/*************************************************************************
* Embedded HTTP Server; Content builder
*
* Copyright 2001 Stanislav Andrianov
*
* Version 0.7
* 
* 6 Nov 2001
*************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>


#include "html2c.h"

HTM_PAGE_DSC_T page_dsc[MAX_PAGE_NUM];
int            page_cnt;
char           out_name[64];
char           cfg_name[64];
char           cur_file_name[64];

/*************************************************************************/
void h2c_init(void){

	memset(page_dsc, 0, sizeof(page_dsc));
	page_cnt = 0;

	strcpy(out_name, "htmpages.c");
	strcpy(cfg_name, "html2c.cfg");
	cur_file_name[0] = 0;
}

/*************************************************************************/
char ext_list[4][4] = {"htm","jpg","gif","txt"}; 
MIME_T h2c_get_mime(char *e){
	int j;

	for (j=0; j<4; j++)
		if (!strcmp(e, ext_list[j]))
			return j+1;

	return MIME_NON;

}

/*************************************************************************/
int h2c_pars_cfg(void){

	FILE *fcfg;
	char  line[128];
	char *tok, *ext;
	MIME_T mime;

	if (!(fcfg = fopen(cfg_name, "r"))){
		perror("Config file open error");
		return 1;
	}
	
	while(fgets(line, 128, fcfg)){
		tok = strtok(line, " \t");
		if (tok){
			if (!strcmp(tok, "PAGE")){
				if (page_cnt >= MAX_PAGE_NUM){
					printf("ERROR: max number of pages exceeded\n");
					return 1;
				}

				tok = strtok(NULL, " \n\t");
				if (!tok){
					printf("ERROR: page filename required\n");
					return 1;
				}
				strcpy(cur_file_name, tok);
				ext = strrchr(tok, '.');
				*ext = 0;
				ext++;

				if ((mime = h2c_get_mime(ext)) == MIME_NON){
					printf("ERROR: unknown mime type\n");
					return 1;
				}
				
				strcpy(page_dsc[page_cnt].filename, cur_file_name);
				strcpy(page_dsc[page_cnt].varname, tok);
				strcat(page_dsc[page_cnt].varname, "_page");
				page_dsc[page_cnt].mime = mime;
				page_cnt++;
			} 
		}
	}

	fclose(fcfg);

	return 0;
}

/*************************************************************************/
unsigned char rdbuf[4096];
int h2c_build_page_var(FILE *fout, int k){
	
	int fd;
	int tot_len, seg_len;
	int j;
	int pk = 0;
	
	tot_len = 0;
	fd = _open(page_dsc[k].filename, _O_RDONLY | _O_BINARY);	
	if (fd < 0){
		perror("Cannot open page source");
		return -2;
	}

	fprintf(fout,"/********* source file %s *************/\n", page_dsc[k].filename);
	fprintf(fout,"unsigned char %s[] = {\n", page_dsc[k].varname);

	while((seg_len = _read(fd, rdbuf, 4096)) > 0){
		for (j=0; j<seg_len; j++){
			if (pk == 0)
				fprintf(fout, "\t");
			if ((seg_len != 4096)&&(j == (seg_len-1)))
				fprintf(fout, "0x%02X", rdbuf[j]);
			else
				fprintf(fout, "0x%02X, ", rdbuf[j]);
			pk++;
			if (pk == 8){
				fprintf(fout, "\n");
				pk = 0;
			}
		}
		tot_len += seg_len;
	}

	fprintf(fout, "\n}; /* end of %s length %d */;\n\n", page_dsc[k].varname, tot_len);
	page_dsc[k].file_len = tot_len;

	_close(fd);

	return 0;
}

/*************************************************************************/
char mime_str[5][16] = {
	"MIME_NON ", "MIME_HTML", "MIME_JPEG", "MIME_GIF ", "MIME_TXT"
};

int h2c_build_page_list(FILE *fout){
	int j;


	fprintf(fout,"/******************** page list *********************/\n");
	fprintf(fout,"HTML_PAGE_LIST_T page_list[] = {\n");

	for (j=0; j<page_cnt; j++)
		fprintf(fout, "    {\"/%s\",\t%s,\t%d,\t%s\t}%c\n",
		     page_dsc[j].filename, page_dsc[j].varname, 
			 page_dsc[j].file_len, mime_str[page_dsc[j].mime], (j==(page_cnt-1))?' ':',');

	fprintf(fout,"}; /* page_list[] */\n");
	fprintf(fout,"int\tpages_cnt = %d;\n\n", page_cnt); 

	return 0;
}

/*************************************************************************/
int h2c_build_header(FILE *fout){

    time_t ltime;

    time( &ltime );

	fprintf(fout,"/*************************************************************************\n");
	fprintf(fout,"* Embedded HTTP Server\n");
	fprintf(fout,"*\n");
	fprintf(fout,"* Copyright 2001 Stanislav Andrianov\n");	
	fprintf(fout,"*\n");
	fprintf(fout,"* file htmpages.c - authomatically generated by html2c\n");
	fprintf(fout,"*\n");
	fprintf(fout,"* DATE: %s", ctime( &ltime ));
	fprintf(fout,"*\n");
	fprintf(fout,"*************************************************************************/\n");
	fprintf(fout,"#include \"htmpages.h\"\n\n\n");

	return 0;
}

/*************************************************************************/
int main(int argc, char *argv[])
{
	FILE *of;
	int  j;

	h2c_init();	

	if (h2c_pars_cfg())
		return 1;

	if (!(of = fopen(out_name, "w+"))){
		perror("Cannot create output file");
		return 1;
	}
	
	h2c_build_header(of);

	/* build page variables */
	for (j=0; j<page_cnt; j++)
		if (h2c_build_page_var(of, j))
			return 1;
	
	h2c_build_page_list(of);

	fclose(of);

	return 0;
}