/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2003 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*                                                                         */
/*   MODULE:  dda_proc.c                                                */
/*   PURPOSE: implimitation functions for proc fs support for managemnet   */
/*			  appl.													       */
/*                                                                         */
/***************************************************************************/

/***********************************************************
	includes
*******************************************************/
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>

#include "cli.h"
#include "dda_proc_fs_defs.h"

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int dda_read_proc_entry(char *dir, char *entry,char *buf,int buf_len)
{
	int  fd, len = 0;
	char proc_path[256];


	strcpy(proc_path,WLAN_PROC_BASE_DIR_PATH);
	strcat(proc_path,dir);
	strcat(proc_path,"/");
	strcat(proc_path,entry);

	if((fd = open(proc_path, O_RDONLY)) == -1)
	{
		fprintf(stderr, "error opening file %s, in proc dir %s, error: %s\n", 
				entry,dir, strerror(errno)); fflush(NULL);
		return -1;
	}
   
    /*For proper use, the proc function should start writing at an offset of "offset"
	in page and write at most "count" bytes */
	len = read(fd, buf, buf_len);

	close(fd);
	
	return len;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int dda_print_proc_entry(char *dir, char *entry)
{
	int  fd, len = 0;
	char proc_path[256];
	char page[4096];


	strcpy(proc_path,WLAN_PROC_BASE_DIR_PATH);
	strcat(proc_path,dir);
	strcat(proc_path,"/");
	strcat(proc_path,entry);

	if((fd = open(proc_path, O_RDONLY)) == -1)
	{
		fprintf(stderr, "error opening file %s, in proc dir %s, error: %s\n", 
				entry,dir, strerror(errno)); fflush(NULL);
		return -1;
	}

	while ((len = read(fd, page, sizeof(page))) > 0) {
		write(1, page, len);
	}

	close(fd);
	
	return len;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int dda_write_proc_entry(char *dir, char *entry  , char *buf,int buf_len)
{
	int fd;
	char proc_path[256];
	strcpy(proc_path,WLAN_PROC_BASE_DIR_PATH);
	strcat(proc_path,dir);
	strcat(proc_path,"/");
	strcat(proc_path,entry);

	if((fd = open(proc_path, O_WRONLY)) == -1)
	{
		fprintf(stderr, "error opening file %s, in proc dir %s, error: %s\n", 
				entry,dir, strerror(errno));
		return -1;
	}

	buf_len = write(fd,(void*)buf,buf_len);
	close(fd);
	return buf_len;
}

