/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2003 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																									*/
/*   MODULE:  command.h																		*/
/*   PURPOSE: Demo application console commands 	 								*/
/*																									*/
/***************************************************************************/
#ifndef _CLI_H_
#define _CLI_H_

#include <linux/types.h> 
#include <sys/reboot.h>

#include "wcfglib.h"

#ifndef __KERNEL__
/* basic data types definitions */
typedef __u8 UINT8;
typedef __u16 UINT16;
typedef __u32 UINT32;
typedef __u32 BOOL;
typedef __u32 UINT;

#define OK   0
#define NOK  1
#define TRUE 1
#define FALSE 0

#define min(x,y) ((x) < (y) ? (x) : (y))
#endif

#define	MAX_PRINT_STRING		128
#define ACCESS_LST_ENTRY_LEN		25
/* Rates constants used in dot11_supRates_T & dot11_extSupRates_T IE */
#define DOT11_RATE_1                                            2
#define DOT11_RATE_2                                            4
#define DOT11_RATE_5_5                                          11
#define DOT11_RATE_6                                            12
#define DOT11_RATE_9                                            18
#define DOT11_RATE_11                                           22
#define DOT11_RATE_12                                           24
#define DOT11_RATE_18                                           36
#define DOT11_RATE_22                                           44
#define DOT11_RATE_24                                           48
#define DOT11_RATE_33                                           66
#define DOT11_RATE_36                                           72
#define DOT11_RATE_48                                           96
#define DOT11_RATE_54                                           108

#define NUM_OF_REGULATORY_DOMAINS_A 2
#define NUM_OF_REGULATORY_DOMAINS_B 8

#define NUM_OF_RANGE_ELEMENTS_US    2
#define NUM_OF_RANGE_ELEMENTS_WORLD 3


#define MIN_BEACON_PERIOD               1
#define MAX_BEACON_PERIOD               65535

#define MIN_DTIM_PERIOD         1
#define MAX_DTIM_PERIOD         255

#define MIN_RTS_THRESHOLD               0
#define MAX_RTS_THRESHOLD               4096

#define MIN_SHORT_RETRY_LIMIT   1
#define MAX_SHORT_RETRY_LIMIT   255

#define MIN_LONG_RETRY_LIMIT    1
#define MAX_LONG_RETRY_LIMIT    255

#define MIN_FRAG_THRESHOLD              256
#define MAX_FRAG_THRESHOLD              4096

#define MIN_TRANSMIT_MSDU_LIFETIME              1
#define MAX_TRANSMIT_MSDU_LIFETIME              0xFFFFFFFF

#define MIN_RECEIVE_MSDU_LIFETIME               1
#define MAX_RECEIVE_MSDU_LIFETIME               0xFFFFFFFF

#define RADIO_PARAM_OVERRIDE_POWERTABLE   1


long wapp_utilsAddSubMenusToConsole(void);
void wdrv_dbPrintFourXTable(void);

#define PRINT_CONFIG_ERR_MSG(str) do { fprintf(stderr,"%s: conferror: %s\n", __func__, str); fflush(stderr); } while(0)
#define PRINT_PROC_WRITE_ERR(dir, entry) do { fprintf(stderr,"%s: writing to proc entry %s/%s failed\n", __func__, dir, entry);fflush(stderr); } while(0)
#define PRINT_PROC_READ_ERR(dir, entry) do { fprintf(stderr,"%s: reading proc entry %s/%s failed\n", __func__, dir, entry);fflush(stderr); } while(0)
//#define PRINT_GET_CONFIG_ERR_MSG(str) fprintf(stderr, "%s: error: %s\n", __func__, str); fflush(stderr);
#define PRINT_TODO_MSG printf("To be implemented (using proc)\n")
#define PRINT_NOT_SUPPORTED_MSG(s) do { fprintf(stderr, "%s: feature is not supported in apdk 5.6\n", s); fflush(stderr); } while(0)


#define linux_reboot()		\
do { 				\
	sync();			\
	sleep(1);		\
	reboot(RB_ENABLE_CAD);	\
	sync();			\
	sleep(1);		\
	reboot(RB_AUTOBOOT);	\
} while(0)


#define MAX_SCREEN_BUFF	80*25
#define MAX_LINE_LEN	600

#define IMMEDIATE 1
#define DEFERRED  0

#endif
