/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																									*/
/*		MODULE:	utilsConsole.c																*/
/*   PURPOSE:	Console implementation													*/
/*																									*/
/***************************************************************************/

/**************************************************************************/
/* The Implementation of the command console is divided to two modules,   */
/* The first module, implemented in the file console.c, is parsing the    */
/* user input according to a table of commands (CommandArray[]), which    */
/* includes the command name, parameters and function. If the user's input*/
/* is found in the table, the command function is called, and the parsed  */
/* parameters are passed to it. The command's functions are implemented   */
/* in command.c module.                                                   */
/**************************************************************************/
//#include "rt_os.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/klog.h>

#ifdef __linux
#include <unistd.h>
#include <linux/unistd.h>
#endif
#ifdef VXWORKS
#include <ioLib.h>
#endif
//#include "pform_types.h"
#include "utilsConsole.h"
#include "wcfglib.h"

#define COMMAND_LEN     128
#define STATUS_LEN      600

#define CONSOLE_MAX_PROMPT_CHARS 15
#define MAX_COMMAND_LINE_LEN     128

#define CONSOLE_IS_CTX_PARAM(p)     ((p) >= NEW_CTX_PRM)
#define CONSOLE_GET_CTX_PARAM(p)    ((ConsoleCommandParameter_T)((p) -  NEW_CTX_PRM))
#define CONSOLE_SET_CTX_PARAM(p,c)  ((p) = (ConsoleCommandParameter_T)(((c) + NEW_CTX_PRM)))

#define MAX_CONSOLE_SUBMENU_LIST         16

#define COMMAND_OK                        0
#define COMMAND_IMPLICIT                  1
#define COMMAND_NOT_FOUND                 2
#define COMMAND_NOT_ENOUGH_PARAMETERS     3
#define COMMAND_BAD_TYPE                  4
#define COMMAND_TO_MANY_PARAMS            5

#define CONSOLE_CONTEXT_TOTAL    16

#define CONSOLE_CTX_PROMPT       (consoleContexts[wapp_utilsConsoleCurrentContext].cPrompt)
#define CONSOLE_CTX_ARRAY        (consoleContexts[wapp_utilsConsoleCurrentContext].cArray)
#define CONSOLE_CTX_OWNER        (consoleContexts[wapp_utilsConsoleCurrentContext].owner)

#define CONSOLE_CTX_COMMAND(i)   (consoleContexts[wapp_utilsConsoleCurrentContext].cArray[(i)])

#define INDEX_NOT_FOUND   -1
#define IMPLICIT_COMMAND  -2
#define LAST_COMMAND  "!" /* the string for 'last command' command */


extern wcfgd *wd;
extern int reportMode;

/************************************************************************
 *          CM Device Data Base Structure                               *
 ************************************************************************/
typedef struct {
    char ConsoleCommand[COMMAND_LEN];
    char ConsoleStatus[STATUS_LEN];
} DevDB_T;

/* current console context */
typedef struct {
   ConsoleCommand_T     *cArray;
   char                cPrompt[CONSOLE_MAX_PROMPT_CHARS];
   long                owner; /* index of owner context, -1 == root */
   ConsoleCommandCB_T   cCB;
   long                valid;
} ConsoleContext_T;

/* current console context reference */
int wapp_utilsConsoleCurrentContext;
DevDB_T               DevDB;
/* Boolean flag indicating current mode */
int consoleMode;
int consoleReady=0; /* 0- if the console is not initiated yet */

unsigned long ConsoleTaskID;
void consoleTask(void);
/* console contexts list */
static ConsoleContext_T consoleContexts[CONSOLE_CONTEXT_TOTAL];

void consoleModeStart(void);
void consoleModeQuit(void);
int  addConsoleCommand(char *cmdname,ConsoleCommandCB_T accfunc,char *desc,ConsoleCommandParameter_T *ccparams, int nparams);
static void consoleSubMenu(ConsoleParameter_T parameters[]);
static void printHelpStrings(void);
void readline(unsigned char *line);

#ifdef TI_INCLUDE_SHELL
void exitToShell(unsigned char **p_buff);
#endif 

long addConsoleMenu(
                   long parent,
                   char *mnuname,
                   ConsoleCommand_T *mnuCommandArray,
                   char *desc,
                   ConsoleCommandParameter_T *ccparams, int nparams,
                   ConsoleCommandCB_T mnuCB);

int wlani_addConsoleCommand(char *cmdname,ConsoleCommandCB_T accfunc,
char *desc,ConsoleCommandParameter_T *ccparams, int nparams);

unsigned char hex(unsigned char c);
unsigned char htol(char str[]);

int checkParameterValidity(unsigned char *parameter,
                  ConsoleCommandParameter_T ParameterType, ConsoleParameter_T *value);
short getCommandIndex(unsigned char *CommandName);

BOOL getDevConsoleCommand(char *string);
BOOL setDevConsoleCommand(char *string);

extern ConsoleCommand_T wapp_utilsConsoleRootCommandArray[];

static int get_printk_console_loglev(int *loglev);

/******************************************************************************
*                      wapp_utilsConsoleInit()											*
*******************************************************************************
DESCRIPTION:	Init console: Create & start console task, init console device 
					database.  						
																										
INPUT:			priority - console task priority
OUTPUT:		   None.
 
RETURN:			OK on suceess, NOK on failure	
																										
******************************************************************************/

#ifdef VXWORKS 

long wapp_utilsConsoleInit(unsigned char priority) {
   int rc;
   int idx;
   int fd;
   
   DevDB.ConsoleCommand[0] = '\0';
	DevDB.ConsoleStatus[0] = '\0';


	/* Initialize serial port to the right mode  */
	fd = open("/tyCo/0", O_RDWR, 0);
	if( fd != ERROR)
	{
		ioctl( fd, FIOSETOPTIONS , OPT_TERMINAL);
		close( fd);
	}


   consoleMode = 1;
   for (idx=0;idx<CONSOLE_CONTEXT_TOTAL;idx++) 
   {
      consoleContexts[idx].valid = 0;
   }

   consoleContexts[0].cArray = wapp_utilsConsoleRootCommandArray;
   strncpy(consoleContexts[0].cPrompt,"MAIN> ",CONSOLE_MAX_PROMPT_CHARS);
   consoleContexts[0].owner = -1;
   consoleContexts[0].cCB = NULL;
   consoleContexts[0].valid = 1;
   wapp_utilsConsoleCurrentContext = 0;

   wapp_utilsAddSubMenusToConsole();
   /* Create the Debug Console Task */
   rc = rt_osTaskCreate( "CSLT", priority, 16384, &ConsoleTaskID ) ;
   if (rc != OS_NOERR)
	{
      printf("Error Creating Console Input Task. Error Code = %ld\n", rc);
		return NOK;
   }

   /* Start the Debug Console Task */
   rc = rt_osTaskStart( ConsoleTaskID, OS_TASK_PREEMPT, consoleTask );
   if (rc != OS_NOERR)
	{
      printf("Error Starting Console Input Task. Error Code = %ld\n", rc);
		return NOK;
   }
	return OK;
}

#else

long wapp_utilsConsoleInit() 
{
   //int rc;
   int idx;
   //int fd;
   
   DevDB.ConsoleCommand[0] = '\0';
	 DevDB.ConsoleStatus[0] = '\0';

   consoleMode = 1;
   for (idx=0; idx<CONSOLE_CONTEXT_TOTAL; idx++)  {
		 consoleContexts[idx].valid = 0;
   }

   consoleContexts[0].cArray = wapp_utilsConsoleRootCommandArray;
   strncpy(consoleContexts[0].cPrompt,"MAIN> ",CONSOLE_MAX_PROMPT_CHARS);
   consoleContexts[0].owner = -1;
   consoleContexts[0].cCB = NULL;
   consoleContexts[0].valid = 1;
   wapp_utilsConsoleCurrentContext = 0;

   wapp_utilsAddSubMenusToConsole();
	 
	 return OK;
}

#endif // VXWORKS

/**************************************************************************
* readline:  Read a line from the console port                            *
***************************************************************************/
void readline(unsigned char *line) {
	//	gets(line);
	fgets(line, MAX_COMMAND_LINE_LEN - 1, stdin);
}

/******************************************************************************
*                      consoleTask()														*
*******************************************************************************
DESCRIPTION:	Consloe task entry point: Wait for user's input, anlayze it and 
					execute the command.  						
																										
INPUT:			None
OUTPUT:		   None.
 
RETURN:			None.
																										
******************************************************************************/
void consoleTask(void) 
{
   unsigned char line[MAX_COMMAND_LINE_LEN];
   int rc, ret, not_read, console_loglev;
   fd_set rfds;
   struct timeval timeout;
   char buf[LOG_BUF_LEN];

   /*   printf("Press <ENTER> to enter Console mode\n\n"); */
   consoleReady = 1;

   if ( get_printk_console_loglev(&console_loglev) )
   {
	   printf("Failed to get printk console log level\n");
	   reportMode = IMMEDIATE;
   }
   else
   {
	   if (console_loglev < DEFAULT_CONSOLE_LOGLEVEL) 
		   reportMode = DEFERRED;
	   else
		   reportMode = IMMEDIATE;
   }

   if (reportMode == DEFERRED) {
			   /* clear the kernel log buffer before starting the console */
			
			   if ( (not_read = klogctl(9, 0, 0)) ){ /* poll the log buffer */ 

					if( klogctl(2, buf, not_read) < 0 ) /* read and clear the buffer*/
						perror("Failed to clear kernel message buffer\n");
			   }

	  }
  
   while(1)
	{
		printf("\n\n>>>\n");

		printf("Console\n");
		printf("Type 'help' for list of commands\n\n");

		consoleModeStart();
		
		printf("%s",CONSOLE_CTX_PROMPT);
		fflush(NULL);

      while(consoleMode) 
	  {
		    if (reportMode == DEFERRED) {
					/* wait at most one second for an input from stdin */
					timeout.tv_sec = 0;
					timeout.tv_usec = 30000; /* 30 millisec */
					FD_ZERO(&rfds);
					FD_SET(0, &rfds);
			
					ret = select(1, &rfds, NULL, NULL, &timeout);
			}
			else {
				ret = 1;
			}

			if (ret) /* there is a data to read at stdin */
			{
		
					 line[0]='\n';
					 readline(line);
					 rc = consoleExecuteCommand(line,CONSOLE_CTX_ARRAY);
					 switch (rc)
						{
						case COMMAND_IMPLICIT:
						   printf("Error: Implicit command.\n");
						   break;
						case COMMAND_NOT_FOUND:
						   printf("Error: Command not found.\n");
						   break;
						case COMMAND_NOT_ENOUGH_PARAMETERS:
						   printf("Error: Not enough paramerers.\n");
						   break;
						case COMMAND_BAD_TYPE:
						   printf("Error: Bad parameter type.\n");
						   break;
						case COMMAND_TO_MANY_PARAMS:
						   printf("Error: To many parameters.\n");
						   break;
					 }
					 
					 printf("%s",CONSOLE_CTX_PROMPT);
					
					 fflush(NULL);
					 
					 
			}
			else /* timeout occured - read and clear kernel message ring buffer */
			{

				if ( (not_read = klogctl(9, 0, 0)) ) { /* poll the log buffer */ 
			
					
					/*ioctl_get_logged_chars(&logged_chars, sizeof(logged_chars), ioctl_sock, wd->dev_name);*/

					ret = klogctl(2, buf, LOG_BUF_LEN - 1); /* read and clear the buffer*/
					
					if (ret < 0) 
						perror("Failed to read kernel message buffer\n");
					else if (ret > 0) {  /* some data was read from the buffer */

							/*printf("\n\n\n BUF_LEN=%d\n\n\n", strlen(buf));*/
							buf[ret] = '\0';
							printf("%s", buf); /* print the data to the console */ 
							fflush(NULL);
							
					}
					
				}
				
			}
			
      }
   }
}

/***************************************************************************
* consoleModeStart: Enable the debug console.										*
***************************************************************************/
void consoleModeStart(void) {
   if (consoleReady == 0)
      return;
   consoleMode = TRUE;
}

/***************************************************************************
* consoleModeQuit: Disable the debug console.										*
***************************************************************************/
void consoleModeQuit(void) {
   consoleMode = FALSE;
}



/***************************************************************************
* quits console mode - replace with owner context (or quit - if root)		*
***************************************************************************/
void consoleQuit(ConsoleParameter_T parameters[]) 
{
/*
   // number of parameters = 0
   unsigned char prnBuffer[WAPP_UTILS_MAX_LINE_LENGTH];
   if (CONSOLE_CTX_OWNER == -1) 
	{ // root console
#ifdef TI_INCLUDE_SHELL
#ifdef __linux__
		int pid;

		puts("Executing shell!");
		pid = fork();

		if (pid == 0) {
			execl("/bin/sh", "/bin/sh", NULL);
		}

		while (waitpid(pid, NULL, 0) < 0) {}

#else
      sprintf(prnBuffer,"Exiting console mode.");
      printf("%s\n",prnBuffer);
      wapp_utilsSetDevConsoleStatus(prnBuffer);
      // consoleModeQuit();
      exitToShell( NULL );
#endif
#endif

	} 
	else  // restore owner console context
      wapp_utilsConsoleCurrentContext = CONSOLE_CTX_OWNER;
  */ 


	if (CONSOLE_CTX_OWNER == -1) {
		tiap_wcfg_light_deinit(wd);
		exit(0);
	}
	else wapp_utilsConsoleCurrentContext = CONSOLE_CTX_OWNER;
}

/***************************************************************************
* prints all the help strings of the command on screen.							*
***************************************************************************/
void consoleHelp(ConsoleParameter_T parameters[]) {
   /* number of parameters = 0  */
   printHelpStrings();
   return;
}



/***************************************************************************
* addConsoleMenu: Adds a console menu.													*
***************************************************************************/
long addConsoleMenu(long parent,
                    char *mnuname,
                   ConsoleCommand_T *mnuCommandArray,
                   char *desc,
                   ConsoleCommandParameter_T *ccparams, int nparams,
                   ConsoleCommandCB_T mnuCB)
{
    int idx,i,j; char *dest;
    int iResult = 0x00;
    long saveConsoleCurrentContext = wapp_utilsConsoleCurrentContext;
    
    do    
    {    
        wapp_utilsConsoleCurrentContext = parent;    
    
    
        for (idx=0;(idx<CONSOLE_CONTEXT_TOTAL) && (consoleContexts[idx].valid);idx++);
        /* idx holds the location for the new menu context */

        if (idx == CONSOLE_CONTEXT_TOTAL)
        {
            iResult = (-1); /* no more place */
            break;
        }

        for (i=0;CONSOLE_CTX_COMMAND(i).FunctionCall != NULL; i++)
        {
	        if (!strcmp(mnuname,CONSOLE_CTX_COMMAND(i).name))
            {
		        iResult = (-1);    /* Menu Command already exists */ 
                break;
            }            
        }
        /* i holds the command location for the new entry  */

        dest = CONSOLE_CTX_COMMAND(i).name;
        if (!strcmp(dest,"nullcommand"))
        {
            iResult = (-1);  /* Command list is full */
            break;
        }
	        
        if (nparams >= MAX_PARAMETER_NUMBER)	
        {
            iResult = (-1); /* one parameter is reserved for internal pourposes */
                             /* it will hold the new context index               */
            break;
        }

        strncpy (dest,mnuname,MAX_COMMAND_NAME_LEN-1);
        dest[MAX_COMMAND_NAME_LEN-1] = 0;

        dest= CONSOLE_CTX_COMMAND(i).HelpString;
        strncpy (dest,desc,MAX_HELP_STRING_LENGTH-1);
        dest[MAX_HELP_STRING_LENGTH-1] = 0;

        CONSOLE_CTX_COMMAND(i).FunctionCall = consoleSubMenu;

        for (j=0; j<MAX_PARAMETER_NUMBER; j++)
	        CONSOLE_CTX_COMMAND(i).ParameterType[j] = (j<nparams ? ccparams[j] : NO_PARAM);

        /* last parameter type is 'special' - holds ref to new console context */
        CONSOLE_SET_CTX_PARAM(CONSOLE_CTX_COMMAND(i).ParameterType[MAX_PARAMETER_NUMBER-1],idx);
        CONSOLE_CTX_COMMAND(i).ParamNumber = nparams;

        /* update submenus context */
        consoleContexts[idx].cArray = mnuCommandArray;
        strncpy(consoleContexts[idx].cPrompt,mnuname,CONSOLE_MAX_PROMPT_CHARS);
        consoleContexts[idx].owner = wapp_utilsConsoleCurrentContext;
        consoleContexts[idx].cCB = mnuCB;
        consoleContexts[idx].valid = 1;
    }
    while(0);
    
	 wapp_utilsConsoleCurrentContext = saveConsoleCurrentContext;
    if(iResult != (-1))
    {
        iResult = idx;
    }
    return iResult;
}

/***************************************************************************
* Adds a console command																	*
***************************************************************************/
int wlani_addConsoleCommand(char *cmdname,ConsoleCommandCB_T accfunc,
   char *desc,ConsoleCommandParameter_T *ccparams, int nparams) {
   
   return addConsoleCommand(cmdname, accfunc, desc, ccparams, nparams);
}

/***************************************************************************
* Adds a console sub-menu											                  *
***************************************************************************/
long addConsoleSubMenu(
                   long parent,
                   char *mnuname,
                   ConsoleCommand_T *mnuCommandArray,
                   char *desc,
                   ConsoleCommandParameter_T *ccparams, int nparams,
                   ConsoleCommandCB_T mnuCB)
{
   return addConsoleMenu(parent,mnuname,mnuCommandArray,desc,ccparams,nparams,mnuCB);
}


/**************************************************************************
* printHelpStrings: Print the command list and help string.					  *
**************************************************************************/
void printHelpStrings(void) {
   short i;
   unsigned char format[80];

   sprintf(format," %%-%ds - %%s.\n",MAX_COMMAND_NAME_LEN - 3);
   printf("Console Commands for this level:\n");
   for (i=0; CONSOLE_CTX_COMMAND(i).FunctionCall != NULL; i++)
	{
      printf(format,CONSOLE_CTX_COMMAND(i).name ,CONSOLE_CTX_COMMAND(i).HelpString );
   }
   return;
}


/******************************************************************************
*                      hex()																	*
*******************************************************************************
DESCRIPTION:	Return the decimal value of an hexadecimal character.  						
																										
INPUT:			c - hexadecimal character.
OUTPUT:		   None.
 
RETURN:			decimal value.
																										
******************************************************************************/
unsigned char hex(unsigned char c)
{
   if (c >= 'A' && c <= 'F')
      return (c - 'A' + 10);
   else if (c >= 'a' && c <= 'f')
      return (c - 'a' + 10);
   return c - '0';
}


/******************************************************************************
*                      htol()																	*
*******************************************************************************
DESCRIPTION:	Return the decimal value of an hexadecimal number.  						
																										
INPUT:			str - hexadecimal string.
OUTPUT:		   None.
 
RETURN:			decimal value.
																										
******************************************************************************/
unsigned char htol(char str[])
{
   unsigned char n;

   n = hex(str[0]) << 4;
   n |= hex(str[1]);
   return n;
}


/******************************************************************************
*                      checkParameterValidity()											*
*******************************************************************************
DESCRIPTION:	Check and find the value of a parameter.        
               The function gets a pointer to the parameter,   
               and the expected type, and returns the value.   
																										
INPUTS:			parameter - pointer to the parameter in the user's input  
               parameterType - expected type of the parameter            
               value - returns the value of the parameter                
OUTPUT:		   None.
 
RETURNS:			TRUE  - parameter parsed correctly.                       
               FALSE - parameter is of the wrong type                    
																										
******************************************************************************/
int checkParameterValidity(unsigned char *parameter,
                  ConsoleCommandParameter_T ParameterType, ConsoleParameter_T *value) {
   short i;
   long intval;
   double realval;
   unsigned long ipval;
   unsigned long uintval;
   unsigned char macval[6];
  //unsigned char *token;
   unsigned int pLength,pPos,pDigitPos;
   unsigned char iCharacter;

   switch (ParameterType){
      case INTEGER_PRM:
         intval = strtol(parameter,NULL,0);
         if ((intval==0) && (strcmp(parameter,"0")!=0) && (strcmp(parameter,"0x0")!=0) && (strcmp(parameter,"00")!=0) && (strcmp(parameter,"0x00")!=0))
            return FALSE;
         value->INTEGER = intval;
         break;

      case SHORT_PRM:
         intval = strtol(parameter,NULL,0);
         if ((intval==0) && (strcmp(parameter,"0")!=0) && (strcmp(parameter,"0x0")!=0) )
            return FALSE;
         value->SHORT = intval;
         break;

      case UNSIGNED_INTEGER_PRM:
         uintval = strtoul(parameter,NULL,0);
         if ((uintval==0) && (strcmp(parameter,"0")!=0) && (strcmp(parameter,"0x0")!=0) && (strcmp(parameter,"00")!=0) && (strcmp(parameter,"0x00")!=0))
            return FALSE;
         value->UNSIGNED_INTEGER = uintval;
         break;

      case REAL_PRM:
#ifdef __linux__
				sscanf(parameter, "%lf", &realval);
#else
				realval = strtod(parameter,NULL);
#endif
         if ((realval==0) && (strcmp(parameter,"0")!=0) && (strcmp(parameter,"0.0")!=0) )
            return FALSE;
         value->REAL = realval;
         break;

      case IP_ADDR_PRM:

         i=0;
         ipval = 0;
         uintval = 0;
         pDigitPos = 0;

         pLength = strlen(parameter);

         if (pLength > 15) return FALSE;

         for (pPos = 0; pPos<pLength;pPos++) 
			{
            if (parameter[pPos] == '.') 
				{
               i++;
               if (i>3) return FALSE;
               if (uintval>255)
                  return FALSE;
               if ((uintval==0)&&(pDigitPos==0))
                  return FALSE;
               ipval <<= 8;
               ipval |= uintval;
               uintval = 0;
               pDigitPos = 0;
            } 
				else if (isdigit(parameter[pPos])) 
				{
               pDigitPos++;
               if (pDigitPos > 3) return FALSE;
               uintval = uintval*10;
               uintval += (parameter[pPos]-'0');
            } 
				else 
				{
               return FALSE;
            }
         }

         if (i<3) return FALSE;

         /* its the last octet */
         if (uintval>255)
            return FALSE;
         if ((uintval==0)&&(pDigitPos==0))
            return FALSE;
         ipval <<= 8;
         ipval |= uintval;

         value->IP_ADDR = ipval;
         break;

      case MAC_ADDR_PRM:

         for (i=0;i<6; i++)
            macval[i] = 0;
         i=0;

         uintval = 0;
         pDigitPos = 0;

         pLength = strlen(parameter);

         if (pLength > 17) return FALSE;

         for (pPos = 0; pPos<pLength;pPos++) 
			{
            if (parameter[pPos] == '-') 
				{
               if (i>=5) return FALSE; /* its the sixth '-' */
               if ((uintval==0)&&(pDigitPos==0))
                  return FALSE;
               macval[i] = uintval;
               i++;
               uintval = 0;
               pDigitPos = 0;
            } 
				else if (isxdigit(parameter[pPos])) {
               pDigitPos++;
               if (pDigitPos > 2) return FALSE;
               uintval = uintval*16;
               if (isdigit(parameter[pPos]))
                  uintval += (parameter[pPos]-'0');
               else
                  uintval += hex(parameter[pPos]);
            } 
				else 
				{
               return FALSE;
            }
         }

         if (i<5) return FALSE;

         /* its the last octet */
         if ((uintval==0)&&(pDigitPos==0))
            return FALSE;
         macval[i] = uintval;

         memcpy(value->MAC_ADDR,macval,6);
         break;
      case STRING_PRM:
         for(i = 0x00;;i++)
         {
             iCharacter = parameter[i];
             if(iCharacter == '\0')                                /* exit with OK on EOL */
             {
                value->STRING = parameter;
                break;
             }
             if(i >= (MAX_COMMAND_LINE_LEN - MAX_COMMAND_NAME_LEN)) /* exit with error on */
             {                                                      /* max. string length */
                 return FALSE;
             }
             if(isgraph(iCharacter) == 0x00)
             {                                                       /* exit with error on      */
                return FALSE;                                        /* non-printable character */
             }
         }
         break;  
  
	  case ARGS_PRM:	/* Takes the rest of the line, including spaces */
         for(i = 0x00;;i++)
         {
			 iCharacter = parameter[i];
             if(iCharacter == '\0' )
			 {
				 value->ARGS = parameter;
				 break;                       /* exit with OK on EOL */
			 }
             if(i >= (MAX_COMMAND_LINE_LEN - MAX_COMMAND_NAME_LEN)) /* exit with error on */
             {                                                      /* max. string length */
           		 value->ARGS = NULL;
				    return FALSE;
             }
             if(isprint(iCharacter) == 0x00)
             {                                                       /* exit with error on      */
	       		value->ARGS = NULL;
               return FALSE;                                        /* non-printable character */
             }
         }
		 break;
      default:
         return FALSE;
   } /* switch */
   return TRUE;
}

/******************************************************************************
*                      getCommandIndex()													*
*******************************************************************************
DESCRIPTION:	Return the index of the command in CommandArray.       
                  Command name can be the prefix name as long as there   
                  is no more then one  command with that prefix.         
                                                                         
INPUTS:			CommandName - Pointer to the user's input command name   

OUTPUT:			None

RETURNS:			index -  Index of command in CommandArray, if command is 
                         found.                                          
               INDEX_NOT_FOUND  - if the command does not exist.        
               IMPLICIT_COMMAND - If the prefix can match more the one  
                                  command                                
                                                                         
******************************************************************************/
short getCommandIndex(unsigned char *CommandName)
{
   short i, index;
   unsigned char *c;

   index = INDEX_NOT_FOUND;
   for (i=0; CONSOLE_CTX_COMMAND(i).FunctionCall != NULL; i++){
      c = strstr(CONSOLE_CTX_COMMAND(i).name, CommandName);
      if ((c!=NULL)&&(c==CONSOLE_CTX_COMMAND(i).name)){
         if (index != INDEX_NOT_FOUND)
            return IMPLICIT_COMMAND;
         index = i;
      }
   }
   return index;
}

/******************************************************************************
*                      consoleExecuteCommand()													*
*******************************************************************************
DECSRIPTION:  Parses the line, devides it to the command   
               name and parameters, checks the validity of the parameters
               and calls the function that will execute the command.     
                                                                         
INPUTS:    line - Pointer to the user's input                       
                                                                         
OUTPUT:    None.

RETURNS:   COMMAND_OK - command was executed OK.                    
                COMMAND_NOT_ENOUGH_PARAMETERS - less than the defined    
                      number of parameters were supplied                 
                COMMAND_BAD_TYPE  - one or more of the parameters is     
                        of the wrong type                                
                COMMAND_TO_MANY_PARAMS - more than the defined number    
                         of parameters were supplied                     
                COMMAND_NOT_FOUND - the command name was not found in    
                         the list of commands                            
                COMMAND_IMPLICIT - more the one command match the prefix 
                                                                         
*******************************************************************************/
int consoleExecuteCommand(unsigned char *line,ConsoleCommand_T *commandArray) {
   unsigned char CommandLine[MAX_COMMAND_LINE_LEN];
   unsigned char CommandName[MAX_COMMAND_NAME_LEN];
   short CommandIndex;
   unsigned char *token;
   const unsigned char seps[] = " \t\n\r";
   short ParameterNumber = 0;

   int valid;
   ConsoleParameter_T         value;
   ConsoleCommandParameter_T  type;
   ConsoleParameter_T         parameters[MAX_PARAMETER_NUMBER];
   ConsoleCommandCB_T         FuncCall;

/* Copy line for strtok parsing  */
   strncpy(CommandLine,line,MAX_COMMAND_LINE_LEN);

   if ( (token  = strtok(CommandLine, seps)) == NULL )
      return FALSE;
   strncpy(CommandName,token,MAX_COMMAND_NAME_LEN);
   CommandIndex = getCommandIndex(CommandName);

   if (CommandIndex == IMPLICIT_COMMAND)
         return COMMAND_IMPLICIT;
   if (CommandIndex == INDEX_NOT_FOUND)
         return COMMAND_NOT_FOUND;
   /* First argument */
   token  = strtok(NULL, seps);


   while (token != NULL)
	{
      ParameterNumber++;
      if (ParameterNumber > CONSOLE_CTX_COMMAND(CommandIndex).ParamNumber)
         return COMMAND_TO_MANY_PARAMS;
      type = CONSOLE_CTX_COMMAND(CommandIndex).ParameterType[ParameterNumber-1];
#if 0
      if( type == ARGS_PRM)
	   {
		  /* restore the original command line, as strtok() puts a NULL, stop on EOL */
		  strncpy(CommandLine,line,MAX_COMMAND_LINE_LEN);
		  token  = strtok(CommandLine, "\r\n");
	   }
#endif
	  /* variable arguments, let the function parse and handle it,
		 ARGS_PRM can be only last. It might be empty */
     valid = checkParameterValidity( token, type, &value );
      if (valid == FALSE)
         return COMMAND_BAD_TYPE;
      parameters[ParameterNumber-1] = value;

	  /* variable arguments, let the function parse and handle it,
		 ARGS_PRM can be only last. It might be empty */
	  if( type == ARGS_PRM)
	  {
        break;
	  }
     token = strtok(NULL, seps);

   }/* endWhile */

   /* ARGS_PRM might be NULL too, this is not an error */
   if( (token == NULL) &&
	   (ParameterNumber < MAX_PARAMETER_NUMBER) &&
	   (CONSOLE_CTX_COMMAND(CommandIndex).ParameterType[ParameterNumber] == ARGS_PRM))
   {
   }
   else
   {

      if (ParameterNumber < CONSOLE_CTX_COMMAND(CommandIndex).ParamNumber)
      return COMMAND_NOT_ENOUGH_PARAMETERS;
   }
   if (CONSOLE_IS_CTX_PARAM(CONSOLE_CTX_COMMAND(CommandIndex).ParameterType[MAX_PARAMETER_NUMBER-1])) 
	{
      /* last param is NEW_CTX_PRM */
      parameters[MAX_PARAMETER_NUMBER-1].INTEGER =
         CONSOLE_GET_CTX_PARAM(CONSOLE_CTX_COMMAND(CommandIndex).ParameterType[MAX_PARAMETER_NUMBER-1]);
   }

   /* if the current command is the 'last command'
      the DB is not updated */
   if (strcmp(CommandName, LAST_COMMAND))
      setDevConsoleCommand(line);

   FuncCall = CONSOLE_CTX_COMMAND(CommandIndex).FunctionCall;
   (*FuncCall)(parameters);

   return COMMAND_OK;
}

/**************************************************************************
*   called when menu context should be switched due to console command    *
**************************************************************************/
void consoleSubMenu(ConsoleParameter_T parameters[]) {

   if (parameters[MAX_PARAMETER_NUMBER-1].INTEGER >= MAX_CONSOLE_SUBMENU_LIST)
      return;
   if (!(consoleContexts[parameters[MAX_PARAMETER_NUMBER-1].INTEGER].valid))
      return;

   /* change context */
   wapp_utilsConsoleCurrentContext = parameters[MAX_PARAMETER_NUMBER-1].INTEGER;
   /* inform context activation */
   consoleContexts[wapp_utilsConsoleCurrentContext].cCB(parameters);
}

/***************************************************************************
*   Adds console command																	*
****************************************************************************/
int addConsoleCommand(char *cmdname,ConsoleCommandCB_T accfunc,char *desc,ConsoleCommandParameter_T *ccparams, int nparams) {
	int i,j; char *dest;

	for (i=0; CONSOLE_CTX_COMMAND(i).FunctionCall != NULL; i++)
		if (!strcmp(cmdname,CONSOLE_CTX_COMMAND(i).name))
			return (-1);   /* Command already exists */

	dest = CONSOLE_CTX_COMMAND(i).name;
	if (!strcmp(dest,"nullcommand"))
		return (-1);  /* Command list is full */

	strncpy (dest,cmdname,MAX_COMMAND_NAME_LEN-1);
	dest[MAX_COMMAND_NAME_LEN-1] = 0;

	dest= CONSOLE_CTX_COMMAND(i).HelpString;
	strncpy (dest,desc,MAX_HELP_STRING_LENGTH-1);
	dest[MAX_HELP_STRING_LENGTH-1] = 0;

	CONSOLE_CTX_COMMAND(i).FunctionCall = accfunc;

	for (j=0; j<MAX_PARAMETER_NUMBER; j++)
		CONSOLE_CTX_COMMAND(i).ParameterType[j] = (j<nparams ? ccparams[j] : NO_PARAM);
	CONSOLE_CTX_COMMAND(i).ParamNumber = nparams;

	return 0;
}


#ifdef TI_INCLUDE_SHELL
#ifndef __linux__
/***************************************************************
*   void exitToShell(voif)                                *
****************************************************************
* DESCRIPTION: Exit to dlg_UI loop to RTOS shell               *
****************************************************************/
void exitToShell(unsigned char **p_buff)
{
	puts("\nExit to VxShell - Enter 'exit' to return\n");
	
	/* transfer control to the shell, interactive mode */
    shellInit (20000, TRUE);

	/* wait for the shell to exit and cleanup from system */
	while(taskNameToId("tShell") != ERROR)
		taskDelay(sysClkRateGet() >> 1);
	errnoSet(OK);

	puts("\nBack to Console...\n");
}
#endif // __linux
#endif /* TI_INCLUDE_SHELL */

/************************************************************************
*                 getDevConsoleCommand()											*
*************************************************************************
DESCRIPTION: Get device last console command.								*
																								*
INPUT:       no.																			*
OUTPUT:      string - pointer to CM console command string.					*
																								*
RETURN:      OK if succeeded, otherwise NOK.										*
																								*
************************************************************************/
BOOL getDevConsoleCommand(char *string)
{
    strncpy(string,DevDB.ConsoleCommand,COMMAND_LEN);
    return OK;
}

/************************************************************************
*                 setDevConsoleCommand()											*
*************************************************************************
DESCRIPTION: Set device new console command.									   *
																								*
INPUT:       string - pointer to CM console command string.					*	
              len - string length.													*
OUTPUT:      no.																			*
																								*
RETURN:      OK if succeeded, otherwise NOK.										*
																								*
*************************************************************************/
BOOL setDevConsoleCommand(char *string)
{
    long len;

    len = strlen(string);
    len = min(len,COMMAND_LEN - 1);
    memcpy(DevDB.ConsoleCommand,string,len);
    DevDB.ConsoleCommand[len] = '\0';
    DevDB.ConsoleStatus[0] = '\0';
    return OK;
}

/************************************************************************
*                 getDevConsoleStatus()											*
*************************************************************************
DESCRIPTION: Get device last console status.									*
																								*
INPUT:       no.																			*
OUTPUT:      string - pointer to CM console status string.					*
																								*
RETURN:      OK if succeeded, otherwise NOK.										*
																								*
************************************************************************/
BOOL getDevConsoleStatus(char *string)
{
    strncpy(string,DevDB.ConsoleStatus,STATUS_LEN);
    return OK;
}

/************************************************************************
*                 wapp_utilsSetDevConsoleStatus()										*
*************************************************************************
DESCRIPTION: Set device result console status.								*
																								*	
INPUT:       no.																			*
OUTPUT:      string - pointer to CM console status string.					*
																								*
RETURN:      OK if succeeded, otherwise NOK.										*
																								*
*************************************************************************/
BOOL wapp_utilsSetDevConsoleStatus(char *string)
{
    long len;

    len = strlen(string);
    len = min(len,STATUS_LEN - 1);
    memcpy(DevDB.ConsoleStatus,string,len);
    DevDB.ConsoleStatus[len] = '\0';
    return OK;
}



int get_printk_console_loglev(int *loglev)
{
	int fd;
	char buf[64];

	fd = open("/proc/sys/kernel/printk", O_RDONLY);
	if (fd < 0) {
		perror("Can't open /proc/sys/kernel/printk");
		return -1;
	}

	if ( read(fd, buf, sizeof(buf)) < sizeof(int) )
	{
		perror("failed to read /proc/sys/kernel/printk");
		return -1;
	}

	sscanf(buf,"%d", loglev); 

	close(fd);

	return 0;

}	
