/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																									*/
/*		MODULE:	utilsConsole.h																*/
/*   PURPOSE:	Console implementation													*/
/*																									*/
/***************************************************************************/
#ifndef _CONSOLE_H_
#define _CONSOLE_H_

//#include <sys/types.h> 
/*typedef __u8 UINT8;
typedef __u16 UINT16;
typedef __u32 UINT32;
typedef __u32 BOOL;
typedef __u32 UINT;*/

#include <netinet/if_ether.h>
#include "cli.h"

#define WAPP_UTILS_MAX_LINE_LENGTH	600

#define MAX_PARAMETER_NUMBER     35
#define MAX_COMMAND_NAME_LEN     20
#define MAX_HELP_STRING_LENGTH   250

typedef enum {
   NO_PARAM       = 0 ,
   INTEGER_PRM    = 1 ,
   REAL_PRM       = 2 ,
   IP_ADDR_PRM    = 3 ,
   MAC_ADDR_PRM   = 4 ,
   STRING_PRM     = 5 ,
   SHORT_PRM	  = 6 ,
   UNSIGNED_INTEGER_PRM    = 7 ,
   ARGS_PRM		  = 8 ,	/* can be only last in the params type list */

   /* no other param types should be defined AFTER this line                 */
   /* the following param types are used for holding 'new context' reference */
   NEW_CTX_PRM    = 9
} ConsoleCommandParameter_T;

typedef union {
   long INTEGER;
   double REAL;
   unsigned long IP_ADDR;
   unsigned char MAC_ADDR[ETH_ALEN];
   unsigned char *STRING;
   UINT16	SHORT;	
   UINT32	UNSIGNED_INTEGER;
   unsigned char *ARGS;	/* same type as STRING */
} ConsoleParameter_T;

typedef void (* ConsoleCommandCB_T)(ConsoleParameter_T parameters[]);

typedef struct CommandStruct
{
   unsigned char     name[MAX_COMMAND_NAME_LEN+4];
   short             ParamNumber;
   ConsoleCommandCB_T FunctionCall;
   ConsoleCommandParameter_T  ParameterType[MAX_PARAMETER_NUMBER];
   unsigned char     HelpString[MAX_HELP_STRING_LENGTH];
} ConsoleCommand_T;

/* The Console has 2 additional commands and 15 more spaces for sub-menus	*/
#define WAPP_UTILS_CONSOLE_MAIN_ARRAY_POSTFIX \
   { "help",  0, consoleHelp,{NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Display this message"} ,\
   { "exit",  0, consoleQuit,{NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Quit the console"} ,    \
   { "dumy0", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy1", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy2", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy3", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy4", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy5", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy6", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy7", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy8", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy9", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy10",0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy11",0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy12",0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy13",0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy14",0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy15",0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy16",0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy17",0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy18",0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy19",0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"}           


/* The Console has 2 additional commands and 15 more spaces for sub-menus	*/
#define WAPP_UTILS_CONSOLE_SUB_MENU_ARRAY_POSTFIX \
   { "help",  0, consoleHelp,{NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Display this message"} ,\
   { "exit",  0, consoleQuit,{NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Quit this sub-menu"} ,    \
   { "dumy0", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy1", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy2", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy3", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy4", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy5", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"},           \
   { "dumy6", 0, NULL,         {NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM,NO_PARAM},"Undefined"}


/* maximum password length - not including terminating zero */
#define PRIORITY_CONSOLE 110

long wapp_utilsConsoleInit(

#ifdef VXWORKS
unsigned char priority
#else
void
#endif
);
void consoleQuit(ConsoleParameter_T parameters[]);
void consoleHelp(ConsoleParameter_T parameters[]);
int  consoleExecuteCommand(unsigned char *line,ConsoleCommand_T *commandArray);
BOOL getDevConsoleStatus(char *string);
BOOL wapp_utilsSetDevConsoleStatus(char *string);
long addConsoleSubMenu(
                   long parent,
                   char *mnuname,
                   ConsoleCommand_T *mnuCommandArray,
                   char *desc,
                   ConsoleCommandParameter_T *ccparams, int nparams,
                   ConsoleCommandCB_T mnuCB);

BOOL getDevConsoleCommand(char *string);

#define LOG_BUF_LEN 1024 /* the length of the kernel message ring buffer*/
#define DEFAULT_CONSOLE_LOGLEVEL 7 /* anything MORE serious than KERN_DEBUG */


#endif
