

#include <stdio.h>
//#include <unistd.h>
//#include <libgen.h>
//#include <signal.h>
//#include <string.h>
#include <errno.h>
//#include <sys/wait.h>
//#include <stdarg.h>
#include <sys/types.h>
#include <sys/socket.h>
//#include <sys/un.h>
#include <linux/route.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <linux/if_bridge.h>

//#include "iwlib.h"
#include "wcfglib.h"
//#include "dda_wext_ioctl.h"
//#include "dda_wpaauth_ioctl.h"
//#include "wcfg_ioctl.h"


/*taken from bridge-utils*/
unsigned int if_nametoindex (const char *ifname)
{
	struct ifreq ifr;
	int fd;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		fprintf(stderr, "wcfglib: %s\n", strerror(errno));
		return 0;
	}

	strncpy (ifr.ifr_name, ifname, sizeof (ifr.ifr_name));
	if (ioctl (fd, SIOCGIFINDEX, &ifr) < 0) {
		fprintf(stderr, "wcfglib: %s\n", strerror(errno));
		close(fd);
		return 0;
	}

	close (fd);
	return ifr.ifr_ifindex;
}


int br_addif(const char *br_name, const char *ifname)
{
	unsigned long args[4];
	unsigned long ifindex;
	struct ifreq ifr;
	int fd, err;

	if (!(ifindex = if_nametoindex(ifname))) {
		fprintf(stderr, "wcfglib: interface %s does not exist!\n", ifname);
		return 1;
	}

	args[0] = BRCTL_ADD_IF;
	args[1] = ifindex;
	args[2] = 0;
	args[3] = 0;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		fprintf(stderr, "wcfglib: %s\n", strerror(errno));
		return 1;
	}

	memcpy(ifr.ifr_name, br_name, IFNAMSIZ);
	((unsigned long *)(&ifr.ifr_data))[0] = (unsigned long)args;

	if( (err = ioctl(fd, SIOCDEVPRIVATE, &ifr)) )
		switch ((err = ioctl(fd, SIOCDEVPRIVATE, &ifr))) {
			case EBUSY:
				fprintf(stderr, "wcfglib: device %s is already a member of a bridge; "
						"can't enslave it to bridge %s.\n", ifname,
						br_name);
				break;

			case ELOOP:
				fprintf(stderr, "wcfglib: device %s is a bridge device itself; "
						"can't enslave a bridge device to a bridge device.\n",
						ifname);
				break;

			default:
				fprintf(stderr, "wcfglib: %s\n", strerror(errno));
				break;
		}

	close(fd);
	return err;
}


int tiap_wcfg_set_ipaddr_or_netmask(const char *devname, int id, __u32 addr)
{
	struct ifreq ifr;
	struct sockaddr_in *sin = (void *) &ifr.ifr_ifru.ifru_addr;
	int fd;

	if (!devname) return 1;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		fprintf(stderr, "wcfglib: %s\n", strerror(errno));
		return 1;
	}

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, devname, IFNAMSIZ);
	sin->sin_family = AF_INET;
	sin->sin_addr.s_addr = addr;

	if (ioctl(fd, id, &ifr) == -1) {
		fprintf(stderr, "%s: %s (addr=%u id=%d)\n", devname, strerror(errno), addr, id);
		return 1;
	}

	close(fd);

	return 0;
}


int tiap_wcfg_dev_set_flags(const char *devname, int flag, int enable)
{
	struct ifreq ifr;
	struct sockaddr_in *sin = (void *) &ifr.ifr_ifru.ifru_addr;
	int fd;

	if (!devname) return 1;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		fprintf(stderr, "%d: %s\n", __LINE__, strerror(errno));
		return 1;
	}

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, devname, IFNAMSIZ);
	sin->sin_family = AF_INET;

	if (ioctl(fd, SIOCGIFFLAGS, &ifr) == -1) {
		fprintf(stderr, "%s: %s\n", devname, strerror(errno));
		return 1;
	}

	if( !enable )
		ifr.ifr_flags &= flag;
	else ifr.ifr_flags |= flag;

	if (ioctl(fd, SIOCSIFFLAGS, &ifr) == -1) {
		fprintf(stderr, "%s: %s\n", devname, strerror(errno));
		return 1;
	}

	return 0;
}


static void set_sockaddr(struct sockaddr_in *sin, __u32 addr, __u16 port)
{
	sin->sin_family = AF_INET;
	sin->sin_addr.s_addr = addr;
	sin->sin_port = port;
}


int tiap_wcfg_set_route(const char *dev, __u32 dst, __u32 genmask, __u32 gateway, const int id)
{
	struct rtentry rm;
	int fd, err;

	if( id != SIOCADDRT && id != SIOCDELRT ) {
		fprintf(stderr, "Cannot add route, invalid id=%d\n", id); 
		return 1;
	}

	memset(&rm, 0, sizeof(rm));
	rm.rt_dev = (char *)dev;
	set_sockaddr((struct sockaddr_in *) &rm.rt_dst, dst, 0);
	set_sockaddr((struct sockaddr_in *) &rm.rt_genmask, genmask, 0);
	set_sockaddr((struct sockaddr_in *) &rm.rt_gateway, gateway, 0);
	rm.rt_flags = RTF_UP;
	if( gateway )
		rm.rt_flags |= RTF_GATEWAY;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		perror("socket");
		return fd;
	}

	if ((err = ioctl(fd, id, &rm)) < 0) {
		fprintf(stderr, "Cannot add route %u (%d): %s.\n", gateway, err, strerror(errno));
		close(fd);
		return err;
	}

	close(fd);
	
	return 0;
}


int config_ip_settings(wcfgd *wd)
{
	/*ip address*/
	if( tiap_wcfg_set_ipaddr_or_netmask(wd->br_name, SIOCSIFADDR, wd->storage.ldb.ipaddr) )
                return 1;

	/*ip address netmask*/
	if( tiap_wcfg_set_ipaddr_or_netmask(wd->br_name, SIOCSIFNETMASK, wd->storage.ldb.netmask) )
                return 1;

	if( tiap_wcfg_dev_set_flags(wd->br_name, IFF_UP, 1) )
		return 1;

	/*
	net = wd->storage.ldb.ipaddr & wd->storage.ldb.netmask;
	if( tiap_wcfg_set_route(wd->br_name, net, wd->storage.ldb.netmask, 0, SIOCADDRT) )
                return 1;
	*/

	/*default gateway*/
	if( tiap_wcfg_set_route(wd->br_name, 0, 0, wd->storage.ldb.default_gw, SIOCADDRT) )
		return 1;

	if( tiap_wcfg_dev_set_flags(wd->dev_name, IFF_UP, 1) )
		return 1;

	/*add wireless device to the bridge*/
	if( br_addif(wd->br_name, wd->dev_name) ) {
		fprintf(stderr, "wcfglib: Warning, failed to add %s device to the the %s bridge, "
				"configuration is not complete.\n", wd->dev_name, wd->br_name);
		fflush(stderr);
	}
	
	return 0;
}


int tiap_wcfg_get_default_gateway(__u32 *gateway_addr)
{
	char delim[]=" \t", *tok;
	FILE *proc_fd;
	char buf[1024];


	if (!(proc_fd = fopen("/proc/net/route","r"))) {
		perror("fopen");
		return 1;
	}
	/* each line in the file has the following format: 
	<Iface>   <Destination>   <Gateway> ....*/

	while (fgets(buf, sizeof(buf), proc_fd) != NULL)
	{
		if (! (tok = strtok(buf, delim))) /*interface*/ {
			fprintf(stderr, "Invalid file format\n");
			return 1;
		}

		tok = strtok(NULL, delim);/*dest address*/

		if (tok)
		{
			if (!strcmp(tok,"00000000")) 
			{
				tok = strtok(NULL, delim); /*gateway address*/
				sscanf(tok,"%X",gateway_addr);
				return 0;
			}
		}
	}

	return 1;
}



void tiap_wcfg_set_storage_ipaddr(wcfgd *wd, const int id, const __u32 addr)
{
int update_storage = 0;

	switch( id )
	{
		case SIOCADDRT:
			wd->storage.ldb.default_gw = addr;
			update_storage = 1;
			break;

		case SIOCSIFADDR:
			wd->storage.ldb.ipaddr = addr;
			update_storage = 1;
			break;

		case SIOCSIFNETMASK:
			wd->storage.ldb.netmask = addr;
			update_storage = 1;
			break;
	}

	if( update_storage )
		wcfg_storage_write(&wd->storage, (const char *)wd->storage_file);
}

int tiap_wcfg_get_ipaddr_or_netmask(const char *devname, int id, __u32 *ip)
{
	struct ifreq ifr;
	struct sockaddr_in *sin = (void *) &ifr.ifr_ifru.ifru_addr;
	int fd;

	if (!devname || !ip) return 1;

	if ((fd = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		fprintf(stderr, "%d: %s\n", __LINE__, strerror(errno));
		return 1;
	}

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, devname, IFNAMSIZ);
	sin->sin_family = AF_INET;

	if (ioctl(fd, id, &ifr) == -1) {
		fprintf(stderr, "%s: %s\n", devname, strerror(errno));
		return 1;
	}

	*ip = sin->sin_addr.s_addr;

	return 0;
}
