/*
 * WCFG         An implementation of the wcfglib library.
 *		See wcfg_lib companion document.
 *
 *
 * Version:     0.1.0
 *
 * Authors:     Alexandra Aronov, <alexandraa@ti.com>
 *              Felix Rubinstein, <felixr@ti.com>
 *
 */

#ifndef __WCFG_NVRAM_H
#define __WCFG_NVRAM_H

#include <linux/config.h>
#include "iwlib.h"
#include "dda_wext_ioctl.h"
#include "dda_wpaauth_ioctl.h"

/* Each time when struct wcfg_storage_t is being changed
   one MUST change (increment) the value of WCFG_MAGIC*/ 
#define WCFG_MAGIC 0x12345694
#define CONFIG_MIPS_AR7WRD 1 // add by leijun
#if defined(CONFIG_MIPS_AR7WI) || defined(CONFIG_MIPS_AR7VWI) || defined(CONFIG_MIPS_AR7WRD)
	#define WCFG_BOOTLOADER "/proc/ticfg/env"
#elif CONFIG_MIPS_PUMAS
	#define WCFG_BOOTLOADER "/proc/avalanche/env"
#else
	#error Unknown board type.
#endif

#define WCFG_WLAN_HWADDR "WLAN_HWADDR"


struct wcfg_storage_t {

	/*linux specific staff*/
	__u8				hwaddr[ETH_ALEN];
	__u32				ipaddr, netmask, default_gw;
	
	/*basic data types*/

	/*standard*/
	__s8               		essid[IW_ESSID_MAX_SIZE+1];
	__u16             		rts;
	__u16             		frag;
	__u8             		retry_long, retry_short;
	__u32				lifetime_tx, lifetime_rx;
	tiap_param_txpower_t		txpow_a, txpow_g;
	
	/*private*/
	__u32              		replev; /* bit map */
	__u8               		priv_opt;
	__u16             		beacon_period;
	__u8               		dtim_period;
	tiap_param_bss_br_state_t    	bss_bridge_state;
	
	__u8               		rate_fallback;
	__u32             		radio_cal;
	__u16             		cal_interval;
	__u8               		energy_detect;
	__u16					pd_low_thresh;
	__u16					pd_high_thresh;
	__u32					aci_state_timeout;
	__u32					nb_state_timeout;
	__u32					aci_rate_thresh;
	__u32					aci_event_thresh;
	__u32					medium_busy_thresh;
	__u16             		hidden_ssid;
	__u8               		multi_domain_cap;
	__u8               		spectrum_mgmt;
	__u8               		legacy_sta;
	__s8               		local_pow_cons;
	__u8               		max_txpow_lev;
	__s8 				country_str[TIAP_PARAM_COUNTRY_STRING_LEN];
	__u8              		oper_state;
	__u8               		wone_num;
	__u8               		wone_timemod;
	__u8				radar_detect;
	__u16				txop_limit;
	__u8				bridge_mode;
	__u8				gpio_owner;

	/*complex data types*/
	tiap_param_auth_algo_t		auth_tbl[TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED];
	tiap_param_wd_t			wd_en, wd_rxfree, wd_beacons, wd_txstuck, wd_fcserrs, wd_aci, wd_nb;
	tiap_param_access_list_t	access_list;
	tiap_param_4x_t			_4x_fs, _4x_4x, _4x_cat, _4x_pb, _4x_mm;
	tiap_param_multi_ssid_t		multi_ssid;
	tiap_param_radio_t		radio;
	tiap_param_channel_rang		ch_rang_a, ch_rang_g;
	tiap_param_vhcf_t		vhcf;
	tiap_param_wone_t		oper_rate[TIAP_PARAM_WONE_DUAL_BAND], oper_add[TIAP_PARAM_WONE_DUAL_BAND],
					oper_ofdm[TIAP_PARAM_WONE_DUAL_BAND], ch[TIAP_PARAM_WONE_DUAL_BAND],
					band[TIAP_PARAM_WONE_DUAL_BAND], pbcc[TIAP_PARAM_WONE_DUAL_BAND],
					rate_adapt[TIAP_PARAM_WONE_DUAL_BAND], pream_mode[TIAP_PARAM_WONE_DUAL_BAND],
					slot_mode[TIAP_PARAM_WONE_DUAL_BAND], erp_mode[TIAP_PARAM_WONE_DUAL_BAND],
					erp_type[TIAP_PARAM_WONE_DUAL_BAND], erp_ti[TIAP_PARAM_WONE_DUAL_BAND],
					dot11_mode[TIAP_PARAM_WONE_DUAL_BAND], cwmin_mode[TIAP_PARAM_WONE_DUAL_BAND],
					usage_time[TIAP_PARAM_WONE_DUAL_BAND];
	
	tiap_param_statbl_t		sta_tbl[TIAP_PARAM_MAX_STATIONS];
	tiap_param_priv_reg_domain_t	priv_regdomain;
	tiap_param_cur_reg_domain_t	cur_regdomain[TIAP_PARAM_WONE_DUAL_BAND];
};

/**
 *	struct storage - stored in the persistent storage 
 *
 *	@magic - unique number to differ previously stored configuration.
 *	@defaults - defaults are set when set to 1, otherwise is set to 0.
 *	@ldb - local db, stores ioctls configuration.
 *	@wpa - stores wpa authenticator configuration.
 */
struct storage {
	unsigned int magic, defaults;
	struct wcfg_storage_t ldb;
	tiap_wpa_auth_t wpa; 
};

int wcfg_storage_read(struct storage *, const char *);
int wcfg_storage_write(const struct storage *, const char *);
void wcfg_storage_setdefaults(struct storage *);
void wcfg_storage_setdefaults_ldb(struct wcfg_storage_t *);
void wcfg_storage_setdefaults_wpa(tiap_wpa_auth_t *);

#endif
