/*
 * WCFG         An implementation of the wcfglib library.
 *		See wcfg_lib companion document.
 *
 *
 * Version:     0.1.0
 *
 * Authors:     Alexandra Aronov, <alexandraa@ti.com>
 *              Felix Rubinstein, <felixr@ti.com>
 *
 */

#ifndef __WCFGLIB_H
#define __WCFGLIB_H

#include <sys/types.h>
#include <netinet/if_ether.h>
#include <linux/if.h>
#include <limits.h>

#include "wcfg_storage.h"

/*number of seconds to wait to decide process is successfully started*/
#define WCFG_MAXWAIT 1
#define WCFG_MAXWPAWAIT 5
#define WCFG_MTDBLOCK "/dev/mtdblock/3"
#define WCFG_PATHPREFIX	"/var/tmp/"
#define WCFG_CFGFILE "/var/tmp/.wcfglib"
#define WCFG_INITCFGFILE "/var/tmp/.wcfglib_init"
#define WCFG_WPAPIDFILE "/var/tmp/.wcfgwpapid"
#define TIAP_SOCKADDR_WPAAUTH "/var/tmp/wpaauth"

#define WCFG_RADIO_PARAM_OVERRIDE_BMASK 0
#define WCFG_RADIO_PARAM_POWER_TABLE 1

#define ENABLE  1
#define DISABLE 0

#define MAX_REPLEV 27
#define MIN_REPLEV -27

typedef struct {
	char storage_file[_POSIX_NAME_MAX+_POSIX_PATH_MAX+1];
	struct storage storage;
	pid_t wpaauth_pid;
	char apmodule_name[_POSIX_NAME_MAX+1];
	char dev_name[IFNAMSIZ];
	char br_name[IFNAMSIZ];
	char fw_filename[_POSIX_NAME_MAX+_POSIX_PATH_MAX+1];
} wcfgd;

typedef struct {
	char storage_file[_POSIX_NAME_MAX+_POSIX_PATH_MAX+1];
	char wpaauth_file[_POSIX_NAME_MAX+_POSIX_PATH_MAX+1];
	char apmodule_file[_POSIX_NAME_MAX+_POSIX_PATH_MAX+1];
	char dev_name[IFNAMSIZ];
	char br_name[IFNAMSIZ];
	char fw_filename[_POSIX_NAME_MAX+_POSIX_PATH_MAX+1];
} wcfginit;

#define UPDATE_STORAGE  1
#define FROM_STORAGE	1

wcfgd *tiap_wcfg_init(wcfginit *);
int tiap_wcfg_light_deinit(const wcfgd *);
int tiap_wcfg_deinit(const wcfgd *);
wcfgd *tiap_wcfg_restart(const wcfgd *);
void tiap_wcfg_system_reboot(void);


/* set/get hardware address */
int tiap_wcfg_set_hwaddr(wcfgd *, const __u8 *);
int tiap_wcfg_get_hwaddr(const wcfgd *, __u8 *);

/* WE standard */
int tiap_wcfg_siwessid(wcfgd *, const char *, int);
int tiap_wcfg_giwessid(const wcfgd *, char *, int from_storage);
int tiap_wcfg_siwrts(wcfgd *, const __s32, int); /*Felix R.: Should it be __s32 ?*/
int tiap_wcfg_giwrts(const wcfgd *wd, __s32 *rts, int from_storage);
int tiap_wcfg_siwfrag(wcfgd *, const __s32, int);
int tiap_wcfg_giwfrag(const wcfgd *, __s32 *, int);
int tiap_wcfg_siwretry(wcfgd *, const __u32, const int, int);
int tiap_wcfg_giwretry(const wcfgd *, __u32 *, int, int from_storage);
int tiap_wcfg_siwtxpow(wcfgd *, const tiap_param_txpower_t *, int);
	
/* Private */
int tiap_wcfg_set_defaults(wcfgd *);

/*basic*/
int tiap_wcfg_set_basic(wcfgd *, const __u32, void *,int);
int tiap_wcfg_get_basic(wcfgd *, const __u32, void *, int from_storage);
int tiap_wcfg_giwtxpow(const wcfgd *, tiap_param_txpower_t *, int from_storage);

/*complex*/
int tiap_wcfg_set_vhcf(const wcfgd *, const tiap_param_vhcf_t *, const int);
int tiap_wcfg_get_vhcf(const wcfgd *, tiap_param_vhcf_t *, int from_storage);

int tiap_wcfg_set_wone(wcfgd *, const tiap_param_wone_t *, const int);
int tiap_wcfg_get_wone(const wcfgd *, tiap_param_wone_t *, int from_storage);

int tiap_wcfg_set_4x(wcfgd *wd, const tiap_param_4x_t *_4x, int idx, const int);
int tiap_wcfg_get_4x(const wcfgd *wd, tiap_param_4x_t *_4x, int from_storage);
int tiap_wcfg_get_sta_table(wcfgd *, tiap_param_statbl_t *);
int tiap_wcfg_get_access_list(wcfgd *wd, tiap_param_access_list_t *tbl, int from_storage);
int tiap_wcfg_set_access_list(wcfgd *wd, const tiap_param_access_list_t *tbl);
int tiap_wcfg_set_auth_type(const wcfgd *wd, const tiap_param_auth_algo_t *tbl);
int tiap_wcfg_get_auth_type(const wcfgd *wd, tiap_param_auth_algo_t *tbl, int from_storage);
int tiap_wcfg_set_multi_ssid(wcfgd *, const tiap_param_multi_ssid_t *, int);
int tiap_wcfg_get_multi_ssid(const wcfgd *wd, tiap_param_multi_ssid_t *multi_ssid, int from_storage);
int tiap_wcfg_set_cfg_pow_tbl(wcfgd *, const tiap_param_radio_t *, int update_storage);
int tiap_wcfg_set_wep_default_key_id(wcfgd *wd, __u8 key_id);
void tiap_wcfg_get_wep_default_key_id(wcfgd *, __u8 *);
int tiap_wcfg_set_def_wep_key(wcfgd *, int, tiap_wpa_wep_key_t *);
int tiap_wcfg_get_def_wep_key(wcfgd *, int, tiap_wpa_wep_key_t *);
int tiap_wcfg_set_psk_passphrase_mode(wcfgd *wd, int value);
int tiap_wcfg_set_psk_passwd(wcfgd *wd, __s8 *passwd, __u8 passwd_len);
void tiap_wcfg_get_psk_passwd(wcfgd *wd, __s8 *passwd, __u8 *passwd_len);
int tiap_wcfg_set_default_cipher(wcfgd *wd, __u32 cipher);
int tiap_wcfg_set_gtk_interval(wcfgd *wd, __u32 intvl);
void tiap_wcfg_get_default_cipher(wcfgd *wd, __u32 *cipher);
void tiap_wcfg_get_radius_port(wcfgd *wd, __u16 *port);
void tiap_wcfg_get_radius_ipaddr(wcfgd *wd, __u32 *ipaddr);
void tiap_wcfg_get_gtk_interval(wcfgd *wd, __u32 *intvl);
void tiap_wcfg_get_psk_passphrase_mode(wcfgd *wd, int *value);
int tiap_wcfg_get_priv_regdomain(wcfgd *, tiap_param_priv_reg_domain_t *, int from_storage);
int tiap_wcfg_set_dot11mode(wcfgd *, int, int, int);
int tiap_wcfg_set_wd(const wcfgd *wd, const tiap_param_wd_t *w, int);
int tiap_wcfg_get_interf_wd_config(const wcfgd *wd, tiap_param_interference_wd_t *param, int from_storage);
int tiap_wcfg_set_interf_wd_config(wcfgd *wd, tiap_param_interference_wd_t *param, int update_storage);
int tiap_wcfg_get_priv_net_stats(wcfgd *, tiap_param_net_stats *);

void  print_mac(const char *mac);
int tiap_wcfg_get_cur_regdomain(wcfgd *, tiap_param_cur_reg_domain_t *, int from_storage);
int tiap_wcfg_set_priv_regdomain(wcfgd *, tiap_param_priv_reg_domain_t *, const int);
int tiap_wcfg_set_ch_range(const wcfgd *, const tiap_param_channel_rang *);
int tiap_wcfg_get_ch_range(const wcfgd *, tiap_param_channel_rang *, int from_storage);
int tiap_wcfg_set_cur_regdomain(wcfgd *, tiap_param_cur_reg_domain_t *, const int);

int tiap_wcfg_set_radius_port(wcfgd *, __u16);
int tiap_wcfg_set_radius_ipaddr(wcfgd *, __u32);
int tiap_wcfg_set_radius_secret(wcfgd *wd, __s8 *, __u8);
void tiap_wcfg_get_radius_secret(wcfgd *wd, __s8 *, __u8 *);

int tiap_wcfg_set_dual_ap(wcfgd *wd, __u8 wone_nr);
int tiap_wcfg_set_gpio_owner(wcfgd *, __u8);

#define TIAP_PARAM_BURST_LIMIT 100

#endif
