/* LICENSE_CODE JUNGO */

#ifndef _AUTH1X_H_
#define _AUTH1X_H_

#include "port_ctl.h"

/* EAPoL state machine params */

/* Params that are read (only) by the state machines.
 * TODO: they should be stored in rg_conf when SNMP support is implemented.
 */
typedef struct {
    /* authenticator PAE state machine constants from 802.1X-2001
     * section 8.5.4.1.2 */
    int quiet_period;
    int re_auth_max;
    int tx_period;

    /* authenticator Key Transmit state machine constants from 802.1X-2001 
     * section 8.5.5.1.2 */
    int key_tx_enabled;

    /* Reauthentication Timer state machine constants from 802.1X-2001 
     * section 8.5.7.1 */
    int re_auth_period;
    int re_auth_enabled;
    
    /* Backend Authentication state machine constants from 802.1X-2001 
     * section 8.5.8.1.2 */
    int supp_timeout;
    int server_timeout;
    int max_req;
} auth1x_params_t;

/* The following vars are set by sources external to state machines and refer
 * to all connections of interface */
typedef struct {
    /* global vars from 802.1X-2001 section 8.5.2.2 */
    int initialize; /* set to non-zero to initialize all state machines */
    auth1x_auth_controlled_port_control_t port_control; /* required port mode */
    int port_enabled;

    /* Controlled Directions state machine vars from 802.1X-2001 
     * section 8.5.9.1 */
    /* required controlled direction */
    auth1x_admin_controlled_direction_t admin_controlled_directions;
    int bridge_detected;
} auth1x_glob_ext_vars_t;

/* main_task interface */

#define WEP_KEYS_NUM 4

typedef enum {
    WEP_TYPE_DISABLED = 0,
    WEP_TYPE_FIXED = 1,
    WEP_TYPE_MIXED = 2,
    WEP_TYPE_DYNAMIC = 3,
} wep_type_t;

typedef struct {
    u8 data[13];
    int sz;
} wep_key_t;

typedef struct wep_keys_t {
    wep_key_t key[WEP_KEYS_NUM];
    int tx_key;
    wep_type_t type; 
    int timeout;
} wep_keys_t;

typedef enum {
    EMI_NONE = 0,
    EMI_RADIUS = 1,
    EMI_MD5 = 2,
    EMI_TLS = 3,
    EMI_TTLS = 4,
} eap_method_id_t;

typedef struct auth_8021x_callbacks_t {
    /* Look up the client specified by id and id_len.
     * If not found - return EMI_NONE, otherwise return EAP method id and set
     * *eap_method_params to its client parameters.
     */
    eap_method_id_t (*client_get)(u8 *id, int id_len, void **eap_method_params);

    /* Called when a new association is sensed. can be null. */
    void (*new_assoc)(void *context, ether_mac_t *mac);

    /* Called when an EAPOL-Start packet is received from a station */
    void (*eapol_start_received)(void *context, ether_mac_t *mac);
} auth1x_port_cb_t;

typedef struct {
    void *dev_id;
    int is_wireless;
    ether_mac_t mac;
    int mtu;
    auth1x_params_t params;
    auth1x_glob_ext_vars_t glob_ext_vars;
    wep_keys_t wep_keys;
    int packet_blocking_enabled;
} auth1x_port_params_t;

typedef struct auth1x_port_t auth1x_port_t;

/* Called on each device that should require
 * 802.1x authentication. returns context on success, NULL otherwise.
 */
auth1x_port_t *auth1x_port_open(auth1x_port_params_t *params,
    auth1x_port_cb_t *cb, auth1x_pc_ops_t *port_ctl_ops, void *context);

/* auth1x_port_reconf() is called whenever any of its params are changed.
 * Note that dev_id and is_wireless fields should not change and are not
 * checked for changes.
 */
void auth1x_port_reconf(auth1x_port_t *auth1x_port,
    auth1x_port_params_t *params);

/* Called on each device that should require no
 * 802.1x authentication.
 */
void auth1x_port_close(auth1x_port_t *auth1x_port);

/* Get mac of auth1x_port_t authenticator */
ether_mac_t *auth1x_port_mac_get(auth1x_port_t *auth1x_port);

#endif

