/* LICENSE_CODE JUNGO */

#ifndef _AUTH1X_CONN_H_
#define _AUTH1X_CONN_H_

#include <net_utils.h>

#include "auth1x.h"
#include "eapol.h"

typedef struct auth1x_conn_t auth1x_conn_t;

typedef struct {
    /* eapol_key_rcv() is called (if not NULL) whenever an EAPoL-Key packet is
     * received from the supplicant. is_encrypted indicates whether the packet 
     * was received encrypted or in the clear. The callback function should copy
     * the contents of EAPoL packet since it may be freed by the caller once the
     * call returns.
     */ 
    void (*eapol_key_rcv)(void *context, eapol_pkt_t *eapol_key,
	int is_encrypted);
 
    /* auth_end() is called in the end of the authentication process.
     * Note that authentication is usually done periodically, so this callback
     * may be called more than once per connection lifetime. is_authenticated
     * is set to non-zero if authentication was successfull, 0 otherwise. If
     * the authentication method used does not provides pmk - pmk will be
     * NULL.  The callback function should copy the contents of pmk since it
     * may be freed by the caller once the call returns.
     */ 
    void (*auth_end)(void *context, auth1x_conn_t *conn, int is_authenticated,
	u8 *pmk, int pmk_len);

    /* conn_term() is called whenever a connection should be closed 
     * the closing itself will be handled by the component who opened
     * the connection, either WEP or WPA */
    void (*conn_term)(void *context, auth1x_conn_t *conn);
} auth1x_conn_cb_t;
 
/* Starts a connection for supplicant supplicant_addr without starting
 * authentication session. This is used for sending EAPoL key.
 * dev_if_auth_8021x is the value returned from dev_if_auth_8021x_start() */
auth1x_conn_t *auth1x_conn_open(auth1x_port_t *port,
    ether_mac_t *supplicant_addr, auth1x_conn_cb_t *cb, void *context);

/* start authentication on an existing connection opend by auth1x_conn_open() */
void auth1x_conn_auth_start(auth1x_conn_t *conn);

/* Terminates the authentication session. auth1x_conn is the value returned
 * from auth1x_conn_open() */
void auth1x_conn_close(auth1x_conn_t *auth1x_conn);

/* Get next auth1x_conn_t in conn_list, NULL means get first */
auth1x_conn_t *auth1x_conn_get_next(auth1x_port_t *port, 
    auth1x_conn_t *auth1x_conn); 

/* Get pmk of auth1x_conn_t */
void auth1x_conn_pmk_get(auth1x_conn_t *auth1x_conn, u8 **pmk, int *pmk_len);

/* Get mac of auth1x_conn_t supplicant */
ether_mac_t *auth1x_conn_mac_get(auth1x_conn_t *auth1x_conn);

void auth1x_conn_eapol_pkt_send(auth1x_conn_t *c, eapol_pkt_t *pkt, 
    int encrypt);

#endif

