/* LICENSE_CODE JUNGO */

#ifndef _EAP_H_
#define _EAP_H_

#include <rg_def.h>
#include "auth1x.h"

/* RFC 2284 values */
typedef enum {
    EAP_REQUEST = 1,
    EAP_RESPONSE = 2,
    EAP_SUCCESS = 3,
    EAP_FAILURE = 4,
} eap_code_t;

typedef enum {
    EAP_NONE = 0,
    EAP_IDENTITY = 1,
    EAP_NOTIFICATION = 2,
    EAP_NAK = 3,
    EAP_MD5_CHALLENGE = 4,
    EAP_OTP = 5,
    EAP_GENERIC_TOKEN_CARD = 6,
    EAP_TLS = 13,
    EAP_TTLS = 21,
} eap_type_t;

typedef struct {
    u8 code;
    u8 id;
    u16 len; /* of the entire eap packet, i.e. including header and data */
} __attribute__ ((packed)) eap_pkt_hdr_t;

typedef struct {
    eap_pkt_hdr_t hdr;
    u8 type;
    u8 *data;
} eap_pkt_t;

/* Function that will be called by the eap method when either a request or
 * success/fail packet is to be sent to the supplicant.
 */
void eap_method_to_supp(void *context, eap_pkt_t *resp);

/* Update state machine */
void eap_method_sm_update(void *context);

/* Function that will be called by the eap method to get current max EAP packet
 * length, as derrived from device MTU. It is used to decide if fragmentation
 * is needed. */
int eap_max_len_get(void *context);

/* Function that will be called by the eap method when authentication was
 * successful and keys are available. The provided keys may be used to change
 * link cipher in wireless devices.
 */
void eap_method_master_key(void *context, u8 *key, int key_len);

eap_method_id_t eap_client_get(void *context, u8 *id, int id_len,
    void **eap_method_params);

typedef struct {
    eap_method_id_t id;

    /* open() is called to initialize with eap-specific parameters and context
     * that should be passed to eap_method_to_supp(). Returns eap context on
     * success, NULL on error */
    void *(*open)(void *params, void *context);
    
    /* close() is called to release all used resources */
    void (*close)(void *eap_context);
    
    /* supp_to_method() is called by the state machine when a response or
     * identity packet was received from the supplicant. resp is freed by the
     * caller (state machine) */
    void (*supp_to_method)(void *eap_context, eap_pkt_t *resp);
} eap_method_t;

#define GETSHORT(s, cp) do { \
	(s) = *(cp)++ << 8; \
	(s) |= *(cp)++; \
} while (0)

#define PUTSHORT(s, cp) do { \
	*(cp)++ = (u_char) ((s) >> 8); \
	*(cp)++ = (u_char) (s); \
} while (0) 

#define GETLONG(l, cp) do { \
	(l) = *(cp)++ << 8; \
	(l) |= *(cp)++; (l) <<= 8; \
	(l) |= *(cp)++; (l) <<= 8; \
	(l) |= *(cp)++; \
} while (0) 

#define PUTLONG(l, cp) do { \
	*(cp)++ = (u_char) ((l) >> 24); \
	*(cp)++ = (u_char) ((l) >> 16); \
	*(cp)++ = (u_char) ((l) >> 8); \
	*(cp)++ = (u_char) (l); \
} while (0) 

#endif

