#ifndef _WEP_PORT_H_
#define _WEP_PORT_H_

#include "auth1x.h"
#include "eapol.h"

typedef struct wep_port_cb_t {
    /* Look up the client specified by id and id_len.
     * If not found - return EMI_NONE, otherwise return EAP method id and set
     * *eap_method_params to its client parameters.
     */
    eap_method_id_t (*client_get)(u8 *id, int id_len, void **eap_method_params);

    /* Wireless control callbacks */

    /* Install a key to the HW into slot key_id (0-3). */
    int (*wlan_ap_set_wep_key)(void *ctx, int key_id, u8 *key, int key_len);

    /* Set the active key - the key used from transmitting. */
    int (*wlan_ap_set_tx_wep_key)(void *ctx, int key_id);
} wep_port_cb_t;

typedef struct wep_port_t wep_port_t;

wep_port_t *wep_port_open(auth1x_port_params_t *params, wep_port_cb_t *cb,
    auth1x_pc_ops_t *port_ctl_ops, void *ctx);

/* XXX - auth1x_port_close is not implemented correctly. all conns are deleted
 * by the auth1x_port_close() this should be checked.
 */
void wep_port_close(wep_port_t *t);

void wep_port_reconf(wep_port_t *t, auth1x_port_params_t *params);

wep_keys_t *wep_port_keys_get(wep_port_t *dev);

/* XXX - should move to the core 802.1X */
eapol_pkt_t *eap_rekey_pkt_create(ether_mac_t *supp_addr, 
    ether_mac_t *auth_addr, u8 *key, int key_len, int key_index, int unicast,
    u8 *pmk, int pmk_len);

#endif
