/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************/
/*																			*/
/*		MODULE:	mgmtTest.c													*/
/*		PURPOSE:	Authorization/association state machine implementation	*/
/*																			*/
/****************************************************************************/
#include "pform_types.h"
#include "rt_os_vxw.h"
#include "utilsReplvl.h"
#include "utilsTcb.h"
#include "utils.h"
#include "wpaTst.h"
#include "memMngr.h"
#include "802_11Defs.h"
#include "bssbridge.h"
#include "mic.h"
#include "wpaIfc.h"
#include <rg_error.h>
#include <802.1x/wep_port.h>
#include <radius/radiusct.h>
#include <802.1x/eap_radius.h>
#include "mgmtSm.h"
#include "tx.h"
#include "mic.h"
#include "public_security_keys.h"

micHandle_t	tstMicHandle;

static char		tstEapolBuffer[2312];

static void buildHeader(UINT32 staMac, dot11_mgmtHeader_T *hdrP);
static void micTest(void);


UINT32 L, R;

//#define ROL32( A, n ) \
// ( ((A) << (n)) | ( ((A) >> (32-(n)))  & ( (1UL << (n)) - 1 ) ) )

#define ROL32( A, n ) \
 ( ((A) << (n)) | ( ((A) >> (32-(n)))  & ( (1UL << (n)) - 1 ) ) )

#define ROR32( A, n ) ROL32( (A), 32-(n) )

/****************************************************
*			Implementation							*
*****************************************************/


void wdrv_wpaTstAssocStation(UINT8 SA, BOOL rsn, UINT8 rsnIeLen, UINT8 rsnIeValue)
{
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	int i, rc;
	char mac[MAC_ADDR_LEN];
	UINT16 index;
	UINT16 listenInterval = 0x5555;
	dot11_SSID_T	ssidTmp;
	char ssidBuf[50];
	dot11_supRates_T ratesTmp;
	UINT16 capabilities;
	dot11_rsne_T	rsnIe;

	wdrv_dbGetApCapabilities(0, &capabilities);
	wdrv_dbGetSsid(ssidBuf);
	wdrv_dbGetStationConfigOperationalRateSet(0, ratesTmp.rates, &ratesTmp.hdr.eleLen);


	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN + FIX_FIELD_LEN + 2 + strlen(ssidBuf) + 2 
						+ ratesTmp.hdr.eleLen; 

	if (rsn)
		totalLen += 2 + rsnIeLen;		

	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	(char *)hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;

	/* first, reset the buffer. */
	memset((char *)hdrP, 0, totalLen + 30);
	
	buildHeader(SA, hdrP);

	hdrP->fc |= DOT11_FC_ASSOC_REQ;
	
	BD->length += sizeof(dot11_mgmtHeader_T);	

	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	wlan_memMngrFillField(&dataP, (char *)&capabilities, FIX_FIELD_LEN, &BD);

	wlan_memMngrFillField(&dataP, (char *)&listenInterval, FIX_FIELD_LEN, &BD);

	ssidTmp.hdr.eleId = DOT11_SSID_ELE_ID;
	ssidTmp.hdr.eleLen = strlen(ssidBuf);
	memcpy(ssidTmp.serviceSetId, ssidBuf, ssidTmp.hdr.eleLen);
	wlan_memMngrFillField(&dataP, (char *)&ssidTmp, ssidTmp.hdr.eleLen + 2, &BD);

	ratesTmp.hdr.eleId = DOT11_SUPPORTED_RATES_ELE_ID;
	wlan_memMngrFillField(&dataP, (char *)&ratesTmp, ratesTmp.hdr.eleLen + 2, &BD);

	if (rsn)	/* We have to build RSN Ie too */
	{
		memset(&rsnIe, 0, sizeof(dot11_rsne_T));

		rsnIe.hdr.eleId = 0xDD;
		rsnIe.hdr.eleLen = rsnIeLen;
		memset(rsnIe.rsne, rsnIeValue, rsnIeLen);
		rsnIe.rsne[0] = 0x00;
		rsnIe.rsne[1] = 0x50;
		rsnIe.rsne[2] = 0xF2;

		wlan_memMngrFillField(&dataP, (char *)&rsnIe, rsnIe.hdr.eleLen + 2, &BD);
	}

	memset(mac, SA, MAC_ADDR_LEN);
	wdrv_dbFindStaIndex(mac, &index);

	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	rc = wdrv_mgmtForwardMgmtFrame(MSDUPtr, index);		/* always send with index 0.*/
	if (rc != OK)
		wdrv_utilsReport(RL_TEST, PRI_INFORMATION, "WDRV_TEST:Failed to send auth Frame\n");

	return;
}

void wdrv_wpaTstReAssocStation(UINT8 SA, BOOL rsn, UINT8 rsnIeLen, UINT8 rsnIeValue)
{
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	int i, rc;
	char mac[MAC_ADDR_LEN];
	UINT16 index;
	UINT16 listenInterval = 0x5555;
	dot11_SSID_T	ssidTmp;
	char ssidBuf[50];
	dot11_supRates_T ratesTmp;
	UINT16 capabilities;
	dot11_rsne_T	rsnIe;
	char			currentAP[6] = {0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC};

	wdrv_dbGetApCapabilities(0, &capabilities);
	wdrv_dbGetSsid(ssidBuf);
	wdrv_dbGetStationConfigOperationalRateSet(0, ratesTmp.rates, &ratesTmp.hdr.eleLen);

	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN + FIX_FIELD_LEN + 2 + strlen(ssidBuf) + 2 
						+ ratesTmp.hdr.eleLen + 6; 

	if (rsn)
		totalLen += 2 + rsnIeLen;	

	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	(char *)hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;

	/* first, reset the buffer. */
	memset((char *)hdrP, 0, totalLen + 30);
	
	buildHeader(SA, hdrP);

	hdrP->fc |= DOT11_FC_REASSOC_REQ;
	
	BD->length += sizeof(dot11_mgmtHeader_T);	

	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	wlan_memMngrFillField(&dataP, (char *)&capabilities, FIX_FIELD_LEN, &BD);

	wlan_memMngrFillField(&dataP, (char *)&listenInterval, FIX_FIELD_LEN, &BD);

	wlan_memMngrFillField(&dataP, (char *)&currentAP, 6, &BD);

	ssidTmp.hdr.eleId = DOT11_SSID_ELE_ID;
	ssidTmp.hdr.eleLen = strlen(ssidBuf);
	memcpy(ssidTmp.serviceSetId, ssidBuf, ssidTmp.hdr.eleLen);
	wlan_memMngrFillField(&dataP, (char *)&ssidTmp, ssidTmp.hdr.eleLen + 2, &BD);

	ratesTmp.hdr.eleId = DOT11_SUPPORTED_RATES_ELE_ID;
	wlan_memMngrFillField(&dataP, (char *)&ratesTmp, ratesTmp.hdr.eleLen + 2, &BD);

	if (rsn)	/* We have to build RSN Ie too */
	{
		memset(&rsnIe, 0, sizeof(dot11_rsne_T));

		rsnIe.hdr.eleId = 0xDD;
		rsnIe.hdr.eleLen = rsnIeLen;
		memset(rsnIe.rsne, rsnIeValue, rsnIeLen);
		rsnIe.rsne[0] = 0x00;
		rsnIe.rsne[1] = 0x50;
		rsnIe.rsne[2] = 0xF2;

		wlan_memMngrFillField(&dataP, (char *)&rsnIe, rsnIe.hdr.eleLen + 2, &BD);
	}

	memset(mac, SA, MAC_ADDR_LEN);
	wdrv_dbFindStaIndex(mac, &index);

	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	rc = wdrv_mgmtForwardMgmtFrame(MSDUPtr, index);		/* always send with index 0.*/
	if (rc != OK)
		wdrv_utilsReport(RL_TEST, PRI_INFORMATION, "WDRV_TEST:Failed to send auth Frame\n");

	return;
}


static void micTest(void)
{
	UINT8		tstMicKey[MICHAEL_KEY_LEN];
	UINT8		micInput[7] = "Michael";
	UINT8		micOutput[MIC_LEN];


	memset(micOutput,0, MIC_LEN);	
	memset(tstMicKey, 0, MICHAEL_KEY_LEN);
	
	mic_init(&tstMicHandle);

	mic_setKey(&tstMicHandle, tstMicKey);
	mic_addBuffer(&tstMicHandle, micInput, 0);
	mic_getMic(&tstMicHandle, micOutput);
	rg_error(LPANIC, "WPA Test MIC test result:	0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X \n", micOutput[0], micOutput[1], micOutput[2], micOutput[3], micOutput[4], micOutput[5], micOutput[6], micOutput[7]);

	mic_setKey(&tstMicHandle, micOutput);
	mic_addBuffer(&tstMicHandle, micInput, 1);
	mic_getMic(&tstMicHandle, micOutput);
	rg_error(LPANIC, "WPA Test MIC test result:	0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X \n", micOutput[0], micOutput[1], micOutput[2], micOutput[3], micOutput[4], micOutput[5], micOutput[6], micOutput[7]);

	mic_setKey(&tstMicHandle, micOutput);
	mic_addBuffer(&tstMicHandle, micInput, 2);
	mic_getMic(&tstMicHandle, micOutput);
	rg_error(LPANIC, "WPA Test MIC test result:	0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X \n", micOutput[0], micOutput[1], micOutput[2], micOutput[3], micOutput[4], micOutput[5], micOutput[6], micOutput[7]);

	mic_setKey(&tstMicHandle, micOutput);
	mic_addBuffer(&tstMicHandle, micInput, 3);
	mic_getMic(&tstMicHandle, micOutput);
	rg_error(LPANIC, "WPA Test MIC test result:	0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X \n", micOutput[0], micOutput[1], micOutput[2], micOutput[3], micOutput[4], micOutput[5], micOutput[6], micOutput[7]);

	mic_setKey(&tstMicHandle, micOutput);
	mic_addBuffer(&tstMicHandle, micInput, 4);
	mic_getMic(&tstMicHandle, micOutput);
	rg_error(LPANIC, "WPA Test MIC test result:	0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X \n", micOutput[0], micOutput[1], micOutput[2], micOutput[3], micOutput[4], micOutput[5], micOutput[6], micOutput[7]);

	mic_setKey(&tstMicHandle, micOutput);
	mic_addBuffer(&tstMicHandle, micInput, 7);
	mic_getMic(&tstMicHandle, micOutput);
	rg_error(LPANIC, "WPA Test MIC test result:	0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X 0x%X \n", micOutput[0], micOutput[1], micOutput[2], micOutput[3], micOutput[4], micOutput[5], micOutput[6], micOutput[7]);
} 

void wdrv_wpaTstVector(void)
{
	micTest();
}

static void buildHeader(UINT32 staMac, dot11_mgmtHeader_T *hdrP)
{
	char	mac[MAC_ADDR_LEN];
	UINT16 type;

	/* build frame control first */
	hdrP->fc = 0;
	
	hdrP->fc |= DOT11_FC_PROT_VERSION;

	hdrP->fc |= DOT11_FC_TYPE_MGMT;

	hdrP->fc |= (FALSE << DOT11_FC_TO_DS_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_FROM_DS_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_MORE_FRAG_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_RETRY_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_PWR_MGMT_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_MORE_DATA_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_WEP_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_ORDER_SHIFT);

	memset(hdrP->SA.addr, staMac, MAC_ADDR_LEN);

	wdrv_dbGetOperationMacAddress(mac);
	memcpy(hdrP->DA.addr, mac, MAC_ADDR_LEN);

	wdrv_dbGetBssid(mac);
	memcpy(hdrP->BSSID.addr, mac, MAC_ADDR_LEN);
}


