/* LICENSE_CODE JUNGO */

#ifndef __ESYSCALL_H_
#define __ESYSCALL_H_

#define OS_INCLUDE_SOCKET
#include <os_includes.h>
#include <sys/socket.h>

#define ERECV 0x1
#define ERECVFROM 0x2
#define ERECVMSG 0x4
#define ERECVALL (ERECV|ERECVFROM|ERECVMSG)
#define ESEND 0x8
#define ESENDTO 0x10
#define ESENDMSG 0x20
#define ESENDALL (ESEND|ESENDTO|ESENDMSG)


/* The callback receives the operation return value (as returned from
 * recv/send/...) and the supplied context.
 */
typedef void (*esyscall_event_t)(int ret, void *o);

/* System call functions, based on event_loop. In addition to the standard
 * arguments, the caller must supply a callback function (cb), context (o) and
 * an optional timeout (ms_timeout != 0.0). The supplied callback function is
 * called  when the operation was finished or when the specified timeout
 * expired. Note that the callback function may be NULL.
 */
void erecv(int s, void *buf, int len, unsigned int flags, esyscall_event_t cb,
    void *o, double ms_timeout);
void erecvfrom(int s, void *buf, int len, unsigned int flags,
    struct sockaddr *from, int *fromlen, esyscall_event_t cb, void *o,
    double ms_timeout);
void esend(int s, const void *msg, int len, unsigned int flags,
    esyscall_event_t cb, void *o, double ms_timeout);
void esendto(int s, const void *msg, int len, unsigned int flags,
    const struct sockaddr *to, int tolen, esyscall_event_t cb, void *o,
    double ms_timeout);
void erecvmsg(int s, struct msghdr *msg, unsigned int flags,
    esyscall_event_t cb, void *o, double ms_timeout);
void esendmsg(int s, const struct msghdr *msg, unsigned int flags,
    esyscall_event_t cb, void *o, double ms_timeout);

/* Cancel only the oldest event with the specified fd and op direction.
 * op should be any combination of ERECV ERECVFROM ERECVMSG ESEND ESENDTO
 * ESENDMSG.
 * If an event of the specified fd and op was found - return the context which
 * was passed to erecv* and esend* in ret_ctx if its not NULL.
 * Return 0 on operation canceled, -1 if no operation found.
 */
int esyscall_cancel(int fd, int op, void **ret_ctx);

/* Cancel only one oldest event according to callback and context.
 * Return 0 on operation canceled, -1 if no operation found.
 */
int esyscall_cancel_by_cb(esyscall_event_t cb, void *o);

#endif

