/* LICENSE_CODE JUNGO */

#ifndef _EVENT_H_
#define _EVENT_H_

#define OS_INCLUDE_TIME
#define OS_INCLUDE_PROCESS
#include <os_includes.h>
#include "rg_def.h"


extern int event_loop_quit;

typedef void (*etimer_func_t)(void *data);
typedef void (*efd_func_t)(int fd, void *data);
typedef void (*esignal_func_t)(int sig, void *data);
typedef void (*esigchild_func_t)(pid_t pid, void *data, int status);
typedef void (*free_t)(void *data);

double get_time_from_boot(void);
double get_time_ms(void);
double event_get_loop_now(void);
time_t event_loop_date_now_get(void);
void ms2tv(struct timeval *tv, double ms);
double tv2ms(struct timeval *tv);

/* Set a timer to be called every ms period */
void event_timer_set_period(double ms, etimer_func_t func, void *data);
/* Set a timer to be called once after ms*/
void event_timer_set(double ms, etimer_func_t func, void *data);
/* Cancel a timer callback */
void event_timer_del(etimer_func_t func, void *data);
/* Check if timer exist on the list. Return 0 if not exit */
int event_timer_exist(etimer_func_t func, void *data);
/* Return the remaining time in milliseconds until the timer will expire. 
 * Return -1 if the timer doesn't exist.
 */
double event_timer_left(etimer_func_t func, void *data);
/* enable accurate timers - but consume 100% cpu for this purpose.
 * useful for pkt_gen.
 */
void event_timer_set_exact(int enable);
    
/* initialization/deinit of the library. Must be called before any other
 * library functions
 */
void event_loop_init(void);
void event_loop_uninit(void);
/* Start the event loop -- should called _exactly_ once per process */
int event_loop(void);
/* Set/cancel callback for fd events */
void event_fd_set(int fd, int type, efd_func_t func, void *data);
void event_fd_clr(int fd, int type);
/* check if ready for operation */
int event_fd_ready(int fd, int type);
/* Set/cancel callback for signal events */
void event_signal_set_flags(int sig, esignal_func_t func, void *data,
    int flags);
void event_signal_set(int sig, esignal_func_t func, void *data);
void event_signal_del(int sig);

#ifndef CONFIG_RG_TARGET_VXWORKS
/* All process related stuff is not relevant for VxWorks. */

/* Register a callback for child exit. The callback must waitpid(pid). */
void event_sigchild_set(pid_t pid, esigchild_func_t func, void *data);
/* When kill != 0, kill process pid synchronously */
void event_sigchild_del(pid_t pid, int kill);
/* Blocking wait for a specific child. Return value 0 indicate we've waited
 * on the requested pid and its status is saved on ret_status.
 * Should be called between event_sigchild_disable() and 
 * event_sigchild_enable().
 */
int event_wait_for_child(pid_t pid, int *ret_status);
/* Stop sigchild signal. Zombies are not handeled */
void event_sigchild_disable(void);
/* Return Handling childs */
void event_sigchild_enable(void);
#endif

#define EFD_READ 1
#define EFD_WRITE 2
#define EFD_EXCEPT 4
#define EFD_ALL (EFD_READ|EFD_WRITE|EFD_EXCEPT)

#define MS_SECOND 1000
#define MS_MIN (MS_SECOND * 60)
#define MS_HOUR (MS_MIN * 60)
#define MS_DAY (MS_HOUR * 24)

#endif

