/* LICENSE_CODE JUNGO */

#ifndef _NET_UTILS_H_
#define _NET_UTILS_H_

#define OS_INCLUDE_INET
#define OS_INCLUDE_NETIF
#define OS_INCLUDE_ROUTE
#include <os_includes.h>

#include <rg_def.h>
#include "net_types.h"

#define BRCTL_GET_BRIDGES 1
#define PROC_NET_DEV "/proc/net/dev"
#define PROC_NET_ROUTE "/proc/net/route"

#define NEW_HOSTNAME "new-computer"

typedef enum {
    IFCFG_SDOWN = 0x0,
    IFCFG_SUP = 0x1, 
    IFCFG_SIPADDR = 0x2, 
    IFCFG_SNETMASK = 0x4,
    IFCFG_SPROMISC = 0x8,
    IFCFG_IS_EXT = 0x10,
} ifconfig_flag_t;
#define IFCFG_SALL (IFCFG_SUP | IFCFG_SIPADDR | IFCFG_SNETMASK)

typedef enum {
    URL_FTP = 1,
    URL_HTTP = 2,
    URL_HTTPS = 3,
    URL_TFTP = 4,
} url_transport_t;

/* URL structure is transport://user:pass@host:port/path?query */
typedef struct {
    url_transport_t transport;
    char *user;
    char *pass;
    char *host;
    unsigned short port;
    int has_port; /* Does the URL contain a port number */
    char *path;
    char *query;
} parsed_url_t;

extern ether_mac_t bcast_mac;

struct dev_if_t;

u32 get_if_ip(char *iface);
u32 get_if_broadcast(char *iface);
int get_if_hw_addr(char *iface, ether_mac_t *mac, int *family);
int get_if_mac(char *iface, ether_mac_t *mac);
int set_if_mac(char *iface, ether_mac_t *mac);
u32 invert_ip_addr(u32 ip_addr);
int route(u32 ip_addr, u32 gw, u32 genmask, char *dev, unsigned short flags,
    int metric, int is_add);
int ifconfig(char *dev, int alias_index, u32 ip_addr, u32 net_mask, u32 remote,
    ifconfig_flag_t flags, int metric, int mtu);
/* Get gateway IP for a specified destination */
struct in_addr get_dest_gw(struct in_addr dest);

int is_ip_format(char *ip_addr);

int is_mac_zero(ether_mac_t *mac);
char *mactoa(ether_mac_t *mac);
ether_mac_t *atomac(char *mac_str);
u32 ip_class_netmask(u32 ip);

/* Compare two subnets (subnet1 defined by 'ip1' and 'netmask1' and subnet2
 * defined by 'ip2' and 'netmask2'.
 * - Return -1 if there is no intersection between the subnets
 * - Return 0 if the subnets are equal
 * - Return 1 if subnet1 contains subnet2
 * - Return 2 if subnet2 contains subnet1
 */
int subnet_compare(u32 ip1, u32 netmask1, u32 ip2, u32 netmask2);

/* Check hostname validity, end points to the end of string, if it ends before
 * the NULL-termination, or NULL if not.
 */
int is_valid_host_name(char *host, char *end);
/* Check full domain name validity, end points to the end of string, if it ends
 * before the NULL-termination, or NULL if not.
 */
int is_valid_domain_name(char *name, char *end);

/* Encode (escape) a given 'url'. The result is allocated, and it's the
 * caller's responsibility to free it
 */
char *url_encode(char *url);

/* Parse 'url' by splitting it into tokens in the parsed_url_t struct 'h'.
 * A call to parsed_url_uninit() must be made when finishing using 'h'
 */
int parse_url(char *url, parsed_url_t *h);

/* Uninitialize a parsed_url_t struct (after it had been allocated by
 * parse_url())
 */
void parsed_url_uninit(parsed_url_t *h);

int netmask_to_bitcount(u32 netmask);
u32 bitcount_to_netmask(int bitcount);
int is_netmask_valid(u32 netmask);
int is_ip_in_subnet(u32 ip, u32 netmask, u32 subnet);
/* Calculate address range (start and end IP's), given an IP in 'ip' and a
 * netmask in 'netmask'
 */
void ip_range_start_get(struct in_addr ip, struct in_addr netmask,
    struct in_addr *start_ip);
void ip_range_end_get(struct in_addr ip, struct in_addr netmask,
    struct in_addr *end_ip);

typedef struct route_t {
    struct route_t *next;
    struct in_addr dest;
    struct in_addr gateway;
    struct in_addr netmask;
    int metric;
    int flags; /* Bit field */
    char interface[IFNAMSIZ];
    int ttl;/* Expiry time of TTL in seconds from boot time, 
             * 0 means infinity */ 
    int installed; /* 1 - route is installed */
} route_t;

/* Allocate a linked-list contains the Kernel route table in route_t nodes */
route_t *route_table_alloc(void);
/* Free route_t linked-lists */
void route_list_free(route_t **head);

/* Returns the route through which a connection to host will go */
route_t *route_to_host_get(struct in_addr host);

/* A generic open socket function */
int sockets_open(int domain, int type, int protocol);

/* A generic wrapper for ioctls to a network interface */
int get_if_data(char *iface, struct ifreq *ifr, int cmd);

/* A function that behaves exactly as inet_aton, except that it does strict
 * checks on the string's format (which is not done by glibc's inet_aton), and
 * zeroes 'inp' if an error occurs.
 * Return value is non-zero if success, zero otherwise.
 * If you are converting a static string (whose format cannot be wrong) and do
 * not need error checking - you can simply use inet_aton().
 */
int rg_inet_aton(char *cp, struct in_addr *inp);

/* Returns 1 if the MAC address is FF:FF:FF:FF:FF:FF, 0 otherwise */
int is_bcast_mac(ether_mac_t mac);

#endif

