/* LICENSE_CODE JUNGO */

#ifndef _PORT_CTL_H_
#define _PORT_CTL_H_

#include <net_utils.h>
#include "port_ctl_types.h"

/* interface to port control */

typedef struct auth1x_pc_t auth1x_pc_t;

/* Main receive packet handler; called whenever EAPOL packet is received in
 * the attached interface. buf points to packet buffer, len contains packet
 * len and context is the context supplied in the call to open(). is_encrypted 
 * is 1 in case the packet was sent encrypted or 0 in case it was sent in the 
 * clear. Packet buffer may be released after the function returns. */
typedef void (*auth1x_pkt_handler_t)(void *context, void *buf, int lenx, 
    int is_encrypted);

typedef struct {
    /* The following functions are mandatory. */

    /* Open a port control on device dev_id. auth1x_pkt_handler and context
     * are the main receive packet handler and it's context.
     * If auth1x_pkt_handler is NULL, no packet handling need to be done.
     * (This is needed for external port control).
     * The function returns NULL on failure */
    auth1x_pc_t *(*open)(void *dev_id,
       	auth1x_pkt_handler_t auth1x_pkt_handler, void *context);

    /* Close port control */
    void (*close)(auth1x_pc_t *t);

    /* send the specified EAPoL packet through the device. Packet should
     * contain ethernet header. Returns number of bytes sent, -1 on error. */
    int (*send_eapol)(auth1x_pc_t *t, void *buf, int len, int encrypt);

    /* Sets mode for the device. Returns 0 on success, 1 otherwise */
    int (*mode_set)(auth1x_pc_t *t, auth1x_mode_t *mode);

    /* Authorize mac on the device. Returns 0 on success, 1 otherwise */
    int (*mac_auth)(auth1x_pc_t *t, ether_mac_t *mac);

    /* Unauthorize mac on the device. Returns 0 on success, 1 otherwise */
    int (*mac_unauth)(auth1x_pc_t *t, ether_mac_t *mac);


    /* The following functions are needed only for wired implementation. They
     * may be NULL in wireless-only implementation. */

    /* Sets pass_single_unauth value of the device. Returns 0 on success, 1
     * otherwise */
    int (*pass_single_unauth_set)(auth1x_pc_t *t, int pass_single_unauth);

    /* Gets pass_single_unauth value of the device. Returns 0 on success, 1
     * otherwise */
    int (*pass_single_unauth_get)(auth1x_pc_t *t,
       	int *pass_single_unauth);


    /* The following functions are needed only for monitoring only. They
     * may be NULL if monitoring is not needed. */

    /* Gets mode for the device. Returns 0 on success, 1 otherwise */
    int (*mode_get)(auth1x_pc_t *t, auth1x_mode_t *mode);

    /* Returns in *mac, next authorized mac. If on input the mac is all zeros
     * then return the first mac in the list. A mac of all zeros on output
     * represents end of list. Returns 0 on success, 1 otherwise
     */
    int (*mac_get_next)(auth1x_pc_t *t, ether_mac_t *mac);

    /* Returns in *use_count the number of open port controls on the device.
     * Used to find if any other user has already running 802.1X on the
     * device. Returns 0 on success, 1 otherwise */
    int (*use_count_get)(auth1x_pc_t *t, int *use_count);

} auth1x_pc_ops_t;

#endif

