/* LICENSE_CODE JUNGO */

#ifndef _PVC_SCAN_H_
#define _PVC_SCAN_H_

#include <rg_def.h>
#include <rg_defines.h>
#include <main/dev_if.h>

/* VPI/VCI pair */
typedef struct {
    u16 itf;
    u16 vpi;
    u16 vci;
} vpi_vci_t;

/* PVC scanning states - found a pair or finished the scanning */
typedef enum {
    PVC_SCAN_INVALID = -1,
    PVC_SCAN_FINISHED = 0,
    PVC_SCAN_FOUND = 1,
} pvc_scan_state_t;

/* PVC scanning results */
typedef struct {
    vpi_vci_t vpi_vci;
    dev_if_type_t device_type; /* Device type to be used for the protocol */
    atm_encapsulation_t encapsulation;
    void *data; /* Specific protocol data */
} pvc_found_t;

/* PPPoE specific data */
typedef struct {
    atm_route_t route; /* Always ATM_ROUTE_BRIDGE */
    char *ac_name; /* Access Concentrator name */
    char *service_name; /* Service name */
} pppoe_data_t;

/* PPPoA specific data - not needed */

/* PVC Scan task context */
typedef struct {
    void *data;
} pvc_scan_t;

/* The callback may be called several times with status PVC_SCAN_FOUND when
 * something is found for a pair; and is always called once with
 * PVC_SCAN_FINISHED indicating that the scanning is complete
 */
typedef void (*pvc_scan_cb)(void *data, pvc_scan_state_t state,
    pvc_found_t *result);

/* Scan 'vpi_vcis' array of 'pairs' size. */
pvc_scan_t *pvc_scan_open(vpi_vci_t *vpi_vcis, int pairs, pvc_scan_cb cb,
    void *data);
void pvc_scan_close(pvc_scan_t *task_info);

#endif
