/* LICENSE_CODE JGPL */

#ifndef _RG_DEF_H_
#define _RG_DEF_H_

#include <rg_config.h>

#define OS_INCLUDE_STRING
#include <os_includes.h>

#include <rg_types.h>

#define MACRO_TO_STR2(A) #A
#define MACRO_TO_STR(A) MACRO_TO_STR2(A)

#ifdef CONFIG_BLUETOOTH
    #include "bt_defs.h"
#endif

#if defined(CONFIG_BLUETOOTH) || defined(CONFIG_HPNA) || defined(CONFIG_HRF)\
    || defined(CONFIG_80211B_FEATURE)
    #define HOME_NETWORKING
#endif

#ifndef MZERO
    #define MZERO(buf) memset(&(buf), 0, sizeof(buf))
#endif

#define MAX_ETH_RAW_FRAME_SIZE 4096

/* OpenRG common URLs and emails */
#define RG_PROD_STR "OpenRG"
#define JUNGO_DOMAIN "jungo.com"
#define JUNGO_SITE "www." JUNGO_DOMAIN
#define JUNGO_URL "http://" JUNGO_SITE
#define RMT_UPG_SITE "update." JUNGO_DOMAIN
#define RMT_UPG_URL "http://" RMT_UPG_SITE "/"
#define RMT_UPG_URL_FULL RMT_UPG_URL "openrg.rmt"
#define RG_SUPP_EMAIL "rg_support@" JUNGO_DOMAIN
#define RG_SALES_EMAIL "sales_rg@" JUNGO_DOMAIN
#define RG_ANON_USERNAME "anonymous"
#define RG_ANON_PWD "OpenRG@" JUNGO_DOMAIN

#define RG_LOG_CMD 0
#define FLASH_UPDATE_INTERVAL 60

#define MAX_LINE_SIZE 1024
#define MAX_VAR_NAME 80
#define MAX_CMD_LEN 300
#define MAX_PATH_LEN 100
#define MAX_QUERY_LEN 1024
#define MAC_IP_LEN 20
#define MAX_MAC_LEN 18
#define DOTTED_IP_LEN 16 	/* length of: xxx.xxx.xxx.xxx\0 */
#define MAX_PROC_CMD_FILENAME 32
#define MAX_ERROR_MSG_LEN MAX_LINE_SIZE
#define MAX_8021X_KEY_CHARS 27 /* 104 bit WEP key */

/* The following are not including '\0' */
#define MAX_WORKGROUP_NAME_LEN 15 
#define MAX_DOMAIN_NAME_LEN 255 
#define MAX_HOST_NAME_LEN 63
#define MAX_SSID_LEN 32

#define MAX_USERNAME_LEN 64
/* username "@" domain */
#define MAX_EMAIL_LEN (MAX_USERNAME_LEN+1+MAX_DOMAIN_NAME_LEN)
#define MAX_PASSWORD_LEN 64
#define MAX_FULLNAME_LEN 128
/* Windows allows use of upto 256 characters for PPTP client password */
#define MAX_SECRET_LEN 256

#define MAX_URL_LEN  \
    ( \
    sizeof("http://") - 1 + \
    MAX_USERNAME_LEN + \
    sizeof(":") - 1 + \
    MAX_PASSWORD_LEN + \
    sizeof("@") - 1 + \
    MAX_HOST_NAME_LEN + \
    sizeof(":65535/") - 1 + \
    MAX_PATH_LEN + \
    MAX_QUERY_LEN \
    )
    
#define MAX_PPP_USERNAME_LEN 100
#define MAX_PPP_PASSWORD_LEN 100

#define MAX_DHCPS_LEASES 256

#define MAX_ENCRYPTION_KEY_LEN 20
#define MAX_WIRELESS_AP_KEY_MAX 26

#define MAX_CERT_SUBJECT_LEN 255
#define MAX_X509_NAME_LEN 64
#define X509_OPENRG_NAME_PREFIX "ORname_"

enum
{
    DO_REBOOT=1,
    DO_DLY_REBOOT,
    DO_HALT,
    DO_UPDATE,
    DO_RCV_UPD_STATUS,
    DO_RESTORE_DEF,
    DO_RUN_FW,
    DO_FW_LOG_SET,
    DO_RUN_FW_FLUSH,
    DO_RUN_FW_CLEAN,
    DO_RESTART_SYSLOGD,
    DO_DHCP_ON,
    DO_DHCP_OFF,
    DO_DNS_UPDATE,
    DO_DNS_ON,
    DO_DNS_OFF,
    DO_UPDATE_NET_FILES,
    DO_NETDEV_UP,
    DO_NETDEV_DOWN,
#ifndef CONFIG_LIBIT
    DO_DHCPC_UPDATE_IP = 70,
    DO_DHCPC_UPDATE_GW,
#endif
#ifdef CONFIG_BLUETOOTH
    DO_BT_ON = 100,
    DO_BT_OFF,
#endif
#ifdef CONFIG_VOIP
    DO_OHPHONE_ON = 300,
    DO_OHPHONE_OFF,
#endif
#ifdef CONFIG_80211B_FEATURE
    DO_80211B_ON = 500,
    DO_80211B_OFF,
#endif
#ifdef CONFIG_USB_RNDIS
    DO_ON_USB_RNDIS_CONN_STATUS_CHANGE = 650,
#endif        
#ifdef CONFIG_PPP
    DO_ON_PPP_UP = 700,
    DO_ON_PPP_DOWN = 701,
    DO_ON_PPP_FINISH = 702,
    DO_ON_PPP_CONN_STATUS_CHANGE = 703,
    DO_ON_PROT_SWAP = 704,
#ifdef CONFIG_PPTPC
    DO_ON_PPP_PPTP_CLIENT_UP = 750,
    DO_ON_PPP_PPTP_CLIENT_DOWN = 751,
#endif
#endif    
    DO_FLASH_COMMIT = 800,
    DO_FLASH_COMMIT_NEEDED = 801,
    DO_RUN_DAEMON = 802,
#ifdef CONFIG_FREESWAN
    DO_IPSEC_START = 900,
    DO_IPSEC_STOP = 901,
    DO_IPSEC_CONN_UP_DOWN = 902,
    DO_RSASIGKEY_COMPLETE = 903,
#if defined(CONFIG_IPSEC_DEBUG) || defined(CONFIG_PLUTO_DEBUG)
    DO_IPSEC_LOG_SET = 904,
#endif
#endif
    DO_DEV_IF_DUMP = 999,/* used for "rg_sysinit send_int 999" */
};

#if !defined(_SEM_SEMUN_UNDEFINED)
       /* union semun is defined by including <sys/sem.h> */
#else
       /* according to X/OPEN we have to define it ourselves */
       union semun {
               int val;                    /* value for SETVAL */
               struct semid_ds *buf;       /* buffer for IPC_STAT, IPC_SET */
               unsigned short int *array;  /* array for GETALL, SETALL */
               struct seminfo *__buf;      /* buffer for IPC_INFO */
       };
#endif  

#include <rg_os.h>

#define RESOLV_CONF		"/etc/resolv.conf"
#define ENS_CONF		"/etc/ens/ens.conf"
#define ETC_RG_DEFAULT          "/etc/rg/rg.default" 
#define ETC_RG_CONF             "/etc/rg/rg.conf" 
#define PAP_SECRET_BASE		"/etc/ppp/pap-secrets"
#define CHAP_SECRET_BASE	"/etc/ppp/chap-secrets"
#define PPTP_SERV_SUFFIX	"pptp_srv"
#define PPTP_CLIENT_SUFFIX	"pptp_cli"
#define PPTPS_FIFO		"/var/run/pptps.fifo"
#define LOCAL_HOST		"localhost"

#define SYSTEM_LOG		0x1 
#define SYSTEM_DAEMON		0x2
#define SYSTEM_CLOSE_FDS	0x4

/* Path for the syslog files */
#define SYSLOG_FILE 		"/var/log/messages"
#ifdef CONFIG_VPN
#define SYSLOG_FILE_FREESWAN    "/var/log/freeswan"
#endif
#ifdef CONFIG_RG_FIREWALL
#define SYSLOG_FILE_FIREWALL 	"/var/log/firewall"
#else
#define SYSLOG_FILE_FIREWALL 	SYSLOG_FILE
#endif

#define MAX_BAD_REBOOT 3

#define PPTPS_MAX_CONNS 100

/* echo needed to keep the fw state alive, otherwise it is deleted after two
 * minutes idle time and the connection will be blocked.
 * The default values below will make ppp discover broken link in 30
 * seconds (5*6)
 */
#ifndef CONFIG_PPP_ECHO_INTERVAL
#define PPP_ECHO_INTERVAL 6
#else
#define PPP_ECHO_INTERVAL (CONFIG_PPP_ECHO_INTERVAL)
#endif

#ifndef CONFIG_PPP_ECHO_FAILURE
#define PPP_ECHO_FAILURE 5
#else 
#define PPP_ECHO_FAILURE (CONFIG_PPP_ECHO_FAILURE)
#endif

#ifndef CONFIG_PPPOE_RETRANSMIT
#define PPPOE_RETRANSMIT 10
#else
#define PPPOE_RETRANSMIT (CONFIG_PPPOE_RETRANSMIT)
#endif

#ifndef CONFIG_PPPOE_MAX_RETRANSMIT_TIMEOUT
#define PPPOE_MAX_RETRANSMIT_TIMEOUT ULONG_MAX
#else
#define PPPOE_MAX_RETRANSMIT_TIMEOUT (CONFIG_PPPOE_MAX_RETRANSMIT_TIMEOUT)
#endif

#define FW_DEFAULT_POLICY "default"

#define INITIAL_RULES_DESCR "Initial Rules"
#define FINAL_RULES_DESCR "Final Rules"
/* Rule group definitions
 * Important Note:
 * ipfilter 3.x doesn't support creating more then one head for the same
 * group. 
 * when a group number is specified, ipfw will add 1 to the number if the
 * direction of the rule is outbound. 
 * Bidir rules will be added once to the defined group number, and again to the
 * incremented group number.
 */
#define GRP_DEFAULT  		0
/* Trusted interfaces configuration */
#define GRP_TRUSTED  		10 
/* Security rules: IP Spoofing, IP options, IP fragments. */
#define GRP_SECURITY 		20 
/* OpenRG Support: RIP, IGMP, DHCP, PPTP, IPSec etc... */
#define GRP_OPENRG		30
/* User-defined rules (grouped by svc_rule_type_t) */
#define GRP_SVC_RMT_MAN		100
#define GRP_SVC_LOCAL_SRV	200
#define GRP_SVC_ACCESS_CTRL	400
#define GRP_SVC_PARENT_CTRL	500
#define GRP_SVC_ADVANCED	600
#define GRP_ESP			700
#define GRP_AH			800
#define GRP_INITIAL_INBOUND	900
#define GRP_INITIAL_OUTBOUND    1000
#define GRP_SVC_DMZ_HOST        1010
#define GRP_FINAL_INBOUND	1100
#define GRP_FINAL_OUTBOUND    	1200
#define GRP_IN			1300
#define GRP_OUT			1400
/* rules needed for a DMZ interface */
#define GRP_DMZ			300


#endif

