/* LICENSE_CODE JUNGO */

#ifndef _RG_ERROR_H_
#define _RG_ERROR_H_

#ifdef  _USE_ORG_REG_ERROR

#define OS_INCLUDE_STDARG
#include <os_includes.h>

/* FIREWALL: syslogd uses this format for deciding which log file to use */
#define SYSLOG_FIREWALL_FMT 	"RGFW-"
#define SYSLOG_NAT_FMT 		"RGNAT"
#define SYSLOG_LOGIN_FMT 	"LOGIN"
#define SYSLOG_CLI_LOGIN_FMT    SYSLOG_LOGIN_FMT "-CLI"
#define SYSLOG_WBM_LOGIN_FMT	SYSLOG_LOGIN_FMT "-WBM"
#define SYSLOG_SYS_UPDOWN_FMT	"SYS_UPDOWN-"

typedef struct {
    int ind;
    char *msg;
} rg_error_msg_t;

typedef enum {
    LDEBUG = 1,
    LINFO = 2,
    LWARN = 3,
    LERR = 4,
    LEXIT = 5,
    LPANIC = 6,
    LSEVERITY_MASK = 0xf,
    LCONSOLE = 0x100, /* From here on, it acts as a bit field. */
} rg_error_level_t;

/* XXX - A temporary hack that defines a level for "good" messages that should
 *       be given high priority than LINFO (the same priority as LERR).
 */
#define LINFO_ERR LERR

typedef enum {
    RL_SYNFLOOD = 0,
    RL_NAT_STATE = 1,
    RL_LAST
} rg_rate_idx_t;

#define RL_TEXT_SYNFLOOD "BLOCK-SYNFLOOD"
#define RL_TEXT_NAT_UNAVAIL "NAT-UNAVAILABLE"
#define RL_TEXT_NAT_STATE "NAT-STATE"

typedef void (*rg_error_func_t)(void *data, unsigned char *buf, 
    rg_error_level_t level);

/* The space before the ',' is important. Dont del it ! */
#define rg_error_f(SEVERITY, FMT, ARGS...) \
    rg_error(SEVERITY, "%s:%d: " FMT, __FUNCTION__, __LINE__ , ## ARGS)

#ifdef OPENRG_DEBUG
#define RG_DEBUG(level, params...) rg_error_f(level, ## params)
#else
#define RG_DEBUG(level, params...) do {} while (0)
#endif
    
/* Every registered function gets a reference to every log message from the
 * level specified and up.
 */
void rg_error_exit_reboot_set(void);

void strace_init(void);

/* Set main_task thread_id for rg_error */
void rg_error_set_mt_id(void);

/* Every registered function gets a reference to every log message from the
 * level specified and up.
 */
void rg_error_register(rg_error_func_t func, void *data, rg_error_level_t 
    level);
int rg_error_unregister(rg_error_func_t func, void *data);

/* Input: IP protocol number. Output: static buffer with protocol name */
char *ip_proto_name_get(int proto);

void rg_openlog(char *ident, int option, int  facility);
int rg_error(rg_error_level_t severity, const char *format, ...);
int rg_verror(rg_error_level_t severity, const char *format, va_list ap);
int rg_perror(rg_error_level_t severity, const char *format, ...);
int rg_vperror(rg_error_level_t severity, const char *format, va_list ap);
void rg_closelog(void);

#ifdef RG_ERROR_INTERNALS
/* Deliver log message directly, must be used only within the main task */
void log_msg_deliver(rg_error_level_t severity, char *msg);
#endif

/* return 1 if limit exceeded, 0 if not yet */
int rg_rate_limit(rg_rate_idx_t index);


#else  /* _USE_ORG_REG_ERROR */

#define rg_error(level,str,args...)

#endif  /* end _USE_ORG_REG_ERROR */

#endif

