/* LICENSE_CODE JUNGO */

/* rg_ip_list_t handling functions */

#ifndef _RG_IPLIST_H_
#define _RG_IPLIST_H_

#define OS_INCLUDE_INET
#include <os_includes.h>

#include <set.h>

typedef enum {
    ADD_FIRST = 0,
    ADD_LAST = 1,
    ADD_SORTED = 2, /* the list is assumed to be sorted in ascending order */
} add_pos_t;

typedef struct rg_ip_list_t {
    struct rg_ip_list_t *next;
    struct in_addr ip;
} rg_ip_list_t;

/* Note: if rg_ip_list_add() is used with either ADD_FIRST or ADD_LAST, then
 * the list might not be sorted, and future addition to the list with
 * ADD_SORTED will not insert the new element in the correct position. The
 * rule should be: if ADD_SORTED is used, then ADD_FIRST/ADD_LAST should not
 * be used on that list.
 */
int rg_ip_list_add(rg_ip_list_t **head, struct in_addr ip, add_pos_t pos);
void rg_ip_list_free(rg_ip_list_t **head);
rg_ip_list_t *rg_ip_list_dup(rg_ip_list_t *head);

/* Write the IP list head to the set 'root', under the path 'path', where each
 * IP has an index: path/%/ip
 */
void rg_ip_list_write_to_set(set_t **root, char *path, rg_ip_list_t *head);

/* Return whether a given IP address exists in a given IP list */
int rg_ip_list_is_exist(rg_ip_list_t *head, struct in_addr ip);
#endif
