/* LICENSE_CODE JUNGO */

#ifndef _SYSCALLS_H_
#define _SYSCALLS_H_
#include <estream.h>
#include <kos/kos_chardev.h>
#include <net_utils.h>

typedef struct {
    unsigned long rx_packets; /* total packets received */
    unsigned long tx_packets; /* total packets transmitted */
    unsigned long rx_bytes; /* total bytes received */
    unsigned long tx_bytes; /* total bytes transmitted */
    unsigned long rx_errors; /* bad packets received */
    unsigned long tx_errors; /* packet transmit problems */
    unsigned long rx_dropped; /* no space in linux buffers */
    unsigned long tx_dropped; /* no space available in linux */
    unsigned long rx_multicast; /* multicast packets received */
    unsigned long tx_multicast; /* multicast packets transmitted */
    unsigned long rx_broadcast; /* broadcast packets received */
    unsigned long tx_broadcast; /* broadcast packets transmitted */
    unsigned long collisions;
} user_net_device_stats_t;

/* estream functions. */

/* Set the socket to nonblock mode */
int sys_sock_set_nonblocking(int sock);

/* Set the interface to send multicast packets */
int sys_dev_set_multicast(int s, u32 if_ip);

/* event functions. */

clock_t sys_get_clock_rate(void);
clock_t sys_get_ticks_from_boot(void);

/* NOTE: Do not use this function directly, instead call mt_time_clock_set */
int sys_set_time(time_t *t);
struct timeval *sys_get_time(struct timeval *tv);

int openrg_module_ctrl(kos_chardev_type_t type, int cmd, void *data);

int sys_rg_chrdev_open(kos_chardev_type_t type, int mode);

void sys_soft_reboot(void);

int sys_flash_sync(int fd);

int sys_if_stats_get(char *if_name, user_net_device_stats_t *if_stats);

int sys_if_flags_set_direct(char *dev, int if_flags);
int sys_if_flags_set(char *dev, ifconfig_flag_t flags);
int sys_if_flags_get(char *dev, int *flags);

int sys_if_snd_len_get(char *dev, int *int_len);

int sys_if_addr_set(char *dev, u32 ip_addr);
int sys_if_addr_get(char *dev, struct sockaddr *ip_addr);

int sys_if_dst_addr_set(char *dev, u32 remote);
int sys_if_bcast_addr_set(char *dev, u32 bcast_addr);

int sys_if_mask_set(char *dev, u32 net_mask);
int sys_if_mask_get(char *dev, struct sockaddr *mask);

int sys_if_ext_set(char *dev, int value);
int sys_if_ext_get(char *dev, int *value);

int sys_if_is_pppoe_set(char *dev, int value);

int sys_if_name_get(int if_idx, char **if_name);
int sys_if_metric_get(char *dev, int *int_metric);
int sys_if_mtu_get(char *dev, int *int_mtu);
int sys_ip_forward_change(int flag);
int sys_ip_ttl_change(int ttl_val);

char *sys_alias_name_get(char *dev_name, int index);

/* Suspend the current process/task for a specified number of seconds */
void sys_sleep(int seconds);

typedef enum {
    ALIAS_CONF_RET_ERROR = -1,
    ALIAS_CONF_RET_SUCCESS = 0,
    ALIAS_CONF_RET_CONTINUE = 1,
} alias_conf_ret_t;

alias_conf_ret_t sys_alias_configure(char *dev, u32 ip_addr, u32 net_mask,
    u32 remote, ifconfig_flag_t flags);

/* Delete an ARP entry for a given device name and ip address */
int sys_arp_delete(char *name, struct in_addr ip);

/* Get the MAC for the specific IP and device using the ARP table.
 * Return 0 on success, -1 if ARP entry not found */
int dev_and_ip_to_mac_get(char *name, struct in_addr ip, ether_mac_t *mac);

/* Create a random buffer containing count integers.
 * Linux: By reading from /dev/urandom
 * VxWorks: By xoring the memory.
 */
void sys_get_random(int *buf, int count);

/* MII Ethernet API */
int sys_if_ioctl_mii_phy_id(char *dev);
int sys_if_ioctl_mii_read(char *dev, u16 phy_id, u16 reg_num);
int sys_if_ioctl_mii_write(char *dev, u16 phy_id, u16 reg_num, u16 val);

#endif
