/* LICENSE_CODE JUNGO */

#ifndef _VPN_UTILS_H_
#define _VPN_UTILS_H_

#include "rg_def.h"

int start_vpn_conns(int do_up);

extern int is_vpn_ipsec_need_start;

typedef enum
{
    RSASIGKEY_STATUS_NOTSET = 0,
    RSASIGKEY_STATUS_INPROCESS = 1,
    RSASIGKEY_STATUS_COMPLETED = 2
} rsasigkey_status_t;

typedef enum
{ 
    TYPE_TUNNEL = 1,
    TYPE_TRANSPORT = 2, 
    TYPE_PASSTHROUGH = 3
} type_t;

typedef enum {
    PUBKEY = 0,
    INKEY = 1,
    MODULUS = 2,
    PUBLICEXPONENT = 3,
    PRIVATEEXPONENT = 4,
    PRIME1 = 5,
    PRIME2 = 6,
    EXPONENT1 = 7,
    EXPONENT2 = 8,
    COEFFICIENT = 9,
    RSA_SEC_ALL = 10
} rsa_secr_fields_t;
    
typedef struct vpn_con_t
{
    struct vpn_con_t *next;
    char *con_name;
    char *host_name;
    int is_ipsec_gateway;
    int ip;
    int gateway;
    int local_net;
    int local_subnet_mask;
    int trans_type;
    int is_compressed;
    int is_startup_auto;
    int is_ip;
    int exch_method;
    int auth_type;
    int shared_rsa;
    char *shared_secret;
    int is_rsa_given;
    int is_local_rsa_given;
    int is_rsa_dns_specify;
    int is_rsa_dns_spec_name;
    int is_local_rsa_dns_name;
    int neg_attmp;
    int rekey_interval;
    char *rsa_public_key;
    char *dns_ip;
    char *dns_name;
    char *dns_local_name;
    int pfs;
    char *spi_num;
    int protocol;
    int ah_key;
    char *ah_auth_key;
    int esp_key;
    char *esp_auth_key;
    char *esp_encrpt_key;
} vpn_con_t;

typedef struct 
{
    char *type; 	//+// `tunnel', `transport'
    char *compress;	//+// noyes
    char *auto_param;	//+// `ignore', `add', `start'
    char *left;		// `%any', `%opportunistic', 
			// `%defaultroute', <ip_addrs>
    char *leftsubnet;	// <network/netmask>, <left/32>
    char *leftnexthop;	// `%direct', `%defaultroute', <ip_addrs>
    char *leftupdown;	// name of script
    char *leftfirewall; // yesno
    char *right;	//+// `%any', `%opportunistic', 
			//+// `%defaultroute', <ip_addrs>
    char *rightsubnet;	//+// <network/netmask>, <left/32>
    char *rightnexthop;	//+// `%direct', `%defaultroute', <ip_addrs>
    char *rightupdown;	// name of script
    char *rightfirewall;// yesno
} vpn_ipsec_conn_parameters_general_t;

typedef struct 
{
    char *keyexchange;	// `ike'
    char *auth;		//+// `esp', `ah'
    char *authby;	//+// `secret', `rsasig'
    char *leftid;	// <ip_addrs>, `@<domain_name>'
    char *leftrsasigkey;	//+// `%dns', <public_key_RFC2537_format>
    char *rightid;	//+// <ip_addrs>, `@<domain_name>'
    char *rightrsasigkey;	//+// `%dns', <public_key_RFC2537_format>
    char *pfs;		//+// yesno
    char *keylife;	//+// <decimal>m, <decimal>h, <decimal>d
		    	// 8.0h (default) ... 24h (max)
    char *rekeymargim;	// like keylife
			// default 9m
    char *rekeyfuzz;	// <decimal>%
    char *keyingtries;	//+// 0 (default),1,2 ...
    char *ikelifetime;	// 1h (default) ... 8h (max)	

} vpn_ipsec_conn_parameters_automatic_keying_t;

typedef struct 
{
    char *spi;		//+// 0x<hex>: 0x100-0xfff
    char *spibase;	// 0x<hex>0: 0x100-0xff0	
    char *esp;		//+// `3des',
		    	//+// `3des-md5-96',
		    	//+// `3des-sha1-96',
		    	//+// `null-md5-96',
		 	//+// `null-sha1-96'	
    char *espenckey;	//+// 192 bit hex key	
    char *leftespenckey;	// 192 bit hex key	
    char *rightespenckey;	// 192 bit hex key	
    char *espauthkey;	//+// 128 bit hex key = for md5
			//+// 160 bit hex key = for sha1
    char *espreplay_windows;	// 0(default)...64
    char *leftespspi;	// 0x<hex>
    char *rightespspi;	// 0x<hex>
    char *ah;		//+// `hmac-md5-96'
			//+// `hmac-sha1-96'
    char *ahkey;	//+// 128 bit hex key = for md5
			//+// 160 bit hex key = for sha1
    char *leftahkey;	// 128 bit hex key = for md5
			// 160 bit hex key = for sha1
    char *rightahkey;	// 128 bit hex key = for md5
			// 160 bit hex key = for sha1
    char *ahreplay_windows;	// 0(default)...64
    char *leftahspi;	// 0x<hex>
    char *rightahspi;	// 0x<hex>
} vpn_ipsec_conn_parameters_manual_keying_t;

typedef struct 
{
    char *interfaces;	// <virtual=physical>: ipsec0=eth1, ipsec1=ppp0
			// "<v1=p1> <v2=p2> ..."
			// `%defaultroute'
    char *forwardcontrol;	// yesno
    char *syslog;	// `daemon.error' (default)
    char *klipsdebug;	// `none', `all'
    char *plutodebug;	// `none', `all'
    char *dumpdir;	// <directory-path>
    char *manualstart;	// `'
			// <name-of-manual-connection>
			// "<name1> <name2> ..."
    char *pluto;	// yesno
    char *plutoload;	// `'
			// <name-of-connection>
			// "<name1> <name2> ..."
			// `%search'
    char *plutostart; 	// `'
			// <name-of-connection>
			// "<name1> <name2> ..."
			// `%search'
    char *plutowait;	// yesno
    char *plutobackgroundload;	// noyes
    char *prepluto;	// <shell-name>
    char *postpluto;	// <shell-name>
    char *fragicmp;	// noyes
    char *noeroutepass;	// noyes
    char *opportunistic;// noyes
    char *hidetos;	// yesno
    char *uniqueids;	// noyes
} vpn_ipsec_setup_section_t;

typedef struct vpn_ipsec_shared_secret_list_
{
    struct vpn_ipsec_shared_secret_list_ *next;
    char *left;		// <ip_addrs>
    char *right;	// <ip_addrs>
    char *shared_secret;// plain text
} vpn_ipsec_shared_secret_list_t;

typedef struct 
{
    char *rsasigkey;    // output of ipsec_rsasigkey
} vpn_ipsec_rsa_t;

typedef struct 
{
    vpn_ipsec_shared_secret_list_t *shared_secret_list;
    vpn_ipsec_rsa_t *rsa;
} vpn_ipsec_ipsec_secrets_t;

typedef struct vpn_ipsec_conn_section_list_
{
    char *name;		//+// <conn-name>, `%default'
    struct vpn_ipsec_conn_section_list_ *next;
    vpn_ipsec_conn_parameters_general_t *general;
    vpn_ipsec_conn_parameters_automatic_keying_t *automatic;
    vpn_ipsec_conn_parameters_manual_keying_t *manual;
} vpn_ipsec_conn_section_list_t;

typedef struct 
{
    vpn_ipsec_conn_section_list_t *conn;
    vpn_ipsec_setup_section_t *setup;
} vpn_ipsec_ipsec_conf_t;

typedef struct 
{
    vpn_ipsec_ipsec_conf_t *conf;
    vpn_ipsec_ipsec_secrets_t *secrets;
} vpn_ipsec_ipsec_t;

int vpn_ipsec_pkg_setup_start(void);
void vpn_ipsec_pkg_setup_stop(void);
int vpn_ipsec_datastruct_update(vpn_con_t *vpn_con_list);
vpn_con_t *vpn_con_list_alloc(char *section, char *entry);
void vpn_con_list_free(vpn_con_t **vpn_con_list);
int vpn_con_list_is_con_up(vpn_con_t **vpn_con_list, int line_num);
int vpn_ipsec_is_conn_up(vpn_con_t *vpn_con_ptr);
void vpn_con_free(vpn_con_t *vpn_con);
int vpn_rsasigkey_conf_load(char *p[]);
void vpn_rsasigkey_conf_free(char *p[]);
u32 get_vpn_con_rmt_ip(vpn_con_t *vpn_con);
int vpn_ipsec_datastruct_save(vpn_ipsec_ipsec_t *ipsec);
int vpn_ipsec_datastruct_alloc(vpn_ipsec_ipsec_t **ipsec_ptr,
    vpn_con_t *vpn, int is_con_list);
int vpn_ipsec_datastruct_free(vpn_ipsec_ipsec_t **ipsec_ptr_ptr);
/* Activate a valid connection, so, the connection can't be a NULL one */
int vpn_ipsec_conn_activate(vpn_con_t *vpn_con, int activate);
int vpn_ipsec_is_pkg_up(void);
int is_rmt_host_connected(vpn_con_t *vpn_con_list, int line_num);
int vpn_ipsec_generate_leftrsasigkey_finish(void);

/* auth/enc protocols */
#define PAP		"PAP"
#define CHAP		"CHAP"
#define MSCHAP		"MS-CHAP"
#define MSCHAPv2	"MS-CHAP-V2"
#define MPPE40		"MPPE-40"
#define MPPE128		"MPPE-128"
#define MPPESTATELESS	"MPPE-STATELESS"

typedef struct pptp_user_t
{
    struct pptp_user_t *next;
    int auth_protocol; /* NONE, ALL, ADVANCED */
    int auth_protocol_advanced;	/* PAP + CHAP + MS-CHAP + MS-CHAPv2 */
    int encrypt_protocol; /* NONE, ALL, ADVANCED */
    int encrypt_protocol_advanced; /* MPPE-40 + MPPE-128 + MPPE-STATELESS */
    
    /* Valuse for:
     * - /etc/ppp/pap-secrets
     * - /etc/ppp/chap-secrets
     */
    char *client;
    char *server;
    char *secret;
    int verbose;
    
    int pptpip; /* what address should pptp connect to */
    int defaultroute; /* should connection be the default route? */
    int access_peersub; /* enable access to peer subnet? */
    int peersubnet; /* subnet of peer's side */
    int netmask; /* 1-32 bits for describing netmask */
    int usepeerdns; /* Yes = use peer dns; No = don't */
    int is_startup_on_boot;
} pptp_user_t;

enum {
    NONE = 1,
    ALL = 2,
    ADVANCED = 3
};

pptp_user_t *pptp_user_list_alloc(char *section, char *entry);
void pptp_user_free(pptp_user_t *pptp_user);
void pptp_user_list_free(pptp_user_t **pptp_user_list);
void pptp_common_write_file_secrets(void);
int is_pptp_encrypt_in_bitmask(char *protocol, int bitmask);
int is_pptp_auth_in_bitmask(char *protocol, int bitmask);

#define MAX_POPTOP_CONNECTIONS 100  /* Limitation as defined in poptop daemon */
extern int is_vpn_poptop_need_start;

typedef struct poptop_package_t {
    int is_startup_on_boot;
    u32 localip_from;
    u32 localip_to;
    u32 remoteip_from;
    u32 remoteip_to;
    int listen_on_all;
    char *server_name;
    int verbose;
    int auth_protocol; /* NONE, ALL, ADVANCED */
    int auth_protocol_advanced; /* PAP + CHAP + MS-CHAP + MS-CHAPv2 */
    int encrypt_protocol; /* NONE, ALL, ADVANCED */
    int encrypt_protocol_advanced; /* MPPE-40 + MPPE-128 + MPPE-STATELESS */
    int idle_timeout;
} poptop_package_t;

poptop_package_t *poptop_package_alloc(char *section);
void poptop_package_free(poptop_package_t **poptop_package_ptr);
int poptop_write_file_conf(poptop_package_t *package);
int vpn_poptop_package_start(void);
int vpn_poptop_package_stop(void);
int is_poptop_package_up(void);
int is_package_valid(poptop_package_t *package);

typedef enum pptp_client_status
{
    PPTP_CLIENT_STATUS_NOTSET = 0,
    PPTP_CLIENT_STATUS_INPROCESS = 1,
    PPTP_CLIENT_STATUS_COMPLETED = 2
} pptp_client_status_t;

int pptp_user_activate(pptp_user_t *pptp_user, int activate);
int is_pptp_client_connected(pptp_user_t *user);
int pptp_client_disconnect(pptp_user_t *user);
int activate_pptp_users(int user_ind, int activate);

#endif
