/* LICENSE_CODE JUNGO */

/* Network library and wrappers for kernel mode development.
 *
 * Prefix KNET stands for Kernel mode NETwork library.
 *
 * All the data types should be of u8, u16 and u32, please refer to
 * procedures/coding/c_data_types.txt
 */
#ifndef _KNET_H_
#define _KNET_H_

#include "kos_endian.h"

/* How to use KNET_IFDEF and KNET_LIN_IFDEF: The order of parameters inside is
 * Linux first, VxWorks later, and inside Linux - 2.4 first and 2.2 later.
 */

#ifdef CONFIG_RG_TARGET_LINUX

#include <linux/version.h>
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
#define KNET_LINUX_24
#define KNET_LIN_IFDEF(A,B) A
#else
#define KNET_LINUX_22
#define KNET_LIN_IFDEF(A,B) B
#endif

#define KNET_IFDEF(A,B) A

#elif defined(CONFIG_RG_TARGET_VXWORKS)

#define KNET_IFDEF(A,B) B

#endif

#ifdef __KERNEL__
#include <kos/kos.h>
#endif

#ifdef CONFIG_RG_TARGET_LINUX
#include <linux/in_systm.h> /* for n_time */
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#include <netinet/in_systm.h> /* for n_time */
#endif

#ifdef __KERNEL__

/* Includes for both OSes */
#ifdef CONFIG_RG_TARGET_LINUX
#include <linux/byteorder/generic.h> /* convertions host/network byte order */
#include <linux/in.h> /* standard well-defined IP protocols */
#include <linux/netdevice.h> /* struct device */
#include <linux/skbuff.h>
#include <linux/if_arp.h>
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/inet.h>
#include <net/tcp.h>
#elif defined(CONFIG_RG_TARGET_VXWORKS)
/* Convertions host/network byte order, standard well-defined IP protocols */
#include <netinet/in.h> 
#include <net/if.h> /* struct ifnet */
#include <net/if_subr.h>
#include <net/mbuf.h>
#include <net/if_dl.h>
#include <net/if_types.h>
#include <netinet/in_var.h>
#include <netinet/tcp.h>
#include <net/systm.h>
#include <etherLib.h>
#include <sys/ioctl.h>
#include <ctype.h>
#include <stdio.h>
#include <inetLib.h>
#endif
#else
#include <rg_def.h>
#define OS_INCLUDE_INET
#include <os_includes.h>
#endif

/* Convertion of values between host and network byte order is done by:
 * htonl(), htons(), ntohl(), ntohs().
 * The functions are available "as is" for both OSes.
 */

/* Network protocol headers */

/* Ethernet header
 *
 * knet_eth_hdr_t is equivalent to struct ethhdr in linux (<linux/if_ether.h>)
 * and struct enet_hdr in VxWorks (<etherLib.h>)
 */

/* Some Ethernet constants */
#define KNET_ETHER_ADDR_LEN 6 /* Ethernet address length */
#define KNET_ETHER_TYPE_LEN 2 /* Ethernet type field length */
#define KNET_ETHER_CRC_LEN 4 /* Ethernet CRC lenght */
#define KNET_ETHER_HDR_LEN ((KNET_ETHER_ADDR_LEN * 2) + KNET_ETHER_TYPE_LEN)
#define KNET_ETHER_ALIGN_FIX (16-KNET_ETHER_HDR_LEN) /* space before ethernet 
						      * header to make the IP
						      * header 16-aligned */
#define KNET_ETHER_MIN_FRAME_LEN 64 /* Minimum frame length, CRC included */
#define KNET_ETHER_MAX_FRAME_LEN 1518 /* Maximum frame length, CRC included */
#define	KNET_ETHER_MTU (KNET_ETHER_MAX_FRAME_LEN - KNET_ETHER_HDR_LEN - \
    KNET_ETHER_CRC_LEN)
/* Is the address a multicast or broadcast. */
#define	KNET_ETHER_IS_MULTICAST(addr) (*(addr) & 0x01)
#define	KNET_ETHER_IS_BROADCAST(addr) (addr[0]==0xff && addr[1]==0xff && \
    addr[2]==0xff && addr[3]==0xff && addr[4]==0xff && addr[5]==0xff)
/* Ethernet type */
#define KNET_IFT_ETHER KNET_IFDEF(ARPHRD_ETHER, IFT_ETHER)
#define KNET_DEV_IS_ETHER(dev) ((dev)->netdev_ift == KNET_IFT_ETHER)

/* These #ifndef's are here mainly for Linux, since these are already defined
 * in Linux (but not in VxWorks).
 */
#ifndef	ICMP_MINLEN
#define	ICMP_MINLEN 8
#endif

#ifndef	ICMP_ECHOREPLY
#define	ICMP_ECHOREPLY 0
#endif

#ifndef	ICMP_UNREACH
#define	ICMP_UNREACH 3
#endif
#ifndef	ICMP_UNREACH_NET
#define	ICMP_UNREACH_NET 0
#endif
#ifndef	ICMP_UNREACH_HOST
#define	ICMP_UNREACH_HOST 1
#endif
#ifndef	ICMP_UNREACH_PROTOCOL
#define	ICMP_UNREACH_PROTOCOL 2
#endif
#ifndef	ICMP_UNREACH_PORT
#define	ICMP_UNREACH_PORT 3
#endif
#ifndef	ICMP_UNREACH_NEEDFRAG
#define	ICMP_UNREACH_NEEDFRAG 4
#endif
#ifndef	ICMP_UNREACH_SRCFAIL
#define	ICMP_UNREACH_SRCFAIL 5
#endif
#ifndef	ICMP_UNREACH_NET_UNKNOWN
#define	ICMP_UNREACH_NET_UNKNOWN 6
#endif
#ifndef	ICMP_UNREACH_HOST_UNKNOWN
#define	ICMP_UNREACH_HOST_UNKNOWN 7
#endif
#ifndef	ICMP_UNREACH_ISOLATED
#define	ICMP_UNREACH_ISOLATED 8
#endif
#ifndef	ICMP_UNREACH_NET_PROHIB
#define	ICMP_UNREACH_NET_PROHIB 9
#endif
#ifndef	ICMP_UNREACH_HOST_PROHIB
#define	ICMP_UNREACH_HOST_PROHIB 10
#endif
#ifndef	ICMP_UNREACH_TOSNET
#define	ICMP_UNREACH_TOSNET 11
#endif
#ifndef	ICMP_UNREACH_TOSHOST
#define	ICMP_UNREACH_TOSHOST 12
#endif
#ifndef	ICMP_UNREACH_ADMIN_PROHIBIT
#define	ICMP_UNREACH_ADMIN_PROHIBIT 13
#endif
#ifndef	ICMP_UNREACH_HOST_PRECEDENCE
#define	ICMP_UNREACH_HOST_PRECEDENCE 14
#endif
#ifndef	ICMP_UNREACH_PRECEDENCE_CUTOFF
#define	ICMP_UNREACH_PRECEDENCE_CUTOFF 15
#endif

#ifndef	ICMP_SOURCEQUENCH
#define	ICMP_SOURCEQUENCH 4
#endif

#ifndef ICMP_REDIRECT
#define	ICMP_REDIRECT 5
#endif
#ifndef	ICMP_REDIRECT_NET
#define	ICMP_REDIRECT_NET 0
#endif
#ifndef	ICMP_REDIRECT_HOST
#define	ICMP_REDIRECT_HOST 1
#endif
#ifndef	ICMP_REDIRECT_TOSNET
#define	ICMP_REDIRECT_TOSNET 2
#endif
#ifndef	ICMP_REDIRECT_TOSHOST
#define	ICMP_REDIRECT_TOSHOST 3
#endif

#ifndef	ICMP_ALTHOSTADDR
#define	ICMP_ALTHOSTADDR 6
#endif

#ifndef	ICMP_ECHO
#define	ICMP_ECHO 8
#endif

#ifndef	ICMP_TIMXCEED
#define	ICMP_TIMXCEED 11
#endif
#ifndef	ICMP_TIMXCEED_INTRANS
#define	ICMP_TIMXCEED_INTRANS 0
#endif
#ifndef	ICMP_TIMXCEED_REASS
#define	ICMP_TIMXCEED_REASS 1
#endif

#ifndef	ICMP_PARAMPROB
#define	ICMP_PARAMPROB 12
#endif
#ifndef	ICMP_PARAMPROB_ERRATPTR
#define	ICMP_PARAMPROB_ERRATPTR 0
#endif
#ifndef	ICMP_PARAMPROB_OPTABSENT
#define	ICMP_PARAMPROB_OPTABSENT 1
#endif
#ifndef	ICMP_PARAMPROB_LENGTH
#define	ICMP_PARAMPROB_LENGTH 2
#endif

#ifndef ICMP_TIMESTAMP
#define ICMP_TIMESTAMP 13
#endif
#ifndef ICMP_TSTAMP
#define	ICMP_TSTAMP 13
#endif

#ifndef ICMP_TIMESTAMPREPLY
#define ICMP_TIMESTAMPREPLY 14
#endif
#ifndef ICMP_TSTAMPREPLY
#define	ICMP_TSTAMPREPLY 14
#endif

#ifndef ICMP_INFO_REQUEST
#define ICMP_INFO_REQUEST 15
#endif
#ifndef ICMP_IREQ
#define	ICMP_IREQ 15
#endif

#ifndef ICMP_INFO_REPLY
#define ICMP_INFO_REPLY 16
#endif
#ifndef ICMP_IREQREPLY
#define	ICMP_IREQREPLY 16
#endif

#ifndef ICMP_ADDRESS
#define ICMP_ADDRESS 17
#endif
#ifndef	ICMP_MASKREQ
#define	ICMP_MASKREQ 17
#endif

#ifndef ICMP_ADDRESSREPLY
#define ICMP_ADDRESSREPLY 18
#endif
#ifndef ICMP_MASKREPLY
#define	ICMP_MASKREPLY 18
#endif

#ifndef	ICMP_TRACEROUTE
#define	ICMP_TRACEROUTE 30
#endif

#ifndef	ICMP_DATACONVERR
#define	ICMP_DATACONVERR 31
#endif

#ifndef	ICMP_MOBILE_REDIRECT
#define	ICMP_MOBILE_REDIRECT 32
#endif

#ifndef	ICMP_IPV6_WHEREAREYOU
#define	ICMP_IPV6_WHEREAREYOU 33
#endif

#ifndef	ICMP_IPV6_IAMHERE
#define	ICMP_IPV6_IAMHERE 34
#endif

#ifndef	ICMP_MOBILE_REGREQUEST
#define	ICMP_MOBILE_REGREQUEST 35
#endif

#ifndef	ICMP_MOBILE_REGREPLY
#define	ICMP_MOBILE_REGREPLY 36
#endif

#ifndef	ICMP_SKIP
#define	ICMP_SKIP 39
#endif

#ifndef	ICMP_PHOTURIS
#define	ICMP_PHOTURIS 40
#endif

#ifndef	ICMP_PHOTURIS_UNKNOWN_INDEX
#define	ICMP_PHOTURIS_UNKNOWN_INDEX 1
#endif

#ifndef	ICMP_PHOTURIS_AUTH_FAILED
#define	ICMP_PHOTURIS_AUTH_FAILED 2
#endif

#ifndef	ICMP_PHOTURIS_DECRYPT_FAILED
#define	ICMP_PHOTURIS_DECRYPT_FAILED 3
#endif

/* Taken from ip-filter/ip_compat.h */
typedef struct {
    u8 ether_dhost[KNET_ETHER_ADDR_LEN]; /* destination mac addr */
    u8 ether_shost[KNET_ETHER_ADDR_LEN]; /* source mac addr */
    u16 ether_type; /* packet protocol type */
} __attribute__ ((packed)) knet_eth_hdr_t;

typedef struct {
    u8 ether_addr_octet[KNET_ETHER_ADDR_LEN];
} __attribute__ ((packed)) knet_ether_addr_t;

/* Ethernet frame types */
#ifdef CONFIG_RG_TARGET_VXWORKS
#include <netinet/if_ether.h>
#elif defined(CONFIG_RG_TARGET_LINUX)
#include <linux/if_ether.h>
#endif
#define KNET_ETH_TYPE_IP KNET_IFDEF(ETH_P_IP, ETHERTYPE_IP)
#define KNET_ETH_TYPE_ARP KNET_IFDEF(ETH_P_ARP, ETHERTYPE_ARP)
#define KNET_ETH_TYPE_RARP KNET_IFDEF(ETH_P_RARP, ETHERTYPE_REVARP)
#define KNET_ETH_TYPE_PPP_DISC 0x8863
#define KNET_ETH_TYPE_PPP_SESSION 0x8864
#define KNET_ETH_TYPE_VLAN_8021Q 0x8100 /* 802.1Q VLAN Extended Header  */
#define KNET_ETH_TYPE_AUTH1X 0x888e /* 802.1x EAP  */

/* Standard well-defined IP protocols
 * (IPPROTO_IP, IPPROTO_ICMP, IPPROTO_IGMP, IPPROTO_TCP, IPPROTO_UDP etc.)
 * They are enum'ed in Linux and #define'd in VxWorks, however since they share
 * the same names, we will not redefine them.
 * Some values are defined only in Linux, some - only in VxWorks, they will be
 * added to either side when needed.
 */

/* IP header
 *
 * Linux: <linux/ip.h> - struct iphdr
 * VxWorks: <netinet/ip.h> - struct ip
 * Taken from ip-filter/ip_compat.h
 */
typedef struct {
#ifdef KOS_ARCH_LITTLE_ENDIAN    
    u8 ip_hl:4; /* header length */
    u8 ip_v:4; /* version */
#else
    u8 ip_v:4; /* version */
    u8 ip_hl:4; /* header length */
#endif
    u8 ip_tos; /* type of service */
    u16 ip_len; /* total length */
    u16 ip_id; /* identification */
    u16 ip_off; /* fragment offset field */
    u8 ip_ttl; /* time to live */
    u8 ip_p; /* protocol */
    u16 ip_sum; /* checksum */
    struct in_addr ip_src; /* source address */
    struct in_addr ip_dst; /* destination address */
} knet_ip_hdr_t;

#define KNET_IP_OFFMASK KNET_IFDEF(IP_OFFSET, IP_OFFMASK)
 
/* TCP header
 *
 * Linux: <linux/tcp.h> - struct tcphdr
 * VxWorks: <netinet/tcp.h> - struct tcphdr
 * Taken from ip-filter/ip_compat.h
 */
typedef struct {
    u16 th_sport; /* source port */
    u16 th_dport; /* destination port */
    u32 th_seq; /* sequence number */
    u32 th_ack; /* acknowledgement number */
#ifdef KOS_ARCH_LITTLE_ENDIAN
    u8 th_res:4; /* unused */
    u8 th_off:4; /* data offset */
#else
    u8 th_off:4; /* data offset */
    u8 th_res:4; /* unused */
#endif
    u8 th_flags;
    u16 th_win; /* window */
    u16 th_sum; /* checksum */
    u16 th_urp; /* urgent pointer */
} knet_tcp_hdr_t;

#define	TH_FIN	0x01
#define	TH_SYN	0x02
#define	TH_RST	0x04
#define	TH_PUSH	0x08
#define	TH_ACK	0x10
#define	TH_URG	0x20
#define	TH_ECN	0x40
#define	TH_CWR	0x80

#define KNET_TCPOPT_EOL KNET_IFDEF(TCPOPT_EOL, TCPOPT_EOL)
#define KNET_TCPOPT_NOP KNET_IFDEF(TCPOPT_NOP, TCPOPT_NOP)
#define KNET_TCPOPT_MSS KNET_IFDEF(TCPOPT_MSS, TCPOPT_MAXSEG)
#define KNET_TCPOPT_WSCALE KNET_IFDEF(TCPOPT_WINDOW, TCPOPT_WINDOW)
#define KNET_TCPOPT_SACK_PERM \
    KNET_IFDEF(TCPOPT_SACK_PERM, TCPOPT_SACK_PERMITTED)

/* UDP header
 *
 * Linux: <linux/udp.h> - struct udphdr
 * VxWorks: <netinet/udp.h> - struct udphdr
 * Taken from ip-filter/ip_compat.h
 */
typedef struct {
    u16 uh_sport; /* source port */
    u16 uh_dport; /* destination port */
    u16 uh_ulen; /* udp length */
    u16 uh_sum; /* udp checksum */
} knet_udp_hdr_t;

/* XXX: All DHCP constants and struct definition were taken from
 * pkg/dhcp/includes/dhcp.h - unite */

#define KNET_DHCP_CHADDR_LEN 16
#define KNET_DHCP_SNAME_LEN 64
#define KNET_DHCP_FILE_LEN 128
#define KNET_DHCP_FIXED_LEN 278 /* Everything but options. */
#define KNET_DHCP_MTU_MAX 1500
#define KNET_DHCP_OPTION_LEN (KNET_DHCP_MTU_MAX - KNET_DHCP_FIXED_LEN)

#define KNET_IPPORT_BOOTPS 67
#define KNET_IPPORT_BOOTPC 68
#define KNET_BOOTREQUEST 1
#define KNET_BOOTREPLY 2

typedef struct {
    u8  op; /* Message opcode/type */
    u8  htype; /* Hardware addr type (see net/if_types.h) */
    u8  hlen; /* Hardware addr length */
    u8  hops; /* Number of relay agent hops from client */
    u32 xid; /* Transaction ID */
    u16 secs; /* Seconds since client started looking */
    u16 flags; /* Flag bits */
    struct in_addr ciaddr; /* Client IP address (if already in use) */
    struct in_addr yiaddr; /* Client IP address */
    struct in_addr siaddr; /* IP address of next server to talk to */
    struct in_addr giaddr; /* DHCP relay agent IP address */
    u8 chaddr[KNET_DHCP_CHADDR_LEN]; /* Client hardware address */
    char sname[KNET_DHCP_SNAME_LEN]; /* Server name */
    char file[KNET_DHCP_FILE_LEN]; /* Boot filename */
    u8 options[KNET_DHCP_OPTION_LEN]; /* Optional parameters (actual length 
				       * dependent on MTU). 
				       */
} knet_dhcp_packet_t;

/* ICMP header
 *
 * Linux: <linux/icmp.h> - struct icmphdr
 * VxWorks: <netinet/ip_icmp.h> - struct icmp
 * Taken from ip-filter/ip_compat.h
 */
typedef struct {
    u8 icmp_type;		/* type of message */
    u8 icmp_code;		/* type sub code */
    u16 icmp_cksum;		/* ones complement cksum of struct */
    union {
	u8 ih_pptr;			/* ICMP_PARAMPROB */
	struct in_addr ih_gwaddr;	/* ICMP_REDIRECT */
	struct knet_ih_idseq {
            u16 icd_id;
            u16 icd_seq;
        } ih_idseq;
	int ih_void;
    } icmp_hun;
    union {
	struct knet_id_ts {
	    n_time its_otime;
	    n_time its_rtime;
	    n_time its_ttime;
	} id_ts;
	struct knet_id_ip {
	    knet_ip_hdr_t idi_ip;
	    /* options and then 64 bits of data */
	} id_ip;
	u32 id_mask;
	u8 id_data[1];
    } icmp_dun;
} knet_icmp_hdr_t;

/* ARP header and message for ethernet
 * IPs and mac-addresses are defined as char arrays in order to create a packed
 * struct
 * use knet_get_u32 and the offset constants below to read the IPs */

/* XXX: VxWorks defines a macro called arp_op */
#if defined(CONFIG_RG_TARGET_VXWORKS) && defined(arp_op)
#undef arp_op
#endif

typedef struct {
    u16 arp_hard_type;
    u16 arp_prot_type;
    u8 arp_hard_size;
    u8 arp_prot_size;
    u16 arp_op;
    u8 arp_msg[2 * KNET_ETHER_ADDR_LEN + 2 * sizeof(struct in_addr)];
} knet_arp_eth_t;

/* ARP constatns */
#define KNET_ARP_OP_REQUEST 1
#define KNET_ARP_OP_REPLY 2
/* offset of ARP message members from ARP-header start */
#define KNET_ARP_SMAC_OFFSET 8
#define KNET_ARP_SIP_OFFSET 14
#define KNET_ARP_DMAC_OFFSET 18
#define KNET_ARP_DIP_OFFSET 24

/* is_external set/get ioctls */
#define RG_IOCTL_KNET_IF_SET_EXT \
    _IOW(RG_IOCTL_PREFIX_KOS, 1, struct ifreq)
#define RG_IOCTL_KNET_IF_GET_EXT \
    _IOWR(RG_IOCTL_PREFIX_KOS, 2, struct ifreq)
#define RG_IOCTL_SIOCSETRGCHRDEVTYPE \
    _IOW(RG_IOCTL_PREFIX_KOS, 3, struct ifreq)
#define RG_IOCTL_KNET_IF_SET_PPPOE \
    _IOW(RG_IOCTL_PREFIX_KOS, 4, struct ifreq)

#ifdef __KERNEL__

typedef struct in_addr in_addr_t; 
char *knet_ntoa(in_addr_t addr);
int knet_aton(char *cp, struct in_addr *inp);

/* Network packets.
 * For handling network packets, Linux uses the sk_buff struct, while
 * VxWorks uses the mbuf struct, taken from BSD. Our internal struct
 * is called knet_buf_t (network buffer).
 * NOTE: Special care should be taken to ALWAYS check the next buffer of
 * a chain while writing multi-OS code.
 */
typedef KNET_IFDEF(struct sk_buff, struct mbuf) knet_buf_t;
typedef KNET_IFDEF(unsigned int, int) knet_pktlen_t;

/* read from un-aligned memory */
u32 knet_get_u32(void *p);

/* Fields access macros.
 * They are written in a BSD style - to access the field, write the macro
 * after -> or . of a knet_pktlen_t variable.
 */
#define knet_pkt_type KNET_IFDEF(pkt_type, m_flags)
#define KNET_MULTICAST KNET_IFDEF(PACKET_MULTICAST, M_MCAST)
#define KNET_BROADCAST KNET_IFDEF(PACKET_BROADCAST, M_BCAST)
#define KNET_PACKET_HOST KNET_IFDEF(PACKET_HOST, 0)
#define KNET_PACKET_OTHERHOST KNET_IFDEF(PACKET_OTHERHOST, 0)

void knet_pkt_type_set(knet_buf_t *buf, int val);

/* knet_rcvif returns the interface the packet arrived/leaving on. */
#define knet_rcvif KNET_IFDEF(dev, m_pkthdr.rcvif)

/* knet_nextpkt returns next packet (network record) in a queue, valid for
 * both Linux and VxWorks.
 */
#define knet_nextpkt KNET_IFDEF(next, m_nextpkt)
 
/* knet_nextbuf returns next buffer in a chain, valid only for VxWorks, for
 * Linux NULL is returned.
 * 
 * IMPORTANT: Code should always check next buffer in the chain, for the full
 * compatibility!
 */
#define knet_nextbuf KNET_IFDEF(NULL, m_next)

/* knet_pktlen returns the length of the whole packet.
 * IMPORTANT: For VxWorks, the length is contained in the first buffer only.
 * The code should verify that this is the first buffer (knet_is_first).
 * knet_buflen returns the length of the current buffer. For Linux, both macros
 * return the same value, however in VxWorks they are different.
 * Use the knet_is_first macro to determine whether this is the first buffer.
 */
#define knet_pktlen KNET_IFDEF(len, m_pkthdr.len)
#define knet_buflen KNET_IFDEF(len, m_len)

/* Returns pointer to buffer's data */
#define knet_data KNET_IFDEF(data, m_data)

/* Returns pointer to buffer's data start */
#define knet_head KNET_IFDEF(head, m_extBuf)

/* knet_is_first returns true if this is the first buffer in the chain.
 * It accepts the pointer to knet_buf_t.
 */
#define knet_is_first(m) KNET_IFDEF(1, (m)->m_flags & M_PKTHDR)

/* knet_is_ext tests whether knet_buf has external storage.
 * For both OSes, this is true.
 */
#define knet_is_ext(m) 1

/* Use this macro to cast the current data pointer to any type 
 * (knet_ip_hdr_t, knet_tcp_hdr_t, knet_udp_hdr_t etc).
 */
#define knet_mtod(m, t) ((t)((m)->knet_data))

/* Network buffer allocation
 * For VxWorks, we allocate buffers from the default pool (_pNetDpool).
 * NOTE: For Linux, dev_alloc_skb() is used.
 * IMPORTANT: To state that this will be the first buffer of a chain,
 * add KNET_FIRST_IN_CHAIN mask to the flags.
 */
#define KNET_DONTWAIT 0
#define KNET_WAIT 1
#define KNET_FIRST_IN_CHAIN 2

knet_buf_t *knet_buf_alloc(int size, int flags);

/* Frees the network buffer.
 * IMPORTANT: In VxWorks, in the case of a buffer chain, all chain is freed.
 * If your intention is to free 1 buffer only, use m_free() instead.
 */
void knet_buf_free(knet_buf_t *buf);

/* Get ip from buf where buf->knet_data points to an ethernet header. 
 * return src or dest ip according to is_src
 */
struct in_addr knet_buf_ip_get(knet_buf_t *buf, int is_src);

/* For VxWorks, the standard m_pullup() is called.
 * Since in Linux there's only one buffer, the function fails
 * if the data size is less than the requested size.
 * In case of error, the buffer is freed.
 */
knet_buf_t *knet_pullup(knet_buf_t *buf, int size);

/* Copy-on-write */
knet_buf_t *knet_buf_cow(knet_buf_t *buf);

/* Copy 'len' bytes of data from network buffer 'buf', starting from byte
 * 'offset', into the data area pointed by dst. Returns the number of bytes
 * copied.
 */
int knet_buf_copydata(knet_buf_t *buf, u8 *dst, int offset, int len);

int knet_buf_copy_to_user(knet_buf_t *buf, u8 *dst, int offset, int len);

/* Manipulating the packet data pointers
 *
 * Original functions:
 * Linux: skb_push, skb_put, skb_pull, skb_reserve.
 */
 
/* Move the data pointer back (to add data to head of packet) */
u8 *knet_buf_data_push(knet_buf_t *buf, u32 len);

/* Move the data pointer forward (to remove data from head of packet) */
u8 *knet_buf_data_pull(knet_buf_t *buf, u32 len);

/* copy on write when buf->knet_data points to un-aligned memory (ethernet
 * header) */
knet_buf_t *knet_buf_cow_align(knet_buf_t *buf);

/* Set the length of the buffer to be 'len', tail = data + len */
void knet_buf_data_adj(knet_buf_t *buf, int len);

/* Copy 'len' bytes of data from data area pointed by src into offset 'offset'
 * in the network buffer 'buf'. The buffer will be extended if necessary. In
 * case the buffer is extended, it may be reallocated. Returns the number of
 * bytes copied.
 */
u32 knet_buf_copyback(knet_buf_t **pbuf, u8 *src, u32 offset, u32 len);

/* Network buffer clone: shallow copy  */
knet_buf_t *knet_buf_clone(knet_buf_t *m, int flags);

/* Network buffer duplicate: deep copy  */
knet_buf_t *knet_buf_dup(knet_buf_t *m, int flags);

/* Concatenate two knet_buf_t's to a single chain, it will replace the first
 * buffer.
 */
int knet_buf_merge(knet_buf_t **buf, knet_buf_t *buf2);

/* Move the tail pointer forward (to add data to end of packet) */ 
u8 *knet_buf_data_put(knet_buf_t *buf, u32 len);

/* Move both data and tail forward (to reserve space for headers) */
void knet_buf_data_reserve(knet_buf_t *buf, u32 len);

/* Set the length of the buffer to be 'len', tail = data + len */
u8 *knet_buf_data_trim(knet_buf_t *buf, u32 len);

/* Move the tail pointer backward (to remove data from the end of packet),
 * decrease the length of the packet by 'len'
 */
void *knet_buf_data_trimby(knet_buf_t *buf, u32 len);

/* Checksum calculations and checks */

/* Used for miscellaneous IP-like checksums, mainly for ICMP 
 * The buf header csum field must be zero when calling this function
 */
u16 knet_ip_compute_csum(void *buf, int buf_hdrlen);

/* Generate a checksum for an outgoing IP datagram. */
void knet_ip_send_csum(knet_buf_t *buf);

/* fix checksum */
u16 knet_fix_csum_16(u16 csum, u16 old, u16 new, u8 is_udp);

/* IP fragment/defragment - must be done by the firewall module,
 * but some API should be provided by the net_lib to help to get the
 * assembled/reassembled packet or queue of buffers information. TODO.
 */

/* Network device structure
 *
 * Linux: <linux/netdevice.h> - struct net_device / device
 * VxWorks: <net/if.h> - struct ifnet
 */
typedef KNET_IFDEF(KNET_LIN_IFDEF(struct net_device, struct device),
    struct ifnet) knet_netdev_t;

#define knet_netdev_hw_hdrlen KNET_IFDEF(hard_header_len, if_hdrlen)

/* Network device registration/unregisration
 *
 * Linux: register_netdev()
 * VxWorks: if_attach()
 */
int knet_netdev_register(knet_netdev_t *dev);
void knet_netdev_unregister(knet_netdev_t *dev);

int knet_netdev_init(knet_netdev_t *);
void knet_netdev_uninit(knet_netdev_t *);

/* Get network device by name. */
knet_netdev_t *knet_netdev_get_by_name(char *name);
knet_netdev_t *knet_netdev_get_by_index(int if_idx);

/* Set promiscuous mode for a device. 
 * 1 - enable promiscuous mode.
 * 0 - disable promiscuous mode.
 */
int knet_netdev_promisc_set(knet_netdev_t *dev, int flag);

/* Interface request structure used for socket ioctl's */
typedef KNET_IFDEF(struct ifreq, struct ifreq) knet_ifreq_t;
typedef KNET_IFDEF(struct ifreq, char) knet_ioctl_data_t;

/* There is a difference in the order of parameters to functions in the
 * struct device in Linux and struct ifnet in VxWorks.
 * Use these macros to define your own callbacks. Note of the return type.
 */

/* KNET int (*ioctl)(KNET_NETDEV_IOCTL(knet_netdev_t *dev, int cmd, 
 *          knet_ioctl_data_t *data);
 *          
 * Linux: int (*do_ioctl)(struct device *dev, struct ifreq *ifr, int cmd);
 * VxWorks: int	(*if_ioctl)(struct ifnet *, int cmd, caddr_t data);
 */
#ifdef CONFIG_RG_TARGET_LINUX
#define KNET_NETDEV_IOCTL(dev, cmd, data) dev, data, cmd
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#define KNET_NETDEV_IOCTL(dev, cmd, data) dev, cmd, data
#endif

/* KNET: int (*start_output)(KNET_NETDEV_START_OUTPUT(knet_netdev_t *dev, 
 *           knet_buf_t *buf));
 *           
 * Linux: int (*hard_start_xmit)(struct sk_buff *skb, struct device *dev);
 * VxWorks: int (*if_output)(struct ifnet *, struct mbuf *, 
 *   struct sockaddr *, struct rtentry *);
 */
#ifdef CONFIG_RG_TARGET_LINUX
#define KNET_NETDEV_START_OUTPUT(dev, buf) buf, dev
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#define KNET_NETDEV_START_OUTPUT(dev, buf) dev, buf, \
    struct sockaddr *__notinuse1, struct rtentry *__notinuse2
#endif

/* KNET: KNET_NETDEV_CALL_OUTPUT(dev, buf)
 * Linux:buf, dev
 * VxWorks: dev, buf, NULL, NULL
 */
#ifdef CONFIG_RG_TARGET_LINUX
#define KNET_NETDEV_CALL_OUTPUT(dev,buf) buf, dev
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#define KNET_NETDEV_CALL_OUTPUT(dev, buf) dev, buf, NULL, NULL
#endif

typedef int (*knet_netdev_output_t)
    (KNET_NETDEV_START_OUTPUT(knet_netdev_t *dev, knet_buf_t *buf));
    
/* KNET: int (*start_xmit)(KNET_NETDEV_START_XMIT(knet_dev_t *dev, 
 *           knet_buf_t *buf, int unit));
 *           
 * Linux: int (*hard_start_xmit)(struct sk_buff *skb, struct device *dev);
 * VxWorks: int (*if_start)(struct ifnet *dev)
 */
#ifdef CONFIG_RG_TARGET_LINUX
#define KNET_NETDEV_START_XMIT(dev, buf) buf, dev
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#define KNET_NETDEV_START_XMIT(dev, buf) dev
#endif

/* KNET: int KNET_NETDEV_OPEN_STOP(knet_dev_t *dev, int if_unit)
 *           
 * Linux: int (*open)(struct device *dev);
 * VxWorks: int (*if_init)(int if_unit);
 */
#ifdef CONFIG_RG_TARGET_LINUX
#define KNET_NETDEV_OPEN_STOP(dev, if_unit) dev
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#define KNET_NETDEV_OPEN_STOP(dev, if_unit) if_unit
#endif

int knet_netdev_send(knet_netdev_t *dev, const char *data, int len,
    int is_data_from_user);

int knet_netdev_receive(knet_buf_t *buf);

void knet_netdev_fields_init(knet_netdev_t *dev);

#define netdev_open KNET_IFDEF(open, if_init)
#ifdef CONFIG_RG_TARGET_LINUX
#define knet_netdev_open(dev) (*dev->open)(dev)
#elif defined (CONFIG_RG_TARGET_VXWORKS)
#define knet_netdev_open(dev) (*dev->if_init)(dev->if_unit)
#endif

#define netdev_stop KNET_IFDEF(stop, if_reset)
#ifdef CONFIG_RG_TARGET_LINUX
#define knet_netdev_stop(dev) (*dev->stop)(dev)
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#define knet_netdev_stop(dev) (*dev->stop)(dev->if_unit)
#endif

#ifdef CONFIG_RG_OS_LINUX
#define netdev_set_mac_func_set(dev, func) dev->set_mac_address = func
#elif defined (CONFIG_RG_OS_VXWORKS)
/* VxWorks doesn't have a set_mac_address function field */
#define netdev_set_mac_func_set(dev, func)    
#endif

/* Access to knet_netdev_t 
 *
 * Implementation notes:
 *
 * To access the fields, the following policy is used:
 * The unified name #define'd with the prefix netdev_, and thus 
 * accessed using . or -> like this:
 * knet_netdev_t dev;
 * dev.netdev_name = kos_alloc(5, 0);
 * strcpy(dev.netdev_name, "ppp");
 * Some of the fields exist the knet_netdev_t of VxWorks but not in Linux 
 * or vice versa. In this case we use a set/get functions to enable the same
 * multi-os solutions for both VxWorks & Linux.
 */
#define netdev_flags KNET_IFDEF(flags, if_flags)
#define netdev_name KNET_IFDEF(name, if_name)
#define netdev_next KNET_IFDEF(next, if_next)
#define netdev_mtu KNET_IFDEF(mtu, if_mtu)
#define netdev_hard_hdr_len KNET_IFDEF(hard_header_len, if_hdrlen)
#define netdev_ift KNET_IFDEF(type, if_type)
#define netdev_hard_addr_len KNET_IFDEF(addr_len, if_addrlen)
#define netdev_if_index KNET_IFDEF(ifindex, if_index)
#define netdev_ioctl KNET_IFDEF(do_ioctl, if_ioctl)
#define netdev_start_xmit KNET_IFDEF(hard_start_xmit, if_start)
#define netdev_start_output KNET_IFDEF(hard_start_xmit, if_output)

/* Head of the global device linked list */
#define KNET_NETDEV_HEAD KNET_IFDEF(dev_base, ifnet)

/* KNET: int (*init) (KNET_NETDEV_INIT_TYPE dev);
 * 
 * VxWorks: int	(*if_init) (int unit);	
 * Linux: int (*init) (struct net_device *dev);
 */    
#define netdev_init KNET_IFDEF(init, if_init)
#define KNET_NETDEV_INIT_TYPE KNET_IFDEF(KNET_LIN_IFDEF(struct net_device *, \
    struct device *), int)

/* Create knet_netdev_t */

/* Linux: alloc and zero knet_netdev_t struct
 * VxWorks: alloc and zero arpcom struct in order to play with MAC address
 */
knet_netdev_t *knet_netdev_zalloc(void);

/* Transmit queue functions */

int knet_netdev_queue_stopped(knet_netdev_t *dev);

#ifdef CONFIG_RG_TARGET_LINUX
#ifdef KNET_LINUX_24
#define knet_netdev_start_queue(dev) netif_start_queue(dev)
#define knet_netdev_stop_queue(dev) netif_stop_queue(dev)
#define knet_netdev_wake_queue(dev) netif_wake_queue(dev)
#else
#define knet_netdev_start_queue(dev)		\
do						\
{						\
    clear_bit(0, (void *)&(dev)->tbusy);	\
    (dev)->start = 1;				\
}						\
while (0)
#define knet_netdev_stop_queue(dev)		\
do						\
{						\
    set_bit(0, (void *)&(dev)->tbusy);		\
    (dev)->start = 0;				\
}						\
while (0)
#define knet_netdev_wake_queue(dev) knet_netdev_start_queue(dev)
#endif
#elif defined(CONFIG_RG_TARGET_VXWORKS)
void knet_netdev_start_queue(knet_netdev_t *dev);
void knet_netdev_stop_queue(knet_netdev_t *dev);
#endif

int knet_netdev_queue_xmit(knet_buf_t *buf);

#ifdef CONFIG_RG_TARGET_VXWORKS
/* dequeue packet from the device queue */
knet_buf_t *knet_netdev_dequeue(knet_netdev_t *dev);

/* Device name
 * 
 * VxWorks:   name is a pointer
 * Linux-2.2: name is a pointer
 * Linux-2.4: name is char []
 */

/* VxWorks: netdev_name contains name and if_unit contains index */
int knet_netdev_separate_name(char *buf, char *name);
#endif

void knet_netdev_copy_name(knet_netdev_t *dev, char *name);

void knet_netdev_free_name(knet_netdev_t *dev);

/* Be careful - in VxWorks the following uses a static buffer for the name */
char *knet_netdev_name_get(knet_netdev_t *dev);

typedef int (*knet_addrlist_loop_cb_t)(struct in_ifaddr *addr, void *arg);

/* Loop on all device's INET addresses. Call cb() for each in_ifaddr found.
 * If cb() returns a non-zero value, stop the iteration and return the 
 * in_ifaddr struct. If all returned zero, return NULL.
 */
struct in_ifaddr *knet_netdev_on_addrlist_loop(knet_netdev_t *dev,
    knet_addrlist_loop_cb_t cb, void *arg);

struct in_addr knet_netdev_ip_get(knet_netdev_t *dev, char *ifa_name);

int knet_netdev_has_ip(knet_netdev_t *dev);

knet_netdev_t *knet_netdev_by_ip_get(struct in_addr ip);

/* check if a given ip is one of dev's addresses */
int knet_is_devs_ip(knet_netdev_t *dev, struct in_addr ip);

/* MAC address:
 * Linux: address is part of device  
 * VxWorks: address is intialized in arpcom
 */

#ifdef CONFIG_RG_TARGET_LINUX
#define KNET_LLADDR(dev) ((dev)->dev_addr)
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#define KNET_LLADDR(dev) ((struct arpcom *)dev)->ac_enaddr
#endif

/* Statistics:
 * VxWorxs - Statistics are in the net device.
 * Linux -   Statistics are in the struct returned from get_stat().
 */
#ifdef CONFIG_RG_TARGET_LINUX
#define KNET_STATISTICS_INC(dev, field, val)				\
do									\
{									\
    if ((dev)->get_stats)						\
    	((dev)->get_stats(dev))->field += val;				\
}									\
while (0)

#elif defined(CONFIG_RG_TARGET_VXWORKS)
#define KNET_STATISTICS_INC(dev, field, val) (*(dev)).field += val
#endif

#define	knet_rx_packets		KNET_IFDEF(rx_packets,	if_ipackets)
#define	knet_rx_errors		KNET_IFDEF(rx_errors,	if_ierrors)
#define	knet_tx_packets 	KNET_IFDEF(tx_packets,	if_opackets)
#define	knet_tx_errors		KNET_IFDEF(tx_errors,	if_oerrors)
#define	knet_collisions		KNET_IFDEF(collisions,	if_collisions)
#define	knet_rx_bytes		KNET_IFDEF(rx_bytes,	if_ibytes)
#define	knet_tx_bytes		KNET_IFDEF(tx_bytes,	if_obytes)
#define	knet_rx_multicast	KNET_IFDEF(multicast,	if_imcasts)
#define	knet_tx_multicast	KNET_IFDEF(multicast,	if_omcasts)
#define	knet_rx_dropped		KNET_IFDEF(rx_dropped,	if_iqdrops)

/* Packet handling hooks */

/* Hook priorities. 
 * Important: this enum just states which priorities exist and their order of
 * importance. The order is specified in the knet module and defined
 * differently for RX and TX
 */
typedef enum {
    KNET_PRI_MULTI_WAN = 1,
    KNET_PRI_FIREWALL = 2,
    KNET_PRI_BRIDGE = 3,
    KNET_PRI_COUNT = 4,
    KNET_PRI_DHCP = 5,
    KNET_PRI_ROUTE = 6,
    KNET_PRI_AUTH1X = 7,
    KNET_PRI_FASTPATH = 8,
    /* Must be last, the number of hook types */
    KNET_NUM_HOOK_TYPES = 8,
} knet_pri_t;

/* Return values of knet RX/TX hook handlers.
 * Important: Be very careful about the return value of a handler because it
 * affects directly the flow of packets in the networks stack.
 */
typedef enum {
    KNET_PKT_HANDLED = 0, /* Packet handled/dropped - no further handling. */
    KNET_PKT_PRI_HANDLED = 1, /* This priority handled - no further hooks of
			       * this priority should be called.
			       */
    KNET_PKT_NOT_HANDLED = 2, /* Packet not handled - all further handling 
			       * allowed. 
			       */
} knet_hook_ret_type_t;

/* TODO: Check conformance to VxWorks solution. */
typedef knet_hook_ret_type_t (*knet_hook_handler_t)(knet_ip_hdr_t **,
    knet_netdev_t *, knet_buf_t **);
typedef knet_hook_ret_type_t (*knet_hook_t)(knet_ip_hdr_t *,
    knet_netdev_t *, knet_buf_t **);

/* KNET hook holders */
extern knet_hook_t net_rx_hook;
extern knet_hook_t net_tx_hook;

int knet_buf_rx_hook_register(knet_hook_handler_t rx_handler,
    knet_pri_t priority);

int knet_buf_tx_hook_register(knet_hook_handler_t tx_handler,
    knet_pri_t priority);

void knet_buf_rx_hook_unregister(knet_hook_handler_t rx_handler,
    knet_pri_t priority);

void knet_buf_tx_hook_unregister(knet_hook_handler_t tx_handler,
    knet_pri_t priority);

int knet_init(void);
void knet_uninit(void);
 
int knet_buf_hook_dev_add(knet_hook_handler_t handler, char *dev,
    knet_pri_t priority, int is_rx);

int knet_buf_hook_dev_del(knet_hook_handler_t handler, char *dev,
    knet_pri_t priority, int is_rx);

int knet_buf_hook_dev_clear(knet_hook_handler_t handler, knet_pri_t priority,
    int is_rx);

int knet_buf_rx_hook_dev_add(knet_hook_handler_t handler, char *dev,
	knet_pri_t priority);

int knet_buf_tx_hook_dev_add(knet_hook_handler_t handler, char *dev,
	knet_pri_t priority);

int knet_buf_rx_hook_dev_del(knet_hook_handler_t handler, char *dev,
	knet_pri_t priority);

int knet_buf_tx_hook_dev_del(knet_hook_handler_t handler, char *dev,
	knet_pri_t priority);

int knet_buf_rx_hook_dev_clear(knet_hook_handler_t handler,
	knet_pri_t priority);

int knet_buf_tx_hook_dev_clear(knet_hook_handler_t handler,
	knet_pri_t priority);

int knet_buf_hook_dev_set(knet_hook_handler_t handler, int val,
    knet_pri_t priority, int is_rx);

int knet_buf_rx_hook_dev_set(knet_hook_handler_t handler, int val,
	knet_pri_t priority);

int knet_buf_tx_hook_dev_set(knet_hook_handler_t handler, int val,
	knet_pri_t priority);

#endif

#ifdef CONFIG_RG_TARGET_VXWORKS
/* Action callback for each rtentry found in the route tree.
 * If return value is zero iteration will continue, else will stop.
 */
typedef int (*knet_route_iter_cb_t)(struct rtentry *rt,void *cb_arg);

/* Call cb() for each rtentry found in the route tree having the correct flags 
 * Return value is zero if iteration was completed (non of the calls to cb()
 * returned a non-zero value). */
int knet_route_table_loop(short flags, knet_route_iter_cb_t cb, void *cb_arg);
#endif 

/* Bytes swap macros for solving endianess issues. Pay attention that these
 * macros are safe when their argument is a function call/complex expression.
 * For example, the following are safe:
 * 
 * knet_swap16(x++);
 * or
 * knet_swap64(f());
 * or
 * knet_swap32(a+b);
 */
#define knet_swap16(__x) \
({ \
    u16 x = (__x); \
    ((u16)( \
    (u16)(((u16)(x) & (u16)0x00ffU) << 8) | \
    (u16)(((u16)(x) & (u16)0xff00U) >> 8))); \
})

#define knet_swap32(__x) \
({ \
    u32 x = (__x); \
    ((u32)( \
    (u32)(((u32)(x) & (u32)0xff000000UL) >> 24) | \
    (u32)(((u32)(x) & (u32)0x00ff0000UL) >>  8) | \
    (u32)(((u32)(x) & (u32)0x0000ff00UL) <<  8) | \
    (u32)(((u32)(x) & (u32)0x000000ffUL) << 24))); \
})

#define knet_swap64(__x) \
({ \
    u64 x = (__x); \
    ((u64)( \
    (u64)(((u64)(x) & (u64)0xff00000000000000ULL) >> 56) | \
    (u64)(((u64)(x) & (u64)0x00ff000000000000ULL) >> 40) | \
    (u64)(((u64)(x) & (u64)0x0000ff0000000000ULL) >> 24) | \
    (u64)(((u64)(x) & (u64)0x000000ff00000000ULL) >>  8) | \
    (u64)(((u64)(x) & (u64)0x00000000ff000000ULL) <<  8) | \
    (u64)(((u64)(x) & (u64)0x0000000000ff0000ULL) << 24) | \
    (u64)(((u64)(x) & (u64)0x000000000000ff00ULL) << 40) | \
    (u64)(((u64)(x) & (u64)0x00000000000000ffULL) << 56))); \
})

#ifdef KOS_ARCH_BIG_ENDIAN
#define knet_cpu_to_le16(x) knet_swap16(x)
#define knet_le16_to_cpu(x) knet_swap16(x)
#define knet_cpu_to_le32(x) knet_swap32(x)
#define knet_le32_to_cpu(x) knet_swap32(x)
#define knet_cpu_to_le64(x) knet_swap64(x)
#define knet_le64_to_cpu(x) knet_swap64(x)
#define knet_be64_to_cpu(x) (x)
#define knet_cpu_to_be64(x) (x)
#elif KOS_ARCH_LITTLE_ENDIAN
#define knet_cpu_to_le16(x) (x)
#define knet_le16_to_cpu(x) (x)
#define knet_cpu_to_le32(x) (x)
#define knet_le32_to_cpu(x) (x)
#define knet_cpu_to_le64(x) (x)
#define knet_le64_to_cpu(x) (x)
#define knet_be64_to_cpu(x) knet_swap64(x)
#define knet_cpu_to_be64(x) knet_swap64(x)
#else
#error KOS_ARCH_XXX_ENDIAN undefined
#endif

#define knet_htonll(x) knet_cpu_to_be64(x)
#define knet_ntohll(x) knet_be64_to_cpu(x)

#endif
