/* LICENSE_CODE JUNGO */

#include "knet_if_ext.h"
#include "kos_chardev.h"

static knet_if_ext_t *knet_if_exts;

static knet_if_ext_t *knet_if_ext_create(knet_netdev_t *dev)
{
    knet_if_ext_t *if_ext;

    if (!(if_ext = kos_alloc(sizeof(knet_if_ext_t), KOS_ALLOC_ATOMIC)))
	return NULL;
    memset(if_ext, 0, sizeof(*if_ext));

    if_ext->next = knet_if_exts;
    knet_if_exts = if_ext;

    if_ext->dev = dev;
    return if_ext;
}

static knet_if_ext_t *knet_if_ext_find(knet_netdev_t *dev)
{
    knet_if_ext_t *if_ext;

    for (if_ext = knet_if_exts; if_ext && if_ext->dev != dev;
	if_ext = if_ext->next);
    return if_ext;
}

void knet_if_ext_free(knet_if_ext_t *ifx)
{
    knet_if_ext_t **if_ext_p, *if_ext;
    kos_interrupt_flags_t fl;

    fl = kos_enter_protected();

    for (if_ext_p = &knet_if_exts; (if_ext = *if_ext_p) && if_ext != ifx;
	if_ext_p = &if_ext->next);
    if (!if_ext)
    {
	kos_exit_protected(fl);
	return;
    }

    *if_ext_p = if_ext->next;
    kos_exit_protected(fl);
    kos_free(if_ext);
}

void knet_if_ext_free_by_netdev(knet_netdev_t *dev)
{
    knet_if_ext_t *if_ext;
    kos_interrupt_flags_t fl;

    fl = kos_enter_protected();
    if ((if_ext = knet_if_ext_find(dev)))
	knet_if_ext_free(if_ext);
    kos_exit_protected(fl);
}

knet_if_ext_t *knet_if_ext_get(knet_netdev_t *dev)
{
    knet_if_ext_t *if_ext;
    kos_interrupt_flags_t fl;

    fl = kos_enter_protected();

    if (!(if_ext = knet_if_ext_find(dev)))
	if_ext = knet_if_ext_create(dev);

    kos_exit_protected(fl);
    return if_ext;
}

static int knet_if_ext_ioctl(kos_chardev_t *context, unsigned int cmd,
    unsigned long arg)
{
    int rc = EINVAL;
    kos_interrupt_flags_t fl;
    knet_netdev_t *dev;
    knet_if_ext_t *devx;
    struct ifreq local_req;

    fl = kos_enter_protected();

    kos_copy_from_user(&local_req, (void *)arg, sizeof(local_req));
    if (!(dev = knet_netdev_get_by_name(local_req.ifr_name)))
	goto exit;
    if (!(devx = knet_if_ext_get(dev)))
    {
	rc = ENOMEM;
	goto exit;
    }
    
    switch (cmd)
    {
    case RG_IOCTL_KNET_IF_GET_EXT:
	local_req.ifr_flags = devx->is_external;
	rc = kos_copy_to_user((void *)arg, &local_req, sizeof(local_req)) ? 
	    EINVAL : 0;
	break;

    case RG_IOCTL_KNET_IF_SET_EXT:
	devx->is_external = local_req.ifr_flags;
	rc = 0;
	break;

    case RG_IOCTL_KNET_IF_SET_PPPOE:
	devx->is_pppoe = local_req.ifr_flags;
	rc = 0;
	break;
	
    default:
	rc = EOPNOTSUPP;
	goto exit;
    }

exit:
    kos_exit_protected(fl);
    return KOS_ERRNO(rc);
}

static kos_reg_chardev_t knet_if_ext_dev = {
    type: KOS_CDT_KNET_IF_EXT,
    ioctl: knet_if_ext_ioctl,
};

void knet_if_ext_init(void)
{
    kos_chardev_register(&knet_if_ext_dev);
}

void knet_if_ext_uninit(void)
{
    while (knet_if_exts)
    {
	knet_if_ext_t *first = knet_if_exts;

	knet_if_exts = knet_if_exts->next;
	kos_free(first);
    }
    kos_chardev_unregister(&knet_if_ext_dev);
}

