/* LICENSE_CODE JUNGO */

#ifndef _KNET_IF_EXT_H_
#define _KNET_IF_EXT_H_

#include "knet.h"

#ifdef __KERNEL__

#ifdef CONFIG_VX_KNET_SYMLINK
#include "knet_vx.h"
#endif

/* a wrapper for knet_netdev_t to add some fields that don't exist in VxWorks.
 * the extension fields here are used even for OSes where they do exist (linux)
 * or where the code can be changed (linux again). */
typedef struct knet_if_ext {
    struct knet_if_ext *next;
    knet_netdev_t *dev;
    void *bridge; /* the bridge that this device is enslaved to */
    int is_external; /* is the device an external device? */
    int is_pppoe; /* Is the device a PPPoE device? */
#ifdef CONFIG_VX_KNET_SYMLINK
    vx_if_pf_t if_pf; /* packet filter structure */
#endif
    void *priv; /* private device context information */
} knet_if_ext_t;

/* get the extension struct for the device.
 * if there isn't any, then create a new struct. */
knet_if_ext_t *knet_if_ext_get(knet_netdev_t *dev);

/* remove the interface extension entry from the entries list */
void knet_if_ext_free(knet_if_ext_t *dev);

/* find the extension entry and free it */
void knet_if_ext_free_by_netdev(knet_netdev_t *dev);

/* [un]init the pool. called from knet_[un]init. */
void knet_if_ext_init(void);
void knet_if_ext_uninit(void);

/* TODO: call knet_if_ext_free[_by_if] on every device removal (linux & vxw) */

#endif
#endif

