/* LICENSE_CODE JUNGO */

/* Utility functions for knet_buf manipulation */

#ifndef _KNET_UTILS_H_
#define _KNET_UTILS_H_

#include <kos/knet.h>

/* Finds the first occurrence of 'what_to_find' in the range 'start_offset' to 
 * 'end_offset' of the buffer 'buf'. 'end_offset' specifies the greatest offset 
 * where the last byte of 'what_to_find' may be found. Returns the offset of the
 * firest occurrence, or -1 if no occurrence were found. */
int knet_buf_find(knet_buf_t *buf, int start_offset, int end_offset, 
    u8 *what_to_find, u32 what_to_find_len);
 
/* Finds the first occurrence of 'what_to_find' in the range 'start_offset' to 
 * 'end_offset' of the buffer 'buf'. The search is case-insensitive.
 * 'end_offset' specifies the greatest offset  where the last byte of
 * 'what_to_find' may be found.
 * Returns the offset of the first occurrence or -1 if not found */
int knet_buf_find_case(knet_buf_t *buf, int start_offset, int end_offset, 
    u8 *what_to_find, u32 what_to_find_len);

/* return 0 if 'len' bytes in knet_buf at offset equals the first 'len' bytes of
 * 'buf' */
int knet_buf_ncmp(knet_buf_t *knet_buf, int offset, u8 *buf, u32 len);

/* return 0 if 'len' bytes in 'knet_buf' at 'offset' equals the first 'len'
 * bytes of 'buf',  case-insensitive */
int knet_buf_case_ncmp(knet_buf_t *knet_buf, u32 offset, u8 *buf, u32 len);

/* locates the first occurrence in the buffer of any of the characters in the
 * string 'delims', starting from offset 'offset'.
 * Returns -1 no characters was found */
int knet_buf_pbrk(knet_buf_t *knet_buf, u32 offset, char *delims);

/* Replaces the segment of length 'old_segment_length' found at 'offset' of 
 * 'buf' with the segment of length 'new_segment_length' pointed to by 'new 
 * segment'. Buffer length will be adjusted if necessary. Returns 0 is
 * successful, -1 otherwise. */
int knet_buf_replace(knet_buf_t **bufp, u32 offset,
    u32 old_segment_len, u8 *new_segment, u32 new_segment_len);

/* reads a number from knet_buf at 'offset', 'byte_read' is the number of digits
 * read (0 will indicate failure) */ 
int knet_buf_uint_read(knet_buf_t *knet_buf, u32 offset, int *byte_read);

/* copy data between start_offset and end_ofset and call inet_ntoa 
 * addr will set to the ip that was read. Returns 0 if the buffer hold a valid
 * ip, else -1 */
int knet_buf_aton(knet_buf_t *buf, int start_offset, int end_offset,
    in_addr_t *addr);

/* count rx non-unicast device statistics
 * multicast packets are supposed to be counted by the device but some devices
 * don't count it */
knet_hook_ret_type_t knet_stats_count_rx(knet_ip_hdr_t **ip, knet_netdev_t *dev,
    knet_buf_t **kbuf);

/* count tx non-unicast device statistics
 * multicast packets are supposed to be counted by the device but some devices
 * don't count it */
knet_hook_ret_type_t knet_stats_count_tx(knet_ip_hdr_t **ip, knet_netdev_t *dev,
    knet_buf_t **kbuf);

#ifdef __KERNEL__
/* Used for debugging, print a packet from a buffer */
void knet_print_buffer(char *prefix, knet_buf_t *buf);
#endif

#endif
