/* LICENSE_CODE JUNGO */

/* OS library and wrappers for kernel mode.
 *
 * Prefix KOS stands for Kernel OS wrappers.
 */
#ifndef _KOS_H_
#define _KOS_H_
/* All main include files will be accessed from kos.h */

/* CONFIG_RG_TARGET_XXX is passed as a compilation flag because in some
 * occasions we have CONFIG_RG_OS=VXWORKS and we are compiling for local
 */

#ifdef CONFIG_RG_TARGET_LINUX
#ifndef __KERNEL__
#error kos.h should be included only from kernel
#endif
#include <linux/config.h> /* Do not include rg_config.h directly because when
			   * compiling inside linux, might not have pkg/include
			   * in the include path.
			   */
#include <sys/types.h>
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/poll.h>
#include <asm/uaccess.h> /* copy_to/from_user */
#include <linux/slab.h> /* kmalloc/kfree */
#include <linux/mm.h> /* get_free_page */
#include <asm/param.h> /* HZ */
#include <linux/timer.h> /* add_timer and Co. */
#include <linux/random.h> /* Random number generation. */
#include <asm/atomic.h> /* atomic operations */
#include <linux/interrupt.h> /* in_interrupt */
#include <linux/time.h> /* do_gettimeofday */
#include <linux/errno.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,4,0)
#include <asm/spinlock.h>
#else
#include <linux/spinlock.h> /* spinlock operations */
#endif
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#include <rg_types.h>
#include <rg_config.h>
#include <vxWorks.h>
#include <memLib.h> /* memalign, malloc, free */
#include <intLib.h> /* intLock, untUnlock */
#include <tickLib.h> /* tickGet */
#include <sysLib.h> /* sysClkRateGet */
#include <stdlib.h> /* malloc, free */
#include <string.h> /* memmove */
#include <wdLib.h> /* watchdog timers */
#include <net/unixLib.h> /* spinlocks */
#include <time.h> /* clock_gettime */
#include <logLib.h>
#include <errno.h>
#include <selectLib.h>
#else
#error No operating system defined
#endif

#include "kos_common.h"

/* General kos initialization routine */
int kos_init(void);
void kos_uninit(void);

#ifndef offsetof
#define offsetof(t,m) (int)((&((t *)0L)->m))
#endif

#ifndef _RG_DEF_H_
/* DOTTED_IP_LEN and MAX_ERROR_MSG_LEN are defined in rg_def.h which we do not
 * want to include from kernel mode.
 */
#define DOTTED_IP_LEN 16
#define MAX_ERROR_MSG_LEN 255
#endif

/* buffer len needed to hold integers */
#define MAX_NUM_LEN 16
#ifndef MAX
#define MAX(a,b)	((a) > (b) ? (a) : (b))
#endif
#ifndef MIN
#define MIN(a,b)	((a) < (b) ? (a) : (b))
#endif
#define	is_digit(x)	(((u8)(x) >= '0') && ((u8)(x) <= '9'))
#define	is_upper(x)	(((u8)(x) >= 'A') && ((u8)(x) <= 'Z'))
#define	is_lower(x)	(((u8)(x) >= 'a') && ((u8)(x) <= 'z'))
#define	is_alpha(x)	(is_upper(x) || is_lower(x))
#define is_ascii(c)  (c > 0)
#define is_space(c)  (c == ' ' || c == '\f' || c == '\n' || c == '\r' ||\
    c == '\t' || c == '\v')
#define is_xlower(c) (is_digit(c) || (c >= 'a' && c <= 'f'))
#define is_xupper(c) (is_digit(c) || (c >= 'A' && c <= 'F'))
#define is_xdigit(c) (is_digit((c)) || ((c) >= 'a' && (c) <= 'f') ||\
    ((c) >= 'A' && (c) <= 'F'))
#define to_lower(c)	(is_upper(c) ? (c) - 'A' + 'a' : (c))
#define	to_upper(x)	(is_lower(x) ? (x) - 'a' + 'A' : (x))
#define to_ascii(c)  (c & 0x7f)

/* Some compilers do not know how to optimise inlines... */
#ifdef CONFIG_GCC_CAN_INLINE
#define INLINE extern __inline__
#else
#define INLINE static
#endif

/* Kernel/usermode operations
 * Note that kos_copy_from/to_user() return int according to the Linux
 * convention (the copy_from_user() return value), instead of a 
 * void * pointer as memcpy() does.
 * 
 * kos_get/put_user() are defined as macros, since they assign
 * a value to an _integer_, not to a pointer.
 */
u32 kos_copy_from_user(void *to, const void *from, u32 n);

u32 kos_copy_to_user(void *to, const void *from, u32 n);

#ifdef CONFIG_RG_TARGET_LINUX
#define kos_get_user(x,p) get_user((x),(p))
#define kos_put_user(x,p) put_user((x),(p))
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#define kos_get_user(x,p) kos_copy_from_user(&(x), (p), sizeof(*(p)))
#define kos_put_user(x,p) kos_copy_to_user((p), &(x), sizeof(x))
#endif

/* Memory allocation
 *
 * Note: In VxWorks, the memory allocation functions cannot be called
 * in interrupt context (ISRs, timer callbacks) for sizes larger than 
 * 512 bytes.
 */
#define KOS_ALLOC_KERNEL 0x01 /* General allocation */
#define KOS_ALLOC_ATOMIC 0x02 /* Allocation in interupt context */
#define KOS_ALLOC_PROTECTED 0x04 /* In linux: the same as KOS_ALLOC_KERNEL
				  * In VxWorks: Used inside sections protected
				  * from interrupts, and thus cannot be
				  * blocking. malloc() and free() are
				  * considered blocking.
				  */

void *__kos_alloc(size_t size, int flags, char *func);
#define kos_alloc(size, flags) __kos_alloc((size), (flags), __FUNCTION__)

void kos_free(void *ptr);

/* Time
 * Access to the kernel's tick counter (jiffies in linux) and
 * timer rate (HZ) is provided by 
 * u32 kos_ticks_get();
 * int kos_tick_rate_get();
 */
#ifdef CONFIG_RG_TARGET_LINUX
extern unsigned long volatile jiffies;
#define kos_ticks_get() jiffies
#define kos_tick_rate_get() HZ
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#define kos_ticks_get() tickGet()
#define kos_tick_rate_get() sysClkRateGet()
#endif

/* Time from boot that is not affected by changes in Time/Date from user */
#define kos_sec_from_boot (kos_ticks_get()/kos_tick_rate_get())

/* Timer callbacks
 * In both OSes, the callback is called in interrupt time, so the same
 * restrictions for memory allocations (see memory section) apply here.
 * The callback is called only once.
 */

#ifdef CONFIG_RG_TARGET_VXWORKS
/* The following is a wrapper needed because VxWorks wdLib does not allow us
 * to examine the status of a watch dog.
 */
typedef struct {
    WDOG_ID id;
    u32 expires; /* In ticks */
} vx_kos_timer_t;
#endif

typedef KOS_IFDEF(struct timer_list, vx_kos_timer_t) kos_timer_t;
typedef KOS_IFDEF(void, int) kos_timer_cb_ret_t;
typedef kos_timer_cb_ret_t (*kos_timer_callback_t)(unsigned long);
#define KOS_TIMER_CB_RETURN(val) return KOS_IFDEF(, val)

/* Allocates and initializes timer */
int kos_timer_init(kos_timer_t *t);

/* After the code has finished working with a timer, it should call this
 * function, since in VxWorks the timer structure is freed by the kernel.
 *
 * If the timer was allocated by kos_alloc(), the code should call
 * kos_free() as well, after calling this function.
 */
int kos_timer_uninit(kos_timer_t *t);

/* Sets timer callback parameters and informs the kernel
 * to consider the timer when running the timers queue.
 * Warning: The timer callback exectues in interrupt context, 
 * so be carefull of what system calls it does.
 */
int kos_timer_start(kos_timer_t *t, int expires, kos_timer_callback_t cb, 
    int data);

/* Removes the timer from the timer queue, cancels the callback. */
int kos_timer_stop(kos_timer_t *t);

/* Timer active
 * Is timer still waiting to get fired: 0 not active, positive - active 
 * For Linux we need to check it ourselves.
 * For VxWorks there is a field holding that data.
 */
int kos_timer_active(kos_timer_t *t);

/* timer modification
 * For Linux there is a function for that.
 * For VxWorks simply starting it again modifies the timer if alredy exist.
 */
int kos_timer_mod(kos_timer_t *t, int expires, kos_timer_callback_t cb, 
    int data);

#ifdef CONFIG_RG_TARGET_LINUX
typedef struct timeval kos_timeval_t;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
typedef struct {
    time_t tv_sec;
    long tv_usec;
} kos_timeval_t;
#endif

/* Returns the best possible estimate of the time. */
void kos_gettime(kos_timeval_t *t);

/* Logging 
 *
 * Implementation notes:
 *
 * rg_error() perfectly suits all the needs of the logging, it should be used.
 * In Linux, we will call printk() with mapping of the 'severities' to kernel
 * levels, in VxWorks - we will call logMsg().
 */

/* TODO: after kernel support for rg_error is finished, provide a normal
 * prototype here.
 * extern int rg_error(int severity, char *fmt, ...); 
 */

#ifdef CONFIG_RG_TARGET_LINUX
#define kos_print printk
#elif defined(CONFIG_RG_TARGET_VXWORKS)
/* logMsg is limited in its arguments number! */
#define MAX_LOG_ARGS 6
void kos_print(char *fmt, ...);
#endif

#ifdef CONFIG_RG_TARGET_LINUX
#define KOS_PANIC(args...) panic(args)
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#define KOS_PANIC(args...) do {	\
    kos_print(args); \
    exit(1); \
} while (0)
#endif

/* This enforces a rate limit: No more than one message (print)
 * every 5 secs to make a denial-of-service attack impossible.
 * All warning kos_print()s should be guarded by this function.
 *
 * NOTE: will not be implemented right now.
 */ 
int kos_net_print_ratelimit(void);
  
/* Interrupt flags handling */
typedef KOS_IFDEF(unsigned long, int) kos_interrupt_flags_t;

kos_interrupt_flags_t kos_save_flags_cli(void);

void kos_restore_flags(kos_interrupt_flags_t flags);

/* Enter a protected from interrupts section in VxWorks (do nothing in Linux)
 * Used to solve reentrancy problems special to VxWorks.
 */
kos_interrupt_flags_t kos_enter_protected(void);

void kos_exit_protected(kos_interrupt_flags_t flags);

int kos_in_interrupt(void);

/* Atomic operations. */
typedef KOS_IFDEF(atomic_t, struct { int counter; }) kos_atomic_t;

void kos_atomic_set(volatile kos_atomic_t *v, int i);

int kos_atomic_read(volatile kos_atomic_t *v);

void kos_atomic_inc(volatile kos_atomic_t *v);

void kos_atomic_dec(volatile kos_atomic_t *v);

/* Spinlocks
 * For Linux, we use spinlocks which save/restore interrupt flags.
 * For VxWorks, we don't use standard splnet/imp from unixLib.o since
 * they use semaphores, which cannot be used in interrupt time.
 */
typedef kos_interrupt_flags_t kos_spinlock_t;
#define kos_splimp kos_save_flags_cli
#define kos_splnet kos_save_flags_cli
#define kos_splx kos_restore_flags

/* Random
 * For VxWorks, we use the libc function.
 * For Linux, we use the random driver API.
 */
int kos_random(void);

/* Pooling
 * We don't use locking in order to avoid wasting time, the worst that will
 * happen by using atomic reads is that we will have a soft limit instead of an
 * hard one. That is, we can get into situations where there is at most a few
 * more than the max size specified. This is caused because we use an atomic
 * read/write and not atomic compare.
 */
struct kos_pool {
    int pool_size_max;
    kos_atomic_t pool_size;
    unsigned int alloc_size;
};

void kos_pool_init(struct kos_pool *pool, unsigned int size);

void *kos_pool_get(struct kos_pool *pool);

void kos_pool_put(struct kos_pool *pool, void *p);

void kos_pool_max_set(struct kos_pool *pool, int sz);

int kos_pool_max_get(struct kos_pool *pool);

/* In Linux, we return -EINVAL (or other errno values).
 * In VxWorks, we return EINVAL (positive).
 */
#define KOS_ERRNO(e) KOS_IFDEF (-(e), (e))
#define KOS_ENOIOCTLCMD KOS_IFDEF(ENOIOCTLCMD, EINVAL)

/* wait queues type & usage has changed in Linux 2.4 */
#ifdef CONFIG_RG_TARGET_LINUX
#define kos_wake_up_int_read kos_wake_up_interruptible
#define kos_wake_up_int_write kos_wake_up_interruptible
#if ( LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) )
#define kos_poll_wait(filp, wait_address, p) poll_wait(filp, *(wait_address), p)
#else
#define kos_poll_wait(filp, wait_address, p) poll_wait(filp, wait_address, p)
#endif
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#define kos_poll_wait(dev, wait_address, p) selNodeAdd((wait_address), (p))
#define kos_wake_up_int_read(x) selWakeupAll(x, SELREAD)
#define kos_wake_up_int_write(x) selWakeupAll(x, SELWRITE)
#endif

typedef KOS_IFDEF(KOS_LIN_IFDEF(struct __wait_queue_head *,
    struct wait_queue *), SEL_WAKEUP_LIST) kos_wait_queue_t;

typedef KOS_IFDEF(struct poll_table_struct *, SEL_WAKEUP_NODE *)
    kos_poll_node_t;

void kos_mod_count_inc(void);

void kos_mod_count_dec(void);

void kos_wake_up_interruptible(kos_wait_queue_t *q);

void kos_waitq_init(kos_wait_queue_t *waitq);

/* kos_select:
 * 
 * Linux:  select returns a mask with bits for read/write, and the system
 * wakes up the wait queue according to mask value.
 *
 * VxWorks: The system tells select whether its a read or write select, and
 * the select implementation wakes the queue by itself.
 */
unsigned int kos_select(kos_poll_node_t poll_node, int read, int write);

int kos_poll_node_del(kos_wait_queue_t *waitq, kos_poll_node_t poll_node);

/* Convert a string to an unsigned long according to base (if zero the function
 * figures the base by itself according to prefix).
 * endp, if not NULL, will point to the character after the last translated.
 */
unsigned long kos_simple_strtoul(char *cp, char **endp, unsigned int base);

#endif

