/* LICENSE_CODE JUNGO */

#ifndef _KOS_CHARDEV_H_
#define _KOS_CHARDEV_H_

#include <rg_ioctl.h>


typedef enum {
    KOS_CDT_DHCP = 1,
    KOS_CDT_BRIDGE = 2,
    KOS_CDT_IPF = 3,
    KOS_CDT_IPF_NAT = 4,
    KOS_CDT_IPF_STATE = 5,
    KOS_CDT_IPF_AUTH = 6,
    KOS_CDT_IPV4 = 7,
    KOS_CDT_BROUTE = 8,
    KOS_CDT_USFS = 9,
    KOS_CDT_CHWAN = 10,
    KOS_CDT_LOG = 11,
    KOS_CDT_KNET_IF_EXT = 12,
    KOS_CDT_RAUL = 13,
    KOS_CDT_ARPRESD = 14,
    KOS_CDT_KLEDS = 15,
    KOS_CDT_HW_BUTTONS = 16,
    KOS_CDT_AUTH1X = 17,
} kos_chardev_type_t;

#ifdef __KERNEL__
#include <kos/kos.h>

#ifdef CONFIG_RG_TARGET_LINUX
#define KOS_CHARDEV_GET_FILP(context) ((context)->filp)
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#define KOS_CHARDEV_GET_FILP(context) NULL
#endif

struct kos_reg_chardev_t;

/* The char device which is passed as context on each ops call */
typedef struct {
    struct kos_reg_chardev_t *reg_dev;
#ifdef CONFIG_RG_TARGET_LINUX
    struct file *filp;
#endif
    void *priv;
} kos_chardev_t;

/* A registration template */
typedef struct kos_reg_chardev_t {
    struct kos_reg_chardev_t *next;
    kos_chardev_type_t type;
    int (*open)  (kos_chardev_t *context);
    int (*close) (kos_chardev_t *context);
    int (*read)  (kos_chardev_t *context, char *buf, int len);
    int (*write) (kos_chardev_t *context, const char *buf, int len);
    int (*ioctl) (kos_chardev_t *context, unsigned int cmd, unsigned long arg);
    unsigned int (*select)(kos_chardev_t *context, kos_poll_node_t poll_node);
    int (*unselect)(kos_chardev_t *context, kos_poll_node_t poll_node);
    kos_atomic_t use_count;
} kos_reg_chardev_t;

/* Register a new device. Return 0 on success, negative value
 * on failure.
 */
int kos_chardev_register(kos_reg_chardev_t *dev);

/* Unregister a device. Return 0 on success, negative value
 * on failure.
 */
int kos_chardev_unregister(kos_reg_chardev_t *dev);

int kos_chardev_init(void);
int kos_chardev_uninit(void);
#endif

#endif
