/** \file mic.h
 * \brief MIC calculation interface
 *
 * This file includes all implimentation of the RSN manager API and 
 * internal functions.
 *
 * \see RsnMgr.h, rsn_api.h
*/

/***************************************************************************
**+----------------------------------------------------------------------+**
**| Michael.h    Reference implementation for Michael                    |**
**|                                                                      |**
**| Copyright (c) 2001 by MacFergus BV                                   |**
**| All rights reserved,                                                 |**
**|                                                                      |**
**| A Michael object implements the computation of the Michael MIC.      |**
**|                                                                      |**
**| Conceptually, the object stores the message to be authenticated.     |**
**| At construction the message is empty.                                |**
**| The append() method appends bytes to the message.                    |**
**| The getMic() method computes the MIC over the message and returns	 |**
**| the result. 														 |**
**| As a side-effect it also resets the stored message                   |**
**| to the empty message so that the object can be re-used               |**
**| for another MIC computation.                                         |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*		MODULE:	mic.h							 		   				   */
/*    PURPOSE:	MIC Base class API										   */
/*																	 	   */
/***************************************************************************/

#ifndef _MIC_H_
#define _MIC_H_

/*#include "pform_types.h"*/
#include "wpaIfc.h"

typedef struct 
{
	UINT8				micValue[MIC_LEN];
	UINT8				micKey[MICHAEL_KEY_LEN];
	UINT32			    micDataLength;
	UINT8				scratchpad[4];
} micHandle_t;



void mic_init(micHandle_t *pHandle);

void mic_setKey(micHandle_t *pHandle, UINT8 *pKey);

STATUS mic_addBuffer(micHandle_t *pHandle, UINT8 *pBuffer, UINT32 bufferLen);

STATUS mic_getMic(micHandle_t *pHandle, UINT8* pMicValue);



#endif /* _MIC_H_ */