/* LICENSE_CODE JUNGO */

#ifndef _RADIUSC_H_
#define _RADIUSC_H_

#include <radius/radiusct.h>

#define RADIUS_AUTHENTICATOR_LEN 16
#define AUTH_STRING_LEN 253

/* RADIUS Access Codes */
typedef enum {
    ACCESS_REQUEST = 1,
    ACCESS_ACCEPT = 2,
    ACCESS_REJECT = 3,
    ACCESS_CHALLENGE = 11,
} radius_codes_t;

typedef struct radius_client_session_t radius_client_session_t;
typedef struct radius_pkt_t radius_pkt_t;

/* RADIUS Attribute */
typedef struct radius_pkt_attr_t {
    struct radius_pkt_attr_t *next;
    u8 type;
    u8 len; /* of entire attribute including type, len and value */
    u8 value[AUTH_STRING_LEN];
} radius_pkt_attr_t;

/* Received packet callback */
typedef void (*radius_pkt_recv_cb_t)(void *context, radius_pkt_t *pkt);

/* Open a RADIUS session */
radius_client_session_t *radius_client_session_open(radius_client_t *client, 
    radius_pkt_recv_cb_t pkt_recv_cb, void *context);

/* Close a RADIUS session */
void radius_client_session_close(radius_client_session_t *session);

/* Toolkit for building RADIUS packets */
/* Create an RADIUS empty packet */
radius_pkt_t *radius_packet_create(void);

/* Send a RADIUS packet */
void radius_client_packet_send(radius_client_session_t *session, 
    radius_pkt_t* pkt);

/* Free a RADIUS packet */
void radius_packet_free(radius_pkt_t *pkt);

/* Add an attribute to an existing packet. Adds the attribute at the end
 * of the packet. value_len is length of attribute without type & length */
int radius_packet_attr_add(radius_pkt_t *pkt, int type, u8 *value, 
    u8 value_len);

/* Get next attribute of the defined type, NULL means first attribute */
radius_pkt_attr_t *radius_packet_attr_get(radius_pkt_t *pkt, int type, 
    radius_pkt_attr_t *last_attr);
 
/* Get code of RADIUS packet */
radius_codes_t radius_packet_code_get(radius_pkt_t *pkt);

/* standard RADIUS attribute-value pairs */

#define	PW_USER_NAME			1	/* string */
#define	PW_USER_PASSWORD		2	/* string */
#define	PW_CHAP_PASSWORD		3	/* string */
#define	PW_NAS_IP_ADDRESS		4	/* ipaddr */
#define	PW_NAS_PORT			5	/* integer */
#define	PW_SERVICE_TYPE			6	/* integer */
#define	PW_FRAMED_PROTOCOL		7	/* integer */
#define	PW_FRAMED_IP_ADDRESS		8	/* ipaddr */
#define	PW_FRAMED_IP_NETMASK		9	/* ipaddr */
#define	PW_FRAMED_ROUTING		10	/* integer */
#define	PW_FILTER_ID		        11	/* string */
#define	PW_FRAMED_MTU			12	/* integer */
#define	PW_FRAMED_COMPRESSION		13	/* integer */
#define	PW_LOGIN_IP_HOST		14	/* ipaddr */
#define	PW_LOGIN_SERVICE		15	/* integer */
#define	PW_LOGIN_PORT			16	/* integer */
#define	PW_OLD_PASSWORD			17	/* string */ /* deprecated */
#define	PW_REPLY_MESSAGE		18	/* string */
#define	PW_LOGIN_CALLBACK_NUMBER	19	/* string */
#define	PW_FRAMED_CALLBACK_ID		20	/* string */
#define	PW_EXPIRATION			21	/* date */ /* deprecated */
#define	PW_FRAMED_ROUTE			22	/* string */
#define	PW_FRAMED_IPX_NETWORK		23	/* integer */
#define	PW_STATE			24	/* string */
#define	PW_CLASS			25	/* string */
#define	PW_VENDOR_SPECIFIC		26	/* string */
#define	PW_SESSION_TIMEOUT		27	/* integer */
#define	PW_IDLE_TIMEOUT			28	/* integer */
#define	PW_TERMINATION_ACTION		29	/* integer */
#define	PW_CALLED_STATION_ID            30      /* string */
#define	PW_CALLING_STATION_ID           31      /* string */
#define	PW_NAS_IDENTIFIER		32	/* string */
#define	PW_PROXY_STATE			33	/* string */
#define	PW_LOGIN_LAT_SERVICE		34	/* string */
#define	PW_LOGIN_LAT_NODE		35	/* string */
#define	PW_LOGIN_LAT_GROUP		36	/* string */
#define	PW_FRAMED_APPLETALK_LINK	37	/* integer */
#define	PW_FRAMED_APPLETALK_NETWORK	38	/* integer */
#define	PW_FRAMED_APPLETALK_ZONE	39	/* string */
#define	PW_CHAP_CHALLENGE               60      /* string */
#define	PW_NAS_PORT_TYPE                61      /* integer */
#define	PW_PORT_LIMIT                   62      /* integer */
#define PW_LOGIN_LAT_PORT               63      /* string */
#define PW_EAP_MESSAGE                  79      /* string */
#define PW_MESSAGE_AUTHENTICATOR        80      /* string */

/* Vendor Specific attributes - PW_VENDOR_SPECIFIC */

typedef struct {
    u32 id;
    u8 type;
    u8 len;
    u8 data[0];
} __attribute__ ((packed)) radius_pkt_vendor_t;

/* Get key from MS-MPPE-Recv-Key and MS-MPPE-Send-Key Microsoft Vendor 
 * attributes, 0 - Success */
int radius_packet_ms_mppe_key_get(radius_client_session_t *session, 
    radius_pkt_t *pkt, u8 **key, int *key_len);

#define PW_VENDOR_MICROSOFT 311
#define PW_MS_MPPE_SEND_KEY 16
#define PW_MS_MPPE_RECV_KEY 17

#endif
