/* LICENSE_CODE JUNGO */

#define OS_INCLUDE_STD
#define OS_INCLUDE_STDARG
#define OS_INCLUDE_IOCTL
#define OS_INCLUDE_SOCKET
#define OS_INCLUDE_INET
/*#include "rg_error.h"*/
#include <os_includes.h>
#include <os_libc_extra.h>
#include <estream.h>
#include <misc_funcs.h>
#include <net_utils.h>
#include <syscalls.h>
#include "str.h"
#include "estream_ssl.h"

#define rg_error(level,str,args...) printf(str,##args)


#ifdef CONFIG_RG_TARGET_VXWORKS
#include "vx_net_fixup.h"
#endif

/* Internal structures */

/* list-glue struct for eserver's list of streams.
 * streams that are opened through eserver are on a linked list
 * of that server, so that it can close all of them when the server
 * itself is closing [optionally]
 */
typedef struct eserver_stream_t {
    estream_t *estream;

    struct eserver_stream_t **prev;
    struct eserver_stream_t *next;
} eserver_stream_t;

int (*estream_ssl_open_func)(estream_t *e, void *ssl_ctx);

/* socket functions */

static void estream_update(estream_t *estream);
static void estream_connect_done(estream_t *estream);

int sock_valid(int sock)
{
    struct sockaddr_in sa;
    int sa_len = sizeof(sa);
    if (getsockname(sock, (struct sockaddr *)&sa, &sa_len)<0)
    {
	rg_error_f(LERR, "socket %d failed getsockname() - %m", sock);
	return -1;
    }
    return 0;
}

int sock_accept(int sock)
{
    struct sockaddr_in sa;
    int sa_len = sizeof(sa);
    int con_sock = -1;

    if (sock_valid(sock)<0)
	goto Error;
    while ((con_sock = accept(sock, (struct sockaddr *)&sa, &sa_len))<0)
    {
	if (errno!=EINTR)
	{
	    rg_error_f(LERR, "socket %d failed accept() - %m", sock);
	    goto Error;
	}
    }
    if (sock_set_sockopt(con_sock, 0)<0)
	goto Error;
    return con_sock;

Error:
    if (con_sock>=0)
	close(con_sock);
    return -1;
}

static char *eserver_get_desc(eserver_t *eserver)
{
    static char desc[30];

    if (eserver->port)
        sprintf(desc, "%d port %d", eserver->sock, ntohs(eserver->port));
    else
	sprintf(desc, "%d", eserver->sock);
    return desc;
}

/* Unlink stream that is being closed from the linked list of server's
 * streams and free the helper list-glue struct
 */
static void eserver_unlink_stream(void *data)
{
    eserver_stream_t *ess = (eserver_stream_t*)data;
    if (ess->prev)
	*(ess->prev) = ess->next;
    if (ess->next)
	ess->next->prev = ess->prev;
    free(ess);    
}

static void eserver_read_efd(int sock, void *o)
{
    eserver_t *eserver = (eserver_t *)o;
    estream_t *estream = NULL;
    eserver_stream_t *ess = NULL;

    if (eserver->closing)
	return;

    estream = estream_accept(eserver->sock, eserver->flags, eserver->func);
    /* did we manage to open the estream */
    if (!estream)
    {
	rg_error_f(LERR, "%s: failed estream_accept()",
	    eserver_get_desc(eserver));
	return;
    }

    /* create list-glue struct */
    ess = zalloc_l(sizeof(eserver_stream_t));
    if (!ess)
	goto Exit;

    /* Add this stream to the list of linked streams */
    ess->estream = estream;
    ess->next = eserver->streams;
    ess->prev = &eserver->streams;
    *(ess->prev) = ess;
    if (ess->next)
	ess->next->prev = &ess->next;

    /* "subscribe" to the on-estream-close notify */
    estream_set_data(estream, ess, eserver_unlink_stream);
    /* In the case estream_push fails, the ess will still be freed from the
     * eserver_unlink_stream func, that is set by estream_set_data
     * and will be called from estream_close
     */
    if (estream_push(estream))
	goto Exit;

    rg_error_f(LDEBUG, "%s: new connection socket %d",
	eserver_get_desc(eserver), estream->sock);

    /* then set the data for the callback */
    estream_set_data(estream, eserver, NULL);
    
    return;
    
Exit:
    if (estream)
	estream_close(estream);
}

eserver_t *eserver_open(int sock, estream_flags_t flags,
    estream_connect_func_t func, void *data)
{
    eserver_t *eserver;
    
    eserver = zalloc_l(sizeof(eserver_t));
    if (!eserver)
	return NULL;
    eserver->sock = sock;
    eserver->func = func;
    eserver->data = data;
    eserver->flags = flags;
    sock_set_sockopt(sock, 0);
    event_fd_set(eserver->sock, EFD_READ, eserver_read_efd, eserver);
    return eserver;
}

void eserver_close(eserver_t *eserver)
{
    eserver_stream_t *ess, *tmp;
    rg_error_f(LDEBUG, "%d: closing", eserver->sock);
    eserver->closing = 1;
    event_fd_clr(eserver->sock, EFD_READ);
    if (eserver->flags & ESTREAM_CLOSE_FD)
	close(eserver->sock);
    /* Unlink the server's streams from the list and close if needed */
    for (ess = eserver->streams; ess; ess = tmp)
    {
	tmp = ess->next;
	ess->next = NULL;
	ess->prev = NULL;
	/* the ess struct will be freed from the eserver_unlink_stream
	 * func, which will be called from estream_close
	 */
	if (ess->estream && !(eserver->flags & ESERVER_INDEPENDENT_STREAMS))
	    estream_close(ess->estream);
    }
    event_timer_set(0, free, eserver);
}

void eserver_estreams_close(eserver_t *eserver, estream_filter_func_t filter,
    void *data)
{
    eserver_stream_t *ess, *next;
    for (ess = eserver->streams; ess; ess = next)
    {
	next = ess->next;
	if (ess->estream && (*filter)(ess->estream, data, 0))
	    estream_close(ess->estream);
    }
}

eserver_t *eserver_open_port(u16 port, estream_flags_t flags,
    estream_connect_func_t func, void *data)
{
    return eserver_open_addr(INADDR_ANY, port, flags, func, data);
}

eserver_t *eserver_open_addr(u32 addr, u16 port, estream_flags_t flags,
    estream_connect_func_t func, void *data)
{
    int sock = -1;
    struct sockaddr_in sa;
    eserver_t *eserver;
    int opt;

    if ((sock = socket(AF_INET, SOCK_STREAM, 0))<0)
    {
	rg_error_f(LERR, "failed socket() - %m");
	goto Error;
    }
    opt = 1;
    if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&opt, sizeof(opt))<0)
    {
	rg_error_f(LERR, "failed setsockopt() - %m");
	goto Error;
    }
	
    MZERO(sa);
    sa.sin_family = AF_INET;
    sa.sin_addr.s_addr = addr;
    sa.sin_port = port;
    if (bind(sock, (struct sockaddr *)&sa, sizeof(sa))<0)
    {
	rg_error_f(LERR, "failed bind() ip %x port %d: - %m", ntohl(addr),
	    ntohs(port));
	goto Error;
    }
    if (listen(sock, 5)<0)
    {
	rg_error_f(LERR, "failed listen() - %m");
	goto Error;
    }
    eserver = eserver_open(sock, flags | ESTREAM_CLOSE_FD, func, data);
    if (!eserver)
    {
	rg_error_f(LERR, "failed eserver_open()");
	goto Error;
    }
    eserver->port = port;
    rg_error_f(LDEBUG, "socket %d opened on ip %s port %d", eserver->sock,
	inet_ntoa(sa.sin_addr), ntohs(eserver->port));
    return eserver;
	
Error:
    if (sock>=0)
	close(sock);
    return NULL;
}

estream_t *estream_connect_tcp(estream_flags_t flags,
    estream_connect_func_t func, u32 ip, u16 port)
{
    return estream_connect_tcp_full(-1, flags, func, ip, port, 0, 0);
}

estream_t *estream_open()
{
    estream_t *estream = zalloc_l(sizeof(estream_t));
    if (!estream)
	return NULL;
    estream->sock = -1;
    return estream;
}

static void estream_timeout_et(void *o)
{
    estream_t *estream = (estream_t *)o;

    if (estream->closing)
	return;
    rg_error_f(LDEBUG, "socket %d: timed out %d ms on %s", estream->sock,
	estream->timeout, estream->connected ? "data" : "connect");
    if (estream->timeout_func)
    {
	estream->timeout_func(estream, NULL, 0);
	if (estream->closing)
	    return;
	estream_update(estream);
    }
    else
	estream_close(estream);
}

static void estream_read_et(void *o);
static void estream_connect_done_et(void *);

void socket_close(int fd)
{
#ifndef CONFIG_RG_TARGET_VXWORKS
    close(fd);
#else
    socket_close_vx(fd);
#endif
}

void estream_close(estream_t *estream)
{
    estream->closing = 1;
    rg_error_f(LDEBUG, "closing socket %d", estream->sock);

    if (estream->sock>=0)
    {
    	event_fd_clr(estream->sock, EFD_ALL);
	if (estream->flags & ESTREAM_CLOSE_FD)
	    socket_close(estream->sock);
    }

    event_timer_del(estream_connect_done_et, estream);
    event_timer_del(estream_timeout_et, estream);
    event_timer_del(estream_read_et, estream);
    
    while (1)
    {
        estream_stack_t *stack = estream->stack;

        if (estream->data_free_func && estream->data)
            estream->data_free_func(estream->data);
        if (!stack)
            break;
        estream->data_free_func = stack->data_free_func;
        estream->data = stack->data;
        estream->stack = stack->next;
        free(stack);
    }

    nfree(estream->in_buf);
    nfree(estream->out_buf);
    if (estream->ssl)
	estream->ssl->close(estream);

    event_timer_set(0, free, estream);
}

static void estream_connect_except_efd(int sock, void *o)
{
    estream_t *estream = (estream_t *)o;

    rg_error_f(LINFO, "socket %d failed connect", estream->sock);

    estream->connect_func(estream, 0);
    if (estream->closing)
	return;

    estream_close(estream);
}

static int estream_got_in_buf(estream_t *estream)
{
    return estream->in_req &&
	(estream->in_req<=estream->in_recv ||
	(estream->in_recv &&
	(estream->in_mode & (ESTREAM_PARTIAL|ESTREAM_LINE|ESTREAM_FOREVER))));
}

static __inline__ int estream_call_read(estream_t *estream,
    estream_func_t func, void *buf, int len)
{
    if (estream->read_filter)
	estream->read_filter(estream, buf, len);
    if (func)
	return func(estream, buf, len);
    return 0;
}

static void estream_read_call_func(estream_t *estream, int is_except)
{
    int rc = 0;
    int in_recv;
    estream_func_t func;
    
    /* did we receive the whole buffer */
    if (!estream_got_in_buf(estream))
	return;

    func = is_except && estream->except_func ? estream->except_func
	: estream->func;
    if (estream->in_mode & ESTREAM_LINE)
    {
	/* we should call the handler for each line/request */
	while (estream->in_recv)
	{
	    int i, line_with_termination;
	    /* locate end of line - 0, "\n" or "\r\n" */
	    for (i=0; i<estream->in_recv; i++)
	    {
		if (!estream->in_buf[i] || estream->in_buf[i]=='\n')
		    break;
	    }
	    /* if not a complete line - wait for more input */
	    if (i==estream->in_recv)
		break;
	    
	    line_with_termination = i+1;
	    if (estream->in_buf[i]=='\n')
	    {
		/* handle the "\r\n" case */
		if (i && estream->in_buf[i-1]=='\r')
		    i--;
		estream->in_buf[i] = '\0';
	    }
	    else if (estream->in_buf[i])
	    {
		/* not a complete line - wait for more input */
		break;
	    }

	    in_recv = i;

	    /* call the user callback */
	    estream->dont_update++;
	    estream->changed = 0;
	    rc = estream_call_read(estream, func, estream->in_buf, in_recv);
	    estream->dont_update--;
	    if (estream->closing)
		return;
	    if (estream->in_recv)
	    {
	        estream->in_recv -= line_with_termination;
	        memmove(estream->in_buf, estream->in_buf+line_with_termination,
		    estream->in_recv);
	    }
	    if (rc || estream->changed)
		break;
	}
    }
    else
    {
	int in_req, req_completed = 0;
	
	in_req = estream->in_req;
	in_recv = estream->in_recv;
	estream->dont_update++;
	/* if got size requested and not in forever mode, clear buffer */
	if (estream->in_req <= estream->in_recv &&
	    !(estream->in_mode & ESTREAM_FOREVER))
	{
	    estream->in_recv -= estream->in_req;
	    estream->changed = 1;
	    estream->in_req = 0;
	    estream->in_mode = ESTREAM_NONE;
	    req_completed = 1;
	}
	if (estream->in_mode&(ESTREAM_FOREVER|ESTREAM_PARTIAL) ||
	    req_completed)
	{
	    /* call the user callback */
	    /* Give the user up to the amount of data requested and NOT the
	     * amount of data that is actually in the buffer. In case of
	     * ESTREAM_FOREVER, in_req is a dummy value so disregard it.
	     */
	    rc = estream_call_read(estream, func, estream->in_buf,
		estream->in_mode&ESTREAM_FOREVER ? in_recv :
		in_req<=in_recv ? in_req : in_recv);
	    if (estream->closing)
		return;
	    if (req_completed)
	    {
		memmove(estream->in_buf, estream->in_buf + in_req,
		    estream->in_recv);
	    }
	}
	estream->dont_update--;
    }

    if (rc)
    {
	rg_error_f(LINFO, "socket %d read_func %s %d",
	    estream->sock, rc<0 ? "failed" : "closed connection", rc);
	estream_close(estream);
    }

    estream_update(estream);
}

void estream_read_abort(estream_t *estream)
{
    rg_error_f(LDEBUG, "socket %d: canceling read req %d bytes",
	estream->sock, estream->in_req);
    estream->changed = 1;
    estream->in_req = 0;
    estream->in_mode = ESTREAM_NONE;
    estream_update(estream);
}

int estream_read_full(estream_t *estream, int len, estream_in_mode_t mode,
    estream_flags_t flags)
{
    estream->changed = 1;
    estream->dont_update++;

    /* cancel previous requests */
    if (estream->in_req)
	estream_read_abort(estream);

    estream->in_mode = mode;
    estream->flags |= flags;

    /* check if we need to allocate a buffer */
    if (estream->in_alloc<len)
    {
	u8 *temp_buf;
	estream->in_alloc = MAX(ESTREAM_DEF_BUF_SIZE, len);
	temp_buf = malloc(estream->in_alloc);
	if (!temp_buf)
	{
	    rg_error_f(LERR, "socket %d: failed allocating %d bytes",
		estream->sock, estream->in_alloc);
	    estream->in_alloc = 0;
	    estream->dont_update--;
	    estream_close(estream);
	    return -1;
	}
	if (estream->in_recv)
	    memcpy(temp_buf, estream->in_buf, estream->in_recv);
	nfree(estream->in_buf);
	estream->in_buf = temp_buf;
    }

    /* initiate request to read the data */
    estream->in_req = len;
    estream->dont_update--;
    estream_update(estream);
    return 0;
}

int estream_read(estream_t *estream, int len)
{
    return estream_read_full(estream, len, ESTREAM_BLOCK, 0);
}

/* calls back whenever new data arrives, and when len bytes
 * have arrived. the data is not flushed after each user callback
 * (it is accumulated).
 */
int estream_read_partial(estream_t *estream, int len)
{
    return estream_read_full(estream, len, ESTREAM_PARTIAL, 0);
}

/* calls the back on every incoming data */
int estream_read_forever(estream_t *estream, int max_buf_size)
{
    estream->in_max_buf_size =
	max_buf_size>0 ? max_buf_size : ESTREAM_DEF_MAX_BUF_SIZE;
    return estream_read_full(estream, estream->in_max_buf_size, 
	ESTREAM_FOREVER, 0);
}

void estream_clear_input(estream_t *estream)
{
    estream->in_recv = 0;
}

/* remove the acknowledged data from the estream recieve buffer */
int estream_read_ack(estream_t *estream, int len)
{
    /* TODO: Decrease the buffer size if it has been doubled */
    if (estream->in_mode != ESTREAM_FOREVER)
	return -1;

    if (len > estream->in_recv)
	len = estream->in_recv;

    if (estream->in_recv > len)
	memmove(estream->in_buf, estream->in_buf + len, estream->in_recv - len);
    estream->in_recv -= len;

    return 0;
}

/* calls back on every input line */
int estream_read_line(estream_t *estream)
{
    return estream_read_full(estream, ESTREAM_MAX_LINE, ESTREAM_LINE, 0);
}

int estream_write(estream_t *estream, void *buf, int len)
{
    estream->changed = 1;
 
    /* Check if there is any actual writing to do or if this is only an update
     * request.
     */
    if (len==-1)
	len = strlen(buf);
    if (estream->write_filter)
	estream->write_filter(estream, buf, len);
    if (len)
    {
	/* check if we need to allocate a buffer */
	if (len + estream->out_len > estream->out_alloc)
	{
	    char *tmp_buf;
	    
	    estream->out_alloc = MAX(ESTREAM_DEF_BUF_SIZE, 
		    len + estream->out_len);
	    tmp_buf = realloc((char *)estream->out_buf, estream->out_alloc);
	    if (!tmp_buf)
	    {
		rg_error_f(LERR, "socket %d: failed allocating buffer %d",
			estream->sock, estream->out_alloc);
		estream->out_alloc = 0;
		estream_close(estream);
		return -1;
	    }
	    estream->out_buf = tmp_buf;
	}

	memcpy(estream->out_buf+estream->out_len, buf, len);
	estream->out_len += len;
    }
    /* If there are no pending write request then there is no actual writing
     * to do, so inform the user the the operation is finished. The user isn't
     * immediately informed otherwise as write requests are commutative, so
     * this preserve normal behaviour.
     */
    else if (!estream->out_len)
    {
	int rc;

	estream->dont_update++;
	rc = estream_call_read(estream, estream->func, NULL, 0);
	estream->dont_update--;
	if (estream->closing)
	    return 0;
	if (rc)
	{
	    rg_error_f(LDEBUG, "socket %d: write_func %s %d",
		estream->sock, rc<0 ? "failed" : "closed connection", rc);
	    estream_close(estream);
	}
    }

    estream_update(estream);
    return 0;
}

static void estream_read_and_except_e(estream_t *estream, int is_except)
{
    int rc;
    char *read_pos;
    int read_len;
    
    if (estream->closing)
	return;
    if (is_except)
    {
	int atmark;
        
	if (!(estream->flags & ESTREAM_NOIP) &&
	    ioctl(estream->sock, SIOCATMARK, (int)&atmark)<0)
	{
	    rg_error_f(LERR, "socket %d: failed ioctl(SIOCATMARK) - %m",
		estream->sock);
	    estream_close(estream);
	    return;
	}
    }
    /* no need to read new data if any of the following occured */
    if (estream->in_recv &&
	(estream->in_mode&(ESTREAM_PARTIAL|ESTREAM_FOREVER) || 
	((estream->in_mode&ESTREAM_LINE) &&
	memchr((char *)estream->in_buf, '\n', estream->in_recv)) ||
	(estream->in_mode&ESTREAM_BLOCK && estream->in_recv>=estream->in_req)))
    {
	estream_read_call_func(estream, is_except);
	return;
    }
    /* check if we need to increase the buffer size for estream forever */
    if (estream->in_mode&ESTREAM_FOREVER && estream->in_req<=estream->in_recv)
    {
	char *tmp_buf;

	if (estream->in_recv == estream->in_max_buf_size)
	{
	    estream_read_call_func(estream, is_except);
	    return;
	}
	estream->in_alloc *= 2;
	if (estream->in_alloc > estream->in_max_buf_size)
	    estream->in_alloc = estream->in_max_buf_size;
	estream->in_req = estream->in_alloc;
	tmp_buf = realloc((char *)estream->in_buf, estream->in_alloc);
	if (!tmp_buf)
	{
	    rg_error_f(LERR, "out of memory");
	    estream_close(estream);
	    return;
	}
	estream->in_buf = tmp_buf;
    }
    read_pos = estream->in_buf + estream->in_recv;
    read_len = estream->in_req - estream->in_recv;
    if (estream->ssl)
	rc = estream->ssl->read(estream, read_pos, read_len);
    else
	rc = read(estream->sock, read_pos, read_len);
    if (rc<0)
    {
	if (errno==EAGAIN || errno==EINTR || errno==EWOULDBLOCK)
	    return;
	rg_error_f(LWARN, "Can't read from fd %d %m(%d)", estream->sock, errno);
	estream_close(estream);
	return;
    }
    if (!rc)
    {
	rg_error_f(LDEBUG, "socket %d: connection closed", estream->sock);
	estream_close(estream);
	return;
    }
    estream->in_recv += rc;
    estream_read_call_func(estream, is_except);
}

static void estream_read_efd(int sock, void *o)
{
    estream_read_and_except_e((estream_t *)o, 0);
}

static void estream_read_et(void *o)
{
    estream_t *estream = (estream_t *)o;

    estream->et_read_set = 0;
    estream_read_efd(estream->sock, estream);
}

static void estream_write_efd(int sock, void *o)
{
    int rc;
    int write_len;
    char *write_pos;
    estream_t *estream = (estream_t *)o;

    if (estream->closing)
	return;
    write_len = estream->out_len - estream->out_sent;
    write_pos = estream->out_buf + estream->out_sent;
    if (estream->ssl)
	rc = estream->ssl->write(estream, write_pos, write_len);
    else
	rc = write(estream->sock, write_pos, write_len);
    if (rc<0)
    {
	if (errno==EAGAIN || errno==EINTR || errno==EWOULDBLOCK)
	    return;
	rg_error_f(LERR, "socket %d: failed write()", estream->sock);
	estream_close(estream);
	return;
    }
    if (!rc)
    {
	rg_error_f(LERR, "socket %d: connection closed", estream->sock);
	estream_close(estream);
	return;
    }
    estream->out_sent += rc;
    if (estream->out_sent==estream->out_len)
    {
	estream->out_sent = estream->out_len = 0;
	estream->dont_update++;
	rc = estream_call_read(estream, estream->func, NULL, 0);
	estream->dont_update--;
	if (estream->closing)
	    return;
	if (rc)
	{
	    rg_error(LDEBUG, "estream_write_efd %d: write_func %s %d",
		estream->sock, rc<0 ? "failed" : "closed connection", rc);
	    estream_close(estream);
	}
    }
    estream_update(estream);
}

static void estream_except_efd(int sock, void *o)
{
    estream_read_and_except_e((estream_t *)o, 1);
}

static void estream_connect_ssl_efd(int sock, void *o)
{
    estream_t *estream = (estream_t *)o;
    int rc;
    /* handle SSL connection after underlaying TCP connection has
     * already beenastablished */
    event_fd_clr(estream->sock, EFD_ALL);
    rc = estream->ssl->accept_connect(estream);
    if (rc==-EWOULDBLOCK)
    {
	event_fd_set(estream->sock,
	    (estream->ssl->should_read(estream) ? EFD_READ : 0) |
	    (estream->ssl->should_write(estream) ? EFD_WRITE : 0),
	    estream_connect_ssl_efd, estream);
	return;
    }
    else if(rc==-1)
    {
	rg_error(LINFO, "estream_connect_ssl_efd %d: failed ssl connetion",
	    estream->sock);
	goto Error;
    }
    estream_connect_done(estream);
    return;

Error:
    if (estream->start_connect && !estream->connected)
    {
	estream->connect_func(estream, 0);
	if (estream->closing)
	    return;
    }
    estream_close(estream);
}

static void estream_connect_done_efd(int sock, void *o)
{
    struct sockaddr_in sa;
    int sa_len = sizeof(sa);
    estream_t *estream = (estream_t *)o;

    event_fd_clr(estream->sock, EFD_ALL);
   
    if (estream->flags & ESTREAM_NOIP)
    {
	estream->local_ip = estream->peer_ip = -1;
	estream->local_port = estream->peer_port = -1;
    }
    else
    {	
	if (getpeername(estream->sock, (struct sockaddr *)&sa, &sa_len)<0)
	{
	    rg_error(LINFO, "estream_connect_done %d: connection failed",
		estream->sock);
	    goto Error;
	}
	estream->peer_ip = sa.sin_addr.s_addr;
	estream->peer_port = sa.sin_port;
	if (getsockname(estream->sock, (struct sockaddr *)&sa, &sa_len)<0)
	{
	    rg_error(LERR, "estream_connect_done %d: failed getsockname, %s",
		estream->sock, strerror(errno));
	    goto Error;
	}
	estream->local_ip = sa.sin_addr.s_addr;
	estream->local_port = sa.sin_port;
    }
    if (!estream->ssl)
	estream_connect_done(estream);
    else
	estream_connect_ssl_efd(estream->sock, estream);
    return;

Error:
    if (estream->start_connect)
    {
	estream->connect_func(estream, 0);
	if (estream->closing)
	    return;
    }
    estream_close(estream);
}

static void estream_connect_done(estream_t *estream)
{
    int rc;
    estream->connected = 1;
    rg_error(LDEBUG, "estream_connect_done_efd %d: connected", estream->sock);
    estream->dont_update++;
    rc = estream->connect_func(estream, 1);
    if (estream->closing)
	return;

    if (rc)
    {
	estream->dont_update--;
	rg_error(LINFO, "estream_connect_done %d: failed connect handler %s %d",
	    estream->sock, rc<0 ? "failed" : "closed connection", rc);
	goto Error;
    }
    estream->dont_update--;
    /* FIXME: for performance, fd events should all be set
     * here, and suspended/resumed in estream_update
     * event_fd_set(estream->sock, EFD_READ, estream_read_efd);
     * event_fd_set(estream->sock, EFD_WRITE, estream_write_efd);
     * event_fd_set(estream->sock, EFD_EXCEPT, estream_except_efd);
     */
    estream_update(estream);
    return;
    
Error:
    estream_close(estream);
}

#define ESTREAM_TIMER_GRANULATIRY 1000

static void estream_update(estream_t *estream)
{
    if (estream->closing || estream->dont_update)
	return;
    
    if (!estream->connected)
    {
	if (estream->timeout)
	{
	    event_timer_set(estream->timeout, estream_timeout_et, estream);
	    /* the exact time does not matter, as long as last_timeout_expire
	     * is 0 (this causes it to be canceled after the connection)
	     */
	    estream->last_timeout_expire = 1;
	}
	else
	{
	    event_timer_del(estream_timeout_et, estream);
	    estream->last_timeout_expire = 0;
	}
	return;
    }

    /* set the incoming fd events */
    if (estream->in_req && !estream->efd_read_set)
    {
	event_fd_set(estream->sock, EFD_READ, estream_read_efd, estream);
	event_fd_set(estream->sock, EFD_EXCEPT, estream_except_efd, estream);
    }
    else if (!estream->in_req && estream->efd_read_set)
	event_fd_clr(estream->sock, EFD_READ|EFD_EXCEPT);
    estream->efd_read_set = estream->in_req;
    if (estream_got_in_buf(estream) && !estream->et_read_set)
    {
	event_timer_set(0, estream_read_et, estream);
        estream->et_read_set = 1;
    }

    /* set the outgoing fd events */
    if (estream->out_len && !estream->efd_write_set)
    {
	event_fd_set(estream->sock, EFD_WRITE, estream_write_efd, estream);
    }
    else if (!estream->out_len && estream->efd_write_set)
	event_fd_clr(estream->sock, EFD_WRITE);
    estream->efd_write_set = estream->out_len;

    /* schedule the timeout event */
    if (estream->timeout && (estream->in_req || estream->out_len))
    {
	double next_expire = event_get_loop_now() + estream->timeout;
	double diff = next_expire - estream->last_timeout_expire;
	if (diff < 0.0)
	    diff = -diff;
	if (!estream->last_timeout_expire ||
	    estream->timeout < ESTREAM_TIMER_GRANULATIRY*5 ||
	    diff > ESTREAM_TIMER_GRANULATIRY)
	{
	    estream->last_timeout_expire = next_expire;
            event_timer_set(estream->timeout, estream_timeout_et, estream);
	}
    }
    else
    {
	if (estream->last_timeout_expire)
	{
	    estream->last_timeout_expire = 0;
            event_timer_del(estream_timeout_et, estream);
	}
    }
}

static void estream_connect_done_et(void *o)
{
    estream_t *estream = (estream_t *)o;

    estream_connect_done_efd(estream->sock, estream);
}

estream_t *estream_accept(int sock, estream_flags_t flags,
    estream_connect_func_t func)
{
    estream_t *estream;
    
    /* create the estream object */
    if (!(estream = estream_open()))
	goto Error;
    
    estream->flags = flags;
    estream->flags |= ESTREAM_CLOSE_FD;
    estream->is_accept = 1;
    /* try to create a socket if we did not receive one */
    if ((estream->sock = sock_accept(sock))<0)
	goto Error;

    if (estream->flags & ESTREAM_SSL)
    {
	if (!estream_ssl_open_func)
	    rg_error(LEXIT, "estream_accept(%d): called with ESTREAM_SSL "
		"before estream_ssl_open_func init", sock);
	if (estream_ssl_open_func(estream, NULL))
	    goto Error;
    }
    estream->connect_func = func;

    /* accept finished - call the next stage handler */
    event_timer_set(0, estream_connect_done_et, estream);
    return estream;

Error:
    estream_close(estream);
    return 0;
}

estream_t *estream_connect_tcp_full(int sock, estream_flags_t flags,
    estream_connect_func_t func, u32 ip, u16 port, u32 src_ip, u16 src_port)
{
    struct sockaddr_in sa;
    int connect_rc;
    
    /* on connection termination close the socket, if we created it */
    flags |= sock<0 ? ESTREAM_CLOSE_FD : 0;
    /* try to create a socket if we did not receive one */
    if (sock<0 && (sock = sock_socket(SOCK_STREAM, src_ip, src_port))<0)
	goto Error;
    if (!sock_set_sockopt(sock, flags)<0)
	goto Error;
    flags |= ESTREAM_SOCKOPT_SET;

    /* connect to the socket */
    MZERO(sa);
    sa.sin_family = AF_INET;
    sa.sin_addr.s_addr = ip;
    sa.sin_port = port;
    rg_error(LDEBUG, "estream_connect_full %d: connecting to %s port %d",
	sock, inet_ntoa(sa.sin_addr), ntohs(port));
    if ((connect_rc = connect(sock, (struct sockaddr *)&sa, sizeof(sa)))<0)
    {
	if (errno!=EWOULDBLOCK && errno!=EINPROGRESS)
	{
	    rg_error(LERR, "estream_connect_src %d: failed connect()",
		sock);
	    goto Error;
	}
    }
    if (!connect_rc)
	flags |= ESTREAM_CONNECTED;
    return estream_connect_fd(sock, flags, func);

Error:
    if (sock>=0 && flags & ESTREAM_CLOSE_FD)
	close(sock);
    return 0;
}

estream_t *estream_connect_fd(int sock, estream_flags_t flags,
    estream_connect_func_t func)
{
    estream_t *estream;
    
    /* create the estream object */
    if (!(estream = estream_open()))
	goto Error;
    estream->flags = flags;
    estream->sock = sock;
    estream->connect_func = func;
    if (estream->flags & ESTREAM_SET_DEF_TIMEOUT)
	estream->timeout = ESTREAM_DEF_TIMEOUT;
    /* Check if sockopts werent set by tcp_full */
    if (!(flags & ESTREAM_SOCKOPT_SET))
	if (sock_set_sockopt(sock, flags))
	    goto Error;

    if (estream->flags & ESTREAM_SSL)
    {
	if (!estream_ssl_open_func)
	    rg_error(LPANIC, "estream_connect(%d): called with ESTREAM_SSL "
		"before estream_ssl_open_func init", sock);
	if (estream_ssl_open_func(estream, NULL))
	    goto Error;
    }

    estream->start_connect = 1;
    estream->connected = estream->flags & ESTREAM_CONNECTED;
    /* handle delayed connection */
    if (!estream->connected)
    {
	event_fd_set(estream->sock, EFD_WRITE, estream_connect_done_efd,
	    estream);
	event_fd_set(estream->sock, EFD_EXCEPT, estream_connect_except_efd,
	    estream);
	estream_update(estream);
	return estream;
    }

    /* connection finished immediately - call the next stage handler */
    event_timer_set(0, estream_connect_done_et, estream);
    rg_error(LDEBUG, "estream_connect_full %d: connected", estream->sock);
    return estream;

Error:
    estream_close(estream);
    return 0;
}

/* normally should be called with src_ip==0 and src_port==0 */
int sock_socket(int type, u32 src_ip, u16 src_port)
{
    int sock = -1;
    struct sockaddr_in sa;

    if ((sock = socket(AF_INET, type, 0))<0)
    {
	rg_error(LERR, "sock_socket: failed socket()");
	return -1;
    }
    MZERO(sa);
    sa.sin_family = AF_INET;
    sa.sin_addr.s_addr = src_ip;
    sa.sin_port = src_port; 
    if (src_port)
    {
	int opt = 1;
	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, 
	    (void *)&opt, sizeof(opt))<0)
	{
	    rg_error(LERR, "sock_socket: failed setsockopt(SO_REUSEADDR)");
	    goto Error;
	}
	if ((bind(sock, (struct sockaddr *)&sa, sizeof(sa)))<0)
	{
	    rg_error(LERR, "sock_socket: failed bind(): %s", strerror(errno));
	    goto Error;
	}
    }
    else
    {
	if ((bind(sock, (struct sockaddr *)&sa, sizeof(sa)))<0)
	{
	    rg_error(LERR, "sock_socket: failed bind()");
	    goto Error;
	}
    }
    
    rg_error(LDEBUG, "sock_socket: created fd %d for ip %s port %d", sock,
	inet_ntoa(sa.sin_addr), ntohs(sa.sin_port));
    return sock;
    
Error:
    if (sock>=0)
	close(sock);
    return -1;
}

int sock_set_sockopt(int sock, estream_flags_t flags)
{
    /* set the socket to nonblock mode */
    int status;
    if ((status = sys_sock_set_nonblocking(sock)))
	return status;

    if (!(flags & ESTREAM_NOIP))
    {
	/* receive out of band data inline */
	int opt = 1;
	if (setsockopt(sock, SOL_SOCKET, SO_OOBINLINE, (char *)&opt,
	    sizeof(opt))<0)
	{
	    return rg_error(LERR, "sock_set_sockopt %d: failed ioctl", sock);
	}
    }
    return 0;
}

void estream_set_timeout(estream_t *estream, int timeout,
    estream_func_t timeout_func)
{
    estream->changed = 1;
    estream->timeout = timeout;
    estream->timeout_func = timeout_func;
    estream_update(estream);
}

void estream_set_data(estream_t *estream, void *data,
    free_t data_free_func)
{
    estream->changed = 1;
    if (estream->data_free_func && estream->data)
	estream->data_free_func(estream->data);
    estream->data_free_func = data_free_func;
    estream->data = data;
}

void estream_set_state(estream_t *estream, int state)
{
    estream->state = state;
}

int estream_get_state(estream_t *estream)
{
    return estream->state;
}

void *estream_get_data(estream_t *estream)
{
    return estream->data;
}

int estream_is_connected(estream_t *estream)
{
    return estream->connected;
}

void estream_set_func(estream_t *estream, estream_func_t func)
{
    estream->changed = 1;
    estream->func = func;
}

void estream_set_read_filter(estream_t *estream, estream_filter_func_t func)
{
    estream->read_filter = func;    
}

void estream_set_write_filter(estream_t *estream, estream_filter_func_t func)
{
    estream->write_filter = func;    
}

size_t estream_get_read_req(estream_t *estream)
{
    return estream->in_req;
}

int estream_push(estream_t *estream)
{
    int retval = 0;
    estream_stack_t *stack = zalloc_l(sizeof(estream_stack_t));

    if (!stack)
    {
	retval = -1;
	goto Exit;
    }
    stack->next = estream->stack;
    estream->stack = stack;
    stack->data = estream->data;
    stack->data_free_func = estream->data_free_func;
    stack->func = estream->func;
    stack->state = estream->state;
    estream->data = NULL;
    estream->data_free_func = NULL;
    estream->state = 0;
Exit:
    return retval;
}

int estream_pop(estream_t *estream)
{
    int retval = 0;
    estream_stack_t *stack = estream->stack;

    if (!stack)
    {
	retval = rg_error(LERR, "estream_pop: empty stack - estream unchanged");
	goto Exit;	    
    }
    if (estream->data_free_func)
	estream->data_free_func(estream->data);
    estream->stack = stack->next;
    estream->data = stack->data;
    estream->data_free_func = stack->data_free_func;
    estream->func = stack->func;
    estream->state = stack->state;
    free(stack);
Exit:
    return retval;
}

int estream_printf(estream_t *estream, char *fmt, ...)
{
    va_list ap;
    str_t *str;
    int rc = 0;

    va_start(ap, fmt);
    str = str_alloc();
    if (str_vsprintf(str, fmt, ap) < 0 || 
	estream_write(estream, str->str, strlen(str->str)))
    {
	rc = -1;
    }

    str_free(str);
    return rc;
}
