/* LICENSE_CODE JGPL */

#define OS_INCLUDE_STD
#define OS_INCLUDE_IO

#include "openrg_gpl.h"
#include "rg_error.h"
#include <os_includes.h>

#define rg_error(level,str,args...) printf(str,##args)
/* OpenRG GPL includes */
#include "memory.h"
/* Note this code is GPL Licence, so do not add any new 
 * openrg incude files here. Talk to TL or SA before any change
 */

static void *malloc_lez(size_t size, int is_zalloc, rg_error_level_t severity)
{
    void *mem;

    mem = is_zalloc ? calloc(size, 1) : malloc(size);
    if (!mem)
    {
	rg_error(severity, "%salloc_%s: failed malloc(%d)", 
	    is_zalloc ? "z" : "m", severity==LERR ? "l" : "e", size);
    }
    return mem;
}

void *zalloc_l(size_t size)
{
    return malloc_lez(size, 1, LERR);
}

void *zalloc_e(size_t size)
{
    return malloc_lez(size, 1, LEXIT);
}

void *malloc_l(size_t size)
{
    return malloc_lez(size, 0, LERR);
}

void *malloc_e(size_t size)
{
    return malloc_lez(size, 0, LEXIT);
}

static void *realloc_le(void *ptr, size_t size, rg_error_level_t err)
{
    void *mem = realloc(ptr, size);
    if (!mem)
	rg_error(err, "realloc_le: failed realloc(%d)", size);
    return mem;
}

void *realloc_l(void *ptr, size_t size)
{
    return realloc_le(ptr, size, LERR);
}

void *realloc_e(void *ptr, size_t size)
{
    return realloc_le(ptr, size, LEXIT);
}

static char *strdup_le(char *str, rg_error_level_t severity)
{
    char *p;

    p = strdup(str);
    if (!p)
    {
	rg_error(severity, "strdup_%s: failed strdup(%s)", 
	    severity==LERR ? "l" : "e", str);
    }
    return p;
}

char *strdup_l(char *str)
{
    return strdup_le(str, LERR);
}

char *strdup_e(char *str)
{
    return strdup_le(str, LEXIT);
}

char *strdup_n(char *str)
{
    return str ? strdup_le(str, LEXIT) : NULL;
}

static void *bufdup_le(void *buf, size_t size, rg_error_level_t severity)
{
    void *mem = malloc_lez(size, 0, severity);

    if (!mem)
    {
	rg_error(severity, "bufdup_%s: failed malloc(%d)", 
	    severity==LERR ? "l" : "e", size);
    }
    else
	memcpy(mem, buf, size);

    return mem;
}

void *bufdup_l(void *buf, size_t size)
{
    return bufdup_le(buf, size, LERR);
}

void *bufdup_e(void *buf, size_t size)
{
    return bufdup_le(buf, size, LEXIT);
}

void _zfree(void **p)
{
    if (!p)
	return;
    free(*p);
    *p = NULL;
}
	
void *memdup(void *src, int len)
{
    void *dst = malloc(len);
    if (!dst)
	return NULL;
    memcpy(dst, src, len);
    return dst;
}

int kmemcpy(char *buf, long pos, int n)
{
#ifdef CONFIG_RG_TARGET_LINUX
    int fd_kmem, ret = -1;

    if (!n)
	return 0;
 
    if ((fd_kmem = open("/dev/kmem", O_RDONLY))==-1)
	return rg_error_f(LWARN, "Failed opening /dev/kmem - %m");
 
    if (lseek(fd_kmem, pos, 0) == -1)
    {
	rg_error_f(LWARN, "Failed lseek on /dev/kmem, pos = 0x%p - %m", pos);
	goto Exit;
    }

    if (read(fd_kmem, buf, n) < n)
    {
	rg_error_f(LWARN, "Failed read on /dev/kmem - %m");
	goto Exit;
    }

    ret = 0;
Exit: 
    close(fd_kmem);
    return ret;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    memcpy((void *)buf, (void *)pos, n);
    return 0;
#endif
}

