/* LICENSE_CODE JUNGO */

/* This file defines functions which are missing in VxWorks. */
#include <vxWorks.h>
#define OS_INCLUDE_TYPES
#define OS_INCLUDE_STD
#define OS_INCLUDE_STDARG
#define OS_INCLUDE_IO
#include <os_includes.h>
#include <limits.h>
#include <fioLib.h>
#include <util/rg_error.h>
#include <os_libc_extra.h>

int alphasort(const void *d1, const void *d2)
{
    struct dirent *p1 = *(struct dirent **)d1;
    struct dirent *p2 = *(struct dirent **)d2;
    return strcmp(p1->d_name, p2->d_name);
}

typedef struct {
    char *buf;
    int len;
    int pos;
    int req;
} str_buffer_t;

static STATUS str_output(char *buffer, int nchars, int outarg)
{
    str_buffer_t *str_buf = (str_buffer_t *)outarg;

    str_buf->req += nchars;

    if (str_buf->pos + nchars + 1 >= str_buf->len)
	nchars = str_buf->len - str_buf->pos - 1;

    if (nchars)
    {
	if (str_buf->buf)
	    memcpy(str_buf->buf + str_buf->pos, buffer, nchars);
	str_buf->pos += nchars;
    }
    return OK;
}

int vsnprintf(char *str, size_t n, const char *fmt, va_list ap)
{
    str_buffer_t str_buf = {str, n, 0, 0};

    if (fioFormatV(fmt, ap, str_output, (int)&str_buf) == ERROR)
	return ERROR;

    if (str_buf.buf && n)
	str_buf.buf[str_buf.pos] = '\0';
    return str_buf.req;
}

int snprintf(char *str, size_t n, const char *fmt, ...)
{
    int ret;
    va_list ap;

    va_start(ap, fmt);
    ret = vsnprintf(str, n, fmt, ap);
    va_end(ap);

    return ret;
}

/* This code is not compiled currently, since although it is absent in
 * VxWorks, IXP library (armMonsoon.a) comes with an implementation.
 */
#if 0
/* Taken from FreeBSD */
int strcasecmp(const char *s1, const char *s2)
{
    const u_char *us1 = (const u_char *)s1, *us2 = (const u_char *)s2;

    while (tolower(*us1) == tolower(*us2++))
    {
	if (*us1++ == '\0')
	    return (0);
    }

    return (tolower(*us1) - tolower(*--us2));
}

int strncasecmp(const char *s1, const char *s2, size_t n)
{
    if (n != 0)
    {
	const u_char *us1 = (const u_char *)s1,	*us2 = (const u_char *)s2;

	do
	{
	    if (tolower(*us1) != tolower(*us2++))
		return (tolower(*us1) - tolower(*--us2));

	    if (*us1++ == '\0')
		break;
	} while (--n != 0);
    }

    return (0);
}

/* Taken from FreeBSD */
char *strdup(const char *str)
{
    size_t len;
    char *copy;

    len = strlen(str) + 1;

    if ((copy = malloc(len)) == NULL)
	return (NULL);

    memcpy(copy, str, len);
    return copy;
}
#endif
