/* LICENSE_CODE JUNGO */

#include <rg_def.h>
#include <fcntl.h>
#include <unistd.h>
#include <util/rg_error.h>

#define rg_error(level,str,args...) printf(str,##args)


#if defined(CONFIG_RG_OPENSSL) && defined(CONFIG_RG_TARGET_VXWORKS)
#include <openssl/rand.h>
#define ENTROPY_BYTES 32 /* Must be larger than ENTROPY_NEEDED (20) because of
			  * an OpenSSL demand, and a multiple of sizeof(int)
			  * because of internal implementation.
			  */
#endif
#include "file.h"

void random_init(void)
{
    int random_seed;

#if defined(CONFIG_RG_OPENSSL) && defined(CONFIG_RG_TARGET_VXWORKS)
    {
	char rnd_seed[ENTROPY_BYTES];

	sys_get_random((int *)rnd_seed, ENTROPY_BYTES / sizeof(int));
	RAND_seed(rnd_seed, sizeof(rnd_seed));
    }
#endif

    sys_get_random(&random_seed, 1);
    srand(random_seed);
}

int random_buf_get(u8 *buf, int size)
{
#ifdef CONFIG_RG_TARGET_LINUX
    int ret = -1;
    int fd = open("/dev/urandom", O_RDONLY);

    if (fd<0)
    {
	rg_error_f(LERR, "cannot open file /dev/urandom");
	goto Exit;
    }
/*  ret = force_read(fd, (char *)buf, size);*/
    ret = read(fd, (char *)buf, size);
	close(fd);
    if (ret != size)
    {
	rg_error_f(LERR, "Error while read file /dev/urandom");
	goto Exit;
    }
    ret = 0;
Exit:
    return ret;
#else
    /* We assume someone was so very kind to initialize a random seed for us.
     * We do not use the whole number received from rand() because it is 
     * between 0 and RAND_MAX (which is defined to be 32767).
     */
    int i;

    for (i = 0; i < size; i++)
	buf[i] = 0xff & rand();

    return 0;
#endif
}

#ifndef CONFIG_RG_OPENSSL
int RAND_bytes(u8 *buf, int size)
{
    return random_buf_get(buf, size);
}
#endif

