/* LICENSE_CODE JUNGO */

#define RG_ERROR_INTERNALS
#include <util/rg_error.h>

#ifdef CONFIG_RG_TARGET_LINUX
#ifdef __KERNEL__ 
#define is_mt_context() 0
#define MAX_LINE_SIZE 1024
#define __LINUX_KERNEL__
/* XXX: linux kernel doesn't have vsnprintf !!! */
#define vsnprintf(str, num, fmt, va) vsprintf(str, fmt, va)
#else
#define is_mt_context() 1
#endif
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#include <stdio.h>
#include <taskLib.h>
/* These numbers are not defined in VxWorks' in.h */
#define IPPROTO_IPIP 4
#define IPPROTO_RSVP 46
#define IPPROTO_GRE 47
#define IPPROTO_ESP 50
#define IPPROTO_AH 51

static int mt_task_id;

static int is_mt_context(void)
{
    return taskIdSelf() == mt_task_id;
}
#else
#error Unsupported OS
#endif

#ifdef __KERNEL__ 
#include <kos/kos.h>
#include <syslog/log_dev.h>
#endif
#ifndef __LINUX_KERNEL__
#include <signal.h>
#include <libc_config.h>
#include <misc_funcs.h>
#include <process_funcs.h>
#include <main/main.h>
#include <netinet/in.h>
#else
#include <linux/kernel.h>
#include <linux/string.h>
#include <linux/in.h>
#endif

void rg_error_set_mt_id(void)
{
#ifdef CONFIG_RG_TARGET_VXWORKS
    mt_task_id = taskIdSelf();
#endif
}

char *ip_proto_name_get(int proto)
{
    int i;
    static char proto_str[] = {"PROTO 255"};
    static struct {
	int proto;
	char *proto_name;
    } protocol_table[] = {
	{IPPROTO_IP, "IP"},
	{IPPROTO_ICMP, "ICMP"},
	{IPPROTO_IGMP, "IGMP"},
	{IPPROTO_IPIP, "IPIP"},
	{IPPROTO_TCP, "TCP"},	
	{IPPROTO_EGP, "EGP"}, 
	{IPPROTO_PUP, "PUP"},
	{IPPROTO_UDP, "UDP"},
	{IPPROTO_IDP, "IDP"}, 
	{IPPROTO_RSVP, "RSVP"},
	{IPPROTO_GRE, "GRE"},
	{IPPROTO_ESP, "ESP"},
	{IPPROTO_AH, "AH"},
	{0, NULL}
    };

    for (i=0; protocol_table[i].proto_name; i++)
    {
        if (protocol_table[i].proto == proto)
   	    return protocol_table[i].proto_name;
    }

    sprintf(proto_str, "PROTO %d", proto);
    return proto_str;
}

/* This function should be static but it create a compilation warning because
 * gcc doesn't 'understand' the weak alias and thinks that its a static 
 * funciton that never beeing used.
 */
void rg_reboot(void)
{
    rg_error(LINFO, "rebooting!");

#ifdef HAVE_KILL
   /* signal the init process (main_task or init) to reboot. */
   kill(1, SIGINT);
#else
   /* This code generates three types of exception: 
    * division by 0 - handled by FPU/Emulator, 
    * unaligned access (float at wrong alignment) - for non-MMU out there
    * segmentation  - 1 is not legal at most MMU platforms.
    */
   *(float *)1 = 1 / 0.0;
#endif
}

void (*openrg_panic_ptr)(void) = rg_reboot;

#ifndef __LINUX_KERNEL__
/* all errors higher or equal rg_error_console_level will also be printed
 * to console 
 */
static int rg_error_console_level = LERR;
static int rg_error_reboot;

/* Make rg_error with LEXIT severity reboot. Should only be used 
 * by the main task.
 */
void rg_error_exit_reboot_set(void)
{
    rg_error_reboot = 1;
}

/* External declaration of main task entry func for stack trace */
extern int MAIN_TASK_ENTRY_FUNC(int argc, char *argv[]);

void *begin_addr;

void strace_init(void)
{
    int temp;
    begin_addr = (void *)&temp + sizeof(void *);
}

static void strace_write(void)
{
    int *end_addr = (int *)&end_addr;
    printf("Main address: %p\n", MAIN_TASK_ENTRY_FUNC);
    printf("Stack:");
    for (; (void *)end_addr <= begin_addr; end_addr++)
	printf(" 0x%x", (unsigned int)*end_addr);
    printf("\n");
}

/* TODO: This logic can (and should) move to syslog task along with all
 * rg_error funcs.
 */
typedef struct rg_error_notify_t {
    struct rg_error_notify_t *next;
    rg_error_func_t func;
    void *data;
    rg_error_level_t level;
} rg_error_notify_t;

static rg_error_notify_t *rg_error_notify;

void rg_closelog(void)
{
    rg_error_notify_t *t, *tmp;
    
    for (t = rg_error_notify; t; t = tmp)
    {
	tmp = t->next;
	free(t->data);
	free(t);
    }
}

void rg_error_register(rg_error_func_t func, void *data, rg_error_level_t 
    level)
{
    rg_error_notify_t *n;
    n = (rg_error_notify_t *)zalloc_e(sizeof(*n));
    n->func = func;
    n->data = data;
    n->level = level;
    n->next = rg_error_notify;
    rg_error_notify = n;
}

/* Return the number of functions unregistred */
int rg_error_unregister(rg_error_func_t func, void *data)
{
    rg_error_notify_t **cur;
    int freed = 0;

    for (cur = &rg_error_notify; *cur;)
    {
	if (data == (*cur)->data && func == (*cur)->func)
	{
	    rg_error_notify_t *tmp = *cur;
	    *cur = tmp->next;
	    free(tmp);
	    freed++;
	}
	else
	    cur = &(*cur)->next;
    }
    return freed;
}
#endif

void log_msg_deliver(rg_error_level_t severity, char *msg)
{
#ifndef __LINUX_KERNEL__ 
    rg_error_notify_t *notify;
    u32 severity_level = severity & LSEVERITY_MASK;

    /* Going through the notify list */
    for (notify = rg_error_notify; notify; notify = notify->next)
    {
	if (notify->level <= severity_level)
	    notify->func(notify->data, msg, severity_level);
    }
    
    if (severity_level >= rg_error_console_level || severity&LCONSOLE)
	console_printf("%s\n", msg);

    switch (severity_level)
    {
    case LEXIT:
	/* Fallthrough */
    case LPANIC:
	strace_write();
	if (LPANIC == severity_level || rg_error_reboot)
	    openrg_panic_ptr();
	break;
    }
#endif
}

/* deliver through logdev */
static void log_dev_enqueue(rg_error_level_t severity, char *msg)
{
#ifdef __KERNEL__ 
    char pri_msg[8];

    sprintf(pri_msg, "<%d>", (unsigned short)severity);
    if (rg_logdev_message_add(pri_msg, strlen(pri_msg)) < strlen(pri_msg))
	return;
    if (rg_logdev_message_add(msg, -1) < 0)
	kos_print("failed queueing a message: \"%s\"\n", msg);
#endif
}

static int _rg_vperror(rg_error_level_t severity, const char *str_error,
    const char *format, va_list ap)
{
    static char msg[MAX_LINE_SIZE];
    static char format_after_m[MAX_LINE_SIZE]; /* will hold format after 
						* expansion of '%m'
						*/
    /* Blocking of the signals is done to protect the static buffer msg[] */
#ifndef __LINUX_KERNEL__ 
    int saved_errno = errno;
#endif
#ifndef __KERNEL__ 
    char name[16]; /* Linux holds first 15 chars of the name */
    sigset_t aset, oset;    

#ifndef CONFIG_RG_TARGET_VXWORKS
    get_process_name(name, sizeof(name));
#else
    strcpy(name, "unknown");
#endif

    sigfillset(&aset);
    sigprocmask(SIG_SETMASK, &aset, &oset);
#else
    kos_interrupt_flags_t f = kos_save_flags_cli();
#endif

    if (format)
    {
	/* handle %m */
	char *p = format_after_m;
	char *p_end = format_after_m + sizeof(format_after_m) - 1;
	const char *format_last = format;
#ifndef __LINUX_KERNEL__ 
	char *format_cur;

	while ((format_cur = strstr(format_last, "%m")))
	{
	    /* copy segment up to '%m' */
	    /* the +1 is to include the terminating null */
	    int chars_to_copy = format_cur - format_last + 1;
	    if (chars_to_copy > p_end - p)
		chars_to_copy = p_end - p;
	    strncpyz(p, format_last, chars_to_copy);
	    /* make p point to the terminating null */
	    p += chars_to_copy - 1;
	    /* add error string */
	    strncpyz(p, strerror(saved_errno), p_end - p);
	    p += strlen(p);
	    format_last = format_cur + 2; /* skip '%m' */
	}
#endif

	/* copy last segment */
	strncpy(p, format_last, p_end - p);
	*p_end = 0;
	p += strlen(p);

	if (str_error && p < p_end)
	{
	    strncpy(p, ": ", p_end-p);
	    *p_end = 0;
	    p += strlen(p);
	    strncpy(p, str_error, p_end-p);
	    *p_end = 0;
	}

	vsnprintf(msg, sizeof(msg), format_after_m, ap);
    }
    else
	strcpy(msg, "error in `rg_error': no message specified");

    if (is_mt_context())
	log_msg_deliver(severity, msg); /* direct delivery */
    else
	log_dev_enqueue(severity, msg); /* deliver through logdev */

#ifndef __KERNEL__ 
    sigprocmask(SIG_SETMASK, &oset, 0);
#else
    kos_restore_flags(f);
#endif
    return -1;
}
 
int rg_error(rg_error_level_t severity, const char *format, ...)
{
    va_list ap;
    int ret;
    
    va_start(ap, format);
    ret = _rg_vperror(severity, NULL, format, ap);
    va_end(ap);

    return ret;
}

int rg_verror(rg_error_level_t severity, const char *format, va_list ap)
{
    return _rg_vperror(severity, NULL, format, ap);
}

#ifndef __LINUX_KERNEL__
int rg_perror(rg_error_level_t severity, const char *format, ...)
{
    va_list ap;
    int ret;
    
    va_start(ap, format);
    ret = _rg_vperror(severity, strerror(errno), format, ap);
    va_end(ap);

    return ret;
}

int rg_vperror(rg_error_level_t severity, const char *format, va_list ap)
{
    return _rg_vperror(severity, strerror(errno), format, ap);
}
#endif

typedef struct {
    rg_rate_idx_t idx;	/* type of message */
    int count;		/* # skipped */
    unsigned long time;	/* last time printed (seconds) */
    char *text_prefix;	/* should distinguish among logs. 0 for varlog */
    char *text;		/* printable type of message */
} rg_rate_limit_t;

static rg_rate_limit_t rg_rate_limits[] = {
    {RL_SYNFLOOD, 0, 0, SYSLOG_FIREWALL_FMT, RL_TEXT_SYNFLOOD},
    {RL_NAT_STATE, 0, 0, SYSLOG_FIREWALL_FMT, RL_TEXT_NAT_STATE},
    {RL_LAST}
};
#if defined(CONFIG_RG_TARGET_VXWORKS) || defined (__LINUX_KERNEL__)
#define get_time_sec() (kos_ticks_get()/kos_tick_rate_get())
#else
#define get_time_sec() time(NULL)
#endif

int rg_rate_limit(rg_rate_idx_t index)
{
    rg_rate_limit_t *rl;
    unsigned long curr = get_time_sec();
    
    for(rl=rg_rate_limits; rl->idx!=RL_LAST && rl->idx!=index; rl++);
    if (rl->idx==RL_LAST)
	return 0;

    if (curr > rl->time) /* once a second - allow and zero counter */
    {
	if (rl->count)
	{
	    /* TODO: if is_mt_context(), use "reported at TIME" */
	    rg_error(LINFO, "%sRATELIMIT: %d messages of type %s reported "
		"%d second(s) ago", rl->text_prefix ? rl->text_prefix : "",
		rl->count, rl->text, curr-rl->time);
	}
	rl->count = 0;
	rl->time = curr;
	return 1;
    }
    rl->count++;
    return 0;
}

