/* LICENSE_CODE JUNGO */

#define OS_INCLUDE_SOCKET
#define OS_INCLUDE_IF_ETHER
#define OS_INCLUDE_IOCTL
#define OS_INCLUDE_INET
#define OS_INCLUDE_TIME
#define OS_INCLUDE_FCNTL
#define OS_INCLUDE_STD
#define OS_INCLUDE_NETIF
#define OS_INCLUDE_REBOOT
#define OS_INCLUDE_NETIF
#define OS_INCLUDE_TYPES
#define OS_INCLUDE_IO
#define OS_INCLUDE_MIB_LIB
#define OS_INCLUDE_SOCKIOS

/*#include <linux/if_arp.h>*/

#include "rg_error.h"
#include <os_includes.h>
/*#include <kos/kos_chardev.h>*/
#include <kos/knet.h>
/*#include "estream.h"*/
#include "event.h"
#include "misc_funcs.h"
/*#include "net_utils.h"*/
#include "syscalls.h"
#include "str.h"
#ifdef CONFIG_RG_TARGET_VXWORKS
#include <muxLib.h>
#include <netinet/in_var.h>
#endif

#define rg_error(level,str,args...) printf(str,##args)
#define rg_perror(level,str,args...) printf(str,##args)


#include <linux/reboot.h> 

#ifdef CONFIG_RG_TARGET_LINUX
/*#include <kos/ipv4_dev.h>*/
#include <unistd.h>


struct net_device_stats {
	unsigned long	rx_packets;	/* total packets received	*/
	unsigned long	tx_packets;	/* total packets transmitted	*/
	unsigned long	rx_bytes;	/* total bytes received 	*/
	unsigned long	tx_bytes;	/* total bytes transmitted	*/
	unsigned long	rx_errors;	/* bad packets received		*/
	unsigned long	tx_errors;	/* packet transmit problems	*/
	unsigned long	rx_dropped;	/* no space in linux buffers	*/
	unsigned long	tx_dropped;	/* no space available in linux	*/

	/* non-unicast counters */
	unsigned long	multicast;	/* multicast packets received	*/
	unsigned long   rx_mcasts;	/* real number of multicast packets 
					 * received (some drivers don't count
					 * in multicast field, some do) */
	unsigned long	tx_mcasts;	/* multicast packets transmitted */
	unsigned long	rx_bcasts;	/* broadcast packets received    */
	unsigned long	tx_bcasts;	/* broadcast packets transmitted */
	
	unsigned long	collisions;

	/* detailed rx_errors: */
	unsigned long	rx_length_errors;
	unsigned long	rx_over_errors;	/* receiver ring buff overflow	*/
	unsigned long	rx_crc_errors;	/* recved pkt with crc error	*/
	unsigned long	rx_frame_errors;/* recv'd frame alignment error */
	unsigned long	rx_fifo_errors;	/* recv'r fifo overrun		*/
	unsigned long	rx_missed_errors;/* receiver missed packet	*/

	/* detailed tx_errors */
	unsigned long	tx_aborted_errors;
	unsigned long	tx_carrier_errors;
	unsigned long	tx_fifo_errors;
	unsigned long	tx_heartbeat_errors;
	unsigned long	tx_window_errors;
	
	/* for cslip etc */
	unsigned long	rx_compressed;
	unsigned long	tx_compressed;
};
#endif

#include <kos/knet.h>
#include <kos/knet_if_ext.h>

/* estream functions. */

/* Set the socket to nonblock mode */
int sys_sock_set_nonblocking(int sock)
{
#ifdef CONFIG_RG_TARGET_VXWORKS
    int on = TRUE;
    if (ioctl(sock, FIONBIO, (int)&on) < 0)
    {
	return rg_error(LERR, 
	    "sock_set_sockopt %d: failed setting non-blocking", sock);
    }
#else
    if (fcntl(sock, F_SETFL, O_NDELAY) < 0)
    {
	return rg_error(LERR, 
	    "sock_set_sockopt %d: failed setting non-blocking", sock);
    }
    /* set current process as the ownder (in case of inherited fd) */
    if (fcntl(sock, F_SETOWN, getpid()) < 0)
    {
	return rg_error(LERR, "sock_set_sockopt %d: failed setting owner",
	    sock);
    }
#endif
    
    return 0;
}

int sys_dev_set_multicast(int s, u32 if_ip)
{
#ifdef CONFIG_RG_TARGET_VXWORKS
    struct in_addr mreq;

    mreq.s_addr = if_ip;
#elif defined(CONFIG_RG_TARGET_LINUX)
    struct ip_mreqn mreq;

    MZERO(mreq);
    mreq.imr_address.s_addr = if_ip;
#endif

    return setsockopt(s, IPPROTO_IP, IP_MULTICAST_IF, (u8 *)&mreq,
	sizeof(mreq));
}

/* event functions. */

clock_t sys_get_clock_rate(void)
#ifdef CONFIG_RG_TARGET_VXWORKS
{
    return sysClkRateGet();
}
#else
{
    return sysconf(_SC_CLK_TCK);
}
#endif

clock_t sys_get_ticks_from_boot(void)
#ifdef CONFIG_RG_TARGET_VXWORKS
{
    return tickGet();
}
#else
{
    struct tms tmp_tms;
    return times(&tmp_tms);
}
#endif

/* NOTE: Do not use this function directly, instead call mt_time_clock_set */
int sys_set_time(time_t *t)
#ifdef CONFIG_RG_TARGET_VXWORKS
{
    struct timespec tp;
    
    tp.tv_sec = *t;
    tp.tv_nsec = 0;

    return clock_settime(CLOCK_REALTIME, &tp);
}
#else
{
    return stime(t);
}
#endif

struct timeval *sys_get_time(struct timeval *tv)
{
#ifdef CONFIG_RG_TARGET_VXWORKS
    struct timespec tp;
    
    clock_gettime(CLOCK_REALTIME, &tp);
    tv->tv_sec = tp.tv_sec;
    tv->tv_usec = tp.tv_nsec / 1000;
#else
    gettimeofday(tv, NULL);
#endif
    return tv;
}

/* net_utils functions */

#ifdef CONFIG_RG_TARGET_VXWORKS
int dev_if_is_end_driver(char *iface)
{
    struct ifnet *dev = ifunit(iface);

    if (!dev)
	return 0;
    return muxDevExists(dev->if_name, dev->if_unit);
}
#endif

int get_if_hw_addr(char *iface, ether_mac_t *mac, int *family)
#ifdef CONFIG_RG_TARGET_BSD
{
    struct arpcom *dev;

    if (!(dev = (struct arpcom *)ifunit(iface)))
	return rg_error_f(LERR, "No device named %s", iface);
    
    if (mac && dev->ac_if.if_type == IFT_ETHER)
	memcpy(*mac, dev->ac_enaddr, sizeof(*mac));
	    
    if (family)
	*family = dev->ac_if.if_type;

    return 0;
}
#else
{
    struct ifreq ifr;

    if (get_if_data(iface, &ifr, SIOCGIFHWADDR) < 0)
	return -1;

    if (mac && ifr.ifr_hwaddr.sa_family == ARPHRD_ETHER)
	memcpy(mac, &ifr.ifr_ifru.ifru_hwaddr.sa_data, 6);

    if (family)
	*family = ifr.ifr_hwaddr.sa_family;
    return 0;
}
#endif

int is_if_ether(char *iface)
{
    int type;

    if (get_if_hw_addr(iface, NULL, &type))
	return 0;
    
#ifdef CONFIG_RG_TARGET_BSD
    return type == IFT_ETHER;
#else
    return type == ARPHRD_ETHER;
#endif
}

#ifdef CONFIG_RG_TARGET_BSD
static int route_exists_cb(struct rtentry *rt, void *arg)
{
    route_t *r = (route_t *)arg;
    struct sockaddr_in *dst = (struct sockaddr_in *)rt_key(rt);
    struct sockaddr_in *gate = (struct sockaddr_in *)rt->rt_gateway;
    struct in_ifaddr *ifa = (struct in_ifaddr *)rt->rt_ifa;
    
    /* Check same destination network */
    if (r->dest.s_addr != dst->sin_addr.s_addr)
	return 0;

    /* Check same gateway in case it is a route to a gateway */
    if (rt->rt_flags & RTF_GATEWAY)
    	return r->gateway.s_addr == gate->sin_addr.s_addr;

    /* Check case that it is a default route insert by VxWorks (local device's
     * IP is the gateway). This check is valid because we already know it is
     * not a route to a gateway.
     */
    if (r->gateway.s_addr != ifa->ia_addr.sin_addr.s_addr)
	return 0;

    return 1;
}
#endif

int route(u32 ip_addr, u32 gw, u32 genmask, char *dev, unsigned short flags,
    int metric, int is_add)
#ifdef CONFIG_RG_TARGET_BSD
{
    int rc = -1;
    struct sockaddr_in rt_gateway, rt_mask, rt_dst;
    char ipaddr[DOTTED_IP_LEN], netmask[DOTTED_IP_LEN];

    MZERO(rt_gateway);
    MZERO(rt_mask);
    MZERO(rt_dst);

    /* Adapt to routing table: Destination=ip&netmask GW=ip */
    if (!gw)
	gw = ip_addr;

    ip_addr &= genmask;
    
    /* The following call to rtrequest will disregard the gw IP if the
     * route we are deleting is not a route to a default GW.
     * This might delete the wrong route to the right subnet - but via a
     * different device (or IP alias), so we first check if the desired
     * route really exists.
     */
    if (!is_add)
    {
	route_t r;
	
	r.gateway.s_addr = gw;
	r.dest.s_addr = ip_addr;
	
	if (!knet_route_table_loop(RTF_UP, route_exists_cb, (void *)&r))
	    return 0;
    }
  
    rt_dst.sin_family = AF_INET;
    rt_dst.sin_len = sizeof(struct sockaddr_in);
    rt_dst.sin_addr.s_addr = ip_addr;
   
    rt_gateway.sin_family = AF_INET;
    rt_gateway.sin_len = sizeof(struct sockaddr_in);
    rt_gateway.sin_addr.s_addr = gw;

    rt_mask.sin_family = AF_INET;
    rt_mask.sin_len = sizeof(struct sockaddr_in);
    rt_mask.sin_addr.s_addr = genmask;

    strncpyz(ipaddr, inet_ntoa(rt_dst.sin_addr), DOTTED_IP_LEN);
    strncpyz(netmask, inet_ntoa(rt_mask.sin_addr), DOTTED_IP_LEN);
    rg_error(LDEBUG, "%s route to: dst(%s) dev(%s) gw(0x%x) net(%s) metric(%d)",
	is_add ? "adding" : "removing", ipaddr, dev ? dev : "-", gw, 
	netmask, metric);
    
    rc = rtrequest(is_add ? RTM_ADD : RTM_DELETE, (struct sockaddr *)&rt_dst, 
	(struct sockaddr *)&rt_gateway,	(struct sockaddr *)&rt_mask,
	flags, NULL);
 
    if (rc)
    {
	rg_perror(LWARN, "route: Failed to %s route to dst(%s) dev(%s) "
	    "metric(%d) gw(0x%x) net(%s) by IOCTL", is_add ? "add" : "remove",
	    ipaddr, dev ? dev : "-", metric, gw, netmask);
    }

    return rc;
}
#else
{
    int s, rc = -1;
    struct rtentry rte;
    struct sockaddr_in *sockptr;
    char ipaddr[DOTTED_IP_LEN], netmask[DOTTED_IP_LEN];

    if ((s = sockets_open(AF_PACKET, SOCK_PACKET, htons(ETH_P_ALL))) < 0)
    {
	rg_perror(LWARN, "route: socket failed");
	return rc;
    }
   
    ip_addr &= genmask;

    MZERO(rte);
   
    sockptr = (struct sockaddr_in *)&rte.rt_dst;
    sockptr->sin_family	= AF_INET;
    sockptr->sin_addr.s_addr = ip_addr;
   
    sockptr = (struct sockaddr_in *)&rte.rt_gateway;
    sockptr->sin_family = AF_INET;
    sockptr->sin_addr.s_addr = gw;
   
    sockptr = (struct sockaddr_in *)&rte.rt_genmask;
    sockptr->sin_family = AF_INET;
    sockptr->sin_addr.s_addr = genmask;
   
    rte.rt_dev = dev;
    rte.rt_metric = metric + 1; /* +1 for binary compatibility to Kernel code */
    rte.rt_flags = flags;
    
    strncpyz(ipaddr, inet_ntoa(((struct sockaddr_in *)&rte.rt_dst)->sin_addr),
	DOTTED_IP_LEN);
    strncpyz(netmask,
	inet_ntoa(((struct sockaddr_in *)&rte.rt_genmask)->sin_addr),
	DOTTED_IP_LEN);
    rg_error(LDEBUG, "%s route to: dst(%s) dev(%s) gw(0x%x) net(%s)",
	is_add ? "adding" : "removing", ipaddr, dev ? dev : "-", gw, netmask);
    
    rc = ioctl(s, is_add ? SIOCADDRT : SIOCDELRT, &rte);
    
    if (rc)
    {
	rg_perror(LWARN, "route: Failed to %s route to dst(%s) dev(%s) gw(0x%x)"
	    " net(%s) by IOCTL", is_add ? "add" : "remove", ipaddr, 
	    dev ? dev : "-", gw, netmask);
    }
    close(s);
    return rc;
}
#endif

struct in_addr get_dest_gw(struct in_addr dest_ip)
#ifdef CONFIG_RG_TARGET_BSD
{
    struct sockaddr_in *req, *ans;
    struct route ro;
    struct in_addr ret;

    ret.s_addr = INADDR_NONE;

    MZERO(ro);
    req = (struct sockaddr_in *)&ro.ro_dst;
    req->sin_len = sizeof(struct sockaddr_in);
    req->sin_family = AF_INET;
    req->sin_addr = dest_ip;
    rtalloc(&ro);

    if (!(ro.ro_rt) || !(ro.ro_rt->rt_flags&(RTF_UP|RTF_GATEWAY)))
	goto Exit;

    ans = (struct sockaddr_in *)ro.ro_rt->rt_gateway;
    ret = ans->sin_addr;
Exit:
    if (ro.ro_rt)
	RTFREE(ro.ro_rt);
    return ret;
}
#elif defined(CONFIG_RG_TARGET_LINUX)
{
    /* XXX: No get_dest_gw() implementation for Linux */
    struct in_addr ret;
    
    ret.s_addr = INADDR_NONE;
    return ret;
}
#endif

int sys_rg_chrdev_open(kos_chardev_type_t type, int mode)
{
    int fd = open("/dev/" OPENRG_CHRDEV_NAME, mode, 0);

    if (fd < 0)
    {
	rg_error(LERR, "can't open " OPENRG_CHRDEV_NAME " %d %m", type);
	return fd;
    }

    if (ioctl(fd, RG_IOCTL_SIOCSETRGCHRDEVTYPE, type))
    {
	return rg_error(LERR, "can't open " OPENRG_CHRDEV_NAME 
	    " for type %d %m", type);
    }

    return fd;
}

/* User-mode utility to call a module's control function. */
int openrg_module_ctrl(kos_chardev_type_t type, int cmd, void *data)
{
    int fd = sys_rg_chrdev_open(type, O_RDWR);
    int ret;

    if (fd < 0)
	return fd;

    ret = ioctl(fd, cmd, (int)data);

    close(fd);
    return ret;
}

#if !defined(CC_FOR_BUILD) && defined(CONFIG_RG_TARGET_LINUX)
/* XXX: This is still here only for compilation of SNMP for Linux. Should 
 * remove or replace with something that compiles for VxWorks.
 */
static int sys_netdev_ioctl(int family, int socket, char *if_name, int cmd,
    struct ifreq *ifr)
{
    int rc, s;

    if ((s = socket) < 0)
    {
	if ((s = sockets_open(family, family==AF_PACKET ? SOCK_PACKET : 
	    SOCK_DGRAM, family==AF_PACKET ? htons(ETH_P_ALL) : 0)) < 0)
	{
	    rg_error(LWARN, "sys_netdev_ioctl: failed, %s", strerror(errno));
	    return -1;
	}
    }
    strncpyz(ifr->ifr_name, if_name, IFNAMSIZ);
    rc = ioctl(s, cmd, ifr);
    if (socket < 0)
	close(s);
    return rc;
}
#endif

#if 0 

int sys_if_stats_get(char *if_name, user_net_device_stats_t *if_stats)
#ifdef CONFIG_RG_TARGET_LINUX
{
#ifndef CC_FOR_BUILD
    struct ifreq ifr;
    struct net_device_stats stats;

    ifr.ifr_data = (void *)&stats;
    if (sys_netdev_ioctl(AF_INET, -1, if_name, SIOCGIFSTATS, &ifr) < 0)
	return -1;

    if_stats->rx_packets = stats.rx_packets;
    if_stats->tx_packets = stats.tx_packets;
    if_stats->rx_bytes = stats.rx_bytes;
    if_stats->tx_bytes = stats.tx_bytes;
    if_stats->rx_errors = stats.rx_errors;
    if_stats->tx_errors = stats.tx_errors;
    if_stats->rx_dropped = stats.rx_dropped;
    if_stats->tx_dropped = stats.tx_dropped;
    if_stats->rx_multicast = stats.rx_mcasts;
    if_stats->tx_multicast = stats.tx_mcasts;
    if_stats->rx_broadcast = stats.rx_bcasts;
    if_stats->tx_broadcast = stats.tx_bcasts;
    if_stats->collisions = stats.collisions;
#endif

    return 0;
}
#elif defined(CONFIG_RG_TARGET_VXWORKS)
{
    struct ifnet *iface;
   
    if (!(iface = ifunit(if_name)))
	return -1;

    if_stats->rx_packets = iface->if_ipackets;
    if_stats->tx_packets = iface->if_opackets;
    if_stats->rx_bytes = iface->if_ibytes;
    if_stats->tx_bytes = iface->if_obytes;
    if_stats->rx_errors = iface->if_ierrors;
    if_stats->tx_errors = iface->if_oerrors;
    if_stats->rx_dropped = iface->if_iqdrops;
    if_stats->tx_dropped = iface->if_snd.ifq_drops;
    if_stats->rx_multicast = iface->if_imcasts;
    if_stats->tx_multicast = iface->if_omcasts;
    if_stats->collisions = iface->if_collisions;
    /* XXX no broadcast counters yet */
    if_stats->rx_broadcast = 0;
    if_stats->tx_broadcast = 0;
    
    return 0;
}
#endif

#endif /* if 0*/

void sys_soft_reboot(void)
{
#ifdef CONFIG_RG_TARGET_LINUX
    reboot(LINUX_REBOOT_CMD_RESTART);
#elif CONFIG_TI_404_MIPS
    void cm_hwReset(void);
    cm_hwReset();
#elif CONFIG_RG_TARGET_VXWORKS
    reboot(BOOT_NORMAL);   
#endif
}

int sys_flash_sync(int fd)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return fsync(fd);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return ioctl(fd, FLASH_SYNC_IOCTL, 0);
#endif
}

static int sys_if_ioctl_prepare(char *dev, struct ifreq *ifr)
{
    int s;

    if ((s = sockets_open(AF_INET, SOCK_DGRAM, 0)) < 0)
	return rg_error_f(LWARN, "Operation failed %m");

    memset(ifr, 0, sizeof(*ifr));
    strncpyz(ifr->ifr_name, dev, sizeof(ifr->ifr_name));

    return s;
}

#ifdef CONFIG_RG_TARGET_LINUX
static int sys_if_ioctl_execute(char *dev, struct ifreq *ifr, int cmd)
{
    int s, ret;
    
    s = sys_if_ioctl_prepare(dev, ifr);
    if (s < 0)
	return -1;
    ret = ioctl(s, cmd, ifr);
    close(s);
    return ret;
}

static int sys_if_ioctl_param_execute(char *dev, int cmd, int param)
{
    int s, ret;
    struct ifreq ifr;
    struct sockaddr_in *sockptr = (struct sockaddr_in *)&(ifr.ifr_addr);
    
    s = sys_if_ioctl_prepare(dev, &ifr);
    if (s < 0)
	return -1;

    switch (cmd)
    {
    case SIOCSIFFLAGS:
	ifr.ifr_flags = param;
	break;
    case SIOCSIFDSTADDR:
	sockptr = (struct sockaddr_in *)&(ifr.ifr_dstaddr);
	/* Fall through */
    case SIOCSIFADDR:
    case SIOCSIFNETMASK:
    case SIOCSIFBRDADDR:
	sockptr->sin_family = AF_INET;
	sockptr->sin_addr.s_addr = param;
	break;	

    default:
	return rg_error(LERR, "Unsupported ioctl command %d", cmd);
    }

    ret = ioctl(s, cmd, &ifr);
    close(s);

    return ret; 
}

static int sys_if_ioctl_mii_execute(int s, int cmd, struct ifreq *ifr)
{
    if (ioctl(s, cmd, ifr))
    {
	/* This is a workaround for old drivers, which implement
	 * SIOCDEVPRIVATE + n ioctls instead MII ioctls */
	int old_cmd = SIOCDEVPRIVATE + cmd - 0x8947;
	if (ioctl(s, old_cmd, ifr))
	{
	    return rg_error_f(LWARN, "Both tried MII ioclts %X/%X failed: %m.",
		cmd, old_cmd);
	}
    }
    return 0;
}
#endif

int sys_if_ioctl_mii_phy_id(char *dev)
{
#ifdef CONFIG_RG_TARGET_LINUX
    struct ifreq ifr;
    u16 *data = (u16 *)(&ifr.ifr_data);
    int ret, s;
    
    if ((s = sys_if_ioctl_prepare(dev, &ifr)) < 0)
	return -1;

    ret = sys_if_ioctl_mii_execute(s, 0x8947, &ifr);
    close(s);

    return ret == -1 ? -1 : data[0];
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#warning MII API is not implemented in VxWorks
    return -1;
#endif
}

int sys_if_ioctl_mii_read(char *dev, u16 phy_id, u16 reg_num)
{
#ifdef CONFIG_RG_TARGET_LINUX
    struct ifreq ifr;
    u16 *data = (u16 *)(&ifr.ifr_data);
    int ret, s;
    
    if ((s = sys_if_ioctl_prepare(dev, &ifr)) < 0)
	return -1;

    data[0] = phy_id;
    data[1] = reg_num;

    ret = sys_if_ioctl_mii_execute(s, 0x8948, &ifr);
    close(s);

    return ret == -1 ? -1 : data[3];
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#warning MII API is not implemented in VxWorks
    return -1;
#endif
}

int sys_if_ioctl_mii_write(char *dev, u16 phy_id, u16 reg_num, u16 val)
{
#ifdef CONFIG_RG_TARGET_LINUX
    struct ifreq ifr;
    u16 *data = (u16 *)(&ifr.ifr_data);
    int ret, s;
    
    if ((s = sys_if_ioctl_prepare(dev, &ifr)) < 0)
	return -1;

    data[0] = phy_id;
    data[1] = reg_num;
    data[2] = val;

    ret = sys_if_ioctl_mii_execute(s, 0x8949, &ifr);
    close(s);
    return ret;
#elif defined(CONFIG_RG_TARGET_VXWORKS)
#warning MII API is not implemented in VxWorks
    return -1;
#endif
}

/* Functions to configure the network interface parameters.
 * Each function corresponds to one of the ioctls used for ifconfig
 */

int sys_if_flags_set_direct(char *dev, int if_flags)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return sys_if_ioctl_param_execute(dev, SIOCSIFFLAGS, if_flags);
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return ifFlagSet(dev, if_flags);
#endif
}

int sys_if_flags_set(char *dev, ifconfig_flag_t flags)
{
    int if_flags = 0;
    
    if (flags & IFCFG_SUP)
        if_flags = IFF_UP | IFF_BROADCAST | IFF_NOTRAILERS | IFF_RUNNING;
    if (cCONFIG_IP_MULTICAST)
	if_flags |= IFF_MULTICAST;
    
    if (flags & IFCFG_SPROMISC)
	if_flags |= IFF_PROMISC;

    return sys_if_flags_set_direct(dev, if_flags);
}

int sys_if_flags_get(char *dev, int *flags)
#ifdef CONFIG_RG_TARGET_LINUX
{
    struct ifreq ifr;
   
    if (sys_if_ioctl_execute(dev, &ifr, SIOCGIFFLAGS))
	return -1;

    *flags = ifr.ifr_flags;
    return 0;
}
#elif defined(CONFIG_RG_TARGET_VXWORKS)
{
    return ifFlagGet(dev, flags);
}
#endif

int sys_if_snd_len_get(char *dev, int *int_len)
#ifdef CONFIG_RG_TARGET_LINUX
{
    struct ifreq ifr;
    
    if (sys_if_ioctl_execute(dev, &ifr, SIOCGIFTXQLEN))
	return -1;

    *int_len = ifr.ifr_ifru.ifru_ivalue;
    return 0;
}
#elif defined(CONFIG_RG_TARGET_VXWORKS)
{
    struct ifnet *iface;
   
    if (!(iface = ifunit(dev)))
	return -1;

    *int_len = iface->if_snd.ifq_len;
    return 0;
}
#endif

int sys_if_addr_set(char *dev, u32 ip_addr)
#ifdef CONFIG_RG_TARGET_LINUX
{
    return sys_if_ioctl_param_execute(dev, SIOCSIFADDR, ip_addr);
}   
#elif defined(CONFIG_RG_TARGET_VXWORKS)
{
    int rc; 
    struct sockaddr_in mask;
    struct in_addr ip;
    
    MZERO(mask);

    ip.s_addr = ip_addr;

    rc = ifAddrSet(dev, inet_ntoa(ip));
    /* Delete route rule 0.0.0.0 added by ifAddrSet */
    if (!ip_addr)
    {
	sys_if_mask_get(dev, (struct sockaddr *)&mask);
	rc |= mRouteDelete(inet_ntoa(ip), mask.sin_addr.s_addr, 0, 0);
    }

    return rc;
}
#endif

int sys_if_addr_get(char *dev, struct sockaddr *ip_addr)
#ifdef CONFIG_RG_TARGET_LINUX
{
    struct ifreq ifr;

    if (sys_if_ioctl_execute(dev, &ifr, SIOCGIFADDR))
	return -1;

    memcpy(ip_addr, &ifr.ifr_addr, sizeof(struct sockaddr));
    return 0;
}
#elif defined(CONFIG_RG_TARGET_VXWORKS)
{
    char *buf;
    int ret = -1;

    buf = zalloc_l(INET_ADDR_LEN);
    if (!buf)
	goto Error;
    
    if (ifAddrGet(dev, buf)!=OK)
	goto Error;
    
    if (!rg_inet_aton(buf, &((struct sockaddr_in *)ip_addr)->sin_addr))
	goto Error;
    
    ip_addr->sa_family = AF_INET;
    
    ret = 0;
Error:
    nfree(buf);
    return ret;
}
#endif

int sys_if_mask_get(char *dev, struct sockaddr *mask)
#ifdef CONFIG_RG_TARGET_LINUX
{
    struct ifreq ifr;

    if (sys_if_ioctl_execute(dev, &ifr, SIOCGIFNETMASK))
	return -1;

    memcpy(mask, &ifr.ifr_addr, sizeof(struct sockaddr));
    return 0;
}
#elif CONFIG_RG_TARGET_VXWORKS
{
    if (ifMaskGet(dev, (int *)&((struct sockaddr_in *)mask)->sin_addr.s_addr)
	!= OK)
    {
	return -1;
    }
    
    return 0;
}
#endif

int sys_if_dst_addr_set(char *dev, u32 remote)
#ifdef CONFIG_RG_TARGET_LINUX
{
    return sys_if_ioctl_param_execute(dev, SIOCSIFDSTADDR, remote);
}
#elif defined(CONFIG_RG_TARGET_VXWORKS)
{
    struct in_addr remote_ip;

    remote_ip.s_addr = remote;
    return ifDstAddrSet(dev, inet_ntoa(remote_ip));
}
#endif

int sys_if_mask_set(char *dev, u32 net_mask)
{
#ifdef CONFIG_RG_TARGET_LINUX
    return sys_if_ioctl_param_execute(dev, SIOCSIFNETMASK, net_mask);
    
#elif defined(CONFIG_RG_TARGET_VXWORKS)
    return ifMaskSet(dev, ntohl(net_mask)); 
#endif
}

int sys_if_bcast_addr_set(char *dev, u32 bcast_addr)
#ifdef CONFIG_RG_TARGET_LINUX
{
    return sys_if_ioctl_param_execute(dev, SIOCSIFBRDADDR, bcast_addr);
}
#elif defined(CONFIG_RG_TARGET_VXWORKS)
{
    struct in_addr bcast_ip;
    bcast_ip.s_addr = bcast_addr;

    return ifBroadcastSet(dev, inet_ntoa(bcast_ip));
}
#endif

int sys_if_ext_set(char *dev, int value)
{
    struct ifreq ifr;

    MZERO(ifr); /* make valgrind happy */
    strncpyz(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
    ifr.ifr_flags = value;
    return openrg_module_ctrl(KOS_CDT_KNET_IF_EXT, RG_IOCTL_KNET_IF_SET_EXT,
	&ifr);
}

int sys_if_ext_get(char *dev, int *value)
{
    struct ifreq ifr;

    strncpyz(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
    if (openrg_module_ctrl(KOS_CDT_KNET_IF_EXT, RG_IOCTL_KNET_IF_GET_EXT, &ifr))
	return -1;
    *value = ifr.ifr_flags;
    return 0;
}

int sys_if_is_pppoe_set(char *dev, int value)
{
    struct ifreq ifr;

    MZERO(ifr); /* make valgrind happy */
    strncpyz(ifr.ifr_name, dev, sizeof(ifr.ifr_name));
    ifr.ifr_flags = value;
    return openrg_module_ctrl(KOS_CDT_KNET_IF_EXT, RG_IOCTL_KNET_IF_SET_PPPOE,
	&ifr);
}

char *sys_alias_name_get(char *dev_name, int index)
{
#ifdef CONFIG_RG_TARGET_LINUX
    static char name[IFNAMSIZ];
    
    if (index >= 0)
    {
	sprintf(name, "%s:%d", dev_name, index);
	return name;
    }
    else
#endif
    	return dev_name;
}

void sys_sleep(int seconds)
#ifdef CONFIG_RG_TARGET_VXWORKS
{
    int total_ticks;

    total_ticks = sys_get_clock_rate() * seconds;
    taskDelay(total_ticks);
}
#else
{
    sleep(seconds);
}
#endif

#ifdef CONFIG_RG_TARGET_VXWORKS
static alias_conf_ret_t sys_vx_alias_remove(char *dev, u32 ip_addr)
{
    int s, ret;
    struct ifreq ifr;
    struct sockaddr_in *ip = (struct sockaddr_in *)&ifr.ifr_addr;
    
    if (!ip_addr)
	return ALIAS_CONF_RET_SUCCESS;
    if ((s = sys_if_ioctl_prepare(dev, &ifr)) < 0)
	return ALIAS_CONF_RET_ERROR;
    ip->sin_family = AF_INET;
    ip->sin_len = sizeof(*ip);
    ip->sin_addr.s_addr = ip_addr;
    
    ret = ioctl(s, SIOCDIFADDR, (int)&ifr);
    close(s);
    
    return ret ? ALIAS_CONF_RET_ERROR : ALIAS_CONF_RET_SUCCESS;
}

static alias_conf_ret_t sys_vx_alias_add(char *dev, u32 ip_addr, u32 net_mask,
    u32 remote)
{
    struct in_aliasreq req;
    struct sockaddr_in *ip = &req.ifra_addr;
    struct sockaddr_in *broadcast = &req.ifra_broadaddr;
    struct sockaddr_in *mask = &req.ifra_mask;
    int s, ret;

    MZERO(req);
    ip->sin_family = broadcast->sin_family = mask->sin_family = AF_INET;
    ip->sin_len = broadcast->sin_len = mask->sin_len = sizeof(*ip);

    strcpy(req.ifra_name, dev);

    ip->sin_addr.s_addr = ip_addr;
    mask->sin_addr.s_addr = net_mask;
    if (remote)
	broadcast->sin_addr.s_addr = remote;
    else
	broadcast->sin_addr.s_addr = ip_addr | ~net_mask;
    
    if ((s = sockets_open(AF_INET, SOCK_DGRAM, 0)) < 0)
    {
	rg_error_f(LWARN, "Operation failed %m");
	return ALIAS_CONF_RET_ERROR;
    }

    ret = ioctl(s, SIOCAIFADDR, (int)&req);
    
    /* BSD kernel also tries to add a default route rule - we don't want to
     * fail if such a route already exists.
     */
    if (ret && errno == EEXIST)
	ret = 0;

    close(s);

    return ret ? ALIAS_CONF_RET_ERROR : ALIAS_CONF_RET_SUCCESS;
}
#endif

alias_conf_ret_t sys_alias_configure(char *dev, u32 ip_addr, u32 net_mask,
    u32 remote, ifconfig_flag_t flags)
{
#ifdef CONFIG_RG_TARGET_VXWORKS
    if (flags == IFCFG_SDOWN)
	return sys_vx_alias_remove(dev, ip_addr);

    return sys_vx_alias_add(dev, ip_addr, net_mask, remote);
#else
    return ALIAS_CONF_RET_CONTINUE;
#endif
}

int sys_if_name_get(int if_idx, char **if_name)
#ifdef CONFIG_RG_TARGET_LINUX
{
    int s;
    int ret = -1;
    struct ifreq ifr;
    
    *if_name = NULL;
    s = sys_if_ioctl_prepare("", &ifr);
    if (s < 0)
	return -1;

    ifr.ifr_ifindex = if_idx;
    if (ioctl(s, SIOCGIFNAME, &ifr))
	goto Exit;
    
    if (!(*if_name = strdup_l(ifr.ifr_name)))
	goto Exit;

    ret = 0;
Exit:
    close(s);
    return ret;
}
#elif defined(CONFIG_RG_TARGET_VXWORKS)
{
    struct ifnet *cur;
    str_t *tmp;
    int ret = -1;
    
    *if_name = NULL;
    for (cur = ifnet; cur && cur->if_index != if_idx; cur = cur->if_next);

    if (!cur)
	return -1;

    tmp = str_alloc();
    str_sprintf(tmp, "%s%d", cur->if_name, cur->if_unit);
    *if_name = strdup_l(tmp->str);
    if (!*if_name)
	goto Error;
    ret = 0;

Error:
    str_free(tmp);
    return ret;
}
#endif

int sys_if_metric_get(char *dev, int *int_metric)
#ifdef CONFIG_RG_TARGET_LINUX
{
    struct ifreq ifr;
   
    if (sys_if_ioctl_execute(dev, &ifr, SIOCGIFMETRIC))
	return -1;

    *int_metric = ifr.ifr_ifru.ifru_ivalue;
    return 0;
}
#elif defined(CONFIG_RG_TARGET_VXWORKS)
{
    if (ifMetricGet(dev, int_metric) != OK)
    	return -1;
    
    return 0;
}
#endif

int sys_if_mtu_get(char *dev, int *int_mtu)
#ifdef CONFIG_RG_TARGET_LINUX
{
    struct ifreq ifr;
   
    if (sys_if_ioctl_execute(dev, &ifr, SIOCGIFMTU))
	return -1;
    
    *int_mtu = ifr.ifr_ifru.ifru_ivalue;
    return 0;
}
#elif defined(CONFIG_RG_TARGET_VXWORKS)
{
    struct ifnet *iface;

    if (!(iface = ifunit(dev)))
	return -1;

    *int_mtu = (int)iface->if_mtu;
    return 0;
}
#endif

#if 0

int sys_ip_forward_change(int flag)
#ifdef CONFIG_RG_TARGET_VXWORKS
{
    M2_IP ip_vars;
     
    MZERO(ip_vars);
    ip_vars.ipForwarding = flag ? M2_ipForwarding_forwarding :
	M2_ipForwarding_not_forwarding;
    return m2IpGroupInfoSet(M2_IPFORWARDING, &ip_vars) == OK ? 0 : -1;
}
#elif defined (CONFIG_RG_TARGET_LINUX)
{
    return openrg_module_ctrl(KOS_CDT_IPV4, IPV4_IOCTL_FWD, (void *)flag);
}
#endif

int sys_ip_ttl_change(int ttl_val)
#ifdef CONFIG_RG_TARGET_VXWORKS
{
    M2_IP ip_vars;
     
    MZERO(ip_vars);
    ip_vars.ipDefaultTTL = ttl_val;
    return m2IpGroupInfoSet(M2_IPDEFAULTTTL, &ip_vars) == OK ? 0 : -1;
}
#elif defined (CONFIG_RG_TARGET_LINUX)
{
    return openrg_module_ctrl(KOS_CDT_IPV4, IPV4_IOCTL_TTL, (void *)ttl_val);
}
#endif

#endif /* if 0*/

int set_if_mac(char *iface, ether_mac_t *mac)
#ifdef CONFIG_RG_TARGET_LINUX
{   
    int s;
    int ret;
    struct ifreq ifr;

    if ((s = sockets_open(AF_PACKET, SOCK_PACKET, htons(ETH_P_ALL))) < 0)
	return rg_error_f(LERR, "Failed to open socket %m");

    MZERO(ifr);
    memcpy(ifr.ifr_name, iface, strlen(iface));
    ifr.ifr_hwaddr.sa_family = ARPHRD_ETHER;
    memcpy(ifr.ifr_hwaddr.sa_data, mac, sizeof(*mac));

    ret = ioctl(s, SIOCSIFHWADDR, &ifr);

    if (ret) 
	rg_error_f(LERR, "SIOCSIFHWADDR failed %m");

    close(s);

    return ret;
}
#else
{
    struct arpcom *dev;
    struct ifaddr *ifap;

    if (!(dev = (struct arpcom *)ifunit(iface)))
	return rg_error_f(LERR, "No device named %s", iface);
    
    memcpy(dev->ac_enaddr, *mac, sizeof(*mac));

    for (ifap = dev->ac_if.if_addrlist; ifap; ifap = ifap->ifa_next)
    {
	if (ifap->ifa_addr->sa_family == AF_LINK)
	    break;
    }

    if (ifap)
    {
	memcpy(LLADDR((struct sockaddr_dl *)ifap->ifa_addr), *mac,
	    sizeof(*mac));
    }
 
    return 0;
}
#endif

#ifdef CONFIG_RG_TARGET_VXWORKS
static int sys_rtentry_cb(struct rtentry *rt, void *list_head)
{
    route_t **head = (route_t **)list_head;

    struct sockaddr_in *dst = (struct sockaddr_in *)rt_key(rt);
    struct sockaddr_in *gate = (struct sockaddr_in *)rt->rt_gateway;
    struct sockaddr_in *mask = (struct sockaddr_in *)rt_mask(rt);

    route_t *tmp = (route_t *)zalloc_l(sizeof(route_t));
    if (!tmp)
	return -1;
	
    snprintf(tmp->interface, sizeof(tmp->interface), "%s%d",
	rt->rt_ifp->if_name, rt->rt_ifp->if_unit);
    tmp->dest.s_addr = dst->sin_addr.s_addr;
    if (rt->rt_flags & RTF_GATEWAY)
	tmp->gateway.s_addr = gate->sin_addr.s_addr;
    tmp->netmask.s_addr = mask ? mask->sin_addr.s_addr : 0xffffffff;
    tmp->metric = 0; /* XXX: Find metric */
    tmp->flags = rt->rt_flags;
    tmp->next = *head;
    *head = tmp;
    
    return 0;
}
#endif

route_t *route_table_alloc(void)
#ifdef CONFIG_RG_TARGET_LINUX
{
    /* TODO: Replace parsing of proc/net/route with an implementation of
     * SIOCRTMSG ioctl
     */
    
    route_t *head = NULL, **cur = &head;
    int is_error = 1, count, ref_count, use;
    FILE *fp = NULL;
    char line[MAX_LINE_SIZE];

    if (!(fp = fopen(PROC_NET_ROUTE, "r")))
	goto Exit;
    
    /* Run over /proc/net/route */
    fgets(line, sizeof(line), fp); /* Skip headers */
    while (fgets(line, sizeof(line), fp))
    {
	route_t *tmp = (route_t *)zalloc_e(sizeof(route_t));

	count = sscanf(line, "%s %x %x %d %d %d %d %x", tmp->interface,
	    &tmp->dest.s_addr, &tmp->gateway.s_addr, &tmp->flags, &ref_count,
	    &use, &tmp->metric, &tmp->netmask.s_addr);

	if (count==8 && tmp->flags&RTF_UP)
	{
	    *cur = tmp;
	    cur = &tmp->next;
	    continue;
	}
	free(tmp); /* Disabled route - don't add to list */
    }

    is_error = 0;
Exit:
    if (fp)
	fclose(fp);
    if (is_error)
	route_list_free(&head);
    return head;
}
#else
{
    route_t *head = NULL;

    if (knet_route_table_loop(RTF_UP, sys_rtentry_cb, (void *)&head))
	route_list_free(&head);
	
    return head;	
}
#endif

/* Delete an ARP entry for a given device name and ip address */
int sys_arp_delete(char *name, struct in_addr ip)
{
    int s;
    struct sockaddr_in ip_addr = {};
    struct arpreq req = {};
    int ret = -1;
    
    if ((s = sockets_open(AF_INET, SOCK_DGRAM, 0)) < 0)
	return rg_error_f(LERR, "Failed to open socket %m");
    
    req.arp_flags = ATF_PERM;
    ip_addr.sin_family = AF_INET;
    ip_addr.sin_addr.s_addr = ip.s_addr;
    memcpy(&req.arp_pa, &ip_addr, sizeof(ip_addr));
#ifdef CONFIG_RG_TARGET_LINUX    
    memcpy(&req.arp_dev, name, strlen(name)+1);
#endif
    if (ioctl(s, SIOCDARP, (char *)&req) < 0)
	goto Exit;

    ret = 0;
Exit:
    close(s);
    return ret;
}

/* Get the MAC for the specific IP using the ARP table.
 * Return 0 on success, -1 if ARP entry not found */
int dev_and_ip_to_mac_get(char *name, struct in_addr ip, ether_mac_t *mac)
{
    int s;
    struct sockaddr_in ip_addr;
    struct arpreq req;
    int ret = -1;
    
    if ((s = sockets_open(AF_INET, SOCK_DGRAM, 0)) < 0)
	return rg_error_f(LERR, "Failed to open socket %m");
    
    MZERO(ip_addr);
    MZERO(req);
    ip_addr.sin_family = AF_INET;
    ip_addr.sin_addr.s_addr = ip.s_addr;
    memcpy(&req.arp_pa, &ip_addr, sizeof(ip_addr));
#ifdef CONFIG_RG_TARGET_LINUX    
    memcpy(&req.arp_dev, name, strlen(name)+1);
#endif
    if (ioctl(s, SIOCGARP, (char *)&req) < 0)
	goto Exit;

    /* The ioctl was succesful, mac will be filled with the ARP cache MAC */
    memcpy(mac, &req.arp_ha.sa_data, sizeof(ether_mac_t));
    ret = 0;
Exit:
    close(s);
    return ret;
}

void sys_get_random(int *buf, int count)
#ifdef CONFIG_RG_TARGET_LINUX
{
    unsigned int nbytes = sizeof(int) * count;
    int fd;

    /* We use urandom because it doesn't block (like random) */
    fd = open("/dev/urandom", O_RDONLY);
    if (fd < 0)
	rg_error(LPANIC, "Cannot open /dev/urandom");
#if 0
    if (force_read(fd, (char *)buf, nbytes) != nbytes)
    {
	close(fd);
	rg_error(LPANIC, "Cannot read /dev/urandom");
    }
#endif
    if (read(fd, (char *)buf, nbytes) != nbytes)
    {
	close(fd);
	rg_error(LPANIC, "Cannot read /dev/urandom");
    }


    close(fd);
}
#else
{
    unsigned int *memptr;
    int i = 0;

    /* Zeroing buf has no meaning - so we don't */

    for (memptr = (unsigned int *)RAM_LOW_ADRS; 
	memptr < (unsigned int *)RAM_HIGH_ADRS; memptr++)
    {
	buf[i] ^= *memptr;
	i = (i + 1) % count;
    }
}
#endif

#ifdef CONFIG_RG_TARGET_VXWORKS
char *sys_vx_inet_ntoa(struct in_addr in)
{
    static char ip_str[INET_ADDR_LEN];

    inet_ntoa_b(in, ip_str);
    return ip_str;
}
#endif
