/* LICENSE_CODE JUNGO */

#ifndef _WPA_ASSOC_H_
#define _WPA_ASSOC_H_

#include <802.1x/auth1x_conn.h>

typedef enum {
    WPA_START_MSG1 = 0,
    WPA_WAIT_MSG2  = 1,
    WPA_SEND_MSG3  = 2,
    WPA_WAIT_MSG4  = 3,
    WPA_DONE       = 4,
} wpa_assoc_state_t;

typedef struct wpa_assoc_t wpa_assoc_t;

struct wpa_assoc_t {
    wpa_assoc_t *next;
    wpa_port_t *port;
    ether_mac_t sta_mac;
    wpa_info_elem_t *sta_info_elem;
    wpa_assoc_state_t ptk_state;  /* Current ptk negotiation state */
    wpa_assoc_state_t gtk_state;  /* Current gtk negotiation state */
    wpa_nonce_t nonce;            /* ANonce or GNonce depending on current 
				   * negotiation 
				   */
    u64 ap_to_sta_replay_count;   /* Replay counter maintained by the AP */
    u64 sta_to_ap_replay_count;   /* Replay counter maintained by the STA */
    wpa_pmk_t pmk;
    int pmk_valid;
    wpa_ptk_t ptk;
    int ptk_neg_retries_left;
    int gtk_neg_retries_left;
    int has_gtk;
    int gtk_updated;
    auth1x_conn_t *auth1x_conn;
    wpa_sta_type_t sta_type;
    wpa_cipher_t pairwise_cipher; /* Cipher of pairwise key installed for the 
				   * station or 0 if no pairwise key is 
				   * installed 
				   */
};

/* Opens a new association */
wpa_assoc_t *wpa_assoc_open(wpa_port_t *port, ether_mac_t sta_mac, 
    wpa_info_elem_t *sta_info_elem);

/* Cleans up and free association */
void wpa_assoc_close(wpa_assoc_t *assoc);

/* Starts 802.1x authentication */
void wpa_assoc_start_auth1x(wpa_assoc_t *assoc); 
    
/* Starts PTK (4 way handshake) negotiation */
void wpa_assoc_start_ptk_neg(wpa_assoc_t *assoc);

/* Starts GTK negotiation */
void wpa_assoc_start_gtk_neg(wpa_assoc_t *assoc);

/* Sends a GTK using a legacy 802.1X EAPOL-Key message */
void wpa_assoc_send_gtk_using_8021x(wpa_assoc_t *assoc);

/* Disassociates station */
void wpa_assoc_disassoc(wpa_assoc_t *t);

/* Deauthenticates a station */
void wpa_assoc_deauth(wpa_assoc_t *t);

/* Returns 1 if we're waiting for this station to acknowledge receipt of the
 * group key
 */ 
int wpa_assoc_is_in_gtk_update(wpa_assoc_t *t);

/* Called when a station type is detected */
void wpa_assoc_sta_type_detected(wpa_assoc_t *t);

/* Called to update the station type */
void wpa_assoc_set_sta_type(wpa_assoc_t *assoc, wpa_sta_type_t sta_type);

void wpa_assoc_free(wpa_assoc_t *assoc);

void wdrv_wpaSetMicFailureReportFromStation(BOOL value);

#endif
