/* LICENSE_CODE JUNGO */

#ifndef _WPA_KEY_UTILS_H_
#define _WPA_KEY_UTILS_H_

#include <wpa/wpa_common.h>

/* Initializes a key counter. */
void wpa_key_utils_init_key_counter(
    wpa_nonce_t *key_counter, /* output - random value */
    ether_mac_t local_addr    /* local MAC address */
    );

/* Calculates a Pairwise Transient Key. */
void wpa_key_utils_calc_ptk(
    wpa_ptk_t *ptk,                 /* output */
    wpa_pmk_t *pmk,                 /* pairwise master key */
    wpa_nonce_t *anonce,            /* authenticator's nonce */
    wpa_nonce_t *snonce,            /* supplicant's nonce */
    ether_mac_t authenticator_addr, /* authenticator's MAC address */
    ether_mac_t supplicant_addr     /* supplicant's MAC address */
    );

/* Calculates a Group Transient Key. */
void wpa_key_utils_calc_gtk(
    wpa_gtk_t *gtk,		   /* output */
    wpa_gmk_t *gmk,		   /* group master key */
    wpa_nonce_t *gnonce,	   /* group's nonce */
    ether_mac_t authenticator_addr /* authenticator's MAC address */
    );

/* Calculates a pre-shared key from a pass-phrase. */
void wpa_key_utils_calc_psk(
    wpa_pmk_t *psk,    /* output - 256 bits of key */
    char *pass_phrase, /* ascii null terminated string up to 63 characters */
    u8 *ssid,          /* string up to 32 bytes */
    int ssid_len       /* length of ssid in bytes */
    );

/* Increment of any value, presented as array of bytes */
void wpa_key_utils_inc(
    u8 *value, /* in/out parameter. array of bytes */
    int size   /* number of bytes in array */
    );

/* Calculate HMAC-MD5 over a buffer */
void wpa_key_util_hmac_md5(
    u8 *digest,   /* output - key, computed according to HMAC-MD5 (rfc2104)
                   * size of output - 16 bytes */
    u8 *data,     /* input buffer - pointer to data stream */
    int data_len, /* length of data stream in bytes */
    u8 *key,      /* input - pointer to authentication key */
    int key_len   /* length of authentication key in bytes */
    );

/* HMAC-SHA1 is the EAPOL-Key encryption algorithm used to protect the
 * distributed GTK
 */
void hmac_sha1(u8 *digest, u8 *text, int text_len, u8 *key, int key_len);

/* Pseudo-random function for generate cryptographic quality random numbers. */
void prf(u8 *key, int key_len, u8 *prefix, int prefix_len, u8 *data, 
    int data_len, u8 *output, int len);

/* Returns the length of key of the specified cipher */
int wpa_key_utils_get_key_len(wpa_cipher_t cipher);

#endif 
