/* LICENSE_CODE JUNGO */

#ifndef _WPA_PORT_H_
#define _WPA_PORT_H_

#include <802.1x/auth1x.h>
#include <wpa.h>
#include <event.h>
#include "wpa_info_elem.h"
#include "wpa_assoc.h"


/* Context for wpa_port objects */
struct wpa_port_t {
    wpa_port_params_t params;              /* User configured parameters */
    wpa_nonce_t key_counter;               /* 256-bit key counter used for nonce
                                            * generation
                                            */
    wpa_pmk_t psk;                         /* Pre-shared key */
    wpa_assoc_t *assoc_list;               /* List of associated stations */
    wpa_info_elem_t *ap_info_elem;         /* AP's information element */
    wpa_gtk_t gtk;                         /* Current group key */
    int current_gtk_index;                 /* Current group key index */
    int gtk_update_in_progress;            /* == 1 if we started a GTK update 
					    * and haven't received acks from 
					    * all stations yet
					    */
    int delayed_gtk_update;                /* == 1 if another GTK update should 
					    * begin as soon as the current one 
					    * ends
					    */
    wpa_cipher_t multicast_cipher;         /* Multicast cipher in use */
    ap_dev_t *ap_dev;                      /* Pointer to underlying layer 
                                            * context
                                            */
    struct timeval last_mic_failure_time;  /* Time of last MIC failure */
    wpa_nonce_t gnonce;                    /* Current GNonce */
    int num_of_legacy_stas;                /* Number of legacy stations 
                                            * currently associated
                                            */
    int num_of_8021x_stas;                 /* Number of 802.1X stations
                                            * currently connected
                                            */
    int num_of_wpa_stas;                   /* Number of WPA stations currently 
                                            * connected
                                            */
    int delayed_preconfigured_key_install; /* Index of preconfigured key that
                                            * was scheduled for delayed 
                                            * installation
                                            */
    int countermeasures_invoked;           /* MIC failure countermeasures are
					    * invoked
					    */
};

/* Called by a wpa_association_t object when a 802.1x authentication with the
 * station ends
 */
void wpa_port_8021x_auth_end(wpa_port_t *port, wpa_assoc_t *assoc, int success);

/* Called by a wpa_association_t object when a 4-way handshake with the
 * associated station ends
 */
void wpa_port_ptk_neg_end(wpa_port_t *port, wpa_assoc_t *assoc, int success);

/* Called by a wpa_association_t object when a group key update for the
 * associated station ends 
 */
void wpa_port_gtk_neg_end(wpa_port_t *port, wpa_assoc_t *assoc, int success);

/* Called by a wpa_association_t object when a station requests a group key
 * update
 */
void wpa_port_gtk_update_request(wpa_port_t *port, wpa_assoc_t *assoc);
 
/* Generates a Nonce from the key counter and increments the key counter */
void wpa_port_generate_nonce(wpa_port_t *port, wpa_nonce_t *nonce); 

/* Called when a MIC failure occurs */
void wpa_port_mic_failure(wpa_port_t *t, wpa_assoc_t *assoc);

/* Called when a station type is detected */
void wpa_port_sta_type_detected(wpa_port_t *t, wpa_assoc_t *assoc);

/* Called to update the counters of each station type */
void wpa_port_update_sta_counter(wpa_port_t *t, wpa_sta_type_t sta_type, 
    int delta);

#endif
