/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |** 
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																									*/
/*		MODULE:	bssbridge.c	      														*/
/*   PURPOSE:	Bssbridge functions.	      			 								*/
/*																									*/
/***************************************************************************/

#ifdef VXWORK

#include <stdarg.h>
#include "pform_types.h"
#include "rt_os_vxw.h"
#include "utilsReplvl.h"
#include "utilsTcb.h"
#include "mainsm.h"
#include "memMngr.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "utils.h"
#include "wpaIfc.h"
#include "802_11Defs.h"
#include "bssbridge.h"

#else
#ifdef __linux__
#include "pform_types_user.h"
#include "linux_ipc_user.h"
#include <signal.h>
#endif  /* __linux__ */
#endif /* vxworks */


#include <802.1x/wep_port.h>
#include <ipc.h>
#include <event.h>
#include <802.1x/eap_radius.h>
#include <wpa/wpa_info_elem.h>
#include <util/random.h>
#include <util/rg_error.h>


/*
 * In the included file <sys/un.h> a sockaddr_un is defined as follows
 * struct sockaddr_un {
 *  short   sun_family;
 *  char    sun_path[108];
 * };
 */
#include <sys/un.h>

#include <stdio.h>
#include <sys/socket.h>
#include <unistd.h>
#include <stdio.h>
#include <netinet/if_ether.h>
#include <sys/ioctl.h>
#include "dda_wpa.h"
#include "wpaIfc.h"
#include "wpa_common.h"
#include "wpa_assoc.h"

#include "dda_wpaauth_ioctl.h"
#include "wcfglib.h"


/*-----------------------------------------------------------*/
/*         Constants										 */
/*-----------------------------------------------------------*/

void wdrv_wpaSetMicFailureReportInRx(BOOL value);
BOOL wdrv_wpaGetMicFailureReportInRx(void);
static int send_ass_res(UINT16 index,int flag);
BOOL wdrv_wpaGetStationSecAttack(UINT16 index);
STATUS canStationAssc(u8 *info_elem, int info_elem_len);	

/* Alarm callback - called by driver with relevant info     */
typedef  void (*wdrv_reportFunc)(char *buffer);

/*-----------------------------------------------------------*/
/*         Local Vartiables									 */
/*-----------------------------------------------------------*/

static int wpaSocket;

/*-----------------------------------------------------------*/
/*         External Variables									 */
/*-----------------------------------------------------------*/
#ifdef VXWORKS
    extern wdrv_reportFunc wdrv_utilsReportCB;
#endif

/*-----------------------------------------------------------*/
/*         Local Prototypes									 */
/*-----------------------------------------------------------*/
static void	wpaTask(tiap_wpa_auth_t *pCfg);
static void wpa_input(int sock, void *data);


/*-----------------------------------------------------------*/
/*         WPA Package Support Section						 */
/*-----------------------------------------------------------*/

/* WEP Port Section  - temporal only, in order to enable to the 802.1X port to work without WPA. */
static void	init8021xPortParams(auth1x_port_params_t *params);

/* 802.1X Port Section */
static auth1x_port_t			*pDev_if_auth_8021x;	/* Returned from the 'auth1x_port_open()' function, 
																	forward to the WPA port in the  'wpa_port_open()' function */
static auth1x_port_params_t		auth_8021x_params;		/* Forwarded in the 'auth1x_port_open()' function */

static auth1x_port_cb_t			auth_8021x_callbacks;	/* Forwarded in the 'auth1x_port_open()' function  */

static auth1x_pc_ops_t 			portCBs;			/* To contain the callbacks that the AP supplies to the 802.1X module */

static void						*pXPortContext;	/* The WPA port context. Used in every call to WPA port functions. */


static unsigned int				apPortHandle = 2;		/* We initializes our AP device in dummy handle. */

static auth1x_pc_t				*pApPort = (auth1x_pc_t *)&apPortHandle;

static	char	pskPwd[MAX_PSK_PWD_LEN + 1];
static  char   	apSsid[64];

static auth1x_pkt_handler_t			eapolPacketHandler = NULL;

/* The following callbacks are forwarded to the 802.1X port as part of the port params in the 'auth1x_port_open()' function */
static auth1x_pc_t *open_by_x_port(void *dev_id,
								   auth1x_pkt_handler_t auth1x_pkt_handler, 
								   void *context);

static void	 close_by_x_port(auth1x_pc_t *t);


/* WPA Port Section */
static ap_dev_cbs_t			ap_dev_cbs;		/* This structure used to hold the Callbacks functions 
											that the WPA package supplies to the AP driver. */
static wpa_port_t			*pWpaPort;		/* Returned from the 'wpa_port_open()' function, 
												used in the 'wpa_port_close()' function */
static wpa_port_params_t	portParams;			/* Forwarded in the 'wpa_port_open()' function */
static ap_dev_ops_t			deviceCBs;		/* To contain the callbacks that the AP supplies to the WPA module */

static void					*pWpaPortContext;	/* The WPA port context. Used in every call to WPA port functions. */

static void		initWpaPortParams(wpa_port_params_t *pPortParams);

/* Radius client Section */
static void *radius_client = NULL;			/* returned from the ;radius_client_open()' function */

static radius_params_t	radiusParams;
static server_list_t		serverList;

static eap_method_radius_params_t clientParams;

static eap_method_id_t client_get(u8 *id, int id_len, void **eap_method_params);

/* The following callbacks are forwarded to the WPA package as part of the port params in the 'wpa_port_open()' function */
static ap_dev_t	*open_by_wpa_port(void *dev_id,
								  ap_dev_cbs_t *cb,
								  void *context);   

static void		close_by_wpa_port(ap_dev_t *t);

static int wpa_wait_for_cfg(tiap_wpa_auth_t *pCfg);

static UINT32				apDeviceHandle = 1;		/* We initializes our AP device in dummy handle. */
static ap_dev_t				pApDev = &apDeviceHandle;

/* General AP Device Section */
static UINT32				local_dev_id = 1;		/* The device ID, we don't use it at all. */


/* Used to forward the EAPOL frame from the WLAN to the WPA package. */
#ifdef VXWORKS
static char eapolBuffer[MAX_DATA_BODY_LENGTH + sizeof(dot11_header_T)];
static char *pEapol;
static UINT32	eapolDataLen;
#endif

/*static void formatEapolToEthernet(mem_MSDU_T*  msduPtr, UINT32 *dataLen);	*/

#define PORT_IN_WPA_PSK_MODE(t) (params.psk_param != WPA_PSK_PARAM_NONE)

static wpa_cipher_t select_multicast_cipher(wpa_port_params_t *params);
void set_ap_info_elem(wpa_port_params_t *params , wpa_info_elem_t **ap_info_elem,int *p_info_elem_len);

static BOOL secAttackTable[MAX_STATIONS];

BOOL	micFailureReportInRx = TRUE;

#define NUMBER_OF_SENDING_HELLO_MSG     50


/****************************************************************************
 *                      wpa_wait_for_cfg()
 ****************************************************************************
 * DESCRIPTION:	this function creates a UNIX domain datagram socket, binds a name to it,
 * then reads from the socket.	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
static int wpa_wait_for_cfg(tiap_wpa_auth_t *pCfg)
{
    int sock, ret;
    struct sockaddr_un name;

    /* Create socket from which to read. */
    sock = socket(AF_UNIX, SOCK_DGRAM, 0);
    if (sock < 0) {
        printf("wpa_wait_for_cfg:error opening datagram socket\n");
        return -1;
    }


    /* Create name. */
    name.sun_family = AF_UNIX;

    strcpy((char*)name.sun_path,TIAP_SOCKADDR_WPAAUTH);

    unlink(TIAP_SOCKADDR_WPAAUTH);
    /* Bind the UNIX domain address to the created socket */
    if (bind(sock, (struct sockaddr *) &name, sizeof(struct sockaddr_un)) < 0 )
	{
		  printf("wpa_wait_for_cfg:error binding datagram socket\n");
          return -1;
    }
    printf("socket %s successfully created, wpa module waits for configuration\n", TIAP_SOCKADDR_WPAAUTH);


    /* Read from the socket */
    if ((ret = read(sock, (char*)pCfg, sizeof(tiap_wpa_auth_t))) < 0)
	{
        printf("wpa_wait_for_cfg: receiving error datagram packet\n");
		close(sock);
		return -1;
	}
	
	if(ret != sizeof(tiap_wpa_auth_t))
	{
        printf("wpa_wait_for_cfg:receive error cfg size ,size receive = %d\n",ret);
		close(sock);
		return -1;
	}


	
    printf("wpa_wait_for_cfg:reeceive configuration\n");
	
    close(sock);

    unlink(TIAP_SOCKADDR_WPAAUTH);
	
	return 0;
}


/****************************************************************************
 *                      wdrv_bssbridgeInit()
 ****************************************************************************
 * DESCRIPTION:	Bssbridge module init function	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
#ifdef VXWORKS
STATUS wdrv_wpaInit(void)
#else
int main(void) 
#endif
{
#ifdef VXWORKS
	int rc, i;
	unsigned long  taskID;

	/* create task */
	rc = rt_osTaskCreate( "WPAT"           ,         /* name              */
                 wdrv_getTasksConfig()->WpaTaskPriority,  /* priority          */
                 wdrv_getTasksConfig()->WpaTaskStackSize, /* supervisor stack  */
				 &taskID);                  /* id                */
	if (rc != OS_NOERR) 
	{
		rg_error(LPANIC, "WDRV_WPA: Error Creating Task. Error Code = %ld\n", rc);
		return NOK;
	}

	/* start task */
	rc = rt_osTaskStart( taskID,
				OS_TASK_PREEMPT,
				wpaTask);
	if (rc != OS_NOERR) 
	{
		rg_error(LPANIC, "WDRV_WPA: Error Starting Task. Error Code = %ld\n", rc);
		return NOK;
	}

#else
	
	tiap_wpa_auth_t Cfg;

	//signal(SIGINT, SIG_IGN);

	wpa_wait_for_cfg(&Cfg);
	
#endif
	wdrv_wpaSetMicFailureReportFromStation(TRUE);

	micFailureReportInRx = TRUE;

	memset(secAttackTable, 0, sizeof(secAttackTable));

#ifdef __linux__
	wpaTask(&Cfg);
#endif
	
	return OK;
}

/****************************************************************************
 *                      init8021xPortParams()
 ****************************************************************************
 * DESCRIPTION:	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	non
 ****************************************************************************/
static void	init8021xPortParams(auth1x_port_params_t *params)
{
	int fd,i;
	struct ifreq mac;	
	memset(params, 0, sizeof(auth1x_port_params_t));

    params->dev_id = (void *)&local_dev_id;
	fd = socket(AF_INET,SOCK_DGRAM,0);
	
	if(fd < 0)
	{
		printf("init8021xPortParams:error create mac ioctl fd\n");
		return; 
	}


	memset(params->mac, 0, ETH_ALEN);
	mac.ifr_hwaddr.sa_family = ARPHRD_ETHER;
	memcpy(mac.ifr_name, "wlan0", IFNAMSIZ);
	ioctl(fd, SIOCGIFHWADDR,&mac);
	close(fd);
	memcpy(params->mac,
		   mac.ifr_hwaddr.sa_data,
		   ETH_ALEN);


	memcpy(portParams.mac,params->mac,ETH_ALEN);
#if 0
	params->mac[0] = 0x0;params->mac[1] = 0x50; params->mac[2] = 0xF1;
	params->mac[3] = 0x12; params->mac[4] = 0x68;params->mac[5] = 0x86; 

/* @@*/
#else
	printf("USER MAC:");
	for(i = 0;i<6;i++)
		printf("%02hhx ",mac.ifr_hwaddr.sa_data[i]);
	printf("\n");
	fflush(NULL);
#endif

	params->is_wireless = 1;
	params->mtu = 1420/* device MTU */;
    
    params->glob_ext_vars.initialize = 1;
    params->glob_ext_vars.port_control = ACPC_AUTO;
    params->glob_ext_vars.port_enabled = 1;
    params->glob_ext_vars.admin_controlled_directions = ACD_BOTH;
    params->glob_ext_vars.bridge_detected = 0;
    /*  params are taken from default params as appear in 802.1X-2001 section
     *  8.5. */
    params->params.quiet_period = 60;
    params->params.re_auth_max = 2;
    params->params.tx_period = 30;
    params->params.key_tx_enabled = 1;
    params->params.re_auth_period = 3600;
    params->params.re_auth_enabled = 0;
    params->params.supp_timeout = 30;
    params->params.server_timeout = 30;
    params->params.max_req = 2;


	/* Now init the AP device CB functions */
	wdrv_wpaApPortInitCallbacks(&portCBs);

	/* Now fill the rest of the AP port CB functions */
	portCBs.open = open_by_x_port;
	portCBs.close = close_by_x_port;
}

/****************************************************************************
 *                      initWpaPortParams()
 ****************************************************************************
 * DESCRIPTION:	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	non
 ****************************************************************************/
static void	initWpaPortParams(wpa_port_params_t *pPortParams)
{



	pPortParams->dev_id = &local_dev_id;

	/* Now init the AP device CB functions */

	wdrv_wpaApDevInitCallbacks(&deviceCBs);
	pPortParams->ap_dev_ops = &deviceCBs;
	
	/* Now fill the rest of the AP device CB functions */
	pPortParams->ap_dev_ops->open = open_by_wpa_port;
	pPortParams->ap_dev_ops->close = close_by_wpa_port;
}


/******************************************************************************
*                      initWpaDb()											  *
*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static STATUS initWpaDb(tiap_wpa_auth_t *pCfg,wpa_info_elem_t **ap_info_elem,int *p_info_elem_len)
{
	int i;
	UINT8	pskHex[WPA_PSK_KEY_LEN];


	/* WPA parameters */
	portParams.allowed_sta_types = pCfg->type.sec.type;
	
	portParams.psk_param = pCfg->type.psk.pass_phrase_mode;

	if (pCfg->type.psk.pass_phrase_mode == WPA_PSK_PARAM_HEX)
		memcpy(portParams.psk.hex.data, pskHex, WPA_PSK_KEY_LEN);
	else if (pCfg->type.psk.pass_phrase_mode == WPA_PSK_PARAM_ASCII)
	{
		memcpy(pskPwd,pCfg->type.psk.psk_passwd,pCfg->type.psk.psk_passwd_len);
		portParams.psk.ascii = pskPwd;
	}

	portParams.rekeying_wep_cipher = pCfg->type.sec.default_cipher;
	
	portParams.gtk_update_interval = pCfg->type.sec.interval;// * 1000000; /*3600 * 1000;*/

/* @@*/
/*	printf("WPA MUDULE,interval: %d\n",pCfg->type.sec.interval);
	printf("WPA MUDULE, cipher %d\n",pCfg->type.sec.default_cipher);
	printf("WPA MUDULE, phrase_mode %d\n",pCfg->type.psk.pass_phrase_mode);
	printf("portParams.psk.ascii %s\n",portParams.psk.ascii);*/
	

	/* WEP Keys parameters */
	for (i = 0; i < WPA_NUM_DEFAULT_KEYS; i++)
	{
		switch (pCfg->type.sec.keys[i].len)
		{
		case 5:
			portParams.wep_keys[i].cipher = WPA_CIPHER_WEP_40;
			memcpy(portParams.wep_keys[i].data, pCfg->type.sec.keys[i].key, WPA_WEP_40_KEY_LEN);

			break;

		case 13:
			portParams.wep_keys[i].cipher = WPA_CIPHER_WEP_104;
			memcpy(portParams.wep_keys[i].data, pCfg->type.sec.keys[i].key, WPA_WEP_104_KEY_LEN);

			break;

		case 29:
			portParams.wep_keys[i].cipher = WPA_CIPHER_WEP_232;
			memcpy(portParams.wep_keys[i].data, pCfg->type.sec.keys[i].key, WPA_WEP_232_KEY_LEN);
			break;

        default:
            

			break;
		}
	}

	portParams.wep_key_index = pCfg->type.sec.wep_def_key_id;

	/* RADIUS parameters */
	radiusParams.servers = &serverList;
	serverList.next = NULL;
	memset(serverList.secret, 0, MAX_RADIUS_SECRET_LEN);
	memcpy(serverList.secret, pCfg->type.radius.secret, pCfg->type.radius.secret_len); 
	serverList.secret[pCfg->type.radius.secret_len] = 0;
	serverList.secret_len = pCfg->type.radius.secret_len;
	serverList.port = pCfg->type.radius.port;
	serverList.ip.s_addr = pCfg->type.radius.serv_ip_addr;
	
/*	printf ("radius %s.\n",radiusParams.servers->secret);
	printf ("radius addr %d.\n",radiusParams.servers->ip.s_addr);
	printf ("radius port %d.\n",radiusParams.servers->port);*/
	/* We have to configure the information element to the HAL before we init the HAL. */
  	set_ap_info_elem(&portParams,ap_info_elem,p_info_elem_len);

#ifndef __linux__
	// Demiurg : This is so ugly !!! Why the heck driver would modify routing table !! ??
	inet_ntoa_b(defaultGateway, defaultGatewayStr);

	status = routeAdd("0.0.0.0", defaultGatewayStr);
	if (status != 0)
	  wdrv_utilsReport(RL_DB, PRI_INFORMATION, "WDRV_DB: ERROR!!!!!! routeAdd Failure .\n");

#endif

	return OK;
}

/******************************************************************************
*                      wpaTask()											  *
*******************************************************************************
DESCRIPTION:	Implementation of the driver management task. 
				Wait in an infinite loop for message. When a message arrives, process it.	   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
#ifdef VXWORKS	
#define LB_DEV_NAME "lo"
#define LB_UNIT_NUM 0
#endif

#ifdef TIWLAN_DEBUG

static void print_cfg_all(tiap_wpa_auth_t *init)
{

	return;

	init->type.psk.psk_passwd[init->type.psk.psk_passwd_len] = NULL;
	printf("PSK PASS:%s\n",init->type.psk.psk_passwd);

	fflush(NULL);
	usleep(1000);
}

#endif 


/******************************************************************************
*                      wpaTask()											  *
*******************************************************************************
DESCRIPTION:	Implementation of the driver management task. 
				Wait in an infinite loop for message. When a message arrives, process it.	   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static void wpaTask(tiap_wpa_auth_t *pCfg) 
{ 

	wpaMessage_T hello_message;
	wpa_info_elem_t *ap_info_elem = NULL;
    int info_elem_len = 0;
	int rc;
	int retryCount  = 0;

#ifdef TIWLAN_DEBUG
	print_cfg_all(pCfg);
#endif

	random_init();
	
	/* create wpa db */
	initWpaDb(pCfg,&ap_info_elem,&info_elem_len);

#if 0

	printf("\n USER info len = %d\n",info_elem_len);
	for(i =0;i<info_elem_len;i++)
	{
		printf("USER1:char %d = 0x%x\n",i,((char*)ap_info_elem)[i]);
	}
	usleep(1000000);
#endif

	/* create hello msg */
	hello_message.type = WPA_HELLO;
	memcpy(hello_message.content.hello.info,(UINT8 *)ap_info_elem,info_elem_len);
	hello_message.content.hello.info_len = info_elem_len;
	
	wpaSocket = ipc_bind_listen_port(0);

	/* send hello to drv*/
	while( retryCount++ < NUMBER_OF_SENDING_HELLO_MSG)
	{
		rc = ipc_connect_buf_write(wpaSocket, &hello_message, sizeof(wpaMessage_T));
	
		if (rc == -1)
		{
			printf("send hello:error %d in 'ipc_connect_buf_write'. trying again...\n", rc);
		}
		else
			break;
		if (wpaSocket < 0)
		{
			rg_error(LPANIC, "WDRV_WPA: WPA socket init failure, trying again... \n");
		}
		
		usleep(2000000);
	}
	
	if(retryCount == NUMBER_OF_SENDING_HELLO_MSG)
	{
		rg_error(LPANIC, "WDRV_WPA: CANT FIND DIVER,ABORT \n");
		return;
	}

	event_loop_init();

	event_fd_set(wpaSocket, EFD_READ, wpa_input, NULL);

	rg_error(LPANIC, "WDRV_WPA: WPA module initialized successfully ,waiting for driver hello..\n");

	event_loop();

}

/******************************************************************************
*                      get_essid()											  *
*******************************************************************************
DESCRIPTION:
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static int get_essid(char *essid)
{
	struct iwreq            wrq;
	int fd;
	
	fd = socket(AF_INET,SOCK_DGRAM,0);
	
	if(fd < 0) {
		perror("");
		return 1;
	}
	
	
	if (essid)
	{
		wrq.u.essid.pointer = (caddr_t) essid;
		wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
		strncpy(wrq.ifr_ifrn.ifrn_name, "wlan0", IFNAMSIZ);
		if( ioctl(fd, SIOCGIWESSID, &wrq) == -1 ) {
			perror("");
			return 1;
		}
		
		printf("%s\n", essid);
	}
	
	return OK;
}

/******************************************************************************
*                      start_wpa_port()											  *
*******************************************************************************
DESCRIPTION:
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/

static BOOL wpa_port_started = FALSE;

static void start_wpa_port(void)
{

	/* Init the 802.1X port params */
	init8021xPortParams(&auth_8021x_params);

	portParams.ssid = apSsid;

	if(get_essid(portParams.ssid) != OK)
	{
		printf("start_wpa_port:error getting system ssid\n");
		return;
	}
/*@@*/
/*	printf("WPA MODULE: ssid received %s.\n",portParams.ssid);*/
	
	portParams.ssid_len = strlen(portParams.ssid);

	/* Init 802.1X port callback */
	auth_8021x_callbacks.client_get = client_get;
	auth_8021x_callbacks.new_assoc = NULL;



	/* Now open the radius client */	
	radius_client = radius_client_open(&radiusParams);

	if (radius_client == NULL)
	{
		rg_error(LPANIC, "WDRV_WPA: Failed to open radius client\n");
		return;
	}

	/* Open 802.1X port */
	pDev_if_auth_8021x = auth1x_port_open(&auth_8021x_params, 
	    &auth_8021x_callbacks, &portCBs, NULL);
	if (pDev_if_auth_8021x == NULL)
	{
		rg_error(LPANIC, "WDRV_WPA: Failed to open X port\n");
		return;
	}
	/* Init the WPA port params */
	initWpaPortParams(&portParams);
	portParams.auth1x_port = pDev_if_auth_8021x;

	/* Now open the WPA port */
	pWpaPort = wpa_port_open(&portParams);
	if (pWpaPort == NULL)
	{
		rg_error(LPANIC, "WDRV_WPA: Failed to open WPA port\n");
		return;
	}


	rg_error(LPANIC, "WDRV_WPA: WPA module receive hello,open port successfully\n");

	wpa_port_started = TRUE;
}
/******************************************************************************
*                      wpa_input()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
#if 0
static void wpa_input(int sock, void *data)
{	
	wpaMessage_T	message;
	int rc;
	int count=0;
	struct timeval tv;

	rc = ipc_accept_buf_read(sock, &message, sizeof(wpaMessage_T));
	gettimeofday(&tv, 0);
	if(!rc)
		printf("\nwpa_input1: received msg %d at %d.%d\n",(int)message.type,(int)tv.tv_sec, (int)tv.tv_usec);

	while ((!rc) && (count < 10))
	{
		count++;

		if (! wpa_port_started && message.type != WPA_DRV_HELLO) {
			printf("%s : WPA is not ready yet!\n", __FUNCTION__);
			return;
		}

		switch (message.type)
		{
			case EAPOL_FRAME:
				if (eapolPacketHandler != NULL)
				{
	#if 0 

					printf("\User receive eapol len %d ",message.content.eapolFrame.eapolDataLen);
					for(i = 0;i<message.content.eapolFrame.eapolDataLen;i++)
					{
						printf("%02x ", message.content.eapolFrame.pEapol[i]);
						if(!(i & 0x7))
							printf("\n");
					}
					fflush(NULL);
					usleep(100);
	#endif
					eapolPacketHandler(pXPortContext,
									   message.content.eapolFrame.pEapol,
									   message.content.eapolFrame.eapolDataLen,
									   message.content.eapolFrame.encrypted);
				}
				break;

			case NOTIFY_STATION_CONNECT:
				
	/*			printf("i'm in NOTIFY_STATION_CONNECT\n");
				fflush(NULL);*/
				if (ap_dev_cbs.sta_assoc_done != NULL)
				{
	#if 0
					int i;
					printf("USER NOTIFY_STATION_CONNECT:\n");
					for(i=0;i<message.content.stationConnect.rsne.hdr.eleLen +2;i++)
					{
						printf("0x%x ",((char*)&message.content.stationConnect.rsne)[i]);
						if(!(i&7))
							printf("\n");
					}
					fflush(NULL);
	#endif
					/* If the eleLen is zero, than the RSN info element is NULL */
					if (message.content.stationConnect.rsne.hdr.eleLen > 0)
					{

						ap_dev_cbs.sta_assoc_done(pWpaPortContext,
												  message.content.stationConnect.mac,
												  (UINT8 *)&(message.content.stationConnect.rsne),
												  message.content.stationConnect.rsne.hdr.eleLen + 2);
					}
					else
						ap_dev_cbs.sta_assoc_done(pWpaPortContext,
												  message.content.stationConnect.mac,
												  NULL,
												  message.content.stationConnect.rsne.hdr.eleLen + 2);
				}
				break;

			case NOTIFY_STATION_DIS_CONNECT:				

				if (ap_dev_cbs.sta_disassoc != NULL)
					ap_dev_cbs.sta_disassoc(pWpaPortContext, message.content.stationDisConnect.mac);
				break;

			case FIRST_DATA_FROM_STATION:				

				if (ap_dev_cbs.first_data_packet_received != NULL)
					ap_dev_cbs.first_data_packet_received(pWpaPortContext, message.content.firstDataFromStation.isEapol, message.content.firstDataFromStation.mac);
				break;

			case MIC_FAILURE:				

				if (ap_dev_cbs.unicast_mic_failure != NULL)
				{
					if (micFailureReportInRx)
						ap_dev_cbs.unicast_mic_failure(pWpaPortContext, message.content.micFailure.mac);
				}
				break;
			case CAN_STA_ASS:
	/*			printf("receive can station ASS\n");*/
				if(canStationAssc(message.content.canStaAss.info_elem,
								  message.content.canStaAss.info_elem_len) == OK)
					send_ass_res(message.content.canStaAss.index,1);
				else
					send_ass_res(message.content.canStaAss.index,0);
				break;
			case WPA_DRV_HELLO:
				start_wpa_port();
				break;

			case SET_STA_ATTACK:
				wdrv_wpaSetStationSecAttack(message.content.StaAttack.index,message.content.StaAttack.flag);
				break;
			default:
				rg_error(LPANIC, "WDRV_WPA: receive unknown msg type %d\n",message.type);
				break;
		} /* switch */

		rc = ipc_accept_buf_read(sock, &message, sizeof(wpaMessage_T));
		gettimeofday(&tv, 0);
		if(!rc)
			printf("\nwpa_input2: received msg %d at %d.%d\n",(int)message.type,(int)tv.tv_sec, (int)tv.tv_usec);

	}

	/*printf("\n\nbefore recvfrom1: %d.%d, event_loop_stamp=%d\n\n",(int)tv1.tv_sec, (int)tv1.tv_usec,event_loop_stamp);
	printf("after recvfrom1: %d.%d, rc=%d\n",(int)tv2.tv_sec, (int)tv2.tv_usec,rc);
	printf("before recvfrom2: %d.%d\n",(int)tv3.tv_sec, (int)tv3.tv_usec);*/
	/*printf("\n\nafter recvfrom2: %d.%d, rc=%d, event_loop_stamp=%d\n\n",(int)tv4.tv_sec, (int)tv4.tv_usec,rc,event_loop_stamp);*/
}
#else
static void wpa_input(int sock, void *data)
{	
	wpaMessage_T	message;
	int rc;

	/* printf("USER:wpa_input:recieve event %d \n",message.type);fflush(NULL);*/

	rc = ipc_accept_buf_read(sock, &message, sizeof(wpaMessage_T));

	if (rc)
	{
	    rg_error(LPANIC, "Got %d bytes. expect %d", rc,
		sizeof(wpaMessage_T));
	    return;
	}

	if (! wpa_port_started && message.type != WPA_DRV_HELLO) {
		printf("%s : WPA is not ready yet!\n", __FUNCTION__);
		return;
	}

	switch (message.type)
	{
		case EAPOL_FRAME:
			if (eapolPacketHandler != NULL)
			{
#if 0 

				printf("\User receive eapol len %d ",message.content.eapolFrame.eapolDataLen);
				for(i = 0;i<message.content.eapolFrame.eapolDataLen;i++)
				{
					printf("%02x ", message.content.eapolFrame.pEapol[i]);
					if(!(i & 0x7))
						printf("\n");
				}
				fflush(NULL);
				usleep(100);
#endif
				eapolPacketHandler(pXPortContext,
								   message.content.eapolFrame.pEapol,
								   message.content.eapolFrame.eapolDataLen,
								   message.content.eapolFrame.encrypted);
			}
			break;

		case NOTIFY_STATION_CONNECT:
			
/*			printf("i'm in NOTIFY_STATION_CONNECT\n");
			fflush(NULL);*/
			if (ap_dev_cbs.sta_assoc_done != NULL)
			{
#if 0
				int i;
				printf("USER NOTIFY_STATION_CONNECT:\n");
				for(i=0;i<message.content.stationConnect.rsne.hdr.eleLen +2;i++)
				{
					printf("0x%x ",((char*)&message.content.stationConnect.rsne)[i]);
					if(!(i&7))
						printf("\n");
				}
				fflush(NULL);
#endif
				/* If the eleLen is zero, than the RSN info element is NULL */
				if (message.content.stationConnect.rsne.hdr.eleLen > 0)
				{
					
					ap_dev_cbs.sta_assoc_done(pWpaPortContext,
											  message.content.stationConnect.mac,
											  (UINT8 *)&(message.content.stationConnect.rsne),
											  message.content.stationConnect.rsne.hdr.eleLen + 2);
				}
				else
				{

					ap_dev_cbs.sta_assoc_done(pWpaPortContext,
											  message.content.stationConnect.mac,
											  NULL,
											  message.content.stationConnect.rsne.hdr.eleLen + 2);
				}
			}
			break;

		case NOTIFY_STATION_DIS_CONNECT:				

			if (ap_dev_cbs.sta_disassoc != NULL)
				ap_dev_cbs.sta_disassoc(pWpaPortContext, message.content.stationDisConnect.mac);
			break;

		case FIRST_DATA_FROM_STATION:				

			if (ap_dev_cbs.first_data_packet_received != NULL)
				ap_dev_cbs.first_data_packet_received(pWpaPortContext, message.content.firstDataFromStation.isEapol, message.content.firstDataFromStation.mac);
			break;

		case MIC_FAILURE:				

			if (ap_dev_cbs.unicast_mic_failure != NULL)
			{
				if (micFailureReportInRx)
					ap_dev_cbs.unicast_mic_failure(pWpaPortContext, message.content.micFailure.mac);
			}
			break;
		case CAN_STA_ASS:
/*			printf("receive can station ASS\n");*/
			if(canStationAssc(message.content.canStaAss.info_elem,
							  message.content.canStaAss.info_elem_len) == OK)
				send_ass_res(message.content.canStaAss.index,1);
			else
				send_ass_res(message.content.canStaAss.index,0);
			break;
		case WPA_DRV_HELLO:
			start_wpa_port();
			break;

		case SET_STA_ATTACK:
			wdrv_wpaSetStationSecAttack(message.content.StaAttack.index,message.content.StaAttack.flag);
			break;
		default:
			rg_error(LPANIC, "WDRV_WPA: receive unknown msg type %d\n",message.type);
			break;
	} /* switch */

}
#endif

/******************************************************************************
*                      wdrv_wpaGetListenSock()											  *
*******************************************************************************
DESCRIPTION:send the specified EAPoL packet through the device. Packet should
 contain ethernet header. Returns number of bytes sent, -1 on error. 
		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static int send_ass_res(UINT16  index,int flag)
{
	wpaMessage_T	msg;
	int rc;

	msg.type = CAN_STA_ASS_RES;
	msg.content.canStaAssRes.index = index;
	msg.content.canStaAssRes.flag  = flag;

	rc = ipc_connect_buf_write(wpaSocket,(void *)&msg, sizeof(wpaMessage_T));
	if (rc == -1)
	{
		printf("send_eapol:error in 'ipc_connect_buf_write'. Error Code = %d\n", rc);
		return NOK;
	}

	return OK;
}

/******************************************************************************
*                      wdrv_wpaGetListenSock()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
int    wdrv_wpaGetListenSock(void)
{
/*	printf("\nreturn wpa sock = %d\n",wpaSocket); fflush(stdout);*/
	return wpaSocket;
}

/******************************************************************************
*                      rg_error()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
int rg_error(rg_error_level_t severity, const char *format, ...)
{
#if 0 
	int rc;
	va_list arg;
	char buffer[REPORT_MSG_LENGTH];

	va_start(arg, format);
	rc = vsprintf(buffer,format,arg);
	va_end(arg);
	if(rc < 0) 
		return -1;

	if(rc > REPORT_MSG_LENGTH) 
	{
		printf("WDRV_WPA: FATAL ERROR, print string is bigger than the maximum allowed!!!\n");
		return - 1;			
	}

#ifdef VXWORKS
		if (wdrv_utilsReportCB != NULL)
			wdrv_utilsReportCB(buffer); 
#else
/*		printf("%s\n",buffer);*/
#endif

#endif /* if 0 */

/*	}*/
	
	return 0;
}

/******************************************************************************
*                      rg_perror()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
int rg_perror(rg_error_level_t severity, const char *format, ...)
{
    va_list ap;
    int ret;
    
    va_start(ap, format);
    ret = rg_error(severity, format, ap);
    va_end(ap);

    return ret;
}

#if 0 
/* NOBUDY USE IT !!!*/
void wdrv_wpaPrintStationRsnIe(void)
{
	wdrv_dbPrintStaRsnIeTable();
}
#endif

/******************************************************************************
*                      open_by_wpa_port()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static ap_dev_t *open_by_wpa_port(void *dev_id,     /* object context */
					  ap_dev_cbs_t *cb, /* callbacks to use */
					  void *context)	/* context to be passed when calling callbacks */
{
	rg_error(LPANIC, "WDRV_WPA: WPA Is Openning AP DEV port\n");

	pWpaPortContext = context;
	
	/* Now init the WPA package callbacks tp be called */
	memset(&ap_dev_cbs, 0, sizeof(ap_dev_cbs_t));
	memcpy(&ap_dev_cbs, cb, sizeof(ap_dev_cbs_t));

	if (ap_dev_cbs.sta_assoc_request == NULL)
		rg_error(LPANIC, "WDRV_WPA: sta_assoc_request callback is NULL\n");
	if (ap_dev_cbs.sta_assoc_done == NULL)
		rg_error(LPANIC, "WDRV_WPA: sta_assoc_done callback is NULL\n");
	if (ap_dev_cbs.sta_disassoc == NULL)
		rg_error(LPANIC, "WDRV_WPA: sta_disassoc callback is NULL\n");
	if (ap_dev_cbs.unicast_mic_failure == NULL)
		rg_error(LPANIC, "WDRV_WPA: unicast_mic_failure callback is NULL\n");
	if (ap_dev_cbs.first_data_packet_received == NULL)
		rg_error(LPANIC, "WDRV_WPA: first_data_packet_received callback is NULL\n");

	
	return &pApDev;
}

/******************************************************************************
*                      close_by_wpa_port()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static void close_by_wpa_port(ap_dev_t *t)
{
}

/******************************************************************************
*                      canStationAssc()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
STATUS canStationAssc(u8 *info_elem, int info_elem_len)
{
	if (ap_dev_cbs.sta_assoc_request != NULL)
	{
		if (info_elem_len) 
			return ap_dev_cbs.sta_assoc_request(pWpaPortContext, info_elem, info_elem_len);
		else
			return ap_dev_cbs.sta_assoc_request(pWpaPortContext, NULL, info_elem_len);
		
	}
		
	else
		return OK;
}

/******************************************************************************
*                      open_by_x_port()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static auth1x_pc_t *open_by_x_port(void *dev_id,
				 auth1x_pkt_handler_t auth1x_pkt_handler, 
				 void *context)
{
	pXPortContext = context;

	/* Now init the EAPOL recv callback */
	eapolPacketHandler = auth1x_pkt_handler;
	if (eapolPacketHandler == NULL)
		rg_error(LPANIC, "WDRV_WPA: eapolPacketHandler callback is NULL\n");
	
	return pApPort;
}

/******************************************************************************
*                      close_by_x_port()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static void close_by_x_port(auth1x_pc_t *t)
{
}


/******************************************************************************
*                      wdrv_wpaGetListenSock()											  *
*******************************************************************************
DESCRIPTION:
 * Look up the client specified by id and id_len.
 * If not found - return EMI_NONE, otherwise return EAP method id and set
 * *eap_method_params to its client parameters.
 *	   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static eap_method_id_t client_get(u8 *id, int id_len, void **eap_method_params)
{
    clientParams.identity = id;
    clientParams.identity_len = id_len;
    clientParams.radius_client = radius_client; // context returned by radius_client_open

    *eap_method_params = &clientParams;
	
    return EMI_RADIUS;
}


/******************************************************************************
*                      wdrv_wpaGetListenSock()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
STATUS wdrv_wpaReConf(wpa_port_params_t *pWpaDbParams)
{
	memcpy(&portParams, pWpaDbParams, sizeof(wpa_port_params_t));

	/* We overrided this value, than set them again. */
	portParams.dev_id = &local_dev_id;
	portParams.auth1x_port = pDev_if_auth_8021x;
	portParams.ap_dev_ops = &deviceCBs;

	wpa_port_reconf(pWpaPortContext, &portParams);
	
	return OK;
}

/******************************************************************************
*                      wdrv_wpaGetListenSock()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
static wpa_cipher_t select_multicast_cipher(wpa_port_params_t *params)
{
    wpa_cipher_t multicast_cipher;
    
    /* If WEP-legacy stations are allowed, we must use the cipher of the 
     * preconfigured key as our multicast cipher
     */
    if (params->allowed_sta_types & WPA_STA_TYPE_WEP_LEGACY)
		multicast_cipher = params->wep_keys[params->wep_key_index].cipher;
  
	/* If there are WEP-802.1X stations, we must use the WEP cipher configured
     * by the user for multicast */ 
    else if ((params->allowed_sta_types & WPA_STA_TYPE_WEP_8021X))
		multicast_cipher = params->rekeying_wep_cipher;

    /* Not configured for backward compatibility, we can use TKIP */
    else
		multicast_cipher = WPA_CIPHER_TKIP;

    return multicast_cipher;
}

/******************************************************************************
*                      wdrv_wpaGetListenSock()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void set_ap_info_elem(wpa_port_params_t *params,wpa_info_elem_t **ap_info_elem,int *p_info_elem_len )
{
/*	int i;*/
    /* If WPA stations are supported, we're going to need an information element
     */
    if (params->allowed_sta_types & WPA_STA_TYPE_WPA)
    {
		/* Build information element */
		*ap_info_elem = wpa_info_elem_open(select_multicast_cipher(params), 
			WPA_CIPHER_TKIP, params->psk_param == WPA_PSK_PARAM_NONE ?
			WPA_AUTH_8021X : WPA_AUTH_PSK);

		if (!*ap_info_elem)
			return;

		*p_info_elem_len = wpa_info_elem_get_len(*ap_info_elem);
		#if 0
		for(i =0;i<*p_info_elem_len;i++)
		{
			printf("USER0:char %d = 0x%x\n",i,((char*)*ap_info_elem)[i]);
		}
		usleep(1000000);
		#endif
	}


#ifdef VXWORKS 	
	wdrv_dbSetRsne(info_elem_len, (UINT8 *)ap_info_elem);	
#endif

    if (*ap_info_elem)
		wpa_info_elem_close(*ap_info_elem);
}

/******************************************************************************
*                      wdrv_wpaGetListenSock()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void wdrv_wpaSetMicFailureReportInRx(BOOL value)
{
	micFailureReportInRx = value;
}

/******************************************************************************
*                      wdrv_wpaGetListenSock()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
BOOL wdrv_wpaGetMicFailureReportInRx(void)
{
	return micFailureReportInRx;
}

/******************************************************************************
*                      wdrv_wpaGetListenSock()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
void wdrv_wpaSetStationSecAttack(UINT16 index, BOOL value)	
{
	secAttackTable[index] = value;
}

/******************************************************************************
*                      wdrv_wpaGetListenSock()											  *
*******************************************************************************
DESCRIPTION:		   
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:		None.                                                    
******************************************************************************/
BOOL wdrv_wpaGetStationSecAttack(UINT16 index)	
{
	return secAttackTable[index];
}

