/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */ 
/*	 MODULE:	4xMngr.c	      										   */
/*   PURPOSE:	4x Management functions.	   			 				   */
/*																		   */ 
/***************************************************************************/
#include "pform_types.h"
#include "802_11Defs.h"
#include "tx.h"
#include "whal.h"
#include "pal.h"
#include "ackEmulUtil.h"
#include "4xMngr.h"
#include "drvDbMain.h"

UINT8 status4x;

void wdrv_4xInit(void)
{
	/* enable TX_DATA interrupt */
	whal_apiEnableTxDataIntr();
	/* set TX_DATA user callback */
   	whal_apiSetTxDataUserCB( (TxDataUser_Callback_T)wdrv_txCheckWaitPcktsFromIntr ); 
}

void wdrv_4xSetStatus(void)
{
	BOOL enable;

	wdrv_dbGet4xEnable(&enable);

	/* 4X FEATURE */
	if (enable == TRUE)
	{
		wdrv_utilsReport(RL_4X, PRI_INFORMATION, "WDRV_4X: 4x Enabled");


		/* 4X Concatenation */
		wdrv_dbGetConcatEnable(&enable);

		if (enable == TRUE)
		{
			wdrv_4xMngr_EnableConcatenation();
			wdrv_utilsReport(RL_4X, PRI_INFORMATION, "WDRV_4X: Concatenation Enabled\n");
		}
		else
		{
			wdrv_4xMngr_DisableConcatenation();
			wdrv_utilsReport(RL_4X, PRI_INFORMATION, "WDRV_4X: Concatenation Disabled\n");
		}
	
		/* 4X Ack Emulation */
		wdrv_dbGetAckEmulEnable(&enable);

		if (enable == TRUE)
		{
			wdrv_4xMngr_EnableAckEmulation();
			wdrv_utilsReport(RL_4X, PRI_INFORMATION, "WDRV_4X: Ack Emulation Enabled\n");
		}
		else
		{
			wdrv_4xMngr_DisableAckEmulation();
			wdrv_utilsReport(RL_4X, PRI_INFORMATION, "WDRV_4X: Ack Emulation Disabled\n");
		}  

		/* CWmin */
		wdrv_dbSetCwMinWinSize(15);
	}

	/* If 4X feature is disabled, we disabled automatically all 4x sub features. */
	else
	{
		wdrv_utilsReport(RL_4X, PRI_INFORMATION, "WDRV_4X: 4x Disabled\n");

		wdrv_4xMngr_DisableConcatenation();
		wdrv_utilsReport(RL_4X, PRI_INFORMATION, "WDRV_4X: Concatenation Disabled\n");

		wdrv_4xMngr_DisableAckEmulation();
		wdrv_utilsReport(RL_4X, PRI_INFORMATION, "WDRV_4X: Ack Emulation Disabled\n");
	}
	
}

/*
 * Concatenation
 * ------------
 */
void wdrv_4xMngr_EnableConcatenation(void)
{
    wdrv_txSetConcatStatus( ON );       
	whal_apiEnable4xTemplate();
}


void wdrv_4xMngr_DisableConcatenation(void)
{
	whal_apiDisable4xTemplate();
    wdrv_txSetConcatStatus( OFF ); 
}

/*
 * AckEmulation
 * ------------
 */
void wdrv_4xMngr_EnableAckEmulation(void)
{
#if 0
   wdrv_aeSetActive(1);
#endif
}

void wdrv_4xMngr_DisableAckEmulation(void)
{
#if 0
   wdrv_aeSetActive(0);
#endif
}

void wdrv_4xMngr_start4x( void )
{
   wdrv_4xMngr_EnableAckEmulation();
   wdrv_4xMngr_EnableConcatenation();
}

void wdrv_4xMngr_stop4x( void )
{
   wdrv_4xMngr_DisableConcatenation();
   wdrv_4xMngr_DisableAckEmulation();
}
