/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
#if 0
/***************************************************************************/
/*																		   */
/*		MODULE:	ackEmulDb.c											   */
/*   PURPOSE:	Ack emulation database 	              					   */
/*															    		   */
/***************************************************************************/
#include "pform_types.h"
#include "ackEmulDb.h"
#include "app_driver_api.h"
#include "802_11Defs.h"
#include "pal.h"

#define MAX_ACIVE_SESSION	2
#define MAX_STANDBY_SESSION 8
#define MAX_AE_STATIONS MAX_STATIONS

#define INDEX_FREE 0
#define INDEX_BUSY 1


typedef struct {
  int	lastUsedIndex;
  int	currentActiveState;
  int	currentStandbyState;
} sessionsTableManager_T;

typedef struct {
	int status;			/* INDEX_FREE or INDEX_BUSY */
	int monitorIndex ; /* Index in wdrv_aeWTargetTable */
}activeIndexTable_T;

typedef struct {
	int Xtag;			
	int DataReceivedFlage;
   int DataReceivedIndex;
}ackEmulationStationTable_T;

static ackEmulationWTargetDbTyple_T wdrv_aeWTargetTable[MAX_ACIVE_SESSION+MAX_STANDBY_SESSION];
static sessionsTableManager_T sessionsTableManager;

static ackEmulationWSourceDbTyple_T wdrv_aeWSourceTable[MAX_AE_STATIONS][MAX_ACIVE_SESSION];

/* This table store the Xtag, DataReceived status and DataReceived Index */ 
static ackEmulationStationTable_T ackEmulationStationTable[MAX_AE_STATIONS];

static void wdrv_aeWTargetDbPrintTuple(int index);

static activeIndexTable_T activeIndexTable[MAX_ACIVE_SESSION];
static void wdrv_aeFindFreeActiveIndex(UINT8 *freeIndex);

static void wdrv_aeWSourceDbInit(void);
static void wdrv_aeWSourceDbResetstation(int stationIndex);




void wdrv_aeDbInit(void)
{
   wdrv_aeWTargetDbInit();
   wdrv_aeWSourceDbInit();     
}	

void wdrv_aeWTargetDbInit(void)
{
	int index;
	for(index=0 ; index< MAX_ACIVE_SESSION+MAX_STANDBY_SESSION; index++)
		wdrv_aeWTargetDbResetTuple(index); 
	for(index=0 ; index< MAX_ACIVE_SESSION; index++)
	{
		activeIndexTable[index].status = INDEX_FREE;
		activeIndexTable[index].monitorIndex = 0xff;
	}
	sessionsTableManager.currentActiveState =0;
	sessionsTableManager.currentStandbyState =0;
}

int wdrv_aeWTargetDbAddSession(UINT8 *pktBuf, UINT16 stationIndex)
{
   int index;
   int freeIndex = 0xff;
   UINT32 currentTimeTick;
   UINT32 oldestTimeTick;
   UINT8  oldestTimeIndex;
   
   /* find free index */
   for(index=0 ; index < MAX_ACIVE_SESSION+MAX_STANDBY_SESSION; index++)
   {
      if(wdrv_aeWTargetTable[index].sPorts	== 0)
      {
         freeIndex = index;
         break;
      }
   }
   if (freeIndex == 0xff)
   {
      /* find old session to delete */
      currentTimeTick = oldestTimeTick = PALSRV_getTick();
      oldestTimeIndex = 0xff;
      for(index=0 ; index< MAX_ACIVE_SESSION+MAX_STANDBY_SESSION; index++)
      {
         if(wdrv_aeWTargetTable[index].timeStamp	< oldestTimeTick)
         {
            oldestTimeTick = wdrv_aeWTargetTable[index].timeStamp;
            oldestTimeIndex = index;
         }
      }
      
      if(PALSRV_msec2ticks(WTARGET_TERMINATE_TIME_OUT) < (currentTimeTick - oldestTimeTick))
      {
         wdrv_aeWTargetDbResetTuple(oldestTimeIndex);
         freeIndex = oldestTimeIndex;
      }      
   }
   
   if (freeIndex != 0xff)
   {
      /* Add new session */

      int ipHeaderLen = ((*(unsigned char*)pktBuf  & 0x0f) * 4);
      wdrv_aeWTargetTable[freeIndex].sPorts = *(UINT16*)(pktBuf +ipHeaderLen);                          
      wdrv_aeWTargetTable[freeIndex].dPorts = *(UINT16*)(pktBuf +ipHeaderLen+DEST_PORT_FIELD);                          
      wdrv_aeWTargetTable[freeIndex].srcIPAddr = *(unsigned long*)(pktBuf+IP_SRC_ADDRESS_FIELD);                      
      wdrv_aeWTargetTable[freeIndex].destIPAddr = *(unsigned long*)(pktBuf+IP_DEST_ADDRESS_FIELD);                     
      wdrv_aeWTargetTable[freeIndex].segmentSize				=0;                    
      wdrv_aeWTargetTable[freeIndex].sequenceNumber =*(unsigned long*)(pktBuf+ipHeaderLen+TCP_SEQUENCE_NUMBER_FIELD);	                    
      wdrv_aeWTargetTable[freeIndex].ackNumber					=0;	                    
      wdrv_aeWTargetTable[freeIndex].timeStamp					=PALSRV_getTick();                      
      wdrv_aeWTargetTable[freeIndex].ackCounter				   =0;	                    
      wdrv_aeWTargetTable[freeIndex].ackTemplate.ipHeaderLen			=0;			
      wdrv_aeWTargetTable[freeIndex].ackTemplate.tcpHeaderLen			=0;			
      wdrv_aeWTargetTable[freeIndex].monitorState				= AE_STANDBY;                     
      wdrv_aeWTargetTable[freeIndex].equalSegmentSizeCounter	=0;          
      wdrv_aeWTargetTable[freeIndex].yTagFlag					=FALSE;  
		wdrv_aeWTargetTable[freeIndex].activeIndex				= 0xff;
      wdrv_aeWTargetTable[freeIndex].sourceStationIndex     = stationIndex;
		wdrv_aeWTargetTable[freeIndex].dataReceivedFlage      = 0;
      wdrv_aeWTargetTable[freeIndex].lastWackInfo           = 0xff;     

      sessionsTableManager.currentStandbyState ++;
   }
   
   return OK;
}



int wdrv_aeWTargetDbFindDataSession(UINT8 *pktBuf,UINT32 *sessionIndex, UINT32 *monitorState)
{
   int index;
   int ipHeaderLen;

   ipHeaderLen = ((*(unsigned char*)pktBuf  & 0x0f) * 4);
   *monitorState = AE_INACTIVE;

	for(index=0 ; index< MAX_ACIVE_SESSION+MAX_STANDBY_SESSION; index++)
	{
      if (wdrv_aeWTargetTable[index].sPorts		== *(UINT16*)(pktBuf +ipHeaderLen) && 
          wdrv_aeWTargetTable[index].dPorts		== *(UINT16*)(pktBuf +ipHeaderLen + DEST_PORT_FIELD) && 
	        wdrv_aeWTargetTable[index].srcIPAddr	== *(unsigned long*)(pktBuf+IP_SRC_ADDRESS_FIELD) &&
	        wdrv_aeWTargetTable[index].destIPAddr	== *(unsigned long*)(pktBuf+IP_DEST_ADDRESS_FIELD))
		{
         *sessionIndex = index;
         *monitorState = wdrv_aeWTargetTable[index].monitorState;

    /*     printf("\nindex = %d  monitorState = %d\n", *sessionIndex, *monitorState);*/

         return OK;
      }
   }
   return NOK;
   
}


int wdrv_aeWTargetDbFindAckSession(UINT8 *pktBuf,UINT32 *sessionIndex, UINT32 *monitorState)
{
   int index;
   int ipHeaderLen;

   ipHeaderLen = ((*(unsigned char*)pktBuf  & 0x0f) * 4);
   *monitorState = AE_INACTIVE;

	for(index=0 ; index< MAX_ACIVE_SESSION+MAX_STANDBY_SESSION; index++)
	{
      if (wdrv_aeWTargetTable[index].sPorts		== *(UINT16*)(pktBuf +ipHeaderLen + DEST_PORT_FIELD) && 
          wdrv_aeWTargetTable[index].dPorts		== *(UINT16*)(pktBuf +ipHeaderLen ) && 
	        wdrv_aeWTargetTable[index].srcIPAddr	== *(unsigned long*)(pktBuf+IP_DEST_ADDRESS_FIELD) &&
	        wdrv_aeWTargetTable[index].destIPAddr	== *(unsigned long*)(pktBuf+IP_SRC_ADDRESS_FIELD))
		{
			*sessionIndex = index;
         *monitorState = wdrv_aeWTargetTable[index].monitorState;
    /*     printf("\nindex = %d  monitorState = %d\n", *sessionIndex, *monitorState);*/

         return OK;
      }
   }
   return NOK;
   
}


void wdrv_aeWTargetDbResetTuple(int index)
{
   UINT8 freeActiveIndex = wdrv_aeWTargetTable[index].activeIndex;
	if (wdrv_aeWTargetTable[index].monitorState == AE_ACTIVE)
	{
		sessionsTableManager.currentActiveState --;
	}
	else
	{
		sessionsTableManager.currentStandbyState --;
	}
	
   wdrv_aeWTargetTable[index].sPorts						=0;                          
   wdrv_aeWTargetTable[index].dPorts						=0;                          
   wdrv_aeWTargetTable[index].srcIPAddr					=0;                      
   wdrv_aeWTargetTable[index].destIPAddr				=0;                     
   wdrv_aeWTargetTable[index].segmentSize				=0;                    
   wdrv_aeWTargetTable[index].sequenceNumber			=0;	                    
   wdrv_aeWTargetTable[index].ackNumber					=0;	                    
   wdrv_aeWTargetTable[index].timeStamp					=0;                      
   wdrv_aeWTargetTable[index].ackCounter				=0;	                    
   wdrv_aeWTargetTable[index].ackTemplate.ipHeaderLen			=0;			
   wdrv_aeWTargetTable[index].ackTemplate.tcpHeaderLen			=0;			
   wdrv_aeWTargetTable[index].monitorState				=AE_INACTIVE;                     
   wdrv_aeWTargetTable[index].equalSegmentSizeCounter	=0;          
   wdrv_aeWTargetTable[index].yTagFlag					=FALSE; 
   wdrv_aeWTargetTable[index].activeIndex		= 0xff;
   wdrv_aeWTargetTable[index].sourceStationIndex     = 0xff;
   wdrv_aeWTargetTable[index].dataReceivedFlage      = 0;
   wdrv_aeWTargetTable[index].lastWackInfo           = 0xff;    
   if(freeActiveIndex != 0xff)

   {
      activeIndexTable[freeActiveIndex].status = INDEX_FREE;
      activeIndexTable[freeActiveIndex].monitorIndex = 0xff;
      
   }
}



void wdrv_aeWTargetDbPrint(void)
{
	int index;
    char* sipByte;
	char* 	dipByte;

   printf("\n current Active State = %d\n", sessionsTableManager.currentActiveState);
   printf(" current Standby State = %d\n", sessionsTableManager.currentStandbyState);
   printf(" --------------------------\n");
   
   printf("|  |   SOURCE IP   |   DEST  IP    |SPORT|DPORT|St|Seg Size|AI|\n");
   printf("|------------------------------------------------------------------------| \n");
   for(index=0 ; index< MAX_ACIVE_SESSION+MAX_STANDBY_SESSION; index++)
   {
	   sipByte = (char*) &wdrv_aeWTargetTable[index].srcIPAddr;
	   dipByte = (char*) &wdrv_aeWTargetTable[index].destIPAddr;
	   
	   printf("|%02d|%03d.%03d.%03d.%03d|%03d.%03d.%03d.%03d|%05d|%05d|%02x|%09lu|%02x|%x|\n",
		   index,
		   sipByte[0],sipByte[1],sipByte[2],sipByte[3], 
		   dipByte[0],dipByte[1],dipByte[2],dipByte[3],
		   wdrv_aeWTargetTable[index].sPorts,
		   wdrv_aeWTargetTable[index].dPorts,
		   wdrv_aeWTargetTable[index].monitorState,
		   wdrv_aeWTargetTable[index].segmentSize,
         wdrv_aeWTargetTable[index].activeIndex,
         wdrv_aeWTargetTable[index].ackCounter);
		   
   }
}




void wdrv_aeWTargetDbSetSessionSequenceNumber(UINT8 index, UINT32 sequenceNumber)
{
   wdrv_aeWTargetTable[index].sequenceNumber = sequenceNumber;
}


void wdrv_aeWTargetDbGetSessionSequenceNumber(UINT8 index, UINT32 *sequenceNumber)
{
      *sequenceNumber = wdrv_aeWTargetTable[index].sequenceNumber;

}


void wdrv_aeWTargetDbSetSessionSegmentSize(UINT8 index, UINT32 segmentSize)
{
   wdrv_aeWTargetTable[index].segmentSize = segmentSize;

}

void wdrv_aeWTargetDbGetSessionSegmentSize(UINT8 index, UINT32 *segmentSize)
{
      *segmentSize = wdrv_aeWTargetTable[index].segmentSize;

}

void wdrv_aeWTargetDbGetIncrSessionEqualSegmentSizeCounter(UINT8 index, UINT8 *equalSegmentSizeCounter)
{
	*equalSegmentSizeCounter = (wdrv_aeWTargetTable[index].equalSegmentSizeCounter ++);
}

int wdrv_aeWTargetDbSetActiveState(UINT8 index , UINT8 *activeIndex)
{
   UINT8 monitorState = AE_ACTIVE ;
   UINT8 freeIndex ,aIndex;
   wdrv_aeFindFreeActiveIndex(&freeIndex);
   
   if(freeIndex == 0xff)
   {
      /* find old active session to delete */
      UINT32 currentTimeTick;
      UINT32 oldestTimeTick;
      UINT8  oldestTimeIndex;
      currentTimeTick = oldestTimeTick = PALSRV_getTick();
      oldestTimeIndex = 0xff;
      for(aIndex=0 ; aIndex< MAX_ACIVE_SESSION; aIndex++)
      {
         if(wdrv_aeWTargetTable[activeIndexTable[aIndex].monitorIndex].timeStamp	< 
            oldestTimeTick)
         {
            oldestTimeTick = wdrv_aeWTargetTable[activeIndexTable[aIndex].monitorIndex].timeStamp;
            oldestTimeIndex = aIndex;
         }
      }
      
      if(PALSRV_msec2ticks(WTARGET_TERMINATE_TIME_OUT) < (currentTimeTick - oldestTimeTick))
      {
         wdrv_aeWTargetDbResetTuple(activeIndexTable[oldestTimeIndex].monitorIndex);
         freeIndex = oldestTimeIndex;
      }
      
   }
   if (freeIndex == 0xff)
   {
      *activeIndex = 0xff;
      return NOK;
   }
   activeIndexTable[freeIndex].status = INDEX_BUSY;
   activeIndexTable[freeIndex].monitorIndex = index;
   wdrv_aeWTargetTable[index].activeIndex = freeIndex;
   *activeIndex = freeIndex;

   sessionsTableManager.currentActiveState ++;
   sessionsTableManager.currentStandbyState --;
   
   wdrv_aeWTargetDbSetSessionMonitorState(index, monitorState);   
   return OK;
}

void wdrv_aeWTargetDbSetSessionMonitorState(UINT8 index, UINT8 monitorState)
{
      wdrv_aeWTargetTable[index].monitorState = monitorState;
}
void wdrv_aeWTargetDbSetSessionEqualSegmentSizeCounter(UINT8 index, UINT8 equalSegmentSizeCounter)
{
	wdrv_aeWTargetTable[index].equalSegmentSizeCounter = equalSegmentSizeCounter; 
}

void wdrv_aeWTargetDbGetSessionAckNumber(UINT8 index, UINT32 *ackNumber)
{
      *ackNumber = wdrv_aeWTargetTable[index].ackNumber;
}

void wdrv_aeWTargetDbSetSessionAckNumber(UINT8 index, UINT32 ackNumber)
{
      wdrv_aeWTargetTable[index].ackNumber = ackNumber;
}



void wdrv_aeWTargetDbGetSessionAckCounter(UINT8 index, UINT32 *ackCounter)
{
      *ackCounter = wdrv_aeWTargetTable[index].ackCounter;
}

void wdrv_aeWTargetDbSetSessionAckCounter(UINT8 index, UINT32 ackCounter)
{
      wdrv_aeWTargetTable[index].ackCounter = ackCounter;
}

void wdrv_aeWTargetDbGetSessionActiveIndex(UINT8 index, UINT8 *activeIndex)
{
      *activeIndex = wdrv_aeWTargetTable[index].activeIndex;

}


void wdrv_aeWTargetDbSetDataReceived(UINT8 index, UINT8 dataReceived)
{
         wdrv_aeWTargetTable[index].dataReceivedFlage = dataReceived;

}
void wdrv_aeWTargetDbGetDataReceived(UINT8 index, UINT8 *dataReceived)
{
         *dataReceived = wdrv_aeWTargetTable[index].dataReceivedFlage;
}
   
void wdrv_aeWTargetDbSetLastWackInfo(UINT8 index, UINT8 lastWackInfo)
{
         wdrv_aeWTargetTable[index].lastWackInfo = lastWackInfo;

}
void wdrv_aeWTargetDbGetLastWackInfo(UINT8 index, UINT8 *lastWackInfo)
{
            *lastWackInfo = wdrv_aeWTargetTable[index].lastWackInfo;

}


void wdrv_aeWTargetDbGetSessionTimeStamp(UINT8 index, UINT32 *timeStamp)
{
      *timeStamp = wdrv_aeWTargetTable[index].timeStamp;
}
void wdrv_aeWTargetDbSetSessionTimeStamp(UINT8 index, UINT32 timeStamp)
{
      wdrv_aeWTargetTable[index].timeStamp= timeStamp;
}


static void wdrv_aeFindFreeActiveIndex(UINT8 *freeIndex)
{
	UINT8 index;
	*freeIndex = 0xff;
	for(index=0 ; index< MAX_ACIVE_SESSION; index++)
	{
		if(activeIndexTable[index].status == INDEX_FREE)
		{
			*freeIndex = index;
			return;
		}
	}	
}


void wdrv_aeWTargetDbSaveAckTemplate(UINT8 index, UINT8 *pIpHeader)
{

   UINT8 ipHeaderLen;
   UINT8 tcpHeaderLen;
   
   ipHeaderLen = ((*(unsigned char*)pIpHeader  & 0x0f) * 4);
   tcpHeaderLen = ((((*(unsigned char*)(pIpHeader + ipHeaderLen+TCP_OFFSET_FIELD))& 0xf0)>>4) * 4);
   PAL_memCopy(wdrv_aeWTargetTable[index].ackTemplate.data, pIpHeader, ipHeaderLen+tcpHeaderLen);
   wdrv_aeWTargetTable[index].ackTemplate.ipHeaderLen = ipHeaderLen;
   wdrv_aeWTargetTable[index].ackTemplate.tcpHeaderLen = tcpHeaderLen;
}

void wdrv_aeWTargetDbUpdateAckTemplate(UINT8 index, UINT32 sequenceNumber)
{
   UINT8 *pSequenceNumber;
   /* Update Template Sequence Number */
   pSequenceNumber = wdrv_aeWTargetTable[index].ackTemplate.data + 
        wdrv_aeWTargetTable[index].ackTemplate.tcpHeaderLen +TCP_SEQUENCE_NUMBER_FIELD;
   *(unsigned long*)(pSequenceNumber) = sequenceNumber;
}



int pcmpt = 0;
int wdrv_aeWTargetDbCmpAckTemplate(UINT8 index, UINT8 *pIpHeader)
{
   int rc;
   UINT8 ipHeaderLen;
   UINT8 tcpHeaderLen;
   UINT8 *pTcpHeader;
   UINT8 *pIpHeaderTemplate;
   UINT8 *pTcpHeaderTemplate;
   UINT8 ipHeaderTemplateLen;
   
   ipHeaderLen = ((*(unsigned char*)pIpHeader  & 0x0f) * 4);
   pIpHeaderTemplate = wdrv_aeWTargetTable[index].ackTemplate.data;
   ipHeaderTemplateLen = wdrv_aeWTargetTable[index].ackTemplate.ipHeaderLen;
   
   /* Comprer IP field: Version ,Header Length, Precedence, TOS , Unused, Total Length */
   if((*(UINT32*)(pIpHeaderTemplate))!= (*(UINT32*)pIpHeader))
   {
      if(pcmpt)
         printf("\n-------> 1 \n");
      return NOK;
   }

   /* Comprer IP field:Fragmentation Flags, Fragment Offset, TTL */
   if((*(UINT32*)(pIpHeaderTemplate+IP_IDENTIFIER_FIELD+2))!= 
      (*(UINT32*)(pIpHeader+IP_IDENTIFIER_FIELD+2)))
   {
      if(pcmpt)
         printf("\n-------> 2 \n");
      return NOK;
   }
   
   pTcpHeader = pIpHeader + ipHeaderLen;
   pTcpHeaderTemplate = pIpHeaderTemplate + ipHeaderTemplateLen;
   
   
      /* Comprer TCP field: Offset, Reserved, Code, Window */
      if((*(UINT32*)(pTcpHeaderTemplate+TCP_ACK_NUMBER_FIELD+4))!= 
      (*(UINT32*)(pTcpHeader+TCP_ACK_NUMBER_FIELD+4)))
   {
         if(pcmpt)
            printf("\n-------> 3 \n");
      return NOK;
   }
      /* Comprer TCP field: Urgent */
   
   if((*(UINT16*)(pTcpHeaderTemplate+TCP_CHECKSUM_FIELD+2))!= 
      (*(UINT16*)(pTcpHeader+TCP_CHECKSUM_FIELD+2)))
   {
      if(pcmpt)
         printf("\n-------> 4 \n");
      return NOK;
   }
   
      
      /* Comprer TCP field: Sequence Number */
   if((*(UINT32*)(pTcpHeaderTemplate+TCP_SEQUENCE_NUMBER_FIELD))!= 
      (*(UINT32*)(pTcpHeader+TCP_SEQUENCE_NUMBER_FIELD)))
   {
      if(pcmpt)
         printf("\n-------> 5 \n");
      
      /* add Ytag */
      return NOK;
   }
   

   return OK;
   
}


void wdrv_aeDbGetXTagStatus(UINT8 sessionIndex, UINT8 *status)
{
      UINT16 stationIndex = wdrv_aeWTargetTable[sessionIndex].sourceStationIndex;
      if(stationIndex != 0xff)
         *status = ackEmulationStationTable[stationIndex].Xtag;
}

void wdrv_aeDbSetXTagStatus(UINT8 sessionIndex, UINT8 status)
{
      UINT16 stationIndex = wdrv_aeWTargetTable[sessionIndex].sourceStationIndex;
         ackEmulationStationTable[stationIndex].Xtag = status;
}

void wdrv_aeDbGetDataReceivedStatus(UINT8 sessionIndex, UINT8 *status, UINT8 *dataReceivedIndex)
{
      UINT16 stationIndex = wdrv_aeWTargetTable[sessionIndex].sourceStationIndex;
      if(stationIndex != 0xff)
      {
         *status = ackEmulationStationTable[stationIndex].DataReceivedFlage;
         *dataReceivedIndex = ackEmulationStationTable[stationIndex].DataReceivedIndex;
      }
}

void wdrv_aeDbSetDataReceivedStatus(UINT8 sessionIndex, UINT8 status)
{
      UINT16 stationIndex = wdrv_aeWTargetTable[sessionIndex].sourceStationIndex;
         ackEmulationStationTable[stationIndex].DataReceivedFlage = status;
         ackEmulationStationTable[stationIndex].DataReceivedIndex = sessionIndex;         
}







/******************************* WSource Data base *************************************/

static void wdrv_aeWSourceDbInit(void)
{
	int stationIndex;
	for (stationIndex =0;stationIndex < MAX_AE_STATIONS;stationIndex++)
		wdrv_aeWSourceDbResetstation(stationIndex);
}

static void wdrv_aeWSourceDbResetstation(int stationIndex)
{
	int activeIndex;
   int  i;
	for (activeIndex =0;activeIndex < MAX_ACIVE_SESSION;activeIndex++)
	{
		wdrv_aeWSourceTable[stationIndex][activeIndex].segmentSize=0;              
      wdrv_aeWSourceTable[stationIndex][activeIndex].ackNumber =0;	                        
      wdrv_aeWSourceTable[stationIndex][activeIndex].ackCounter =0;	                        
      wdrv_aeWSourceTable[stationIndex][activeIndex].timeStamp =0;	
      wdrv_aeWSourceTable[stationIndex][activeIndex].ackReorderProblem = REORDER_PROBLEM_OFF;
		wdrv_aeWSourceTable[stationIndex][activeIndex].ackTemplate.ipHeaderLen =0;	
		wdrv_aeWSourceTable[stationIndex][activeIndex].ackTemplate.tcpHeaderLen =0;	
	}			
}

void wdrv_aeWSourceDbResetSession(int stationIndex,int activeIndex)
{
   wdrv_aeWSourceTable[stationIndex][activeIndex].segmentSize=0;              
   wdrv_aeWSourceTable[stationIndex][activeIndex].ackNumber =0;	                        
   wdrv_aeWSourceTable[stationIndex][activeIndex].ackCounter =0;	                        
   wdrv_aeWSourceTable[stationIndex][activeIndex].timeStamp =0;
   wdrv_aeWSourceTable[stationIndex][activeIndex].ackReorderProblem = REORDER_PROBLEM_OFF;
   wdrv_aeWSourceTable[stationIndex][activeIndex].ackTemplate.ipHeaderLen =0;	
   wdrv_aeWSourceTable[stationIndex][activeIndex].ackTemplate.tcpHeaderLen =0;	
}

int pytag =0;
void wdrv_aeWSourceSaveAckTemplate(UINT8 stationIndex,UINT8 activeIndex,
                                   UINT8 *dataBuf,UINT16 dataLen,UINT16 segmentSize)
{
   
   UINT8 *pWlanSnapHeader;
   UINT8 *pIpHeader;
   UINT8 *pTcpHeader;
   UINT8 *pTemplateData;
   UINT8 ipHeaderLen;
   UINT8 tcpHeaderLen;
   UINT32 ackNumber;
   
    wdrv_aeWSourceDbSetSessionTimeStamp(stationIndex,activeIndex,PALSRV_getTick());   

   pWlanSnapHeader = dataBuf + WLAN_HDR_LEN;
   pIpHeader = pWlanSnapHeader + WLAN_SNAP_HDR_LEN;
   
   ipHeaderLen = ((*(unsigned char*)pIpHeader  & 0x0f) * 4);
   pTcpHeader = pIpHeader + ipHeaderLen;
   tcpHeaderLen = ((((*(unsigned char*)(pTcpHeader+TCP_OFFSET_FIELD))& 0xf0)>>4) * 4);
   pTemplateData = wdrv_aeWSourceTable[stationIndex][activeIndex].ackTemplate.data;
   PAL_memCopy(pTemplateData,dataBuf, dataLen);
   wdrv_aeWSourceTable[stationIndex][activeIndex].ackTemplate.ipHeaderLen = ipHeaderLen;
   wdrv_aeWSourceTable[stationIndex][activeIndex].ackTemplate.tcpHeaderLen = tcpHeaderLen;

   wdrv_aeWSourceTable[stationIndex][activeIndex].segmentSize = segmentSize;    
   
   ackNumber = *(unsigned long*)(pIpHeader+ipHeaderLen+TCP_ACK_NUMBER_FIELD);
   wdrv_aeWSourceTable[stationIndex][activeIndex].ackNumber = ackNumber;
   wdrv_aeWSourceTable[stationIndex][activeIndex].ackCounter =	ackNumber/(segmentSize*2);	
   wdrv_aeWSourceTable[stationIndex][activeIndex].ackReorderProblem = REORDER_PROBLEM_OFF;

   if (pytag)
   {
#ifdef VXWORKS
      d(pTemplateData,dataLen,1);
      printf("\nactiveIndex = %d\n",activeIndex);
#else
			puts("What do you want from me ?");
#endif
   }
      /* reset the Ip & TCP Checksum */
   *(UINT16*)(pTemplateData+WLAN_HDR_LEN+WLAN_SNAP_HDR_LEN+IP_CHECKSUM_FIELD) = 0x0000;
   *(UINT16*)(pTemplateData+WLAN_HDR_LEN+WLAN_SNAP_HDR_LEN+ipHeaderLen+TCP_CHECKSUM_FIELD) = 0x0000;
   
}


void wdrv_aeWSourceDbGetSessionAckCounter(UINT16 stationIndex, UINT8 activeIndex, UINT32 *ackCounter)
{
      *ackCounter = wdrv_aeWSourceTable[stationIndex][activeIndex].ackCounter;
}

void wdrv_aeWSourceDbSetSessionAckCounter(UINT16 stationIndex, UINT8 activeIndex, UINT32 ackCounter)
{
      wdrv_aeWSourceTable[stationIndex][activeIndex].ackCounter= ackCounter;
}

void wdrv_aeWSourceDbGetSessionAckNumber(UINT16 stationIndex, UINT8 activeIndex, UINT32 *ackNumber)
{
      *ackNumber = wdrv_aeWSourceTable[stationIndex][activeIndex].ackNumber;
}

void wdrv_aeWSourceDbSetSessionAckNumber(UINT16 stationIndex, UINT8 activeIndex, UINT32 ackNumber)
{
      wdrv_aeWSourceTable[stationIndex][activeIndex].ackNumber= ackNumber;
}

void wdrv_aeWSourceDbGetSessionSegmentSize(UINT16 stationIndex, UINT8 activeIndex, UINT32 *segmentSize)
{
      *segmentSize = wdrv_aeWSourceTable[stationIndex][activeIndex].segmentSize;
}

void wdrv_aeWSourceDbSetSessionSegmentSize(UINT16 stationIndex, UINT8 activeIndex, UINT32 segmentSize)
{
      wdrv_aeWSourceTable[stationIndex][activeIndex].segmentSize= segmentSize;
}


void wdrv_aeWSourceDbGetSessionTimeStamp(UINT16 stationIndex, UINT8 activeIndex, UINT32 *timeStamp)
{
      *timeStamp = wdrv_aeWSourceTable[stationIndex][activeIndex].timeStamp;
}
void wdrv_aeWSourceDbSetSessionTimeStamp(UINT16 stationIndex, UINT8 activeIndex, UINT32 timeStamp)
{
      wdrv_aeWSourceTable[stationIndex][activeIndex].timeStamp= timeStamp;
}

void wdrv_aeWSourceDbGetSessionAckReorderProblem(UINT16 stationIndex, UINT8 activeIndex, UINT32 *ackReorderProblem)
{
      *ackReorderProblem = wdrv_aeWSourceTable[stationIndex][activeIndex].ackReorderProblem;
}
void wdrv_aeWSourceDbSetSessionAckReorderProblem(UINT16 stationIndex, UINT8 activeIndex, UINT32 ackReorderProblem)
{
      wdrv_aeWSourceTable[stationIndex][activeIndex].ackReorderProblem= ackReorderProblem;
}

void wdrv_aeWSourceDbGetAckTemplate(UINT16 stationIndex, UINT8 activeIndex,UINT8 **pTeplate,
													 UINT8 *ipHeaderLen, UINT8 *tcpHeaderLen)
{																					
      *ipHeaderLen   = wdrv_aeWSourceTable[stationIndex][activeIndex].ackTemplate.ipHeaderLen;
      *tcpHeaderLen  = wdrv_aeWSourceTable[stationIndex][activeIndex].ackTemplate.tcpHeaderLen;
      *pTeplate      = wdrv_aeWSourceTable[stationIndex][activeIndex].ackTemplate.data;
}


void wdrv_aeWSourceDbUpdateTemplate(UINT8 *pktBuf,UINT8 stationIndex,UINT8 *sessionIndex)
{
   int index;
   int ipHeaderLen;
   UINT8 *pTmpIpHeader;
   int tmpIpHeaderLen;
   UINT32 ackNumber;
   UINT32 prevAckNumber;
   UINT32 segmentSize;
   UINT32 reorderProblemStatus;
   *sessionIndex =0xff;

   ipHeaderLen = ((*(unsigned char*)pktBuf  & 0x0f) * 4);
   /* Find WSource session */
   for(index=0 ; index< MAX_ACIVE_SESSION; index++)
	{
      pTmpIpHeader = (wdrv_aeWSourceTable[stationIndex][index].ackTemplate.data)+ WLAN_HDR_LEN+WLAN_SNAP_HDR_LEN;
      tmpIpHeaderLen = wdrv_aeWSourceTable[stationIndex][index].ackTemplate.ipHeaderLen;
      if (	*(UINT16*)(pTmpIpHeader +tmpIpHeaderLen + DEST_PORT_FIELD)== *(UINT16*)(pktBuf +ipHeaderLen + DEST_PORT_FIELD) && 
            *(UINT16*)(pTmpIpHeader +tmpIpHeaderLen )	== *(UINT16*)(pktBuf +ipHeaderLen ) && 
	         *(unsigned long*)(pTmpIpHeader+IP_DEST_ADDRESS_FIELD) 	== *(unsigned long*)(pktBuf+IP_DEST_ADDRESS_FIELD) &&
            *(unsigned long*)(pTmpIpHeader+IP_SRC_ADDRESS_FIELD)	== *(unsigned long*)(pktBuf+IP_SRC_ADDRESS_FIELD))
      {
         /* Update ackNumber , ackCounter and reorder problem flag */
         ackNumber = *(unsigned long*)(pktBuf+ipHeaderLen+TCP_ACK_NUMBER_FIELD);
         segmentSize = wdrv_aeWSourceTable[stationIndex][index].segmentSize;
			prevAckNumber = wdrv_aeWSourceTable[stationIndex][index].ackNumber;
         
         
         if((prevAckNumber % (segmentSize*2))>(ackNumber % (segmentSize*2)))
         {
            reorderProblemStatus = wdrv_aeWSourceTable[stationIndex][index].ackReorderProblem;

            switch(reorderProblemStatus)
            {
            case REORDER_PROBLEM_OFF:
               wdrv_aeWSourceTable[stationIndex][index].ackReorderProblem = REORDER_PROBLEM_ON;
               break;
            case REORDER_PROBLEM_ON:
               wdrv_aeWSourceTable[stationIndex][index].ackReorderProblem = REORDER_PROBLEM_PRE;
               break;
            case REORDER_PROBLEM_PRE:
            default:
               break;
            }
         }
			
         wdrv_aeWSourceTable[stationIndex][index].ackNumber = ackNumber;
         wdrv_aeWSourceTable[stationIndex][index].ackCounter =	ackNumber/(segmentSize*2);	
          *(unsigned long*)(pTmpIpHeader+tmpIpHeaderLen+TCP_SEQUENCE_NUMBER_FIELD) =
          *(unsigned long*)(pktBuf+ipHeaderLen+TCP_SEQUENCE_NUMBER_FIELD);
         
         *sessionIndex = index;
 /*        printf("\nindex =  = %d ackNumber %d , indentifier %d\n", *sessionIndex,ackNumber,
            *(UINT16*)(pktBuf+IP_IDENTIFIER_FIELD));*/
       
         
         return ;
      }
   }
   return ;
   
}
#endif