/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*		MODULE:	ackEmulDb.h											   */
/*   PURPOSE:	Ack emulation database 	              					   */
/*											    						   */
/***************************************************************************/

#ifndef _ACK_EMULATION_DB_H_
#define _ACK_EMULATION_DB_H_

#include "pform_types.h"

#define MAX_ACK_TEMPLATE_SIZE 100

#define IP_PROTOCOL_NUMBER          0x0800
#define TCP_PROTOCOL                0x06
#define UDP_PROTOCOL                0x11  
#define IP_TOTAL_LEN_FIELD          2
#define IP_IDENTIFIER_FIELD         4

#define IP_PROTOCOL_FIELD           9
#define IP_CHECKSUM_FIELD           10
#define DEST_PORT_FIELD             2
#define IP_SRC_ADDRESS_FIELD		   12
#define IP_DEST_ADDRESS_FIELD		   16


#define TCP_SEQUENCE_NUMBER_FIELD	4
#define TCP_ACK_NUMBER_FIELD        8
#define TCP_OFFSET_FIELD	         12
#define TCP_CHECKSUM_FIELD	         16

#define WTARGET_ACTIVE_TIME_OUT 500
#define WTARGET_TERMINATE_TIME_OUT 1000
#define WSOURCE_SESSION_TIME_OUT 700
#if 0

typedef struct {  
   UINT8 data[MAX_ACK_TEMPLATE_SIZE];
   UINT8 ipHeaderLen;
   UINT8 tcpHeaderLen;
} ackTemplate_T;

typedef struct {  
   UINT16	sPorts;                              /* Include source port  */
   UINT16	dPorts;                              /* Include destination port */
   UINT32	srcIPAddr;                          /* source ip address */
   UINT32	destIPAddr;                         /* destination ip address */
   UINT32   segmentSize;                        /* Stable segment size */
   UINT32   sequenceNumber;	                  /* The data sequence number */
   UINT32   ackNumber;	                        /* Specifies the next byte expected.*/
   UINT32   timeStamp;                          /*    */
   UINT32   ackCounter;	                        /* The number of 2 * Segement_size in ACK number  */
   ackTemplate_T    ackTemplate;			      	/* Ack template block */ 
   UINT8    monitorState;                       /* current monitor state */
   UINT8    equalSegmentSizeCounter;            /* count the number of equal segment size. */
   UINT8    yTagFlag;                           /* True if send Y tag */
   UINT8    activeIndex;                        /* Active session index */
   UINT16   sourceStationIndex;                 /* Source station index */
   UINT8    dataReceivedFlage;                  /* True if data packet receive for this active session */
   UINT8    lastWackInfo;                       /* The low bit of last Wack ack counter */
   
} ackEmulationWTargetDbTyple_T;


typedef struct {  
   UINT32   segmentSize;                        /* Stable segment size */
   UINT32   ackNumber;	                        /* Specifies the next byte expected.*/
   UINT32   ackCounter;	                        /* The number of 2 * Segement_size in ACK number  */
   UINT32   timeStamp;	                        /* The number of 2 * Segement_size in ACK number  */
   UINT32   ackReorderProblem;                
   ackTemplate_T    ackTemplate;			      	/* Ack template block */ 
}ackEmulationWSourceDbTyple_T;


/* Ack emulation monitor states */
typedef enum {
	REORDER_PROBLEM_OFF =0,
	REORDER_PROBLEM_ON,
	REORDER_PROBLEM_PRE,
} wdrv_eaAckReorderProblem_T;


typedef enum {
	AE_INACTIVE =0,
	AE_STANDBY,
	AE_CANDIDATE_ACTIVE,
	AE_ACTIVE,
   AE_TERMINATE,
} wdrv_eaMonitorStatus_T;


void wdrv_aeDbInit(void);
void wdrv_aeWTargetDbPrint(void);
int wdrv_aeWTargetDbFindDataSession(UINT8 *pktBuf,UINT32 *sessionIndex, UINT32 *monitorState);
int wdrv_aeWTargetDbFindAckSession(UINT8 *pktBuf,UINT32 *sessionIndex, UINT32 *monitorState);
int wdrv_aeWTargetDbAddSession(UINT8 *pktBuf, UINT16 stationIndex);

int wdrv_aeWTargetDbDelSession(UINT8 SessionIndex);
int wdrv_aeWTargetDbSetActiveState(UINT8 index , UINT8 *activeIndex);
void wdrv_aeWTargetDbSaveAckTemplate(UINT8 index, UINT8 *pIpHeader);
void wdrv_aeWTargetDbUpdateAckTemplate(UINT8 index, UINT32 sequenceNumber);
int wdrv_aeWTargetDbCmpAckTemplate(UINT8 index, UINT8 *pIpHeader);
void wdrv_aeWTargetDbResetTuple(int index);

void wdrv_aeWTargetDbGetSessionSequenceNumber(UINT8 index, UINT32 *sequenceNumber);
void wdrv_aeWTargetDbGetSessionAckNumber(UINT8 index, UINT32 *ackNumber);
void wdrv_aeWTargetDbGetSessionSegmentSize(UINT8 index, UINT32 *segmentSize);
void wdrv_aeWTargetDbGetIncrSessionEqualSegmentSizeCounter(UINT8 index, UINT8 *equalSegmentSizeCounter);
void wdrv_aeWTargetDbGetSessionAckCounter(UINT8 index, UINT32 *ackCounter);
void wdrv_aeWTargetDbGetSessionMmonitorState(UINT8 index, UINT8 *monitorState);
void wdrv_aeWTargetDbGetSessionActiveIndex(UINT8 index, UINT8 *activeIndex);
void wdrv_aeWTargetDbGetDataReceived(UINT8 index, UINT8 *dataReceived);
void wdrv_aeWTargetDbGetLastWackInfo(UINT8 index, UINT8 *lastWackInfo);

void wdrv_aeWTargetDbSetSessionSequenceNumber(UINT8 index, UINT32 sequenceNumber);
void wdrv_aeWTargetDbSetSessionAckNumber(UINT8 index, UINT32 ackNumber);
void wdrv_aeWTargetDbSetSessionSegmentSize(UINT8 index, UINT32 segmentSize);
void wdrv_aeWTargetDbSetSessionAckCounter(UINT8 index, UINT32 ackCounter);
void wdrv_aeWTargetDbSetSessionMonitorState(UINT8 index, UINT8 monitorState);
void wdrv_aeWTargetDbSetSessionEqualSegmentSizeCounter(UINT8 index, UINT8 equalSegmentSizeCounter);
void wdrv_aeWTargetDbSetDataReceived(UINT8 index, UINT8 dataReceived);
void wdrv_aeWTargetDbSetLastWackInfo(UINT8 index, UINT8 lastWackInfo);
void wdrv_aeWTargetDbGetSessionTimeStamp(UINT8 index, UINT32 *timeStamp);
void wdrv_aeWTargetDbSetSessionTimeStamp(UINT8 index, UINT32 timeStamp);
void wdrv_aeWTargetDbInit(void);

void wdrv_aeWSourceDbUpdateTemplate(UINT8 *pktBuf,UINT8 stationIndex,UINT8 *sessionIndex);
void wdrv_aeWSourceDbResetSession(int stationIndex,int activeIndex);

void wdrv_aeWSourceSaveAckTemplate(UINT8 stationIndex,UINT8 activeIndex,
											  UINT8 *dataBuf,UINT16 dataLen , UINT16 segmentSize);

void wdrv_aeWSourceDbGetSessionAckCounter(UINT16 stationIndex, UINT8 activeIndex, UINT32 *ackCounter);

void wdrv_aeWSourceDbSetSessionAckCounter(UINT16 stationIndex, UINT8 activeIndex, UINT32 ackCounter);

void wdrv_aeWSourceDbGetSessionAckNumber(UINT16 stationIndex, UINT8 activeIndex, UINT32 *ackNumber);
void wdrv_aeWSourceDbSetSessionAckNumber(UINT16 stationIndex, UINT8 activeIndex, UINT32 ackNumber);

void wdrv_aeWSourceDbGetSessionSegmentSize(UINT16 stationIndex, UINT8 activeIndex, UINT32 *segmentSize);
void wdrv_aeWSourceDbSetSessionSegmentSize(UINT16 stationIndex, UINT8 activeIndex, UINT32 segmentSize);

void wdrv_aeWSourceDbGetSessionTimeStamp(UINT16 stationIndex, UINT8 activeIndex, UINT32 *timeStamp);
void wdrv_aeWSourceDbSetSessionTimeStamp(UINT16 stationIndex, UINT8 activeIndex, UINT32 timeStamp);

void wdrv_aeWSourceDbGetSessionAckReorderProblem(UINT16 stationIndex, UINT8 activeIndex, UINT32 *ackReorderProblem);
void wdrv_aeWSourceDbSetSessionAckReorderProblem(UINT16 stationIndex, UINT8 activeIndex, UINT32 ackReorderProblem);


void wdrv_aeWSourceDbGetAckTemplate(UINT16 stationIndex, UINT8 activeIndex, UINT8 **pTeplate,
													 UINT8 *ipHeaderLen, UINT8 *tcpHeaderLen);


void wdrv_aeDbGetXTagStatus(UINT8 sessionIndex, UINT8 *status);
void wdrv_aeDbSetXTagStatus(UINT8 sessionIndex, UINT8 status);
void wdrv_aeDbGetDataReceivedStatus(UINT8 sessionIndex, UINT8 *status, UINT8 * dataReceivedIndex);
void wdrv_aeDbSetDataReceivedStatus(UINT8 sessionIndex, UINT8 status);

#endif
#endif
