/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
#if 0
/***************************************************************************/
/*																		   */
/*		MODULE:	ackEmulUtil.c											   */
/*   PURPOSE:	Ack emulation utility 	              					   */
/*															    		   */
/***************************************************************************/
#include "pform_types.h"
#include "ackEmulDb.h"
#include "802_11Defs.h"
#include "memMngr.h"
#include "pal.h"
#include "whal.h"

/* The minimum of segment size in byte to lock the segment size */
#define MIN_LOCK_SEGMENT_SIZE 500

typedef enum {
	OTHER    =0,
      TCP_ACK_ONLY ,
      TCP_DATA    ,
} packetInclude_T;


/* TI Snap Header */
/*  -------------------------------------------
   | dsap | SSAP | Control |    OUI   |  Type  |
   | 0xAA | 0xAA |   0x00  | 0x080028 | 0x60D0 |
    -------------------------------------------  */

#define TI_SNAP_HEADER_LEN 8
#define TI_SNAP_AE_TYPE 0x02
#define TI_SNAP_AE_LEN  6

#define K_FACTOR 14

static char tiSnapHeader[TI_SNAP_HEADER_LEN]={0xAA,0xAA,0x00,0x08,0x00,0x28,0x60,0xD0};

static void wdrv_ackEmulationPktType(UINT8 *pWlanSnapHeader ,UINT8 *pIpHeader , packetInclude_T *packetInclude, UINT16 *tcpDataSize);
static void wdrv_ackEmulationDataStandbyState(UINT8 sessionIndex, UINT8 *pIpHeader);
static void wdrv_ackEmulationAckStandbyState(UINT8 sessionIndex, UINT8 *pIpHeader);
static void wdrv_ackEmulationAckCandidateActivState(UINT8 sessionIndex, UINT8 *pIpHeader,
																	 UINT8 *addYTag, UINT8 *activeIndex,
																	 UINT32 *segmentSize);
static void wdrv_ackEmulationAckActivState(UINT8 sessionIndex, UINT8 *pIpHeader,UINT8 *dropAck);
static void wdrv_ackEmulationAckTerminateState(UINT8 sessionIndex);

static void wdrv_ackEmulationAddYTag(UINT8 *pDot11Header, UINT16 firstBdLength,
												 UINT8 activeIndex, UINT32 segmentSize);


static int wdrv_aeChackSnapWithYtag(UINT8 *pSnapHeader, UINT8 *tiSnapLen,
										  char *activeIndex, UINT16 *segmentSize);

			
static void wdrv_aeGenerateAck(UINT16 stationIndex, UINT8 activeIndex ,UINT32 ackNumber);

static int ackEmulationActive = FALSE;

static UINT16 wdrv_IpChecksumCalc(UINT16 len_ip_header, UINT8 *buff);
static UINT16 wdrv_TcpChecksumCalc(UINT16 len_tcp_header,UINT8 *IpSource, UINT8 *IpDest ,UINT8 *buff);


int aeul = 0;

int rxCounter = 0;
STATUS wdrv_ackEmulationRxPacket(mem_MSDU_T *pMsdu)
{
   
   UINT8 rc;
   UINT8 *dataBuf, SA[MAC_ADDR_LEN];
   dot11_header_T *pDot11Header;
   UINT8 *pWlanSnapHeader;
   UINT8 *pSnapHeader;
   UINT8 *pIpHeader;
   UINT16 tcpDataSize;
   packetInclude_T packetInclude;
   UINT32 sessionIndex, monitorState;
   char activeIndex = 0xff;
   UINT16 segmentSize = 0;
   UINT16 stationIndex;
   UINT8 stationActive = FALSE;
   UINT8 tiSnapLen = 0;
   UINT16 dataLen;
UINT8 WTsessionIndex =0xff;
   UINT16 XTag =0;


   if (ackEmulationActive == FALSE)
      return OK;


   dataBuf = memMgr_BufData(pMsdu->firstBDPtr);
   dataLen = memMgr_BufLength(pMsdu->firstBDPtr);
   pDot11Header = (dot11_header_T*) dataBuf;
rxCounter++;
   /* Find the station Index  */
   PAL_memCopy( SA, (UINT8*)&pDot11Header->address2, MAC_ADDR_LEN );
   if( wdrv_dbFindStaIndex(SA, &stationIndex) == OK )
   {
      if (wdrv_mgmtIsStaAssoc(stationIndex))
      { 
         stationActive = TRUE;
      }
   }
         

   //   pDot11Header->fc = ((pDot11Header->fc)|0x8000);
   
   XTag = ((pDot11Header->fc)&0x8000)>>15;
//      printf("\n 1 xtag = %x %x\n",pDot11Header->fc, XTag);
   pSnapHeader = dataBuf + WLAN_HDR_LEN;
   
   /* Chack if SNAP with Y TAG */
   rc = wdrv_aeChackSnapWithYtag(pSnapHeader, &tiSnapLen, &activeIndex, &segmentSize);
   if(rc == OK)
   {
      /* remove the TI SMAP */
      memmove(pSnapHeader,pSnapHeader + tiSnapLen, dataLen - WLAN_HDR_LEN - tiSnapLen);
      pMsdu->firstBDPtr->length -= tiSnapLen ;
      pMsdu->dataLen -= tiSnapLen;

//      printf("\n activeIndex %d segmentSize %d\n",activeIndex, segmentSize);

      if (stationActive == TRUE)
      { 
         
//      printf("\n  stationIndex %d\n", stationIndex);

         /* Update table and save template */
         wdrv_aeWSourceSaveAckTemplate(stationIndex,activeIndex,dataBuf,pMsdu->dataLen,segmentSize);
         
      }
      
      return OK;
   }
	
   /* Packet with out ack emulation TI Snap */
   pWlanSnapHeader = dataBuf + WLAN_HDR_LEN;
   pIpHeader = pWlanSnapHeader + WLAN_SNAP_HDR_LEN;
	
   wdrv_ackEmulationPktType(pWlanSnapHeader,pIpHeader,&packetInclude,&tcpDataSize);
	
	switch (packetInclude)
   {
   case TCP_DATA:
      
      rc = wdrv_aeWTargetDbFindDataSession(pIpHeader ,&sessionIndex, &monitorState);
      if(rc == OK)
      {
         wdrv_aeDbSetXTagStatus(sessionIndex, XTag);
      } 
      switch (monitorState)
      {
      case AE_INACTIVE:
         if(aeul)
            rc = wdrv_aeWTargetDbAddSession(pIpHeader, stationIndex);
         
			break;
		case AE_STANDBY:
         wdrv_ackEmulationDataStandbyState(sessionIndex, pIpHeader);
			break;
		case AE_CANDIDATE_ACTIVE:
			break;
		case AE_ACTIVE:
     //    wdrv_aeWTargetDbSetDataReceived(sessionIndex, TRUE);
         wdrv_aeDbSetDataReceivedStatus(sessionIndex, TRUE);
      default:
			break;
      }
      /*         printf("\n rc= %d  sessionIndex = %d monitorState = %d\n",rc, sessionIndex , monitorState);*/
      break;
      case TCP_ACK_ONLY:
       wdrv_aeWSourceDbUpdateTemplate(pIpHeader,stationIndex,&WTsessionIndex);
    //     if (WTsessionIndex != 0xff)
      //      printf("\n sessionIndex = %x \n",WTsessionIndex);
         break;
      case OTHER:
      default:
         break;
   }
   return OK;
}

int tx = 0;

STATUS wdrv_ackEmulationTxPacket(mem_MSDU_T *pMsdu,int *discardPacket)
{
   UINT8		rc;
   UINT8		*pDot11Header;
   UINT8    *pWlanSnapHeader;
   UINT8		*pIpHeader;
   UINT16	tcpDataSize;
   packetInclude_T packetInclude;
   UINT32	sessionIndex = 0xff;
   UINT32	monitorState = 0xff;
   UINT8		addYTag =FALSE;
   UINT8		dropAck =FALSE;
   UINT16	firstBdLength = 0;
   UINT32	firstBddataOffset = 0;
   UINT32	secondBddataOffset = 0;
	UINT8 	activeIndex = 0xff;
   UINT32	segmentSize = 0;

   *discardPacket = FALSE;
   if (ackEmulationActive == FALSE)
      return OK;
tx++;

   firstBdLength = memMgr_BufLength(pMsdu->firstBDPtr);
   firstBddataOffset = memMgr_BufOffset(pMsdu->firstBDPtr) ;
   pDot11Header = memMgr_BufData(pMsdu->firstBDPtr)+firstBddataOffset;

   if (firstBdLength == WLAN_HDR_LEN + WLAN_SNAP_HDR_LEN)
   {
      /* Packet from Ethernet  */
      if(pMsdu->firstBDPtr->nextBDPtr != NULL)
      {
         secondBddataOffset = memMgr_BufOffset(pMsdu->firstBDPtr->nextBDPtr);
         pIpHeader = memMgr_BufData(pMsdu->firstBDPtr->nextBDPtr)+ secondBddataOffset;
      }
   }
   else
   {
      /* Packet from Wlan  */
      pIpHeader = pDot11Header + WLAN_HDR_LEN + WLAN_SNAP_HDR_LEN;
   }

   pWlanSnapHeader = pDot11Header + WLAN_HDR_LEN;
   
   wdrv_ackEmulationPktType(pWlanSnapHeader,pIpHeader ,&packetInclude,&tcpDataSize);

   switch (packetInclude)
   {
   case TCP_ACK_ONLY:

      rc = wdrv_aeWTargetDbFindAckSession(pIpHeader ,&sessionIndex, &monitorState);

      switch (monitorState)
      {
      case AE_INACTIVE:

         break;
      case AE_STANDBY:
         
         wdrv_ackEmulationAckStandbyState(sessionIndex, pIpHeader);
         break;
      case AE_CANDIDATE_ACTIVE:
			wdrv_ackEmulationAckCandidateActivState(sessionIndex,
																 pIpHeader,
																 &addYTag,
																 &activeIndex,
																 &segmentSize);
			if(addYTag == TRUE)
			{
 //          d(pDot11Header,firstBdLength,1);
	 			wdrv_ackEmulationAddYTag(pDot11Header, firstBdLength, activeIndex, segmentSize);

//            printf("\nactiveIndex %d segmentSize %d\n",activeIndex, segmentSize);

				pMsdu->firstBDPtr->length += TI_SNAP_HEADER_LEN+2+TI_SNAP_AE_LEN;
				pMsdu->dataLen += TI_SNAP_HEADER_LEN+2+TI_SNAP_AE_LEN;
//           d(pDot11Header,pMsdu->dataLen ,1);
//           d(pDot11Header,pMsdu->firstBDPtr->length ,1);
//           d(pIpHeader,pMsdu->firstBDPtr->nextBDPtr->length ,1);

			}

			break;
      case AE_ACTIVE:
           /* Comper Template and drop packet */
         wdrv_ackEmulationAckActivState(sessionIndex, pIpHeader,&dropAck);
         
            *discardPacket = dropAck;
         break;

      case AE_TERMINATE:
         wdrv_ackEmulationAckTerminateState(sessionIndex);

         break;
      default:

         break;
      }

      break;
   case TCP_DATA:
      
       break;
      case OTHER:
      default:
        break;
   }
   return OK;
}



static void wdrv_ackEmulationPktType(UINT8 *pWlanSnapHeader ,UINT8 *pIpHeader , packetInclude_T *packetInclude, UINT16 *tcpDataSize)
{
   UINT16 pktTotalLen;
   UINT8 ipHeaderLen, tcpHeaderLen;

   *tcpDataSize =0;



   /* Check if the packet is TCP/IP */
   if ((((Wlan_LlcHeader_T*)pWlanSnapHeader)->Type == IP_PROTOCOL_NUMBER) &&
      (*(unsigned char*) ((ULONG)pIpHeader+(ULONG)IP_PROTOCOL_FIELD)== TCP_PROTOCOL))
   {
      /* Check if the packet include data or Ack only */
      pktTotalLen = (*(UINT16*)(pIpHeader + IP_TOTAL_LEN_FIELD));
      ipHeaderLen = ((*(unsigned char*)pIpHeader  & 0x0f) * 4);
      tcpHeaderLen = ((((*(unsigned char*)(pIpHeader + ipHeaderLen+TCP_OFFSET_FIELD))& 0xf0)>>4) * 4);
      *tcpDataSize = pktTotalLen - (ipHeaderLen + tcpHeaderLen);

      if(*tcpDataSize > 0)
         *packetInclude = TCP_DATA;
      else 
         *packetInclude = TCP_ACK_ONLY;

   }
   else
   {
      *packetInclude = OTHER;
   }
   
}



static void wdrv_ackEmulationDataStandbyState(UINT8 sessionIndex, UINT8 *pIpHeader)
{
	UINT8 ipHeaderLen = 0;
	UINT8 *tcpHeaderPtr;

	UINT32 prevSequenceNumber =0;
	UINT32 currentSequenceNumber =0;
	
	UINT32 prevSegmentSize = 0;
	UINT32 currentSegmentSize =0;

	UINT8 equalSegmentSizeCounter =0;



	/* Calculate Current Sequence Number */
	ipHeaderLen = ((*(unsigned char*)pIpHeader  & 0x0f) * 4);
	currentSequenceNumber = *(unsigned long*)(pIpHeader+ipHeaderLen+TCP_SEQUENCE_NUMBER_FIELD);

	wdrv_aeWTargetDbGetSessionSequenceNumber(sessionIndex,&prevSequenceNumber);
	/* Calclate Segment Size */
	wdrv_aeWTargetDbGetSessionSegmentSize(sessionIndex,&prevSegmentSize);
	currentSegmentSize = currentSequenceNumber - prevSequenceNumber;

	if (prevSegmentSize == currentSegmentSize)
	{
		/* Increase Equal Segment Size Counter */
		wdrv_aeWTargetDbGetIncrSessionEqualSegmentSizeCounter(sessionIndex,&equalSegmentSizeCounter);
		if(equalSegmentSizeCounter == 2 && currentSegmentSize > MIN_LOCK_SEGMENT_SIZE)
		{
			/* Monitor state -> AE_CANDIDATE_ACTIVE */
/*         printf("\n Session %d chabge state to AE_CANDIDATE_ACTIVE with segment size %d\n",
            sessionIndex,
            currentSegmentSize);*/
			wdrv_aeWTargetDbSetSessionMonitorState(sessionIndex,AE_CANDIDATE_ACTIVE);
		}

	}
	else
	{
		wdrv_aeWTargetDbSetSessionSegmentSize(sessionIndex,currentSegmentSize);
		wdrv_aeWTargetDbSetSessionEqualSegmentSizeCounter(sessionIndex,0);
	}
	
	wdrv_aeWTargetDbSetSessionSequenceNumber(sessionIndex,currentSequenceNumber);	
}


static void wdrv_ackEmulationAckStandbyState(UINT8 sessionIndex, UINT8 *pIpHeader)
{

   UINT8 ipHeaderLen = 0;
	UINT8 *tcpHeaderPtr;
	UINT32 currentAckNumber =0;

   	/* Calculate Current Ack Number */
	ipHeaderLen = ((*(unsigned char*)pIpHeader  & 0x0f) * 4);
	currentAckNumber = *(unsigned long*)(pIpHeader+ipHeaderLen+TCP_ACK_NUMBER_FIELD);

   	/* Set Current Ack Number */
	wdrv_aeWTargetDbSetSessionAckNumber(sessionIndex,currentAckNumber);



}

static void wdrv_ackEmulationAckCandidateActivState(UINT8 sessionIndex,
																	 UINT8 *pIpHeader,
																	 UINT8 *addYTag,
																	 UINT8 *activeIndex,
																	 UINT32 *segmentSize)
{

   UINT8 ipHeaderLen = 0;
	UINT32 prevAckNumber =0;
	UINT32 currentAckNumber =0;
   UINT32 ackCounter =0;

   	/* Calculate Current Ack Number */
	ipHeaderLen = ((*(unsigned char*)pIpHeader  & 0x0f) * 4);
	currentAckNumber = *(unsigned long*)(pIpHeader+ipHeaderLen+TCP_ACK_NUMBER_FIELD);

	wdrv_aeWTargetDbGetSessionSegmentSize(sessionIndex,segmentSize);
	wdrv_aeWTargetDbGetSessionAckNumber(sessionIndex,&prevAckNumber);
   
   if((currentAckNumber - prevAckNumber) == *segmentSize *2)
   {
      /* Change state to Active and update Active index */
      if(wdrv_aeWTargetDbSetActiveState(sessionIndex, activeIndex) == OK)
      {
         
 
         /* Calculate Ack Counter */
         ackCounter = currentAckNumber / (*segmentSize *2);
         wdrv_aeWTargetDbSetSessionAckCounter(sessionIndex,ackCounter) ;
         /* Save template update Active index */
         wdrv_aeWTargetDbSaveAckTemplate(sessionIndex,pIpHeader);
         /* add addYTag */
			*addYTag = TRUE;

      }
   }
   wdrv_aeWTargetDbSetSessionAckNumber(sessionIndex,currentAckNumber);

}

int cdrop = 0;
int cndrop = 0;
static void wdrv_ackEmulationAckActivState(UINT8 sessionIndex, UINT8 *pIpHeader,UINT8 *dropAck)
{
   int rc =NOK;

   UINT8 ipHeaderLen = 0;
   UINT32 currentAckNumber   =0;
   UINT32 prevAckNumber   =0;
   UINT32 tempAckNumber   =0;
   UINT32 segmentSize =0;
   UINT32 currentAckCounter =0;
   UINT32 prevAckCounter =0;
   UINT32 segmentNumber = 0;
   UINT8  activeIndex = 0xff;
   UINT8 XTag = 0;
   UINT8 dataReceivedFlage = 0;
   UINT8 tindex;
   UINT32 prevTimeStamp;
   UINT32 currentTimeStamp;
   UINT8 newWackInfo = 0;
   UINT8 prevWackInfo;
   UINT8 prevownershipBit;
   UINT8 prevactvIndxBits;
   UINT8 prevackCounterBits;
   
   static UINT8   prevWackSessionIndex = 0xff;
   static UINT32  prevWackAckCounter = 0;
   
   currentTimeStamp =PALSRV_getTick(); 
   
   wdrv_aeWTargetDbGetSessionTimeStamp(sessionIndex, &prevTimeStamp);
   if(prevTimeStamp != 0)
   {
      if(PALSRV_msec2ticks(WTARGET_ACTIVE_TIME_OUT) < (currentTimeStamp - prevTimeStamp))
      {
         /* Change session state to terminate */
//         printf("\n AE_TERMINATE \n");
         wdrv_aeWTargetDbSetSessionMonitorState(sessionIndex,AE_TERMINATE);
         return;
      }
   }

   
   
   whal_apiGetSend4xWackInfo(&prevWackInfo);           
   prevackCounterBits = (prevWackInfo&0x0f);
   prevactvIndxBits = (prevWackInfo&0x10)>>4;
   prevownershipBit = (prevWackInfo&0x80)>>7;
   
   if (prevWackSessionIndex == 0xff)
      dataReceivedFlage = FALSE;
   else
      //           wdrv_aeWTargetDbGetDataReceived(prevWackSessionIndex,&dataReceivedFlage);
      wdrv_aeDbGetDataReceivedStatus(prevWackSessionIndex,&dataReceivedFlage,&tindex);
   
   /* Calculate Current Ack Number */
   ipHeaderLen = ((*(unsigned char*)pIpHeader  & 0x0f) * 4);
   currentAckNumber = *(unsigned long*)(pIpHeader+ipHeaderLen+TCP_ACK_NUMBER_FIELD);
   
   wdrv_aeWTargetDbGetSessionSegmentSize(sessionIndex,&segmentSize);
   wdrv_aeWTargetDbGetSessionAckNumber(sessionIndex,&prevAckNumber);
   
   wdrv_aeWTargetDbGetSessionActiveIndex(sessionIndex, &activeIndex);

   /* Calculate Ack Counter */
   currentAckCounter = currentAckNumber / (segmentSize*2);
   if((currentAckNumber - prevAckNumber) == segmentSize *2)
   {
   /*   printf("\n currentAckCounter = %x prevAckCounter = %x \n",currentAckCounter ,prevAckCounter);
   printf("\n low = %x \n",currentAckCounter%8);
      printf("\n low = %x \n",currentAckCounter&0x7);*/

      wdrv_aeDbGetXTagStatus(sessionIndex,&XTag);
      
      if(XTag)
      {         
         
         wdrv_aeWTargetDbGetSessionAckCounter(sessionIndex, &prevAckCounter) ;
         
         if ((prevownershipBit == 1)&&(dataReceivedFlage)&&(prevAckCounter < prevWackAckCounter))
         {
            
            wdrv_aeWTargetDbSetSessionAckCounter(prevWackSessionIndex,prevWackAckCounter);        
         }
         
         
         if ((prevownershipBit == 1)|| (activeIndex == prevactvIndxBits))
         {
            wdrv_aeWTargetDbGetSessionAckCounter(sessionIndex, &prevAckCounter) ;
            if ((currentAckCounter > prevAckCounter) && (currentAckCounter < prevAckCounter + 5))
            {                            
               rc = wdrv_aeWTargetDbCmpAckTemplate(sessionIndex, pIpHeader);
               if (rc == OK)
               {  
                  *dropAck = TRUE;        
                  //printf("\n currentAckNumber %x \n",currentAckNumber);
               }
            }
         }
      }
   }
   
   if(*dropAck == TRUE)
   {
      newWackInfo = (((currentAckCounter % K_FACTOR)+1)&0xf); // Add currentAckCounter MOD K_FACTOR to bit 0-3   
      newWackInfo = newWackInfo | (activeIndex << 4);  // Add activeIndex to bit 4    
      /*               printf("\n currentAckCounter %x activeIndex %x newWackInfo%x\n",currentAckCounter,activeIndex,newWackInfo);*/
      whal_apiSetSend4xWackInfo(newWackInfo); 
      prevWackSessionIndex = sessionIndex;
      prevWackAckCounter   = currentAckCounter;
      wdrv_aeWTargetDbSetDataReceived(sessionIndex, FALSE);
      wdrv_aeWTargetDbSetSessionTimeStamp(sessionIndex, currentTimeStamp);
      cdrop ++;
   }
   else
   {

      if((prevAckNumber % (segmentSize*2))>(currentAckNumber % (segmentSize*2)))
		{
			/* Ack Reorder Problem */
			newWackInfo = 0xF; 
		}
		else
		{
         if(prevackCounterBits == 0xF)
         {
            /* chack if this wack info send */
            if((prevownershipBit == 1)&&(dataReceivedFlage))
               newWackInfo = 0;
         }
         else
         {
               newWackInfo = 0;
         }
		}
      newWackInfo = newWackInfo | (activeIndex << 4);  // Add activeIndex to bit 4    
      whal_apiSetSend4xWackInfo(newWackInfo); 

      wdrv_aeWTargetDbSetSessionAckCounter(sessionIndex,currentAckCounter) ;
      cndrop ++;

   }
   wdrv_aeWTargetDbSetSessionAckNumber(sessionIndex,currentAckNumber);
   segmentNumber = *(unsigned long*)(pIpHeader+ipHeaderLen+TCP_SEQUENCE_NUMBER_FIELD);
   wdrv_aeWTargetDbUpdateAckTemplate(sessionIndex,segmentNumber);

}


static void wdrv_ackEmulationAckTerminateState(UINT8 sessionIndex)
{
   UINT32 prevTimeStamp;
   UINT32 currentTimeStamp = PALSRV_getTick();

   wdrv_aeWTargetDbGetSessionTimeStamp(sessionIndex, &prevTimeStamp);
   if(prevTimeStamp != 0)
   {
      if(PALSRV_msec2ticks(WTARGET_TERMINATE_TIME_OUT) < (currentTimeStamp - prevTimeStamp))
      {
         /* Reset Session */
            //      printf("\n ResetTuple \n");
         wdrv_aeWTargetDbResetTuple(sessionIndex); 
      }
   }
   return;
}

static void wdrv_ackEmulationAddYTag(UINT8 *pDot11Header, UINT16 firstBdLength,
												 UINT8 activeIndex, UINT32 segmentSize)
{
	UINT8 *tempPtr;
	UINT32 moveLen;
	
	tempPtr = pDot11Header + WLAN_HDR_LEN;
	moveLen = firstBdLength - WLAN_HDR_LEN;
/*d(pDot11Header,firstBdLength,1);*/
	memmove(tempPtr+TI_SNAP_HEADER_LEN+2+TI_SNAP_AE_LEN ,tempPtr,moveLen);
	PAL_memCopy(tempPtr,tiSnapHeader,TI_SNAP_HEADER_LEN);
	tempPtr += TI_SNAP_HEADER_LEN;
   *(UINT8*)tempPtr = TI_SNAP_AE_TYPE; 
	tempPtr ++;
	*(UINT8*)tempPtr = TI_SNAP_AE_LEN; /* Length */
	tempPtr ++;
	*(UINT16*)tempPtr = (UINT16)activeIndex; /* Active Index */
	tempPtr +=2;
	*(UINT32*)tempPtr = (UINT32)segmentSize; /* Segment Size */
/*d(pDot11Header,firstBdLength+8+5,1);*/

}

static int wdrv_aeChackSnapWithYtag(UINT8 *pSnapHeader, UINT8 *tiSnapLen,
                                    char *activeIndex, UINT16 *segmentSize)
{
   
   /* Chack if SNAP with Y TAG */
   if ((PAL_memCmp(pSnapHeader,tiSnapHeader,TI_SNAP_HEADER_LEN)== 0)&&
      (*(UINT8*)(pSnapHeader + TI_SNAP_HEADER_LEN) == TI_SNAP_AE_TYPE))
   {
      
      /* This packet include Ack with Y TAG */
      UINT8* tempPtr;
      UINT8 templen;
      tempPtr = pSnapHeader + TI_SNAP_HEADER_LEN+1;
      templen = *(UINT8*)tempPtr;
      *tiSnapLen = TI_SNAP_HEADER_LEN + 2 + templen;     
      tempPtr ++;
      *activeIndex = *(char*)tempPtr;
      tempPtr ++;
      *segmentSize = *(UINT16*)tempPtr;
      return OK;
   }
   else
   {
      return NOK;
   }
}

int genera = 0;

void wdrv_aeWackReceive(UINT16 station, UINT8 wackInfo)
{
   UINT8 ackCounterlowBits;
   UINT8 TempAckCounterlowBits;
   UINT8 activeIndex;
   UINT8 ownershipBit;
   UINT32 oldAckNumber;
	UINT32 newAckNumber;
	UINT32 oldAckCounter;
	UINT32 newAckCounter;
	UINT32 segmentSize;
        UINT32 prevWackTimeStamp;
        UINT32 currentWackTimeStamp;
	UINT32  reorderProblemStatus;

   ackCounterlowBits = (wackInfo & 0xf);
   activeIndex      = (wackInfo & 0x10) >> 4;
   ownershipBit     = (wackInfo & 0x80) >> 7;

   
   if (ackCounterlowBits == 0)
      ackCounterlowBits = 0xF;
   else
      ackCounterlowBits --;


   if(ackCounterlowBits == 0xF)
		return;
      
	if(ackCounterlowBits == 0xE)
	{
		/* Reorder problem is possible */
		wdrv_aeWSourceDbSetSessionAckReorderProblem(station,activeIndex,REORDER_PROBLEM_ON);
		return;
	}
      wdrv_aeWSourceDbGetSessionTimeStamp(station,activeIndex,&prevWackTimeStamp);
      currentWackTimeStamp = PALSRV_getTick();
      
      if(PALSRV_msec2ticks(WSOURCE_SESSION_TIME_OUT) < (currentWackTimeStamp - prevWackTimeStamp))
      {
         /* reset Wsource session */
        // printf("\nwdrv_aeWSourceDbResetSession\n");
         wdrv_aeWSourceDbResetSession(station, activeIndex);

         return;
      }

      wdrv_aeWSourceDbSetSessionTimeStamp(station,activeIndex,currentWackTimeStamp);

      // printf("\n  ackCounterlowBits %x activeIndex %x\n", ackCounterlowBits,activeIndex);
 wdrv_aeWSourceDbGetSessionSegmentSize(station,activeIndex,&segmentSize); 
   if(segmentSize != 0)
   {
      wdrv_aeWSourceDbGetSessionAckNumber(station,activeIndex,&oldAckNumber); 
      wdrv_aeWSourceDbGetSessionAckCounter(station,activeIndex,&oldAckCounter); 
      
      

      /* newACK_counter = (Low_counter - (oldACK_counter mod 2^k) + 2^k + 3) mod 2^k - 3 + oldACK_counter*/
      newAckCounter = ((ackCounterlowBits - (oldAckCounter % K_FACTOR) + K_FACTOR + 3) % K_FACTOR) -3 + oldAckCounter;

      if(newAckCounter <= oldAckCounter)
         return;

      wdrv_aeWSourceDbGetSessionAckReorderProblem(station,activeIndex,&reorderProblemStatus);
      if(reorderProblemStatus == REORDER_PROBLEM_ON)     
         newAckNumber = (newAckCounter * (segmentSize*2));
      else
      /* newACK_number = oldACK_number mod (Segment_Size*2) + Rx_Ack_count * (Segment_Size*2)*/      
      newAckNumber = (oldAckNumber % (segmentSize*2)) + (newAckCounter * (segmentSize*2));

      /* Generate ack */
      genera++;
     	wdrv_aeGenerateAck(station,activeIndex,newAckNumber);
      
      wdrv_aeWSourceDbSetSessionAckNumber(station,activeIndex,newAckNumber); 
      wdrv_aeWSourceDbSetSessionAckCounter(station,activeIndex,newAckCounter); 
 //     printf("\n newAckCounter %d oldAckCounter %d\n",newAckCounter ,oldAckCounter);
   }
   return;

}

int gen =0;
static void wdrv_aeGenerateAck(UINT16 stationIndex, UINT8 activeIndex ,UINT32 ackNumber)
{
   mem_MSDU_T *pMsdu;
   UINT8 ipHeaderLen;
   UINT8 tcpHeaderLen;
   UINT8 *pTeplate;
   UINT8 *pNewPkt;
   UINT16  newPktLen;
   UINT8 *ipSource, *ipDest;
   UINT8 *pIpHeader, *pTcpHeader;
   UINT16 ipChecksum =0;
   UINT16 tcpChecksum =0;
   
   gen++;
   wdrv_aeWSourceDbGetAckTemplate(stationIndex, activeIndex, &pTeplate,&ipHeaderLen, &tcpHeaderLen);
   if(ipHeaderLen > 0)
   {
      newPktLen = WLAN_HDR_LEN+WLAN_SNAP_HDR_LEN+ipHeaderLen+tcpHeaderLen;
      wlan_memMngrAllocMSDU(&pMsdu,newPktLen);
      
      
      pNewPkt = memMgr_BufData(pMsdu->firstBDPtr);
      PAL_memCopy(pNewPkt,pTeplate,newPktLen);
      
      pMsdu->firstBDPtr->dataOffset = 0;
      pMsdu->firstBDPtr->length = newPktLen;
      pMsdu->dataLen = newPktLen;
      
      /* Calclate IP Checksum */
      pIpHeader = pNewPkt+WLAN_HDR_LEN+WLAN_SNAP_HDR_LEN;
      ipChecksum = wdrv_IpChecksumCalc(ipHeaderLen,pIpHeader);
      
      /* Calclate TCP Checksum */
      
      pTcpHeader = pIpHeader + ipHeaderLen;


 //     printf("\nackNumber %x\n",ackNumber);

      *(UINT32*)(pTcpHeader+TCP_ACK_NUMBER_FIELD) = ackNumber;
      ipSource = pIpHeader+IP_SRC_ADDRESS_FIELD;
      ipDest   = pIpHeader+IP_DEST_ADDRESS_FIELD;
      tcpChecksum = wdrv_TcpChecksumCalc(tcpHeaderLen, ipSource, ipDest, pTcpHeader);
      /* Add the Checksum to the new packet */
      *(UINT16*)(pIpHeader+IP_CHECKSUM_FIELD) = ipChecksum;
      *(UINT16*)(pTcpHeader+TCP_CHECKSUM_FIELD) = tcpChecksum;
      
//      d(pNewPkt,newPktLen,1);
 //     wlan_memMngrFreeMSDU(pMsdu->handle);
     wdrv_bssBridgeFromWlanFrame(pMsdu,TRUE, FALSE,stationIndex);		/* FALSE in micOK because in is not received from the WLAN,
																	it was generated internally in the AP.*/
   }
}

/****************************************************************************
 *                      wdrv_aeSetActive()
 ****************************************************************************
 * DESCRIPTION:	Enable/ Desable Ack Emulation
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/

void wdrv_aeSetActive(int status)
{
   
   if (status == TRUE)
   {
      (void)whal_apiSetSend4xWackInfo(0);
      (void)whal_apiEnable4xWackFeature();
   }
   if (status == FALSE)
   {
      (void)whal_apiSetSend4xWackInfo(0);
      (void)whal_apiDisable4xWackFeature();
   }
   
      ackEmulationActive = status;
}

int wdrv_aeGetActive()
{
   return(ackEmulationActive);
}


void wdrv_aePrintActive()
{
  printf("\n ackEmulationActive  = %d\n",ackEmulationActive);
}


/*
**************************************************************************
Function: wdrv_IpChecksumCalc
Description: Calculate the 16 bit IP Checksum.
***************************************************************************
*/


static UINT16 wdrv_IpChecksumCalc(UINT16 len_ip_header, UINT8 *buff)
{
UINT16 word16;
UINT32 sum=0;

UINT16 i;
	// make 16 bit words out of every two adjacent 8 bit words in the packet
	// and add them up
	for (i=0;i<len_ip_header;i=i+2){
		word16 = *(UINT16*)(buff+i);
		sum = sum + (UINT32) word16;	
	}
	
	// take only 16 bits out of the 32 bit sum
//temp = (sum & 0xffff0000)>>16;
sum = (sum & 0x0000ffff)+((sum & 0xffff0000)>>16);
	// one's complement the result
	sum = ~sum;
	
return ((UINT16) sum);
}

/*
**************************************************************************
Function: wdrv_TcpChecksumCalc
Description: Calculate the 16 bit TCP Checksum.
***************************************************************************
*/

static UINT16 wdrv_TcpChecksumCalc(UINT16 len_tcp_header,UINT8 *IpSource, UINT8 *IpDest ,UINT8 *buff)
{
UINT16 word16;
UINT32 sum=0;
UINT16 i;


	
	// add a padding byte = 0 at the end of packet

		buff[len_tcp_header]=0;
	

	// make 16 bit words out of every two adjacent 8 bit words in the packet
	// and add them up
	for (i=0;i<len_tcp_header;i=i+2){
		word16 = *(UINT16*)(buff+i);
		sum = sum + (UINT32) word16;	
	}

   	
   // add the TCP pseudo header which contains:
	// the IP source and destinationn addresses, TCP protocol & TCP length,

   
   word16 = *(UINT16*)IpSource;
   sum = sum + (UINT32) word16;	
   word16 = *(UINT16*)(IpSource+2);
   sum = sum + (UINT32) word16;	
   word16 = *(UINT16*)IpDest;
   sum = sum + (UINT32) word16;	
   word16 = *(UINT16*)(IpDest+2);
   sum = sum + (UINT32) word16;	
   sum = sum + (UINT32)len_tcp_header + 0x06;
   
      
      
      
      // take only 16 bits out of the 32 bit sum
sum = (sum & 0x0000ffff)+((sum & 0xffff0000)>>16);
	// one's complement the result
	sum = ~sum;
	
return ((UINT16) sum);
}

/*  Debug Function */ 
void gener(UINT16 stationIndex, UINT8 activeIndex ,UINT32 ackNumber)
{
	wdrv_aeGenerateAck(stationIndex,activeIndex,ackNumber);
}

void setAeUl(int status)
{
   aeul = status;
   wdrv_aeWTargetDbInit();
}


void showAeUl()
{
   if(aeul)
      printf("\nAck emulation Up Load - ON\n");
      else
      printf("\nAck emulation Up Load - OFF\n");
}
#endif