/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																									*/
/*		MODULE:	Shutdown.c																		*/
/*    PURPOSE:	Wireless Enable/Disable feature							*/
/*																									*/
/***************************************************************************/
#include "pform_types.h"
#include "mgmtSm.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "whal.h"
#include "whalAcxCmd.h"
#include "mgmtIf.h"
#include "whalInterrupt.h"
#include "mainsm.h"
#include "rx.h"
#include "dda.h"


/*************************************************************************
 *                        wdrv_EnableWlan                                *
 *************************************************************************
DESCRIPTION: 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:
**************************************************************************/

void wdrv_WlanServiceEnable ( void )
{
	UINT8	SavedEnableWA;
	BOOL	WlanState;
	
	wdrv_dbGetWlanServiceState( &WlanState );
	
	printf("============================================================\n");
	printf("  wdrv_WlanServiceEnable : current state is %s\n", (WlanState)?"Enabled":"Disabled" );
	printf("============================================================\n");
	
	if (WlanState)
	{
		return;
	}	
	
	wdrv_dbSetWlanServiceState( TRUE );
   	wdrv_dbGetEnRestart	(&SavedEnableWA );
   	
  	
   	wdrv_dbSetEnRestart (SavedEnableWA, TRUE );
    
    wdrv_EnableWlanService();
	tiwlan_ddaLedAction(TIAP_LED_ON);
}


void wdrv_WlanServiceDisable ( void )
{
	int 	i, state;
	char	mac[6];
	UINT8	CurrentEnableWA;
	BOOL	WlanState;
	
	wdrv_dbGetWlanServiceState( &WlanState );
	
	printf("============================================================\n");
	printf("  wdrv_WlanServiceDisable : current state is %s\n", (WlanState)?"Enabled":"Disabled" );
	printf("============================================================\n");
	
	if (!WlanState)
	{
		return;
	}	
		
   	wdrv_dbGetEnRestart				( &CurrentEnableWA );   /* Get current WA */
   	wdrv_dbSetEnRestart				( 0, FALSE );                  /* Disable WA, do not update nvram */
	wdrv_dbSetWlanServiceState		( FALSE );
   	wdrv_dbSetEnRestart				(  CurrentEnableWA, TRUE );   /* Save current WA in order to reproduce it later */
	
	whal_acxRxIntrDisable();
	wdrv_rxDisable();
	//whal_acxCmdDisableRx();

	for (i = 0; i < MAX_STATIONS; i++)
	{
		wdrv_mgmtGetState(i, &state);
		if (state != STATE_IDLE)
		{
			wdrv_dbGetStaMacAddress(i, mac);
			
		    /* send a deauhtentication to all connected STAs */
			wdrv_mgmtDeAuthMng(mac, STATUS_UNSPECIFIED);
			printf("sending deauth to MAC: %02X-%02X-%02X-%02X-%02X-%02X\n",
					(UINT8)*(mac), (UINT8)*(mac+1), (UINT8)*(mac+2), (UINT8)*(mac+3),
					(UINT8)*(mac+4), (UINT8)*(mac+5));
			do
			{
				wdrv_mgmtGetState(i, &state);
				/* wait for response 100ms */				
				PAL_sleepLong(100);
			}
			while(state != STATE_IDLE);
		}	
	}
	
	wdrv_rxEnable();
	whal_acxCmdDisableRx();
	whal_acxRxIntrEnable();
	whal_apiStopBss();
	tiwlan_ddaLedAction(TIAP_LED_OFF);
}
