/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*                                                                         */
/*     MODULE:  WoneBalance.c                                              */
/*    PURPOSE:  Wone feature dynamic balancing mechanism implementation    */
/*                                                                         */
/***************************************************************************/

#define WONE_BALANCE_C

#include "WoneBalance.h"
#include "pform_types.h"
#include "envList.h"
#include "drvDbMain.h"
#include "mgmtSm.h"
#include "mainsm.h"
/*#include "whalAcxConfig.h"*/
#include "drvDbHash.h"
#include "whalAcxConfig.h"

#define  WONE_RX_PACKET_THRESHOLD  2
#define  WONE_TX_PACKET_THRESHOLD  2
#define  RxSCALE(rx) (rx)
#define  TxSCALE(tx) (tx)

#define WONE_SLICE_MAX 30


UINT32              wone_balance_period = 1000;
PALSRV_TcbObjTYPE   WoneBalanceTimer;

extern  EnvList_T   *staEnvList[];

static  UINT8   PrevSlice   [MAX_WONE_NET];
static  BOOL    WoneBalanceEn;

static  void wdrv_WoneSliceUpdateTimerCB(UINT32 data);
static  UINT8  woneSliceMin;
static  UINT8  woneSliceMax; 

/****************************************************************************
 *                      wdrv_WoneBalanceSuspend()
 ****************************************************************************
 * DESCRIPTION: This function suspends dynamic slices calculation basing
 *              existing rx/tx statistics.
 *
 * INPUTS:  None
 *
 * OUTPUT:  None
 *
 * RETURNS: None
 ****************************************************************************/
void    wdrv_WoneBalanceSuspend( void )
{
    WoneBalanceEn   =   FALSE;
}

/****************************************************************************
 *                      wdrv_WoneBalanceResume()
 ****************************************************************************
 * DESCRIPTION: This function resumes dynamic slices calculation basing
 *              existing rx/tx statistics.
 *
 * INPUTS:  None
 *
 * OUTPUT:  None
 *
 * RETURNS: None
 ****************************************************************************/
void    wdrv_WoneBalanceResume( void )
{
    UINT8   WoneIndex,TotalWones;

    wdrv_dbGetNumSupWone(&TotalWones);

    for (WoneIndex = 0; WoneIndex < MAX_WONE_NET ; WoneIndex++)
    {
        PrevSlice[WoneIndex] = 0;
    }

    WoneBalanceEn   =   TRUE;
}

/****************************************************************************
 *                      wdrv_WoneBalanceReset()
 ****************************************************************************
 * DESCRIPTION: This function resets the slices to mid-values and suspends
 *              their dynamic calculation.
 *
 * INPUTS:  None
 *
 * OUTPUT:  None
 *
 * RETURNS: None
 ****************************************************************************/
void    wdrv_WoneBalanceReset( void )
{
    UINT8   WoneIndex,TotalWones;

    wdrv_WoneBalanceSuspend();

    wdrv_dbGetNumSupWone(&TotalWones);
   /* Override the defaults saved in NVRAM and set all slices to equal values */
    for (WoneIndex = 0; WoneIndex < TotalWones ; WoneIndex++)
    {
        wdrv_dbUpdateUsageTime(WoneIndex, ( woneSliceMax + woneSliceMin ) / TotalWones );
    }

    whal_acxConfigWoneSlice();
}


/****************************************************************************
 *                      wdrv_PeriodicWoneSliceUpdate()
 ****************************************************************************
 * DESCRIPTION: This function performs dynamic slices calculation basing
 *              existing rx/tx statistics.
 *
 * INPUTS:  None
 *
 * OUTPUT:  None
 *
 * RETURNS: None
 ****************************************************************************/

void wdrv_PeriodicWoneSliceUpdate ( void )
{
    int     i,state;
    UINT8   WoneIndex,TotalWones;
    UINT32  rxCounter = 0 , txCounter = 0;
    UINT32  rxAcc       [MAX_WONE_NET];
    UINT32  txAcc       [MAX_WONE_NET];
    UINT32  TotalUsage;
    UINT8   NewSlice    [MAX_WONE_NET];
    UINT8   tmp;
    BOOL    Update = FALSE;

    if (! WoneBalanceEn)
    {
        return ;
    }

    wdrv_dbGetNumSupWone(&TotalWones);

    if (TotalWones == WONE_SINGLE_BAND)
    {
        return ;
    }

    for (WoneIndex = 0; WoneIndex < MAX_WONE_NET ; WoneIndex++)
    {
        rxAcc[WoneIndex]        = 0;
        txAcc[WoneIndex]        = 0;
    }

   /*-----------------------------------------------------------------------*/
   /*       Gather current tx and accumulated rx statistics.                */
   /*                                                                       */
    for (i = 0; i < MAX_STATIONS; i++)
    {
        wdrv_mgmtGetState(i, &state);
        if (state != STATE_IDLE)
        {
            wdrv_dbGetStationWoneIndex          (i,&WoneIndex);
            wdrv_dbGetStationRxPacketCounter    (i,&rxCounter);
            wdrv_dbResetStationRxPacketCounter  (i);
            wdrv_dbGetStationTxPacketCounter    (i,&txCounter);
            wdrv_dbResetStationTxPacketCounter  (i);
            if (WoneIndex < MAX_WONE_NET)
            {
                if( rxCounter > WONE_RX_PACKET_THRESHOLD )
                {
	              /* Count in the station, which sent us more than threshold packets */
                    rxAcc[WoneIndex]++;
                }
                if( txCounter/*staEnvList[i]->CurrNumOfEnv*/ > WONE_TX_PACKET_THRESHOLD )
                {
	              /* Count in the station, which has more than threshold packets queued */
                    txAcc[WoneIndex]++;
                }
            }
        }

    }
   /*-----------------------------------------------------------------------*/

   /*-----------------------------------------------------------------------*/
   /*                          Calculate new Slices                         */
   /*                                                                       */
   /*                                                                       */
   /*                                                                       */
   /*                                                                       */
   /*    The expression is as follows:                                      */
   /*                                                                       */
   /*               RxStations(band) * Scale + TxStations(band) * Scale     */
   /*  X(band) = ---------------------------------------------------------  */
   /*               sum( RxStations(i) * Scale + TxStations(i) * Scale )    */
   /*                                                                       */
   /*                                                                       */
   /*  slice(band) =  MIN_slice + ( X(band) * (MAX_slice - MIN_slice) )     */
   /*                                                                       */
   /*                                                                       */
   /*                                                                       */

    for (WoneIndex = 0, TotalUsage = 0; WoneIndex < TotalWones ; WoneIndex++)
    {
        TotalUsage += RxSCALE(rxAcc[WoneIndex]) + TxSCALE(txAcc[WoneIndex]);
    }

    for (WoneIndex = 0,tmp = 0; WoneIndex < TotalWones - 1  ; WoneIndex++)
    {
        if (TotalUsage == 0)
        {
            NewSlice[WoneIndex] = ( woneSliceMax + woneSliceMin ) / TotalWones;
        }
        else
        {
            NewSlice[WoneIndex] =
                ( RxSCALE(rxAcc[WoneIndex]) + TxSCALE(txAcc[WoneIndex]) ) *
                ( woneSliceMax - woneSliceMin )
                            / TotalUsage + woneSliceMin;
        }

        if (PrevSlice[WoneIndex] != NewSlice[WoneIndex])
        {
            Update = TRUE;
            PrevSlice[WoneIndex]  = NewSlice[WoneIndex];
        }

        tmp += NewSlice[WoneIndex];
    }

    NewSlice[WoneIndex] = woneSliceMin + woneSliceMax - tmp;
    if (PrevSlice[WoneIndex] != NewSlice[WoneIndex])
    {
        Update = TRUE;
        PrevSlice[WoneIndex]  = NewSlice[WoneIndex];
    }

   /*-----------------------------------------------------------------------*/

   /*-----------------------------------------------------------------------*/
   /*                    Update Slices in DB and firmware                   */
   /*                                                                       */

    if (Update && WoneBalanceEn)
    {
/*      printf("==== Wone slice update ====\n"); */
        for (WoneIndex = 0; WoneIndex < TotalWones  ; WoneIndex++)
        {
            wdrv_dbUpdateUsageTime( WoneIndex, NewSlice[WoneIndex] );
/*              printf( "slice[%d]:%d ",WoneIndex, NewSlice[WoneIndex] ); */
        }
/*      printf("\n"); */
        whal_acxConfigWoneSlice();
    }

   /*-----------------------------------------------------------------------*/

}


void wdrv_WoneStartBalanceTimer(void)
{
	UINT8   TotalWones;

	wdrv_dbGetNumSupWone(&TotalWones);
	
	if (TotalWones != WONE_SINGLE_BAND)
	{
		PALSRV_tcbNew(&WoneBalanceTimer);
		PALSRV_tcbQueueAct(&WoneBalanceTimer, wone_balance_period,
							wdrv_WoneSliceUpdateTimerCB, 0, 0);
	}
}

/****************************************************************************
 *                      wdrv_StartWoneBalance()
 ****************************************************************************
 * DESCRIPTION: This function performs dynamic slices calculation basing
 *              existing rx/tx statistics.
 *
 * INPUTS:  None
 *
 * OUTPUT:  None
 *
 * RETURNS: None
 ****************************************************************************/

void wdrv_WoneBalanceInit ( void )
{
    UINT8   WoneIndex,TotalWones;
	UINT8   totalUsageTime = 0;
	extern BOOL recoveryIsInProcess;

    wdrv_dbGetNumSupWone(&TotalWones);

    if (TotalWones == WONE_SINGLE_BAND)
    {
        return ;
    }

    for (WoneIndex = 0; WoneIndex < MAX_WONE_NET ; WoneIndex++)
    {

       UINT8 tempUsageTime = 0;
       
	   PrevSlice[WoneIndex] = 0;
       wdrv_dbGetUsageTime(WoneIndex ,&tempUsageTime);

       totalUsageTime += tempUsageTime;
         
    }


	/* removed due to lack of the floating point support in Linux kernel on mips */
    /* woneSliceMax = _min((totalUsageTime * 0.85),WONE_SLICE_MAX); */
    woneSliceMax = _min((totalUsageTime * 85/100),WONE_SLICE_MAX); 
    woneSliceMin =  totalUsageTime - woneSliceMax;

    WoneBalanceEn = TRUE;


	/* the timer must not be called during the recovery,
	 * since tiwlan_ddaResume would call it for the second time,
	 * (since PALSRV_tcbNew adds it to the list).
	 */
	if (recoveryIsInProcess == FALSE)
			wdrv_WoneStartBalanceTimer();
}

/****************************************************************************
 *                      wdrv_WoneBalanceUnload()
 ****************************************************************************
 * DESCRIPTION: This function unloads WoneBalance module
 *
 * INPUTS:  None
 *
 * OUTPUT:  None
 *
 * RETURNS: None
 ****************************************************************************/
void wdrv_WoneBalanceUnload(void)
{
	UINT8 TotalWones;

	wdrv_dbGetNumSupWone(&TotalWones);

	if (TotalWones != WONE_SINGLE_BAND)
	{
		PALSRV_tcbDelete(&WoneBalanceTimer);
	}
	printf("WoneBalance unloaded !!!\n");
}


void wdrv_WoneSliceUpdateTimerCB(UINT32 data)
{
	wdrv_PeriodicWoneSliceUpdateRequest();

    PALSRV_tcbQueueAct (&WoneBalanceTimer,
                        wone_balance_period,
                        wdrv_WoneSliceUpdateTimerCB,
						0, 0);
}

/****************************************************************************
 *                      wdrv_StopWoneBalance()
 ****************************************************************************
 * DESCRIPTION: This function performs dynamic slices calculation basing
 *              existing rx/tx statistics.
 *
 * INPUTS:  None
 *
 * OUTPUT:  None
 *
 * RETURNS: None
 ****************************************************************************/

void wdrv_WoneBalanceShutdown ( void )
{
	UINT8 TotalWones;

	wdrv_dbGetNumSupWone(&TotalWones);
	if (TotalWones != WONE_SINGLE_BAND)
	{
		WoneBalanceEn = FALSE;
		PALSRV_tcbCancel (&WoneBalanceTimer );
	}
}

#undef WONE_BALANCE_C
