/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																									*/
/*		MODULE:	bssbridge.c	      														*/
/*   PURPOSE:	Bssbridge functions.	      			 								*/
/*																									*/
/***************************************************************************/
#include "pform_types.h"
#include <stdio.h>
#include "bssbridge.h"
#include "mainsm.h"
#include "802_11Defs.h"
#include "tx.h"
#include "ackEmulUtil.h"
#include "pal.h"
#include "dda_wpa_interface.h"
/*#include "wpaIfc.h" 
#include "mic.h" 
*/
#include "dda_contexts.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "mgmtIf.h"
#include "dda.h"

/*#include "ddc_stubs.h"*/
/*#include <netinet/in.h>*/


/*
This is an Ethernet Version 2 frame:

       +--------------+
       |              | The destination address is a six byte Media Access
       | Destination  | Control (MAC) address, usually burned into the
       |   6 bytes    | ROM of the Ethernet card.
       +--------------+
       |              | The source address is a six byte MAC address, and
       |   Source     | can signify a physical station or a broadcast.
       |   6 bytes    |
       +--------------+
       |     Type     | The Type field it must be grater then 1500 dec.
       |    2 bytes   |
       +--------------+
       |              |  Any higher layer information is placed in the
       |    Data      |  data field, which could contain protocol
       |              |  information or user data.
       ~              ~
       ~              ~
       |  46 to 1500  |
       |    bytes     |
       |              |
       +--------------+
       |     FCS      |  
       |   4 bytes    |  
       +--------------+  
                         


*/
/*

        802.2 SNAP DATA Frame 
       +----------------+
       |                |
       |  Destination   |
       |    6 bytes     |
       +----------------+
       |                |
       |     Source     |
       |    6 bytes     |
       +----------------+
       |  Frame Length  | Must be <= 1500 Dec.
       |    2 bytes     |
       +----------------+
       |  DSAP - 1 byte | = 0xAA ( SNAP )
       +----------------+
       |  SSAP - 1 byte | = 0xAA ( SNAP )
       +----------------+
       |Control - 1 byte| = 0x03 
       +----------------+ 
       | OUI - 3 bytes  | = 0x0
       |                | 
       +----------------+
       | Type - 2 bytes |  = Ethernet type (IP=0x0800)
       +----------------+ 
       |                | 
       |      Data      |
       |                |
       ~                ~
       ~                ~
       |   46 to 1500   |
       |     bytes      |
       |                |
       +----------------+
       |      FCS       |
       |    4 bytes     |
       +----------------+




        802.11 DATA Frame
       +----------------+
       |                |
       |  Frame Control |
       |    2 bytes     | 
       +----------------+
       |                |
       |  Duration ID   |
       |    2 bytes     |        
       +----------------+
       |                |
       |   Address 1    |
       |    6 bytes     |
       +----------------+
       |                |
       |   Address 2    |
       |    6 bytes     |
       +----------------+
       |                |
       |   Address 3    |
       |    6 bytes     |
       +----------------+
       |                |
       | Sequence Cntrl |
       |    2 bytes     |
       +----------------+
       |                |
       |   Address 4    |
       |    6 bytes     |
       +----------------+
       |  DSAP - 1 byte | = 0xAA ( SNAP )
       +----------------+
       |  SSAP - 1 byte | = 0xAA ( SNAP )
       +----------------+
       |Control - 1 byte| = 0x03 
       +----------------+ 
       | OUI - 3 bytes  | = 0x0
       |                | 
       +----------------+
       | Type - 2 bytes |  = Ethernet type (IP=0x0800)
       +----------------+ 
       |                | 
       |      Data      |
       |                |
       ~                ~
       ~                ~
       |   46 to 1500   |
       |     bytes      |
       |                |
       +----------------+
       |      FCS       |
       |    4 bytes     |
       +----------------+

*/

#pragma pack(2)
typedef struct
{
  macAddress_T	DstAddr;
  macAddress_T	SrcAddr;
  UINT16    Length;
  UINT8     DSAP;
  UINT8     SSAP;
  UINT8     Control;
  UINT8     OUI[3];
  UINT16		Type;
} LlcSnapHeader_T;
#pragma pack()


#define SNAP_OUI_802_1H_BYTE0 0x00
#define SNAP_OUI_802_1H_BYTE1 0x00
#define SNAP_OUI_802_1H_BYTE2 0xf8
UINT8 SNAP_OUI_802_1H[] = { SNAP_OUI_802_1H_BYTE0, SNAP_OUI_802_1H_BYTE1, SNAP_OUI_802_1H_BYTE2 };


UINT8 SNAP_OUI_RFC1042[] = { SNAP_OUI_RFC1042_BYTE0, SNAP_OUI_RFC1042_BYTE1, SNAP_OUI_RFC1042_BYTE2 };


#define SNAP_CHANNEL_ID                    0xAA
#define BSS_PERF_DBG
typedef enum tETHERTYPES {

  ETHERTYPE_APPLE_AARP = 0x80f3,
  ETHERTYPE_DIX_II_IPX = 0x8137

} ETHERTYPES, *PETHERTYPES;

extern void wdrv_dbGetWlanServiceState ( BOOL *state );

/* Gather all statistics in one structure */
static struct bssBrgStat {
   UINT32 pktsUnicWlanToWlan;
   UINT32 pktsUnicWlanToEthr;
   UINT32 pktsUnicEthrToWlan;
   UINT32 pktsBrcstFromWlan;
   UINT32 pktsBrcstFromEthr;
   UINT32 pktsFromEthrToWlanErrStaNotFound;
   UINT32 pktsDroppedDueToIsolation;
} bssBrgStat = {0};

static BOOL isEapol(mem_MSDU_T *msduPtr);

static BOOL bssBridgeExcludeUnencrypted;
static UINT32 bssBridgeExcludedCount;
static UINT32 bssBridgeTkipMicFailureCount;

/****************************************************************************
 *                      wdrv_bssbridgeInit()
 ****************************************************************************
 * DESCRIPTION:	Bssbridge module init function	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
STATUS wdrv_bssbridgeInit(void)
{
	wdrv_dbGetPrivacyExcludeUnencrypted(&bssBridgeExcludeUnencrypted);
	bssBridgeExcludedCount = 0;
	bssBridgeTkipMicFailureCount = 0;

	wdrv_mainsmSendModuleUp(WDRV_BSS_BRIDGE_MODULE_ID);

	return OK;
}


/****************************************************************************
 *                      copyMsdu()
 ****************************************************************************
 * DESCRIPTION:
 *
 * INPUTS:
 *
 *
 *
 * OUTPUT:	None
 *
 * RETURNS:	OK or NOK
 ****************************************************************************/
STATUS copyMsdu(mem_MSDU_T *pDstMsdu, mem_MSDU_T *pSrcMsdu )
{
    mem_BD_T* srcBdPtr;
    mem_BD_T* dstBdPtr;
    mem_BD_T* tmpDstBdPtr;

   srcBdPtr = pSrcMsdu->firstBDPtr;
   dstBdPtr = pDstMsdu->firstBDPtr;

   dstBdPtr->length =  srcBdPtr->length;
   dstBdPtr->dataOffset = srcBdPtr->dataOffset;
   PAL_memCopy( dstBdPtr->data +  dstBdPtr->dataOffset,
           srcBdPtr->data + srcBdPtr->dataOffset,
           srcBdPtr->length );
  
   pDstMsdu->dataLen =  srcBdPtr->length;

   while( pDstMsdu->dataLen <  pSrcMsdu->dataLen ){

       srcBdPtr = srcBdPtr->nextBDPtr;
       
       if( wlan_memMngrAllocDataBuf(&tmpDstBdPtr, DATA_BUF_LEN) == NOK){
			wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION,
							"WDRV_DB: BSS bridge, copyMsdu - fail to allocate payloadBdPtr\n");
            return NOK;
       }
       dstBdPtr->nextBDPtr = tmpDstBdPtr;
       dstBdPtr = tmpDstBdPtr;
    
       dstBdPtr->length =  srcBdPtr->length;
       dstBdPtr->dataOffset = srcBdPtr->dataOffset;
	    PAL_memCopy( dstBdPtr->data +  dstBdPtr->dataOffset,
          srcBdPtr->data + srcBdPtr->dataOffset,
          srcBdPtr->length );
       
       pDstMsdu->dataLen +=  srcBdPtr->length;
   }


   pDstMsdu->dataLen = pSrcMsdu->dataLen;
   return OK;
}





/****************************************************************************
 *                      wdrv_bssbridgeInit()
 ****************************************************************************
 * DESCRIPTION:	Bssbridge module init function
 *
 * INPUTS:
 *
 *
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void wdrv_bssBridgeFromWlanFrame(mem_MSDU_T *pMsdu,UINT32 pktEmulate, BOOL micOK, UINT16 srcStaIndex)
{
	typedef enum 
	{
	  WLAN = 0x1,
	  END  = 0x2,
	} dest_T;

	mem_MSDU_T *pBrctMsdu;
	UINT8 stationCipher = CIPHER_NONE;
	BOOL	srcAuthorized = FALSE, wepOn = FALSE, frameIsEapolToAp = FALSE;
	UINT8 *dataBuf, DA[6], apMac[6], SA[6];
	register dot11_header_T  *pDot11Hdr;
	UINT16 staIndex;
	dest_T DEST = 0;
	ULONG txFlags = WDRV_TX_LOW_PRIORITY;
    UINT8 wone_index, totalWones;
    mem_MSDU_T *pCopyMsdu = NULL;
	BOOL		bssBridgeIsolationOn;

	bm_trace(2,0,0);

	dataBuf = memMgr_BufData(pMsdu->firstBDPtr);
	pDot11Hdr = (dot11_header_T*) dataBuf;

#ifdef BSS_PERF_DBG
   MACDUP2( DA, (UINT8*)&pDot11Hdr->address3);
   MACDUP2( SA, (UINT8*)&pDot11Hdr->address2);
#else
	PAL_memCopy( DA, (UINT8*)&pDot11Hdr->address3, 6 );
	PAL_memCopy( SA, (UINT8*)&pDot11Hdr->address2, 6 );
#endif
	wdrv_dbGetBssid(apMac);

#ifdef BSS_PERF_DBG
	if ( (isEapol(pMsdu)) && MACCMP(apMac, DA))
#else
	if ((isEapol(pMsdu)) && (PAL_memCmp(apMac, DA, MAC_ADDR_LEN) == 0))
#endif
		frameIsEapolToAp = TRUE;

#if 0
	/* First we have to notify the WPA module if it the first data frame from this station. */
	/* so we have to find out whether it is a data or EAPOL */
    if( wdrv_dbFindStaIndex(SA, &staIndex) == OK )
	{
		if (wdrv_dbIsDataFirstFromStation(staIndex))
			dda_wpa_first_dataFromStation(staIndex, frameIsEapolToAp);
	}
	else
	{
		if(REPORT_LEVEL(RL_WPA)) 
			wdrv_utilsReport(RL_WPA, PRI_INFORMATION, "WDRV_DB: BSS bridge, Error, frame received from WLAN but has no index\n");
		wlan_memMngrFreeMSDU(pMsdu->handle);
		return;
	}
#else
    if (wdrv_dbIsDataFirstFromStation(srcStaIndex))
        dda_wpa_first_dataFromStation(srcStaIndex, frameIsEapolToAp);
#endif


    /* At this point the WPA task may delete the station from the table. */  	
    if( wdrv_dbFindStaIndex(SA, &staIndex) != OK )
	{
		if(REPORT_LEVEL(RL_WPA)) 
			wdrv_utilsReport(RL_WPA, PRI_INFORMATION, "WDRV_DB: BSS bridge, station removed from database becasue of first data\n");
		wlan_memMngrFreeMSDU(pMsdu->handle);
		return;
	}

	if(REPORT_LEVEL(RL_BRIDGE)) 
		wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, SRC station %d (%02hx-%02hx-%02hx-%02hx-%02hx-%02hx) was found\n",staIndex,
						 SA[0],SA[1],SA[2],SA[3],SA[4],SA[5]);


	srcAuthorized = wdrv_mgmtIsStationAuthorized(staIndex);
	if  (((pDot11Hdr->fc) & DOT11_FC_WEP) == DOT11_FC_WEP)
		wepOn = TRUE; 
	wdrv_dbGetStationSuiteType(staIndex, &stationCipher);
	
	if (srcAuthorized == FALSE)
	{
		/* Drop all frames with WEP ON received from non-authorized station. */
		if (wepOn == TRUE)
		{
			wlan_memMngrFreeMSDU(pMsdu->handle);
			return;
		}
		
		else	/* WEP is off, than except only EAPOL frames. */
		{
			if (frameIsEapolToAp == TRUE)
			{
				dda_wpa_notify_eapolReceived(pMsdu, FALSE);
				return;
			}			
			else		/* Not EAPOL */
			{
				wlan_memMngrFreeMSDU(pMsdu->handle);
				return;
			}

		}
	} 
	if(REPORT_LEVEL(RL_BRIDGE)) 
		wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, SRC station is authorized\n");


	/* If we are here, it means that the station is authorized. */
	
	/* Exclude if needed any frames with WEP OFF, but forward EAPOLS */
	if (wepOn == FALSE)
	{
		if (frameIsEapolToAp == TRUE)		/* EAPOL */
		{
			dda_wpa_notify_eapolReceived(pMsdu, FALSE);
			return;
		}
		else		/* Not EAPOL */
		{
			if (bssBridgeExcludeUnencrypted == TRUE)
			{
				wlan_memMngrFreeMSDU(pMsdu->handle);
				bssBridgeExcludedCount++;
				return;
			}
		}
	}
	
	else	/* WEP is ON */
	{
		if (stationCipher == CIPHER_TKIP)		/* if TKIP, check MIC */
		{

			if (micOK == FALSE)			/* MIC Failure */
			{
				dda_wpa_SetStationSecAttack(staIndex, TRUE);		/* Set the security attack flag for this station */
				dda_wpa_rxMicFailure(staIndex);
				bssBridgeTkipMicFailureCount++;
				wlan_memMngrFreeMSDU(pMsdu->handle);
				return;
			}

		}
		
		/* Now if it is a EAPOL we have to forward it to WPA */
		if (frameIsEapolToAp)
		{
			dda_wpa_notify_eapolReceived(pMsdu, TRUE);
			return;
		}
	}
	
	
	/*--------------------------------------------------*/
	/*             Destination determination            */
	/*--------------------------------------------------*/
   
	wdrv_dbGetBssIsolationState(&bssBridgeIsolationOn);
	if( DA[0] & 1 ) 
	{ 
      /* broadcast or multicast, bit40 of MAC address is set */
		/* we allow forwarding it to the WLAN only if the broadcst MAC is authorized. */    
		DEST = END;

		if (wdrv_mgmtIsBroadcastAuthorized() && !bssBridgeIsolationOn)
			DEST |= WLAN;

		bssBrgStat.pktsBrcstFromWlan++;
   }
    
   else 
   {  
     /* Unicast */
    if( wdrv_dbFindStaIndex(DA, &staIndex) == OK )
	{
		if(REPORT_LEVEL(RL_BRIDGE)) 
			wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, wdrv_dbFindStaIndex(%02hx-%02hx-%02hx-%02hx-%02hx-%02hx, %d) == OK\n",
							 DA[0],DA[1],DA[2],DA[3],DA[4],DA[5],staIndex);
		 if (wdrv_mgmtIsStationAuthorized(staIndex))
		 {
			 if(REPORT_LEVEL(RL_BRIDGE)) 
				 wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, DEST = WLAN\n");
			 DEST = WLAN; 
			 bssBrgStat.pktsUnicWlanToWlan++;  
		 }
		 else
		 {
			 /* station is not authorized, but still appears in the table (e.g. authenticated).
				One case is fast roaming. Station is disassociation from this AP, but now
				associated on another AP, probably on the same LAN, maybe this AP connected
				directly with this AP.
				For that, forward the packet on the bridge..
			 */
			 if(REPORT_LEVEL(RL_BRIDGE)) 
				 wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, DEST = END\n");
			 DEST = END;
			 bssBrgStat.pktsUnicWlanToEthr++;
		 }

      }
      else 
	  { 
		  if(REPORT_LEVEL(RL_BRIDGE)) 
			  wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, DEST = END\n");
         DEST = END;
         bssBrgStat.pktsUnicWlanToEthr++;
      }
   }
   
   if(DEST == WLAN && bssBridgeIsolationOn)
   {
		wlan_memMngrFreeMSDU(pMsdu->handle);
		bssBrgStat.pktsDroppedDueToIsolation++;
		return;
   }

#if 0
	if(pktEmulate == FALSE)
      rc = wdrv_ackEmulationRxPacket(pMsdu);
#endif

   if( DEST == END ){
      /* Packet destined to the END, send it and return. */
	   if(REPORT_LEVEL(RL_BRIDGE)) 
		   wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, Packet destined to the END, send it and return.\n");
   	  
	   wdrv_bssBridgeSendPktToEthernet(pMsdu);
      return;
   }

   
   if( DEST == WLAN ) { 
         txFlags |= WDRV_TX_UNICAST_MSDU;
   }

   else /*DEST = WLAN | END */ 
   {
	  if(REPORT_LEVEL(RL_BRIDGE)) 
		  wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "BSS bridge, DEST = WLAN | END.\n");

      if( wlan_memMngrAllocMSDU ( &pBrctMsdu, pMsdu->dataLen ) == OK )
      {
         if(copyMsdu( pBrctMsdu, pMsdu ) == OK)
			 wdrv_bssBridgeSendPktToEthernet(pMsdu);
      }
      else
      {
          if (REPORT_LEVEL(RL_MEM))
              wdrv_utilsReport(RL_MEM, PRI_INFORMATION,"failed in wlan_memMngrAllocMSDU");
		  wlan_memMngrFreeMSDU(pMsdu->handle);
		  return;
      }

      txFlags |= WDRV_TX_BRCST_MTCST_MSDU;   
   }

   if(!(txFlags & WDRV_TX_BRCST_MTCST_MSDU))
   {
	   pBrctMsdu = pMsdu;
   }
   else
   {
	   dataBuf = memMgr_BufData(pBrctMsdu->firstBDPtr);
       pDot11Hdr = (dot11_header_T*) dataBuf;
   }

   /*--------------------------------------------------*/
   /* Sending the original buffer to the WLAN          */
   /*--------------------------------------------------*/
   pDot11Hdr->fc &= ~( DOT11_FC_TO_DS | DOT11_FC_MORE_FRAG );
   pDot11Hdr->fc |= DOT11_FC_FROM_DS;

#ifdef BSS_PERF_DBG
   MACDUP2( (void*)&pDot11Hdr->address3, (void*)&pDot11Hdr->address2);
   MACDUP2( (void*)&pDot11Hdr->address2, (void*)&pDot11Hdr->address1);
   MACDUP2( (void*)&pDot11Hdr->address1, DA);
#else
   PAL_memCopy( (void*)&pDot11Hdr->address3, (void*)&pDot11Hdr->address2, 6 );
   PAL_memCopy( (void*)&pDot11Hdr->address2, (void*)&pDot11Hdr->address1, 6 );
   PAL_memCopy( (void*)&pDot11Hdr->address1, DA, 6);
#endif

   pBrctMsdu->lastBDPtr = pBrctMsdu->firstBDPtr;

   if (txFlags & WDRV_TX_BRCST_MTCST_MSDU)
   {
      wdrv_dbGetNumSupWone(&totalWones);
      if(totalWones == WONE_DUAL_BAND)
      {
         /* duplicate broadcast packet for second wone */

            if ( wlan_memMngrAllocMSDU(&pCopyMsdu, memMgr_MsduDataSize(pBrctMsdu)) == OK )
            {
                wlan_memMngrCopyMsdu( pCopyMsdu, pBrctMsdu);
                wdrv_txSendMsdu( pCopyMsdu, 0, txFlags,SECOND_WONE_NET);
            } 
            else
            {
                if (REPORT_LEVEL(RL_MEM))
                    wdrv_utilsReport(RL_MEM, PRI_INFORMATION,"1 fail to wlan_memMngrAllocMSDU");
            }
      }
      wone_index = FIRST_WONE_NET;
   
   } 
   else
   {
      wdrv_dbGetStationWoneIndex(staIndex, &wone_index);
   }

   wdrv_txSendMsdu( pBrctMsdu, staIndex, txFlags,wone_index);
}



/****************************************************************************
 *                      wdrv_bssBridgeSendPktToEthernet()                   *
 ****************************************************************************
 * DESCRIPTION:	Send the WLAN packet to the END using Ethernet header.	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void wdrv_bssBridgeSendPktToEthernet(mem_MSDU_T *pMsdu)
{
   
   EthernetHeader_T EthHeader;
   Wlan_LlcHeader_T  *pWlanSnapHeader;   
   UINT8 			*dataBuf;
   register dot11_header_T  	*pdot11Header;
   UINT32   		lengthDelta;
   UINT8          createEtherIIHeader;


   dataBuf = memMgr_BufData(pMsdu->firstBDPtr);
   pdot11Header = (dot11_header_T*) dataBuf;
   pWlanSnapHeader = (Wlan_LlcHeader_T*)((ULONG)dataBuf + (ULONG)WLAN_HDR_LEN);


   /* See if the LLC header in the frame shows the SAP SNAP... */
   if ( ( SNAP_CHANNEL_ID == pWlanSnapHeader->DSAP ) &&
        ( SNAP_CHANNEL_ID == pWlanSnapHeader->SSAP ) &&
        ( LLC_CONTROL_UNNUMBERED_INFORMATION ==  pWlanSnapHeader->Control ) ) 
   {

      /* Check for the Bridge Tunnel OUI in the SNAP Header... */
      if((SNAP_OUI_802_1H_BYTE0 == pWlanSnapHeader->OUI[ 0 ]) &&
         (SNAP_OUI_802_1H_BYTE1 == pWlanSnapHeader->OUI[ 1 ]) &&
         (SNAP_OUI_802_1H_BYTE2 == pWlanSnapHeader->OUI[ 2 ])) {
         /* Strip the SNAP header by skipping over it.                  */
         /* Start moving data from the Ethertype field in the SNAP      */
         /* header.  Move to the TypeLength field in the 802.3 header.  */
         createEtherIIHeader = TRUE;        
         }

      /* Check for the RFC 1042 OUI in the SNAP Header   */
      else {
           /* See if the Ethertype is in our selective translation table  */
           /* (Appletalk AARP and DIX II IPX are the two protocols in     */
           /* our 'table')                                                */
            if (( ETHERTYPE_APPLE_AARP == htons(pWlanSnapHeader->Type) ) ||
                ( ETHERTYPE_DIX_II_IPX == htons(pWlanSnapHeader->Type) )) {
               /* Strip the SNAP header by skipping over it. */
                   createEtherIIHeader = FALSE;
            }
            /* All the rest SNAP types are transformed to EthernetII format */
            else {
               createEtherIIHeader = TRUE;
            }
         }
      }

   else { 
         /* Non-SNAP packets strip out the WLAN header and create the dst,src,len header. */
      createEtherIIHeader = FALSE;
   }
      
   
   /* Prepare the Ethernet header. */
#ifdef BSS_PERF_DBG
   MACDUP2( &EthHeader.DstAddr, &pdot11Header->address3);
   MACDUP2( &EthHeader.SrcAddr, &pdot11Header->address2);
#else
   CopyMacAddress(&EthHeader.DstAddr, &pdot11Header->address3);
   CopyMacAddress(&EthHeader.SrcAddr, &pdot11Header->address2);
#endif
   
   if( createEtherIIHeader == TRUE ){
      /* The LEN/TYPE bytes are set to TYPE, the entire WLAN+SNAP is removed.*/
      lengthDelta = WLAN_HDR_LEN + WLAN_SNAP_HDR_LEN - ETHERNET_HDR_LEN;
      EthHeader.TypeLength = pWlanSnapHeader->Type;
   }
   else {
      /* The LEN/TYPE bytes are set to frame LEN, only the WLAN header is removed, */
      /* the entire 802.3 or 802.2 header is not removed.*/
      EthHeader.TypeLength = htons(pMsdu->dataLen - WLAN_HDR_LEN);
      lengthDelta = WLAN_HDR_LEN - ETHERNET_HDR_LEN;
   }

   /* Replace the 802.11 header and the LLC with Ethernet packet. */
   dataBuf += lengthDelta;
   PAL_memCopy( dataBuf, (void*)&EthHeader, ETHERNET_HDR_LEN );

   pMsdu->firstBDPtr->dataOffset = lengthDelta;
   pMsdu->dataLen -= lengthDelta;
   pMsdu->firstBDPtr->length -= lengthDelta;

   if (REPORT_LEVEL(RL_BRIDGE)) {
   		wdrv_utilsReport(RL_BRIDGE, PRI_INFORMATION, "w->e :: msdu->len=%d :: msdu->firstBD->len=%d",
   			pMsdu->dataLen, pMsdu->firstBDPtr->length );
      }
   	 
   /* call DDA */
   tiwlan_ddaReceive(pMsdu);
}



/****************************************************************************
 *                      wdrv_bssBridgePktFromEthrToWlan()                   *
 ****************************************************************************
 * DESCRIPTION:	Bssbridge module init function	
 * 
 * INPUTS:
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
STATUS wdrv_bssBridgePktFromEthrToWlan(mem_MSDU_T *pMsdu)
{
   EthernetHeader_T   *pEthHeader;
   dot11_header_T     *pdot11Header;
   Wlan_LlcHeader_T  *pWlanSnapHeader;  
   UINT8 BSSID[6];
   UINT16 staIndex;
   ULONG txFlags = WDRV_TX_LOW_PRIORITY | WDRV_TX_FROM_ETHER;
   UINT16 lenType = 0;
   UINT8 createSNAPHeader;
   UINT8 bytesToskeep;
   mem_BD_T	   *BDptr;
   BOOL			WlanState;
   UINT8 wone_index, totalWones;
   mem_MSDU_T *pCopyMsdu = NULL;

   wdrv_dbGetWlanServiceState( &WlanState );

   /* The data packet is placed in the second buffer in order to keep place for 
      for the WLAN + SNAP header.
   */
   pEthHeader = (EthernetHeader_T*)memMgr_BufData(pMsdu->firstBDPtr->nextBDPtr);

   BDptr=pMsdu->firstBDPtr;
   while(BDptr->nextBDPtr)
		BDptr = BDptr->nextBDPtr;
   pMsdu->lastBDPtr = BDptr;

	if ( !WlanState )
	{
		wlan_memMngrFreeMSDU(pMsdu->handle);
		return OK;
	}

   if( (*(char*)pEthHeader) & 0x01 )
   {
   		if (wdrv_mgmtIsBroadcastAuthorized() == FALSE)
		{
			 wlan_memMngrFreeMSDU(pMsdu->handle);
			 return OK;
		}

		if (wdrv_dbGetNumStations() == 0)
		{
			wlan_memMngrFreeMSDU(pMsdu->handle);
			return OK;
		}

		txFlags |= WDRV_TX_BRCST_MTCST_MSDU;
		bssBrgStat.pktsBrcstFromEthr++;
   }
   
   else
   {
      txFlags |= WDRV_TX_UNICAST_MSDU;
      if( wdrv_dbFindStaIndex((char*)&pEthHeader->DstAddr, &staIndex) != OK )
	  {
         wlan_memMngrFreeMSDU(pMsdu->handle);
         bssBrgStat.pktsFromEthrToWlanErrStaNotFound++;
         return OK;
	  }
	  if (wdrv_mgmtIsStationAuthorized(staIndex) == FALSE)
	  {
         wlan_memMngrFreeMSDU(pMsdu->handle);
         return OK;
      }
      bssBrgStat.pktsUnicEthrToWlan++;
   }

   /*--------------------------------------------------------------*/
   /*--------------------------------------------------------------*/
   /* Phase 1: Create the WLAN Header in the head of the first BD. */
   /*--------------------------------------------------------------*/
   /*--------------------------------------------------------------*/
   /* The WLAN header will be created in the first BS */
   pdot11Header = (dot11_header_T*)memMgr_BufData(pMsdu->firstBDPtr);
   
   /* The first BD contains the WLAN and SNAP headers only. */
   pMsdu->firstBDPtr->length = WLAN_HDR_LEN;
   pMsdu->firstBDPtr->dataOffset = 0;
   pMsdu->dataLen += WLAN_HDR_LEN;

   pMsdu->headerLen = WLAN_HDR_LEN;
   pdot11Header->fc = DOT11_FC_DATA | DOT11_FC_FROM_DS;

#ifdef BSS_PERF_DBG
   MACDUP2((void*)&pdot11Header->address1, (void*)&pEthHeader->DstAddr);
   wdrv_dbGetBssid(BSSID);
   MACDUP2((void*)&pdot11Header->address2, (void*)BSSID);
   MACDUP2((void*)&pdot11Header->address3, (void*)&pEthHeader->SrcAddr);
#else
   CopyMacAddress((void*)&pdot11Header->address1,(void*)&pEthHeader->DstAddr);
   wdrv_dbGetBssid(BSSID);
   CopyMacAddress((void*)&pdot11Header->address2, (void*)BSSID);
   CopyMacAddress((void*)&pdot11Header->address3,(void*)&pEthHeader->SrcAddr);
#endif

   /*--------------------------------------------------------------*/
   /*--------------------------------------------------------------*/
   /* Phase 2: Detect the packet type and decide if to create a    */
   /*          new SNAP or leave the original LLC.                 */
   /*--------------------------------------------------------------*/
   /*--------------------------------------------------------------*/
   if ( htons(pEthHeader->TypeLength) > 1500 ){
      /*----------------------------------------------------------------*/
      /*                    Ethernet Packet                             */
      /*----------------------------------------------------------------*/
      /* Need to cut the Ethernet header off and create SNAP in the     */
      /* first BD. (just remember the type field.)                      */
      /*----------------------------------------------------------------*/
      bytesToskeep = ETHERNET_HDR_LEN;
      // Move the Ethertype into the SNAP header
      lenType = pEthHeader->TypeLength;
      createSNAPHeader = TRUE;
   }
   
   else if(( *(unsigned char*)((ULONG)pEthHeader+14) == 0xFF )&&( *(unsigned char*)((ULONG)pEthHeader+15) == 0xFF)) {
      /*----------------------------------------------------------------*/
      /*                    RAW 802.3  Packet                           */
      /*----------------------------------------------------------------*/
      /* Just jump over this header.                                    */
      /*----------------------------------------------------------------*/
      bytesToskeep = IEEE802_3_HDR_LEN;
      createSNAPHeader = FALSE; 
   }
   else{
      /*----------------------------------------------------------------*/
      /*                   802.2 Packet (RAW or SNAP)                    */
      /*----------------------------------------------------------------*/
      /* Cut off the first 14 bytes, the LLC will be glued to the WLAN  */
      /* header.                                                        */
      /*----------------------------------------------------------------*/
      bytesToskeep = ETHERNET_HDR_LEN;
      createSNAPHeader = FALSE;
   }

   /* In the second BD we skeep header bytes accourding to the frame type */
   pMsdu->dataLen -= bytesToskeep;
   pMsdu->firstBDPtr->nextBDPtr->length -= bytesToskeep;
   pMsdu->firstBDPtr->nextBDPtr->dataOffset = bytesToskeep;


   /*--------------------------------------------------------------*/
   /*--------------------------------------------------------------*/
   /* Phase 3: If we need to create SNAP header, it is created in  */
   /*          the first BD, after the WLAN header.                */
   /*--------------------------------------------------------------*/
   /*--------------------------------------------------------------*/   
   if ( createSNAPHeader == TRUE ){
      /*--------------------------------------------------------*/
      /* Create the SNAP Header                                 */
      /*--------------------------------------------------------*/
      pMsdu->firstBDPtr->length += WLAN_SNAP_HDR_LEN;
      pMsdu->dataLen += WLAN_SNAP_HDR_LEN;
      pWlanSnapHeader = (Wlan_LlcHeader_T*)((ULONG)pdot11Header+WLAN_HDR_LEN);
      pWlanSnapHeader->DSAP = SNAP_CHANNEL_ID;
      pWlanSnapHeader->SSAP = SNAP_CHANNEL_ID;
      pWlanSnapHeader->Control = LLC_CONTROL_UNNUMBERED_INFORMATION;
      
      /* Check to see if the Ethertype matches anything in the translation     */
      /* table (Appletalk AARP or DixII/IPX).  If so, add the 802.1h           */
      /* SNAP.                                                                 */
      if ( ( ETHERTYPE_APPLE_AARP == htons(lenType) ) ||
           ( ETHERTYPE_DIX_II_IPX == htons(lenType) )    ) {
        
        /* Fill out the SNAP Header with 802.1H extention   */
#ifdef BSS_PERF_DBG
         UST1( &pWlanSnapHeader->OUI, 0) = UST1( SNAP_OUI_802_1H, 0);
         UST1( &pWlanSnapHeader->OUI, 1) = UST1( SNAP_OUI_802_1H, 1);
         UST1( &pWlanSnapHeader->OUI, 2) = UST1( SNAP_OUI_802_1H, 2);
#else
        PAL_memCopy( &pWlanSnapHeader->OUI, SNAP_OUI_802_1H, 
                        sizeof( pWlanSnapHeader->OUI ) );
#endif
      }
      else {
#ifdef BSS_PERF_DBG
         UST1( &pWlanSnapHeader->OUI, 0) = UST1( SNAP_OUI_RFC1042, 0);
         UST1( &pWlanSnapHeader->OUI, 1) = UST1( SNAP_OUI_RFC1042, 1);
         UST1( &pWlanSnapHeader->OUI, 2) = UST1( SNAP_OUI_RFC1042, 2);
#else
         /* otherwise, add the RFC1042 SNAP   */
         PAL_memCopy( &pWlanSnapHeader->OUI, SNAP_OUI_RFC1042, 
                    sizeof( pWlanSnapHeader->OUI) );
#endif
      }

      pWlanSnapHeader->Type = lenType;
   }

   
   if (txFlags & WDRV_TX_BRCST_MTCST_MSDU)
   {
      wdrv_dbGetNumSupWone(&totalWones);
      if(totalWones == WONE_DUAL_BAND)
      {
         /* duplicate broadcast packet for second wone */

         if ( wlan_memMngrAllocMSDU(&pCopyMsdu, memMgr_MsduDataSize(pMsdu)) == OK )
         {
            wlan_memMngrCopyMsdu( pCopyMsdu, pMsdu);
            wdrv_txSendMsdu( pCopyMsdu, 0, txFlags,SECOND_WONE_NET);
         }
         else
         {
             if (REPORT_LEVEL(RL_MEM))
                 wdrv_utilsReport(RL_MEM, PRI_INFORMATION,"fail to wlan_memMngrAllocMSDU");
         }
      }
      wone_index = FIRST_WONE_NET;
   } 
   else
   {
      wdrv_dbGetStationWoneIndex(staIndex, &wone_index);

   }
      
   wdrv_txSendMsdu( pMsdu, staIndex, txFlags ,wone_index);
   return OK;
}


/****************************************************************************
 *                      wdrv_bssBridgeShowStatistics()                      *
 ****************************************************************************
 * DESCRIPTION:
 * 
 * INPUTS:
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/

int wdrv_bssBridgeShowStatistics( char *buf ){
   int len = 0;

   len += sprintf(buf+len,"------------------------- Bss Bridge Statistics ---------------------\n");
   len += sprintf(buf+len,"Unicast:  Wlan->Wlan     : %8d\n", bssBrgStat.pktsUnicWlanToWlan );
   len += sprintf(buf+len,"          Wlan->Ethernet : %8d\n", bssBrgStat.pktsUnicWlanToEthr );         
   len += sprintf(buf+len,"          Ethrnet->Wlan  : %8d\n", bssBrgStat.pktsUnicEthrToWlan );
   len += sprintf(buf+len,"\n");
   len += sprintf(buf+len,"Broacast: From Wlan      : %8d\n", bssBrgStat.pktsBrcstFromWlan  );
   len += sprintf(buf+len,"          From Ethernet  : %8d\n", bssBrgStat.pktsBrcstFromEthr  );
   len += sprintf(buf+len,"\n");
   len += sprintf(buf+len,"Error:    Unouwn Station : %8d\n", bssBrgStat.pktsFromEthrToWlanErrStaNotFound );
   len += sprintf(buf+len,"BSS Isolation dropped    : %8d\n", bssBrgStat.pktsDroppedDueToIsolation);    
   len += sprintf(buf+len,"---------------------------------------------------------------------\n");                                              
	
   return len;
}

void wdrv_bssBridgeGetLanSendUnicast(UINT32 *LanSendUnicast)
{
	*LanSendUnicast = bssBrgStat.pktsUnicWlanToEthr;
}

void wdrv_bssBridgeGetLanSendMulticast(UINT32 *LanSendMulticast)
{
	*LanSendMulticast = bssBrgStat.pktsBrcstFromWlan;
}

void wdrv_bssBridgeGetLanRecvUnicast(UINT32 *LanRecvUnicast)
{
	*LanRecvUnicast = bssBrgStat.pktsUnicEthrToWlan;
}

void wdrv_bssBridgeGetLanRecvMulticast(UINT32 *LanRecvMulticast)
{
	*LanRecvMulticast = bssBrgStat.pktsBrcstFromEthr;
}

/* WPA support section */
static BOOL isEapol(mem_MSDU_T *pMsdu)
{
	Wlan_LlcHeader_T	*pWlanSnapHeader;   
	UINT8 				*dataBuf;

	dataBuf = memMgr_BufData(pMsdu->firstBDPtr);
	pWlanSnapHeader = (Wlan_LlcHeader_T*)((ULONG)dataBuf + (ULONG)WLAN_HDR_LEN);

	/* See if the frame is EAPOL... */
	if (( SNAP_CHANNEL_ID == pWlanSnapHeader->DSAP ) &&
		( SNAP_CHANNEL_ID == pWlanSnapHeader->SSAP ) &&
		( LLC_CONTROL_UNNUMBERED_INFORMATION ==  pWlanSnapHeader->Control ) &&
		(SNAP_OUI_RFC1042_BYTE0 == pWlanSnapHeader->OUI[ 0 ]) &&
		(SNAP_OUI_RFC1042_BYTE1 == pWlanSnapHeader->OUI[ 1 ]) &&
		(SNAP_OUI_RFC1042_BYTE2 == pWlanSnapHeader->OUI[ 2 ]) &&             
		(EAPOL_ETHER_TYPE == htons(pWlanSnapHeader->Type)))
		return TRUE;

	return FALSE;
}


UINT32 wdrv_bssbridgeGetExcludedCount(void)
{
	return bssBridgeExcludedCount;		
}

void wdrv_bssbridgeSetExcludeUnencrypted(BOOL value)
{
	bssBridgeExcludeUnencrypted = value;
}

UINT32 wdrv_bssbridgeGetTKIPMicFailure(void)
{
	return bssBridgeTkipMicFailureCount;		
}
