/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2003 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*		MODULE:		drvDbDefs.h											   */
/*		PURPOSE:	All db typesd defs									   */
/*																	 	   */
/***************************************************************************/

#ifndef _DB_DEFS_H
#define _DB_DEFS_H



/*****************************
@@	incldues 
******************************/
#include "802_11Defs.h"
#ifdef __cplusplus
extern "C" {
#endif



/************************
@@	defines 
************************/

/*  init db ERROR codes */
#define INIT_RET_CODE		int
#define DDC_INIT_SUCCESS			 0
#define DDC_INIT_CFG_RANGE_ERROR    -1
#define DDC_INIT_CFG_LOGICAL_ERROR  -2
#define DDC_INIT_RESOURCE_ERROR     -3
#define DDC_INIT_HW_INIT_ERROR      -4


#define MAX_VHCF_CLASSIFIERS   2
#define NUM_OF_TX_POWER_LEVELS  5
#define COUNTRY_STRING_LENGTH   3
#define MAX_NUM_OF_RANGES_A     8
#define MAX_NUM_OF_RANGES_B     4
#define MGMT_ACCESS_LIST_SIZE   64
#define NUM_MULTIPLE_SSID                 16
#define WHAL_SSID_STRING_LEN              32
#define MAX_AUTH_ALGORITHM_SUPPORTED      2
#define DEF_DYN_ALG_TIMEOUT			100 /* msec */
#define DOT11_WEP_KEY_MAPPING_LEN         10
#define DEF_WEP_KEY_MAPPING_LEN     10

#define MAX_WONE_NET 2

#define MAX_STATIONS 64
	
/*********************************
@@	types/unums/structs
**********************************/


typedef enum {
    WLAN_BAND_B,
    WLAN_BAND_A
}wlan_bandType;


#pragma pack(1)
   typedef struct
   {
      UINT16 authAlgo;
      BOOL enable;
   }tAUTH_ALGO_ENTRY;
#pragma pack()


#pragma pack(1)
   typedef struct
   {
      UINT8         startCh;            /* start channel */
      UINT8         stopCh;             /* stop channel  */
      UINT8	        stepOpt;            /* Default step value "a"=4, "b"=1 */
      INT8          powerLimit;         /* Power limit in dBm (default = 200 dBm) */
   }tChannelRangeElementStruct;
#pragma pack()
	
typedef enum {
	MGMT_ACCESS_WHITE_LIST, 
	MGMT_ACCESS_BLACK_LIST
} mgmt_access_list_type_t;

typedef struct    
{				  
    UINT8		  acID; /* Use AccessCategory_e values. This tells us which EDCFCfg is 
	                       being read or written from the NUM_ACCESS_CATEGORIES available */
    UINT8		  trafficType;  /* Use TrafficAccessType_e values */
    UINT8		  aifsn;    /* slots number interval to start tx. */
    UINT8		  cwMin;
    UINT16		  cwMax;
    UINT16		  txopLimit;
    UINT32		  dot11EDCATableMSDULifetime;  /* For future use, default - 512 TUs. */
    UINT16        rxTimeout;    /* For future use, default - 0 (no timeout) */
    UINT8         ackPolicy;    /* For future use, Use ACKPolicy_e values */
    UINT8         shortRetryLimit;   /* For future use, default - 7 */
    UINT8         longRetryLimit;    /* For future use, default - 4 */
    BOOL	      retryPremption;    /* For future use. */
    BOOL          txopContinuation;
}db_edcf_info_t;
   

/*================
  vHCF Parameters 
=================*/
#pragma pack(1)
typedef enum
{
	VHCF_PROT_NONE = 0,
	VHCF_PROT_TCP  = 1,
	VHCF_PROT_UDP  = 2
}VhcfProtocol_e;

typedef struct
{
	VhcfProtocol_e	Protocol;
	UINT16	DestPort;
	UINT32	DestIp;
}tVHCF_CASSIFIER;

typedef struct
{
  BOOL		vhcfEnable;
  tVHCF_CASSIFIER  vhcfClassifier[MAX_VHCF_CLASSIFIERS];
}tVHCF_PARAMS;
#pragma pack()

/* wone config struct */
typedef struct 
{
    dot11_supRates_T       apRates; 
    dot11_supRates_T       apAddRates;
    dot11_extSupRates_T    apOFDMRates;
    UINT8                  channel;
    UINT8                  band;   
    UINT8                  UsageTime;        
    UINT8                  PreambleMode;
    UINT8                  PbccAlgorithmMode;
    UINT8                  RateAdaptation;
    UINT8                  CwMinMode;
    UINT8                  SlotTimeMode;
    UINT8                  ERPProtectionMode;
    UINT8                  ERPProtectionType;
    UINT8                  ERPProtectionTI;
    UINT8                  Dot11Mode; 
} wone_config_T;

/* reg domain init struct */
typedef struct {
	UINT8     dot11MultiDomainCapabilityEnable;
	UINT8     dot11SpectrumManagementRequired;
	UINT8     whalEnableLegacyStations;
	UINT8     dot11CurrentRegDomain;
	INT8     dot11LocalPowerConstraint;
	UINT8     dot11PrivateRegulatoryDomain;
	UINT8     whalMaxTxPowerLevel;

	UINT8     txPowerLevels_a[NUM_OF_TX_POWER_LEVELS];      
	INT8      txPowerLevels_g[NUM_OF_TX_POWER_LEVELS]; 

	INT8      dot11CountryString[COUNTRY_STRING_LENGTH];
	
	INT8      privateRegDomain;

	UINT8     numOfRanges_802_11_a;
	tChannelRangeElementStruct privateRangeElements_a[MAX_NUM_OF_RANGES_A];
	UINT8     numOfRanges_802_11_b;
	tChannelRangeElementStruct privateRangeElements_b[MAX_NUM_OF_RANGES_B];
	
} RegDomainInitStrcut;

/* access list init struct*/
typedef struct {
	UINT8 mgmt_access_enabled;
	char  mgmt_access_list[MGMT_ACCESS_LIST_SIZE][MAC_ADDR_LEN];
	UINT8 mgmt_access_list_type;
}mgmt_accesslist_init_struct;

typedef struct RadioParams_t {
   UINT32 overrideBitmask;
   /* power table */
   UINT32 powerTable_paramType;
   INT8   powerTable_param[20];
   /* add more prameters above this line */
} RadioParams_t;

typedef enum
{
	BSS_TRAFFIC_BLOCK,
	BSS_TRAFFIC_EN,
	BSS_TRAFFIC_DROP
} bssIsolationState_t;

/* Interference WA state machine configuration*/
typedef struct {
	UINT32 aciStateTimeout;
	UINT32 nbStateTimeout;
	UINT32 aciRateThreshold;
	UINT32 aciEventThreshold;
	UINT32 mediumBusyThresh;
	UINT16 pdUnderAciLowThrsh;
	UINT16 pdUnderAciHighThrsh;
}InterferenceWAparams_t;

/* global params init struct*/
typedef struct {

	char   APMAC[MAC_ADDR_LEN];
	UINT16 apHiddenSSID;
    UINT16 apMultipleSsidFlag;
    UINT8  apMultipleSsidTable[NUM_MULTIPLE_SSID][WHAL_SSID_STRING_LEN];                  
	INT8   dot11SsID[WHAL_SSID_STRING_LEN];
	UINT8  dot11SsIDLength;
	BOOL   enable4x;
	BOOL   enableConcat;

	UINT32 bssBridgeState;
	
	/*reg domain init strcut */
	RegDomainInitStrcut RegInit;

	mgmt_accesslist_init_struct mgmtAccessList;
	/* wone */
	UINT8   usageTimeMode;
    UINT8   numSupWone;	
	wone_config_T  wone_info[MAX_WONE_NET];

	UINT8   FeatureSet;
	BOOL    enable4xMixMode;
	BOOL    privacyOptionImplemented;
	UINT32	wpaMode; /* bit map */

	UINT32  RadioCalibrateOption;
	UINT16  RadioCalibrateInterval;

	UINT8   enableEnergyDetect;
   

	UINT8   RateFallback;
	UINT16  apBeaconPeriod;
	UINT16  apRtsThreshold;
	UINT16  apFragThreshold;
	UINT8   apShortRetryLimit;
	UINT8   apLongRetryLimit;
	UINT32  apMaxTxLifetime;
	UINT32  apMaxRxLifetime;
	UINT8   apDtimPeriod;
	

    tAUTH_ALGO_ENTRY  dot11AuthenticationAlgorithmTable[MAX_AUTH_ALGORITHM_SUPPORTED];

	BOOL  WlanServiceState ;
	UINT8 EnRestart;
	UINT8 RxFreeMemOptionWA;
	UINT8 BeaconNotSentOptionWA;
	UINT8 TxStuckOptionWA;
	UINT8 aciOptionWA;
	UINT8 nbOptionWA;

    InterferenceWAparams_t interfWAparams;

	UINT16 ConsFcsErrOptionWA;


	RadioParams_t dbRadioParams;

	UINT8 RadarDetectMode;
	UINT8 ApBridgeMode;

    tVHCF_PARAMS vhcfParams;

} initConfigStruct;


#ifdef __cplusplus
}
#endif

#endif /* _DB_DEFS_H */
