/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*		MODULE:		drvDbHash.h											   */
/*		PURPOSE:	Hash interface functions							   */
/*																	 	   */
/***************************************************************************/

#ifndef _DB_HASH_H
#define _DB_HASH_H

#include "pform_types.h"
#include "802_11Defs.h"
#include "utils.h"
#include "drvDbMain.h"
#include "mgmtSm.h"
#include "tx.h"
#include "powerSave.h"
#include "dda_wpa.h"


#define INVALID_INDEX	0xFFFF

typedef enum
{
	ASSOCIATE = 1,
	DISASSOCIATE = 2,
} algo_action_t;

#if HASH_DEBUG
#define HASH_ENTRIES_NUM	4	/* number of hash entries */
/* return last 2 bits of the mac address */
#define HASH_FUNC(mac) (*(mac + MAC_ADDR_LEN - 1) & 0x03)
#else
#define HASH_ENTRIES_NUM	64	/* number of hash entries */
/* return last 6 bits of the mac address */
#define HASH_FUNC(mac) (*(mac + MAC_ADDR_LEN - 1) & 0x3F)
#endif

#define	MAX_PRINT_STRING		128	 

   
typedef struct
{
	BOOL concatEn;
	BOOL ackEmulEn;

} station4x_cap_T;

/* structure that holds the station partameters as received in the (re)assoc request */
typedef struct 
{
	UINT16	listenInterval;		/* station listen interval*/
	UINT16	capabilities;		/* station capabilities*/
	BOOL	fourXEnable;				/* station 4x support */
	BOOL	fourXActive;				/* station 4x support */
	BOOL	TIprotectionSupported; /* TRUE if trick packets supported, otherwise FALSE */
	BOOL	CommonFeatureSet;
	station4x_cap_T fourX_cap;
	UINT32	ConcatMaxSize;
	BOOL    TIcapEn;
	UINT8	activeRate;			/* station active rate, used for printing purposes only  */
	UINT16	activeRateAcx;		/* station active rate in ACX units, used by the TX to configure the ACX per frame. */
	UINT16  currentActiveRateWithFallback;  /* updated every transmit */
	/* Station supported rates and extended rates */
	/* from (re)association request */
	UINT8	ratesLen;
	UINT8	rates[DOT11_SUPPORTED_RATES_MAX];
	UINT8	extRatesLen;
	UINT8	extRates[DOT11_EXT_SUPPORTED_RATES_MAX];
	UINT16  bitWiseSupRates;    /* All supported rates in bit wise ACX format */
	/* The same supported rates but without */
	/* the rates that are not supported by the AP */
	UINT8	apRatesLen;
	UINT8	apRates[DOT11_SUPPORTED_RATES_MAX];
	UINT8	apExtRatesLen;
	UINT8	apExtRates[DOT11_EXT_SUPPORTED_RATES_MAX];
    UINT8	WoneIdentifierIndex;
    UINT32  rxPacketCounter;
	UINT32  txPacketCounter;

	/* WPA section */ 
	dot11_rsne_T	rsnIe;
	UINT8		cipherSuiteType;
    UINT8       encryptionKeyLen;
    UINT8       encryptionKey[WPA_MAX_WEP_KEY_LEN];
	UINT8		txMicKey[MICHAEL_KEY_LEN];
	UINT8		rxMicKey[MICHAEL_KEY_LEN];
	BOOL		dataReceivedFromStation;
	BOOL		generateTxMicFailure;
	UINT32      wpaReqReason;
} stationParams_T; 

typedef struct staHashEntry_T staHashEntry_T;
/* the following structure defines a station entry in the hash & station tables */
struct staHashEntry_T
{
	staHashEntry_T *nextSta;		    /* points to the next entry in the linked list          */
	staHashEntry_T *prevSta;		    /* points to the previous entry in the linked list      */
	BOOL	free;					    /* indicates whether the entry is free or not           */
	char	macAddr[MAC_ADDR_LEN];	    /* the station mac address                              */
    char    ssid[WHAL_SSID_STRING_LEN]; /* the station's SSID (used in multiple SSID features)  */
    int     ssidLength;                 /* the actual length of the SSID                        */
	UINT16	index;					    /* the station index                                    */
}; 


UINT8 convertAcxRateToDOT11Rate(UINT16 rateAcx);
STATUS	wdrv_dbHashInit(void);
void wdrv_dbGetStaSsidByIndex(UINT16 index, char *ssid, int *SsidLen);
STATUS	wdrv_dbFindAndInsertStaIndex(char *mac, UINT8 woneIndex, UINT16 *idx);
STATUS	wdrv_dbFindStaIndex(char *mac, UINT16 *idx);
BOOL    wdrv_dbIsTableIndexExists(UINT16 index);
void	wdrv_dbRemoveStaIndex(UINT16 idx);
void	wdrv_dbGetStaMacAddress(UINT16 index, char *macAddressPtr);
void	wdrv_dbSetStationRates(UINT16 index, dot11_supRates_T *rates, dot11_extSupRates_T *extRates, BOOL fourXSupport);
void	wdrv_dbGetActiveRateAcx(UINT16 index, UINT8 *activeRate);
BOOL    wdrv_dbIsRateSupported(UINT16 index, UINT16 rateAcx);
void	wdrv_dbGetActiveRate(UINT16 index, UINT8 *activeRate);
void	wdrv_dbGetStationAPRates(UINT16 index, char *value, UINT8 *len);
void	wdrv_dbGetStationAPExtRates(UINT16 index, char *value, UINT8 *len);
void	wdrv_dbGetStationAcxBitWiseRates(UINT16 index, UINT16 *acxBitWiseRates);
void	wdrv_dbSetCurrentActiveRateWithFallback(UINT16 index, UINT16 *currentActiveRateWithFallback);
void	wdrv_dbGetCurrentActiveRateWithFallback(UINT16 index, UINT16 *currentActiveRateWithFallback);
void	wdrv_dbSetListenInterval(UINT16 index, UINT16 *listenInterval);
void	wdrv_dbGetListenInterval(UINT16 index, UINT16 *listenInterval);
void	wdrv_dbSetStationCapabilities(UINT16 index, UINT16 *capabilities);
void	wdrv_dbGetStationCapabilities(UINT16 index, UINT16 *capabilities);
void	wdrv_dbPerformPBCCAlgorithmStatic(UINT16 index, algo_action_t action);
void	wdrv_dbPerformPreambleAlgorithmStatic(UINT16 index, algo_action_t action);
void	wdrv_dbPerformSlotTimeAlgorithmStatic(UINT16 index, algo_action_t action);
void	wdrv_dbPerformERPProtectionAlgorithmStatic(UINT16 index, algo_action_t action);
void	wdrv_dbPerformERPProtectionAlgorithmDynamic(BOOL mode);
void    wdrv_dbSetStationTIcapEnable(UINT16 index, BOOL fourXSupport);
void	wdrv_dbGetStationTIcapEnable(UINT16 index, BOOL *mode);
void	wdrv_dbSetStationFourXActive(UINT16 index, BOOL fourXActive);
void	wdrv_dbGetStationFourXActive(UINT16 index, BOOL *mode);
void wdrv_dbSetStationConcatMaxSize(UINT16 index, UINT16 size);
void wdrv_dbGetStationConcatMaxSize(UINT16 index, UINT16 *size);
void wdrv_dbSetStationConcatSupport(UINT16 index, BOOL support);
void wdrv_dbGetStationConcatSupport(UINT16 index, BOOL *mode);
void wdrv_dbSetStationAckEmulSupport(UINT16 index, BOOL support);
void wdrv_dbGetStationAckEmul(UINT16 index, BOOL *mode);
void wdrv_dbSetTIprotectionSupport(UINT16 index, BOOL *support);
void wdrv_dbGetTIprotectionSupport(UINT16 index, BOOL *support);
void wdrv_dbSetCommonFeatureSet(UINT16 index, UINT8 *set);
void wdrv_dbGetCommonFeatureSet(UINT16 index, UINT8 *set);
STATUS wdrv_dbSetStaSsidByIndex(UINT16 index, char *ssid, int SsidLen);

/* Wone */
void    wdrv_dbSetStationWoneIndex(UINT16 index, UINT8 woneIndex);
void    wdrv_dbGetStationWoneIndex(UINT16 index, UINT8 * woneIndex);

void    wdrv_dbGetStationRxPacketCounter(UINT16 index, UINT32 *unicastCounter);
void    wdrv_dbGetStationTxPacketCounter(UINT16 index, UINT32 *unicastCounter);
void    wdrv_dbResetStationRxPacketCounter(UINT16 index);
void    wdrv_dbResetStationTxPacketCounter(UINT16 index);

int 	wdrv_dbPrintFourXTable(char *buf);

BOOL wdrv_dbIsStationPBCCCapable(UINT16 index);
BOOL wdrv_dbIsStationShortSlotTimeCapable(UINT16 index);
BOOL wdrv_dbIsStationERP(UINT16 index);


/* functions called by the console */
int 	wdrv_dbPrintNumOfStations(char *buf);
int  	wdrv_dbPrintStaTable(char *buf);
int	    wdrv_dbPrintHashTable(char *buf);
int		wdrv_dbPrintRatesTable(char *buf);
int     wdrv_printRateInMbpsUnit(UINT8 rate ,char *buf);
void	wdrv_dbResetStationParams(UINT16 index);
void	wdrv_dbGetNumOfStationNotPBCCCapable(UINT16 *num);
void	wdrv_dbGetCurrentPBCCSupportMode(BOOL *support);
void	wdrv_dbGetNumOfStationNotSlotTimeCapable(UINT16 *num);
void	wdrv_dbGetCurrentSlotTimeSupport(BOOL *support);
void	wdrv_dbGetNumOfStationNotShortPreambleCapable(UINT16 *num);
void	wdrv_dbGetCurrentShortPreambleSupport(BOOL *support);
void	wdrv_dbGetCurrentERPProtectionMode(BOOL *support);
void    wdrv_dbSetCurrentERPProtectionMode(BOOL support);
void    wdrv_dbGetCurrentERPNonPresent(BOOL *value);
void    wdrv_dbSetCurrentERPNonPresent(BOOL value);
void    wdrv_dbGetCurrentShortSlotTimeSupport(BOOL *support);
void    wdrv_dbSetCurrentShortSlotTimeSupport(BOOL support);
void    wdrv_dbGetStationsNumForERP(UINT16 *nonERPStaNum, UINT16 *ERPStaNum);
void 	wdrv_dbSetCurrentPBCCSupportMode(BOOL support);
void    wdrv_dbSetPBCCStationsSupport(BOOL mode);

/* WPA section */
void	wdrv_dbPrintStaRsnIeTable(void);

void	wdrv_dbSetStationRsnIe(UINT16 index, dot11_rsne_T *rsnInfoEle);
void	wdrv_dbGetStationRsnIe(UINT16 index, dot11_rsne_T *rsnInfoEle);

void	wdrv_dbSetStationSuiteType(UINT16 index, UINT8 cipher);	
void	wdrv_dbSetStationEncryptionKey(UINT16 index, UINT16 keySize, UINT8 *keyValue);
void	wdrv_dbGetStationEncryptionKey(UINT16 index, UINT16 *keySize, UINT8 *keyValue);
void	wdrv_dbSetStationTxMicKey(UINT16 index, UINT8 *keyValue);	
void	wdrv_dbSetStationRxMicKey(UINT16 index, UINT8 *keyValue);	
void	wdrv_dbGetStationTxMicKey(UINT16 index, UINT8 *keyValue);	
void	wdrv_dbGetStationRxMicKey(UINT16 index, UINT8 *keyValue);	
BOOL	wdrv_dbIsDataFirstFromStation(UINT16 index);	
void	wdrv_dbGenerateTxMicFailure(char *mac);	
void    wdrv_dbGetStaSsidByIndex(UINT16 index, char *ssid, int *SsidLen);

inline void    wdrv_dbIncreaseStationRxPacketCounter(UINT16 index);     
inline void    wdrv_dbIncreaseStationTxPacketCounter(UINT16 index);
inline void    wdrv_dbGetStationSuiteType(UINT16 index, UINT8 *cipher);
inline void    wdrv_dbSetWpaReq(UINT16 index, UINT32 reason);
inline void    wdrv_dbGetWpaReq(UINT16 index, UINT32 *reason);

UINT16 wdrv_dbGetNumStations(void);

#endif
