/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2003 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*		MODULE:		drvDbMain.c											   */
/*		PURPOSE:	AP Main database. This file holds the main database	   */
/*					of the AP, such as system global variable and MIB tables.*/
/*																	 	   */
/***************************************************************************/


#include "drvDbHash.h"
#include "whalDefs.h"
#include "whalParams.h"
#include "4xMngr.h"
#include "utils.h"
#include "drvDbMain.h"
#include "drvDbRegDomain.h"
#include "bssbridge.h"
#include "tx.h"
#include "rx.h"
#include "dda_wpa.h"
#include "pal.h"
#include "mgmtAccess.h"




typedef struct
{
	UINT32	suite;
	BOOL	enable;
} suite_t;


/*-----------------------------------------------------------*/
/*         EDCF Definitions                                  */
/*-----------------------------------------------------------*/

#define AIFS_PIFS  1  /* slots number to start tx at PIFS interval. */
#define AIFS_DIFS  2  /* slots number to start tx at DIFS interval (normal DCF access). */
#define MAX_AIFS   0xFF
#define MAX_EDCF_SLOTS  10   /* FW allows AIFSN up to this number of additional slots beyond DIFS. */
#define NO_RX_TIMEOUT   0    /* rxTimeout default value */
#define SINGLE_MPDU_TXOP_LIMIT  0    /* Only send a single MPDU when a TXOP is obtained. */
#define MSDU_LIFETIME_DEFAULT   512  /* In TUs (1024 usec Time Units). */

#if 0 
typedef enum
{
    ACK_IMMEDIATE_POLICY = 0,   /* means expect a standard ACK in response to each MPDU */
    NO_ACK_POLICY		 = 1    /* No ACK is expected when transmitting. */
}ACKPolicy_e;


typedef enum
{
    AC0 = 0,
    AC_DCF = AC0, /* use category 0 for normal DCF access upon initialization - all queues initialized to this */
    AC1 = 1,
    AC2 = 2,
    AC3 = 3,
    NUM_ACCESS_CATEGORIES = 4
}AccessCategory_e;



typedef enum 
{
    DATA_DCF = 0,       /* MSDUs are sent completely including retrys - normal legacy traffic */
    QOS_DATA_EDCF = 1   /* MPDUs are sent according to TXOP limits. */
}TrafficAccessType_e;

#endif

/*-----------------------------------------------------------*/
/*         wep  params                                       */
/*-----------------------------------------------------------*/


static defaultWepKey_T defaultWepKeys[WPA_NUM_DEFAULT_KEYS];
static UINT8 wepDefaultKeyId;


/*-----------------------------------------------------------*/
/*         Wone  Info                                        */
/*-----------------------------------------------------------*/


typedef struct 
{
    UINT8 type;
    UINT8 Priority;
    UINT8 globalQid;
    UINT32 threshold; /* Limit buffer pool usage by a specific tx queue in the ACX in 5 percent
					    units from ACX buffer pool. 0 = no limit, so no need to send IE. */
	BOOL edcfInfoRequired;  /* TRUE = need to configure EDCF parameters to FW for this tx queue. */
	edcf_info_t  edcfInfo;  /* defined in whal.h */
}queue_info_t;

typedef struct 
{
   queue_info_t            queue_info[MAX_NUM_Q_PER_WONE];
   UINT8                   numOfQueue;
}
queues_table_t;

typedef struct 
{
    
    dot11_supRates_T       apRates;
    dot11_supRates_T       apRatesTemplate;
    dot11_extSupRates_T    apOFDMRates;
    UINT16                 apCapabilities;
    UINT16                 defaultActiveRateAcx;
    UINT16                 defaultActiveRateAcxWithFallback;
    UINT16                 allApSupRatesLen;
    queues_table_t         queue_table;
    UINT8                  channel;
    UINT8                  band;
    UINT8                  primaryAcxQindex;
    UINT8                  psBroadcastQindex;
    UINT8                  UsageTime;      
    UINT8                  PreambleMode;
    UINT8                  PbccAlgorithmMode;
    UINT8                  RateAdaptation;
    UINT8                  CwMinMode;
    UINT8                  SlotTimeMode;
    UINT8                  ERPProtectionMode;
    UINT8                  ERPProtectionType;
    UINT8                  ERPProtectionTI;
    UINT8                  defaultActiveRate;
    UINT8                  maxActiveRate;
    UINT8                  maxCCKactiveRate;
    UINT8                  maxCCKbasicRate;
    UINT8                  maxOFDMbasicRate;
    UINT8                  minCCKbasicRate;
    UINT8                  minOFDMbasicRate;
    UINT8                  allApSupRates[DOT11_SUPPORTED_RATES_MAX +
                                         DOT11_EXT_SUPPORTED_RATES_MAX];
    UINT8                  Dot11Mode;
    UINT8                  PowerLevel;
} wone_info_T;




static UINT8 numSupWone;    /* Number of supported wone object. Default is 1  */
static UINT8 usageTimeMode;      /* 0 = short usage time, 1 = long usage time */




static wone_info_T wone_info[MAX_WONE_NET];  /* Wone Info table */



/*-----------------------------------------------------------*/
/*         Local Variables                                   */
/*-----------------------------------------------------------*/

static tVHCF_PARAMS  vhcfParams;
/* Inplements the privacy part in the station config table */
static struct 
{
	UINT16	disassociateReason;
	char	disassociateStation[MAC_ADDR_LEN];
	UINT16	deauthenticateReason;
	char	deauthenticateStation[MAC_ADDR_LEN];
	UINT16	deauthenticateFailStatus;
	char	deauthenticateFailStation[MAC_ADDR_LEN];
} dot11StationConfigTable; /* TODO */


/* Dynamic Algorithms Timestamps*/
static struct
{
	UINT32	pbccTimeStamp;
	UINT32  erpTimeStamp;
	UINT32  RxTxBrateTimestamp;
	UINT32  RxTxBrateCount;
	BOOL	pbccValid;
	BOOL	erpValid;
} DynamicAlgInfo; 

/* AP's MAC address*/
static char APMAC[MAC_ADDR_LEN] = {0xaa,0xaa,0xbb,0xcc,0xdd,0xee};
/* AP's BSSID */
static char BSSID[MAC_ADDR_LEN];
/* AP's SSID */
static dot11_SSID_T	apSSID;
/* AP's hidden SSID status */
static UINT16 apHiddenSSID;

/* Multiple SSID flag */
static UINT16 apMultipleSsidFlag; 
/* Multiple SSID Table */
static char apMultipleSsidTable[NUM_MULTIPLE_SSID][WHAL_SSID_STRING_LEN];
/* Multiple SSID counter */
static UINT32 apNumOfMultipleSsid = 0;

/* AP's beacon period. */
static UINT16 apBeaconPeriod;	
/* AP's DTIM period. */
static UINT8 apDtimPeriod;	
/* AP's RTS threshold. */
static UINT16 apRtsThreshold;	
/* AP's short retry limit. */
static UINT8 apShortRetryLimit;	
/* AP's long retry limit. */
static UINT8 apLongRetryLimit;	
/* AP's Frag threshold. */
static UINT16 apFragThreshold;	
/* AP's max transmit lifetime. */
static UINT32 apMaxTxLifetime;	
/* AP's max receive lifetime. */
static UINT32 apMaxRxLifetime;	
/* AP's BSS isolation state */
static BOOL bssBridgeIsolationOn;
/* AP's 4x mode*/
static dot11_TIcap_T	TIcap;
static BOOL	enable4x;
static BOOL	enableConcat;
static BOOL	enableAckEmul;
static BOOL enable4xMixMode;

static UINT8 FeatureSet;
/* AP's Contention window minimum */
static UINT16	apCWMinWindow;
/* AP uses this to decide how to set the supported rates IE in (re)associate response */
static UINT8 apRespRateMode;

/* RSN */
static dot11_rsne_T	apRsne; 

static char TI_OUI[DOT11_OUI_LEN] = {0x08, 0x00, 0x28};

static BOOL privacyOptionImplemented;

static BOOL privacyInvoked;

static BOOL excludeUnencrypted;


static BOOL  authOpenSystemEnable;
static UINT8 authOpenSystemIndex;

static BOOL  authSharedKeyEnable;
static UINT8 authSharedKeyIndex;

static BOOL  Agile; 


static UINT8 RateFallback;


static UINT16 SlotTimeThreshold;



static UINT32 DynamicAlgorithmsTimeout;

static UINT32 RadioCalibrateOption;
static UINT16 RadioCalibrateInterval;


/*		WPA Section		*/

/* broadcast suite*/
UINT8	broadcastCipherSuiteType;		


static UINT32	wpaMode;

static BOOL WlanServiceState ;
static UINT8 SavedEnRestart;
static BOOL RestoreSavedEnRestart;
/* Driver Workaround variables */
static UINT8 EnRestart;
static UINT8 RxFreeMemOptionWA;
static UINT8 BeaconNotSentOptionWA;
static UINT8 TxStuckOptionWA;
static UINT16 ConsFcsErrOptionWA;
static UINT8  aciOptionWA;
static UINT8  nbOptionWA;

/* Interference WA State Machine configuration */
static UINT32 aciStateTimeout;
static UINT32 nbStateTimeout;
static UINT32 aciRateThreshold;
static UINT32 aciEventThreshold;
static UINT32 mediumBusyThresh;

/* FW Workaround variables */
static UINT8  EnEnergyDetect;
static UINT16 pdUnderAciLowThrsh;
static UINT16 pdUnderAciHighThrsh;

/* radio support variable */
static UINT8 RadioFrequency = RADIO_BAND_2_4_AND_5GHZ;

/* radio parameters */
static RadioParam_t dbRadioParams;

/* packet bursting Information */
static UINT16 txopLimit;    

extern UINT16 configTxopLimit;

static UINT8 RadarDetectMode;
static UINT8 ApBridgeMode;

static PAL_spinLockHandle drvDbMainLock;

/*-----------------------------------------------------------*/
/*         Local Functions                                   */
/*-----------------------------------------------------------*/
static void	   initMibTables(void);	
static BOOL    isBasic(UINT8 rate);
static int     initDefaultActiveRate(UINT8 woneIndex);
static int     initSecurity(initConfigStruct *pInitConfigStruct);
INIT_RET_CODE  checkInitStructValid(initConfigStruct *pInitConfigStruct);	

/*************************************************************************
 *                        edcfInfoInit                                   *
 *************************************************************************
DESCRIPTION: Initialize Tx queue's EDCF parameters.
			 Currently assumes that it is called only for low and high priority
			 tx queues and only if vHCF or concatination is enabled!!!
**************************************************************************/
void edcfInfoInit(int queueType ,edcf_info_t *edcfInfoPtr)
{
	
   edcfInfoPtr->acID = queueType + 1; /* TODO-AK, update values for full EDCF (WME)!!! */
   edcfInfoPtr->dot11EDCATableMSDULifetime = MSDU_LIFETIME_DEFAULT;
   edcfInfoPtr->rxTimeout = NO_RX_TIMEOUT;
   edcfInfoPtr->ackPolicy = ACK_IMMEDIATE_POLICY;
   edcfInfoPtr->shortRetryLimit = 7; /* Temp, currently not implemented in the FW. */
   edcfInfoPtr->retryPremption = FALSE;
   
   if(queueType == WONE_HIGH_PRIOR_QUEUE)
    {

       edcfInfoPtr->trafficType =  QOS_DATA_EDCF;
       edcfInfoPtr->aifsn =  AIFS_PIFS;
       edcfInfoPtr->cwMin = 0;
       edcfInfoPtr->cwMax = 0;
       edcfInfoPtr->longRetryLimit = 4;  /* Temp, currently not implemented in the FW. */
	edcfInfoPtr->txopContinuation = FALSE;
       edcfInfoPtr->txopLimit = 0;

    }
    else
    {   
       int enablePb = ((enableConcat) && (!vhcfParams.vhcfEnable) && (numSupWone == WONE_SINGLE_BAND));
       edcfInfoPtr->trafficType = (enablePb) ? QOS_DATA_EDCF : DATA_DCF;
       edcfInfoPtr->aifsn = (enablePb) ? AIFS_DIFS : AIFS_DIFS ;
       edcfInfoPtr->cwMin = (enablePb) ? 0xF : 0xF ;
       edcfInfoPtr->cwMax = (enablePb) ? 0x3FF : 0x3FF; 
       edcfInfoPtr->longRetryLimit = 2;  /* Temp, currently not implemented in the FW. */
       edcfInfoPtr->txopContinuation = (enablePb) ? TRUE : FALSE;
       edcfInfoPtr->txopLimit = (enablePb) ? txopLimit : 0;
    }
}


/*************************************************************************
 *                        initMibTables                                  *
 *************************************************************************
DESCRIPTION: This function initializes wpa mib table. 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
static BOOL LookforBasicRate(int woneIndex,initConfigStruct *pInitConfigStruct)
{
	int i = 0,break_flag;
	while(1)
	{
		break_flag = 1;	
		if(pInitConfigStruct->wone_info[woneIndex].apRates.hdr.eleLen > i)
		{
			break_flag = 0;
			if(pInitConfigStruct->wone_info[woneIndex].apRates.rates[i] & 0x80)
			{
				return TRUE;
			}
		}
		if(pInitConfigStruct->wone_info[woneIndex].apOFDMRates.hdr.eleLen > i)
		{
			break_flag = 0;
			if(pInitConfigStruct->wone_info[woneIndex].apOFDMRates.rates[i] & 0x80)
			{
				return TRUE;
			}
		}
		if(pInitConfigStruct->wone_info[woneIndex].apAddRates.hdr.eleLen > i)
		{
			break_flag = 0;
			if(pInitConfigStruct->wone_info[woneIndex].apAddRates.rates[i] & 0x80)
			{
				return TRUE;
			}
		}
		i++;
		
		if(break_flag)
			break;
	}
	return FALSE; 
}

/*************************************************************************
 *                        initMibTables                                  *
 *************************************************************************
DESCRIPTION: This function initializes wpa mib table. 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
INIT_RET_CODE  checkInitStructValid(initConfigStruct *pInitConfigStruct)
{
	int woneIndex;
	for(woneIndex = 0;woneIndex < pInitConfigStruct->numSupWone;woneIndex++)
	{
		switch(pInitConfigStruct->wone_info[woneIndex].Dot11Mode)
		{
		case DOT11_MODE_MIXED:

			/* check band*/
			if(pInitConfigStruct->wone_info[woneIndex].band != RADIO_BAND_2_4GHZ)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* check channel*/
			if(
				(pInitConfigStruct->wone_info[woneIndex].channel < 1 ) |
				(pInitConfigStruct->wone_info[woneIndex].channel > 14 )
			  )
				return DDC_INIT_CFG_LOGICAL_ERROR;

			/* allowed only in DOT11_MODE_A_ONLY mode */
			if (pInitConfigStruct->RadarDetectMode)
				return DDC_INIT_CFG_LOGICAL_ERROR;

						/* check at list one basic rate*/


			break;
		
		case DOT11_MODE_B_ONLY:

			/* additional rates*/
			if(pInitConfigStruct->wone_info[woneIndex].apAddRates.hdr.eleLen != 0)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* ofdm rates*/
			if(pInitConfigStruct->wone_info[woneIndex].apOFDMRates.hdr.eleLen != 0)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* chack band*/
			if(pInitConfigStruct->wone_info[woneIndex].band != RADIO_BAND_2_4GHZ)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* check channel*/
			if(
				(pInitConfigStruct->wone_info[woneIndex].channel < 1 ) |
				(pInitConfigStruct->wone_info[woneIndex].channel > 14 )
			  )
				return DDC_INIT_CFG_LOGICAL_ERROR;

			/* allowed only in DOT11_MODE_A_ONLY mode */
			if (pInitConfigStruct->RadarDetectMode)
				return DDC_INIT_CFG_LOGICAL_ERROR;

			break;
		
		case DOT11_MODE_B_ONLY_PLUS:			

			/* additional rates*/
			if(pInitConfigStruct->wone_info[woneIndex].apAddRates.hdr.eleLen != 0)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* ofdm rates*/
			if(pInitConfigStruct->wone_info[woneIndex].apOFDMRates.hdr.eleLen != 0)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* check band*/
			if(pInitConfigStruct->wone_info[woneIndex].band != RADIO_BAND_2_4GHZ)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* check channel*/
			if(
				(pInitConfigStruct->wone_info[woneIndex].channel < 1 ) |
				(pInitConfigStruct->wone_info[woneIndex].channel > 14 )
			  )
				return DDC_INIT_CFG_LOGICAL_ERROR;

			/* allowed only in DOT11_MODE_A_ONLY mode */
			if (pInitConfigStruct->RadarDetectMode)
				return DDC_INIT_CFG_LOGICAL_ERROR;

			break;
		
		case DOT11_MODE_OFDM_ONLY:

			/* check band*/
			if(pInitConfigStruct->wone_info[woneIndex].band != RADIO_BAND_2_4GHZ)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* check channel*/
			if(
				(pInitConfigStruct->wone_info[woneIndex].channel < 1 ) |
				(pInitConfigStruct->wone_info[woneIndex].channel > 14 )
			  )
				return DDC_INIT_CFG_LOGICAL_ERROR;

			/* allowed only in DOT11_MODE_A_ONLY mode */
			if (pInitConfigStruct->RadarDetectMode)
				return DDC_INIT_CFG_LOGICAL_ERROR;

			break;

		case DOT11_MODE_A_ONLY:
			
			/* base rates*/
			if(pInitConfigStruct->wone_info[woneIndex].apRates.hdr.eleLen != 0)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* additional rates*/
			if(pInitConfigStruct->wone_info[woneIndex].apAddRates.hdr.eleLen == 0)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* ofdm rates*/
			if(pInitConfigStruct->wone_info[woneIndex].apOFDMRates.hdr.eleLen != 0)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* chack band*/
			if(pInitConfigStruct->wone_info[woneIndex].band != RADIO_BAND_5GHZ)
				return DDC_INIT_CFG_LOGICAL_ERROR;
			/* check channel*/
			if(				
				(pInitConfigStruct->wone_info[woneIndex].channel < 36 ) |
				(pInitConfigStruct->wone_info[woneIndex].channel > 161 )
			  )
			{
				printf("\n\nwrong channel: %d\n\n\n", pInitConfigStruct->wone_info[woneIndex].channel);
				return DDC_INIT_CFG_LOGICAL_ERROR;
			}
			
			break;
		
		default:
			return DDC_INIT_CFG_RANGE_ERROR;
		}

		/* check for basic rates */
		if(LookforBasicRate(woneIndex,pInitConfigStruct) == FALSE)
		{
			printk("LookforBasicRate:no basic rates found for wone index %d.\n",woneIndex);
			return DDC_INIT_CFG_LOGICAL_ERROR;
		}
	}

	/*vHCF doesn't work in wOne mode */
	if((pInitConfigStruct->vhcfParams.vhcfEnable) && (pInitConfigStruct->numSupWone > 1) )
	{
		printf("checkInitStructValid:vHCF doesn't work in wOne mode.\n");
		return DDC_INIT_CFG_LOGICAL_ERROR;
	}

	/* multiple SSID doesn't work with WEP */
	if((pInitConfigStruct->apMultipleSsidFlag) && ( pInitConfigStruct->wpaMode == WPA_STA_TYPE_WEP_8021X))
	{
		printf("checkInitStructValid:multiple SSID doesn't work with WEP.\n");
		return DDC_INIT_CFG_LOGICAL_ERROR;
	}

	if (pInitConfigStruct->ApBridgeMode > 1)
	{
		printf("%s: ap bridge mode %d.\n", __func__, pInitConfigStruct->ApBridgeMode);
		return DDC_INIT_CFG_LOGICAL_ERROR;
	}


	return DDC_INIT_SUCCESS;
}

/*************************************************************************
 *                        wdrv_dbMainInit                                    *
 *************************************************************************
DESCRIPTION: This function initializes the AP's main database:
			1. Reset the MIB tables.                 
			2. Load the database variables with the values from the nvram.
			3. Fixes the AP's defaulte active rate.
			4. Builds information elements based on the database. Thius elements
				are sent in management messges.
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      OK : Initialization succesfull.
             NOK: Initialization unsuccesfull.                                                                                                                                             
************************************************************************/
INIT_RET_CODE wdrv_dbMainInit(initConfigStruct *pInitConfigStruct)
{
	INIT_RET_CODE ret;
	int i, j;
	UINT16 MaxConcatSize;
    int wone_index;
    UINT16   value16;
    BOOL     value;
    UINT8 global_Qid =0;
	UINT8 apDomain;

	PAL_spinLock_init(&drvDbMainLock);

	/* check params validity*/
	if(((ret = checkInitStructValid(pInitConfigStruct)) != DDC_INIT_SUCCESS ))
	{
		printf("checkInitStructValid failed\n");
		return ret;
	}

	/* init mib parameters*/
	initMibTables();
   
	/* init mac, bssid (same parameter in init stage)*/
    PAL_memCopy(APMAC,pInitConfigStruct->APMAC,MAC_ADDR_LEN);
    PAL_memCopy(BSSID,pInitConfigStruct->APMAC,MAC_ADDR_LEN);
   
	/* update ap ssid params */
	apSSID.hdr.eleId = DOT11_SSID_ELE_ID;
	apSSID.hdr.eleLen = pInitConfigStruct->dot11SsIDLength;
	PAL_memCopy(apSSID.serviceSetId,pInitConfigStruct->dot11SsID,apSSID.hdr.eleLen);
	apHiddenSSID = pInitConfigStruct->apHiddenSSID;

	apMultipleSsidFlag = pInitConfigStruct->apMultipleSsidFlag;
	if (apMultipleSsidFlag)
	{
		PAL_memCopy((char*)apMultipleSsidTable,(char*)pInitConfigStruct->apMultipleSsidTable,NUM_MULTIPLE_SSID*WHAL_SSID_STRING_LEN);
		
		while (apNumOfMultipleSsid < NUM_MULTIPLE_SSID) 
        {
            if (apMultipleSsidTable[apNumOfMultipleSsid][0] == '\0') 
                break;
            apNumOfMultipleSsid++;
        }

	}
	vhcfParams    = pInitConfigStruct->vhcfParams;
	enable4x      = pInitConfigStruct->enable4x;
	enableConcat  = pInitConfigStruct->enableConcat;
	enableAckEmul = FALSE;
	#ifdef TIWLAN_DEBUG
		txopLimit     = configTxopLimit; /* MicroSec */
	#else
		txopLimit     = 3000; /* MicroSec */
	#endif
	
    /* init 802.11 d & h DB */
    if((ret = wdrv_dbRegDomainInit(&pInitConfigStruct->RegInit,
								   pInitConfigStruct->wone_info[0].band	
								  )) != DDC_INIT_SUCCESS ) 
	{
		printf("wdrv_dbRegDomainInit failed\n");
		return ret;
	}

	/* init mgmt access list params*/
	if((ret=mgmtAccessInit(&(pInitConfigStruct->mgmtAccessList))) != OK)
	{
		printf("\n\r error init access list");
		return DDC_INIT_CFG_LOGICAL_ERROR;
	}

	bssBridgeIsolationOn = !(pInitConfigStruct->bssBridgeState == BSS_TRAFFIC_EN);

	usageTimeMode = pInitConfigStruct->usageTimeMode;
	numSupWone    = pInitConfigStruct->numSupWone;

	Agile = FALSE;

    for (wone_index = 0;wone_index < numSupWone; wone_index++)
    {
        PAL_memSet(&wone_info[wone_index].queue_table,0,sizeof(queues_table_t));
		/* If vHCF is enabled set 3 Tx queues per Wone (low priority, high priority
		   and ps-bcast), otherwise (default) set 2 Tx queues (regular and ps-bcast). */
		wone_info[wone_index].queue_table.numOfQueue = (vhcfParams.vhcfEnable) ? 3 : 2; 

        for(i=0;i<wone_info[wone_index].queue_table.numOfQueue -1;i++)
        {
              wone_info[wone_index].queue_table.queue_info[i].globalQid = global_Qid;
              wone_info[wone_index].queue_table.queue_info[i].Priority = global_Qid;
              global_Qid ++;
              wone_info[wone_index].queue_table.queue_info[i].type = 0; /*regular */

			  if (vhcfParams.vhcfEnable ||  enableConcat)
			  {
				 /* Set ACX pull usage limit for low priority 50% (10*5), and no limit for high priority. */ 
			     wone_info[wone_index].queue_table.queue_info[i].threshold = 0;
				 /*  Temp - remove this:  (i == WONE_LOW_PRIOR_QUEUE) ? 10 : 0;  */

				 /* Init queue EDCF parmeters. */
				 edcfInfoInit( i, &(wone_info[wone_index].queue_table.queue_info[i].edcfInfo) );
				 wone_info[wone_index].queue_table.queue_info[i].edcfInfoRequired = TRUE;
			  }
			  /* If vHCF is disabled, ACX pool threshold and EDCF IEs are not needed for now. */
			  else 
			  {
			     wone_info[wone_index].queue_table.queue_info[i].threshold = 0;
			     wone_info[wone_index].queue_table.queue_info[i].edcfInfoRequired = FALSE;
			  }
        }
        /* Set PS brodcast Queue */
        wone_info[wone_index].queue_table.queue_info[i].globalQid = global_Qid;
        wone_info[wone_index].queue_table.queue_info[i].Priority = global_Qid;
        wone_info[wone_index].queue_table.queue_info[i].type = 1; /* PS broadcast */
		wone_info[wone_index].queue_table.queue_info[i].threshold = 0; /* no limit */
		wone_info[wone_index].queue_table.queue_info[i].edcfInfoRequired = FALSE;

        global_Qid ++;

 /*       if(enableConcat)
        {
           for(i=0;i<wone_info[wone_index].queue_table.numOfQueue ;i++)
           {
              packetBurstInfoInit( i, &(wone_info[wone_index].queue_table.queue_info[i].packetBurstInfo) );
           }
        } */         

			/* Channel & Band*/
		wone_info[wone_index].band    = pInitConfigStruct->wone_info[wone_index].band;
		wone_info[wone_index].channel = pInitConfigStruct->wone_info[wone_index].channel;

        /* Regulatory Domain */
        if((ret = wdrv_dbGetdot11CurrentRegDomain(wone_info[wone_index].band, &apDomain)) != OK)
		{
			printf("wdrv_dbGetdot11CurrentRegDomain failed\n");
			return ret;
		}
		
		/* check if channel allowed */
		if (IsChannelAllowed(wone_index,wone_info[wone_index].channel) != TRUE)
		{
			if ( (ret = wdrv_dbRegDomainGetDefaultChannel(apDomain, &(wone_info[wone_index].channel))) != OK)
			{
				printf("wdrv_dbRegDomainGetDefaultChannel failed\n");
				return ret;
			}
			printf("setting channel to default according to regulatory domain......\n");
		}

		wone_info[wone_index].Dot11Mode     = pInitConfigStruct->wone_info[wone_index].Dot11Mode;

        /* build the apRates from the OperationalRateSet & OperationalAdditionalRateSet */
        /* depending on the Dot11 mode */
        wone_info[wone_index].apRates.hdr.eleId = DOT11_SUPPORTED_RATES_ELE_ID;  
        
        if (wone_info[wone_index].band == RADIO_BAND_2_4GHZ)
        {
			wone_info[wone_index].apRates.hdr.eleLen = pInitConfigStruct->wone_info[wone_index].apRates.hdr.eleLen;
			PAL_memCopy(wone_info[wone_index].apRates.rates,
				   pInitConfigStruct->wone_info[wone_index].apRates.rates,
				   wone_info[wone_index].apRates.hdr.eleLen);
        }
        else
        {
            wone_info[wone_index].apRates.hdr.eleLen = 0;
        }


        if ((wone_info[wone_index].Dot11Mode != DOT11_MODE_B_ONLY) && 
            (wone_info[wone_index].Dot11Mode != DOT11_MODE_B_ONLY_PLUS) &&
			(wone_info[wone_index].channel != 14))
        {

			PAL_memCopy((char*)&wone_info[wone_index].apRates.rates[pInitConfigStruct->wone_info[wone_index].apRates.hdr.eleLen],
				   pInitConfigStruct->wone_info[wone_index].apAddRates.rates,
				   pInitConfigStruct->wone_info[wone_index].apAddRates.hdr.eleLen
				  );

            wone_info[wone_index].apRates.hdr.eleLen += pInitConfigStruct->wone_info[wone_index].apAddRates.hdr.eleLen;				   
        }

        /* Get CWmin */
		wone_info[wone_index].CwMinMode = pInitConfigStruct->wone_info[wone_index].CwMinMode;

		if (wone_info[wone_index].CwMinMode == DOT11_CWMIN_ENABLED)
            apCWMinWindow = 15;
        else if (wone_info[wone_index].CwMinMode == DOT11_CWMIN_DISABLED)
            apCWMinWindow = 31;
        else /* behave as dot11mode dependent */
        {
            if (wone_info[wone_index].Dot11Mode == DOT11_MODE_B_ONLY || wone_info[wone_index].Dot11Mode == DOT11_MODE_B_ONLY_PLUS)
                apCWMinWindow = 31;
            else
                apCWMinWindow = 15;
        }

        /* build the apOFDMRates from the OperationalOFDMRateSet */
        if (wone_info[wone_index].band == RADIO_BAND_2_4GHZ)
        {
            wone_info[wone_index].apOFDMRates.hdr.eleId = DOT11_EXT_SUPPORTED_RATES_ELE_ID;
            if ((wone_info[wone_index].Dot11Mode != DOT11_MODE_B_ONLY) && 
                (wone_info[wone_index].Dot11Mode != DOT11_MODE_B_ONLY_PLUS) &&
				(wone_info[wone_index].channel != 14))
            {
            
				wone_info[wone_index].apOFDMRates.hdr.eleLen = pInitConfigStruct->wone_info[wone_index].apOFDMRates.hdr.eleLen;
				PAL_memCopy(wone_info[wone_index].apOFDMRates.rates,
					   pInitConfigStruct->wone_info[wone_index].apOFDMRates.rates,
					   pInitConfigStruct->wone_info[wone_index].apOFDMRates.hdr.eleLen);

			
			}
        }  
        else
        {
            wone_info[wone_index].apOFDMRates.hdr.eleLen = 0;
        }


        wone_info[wone_index].allApSupRatesLen = 0;

        for (i=0; i<wone_info[wone_index].apRates.hdr.eleLen; i++)
        {
            wone_info[wone_index].allApSupRates[wone_info[wone_index].allApSupRatesLen++] = 
            wone_info[wone_index].apRates.rates[i];
        }

        for (i=0; i<wone_info[wone_index].apOFDMRates.hdr.eleLen; i++)
        {
            wone_info[wone_index].allApSupRates[wone_info[wone_index].allApSupRatesLen++] =
            wone_info[wone_index].apOFDMRates.rates[i];
        }
        /* Sort allApSupRates in ascending order */
        for (i=0; i<wone_info[wone_index].allApSupRatesLen; i++)
        {
            for (j=i+1; j<wone_info[wone_index].allApSupRatesLen; j++)
            {
                if ((wone_info[wone_index].allApSupRates[j] & 0x7F) < (wone_info[wone_index].allApSupRates[i] & 0x7F))
                {
                    char temp = wone_info[wone_index].allApSupRates[i];
                    wone_info[wone_index].allApSupRates[i] = wone_info[wone_index].allApSupRates[j];
                    wone_info[wone_index].allApSupRates[j] = temp;
                }
            }
        }
        /* Add to handle special test case where a rate is set more than once */
        for (i=1; i<wone_info[wone_index].allApSupRatesLen; i++)
        {
            if (wone_info[wone_index].allApSupRates[i] == wone_info[wone_index].allApSupRates[i-1])
            {
                for (j=i; j<wone_info[wone_index].allApSupRatesLen-1; j++)
                {
                    wone_info[wone_index].allApSupRates[j] = wone_info[wone_index].allApSupRates[j+1];
                }
                wone_info[wone_index].allApSupRatesLen--;
            }
        }

		wone_info[wone_index].ERPProtectionTI = pInitConfigStruct->wone_info[wone_index].ERPProtectionTI;
	
		FeatureSet = pInitConfigStruct->FeatureSet;
       
		TIcap.hdr.eleId = DOT11_TI_CAP_ELE_ID;
        TIcap.hdr.eleLen = DOT11_4X_FEATURE_SET_ZERO_LEN;
        PAL_memCopy(TIcap.cap, TI_OUI, DOT11_OUI_LEN);
        TIcap.cap[DOT11_OUI_LEN] = FeatureSet;
        if (FeatureSet == 0)
            TIcap.hdr.eleLen = DOT11_4X_FEATURE_SET_ZERO_LEN;
        else
        {
            int curr = DOT11_4X_FEATURE_SET_ZERO_LEN;
            if (enable4x)
            {
                /* declare concatenation */
                if (enableConcat)
                {
                    MaxConcatSize = ENDIAN_HANDLE_WORD(WLAN_MAX_CONCAT_SIZE);
                    TIcap.cap[curr] = TI_CAP_4X_CONCATENATION;
                    curr += sizeof(UINT16);
                    TIcap.cap[curr] = 2;
                    curr += sizeof(UINT16);
                    PAL_memCopy(&TIcap.cap[curr], (char *)&MaxConcatSize, sizeof(MaxConcatSize));
                    curr += 2;
                }
                /* declare Ack emulation */
                if (enableAckEmul)
                {
                    TIcap.cap[curr] = TI_CAP_4X_TCP_ACK_EMUL;
                    curr += sizeof(UINT16);
                    TIcap.cap[curr] = 0;
                    curr += sizeof(UINT16);
                }
            }
            /* declare trick packet */
            if (wone_info[wone_index].ERPProtectionTI)
            {
                TIcap.cap[curr] = TI_CAP_TRICK_PACKET_ERP;
                curr += sizeof(UINT16);
                TIcap.cap[curr] = 0;
                curr += 2;
            }

            if (curr <= DOT11_TI_CAP_MAX_LEN)
                TIcap.hdr.eleLen = curr;
            else
            {
                printf("there is not enough space allocated for 4x IE (need:%d, have:%d)\n",
                       curr, DOT11_TI_CAP_MAX_LEN);
                return DDC_INIT_RESOURCE_ERROR;
            }
        }

        if((ret = initDefaultActiveRate(wone_index)) != OK)
		{
			printf("initDefaultActiveRate failed\n");
			return DDC_INIT_CFG_LOGICAL_ERROR;
		}

		wone_info[wone_index].PbccAlgorithmMode = pInitConfigStruct->wone_info[wone_index].PbccAlgorithmMode;
        if (wone_info[wone_index].PbccAlgorithmMode != PBCC_ALGO_DISABLED)
            wdrv_dbSetCurrentPBCCSupportMode(TRUE);

		wone_info[wone_index].RateAdaptation = pInitConfigStruct->wone_info[wone_index].RateAdaptation;
		wone_info[wone_index].PreambleMode = pInitConfigStruct->wone_info[wone_index].PreambleMode;
		wone_info[wone_index].SlotTimeMode = pInitConfigStruct->wone_info[wone_index].SlotTimeMode;
		wone_info[wone_index].ERPProtectionMode = pInitConfigStruct->wone_info[wone_index].ERPProtectionMode;
		wone_info[wone_index].ERPProtectionType = pInitConfigStruct->wone_info[wone_index].ERPProtectionType;
        wone_info[wone_index].apCapabilities = 0;
        wone_info[wone_index].apCapabilities |= (TRUE << CAP_ESS_SHIFT);
        if (wone_info[wone_index].PbccAlgorithmMode != PBCC_ALGO_DISABLED)
            wone_info[wone_index].apCapabilities |= (1 << CAP_PBCC_SHIFT);
        wone_info[wone_index].apCapabilities |= (Agile << CAP_AGILE_SHIFT);
        if (wone_info[wone_index].PreambleMode != PREAMBLE_MODE_LONG)
        {
            wone_info[wone_index].apCapabilities |= (1 << CAP_PREAMBLE_SHIFT);
        }
        if (wone_info[wone_index].SlotTimeMode != SLOTTIME_MODE_DISABLED)
        {
            wone_info[wone_index].apCapabilities |= (1 << CAP_SLOTTIME_SHIFT);
        }


        PAL_memSet(&DynamicAlgInfo, 0, sizeof(DynamicAlgInfo));
        		
		wone_info[wone_index].UsageTime = pInitConfigStruct->wone_info[wone_index].UsageTime;
        /* if usage time mode is short -> divide UsageTime n=by 2 */
		if(usageTimeMode == 0)
          wone_info[wone_index].UsageTime = ((wone_info[wone_index].UsageTime)/2);
	} /* for wone */

	
	enable4xMixMode		     = pInitConfigStruct->enable4xMixMode;		
	apRespRateMode		     = ASSOCRESP_RATE_AP_AND_STA;
	RadioCalibrateOption     = pInitConfigStruct->RadioCalibrateOption;
	RadioCalibrateInterval   = pInitConfigStruct->RadioCalibrateInterval;
	RateFallback		     = pInitConfigStruct->RateFallback;

	apBeaconPeriod    = pInitConfigStruct->apBeaconPeriod;
	apDtimPeriod      = pInitConfigStruct->apDtimPeriod;
	apRtsThreshold    = pInitConfigStruct->apRtsThreshold;
	apShortRetryLimit = pInitConfigStruct->apShortRetryLimit;
	apLongRetryLimit  = pInitConfigStruct->apLongRetryLimit;
	apFragThreshold   = pInitConfigStruct->apFragThreshold;
	apMaxTxLifetime   = pInitConfigStruct->apMaxTxLifetime;
	apMaxRxLifetime   = pInitConfigStruct->apMaxRxLifetime; 
	EnEnergyDetect	  = pInitConfigStruct->enableEnergyDetect;
	
	DynamicAlgorithmsTimeout = DEF_DYN_ALG_TIMEOUT;

	
	for (i = 0; i < MAX_AUTH_ALGORITHM_SUPPORTED; i++)
   {

		   value16 = pInitConfigStruct->dot11AuthenticationAlgorithmTable[i].authAlgo;
		   value   = pInitConfigStruct->dot11AuthenticationAlgorithmTable[i].enable;

		   if (value16 == MIB_OPEN_SYSTEM)
		   {
			   authOpenSystemIndex = i;

			   if (value == TRUE)
				   authOpenSystemEnable = TRUE;
			   else
				   authOpenSystemEnable = FALSE;
		   }

		   else if (value16 == MIB_SHARED_KEY)
		   {
			   authSharedKeyIndex = i;

			   if (value == TRUE)
				   authSharedKeyEnable = TRUE;
			   else
				   authSharedKeyEnable = FALSE;
		   }

		   else
		   {
			   printf("WDRV_DB: FATAL ERROR, receive error auth algorithm from init struct.\n");
			   return DDC_INIT_CFG_RANGE_ERROR;
		   }
   }

	/* security parametrs */	
    if((ret = initSecurity(pInitConfigStruct)) != DDC_INIT_SUCCESS)
	{
		printf("initSecurity failed\n");
		return ret;
	}


	WlanServiceState = pInitConfigStruct->WlanServiceState;
 	

	EnRestart = pInitConfigStruct->EnRestart;
    SavedEnRestart =        EnRestart;
    RestoreSavedEnRestart =        FALSE;
    if ( !WlanServiceState )	 
		EnRestart = 0;
    

	RxFreeMemOptionWA     = pInitConfigStruct->RxFreeMemOptionWA; 
    BeaconNotSentOptionWA = pInitConfigStruct->BeaconNotSentOptionWA;
    TxStuckOptionWA       = pInitConfigStruct->TxStuckOptionWA;
    ConsFcsErrOptionWA    = pInitConfigStruct->ConsFcsErrOptionWA; 
	aciOptionWA			  = pInitConfigStruct->aciOptionWA;
	nbOptionWA			  = pInitConfigStruct->nbOptionWA;
	aciStateTimeout		  = pInitConfigStruct->interfWAparams.aciStateTimeout;
	nbStateTimeout		  = pInitConfigStruct->interfWAparams.nbStateTimeout;
	aciRateThreshold	  = pInitConfigStruct->interfWAparams.aciRateThreshold;
	aciEventThreshold	  = pInitConfigStruct->interfWAparams.aciEventThreshold;
	pdUnderAciLowThrsh	  = pInitConfigStruct->interfWAparams.pdUnderAciLowThrsh;
	pdUnderAciHighThrsh   = pInitConfigStruct->interfWAparams.pdUnderAciHighThrsh;
	mediumBusyThresh	  = pInitConfigStruct->interfWAparams.mediumBusyThresh;

   
	if(pInitConfigStruct->wone_info[0].Dot11Mode == DOT11_MODE_OFDM_ONLY)
		SlotTimeThreshold = 3;
	else
		SlotTimeThreshold = 0;

   /******************************************
    * Radio Params - Read them all from NVRAM
    ******************************************/
   PAL_memSet( (char*)&dbRadioParams, 0, sizeof( dbRadioParams));

   dbRadioParams.overrideBitmask      = pInitConfigStruct->dbRadioParams.overrideBitmask;
   dbRadioParams.powerTable_paramType = pInitConfigStruct->dbRadioParams.powerTable_paramType;
   PAL_memCopy(dbRadioParams.powerTable_param,
		  pInitConfigStruct->dbRadioParams.powerTable_param,
		  MAX_RADIO_PARAM_POWER_TABLE);
   
   RadarDetectMode = pInitConfigStruct->RadarDetectMode;

   /* Wi-Fi mode */
   if ((ApBridgeMode = pInitConfigStruct->ApBridgeMode)) {
	   /* Reflects Wi-Fi Alliance changes */
	   apBeaconPeriod = 100;
	   apDtimPeriod = 1;
   }

   printf("Configuration succeeded !!!\n");

   return DDC_INIT_SUCCESS;
}/* END wdrv_dbMainInit() */


/*************************************************************************
 *                        initSecurity                                    *
 *************************************************************************
DESCRIPTION: This function initializes wpa main database:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      OK : Initialization succesfull.
             NOK: Initialization unsuccesfull.                                                                                                                                             
************************************************************************/
static int initSecurity(initConfigStruct *pInitConfigStruct)
{
    int    wone_index;

	wpaMode = pInitConfigStruct->wpaMode;
	
	privacyOptionImplemented = pInitConfigStruct->privacyOptionImplemented;
	/* broadcastCipherSuiteType is updated by WPA authenticator, meanwhile
	   initialized to default value */
	broadcastCipherSuiteType = CIPHER_NONE;
	
	/* First, we have to apply some rules in order to be compliant with the automation interface */
	/* Rule no. 1: If the working mode is anything except  'Legacy & Privacy On' and 'Legacy & Privacy Off' 
			the 'Privacy Option' parameter is override to TRUE in the database (not in NVRAM).*/
	
	if ((wpaMode & WPA_STA_TYPE_WPA) || (wpaMode & WPA_STA_TYPE_WEP_8021X))
		privacyOptionImplemented = TRUE;
		
	privacyInvoked = privacyOptionImplemented;
	excludeUnencrypted = privacyOptionImplemented;


	/* The 'Auth Algorithm' parameter will be override to be open system (in database and not in NVRAM), 
	in all of the working modes except 'Legacy & Privacy On' 
	And in the case that the working mode is legacy but privacy is off. */	
	
	if (wpaMode == WPA_STA_TYPE_WEP_LEGACY)
	{
		if (privacyOptionImplemented == FALSE)
		{
			authOpenSystemEnable = TRUE;
			authSharedKeyEnable = FALSE;
		}
	}
	else	/* wpaMode != WPA_STA_TYPE_WEP_LEGACY */
	{
		authOpenSystemEnable = TRUE;
		authSharedKeyEnable = FALSE;
	}
	
	/* Now build the capabilities because the privacy option maybe had been changed in this function. */
    for (wone_index = 0;wone_index < numSupWone; wone_index++)
    {
        wone_info[wone_index].apCapabilities |= (privacyOptionImplemented << CAP_PRIVACY_SHIFT);
    }

	return DDC_INIT_SUCCESS;
}

/*************************************************************************
 *                        initDefaultActiveRate                          *
 *************************************************************************
DESCRIPTION: This function initializes the AP's default active rate.
				THe default active rate is the highest basic rate supportted 
				by the AP.
			There are 2 variables:
			1. defaultActiveRate is the default active rate in standard units.
			2. defaultActiveRateAcx is the default active rate in ACX units.
			3. defaultActiveRateAcxWithFallback is the default active rate in
               ACX units with other rates used in FW rate fallback.
                                                                                         
INPUT:       

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
static int initDefaultActiveRate(UINT8 woneIndex)
{
	int i;
	UINT16 activeRateAcx = 0;
	
	wone_info[woneIndex].maxActiveRate = 0;
	wone_info[woneIndex].maxCCKactiveRate = DOT11_RATE_1;
	wone_info[woneIndex].maxCCKbasicRate = DOT11_RATE_1;
    wone_info[woneIndex].maxOFDMbasicRate = DOT11_RATE_6;
	wone_info[woneIndex].minCCKbasicRate = DOT11_RATE_11 | 0x80;
    wone_info[woneIndex].minOFDMbasicRate = DOT11_RATE_54 | 0x80;
	wone_info[woneIndex].defaultActiveRate = 0;
	wone_info[woneIndex].defaultActiveRateAcx = 0;
	wone_info[woneIndex].defaultActiveRateAcxWithFallback = 0;

	for (i = 0; i < wone_info[woneIndex].allApSupRatesLen; i++)
	{
		/* Find max of all rates (basic and not basic) */
		wone_info[woneIndex].maxActiveRate = 
                    _max(wone_info[woneIndex].allApSupRates[i] & 0x7F, wone_info[woneIndex].maxActiveRate);

		if(((wone_info[woneIndex].allApSupRates[i] & 0x7F) == DOT11_RATE_1) ||
		   ((wone_info[woneIndex].allApSupRates[i] & 0x7F) == DOT11_RATE_2) ||
		   ((wone_info[woneIndex].allApSupRates[i] & 0x7F) == DOT11_RATE_5_5) ||
		   ((wone_info[woneIndex].allApSupRates[i] & 0x7F) == DOT11_RATE_11) ||
		   ((wone_info[woneIndex].allApSupRates[i] & 0x7F) == DOT11_RATE_22))			
		{
			if((wone_info[woneIndex].allApSupRates[i] & 0x7F) > wone_info[woneIndex].maxCCKactiveRate)
				wone_info[woneIndex].maxCCKactiveRate = wone_info[woneIndex].allApSupRates[i] & 0x7F;
		}
		   

		if (isBasic(wone_info[woneIndex].allApSupRates[i]) == TRUE)
		{
			/* Find min/max of basic rates only */
			wone_info[woneIndex].defaultActiveRate = 
                 max(wone_info[woneIndex].allApSupRates[i], wone_info[woneIndex].defaultActiveRate);

			switch (wone_info[woneIndex].allApSupRates[i] & 0x7F)
			{
				case DOT11_RATE_1:
					activeRateAcx = ACX_RATE_1;
					if(wone_info[woneIndex].maxCCKbasicRate < wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].maxCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
					if(wone_info[woneIndex].minCCKbasicRate > wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].minCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				case DOT11_RATE_2:
					activeRateAcx = ACX_RATE_2;
					if(wone_info[woneIndex].maxCCKbasicRate < wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].maxCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
					if(wone_info[woneIndex].minCCKbasicRate > wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].minCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				case DOT11_RATE_5_5:
					activeRateAcx = ACX_RATE_5_5;
					if(wone_info[woneIndex].maxCCKbasicRate < wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].maxCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
					if(wone_info[woneIndex].minCCKbasicRate > wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].minCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				case DOT11_RATE_6:
					activeRateAcx = ACX_RATE_6;
                    if(wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if(wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				case DOT11_RATE_9:
					activeRateAcx = ACX_RATE_9;
                    if(wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if(wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];

					break;

				case DOT11_RATE_11:
					activeRateAcx = ACX_RATE_11;
					if(wone_info[woneIndex].maxCCKbasicRate < wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].maxCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
					if(wone_info[woneIndex].minCCKbasicRate > wone_info[woneIndex].allApSupRates[i])
						wone_info[woneIndex].minCCKbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				case DOT11_RATE_12:
					activeRateAcx = ACX_RATE_12;
                    if(wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if(wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];

					break;

				case DOT11_RATE_18:
					activeRateAcx = ACX_RATE_18;
                    if(wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if(wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
					
					break;

				case DOT11_RATE_22:
					activeRateAcx = ACX_RATE_22;
					break;

				case DOT11_RATE_24:
					activeRateAcx = ACX_RATE_24;
                    if(wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if(wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];

					break;

				case DOT11_RATE_36:
					activeRateAcx = ACX_RATE_36;
                    if(wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if(wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				case DOT11_RATE_48:
					activeRateAcx = ACX_RATE_48;
                    if(wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if(wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				case DOT11_RATE_54:
					activeRateAcx = ACX_RATE_54;
                    if(wone_info[woneIndex].maxOFDMbasicRate < wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].maxOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
                    if(wone_info[woneIndex].minOFDMbasicRate > wone_info[woneIndex].allApSupRates[i])
                       wone_info[woneIndex].minOFDMbasicRate = wone_info[woneIndex].allApSupRates[i];
					break;

				default:
					wdrv_utilsReport(RL_DB, PRI_INFORMATION,
                                     "WDRV_DB: FATAL ERROR, Invalid default active rate.\n");
			}

			/* Find max of basic rates only in ACX units */
			wone_info[woneIndex].defaultActiveRateAcx = 
                max(activeRateAcx, wone_info[woneIndex].defaultActiveRateAcx);

			/* Set all basic rates in ACX units, used when fallback is enabled */
			wone_info[woneIndex].defaultActiveRateAcxWithFallback |= activeRateAcx;
		}
	}

	return OK;
}

/*************************************************************************
 *                        initMibTables                                  *
 *************************************************************************
DESCRIPTION: This function initializes wpa mib table. 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
static void	initMibTables(void)
{
	PAL_memSet(&dot11StationConfigTable, 0, sizeof(dot11StationConfigTable));
}

/*************************************************************************
 *                        wdrv_dbGetBssid                                  *
 *************************************************************************
DESCRIPTION: get bss id
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetBssid(char *value)
{
	PAL_memCopy(value, BSSID, MAC_ADDR_LEN);	
}

/*************************************************************************
 *                        wdrv_dbGetBssid                                  *
 *************************************************************************
DESCRIPTION:Returns the SSID 
			value is accepted to point to a buffer in len MAX_SSID_LENGTH + 1 

                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetSsid(char *value)
{
	PAL_memCopy(value, apSSID.serviceSetId, apSSID.hdr.eleLen);	
	*(value + apSSID.hdr.eleLen) = 0;
}


/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetHiddenSsidStatus(UINT16 *value)
{
    *value = apHiddenSSID;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetElementFromMultipleSsidActualTable(char *value, UINT16 index)
{
    if ((index == 0) || (index > apNumOfMultipleSsid)) 
    {
        *value = '\0';
        return;
    }
    PAL_memCopy(value,apMultipleSsidTable[index-1],WHAL_SSID_STRING_LEN);
}


/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbRemoveElementFromMultipleSsidTable(UINT8 index)
{

}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMultipleSsidStatus(UINT16 *value)
{
    *value = apMultipleSsidFlag;
}
/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetMultipleSsidStatus(UINT16 status)
{
}

 
/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetApCapabilities(UINT8 woneIndex, UINT16 *cap)
{
	PAL_memCopy(cap, &(wone_info[woneIndex].apCapabilities), FIX_FIELD_LEN);
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetApCapabilities(UINT8 woneIndex, UINT16 cap)
{
	PAL_memCopy(&(wone_info[woneIndex].apCapabilities), &cap, FIX_FIELD_LEN);
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetDefaultActiveRate(UINT8 woneIndex ,UINT8 *defaultRate)
{
	*defaultRate = wone_info[woneIndex].defaultActiveRate;
}




/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMaxCCKbasicRate(UINT8 woneIndex ,UINT8 *maxBasicCCKRate)
{
	*maxBasicCCKRate = wone_info[woneIndex].maxCCKbasicRate;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMinCCKbasicRate(UINT8 woneIndex ,UINT8 *minBasicCCKRate)
{
	*minBasicCCKRate = wone_info[woneIndex].minCCKbasicRate;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMaxOFDMbasicRate(UINT8 woneIndex ,UINT8 *maxBasicOFDMRate)
{
	*maxBasicOFDMRate = wone_info[woneIndex].maxOFDMbasicRate;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMinOFDMbasicRate(UINT8 woneIndex ,UINT8 *minBasicOFDMRate)
{
	*minBasicOFDMRate = wone_info[woneIndex].minOFDMbasicRate;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMaxCCKactiveRate(UINT8 woneIndex ,UINT8 *maxActiveCCKRate)
{
	*maxActiveCCKRate = wone_info[woneIndex].maxCCKactiveRate;	
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetDefaultActiveRateAcx(UINT8 woneIndex ,UINT16 *defaultRate, 
									UINT16 *defaultRateWithFallback, UINT8 pktType)
{
	if (pktType == TXATTR_BRCST_MTCST)
	{
		if (wone_info[woneIndex].Dot11Mode == DOT11_MODE_A_ONLY)
		{	
			*defaultRate = BRCST_MTCST_OFDM_ACX_RATE;
			*defaultRateWithFallback = BRCST_MTCST_OFDM_ACX_RATE;
		}
		else  
		{
			*defaultRate = BRCST_MTCST_ACX_RATE;
			*defaultRateWithFallback = BRCST_MTCST_ACX_RATE;
		}
	}
	else
	{
		*defaultRate = wone_info[woneIndex].defaultActiveRateAcx;
		*defaultRateWithFallback = wone_info[woneIndex].defaultActiveRateAcxWithFallback;
	}
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetMaxActiveRate(UINT8 *activeRate)
{
	*activeRate = wone_info[0].maxActiveRate;
}
/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbRestoreNvramDefaults(void)
{
}


/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetDynamicAlgorithmsTimeout(UINT32 *value)
{
	*value = DynamicAlgorithmsTimeout;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetDynamicAlgorithmsTimeout(UINT32 value)
{
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRadioCalibrateOption(UINT32 *value)
{
	*value = RadioCalibrateOption;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetRadioCalibrateOption(UINT32 value)
{
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRadioCalibrateInterval(UINT16 *value)
{
	*value = RadioCalibrateInterval;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetRadioCalibrateInterval(UINT16 value)
{
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPreambleMode(UINT8 woneIndex ,UINT8 *value)
{
	*value = wone_info[woneIndex].PreambleMode;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetPreambleMode(UINT8 woneIndex ,UINT8 value)
{
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetSlotTimeMode(UINT8 woneIndex ,UINT8 *value)
{
	*value = wone_info[woneIndex].SlotTimeMode;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetSlotTimeMode(UINT8 woneIndex ,UINT8 value)
{
}
	

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetSlotTimeThreshold(UINT16 *value)
{
	*value = SlotTimeThreshold;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetSlotTimeThreshold(UINT16 value)
{

}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetERPProtectionMode(UINT8 woneIndex ,UINT8 *value)
{
	*value = wone_info[woneIndex].ERPProtectionMode;	
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION: Sets the ERP Protection mode in runtime without updating the NVRAM
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/

void wdrv_dbSetERPprotectionMode(UINT8 woneIndex ,UINT8 value)
{	
	 
	 PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */
	 wone_info[woneIndex].ERPProtectionMode = value;
	 PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetERPProtectionType(UINT8 woneIndex ,UINT8 *value)
{
	*value = wone_info[woneIndex].ERPProtectionType;	
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetERPProtectionType(UINT8 woneIndex ,UINT8 value)
{
}


/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetDefaultWepKey(UINT8 index,UINT16 keySize,UINT8 *keyValue)
{

	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION - incase of restart workaround*/

    if(keyValue == NULL)
    {
        PAL_memSet(defaultWepKeys[index].WepKey,0,keySize);
        defaultWepKeys[index].keySize = 0;

    }else
    {
        PAL_memCopy(defaultWepKeys[index].WepKey,keyValue,keySize);
        defaultWepKeys[index].keySize = keySize;

    }
	
	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */

}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetDefaultWepKey(UINT8 index,UINT16 *keySize,UINT8 *keyValue)
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION - incase of restart workaround*/

    PAL_memCopy(keyValue,defaultWepKeys[index].WepKey,defaultWepKeys[index].keySize);
    *keySize = defaultWepKeys[index].keySize;

	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}
/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetWepDefaultKeyId(UINT8 *index)
{
    *index = wepDefaultKeyId;
}

void wdrv_dbGetPrivacyWEPDefaultKeyId(UINT8 *value)
{
    *value = wepDefaultKeyId;
}
/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetWepDefaultKeyId(UINT8 index)
{
    wepDefaultKeyId = index;
}
/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetWEP_DefaultKeyId(UINT8 *index)
{
	*index = wepDefaultKeyId;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetERPProtectionTI(UINT8 woneIndex ,UINT8 *value)
{
	*value = wone_info[woneIndex].ERPProtectionTI;	
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetERPProtectionTI(UINT8 woneIndex ,UINT8 value)
{
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetFeatureSet(UINT8 *value)
{
	*value = FeatureSet;	
}
 
/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetFeatureSet(UINT8 value)
{
}


/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetDot11Mode(UINT8 woneIndex ,UINT8 *value)
{
	*value = wone_info[woneIndex].Dot11Mode;	
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetDot11Mode(UINT8 woneIndex ,UINT8 value)
{ 
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPBCCAlgorithmMode(UINT8 woneIndex ,UINT8 *value)
{
	*value = wone_info[woneIndex].PbccAlgorithmMode;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetPBCCAlgorithmMode(UINT8 woneIndex ,UINT8 value)
{
}


/*************************************************************************
 *					wdrv_dbGetPowerLevel					                                  *
 *************************************************************************
DESCRIPTION:
 This is the 1st phase of implementation of Get PowerLevel. 
 In this phase there is no band involved and required, due  
 to a FW limitation (only one value can be sent down.)      
 Second phase (TBD..) will involve also the woneIndex, so   
 the interface will be probably changed (and so this note..) 
                                                                                                  
INPUT:  channel     
  
OUTPUT:  power level      

RETURN:  void    
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPowerLevel(UINT8 channel, UINT8 *value)
{	
    INT8  powerAdbm, poweBdbm, userTxPower, maxTxPower=0, txPowerPerLevel, band;
    UINT8 status, index, powerConst, step, ch;
    UINT8 currentRegDomain, numOfRanges, isPrivateDomain, userTxPowerLevel, TotalWones;
    tChannelRangeElementStruct rangeElementStruct;
    UINT32 level;

    wdrv_dbGetMaxTxPowerLevel(&userTxPowerLevel);
    wdrv_dbGetdot11MultiDomainCapabilityStatus(&status);

    if (status == 0)     /* 802.11d is unused */
    {
        *value = userTxPowerLevel;
        return;
    }
    else                 /* 802.11d is used */
    {
        /* Check power level for band A for given channel */
        powerAdbm = 0;
        wdrv_dbGetTxPowerLevels(RADIO_BAND_5GHZ, userTxPowerLevel, &userTxPower);
        wdrv_dbGetRegDomainInformation(WLAN_BAND_A, 
                                       &currentRegDomain, 
                                       &numOfRanges, 
                                       &isPrivateDomain);

        for (index = 1; index <= numOfRanges; index++) 
        {
            if (isPrivateDomain) 
                wdrv_dbGetPrivateRangeElement(WLAN_BAND_A,index,&rangeElementStruct);
            else
                wdrv_dbGetRangeElement(WLAN_BAND_A,
                                       (IntegerRegulatoryDomains_e)currentRegDomain,
                                       index,
                                       &rangeElementStruct);
            step = rangeElementStruct.stepOpt & 0xF;
            for (ch =  rangeElementStruct.startCh;
                 ch <= rangeElementStruct.stopCh;
                 ch += step) 
            {
                if (ch == channel)
                {
                    powerAdbm = rangeElementStruct.powerLimit;
                    break;
                }
            }
            if (powerAdbm > 0)
            {
                wdrv_dbGetdot11LocalPowerConstraint(&powerConst);
                powerAdbm -= powerConst;
                if (powerAdbm > userTxPower)
                    powerAdbm = userTxPower;
/*                if (powerAdbm < -128)
                    powerAdbm = -128;
                else if (powerAdbm > 127)
                    powerAdbm = 127;*/
                break;
            }
        }

        /* Check power level for band B for given channel */
        poweBdbm = 0;
        wdrv_dbGetTxPowerLevels(RADIO_BAND_2_4GHZ, userTxPowerLevel, &userTxPower);
        wdrv_dbGetRegDomainInformation(WLAN_BAND_B,
                                       &currentRegDomain, 
                                       &numOfRanges, 
                                       &isPrivateDomain);

        for (index = 1; index <= numOfRanges; index++) 
        {
            if (isPrivateDomain) 
                wdrv_dbGetPrivateRangeElement(WLAN_BAND_B,index,&rangeElementStruct);
            else
                wdrv_dbGetRangeElement(WLAN_BAND_B,
                                       (IntegerRegulatoryDomains_e)currentRegDomain,
                                       index,
                                       &rangeElementStruct);
            step = rangeElementStruct.stepOpt & 0xF;
            for (ch =  rangeElementStruct.startCh;
                 ch <= rangeElementStruct.stopCh;
                 ch += step) 
            {
                if (ch == channel)
                {
                    poweBdbm = rangeElementStruct.powerLimit;
                    break;
                }
            }
            if (poweBdbm > 0)
            {
                wdrv_dbGetdot11LocalPowerConstraint(&powerConst);
                poweBdbm -= powerConst;
                if (poweBdbm > userTxPower)
                    poweBdbm = userTxPower;
/*                if (poweBdbm < -128)   warning: comparison is always 0 due to limited range of data type
                    poweBdbm = 128;
                else if (poweBdbm > 127)
                    poweBdbm = 127;	*/
                break;
            }
        }
        /* Get the lower between the TxPowers of A and B */
        if ((powerAdbm == 0) && (poweBdbm == 0))
        {
            *value = 0;     /* channel not exists */
            return;
        }
        else if (powerAdbm == 0)   /* channel belongs to B band */
        {
            maxTxPower = poweBdbm;
            band = RADIO_BAND_2_4GHZ;
        }
        else if (poweBdbm == 0)   /* channel belongs to A band */
        {
            maxTxPower = powerAdbm;
            band = RADIO_BAND_5GHZ;
        }
        else                    /* channel contains in both A and B bands */
        {
            wdrv_dbGetNumSupWone(&TotalWones);
            if (TotalWones == WONE_SINGLE_BAND)
            {
                wdrv_dbGetBand(0,&band);
            }
            else
            {
                maxTxPower = min(powerAdbm,poweBdbm);
                band = (powerAdbm < poweBdbm) ? RADIO_BAND_5GHZ : RADIO_BAND_2_4GHZ;
            }
        }
    }
    /* Now we have the maxTxPower - should find the correct entry in power level tables. */
    /*level = 1;
    wdrv_dbGetTxPowerLevels(level, &txPowerPerLevel);
    if (maxTxPower >= txPowerPerLevel) 
    {
        *value = (UINT8)level;
        return;
    } */
    for (level=1; level <= 5; level++) 
    {
        wdrv_dbGetTxPowerLevels(band, level, &txPowerPerLevel);
        if (txPowerPerLevel <= maxTxPower) 
        {
            *value = (UINT8)level;
            return;
        }
    }
    /* txPower is below the minimum level */
	*value = 5;
}


/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRateFallback(UINT8 *value)
{
	*value = RateFallback;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetRateFallback(UINT8 value)
{
	
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRateAdaptation(UINT8 woneIndex, UINT8 *value)
{
	*value = wone_info[woneIndex].RateAdaptation;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetRateAdaptation(UINT8 woneIndex ,UINT8 value)
{
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGet4xEnable(BOOL *value)
{
	*value = enable4x;	
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSet4xEnable(BOOL value)
{
	enable4x = value;

	wdrv_dbSetCwMinWinSize(15);
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetConcatEnable(BOOL *value)
{
	*value = enableConcat;	
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetConcatEnable(BOOL value)
{
	enableConcat = value;
	
	if (value == TRUE)
		wdrv_4xMngr_EnableConcatenation();
	else
		wdrv_4xMngr_DisableConcatenation(); 
}
/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetAckEmulEnable(BOOL *value)
{
	*value = enableAckEmul;	
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetAckEmulEnable(BOOL value)
{
	enableAckEmul = value;
	
	if (value == TRUE)
		wdrv_4xMngr_EnableAckEmulation();
	else
		wdrv_4xMngr_DisableAckEmulation(); 
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCwMinMode(UINT8 woneIndex, UINT8 *value)
{
	*value = wone_info[woneIndex].CwMinMode;	
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCwMinWinSize(UINT16 *value)
{
	*value = apCWMinWindow;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetCwMinWinSize(UINT16 value)
{
	apCWMinWindow = value;
	
	/* pass the updated CWmin to the HAL */
	whal_apiSet4xCwMin(apCWMinWindow);
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetCwMinMode(UINT8 woneIndex, UINT8 value)
{
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetBssIsolationState(UINT8 value)
{
	bssBridgeIsolationOn = !(value == BSS_TRAFFIC_EN);
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetBssIsolationState(BOOL *value)
{
	*value = bssBridgeIsolationOn;
}


/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetTIInfoElement(char *value, UINT8 *len)
{
	*len = TIcap.hdr.eleLen;
	PAL_memCopy(value, TIcap.cap, TIcap.hdr.eleLen);	
}
/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void	wdrv_dbGet4xMixMode(BOOL *value)
{
   *value = enable4xMixMode; 
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSet4xMixMode(BOOL value)
{
//   whal_nvram_set4xMixMode( value, IMMEDIATELY );
   enable4xMixMode = value;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetWlanServiceState ( BOOL *state )
{
	*state = WlanServiceState;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetWlanServiceState ( BOOL state )
{
//	whal_nvram_setWlanServiceState( state, IMMEDIATELY );
   /*-------------------------------------------------------------*/
   /* Only in case we are going from OFF to ON,                   */
   /* let the wdrv_dbGetEnRestart retreive the saved transitions. */
	if ( !WlanServiceState && state ) 
	{
		RestoreSavedEnRestart = TRUE;
	}
	else
	{
		RestoreSavedEnRestart = FALSE;
	}
   /*-------------------------------------------------------------*/
	WlanServiceState = state;
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetEnRestart(UINT8 *value)
{
   	if (RestoreSavedEnRestart)
	{
	   /*------------------------------------*/
	   /* Return saved transitions only once */
		*value = SavedEnRestart;
		RestoreSavedEnRestart = FALSE;
	   /*------------------------------------*/
	}
	else
	{
        *value = EnRestart; 
    }
}

/*************************************************************************
 *										                                  *
 *************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetEnRestart(UINT8 value, BOOL updateNvram)
{
//   if (updateNvram)
//	  whal_nvram_setEnableRestartWA( value, IMMEDIATELY );
   if (WlanServiceState)
   {
      EnRestart = value;
   }
   else
   {
	    SavedEnRestart = value;
   }
}
/*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetWAoptions(UINT8 *rxFreeMem,UINT8 *bcnNotSent,UINT8 *txStuck,
						  UINT16 *consFcsErr, UINT8 *aci, UINT8 *nb)
{
   *rxFreeMem  = RxFreeMemOptionWA; 
   *bcnNotSent = BeaconNotSentOptionWA; 
   *txStuck    = TxStuckOptionWA; 
   *consFcsErr = ConsFcsErrOptionWA; 
   *aci		   = aciOptionWA;
   *nb		   = nbOptionWA;
}

/*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetWAoptions(UINT8 rxFreeMem,UINT8 bcnNotSent,UINT8 txStuck,
						  UINT16 consFcsErr, UINT8 aci, UINT8 nb)
{
}

/*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetEnergyDetect(UINT8 *value)
{
   *value = EnEnergyDetect; 
}

/*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetEnergyDetect(UINT8 value)
{
   EnEnergyDetect = value;
}


/*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRespRateMode(UINT8 *value)
{
	*value = apRespRateMode;
}

/*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetRespRateMode(UINT8 value)
{
}


/*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRadioFrequency(UINT8 *value)
{
    *value = RadioFrequency;
}

/*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetRadioFrequency(UINT8 value)
{
    RadioFrequency = value;
}


/*************************************************************************
 *                        wdrv_dbRatesSupport					         *
 *************************************************************************
DESCRIPTION: This function checks whether the AP supports the station rates or not                 
                                                                                                   
INPUT:       The station rates resides in one or two information elements

OUTPUT:      

RETURN:      OK : Rates supportted.
             NOK: Rates not supportted.                                                                                                                                             
************************************************************************/
BOOL wdrv_dbRatesSupport(UINT8 woneIndex, dot11_supRates_T *ratesP, dot11_extSupRates_T *extRatesP)
{
	UINT8 i, j;
	BOOL	rateFound;
	
	/* now, for every basic rate of the AP, find it in the station rates */
	for (i = 0; i < wone_info[woneIndex].allApSupRatesLen; i++)
	{
		if (isBasic(wone_info[woneIndex].allApSupRates[i]) == TRUE)
		{
		   rateFound = FALSE;
		
		   for (j = 0; j < ratesP->hdr.eleLen; j++)
		   {
			   if (wone_info[woneIndex].allApSupRates[i] == (ratesP->rates[j] | 0x80))
			   {
				   rateFound = TRUE;
				   break;
			   }
		   }

		   /* If rate was not found and the extended supported rates exist -> search it */
		   if ((extRatesP != NULL) && (rateFound == FALSE))
		   {
			   for (j = 0; j < extRatesP->hdr.eleLen; j++)
			   {
				   if (wone_info[woneIndex].allApSupRates[i] == (extRatesP->rates[j] | 0x80))
				   {
					   rateFound = TRUE;
					   break;
				   }
			   }
		   }

		   if (rateFound == FALSE)
			   return FALSE;
		}
	}
	return TRUE;
}

/*************************************************************************
DESCRIPTION:return true if rate is basic rate, i.e., the msb is set 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
static BOOL isBasic(UINT8 rate)
{
	if (rate & 0x80)
		return TRUE;
	else
		return FALSE;
}

/*************************************************************************
 *                        wdrv_dbSSIDSupport					         *
 *************************************************************************
DESCRIPTION: This function checks whether the AP supports the station SSID or not                 
                                                                                                   
INPUT:       The station SSID as received in the assoc/reassoc request.

OUTPUT:      

RETURN:      OK : SSID supportted.
             NOK: SSID not supportted.                                                                                                                                             
************************************************************************/
BOOL wdrv_dbSSIDSupport(dot11_SSID_T	*ssid)
{
	BOOL	compare = FALSE;
    int     index;

	if ((ssid->hdr.eleLen) != (apSSID.hdr.eleLen))
    {
        if (apMultipleSsidFlag == FALSE)
		return FALSE;
    }
    else    /* length matches */
		compare = (!PAL_memCmp(ssid->serviceSetId, apSSID.serviceSetId, apSSID.hdr.eleLen));
    if (compare == TRUE)
        return TRUE;
    if (apMultipleSsidFlag == FALSE)
        return FALSE;

    /* Check Multiple SSID */
    for (index = 0; index < NUM_MULTIPLE_SSID; index++) 
    {
        if (apMultipleSsidTable[index][0] == '\0')  /* end of table */
            return FALSE;

        if ((ssid->hdr.eleLen) != strlen(apMultipleSsidTable[index]))
            continue;
        compare = (PAL_memCmp(ssid->serviceSetId, apMultipleSsidTable[index], ssid->hdr.eleLen)==0);
        if (compare)
            return TRUE;
    }
	return FALSE;
}

/*************************************************************************
 *                        wdrv_dbAuthAlgoSupported					     *
 *************************************************************************
DESCRIPTION: This function checks whether the AP supports the authentication algorithm                 
                                                                                                   
INPUT:       The authentication algorithm as received in the authentication frame.

OUTPUT:      

RETURN:      OK : Algorithm supportted.
             NOK: Algorithm not supportted.                                                                                                                                             
************************************************************************/
BOOL wdrv_dbAuthAlgoSupported(UINT16 algoNumber)
{
	int i;
	UINT16	algo;
	BOOL	enable;

	for (i = 0; i < MAX_AUTH_ALGORITHM_SUPPORTED; i++)
	{
		wdrv_dbGetAuthAlgo(i, &algo);
		wdrv_dbGetAuthAlgoEnable(i, &enable);
		
		if ((algo == algoNumber + 1) && (enable == TRUE))
			return TRUE;
	}
	return FALSE;
}


/*************************************************************************
DESCRIPTION: Returns TRUE if input MAC address is 
			 equal to the AP's MAC address, otherwise, returns FALSE. 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
BOOL wdrv_dbApSupport(char *mac)
{
	BOOL	compare;

	compare = (!PAL_memCmp(mac, APMAC, MAC_ADDR_LEN));

	return compare;
}

/*************************************************************************
 *                        wdrv_dbCapabilitiesSupport			         *
 *************************************************************************
DESCRIPTION: This function checks whether the AP supports the station
			 capabilities as received in the association request.
			 Right now it checks the provacy bit only.	
                                                                                                   
INPUT:       The station capabilities.

OUTPUT:      

RETURN:      OK : Capabilities supportted.
             NOK: Capabilities supportted.                                                                                                                                             
************************************************************************/
BOOL wdrv_dbCapabilitiesSupport(UINT8 woneIndex ,UINT16	*caps)
{
	
	/* if AP & station are equal in their privacy bit, associate the station. */
	if (((wone_info[woneIndex].apCapabilities >> CAP_PRIVACY_SHIFT) & CAP_PRIVACY_MASK) ==
        (((*caps) >> CAP_PRIVACY_SHIFT) & CAP_PRIVACY_MASK))
		return TRUE;

	/* if the station has privacy, and the AP does not have, don't associate the station */
	if (((*caps >> CAP_PRIVACY_SHIFT) & CAP_PRIVACY_MASK) &&
         !((wone_info[woneIndex].apCapabilities >> CAP_PRIVACY_SHIFT) & CAP_PRIVACY_MASK))
		return FALSE;

	return TRUE;

}


/*************************************************************************
 *                        wdrv_dbPrivacySupport			         *
 *************************************************************************
DESCRIPTION: This function checks whether the AP supports provacy 
			 capabilities 	
                                                                                                   
INPUT:       The Wone Index.

OUTPUT:      

RETURN:      OK : Capabilities supportted.
             NOK: Capabilities supportted.                                                                                                                                             
************************************************************************/
void wdrv_dbPrivacySupport(UINT8 woneIndex ,BOOL *value)
{

    if (((wone_info[woneIndex].apCapabilities >> CAP_PRIVACY_SHIFT) & CAP_PRIVACY_MASK))
        *value = OK;
    else
        *value = NOK;
}


/***********************************************************************************
 The following functions implement the MIB. Each group is correspond to a mib table 
************************************************************************************/	


/*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigAuthRspTimeout(UINT32 *value)
{
	*value = 2000;
}

/*************************************************************************
DESCRIPTION:
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigPrivacyOptionImplemented(BOOL *value)
{
	*value = privacyOptionImplemented;
}


/*************************************************************************
DESCRIPTION:  802.11b rates 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigOperationalRateSet(UINT8 woneIndex, char *value, UINT8 *len)
{
	*len = wone_info[woneIndex].apRates.hdr.eleLen;       
   	PAL_memCopy(value, wone_info[woneIndex].apRates.rates, wone_info[woneIndex].apRates.hdr.eleLen);	
}

/*************************************************************************
DESCRIPTION:  802.11g OFDM rates 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigOperationalOFDMRateSet(UINT8 woneIndex, char *value, UINT8 *len)
{
	*len = wone_info[woneIndex].apOFDMRates.hdr.eleLen;
	PAL_memCopy(value, wone_info[woneIndex].apOFDMRates.rates, wone_info[woneIndex].apOFDMRates.hdr.eleLen);	
}

/*************************************************************************
DESCRIPTION:  802.11g all rates 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigOperationalAllRateSet(UINT8 woneIndex, char *value, UINT8 *len)
{
	*len = wone_info[woneIndex].allApSupRatesLen;
	PAL_memCopy(value, wone_info[woneIndex].allApSupRates, wone_info[woneIndex].allApSupRatesLen);	
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigBeaconPeriod(UINT16 *value)
{
	*value = apBeaconPeriod;
}
/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigDtimPeriod(UINT8 *value)
{
	*value = apDtimPeriod;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigDisassociateReason(UINT16 *value)
{
	*value = dot11StationConfigTable.disassociateReason;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigDisassociateStation(char *value)
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */
	PAL_memCopy(value, dot11StationConfigTable.disassociateStation, MAC_ADDR_LEN);
  	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigDeauthenticateReason(UINT16 *value)
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */
	*value = dot11StationConfigTable.deauthenticateReason;
  	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigDeauthenticateStation(char *value)
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */
	PAL_memCopy(value, dot11StationConfigTable.deauthenticateStation, MAC_ADDR_LEN);
  	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigDeauthenticateFailStatus(UINT16 *value)
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */
	*value = dot11StationConfigTable.deauthenticateFailStatus;
  	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetStationConfigeauthenticateFailStation(char *value)
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */
	PAL_memCopy(value, dot11StationConfigTable.deauthenticateFailStation, MAC_ADDR_LEN);
  	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}


/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigAuthRspTimeout(UINT32 value)
{
	
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigPrivacyOptionImplemented(BOOL value)
{
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigOperationalRateSet(UINT8 woneIndex, char *value, UINT8 len)
{
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigOperationalAdditionalRateSet(UINT8 woneIndex, char *value, UINT8 len)
{
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigOperationalOFDMRateSet(UINT8 woneIndex, char *value, UINT8 len)
{
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigBeaconPeriod(UINT16 value)
{
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigDtimPeriod(UINT8 value)
{
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigDisassociateReason(UINT16 value)
{
	dot11StationConfigTable.disassociateReason = value;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigDisassociateStation(char *value)
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */ 
	PAL_memCopy(dot11StationConfigTable.disassociateStation, value, MAC_ADDR_LEN);
  	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigDeauthenticateReason(UINT16 value)
{
	dot11StationConfigTable.deauthenticateReason = value;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigDeauthenticateStation(char *value)
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */
	PAL_memCopy(dot11StationConfigTable.deauthenticateStation, value, MAC_ADDR_LEN);
  	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigDeauthenticateFailStatus(UINT16 value)
{
	dot11StationConfigTable.deauthenticateFailStatus = value;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetStationConfigDeauthenticateFailStation(char *value)
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */
	PAL_memCopy(dot11StationConfigTable.deauthenticateFailStation, value, MAC_ADDR_LEN);
  	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}

/**********************************************************************
	Authentication Algorithms table
************************************************************************/

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetAuthAlgo(UINT32 index, UINT16 *value)
{
	if (index == authOpenSystemIndex)
		*value = MIB_OPEN_SYSTEM;
	else if (index == authSharedKeyIndex)
		*value = MIB_SHARED_KEY;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetAuthAlgoEnable(UINT32 index, BOOL *value)
{
	if (index == authOpenSystemIndex)
		*value = authOpenSystemEnable;
	else if (index == authSharedKeyIndex)
		*value = authSharedKeyEnable;
}


/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetAuthAlgo(UINT32 index, UINT16 value)
{
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetAuthAlgoEnable(UINT32 index, BOOL value)
{
}


 
/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPrivacyInvoked(BOOL *value)
{
	*value = privacyInvoked;	
}


/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPrivacyExcludeUnencrypted(BOOL *value)
{
	*value = excludeUnencrypted;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPrivacyWEPExcludedCount(UINT32 *value)
{
	* value = wdrv_bssbridgeGetExcludedCount();
}



/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetPrivacyInvoked(BOOL value)
{
	privacyInvoked = value;
	wdrv_txSetPrivacyInvoked(value);
}


/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetPrivacyExcludeUnencrypted(BOOL value)
{
	excludeUnencrypted = value;	
	whal_apiSetWepExcludeUnencrypted(value);
	wdrv_bssbridgeSetExcludeUnencrypted(value);
}

/*************************************
	MultiDomainCapability table
**************************************/

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetChannel(UINT8 woneIndex ,UINT8 *value)
{	
	*value = wone_info[woneIndex].channel;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetChannel(UINT8 woneIndex ,UINT8 value)
{
	wone_info[woneIndex].channel = value;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetBand(UINT8 woneIndex ,UINT8 *value)
{	
	*value = wone_info[woneIndex].band;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetBand(UINT8 woneIndex ,UINT8 value)
{
}
/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPrimaryAcxQindex(UINT8 woneIndex ,UINT8 *value)
{	
	*value = wone_info[woneIndex].primaryAcxQindex;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetPrimaryAcxQindex(UINT8 woneIndex ,UINT8 value)
{
    wone_info[woneIndex].primaryAcxQindex = value;
}



/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPsBroadcastQindex(UINT8 woneIndex ,UINT8 *value)
{	
	*value = wone_info[woneIndex].psBroadcastQindex;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetPsBroadcastQindex(UINT8 woneIndex ,UINT8 value)
{
    wone_info[woneIndex].psBroadcastQindex = value;
}


/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetUsageTime(UINT8 woneIndex ,UINT8 *value)
{	
	*value = wone_info[woneIndex].UsageTime;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetUsageTime(UINT8 woneIndex ,UINT8 value)
{
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbUpdateUsageTime(UINT8 woneIndex ,UINT8 value)
{
    wone_info[woneIndex].UsageTime = value;
}




/*************************************
	Operation table
**************************************/

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetOperationMacAddress(char *value)
{
	PAL_memCopy(value, APMAC, MAC_ADDR_LEN);	
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetOperationRTSThreshold(UINT16 *value)
{
	*value = apRtsThreshold;	
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetOperationShortRetryLimit(UINT8 *value)
{
	*value = apShortRetryLimit;	
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetOperationLongRetryLimit(UINT8 *value)
{
	*value = apLongRetryLimit;	
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetOperationFragmentationThreshold(UINT16 *value)
{
	*value = apFragThreshold;	
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetOperationMaxTransmitMSDULifetime(UINT32 *value)
{	
	*value = apMaxTxLifetime;	
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetOperationMaxReceiveLifetime(UINT32 *value)
{
	*value = apMaxRxLifetime;	
}


/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetOperationMacAddress(char *value)
{
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetOperationRTSThreshold(UINT16 value)
{
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetOperationShortRetryLimit(UINT8 value)
{
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetOperationLongRetryLimit(UINT8 value)
{
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetOperationFragmentationThreshold(UINT16 value)
{
}
/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetOperationMaxTransmitMSDULifetime(UINT32 value)
{
}
/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetOperationMaxReceiveLifetime(UINT32 value)
{
}

/*************************************
	Counters table
**************************************/

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersTransmittedFragmentCount(UINT32 *value)
{
	whal_apiGetTxFragmentCount(0,value);
}
/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersMulticastTransmittedFrameCount(UINT32 *value)
{
	wdrv_txGetTxMulticastCount(value);
}
/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersFailedCount(UINT32 *value)
{
	whal_apiGetFailedCount(0,value);
}
/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersRetryCount(UINT32 *value)
{
	whal_apiGetRetryCount(0,value);	
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersMultipleRetryCount(UINT32 *value)
{
	whal_apiGetMultipleRetryCount(0,value);	
}
/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersFrameDuplicateCount(UINT32 *value)
{
	wdrv_rxGetCountersFrameDuplicateCount(value);
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersRTSSuccessCount(UINT32 *value)
{
	whal_apiGetRtsSuccCount(0,value);
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersRTSFailureCount(UINT32 *value)
{
	whal_apiGetRtsFailCount(0,value);
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersACKFailureCount(UINT32 *value)
{
	whal_apiGetAckFailCount(0,value);	
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersReceivedFragmentCount(UINT32 *value)
{
	wdrv_rxGetReceivedFragmentCount(value);
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersMulticastReceivedFrameCount(UINT32 *value)
{
	wdrv_rxGetMulticastReceivedFrameCount(value);
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersFCSErrorCount(UINT32 *value)
{
	whal_apiGetFcsError(value);	
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersTransmittedFrameCount(UINT32 *value)
{
	whal_apiGetTxFrameCount(0,value);	
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetPBCCdynamicStatus(BOOL status)
{
	DynamicAlgInfo.pbccValid = status;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPBCCdynamicStatus(BOOL *status)
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */
	*status = DynamicAlgInfo.pbccValid;
	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetERPdynamicStatus(BOOL status)
{
	DynamicAlgInfo.erpValid = status;
}
/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetERPdynamicStatus(BOOL *status)
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */
	*status = DynamicAlgInfo.erpValid;
	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}
/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetNotPBCCTimestamp(UINT32 timestamp)
{
	DynamicAlgInfo.pbccTimeStamp = timestamp;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetNotPBCCTimestamp(UINT32 *timestamp)
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */
	*timestamp = DynamicAlgInfo.pbccTimeStamp;
	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetRxTxBrate(UINT32 timestamp, UINT32 count)
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */
	DynamicAlgInfo.RxTxBrateTimestamp = timestamp;
	if (count != 0)
	{
		DynamicAlgInfo.RxTxBrateCount += count;
	}
	else
	{
		DynamicAlgInfo.RxTxBrateCount = 0;
	}
	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRxTxBrate(UINT32 *timestamp, UINT32 *count)
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */
	*timestamp = DynamicAlgInfo.RxTxBrateTimestamp;
	*count = DynamicAlgInfo.RxTxBrateCount;
	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetNotErpTimestamp(UINT32 timestamp)
{
	DynamicAlgInfo.erpTimeStamp = timestamp;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetNotErpTimestamp(UINT32 *timestamp)
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */
	*timestamp = DynamicAlgInfo.erpTimeStamp;
	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}


/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetNumSupWone(UINT8 value, BOOL rebootNow)
{
}


/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetNumSupWone(UINT8 * TotalWones)
{
	*TotalWones = numSupWone;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetNumSupWoneQueues	(UINT8 woneIndex, UINT8 * WoneTxQueues )
{
	*WoneTxQueues = wone_info[woneIndex].queue_table.numOfQueue;
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetAcxQindex	(	UINT8 		woneIndex,
								UINT8 		localQueueId,
								UINT8 *		pType,
								UINT8 *		pPriority	)
{                                                                                                                                              

    *pType = (UINT8)(wone_info[woneIndex].queue_table.queue_info[localQueueId].type);
	
	*pPriority	=	(UINT8)(wone_info[woneIndex].queue_table.queue_info[localQueueId].Priority);
}

/*************************************************************************
DESCRIPTION:  
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetAcxGlobalQindex(UINT8 woneIndex,int localQueueType ,int *globalQid)
{
    UINT8 localQid ;
	
    localQid =  localQueueType;

    if (wone_info[woneIndex].queue_table.numOfQueue == 2 /* NO Vhcf */)
    {
        if ((localQueueType == WONE_HIGH_PRIOR_QUEUE) ||
            (localQueueType == WONE_PS_BCAST_QUEUE))
        {
            localQid --;
        }
    }
    *globalQid = wone_info[woneIndex].queue_table.queue_info[localQid].globalQid;
}



/************************************************************************************
 *                        wdrv_dbSetVhcfParams                                      *
 ************************************************************************************
  DESCRIPTION: Configure vHCF parameters (enable/disable and classifiers).
 ************************************************************************************/
void wdrv_dbSetVhcfParams(tVHCF_PARAMS * vhcfParamsPtr)
{
  UINT8  idx;

  vhcfParams.vhcfEnable = vhcfParamsPtr->vhcfEnable;
  for(idx = 0; idx < MAX_VHCF_CLASSIFIERS ; idx++)
  {
	  vhcfParams.vhcfClassifier[idx].Protocol = vhcfParamsPtr->vhcfClassifier[idx].Protocol;
	  vhcfParams.vhcfClassifier[idx].DestPort = vhcfParamsPtr->vhcfClassifier[idx].DestPort;
	  vhcfParams.vhcfClassifier[idx].DestIp   = ntohl(vhcfParamsPtr->vhcfClassifier[idx].DestIp);

  }
}

/************************************************************************************
 *                        wdrv_dbGetVhcfParams                                      *
 ************************************************************************************
  DESCRIPTION: Provide the current vHCF parameters (enable/disable and classifiers).
 ************************************************************************************/
void wdrv_dbGetVhcfParams(tVHCF_PARAMS * vhcfParamsPtr)
{
  UINT8  idx;

  vhcfParamsPtr->vhcfEnable = vhcfParams.vhcfEnable;
  for(idx = 0; idx < MAX_VHCF_CLASSIFIERS ; idx++)
  {
	  vhcfParamsPtr->vhcfClassifier[idx].Protocol = vhcfParams.vhcfClassifier[idx].Protocol;
	  vhcfParamsPtr->vhcfClassifier[idx].DestPort = vhcfParams.vhcfClassifier[idx].DestPort;
	  vhcfParamsPtr->vhcfClassifier[idx].DestIp   = vhcfParams.vhcfClassifier[idx].DestIp;
  }
}

/************************************************************************************
 *                        wdrv_dbIsVhcfEnabled                                      *
 ************************************************************************************
  DESCRIPTION: Return TRUE if vHCF is currently enabled, FALSE if not.
 ************************************************************************************/
BOOL wdrv_dbIsVhcfEnabled(void)
{
  return vhcfParams.vhcfEnable;
}


/************************************************************************************
 *                        wdrv_dbGetEdcfParams                                      *
 ************************************************************************************
  DESCRIPTION:  Provide the requested tx queue EDCF parameters. 
				Return FALSE if not required (vHCF disabled or PS-Broadcast queue).
 ************************************************************************************/
BOOL wdrv_dbGetEdcfParams(UINT8 woneIndex, UINT8 localQueueId, edcf_info_t * edcfInfoPtr)
{
	if (wone_info[woneIndex].queue_table.queue_info[localQueueId].edcfInfoRequired)
	{
	    PAL_memCopy( (void *)edcfInfoPtr,
				(void *)&(wone_info[woneIndex].queue_table.queue_info[localQueueId].edcfInfo),
				sizeof(edcf_info_t) );
		return TRUE;
	}
	else
		return FALSE;
}


/************************************************************************************
 *                        wdrv_dbGetQueueThresh                                     *
 ************************************************************************************
  DESCRIPTION:  Provide the requested tx queue buffers usage limit (in 5% units). 
				Return FALSE if not required (vHCF disabled or not the low priority queue).
 ************************************************************************************/
BOOL wdrv_dbGetQueueThresh(UINT8 woneIndex, UINT8 localQueueId, UINT32 * thresholdPtr)
{
	if (wone_info[woneIndex].queue_table.queue_info[localQueueId].threshold)
	{
		*thresholdPtr = wone_info[woneIndex].queue_table.queue_info[localQueueId].threshold;
		return TRUE;
	}
	else
		return FALSE;
}

/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRsne(dot11_rsne_T *rsne)	
{
	PAL_spinLock(&drvDbMainLock); /* START OF CRITICAL SECTION */
	PAL_memSet(rsne, 0, sizeof(dot11_rsne_T));	
	rsne->hdr.eleId = DOT11_RSNE_MS_ELE_ID;
	rsne->hdr.eleLen = apRsne.hdr.eleLen;
/*	printf("############################ RSN IE len = %d\n", rsne->hdr.eleLen);*/
	PAL_memCopy(rsne->rsne, apRsne.rsne, apRsne.hdr.eleLen);	

  	PAL_spinUnLock(&drvDbMainLock);  /* END OF CRITICAL SECTION */
}

/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetRsne(UINT8 len, UINT8 *pInfo)
{
	PAL_memSet(&apRsne, 0, sizeof(dot11_rsne_T));	
	if (pInfo != NULL)
		PAL_memCopy(&apRsne, pInfo, len);
}

/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetBroadcastSuiteType(UINT8 *cipher)
{
	*cipher = broadcastCipherSuiteType;	
}

/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetBroadcastSuiteType(UINT8 cipher)
{
	broadcastCipherSuiteType = cipher;	
}




/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetWpaMode(UINT32 mode)	
{
}

/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetWpaMode(UINT32 *mode)
{	
	*mode = wpaMode;
}


/*************************************************************************
DESCRIPTION: 
 This routine is called by the various primitives in order to check whether a Privacy is activated 
 in the AP. For example: If Privacy is enabled, then the Multiple SSID should not enabled.         
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
BOOL wdrv_dbIsPrivacyEnabled(void)
{
	UINT32 mode;
    BOOL option;

    wdrv_dbGetWpaMode(&mode);

	if (mode & DOT11_WPA_STA_TYPE_WEP_LEGACY)
    {
        wdrv_dbGetStationConfigPrivacyOptionImplemented(&option);
        if (option)
            return TRUE;
    }
	if (mode & DOT11_WPA_STA_TYPE_WEP_8021X)
		return TRUE;
	if (mode & DOT11_WPA_STA_TYPE_WPA)
		return TRUE;

    return FALSE;
}



/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetUsageTimeMode(UINT8 value, BOOL rebootNow)	
{
}

/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetUsageTimeMode(UINT8 *value)
{	
	*value = usageTimeMode;
}

/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRadioParams( RadioParam_t *ptr)
{
   PAL_memCopy( ptr, &dbRadioParams, sizeof(dbRadioParams));
}/* END wdrv_dbGetRadioParams() */

/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetRadioParams( RadioParam_t *ptr, UINT32 parameterType, BOOL updateNvram)
{
   switch( parameterType)
   {
   case RADIO_PARAM_OVERRIDE_BMASK:
      dbRadioParams.overrideBitmask = ptr->overrideBitmask;
      break;
   case RADIO_PARAM_POWER_TABLE:
      dbRadioParams.powerTable_paramType = ptr->powerTable_paramType;
      PAL_memCopy( dbRadioParams.powerTable_param,
         ptr->powerTable_param,
         sizeof( dbRadioParams.powerTable_param));
      break;
   default:
      break;
   }

}/* END wdrv_dbSetRadioParams() */

/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetCountersWEPUndecryptableCount(UINT32 *val)
{
	*val = whal_ParamsGetWepParams()->dot11WEPUndecryptableCount;
}
/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPrivacyWEPICVErrorCount(UINT32 *val)
{
	*val = whal_ParamsGetWepParams()->dot11WEPICVErrorCount;
}

/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetPrivacyTKIPMicFailure(UINT32 *val)
{
	*val = wdrv_bssbridgeGetTKIPMicFailure();
}


/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetRadarDetectMode(UINT8 val)
{
}

/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetRadarDetectMode(UINT8 *val)
{
	*val = RadarDetectMode;
}

/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbSetBurstLimit(UINT16 val)
{
}

/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
void wdrv_dbGetBurstLimit(UINT16 *val)
{
	*val = txopLimit;
}


/*************************************************************************
DESCRIPTION: 
                                                                                       
INPUT:       

OUTPUT:      

RETURN:      
                                                                                                                                                          
************************************************************************/
UINT8 wdrv_dbGetApBridgeMode(void)
{
	return ApBridgeMode;
}


void wdrv_dbSetAciStateTimeout(UINT32 timeout)
{
}
void wdrv_dbGetAciStateTimeout( UINT32 *timeout)
{
	*timeout = aciStateTimeout;
}

void wdrv_dbSetNbStateTimeout(UINT32 timeout)
{
}
void wdrv_dbGetNbStateTimeout( UINT32 *timeout)
{
	*timeout = nbStateTimeout;
}

void wdrv_dbSetAciRateThreshold(UINT32 thresh)
{
}

void wdrv_dbGetAciRateThreshold( UINT32 *thresh)
{
	*thresh = aciRateThreshold;
}

void wdrv_dbSetAciEventsThreshold(UINT32 thresh)
{
}

void wdrv_dbGetAciEventsThreshold( UINT32 *thresh)
{
	*thresh = aciEventThreshold;
}


void wdrv_dbSetPacketDetectHighThresh(UINT16 pdHighThresh)
{
}

void wdrv_dbGetPacketDetectHighThresh(UINT16 *pdHighThresh)
{
	*pdHighThresh = pdUnderAciHighThrsh;
}

void wdrv_dbSetPacketDetectLowThresh(UINT16 pdLowThresh)
{
}

void wdrv_dbGetPacketDetectLowThresh(UINT16 *pdLowThresh)
{
	*pdLowThresh = pdUnderAciLowThrsh;
}

void wdrv_dbSetMediumBusyThresh(UINT32 mediumBusyThresh)
{
}
void wdrv_dbGetMediumBusyThresh(UINT32 *thresh)
{
	*thresh = mediumBusyThresh;
}

