/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																									*/
/*		MODULE:	mainsm.c																		*/
/*    PURPOSE:	Driver's main state machine											*/
/*																									*/
/***************************************************************************/
#include "pform_types.h"
#include "mainsm.h"
#include "mgmtSm.h"
#include "tx.h"
#include "rx.h"
/*#include "whal.h"*/
#include "drvDbInit.h"
#include "WoneBalance.h"
#include "bssbridge.h"
#include "rx.h"
#include "4xMngr.h"
#include "drvDbMain.h"
#include "whalApi.h"
#include "whalRxMaster.h"
#include "dda_contexts.h"
#include "dda.h"
#include "drvDbRegDomain.h"
#include "dda_wpa_interface.h"


/* updated during initialization */
static UINT32 UpModulesToGo = 0;


/* Configuration parameters for the driver */
WDRV_CONFIG	wdrvConfig;

PALSRV_TcbObjTYPE RestartWatchDogTimer;

extern void whal_txMasterUnload(void);
extern STATUS whal_apiRestartCheckIfNeeded(watchdogFailCB_T coreCB);
extern int whal_acxConfigWoneConfig (void);

static void wdrv_RestartHandler(long type);
static void wdrv_RestartTimerCB(UINT32 data);
static void wdrv_RestartWatchDog_Start(void);
static void wdrv_RestartWatchDog_Stop(void);
STATUS 		mainsmInit(void);
static void mainsmUnload(void);
static STATUS wdrv_check_channel(UINT8 wone_index,UINT8 channel);
static BOOL isChannelInSafeRadarList(UINT8 channel);


/***************************************************************************
*  radar detection alg. defs
***************************************************************************/
#include "public_dcs.h"
#define NUMBER_OF_RADARS                 3 
#define RESOLUTION		                 20
#define EVENET_DIFF_VECTOR_SIZE          NUM_OF_DCS_SAMPLES
#define CHANNEL_SELC_WAIT_FOR_REPLY_TIME 61000
#define RADAR_DET_TEST_ONLY
/*#define RADAR_DET_DEMO_MAILBOX*/
#ifdef RADAR_DET_DEMO_MAILBOX
UINT16 tst_time_deff_vec[] = {1500,1500,1500,1500,1500,1430,1500,1440,1500,1500,
						      1500,1500,1500,1500,1500,1500,1500,1500,1500,1500,
						      1500,1500,1500,1500,1500,1500,1500,1500,1500,1500
							};
const UINT16  tst_event_num  = 30;
#endif  

UINT16  const Radar_Minimal_Number_OF_PULSES[NUMBER_OF_RADARS] = {9,5,35}; //50% of number of expected pulses
UINT32  const Radar_Interval[NUMBER_OF_RADARS] = {1429,556,3030};  //usec
#define DEFAULT_RADAR_SAFE_CHANNEL        36
#define SAFE_RADAR_FREQUANCY_START_RANGE  5150
#define SAFE_RADAR_FREQUANCY_END_RANGE    5250


/******************************************************************************
*                      mainsmTask()											  *
*******************************************************************************
DESCRIPTION:	Implementation of the driver main state machine.    
                                                                         
INPUTS:			None.

OUTPUT:			None

RETURNS:			None.                                                    
******************************************************************************/

STATUS wdrv_mainSm_getEvent(mainsmMessage_T  *pMessageInfo)
{

     switch (pMessageInfo->type) 
	 {

		case WDRV_MAINSM_MODULE_UP:
			MODULE_CLEAR_BIT(UpModulesToGo, pMessageInfo->content.MODULE_up.moduleID);
			break;

		case WDRV_MAINSM_RESTART:
			wdrv_RestartHandler(pMessageInfo->type);
			break;

        case WDRV_MAINSM_WONE_BALANCE:
           wdrv_PeriodicWoneSliceUpdate();
           break;

		case WDRV_MAINSM_ACTIVATE_RESTART:
		   wdrv_ActivateRestart();
		   break;

		case WDRV_MAINSM_ENABLE_WLAN_SERVICE:
		   wdrv_RestartHandler(pMessageInfo->type);
		   break;
           
        default:
		   printf("mainsm_getEvent:receive error type %d",(int)pMessageInfo->type);	
		   return NOK; 
		   break;
	 }

	 return OK;
}


/***************************************************************************
*main state machine module init function												*
***************************************************************************/
STATUS mainsmInit(void) 
{
	if(REPORT_LEVEL(RL_MAINSM)) 
		 wdrv_utilsReport(RL_MAINSM, PRI_INFORMATION, "WDRV_MAINSM: Main state machine Initialization ended OK.\n\n");

   return OK;
}

/***************************************************************************
*main state machine module unload function												*
***************************************************************************/
void mainsmUnload(void)
{
	PALSRV_tcbDelete(&RestartWatchDogTimer);
	printf("mainSm unloaded !!!\n");
}

/***************************************************************************
* Send message to mainsm message queue.												*
***************************************************************************/
STATUS wdrv_mainsmSendModuleUp(int id) {
   
   mainsmMessage_T MessageInfo;

   MessageInfo.type = WDRV_MAINSM_MODULE_UP;
   MessageInfo.content.MODULE_up.moduleID = id;

   wdrv_DDA_mainSm_setEvent(&MessageInfo);
   
   return OK;
}

/***************************************************************************
* Send message to mainsm message queue.												*
***************************************************************************/
STATUS wdrv_EnableWlanService(void) 
{
	mainsmMessage_T MessageInfo;

	MessageInfo.type = WDRV_MAINSM_ENABLE_WLAN_SERVICE;

	wdrv_DDA_mainSm_setEvent(&MessageInfo);

	return OK;
}


/***************************************************************************
* Driver init function																		*
***************************************************************************/
int wdrv_init(initConfigStruct *pInitConfig) 
{

	int rc;
	unsigned long retriesLeft = 5;

	do	
	{		
		/* DATABASE INITIALIZATION MUST BE BEFORE THE OTHER MODULES */
		MODULE_ADD_BIT(UpModulesToGo,WDRV_DB_MODULE_ID);

		rc = wdrv_dbInit(pInitConfig);
		if (rc != OK)
		{
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "Could not initialize WLAN driver database\n");
			break;
		}
		else
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "WLAN driver database is up\n");

#if 0
        /* Ack Emulation DataBase Initialization */
        wdrv_aeDbInit();
#endif      		

		MODULE_ADD_BIT(UpModulesToGo,WDRV_HAL_MODULE_ID);

		rc = whal_apiInit((whal_UsrInitFuncCB)wdrv_mainsmSendModuleUp,  WDRV_HAL_MODULE_ID);
		if (rc != OK)
		{
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "Could not initialize WLAN HAL layer \n");
			break;
		}
		else
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "WLAN HAL layer is up\n");		

		MODULE_ADD_BIT(UpModulesToGo,WDRV_BSS_BRIDGE_MODULE_ID);

		rc = wdrv_bssbridgeInit();
		if (rc != OK)
		{
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "Could not initialize WLAN driver BssBridge\n");
			break;
		}
		else
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "BssBridge is up\n");		

		MODULE_ADD_BIT(UpModulesToGo, WDRV_MGMT_MODULE_ID);		

		rc = wdrv_mgmtInit();
		if (rc != OK)
		{
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "Could not initialize WLAN driver management\n");
			break;
		}
		else
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "Mgmt is up\n");

		MODULE_ADD_BIT(UpModulesToGo, WDRV_RX_MODULE_ID);

		rc = wdrv_rxInit();
		if (rc != OK)
		{
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "Could not initialize WLAN driver Rx\n");
			break;
		}
		else
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "Rx is up\n");		
		
		MODULE_ADD_BIT(UpModulesToGo, WDRV_TX_MODULE_ID);

		rc = wdrv_txInit();
		if (rc != OK)
		{
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "Could not initialize WLAN driver Tx\n");
			break;
		}
		else
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "Tx is up\n");		

		rc = wlan_memMngrInit();
		if (rc != OK)
		{
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "Could not initialize WLAN driver database\n");
			break;
		}
		else
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "MemMngr is up\n");

		rc = mainsmInit();
		if (rc != OK)
		{
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "Could not initialize WLAN driver main state machine\n");
			break;
		}
		else
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "main state machine is up\n");

		/* Wait 2 seconds for all driver tasks to come up */
		PAL_sleepLong(2000);

		if(REPORT_LEVEL(RL_MAINSM)) 
			 wdrv_utilsReport(RL_MAINSM, PRI_INFORMATION, "WDRV_MAINSM: Waiting for driver tasks to wake up\n");

		while(UpModulesToGo && (--retriesLeft)) 
		{
		  PAL_sleepLong(5000);
			if(REPORT_LEVEL(RL_MAINSM)) 
				 wdrv_utilsReport(RL_MAINSM, PRI_INFORMATION, "WDRV_MAINSM: Next WLAN driver wake up retry ... UpModulesToGo=%d\n",UpModulesToGo);
		}

		if (!retriesLeft) 
		{
			rc = NOK;	
			wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "Could not initialize WLAN driver, wake up problem\n");
			break;
		}
	}
	while(0);

	if (!rc)		
		wdrv_utilsReport(RL_MAINSM, PRI_INFORMATION, "WDRV_MAINSM: WLAN Driver initialized successfully\n\n");
	else
		wdrv_utilsReport(RL_MAINSM, PRI_INFORMATION, "WDRV_MAINSM: WLAN Driver initialization Failure\n\n");

	return rc;
}

/***************************************************************************
* Driver unload function																		*
***************************************************************************/
void wdrv_unload(void)
{
	mainsmUnload();	
	wdrv_mgmtUnload();
	wdrv_rxUnload();
	wdrv_txUnload();
	wdrv_WoneBalanceUnload();
	whal_apiUnload(0);
	whal_apiUnload(1);
	whal_apiTxCleanAll();
	whal_rxMasterUnload();
	whal_txMasterUnload();
	wdrv_dbUnload();
	wlan_memMngrFreeAll();
}

/***************************************************************************
* Driver start function																		*
***************************************************************************/
STATUS wdrv_start(void)
{
	BOOL	WlanState;
	UINT8   channel,band,radarDetectMode; 
	UINT8 TotalWones;
	int i;

	wdrv_dbGetWlanServiceState( &WlanState );

	wdrv_4xSetStatus();

	/* check if we need to choose channel or the user choose one for use(DCS use) */
	wdrv_dbGetNumSupWone(&TotalWones);
	for(i=0 ; i < TotalWones; i++)
	{
		wdrv_dbGetBand(i,&band);

		if(band == RADIO_BAND_5GHZ)
		{
			wdrv_dbGetRadarDetectMode(&radarDetectMode);

			/* radar detection use !*/
			if(radarDetectMode)
			{
				UINT8 energy_en; 
				/* get channel*/
				wdrv_dbGetChannel(i,&channel);

				/*if this channel needs to be checked*/
				if(!isChannelInSafeRadarList(channel))
				{
				/* enable energy detction case disable*/
				wdrv_dbGetEnergyDetect(&energy_en);
				if(!energy_en)
					whal_apiPhyWrite(0x20b7, 0x140A); /* enable energy detect */

				/* finding channel ,using radar detection algorithm*/
					if(wdrv_check_channel(i,channel) != OK)
				{
						wdrv_utilsReport(RL_MAINSM, PRI_EMERGENCY, "after running channel selection ,replacing configure channel %d with safe channel %d.\n",channel,DEFAULT_RADAR_SAFE_CHANNEL);
						channel = DEFAULT_RADAR_SAFE_CHANNEL;	
					}
				/* case energy detection was off ,return it to original status*/
				if(!energy_en)
					whal_apiPhyWrite(0x20b7, 0xFFEF); /* disable energy detect */
			}

				wdrv_utilsReport(RL_MAINSM, PRI_INFORMATION, "wdrv_start:ap work in A mode,band 5 GHZ,cahnnel %d active.\n",channel);
				wdrv_dbSetChannel(i,channel);
				whal_apiSetRadioChannel(i,channel);

			}
		}/* if band */

	}/* for wones*/
	
	wdrv_rxEnable();

	if(WlanState)
		whal_apiStartBss();

	wdrv_txEnable();

    /*wdrv_endStart();*/

	wdrv_RestartWatchDog_Start();

	return OK;
}

/***************************************************************************
*  print time from boot
***************************************************************************/
void PrintTimeSinceBoot(void)
{
	ULONG counterSeconds;
	ULONG days,hours,minutes,seconds,temp;

	counterSeconds = PALSRV_getTick() / 100;
	days = counterSeconds/86400;
	temp = counterSeconds - days*86400;
	hours = temp/3600;
	temp -= hours*3600;
	minutes = temp/60;
	seconds = temp - minutes*60;
	
	printf ("@@@@@@@@@@@@ TIME SINCE BOOT: %d[d]:%d[h]:%d[m]:%d[s] @@@@@@@@@@@@\n",(int)days,(int)hours,(int)minutes,(int)seconds);
}


/***************************************************************************
*  fail status handler
***************************************************************************/
void wdrv_watchdogFailStatusHandler(void)
{
	wdrv_DDA_restart_setEvent();
}

BOOL recoveryIsInProcess = FALSE;

/***************************************************************************
*  ative restart
***************************************************************************/
void wdrv_ActivateRestart(void){
	BOOL	WlanState;
	unsigned long flags = 0;
	int     Status;
	/*int start_jiffies = jiffies;*/

	recoveryIsInProcess = TRUE;

	/*printk("Recovery start : %ld\n", jiffies - start_jiffies);*/

	wdrv_RestartWatchDog_Stop();

	wdrv_WoneBalanceShutdown();
	wdrv_mgmtRadioCalibrateStop();
	/*wdrv_WoneBalanceUnload();*/

	PAL_intrLock(flags);

	tiwlan_ddaSuspend();
	wdrv_dbGetWlanServiceState( &WlanState );

	PrintTimeSinceBoot();
	whal_apiUpdateFWWAStatistics();
	/*wdrv_RestartWatchDog_Stop();*/

	whal_apiUnload(0);    

	whal_apiUnload(1);
	
	whal_apiTxCleanAll();

	whal_rxMasterUnload();

	whal_txMasterUnload();

	PAL_intrUnLock(flags);

	Status = whal_apiInit(0, 0);
	if (Status != OK)
	{
		printf("wdrv_ActivateRestart: FATAL ERROR, Failed in whal_apiInit\n");
		return; 
	}

	PAL_intrLock(flags);

	whal_apiSetRxDataCB((RxData_Callback_T)wdrv_rxPacketRecvNotify, 1);

	if (WlanState) 	
		whal_apiStartBss();

	whal_apiEnableAllInterrupts();

	//	wpa_restart();
	dda_wpa_restart_module();

	tiwlan_ddaResume();
	wdrv_WoneStartBalanceTimer();
	wdrv_mgmtRadioCalibrateInit();
	PAL_intrUnLock(flags);

	recoveryIsInProcess = FALSE;
	wdrv_RestartWatchDog_Start();
}

/***************************************************************************
*  restrart timer cb
***************************************************************************/
static void wdrv_RestartTimerCB(UINT32 data)
{
	mainsmMessage_T MessageInfo;

	MessageInfo.type = WDRV_MAINSM_RESTART;
   
	wdrv_DDA_mainSm_setEvent(&MessageInfo);

	/* reschedule the timer */
	PALSRV_tcbQueueAct(&RestartWatchDogTimer,
					   WATCH_DOG_TIMER_VAL,
					   wdrv_RestartTimerCB,
					   0, 0);
}


/***************************************************************************
* 
***************************************************************************/
static void wdrv_RestartHandler(long type)
{	
	UINT8 	en;

	wdrv_dbGetEnRestart(&en);
	if (!en)
		return;

	if(type == WDRV_MAINSM_RESTART)
	{
		if (whal_apiRestartCheckIfNeeded(wdrv_watchdogFailStatusHandler) == OK)
			wdrv_DDA_restart_setEvent();	
	}
    else if(type == WDRV_MAINSM_ENABLE_WLAN_SERVICE)
	{
		printf ("@@@@@@@@@@@@  ENABLE WLAN SERVICE @@@@@@@@@@@@\n");
		wdrv_DDA_restart_setEvent();
	}
}


/***************************************************************************
*  
***************************************************************************/
static void wdrv_RestartWatchDog_DeviceError_Callback(void)
{
	UINT8 	en;
    
	wdrv_dbGetEnRestart(&en);
	if (!en)
		return;

	wdrv_DDA_restart_setEvent();
}


/***************************************************************************
*  
***************************************************************************/
void wdrv_PeriodicWoneSliceUpdateRequest (void)
{
	mainsmMessage_T MessageInfo;

	MessageInfo.type = WDRV_MAINSM_WONE_BALANCE;
   
	wdrv_DDA_mainSm_setEvent(&MessageInfo);
}


/***************************************************************************
*  
***************************************************************************/
static void wdrv_RestartWatchDog_Start(void)
{
	UINT8 en;

	wdrv_dbGetEnRestart(&en);
	if (en)
		wdrv_utilsReport(RL_MAINSM, PRI_INFORMATION, "FW Watchdog is Enabled\n");
	else
		wdrv_utilsReport(RL_MAINSM, PRI_INFORMATION, "FW Watchdog is Disabled\n");
	
	PALSRV_tcbNew(&RestartWatchDogTimer);

	PALSRV_tcbQueueAct(&RestartWatchDogTimer,
					   WATCH_DOG_TIMER_VAL,
					   wdrv_RestartTimerCB,
					   0, 0);
	
	whal_apiEnableDeviceErrorIntr(wdrv_RestartWatchDog_DeviceError_Callback);
}


/***************************************************************************
*  
***************************************************************************/
static void wdrv_RestartWatchDog_Stop(void)
{
	whal_apiDisableDeviceErrorIntr();

	PALSRV_tcbCancel(&RestartWatchDogTimer);
}





STATUS wdrv_radar_found(UINT16 CCA_Events_Diff[], UINT8 Number_Of_CCA_Events)
{
	int EventNumber,RadarNumber;
	UINT8 PulseCount ;

/*	printk("\nwdrv_radar_found:start check");*/
	/* run and check all RADARS ,this loop is no use !!! */
	for(RadarNumber = 0; RadarNumber < NUMBER_OF_RADARS; RadarNumber++)
	{	 
		PulseCount = 0;
		for(EventNumber = 0; EventNumber < Number_Of_CCA_Events; EventNumber++)	
		{
			if(
				((CCA_Events_Diff[EventNumber] - Radar_Interval[RadarNumber])	< RESOLUTION) ||
				((Radar_Interval[RadarNumber] - CCA_Events_Diff[EventNumber])	< RESOLUTION)
			  )
			{
#ifdef RADAR_DET_TEST_ONLY
				printk("\nfound pulse %d ,radar %d" ,PulseCount,RadarNumber);
#endif
				PulseCount++;
			}

			if(PulseCount > Radar_Minimal_Number_OF_PULSES[RadarNumber])
				return OK;
		}
	}

	return NOK;
}	

/***************************************************************************
*  radar detection alg.
***************************************************************************/
STATUS wdrv_check_channel(UINT8 wone_index,UINT8 channel)
{
	UINT16 CCA_Events_Diff[EVENET_DIFF_VECTOR_SIZE];
	UINT8  Number_Of_CCA_Events;
	


		/* if channel allowed*/
	if(IsChannelAllowed(wone_index,channel) == FALSE)
		return NOK;

		/* send  start channel scan cmd*/
#ifndef RADAR_DET_DEMO_MAILBOX 
	if(whal_apiRadarDetSendStartCmd(channel,RADIO_BAND_5GHZ) != OK)
		{
		wdrv_utilsReport(RL_MAINSM, PRI_ERROR, "wdrv_check_channel:could not send radar detection \nstart cmd for channel %d\n",channel);
		return NOK;
		}
#endif
		/* wait 61 second */
	wdrv_utilsReport(RL_MAINSM, PRI_INFORMATION, "wdrv_check_channel:channel selection process,\n checking channel %d validity,please wait(it can take a moment)... \n",channel);
		PAL_sleepLong(CHANNEL_SELC_WAIT_FOR_REPLY_TIME);

		/* send stop cmd */
#ifndef RADAR_DET_DEMO_MAILBOX 
	if(whal_apiRadarDetSendStopCmd(CCA_Events_Diff,&Number_Of_CCA_Events) != OK) 
		{
		wdrv_utilsReport(RL_MAINSM, PRI_ERROR, "wdrv_check_channel:could not send radar detection stop, channel%d.\n",channel);
		return NOK;
		}
#else
		printk("\nRD TST:copying data");
		memcpy((char*)CCA_Events_Diff,(char*)tst_time_deff_vec,tst_event_num * sizeof(short));
		Number_Of_CCA_Events = tst_event_num;
#endif

#ifdef RADAR_DET_TEST_ONLY
		/* activate algorithm with mailbox input*/
		{
			int i;
			printk("\nNumber_Of_CCA_Events = %d",Number_Of_CCA_Events);
			for(i = 0;i<Number_Of_CCA_Events;i++)
				printk("\n%d.sample val = %d",i,CCA_Events_Diff[i]);
		}
#endif
	if(wdrv_radar_found(CCA_Events_Diff,Number_Of_CCA_Events) == OK)		
		{
		wdrv_utilsReport(RL_MAINSM, PRI_NOTICE, "wdrv_check_channel:found radar on channel %d,looking for another channel\n",channel);
		return NOK;
		}
		
	return OK;
	
}

/***************************************************************************
*  convert cahnnel to frequancy.
***************************************************************************/
static UINT32 convertCahnneltoFreq(int channel)
{
	return (5000 + 5*channel);
}
/***************************************************************************
*  chaeck if channel is in radar safe channel list
***************************************************************************/
BOOL isChannelInSafeRadarList(UINT8 channel)
{
	UINT32 freq =  convertCahnneltoFreq(channel);

	if(
		(freq < SAFE_RADAR_FREQUANCY_START_RANGE  ) ||
		(freq > SAFE_RADAR_FREQUANCY_END_RANGE  )
	  )
	  return FALSE;

	return TRUE;
}
