/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																									*/
/*		MODULE:	mainsm.h																		*/
/*    PURPOSE:	Driver's main state machine											*/
/*																									*/
/***************************************************************************/

#ifndef _MAINSM_H
#define _MAINSM_H

#include "pform_types.h"
#include "memMngr.h"
#include "drvDbDefs.h"

#define WDRV_MAINSM_QSIZE         15

/* MAINSM Incoming Messages type definition */
#define WDRV_MAINSM_MODULE_UP		 1
#define WDRV_MAINSM_RESTART			 2
#define WDRV_MAINSM_WONE_BALANCE     3
#define WDRV_MAINSM_ACTIVATE_RESTART 4
#define WDRV_MAINSM_ENABLE_WLAN_SERVICE 6


/* module ID's */
#define WDRV_UTIL_MODULE_ID			0
#define WDRV_BSS_BRIDGE_MODULE_ID	1
#define WDRV_DB_MODULE_ID			2
#define WDRV_END_MODULE_ID			3	
#define WDRV_MGMT_MODULE_ID			4	
#define WDRV_PWR_SV_MODULE_ID		5	
#define WDRV_RX_MODULE_ID			6	
#define WDRV_TX_MODULE_ID			7	
#define WDRV_MEM_MODULE_ID			8
#define WDRV_HAL_MODULE_ID			9
#define WDRV_WPA_MODULE_ID			10



#define MODULE_BIT(i)         (1<<(i))
#define MODULE_ADD_BIT(m,i)   ((m) |= MODULE_BIT(i))
#define MODULE_CLEAR_BIT(m,i) ((m) &= ~(MODULE_BIT(i)))


typedef struct 
{
	/* wdrv tasks priority */
	UINT32 RxTaskPriority;
	UINT32 TxTaskPriority;
	UINT32 WpaTaskPriority;
	UINT32 MgmtTaskPriority;
	UINT32 MainSmTaskPriority;

	/* wdrv tasks stack size */
	UINT32 RxTaskStackSize;
	UINT32 TxTaskStackSize;
	UINT32 WpaTaskStackSize;
	UINT32 MgmtTaskStackSize;
	UINT32 MainSmTaskStackSize;

} WDRV_TASKS_CONFIG;

typedef struct 
{
	/* driver tasks configuration */
	WDRV_TASKS_CONFIG wdrvTasksConfig;
	/* Add here more drvice configuration parameters */
} WDRV_CONFIG;

typedef struct {
   long type;
   union {
      long MessageInfo[3];
      struct {
         long  moduleID;
      } MODULE_up;
   } content;
} mainsmMessage_T;


/* User init ack callback function */
typedef void (*wdrv_UsrInitFuncCB)(STATUS status);
/* MSDU annuncment callback function */
typedef void (*wdrv_msduRcvdCBFunc)(mem_MSDU_T *pMsdu);
/* Address removal from bridge callback function */
typedef void (*wdrv_addrDelCBFunc)(char *mac_addr);
/* Core Callback that called when mailbox error occurs */
typedef void (*watchdogFailCB_T)(void); 

STATUS wdrv_mainsmSendModuleUp(int id);
STATUS wdrv_EnableWlanService(void);
int    wdrv_init(initConfigStruct *pInitConfig);
STATUS wdrv_start(void);
void   wdrv_restart(void);
void   wdrv_unload(void);
void   wdrv_ActivateRestart(void);

extern void wdrv_PeriodicWoneSliceUpdateRequest ( void );

STATUS wdrv_mainSm_getEvent(mainsmMessage_T  *pMessageInfo);
void wdrv_mgmtGetState(UINT16 index, UINT32 *state);


#endif /*_MAINSM_H*/
