/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************/
/*																			*/
/*		MODULE:	mgmtTest.c													*/
/*		PURPOSE:	Authorization/association state machine implementation	*/
/*																			*/
/****************************************************************************/
#include "pform_types.h"
#include "mgmtSm.h"
#include "memMngr.h"
#include "drvDbHash.h"
#include "mgmtBuilder.h"
#include "mgmtParser.h"
#include "802_11Defs.h"
#include "mainsm.h"
#include "utils.h"
#include "tx.h"

#if 0

/* Management messages structure */
typedef struct {
   int type;
   union {
      long MessageInfo[3];
      struct {
            mem_MSDU_T*  msduPtr;	   
      } frame;
  } content;
} stubMessage_T;

static dot11_mgmtFrame_T		tstDeFragMmpdu;

dot11_mgmtFrame_T nullFrame;

/* task variables */
#define WDRV_MGMT_TST_PRIORITY     174
#define WDRV_MGMT_TST_QSIZE         20
unsigned long mgmtTstQueue = 0;

static void mgmtTstTask(void);
static void buildHeader(UINT32 staMac, dot11_mgmtHeader_T *hdrP);

STATUS wdrv_txInit(void)
{
	int rc, i;
	unsigned long  taskID;

	/* create task */
	rc = rt_osTaskCreate( "MGMTTST"           ,         /* name              */
				 WDRV_MGMT_TST_PRIORITY ,         /* priority          */
				 16384            ,         /* supervisor stack  */
				 &taskID);                  /* id                */
	if (rc != OS_NOERR) 
	{
		if(REPORT_LEVEL(RL_TEST)) 
			wdrv_utilsReport(RL_TEST, PRI_EMERGENCY, "WDRV_TEST: Error Creating Task. Error Code = %ld\n", rc);
		return NOK;
	}

	/* start task */
	rc = rt_osTaskStart( taskID,
				OS_TASK_PREEMPT,
				mgmtTstTask);
	if (rc != OS_NOERR) 
	{
		if(REPORT_LEVEL(RL_TEST)) 
			wdrv_utilsReport(RL_TEST, PRI_EMERGENCY, "WDRV_TEST: Error Starting Task. Error Code = %ld\n", rc);
		return NOK;
	}

	/* create a messageQ for the MGMT incoming msgs */
	rc = rt_osQueueCreate("MGMTTSTQ", WDRV_MGMT_TST_QSIZE, &mgmtTstQueue);
	if (rc != 0) 
	{
		if(REPORT_LEVEL(RL_TEST)) 
			wdrv_utilsReport(RL_TEST, PRI_EMERGENCY, "WDRV_TEST: Error Creating Message Queue\n");
		return NOK;
	}

	if(REPORT_LEVEL(RL_TEST)) 
		wdrv_utilsReport(RL_TEST, PRI_INFORMATION, "WDRV_TEST: Management Test module initialized successfully\n");

	wdrv_utilsSetReportLevel((WDRV_UTILS_REPORT_LEVEL)(RL_TEST));

	wdrv_mainsmSendModuleUp(WDRV_TX_MODULE_ID);

	return OK;
}

void mgmtTstTask(void) {

	int               rc, i;
	stubMessage_T     stubMsg;

/**************************************************************
      The loop of the task, implementing the SM
 **************************************************************/

	while (1) 
	{

		/* Wait for a message to come in                      */
		if ((rc = rt_osQueueReceive(mgmtTstQueue,
									OS_WAIT_FOREVER, 
									(unsigned long *)&(stubMsg.type))) == 0) 
		{
					wlan_memMngrFreeMSDU(stubMsg.content.frame.msduPtr->handle);
		} /* if got a message */
	} /* while (1) */

}

STATUS wdrv_txSendMsdu( mem_MSDU_T *msduPtr, UINT16 stationIndex, ULONG txFlags )
{
	TxMessage_T txMsg;

	txMsg.type = WDRV_TX_MSDU_TRESHOLD;
	txMsg.msg.MessageInfo[0] = (long)msduPtr;

	rt_osQueueSend(mgmtTstQueue , (UINT32 *)&txMsg);

	return OK;
}


int whal_testInit()
{

}

int  whal_apiInit(whal_UsrInitFuncCB CbFunc, int retArg)
{
	wdrv_mainsmSendModuleUp(WDRV_HAL_MODULE_ID);
}

int  whal_apiStartBss()
{

}

int whal_nvram_getMacAddress(UINT8 *pbOutputArray)
{
	memset(pbOutputArray, 0xAA, MAC_ADDR_LEN);
}

int whal_nvram_getSsid(UINT8 *pbOutputArray)
{
	memset(pbOutputArray, 0x00, DOT11_SSID_MAX - 1);
	*(pbOutputArray + DOT11_SSID_MAX - 1) = 0x01;
}

int whal_nvram_setMacAddress(UINT8 *pbInputArray,UINT8 bSaveDelay)
{

}

int whal_nvram_setSsid(UINT8 *pbInputArray,UINT8 bSaveDelay)
{

}

int whal_nvram_getOperationalRateSet(UINT8 woneIndex, SINT8 *pbOutputArray, UINT16 wLen)
{

}

int whal_nvram_setOperationalRateSetLength(UINT8 woneIndex, UINT16 wInput,UINT8 bSaveDelay)
{

}

void whal_nvram_getOperationalRateSetLength(UINT8 woneIndex,UINT8 *pwOutput)
{

}

int whal_nvram_setOperationalRateSet(UINT8 woneIndex,						 
                                     SINT8 *pbInputArray, 
                                     UINT16 wLen,
                                     UINT8 bSaveDelay)
{

}

void whal_utilsSniffer(char *pPacket)
{

}

MGR_HANDLE whal_apiRxRecv(int *Len, RxAttr_T *pRxAttr)
{

}

STATUS wdrv_rxInit(void)
{
	wdrv_mainsmSendModuleUp(WDRV_RX_MODULE_ID);
	return OK;
}

void wdrv_rxEnable(void)
{
}

STATUS wdrv_txEnable(void)
{
}

STATUS wdrv_bssbridgeInit(void) 
{
   wdrv_mainsmSendModuleUp(WDRV_BSS_BRIDGE_MODULE_ID);

	return OK;
}

void whal_nvram_getBeaconPeriod(UINT16 *pwOutput)
{
}

int whal_nvram_setBeaconPeriod(UINT16 wInput,UINT8 bSaveDelay)
{
}

void whal_nvram_getDTIMPeriod(UINT8 *pwOutput)
{
}

void whal_nvram_getPrivacyOptionImplemented(BOOL *puiOutput)
{
}

void whal_nvram_getPrivacyOptionImplemented(BOOL *puiOutput)
{
}

int whal_nvram_setDTIMPeriod(UINT8 wInput,UINT8 bSaveDelay)
{
}

void whal_nvram_getAuthenticationResponceTimeOut(UINT32 *puiOutput)
{
	*puiOutput = 20000;
}

int whal_nvram_setAuthenticationResponceTimeOut(UINT32 uiInput,
                                                UINT8 bSaveDelay)
{
	return 1;
}

#endif