/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************/
/*																			*/
/*		MODULE:	mgmtTest.c													*/
/*		PURPOSE:	Authorization/association state machine implementation	*/
/*																			*/
/****************************************************************************/
#include "pform_types.h"
#include "mgmtSm.h"
#include "memMngr.h"
#include "drvDbHash.h"
#include "mgmtBuilder.h"
#include "mgmtParser.h"
#include "802_11Defs.h"
#include "mainsm.h"
#include "utils.h"
#include "mgmtSwap.h"
#include "mgmtIf.h"


/* task variables */
static void buildHeader(UINT32 staMac, dot11_mgmtHeader_T *hdrP);


void wdrv_mgmtTstNull(UINT16 fc, UINT32 SA, UINT32 DA, UINT32 BSSID)
{
	dot11_mgmtFrame_T nullFrame;
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	int  rc;
	
	PAL_memSet(&nullFrame, 0, sizeof(dot11_mgmtFrame_T));
	
	totalLen = sizeof(dot11_mgmtFrame_T);

	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;
	
	BD->length = 0;

	(char *)hdrP = BD->data + BD->dataOffset;

	PAL_memCopy(&(hdrP->fc), &fc, 2);
	PAL_memSet(&(hdrP->duration), 0, 2);
	PAL_memSet(hdrP->SA.addr, SA, MAC_ADDR_LEN);
	PAL_memSet(hdrP->DA.addr, DA, MAC_ADDR_LEN);
	PAL_memSet(hdrP->BSSID.addr, BSSID, MAC_ADDR_LEN);
	PAL_memSet(&(hdrP->seqCtrl), 0, 2);

	BD->length += sizeof(dot11_mgmtHeader_T);	

	dataP = BD->data + BD->dataOffset + BD->length;

	wlan_memMngrFillField(&dataP, (char *)&(nullFrame.body), MAX_MGMT_BODY_LENGTH, &BD);

	wdrv_utilsReport(RL_TEST, PRI_INFORMATION, "WDRV_TEST:Total Len: %d\n", totalLen);

	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	rc = wdrv_mgmtForwardMgmtFrame(MSDUPtr, 0);		/* always send with index 0.*/
	if (rc != OK)
		wdrv_utilsReport(RL_TEST, PRI_INFORMATION, "WDRV_TEST:Failed to send null Frame\n");

	return;
}

void wdrv_mgmtTstAuth1(UINT32 SA, UINT16 algoNum)
{
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	int rc;
	char mac[MAC_ADDR_LEN];
	UINT16 index;
	UINT16 status = 0;
	UINT16 seqNum = 1;

	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN + FIX_FIELD_LEN + FIX_FIELD_LEN;
		
	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	(char *)hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;

	buildHeader(SA, hdrP);

	hdrP->fc |= DOT11_FC_AUTH;

	BD->length += sizeof(dot11_mgmtHeader_T);	

	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	wlan_memMngrFillField(&dataP, (char *)&algoNum, FIX_FIELD_LEN, &BD);

	wlan_memMngrFillField(&dataP, (char *)&seqNum, FIX_FIELD_LEN, &BD);

	wlan_memMngrFillField(&dataP, (char *)&status, FIX_FIELD_LEN, &BD);

	PAL_memSet(mac, SA, MAC_ADDR_LEN);
	wdrv_dbFindStaIndex(mac, &index);

	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	rc = wdrv_mgmtForwardMgmtFrame(MSDUPtr, index);		/* always send with index 0.*/
	if (rc != OK)
		wdrv_utilsReport(RL_TEST, PRI_INFORMATION, "WDRV_TEST:Failed to send auth Frame\n");

	return;
}

void wdrv_mgmtTstAuth3(UINT32 SA, UINT16 algoNum, UINT32 challenge)
{
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	dot11_Chlg_T challengeTmp;
	int rc;
	char mac[MAC_ADDR_LEN];
	UINT16 index;
	UINT16 status = 0;
	UINT16 seqNum = 3;

	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN + FIX_FIELD_LEN + FIX_FIELD_LEN + sizeof(dot11_Chlg_T);
		
	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	(char *)hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;

	buildHeader(SA, hdrP);

	hdrP->fc |= DOT11_FC_AUTH;

	BD->length += sizeof(dot11_mgmtHeader_T);	

	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	wlan_memMngrFillField(&dataP, (char *)&algoNum, FIX_FIELD_LEN, &BD);

	wlan_memMngrFillField(&dataP, (char *)&seqNum, FIX_FIELD_LEN, &BD);

	wlan_memMngrFillField(&dataP, (char *)&status, FIX_FIELD_LEN, &BD);

	challengeTmp.hdr.eleId = DOT11_CHALLENGE_TEXT_ELE_ID;
	challengeTmp.hdr.eleLen = DOT11_CHALLENGE_TEXT_MAX;
	PAL_memSet(challengeTmp.text, 0, DOT11_CHALLENGE_TEXT_MAX - 1);
	challengeTmp.text[DOT11_CHALLENGE_TEXT_MAX - 1] = challenge;
	wlan_memMngrFillField(&dataP, (char *)&challengeTmp, sizeof(dot11_Chlg_T), &BD);

	PAL_memSet(mac, SA, MAC_ADDR_LEN);
	wdrv_dbFindStaIndex(mac, &index);

	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	rc = wdrv_mgmtForwardMgmtFrame(MSDUPtr, index);		/* always send with index 0.*/
	if (rc != OK)
		wdrv_utilsReport(RL_TEST, PRI_INFORMATION, "WDRV_TEST:Failed to send auth Frame\n");

	return;
}

void wdrv_mgmtTstAssoc(UINT32 SA, BOOL pbcc, BOOL privacy, BOOL fourX)
{
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	int rc;
	char mac[MAC_ADDR_LEN];
	UINT16 index;
	UINT16 listenInterval = 0x5555;
	dot11_SSID_T	ssidTmp;
	char ssidBuf[50];
	dot11_supRates_T ratesTmp;
	dot11_extSupRates_T extRates;
	dot11_TIcap_T	TIcapIE;
	UINT16 capabilities = 0;

	wdrv_dbGetSsid(ssidBuf);
	wdrv_dbGetStationConfigOperationalRateSet(0,ratesTmp.rates, &ratesTmp.hdr.eleLen);

	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN + FIX_FIELD_LEN + 2 + strlen(ssidBuf) + 2 
						+ ratesTmp.hdr.eleLen; 

	/* Get the rates to be transmitted in Extended Supported Rates IE */
	extRates.hdr.eleId = DOT11_EXT_SUPPORTED_RATES_ELE_ID;
	wdrv_dbGetStationConfigOperationalOFDMRateSet(0,extRates.rates, &(extRates.hdr.eleLen));
	if (extRates.hdr.eleLen > 0)
	{
		totalLen += sizeof(dot11_eleHdr_T) + extRates.hdr.eleLen;
	}

	if (fourX)
	{
		wdrv_dbGetTIInfoElement(TIcapIE.cap, &(TIcapIE.hdr.eleLen));
		totalLen += 2 + TIcapIE.hdr.eleLen;
	}
		
	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	(char *)hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;

	buildHeader(SA, hdrP);

	hdrP->fc |= DOT11_FC_ASSOC_REQ;
	
	BD->length += sizeof(dot11_mgmtHeader_T);	

	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	capabilities |= (TRUE << CAP_ESS_SHIFT);
	
	if (privacy)
		capabilities |= (TRUE << CAP_PRIVACY_SHIFT);

	if (pbcc)
		capabilities |= (TRUE << CAP_PBCC_SHIFT);
	
	wlan_memMngrFillField(&dataP, (char *)&capabilities, FIX_FIELD_LEN, &BD);

	wlan_memMngrFillField(&dataP, (char *)&listenInterval, FIX_FIELD_LEN, &BD);

	ssidTmp.hdr.eleId = DOT11_SSID_ELE_ID;
	ssidTmp.hdr.eleLen = strlen(ssidBuf);
	PAL_memCopy(ssidTmp.serviceSetId, ssidBuf, ssidTmp.hdr.eleLen);
	wlan_memMngrFillField(&dataP, (char *)&ssidTmp, ssidTmp.hdr.eleLen + 2, &BD);

	ratesTmp.hdr.eleId = DOT11_SUPPORTED_RATES_ELE_ID;
	wlan_memMngrFillField(&dataP, (char *)&ratesTmp, ratesTmp.hdr.eleLen + 2, &BD);

	/*#ifdef DOT_11G*/
	if (extRates.hdr.eleLen > 0)
	{
		wlan_memMngrFillField(&dataP, (char *)&extRates, sizeof(dot11_eleHdr_T) + extRates.hdr.eleLen, &BD);
	}
	/*#endif*/

	if (fourX)
	{
		TIcapIE.hdr.eleId = DOT11_TI_CAP_ELE_ID;
		wlan_memMngrFillField(&dataP, (char *)&TIcapIE, TIcapIE.hdr.eleLen + 2, &BD);
	}

	PAL_memSet(mac, SA, MAC_ADDR_LEN);
	wdrv_dbFindStaIndex(mac, &index);

	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	rc = wdrv_mgmtForwardMgmtFrame(MSDUPtr, index);		/* always send with index 0.*/
	if (rc != OK)
		wdrv_utilsReport(RL_TEST, PRI_INFORMATION, "WDRV_TEST:Failed to send auth Frame\n");

	return;
}

void wdrv_mgmtTstReassoc(UINT32 SA, BOOL pbcc, BOOL privacy, BOOL fourX)
{
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	int  rc;
	char mac[MAC_ADDR_LEN];
	UINT16 index;
	UINT16 listenInterval = 0x5555;
	dot11_SSID_T	ssidTmp;
	char ssidBuf[50];
	dot11_supRates_T ratesTmp;
	dot11_extSupRates_T extRates;
	UINT16 capabilities = 0;
	char currApTmp[MAC_ADDR_LEN];
	dot11_TIcap_T	TIcapIE;

	wdrv_dbGetSsid(ssidBuf);
	wdrv_dbGetStationConfigOperationalRateSet(0,ratesTmp.rates, &ratesTmp.hdr.eleLen);

	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN + FIX_FIELD_LEN + 2 + strlen(ssidBuf) + 2 
				+ ratesTmp.hdr.eleLen + MAC_ADDR_LEN;

	/* Get the rates to be transmitted in Extended Supported Rates IE */
	extRates.hdr.eleId = DOT11_EXT_SUPPORTED_RATES_ELE_ID;
	wdrv_dbGetStationConfigOperationalOFDMRateSet(0,extRates.rates, &(extRates.hdr.eleLen));
	if (extRates.hdr.eleLen > 0)
	{
		totalLen += sizeof(dot11_eleHdr_T) + extRates.hdr.eleLen;
	}

	if (fourX)
	{
		wdrv_dbGetTIInfoElement(TIcapIE.cap, &(TIcapIE.hdr.eleLen));
		totalLen += 2 + TIcapIE.hdr.eleLen;
	}

	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	(char *)hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;

	buildHeader(SA, hdrP);

	hdrP->fc |= DOT11_FC_REASSOC_REQ;

	BD->length += sizeof(dot11_mgmtHeader_T);	

	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	capabilities |= (TRUE << CAP_ESS_SHIFT);

	if (privacy)
		capabilities |= (TRUE << CAP_PRIVACY_SHIFT);
	if (pbcc)
		capabilities |= (TRUE << CAP_PBCC_SHIFT);
	
	wlan_memMngrFillField(&dataP, (char *)&capabilities, FIX_FIELD_LEN, &BD);

	wlan_memMngrFillField(&dataP, (char *)&listenInterval, FIX_FIELD_LEN, &BD);

	wdrv_dbGetOperationMacAddress(currApTmp);
	wlan_memMngrFillField(&dataP, (char *)&currApTmp, MAC_ADDR_LEN, &BD);

	ssidTmp.hdr.eleId = DOT11_SSID_ELE_ID;
	ssidTmp.hdr.eleLen = strlen(ssidBuf);
	PAL_memCopy(ssidTmp.serviceSetId, ssidBuf, ssidTmp.hdr.eleLen);
	wlan_memMngrFillField(&dataP, (char *)&ssidTmp, ssidTmp.hdr.eleLen + 2, &BD);

	ratesTmp.hdr.eleId = DOT11_SUPPORTED_RATES_ELE_ID;
	wlan_memMngrFillField(&dataP, (char *)&ratesTmp, ratesTmp.hdr.eleLen + 2, &BD);

	/*#ifdef DOT_11G*/
	if (extRates.hdr.eleLen > 0)
	{
		wlan_memMngrFillField(&dataP, (char *)&extRates, sizeof(dot11_eleHdr_T) + extRates.hdr.eleLen, &BD);
	}
	/*#endif*/

	if (fourX)
	{
		TIcapIE.hdr.eleId = DOT11_TI_CAP_ELE_ID;
		wlan_memMngrFillField(&dataP, (char *)&TIcapIE, TIcapIE.hdr.eleLen + 2, &BD);
	}

	PAL_memSet(mac, SA, MAC_ADDR_LEN);
	wdrv_dbFindStaIndex(mac, &index);

	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	rc = wdrv_mgmtForwardMgmtFrame(MSDUPtr, index);		/* always send with index 0.*/
	if (rc != OK)
		wdrv_utilsReport(RL_TEST, PRI_INFORMATION, "WDRV_TEST:Failed to send auth Frame\n");

	return;
}

void wdrv_mgmtTstDeAuth(UINT32 SA, UINT16 reason)
{
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	int rc;
	char mac[MAC_ADDR_LEN];
	UINT16 index;

	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN;
		
	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	(char *)hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;

	buildHeader(SA, hdrP);

	hdrP->fc |= DOT11_FC_DEAUTH;

	BD->length += sizeof(dot11_mgmtHeader_T);	

	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	wlan_memMngrFillField(&dataP, (char *)&reason, FIX_FIELD_LEN, &BD);

	PAL_memSet(mac, SA, MAC_ADDR_LEN);
	wdrv_dbFindStaIndex(mac, &index);

	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	rc = wdrv_mgmtForwardMgmtFrame(MSDUPtr, index);		/* always send with index 0.*/
	if (rc != OK)
		wdrv_utilsReport(RL_TEST, PRI_INFORMATION, "WDRV_TEST:Failed to send auth Frame\n");

	return;
}

void wdrv_mgmtTstDisAssoc(UINT32 SA, UINT16 reason)
{
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	int  rc;
	char mac[MAC_ADDR_LEN];
	UINT16 index;

	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN;
		
	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	(char *)hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;

	buildHeader(SA, hdrP);

	hdrP->fc |= DOT11_FC_DISASSOC;

	BD->length += sizeof(dot11_mgmtHeader_T);	

	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	wlan_memMngrFillField(&dataP, (char *)&reason, FIX_FIELD_LEN, &BD);

	PAL_memSet(mac, SA, MAC_ADDR_LEN);
	wdrv_dbFindStaIndex(mac, &index);

	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	rc = wdrv_mgmtForwardMgmtFrame(MSDUPtr, index);		/* always send with index 0.*/
	if (rc != OK)
		wdrv_utilsReport(RL_TEST, PRI_INFORMATION, "WDRV_TEST:Failed to send auth Frame\n");

	return;
}

void wdrv_mgmtTstClass3(UINT32 SA, UINT16 index)
{
	char mac[MAC_ADDR_LEN];

	PAL_memSet(mac, SA, MAC_ADDR_LEN);
	wdrv_mgmtClass3Notify(mac, index, 0);
}

static void buildHeader(UINT32 staMac, dot11_mgmtHeader_T *hdrP)
{
	char	mac[MAC_ADDR_LEN];

	/* build frame control first */
	hdrP->fc = 0;
	
	hdrP->fc |= DOT11_FC_PROT_VERSION;

	hdrP->fc |= DOT11_FC_TYPE_MGMT;

	hdrP->fc |= (FALSE << DOT11_FC_TO_DS_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_FROM_DS_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_MORE_FRAG_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_RETRY_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_PWR_MGMT_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_MORE_DATA_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_WEP_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_ORDER_SHIFT);

	PAL_memSet(hdrP->SA.addr, staMac, MAC_ADDR_LEN);

	wdrv_dbGetOperationMacAddress(mac);
	PAL_memCopy(hdrP->DA.addr, mac, MAC_ADDR_LEN);

	wdrv_dbGetBssid(mac);
	PAL_memCopy(hdrP->BSSID.addr, mac, MAC_ADDR_LEN);
}
      
