/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************/
/*                                                                          */
/*      MODULE:	mgmtAccess.c                                                */
/*      PURPOSE: MAC access list functionality                              */
/*      CREATOR: Alexander Sirotkin                                         */
/*               demiurg@ti.com                                             */
/*      UPDATES:                                                            */
/*              July,09,2003 bug fixing, indentation                        */
/*                                                                          */
/****************************************************************************/

#include "mgmtAccess.h"
#include "app_driver_api.h"
#include "pal.h"


/*#include <stdio.h>
#include <stdlib.h>
*/

static UINT8 mgmt_access_enabled = 0;
static char mgmt_access_list[MGMT_ACCESS_LIST_SIZE][MAC_ADDR_LEN];
static mgmt_access_list_type_t mgmt_access_list_type = MGMT_ACCESS_BLACK_LIST;
static int NumOfStationsInList = 0;


STATUS mgmtAccessInit(mgmt_accesslist_init_struct *pInitAccessList)
{
	
    UINT8 type;
    int i=0;
    
	mgmt_access_enabled = pInitAccessList->mgmt_access_enabled;
	
	type = pInitAccessList->mgmt_access_list_type;
  	mgmt_access_list_type = (type == 0) ? MGMT_ACCESS_WHITE_LIST : MGMT_ACCESS_BLACK_LIST;
    
	memcpy((char*)mgmt_access_list,
		   (char*)(pInitAccessList->mgmt_access_list),
			MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN);

	wdrv_accessSetAuthCallback(mgmtAccessCallback);
	wdrv_accessSetAssocCallback(mgmtAccessCallback);

    for (i=0; i<MGMT_ACCESS_LIST_SIZE; i++) 
    {
        if( MACCMPZ( mgmt_access_list[i]))
            break;
        NumOfStationsInList++;
    }

	return OK;
}


/* Given a buffer, copy to it the B/W buffer */
STATUS mgmtAccessGetList(char *list, int *pLength) 
{
    if (*pLength < MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN) 
        return NOK;
	PAL_memCopy(list,mgmt_access_list,MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN);
    *pLength = MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN;
    return OK;
}

void mgmtAccessGetEnabled(UINT8 *enabled)
{
	*enabled = mgmt_access_enabled;
}

void mgmtAccessGetType(mgmt_access_list_type_t *type)
{
	*type = mgmt_access_list_type;
}

/* Add a given MAC Address to the B/W table */
STATUS mgmtAccessSetMAC(char *mac_addr) 
{
    int i;
    UINT16 idx;

    if (NumOfStationsInList == MGMT_ACCESS_LIST_SIZE) 
    {
        return ERROR;
    }

    /* an empty MAC (00-00-00-00-00-00), a broadcast MAC (ff-ff-ff-ff-ff-ff) */
    if( MACCMPZ(mac_addr) || MACCMPF(mac_addr))
        return ERROR;

    for (i=0; i<MGMT_ACCESS_LIST_SIZE; i++) 
    {
        if( MACCMP( mgmt_access_list[i], mac_addr)) 
            return OK;  /* already exists */
    }

    MACDUP2( mgmt_access_list[NumOfStationsInList], mac_addr);
    NumOfStationsInList++;
    if (mgmt_access_enabled && (mgmt_access_list_type == MGMT_ACCESS_BLACK_LIST)) 
    {
        if (wdrv_dbFindStaIndex(mac_addr,&idx) == OK) 
        {   /* station is already exists - send DIS_ASSOC msg and remove it. */
            wdrv_mgmtDeAuthMng(mac_addr,12/*STATUS_ASSOC_DENIED_OUT_OF_SCOPE*/);
/*            wdrv_mgmtSendDisAssocUnicastByIndex(12,idx);
            wdrv_dbRemoveStaIndex(idx);*/
        }
    }
	return OK;
}


/* Remove a given index from the B/W table */
STATUS mgmtAccessRemMAC(UINT16 index)
{
    int i;
    UINT16 idx;
    UINT8 mac_addr[MAC_ADDR_LEN];
    register void *pTmp;
   
    if ((index == 0) || (index > MGMT_ACCESS_LIST_SIZE))
	    return NOK;
 
    if (index > NumOfStationsInList)
        return OK;


    MACDUP2( mac_addr, mgmt_access_list[index-1]);
    
    /* shrink the table by one */
    for( i=index; i<NumOfStationsInList; ++i) {
       register void *pTmp1 = (void*)(mgmt_access_list[i]);
       pTmp  = (void*)(mgmt_access_list[i-1]);
       MACDUP2( pTmp, pTmp1);
    }
   
    --NumOfStationsInList;
    /* Set last MAC address to 0 */
    pTmp = (void*)(mgmt_access_list[NumOfStationsInList]);
    UST2( pTmp, 0) = 0;
    UST2( pTmp, 2) = 0;
    UST2( pTmp, 4) = 0;

  
    if (mgmt_access_enabled && (mgmt_access_list_type == MGMT_ACCESS_WHITE_LIST))
    {
	   
        if (wdrv_dbFindStaIndex(mac_addr,&idx) == OK) 
        {   /* station is already exists - send DIS_ASSOC msg and remove it. */

	     wdrv_mgmtDeAuthMng(mac_addr,12/*STATUS_ASSOC_DENIED_OUT_OF_SCOPE*/);
            /*wdrv_mgmtSendDisAssocUnicastByIndex(12,idx);
            wdrv_dbRemoveStaIndex(idx);*/
        }
    }

    return OK;
}


/* Remove a given index from the B/W table using an index*/
STATUS mgmtAccessRemMACbyAddr(char *mac_addr)
{
    int i;

    for (i=0; i < NumOfStationsInList; i++) 
    {
       if( MACCMP( mgmt_access_list[i], mac_addr)) 
            break;
    }

    if (i == NumOfStationsInList)
        return OK;  /* not found */

    return mgmtAccessRemMAC(i+1);
}

STATUS mgmtAccessCallback(char * mac_addr, int mac_addr_len) 
{
	int i;

/* 	printf(" %02hhx-%02hhx-%02hhx-%02hhx-%02hhx-%02hhx\n",  */
/* 					mac_addr[0],  mac_addr[1], */
/* 					mac_addr[2],  mac_addr[3], */
/* 					mac_addr[4],  mac_addr[5]); */

	if (! mgmt_access_enabled)
		return OK;

	if (mac_addr_len != MAC_ADDR_LEN)
		return NOK;

	/* TODO : hash */
	/* TODO : critical section */

	for (i=0; i<NumOfStationsInList; ++i) 
    {
        if ( MACCMP( mac_addr, mgmt_access_list[i]))
            return (mgmt_access_list_type == MGMT_ACCESS_BLACK_LIST) ? NOK : OK;
	}

	return (mgmt_access_list_type == MGMT_ACCESS_BLACK_LIST) ? OK : NOK;
}


/* Given a buffer, prints to it the B/W table contents. The buffer is ready to
   be printed out */
void mgmtAccessListStatus(char *list, int max_list, int *pNumOfEntries) 
{
    int i, res, count;
    *list = '\0';
    
    
    for (i=0, count=0, *pNumOfEntries=0; i<MGMT_ACCESS_LIST_SIZE; ++i) 
    {
       /* if empty mac, return */

        if( MACCMPZ( mgmt_access_list[i]))
        {
            return;     /* end of table */
        }
        res = sprintf(list+count,   
                      "%2d : %02hx-%02hx-%02hx-%02hx-%02hx-%02hx\n", 
                      i+1, 
                      mgmt_access_list[i][0], mgmt_access_list[i][1],
                      mgmt_access_list[i][2], mgmt_access_list[i][3],
                      mgmt_access_list[i][4], mgmt_access_list[i][5]);
        if (count+5 > max_list)    /* almost end of given buffer */
            return;
        if (res<0) 
            return;
        else
            count+=res;
        (*pNumOfEntries)++;
    }
}


