/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************/
/*																			*/
/*		MODULE:		mgmtBuilder.c											*/
/*		PURPOSE:	Management builder object								*/
/*																			*/
/****************************************************************************/
#include "mgmtSm.h"
#include "mainsm.h"
#include "memMngr.h"
#include "drvDbHash.h"
#include "802_11Defs.h"
#include "mgmtBuilder.h"
#include "mgmtParser.h"
#include "mgmtSm.h"
#include "drvDbMain.h"
#include "utils.h"
#include "mgmtSwap.h"
#include "tx.h"
#include "pal.h"
#include "dda.h"

static	UINT16 seed;
static  BOOL   seedInitialized = FALSE;	 
/*-----------------------------------------------------------*/
/*         Local Functions                                   */
/*-----------------------------------------------------------*/
static void buildHeader(char *dstMac, dot11_mgmtHeader_T *hdrP);

/*************************************************************************
 *                        wdrv_mgmtSendAuth                              *
 *************************************************************************
DESCRIPTION: This function sends an authentication frame                 
                                                                                                   
INPUT:      Authentication status
			Authentication request information
			Station index

OUTPUT:      

RETURN:      

************************************************************************/
void wdrv_mgmtSendAuth(UINT16 status, dot11_auth_T *protoPtr, UINT16 index)
{
	ULONG	txFlags = 0;
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	dot11_Chlg_T challenge;
	UINT16 transSeqNum;
	char mac[MAC_ADDR_LEN];
    UINT8 wone_index;

	/* compute the total len of the frame */
	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN + FIX_FIELD_LEN + FIX_FIELD_LEN;
		
	if (*(protoPtr->algoNumber) == SHARED_KEY && (*(protoPtr->transSeqNumber) + 1) == 2)	/* means we are about to send shared key no.2.*/
		totalLen += sizeof(dot11_Chlg_T);
	
	/* Allocate data buffers */
	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;
	
	/* Get station MAC */
	wdrv_dbGetStaMacAddress(index, mac);

	/* we have to init the random seed */
	if (!seedInitialized)
	{
		seed = mac[0];
		seedInitialized = TRUE;
	}

	/* Build the header */
	buildHeader(mac, hdrP);

	hdrP->fc |= DOT11_FC_AUTH;

	BD->length += sizeof(dot11_mgmtHeader_T);	

	/* Now fill the body */
	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	wlan_memMngrFillField(&dataP, (char *)protoPtr->algoNumber, FIX_FIELD_LEN, &BD);

	transSeqNum = *(protoPtr->transSeqNumber) + 1;
	wlan_memMngrFillField(&dataP, (char *)&transSeqNum, FIX_FIELD_LEN, &BD);

	wlan_memMngrFillField(&dataP, (char *)&status, FIX_FIELD_LEN, &BD);

	if ((*(protoPtr->algoNumber) == SHARED_KEY) && (transSeqNum == 2))
	{
		challenge.hdr.eleId = DOT11_CHALLENGE_TEXT_ELE_ID;
		challenge.hdr.eleLen = DOT11_CHALLENGE_TEXT_MAX;

		if (status == STATUS_SUCCESSFUL)
		{
			int   j;

			/* random generator - to be changed to DDA functions which use OS API */
			tiwlan_ddaSrand(seed);
			for (j = 0; j < DOT11_CHALLENGE_TEXT_MAX; j++)
				challenge.text[j] = tiwlan_ddaRand();
			seed = tiwlan_ddaRand();
		}
		else
			PAL_memSet(challenge.text, 0, DOT11_CHALLENGE_TEXT_MAX);

		wlan_memMngrFillField(&dataP, (char *)&challenge, sizeof(dot11_Chlg_T), &BD);
		wdrv_mgmtSetChallenge(index, challenge.text);
	}

	if(REPORT_LEVEL(RL_MGMT)) 
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Sending Auth\n");

	/* Swap */
	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	if (wdrv_mgmtLogEnable())
		wdrv_mgmtPrintFrame(MSDUPtr);

	/* Send it to the Tx module */
	txFlags = WDRV_TX_HIGH_PRIORITY | WDRV_TX_UNICAST_MSDU | WDRV_TX_FORCE_UN_ENCRYPTED | WDRV_TX_PS_NOT_ALLOWED;

    wdrv_dbGetStationWoneIndex(index, &wone_index);
	if (wone_index >= MAX_WONE_NET)
	{
		printf("%s: staIndex=%d, woneIndex = %d  !!!\n",__FUNCTION__,index,wone_index);
		wlan_memMngrFreeMSDU(MSDUPtr->handle);
		return;
	}
	
    wdrv_txSendMsdu(MSDUPtr, index, txFlags,wone_index);

	/* Fill the MIB tables */
	if (status != STATUS_SUCCESSFUL)
	{
		wdrv_dbSetStationConfigDeauthenticateFailStatus(status);	
		wdrv_dbSetStationConfigDeauthenticateFailStation(mac);	
	}
	else
		wdrv_mgmtUpdateAuthTimeStamp(index);

	return;
}

/*************************************************************************
 *                        wdrv_mgmtSendDeAuthUnicastByIndex                              *
 *************************************************************************
DESCRIPTION: Sends De authentication unicast frame by the station index                
                                                                                                   
INPUT:      Authentication status
			Station index

OUTPUT:      

RETURN:      

************************************************************************/
void wdrv_mgmtSendDeAuthUnicastByIndex(UINT16 status, UINT16 index)
{
	ULONG txFlags = 0;
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	char mac[MAC_ADDR_LEN];
    UINT8 wone_index;

	/* compute the total len of the frame */
	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN;
		
	/* Allocate data buffers */
	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;
	
	/* Get station MAC */
	wdrv_dbGetStaMacAddress(index, mac);

	/* Build the header */
	buildHeader(mac, hdrP);

	hdrP->fc |= DOT11_FC_DEAUTH;

	BD->length += sizeof(dot11_mgmtHeader_T);	

	/* Now fill the body */
	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	wlan_memMngrFillField(&dataP, (char *)&status, FIX_FIELD_LEN, &BD);

	if(REPORT_LEVEL(RL_MGMT))
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Sending De Auth\n");

	/* Swap */
	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	if (wdrv_mgmtLogEnable())
		wdrv_mgmtPrintFrame(MSDUPtr);

	/* Send it to the Tx module */
	txFlags = WDRV_TX_HIGH_PRIORITY | WDRV_TX_UNICAST_MSDU | WDRV_TX_FORCE_UN_ENCRYPTED;

    wdrv_dbGetStationWoneIndex(index, &wone_index);
	if (wone_index >= MAX_WONE_NET)
	{
		printf("%s: staIndex=%d, woneIndex = %d  !!!\n",__FUNCTION__,index,wone_index);
		wlan_memMngrFreeMSDU(MSDUPtr->handle);
		return;
	}

	wdrv_txSendMsdu(MSDUPtr, index, txFlags,wone_index);

	/* Fill the MIB tables */
	wdrv_dbSetStationConfigDeauthenticateReason(status);
	wdrv_dbSetStationConfigDeauthenticateStation(mac);

	return;
}

/*************************************************************************
 *                        wdrv_mgmtSendDeAuthUnicastByMac                              *
 *************************************************************************
DESCRIPTION: Sends Deauthentication unicast frame by the station mac                 
                                                                                                   
INPUT:      Association status
			Station mac

OUTPUT:      

RETURN:      

************************************************************************/
void wdrv_mgmtSendDeAuthUnicastByMac(UINT16 status, char *mac, UINT8 woneIndex)
{
	ULONG	txFlags;
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;

	/* compute the total len of the frame */
	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN;
		
	/* Allocate data buffers */
	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;
	
	/* Build the header */
	buildHeader(mac, hdrP);

	hdrP->fc |= DOT11_FC_DEAUTH;

	BD->length += sizeof(dot11_mgmtHeader_T);	

	/* Now fill the body */
	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	wlan_memMngrFillField(&dataP, (char *)&status, FIX_FIELD_LEN, &BD);

	if(REPORT_LEVEL(RL_MGMT))
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Sending De Auth\n");

	/* Swap */
	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	if (wdrv_mgmtLogEnable())
		wdrv_mgmtPrintFrame(MSDUPtr);

	/* Send it to the Tx module */
	txFlags = WDRV_TX_HIGH_PRIORITY | WDRV_TX_UNICAST_MSDU | WDRV_TX_FORCE_UN_ENCRYPTED;
	wdrv_txSendMsdu(MSDUPtr, INVALID_INDEX, txFlags, woneIndex);

	/* Fill the MIB tables */
	wdrv_dbSetStationConfigDisassociateReason(status);
	wdrv_dbSetStationConfigDisassociateStation(mac);

	return;
}

/*************************************************************************
 *                        wdrv_mgmtSendDeAuthBcast                              *
 *************************************************************************
DESCRIPTION: Sends De authentication broadcast frame                 
                                                                                                   
INPUT:      Authentication status

OUTPUT:      

RETURN:      

************************************************************************/
void wdrv_mgmtSendDeAuthBcast(UINT16 status)
{
	ULONG txFlags = 0;
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	char mac[MAC_ADDR_LEN] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

    UINT8 wone_index, totalWones;
    mem_MSDU_T *pCopyMsdu = NULL;
	
	/* compute the total len of the frame */
	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN;
		
	/* Allocate data buffers */
	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;
	
	/* Build the header */
	buildHeader(mac, hdrP);

	hdrP->fc |= DOT11_FC_DEAUTH;

	BD->length += sizeof(dot11_mgmtHeader_T);	

	/* Now fill the body */
	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	wlan_memMngrFillField(&dataP, (char *)&status, FIX_FIELD_LEN, &BD);

	if(REPORT_LEVEL(RL_MGMT))
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Sending De Auth\n");

	/* Swap */
	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	if (wdrv_mgmtLogEnable())
		wdrv_mgmtPrintFrame(MSDUPtr);

	/* Send it to the Tx module */
	txFlags = WDRV_TX_HIGH_PRIORITY | WDRV_TX_BRCST_MTCST_MSDU | WDRV_TX_FORCE_UN_ENCRYPTED;
    
    wdrv_dbGetNumSupWone(&totalWones);
    if (totalWones == WONE_DUAL_BAND)
    {
       /* duplicate broadcast packet for second wone */
     
       if ( wlan_memMngrAllocMSDU(&pCopyMsdu, memMgr_MsduDataSize(MSDUPtr)) == OK )
       {
          wlan_memMngrCopyMsdu( pCopyMsdu, MSDUPtr);
          wdrv_txSendMsdu( pCopyMsdu, 0, txFlags,SECOND_WONE_NET);
       }
       else
       {
           if (REPORT_LEVEL(RL_MEM))
               wdrv_utilsReport(RL_RX, PRI_INFORMATION,"fail to wlan_memMngrAllocMSDU");
       }


    }
    wone_index = FIRST_WONE_NET;
   
    wdrv_txSendMsdu(MSDUPtr, 0, txFlags,wone_index);

	/* Fill the MIB tables */
	wdrv_dbSetStationConfigDeauthenticateReason(status);
	wdrv_dbSetStationConfigDeauthenticateStation(mac);

	return;
}

/*************************************************************************
 *                        wdrv_mgmtUpdateApCapability                    *
 *************************************************************************
DESCRIPTION: This function updates the preamble bit & short slot time bit in the AP capability
	If PreambleMode = Always Long then clear preamble bit in AP capability
	If PreambleMode = Always Short then set preamble bit in AP capability
	If PreambleMode = Local STAs then Overwrite AP capability preamble bit
                      with the station capability
    The short slot time bit is depend on the current short slot time state.
                                                                                                   
INPUT:      UINT16 *apCap - the AP capabilities
			UINT16 index - Station index

OUTPUT:     apCap - the AP capabilities after update

RETURN:      
************************************************************************/
void wdrv_mgmtUpdateApCapability(UINT16 index, UINT16 *apCap)
{
UINT16 staCap;
UINT8 preambleMode;
BOOL currentShortSlotTimeSupport;

	wdrv_dbGetPreambleMode(0,&preambleMode);

	if (preambleMode == PREAMBLE_MODE_LOCAL_STATIONS)
	{
		*apCap &= ~(CAP_PREAMBLE_MASK << CAP_PREAMBLE_SHIFT);
		wdrv_dbGetStationCapabilities(index, &staCap);
		*apCap |= (staCap & (CAP_PREAMBLE_MASK << CAP_PREAMBLE_SHIFT));
	}
	else if (preambleMode == PREAMBLE_MODE_LONG)
	{
		*apCap &= ~(CAP_PREAMBLE_MASK << CAP_PREAMBLE_SHIFT);
	}
	else if (preambleMode == PREAMBLE_MODE_SHORT)
	{
		*apCap |= (CAP_PREAMBLE_MASK << CAP_PREAMBLE_SHIFT);
	}

    if(wdrv_dbGetApBridgeMode() && !wdrv_dbIsRateSupported(index, ACX_RATE_ALL_OFDM_MASK))
    {
      
       *apCap &= ~(CAP_PREAMBLE_MASK << CAP_PREAMBLE_SHIFT);
    }

	wdrv_dbGetCurrentShortSlotTimeSupport(&currentShortSlotTimeSupport);
	if (currentShortSlotTimeSupport)
	{
		*apCap |= (CAP_SLOTTIME_MASK << CAP_SLOTTIME_SHIFT);
	}
	else
	{
		*apCap &= ~(CAP_SLOTTIME_MASK << CAP_SLOTTIME_SHIFT);
	}
}


/*************************************************************************
 *                        wdrv_mgmtSendAssocRsp                              *
 *************************************************************************
DESCRIPTION: This function sends an association response frame                 
                                                                                                   
INPUT:      association status
			Station index

OUTPUT:      

RETURN:      

************************************************************************/
void wdrv_mgmtSendAssocRsp(UINT16 status, UINT16 index)
{
	ULONG txFlags = 0;
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	UINT16	cap;
	dot11_supRates_T	rates;
	dot11_extSupRates_T	extRates;
	char mac[MAC_ADDR_LEN];
	UINT16 aid;
	dot11_TIcap_T	TIcapIE;
	BOOL	TIcap;
	UINT8 respRateMode;
	UINT8 CommonFeatureSet;
	int curr;
	UINT8 ERPProtectionTI;
	BOOL FourXsupport,enableConcat,enableAckEmul;
	UINT16 MaxConcatSize,size;
	BOOL ERPProtectionTIStaSupport,ConcatStaSupport,AckEmulStaSupport;
        UINT8 wone_index;

	wdrv_dbGetRespRateMode(&respRateMode);

	rates.hdr.eleId = DOT11_SUPPORTED_RATES_ELE_ID;
	extRates.hdr.eleId = DOT11_EXT_SUPPORTED_RATES_ELE_ID;
	if (respRateMode == ASSOCRESP_RATE_ALL_AP)
	{
		/* Get the rates to be transmitted in Legacy Supported Rates IE */
		wdrv_dbGetStationConfigOperationalRateSet(0,rates.rates, &(rates.hdr.eleLen));
		/* Get the rates to be transmitted in Extended Supported Rates IE */
		wdrv_dbGetStationConfigOperationalOFDMRateSet(0,extRates.rates, &(extRates.hdr.eleLen));
	}
	else
	{
		/* Get the AP rates supported by STA */
		wdrv_dbGetStationAPRates(index, rates.rates, &(rates.hdr.eleLen));
		/* Get the AP extended rates supported by STA */
		wdrv_dbGetStationAPExtRates(index, extRates.rates, &(extRates.hdr.eleLen));
	}

	/* compute the total len of the frame */
	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN + FIX_FIELD_LEN + FIX_FIELD_LEN + sizeof(dot11_eleHdr_T) + rates.hdr.eleLen;

	if (extRates.hdr.eleLen > 0)
	{
		totalLen += sizeof(dot11_eleHdr_T) + extRates.hdr.eleLen;
	}

	wdrv_dbGetStationTIcapEnable(index, &TIcap);

	if (TIcap == TRUE)
	{
		wdrv_dbGetCommonFeatureSet(index, &CommonFeatureSet);
		wdrv_dbGetTIInfoElement(TIcapIE.cap, &(TIcapIE.hdr.eleLen));
		
		curr = DOT11_4X_FEATURE_SET_ZERO_LEN;

		TIcapIE.cap[DOT11_OUI_LEN] = 0;
		TIcapIE.cap[DOT11_OUI_LEN] |= (char)CommonFeatureSet;
		TIcapIE.hdr.eleId = DOT11_TI_CAP_ELE_ID;
		
		/* Build the TIcap IE */
		if(CommonFeatureSet > 0)
		{			
			wdrv_dbGet4xEnable(&FourXsupport);
			if(FourXsupport)
			{
				/* declare concatenation */
				wdrv_dbGetConcatEnable(&enableConcat);
				wdrv_dbGetStationConcatSupport(index, &ConcatStaSupport);
				if(enableConcat && ConcatStaSupport)
				{          
					wdrv_dbGetStationConcatMaxSize(index, &size);
					MaxConcatSize = ENDIAN_HANDLE_WORD(size);
					TIcapIE.cap[curr] = TI_CAP_4X_CONCATENATION;
					curr += sizeof(UINT16);
					TIcapIE.cap[curr] = 2;
					curr += sizeof(UINT16);
					PAL_memCopy(&TIcapIE.cap[curr], (char *)&MaxConcatSize, sizeof(MaxConcatSize));
					curr += 2;
				}
				/* declare Ack emulation */
				wdrv_dbGetAckEmulEnable(&enableAckEmul);
				wdrv_dbGetStationAckEmul(index, &AckEmulStaSupport);
				if(enableAckEmul && AckEmulStaSupport)
				{
					TIcapIE.cap[curr] = TI_CAP_4X_TCP_ACK_EMUL;
					curr += sizeof(UINT16);
					TIcapIE.cap[curr] = 0;
					curr += sizeof(UINT16);
				}
			}
			/* declare trick packet */
			wdrv_dbGetERPProtectionTI(0,&ERPProtectionTI);
			wdrv_dbGetTIprotectionSupport(index, &ERPProtectionTIStaSupport);
			if(ERPProtectionTI && ERPProtectionTIStaSupport)
			{
				TIcapIE.cap[curr] = TI_CAP_TRICK_PACKET_ERP;
				curr += sizeof(UINT16);
				TIcapIE.cap[curr] = 0;
				curr += 2;
			}
		}

		TIcapIE.hdr.eleLen = curr;
		totalLen += sizeof(dot11_eleHdr_T) + TIcapIE.hdr.eleLen;
	}
	
	/* Allocate data buffers */
	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;
	
	/* Get station MAC */
	wdrv_dbGetStaMacAddress(index, mac);

	/* Build the header */
	buildHeader(mac, hdrP);

	hdrP->fc |= DOT11_FC_ASSOC_RESP;

	BD->length += sizeof(dot11_mgmtHeader_T);	

	/* Now fill the body */
	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	wdrv_dbGetApCapabilities(0,&cap);

	wdrv_mgmtUpdateApCapability(index, &cap);
	
	wlan_memMngrFillField(&dataP, (char *)&cap, FIX_FIELD_LEN, &BD);

	wlan_memMngrFillField(&dataP, (char *)&status, FIX_FIELD_LEN, &BD);

	aid = wdrv_mgmtGetStationAid(index);
	
	wlan_memMngrFillField(&dataP, (char *)&aid, FIX_FIELD_LEN, &BD);
	
	wlan_memMngrFillField(&dataP, (char *)&rates, sizeof(dot11_eleHdr_T) + rates.hdr.eleLen, &BD);

	/*#ifdef DOT_11G*/
	if (extRates.hdr.eleLen > 0)
	{
		wlan_memMngrFillField(&dataP, (char *)&extRates, sizeof(dot11_eleHdr_T) + extRates.hdr.eleLen, &BD);
	}
	/*#endif*/

	if (TIcap == TRUE)
		wlan_memMngrFillField(&dataP, (char *)&TIcapIE, TIcapIE.hdr.eleLen + sizeof(dot11_eleHdr_T), &BD);

	if(REPORT_LEVEL(RL_MGMT))
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Sending Assoc Rsp\n");

	/* Swap */
	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	if (wdrv_mgmtLogEnable())
		wdrv_mgmtPrintFrame(MSDUPtr);

	/* Send it to the Tx module */
	txFlags = WDRV_TX_HIGH_PRIORITY | WDRV_TX_UNICAST_MSDU | WDRV_TX_FORCE_UN_ENCRYPTED | WDRV_TX_PS_NOT_ALLOWED;
    wdrv_dbGetStationWoneIndex(index, &wone_index);
	if (wone_index >= MAX_WONE_NET)
	{
		printf("%s: staIndex=%d, woneIndex = %d  !!!\n",__FUNCTION__,index,wone_index);
		wlan_memMngrFreeMSDU(MSDUPtr->handle);
		return;
	}

	wdrv_txSendMsdu(MSDUPtr, index, txFlags,wone_index);

	if (status == STATUS_SUCCESSFUL)
		wdrv_mgmtUpdateAssocTimeStamp(index);

	return;
}

/*************************************************************************
 *                        wdrv_mgmtSendReAssocRsp                              *
 *************************************************************************
DESCRIPTION: This function sends a REassociation response frame                 
                                                                                                   
INPUT:      Reassociation status
			Station index

OUTPUT:      

RETURN:      

************************************************************************/
void wdrv_mgmtSendReAssocRsp(UINT16 status, UINT16 index)
{
	ULONG txFlags;
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	UINT16	cap;
	dot11_supRates_T	rates;
	dot11_extSupRates_T	extRates;
	char mac[MAC_ADDR_LEN];
	UINT16 aid;
	dot11_TIcap_T	TIcapIE;
	BOOL	TIcap;
	UINT8 respRateMode;
	UINT8 CommonFeatureSet;
	int curr;
	UINT8 ERPProtectionTI;
	BOOL FourXsupport,enableConcat,enableAckEmul;
	UINT16 MaxConcatSize, size;
	BOOL ERPProtectionTIStaSupport,ConcatStaSupport,AckEmulStaSupport;
    UINT8 wone_index;
	wdrv_dbGetRespRateMode(&respRateMode);

	rates.hdr.eleId = DOT11_SUPPORTED_RATES_ELE_ID;
	extRates.hdr.eleId = DOT11_EXT_SUPPORTED_RATES_ELE_ID;
	if (respRateMode == ASSOCRESP_RATE_ALL_AP)
	{
		/* Get the rates to be transmitted in Legacy Supported Rates IE */
		wdrv_dbGetStationConfigOperationalRateSet(0,rates.rates, &(rates.hdr.eleLen));
		/* Get the rates to be transmitted in Extended Supported Rates IE */
		wdrv_dbGetStationConfigOperationalOFDMRateSet(0,extRates.rates, &(extRates.hdr.eleLen));
	}
	else
	{
		/* Get the AP rates supported by STA */
		wdrv_dbGetStationAPRates(index, rates.rates, &(rates.hdr.eleLen));
		/* Get the AP extended rates supported by STA */
		wdrv_dbGetStationAPExtRates(index, extRates.rates, &(extRates.hdr.eleLen));
	}

	/* compute the total len of the frame */
	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN + FIX_FIELD_LEN + FIX_FIELD_LEN + sizeof(dot11_eleHdr_T) + rates.hdr.eleLen;

	if (extRates.hdr.eleLen > 0)
	{
		totalLen += sizeof(dot11_eleHdr_T) + extRates.hdr.eleLen;
	}

	wdrv_dbGetStationTIcapEnable(index, &TIcap);

	if (TIcap == TRUE)
	{
		wdrv_dbGetCommonFeatureSet(index, &CommonFeatureSet);
		wdrv_dbGetTIInfoElement(TIcapIE.cap, &(TIcapIE.hdr.eleLen));
		TIcapIE.cap[DOT11_OUI_LEN] |= CommonFeatureSet;
		TIcapIE.hdr.eleId = DOT11_TI_CAP_ELE_ID;
		
		curr = DOT11_4X_FEATURE_SET_ZERO_LEN;
		/* Build the TIcap IE */
		if(CommonFeatureSet > 0)
		{			
			wdrv_dbGet4xEnable(&FourXsupport);
			if(FourXsupport)
			{
				/* declare concatenation */
				wdrv_dbGetConcatEnable(&enableConcat);
				wdrv_dbGetStationConcatSupport(index, &ConcatStaSupport);
				if(enableConcat && ConcatStaSupport)
				{
					wdrv_dbGetStationConcatMaxSize(index, &size);
					MaxConcatSize = ENDIAN_HANDLE_WORD(size);
					TIcapIE.cap[curr] = TI_CAP_4X_CONCATENATION;
					curr += sizeof(UINT16);
					TIcapIE.cap[curr] = 2;
					curr += sizeof(UINT16);
					PAL_memCopy(&TIcapIE.cap[curr], (char *)&MaxConcatSize, sizeof(MaxConcatSize));
					curr += 2;
				}
				/* declare Ack emulation */
				wdrv_dbGetAckEmulEnable(&enableAckEmul);
				wdrv_dbGetStationAckEmul(index, &AckEmulStaSupport);
				if(enableAckEmul && AckEmulStaSupport)
				{
					TIcapIE.cap[curr] = TI_CAP_4X_TCP_ACK_EMUL;
					curr += sizeof(UINT16);
					TIcapIE.cap[curr] = 0;
					curr += sizeof(UINT16);
				}
			}
			/* declare trick packet */
			wdrv_dbGetERPProtectionTI(0,&ERPProtectionTI);
			wdrv_dbGetTIprotectionSupport(index, &ERPProtectionTIStaSupport);			
			if(ERPProtectionTI && ERPProtectionTIStaSupport)
			{
				TIcapIE.cap[curr] = TI_CAP_TRICK_PACKET_ERP;
				curr += sizeof(UINT16);
				TIcapIE.cap[curr] = 0;
				curr += 2;
			}
		}

		TIcapIE.hdr.eleLen = curr;
		printf("AP's 4x IE len = %d\n",TIcapIE.hdr.eleLen);
		totalLen += sizeof(dot11_eleHdr_T) + TIcapIE.hdr.eleLen;
	}
		
	/* Allocate data buffers */
	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;
	
	/* Get station MAC */
	wdrv_dbGetStaMacAddress(index, mac);

	/* Build the header */
	buildHeader(mac, hdrP);

	hdrP->fc |= DOT11_FC_REASSOC_RESP;

	BD->length += sizeof(dot11_mgmtHeader_T);	

	/* Now fill the body */
	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	wdrv_dbGetApCapabilities(0,&cap);

	wdrv_mgmtUpdateApCapability(index, &cap);

	wlan_memMngrFillField(&dataP, (char *)&cap, FIX_FIELD_LEN, &BD);

	wlan_memMngrFillField(&dataP, (char *)&status, FIX_FIELD_LEN, &BD);

	aid = wdrv_mgmtGetStationAid(index);
	
	wlan_memMngrFillField(&dataP, (char *)&aid, FIX_FIELD_LEN, &BD);
	
	wlan_memMngrFillField(&dataP, (char *)&rates, sizeof(dot11_eleHdr_T) + rates.hdr.eleLen, &BD);

	/*#ifdef DOT_11G*/
	if (extRates.hdr.eleLen > 0)
	{
		wlan_memMngrFillField(&dataP, (char *)&extRates, sizeof(dot11_eleHdr_T) + extRates.hdr.eleLen, &BD);
	}
	/*#endif*/

	if (TIcap == TRUE)
		wlan_memMngrFillField(&dataP, (char *)&TIcapIE, TIcapIE.hdr.eleLen + sizeof(dot11_eleHdr_T), &BD);

	if(REPORT_LEVEL(RL_MGMT))
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Sending Re Assoc Rsp\n");

	/* Swap */
	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	if (wdrv_mgmtLogEnable())
		wdrv_mgmtPrintFrame(MSDUPtr);

	/* Send it to the Tx module */
	txFlags = WDRV_TX_HIGH_PRIORITY | WDRV_TX_UNICAST_MSDU | WDRV_TX_FORCE_UN_ENCRYPTED | WDRV_TX_PS_NOT_ALLOWED;
    wdrv_dbGetStationWoneIndex(index, &wone_index);
	if (wone_index >= MAX_WONE_NET)
	{
		printf("%s: staIndex=%d, woneIndex = %d  !!!\n",__FUNCTION__,index,wone_index);
		wlan_memMngrFreeMSDU(MSDUPtr->handle);
		return;
	}

	wdrv_txSendMsdu(MSDUPtr, index, txFlags,wone_index);

	if (status == STATUS_SUCCESSFUL)
		wdrv_mgmtUpdateAssocTimeStamp(index);

	return;
}

/*************************************************************************
 *                        wdrv_mgmtSendNullPacket                        *
 *************************************************************************
DESCRIPTION: Sends Null unicast frame by the station index                 
                                                                                                   
INPUT:      Station index

OUTPUT:      

RETURN:      

************************************************************************/
STATUS wdrv_mgmtSendNullPacket(UINT16 index)
{
	mem_MSDU_T* pMsdu;
	char 		mac[MAC_ADDR_LEN];
	ULONG		txFlags;
	dot11_header_T     *pdot11Header;
	UINT8 		BSSID[6];
    UINT8 wone_index;

	if (wlan_memMngrAllocMSDU(&pMsdu, WLAN_HDR_LEN) != OK)
		return NOK;

	pdot11Header = (dot11_header_T*)memMgr_BufData(pMsdu->firstBDPtr);

	/* The first BD contains the WLAN and SNAP headers only. */
	pMsdu->firstBDPtr->length = WLAN_HDR_LEN;
	pMsdu->firstBDPtr->dataOffset = 0;
	pMsdu->firstBDPtr->nextBDPtr = NULL;
	pMsdu->dataLen = WLAN_HDR_LEN;

	pMsdu->headerLen = WLAN_HDR_LEN;
	pdot11Header->fc = DOT11_FC_DATA_NULL_FUNCTION | DOT11_FC_FROM_DS;

	wdrv_dbGetStaMacAddress(index, mac);

	PAL_memCopy((void*)&pdot11Header->address1,mac,6); /* dest address */
	wdrv_dbGetBssid(BSSID);
	PAL_memCopy((void*)&pdot11Header->address2, (void*)BSSID,6); /* BSSID */
	PAL_memCopy((void*)&pdot11Header->address3, (void*)BSSID,6); /* source address */

	txFlags = WDRV_TX_LOW_PRIORITY | WDRV_TX_UNICAST_MSDU;

    wdrv_dbGetStationWoneIndex(index, &wone_index);
	if (wone_index >= MAX_WONE_NET)
	{
		printf("%s: staIndex=%d, woneIndex = %d  !!!\n",__FUNCTION__,index,wone_index);
		wlan_memMngrFreeMSDU(pMsdu->handle);
		return NOK;
	}

	wdrv_txSendMsdu(pMsdu, index, txFlags,wone_index);

	return OK;
}


/*************************************************************************
 *                        wdrv_mgmtSendDisAssocUnicastByIndex                              *
 *************************************************************************
DESCRIPTION: Sends Disassociation unicast frame by the station index                 
                                                                                                   
INPUT:      Association status
			Station index

OUTPUT:      

RETURN:      

************************************************************************/
void wdrv_mgmtSendDisAssocUnicastByIndex(UINT16 status, UINT16 index)
{
	ULONG	txFlags = 0;
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	char mac[MAC_ADDR_LEN];
    UINT8 wone_index;

	/* compute the total len of the frame */
	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN;
		
	/* Allocate data buffers */
	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;
	
	/* Get station MAC */
	wdrv_dbGetStaMacAddress(index, mac);

	/* Build the header */
	buildHeader(mac, hdrP);

	hdrP->fc |= DOT11_FC_DISASSOC;

	BD->length += sizeof(dot11_mgmtHeader_T);	

	/* Now fill the body */
	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	wlan_memMngrFillField(&dataP, (char *)&status, FIX_FIELD_LEN, &BD);

	if(REPORT_LEVEL(RL_MGMT))
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Sending Dis Assoc\n");

	/* Swap */
	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	if (wdrv_mgmtLogEnable())
		wdrv_mgmtPrintFrame(MSDUPtr);

	/* Send it to the Tx module */
	txFlags = WDRV_TX_HIGH_PRIORITY | WDRV_TX_UNICAST_MSDU | WDRV_TX_FORCE_UN_ENCRYPTED;
    wdrv_dbGetStationWoneIndex(index, &wone_index);
	if (wone_index >= MAX_WONE_NET)
	{
		printf("%s: staIndex=%d, woneIndex = %d  !!!\n",__FUNCTION__,index,wone_index);
		wlan_memMngrFreeMSDU(MSDUPtr->handle);
		return;
	}

	wdrv_txSendMsdu(MSDUPtr, index, txFlags,wone_index);

	/* Fill the MIB tables */
	wdrv_dbSetStationConfigDisassociateReason(status);
	wdrv_dbSetStationConfigDisassociateStation(mac);

	return;
}

/*************************************************************************
 *                        wdrv_mgmtSendDisAssocBcast                              *
 *************************************************************************
DESCRIPTION: Sends Dis Association broadcast frame                 
                                                                                                   
INPUT:      Authentication status

OUTPUT:      

RETURN:      

************************************************************************/
void wdrv_mgmtSendDisAssocBcast(UINT16 status)
{
	ULONG txFlags = 0;
	mem_MSDU_T* MSDUPtr;
	UINT32	totalLen;
	dot11_mgmtHeader_T *hdrP;
	mem_BD_T *BD;
	char *dataP;
	char mac[MAC_ADDR_LEN] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

    UINT8 wone_index, totalWones;
    mem_MSDU_T *pCopyMsdu = NULL;
	
	/* compute the total len of the frame */
	totalLen = sizeof(dot11_mgmtHeader_T) + FIX_FIELD_LEN;
		
	/* Allocate data buffers */
	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = sizeof(dot11_mgmtHeader_T);

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = 0;

	hdrP = (dot11_mgmtHeader_T *)(BD->data + BD->dataOffset) ;
	
	/* Build the header */
	buildHeader(mac, hdrP);

	hdrP->fc |= DOT11_FC_DISASSOC;

	BD->length += sizeof(dot11_mgmtHeader_T);	

	/* Now fill the body */
	dataP = (char *)(BD->data + BD->dataOffset) + sizeof(dot11_mgmtHeader_T);

	wlan_memMngrFillField(&dataP, (char *)&status, FIX_FIELD_LEN, &BD);

	if(REPORT_LEVEL(RL_MGMT))
		wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: Sending Dis Assoc\n");

	/* Swap */
	wdrv_mgmtSwap((dot11_mgmtFrame_T *)hdrP);
	
	if (wdrv_mgmtLogEnable())
		wdrv_mgmtPrintFrame(MSDUPtr);

	/* Send it to the Tx module */
	txFlags = WDRV_TX_HIGH_PRIORITY | WDRV_TX_BRCST_MTCST_MSDU | WDRV_TX_FORCE_UN_ENCRYPTED;
	
    wdrv_dbGetNumSupWone(&totalWones);
    if (totalWones == WONE_DUAL_BAND)
    {
       /* duplicate broadcast packet for second wone */
     
       if ( wlan_memMngrAllocMSDU(&pCopyMsdu, memMgr_MsduDataSize(MSDUPtr)) == OK )
       {
          wlan_memMngrCopyMsdu( pCopyMsdu, MSDUPtr);
          wdrv_txSendMsdu( pCopyMsdu, 0, txFlags,SECOND_WONE_NET);
       }
       else
       {
           if (REPORT_LEVEL(RL_MEM))
               wdrv_utilsReport(RL_RX, PRI_INFORMATION,"fail to wlan_memMngrAllocMSDU");
       }

    }
    wone_index = FIRST_WONE_NET;
    
    wdrv_txSendMsdu(MSDUPtr, 0, txFlags,wone_index);

	/* Fill the MIB tables */
	wdrv_dbSetStationConfigDisassociateReason(status);
	wdrv_dbSetStationConfigDisassociateStation(mac);

	return;
}

/*************************************************************************
 *                        buildHeader                              *
 *************************************************************************
DESCRIPTION: Build a mgmt frame header                 
                                                                                                   
INPUT:      Destination mac
			Header pointer.

OUTPUT:      

RETURN:      

************************************************************************/
static void buildHeader(char *dstMac, dot11_mgmtHeader_T *hdrP)
{

	/* build frame control first */
	hdrP->fc = 0;
	
	hdrP->fc |= DOT11_FC_PROT_VERSION;

	hdrP->fc |= DOT11_FC_TYPE_MGMT;

	hdrP->fc |= (FALSE << DOT11_FC_TO_DS_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_FROM_DS_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_MORE_FRAG_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_RETRY_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_PWR_MGMT_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_MORE_DATA_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_WEP_SHIFT);

	hdrP->fc |= (FALSE << DOT11_FC_ORDER_SHIFT);

	/* fill DA*/
	PAL_memCopy(hdrP->DA.addr, dstMac, MAC_ADDR_LEN);

	/* fill SA*/
	wdrv_dbGetOperationMacAddress(hdrP->SA.addr);

	/* fill BSSID*/
	wdrv_dbGetBssid(hdrP->BSSID.addr);
}


