/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*		MODULE:	mgmtSm.h												   */
/*    PURPOSE:	Authorization/association state machine implementation	   */
/*																	 	   */
/***************************************************************************/

#ifndef _MGMT_SM_H
#define _MGMT_SM_H

#define SM_DEBUG	1

#include "memMngr.h"
#include "pal.h"

#define OPEN_SYSTEM	0
#define SHARED_KEY	1

/* state machine states */
typedef enum {
	STATE_IDLE					= 0,
	STATE_AUTH_KEY				= 1,
	STATE_AUTHENTICATED			= 2,
	STATE_WAIT_FOR_ASS_RES  	= 3,	
	STATE_ASSOCIATED			= 4,
	STATE_AUTHORIZED			= 5,
	NUM_STATES					= 6
} states;



/* 802.11 events */
typedef enum {
	EV_11_NONE			  =	0,
	EV_11_AUTH_1		  =	1,
	EV_11_AUTH_3		  =	2,	
	EV_11_DE_AUTH		  =	3,	
	EV_11_ASSOC_REQ		  =	4,
	EV_11_ASSOC_REQ_REP	  =	5,
	EV_11_RE_ASSOC_REQ	  =	6,
	EV_11_DIS_ASSOC		  =	7,
} wdrv_mgmtProtoEv_T;

/* State machine inputs */
typedef enum {
	INPUT_AUTH_1_OS_A				=	0,
	INPUT_AUTH_1_SK_A				=	1,
	INPUT_AUTH_1_D					=	2,	
	INPUT_AUTH_3_A					=	3,
	INPUT_AUTH_3_D					=	4,	
	INPUT_DE_AUTH_STATION			=	5,		/* A station sent DeAuthentication message */
	INPUT_ASSOC_REQ_CHECK           =   6,   
	INPUT_ASSOC_REQ_A				=	7,
	INPUT_ASSOC_REQ_D				=	8,
	INPUT_RE_ASSOC_REQ_CHECK		=	9,
	INPUT_RE_ASSOC_REQ_A			=	10,
	INPUT_RE_ASSOC_REQ_D			=	11,
	INPUT_DIS_ASSOC_STATION			=	12,		/* A station sent Disassociation message */
	INPUT_CLASS_3					=	13,		
	INPUT_AUTH_TIMEOUT				=	14,
	INPUT_DE_AUTH_MNG_UNICAST		=	15,		/* Management unicast DeAuthentication  - by console command */
	INPUT_DE_AUTH_MNG_BROADCAST		=	16,		/* Management broadcast DeAuthentication  - by console command */
	INPUT_DIS_ASSOC_MNG_UNICAST		=	17,		/* Management unicast Disassociation  - by console command */
	INPUT_DIS_ASSOC_MNG_BROADCAST	=	18,		/* Management broadcast Disassociation  - by console command */
	INPUT_MAC_AUTHORIZE				=	19,		/* Management broadcast Disassociation  - by console command */
	INPUT_MAC_DE_AUTHORIZE			=	20,		/* Management broadcast Disassociation  - by console command */
} wdrv_mgmtSmInput_T;

/* Status codes */
typedef enum {
	STATUS_SUCCESSFUL												=	0,
	STATUS_UNSPECIFIED												=	1,												
	STATUS_AUTH_NO_LONGER_VALID										=	2,										
	STATUS_DE_AUTH_STATION_IS_LEAVING								=	3,								
	STATUS_DIS_ASSOC_INACTIVITY										=	4,										
	STATUS_DIS_ASSOC_UNABLE_TO_HANDLE_ALL_CURRENTLY_ASSOC_STATIONS	=	5,		
	STATUS_CLASS_2_FROM_NON_AUTH_STATION							=	6,								
	STATUS_CLASS_3_FROM_NON_ASSOC_STATION							=	7,								
	STATUS_DIS_ASSOC_STATION_IS_LEAVING								=	8,									
	STATUS_STATION_REQUESTING_ASSOC_IS_NOT_AUTH						=	9,							
	STATUS_CAPABILITIES_FAILURE										=	10,											
	STATUS_REASSOC_DENIED_UNABLE_TO_CONFIRM_ASSOC					=	11,						
	STATUS_ASSOC_DENIED_OUT_OF_SCOPE								=	12,									
	STATUS_ALGO_IS_NOT_SUPPORTTED									=	13,										
	STATUS_AUTH_TRANSC_NUMBER_OUT_OF_SEQUENCE						=	14,							
	STATUS_AUTH_REJ_CHALLENGE_FAILURE								=	15,									
	STATUS_AUTH_REJ_TIMEOUT											=	16,											
	STATUS_ASSOC_DENIED_UNABLE_TO_HANDLE_ADDITIONAL_ASSOC			=	17,				
	STATUS_ASSOC_DENIED_RATE_FAILURE								=	18,									
	STATUS_ASSOC_DENIED_PREAMBLE_FAILURE							=	19,
	STATUS_ASSOC_DENIED_PBCC_FAILURE								=	20,
	STATUS_ASSOC_DENIED_AGILITY_FAILURE								=	21,
	STATUS_MAX														=	22,
	STATUS_INVALID													=	65535,
} wdrv_mgmtStatus_T;

typedef enum
{
	TEMPLATE_SET_PBCC = 0,
	TEMPLATE_SET_ERP = 1,
	TEMPLATE_SET_INVALID
} wdrv_mgmtSetTemplate_T;

typedef enum
{
	INPUT_SLOTTIME_B_STA_IN_BSS = 0,
	INPUT_SLOTTIME_G_STA_ONLY = 1,
	INPUT_SLOTTIME_CHECK_TIMEOUT
} wdrv_mgmtSlotTimeInputs_T;

typedef enum
{
	STATE_SLOTTIME_B_STA_IN_BSS = 0,
	STATE_SLOTTIME_G_STA_ONLY
} wdrv_mgmtSlotTimeState_T;

/*Management Incoming Messages type definition */
typedef enum {
	MGMT_FRAME				=	1,	/* mgmt frame arrived */
	CLASS3					=	2,	/* data received from un associated station */
	AUTH_RESPONSE_TIMEOUT	=	3,	/* auth rsp timeout message received from the TCB */
	AGING					=	4,	/* aging message received from the TCB */
	DEAUTH_MNG				=	5,	/* De authentication message received from the operator via the console */
	DISASSOC_MNG			=	6,	/* DisAssociation message received from the operator via the console */
	BEACON_FRAME			=   7,  /* Beacon frame received ( for ERP and PBCC usage ) */
	BEACON_ERP_TIMEOUT		=   8,  /* Beacon timeout received (for ERP usage) */
	BEACON_PBCC_TIMEOUT  	=   9,  /* Beacon timeout received (for PBCC usage) */
	DYNAMIC_ALG_TIMEOUT     =   10, /* PBCC, ERP, Slot time enhanced dynamic algorithms timeout event */
	SET_TEMPLATE_REQ		=   11, /* Request to MGMT task to update Beacon and/or ProbeRsp template */
	RADIO_CALIBRATE_TIMEOUT	=   12, /* RADIO CALIBRATE timeout event */
	MAC_AUTHORIZE			=	13,	/* DisAssociation message received from the operator via the console */
	MAC_DE_AUTHORIZE		=	14,	/* DisAssociation message received from the operator via the console */
	WPA_ASSOC_REQ_REP		=	15,	/* response for asso request from wpa module*/ 
	RADIO_CALIBRATE_TIMEOUT_CRT
} mgmtMsgType;

/* Management messages structure */
typedef struct {
   int type;
   union {
      long MessageInfo[3];
      struct {
            mem_MSDU_T*  msduPtr;	   
			UINT16	index;	
      } frame;
      struct {
			char    macAddress[MAC_ADDR_LEN];
			UINT16	index;	
            UINT8 wone_index;
      } class3;
      struct {
			UINT16	index;	
      } authTimeout;
       struct {
		    char    macAddress[MAC_ADDR_LEN];		
		    UINT16	reason;			
      } deAuthMng;
       struct {
		    char    macAddress[MAC_ADDR_LEN];		
		    UINT16	reason;			
      } disAssocMng;
       struct {
		    char    macAddress[MAC_ADDR_LEN];		
      } macAuthorize;
       struct {
		    char    macAddress[MAC_ADDR_LEN];		
      } macDeAuthorize;
	  
	  struct {
			UINT16	index;
			UINT8   flag;
      }wpaAssCheckRep;
	
  } content;
} mgmtMessage_T;


BOOL		wdrv_mgmtIsChallengeEqual(UINT16 index, char *challengeP);
UINT16		wdrv_mgmtGetStationAid(UINT16 index);
void		wdrv_mgmtSetLog(BOOL mode);
void		wdrv_mgmtSetAging(BOOL mode);
BOOL		wdrv_mgmtAgingEnable(void);
BOOL		wdrv_mgmtLogEnable(void); 
void		wdrv_mgmtReset(void);
int 		wdrv_mgmtPrintChallenge(UINT16 index,char *buf);
void		wdrv_mgmtSetChallenge(UINT16 index, char *text);
void		wdrv_mgmtUpdateAuthTimeStamp(UINT16 idx);
void		wdrv_mgmtUpdateAssocTimeStamp(UINT16 idx);
UINT32		wdrv_mgmtReadAuthTimeStamp(UINT16 idx);
UINT32		wdrv_mgmtReadAssocTimeStamp(UINT16 idx);
UINT32		wdrv_mgmtReadTimeStamp(UINT16 idx);
void		wdrv_mgmtGetState(UINT16 index, UINT32 *state);
void		wdrv_mgmtGetStateString(UINT16 index, char *stateString);
BOOL		wdrv_mgmtIsStaAssoc(UINT16 idx);
void 		wdrv_mgmtIsERPexternalActive(BOOL *active);
void 		wdrv_mgmtIsPBCCexternalActive(BOOL *active);
void 		wdrv_mgmtSetNotPBCCTimestamp(UINT32 timestamp);
void 		wdrv_mgmtSetNotShortTimeSlotTimestamp(UINT32 timestamp);
void 		wdrv_mgmtSetNotErpTimestamp(UINT32 timestamp);
void		wdrv_mgmtShortSlotTimeSM(int input);
void        wdrv_mgmtSetHiddenSSID(char *pSSID, int len);
/*void        wdrv_mgmtProbeRequestPacketMgmt(mem_BD_T *pduPtr, UINT32 mpduLen, RxAttr_T rxAttr);*/
STATUS		wdrv_mgmt_getEvent(mgmtMessage_T	*pMessage);
void        wdrv_mgmtRadioCalibrateStop(void);
void        wdrv_mgmtRadioCalibrateInit(void);
STATUS		wdrv_mgmtInit(void);
void		wdrv_mgmtUnload(void);
void wdrv_mgmtMacAuthorize(char *mac);
	
#endif /* _MGMT_SM_H */
