/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************/
/*																			*/
/*		MODULE:		mgmtSm.c												*/
/*		PURPOSE:	mgmt state machine system check implementation			*/
/*      Updates:    ZZ, 09,July,2003                                        */
/*																			*/
/****************************************************************************/
#include "mainsm.h"
#include "mgmtSystem.h"
#include "802_11Defs.h"
#include "mgmtSm.h"
#include "pal.h"
#include "drvDbHash.h"
#include "drvDbMain.h"
#include "dda_wpa.h"
/*#include "wpaIfc.h"*/


/* Mgmt Access Control */

typedef STATUS (*wdrv_accessCallback_t)(char * mac_addr, int mac_addr_len);
static wdrv_accessCallback_t wdrv_accessAuthCallback = NULL, wdrv_accessAssocCallback = NULL;


static int wdrv_accessAuthCheck(int index)
{
    char mac[MAC_ADDR_LEN];
    int res = OK;

    if ( wdrv_accessAuthCallback != NULL) 
    {
        wdrv_dbGetStaMacAddress(index, mac);
        res =  wdrv_accessAuthCallback(mac, MAC_ADDR_LEN);
    }
    
    return res;
}


static STATUS wdrv_accessAssocCheck(int index) 
{
    char mac[MAC_ADDR_LEN];
    int res = OK;
    
    if ( wdrv_accessAssocCallback != NULL) 
    {
        wdrv_dbGetStaMacAddress(index, mac);
        res =  wdrv_accessAssocCallback(mac, MAC_ADDR_LEN);
    }
    
    return res;
}


void wdrv_accessSetAuthCallback(wdrv_accessCallback_t callback) 
{
    wdrv_accessAuthCallback = callback;
}


void wdrv_accessSetAssocCallback(wdrv_accessCallback_t callback) 
{
    wdrv_accessAssocCallback = callback;
}


/*************************************************************************
 *                        wdrv_mgmtSystemCheck                           *
 *************************************************************************
DESCRIPTION: This function tests a frame received from the station under the system condition
			 Called by the mgmt sm upon receiving a mgmt frame.
                                                                                                   
INPUT:       index - a station index 
			 protoEvent - the event as retrieved from the frame by the mgmt parser. 
			 protoPtr - the frame information as retrieved from the frame by the mgmt parser
			 
OUTPUT:      systemInfo - pointer to the system information structure. This structure contains the following:
			 event - a system event. Unlike the protoEvent which is a general one, 
					the system event describes a particular event corresponds to the system resources status.
			 status - result of the process. (successful or not, and if not successful the reason for it)

RETURN:      

************************************************************************/
void wdrv_mgmtSystemCheck(UINT16 index, UINT32 protoEvent, void *protoPtr, wdrv_mgmtSysInfo_T *systemInfo)
{
	dot11_auth_T			*authP;
	dot11_deAuth_T			*deAuthP;
	dot11_assocReq_T		*assocReqP;
	dot11_reAssocReq_T		*reAssocReqP;
	dot11_disAssoc_T		*disAssocP;
    UINT8                   woneIndex =0;
	int wpaMode;


	systemInfo->status = STATUS_UNSPECIFIED;

    wdrv_dbGetStationWoneIndex(index ,&woneIndex);
	if(woneIndex >= MAX_WONE_NET)
	{
		printf("wdrv_mgmtSystemCheck: staIndex=%d, woneIndex=%d\n",index,woneIndex);
		return; 
	}

	wdrv_dbGetWpaMode(&wpaMode);

	/* for each protocol event, different system conditions has to be tested */
	switch (protoEvent)
	{
		case EV_11_AUTH_1:
			if(REPORT_LEVEL(RL_MGMT)) 
				wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: System input = EV_11_AUTH_1\n");

            if (wdrv_accessAuthCheck(index) != OK) 
            {
				systemInfo->event = INPUT_AUTH_1_D;
				systemInfo->status = STATUS_ASSOC_DENIED_OUT_OF_SCOPE;
				return;
            }

			authP = protoPtr;
			if (!wdrv_dbAuthAlgoSupported(*(authP->algoNumber)))
			{
				systemInfo->event = INPUT_AUTH_1_D;
				systemInfo->status = STATUS_ALGO_IS_NOT_SUPPORTTED;
				return;
			}
			if (*(authP->algoNumber) == OPEN_SYSTEM)
			{
				systemInfo->event = INPUT_AUTH_1_OS_A;
				systemInfo->status = STATUS_SUCCESSFUL;
				return;
			}
			systemInfo->event = INPUT_AUTH_1_SK_A;
			systemInfo->status = STATUS_SUCCESSFUL;
			return;

		case EV_11_AUTH_3:
			if(REPORT_LEVEL(RL_MGMT)) 
				wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: System input = EV_11_AUTH_3\n");

            if (wdrv_accessAuthCheck(index) != OK) 
            {
				systemInfo->event = INPUT_AUTH_3_D;
				systemInfo->status = STATUS_ASSOC_DENIED_OUT_OF_SCOPE;
				return;
            }
			authP = protoPtr;
			if ((*authP->algoNumber) != SHARED_KEY)
			{
				systemInfo->event = INPUT_AUTH_3_D;
				if ((*authP->algoNumber) == OPEN_SYSTEM)
					systemInfo->status = STATUS_UNSPECIFIED;
				else
					systemInfo->status = STATUS_ALGO_IS_NOT_SUPPORTTED;

				return;
			}
			if (!authP->wepOn)
			{
				systemInfo->event = INPUT_AUTH_3_D;
				systemInfo->status = STATUS_AUTH_REJ_CHALLENGE_FAILURE;
				return;
			}
			if (!wdrv_mgmtIsChallengeEqual(index, authP->challenge->text))
			{
				systemInfo->event = INPUT_AUTH_3_D;
				systemInfo->status = STATUS_AUTH_REJ_CHALLENGE_FAILURE;
				return;
			}
			systemInfo->event = INPUT_AUTH_3_A;
			systemInfo->status = STATUS_SUCCESSFUL;
			return;

		case EV_11_DE_AUTH:
			if(REPORT_LEVEL(RL_MGMT)) 
				wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: System input = EV_11_DE_AUTH\n");

			deAuthP = protoPtr;
			systemInfo->event = INPUT_DE_AUTH_STATION;
			systemInfo->status = *(deAuthP->reason);
			return;

		case EV_11_ASSOC_REQ:
			if(REPORT_LEVEL(RL_MGMT)) 
				wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: System input = EV_11_ASSOC_REQ\n");

			assocReqP = protoPtr;
            if (wdrv_accessAssocCheck(index) != OK) 
            {
				systemInfo->event = INPUT_ASSOC_REQ_D;
				systemInfo->status = STATUS_ASSOC_DENIED_OUT_OF_SCOPE;
				
				return;
            }

			if (!wdrv_dbSSIDSupport(assocReqP->SSID))
			{
				systemInfo->event = INPUT_ASSOC_REQ_D;
				systemInfo->status = STATUS_ASSOC_DENIED_OUT_OF_SCOPE;
				return;
			}
   		
			if (!wdrv_dbRatesSupport(woneIndex ,assocReqP->supportedRates, assocReqP->extSupportedRates))
			{
				systemInfo->event = INPUT_ASSOC_REQ_D;
				systemInfo->status = STATUS_ASSOC_DENIED_RATE_FAILURE;
				return;
			}
		
			if (!wdrv_dbCapabilitiesSupport(woneIndex,assocReqP->capabilities))
			{
				systemInfo->event = INPUT_ASSOC_REQ_D;
				systemInfo->status = STATUS_CAPABILITIES_FAILURE;
				return;
			}
		
		
			systemInfo->event  = INPUT_ASSOC_REQ_CHECK;
			systemInfo->status = STATUS_SUCCESSFUL;
			return;

		case EV_11_RE_ASSOC_REQ:
			if(REPORT_LEVEL(RL_MGMT)) 
				wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: System input = EV_11_RE_ASSOC_REQ\n");

            
			reAssocReqP = protoPtr;
            if (wdrv_accessAssocCheck(index) != OK) 
            {
				systemInfo->event = INPUT_RE_ASSOC_REQ_D;
				systemInfo->status = STATUS_ASSOC_DENIED_OUT_OF_SCOPE;
				return;
            }
			if (!wdrv_dbSSIDSupport(reAssocReqP->SSID))
			{
				systemInfo->event = INPUT_RE_ASSOC_REQ_D;
				systemInfo->status = STATUS_ASSOC_DENIED_OUT_OF_SCOPE;
				return;
			}
			if (!wdrv_dbRatesSupport(woneIndex, reAssocReqP->supportedRates, reAssocReqP->extSupportedRates))
			{
				systemInfo->event = INPUT_RE_ASSOC_REQ_D;
				systemInfo->status = STATUS_ASSOC_DENIED_RATE_FAILURE;
				return;
			}
			if (!wdrv_dbCapabilitiesSupport(woneIndex,reAssocReqP->capabilities))
			{
				systemInfo->event = INPUT_RE_ASSOC_REQ_D;
				systemInfo->status = STATUS_CAPABILITIES_FAILURE;
				return;
			}


			systemInfo->event  = INPUT_RE_ASSOC_REQ_CHECK;
			systemInfo->status = STATUS_SUCCESSFUL;
			return;
			

		case EV_11_DIS_ASSOC:
			if(REPORT_LEVEL(RL_MGMT)) 
				wdrv_utilsReport(RL_MGMT, PRI_INFORMATION, "WDRV_MGMT: System input = EV_11_DIS_ASSOC\n");

			disAssocP = protoPtr;
			systemInfo->event = INPUT_DIS_ASSOC_STATION;
			systemInfo->status = *(disAssocP->reason);
			return;
	}
}

