/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*		MODULE:	mpduCollec.c											   */
/*    	PURPOSE: Collection of the MPDUs for each MSDU/MMPDU			   */
/*																		   */
/***************************************************************************/

#include "pal.h"
#include "rx.h"
#include "802_11Defs.h"
#include "memMngr.h"
#include "bssbridge.h"

typedef struct {
	UINT32		seqNumber; 	/* The sequence number of the last MSDU received from the station  */
	UINT32		fragNumber;	/* The sequence number of the last fragment received from the station  */
	mem_BD_T	*lastPtr;  	/* Pointer to the last fragment in the collection.  */
    mem_MSDU_T  *msduPtr;    /* Pointer to the MSDU structure */
	BOOL		collect;  	/* A Boolean, set to TRUE if the station is in a middle of a fragment collection. */
	UINT32		timeStamp;  	/* A timestamp contains the receive time of the first fragment in the collection, if any. */
} CollectEntry_T;



CollectEntry_T* 	CollectTable;


UINT32  dupDetectTable[MAX_STATIONS];

static UINT32		mpduValidFrames = 0;
static UINT32		duplicateMsdu = 0;

#if 0
static void freeCollectEntry(CollectEntry_T* collectEnt)
{
/*  	CollectEnt->seqNumber  - Stays the same */
  	collectEnt->fragNumber = 0;
  	collectEnt->lastPtr = NULL;
  	collectEnt->msduPtr = NULL;
  	collectEnt->collect = FALSE;
  	collectEnt->timeStamp = 0;
}
#endif

STATUS wdrv_rxMpduCollectReset(void)
{
    /* reset */
	PAL_memSet(CollectTable, 0, (MAX_STATIONS * sizeof(CollectEntry_T)));

    return OK;
}

#if 0
STATUS wdrv_rxFragCollectInit(void)
{
	int index;

	CollectTable = (CollectEntry_T*)PAL_memAlloc(0,MAX_STATIONS*sizeof(CollectEntry_T),0,0);

	if (CollectTable == NULL) {
      	if (REPORT_LEVEL(RL_RX)) {
	    	wdrv_utilsReport(RL_RX,PRI_EMERGENCY,
    	    "FATAL ERROR: Could not allocate memory for Mpdu Collector - Aborting\n");
      	}
        return NOK;
    }

	wdrv_rxMpduCollectReset();
	for (index=0; index<MAX_STATIONS; index++)
    {
       freeCollectEntry(&(CollectTable[index]));
       CollectTable[index].seqNumber = 0xFFFFFFFF; 
    }
	return OK;
}
#endif

void wdrv_rxDupDetectInit(void)
{
   int index;
   for (index=0; index<MAX_STATIONS; index++)
   {
      dupDetectTable[index] = 0xFFFFFFFF; 
   }
}




collect_status_T wdrv_rxDupMpduDetect(mem_BD_T* pMpdu, UINT16 staInd,
						UINT32 mpduLen, mem_MSDU_T **msduPtr)
{
    dot11_header_T*	pHeader;
    UINT32          seqNumber;
    UINT32 prevSeqNumber = dupDetectTable[staInd];
/*    printf("wdrv_rxMpduCollect  CollectTable %x CollectTable[staInd] %x &(CollectTable[staInd]) %x "
            " collectEnt %x\n",CollectTable, CollectTable[staInd],&(CollectTable[staInd]),collectEnt);
    */
    *msduPtr = NULL;
    pHeader = (dot11_header_T*)(pMpdu->data);
/*
	fragNumber = (pHeader->seqCtrl & 0x000F);
    seqNumber = ((pHeader->seqCtrl & 0xFFF0) >> 4);
    */

	
    pHeader->seqCtrl = htons(pHeader->seqCtrl);

    seqNumber = ((((pHeader->seqCtrl & 0x00FF) << 4) & 0xFF0) | (((pHeader->seqCtrl & 0xF000) >> 12) & 0xF));


   /*We should check retry bit according to section 9.2.9 in 802.11-1999. */
   if ((seqNumber == prevSeqNumber) && (pHeader->fc & DOT11_FC_RETRY))
   {
      /* Duplicate MSDU - new MSDU with the same Sequence number as previous MSDU */
      wlan_memMngrFreeBD(pMpdu->handle);
      duplicateMsdu++;
      /*if (REPORT_LEVEL(RL_RX))
      {
         wdrv_utilsReport(RL_RX,PRI_EMERGENCY,
                          "Rx: duplicate MSDU seqNumber = %d", seqNumber);
      }*/

      return MPDU_DROP;
   }
		
  /* New MSDU */
   mpduValidFrames++;
   wlan_memMngrAllocMSDUBufferOnly(msduPtr);
   if (*msduPtr == NULL)
   {
      if (REPORT_LEVEL(RL_RX))
      {
         wdrv_utilsReport(RL_RX,PRI_EMERGENCY,
                          "Rx: Can not allocate new MSDU buffer");
      }
      wlan_memMngrFreeBD(pMpdu->handle);
      return MPDU_DROP;
   }


   (*msduPtr)->headerLen = WLAN_HDR_LEN;
   (*msduPtr)->firstBDPtr = pMpdu;
   (*msduPtr)->dataLen = mpduLen;
   
   dupDetectTable[staInd] = seqNumber;

   return MSDU_READY;
}


#if 0

void wdrv_rxFragCollectUnload(void)
{
	if(CollectTable)
		PAL_memFree(0,CollectTable,0);
}

collect_status_T wdrv_rxMpduCollect(mem_BD_T* pMpdu, UINT16 staInd,
						UINT32 mpduLen, mem_MSDU_T **msduPtr)
{
    dot11_header_T*	pHeader;
    dot11_Fc_Type_T mpduType;
    UINT8           fragNumber;
    UINT32          seqNumber;
    UINT32          moreFrag;
    CollectEntry_T* collectEnt = &(CollectTable[staInd]);

/*    printf("wdrv_rxMpduCollect  CollectTable %x CollectTable[staInd] %x &(CollectTable[staInd]) %x "
            " collectEnt %x\n",CollectTable, CollectTable[staInd],&(CollectTable[staInd]),collectEnt);
    */
    *msduPtr = NULL;
    pHeader = (dot11_header_T*)(pMpdu->data);
/*
	fragNumber = (pHeader->seqCtrl & 0x000F);
    seqNumber = ((pHeader->seqCtrl & 0xFFF0) >> 4);
    */

	
    pHeader->seqCtrl = htons(pHeader->seqCtrl);

	fragNumber = (((pHeader->seqCtrl & 0x0F00) >> 8) & 0xF);
    seqNumber = ((((pHeader->seqCtrl & 0x00FF) << 4) & 0xFF0) | (((pHeader->seqCtrl & 0xF000) >> 12) & 0xF));
    moreFrag = (pHeader->fc & DOT11_FC_MORE_FRAG);

    /*
    if (REPORT_LEVEL(RL_RX)) {
        wdrv_utilsReport(RL_RX,PRI_EMERGENCY,
        "Rx: 1. fragNumber = %d seqNumber = %d seqCtrl %x", fragNumber, seqNumber, pHeader->seqCtrl);
    }
    */

    if ((seqNumber == collectEnt->seqNumber) &&
		(collectEnt->collect == TRUE)) {
        /* This MPDU belongs to the current collection of the MSDU */
        if (fragNumber == (collectEnt->fragNumber+1)) {
            /* This is the next MPDU of the current MSDU */
            /* Update the new MPDU */
            collectEnt->fragNumber++;
            collectEnt->msduPtr->dataLen += (mpduLen - WLAN_HDR_LEN); /* Update MSDU length */
            pMpdu->dataOffset += WLAN_HDR_LEN; /* Point to the start of the Data */
            pMpdu->length -= WLAN_HDR_LEN;
            collectEnt->lastPtr->nextBDPtr = pMpdu; /* Update the BD list */
	        collectEnt->lastPtr = pMpdu;
            mpduValidFrames++;
        }
        else if (fragNumber <= collectEnt->fragNumber) {
            /* Order ERROR or Duplication- NEED to Drop the MPDU */
            wlan_memMngrFreeBD(pMpdu->handle);
            duplicateMpdu++;
            return MPDU_DROP;
        }
        else {
            /* We lost one of the Fragments
               fragNumber > (collectEnt->fragNumber+1)
	           Need to free all the Fragments of the current MSDU */
            wlan_memMngrFreeBD(pMpdu->handle);
            wlan_memMngrFreeMSDU(collectEnt->msduPtr->handle);
            freeCollectEntry(collectEnt);
            msduFreeNotInOrder++;
            return MSDU_DROP;
        }
    }
    else {  /* This MPDU is part of a new MSDU */
        if (collectEnt->collect == TRUE) {
            /* Still in the middle of collecting the previous MSDU
               Need to drop the previous MSDU */
            wlan_memMngrFreeMSDU(collectEnt->msduPtr->handle);
            freeCollectEntry(collectEnt);
            msduFreeNotInOrder++;
        }
		else 
            /*We should check retry bit according to section 9.2.9 in 802.11-1999. */
		if ((seqNumber == collectEnt->seqNumber) && (pHeader->fc & DOT11_FC_RETRY)) {
			if (fragNumber != 0) {
				/* Still Duplicate MPDU !!!, the last fragment received again as retry (the 
				other size didn't receive the ACK and send the last fragment again) */
				wlan_memMngrFreeBD(pMpdu->handle);
				duplicateMpdu++;
#if 0 
	      		if (REPORT_LEVEL(RL_RX)) {
		    		wdrv_utilsReport(RL_RX,PRI_EMERGENCY,
    				"Rx: duplicate last fragment MPDU fragNumber = %d seqNumber = %d", fragNumber, seqNumber);
      			}
#endif
			}
			else {
				/* Duplicate MSDU - new MSDU with the same Sequence number as previous MSDU */
				wlan_memMngrFreeBD(pMpdu->handle);
				duplicateMsdu++;
	      		if (REPORT_LEVEL(RL_RX)) {
		    		wdrv_utilsReport(RL_RX,PRI_EMERGENCY,
    				"Rx: duplicate MSDU seqNumber = %d", seqNumber);
      			}
			}
            return MPDU_DROP;
		}
        if (fragNumber != 0) {
            /* MPDU not in order.
               This is should first MPDU and must be 0
			   Need to drop the MPDU */
            wlan_memMngrFreeBD(pMpdu->handle);
            msduFreeNotInOrder++;
	      	if (REPORT_LEVEL(RL_RX)) {
		    	wdrv_utilsReport(RL_RX,PRI_EMERGENCY,
    		    "Rx: MPDU not in order. fragNumber = %d seqNumber = %d", fragNumber, seqNumber);
      		}
            return MPDU_DROP;
        }
        /* New MSDU */
        mpduValidFrames++;
        wlan_memMngrAllocMSDUBufferOnly(&(collectEnt->msduPtr));
        if (collectEnt->msduPtr == NULL) {
	      	if (REPORT_LEVEL(RL_RX)) {
		    	wdrv_utilsReport(RL_RX,PRI_EMERGENCY,
    		    "Rx: Can not allocate new MSDU buufer");
      		}
            wlan_memMngrFreeBD(pMpdu->handle);
            return MPDU_DROP;
        }
        collectEnt->msduPtr->headerLen = WLAN_HDR_LEN;
        collectEnt->msduPtr->firstBDPtr = pMpdu;
        collectEnt->lastPtr = pMpdu;
        collectEnt->msduPtr->dataLen = mpduLen;
        collectEnt->collect = TRUE;
        collectEnt->seqNumber = seqNumber;
    }

    /* Check if the MSDU collection finished */
    if (!moreFrag) {
        /* MSDU collection terminated */
        *msduPtr = collectEnt->msduPtr;
        freeCollectEntry(collectEnt);
        return MSDU_READY;
    }

    return MSDU_IN_PROGRESS;
}
#endif

void fragmentCollectPrint(void)
{
    printf("RX: Received Fragments - %d\n", mpduValidFrames);
    printf("RX: Duplicate MSDU - %d\n", duplicateMsdu);
}

void wdrv_rxGetCountersFrameDuplicateCount(UINT32* count)
{
    *count = duplicateMsdu;
}

void wdrv_rxGetReceivedFragmentCount(UINT32* count)
{
    *count = mpduValidFrames;
}
#if 0
void wdrv_rxTerminateCollection(UINT16 index) 
{
    CollectEntry_T* collectEnt = &(CollectTable[index]);

    if (collectEnt->collect == TRUE)
	{
		wlan_memMngrFreeMSDU(collectEnt->msduPtr->handle);
		freeCollectEntry(collectEnt);
	}
}
#endif
