/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2004    Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/**************************************************************************/
/*		MODULE:	rx.c													   */
/*    	PURPOSE:	Rx module functions	 								   */
/**************************************************************************/

#include "pal.h"
#include "rx.h"
#include "mainsm.h"
#include "802_11Defs.h"
#include "whal.h"
#include "memMngr.h"
#include "bssbridge.h"
#include "tx.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "mgmtSm.h"
#include "mgmtIf.h"
#include "4xMngr.h"
#include "whalInterrupt.h"
#include "wlanSniffer.h"
#include "powerSave.h"
#include "dda_contexts.h"
#include "dda.h"


/* COPY_dot11_AP_header_T()
 * Copies dot11_AP_header_T if both src and dst are at least 2 byte aligned.
 * Spare the PAL_memCopy() overhead, for this small structure.
 */
static __inline__ void COPY_dot11_AP_header_T( dot11_AP_header_T* dst, dot11_AP_header_T* src)
{
   dst->fc       = src->fc;
   dst->duration = src->duration;
   MACDUP2( &(dst->address1), &(src->address1));
   MACDUP2( &(dst->address2), &(src->address2));
   MACDUP2( &(dst->address3), &(src->address3));
   dst->seqCtrl  = src->seqCtrl;
}

/* COPY_Wlan_LlcHeader_T() macro. Copy Wlan_LlcHeader_T structure.
 * Spare the PAL_memCopy() overhead, for this small structure 8 bytes.
 */
#define COPY_Wlan_LlcHeader_T(dst,src) { \
   UST2((dst),0) = UST2((src),0); \
   UST2((dst),2) = UST2((src),2); \
   UST2((dst),4) = UST2((src),4); \
   UST2((dst),6) = UST2((src),6); \
}


#define MAX_NUM_OF_MSG             32
#define RX_TIMEOUT                 5000 /* message queue timeout in ms */

#define MAX_DECONACT_LEN  1600 /* Maximum length of real ip packet inside concatenation packet */   

extern STATUS wdrv_rxMpduCollectReset(void);
extern void fragmentCollectPrint(void);
/*extern STATUS wdrv_rxFragCollectInit(void);*/
extern collect_status_T wdrv_rxMpduCollect(mem_BD_T* pMpdu, UINT16 staInd,
						UINT32 mpduLen, mem_MSDU_T **msduPtr);
extern void whal_rxMasterFreeBuffer( mem_BD_T *pBd );

PALSRV_TcbObjTYPE rxTimer;

UINT32 enableRxFlag = FALSE;		/* Enable/Disable the RX path */

STATUS wdrv_rxRecvPackets(void);
static void rxTimerCB(UINT32 data);
STATUS wdrv_rx4xWackRecvNotify(UINT8 *destMacAddress, UINT8 wackInfo);
extern collect_status_T wdrv_rxDupMpduDetect(mem_BD_T* pMpdu, UINT16 staInd,
						UINT32 mpduLen, mem_MSDU_T **msduPtr);

extern UINT8   tiSnapDataArray[8];

int deConcatDbg = 0;
int parseMsdu = 0;
static UINT16 HiddenSSID;
static UINT16 MultipleSsidFlag;

/*************** 4x NOTES ******************

  4x features may be set for PBCC modulation (TI proprietary).
  
  Basic structure of WLAN frame:

  Bytes  description
  ---------------------------------
  18     PLCP preamble
   4     PLCP header
         MPDU

  MPDU structure:

  Bytes     description
  ---------------------------------
  24 or 30  MAC header information
            payload
   4        FCS

  Total overhead per frame = 50 or 56 bytes.

  According to the concatenation algorithm, STA sends multiple MSDUs destined
  for the same DA in a single MPDU. Only unicast data frames may be
  concatenated.

  4x allows maximum concatenated MPDU size of 4kB.
  In order to optimize the number of MSDUs that can be packaged,
  MTU = 1300 bytes.

  Concatenated MPDU format:

  Bytes     description
  ---------------------------------
  24        802.11 header (not 30!)
   8        TI SNAP
   4        4x Concatenation Header
   n        Concatenated MSDU(s)

  TI SNAP header format:

  Bytes     description
  ---------------------------------
   1        DSAP     = 0xAA
   1        SSAP     = 0xAA
   1        Control  = 0x00
   3        OUI      = 0x080028
   2        Type     = 0x60D0

  4x Concatenation Header format:

  Bytes     description
  ---------------------------------
   1        Type     = 0x01
   1        Length   = 0x02
   2        Flags    = bits 15:1 = 0, bit 0 = "more"

  The "more" bit is used in ACK emulation, Indicating there are more frames
  waiting in the host Tx queue, destined for the same DA. Receiving host uses
  "more" to encapsulate TCP ACKs information in the 802.11 ACX frames
  (4x WACKs).

  Concatenated MSDU format: STA->AP:

  Bytes     description
  ---------------------------------
   2        MSDU length
   6        DA = MSDU Destination Address
   8        SNAP
   n        MSDU Payload
   0-3      Padding 0's; to make the MSDU payload length a multiple of 4 bytes

  Concatenated MSDU format: AP->STA:

  Bytes     description
  ---------------------------------
   2        MSDU length = address (6) + snap (8) + payload (n)
   6        SA = MSDU Source Address
   8        SNAP
   n        MSDU Payload
   0-3      Padding 0's; to make the MSDU payload length a multiple of 4 bytes


 *******************************************/

/*****************
 * Helper macros *
 *****************/

/* Wlan_LlcHeader_T has 8 bytes long and 2 bytes alignment */
#define LICHDR_CMPRX( a, b) \
   ((XORUST2((a),(b),0) | XORUST2((a),(b),2) | XORUST2((a),(b),4) | XORUST2((a),(b),6)) == 0)

/*************************************************************************
 *                        wdrv_rxParse4xFrame                            *
 *************************************************************************
DESCRIPTION: This gets MPDU and checks if it is a 4x frame, if yes send
             it to the propiate destination.                
                                                                                                   
INPUT:       msduPtr     - Pointer to the first MSDU.
             
             
OUTPUT:     

RETURN:      OK : Deconcatenation OK.
             NOK: Deconcatenation Faild.                                                                                                                                             
************************************************************************/
Wlan4XType_T wdrv_rxParse4xFrame( mem_MSDU_T* firstMsduPtr )
{
    char bssid[6];
   dot114xMsdu_T     *pdot11Header;
   dot11_DataMsduHeader_T  *msduh;
   Wlan4XType_T       type4x = NOT_4X_MSDU;
   register void *paddr3; 
 
   wdrv_dbGetBssid( bssid);
   pdot11Header = (dot114xMsdu_T*)memMgr_BufData( firstMsduPtr->firstBDPtr);
   msduh = &(pdot11Header->msduHeader);
   paddr3 = (void*)&(msduh->dot11Header.address3);

   /*  Verify that the MPDU has been sent to the AP. */
   if( MACCMP( paddr3, bssid))  {
      /* Verify a TI SNAP header. */
      void* psnaph = (void*)&(msduh->snapHeader);
      if( LICHDR_CMPRX( psnaph, tiSnapDataArray)) {
         UINT8 tp = pdot11Header->header4x.type;
         if( (tp == CONCATENATION) || (tp == MANAGMENT_4X)) {
            type4x = tp;
         }
      }
   }

   return type4x;
}/* END wdrv_rxParse4xFrame() */


/*************************************************************************
 *                        wdrv_rxDeConcatMsdu                            *
 *************************************************************************
DESCRIPTION: This function de concatenates number of MSDUs froma single 
             MPDU.                
                                                                                                   
INPUT:       msduPtr     - Pointer to the first MSDU.
             
             
OUTPUT:     

RETURN:      OK : Deconcatenation OK.
             NOK: Deconcatenation Faild.                                                                                                                                             
************************************************************************/
extern int halRestartSession;
STATUS wdrv_txDeConcatMsdu( mem_MSDU_T* firstMsduPtr, UINT16 staIndex, UINT8 MicStatus)
{
   UINT32         concatLength;        /* Total length of the concatenated MSDU */
   UINT32         parsedLen;           /* Accumulated length of parsed fields   */
   dot114xMsdu_T *pdot11Header;        /* Concatenated packet header */
   register UINT32 *ptr1;
   register UINT32 *ptr2;   
   register UINT8  *currPtr;             /* current pointer navigating throught the concat MSDU */
   UINT8         *newDot11HeaderPtr;
   Wdrv4xConcatHeader_T *concatHdrPtr; /* current packet header of the concatenated MSDU */
   UINT16		 currLength;
   dot11_header_T       *pHdr;
   int   msduNum = 0;               /* MSDU counter */
   BOOL  micOK   = TRUE;
   BOOL  wepOn   = FALSE;
   UINT8 stationCipher = CIPHER_NONE;
   UINT32 bufToken;
   UINT8  *buffer;
   mem_BD_T *tempBDPtr;
   
   /* Header of the packet */
   pHdr = (dot11_header_T*)memMgr_BufData( firstMsduPtr->firstBDPtr);
   
   /* WEP and MIC */
   wdrv_dbGetStationSuiteType( staIndex, &stationCipher);
   if( ((pHdr->fc) & DOT11_FC_WEP) == DOT11_FC_WEP) {
      wepOn = TRUE; 
   }
   if( (wepOn == TRUE) && (stationCipher == CIPHER_TKIP)) {
      /* We have to check here whether the MIc received is valid,
       * and to set the micOK value
       */
      micOK =  (MicStatus == OK) ? TRUE:FALSE;
   }

   /* if packet is not TI concatenated format, transfer it to the BSS bridge,
    * and exit.
    */
   if( wdrv_rxParse4xFrame( firstMsduPtr ) != CONCATENATION ) {
     /* Announce the new MSDU to the bssbridge. */
      wdrv_bssBridgeFromWlanFrame( firstMsduPtr, FALSE,  micOK, staIndex);
	  return OK;
   }
   /* In a case of a MIC failure we don't de concat the MSDU. 
    * Instead we forward it as is to the bridge, then it will be dropped.
    */
   
   if( micOK ==  FALSE) {
     /* 
      * Announce the new MSDU to the bssbridge.
      */
      wdrv_bssBridgeFromWlanFrame( firstMsduPtr, FALSE, FALSE, staIndex);
      return OK;
   }
      
   /*
    * The 802.11 header is located in the beggining of the concat frame.
    * concatLength = WLAN HDR   + SNAP + CCatHDR + sum( 2 + MSDU_len + pad)
    * For example:
    *        0x740 = 0x18 (=24) + 8    +  4      + sum( 2 + MSDU_len + pad) =
    *                      0x24 (=36)            + ( 2 + 0x522 + 0) + ( 2 + 0x1f6 + 0)
    *
     */
   pdot11Header = (dot114xMsdu_T*)memMgr_BufData( firstMsduPtr->firstBDPtr);
   concatLength =  firstMsduPtr->dataLen;

   /*
    * The parsed size is currently the wlan snap and 4x headers.
    * = sizeof( dot114xMsdu_T ) = WLAN HDR 24+ SNAP 8+ ConcatHdr 4 = 36
    * Point to the start of the first MSDU.
    */ 
   parsedLen =  sizeof( dot114xMsdu_T );
   currPtr   =  (UINT8*)pdot11Header + sizeof( dot114xMsdu_T);
   /*
    * Start parsing the concatenated MPDU and create an MSDUs. zero copy
    * solution
    */
   while( parsedLen + sizeof( Wdrv4xConcatHeader_T)  < concatLength ) 
    {
      mem_MSDU_T* buildMsduPtr;
      unsigned int numOfPadBytes;
      UINT32 lenc, i;
	  int    sts;

      /* count the MSDUs */
      msduNum++;

      /* cast currPtr to Wdrv4xConcatHeader_T */
      concatHdrPtr =  (Wdrv4xConcatHeader_T*)currPtr;
                  
      /* If MSDU is too long for deconcatenation (1600), report and free it.
       * Thia is done to abort if there are unacceptable situation in FW,
       * to avoid crash.
       */
	  /* lior - endianess fix */
      currLength = PAL_BE2UINT16(concatHdrPtr->len);
      if( currLength > MAX_DECONACT_LEN) {
         if( REPORT_LEVEL(RL_4X))  {
                wdrv_utilsReport(RL_4X,PRI_EMERGENCY,
                                 "RX: Error Parsing Deconcat Packet Len %d > Max len %d in msdu %d hdrptr=%x len=%d",
                                 currLength,
                                 MAX_DECONACT_LEN,
                                 msduNum,
                                 (int)concatHdrPtr,
                                 firstMsduPtr->dataLen);
            }
       
         if( wlan_memMngrFreeMSDU( firstMsduPtr->handle) != OK) {
            wdrv_utilsReport( RL_RX, PRI_EMERGENCY,
               "wdrv_txDeConcatMsdu: wlan_memMngrFreeMSDU failed!!");
         }
         return NOK;
      }/* endIf */
      
      /* point to SNAP within current MSDU */
      currPtr += sizeof( Wdrv4xConcatHeader_T);

      /*
       *  Zero Padding checking. Payload is to be padded to be 4 byte aligned.
       *  currLength should be = (DA+SNAP)(=14) + payloadLength.
       */
      numOfPadBytes = (currLength + 2) & 3;
      numOfPadBytes = (4 - numOfPadBytes) & 3;
      
#if 1      
          /*******************************************************************/
          /*     Deconcatenation with COPY Solution                          */
          /*******************************************************************/         
      
	  sts = tiwlan_ddaRxAllocBufferIfNeeded(currLength+sizeof(dot11_AP_header_T)-sizeof(macAddress_T),
									  &bufToken,
									  &buffer);
	  if (sts == OK)
	  {
		  if (buffer)
		  {
			   if(wlan_memMngrAllocBDs(1, &tempBDPtr) != OK)
			   {
				   wdrv_utilsReport(RL_RX, PRI_WARNING, "wdrv_txDeConcatMsdu: BD allocation failed!!!");
				   tiwlan_ddaRxFreeBuffer(bufToken);
				   if( wlan_memMngrFreeMSDU( firstMsduPtr->handle) != OK) {
					   wdrv_utilsReport( RL_RX, PRI_EMERGENCY,
						  "wdrv_txDeConcatMsdu: wlan_memMngrFreeMSDU failed!!\n");
				   }
				   return NOK;
			   }
	
			   tempBDPtr->data = buffer;
			   tempBDPtr->dataOffset = 0;
			   tempBDPtr->bufToken = bufToken;
			   tempBDPtr->length = currLength+sizeof(dot11_AP_header_T)-sizeof(macAddress_T);
			   tempBDPtr->freeArgs[3] = halRestartSession;
			   tempBDPtr->freeFunc = (void*)whal_rxMasterFreeBuffer;
	
			   if(wlan_memMngrAllocMSDUBufferOnly(&buildMsduPtr) != OK)
			   {
				   wdrv_utilsReport(RL_RX, PRI_WARNING, "wdrv_txDeConcatMsdu: MSDU allocation failed!!!");
				   wlan_memMngrFreeBD(tempBDPtr->handle);
				   if( wlan_memMngrFreeMSDU( firstMsduPtr->handle) != OK) {
					   wdrv_utilsReport( RL_RX, PRI_EMERGENCY,
						  "wdrv_txDeConcatMsdu: wlan_memMngrFreeMSDU failed!!\n");
				   }
				   return NOK;
			   }
			   buildMsduPtr->firstBDPtr = tempBDPtr;
			   buildMsduPtr->lastBDPtr = tempBDPtr;
			   buildMsduPtr->headerLen = WLAN_HDR_LEN;
			   buildMsduPtr->dataLen = tempBDPtr->length;
		  }
		  else
		  {
			   if(REPORT_LEVEL(RL_RX))
				   wdrv_utilsReport( RL_RX, PRI_EMERGENCY,
									"wdrv_txDeConcatMsdu: failed to alloc rx data buffer!!\n");
			   if( wlan_memMngrFreeMSDU( firstMsduPtr->handle) != OK) {
				   wdrv_utilsReport( RL_RX, PRI_EMERGENCY,
					  "wdrv_txDeConcatMsdu: wlan_memMngrFreeMSDU failed!!\n");
			   }
			   return NOK;
		  }
	  }
	  else /* no OS buffer needed, allocate MSDU from memMngr */
	  {
		  if( wlan_memMngrAllocMSDU( &buildMsduPtr, currLength +
			  sizeof(dot11_AP_header_T) - sizeof(macAddress_T)/* =24-6 */) == NOK)
		  {
   			  wlan_memMngrFreeMSDU( firstMsduPtr->handle);
			  return NOK;
		  }
	  }
#if 0
      /*
       * Allocate MSDU buildMsduPtr. we want to build a deconcatenatedd MSDU.
       */
      if( wlan_memMngrAllocMSDU( &buildMsduPtr, currLength +
          sizeof(dot11_AP_header_T) - sizeof(macAddress_T)/* =24-6 */) == NOK)
      {
         wdrv_utilsReport( RL_RX, PRI_WARNING,
            "wdrv_txDeConcatMsdu: wlan_memMngrAllocMSDU failed!!");
         if( wlan_memMngrFreeMSDU( firstMsduPtr->handle) != OK)  {
            wdrv_utilsReport( RL_RX, PRI_EMERGENCY,
               "wdrv_txDeConcatMsdu: wlan_memMngrFreeMSDU failed!!");
         }
         return NOK;
      }
#endif

      /*
       * Create MSDU with one buffer for the wlan header.
       * and copy the 802.11 header from the MSDU to it. 
       * both newDot11HeaderPtr and pdot11Header are at least 2 byte aligned.
       */
      newDot11HeaderPtr =  memMgr_BufData( buildMsduPtr->firstBDPtr);
      COPY_dot11_AP_header_T( (dot11_AP_header_T*)newDot11HeaderPtr,
         (dot11_AP_header_T*)pdot11Header);
      /*
       *  Copy the DA from the concatenated MSDU to the header.
       */ 
      MACDUP2( (void*)&(((dot11_AP_header_T*)newDot11HeaderPtr)->address3),
               (void*)&(concatHdrPtr->SaDa) );
     /*
      * Reading the concatenation More bit, if it is exists, set it in each MSDU's
      * order bit. (Used by the Ack emulation)
      */
	  /* lior - endianess fix */
      if ( pdot11Header->header4x.txFlags == PAL_BE2UINT16(WLAN_4X_CONCAT_MORE_BIT) ) {
         ((dot11_AP_header_T*)newDot11HeaderPtr)->fc |= DOT11_FC_ORDER;        
      } else {
         ((dot11_AP_header_T*)newDot11HeaderPtr)->fc &= ~DOT11_FC_ORDER;
      }
      
      /*
       *  Copy the SNAP + Payload length to the new data buffer.
       *  ( 2 copies solution ). 
       */
      /* ptr1 and ptr2 should be 4 aligned, copy longs - reduces CPU load
       * comparing to memcpy call.
       */
      ptr1 = (UINT32*)((UINT32)(newDot11HeaderPtr) + sizeof( dot11_AP_header_T));
      ptr2 = (UINT32*)currPtr;
      lenc = currLength - sizeof(macAddress_T);
      /* copy longs */
      for( i=0; i<lenc; i+=4) {
         *ptr1++ = *ptr2++;
      }
      
      currPtr += currLength - sizeof(macAddress_T) + numOfPadBytes;
	                              
      buildMsduPtr->firstBDPtr->length = currLength - sizeof(macAddress_T) + sizeof(dot11_AP_header_T);
      buildMsduPtr->dataLen =  buildMsduPtr->firstBDPtr->length;
      buildMsduPtr->firstBDPtr->dataOffset = 0;
      buildMsduPtr->firstBDPtr->nextBDPtr = NULL;
#else
          /*******************************************************************/
          /*     Deconcatenation with NO COPY Solution                       */
          /*******************************************************************/ 
        if ( wlan_memMngrAllocMSDU(&buildMsduPtr, sizeof(dot11_AP_header_T) )==NOK)
        {
            printf("Concat - fail to allocate buildMsduPtr "); 
            return NOK;
        }

         /*
          *  Copy the 802.11 header to the first buffer. (BssBridge needs the 802.11
          *  and the SNAP in the same buffer for Ethernet header translation. 
          */
        newDot11HeaderPtr =  memMgr_BufData(buildMsduPtr->firstBDPtr);
        PAL_memCopy( newDot11HeaderPtr, (char*)pdot11Header, sizeof(dot11_AP_header_T));
         
         /*
          *  Copy the DA from the concatenated MSDU to the header.
          */ 
        PAL_memCopy( (void*)&(((dot11_AP_header_T*)newDot11HeaderPtr)->address3), 
                (void*)&(concatHdrPtr->SaDa),sizeof(macAddress_T) );

         /*
     	  * Reading the concatenation More bit, if it is exists, set it in each MSDU's
     	  * order bit. (Used by the Ack emulation)
          */
         if ( pdot11Header->header4x.txFlags == WLAN_4X_CONCAT_MORE_BIT )
             ((dot11_AP_header_T*)newDot11HeaderPtr)->fc |= DOT11_FC_ORDER;        
         else
             ((dot11_AP_header_T*)newDot11HeaderPtr)->fc &= ~DOT11_FC_ORDER;  

         /*
          *  Copy the SNAP header to the first buffer. (BssBridge needs the 802.11
          *  and the SNAP in the same buffer for Ethernet header translation. 
          */
         PAL_memCopy( (void*)((UINT32)(newDot11HeaderPtr)+sizeof(dot11_AP_header_T)), 
                 currPtr, sizeof(Wlan_LlcHeader_T) );

         currPtr+= sizeof(Wlan_LlcHeader_T);

         buildMsduPtr->firstBDPtr->length = sizeof( dot11_AP_header_T ) + sizeof(Wlan_LlcHeader_T);

         /*
          * Create a new BD, link it concatenated to the data buffer's SNAP header.
          */

		 /* DataBuf might be a NULL pointer !!!!!!!!!!!!!!!!!!!!!!!!!!
		    The following code will cause exception !!!!!!!!!!!!!!!! */

         if ( wlan_memMngrAllocBDs(1, &payloadBdPtr) == NOK)
         {
             printf("Concat - fail to allocate payloadBdPtr\n ");
             if (wlan_memMngrFreeMSDU(buildMsduPtr->handle) != OK)
             {
                 wdrv_utilsReport(RL_RX, PRI_EMERGENCY,"wdrv_txDeConcatMsdu: wlan_memMngrFreeMSDU failed!!");
             }

             return NOK;
         }
         payloadBdPtr->dataBuf = firstMsduPtr->firstBDPtr->dataBuf;
         payloadBdPtr->data = currPtr;
         payloadBdPtr->length = currLength - sizeof(macAddress_T) - sizeof(Wlan_LlcHeader_T);
         payloadBdPtr->dataOffset = 0;
         payloadBdPtr->nextBDPtr = NULL;

         /*
          *  Link the MSDU first BD (wlan header) to the second one (snap+payload)
          *  creating a complete MSDU.
          */
         buildMsduPtr->firstBDPtr->nextBDPtr =  payloadBdPtr;
         payloadBdPtr->dataBuf->refCount++;
         buildMsduPtr->dataLen =  buildMsduPtr->firstBDPtr->length + payloadBdPtr->length;

         currPtr += currLength - sizeof(macAddress_T) - sizeof(Wlan_LlcHeader_T) + numOfPadBytes;;          
#endif   /* 1 */
      
      /* 
       * Announce the new MSDU to the bssbridge.
       */
      wdrv_bssBridgeFromWlanFrame( buildMsduPtr, FALSE, TRUE, staIndex);
      parsedLen += currLength + 2 + numOfPadBytes;
   } /* endWhile ( parsedLen < concatLength ) */
    
   /*
    *  Free the concatenated packet.
    */
   if( wlan_memMngrFreeMSDU( firstMsduPtr->handle) != OK) {
      wdrv_utilsReport( RL_RX, PRI_EMERGENCY,
          "wdrv_txDeConcatMsdu: wlan_memMngrFreeMSDU failed!!");
   }

   return OK;
}/* END wdrv_txDeConcatMsdu() */


/*************************************************************************
 *                        wdrv_rxInit                                    *
 *************************************************************************
DESCRIPTION: This function initializes the Rx module.

INPUT:

OUTPUT:

RETURN:      OK : Initiation succesfull.
             NOK: Initiation unsuccesfull.
************************************************************************/
STATUS wdrv_rxInit(void)
{
   
   /*----------------------------------------------*/
   /* Connect the Rx recieve routine to the HAL.   */
   /*----------------------------------------------*/
	whal_apiSetRxDataCB((RxData_Callback_T)wdrv_rxPacketRecvNotify, 1);

   /*---------------------------------------------------*/
   /* Connect the 4xWack  recieve routine to the HAL.   */
   /*---------------------------------------------------*/
	whal_apiSetRecv4xWackInfoCB((Recv4xWackInfo_Callback_T)wdrv_rx4xWackRecvNotify);

   /* init Hidden SSID if needed */
   wdrv_dbGetHiddenSsidStatus(&HiddenSSID);
   /* Multiple SSID */
   wdrv_dbGetMultipleSsidStatus(&MultipleSsidFlag);

   /*wdrv_rxFragCollectInit();*/

   if(PALSRV_tcbNew(&rxTimer))
   {
	   wdrv_utilsReport(RL_DB, PRI_EMERGENCY,
       			"RX: Failed to allocate RX timer\n");
       return NOK;
   }

   /* L.M consider whether required */
   PALSRV_tcbQueueAct(&rxTimer,
					  RX_TIMEOUT, 
                      rxTimerCB,
					  0, 0);                    

   /*--------------------------------------------------------*/
   /* Announce the main state machien that init is completed.*/
   /*--------------------------------------------------------*/
   wdrv_mainsmSendModuleUp(WDRV_RX_MODULE_ID);

   return OK;
}

/*************************************************************************
 *                        wdrv_rxUnload                                    *
 *************************************************************************
DESCRIPTION: This function deinitializes the Rx module.

INPUT:

OUTPUT:

RETURN:     
************************************************************************/
void wdrv_rxUnload(void)
{
	PALSRV_tcbDelete(&rxTimer);

	printf("RX unloaded !!!\n");
}


UINT32 rxMpduNum = 0;
UINT32 rxMpduControlFrame = 0;
UINT32 rxMpduNotAllow = 0;
UINT32 rxNullDataFrame = 0;
UINT32 rxMsduReady = 0;
UINT32 rxMsduReadyData = 0;
UINT32 rxMsduReadyMgmt = 0;
UINT32 rxBeacon = 0;
UINT32 rxMulticastFrameCount = 0;
UINT32 queueTimeout = 0;
UINT32 rxTresholdNotifyCounter = 0;
UINT32 rxMsduOvf = 0;

static void rxTimerCB(UINT32 data)
{
   RxMessage_T rxMsg;

   rxMsg.type = MSG_MPDU_RECEIVE;
   wdrv_DDA_rx_setEvent(&rxMsg);

   /* reschedule the timer */
   PALSRV_tcbQueueAct(&rxTimer,
					  RX_TIMEOUT,
                      rxTimerCB,
					  0, 0);
}

/*************************************************************************
 *                        wdrv_rxTaskMain                                *
 *************************************************************************
DESCRIPTION: This function implements the Rx Task flow.

OUTPUT:

RETURN:      OK:
************************************************************************/
STATUS wdrv_rx_getEvent(RxMessage_T *pRxMessage)
{
	switch (pRxMessage->type)
	{            
		case MSG_MPDU_RECEIVE:
			bm_trace(5,0,0);
			wdrv_DDA_init_rx_sched_count();
			wdrv_rxRecvPackets();
			break;

		case MSG_4X_WACK:
			/* Build and send ack packet */
#if 0
			wdrv_aeWackReceive(pRxMessage->msg.wack.station, pRxMessage->msg.wack.info);
#endif
	        break;

		default:
			if (REPORT_LEVEL(RL_RX))
				wdrv_utilsReport(RL_RX,PRI_EMERGENCY, "RX: wrong event %d ",pRxMessage->type);
			return NOK;
	}

	return OK;
}

/*************************************************************************
 *                        wdrv_rxPacketRecvNotify                        *
 *************************************************************************
DESCRIPTION:

OUTPUT:

RETURN:  
************************************************************************/
STATUS wdrv_rxPacketRecvNotify(void)
{
   RxMessage_T rxMsg;

   rxTresholdNotifyCounter++;

   rxMsg.type = MSG_MPDU_RECEIVE;

   bm_trace(4,0,0);

   whal_acxRxIntrDisable();

   wdrv_DDA_rx_setEvent(&rxMsg);

   return OK;
}

/*************************************************************************
 *                        wdrv_rx4xWackRecvNotify                        *
 *************************************************************************
DESCRIPTION:

OUTPUT:

RETURN:  
************************************************************************/
STATUS wdrv_rx4xWackRecvNotify(UINT8 *destMacAddress, UINT8 wackInfo)
{
#if 0
   UINT16  staIndex;
   RxMessage_T rxMsg;

   if (wdrv_aeGetActive() == FALSE)
      return OK;

   if( wdrv_dbFindStaIndex(destMacAddress, &staIndex) == NOK )
      return OK; 
 
   rxMsg.type = MSG_4X_WACK;
   rxMsg.msg.wack.station = staIndex;
   rxMsg.msg.wack.info = wackInfo;
   wdrv_DDA_rx_setEvent(&rxMsg);
#endif
   return OK;

}


/*************************************************************************
 *                        wdrv_rxEnhancedDynamicAlgoPerform              *
 *************************************************************************
DESCRIPTION: update timestamp for ERP, PBCC, Slot time enhanced dynamic algorithms

OUTPUT: None

RETURN: None  
************************************************************************/
void wdrv_rxEnhancedDynamicAlgoPerform(UINT16 index, RxAttr_T *pRxAttr)
{
	UINT32 timestamp;
	BOOL   cap;
	BOOL   pbccDisableValid,erpEnableValid;
	BOOL   externalActive;
    BOOL   mixMode4x, status4x;
    UINT8  PBCCalgoMode,ERPProtectionMode;
	UINT8  activeRate;


	timestamp = wdrv_mgmtReadTimeStamp(index);

    wdrv_dbGetPBCCAlgorithmMode(0,&PBCCalgoMode);
    if(PBCCalgoMode == PBCC_ALGO_ENHANCED_DYNAMIC)
    {
    	/* check whether this STA is PBCC capable, and if not update the corresponding timestamp */
    	if(((cap = wdrv_dbIsStationPBCCCapable(index)) == FALSE))
    	{
    		wdrv_dbSetNotPBCCTimestamp(timestamp);
    		/* in case of external interference active, we can't use PBCC any way*/
    		wdrv_mgmtIsPBCCexternalActive(&externalActive);
    		if(!externalActive)
    		{
    			/* in case we've already disabled the PBCC, don't disable it again */
    			wdrv_dbGetPBCCdynamicStatus(&pbccDisableValid);
    			if(!pbccDisableValid)
    			{
    				/* disable PBCC */
    				wdrv_dbSetPBCCStationsSupport(FALSE);
					/*whal_apiSetPbcc(0);*/
					wdrv_mgmtSendTemplateUpdateRequest(TEMPLATE_SET_PBCC, 0);    				
    				wdrv_dbSetCurrentPBCCSupportMode(FALSE);
    
    				/* If 4x can work only in PBCC, disable the 4X .*/
    				wdrv_dbGet4xMixMode(&mixMode4x);
    				wdrv_dbGet4xEnable(&status4x);
					wdrv_dbGetMaxActiveRate(&activeRate);
    				if(( mixMode4x == FALSE )&&( status4x == TRUE )&&(activeRate <= DOT11_RATE_22))         
    					wdrv_4xMngr_stop4x();
    				
    				wdrv_dbSetPBCCdynamicStatus(TRUE);
    			}
    		}
    	}
    }

    /* Update timestamp if received a data frame in B/B+ rate */
	/* This is used for the short slot time algorithm */
	if (!(pRxAttr->Status & 0x4)) /* if OFDM bit is turned off */
	{
		wdrv_dbSetRxTxBrate(timestamp, 1);
	}

    wdrv_dbGetERPProtectionMode(0,&ERPProtectionMode);				
    if (ERPProtectionMode == PROTECTION_MODE_ENHANCED_DYNAMIC)
    {
        /* check whether this STA is ERP, and if not update the corresponding timestamp */
        if(((cap = wdrv_dbIsStationERP(index)) == FALSE))
        {
            wdrv_dbSetNotErpTimestamp(timestamp);
            /* in case of external interference active, we have to activate protection  any way*/
            wdrv_mgmtIsERPexternalActive(&externalActive);
            if(!externalActive)
            {
                /* in case we've already enabled protection, don't enable it again */
                wdrv_dbGetERPdynamicStatus(&erpEnableValid);
                if(!erpEnableValid)
                {
                    /* enable protection */
                    /*whal_apiSetERPProtection(1,1);*/
					wdrv_mgmtSendTemplateUpdateRequest(TEMPLATE_SET_ERP, 1);
                    wdrv_dbSetERPdynamicStatus(TRUE);
                }
            }
        }
    }
}


static int ProbeRespCounter = 0;

/*************************************************************************
 *                        wdrv_rxRecvPackets                             *
 *************************************************************************
DESCRIPTION:

OUTPUT:

RETURN:  
************************************************************************/
#ifdef ERP_DYNAMIC_DEBUG
UINT32 TrcBuf[256];
char   TrcInd=0;
BOOL   TrcEn=TRUE;
BOOL   first = TRUE;
BOOL   printEn=TRUE;
#endif

STATUS wdrv_rxRecvPackets(void)
{
   dot11_header_T*		pHeader;
   mem_BD_T*			pMpdu;
   mem_MSDU_T*			msduPtr;
   UINT32				mpduLen;
   RxAttr_T				rxAttr;
   dot11_Fc_Type_T 		mpduType;
   UINT16          		staInd;
   collect_status_T     collectStat = MSDU_IN_PROGRESS;
   BOOL    				assocInd=FALSE;
   BOOL					BeaconFlag=FALSE;
   UINT8  				ERPProtectionMode;
   UINT8				PBCCalgoMode;
   dot11_mgmtHeader_T*  pDot11Header;

   UINT8 band;


   int rxCount = 10;
   /* Enter loop only if enableRxFlag flag is set.  */
   while( enableRxFlag && rxCount)
   {
	  rxCount--;
      rxAttr.Status = 0;
	  pMpdu = (mem_BD_T *)whal_apiRxRecv( &mpduLen, &rxAttr);

      if( pMpdu == NULL)
      {
         /* There are no more MPDU waiting in the Rx queue */
         break;
      }

      /* Hidden SSID support, 15/6/03 */
      pDot11Header = (dot11_mgmtHeader_T *)(pMpdu->data);

      if (pDot11Header->fc == DOT11_FC_PROBE_REQ)
      {
          if (HiddenSSID || MultipleSsidFlag) 
          {
              ProbeRespCounter++;
              wdrv_mgmtProbeRequestPacketMgmt(pMpdu, mpduLen, rxAttr.woneIndex);

              /* free rx buffers */
              wlan_memMngrFreeBD(pMpdu->handle);
              continue;
          }
      }
      
      if (REPORT_LEVEL(RL_RX_SNIFF))
      {
         /* Sniff the MPDU */
    	    whal_utilsSniffer( pMpdu->data);
      }
      
      ++rxMpduNum;
      pHeader = (dot11_header_T*)(pMpdu->data);
      mpduType = (pHeader->fc & DOT11_FC_TYPE_MASK);


	  /* Filter out Broadcast Data Packets sent from other APs or STAs in ad-hoc */
	  if((pHeader->address1.addr[0] & 1) && (mpduType == DOT11_FC_TYPE_DATA))
	  {
		  wlan_memMngrFreeBD( pMpdu->handle);
		  continue;
	  }


      wdrv_dbGetBand(rxAttr.woneIndex,&band);

      /* check if beacon from other BSSs used in protection/PBCC algorithms */
      if((pHeader->fc & DOT11_FC_TYPESUBTYPE_MASK) == DOT11_FC_BEACON)
	  {
#ifdef ERP_DYNAMIC_DEBUG

		  int i,k;
          UINT32 CurrTimestamp = PALSRV_msec2ticks();
		  
		  
		  if(TrcEn || first)
		  {
			  first = FALSE;
			  TrcBuf[TrcInd++] = CurrTimestamp;
		  }

		  if(!TrcEn && !first && printEn)
		  {
			  printEn = FALSE;
			  printf("Trace Dump (curr=%d)\n",TrcInd);
			  k = TrcInd;
			  for(i=0; i<10; i++)
			  {
				  printf("TrcBuf[%d] = %d\n", k%256, TrcBuf[k%256]);
				  k++;
			  }
		  }		  
#endif

		  BeaconFlag = TRUE;
	  }

	  /* Skip defragmentation of beacons at this stage */
	  if(!BeaconFlag)
	  {
          UINT16 pwrMgmtStatus;

		  /* check the Destination address with the Management SM to receive the
		  STA index */
		  if( wdrv_mgmtIsSrcAllowed( pHeader->address2.addr, mpduType, 
                                     rxAttr.woneIndex, &staInd, &assocInd) == FALSE)
		  {
			 if( REPORT_LEVEL(RL_RX))
			 {
				wdrv_utilsReport(RL_RX,PRI_EMERGENCY,
				   "RX: Source is not allowed.");
			 }
			 wlan_memMngrFreeBD( pMpdu->handle);
			 ++rxMpduNotAllow;
			 continue;
		  }

          /* Increase statoin packet pounter */
          wdrv_dbIncreaseStationRxPacketCounter(staInd);
         

		  /* update timestamp for ERP, PBCC, Slot time enhanced dynamic algorithms */
		  
          wdrv_dbGetBand(rxAttr.woneIndex,&band);
          if(assocInd && (band == RADIO_BAND_2_4GHZ) )
			  wdrv_rxEnhancedDynamicAlgoPerform(staInd, &rxAttr);
		  
		  /* Update the Power Save Status of this STA */
		  if( pHeader->fc & DOT11_FC_PWR_MGMT)
			  pwrMgmtStatus = TRUE;
		  else
			  pwrMgmtStatus = FALSE;
			 
		  wdrv_psSetPsStatus( staInd, rxAttr.woneIndex, pwrMgmtStatus );		  
		  
		  /* Control Frames */
		  if( mpduType == DOT11_FC_TYPE_CTRL)
		  {
			 if ((pHeader->fc & DOT11_FC_SUB_MASK) == DOT11_FC_SUB_PS_POLL)
			 {
				/* Handle PS-POLL requests */
				wdrv_txSendPsPollMsg((UINT32)staInd );
			 }
			 /* Free MPDU */
			 wlan_memMngrFreeBD( pMpdu->handle);
			 ++rxMpduControlFrame;
			 continue;
		  }
		  
		  /* NULL data frames */
		  if( (pHeader->fc & DOT11_FC_TYPESUBTYPE_MASK) == DOT11_FC_DATA_NULL_FUNCTION)
		  {
			 /* Free MPDU */
			 wlan_memMngrFreeBD(pMpdu->handle);
			 ++rxNullDataFrame;
			 continue;
		  }
		  /* Send the MPDU to the fragment collector */
		  collectStat =  wdrv_rxDupMpduDetect/*wdrv_rxMpduCollect*/( pMpdu, staInd, mpduLen, &msduPtr);
	  }

      if( collectStat == MSDU_READY || BeaconFlag)
      {
      /* Send the MSDU packet to the Bridge or Management or
         continue to the next MPDU */
         rxMsduReady++;
         
         if ((pHeader->address3.addr[0]) & 0x01)
         {
            ++rxMulticastFrameCount;
         }
         switch( mpduType)
         {
		 case (DOT11_FC_TYPE_DATA): 
			 wdrv_txDeConcatMsdu( msduPtr, staInd,rxAttr.Mic);
			 ++rxMsduReadyData;
			 break;
		 case (DOT11_FC_TYPE_MGMT):
			if(!BeaconFlag)
			{
				++rxMsduReadyMgmt;
				wdrv_mgmtForwardMgmtFrame( msduPtr, staInd);
			}
			else /* Beacon */
			{
				++rxBeacon;
				wdrv_dbGetERPProtectionMode(0,&ERPProtectionMode);
				wdrv_dbGetPBCCAlgorithmMode(0,&PBCCalgoMode);
                if (((ERPProtectionMode == PROTECTION_MODE_DYNAMIC) ||
					(ERPProtectionMode == PROTECTION_MODE_ENHANCED_DYNAMIC) ||
					(PBCCalgoMode == PBCC_ALGO_DYNAMIC) ||
					(PBCCalgoMode == PBCC_ALGO_ENHANCED_DYNAMIC)) &&
                    (band == RADIO_BAND_2_4GHZ))
				{				
					if (wdrv_mgmtForwardBeacon(pMpdu, mpduLen) == NOK)
					{
					/* Free MPDU */
    				wlan_memMngrFreeBD(pMpdu->handle); /* 15/07 */
					++rxMsduOvf;
					}
				}
                else
                {
					
                    /* Free MPDU */
    			    wlan_memMngrFreeBD(pMpdu->handle); 
                }


				BeaconFlag = FALSE;
			}
            break;
         default:
            if( REPORT_LEVEL(RL_RX))
            {
               wdrv_utilsReport( RL_RX,PRI_EMERGENCY,
                  "RX: Type of MPDU is not MNGM/DATA/CNTR");
            }
            break;
         }
      }
    }/*endWhile*/
    
    /* Reenable RX interrupts  */
    whal_acxRxIntrEnable();

	return OK;
}

/*************************************************************************
 *                        wdrv_rxEnable                                  *
 *************************************************************************
DESCRIPTION:

OUTPUT:

RETURN:  
************************************************************************/
STATUS wdrv_rxEnable(void)
{
	enableRxFlag = TRUE;
	return TRUE;
}

/*************************************************************************
 *                        wdrv_rxDisable                                 *
 *************************************************************************
DESCRIPTION:

OUTPUT:

RETURN:  
************************************************************************/
STATUS wdrv_rxDisable(void)
{
	enableRxFlag = FALSE;
	return TRUE;
}

/*************************************************************************
 *                        rxPrintStatus                                  *
 *************************************************************************
DESCRIPTION:

OUTPUT:

RETURN:  
************************************************************************/
void rxPrintStatus(void)
{
    printf("RX: Number of MPDU received - %d\n", rxMpduNum);
    printf("RX: Number of control frames - %d\n", rxMpduControlFrame);
    printf("RX: Number of NOT ALLOWED MPDU -  %d\n", rxMpduNotAllow);
    printf("RX: Number of Null Data frames -  %d\n", rxNullDataFrame);
    printf("RX: Number of MSDU ready - %d\n", rxMsduReady);
    printf("RX: Number of MSDU ready (DATA) - %d\n", rxMsduReadyData);
	printf("RX: Number of Beacons ready     - %d\n", rxBeacon);
    printf("RX: Number of MSDU ready (Management) - %d\n", rxMsduReadyMgmt);
    printf("RX: Queue Timeout - %d\n", queueTimeout);
    printf("RX: Number of treshold notifications - %d\n", rxTresholdNotifyCounter ); 
    printf("RX: Number of MSDU BD Overflow - %d\n", rxMsduOvf ); /* 15/07 */
    fragmentCollectPrint();
}

void wdrv_rxGetMulticastReceivedFrameCount(UINT32* count)
{
    *count = rxMulticastFrameCount;
}

void wdrv_rxGetRxMsduReadyCount(UINT32* count) 
{
   *count = rxMsduReady;
}

