/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																									*/
/*	  MODULE:	powerSave.h	                                                */ 
/*   PURPOSE:	Power Save module implementation.	 							   */ 
/*																									*/
/***************************************************************************/
#ifndef TX_POWER_SAVE_H
#define TX_POWER_SAVE_H


/*************************************************************************
 *                        wdrv_psInit                                    *
 *************************************************************************
DESCRIPTION: This function initializes the Power save module.                 
                                                                                                   
INPUT:      numOfStations - Number of station in the system 

OUTPUT:      

RETURN:      OK : Initiation succesfull.
             NOK: Initiation unsuccesfull.                                                                                                                                             
************************************************************************/
STATUS wdrv_psInit(UINT16 numOfStations );

/*************************************************************************
 *                        wdrv_psUnload                                    *
 *************************************************************************
DESCRIPTION: This function deinitializes the Power save module.                 
                                                                                                   
INPUT:      numOfStations - Number of station in the system 

OUTPUT:      

RETURN:                                                                                                                                                   
************************************************************************/
void wdrv_psUnload(UINT16 numOfStations);

/*************************************************************************
 *                        wdrv_psInit                                    *
 *************************************************************************
DESCRIPTION: This function initializes the Power save module.                 
                                                                                                   
INPUT:      staIndex - The station index.

OUTPUT:      

RETURN:  TRUE  - PS in ON .  
         FALSE - PS is OFF.                                                                                                                                              
************************************************************************/
inline void wdrv_psSetPsStatus( UINT16 staIndex, UINT8 woneIndex, UINT16 status );

/*************************************************************************
 *                        wdrv_psGetStaStatus                            *
 *************************************************************************
DESCRIPTION: This function returns the power save status of the station.                
                                                                                                   
INPUT:      staIndex - The station index.

OUTPUT:      

RETURN: TRUE  - PS in ON .  
        FALSE - PS is OFF.                                                                                                                                               
************************************************************************/
UINT16 wdrv_psGetStaStatus( UINT16 stationIndex );

/*************************************************************************
 *                        wdrv_psPsOnAndPksInQueue                       *
 *************************************************************************
DESCRIPTION: This function returns TRUE if the station is in PS mode and
              there are more packets in the PS queue.                 
                                                                                                   
INPUT:      staIndex - The station index.

OUTPUT:      

RETURN: TRUE  - Station in PS more packets to send .  
        FALSE - Otherwise.                                                                                                                                               
************************************************************************/
UINT16 wdrv_psPsOnAndPksInQueue( UINT16 staIndex );

/*************************************************************************
 *                        wdrv_psStoreEnv                                *
 *************************************************************************
DESCRIPTION: This function saves the envelop in the envelop list.                
                                                                                                   
INPUT:      staIndex - The station index.
            envelopPtr - Pointer to the envelop to save.            

OUTPUT:      

RETURN:      OK : The envelop has been inserted, there was enough place in 
                  the list.
             NOK: The list was full, the first envelop is dropt, and returned
                  by *envelopPtr.                                                                                                                                                
************************************************************************/
STATUS wdrv_psStoreEnv( UINT16 staIndex, TxEnvelop_T **envelopPtr );


/*************************************************************************
 *                        wdrv_psWatchEnv                                *
 *************************************************************************
DESCRIPTION: This function saves the envelop in the envelop list.                
                                                                                                   
INPUT:      staIndex - The station index.
            envelopPtr - Pointer to the envelop to save.            

OUTPUT:      

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.                                                                                                                                                 
************************************************************************/
STATUS wdrv_psWatchEnv( UINT16 staIndex, TxEnvelop_T **envelopPtr);

/*************************************************************************
 *                        wdrv_psGetEnv                                  *
 *************************************************************************
DESCRIPTION: This function saves the envelop in the envelop list.                
                                                                                                   
INPUT:      staIndex - The station index.
            envelopPtr - Pointer to the envelop to save.            

OUTPUT:      

RETURN:      OK : There was an envelop in the list, and it is assigned 
                  to the *envelopPtr.
             NOK: The list was empty, *envelopPtr is trush.                                                                                                                                                 
************************************************************************/
STATUS wdrv_psGetEnv( UINT16 staIndex, TxEnvelop_T **envelopPtr);


/*************************************************************************
 *                        wdrv_psGetTotalStaInPSMode                     *
 *************************************************************************
DESCRIPTION: Returns the number of station currently in PS mode.

OUTPUT:      

RETURN: The number of packets in PS mode.                                                                                                                                               
************************************************************************/
UINT32  wdrv_psGetTotalStaInPSMode( void );

/*************************************************************************
 *                        wdrv_psGetTotalWoneStaInPSMode                     *
 *************************************************************************
DESCRIPTION: Returns the number of station currently in PS mode for wone.

OUTPUT:      

RETURN: The number of packets in PS mode for wone.                                                                                                                                               
************************************************************************/
UINT32  wdrv_psGetTotalWoneStaInPSMode( UINT8 wone_Index );

/*************************************************************************
 *                        wdrv_psGetTotalStaInPSMode                     *
 *************************************************************************
DESCRIPTION: Returns the number of packets currently in PS mode.

OUTPUT:      

RETURN: The number of packets in PS mode.                                                                                                                                               
************************************************************************/
int wdrv_powerSavePrintDb(char *buf);

/*************************************************************************
 *                        wdrv_txPowerSaveAgingFunc                      *
 *************************************************************************
DESCRIPTION: Power Save aging managment.

OUTPUT:      

RETURN:                                                                                                                                               
************************************************************************/
void wdrv_txPowerSaveAgingFunc(void);

/****************************************************************************
 *                      wdrv_powerSaveCleanStaList()                       *
 ****************************************************************************
 * DESCRIPTION: Clean all the packets stored in a station power save list 
 *              and set the PS status to FALSE, used for disconnecting a 
 *              staion from the AP.
 * INPUTS: stationIndex - The station Index
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void wdrv_powerSaveCleanStaList( UINT16 stationIndex, UINT8 woneIdx );

/****************************************************************************
 *                      wdrv_psGetPsPrevTxFail()                       *
 ****************************************************************************
 * DESCRIPTION: Return the status of previous Ps unicast packet send 
 *
 * INPUTS: stationIndex - The station Index
 *		   woneIndex    - The wone Index   
 *		
 * 
 * OUTPUT:	prevTxFail - Indecate if this station fail to send previous 
 *          unicast packet
 * 
 ****************************************************************************/

void  wdrv_psGetPsPrevTxFail( UINT8 stationIndex ,UINT8 *prevTxFail);

/****************************************************************************
 *                      wdrv_psSetPsPrevTxFail()                       *
 ****************************************************************************
 * DESCRIPTION: Set the PS Prev send fail flag.
 *
 * INPUTS: stationIndex - The station Index
 *		      
 *		
 * 
 * 
 ****************************************************************************/

void  wdrv_psSetPsPrevTxFail( UINT8 stationIndex);

/****************************************************************************
 *                      wdrv_psGetPsPrevTxFailCounter()                       *
 ****************************************************************************
 * DESCRIPTION: return the number of station with PS Prev send fail flag on.
 * 
 ****************************************************************************/


inline void  wdrv_psGetPsPrevTxFailCounter(UINT8 *PsTxFailurs);

/*************************************************************************
 *                        wdrv_psPowerSaveStatusChangeFunc                *
 *************************************************************************
DESCRIPTION: This function initializes the Power save module.                 
                                                                                                   
INPUT:      staIndex - The station index.

OUTPUT:      

RETURN:                                                                                                                                                
************************************************************************/
void wdrv_psPowerSaveStatusChangeFunc(UINT32 staIndex, UINT16 status, UINT8 woneIndex) ;

/****************************************************************************
 *                      wdrv_powerSaveCleanStaListFunc()                       *
 ****************************************************************************
 * DESCRIPTION: Clean all the packets stored in a station power save list 
 *              and set the PS status to FALSE, used for disconnecting a 
 *              staion from the AP.
 * INPUTS: stationIndex - The station Index
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void wdrv_powerSaveCleanStaListFunc( UINT16 stationIndex ) ;

#endif /* TX_POWER_SAVE_H */

