/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2003  Texas Instruments Incorporated        |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*		MODULE:	tx.c													   */
/*    PURPOSE:	Tx module functions	 									   */
/*																		   */
/***************************************************************************/
#include "pal.h"
#include "tx.h"
#include "mainsm.h"
#include "mgmtIf.h"
#include "envList.h"
#include "powerSave.h"
#include "whal.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "mgmtSm.h"
#include "concatenator.h"
#include "802_11Defs.h"
#include "public_security_keys.h"
#include "whalTx.h"
#include "ackEmulDb.h"
#include "dda_contexts.h"
#include "dda.h"


#define TOTAL_NUM_OF_DATA_ENVELOPS      		128/* was 256, must be 128, since there are not enough skbs */  
#define TOTAL_NUM_OF_WONE_WHIT_DATA_ENVELOPS    64 /*120*/
#define TOTAL_NUM_OF_MNGT_ENVELOPS      		64
#define PS_AGING_INTERVAL            			10000 /* 10 SEC */

#define WONE_CONCAT_MAX_SIZE 2772 /* in wone mode the concatenation size is limited to 2 big packets to prevent other band starvation */

/* If vHCF enabled, limit num of waiting low prioriy MSDUs from Ethernet, to prevent 
	Ethernet BDs starvation. Should be less than TNETD53XX_EMAC_RCR_NUM_ENTRIES */
#define VHCF_LIMIT_WAIT_LOW_PRIOR_ETHER  32 


/*-----------------------------------------------------------*/
/*         Local Variables                                   */
/*-----------------------------------------------------------*/
static EnvList_T   *freeDataEnvList;        /* A list to keep the data free envelops. */
static EnvList_T   *freeMngtEnvList;        /* A list to keep the managment free envelops. */
static EnvList_T   *highPrioWaitEnvList[MAX_WONE_NET]; /* A high priority list to keep the 
                                     envelops waiting to be sent. */
static EnvList_T   *lowPrioWaitEnvList[MAX_WONE_NET];  /* A high priority list to keep the 
                                     envelops waiting to be sent. */

static EnvList_T   *staEnvList[MAX_STATIONS];


PALSRV_TcbObjTYPE waitListTimer;
PALSRV_TcbObjTYPE psAgingTimer;

static PAL_atomic_t inTx;

static struct Tx_Statistics {
   UINT32 unicAllocFail  ;
   UINT32 brcstAllocFail ;
   UINT32 acxQueUnderflow;
   UINT32 unicastSent    ;
   UINT32 brcstSent      ;
   UINT32 lowPrioSent    ;
   UINT32 highPrioSent   ;
   UINT32 psPollEvents   ;
   UINT32 psWakeEvents   ;
   UINT32 sentPsPackets  ;
   UINT32 concatPackets  ;
   UINT32 unConcatPackets;
   UINT32 unicastQsent   ;
   UINT32 broadcastQsent ;
   UINT32 psunicAllocFail;
   UINT32 psprevFailSend ;
} Tx_Statistics = {0};


static BOOL txPrivacyInvoked;

/*static BOOL dataMsgExistsInTxMsgQ = FALSE; *//* This flag notify if there is already TIMEOUT/THRESHOLD
                                            messages in the txMsgQ. */
int concatOn = 1;
BOOL concatDbg = FALSE;

UINT32 maxConcatSize = MAX_FRAG_THRESHOLD;


/*static int TransmittedMsduCount[NUM_TX_QUEUES];*/
extern int whal_apiTxGetBufsNum(TxQueueId_T TxQid, UINT32 *FreeDescNum, UINT32 *UsedDescNum);
static int waitLowPriorFromEther = 0; /* Count waiting low priority MSDUs from Ethernet. */
static int waitPktFromEther[MAX_WONE_NET]      = {0,0};  /* Count waiting MSDUs from Ethernet(for Wone threshold). */
static int overloadWaitPktFromEther[MAX_WONE_NET]      = {0,0};  /* Count the overload waiting packet from ethernet). */

/*--------------------------------------------------------------*/
/*              Rate Adaptation Table                           */
/*--------------------------------------------------------------*/
typedef struct {
   UINT32  expirTimeTick;
   UINT32  initExpirTimeTick;
   UINT32  txCount;
   UINT32  txFailCount;
   UINT8   currRateIndex;
   UINT8   maxRateIndex;
   UINT8   nextRateIndex;
   UINT8   stepUpFlag;
   UINT8   mode;
} rateAdaptParams_Type;

static rateAdaptParams_Type rateAdapTable[MAX_STATIONS];


/*-----------------------------------------------------------*/
/*         Local Functions                                   */
/*-----------------------------------------------------------*/
void wdrv_txFreeEnv( TxEnvelop_T **envelopPtr );
void wdrv_txSendPSAgingTimerCB( UINT32 data );
STATUS sendWaitMsdu( EnvList_T  *envList);
static STATUS sendPsMsdu( UINT16 stationIndex);
STATUS wdrv_txSendMsduToHal(TxEnvelop_T* envelopPtr, TxQueueId_T txQid);
static void wdrv_txSetTxAttrProtection(TxAttr_T *pTxAttr, UINT16 StaIndex, UINT8 WoneIndex);
static void wdrv_txRateAdaptFailAnnounce(UINT8 *DestMacAddress, UINT8 ActualTxRate, UINT8 TxStatus);
static UINT16 tx_getStaRate( UINT16 staIndex, UINT16 *pRateWithFallback);
static STATUS tx_getRateIndexByRate( UINT16 rate, UINT8 mode, char* retVal );
static void wdrv_txUpdateRateAdaptation( UINT16 staIndex );
static BOOL tx_isFallbackPossible( UINT16 staIndex);
static BOOL tx_isStepupPossible( UINT16 staIndex);
static void tx_updateTxBrateTimestamp(UINT16 rate);
static void vhcfClassifyTxPacket(mem_MSDU_T *msduPtr, ULONG * txFlags);
static void wdrv_txWaitListCheckTimerCB( UINT32 data );

/*************************************************************************
 *                        wdrv_txInit                                    *
 *************************************************************************
DESCRIPTION: This function initializes the Tx module.                 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      OK : Initiation succesfull.
             NOK: Initiation unsuccesfull.                                                                                                                                             
************************************************************************/
STATUS wdrv_txInit(void) 
{
   int i;
   TxEnvelop_T  *tmpEnvPtr;
   UINT16 maxWaitDataEnv, maxWaitMgmtEnv;
   UINT8 woneIndex ,numSupWone;

   PAL_atomic_set(&inTx, 0);

   wdrv_dbGetNumSupWone(&numSupWone);

   maxWaitMgmtEnv =  TOTAL_NUM_OF_MNGT_ENVELOPS - 5;
   maxWaitDataEnv =  TOTAL_NUM_OF_DATA_ENVELOPS - 10;
 
   /*----------------------------------------------------------------------------*/
   /* Initialize the data envelops free list, and fill it up with empty envelps. */
   /*----------------------------------------------------------------------------*/
   if( (freeDataEnvList = wdrv_txNewEnvList( TOTAL_NUM_OF_DATA_ENVELOPS )) == NULL ){
      wdrv_utilsReport(RL_TX, PRI_EMERGENCY,
       		"TX: Not enough memory to Init freeEnvList");
      return NOK;
   }
   for( i=0 ; i < TOTAL_NUM_OF_DATA_ENVELOPS ; i++ )
   {
      if((tmpEnvPtr = PAL_memAlloc( 0,sizeof( TxEnvelop_T ),0,0)) == NULL ){
         wdrv_utilsReport(RL_TX, PRI_EMERGENCY,
            "TX: Not enough memory to Init envelop");
         return NOK;
      }
	  PAL_memSet(tmpEnvPtr, 0, sizeof( TxEnvelop_T ));         
      if( wdrv_txEnvListIns( freeDataEnvList, &tmpEnvPtr ) == NOK ){
         wdrv_utilsReport(RL_TX, PRI_EMERGENCY,
            "TX: Init : Fail to insert envelop to free queue.");
         return NOK;
      }
   } /* for(....) */

   /*----------------------------------------------------------------------------*/
   /* Initialize the mngt envelops free list, and fill it up with empty envelps. */
   /*----------------------------------------------------------------------------*/
   if( (freeMngtEnvList = wdrv_txNewEnvList( TOTAL_NUM_OF_MNGT_ENVELOPS )) == NULL ){
      wdrv_utilsReport(RL_TX, PRI_EMERGENCY,
       		"TX: Not enough memory to Init freeEnvList");
      return NOK;
   }
   for( i=0 ; i < TOTAL_NUM_OF_MNGT_ENVELOPS ; i++ )
   {
      if((tmpEnvPtr = PAL_memAlloc( 0,sizeof( TxEnvelop_T ),0,0)) == NULL ){
         wdrv_utilsReport(RL_TX, PRI_EMERGENCY,
            "TX: Not enough memory to Init envelop");
         return NOK;
      }
	  PAL_memSet(tmpEnvPtr, 0, sizeof( TxEnvelop_T ));         
      if( wdrv_txEnvListIns( freeMngtEnvList, &tmpEnvPtr ) == NOK ){
         wdrv_utilsReport(RL_TX, PRI_EMERGENCY,
            "TX: Init : Fail to insert envelop to free queue.");
         return NOK;
      }
   } /* for(....) */


   for (woneIndex=0; woneIndex < numSupWone ; woneIndex++) {
   /*-------------------------------------------------------------------------*/
   /* Initialize the envelops high priority waiting list, and leave it empty. */
   /*-------------------------------------------------------------------------*/
      if ( (highPrioWaitEnvList[woneIndex] = wdrv_txNewEnvList( maxWaitMgmtEnv)) == NULL ) {
		  wdrv_utilsReport(RL_DB, PRI_EMERGENCY,
				"TX: Not enough memory to Init waitEnvList");
		  return NOK;
	  }

   /*------------------------------------------------------------------------*/
   /* Initialize the envelops low priority waiting list, and leave it empty. */
   /*------------------------------------------------------------------------*/
      if ( (lowPrioWaitEnvList[woneIndex] = wdrv_txNewEnvList( maxWaitDataEnv)) == NULL ) {
		  wdrv_utilsReport(RL_DB, PRI_EMERGENCY,
				"TX: Not enough memory to Init waitEnvList");
		  return NOK;
	  }

   }
   
   /*------------------------------------------------------------------------*/
   /* Initialize the envelops for the stations waiting list.                 */
   /*------------------------------------------------------------------------*/ 
   for( i=0; i <  MAX_STATIONS; i++ ){
	   if( (staEnvList[ i ] = wdrv_txNewEnvList( maxWaitDataEnv)) == NULL )
	   {
      		wdrv_utilsReport(RL_DB, PRI_EMERGENCY,
       			"TX: Not enough memory to Init staEnvList");
      		return NOK;
   		}
   	}	
	
   /*------------------------------*/
   /* Init the Power Save module.  */
   /*------------------------------*/
   if( wdrv_psInit( MAX_STATIONS ) == NOK )
      return NOK;

   /* clean Tx queue periodically. */

   /*--------------------------------------------------------*/
   /* Create a periodic timer for waiting list checker.      */
   /*--------------------------------------------------------*/  
   if(PALSRV_tcbNew(&waitListTimer))
   {
	   wdrv_utilsReport(RL_DB, PRI_EMERGENCY,
       			"TX: Failed to allocate wait list timer\n");
       return NOK;
   }

   PALSRV_tcbQueueAct(&waitListTimer,
					  10, 
                      wdrv_txWaitListCheckTimerCB,
					  0, 0);
   
   /*--------------------------------------------------------*/
   /* Create a periodic timer for power save aging.          */
   /*--------------------------------------------------------*/  
   if(PALSRV_tcbNew(&psAgingTimer))
   {
	   wdrv_utilsReport(RL_DB, PRI_EMERGENCY,
       			"TX: Failed to allocate PS aging timer\n");
       return NOK;
   }

   PALSRV_tcbQueueAct(&psAgingTimer,
					  PS_AGING_INTERVAL, 
                      wdrv_txSendPSAgingTimerCB,
					  0, 0);

   /*--------------------------------------------------------*/
   /* Init the WEP ctrl										          */
   /*--------------------------------------------------------*/
   wdrv_dbGetPrivacyInvoked(&txPrivacyInvoked);


   /*--------------------------------------------------------*/
   /* Init rate failoure announcment callback for the HAL.   */
   /*--------------------------------------------------------*/
   whal_apiSetTxStatusCB((TxStatus_Callback_T) wdrv_txRateAdaptFailAnnounce);

   /*--------------------------------------------------------*/
   /* Announce the main state machien that init is completed.*/
   /*--------------------------------------------------------*/
   wdrv_mainsmSendModuleUp(WDRV_TX_MODULE_ID);

   return OK;
}

/*************************************************************************
 *                        wdrv_txUnload                                  *
 *************************************************************************
DESCRIPTION: This function unloads TX module

OUTPUT:      

RETURN:      OK:                                                                                                                                            
************************************************************************/
void wdrv_txUnload(void)
{
	UINT8 numSupWone;
	int   i;
	TxEnvelop_T *envelop;

	wdrv_dbGetNumSupWone(&numSupWone);

    /* for each list: free all envelops from the list and them free the list itself */ 
	if(freeDataEnvList)
	{
		while(TRUE)
		{
			if(wdrv_txEnvListGetFirst(freeDataEnvList, &envelop) == OK)
				PAL_memFree(0,(void *)envelop,0);
			else
				break;
		}
		
		wdrv_txFreeEnvList(freeDataEnvList);
	}

	if(freeMngtEnvList)
	{
		while(TRUE)
		{
			if(wdrv_txEnvListGetFirst(freeMngtEnvList, &envelop) == OK)
				PAL_memFree(0,(void *)envelop,0);
			else
				break;
		}
		
		wdrv_txFreeEnvList(freeMngtEnvList);
	}

	for(i=0; i<numSupWone; i++)
	{
		if(highPrioWaitEnvList[i])
		{
			while(TRUE)
			{
				if(wdrv_txEnvListGetFirst(highPrioWaitEnvList[i], &envelop) == OK)
					PAL_memFree(0,(void *)envelop,0);
				else
					break;
			}
			
			wdrv_txFreeEnvList(highPrioWaitEnvList[i]);
		}

		if(lowPrioWaitEnvList[i])
		{
			while(TRUE)
			{
				if(wdrv_txEnvListGetFirst(lowPrioWaitEnvList[i], &envelop) == OK)
					PAL_memFree(0,(void *)envelop,0);
				else
					break;
			}
				
			wdrv_txFreeEnvList(lowPrioWaitEnvList[i]);
		}
	}

	/* all envelops are free, now just free the stations lists */
	for(i=0; i<MAX_STATIONS; i++)
	{
		if(staEnvList[i])
			wdrv_txFreeEnvList(staEnvList[i]);
	}

	/* delete timers */
	PALSRV_tcbDelete(&waitListTimer);
	PALSRV_tcbDelete(&psAgingTimer);

	/* unload power save sub-module */
	wdrv_psUnload(MAX_STATIONS);	

	printf("TX unloaded !!!\n");
}


/*************************************************************************
 *                        wdrv_tx_getEvent                                *
 *************************************************************************
DESCRIPTION: This function implements the Tx Task flow.

OUTPUT:      

RETURN:      OK:                                                                                                                                            
************************************************************************/


STATUS wdrv_tx_getEvent(TxMessage_T *pTxMessage)
{
    UINT8 numSupWone;      
    UINT8 PsTxFailurs = 0;
    UINT8 woneIndex = 0;
	BOOL  WlanState;


	wdrv_dbGetNumSupWone(&numSupWone);
    
    /* Send MSDU to PS station in case of previous fail */ 
    wdrv_psGetPsPrevTxFailCounter(&PsTxFailurs);
    if(PsTxFailurs != 0)
    {
		UINT8 Ind;
		UINT8 prevTxFail = FALSE;

        for(Ind = 0;Ind < MAX_STATIONS ;Ind++)
        {
            wdrv_psGetPsPrevTxFail(Ind, &prevTxFail);
            if(prevTxFail == TRUE)
            {
                if (wdrv_psGetStaStatus(Ind)) {
                    /* Still in PS, send one packet */
                    sendPsMsdu(Ind);   
                    Tx_Statistics.psprevFailSend++;
                } 
                else
                {
                    /* Station is awake - try to send it all the stored packets */
                    while((sendPsMsdu(Ind) == OK));
                }

            }
        }
    }
                    
	switch (pTxMessage->type)
	{
		case WDRV_TX_MSDU_TIMEOUT:		
		case WDRV_TX_MSDU_SEND:
			
			bm_trace(10,pTxMessage->type,0);

			PAL_atomic_set(&inTx, 1);
		
			if(pTxMessage->type == WDRV_TX_MSDU_TIMEOUT)
				wdrv_DDA_init_tx_sched_count();

			/*dataMsgExistsInTxMsgQ = FALSE;*/
			/* Make sure that always the current highest priority MSDU is sent!! */
			while(1)
			{
				UINT8  continue_flag;
				
				for(woneIndex = 0, continue_flag = 0; woneIndex <  numSupWone; woneIndex++ )	
				{
					if( sendWaitMsdu(highPrioWaitEnvList[woneIndex]) == OK )
					   continue_flag++; 
				}
				
				if ( continue_flag != 0 )
					continue;
				
				for(woneIndex = 0, continue_flag = 0; woneIndex <  numSupWone; woneIndex++ )	
				{
					if( sendWaitMsdu(lowPrioWaitEnvList[woneIndex]) == OK )
					   continue_flag++; 
				}
				
				if ( continue_flag != 0 )
					continue;
				
				break;
			}
			
			PAL_atomic_set(&inTx, 0);

			break;


		case WDRV_TX_COMPLETE:
			wdrv_txCheckWaitPckts();			
			wdrv_dbGetWlanServiceState( &WlanState );
			if(WlanState)
				tiwlan_ddaLedAction(TIAP_LED_BLINK_OFF);
			bm_trace(9,0,0);
			break;
        
		case WDRV_TX_MSDU_PS_POLL:			
			Tx_Statistics.psPollEvents++;
			sendPsMsdu( pTxMessage->msg.psMsg.staIndex);
			break;
        
        
		case WDRV_TX_MSDU_PS_STATUS_CHANGE:
			wdrv_psPowerSaveStatusChangeFunc(pTxMessage->msg.psMsg.staIndex, 
				pTxMessage->msg.psMsg.status, pTxMessage->msg.psMsg.woneIndex);
        
			/* its only when the station changes from Power Save to Wake up mode */ 
			if(pTxMessage->msg.psMsg.status == FALSE)
			{
				Tx_Statistics.psWakeEvents++;
				while((sendPsMsdu( pTxMessage->msg.psMsg.staIndex) == OK));
			}
			break;
        
		case WDRV_TX_MSDU_PS_CLEAN_STATION_LIST:
			wdrv_powerSaveCleanStaListFunc( pTxMessage->msg.psMsg.staIndex ) ;
			wdrv_psPowerSaveStatusChangeFunc( (UINT32)pTxMessage->msg.psMsg.staIndex , FALSE, pTxMessage->msg.psMsg.woneIndex);
			break ;
        
        
		case WDRV_TX_PS_AGING:
			wdrv_txPowerSaveAgingFunc();
			break;

		default:
			break;        
	}

	return OK;
}


/*************************************************************************
 *                        wdrv_txSendMsdu                                *
 *************************************************************************
DESCRIPTION: This prepares the msdu to be sent. It takes an empty envelop              
             from the free list, put the msdu in to it, and insert the 
             envelop to the waiting list.
INPUT:       msduPtr : Pointer to the MSDU to send.
             priority : The Tx priority. (High \ Low) 

OUTPUT:      

RETURN:      OK : MSDU has been inserted to the waiting list.
             NOK: Couldn't insert the MSDU to the waiting list.                                                                                                                                             
************************************************************************/
STATUS wdrv_txSendMsdu( mem_MSDU_T *msduPtr, UINT16 stationIndex, 
                       ULONG txFlags ,UINT8 woneIndex)
{
   TxEnvelop_T *envelopPtr;
   TxMessage_T txMsg;   
   EnvList_T   *freeEnvList;
   EnvList_T   *waitEnvList;
   UINT8 numSupWone = 0;
   unsigned long flags;
   BOOL        busy;


   wdrv_dbGetNumSupWone(&numSupWone);
   
   /*if (REPORT_LEVEL(RL_TX_SNIFF))
      whal_utilsSniffer(msduPtr->firstBDPtr->data);*/

#if 0             /* No Ack emulation support  */
   rc = wdrv_ackEmulationTxPacket(msduPtr, &discardPacket);

   if (discardPacket == TRUE)
   {
      wlan_memMngrFreeMSDU(msduPtr->handle);
      return NOK;
   }
#endif

   /* If packet matches vHCF classification, set high priority and vHCF flags. */
   vhcfClassifyTxPacket(msduPtr, &txFlags);

   PAL_intrLock(flags);  /* START OF CRITICAL SECTION */

   if( txFlags & WDRV_TX_HIGH_PRIORITY )
   {
      waitEnvList = highPrioWaitEnvList[woneIndex];
      freeEnvList = freeMngtEnvList;
   }
   else   /* Low priority MSDU */
   {
      
	  /* If dual AP is enabled and this is a low priority MSDU from Ethernet 
	       and such MSDUs waiting in list have reached the limit,
	       free the MSDU and exit (to avoid Ethernet BDs starvation). */
	  if (((txFlags & WDRV_TX_FROM_ETHER) && (numSupWone == WONE_DUAL_BAND) &&
           (waitPktFromEther[woneIndex] >= TOTAL_NUM_OF_WONE_WHIT_DATA_ENVELOPS ))||
	  /* If vHCF is enbled and this is a low priority MSDU from Ethernet 
	       and such MSDUs waiting in list have reached the limit,
	       free the MSDU and exit (to avoid Ethernet BDs starvation). */
          (wdrv_dbIsVhcfEnabled() && (txFlags & WDRV_TX_FROM_ETHER) &&
           (waitLowPriorFromEther >= VHCF_LIMIT_WAIT_LOW_PRIOR_ETHER)))    
	  {
		  wlan_memMngrFreeMSDU(msduPtr->handle);
          PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */ 
          overloadWaitPktFromEther[woneIndex]++;
		  return NOK;
	  }

	  else
	  {
      	  waitEnvList = lowPrioWaitEnvList[woneIndex];
		  freeEnvList = freeDataEnvList;
	  }
   }

   /*--------------------------------------------------------*/
   /* Get a free envelop from the free list.                .*/
   /*--------------------------------------------------------*/
   if((wdrv_txEnvListGetFirst( freeEnvList, &envelopPtr)) == NOK)
   {
      if (REPORT_LEVEL(RL_TX))
         wdrv_utilsReport(RL_TX, PRI_EMERGENCY,
            "TX: send: Fail to get envelop from free queue.");
      wlan_memMngrFreeMSDU(msduPtr->handle ); 
      PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */ 
      return NOK;
   }


   /*--------------------------------------------------------*/
   /* Assign the MSDU and the attributes to the envelop.     */
   /*--------------------------------------------------------*/
   envelopPtr->msduPtr = msduPtr;
   envelopPtr->staIndex = stationIndex;
   envelopPtr->woneIndex  = woneIndex;
   envelopPtr->txFlags  = txFlags;


   /*--------------------------------------------------------*/
   /* Insert the envelop to the waiting list.                */
   /*--------------------------------------------------------*/   
   if((wdrv_txEnvListIns( waitEnvList ,&envelopPtr )) == NOK ){
      if (REPORT_LEVEL(RL_TX))
         wdrv_utilsReport(RL_TX, PRI_EMERGENCY,
            "TX: send: Fail to insert MSDU to the waiting list wone = %d",woneIndex);

         wlan_memMngrFreeMSDU(msduPtr->handle );
         wdrv_txFreeEnv( &envelopPtr );
         PAL_intrUnLock(flags);   /* END OF CRITICAL SECTION */
         return NOK;
   }
   
   /* If Wone mode and MSDU from Ethernet, increment counter (for limit the packet in tx Queue). */
   if((numSupWone == WONE_DUAL_BAND) && (txFlags & WDRV_TX_FROM_ETHER))
      waitPktFromEther[woneIndex]++;

   /* If a low priority MSDU from Ethernet, increment counter (for limit in vHCF case). */
   if ( (txFlags & WDRV_TX_FROM_ETHER) && (txFlags & WDRV_TX_LOW_PRIORITY) ) 
	   waitLowPriorFromEther++;


   /*--------------------------------------------------------*/
   /* Insert the envelop to a station envelop list.          */
   /*--------------------------------------------------------*/
   if( (txFlags & WDRV_TX_LOW_PRIORITY) &&
       (txFlags & WDRV_TX_UNICAST_MSDU) ){
   		wdrv_txStaEnvListIns( staEnvList[stationIndex] ,&envelopPtr );
	}   

#if 0
{
	
   /* decide which event will be sent to the DDA */
   if ( ((txFlags & WDRV_TX_FROM_ETHER) && (txFlags & WDRV_TX_LOW_PRIORITY)) ||
		(txFlags & WDRV_TX_VHCF_MSDU)) 
	   event = WDRV_TX_MSDU_SCHED;
   else
	   event = WDRV_TX_MSDU_SEND;  
}
#endif

   PAL_intrUnLock(flags);   /* END OF CRITICAL SECTION */ 
   PAL_atomic_read(&inTx, busy);
   if(!busy)
   {
	   bm_trace(7,WDRV_TX_MSDU_SEND,0);
	   txMsg.type = WDRV_TX_MSDU_SEND;   
	   wdrv_DDA_tx_setEvent(&txMsg);
   }
   else
	   bm_trace(8,0,0);

   return OK;
}


/*************************************************************************
 *                        wdrv_txSendPsPollMsg                           *
 *************************************************************************
DESCRIPTION: This function sends ps-poll message.

INPUT :      staIndex : The index of the station in the main db table.
OUTPUT:      

RETURN:      OK:                                                                                                                                            
************************************************************************/
void wdrv_txSendPsPollMsg( UINT32 staIndex )
{
    TxMessage_T txMsg;

    txMsg.type = WDRV_TX_MSDU_PS_POLL;
    txMsg.msg.psMsg.staIndex = staIndex;
	
	/* L.M this event was treated as urgent, check if we have to implement this in the DDA */
    wdrv_DDA_tx_setEvent(&txMsg);
}

/*************************************************************************
 *                        wdrv_txSendPSAgingTimerCB                          *
 *************************************************************************
DESCRIPTION: This function send a message to the Tx task to commit a PS
             Aging.

INPUT :     
OUTPUT:      

RETURN:                                                                                                                                             
************************************************************************/
void wdrv_txSendPSAgingTimerCB( UINT32 data )
{
   TxMessage_T txMsg;

   txMsg.type = WDRV_TX_PS_AGING;
   wdrv_DDA_tx_setEvent(&txMsg);

   PALSRV_tcbQueueAct(&psAgingTimer,
					  PS_AGING_INTERVAL, 
                      wdrv_txSendPSAgingTimerCB,
					  0, 0); 
}

/*************************************************************************
 *                        wdrv_txSendPsChangeStateMsg                    *
 *************************************************************************
DESCRIPTION: This function sends ps-Status Change message.

INPUT :      staIndex : The index of the station in the main db table.
OUTPUT:      

RETURN:      OK:                                                                                                                                            
************************************************************************/
void wdrv_txSendPsChangeStateMsg( UINT32 staIndex ,UINT16 status, UINT8 woneIndex)
{
   TxMessage_T txMsg;

   txMsg.type = WDRV_TX_MSDU_PS_STATUS_CHANGE;
   txMsg.msg.psMsg.staIndex  = staIndex;
   txMsg.msg.psMsg.status    = status ;
   txMsg.msg.psMsg.woneIndex = woneIndex ;
   wdrv_DDA_tx_setEvent(&txMsg);
}


/****************************************************************************
 *                      wdrv_txSendPsCleanStaListMsg()                       *
 ****************************************************************************
 * DESCRIPTION: Clean all the packets stored in a station power save list 
 *              and set the PS status to FALSE, used for disconnecting a 
 *              staion from the AP.
 * INPUTS: stationIndex - The station Index
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	None
 ****************************************************************************/
void  wdrv_txSendPsCleanStaListMsg(UINT16 staIndex, UINT8 woneIndex) 
{
   TxMessage_T txMsg;

   txMsg.type = WDRV_TX_MSDU_PS_CLEAN_STATION_LIST;
   txMsg.msg.psMsg.staIndex = staIndex;
   txMsg.msg.psMsg.woneIndex = woneIndex ;
   wdrv_DDA_tx_setEvent(&txMsg);
}

/*************************************************************************
 *                        wdrv_txCheckWaitPcktsFromIntr                          *
 *************************************************************************
DESCRIPTION: This callback function is called from the HAL on TX complete
			 interrupt reception

OUTPUT:      

RETURN:      OK:                                                                                                                                            
************************************************************************/
void wdrv_txCheckWaitPcktsFromIntr( void )
{
	TxMessage_T msg;
	msg.type = WDRV_TX_COMPLETE;
	
	bm_trace(6,0,0);
	/*whal_acxTxCmpltIntrDisable();*/
	wdrv_DDA_tx_setEvent(&msg);
}

/*************************************************************************
 *                        wdrv_txCheckWaitPckts                          *
 *************************************************************************
DESCRIPTION: This function verifies if there are and waiting packets in 
             the waiting list, if there are, send a TIMEOUT message to the
             Tx task.

OUTPUT:      

RETURN:      OK:                                                                                                                                            
************************************************************************/	
void wdrv_txCheckWaitPckts( void )
{
   TxMessage_T msg;
   int   waitingPackets = FALSE;
   UINT8 woneIndex ,numSupWone, LocalQid, WoneTxQueues;
   TxQueueId_T 	TxQid;
   unsigned long flags;

   wdrv_dbGetNumSupWone(&numSupWone);
   
   PAL_intrLock(flags);  /* START OF CRITICAL SECTION */

   for (woneIndex = 0,TxQid = 0; woneIndex < numSupWone; woneIndex++)
   {
	   wdrv_dbGetNumSupWoneQueues( woneIndex, &WoneTxQueues );
	   for (LocalQid=0; LocalQid<WoneTxQueues; LocalQid++,TxQid++)                              
	   {
		   whal_apiTxClean(TxQid);
	   }
   }

   for(woneIndex=0; woneIndex < numSupWone ; woneIndex++)
   {  
      if(( highPrioWaitEnvList[woneIndex]->CurrNumOfEnv != 0 ) ||
         ( lowPrioWaitEnvList [woneIndex]->CurrNumOfEnv != 0 ) )
      {
		 waitingPackets = TRUE;
         break;
      }
   }

   if(waitingPackets == TRUE)
   {
         PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */		 
		 msg.type = WDRV_TX_MSDU_TIMEOUT;
         wdrv_DDA_tx_setEvent(&msg);
   }
   else
   {
        PAL_intrUnLock(flags);  /* END OF CRITICAL SECTION */
   }
}

void wdrv_txWaitListCheckTimerCB( UINT32 data )
{
    wdrv_txCheckWaitPckts();

	/* reschedule the timer */
    PALSRV_tcbQueueAct(&waitListTimer,
					  10, 
                      wdrv_txWaitListCheckTimerCB,
					  0, 0);
}

/*************************************************************************
 *                        wdrv_txMakeConcatDecision                          *
 *************************************************************************
DESCRIPTION: This function makes the decision regarding the concatenation

OUTPUT:      Three different decisions can be done:
             1. Don't transmit any packet (returned NOK)
             2. Transmit only one packet (return OK)
             3. Concatenate (returned OK and max permitted msdu number)

RETURN:      OK:                                                                                                                                            
************************************************************************/
#define HOST_Q_THRESHOLD 4
#define ACX_Q_THRESHOLD  0
#define ACX_Q_UPPER_THRESHOLD 5
int wdrv_txMakeConcatDecision(TxQueueId_T TxQid,
                              TxEnvelop_T *envelopPtr,
                              BOOL        Sta4xActive,  
                              BOOL        *MakeConcatenation, 
                              UINT32      *ConcatMaxCount, 
							  BOOL		  *allocFail)
{

    UINT32      	FreeHostDescNum;
    UINT32      	CurrAcxDescNum;
	UINT32			BusyQueueThreshold;
	UINT8			numSupWone;


   wdrv_dbGetNumSupWone(&numSupWone);
   if(numSupWone == 1)
	   BusyQueueThreshold = 2;
   else
	   BusyQueueThreshold = 1;


  *allocFail = FALSE;

  if( whal_apiTxGetBufsNum(TxQid, &FreeHostDescNum, &CurrAcxDescNum) == NOK)
  {
    /* No Host or Acx descriptor available, return */
	*allocFail = TRUE;
    return NOK;
  }

  if(CurrAcxDescNum == 0)
	  Tx_Statistics.acxQueUnderflow++;

  if(concatOn && Sta4xActive)
  {
		/* If there are enough packets in the ACX Q and there is a free space in the host Q
           we want to transmit only in case we have enough packets to concatenate (more than 2) */
  		/*if(CurrAcxDescNum > ACX_Q_UPPER_THRESHOLD)
			return NOK;*/
        if ((FreeHostDescNum > HOST_Q_THRESHOLD) && (CurrAcxDescNum > ACX_Q_THRESHOLD))
        {   
            if(staEnvList[envelopPtr->staIndex]->CurrNumOfEnv > BusyQueueThreshold)
            {
                *MakeConcatenation = TRUE;
                *ConcatMaxCount = (FreeHostDescNum-1) >> 2; /*/MIN_NUM_OF_BDs_IN_MSDU;*/
                return OK;
            }
            else
            {
                return NOK;
            }
        }
        /* if there are not enough desc. in the ACX Q, pass the packet down unconditionally */
        else if(CurrAcxDescNum <= ACX_Q_THRESHOLD)
        {      
            *ConcatMaxCount = (FreeHostDescNum-1) >> 2; /*/MIN_NUM_OF_BDs_IN_MSDU;*/
            if((*ConcatMaxCount > 1) && (staEnvList[envelopPtr->staIndex]->CurrNumOfEnv > 1))
            {
                *MakeConcatenation = TRUE;
            }
            else
            {
                *MakeConcatenation = FALSE;
            }
            return OK;
        }
        else /* (FreeHostDescNum <= HOST_Q_THRESHOLD) */
        {
            return NOK;
        }
  }


  else /* Concatenation disable */
  {
     *MakeConcatenation = FALSE;
     return OK;
  }
}


/*************************************************************************
 *                        sendWaitMsdu                                   *
 *************************************************************************
DESCRIPTION: This function tries to send one packet form the waiting queue
             to the acx100.
INPUT :      *envList : Pointer to the envelop list to handle.
OUTPUT:      

RETURN:      OK:  The first MSDU has been sent, envelop returned to the 
                  free list.
             NOK: The MSDU could not have been sent.   
************************************************************************/
STATUS sendWaitMsdu( EnvList_T  *envList)
{
    TxEnvelop_T  *envelopPtr;    
    BOOL   allocFail;
    UINT32 ucastQueueIndex;    
    mem_MSDU_T *buildMsduPtr;
	unsigned long flags;
        	

    /*--------------------------------------------------------------*/
    /*   Look at the first MSDU envelop from the waiting queue.     */
    /*--------------------------------------------------------------*/
	PAL_intrLock(flags);
    if( wdrv_txEnvListWatchFirst( envList ,&envelopPtr) != OK ){
        /* List is empty - return NOK */
        goto fail;
    }	
    
    /* Get Tx queue index according to MSDU priority, and also for PS-Broadcast. */
    if (envelopPtr->txFlags & WDRV_TX_HIGH_PRIORITY)
        wdrv_dbGetAcxGlobalQindex(envelopPtr->woneIndex, WONE_HIGH_PRIOR_QUEUE, &ucastQueueIndex);
    else 
        wdrv_dbGetAcxGlobalQindex(envelopPtr->woneIndex, WONE_LOW_PRIOR_QUEUE, &ucastQueueIndex);
        
    
    if ( envelopPtr->txFlags & WDRV_TX_BRCST_MTCST_MSDU )
    {
        /*-----------------------------------------------------------*/
        /*                  Broadcast MSDU                           */
        /*-----------------------------------------------------------*/
        
        if (wdrv_psGetTotalWoneStaInPSMode(envelopPtr->woneIndex) != 0 )
        {
            /*-----------------------------------------------------------*/
            /*                  BCAST w/ Power Save                      */
            /*-----------------------------------------------------------*/
            
			UINT32 bcastQueueIndex;

            /*  Power save is active, send the BRCST MSDU to the BRCST queue.  */
            wdrv_txEnvListGetFirst( envList, &envelopPtr);
			PAL_intrUnLock(flags);
            wdrv_dbGetAcxGlobalQindex(envelopPtr->woneIndex, WONE_PS_BCAST_QUEUE, &bcastQueueIndex);
            /* Verify if there is a space in the acx100 buffers.         */
            /* check the broadcast queue size                */ 						
            if ( whal_apiTxCheckSize(bcastQueueIndex, envelopPtr->msduPtr->dataLen)==NOK )
            {
                /* Not place in BDRAM -> Discards the packet */				
                Tx_Statistics.brcstAllocFail++;
                wlan_memMngrFreeMSDU( (envelopPtr)->msduPtr->handle  );
                wdrv_txFreeEnv( &envelopPtr );
                goto succ;
            }
            else
            {
                /* put the packet in the BCAST SDRAM QUEUE */
                wdrv_txSendMsduToHal( envelopPtr, bcastQueueIndex );
                Tx_Statistics.broadcastQsent++;                
                wdrv_txFreeEnv( &envelopPtr );  
                goto succ;
            } 			
        }  /* Power save mode is activated. */
        
        else
        {
            /*-----------------------------------------------------------*/
            /*                  BCAST without Power Save                 */
            /*-----------------------------------------------------------*/            
            /* Send as unicast packet */
            if ( whal_apiTxCheckSize(ucastQueueIndex, envelopPtr->msduPtr->dataLen)==NOK )
            {
                /* No room for packet -> count and leave it in the waiting Q for the next iteration */
                Tx_Statistics.unicAllocFail++;
                goto fail;
            }
            
            else
            { 
                /* send The MSDU */
                wdrv_txEnvListGetFirst( envList, &envelopPtr);
				PAL_intrUnLock(flags);
                wdrv_txSendMsduToHal( envelopPtr, ucastQueueIndex );
                Tx_Statistics.unicastQsent++;
                wdrv_txFreeEnv( &envelopPtr );
                goto succ;
            }
        } /* broadcast, no power save */
    } /* Broadcast MSDU */
    
    else 
    {
        /*-----------------------------------------------------------*/
        /*                  Unicast MSDU                             */
        /*-----------------------------------------------------------*/
        
		UINT32 MaxConcatCount = 0;		
		UINT16 staMaxConcatSize;
        BOOL   MakeConcatenation = FALSE;
		EnvList_T *staEnvListPtr = NULL;
		 
		if(envelopPtr->staIndex != INVALID_INDEX)		
			staEnvListPtr = staEnvList[envelopPtr->staIndex];
                
        if ((wdrv_psGetStaStatus( envelopPtr->staIndex ) == TRUE) &&
            (envelopPtr->txFlags & WDRV_TX_PS_NOT_ALLOWED)) {
            wdrv_powerSaveCleanStaListFunc(envelopPtr->staIndex);
        }
        
        /* Verify if the station is in PS mode, if yes, store the    */
        /* envelop in the PS module (DATA ONLY!), otherwise send it. */
        if( (wdrv_psGetStaStatus( envelopPtr->staIndex ) == TRUE ) &&
            (!(envelopPtr->txFlags & WDRV_TX_PS_NOT_ALLOWED)))
        {
            wdrv_txEnvListGetFirst( envList, &envelopPtr);
            
            if(envelopPtr->txFlags & WDRV_TX_LOW_PRIORITY)
                wdrv_txStaEnvListGetFirst( staEnvList[envelopPtr->staIndex], &envelopPtr);
            
            if(wdrv_psStoreEnv( envelopPtr->staIndex, &envelopPtr) == NOK )
            {                
                wlan_memMngrFreeMSDU( (envelopPtr)->msduPtr->handle  );
                wdrv_txFreeEnv( &envelopPtr );
            }
			PAL_intrUnLock(flags);
            goto succ;
        }
        
        
        /* If we got here then this is a unicast packet with no power save. */
        
        if( envelopPtr->txFlags & WDRV_TX_LOW_PRIORITY )
        { 
			BOOL	sta4xActive;
            /**************************************************************************/
            /*                Concatenation  Decision                                 */
            /***************************************************************************/
            wdrv_dbGetStationFourXActive(envelopPtr->staIndex, &sta4xActive);
            

            /* make decision regarding concatenation: concatenate/don't concatenate/send nothing */
			/* WA for downstream 4x traffic in WONE */
            if (sta4xActive)
            {				
                if (wdrv_txMakeConcatDecision(ucastQueueIndex,
                    envelopPtr,
                    sta4xActive,
                    &MakeConcatenation, 
                    &MaxConcatCount,
                    &allocFail) == NOK)
                {
                    if(allocFail)
					{
						bm_trace(3,0,0);
                        Tx_Statistics.unicAllocFail++;
					}
					goto fail;
                }
            
				/* Allocate one MSDU + BD for the concatenation header. */
				if ( MakeConcatenation && 
					(wlan_memMngrAllocMSDU(&buildMsduPtr, WLAN_HDR_LEN + WLAN_SNAP_HDR_LEN + WLAN_4X_CONCAT_HDR_LEN ) ==NOK))
				{
					MakeConcatenation = FALSE; 
				}
			}
        }               
                
        if( MakeConcatenation )            
        {
            /**************************************************************************/
            /*                            Concatenation                               */
            /**************************************************************************/
            
            int  CurrConcatMsduCnt;
            TxEnvelop_T    *firstEnvPtr;
            TxEnvelop_T    *nextEnvPtr;
            int NumOfConcatPkts = 1;
            UINT32 concateSize = WLAN_HDR_LEN+ WLAN_SNAP_HDR_LEN + WLAN_4X_CONCAT_HDR_LEN;
            UINT16 concatFlags = 0;
            UINT16 newConcatSize = 0;
            			
            wdrv_dbGetStationConcatMaxSize(envelopPtr->staIndex, &staMaxConcatSize); 
			/* TODO: use STA's maxConcatSize (instead the global one) in next release */
            
            /* Get the first envelop both from the waiting queue and the station list. */
            wdrv_txEnvListGetFirst( envList, &envelopPtr); 

            wdrv_txStaEnvListGetFirst(staEnvListPtr ,&envelopPtr);
            firstEnvPtr = envelopPtr;
            concateSize += firstEnvPtr->msduPtr->dataLen;
            
            
            /* Continue concatenation until:
            There are no more envelopes in the wait list OR
            There are no more free host descriptors OR
            The max length achieved */
            
            CurrConcatMsduCnt = (int)MaxConcatCount;
            
            Tx_Statistics.concatPackets++;
            			
            while( ( CurrConcatMsduCnt > 0) && 
                ( wdrv_txEnvListWatchFirst(staEnvListPtr,&nextEnvPtr) == OK) &&
                ( (newConcatSize=(concateSize+nextEnvPtr->msduPtr->dataLen)) <= maxConcatSize) )
            {
                
                /* There is another envelop to concat, and no length overflow. */
                /* link the next envelops MSDUs .                               */
                envelopPtr->msduPtr->nextConcatMSDU = nextEnvPtr->msduPtr;
                
                /* Not the first envelop, should be returned to the free list. */
                if( envelopPtr != firstEnvPtr)
                {
				   if (envelopPtr->txFlags & WDRV_TX_FROM_ETHER)
				   {
					  if (waitLowPriorFromEther > 0 )
							waitLowPriorFromEther--;
						
					  if (waitPktFromEther[envelopPtr->woneIndex] > 0)
						 waitPktFromEther[envelopPtr->woneIndex]--;
				   }

                    wdrv_txEnvListIns( freeDataEnvList ,&envelopPtr );
                }
                
                /* Get the next envelop  from the list. */
                wdrv_txStaEnvListGetFirst(staEnvListPtr, &envelopPtr);

                /* Remove the envelop from the waiting Queue */
                wdrv_txEnvListGetMiddle( envList, envelopPtr);
					
                
                concateSize = newConcatSize;
                
                NumOfConcatPkts++;
                Tx_Statistics.concatPackets++;
                
                CurrConcatMsduCnt-=2;
            }
            
            envelopPtr->msduPtr->nextConcatMSDU = NULL;
            
            /* Last envelop should be returned to the free list. */
            if( envelopPtr != firstEnvPtr)
            {
                
               if (envelopPtr->txFlags & WDRV_TX_FROM_ETHER)
               {
                  if (waitLowPriorFromEther > 0 )
                    waitLowPriorFromEther--;
                
                  if (waitPktFromEther[envelopPtr->woneIndex] > 0)
                     waitPktFromEther[envelopPtr->woneIndex]--;
               }
                
                wdrv_txEnvListIns( freeDataEnvList ,&envelopPtr );
            }
                        
#if 0 /* No Ack emulation in this version */
            wdrv_dbGetStationAckEmul(firstEnvPtr->staIndex, &AckEmulStaSupport);
            if( (AckEmulStaSupport == TRUE) &&
                (wdrv_txEnvListGetCurrentStored(staEnvListPtr) > 1))
                concatFlags = concatFlags | WLAN_4X_CONCAT_MORE_BIT; 
#endif            
            
            /* MSDU list is ready send to the concatenator. */			
            wdrv_txConcatMsduList( buildMsduPtr, &(firstEnvPtr->msduPtr), concatFlags);
			PAL_intrUnLock(flags);
            
            if( concatDbg )
                wdrv_utilsReport(RL_4X, PRI_INFORMATION, "Tx - Sending %d concatenated MSDUs\n", NumOfConcatPkts );
            
			wdrv_dbIncreaseStationTxPacketCounter(firstEnvPtr->staIndex);
            wdrv_txSendMsduToHal( firstEnvPtr, ucastQueueIndex );
            Tx_Statistics.unicastQsent++;
            wdrv_txFreeEnv( &firstEnvPtr );
			/*wdrv_DDA_queue_start();*/
            goto succ;            
        }
        else {
            /**********************************************************************/
            /*                    Send a single MSDU                              */
            /**********************************************************************/
            
            /* Verify if there is a space in the acx buffers.         */
            
            if( concatDbg ){
                wdrv_utilsReport(RL_4X, PRI_INFORMATION, " Send Single MSDU with concat on - STA=%d txFlags=0x%X\n",
                    envelopPtr->staIndex, (int)envelopPtr->txFlags);
            }
            
            envelopPtr->msduPtr->nextConcatMSDU = NULL;
            
            if( whal_apiTxCheckSize(ucastQueueIndex, envelopPtr->msduPtr->dataLen) == NOK )
            {
                Tx_Statistics.unicAllocFail++;
				bm_trace(3,0,0);
                goto fail;
            }
            else { 							
                /*          Send The MSDU       */
                wdrv_txEnvListGetFirst( envList, &envelopPtr);
                
                if( (envelopPtr->txFlags & WDRV_TX_LOW_PRIORITY) &&
                    (envelopPtr->txFlags & WDRV_TX_UNICAST_MSDU) ) {                    
                    /* Get the next envelop  from the station list. */
                    wdrv_txStaEnvListGetFirst( staEnvList[envelopPtr->staIndex] ,&envelopPtr);
                }
				PAL_intrUnLock(flags);
                
				if( (envelopPtr->txFlags & WDRV_TX_LOW_PRIORITY) &&
                    (envelopPtr->txFlags & WDRV_TX_UNICAST_MSDU) )
					wdrv_dbIncreaseStationTxPacketCounter(envelopPtr->staIndex);
					
				Tx_Statistics.unConcatPackets++;
				wdrv_txSendMsduToHal( envelopPtr, ucastQueueIndex );
                Tx_Statistics.unicastQsent++;
                wdrv_txFreeEnv( &envelopPtr );
                goto succ;
                
            } /* A single MSDU */
        } /* unicast no power save */
   } /* Unicast MSDU */

succ:
	return OK;

fail:
	PAL_intrUnLock(flags);
	return NOK;	
}



/*************************************************************************
 *                        wdrv_txFreeEnv                             *
 *************************************************************************
DESCRIPTION: This function frees the envelop's (and MSDU's buffers in Slave mode),
             and put the envelop in the free list.
INPUT :    *envelopPtr : Pointer to the envelop to free.
OUTPUT:     

RETURN:        
************************************************************************/
void wdrv_txFreeEnv( TxEnvelop_T **envelopPtr )
{
   EnvList_T   *freeEnvList;
   unsigned long flags;

   PAL_intrLock(flags);

   /* In Master mode the MSDU must be freed before calling to this function */
   if((*envelopPtr)->txFlags & WDRV_TX_HIGH_PRIORITY ){
      freeEnvList = freeMngtEnvList;
   }
   else
   { 
      freeEnvList = freeDataEnvList;
	  if( ( (*envelopPtr)->txFlags & WDRV_TX_FROM_ETHER   ) &&
	      ( (*envelopPtr)->txFlags & WDRV_TX_LOW_PRIORITY ) )
      {
		  if(waitLowPriorFromEther > 0) 
			  waitLowPriorFromEther--;

          if (waitPktFromEther[(*envelopPtr)->woneIndex] > 0)
             waitPktFromEther[(*envelopPtr)->woneIndex]--;
      }
   }

   wdrv_txEnvListIns( freeEnvList , envelopPtr );

   PAL_intrUnLock(flags);

   return;
}




/*************************************************************************
 *                        sendPsMsdu                                     *
 *************************************************************************
DESCRIPTION: This function takes an envelop from the PS list of a given 
             station and send it, if there is no space in the MAC buffers
             the packet is kept in the head of the low priority waiting 
             list.
INPUT :    stationIndex - The index of the station to take the packet 
                          from.
OUTPUT:     

RETURN:        
************************************************************************/
STATUS sendPsMsdu( UINT16 stationIndex)
{
    TxEnvelop_T  *envelopPtr;
    UINT32 ucastQueueIndex = 0;
	unsigned long flags;

	/* this function must be reentrant, we have to create critical section
	   between envelope watch and get */
	PAL_intrLock(flags);
    
    /*--------------------------------------------------------------*/
    /*   Look at the first MSDU envelop from the power save queue.  */
    /*--------------------------------------------------------------*/
    if( wdrv_psWatchEnv( stationIndex ,&envelopPtr) != OK ){
        /* List is empty - return NOK */
        return NOK;
    }
    
    if (envelopPtr->txFlags & WDRV_TX_HIGH_PRIORITY)
        wdrv_dbGetAcxGlobalQindex(envelopPtr->woneIndex, WONE_HIGH_PRIOR_QUEUE, &ucastQueueIndex);
    else 
        wdrv_dbGetAcxGlobalQindex(envelopPtr->woneIndex, WONE_LOW_PRIOR_QUEUE, &ucastQueueIndex);
    
    
    /*-----------------------------------------------------------*/
    /*                  Send    MSDU                             */
    /*-----------------------------------------------------------*/
    if( whal_apiTxCheckSize(ucastQueueIndex, envelopPtr->msduPtr->dataLen) == NOK ){
        Tx_Statistics.psunicAllocFail++;
        wdrv_psSetPsPrevTxFail(stationIndex);
        return NOK;
    }
    
    Tx_Statistics.sentPsPackets++;
    
    /* There is a place for the packet, take it from the list and send it. */
    wdrv_psGetEnv(  stationIndex ,&envelopPtr);	

	PAL_intrUnLock(flags);
    
    /* Controlling the MORE_DATA field. */
    if( wdrv_psPsOnAndPksInQueue( stationIndex ) == TRUE ){
        dot11_header_T *pDot11Hdr = (dot11_header_T*)memMgr_BufData(envelopPtr->msduPtr->firstBDPtr);
        pDot11Hdr->fc |= DOT11_FC_MORE_DATA;
    }
    
	wdrv_dbIncreaseStationTxPacketCounter(envelopPtr->staIndex);
	wdrv_txSendMsduToHal( envelopPtr, ucastQueueIndex );
    Tx_Statistics.unicastQsent++;
    wdrv_txFreeEnv( &envelopPtr );  

	return OK;
}



/*************************************************************************
 *                        wdrv_txSetTxAttrProtection                     *
 *************************************************************************
DESCRIPTION:                                                                
      
INPUT :                                                     
OUTPUT:     

RETURN:        
************************************************************************/
static void wdrv_txSetTxAttrProtection(TxAttr_T *pTxAttr, UINT16 StaIndex, UINT8 WoneIndex)
{
	UINT8 ERPProtectionType;
	UINT8 ERPProtectionTI;
	BOOL currentERPProtectionMode;
    UINT8 band;

	wdrv_dbGetERPProtectionType(0,&ERPProtectionType);
	wdrv_dbGetERPProtectionTI(0,&ERPProtectionTI);
	wdrv_dbGetCurrentERPProtectionMode(&currentERPProtectionMode);
	
    wdrv_dbGetBand(WoneIndex, &band);

	if ((currentERPProtectionMode == TRUE) &&
		(pTxAttr->Rate & ACX_RATE_ALL_OFDM_MASK)&&
        (band == RADIO_BAND_2_4GHZ))
	{
		BOOL trick_packet_support;

		wdrv_dbGetTIprotectionSupport(StaIndex, &trick_packet_support);
		
		if(ERPProtectionTI == PROTECTION_TI_ENABLED && trick_packet_support)
			pTxAttr->ProtectionType = TXATTR_PROTECTION_TI_TRICK;	
		else
		{
			if (ERPProtectionType == PROTECTION_TYPE_RTS_CTS)
			{
				pTxAttr->ProtectionType = TXATTR_PROTECTION_RTS_CTS;
			}
			else
			{
				pTxAttr->ProtectionType = TXATTR_PROTECTION_CTS_TOSELF;
			}
		}
	}
	else
	{
		pTxAttr->ProtectionType = TXATTR_PROTECTION_NONE;
	}
}


/*************************************************************************
 *                        wdrv_txSendMsduToHal                           *
 *************************************************************************
DESCRIPTION: This function prepares and sends the envelop to the HAL layer
      
INPUT :    *envelopPtr : Pointer to the envelop to send
OUTPUT:     

RETURN:        
************************************************************************/
STATUS wdrv_txSendMsduToHal(TxEnvelop_T* envelopPtr, TxQueueId_T txQid)
{
	TxAttr_T txAttr;
	BOOL     dstAuthorized;	
	BOOL     forceUnEncrypted;

	txAttr.ShortPreamble = 0;

	/*--------------------------------------------*/
	/*--------------------------------------------*/
	/*   Rate and ACX queues managment.           */
	/*--------------------------------------------*/
	/*--------------------------------------------*/
	if(envelopPtr->txFlags & WDRV_TX_BRCST_MTCST_MSDU ){
	    /*--------------------------------------------*/	   
	    /*            A broadcast packet              */
	    /*--------------------------------------------*/
		wdrv_dbGetDefaultActiveRateAcx(envelopPtr->woneIndex, &(txAttr.Rate), 
									   &(txAttr.RateWithFallback), TXATTR_BRCST_MTCST);
		txAttr.ProtectionType = TXATTR_PROTECTION_NONE;
		txAttr.Broadcast = TRUE;
		Tx_Statistics.brcstSent++;

    } 

	else {
	    /*--------------------------------------------*/	   
	    /*           Unicast packet                   */
		/*--------------------------------------------*/
		txAttr.Broadcast = FALSE;
		Tx_Statistics.unicastSent++;

		if ((envelopPtr->staIndex == INVALID_INDEX) ||          /* station is not in the database */
			(!wdrv_mgmtIsStaAssoc(envelopPtr->staIndex)) ||     /* station is in the database but not associated yet. */
			( (envelopPtr->txFlags & WDRV_TX_HIGH_PRIORITY) &&  /* high priority frame     */
			 !(envelopPtr->txFlags & WDRV_TX_VHCF_MSDU) ) )     /* but not a video frame!! */
		{
			wdrv_dbGetDefaultActiveRateAcx(envelopPtr->woneIndex, &(txAttr.Rate), &(txAttr.RateWithFallback), TXATTR_UNICAST);
			txAttr.ProtectionType = TXATTR_PROTECTION_NONE;
		}
		else
		{
			wdrv_txUpdateRateAdaptation( envelopPtr->staIndex );
			txAttr.Rate = tx_getStaRate( envelopPtr->staIndex, &txAttr.RateWithFallback );	/* station is in the database and associated. */
			tx_updateTxBrateTimestamp(txAttr.Rate);
			wdrv_dbSetCurrentActiveRateWithFallback(envelopPtr->staIndex, &txAttr.RateWithFallback);
			/* Call this after setting txAttr.Rate */
			wdrv_txSetTxAttrProtection(&txAttr, envelopPtr->staIndex, envelopPtr->woneIndex);

			{
				BOOL currentShortPreambleSupport;
				/* Apply Short preamble on Unicast Data frames only */
				wdrv_dbGetCurrentShortPreambleSupport(&currentShortPreambleSupport);
				if (currentShortPreambleSupport)
				{                
					dot11_header_T *pDot11Hdr = 
					(dot11_header_T*)memMgr_BufData(envelopPtr->msduPtr->firstBDPtr);
					if ((pDot11Hdr->fc & DOT11_FC_TYPE_MASK) == DOT11_FC_TYPE_DATA)
						txAttr.ShortPreamble = 1;
				}
			}
		}
	}

	/*--------------------------------------------*/
	/*--------------------------------------------*/
	/*   Securety settings for data and mngt ment.*/
	/*--------------------------------------------*/
	/*--------------------------------------------*/
	if( (envelopPtr->txFlags & WDRV_TX_HIGH_PRIORITY) &&
       !(envelopPtr->txFlags & WDRV_TX_VHCF_MSDU) ) {    /* Not a video frame!! (only mgmt) */
	   /*--------------------------------------------*/
	   /*           Managment packet                 */
		/*--------------------------------------------*/
	   Tx_Statistics.highPrioSent++;
	}
   
	else 
	{
	   /*--------------------------------------------*/	   
	   /*         Low Priority (Data) queue          */
		/*--------------------------------------------*/

		/****************************************************************/
		/*   Note that the vHCF MSDU goes through this code although    */
		/*     it is transmitted through the HIGH-PRIORITY queue!!!     */
		/****************************************************************/
		if(envelopPtr->txFlags & WDRV_TX_VHCF_MSDU)
			Tx_Statistics.highPrioSent++;
		else
			Tx_Statistics.lowPrioSent++;
	}

	/* TODO_WPA - move the TKIP support to the HAL */
	if(envelopPtr->txFlags & WDRV_TX_FORCE_UN_ENCRYPTED)
		forceUnEncrypted = TRUE;
	else
		forceUnEncrypted = FALSE;

	if(envelopPtr->txFlags & WDRV_TX_BRCST_MTCST_MSDU )
		dstAuthorized = wdrv_mgmtIsBroadcastAuthorized();
	else if(envelopPtr->staIndex != INVALID_INDEX)
		dstAuthorized = wdrv_mgmtIsStationAuthorized(envelopPtr->staIndex);
	else
		dstAuthorized = TRUE;


	/* Now, if the station is not authorized, only MGMT & EAPOLs frames are allowed */
	/* MGMT & EAPOLs are high priority, so all low priority frames are dropped. */
	/* Video frames are high priority, we have to make sure we are dropping them too. */
	if  (dstAuthorized == FALSE) 
	{
		if ((envelopPtr->txFlags & WDRV_TX_LOW_PRIORITY) ||
          (envelopPtr->txFlags & WDRV_TX_VHCF_MSDU))
		{
			/* Drop it*/
			wlan_memMngrFreeMSDU( (envelopPtr)->msduPtr->handle  );
			return OK;
		}
	}
	
   txAttr.Tkip = FALSE;
   if ((txPrivacyInvoked) && (dstAuthorized == TRUE) && (forceUnEncrypted == FALSE))
	{	
	    UINT8    stationCipher = CIPHER_NONE;
		dot11_header_T *pDot11Hdr = 
		(dot11_header_T*)memMgr_BufData(envelopPtr->msduPtr->firstBDPtr);
		pDot11Hdr->fc |= DOT11_FC_WEP;

		/* Now, if the suite is TKIP we have to calculate the MIC and set the TKIP flag, otherwise, set the WEP flag. */
		if(envelopPtr->txFlags & WDRV_TX_BRCST_MTCST_MSDU )
		{
			wdrv_dbGetBroadcastSuiteType(&stationCipher);
			if (stationCipher == CIPHER_TKIP)
				txAttr.Tkip = TRUE;
		}

		else
		{
			if(envelopPtr->staIndex != INVALID_INDEX)
			   wdrv_dbGetStationSuiteType(envelopPtr->staIndex, &stationCipher);

			if (stationCipher == CIPHER_TKIP)
				txAttr.Tkip = TRUE;
		}

	}

#if 0
   if(whal_apiSendMsdu(txQid, envelopPtr->msduPtr, &txAttr) == OK)
   {
	   TransmittedMsduCount[txQid]++;
	   if(TransmittedMsduCount[txQid] >= 8)
	   {
		   wdrv_txCheckWaitPckts();
		   TransmittedMsduCount[txQid] = 0;
		   whal_apiRestartWatchDog_PacketTransmitted();
	   }
   }
#else
   whal_apiSendMsdu(txQid, envelopPtr->msduPtr, &txAttr);
#endif


#if 0
{
   UINT32	Qsize;
   /* each time the counter reaches half a buffer, clean up the approprite queue */
   TransmittedMsduCount[txQid]++;
   whal_apiTxGetAcxQsize(txQid, &Qsize);
   Qsize >>= 1;
   if(TransmittedMsduCount[txQid] >= Qsize)
   {
	   whal_apiTxClean(txQid);
	   TransmittedMsduCount[txQid] = 0;
   }
}
#endif
   return(OK);
}


/*--------------------------------------------------------------*/
/*              Rate Adaptation MACROS                          */
/*--------------------------------------------------------------*/
#define TS_EXCEEDS(currTime,expTime) (currTime > expTime)
#define TS_ADVANCE(currTime,expTime,delta) (expTime = currTime + (delta))

/* These rate adaptation interval constants are in seconds */
#define RATE_ADAPT_CHECK_INTERVAL 1000
#define RATE_ADAPT_INIT_INTERVAL 60000

int gRateAdaptInitEnable=1;

typedef struct TxRateEntry_T
{
	UINT16 flags;
	UINT16 rate;
	UINT16 fallback_threshold; /* in percentage */ 
	UINT16 stepup_threshold;   /* in percentage */
} txRateEntry_T;


#define WDRV_TX_RATE_NUM_SETPUP_PKTS   10 /* The amount of packets to commite the alghorithem after step up.*/ 
#define WDRV_TX_RATE_NUM_CONT_PKTS     30 /* The amount of packets to commite the alghorithem in contiuse.  */ 

/* Tx Rate Modes Constants */
#define TX_RATE_CCK       0
#define TX_RATE_PBCC      1
#define TX_RATE_OFDM54    2
#define TX_RATE_OFDM12    3
#define TX_RATE_OFDM9     4
#define TX_RATE_OFDM6     5
#define TX_RATE_OFDMA     6

/* Tx Rate Flags Constants */
#define TX_RATE_4x_ENA    0x0001
#define TX_RATE_4x_DIS    0x0000

#define TX_NUM_MODE_ENTRIES 7
#define TX_NUM_RATE_ENTRIES 10

#define SU_1 30
#define FB_1 50

#define SU_2 30
#define FB_2 50

#define SU_5_5 30
#define FB_5_5 50

#define SU_6 30
#define FB_6 50

#define SU_9 30
#define FB_9 50

#define SU_11 30
#define FB_11 50

#define SU_12 30
#define FB_12 45

#define SU_18 30
#define FB_18 45

#define SU_22 30
#define FB_22 50

#define SU_24 30
#define FB_24 45

#define SU_36 10
#define FB_36 45

#define SU_48 5
#define FB_48 45

#define SU_54 0
#define FB_54 20


static UINT16 RatesMapMask[TX_NUM_MODE_ENTRIES] =
{
	/* CCK */
	(ACX_RATE_1 | ACX_RATE_2 | ACX_RATE_5_5 | ACX_RATE_11),

	/* PBCC */
	(ACX_RATE_1 | ACX_RATE_2 | ACX_RATE_5_5 | ACX_RATE_11 | ACX_RATE_22),

	/* OFDM54 */
	(ACX_RATE_1 | ACX_RATE_2 | ACX_RATE_5_5 | ACX_RATE_11 | ACX_RATE_18 |
	ACX_RATE_24 | ACX_RATE_36 | ACX_RATE_48 | ACX_RATE_54),

	/* OFDM12 */
	(ACX_RATE_1 | ACX_RATE_2 | ACX_RATE_5_5 | ACX_RATE_11 | ACX_RATE_12),

	/* OFDM9 */
	(ACX_RATE_1 | ACX_RATE_2 | ACX_RATE_5_5 | ACX_RATE_9),

	/* OFDM6 */
	(ACX_RATE_1 | ACX_RATE_2 | ACX_RATE_5_5 | ACX_RATE_6),

	/* OFDMA */
	(ACX_RATE_6 | ACX_RATE_9 | ACX_RATE_12 | ACX_RATE_18 | ACX_RATE_24 | ACX_RATE_36 | ACX_RATE_48 | ACX_RATE_54)

};

static txRateEntry_T RatesMap[TX_NUM_MODE_ENTRIES][TX_NUM_RATE_ENTRIES] =
{
	/* CCK */
	{
	{ TX_RATE_4x_DIS, ACX_RATE_1,   FB_1,   SU_1 },
	{ TX_RATE_4x_DIS, ACX_RATE_2,   FB_2,   SU_2 },
	{ TX_RATE_4x_DIS, ACX_RATE_5_5, FB_5_5, SU_5_5 },
	{ TX_RATE_4x_ENA, ACX_RATE_11,  FB_11,  SU_11 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 }
	},

	/* PBCC */
	
	{
	{ TX_RATE_4x_DIS, ACX_RATE_1,                  FB_1,   SU_1 },
	{ TX_RATE_4x_DIS, ACX_RATE_2,                  FB_2,   SU_2 },
	{ TX_RATE_4x_DIS, ACX_RATE_5_5 | ACX_RATE_PBCC,FB_5_5, SU_5_5 },
	{ TX_RATE_4x_ENA, ACX_RATE_11  | ACX_RATE_PBCC,FB_11,  SU_11 },
	{ TX_RATE_4x_ENA, ACX_RATE_22  | ACX_RATE_PBCC,FB_22,  FB_22 },
	{ 0,           0,                        12, 2 },
	{ 0,           0,                        12, 2 },
	{ 0,           0,                        12, 2 },
	{ 0,           0,                        12, 2 },
	{ 0,           0,                        12, 2 }
	},
	
	/* OFDM54 */
	{
	{ TX_RATE_4x_DIS, ACX_RATE_1,   FB_1,   SU_1 },
	{ TX_RATE_4x_DIS, ACX_RATE_2,   FB_2,   SU_2 },
	{ TX_RATE_4x_DIS, ACX_RATE_5_5, FB_5_5, SU_5_5 },
	{ TX_RATE_4x_ENA, ACX_RATE_11,  FB_11,  SU_11 },
	{ TX_RATE_4x_ENA, ACX_RATE_18,  FB_18,  SU_18 },
	{ TX_RATE_4x_ENA, ACX_RATE_24,  FB_24,  SU_24 },
	{ TX_RATE_4x_ENA, ACX_RATE_36,  FB_36,  SU_36 },
	{ TX_RATE_4x_ENA, ACX_RATE_48,  FB_48,  SU_48 },
	{ TX_RATE_4x_ENA, ACX_RATE_54,  FB_54,  SU_54 },
	{ 0,           0,         12, 2 }
	},

	/* OFDM12 */
	{
	{ TX_RATE_4x_DIS, ACX_RATE_1,   FB_1,   SU_1 },
	{ TX_RATE_4x_DIS, ACX_RATE_2,   FB_2,   SU_2 },
	{ TX_RATE_4x_DIS, ACX_RATE_5_5, FB_5_5, SU_5_5 },
	{ TX_RATE_4x_ENA, ACX_RATE_11,  FB_11,  SU_11 },
	{ TX_RATE_4x_ENA, ACX_RATE_12,  FB_12,  SU_12 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 }
	},

	/* OFDM9 */
	{
	{ TX_RATE_4x_DIS, ACX_RATE_1,   FB_1,   SU_1 },
	{ TX_RATE_4x_DIS, ACX_RATE_2,   FB_2,   SU_2 },
	{ TX_RATE_4x_DIS, ACX_RATE_5_5, FB_5_5, SU_5_5 },
	{ TX_RATE_4x_DIS, ACX_RATE_9,   FB_9,   SU_9 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 }
	},

	/* OFDM6 */
	{
	{ TX_RATE_4x_DIS, ACX_RATE_1,   FB_1,   SU_1 },
	{ TX_RATE_4x_DIS, ACX_RATE_2,   FB_2,   SU_2 },
	{ TX_RATE_4x_DIS, ACX_RATE_5_5, FB_5_5, SU_5_5 },
	{ TX_RATE_4x_DIS, ACX_RATE_6,   FB_6,   SU_6 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 },
	{ 0,           0,         12, 2 }
	},

    /* OFDM A */
    {
    { TX_RATE_4x_DIS, ACX_RATE_6,   FB_6,   SU_6 },
    { TX_RATE_4x_DIS, ACX_RATE_9,   FB_6,   SU_6 },
    { TX_RATE_4x_ENA, ACX_RATE_12,  FB_12,  SU_12 },
    { TX_RATE_4x_ENA, ACX_RATE_18,  FB_18,  SU_18 },
    { TX_RATE_4x_ENA, ACX_RATE_24,  FB_24,  SU_24 },
    { TX_RATE_4x_ENA, ACX_RATE_36,  FB_36,  SU_36 },
    { TX_RATE_4x_ENA, ACX_RATE_48,  FB_48,  SU_48 },
    { TX_RATE_4x_ENA, ACX_RATE_54,  FB_54,  SU_54 },
    { 0,           0,         12, 2 },
    { 0,           0,         12, 2 }


	}
};

/* this function is for testing, called from VxWorks shell */
void tx_SetThresholds(int rate, UINT16 fallback, UINT16 stepup,UINT8 band)
{

	switch (rate)
	{
    
		case 54:
        if(band == RADIO_BAND_5GHZ)
        {
			RatesMap[TX_RATE_OFDMA][7].fallback_threshold = fallback;
			RatesMap[TX_RATE_OFDMA][7].stepup_threshold = stepup;
        }
        else
        {
			RatesMap[TX_RATE_OFDM54][8].fallback_threshold = fallback;
			RatesMap[TX_RATE_OFDM54][8].stepup_threshold = stepup;
        }
			break;

		case 48:
			RatesMap[TX_RATE_OFDM54][7].fallback_threshold = fallback;
			RatesMap[TX_RATE_OFDM54][7].stepup_threshold = stepup;
			break;

		case 36:
			RatesMap[TX_RATE_OFDM54][6].fallback_threshold = fallback;
			RatesMap[TX_RATE_OFDM54][6].stepup_threshold = stepup;
			break;

		case 24:
			RatesMap[TX_RATE_OFDM54][5].fallback_threshold = fallback;
			RatesMap[TX_RATE_OFDM54][5].stepup_threshold = stepup;
			break;

		case 18:
			RatesMap[TX_RATE_OFDM54][4].fallback_threshold = fallback;
			RatesMap[TX_RATE_OFDM54][4].stepup_threshold = stepup;
			break;

		case 12:
			RatesMap[TX_RATE_OFDM12][4].fallback_threshold = fallback;
			RatesMap[TX_RATE_OFDM12][4].stepup_threshold = stepup;
			break;

		case 9:
			RatesMap[TX_RATE_OFDM9][3].fallback_threshold = fallback;
			RatesMap[TX_RATE_OFDM9][3].stepup_threshold = stepup;
			break;

		case 6:
			RatesMap[TX_RATE_OFDM6][3].fallback_threshold = fallback;
			RatesMap[TX_RATE_OFDM6][3].stepup_threshold = stepup;
			break;

		case 22:
			RatesMap[TX_RATE_PBCC][4].fallback_threshold = fallback;
			RatesMap[TX_RATE_PBCC][4].stepup_threshold = stepup;
			break;

		case 11:
			RatesMap[TX_RATE_OFDM54][3].fallback_threshold = fallback;
			RatesMap[TX_RATE_OFDM54][3].stepup_threshold = stepup;
			RatesMap[TX_RATE_PBCC][3].fallback_threshold = fallback;
			RatesMap[TX_RATE_PBCC][3].stepup_threshold = stepup;
			RatesMap[TX_RATE_CCK][3].fallback_threshold = fallback;
			RatesMap[TX_RATE_CCK][3].stepup_threshold = stepup;
			break;

		case 5:
			RatesMap[TX_RATE_OFDM54][2].fallback_threshold = fallback;
			RatesMap[TX_RATE_OFDM54][2].stepup_threshold = stepup;
			RatesMap[TX_RATE_PBCC][2].fallback_threshold = fallback;
			RatesMap[TX_RATE_PBCC][2].stepup_threshold = stepup;
			RatesMap[TX_RATE_CCK][2].fallback_threshold = fallback;
			RatesMap[TX_RATE_CCK][2].stepup_threshold = stepup;
			break;

		case 2:
			RatesMap[TX_RATE_OFDM54][1].fallback_threshold = fallback;
			RatesMap[TX_RATE_OFDM54][1].stepup_threshold = stepup;
			RatesMap[TX_RATE_PBCC][1].fallback_threshold = fallback;
			RatesMap[TX_RATE_PBCC][1].stepup_threshold = stepup;
			RatesMap[TX_RATE_CCK][1].fallback_threshold = fallback;
			RatesMap[TX_RATE_CCK][1].stepup_threshold = stepup;
			break;

		case 1:
			RatesMap[TX_RATE_OFDM54][0].fallback_threshold = fallback;
			RatesMap[TX_RATE_OFDM54][0].stepup_threshold = stepup;
			RatesMap[TX_RATE_PBCC][0].fallback_threshold = fallback;
			RatesMap[TX_RATE_PBCC][0].stepup_threshold = stepup;
			RatesMap[TX_RATE_CCK][0].fallback_threshold = fallback;
			RatesMap[TX_RATE_CCK][0].stepup_threshold = stepup;
			break;

		default:
			printf("wrong rate\n");
			break;
	}
}



/*************************************************************************
 *                        tx_getRateIndexByRate                              *
 *************************************************************************
DESCRIPTION:  Get Rate index of rate  
      
INPUT :   {rate,flags} - the entry that is searched in RatesMap
          mode - rate mode of operation (CCK, PBCC or ODFM)

OUTPUT:   retVal - the first index in which rate is found in RatesMap

RETURN:   OK - rate was found
          NOK - rate was not found
************************************************************************/
static STATUS tx_getRateIndexByRate( UINT16 rate, UINT8 mode, char* retVal )
{
	int index;

	/* Reset the Preamble bit */
	rate &= (~ACX_RATE_PREAMBLE);

	for( index=0; index < TX_NUM_RATE_ENTRIES; index ++ )
	{
		if ( rate == RatesMap[ mode ][ index ].rate )
		{
			*retVal = index;
			return OK;
		}
	}
	
	return NOK;
}

/*************************************************************************
 *                        tx_getStaRate                                  *
 *************************************************************************
DESCRIPTION:  Get the station rate.
      
INPUT :   staIndex - the station index
OUTPUT:     

RETURN:   rate - the station rate
************************************************************************/
static UINT16 tx_getStaRate( UINT16 staIndex, UINT16 *pRateWithFallback)
{
	UINT16 rate;

	rate = RatesMap[ rateAdapTable[staIndex].mode ][ rateAdapTable[staIndex].currRateIndex ].rate;

	if (pRateWithFallback != NULL)
	{
        register UINT16 mask;
		UINT16 acxBitWiseRates;

		wdrv_dbGetStationAcxBitWiseRates(staIndex, &acxBitWiseRates);

		acxBitWiseRates &= RatesMapMask[rateAdapTable[staIndex].mode];

		/* rate includes only one rate represent by one bit (and PBCC if set) */
		/* Generate a mask that will mask rates higher than rate itself, */
		/* used for generating the rateFallback */
		mask = ((rate & ACX_RATE_MASK) << 1) - 1;
		*pRateWithFallback = (acxBitWiseRates & mask) | (rate & (~ACX_RATE_MASK));

		/* in case the rate is 54 or 48, we don't get the lowest rates during fallback,
		   so we drop 18Mbps and 5.5Mbps rates */
		if(rate == ACX_RATE_54)
			*pRateWithFallback &= ~(ACX_RATE_18 | ACX_RATE_5_5);
		else if(rate == ACX_RATE_48)
			*pRateWithFallback &= ~ACX_RATE_18;
	}

	return (rate);
}


/*************************************************************************
 *                        tx_updateTxBrateTimestamp                      *
 *************************************************************************
DESCRIPTION:  Update the Tx B-rate timestamp.
      
INPUT :   rate - the rate to be transmited
OUTPUT:   NONE     

RETURN:   NONE
************************************************************************/
static void tx_updateTxBrateTimestamp(UINT16 rate)
{
	/* Update timestamp if going to transmit a data frame in B/B+ rate */
	/* This is used for the short slot time algorithm */
	rate &= ACX_RATE_ALL_OFDM_MASK;
	if (rate == 0) /* if OFDM rate bits are turned off */
	{
		wdrv_dbSetRxTxBrate(PALSRV_getTick(), 1);
	}
}


/*************************************************************************
 *                        wdrv_txRateAdapSetRate                         *
 *************************************************************************
DESCRIPTION:  Setting a station rate parameters in the rate adaptation table.
      
INPUT :   
OUTPUT:     

RETURN:        
************************************************************************/
STATUS wdrv_txRateAdapSetRate( UINT16 stationIndex, UINT16 rate ){

   UINT8 rateIndex;
   BOOL modeConcat, mixMode4x;

	/* Choose the rate adaptation table to use */
	if (rate & ACX_RATE_ALL_OFDM_MASK & RatesMapMask[TX_RATE_OFDM54])
	{
        UINT8 woneIndex,band;

        wdrv_dbGetStationWoneIndex(stationIndex, &woneIndex);
		if(woneIndex >= MAX_WONE_NET)
		{
			printf("wdrv_txRateAdapSetRate: staIndex=%d, woneIndex=%d\n",stationIndex,woneIndex);
			return NOK; 
		}
        wdrv_dbGetBand(woneIndex,&band);		

        if(band == RADIO_BAND_5GHZ)
        {
            rateAdapTable[stationIndex].mode = TX_RATE_OFDMA;

        }else
        {
			rateAdapTable[stationIndex].mode = TX_RATE_OFDM54;
		}
	}
	else if (rate & ACX_RATE_ALL_OFDM_MASK & RatesMapMask[TX_RATE_OFDM12])
	{
		rateAdapTable[stationIndex].mode = TX_RATE_OFDM12;
	}
	else if (rate & ACX_RATE_ALL_OFDM_MASK & RatesMapMask[TX_RATE_OFDM9])
	{
		rateAdapTable[stationIndex].mode = TX_RATE_OFDM9;
	}
	else if (rate & ACX_RATE_ALL_OFDM_MASK & RatesMapMask[TX_RATE_OFDM6])
	{
		rateAdapTable[stationIndex].mode = TX_RATE_OFDM6;
	}
	else if (rate & ACX_RATE_PBCC)
	{
		rateAdapTable[stationIndex].mode = TX_RATE_PBCC;
	}
	else
	{
		rateAdapTable[stationIndex].mode = TX_RATE_CCK;
	}

   wdrv_dbGetStationConcatSupport( stationIndex, &modeConcat );
   /* If we are in CCK mode and MIX mode is not supported disable 4X */
   wdrv_dbGet4xMixMode(&mixMode4x);
   if((mixMode4x == FALSE )&&( rateAdapTable[stationIndex].mode == TX_RATE_CCK ))
     modeConcat = FALSE; 

    if( modeConcat == TRUE )
    	wdrv_dbSetStationFourXActive(stationIndex, TRUE);
    else
        wdrv_dbSetStationFourXActive(stationIndex, FALSE);
    
   if ( tx_getRateIndexByRate( rate, rateAdapTable[stationIndex].mode, &rateIndex ) == NOK )
      wdrv_utilsReport(1, 1, "Tx - txRateAdapSetRate Error, rate=0x%x, Pbcc=0x%x\n", rate, (rateAdapTable[stationIndex].mode==TX_RATE_PBCC));

   if (REPORT_LEVEL(RL_4X))
	   wdrv_utilsReport(RL_4X, PRI_INFORMATION, "WDRV_4X: Setting Tx rate first time, rate Index is: %d\n", rateIndex);

   rateAdapTable[stationIndex].currRateIndex = rateIndex;
   rateAdapTable[stationIndex].maxRateIndex = rateIndex;

   rateAdapTable[stationIndex].expirTimeTick = PALSRV_getTick() + PALSRV_msec2ticks(RATE_ADAPT_CHECK_INTERVAL);
   rateAdapTable[stationIndex].initExpirTimeTick = PALSRV_getTick() + PALSRV_msec2ticks(RATE_ADAPT_INIT_INTERVAL);
   rateAdapTable[stationIndex].txCount = 0;
   rateAdapTable[stationIndex].txFailCount = 0;
   rateAdapTable[stationIndex].nextRateIndex = 0;
   rateAdapTable[stationIndex].stepUpFlag = FALSE;
   return OK;
}



/*************************************************************************
 *                        wdrv_txRateAdaptFailAnnounce                          *
 *************************************************************************
DESCRIPTION:  This function is called by the HAL when the rate used to send 
              the packet was deferent from the requested rate.
      
INPUT :   
OUTPUT:     

RETURN:        
************************************************************************/
static void wdrv_txRateAdaptFailAnnounce(UINT8 *DestMacAddress, UINT8 ActualTxRate, UINT8 TxStatus)
{
   UINT16  staIndex;

   if( wdrv_dbFindStaIndex(DestMacAddress, &staIndex) == NOK ){
      return; 
   }

   rateAdapTable[staIndex].txFailCount++;
}


/*************************************************************************
 *                        tx_isFallbackPossible                                  *
 *************************************************************************
DESCRIPTION:  Check if can do Fallback (Lower rate is supported)
              If true, update nextRateIndex
      
INPUT :   staIndex - the station index
OUTPUT:     

RETURN:   TRUE  - Stepup is possible
          FALSE - Stepup is not possible
************************************************************************/
static BOOL tx_isFallbackPossible( UINT16 staIndex)
{
	int rateIndex;


	rateIndex = rateAdapTable[staIndex].currRateIndex;
	
	while (rateIndex > 0)
	{
		UINT16 rate;

		rateIndex--;
		rate = RatesMap[rateAdapTable[staIndex].mode][rateIndex].rate;
		if (wdrv_dbIsRateSupported(staIndex, rate))
		{
			rateAdapTable[staIndex].nextRateIndex = rateIndex;

			return TRUE;
		}
	}

	return FALSE;
}


/*************************************************************************
 *                        tx_isStepupPossible                                  *
 *************************************************************************
DESCRIPTION:  Check if can do Stepup (Higher rate is supported)
              If true, update nextRateIndex
			        
INPUT :   staIndex - the station index
OUTPUT:     

RETURN:   TRUE  - Stepup is possible
          FALSE - Stepup is not possible
************************************************************************/
static BOOL tx_isStepupPossible( UINT16 staIndex)
{
	int rateIndex;


	rateIndex = rateAdapTable[staIndex].currRateIndex;
	
	while (rateIndex < rateAdapTable[staIndex].maxRateIndex)
	{
		UINT16 rate;

		rateIndex++;
		rate = RatesMap[rateAdapTable[staIndex].mode][rateIndex].rate;
		if (wdrv_dbIsRateSupported(staIndex, rate))
		{
			rateAdapTable[staIndex].nextRateIndex = rateIndex;
			return TRUE;
		}
	}

	return FALSE;
}


/*************************************************************************
 *                        wdrv_txUpdateRateAdaptation                    *
 *************************************************************************
DESCRIPTION:  
      
INPUT :   
OUTPUT:     

RETURN:        
************************************************************************/
static void wdrv_txUpdateRateAdaptation( UINT16 staIndex )
{
   enum {
      WLAN_DO_NOTHING=0,
      WLAN_FALLBACK,
		WLAN_STEPUP,
		WLAN_RESET_COUNTERS,
		WLAN_INIT
   };

   UINT32  txFailPercent = 0;
   UINT8   action;
   UINT16  capabilities;
   UINT8   pbcc;
   UINT32  timeStamp;
   UINT16  flags, rate;
   UINT16  fallback_threshold, stepup_threshold;
   UINT8   RateAdaptation;
   BOOL    mixMode4x ,Support4x;

   static  UINT32 PreviousTimeStamp = 0;



   wdrv_dbGetRateAdaptation(0,&RateAdaptation);
   if (!RateAdaptation)
   {
       return;
   }

   if (wdrv_dbIsTableIndexExists(staIndex) == FALSE)    /* bad index number */
      return;


   timeStamp = PALSRV_getTick();
   rateAdapTable[staIndex].txCount++;
   action = WLAN_DO_NOTHING;

   /* Verify if the checking time has come. */
   if( TS_EXCEEDS(timeStamp, rateAdapTable[staIndex].initExpirTimeTick) && gRateAdaptInitEnable )
   {
      /* Init the rate adaptation every RATE_ADAPT_INIT_INTERVAL seconds */
	  action = WLAN_INIT;
	  rateAdapTable[staIndex].stepUpFlag = TRUE; /* Flag is on for next time. */
   }
   else if( TS_EXCEEDS(timeStamp, rateAdapTable[staIndex].expirTimeTick) )
   {
      fallback_threshold = RatesMap[ rateAdapTable[staIndex].mode ][ rateAdapTable[staIndex].currRateIndex ].fallback_threshold;
      stepup_threshold = RatesMap[ rateAdapTable[staIndex].mode ][ rateAdapTable[staIndex].currRateIndex ].stepup_threshold;

#if 0 /* Increase thresholds to reduce algorithm sensitivity for vHCF if needed.  */
	  if (wdrv_dbIsVhcfEnabled())
	  {
		  fallback_threshold += 20;
		  stepup_threshold   += 20;
	  }
#endif

      /* If step up has just done wait for 10 packets only. */ 
      if((rateAdapTable[staIndex].stepUpFlag == TRUE) && 
         (rateAdapTable[staIndex].txCount > WDRV_TX_RATE_NUM_SETPUP_PKTS ))
      {
         /* Calculate the packet fail percents. */
         txFailPercent = (rateAdapTable[staIndex].txFailCount * 100) / rateAdapTable[staIndex].txCount;
         

         
         rateAdapTable[staIndex].stepUpFlag = FALSE; /* Flag is off for next time. */
         
		 if((txFailPercent > fallback_threshold)&&(tx_isFallbackPossible(staIndex)))
         {
            action = WLAN_FALLBACK;

         }
         else 
         {
            action = WLAN_RESET_COUNTERS;

         }
      }
      
      else if (rateAdapTable[staIndex].txCount > WDRV_TX_RATE_NUM_CONT_PKTS )
      { 
         /* Calculate the packet fail percents. */
         txFailPercent = (rateAdapTable[staIndex].txFailCount * 100) / rateAdapTable[staIndex].txCount;

         if ((txFailPercent > fallback_threshold)&&(tx_isFallbackPossible(staIndex)))
         {
            action = WLAN_FALLBACK;  
         }
         else if ((txFailPercent < stepup_threshold) && 
                 (tx_isStepupPossible(staIndex)))
         {
             action = WLAN_STEPUP;

             rateAdapTable[staIndex].stepUpFlag = TRUE; /* Flag is on for next time. */

         }
         else 
         {
            action = WLAN_RESET_COUNTERS;
         }
      }
   } /* Time for checking */

   
   if( action == WLAN_DO_NOTHING ){
      return;
   }

   /*--------------------------------------------------------------
              Make the action
     --------------------------------------------------------------*/
	wdrv_dbGetStationCapabilities(staIndex, &capabilities);
	pbcc = capabilities & DOT11_CAPS_PBCC;
	wdrv_dbGet4xMixMode(&mixMode4x);
    wdrv_dbGet4xEnable(&Support4x);


	if ( action == WLAN_FALLBACK )
	{
		rateAdapTable[staIndex].currRateIndex = rateAdapTable[staIndex].nextRateIndex;


        if(Support4x == TRUE)
        {
		flags = RatesMap[rateAdapTable[staIndex].mode][rateAdapTable[staIndex].currRateIndex].flags;
		rate = RatesMap[rateAdapTable[staIndex].mode][rateAdapTable[staIndex].currRateIndex].rate;
		
		if ( !(flags & TX_RATE_4x_ENA) ||
			((rate == ACX_RATE_11) && (mixMode4x == FALSE)) )
		{
			wdrv_dbSetStationFourXActive(staIndex, FALSE);
			if (REPORT_LEVEL(RL_4X))
				wdrv_utilsReport(RL_4X, PRI_INFORMATION, "WDRV_4X: Setting 4X Non Active\n");
		}
		else
		{
			if (REPORT_LEVEL(RL_4X))
				wdrv_utilsReport(RL_4X, PRI_INFORMATION, "WDRV_4X: Setting 4X Active\n");
		}
        }

		if (REPORT_LEVEL(RL_TX))
			wdrv_utilsReport(RL_TX, PRI_INFORMATION,
                "Tx: Station %d  - Rate Fallback to rate %d, bitMap 0x%x Fail percent %d timeStamp %d timeStampInterval %d\n",
                staIndex, rateAdapTable[staIndex].currRateIndex, tx_getStaRate( staIndex, NULL), txFailPercent,timeStamp,timeStamp - PreviousTimeStamp);
		
	}

	else if ( action == WLAN_STEPUP )
	{
		rateAdapTable[staIndex].currRateIndex = rateAdapTable[staIndex].nextRateIndex;

        if(Support4x == TRUE)
        {
		flags = RatesMap[rateAdapTable[staIndex].mode][rateAdapTable[staIndex].currRateIndex].flags;
		rate = RatesMap[rateAdapTable[staIndex].mode][rateAdapTable[staIndex].currRateIndex].rate;
		if ( (flags & TX_RATE_4x_ENA) &&
			((rate != ACX_RATE_11) || (mixMode4x == TRUE)) )
		{
			wdrv_dbSetStationFourXActive(staIndex, TRUE);
			if (REPORT_LEVEL(RL_4X))
				wdrv_utilsReport(RL_4X, PRI_INFORMATION, "WDRV_4X: Setting 4X Active\n");
		}
		else
		{
			if (REPORT_LEVEL(RL_4X))
				wdrv_utilsReport(RL_4X, PRI_INFORMATION, "WDRV_4X: Setting 4X Non Active\n");
		}
        }

		if (REPORT_LEVEL(RL_TX))
			wdrv_utilsReport(RL_TX, PRI_INFORMATION,
             "Tx: Station %d  - Rate Step Up to rate %d, bitMap 0x%x Fail percent %d timeStamp %d timeStampInterval %d\n",
             staIndex, rateAdapTable[staIndex].currRateIndex, tx_getStaRate( staIndex, NULL), txFailPercent,timeStamp,timeStamp- PreviousTimeStamp);

        
	}

	else if ( action == WLAN_INIT )
	{
		rateAdapTable[staIndex].currRateIndex = rateAdapTable[staIndex].maxRateIndex;
		
      flags = RatesMap[rateAdapTable[staIndex].mode][rateAdapTable[staIndex].currRateIndex].flags;
		rate = RatesMap[rateAdapTable[staIndex].mode][rateAdapTable[staIndex].currRateIndex].rate;
		if ( (flags & TX_RATE_4x_ENA) &&
			((rate != ACX_RATE_11) || (mixMode4x == TRUE)) )
		{
			wdrv_dbSetStationFourXActive(staIndex, TRUE);
			if (REPORT_LEVEL(RL_4X))
				wdrv_utilsReport(RL_4X, PRI_INFORMATION, "WDRV_4X: Setting 4X Active\n");
		}
		else
		{
			if (REPORT_LEVEL(RL_4X))
				wdrv_utilsReport(RL_4X, PRI_INFORMATION, "WDRV_4X: Setting 4X Non Active\n");
		}


		if (REPORT_LEVEL(RL_TX))
			wdrv_utilsReport(RL_TX, PRI_INFORMATION,
                "Tx: Station %d  - Rate Init to rate %d, bitMap 0x%x timeStamp %d timeStampInterval %d\n",
                staIndex, rateAdapTable[staIndex].currRateIndex, tx_getStaRate( staIndex, NULL),timeStamp,timeStamp- PreviousTimeStamp);


		
		TS_ADVANCE(timeStamp, rateAdapTable[staIndex].initExpirTimeTick, PALSRV_msec2ticks(RATE_ADAPT_INIT_INTERVAL));
	}

   /*--------------------------------------------------------------
       action != WLAN_DO_NOTHING, we need to reset counters.
     --------------------------------------------------------------*/
   rateAdapTable[staIndex].txCount = 0;
   rateAdapTable[staIndex].txFailCount = 0;
   TS_ADVANCE( timeStamp, rateAdapTable[staIndex].expirTimeTick,
      (rateAdapTable[staIndex].stepUpFlag? PALSRV_msec2ticks(50) : PALSRV_msec2ticks(RATE_ADAPT_CHECK_INTERVAL))); 

   PreviousTimeStamp = timeStamp;

}


/*************************************************************************
 *                        wdrv_txPrintRateAdaptation                     *
 *************************************************************************
DESCRIPTION:  
      
INPUT :   
OUTPUT:     

RETURN:        
************************************************************************/
int wdrv_txPrintRateAdaptation( char *buf )
{
   int i ,len = 0;
   BOOL	sta4xActive;

   len +=  sprintf(buf + len, "\n");
   len +=  sprintf(buf + len, "         Rate Adaptation Table         ");
   len +=  sprintf(buf + len, "---------------------------------------");
   len +=  sprintf(buf + len, " STA     Tx     Fail     Rate     PBCC     Max_Rate	Active");

   for( i=0 ; i < MAX_STATIONS ; i++ )
   {
      if( wdrv_mgmtIsStaAssoc(i) )
	  {
		   wdrv_dbGetStationFourXActive(i, &sta4xActive); 
         len += sprintf(buf + len, "%4d   %4d   %6d   %6d   %6d     %6d	%d", i, rateAdapTable[i].txCount,
         rateAdapTable[i].txFailCount, rateAdapTable[i].currRateIndex,
         (rateAdapTable[i].mode==TX_RATE_PBCC), rateAdapTable[i].maxRateIndex, sta4xActive);
      }
   }

   return len;
}







/*************************************************************************
 *                        wdrv_txPrintStatistics                         *
 *************************************************************************
DESCRIPTION:  
      
INPUT :   
OUTPUT:     

RETURN:        
************************************************************************/
int wdrv_txPrintStatistics( char *buf )
{
   UINT8 woneIndex ,numSupWone;
   int len = 0;

   wdrv_dbGetNumSupWone(&numSupWone);

   len += sprintf(buf + len,"---------------------------- Tx Statistics -----------------------\n");
   len += sprintf(buf + len,"Unicast MSDUs  Sent        : %10u \n", Tx_Statistics.unicastSent);
   len += sprintf(buf + len,"Unicast Pkts Sent          : %10u \n", Tx_Statistics.concatPackets+Tx_Statistics.unConcatPackets);
   len += sprintf(buf + len,"Broadcasts Sent            : %10u \n", Tx_Statistics.brcstSent  );
   len += sprintf(buf + len,"Concat Pkts Sent           : %10u \n", Tx_Statistics.concatPackets );
   len += sprintf(buf + len,"Non Concat Pkts Sent       : %10u \n", Tx_Statistics.unConcatPackets );  
   len += sprintf(buf + len,"Uincast    Alloc Fails     : %10u \n", Tx_Statistics.unicAllocFail ); 
   len += sprintf(buf + len,"Broadcasts Alloc Fails     : %10u \n", Tx_Statistics.brcstAllocFail );   
   len += sprintf(buf + len,"Acx Queue Underflow        : %10u \n", Tx_Statistics.acxQueUnderflow );
   len += sprintf(buf + len,"Low  Priority Sent         : %10u \n", Tx_Statistics.lowPrioSent );
   len += sprintf(buf + len,"High Priority Sent         : %10u \n", Tx_Statistics.highPrioSent );
   len += sprintf(buf + len,"Unicast Queue              : %10u \n", Tx_Statistics.unicastQsent );
   len += sprintf(buf + len,"Broadcast Queue                    : %10u \n", Tx_Statistics.broadcastQsent );
   len += sprintf(buf + len,"Ps-Poll events                     : %10u \n", Tx_Statistics.psPollEvents );
   len += sprintf(buf + len,"Ps-Wake events                     : %10u \n", Tx_Statistics.psWakeEvents );
   len += sprintf(buf + len,"Ps Uincast    Alloc Fails          : %10d \n", Tx_Statistics.psunicAllocFail ); 
   len += sprintf(buf + len,"Ps Prev Fail Send                  : %10d \n", Tx_Statistics.psprevFailSend ); 
   len += sprintf(buf + len,"PS Packets Sent                    : %10u \n", Tx_Statistics.sentPsPackets );
   for (woneIndex=0; woneIndex < numSupWone ; woneIndex++)
      len += sprintf(buf + len,"Max Env in High Queue[%d]          : %10u \n",woneIndex, wdrv_txEnvListGetMaxStored( highPrioWaitEnvList[woneIndex] ));
   for (woneIndex=0; woneIndex < numSupWone ; woneIndex++)
      len += sprintf(buf + len,"Max Env in Low  Queue[%d]          : %10u \n",woneIndex, wdrv_txEnvListGetMaxStored( lowPrioWaitEnvList[woneIndex] ));
   for (woneIndex=0; woneIndex < numSupWone ; woneIndex++)
      len += sprintf(buf + len,"Pkts in High  Queue[%d]            : %10u \n",woneIndex, wdrv_txEnvListGetCurrentStored( highPrioWaitEnvList[woneIndex] ));
   for (woneIndex=0; woneIndex < numSupWone ; woneIndex++)
      len += sprintf(buf + len,"Pkts in Low   Queue[%d]            : %10u \n",woneIndex, wdrv_txEnvListGetCurrentStored( lowPrioWaitEnvList[woneIndex] ));
   for (woneIndex=0; woneIndex < numSupWone ; woneIndex++)
      len += sprintf(buf + len,"Overload packet in Low   Queue[%d] : %10u \n",woneIndex, overloadWaitPktFromEther[woneIndex] );


   len += sprintf(buf + len,"Pkts in Mngt Free Queue    : %10u \n", wdrv_txEnvListGetCurrentStored( freeMngtEnvList ));
   len += sprintf(buf + len,"Pkts in Data Free Queue    : %10u \n", wdrv_txEnvListGetCurrentStored( freeDataEnvList ));
   len += sprintf(buf + len,"-----------------------------------------------------------------");
	
   return len;
}


/*************************************************************************
 *                        wdrv_txEnable                                  *
 *************************************************************************
DESCRIPTION:  
      
INPUT :   
OUTPUT:     

RETURN:        
************************************************************************/
STATUS wdrv_txEnable(void)
{
#if 1
	/* enable TX_DATA interrupt */
	whal_apiEnableTxDataIntr();
	/* set TX_DATA user callback */
   	whal_apiSetTxDataUserCB( (TxDataUser_Callback_T)wdrv_txCheckWaitPcktsFromIntr );
#endif
	return OK;
}


/*************************************************************************
 *                        wdrv_txGetTxMulticastCount                     *
 *************************************************************************
DESCRIPTION:  
      
INPUT :   
OUTPUT:     

RETURN:        
************************************************************************/
void wdrv_txGetTxMulticastCount(UINT32 *counter)
{
	*counter = Tx_Statistics.brcstSent;
}

/*************************************************************************
 *                        wdrv_txSetConcatStatus                         *
 *************************************************************************
DESCRIPTION:  Setting the concatenation mode.
      
INPUT :   
OUTPUT:     

RETURN:        
************************************************************************/
void wdrv_txSetConcatStatus( UINT8 status )
{
	UINT8 numSupWone;

    concatOn = status;

    if( concatOn )
	{
        maxConcatSize = WLAN_MAX_CONCAT_SIZE;
		wdrv_dbGetNumSupWone(&numSupWone);
		if((numSupWone > 1) && (maxConcatSize > WONE_CONCAT_MAX_SIZE))
			maxConcatSize = WONE_CONCAT_MAX_SIZE;
	}
    else
        maxConcatSize =  MAX_FRAG_THRESHOLD;
}


/*************************************************************************
 *                        wdrv_txSetMaxConcatSize                        *
 *************************************************************************
DESCRIPTION:  Setting the max concatenation size.
      
INPUT :   
OUTPUT:     

RETURN:        
************************************************************************/
void wdrv_txSetMaxConcatSize( UINT32 size )
{
    maxConcatSize = size;
}


/*************************************************************************
 *                        wdrv_txGetMaxConcatSize                        *
 *************************************************************************
DESCRIPTION:  Setting the max concatenation size.
      
INPUT :   
OUTPUT:     

RETURN:        
************************************************************************/
int wdrv_txGetMaxConcatSize( void )
{
    return (int)maxConcatSize;
}

void wdrv_txSetConcatDebug(BOOL mode)
{
	concatDbg = mode;
}


/*************************************************************************
 *                        vhcfClassifyTxPacket                           *
 *************************************************************************
DESCRIPTION:  If packet matches vHCF classification, mark it as high priority.
      
INPUT :   MSDU-Pointer, Tx-Flags-Pointer
OUTPUT:   Update Tx-Flags (MSDU priority).

RETURN:        
************************************************************************/
static void vhcfClassifyTxPacket(mem_MSDU_T *msduPtr, ULONG * txFlags)
{
   UINT8	*pDot11Header;
   UINT8    *pWlanSnapHeader;
   UINT8	*pIpHeader;
   UINT16	firstBdLength = 0;
   UINT32	firstBddataOffset = 0;   
   int		ipHeaderLen;
   register UINT idx;
   tVHCF_PARAMS  vhcfParams;


   if( (*txFlags) & WDRV_TX_BRCST_MTCST_MSDU )  /* If it's a broadcast MSDU, exit. */
	   return;

   wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF configuration. */

   if( !vhcfParams.vhcfEnable )  /* If vHCF is disabled, exit. */
	   return;
	   
	/* Prepare packet attributes:
      ========================== */

   firstBdLength = memMgr_BufLength(msduPtr->firstBDPtr);
   firstBddataOffset = memMgr_BufOffset(msduPtr->firstBDPtr);
   pDot11Header = memMgr_BufData(msduPtr->firstBDPtr) + firstBddataOffset;
   pWlanSnapHeader = pDot11Header + WLAN_HDR_LEN;

   if ( ((Wlan_LlcHeader_T*)pWlanSnapHeader)->Type != ntohs(IP_PROTOCOL_NUMBER) )
   {
	   return;    /* If it's not an IP packet, exit. */	   
   }

   if (firstBdLength == WLAN_HDR_LEN + WLAN_SNAP_HDR_LEN)  /* Packet from Ethernet: */
   {
	  UINT32	secondBddataOffset;

      if(msduPtr->firstBDPtr->nextBDPtr != NULL)  /* If there is no data, exit. */
	  {
		  secondBddataOffset = memMgr_BufOffset(msduPtr->firstBDPtr->nextBDPtr);
		  pIpHeader = memMgr_BufData(msduPtr->firstBDPtr->nextBDPtr) + secondBddataOffset;
	  }
	  else
		  return;
   }
   else   /* Packet from Wlan: */
      pIpHeader = pDot11Header + WLAN_HDR_LEN + WLAN_SNAP_HDR_LEN;

   ipHeaderLen = ((*(unsigned char*)pIpHeader & 0x0F) * 4);

   /* If the packet matches a classifier, mark it as high-priority:
      ============================================================= */

   for(idx = 0; idx < MAX_VHCF_CLASSIFIERS ; idx++)  /* Go over the classifiers. */
   {
	   UINT8	protocol;
	   UINT32	destIp;
	   UINT16	destPort;

	  if (vhcfParams.vhcfClassifier[idx].Protocol == VHCF_PROT_NONE)
	  {
		  continue;  /* If classifier is not used, continue to the next one. */
	  }		  
	  
	  /* Packet attributes: */

		protocol = * (unsigned char*) ((ULONG)pIpHeader + (ULONG)IP_PROTOCOL_FIELD);
		destIp   = * (unsigned long*) (pIpHeader + IP_DEST_ADDRESS_FIELD);
		destPort = * (UINT16*) (pIpHeader + ipHeaderLen + DEST_PORT_FIELD);

	  if ( ((protocol == TCP_PROTOCOL) && (vhcfParams.vhcfClassifier[idx].Protocol == VHCF_PROT_TCP))
	    || ((protocol == UDP_PROTOCOL) && (vhcfParams.vhcfClassifier[idx].Protocol == VHCF_PROT_UDP)) )
	  {
		  if ( (  vhcfParams.vhcfClassifier[idx].DestIp   == ntohl(destIp)) &&
			   ( (vhcfParams.vhcfClassifier[idx].DestPort == ntohs(destPort)) ||
			     (vhcfParams.vhcfClassifier[idx].DestPort == 0       ) ) ) /* if port is 0 ignore it */
		  {
			  /* Packet matches current classifier - set high-priority and vHCF flags and exit. */
			  *txFlags &= ~((ULONG)WDRV_TX_LOW_PRIORITY);
			  *txFlags |= WDRV_TX_HIGH_PRIORITY | WDRV_TX_VHCF_MSDU;
			  return;
 		  }
	  }
   }

   return;
}

void wdrv_txSetPrivacyInvoked(BOOL value)
{
	txPrivacyInvoked = value;
}

UINT16 tx_getStaActiveRate(UINT16 staIndex)
{
	if (staIndex < MAX_STATIONS)
		return RatesMap[rateAdapTable[staIndex].mode][rateAdapTable[staIndex].currRateIndex].rate;
	
	return 0xFFFF;
}

