/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*																		   */
/*		MODULE:	txInit.h												   */
/*    PURPOSE:	Tx module init functions	 							   */
/*																		   */
/***************************************************************************/
#ifndef _TX_INIT_H_
#define _TX_INIT_H_

#include "pform_types.h"
#include "envList.h"
#include "pal.h"

/* Tx Flags */
#define   WDRV_TX_HIGH_PRIORITY      0x01
#define   WDRV_TX_LOW_PRIORITY       0x02
#define   WDRV_TX_UNICAST_MSDU       0x04
#define   WDRV_TX_BRCST_MTCST_MSDU   0x08
#define   WDRV_TX_VHCF_MSDU			 0x10
#define   WDRV_TX_FROM_ETHER		 0x20
#define   WDRV_TX_FORCE_UN_ENCRYPTED 0x40
#define   WDRV_TX_PS_NOT_ALLOWED	 0x80
#define   WDRV_TX_EAPOL_FRAME		 0x100

typedef enum {
   WDRV_TX_MSDU_TIMEOUT,
   WDRV_TX_MSDU_SEND,
   WDRV_TX_COMPLETE,
   WDRV_TX_MSDU_PS_POLL,
   WDRV_TX_PS_AGING,
   WDRV_TX_MSDU_PS_STATUS_CHANGE,
   WDRV_TX_MSDU_PS_CLEAN_STATION_LIST,
} TxMsg_T;


typedef struct {
   TxMsg_T type;  /* Type of message (from the list above) */
   union {
      long MessageInfo[3]; /* queue-message size */

      /* Ps message types. 802.11(Ps-poll, Ps-wake) */
      struct {
         long  staIndex;
         long  status /* reserve1 */;
         long  woneIndex;
      } psMsg;
   } msg;
} TxMessage_T;


/*******************************************************************/
/*                    Testing routines                             */ 
/*******************************************************************/
void wdrv_txSendMsduTest( void );


/*******************************************************************/


/*************************************************************************
 *                        wdrv_txInit                                    *
 *************************************************************************
DESCRIPTION: This function initializes the Tx module.                 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:      OK : Initiation succesfull.
             NOK: Initiation unsuccesfull.                                                                                                                                             
************************************************************************/
STATUS wdrv_txInit(void);

/*************************************************************************
 *                        wdrv_txUnload                                  *
 *************************************************************************
DESCRIPTION: This function deinitializes the Tx module.                 
                                                                                                   
INPUT:       

OUTPUT:      

RETURN:     

************************************************************************/
void wdrv_txUnload(void);

/*************************************************************************
 *                        wdrv_txSendMsdu                                *
 *************************************************************************
DESCRIPTION: This prepares the msdu to be sent. It takes an empty envelop              
             from the free list, put the msdu in to it, and insert the 
             envelop to the waiting list.
INPUT:       msduPtr : Pointer to the MSDU to send.
             priority : The Tx priority. (High \ Low) 
             woneIndex : wone index

OUTPUT:      

RETURN:      OK : MSDU has been inserted to the waiting list.
             NOK: Couldn't insert the MSDU to the waiting list.                                                                                                                                             
************************************************************************/
STATUS wdrv_txSendMsdu( mem_MSDU_T *msduPtr, UINT16 stationIndex, 
                       ULONG txFlags, UINT8 woneIndex);


/*************************************************************************
 *                        wdrv_txCheckWaitPckts                          *
 *************************************************************************
DESCRIPTION: This function sends ps-poll message.

INPUT :      staIndex : The index of the station in the main db table.
OUTPUT:      

RETURN:      OK:                                                                                                                                            
************************************************************************/
void wdrv_txSendPsPollMsg( UINT32 staIndex );

/*************************************************************************
 *                        wdrv_txSendPsWakeMsg                           *
 *************************************************************************
DESCRIPTION: This function sends ps-wake message.

INPUT :      staIndex : The index of the station in the main db table.
OUTPUT:      

RETURN:      OK:                                                                                                                                            
************************************************************************/
void wdrv_txSendPsWakeMsg( UINT32 staIndex );

/*************************************************************************
 *                        wdrv_txCheckWaitPckts                          *
 *************************************************************************
DESCRIPTION: This function verifies if there are and waiting packets in 
             the waiting list, if there are, send a TIMEOUT message to the
             Tx task.

OUTPUT:      

RETURN:      OK:                                                                                                                                            
************************************************************************/
void wdrv_txCheckWaitPckts (void );
void wdrv_txCheckWaitPcktsFromIntr (void);
void wdrv_txWaitListCheckTiemrCB (void );

/*************************************************************************
 *                        wdrv_txFreeEnv                                 *
 *************************************************************************
DESCRIPTION: This function free the envelop's buffers and put it in the
             free envelops list.

INPUT :      envelopPtr : The envelop to free.
OUTPUT:      

RETURN:      OK:                                                                                                                                            
************************************************************************/
void wdrv_txFreeEnv( TxEnvelop_T **envelopPtr );

/*************************************************************************
 *                        wdrv_txEnable                                  *
 *************************************************************************
DESCRIPTION: 
             

INPUT :      
OUTPUT:      

RETURN:      OK:                                                                                                                                            
************************************************************************/ 
STATUS wdrv_txEnable(void);

/*************************************************************************
 *                        wdrv_txSendPSAgingTimerCB                      *
 *************************************************************************
DESCRIPTION: This function send a message to the Tx task to commit a PS
             Aging.

INPUT :     
OUTPUT:      

RETURN:                                                                                                                                             
************************************************************************/
void wdrv_txSendPSAgingTimerCB( UINT32 data );

/*************************************************************************
 *                        wdrv_txSendPsChangeStateMsg                    *
 *************************************************************************
DESCRIPTION: This function sends ps-Status Change message.

INPUT :      staIndex : The index of the station in the main db table.
OUTPUT:      

RETURN:      OK:                                                                                                                                            
************************************************************************/
void wdrv_txSendPsChangeStateMsg( UINT32 staIndex ,UINT16 status, UINT8 woneIndex);

/****************************************************************************
 *                      wdrv_txSendPsCleanStaListMsg()                       *
 ****************************************************************************
 * DESCRIPTION: Clean all the packets stored in a station power save list 
 *              and set the PS status to FALSE, used for disconnecting a 
 *              staion from the AP.
 * INPUTS: stationIndex - The station Index
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	None
 ****************************************************************************/
void  wdrv_txSendPsCleanStaListMsg(UINT16 stationIndex, UINT8 woneIdx) ;

/*************************************************************************
 *                        wdrv_txRateAdapSetRate                         *
 *************************************************************************
DESCRIPTION:  Setting a station rate parameters in the rate adaptation table.
      
INPUT :   
OUTPUT:     

RETURN:        
************************************************************************/
STATUS wdrv_txRateAdapSetRate( UINT16 stationIndex, UINT16 rate);


/*************************************************************************
 *                        wdrv_txSetConcatStatus                         *
 *************************************************************************
DESCRIPTION:  Setting the concatenation mode.
      
INPUT :   
OUTPUT:     

RETURN:        
************************************************************************/
void wdrv_txSetConcatStatus( UINT8 status );

/*************************************************************************
 *                        wdrv_txSetMaxConcatSize                        *
 *************************************************************************
DESCRIPTION:  Getting the max concatenation size.
      
INPUT :   
OUTPUT:     

RETURN:        
************************************************************************/
void wdrv_txSetMaxConcatSize( UINT32 size );


/*************************************************************************
 *                        wdrv_tx_getEvent		                         *
 *************************************************************************
DESCRIPTION:  Called by DDA upon TX event reception
      
INPUT :   
OUTPUT:     

RETURN:        
************************************************************************/
STATUS wdrv_tx_getEvent(TxMessage_T *pTxMessage);


/*************************************************************************
 *                        wdrv_txGetMaxConcatSize                        *
 *************************************************************************
DESCRIPTION:  Setting the max concatenation size.
      
INPUT :   
OUTPUT:     

RETURN:        
************************************************************************/
int wdrv_txGetMaxConcatSize( void );

void wdrv_txGetTxMulticastCount(UINT32 *counter);


/* 4X functions. */
void wdrv_txSetConcatDebug(BOOL value);


/* Statistics */
int  wdrv_txPrintStatistics(char *buf);
int  wdrv_txPrintRateAdaptation( char *buf);

void wdrv_txSetPrivacyInvoked(BOOL value);

UINT16 tx_getStaActiveRate(UINT16 staIndex);

#endif
