/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  acx_hw.c
 *   PURPOSE: Handle all board specific issues for the acx100
 *
 ****************************************************************************/

#define _ACX_HW_C_

#ifndef __KERNEL__
#include "tnetd53xx.h"
#define OPTION_TXWDT_ANTI_BUBBLE 1
#include "sysapi.h"
#endif

#include "acx_hw.h"
#include "vlynq.h"
#include "pal.h"
#define	TNETD53XX_GPIO_BASE				0x08610900
#define TNETD53XX_GPIO_DOUTR	 			(TNETD53XX_GPIO_BASE+0x04)
#define TNETD53XX_GPIO_PDIRR	 		  (TNETD53XX_GPIO_BASE+0x0008)
#define TNET_PUMA_GPIO_DOUTR_SCC_RINGIN3	0x00002000
#define TNET_PUMA_GPIO_PDIRR_SCC_RINGIN3	0x00002000

/*****************
 Static Variables
 *****************/
static UINT32 vlynqErrorCount = 0;

static int boardInitVlynqDriverAcx111(void);

/*************************
 Endian Swapping routines
 *************************/
#ifdef USE_ENDIAN_SWAP_ROUTINES

typedef union ttSwap32
{
   UINT32 u;
   unsigned char b[sizeof(UINT32)];
} tSwap32;

typedef union ttSwap16
{
   UINT16 u;
   unsigned char b[sizeof(UINT16)];
} tSwap16;

UINT32 endian_handle_u32(UINT32 x)
{
   register tSwap32 t,m;
   t.u = x;
   m.b[0] = t.b[3];
   m.b[1] = t.b[2];
   m.b[2] = t.b[1];
   m.b[3] = t.b[0];
   return m.u;
}/* END endian_handle_u32() */

UINT16 endian_handle_u16(UINT16 x)
{
   register tSwap16 t,m;
   t.u = x;
   m.b[0] = t.b[1];
   m.b[1] = t.b[0];
   return m.u;
}/* END endian_handle_u16() */

#endif /* USE_ENDIAN_SWAP_ROUTINES */

/***********
 * CARDBUS *
 ***********/
/*
 * ACX100 addresses
 * ----------------
 * chip select and timing configuration
 */
#define BOARD_ACX100_REG_ADDRESS		(TNETD53XX_CS3_BASE | K1BASE)  
#define BOARD_ACX100_MEM_ADDRESS		(TNETD53XX_CS3_BASE | 0x400000 |K1BASE )  

/* ACX100 chip select configuration registers */
/* 
 * Base+0x14: Async Device CS3 - Avalanche CS3 
 * -------------------------------------------
   0  0  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  1  S  S  reset value 
  |31|30|29|28|27|26|25|24|23|22|21|20|19|18|17|16|15|14|13|12|11|10| 9| 8| 7| 6| 5| 4| 3| 2| 1| 0| 
     |EW| Wsu       | Wstrobe         | Whold  | Rsu       | Rstrobe         | Rhold  | TA  |ASize| 

  ExtWait(EW) -Extend ASYNC cycles based on external WAIT signal 
  Wsu     -Cycles where the address, data, byte enables, and chip select are driven prior to the Write Strobe. 
  Wstrobe -Cycles where the MWE is held low 
  Whold   -Cycles where the address, data, byte enables and chip select is held after the Write strobe deassertion 
  Rsu     -Cycles where the address, byte enables, and chip select are driven prior to the Read Strobe and enable. 
  Rstrobe -Cycles where the MRE is held low 
  Rhold   -Cycles where the address, byte enables and chip select are held after the strobe deassertion 
  TA      -Cycles where the bus is held hiz between cycles of differing types. Async Read to Async Read to the 
           same device does not incur this delay. 
  ASize   -Bus width (0=8, 1=16, 2=32, 3=64) 64 bit mode is reserved for the future 
*/
#define CPU_CS3_ADDR			(PHYS_TO_K1(TNETD53XX_MEMC_ASYNC_CS3_REG))
#define CS3_WSU      2  /*  8ns */
#define CS3_WSTROBE  8  /* 64ns */
#define CS3_WHD      2  /*  8ns */
#define CS3_RSU      1  /*  8ns */
#define CS3_RSTROBE  8  /* 64ns */
#define CS3_RHD      1  /*  8ns */
#define CS3_TA       1  /*  8ns */
#define CS3_SIZE     1  /* 16bit*/

/* set value to 0x60c44e25 */
#define CPU_CS3_CONFIG  0x40000000 | \
	((CS3_WSU<<26)|(CS3_WSTROBE<<20)|(CS3_WHD<<17) \
	|(CS3_RSU<<13)|(CS3_RSTROBE<< 7)|(CS3_RHD<< 4)|(CS3_TA<<2)|CS3_SIZE)

/* 
 * Base+0x04: External Wait Control and Timout 
 * -------------------------------------------
    1  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0  1  0  0  0  0  0  0  0  reset value 
  |31|30|29|28|27|26|25|24|23|22|21|20|19|18|17|16|15|14|13|12|11|10| 9| 8| 7| 6| 5| 4| 3| 2| 1| 0| 
     |WP|                                                                 |   Max_External_Wait   | 

  WaitPolarity:      The Polarity of the external WAIT pin: 1=wait if external pin is one, and 0=wait if pin is zero. 
  Max_External_Wait: Number of 16 clock periods + 1 The memory controller will wait for an extended ASYNC cycle 
                     before the cycle is terminated and the AT (AsyncTimedout) bit is set. 
*/
#define CPU_XTRL_WAIT_CTRL			(PHYS_TO_K1(TNETD53XX_MEMC_XTL_CNTRL_REG))
#define MAX_EXTERNAL_ASYNC_WAIT_CYCLES	0x80

/*********
 * VLYNQ *
 *********/
/*
 * ACX111 addresses
 * ----------------
 * VLYNQ Portal Configuration
 */
#define VLYNQ_BOARD_ACX111_REG_ADDRESS		PHYS_TO_K1(VLYNQ_BOARD_ACX_PORTAL_BASE_ADDR + VLYNQ_ACX111_MEM_SIZE)
#define VLYNQ_BOARD_ACX111_MEM_ADDRESS		PHYS_TO_K1(VLYNQ_BOARD_ACX_PORTAL_BASE_ADDR)


/****************************************************************************
 *                      report_acx111_vlynq()
 ****************************************************************************
 * DESCRIPTION:	VLYNQ driver error reporter.
 * 
 * INPUTS:
 *  VLYNQ_DEV    *pDev,         - This VLYNQ
 *  eVLYNQ_DVC    aSrcDvc,      - Who casused the event, local or remote?
 *  UINT32        dwStatRegVal  - Value of the relevant status register
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	None.
 *
 ****************************************************************************/
static void report_acx111_vlynq
(
   VLYNQ_DEV    *pDev,     /* This VLYNQ */
   eVLYNQ_DVC    aSrcDvc,  /* Who casused the event, local or remote? */
   UINT32        dwStatRegVal    /* Value of the relevant status register */
)
{   
   vlynqErrorCount++;
}

UINT32 get_acx111_vlynq_errorCount(void)
{
	return vlynqErrorCount;
}

void set_acx111_vlynq_errorCount(UINT32 val)
{
	vlynqErrorCount = val;
}

/****************************************************************************
 *                      reset_acx111_vlynq()
 ****************************************************************************
 * DESCRIPTION:	VLYNQ driver reset callback.
 * 
 * INPUTS:
 *  VLYNQ_DEV    *pDev,         - This VLYNQ
 *  eVLYNQ_RESET_MODE mode      - mode
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	None.
 *
 ****************************************************************************/
void reset_acx111_vlynq( VLYNQ_DEV *pDev, eVLYNQ_RESET_MODE mode)
{
   volatile UINT32 *ptrPDat;
   unsigned long flags;

   /*unsigned int board;

   board = ReadBoardType();*/
   ptrPDat  = (UINT32*)PHYS_TO_K1(TNETD53XX_GPIO_DOUTR);

   if( VLYNQ_RESET_ASSERT == mode)
   {
      volatile UINT32 *ptrPDir;
      ptrPDir  = (UINT32*)PHYS_TO_K1(TNETD53XX_GPIO_PDIRR);

      puts("reset_acx111_vlynq() -- VLYNQ_RESET_ASSERT");
      /* RESET, don't let other tasks to interfere. */
      PAL_intrLock(flags); /* lock interrupts */
      *ptrPDir &= ~TNET_PUMA_GPIO_PDIRR_SCC_RINGIN3;   /* enable GPIO13 */
      if(0/*(WA1130V == board) || (WA123AG == board)*/)
      {
         *ptrPDat |= TNET_PUMA_GPIO_DOUTR_SCC_RINGIN3;   /* put it on '1' */
      }
      else
      {
         *ptrPDat &= ~TNET_PUMA_GPIO_DOUTR_SCC_RINGIN3;   /* put it on '0' */
      }
      PAL_intrUnLock(flags);   /* unlock interrupts */

      /* Hold GPIO13 in '0' for a few ticks */
      PAL_sleepLong(40);
   }
   else if( VLYNQ_RESET_DEASSERT == mode)
   {
      /* RELEASE RESET. */
      puts("reset_acx111_vlynq() -- VLYNQ_RESET_DEASSERT");
      PAL_intrLock(flags);
      if(0/* (WA1130V == board) || (WA123AG == board) */)
      {
         *ptrPDat &= ~TNET_PUMA_GPIO_DOUTR_SCC_RINGIN3; /* put GPIO13 on '0' */
      }
      else
      {
         *ptrPDat |= TNET_PUMA_GPIO_DOUTR_SCC_RINGIN3; /* put GPIO13 on '1' */
      }
      PAL_intrUnLock(flags);

      /* Wait minimum for the remote device. 0.5msec should be enough for ACX. */
      PAL_sleepLong(40);
   }
   else if( VLYNQ_RESET_INITFAIL == mode)
   {
      /* Initilization failure. Hold remote device in reset state. */
      puts("reset_acx111_vlynq() -- VLYNQ_RESET_INITFAIL, LOCK REMOTE");
      PAL_intrLock(flags); /* lock interrupts */
      if(0/* (WA1130V == board) || (WA123AG == board)*/ )
      {
         *ptrPDat |= TNET_PUMA_GPIO_DOUTR_SCC_RINGIN3;
      }
      else
      {
         *ptrPDat &= ~TNET_PUMA_GPIO_DOUTR_SCC_RINGIN3;
      }
      PAL_intrUnLock(flags);   /* unlock interrupts */
   }
   else if( VLYNQ_RESET_LINKESTABLISH == mode)
   {
      /* Called if link established, boost VLYNQ clock  */
      puts("reset_acx111_vlynq() -- VLYNQ_RESET_LINKESTABLISH, boosting clock");
      if(0/* (WA1130V == board) || (WA123AG == board) */)
      {
         pDev->clk_div = ACX111_VLYNQ_FINAL_CLOCK_DIV_REMOTE;
      }
      else
      {
         pDev->clk_div = ACX111_VLYNQ_FINAL_CLOCK_DIV;
      }
      PAL_sleepLong(40);
   }
   else if( VLYNQ_RESET_INITFAIL2 == mode)
   {
      puts("reset_acx111_vlynq() -- VLYNQ_RESET_INITFAIL2");
   }
   else if (VLYNQ_RESET_INITOK == mode)
   {
      puts("reset_acx111_vlynq() -- INITOK");
      PAL_sleepLong(40);
      
   }
   else if( VLYNQ_RESET_UNLOAD == mode)
   {
      puts("reset_acx111_vlynq() -- UNLOAD");
      /* TODO: Disable all the driver clients safely. */
   }
   else if( VLYNQ_RESET_ONFINALRELEASE == mode)
   {
      /* Free resources after VLYNQ device is dead */
      PAL_memFree( 0,glb_pBusDrv,0);
      glb_pBusDrv = NULL;
   }
} /* END reset_acx111_vlynq() */

int boardUnloadAcx111(void)
{
   int status = OK;
   
   if( glb_pBusDrv != NULL)
   {
      VLYNQ_DEV *pDev;
      pDev = (VLYNQ_DEV *)glb_pBusDrv;
      status = tnetd53xx_vlynq_Unload( pDev);
   }

   return status;
}

/****************************************************************************
 *                      boardInitAcx111Addr()
 ****************************************************************************
 * DESCRIPTION:	Return the ACX111 memory and register addresses 
 * 
 * INPUTS:	UINT32 *RegAddr, UINT32 *MemAddr - pointers for results.
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK if successful or error code if fail
 *
 ****************************************************************************/
int boardInitAcx111Addr(UINT32 *RegAddr, UINT32 *MemAddr)
{
   /* Init the VLYNQ module, only once, report error if failed */
   if( glb_pBusDrv == NULL)
   {
      int r;

      r = boardInitVlynqDriverAcx111();
      if( TNETD53XX_ERR_OK != r)
      {
         /* If fail, free allocated system resources. */
         if( glb_pBusDrv != NULL)
         {
            PAL_memFree( 0,glb_pBusDrv,0);
            glb_pBusDrv = NULL;
         }
         printf("boardInitAcx111Addr() - VLYNQ driver initialization failure code %d\n",r);
         return r;
      }
   }

   /* Save results if possible, get here only if OK */
   if( (RegAddr != NULL) && (MemAddr != NULL))
   {
	   *RegAddr = VLYNQ_BOARD_ACX111_REG_ADDRESS;
	   *MemAddr = VLYNQ_BOARD_ACX111_MEM_ADDRESS;
      return OK;
   }

   return TNETD53XX_ERR_ERROR;
}

int boardInitAcx111Intr(INT_HANDLER whalIntrHandler)
{
   VLYNQ_DEV *pDev = (VLYNQ_DEV *)glb_pBusDrv;
   int retCode = TNETD53XX_ERR_OK;

   if( (pDev == NULL) || (pDev->state != VLYNQ_DRV_STATE_RUN))
      retCode = TNETD53XX_ERR_DEVICE_NOT_READY;

   if( retCode == TNETD53XX_ERR_OK)
      retCode = tnetd53xx_vlynq_Install_ISR(
         pDev, 0, VLYNQ_REMOTE_07,
         VLYNQ_INTR_ACTIVE_HIGH, VLYNQ_INTR_LEVEL,
         (VLYNQ_INTR_CNTRL_ISR)whalIntrHandler,
         NULL);
   /* ACX writes 0x80 in INTPSR to cause HINT IRQ */

   if( retCode == TNETD53XX_ERR_OK)
      retCode = tnetd53xx_vlynq_IRQ_Enable( pDev, 0, 1);

   return retCode;
}


/****************************************************************************
 *                      boardInitVlynqDriver()
 ****************************************************************************
 * DESCRIPTION:	Configure VLYNQ driver for accessing the ACX.
 * 
 * INPUTS:	global void * glb_pBusDrv.
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK if successful, else error.
 *
 ****************************************************************************/
static int boardInitVlynqDriverAcx111(void)
{
   VLYNQ_DEV *pDev;
   int retCode;
#if 0   
   unsigned int board;

   board = ReadBoardType();
#endif

   /* allocate memory */
   glb_pBusDrv = PAL_memAlloc(0, sizeof(VLYNQ_DEV),0,0);
   if( glb_pBusDrv == NULL)
      return TNETD53XX_ERR_ALLOC_FAIL;
   pDev = (VLYNQ_DEV *)glb_pBusDrv;

   pDev->state = VLYNQ_DRV_STATE_UNINIT;
   pDev->pIntrCB = NULL;

   /* Fill the driver structure, according to the ACX type */
   pDev->dev_idx = 1;                           /* the one and only VLYNQ module */
   if(0/* (WA1130V == board) || (WA123AG == board) */)
   {
      pDev->clk_source = VLYNQ_CLK_SOURCE_REMOTE;  /* clock coming out from ACX */
   }
   else
   {
      pDev->clk_source = VLYNQ_CLK_SOURCE_LOCAL;   /* clock coming out from local   */
   }
   pDev->clk_div = ACX111_VLYNQ_INITIAL_CLOCK_DIV;  /* start with 125/4MHz=31.25MHz clock */
   /* local portal */
   pDev->local_mem.TxMap = VLYNQ_BOARD_ACX111_MEM_ADDRESS;  /* remote window at local */
   pDev->remote_mem.RxOffset[0]  =  VLYNQ_ACX111_MEM_OFFSET;   /* ACX memory region */
   pDev->remote_mem.RxSize[0]    =  VLYNQ_ACX111_MEM_SIZE  ;
   pDev->remote_mem.RxOffset[1]  =  VLYNQ_ACX111_REG_OFFSET;   /* ACX registers region */
   pDev->remote_mem.RxSize[1]    =  VLYNQ_ACX111_REG_SIZE  ;
   /* remote portal */
   pDev->remote_mem.TxMap        =  0 /* ACX111_VL1_REMOTE_BASE*/ ; /* Host SDRAM */
   pDev->local_mem.RxOffset[0]   =  EVM3_SDRAM_MEM_ADDR;    /* Physical address of SDRAM start */
   pDev->local_mem.RxSize[0]     =  ACX111_VL1_REMOTE_SIZE; /* map all host SDRAM */
   /* callbacks */
   pDev->reset_cb    = reset_acx111_vlynq;
   pDev->report_cb   = report_acx111_vlynq;

   /* Init the VLYNQ driver */
   retCode = tnetd53xx_vlynq_Init( pDev, ACX111_VLYNQ_INIT_OPT   );
   logMsg("boardInitVlynqDriverAcx111() -- return code = %d,"
      " driver state = %d\n", retCode, pDev->state,0,0,0,0);

   return retCode;
}

/*********
 * PCI   *
 *********/


/*************************
 * Common initialization *
 *************************/

#ifndef __linux__

/****************************************************************************
 *                      boardInitAcx100Addr()
 ****************************************************************************
 * DESCRIPTION:	Configure memory controller, returning the ACX100
 *                Memory and register addresses
 * 
 * INPUTS:	UINT32 *RegAddr, UINT32 *MemAddr - pointers for results.
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK if successful, else error.
 *
 ****************************************************************************/
int boardInitAcx100Addr(UINT32 *RegAddr, UINT32 *MemAddr)
{
   /* Initialize bus interface */
	*(volatile UINT32 *)CPU_CS3_ADDR = CPU_CS3_CONFIG;
	*(volatile UINT32 *)CPU_XTRL_WAIT_CTRL = MAX_EXTERNAL_ASYNC_WAIT_CYCLES;

   /* return results */
   if( (RegAddr != NULL) && (MemAddr != NULL))
   {
	   *RegAddr = BOARD_ACX100_REG_ADDRESS;
	   *MemAddr = BOARD_ACX100_MEM_ADDRESS;
      return OK;
   }

   return TNETD53XX_ERR_ERROR;
}


/****************************************************************************
 *                      boardInitAcx100Addr()
 ****************************************************************************
 * DESCRIPTION:	Registers and enables HINT innterrupt 
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK if success, else error
 *
 ****************************************************************************/
int boardInitAcx100Intr(INT_HANDLER whalIntrHandler)
{
	/*
	 * Enable interrupt #1 in the Avalanch 
	 * Convert Bubble(docsis) to ACX100 because both use interrupt #1 
	 */
	return sysAntiBubbleIntInit((typ_USR_TxWdt_ISR)whalIntrHandler);
}

#if (WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)

/****************************************************************************
 *                      boardInitAcx111Addr()
 ****************************************************************************
 * DESCRIPTION:	Return the ACX111 memory and register addresses 
 * 
 * INPUTS:	UINT32 *RegAddr, UINT32 *MemAddr - pointers for results.
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK if successful or error code if fail
 *
 ****************************************************************************/
int boardInitAcx111Addr(UINT32 *RegAddr, UINT32 *MemAddr)
{
   /* Init the VLYNQ module, only once, report error if failed */
   if( glb_pBusDrv == NULL)
   {
      int r;

      r = boardInitVlynqDriverAcx111();
      if( TNETD53XX_ERR_OK != r)
      {
         /* If fail, free allocated system resources. */
         if( glb_pBusDrv != NULL)
         {
            PAL_memFree( 0,glb_pBusDrv,0);
            glb_pBusDrv = NULL;
         }
         printf("boardInitAcx111Addr() - VLYNQ driver initialization failure code %d\n",r);
         return r;
      }
   }

   /* Save results if possible, get here only if OK */
   if( (RegAddr != NULL) && (MemAddr != NULL))
   {
	   *RegAddr = VLYNQ_BOARD_ACX111_REG_ADDRESS;
	   *MemAddr = VLYNQ_BOARD_ACX111_MEM_ADDRESS;
      return OK;
   }

   return TNETD53XX_ERR_ERROR;
}

/****************************************************************************
 *                      boardInitAcx111Addr()
 ****************************************************************************
 * DESCRIPTION:	Registers and enables HINT innterrupt 
 * 
 * INPUTS:	INT_HANDLER whalIntrHandler
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK if successful or error code if fail
 *
 ****************************************************************************/
int boardInitAcx111Intr(INT_HANDLER whalIntrHandler)
{
   VLYNQ_DEV *pDev = (VLYNQ_DEV *)glb_pBusDrv;
   int retCode = TNETD53XX_ERR_OK;

   if( (pDev == NULL) || (pDev->state != VLYNQ_DRV_STATE_RUN))
      retCode = TNETD53XX_ERR_DEVICE_NOT_READY;

   if( retCode == TNETD53XX_ERR_OK)
      retCode = tnetd53xx_vlynq_Install_ISR(
         pDev, 0, VLYNQ_REMOTE_07,
         VLYNQ_INTR_ACTIVE_HIGH, VLYNQ_INTR_LEVEL,
         (VLYNQ_INTR_CNTRL_ISR)whalIntrHandler,
         NULL);
   /* ACX writes 0x80 in INTPSR to cause HINT IRQ */

   if( retCode == TNETD53XX_ERR_OK)
      retCode = tnetd53xx_vlynq_IRQ_Enable( pDev, 0, 1);

   return retCode;
}

/****************************************************************************
 *                      boardUnloadAcx111()
 ****************************************************************************
 * DESCRIPTION:	Unloads ACX111 driver
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK if successful or error code if fail
 *
 ****************************************************************************/
int boardUnloadAcx111(void)
{
   int status = OK;
   
   if( glb_pBusDrv != NULL)
   {
      VLYNQ_DEV *pDev;
      pDev = (VLYNQ_DEV *)glb_pBusDrv;
      status = tnetd53xx_vlynq_Unload( pDev);
   }

   return status;
}


#elif (WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)

/****************************************************************************
 *                      board_pci_find_acx111()
 ****************************************************************************
 * DESCRIPTION:	Find the first ACX111 and return details in pAcx.
 * 
 * INPUTS:	int from (= 1), tPCI_ACX111_DEV_INFO *pAcx
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	Returns OK or fail if not found or HW error.
 *
 ****************************************************************************/
static int board_pci_find_acx111( int from, tPCI_ACX111_DEV_INFO *pAcx)
{
   extern   PCI_DEVICE_T *ptr_pci_interface;
   int status = NOK;
   UINT16 acx111_id = ACX111_PCI_DEVICE_ID;
   UINT16 TI_id     = PCIIF_PCI_VENDOR_ID;
   int i;
   PCI_DEVICE_T *ptrDev;

   /* SLOT#0 is the PCI bridge, so do not bother looking in it */
   if( (from <= 0) ||
       (from >= PCI_MAX_DEVICES_SUPPORTED) ||
       (ptr_pci_interface == NULL) )
   {
      return status;
   }

   /* look for the device */
   for( i = from ; i < PCI_MAX_DEVICES_SUPPORTED; i++)
   {
      ptrDev = &ptr_pci_interface[i];

      if( (ptrDev->device == acx111_id) && (ptrDev->vendor == TI_id) )
      {
         /* fill ACX device structure in relevant information */
         pAcx->slotNum = ptrDev->devfn;
         pAcx->devID   = ptrDev->device;
         pAcx->venID   = ptrDev->vendor;
         pAcx->MemAddr = ptrDev->resource[1].base_address;
         pAcx->MemSize = ptrDev->resource[1].size;
         pAcx->RegAddr = ptrDev->resource[0].base_address;
         pAcx->RegSize = ptrDev->resource[0].size;

         /* Exit with status OK */
         status = OK;
         break;
      }
   }

   return status;
}

/****************************************************************************
 *                      boardInitAcx111Addr()
 ****************************************************************************
 * DESCRIPTION:	Return the ACX111 memory and register addresses 
 * 
 * INPUTS:	UINT32 *RegAddr, UINT32 *MemAddr - pointers for results.
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK if successful or error code if fail
 *
 ****************************************************************************/

/* ACX PCI controller */
#define ACX111_PCI_REGCTL_MASTER_ON_VAL     0x02100047
#define ACX111_PCI_REGBAR_MASTER_ON_VAL     0x00001080

#define ACX111_PCI_REGCTL_MASTER_OFF_VAL    0x02100003
#define ACX111_PCI_REGBAR_MASTER_OFF_VAL    0x00000000

int boardInitAcx111Addr(UINT32 *RegAddr, UINT32 *MemAddr)
{
   int status = OK;
   tPCI_ACX111_DEV_INFO *pPciDev = (tPCI_ACX111_DEV_INFO *)glb_pBusDrv;

   /* Init the PCI bus only once */
   if( glb_pBusDrv == NULL)
   {
      glb_pBusDrv = calloc( 1, sizeof( tPCI_ACX111_DEV_INFO));
      if( glb_pBusDrv == NULL)
         return TNETD53XX_ERR_ALLOC_FAIL;

      pPciDev = (tPCI_ACX111_DEV_INFO *)glb_pBusDrv;
	   pci_initialize(TNETD53XX_PCIC_BASE, 0, 0);
      status  = board_pci_find_acx111( 1, pPciDev);

      /* enable ACX PCI bus master support to access Ava-I. */
      if( OK == status)
         status = pci_write_config_dword( 0, pPciDev->slotNum, 0x04, ACX111_PCI_REGCTL_MASTER_ON_VAL);
      if( OK == status)
         status = pci_write_config_dword( 0, pPciDev->slotNum, 0x1c, ACX111_PCI_REGBAR_MASTER_ON_VAL);

      if( OK != status)
      {
         /* If fail, free allocated system resources. */
         if( glb_pBusDrv != NULL)
         {
            PAL_memFree( 0,glb_pBusDrv,0);
            glb_pBusDrv = NULL;
         }
         logMsg("boardInitAcx111Addr() - PCI driver initialization failure\n",
            0,0,0,0,0,0);
         return status;
      }
   }

   /* Save results if possible, status = OK and driver is valid */
   if( (RegAddr != NULL) && (MemAddr != NULL))
   {
	   *RegAddr = pPciDev->RegAddr;
	   *MemAddr = pPciDev->MemAddr;
   }

   return status;
}

/****************************************************************************
 *                      boardInitAcx111Addr()
 ****************************************************************************
 * DESCRIPTION:	Registers and enables HINT innterrupt 
 * 
 * INPUTS:	INT_HANDLER whalIntrHandler
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK if successful or error code if fail
 *
 ****************************************************************************/
int boardInitAcx111Intr(INT_HANDLER whalIntrHandler)
{
   int status = OK;

  /* TODO: Connect all PCI IRQs to a single generic PCI routine
     in a similar manner to VLYNQ. Pass PCI_DEVICE_T* as argument.
   sysHwIntConnect();
   sysHwPCIEnable();  called by the PCI driver,
   thus connecting the ISRs must be before PCI init or by a callback means.
   */
   if( glb_pBusDrv != NULL)
   {
 	   sysHwIntConnect(
         (TNETD53XX_INTR_CNTRL_SRC_PRIMARY | TNETD53XX_INTR_CNTRL_SRC_PCIA),
		   (VOIDFUNCPTR)whalIntrHandler, (void*)glb_pBusDrv);
	   sysHwIntConnect(
         (TNETD53XX_INTR_CNTRL_SRC_PRIMARY | TNETD53XX_INTR_CNTRL_SRC_PCIC),
		   (VOIDFUNCPTR)whalIntrHandler, (void*)glb_pBusDrv);
   }
   else
   {
      status = NOK;
   }

   return status;
}

/****************************************************************************
 *                      boardUnloadAcx111()
 ****************************************************************************
 * DESCRIPTION:	Unloads ACX111 driver
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK if successful or error code if fail
 *
 ****************************************************************************/

/* TNETW1130 */
#define ACX_ECPU_CTRL_REG                   0x0108
#define ACX_REG_SLV_END_CTL					  0x0020
#define ACX_REG_SLV_SOFT_RESET              0x0000
#define ACX_EE_START_REG                    0x0100

int boardUnloadAcx111(void)
{
   int status = OK;
   if( glb_pBusDrv != NULL)
   {
      tPCI_ACX111_DEV_INFO *pPciDev = (tPCI_ACX111_DEV_INFO *)glb_pBusDrv;
   
      /* Close PCI interrupts (global to all devices) */
      sysHwPCIDisable();

      /* Disable Host memory PCI window  */
      pciif_pci_disable_local_window();

      /* Inhibit ACX from mastering the PCI bus. */
      pci_write_config_dword( 0, pPciDev->slotNum, 0x04, ACX111_PCI_REGCTL_MASTER_OFF_VAL);
      pci_write_config_dword( 0, pPciDev->slotNum, 0x1c, ACX111_PCI_REGBAR_MASTER_OFF_VAL);

      /* Halt the ACX device, and reset its eCPU */
      whal_acxRegWrite( ACX_ECPU_CTRL_REG, 1);
      whal_acxRegWrite( ACX_REG_SLV_SOFT_RESET, 1);
      rt_osTimerWakeAfter(4);
      whal_acxRegWrite( ACX_REG_SLV_SOFT_RESET, 0);
      rt_osTimerWakeAfter(4);

      /* Now, boot the ACX from serial EEPROM. In PCI Serial EEPROM must be valid. */
      whal_acxRegWrite( ACX_EE_START_REG, 1);
      rt_osTimerWakeAfter(4);

      /* Reinitialize PCI controller, reenumerting all devices */
      pciif_pci_reinit_driver();

      /* Free up system resources, and invalidate the bus driver. */
      PAL_memFree( 0,glb_pBusDrv,0);
      glb_pBusDrv = NULL;
   }

   return status;
}


#endif /* WLAN_BUS_TYPE */

#endif /* __linux */

#undef _ACX_HW_C_

