/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  acx_hw.c
 *   PURPOSE: Handle all board specific issues for the acx100
 *
 ****************************************************************************/

#ifndef _ACX_HW_H
#define _ACX_HW_H

#include "pal.h"
#include "../../../../include/cy_codepattern.h"

#define EVM3_SDRAM_MEM_ADDR       0x14000000

/*
 * Compilation definition, see config.mk.
 * --------------------------------------
 */
#define   WLAN_BUS_TYPE_CARDBUS  1
#define   WLAN_BUS_TYPE_VLYNQ    2
#define   WLAN_BUS_TYPE_PCI      3

#define   WLAN_ADAPTOR_VERSION_1100  1100
#define   WLAN_ADAPTOR_VERSION_1130  1130

#define   WLAN_SLAVE_MODE   1
#define   WLAN_MASTER_MODE  2

#ifndef WLAN_BUS_TYPE
#error  WLAN_BUS_TYPE must be defined
#if 0	/* clean up warning */
/*#else
#if (WLAN_BUS_TYPE == WLAN_BUS_TYPE_CARDBUS)
#warning WLAN Bus Type is CardBus
#elif (WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)
#warning WLAN Bus Type is VLYNQ
#elif (WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)
#warning WLAN Bus Type is PCI
#else
#error Check WLAN_BUS_TYPE definition.
#endif*/
#endif /* 0 */
#endif

#ifndef WLAN_ADAPTOR_VERSION
#error  WLAN_ADAPTOR_VERSION must be defined
#if 0	/* clean up warning */
/*#else
#if (WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1100)
#warning WLAN Adaptor is set to TNETW1100 (ACX100)
#elif (WLAN_ADAPTOR_VERSION == WLAN_ADAPTOR_VERSION_1130)
#warning WLAN Adaptor is set to TNETW1130 (ACX111)
#else
#error  Check WLAN_ADAPTOR_VERSION definition
#endif*/
#endif /* 0 */
#endif

#ifndef WLAN_ADAPTOR_WORK_MODE
#error  WLAN_ADAPTOR_WORK_MODE must be defined
#if 0	/* clean up warning */
/*#else
#if (WLAN_ADAPTOR_WORK_MODE == WLAN_SLAVE_MODE)
#warning WLAN Adaptor work mode is Generic Slave
#elif (WLAN_ADAPTOR_WORK_MODE == WLAN_MASTER_MODE)
#warning WLAN Adaptor work mode is Master Mode with indirect host memory organization
#else
#error  Check WLAN_ADAPTOR_WORK_MODE definition
#endif*/
#endif /* 0 */
#endif


/*
 * Bus driver, can be any bus driver.
 * ---------------------------------------
 */
#ifdef _ACX_HW_C_
void *glb_pBusDrv = NULL;
#else
extern void *glb_pBusDrv;
#endif

/*
 * VLYNQ Stuff
 * -----------
 */
/* VLYNQ testing */
#define VLYNQ_REMOTE 0  /* Set this flag to 1 to compile remote simulator */
#define VLYNQ_LOCAL  1  /* Set this flag to 1 to compile local  simulator */
#if( (VLYNQ_REMOTE) && (VLYNQ_LOCAL))
#error You cannot set both VLYNQ_LOCAL and VLYNQ_REMOTE ***
#endif

#define VLYNQ_BOARD_ACX_PORTAL_BASE_ADDR  0x04000000  /* PHYS_ADDR of VLYNQ Portal  */

/*
 * ACX111 definitions
 * ------------------
 */
#define VLYNQ_ACX111_MEM_OFFSET     0xC0000000  /* Physical address of ACX111 memory */
#define VLYNQ_ACX111_MEM_SIZE       0x00040000  /* Total size of the ACX111 memory   */
#define VLYNQ_ACX111_SRAM_SIZE      0x0001E000  /* ACX111 SRAM size memory 40kB ZWS + 80kB */
#define VLYNQ_ACX111_REG_OFFSET     0xF0000000  /* PHYS_ADDR of ACX111 control registers   */
#define VLYNQ_ACX111_REG_SIZE       0x00022000  /* Size of ACX111 registers area, MAC+PHY  */
#define ACX111_VL1_REMOTE_BASE      0xD0000000  /* Host Window for ACX111 (Master Mode) */

#ifndef __linux__
#error "no define linux"
#define ACX111_VL1_REMOTE_SIZE      ReadRamSize() /* Give unlimited access in this version */
#else 
//#define ACX111_VL1_REMOTE_SIZE 0x1000000

#if (DRAM_SIZE == 32)
  #define ACX111_VL1_REMOTE_SIZE 0x2000000
#elif (DRAM_SIZE == 16)
#error "is 16M now"
  #define ACX111_VL1_REMOTE_SIZE 0x1000000
#endif

#endif

#ifndef __linux__

#define ACX111_VLYNQ_INIT_OPT    \
     (VLYNQ_OPTION_INIT_HOST_REGIONS   |  \
      VLYNQ_OPTION_INIT_REMOTE_REGIONS |  \
      VLYNQ_OPTION_INIT_INTERRUPTS     |  \
      VLYNQ_OPTION_INIT_CHECK_LINK     |  \
      VLYNQ_OPTION_INIT_CONFIG_CLK     |  \
      VLYNQ_OPTION_INIT_CLEAR_ERRORS)

#else
#define ACX111_VLYNQ_INIT_OPT    \
     (VLYNQ_OPTION_INIT_HOST_REGIONS   |  \
      VLYNQ_OPTION_INIT_REMOTE_REGIONS |  \
      VLYNQ_OPTION_INIT_CHECK_LINK     |  \
      VLYNQ_OPTION_INIT_CONFIG_CLK     |  \
      VLYNQ_OPTION_INIT_CLEAR_ERRORS)

#endif /* __linux */

/* Clock division factors range[1..8] */
#define ACX111_VLYNQ_INITIAL_CLOCK_DIV       4
#define ACX111_VLYNQ_FINAL_CLOCK_DIV         2  /* clock src is host CPU */
#define ACX111_VLYNQ_FINAL_CLOCK_DIV_REMOTE  1  /* clock src is ACX */

/* Conversion from pointer to HOST CPU SDRAM that is used by the HOST CPU,
   into a pointer that the ACX111 eCPU can use to access the same memory cell.
   DMA CTL registers MUST (!) be setup before using these macros or before running
   the FW.
 */
#define VLYNQ_HOST2ACX111_SDRAM_PTR(ptr)     (PHYS_ADDR( (UINT32)(ptr) ))
/* Conversion from pointer to HOST CPU SDRAM that is used by the ACX111 eCPU,
   into a pointer that the HOST CPU can use to access the same memory cell.
   The pointer is to uncached memory or cached.
 */
#define VLYNQ_ACX111_TO_HOST_SDRAM_PTR_PHYS(ptr)     \
  (PHYS_ADDR((UINT32)(ptr)))
#define VLYNQ_ACX111_TO_HOST_SDRAM_PTR_CACHED(ptr)   \
  (PHYS_TO_K0(VLYNQ_ACX111_TO_HOST_SDRAM_PTR_PHYS(ptr)))
#define VLYNQ_ACX111_TO_HOST_SDRAM_PTR_UNCACHED(ptr) \
  (PHYS_TO_K1(VLYNQ_ACX111_TO_HOST_SDRAM_PTR_PHYS(ptr)))


/* Conversion from pointer to HOST CPU SDRAM that is used by the HOST CPU,
   into a pointer that the ACX111 eCPU can use to access the same memory cell.
   DMA CTL registers MUST (!) be setup before using these macros or before running
   the FW. Host CPU PCI slave base adderss = 0x00000000, 16MB window.
 */
#define PCI_HOST2ACX111_SDRAM_PTR(ptr)     (PHYS_ADDR( (UINT32)(ptr) ))
/* Conversion from pointer to HOST CPU SDRAM that is used by the ACX111 eCPU,
   into a pointer that the HOST CPU can use to access the same memory cell.
   The pointer is to uncached memory or cached.
 */
#define PCI_ACX111_TO_HOST_SDRAM_PTR_PHYS(ptr)     \
  (PHYS_ADDR((UINT32)(ptr)))
#define PCI_ACX111_TO_HOST_SDRAM_PTR_CACHED(ptr)   \
  (PHYS_TO_K0(PCI_ACX111_TO_HOST_SDRAM_PTR_PHYS(ptr)))
#define PCI_ACX111_TO_HOST_SDRAM_PTR_UNCACHED(ptr) \
  (PHYS_TO_K1(PCI_ACX111_TO_HOST_SDRAM_PTR_PHYS(ptr)))

/*
 * HW access flags settings according to WLAN bus type
 * ---------------------------------------------------
 */
#if (WLAN_BUS_TYPE == WLAN_BUS_TYPE_CARDBUS)
/*#warning Cardbus is using endian swap*/
#define USE_ENDIAN_SWAP 1
#elif (WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)
/*#warning VLYNQ is using endian swap*/
#define USE_ENDIAN_SWAP 0
#elif (WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)
/*#warning PCI is using endian swap*/
#define USE_ENDIAN_SWAP 0
#endif   /* WLAN_BUS_TYPE  */


/*
 * Endian macros
 * -------------
 */
#define LO_BYTE(x)			((x) & 0xff)		
#define HI_BYTE(x)			(((x) >> 8) & 0xff)
#define LO_WORD(x)			((x) & 0xffff)		
#define HI_WORD(x)			(((x) >> 16) & 0xffff)

#if USE_ENDIAN_SWAP
#if( (WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI) || (WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ))
/* Make sure to read from the device only once. */
#define USE_ENDIAN_SWAP_ROUTINES
/*extern  UINT16 endian_handle_u16(UINT16 x);
extern  UINT32 endian_handle_u32(UINT32 x);
#define ENDIAN_HANDLE_WORD(x)	endian_handle_u16((UINT16)(x))
#define ENDIAN_HANDLE_LONG(x) endian_handle_u32((UINT32)(x))
*/
#else
/*
#undef  USE_ENDIAN_SWAP_ROUTINES
#define ENDIAN_HANDLE_WORD(x)	((LO_BYTE(x) << 8) | HI_BYTE(x))
#define ENDIAN_HANDLE_LONG(x) ((ENDIAN_HANDLE_WORD(LO_WORD(x)) << 16) | (ENDIAN_HANDLE_WORD(HI_WORD(x))))
*/
#endif /* WLAN_BUS_TYPE */

#else
/*#define ENDIAN_HANDLE_WORD(x) (x)
#define ENDIAN_HANDLE_LONG(x) (x)
*/
#endif

typedef	void (*INT_HANDLER)(void *);
#define MAKE_UNCACHED_PTR(ptr)   (PHYS_TO_K1(ptr))


/* TNETW1130 */
#if 0
int boardInitAcx111Addr(UINT32 *RegAddr, UINT32 *MemAddr);
int boardInitAcx111Intr(INT_HANDLER whalIntrHandler);
int boardUnloadAcx111(void);
UINT32 get_acx111_vlynq_errorCount(void);
#endif // 0
#endif   /* _ACX_HW_H */


