/*******************************************************************************************
*	tnetd53xx.h		SHARED TYPEDEFs, CONSTs, and API for TNETD53xx
********************************************************************************************
*	PLEASE, READ THE NOTES BELOW TO BETTER UNDERSTAND HOW THIS FILE IS STRUCTURED.
*
*	These are const, typedef, and api definitions for Avalanche.
*
*	NOTES:
*	1. This file may be included into both C and Assembly files.
*		- for .s files, please do #define _ASMLANGUAGE in your ASM file to
*			avoid C data types (typedefs) below;
*		- for .c files, you don't have to do anything special.
*
*	2. This file has a number of sections for each Avalanch sub-system. When adding
*		a new constant, find the subsystem you are working on and follow the
*		name pattern. If you are adding another typedef for your interface, please,
*		place it with other typedefs and function prototypes.
*
*	3. Please, DO NOT add any macros or types that are local to a subsystem to avoid
*		cluttering. Include such items directly into the module's .c file or have a
*		local .h file to pass data between smaller modules. This file defines only
*		shared items.
******************************************************************************************** */

#ifndef ___TNETD53XX_H___
#define ___TNETD53XX_H___

/* Define the following to use workarounds for PG1.2 bugs in the hardware */
#define	TNETD53XX_PG12_BUGS

//#include "hswpform.h"
//#include "tnetd53xxerr.h"

//#include "hardware.h"
//#include "tnetd53xx_emac.h"
//#include "tnetd53xx_vlynq.h"

/******************************************************************************************** 
*
* NOTE: Below are the Physical Addresses (PA) for the various devices, to access these devices
*       without an MMU you need to either access it from KSEG0 or KSEG1 (add 8000:0000 or
*       A000:0000).  KSEG1 is preferred for since it is uncached memory. 
*
*		 In C, use the macros PHYS_TO_K1 or PHYS_TO_K0, as needed.       
*
*********************************************************************************************/


/*******************************************************************************************
*  Clock relationships
******************************************************************************************** */
#define TNETD53XX_VBUS_FREQ					(TNETD53XX_MIPS_FREQ/2) /* This is defined in the board definitions */


/*#define	TNETD53XX_MIPS_FREQ_DEFAULT			25000000		*//* MHz */

/*******************************************************************************************
*	Internal SRAM
******************************************************************************************** */

/*******************************************************************************************
*	Boardband I/F space 0
******************************************************************************************** */
#define TNETD53XX_BBIF_SPACE0_BASE             0x01000000

/*******************************************************************************************
*	Boardband I/F space 1
******************************************************************************************** */
#define TNETD53XX_BBIF_SPACE1_BASE             0x01800000

/*******************************************************************************************
*	Boardband I/F control space
******************************************************************************************** */
#define TNETD53XX_BBIF_BASE			0x02000000

#define TNETD53XX_BBIF_CTRL_REG                        (TNETD53XX_BBIF_BASE+0x0000)
#define TNETD53XX_BBIF_CTRL_MODE_MASK				0x00000003
#define	TNETD53XX_BBIF_CTRL_DISABLE				0x00000000
#define	TNETD53XX_BBIF_CTRL_LBT4040_INT				0x00000001
#define	TNETD53XX_BBIF_CTRL_XBUS				0x00000002
#define	TNETD53XX_BBIF_CTRL_LBT4040_EXT				0x00000003

#define TNETD53XX_BBIF_ADDR_XLAT0_REG 			(TNETD53XX_BBIF_BASE+0x0004)
#define TNETD53XX_BBIF_ADDR_XLAT0_EXTMEMBASE			0xff000000

#define TNETD53XX_BBIF_ADDR_XLAT1_REG 			(TNETD53XX_BBIF_BASE+0x0008)
#define TNETD53XX_BBIF_ADDR_XLAT1_EXTMEMBASE			0xff000000

#define TNETD53XX_BBIF_XBUS_BURST_LEN_REG 		(TNETD53XX_BBIF_BASE+0x000c)
#define TNETD53XX_BBIF_XBUS_BURST_LEN_BLEN			0x000007f8

#define TNETD53XX_BBIF_XBUS_INFIFO_LEVELS_REG 		(TNETD53XX_BBIF_BASE+0x0010)
#define TNETD53XX_BBIF_XBUS_INFIFO_LEVELS_STOP			0x0000000f
#define TNETD53XX_BBIF_XBUS_INFIFO_LEVELS_RESTART		0x000000f0

#define TNETD53XX_BBIF_XBUS_OUTFIFO_LEVELS_REG 		(TNETD53XX_BBIF_BASE+0x0014)
#define TNETD53XX_BBIF_XBUS_OUTFIFO_LEVELS_STOP			0x0000000f
#define TNETD53XX_BBIF_XBUS_OUTFIFO_LEVELS_START		0x000000f0

/*******************************************************************************************
*	ATM SAR
******************************************************************************************** */

#define	TNETD53XX_ATM_RX_BUFFER_SIZE			1564	/* Max length of buffer allocated for an ATM packet by end Driver */
#define	TNETD53XX_ATM_SAR_FREQ_REG			0xa3012280

#define	TNETD_ATM_EVENT_RX	0x00000280	/* receive data available on RCR A or B */
#define	TNETD_ATM_EVENT_TX	0x00000014	/* transmit completed on TCR A or B */

/*******************************************************************************************
*	USB MCU space
******************************************************************************************** */

/*******************************************************************************************
*	DSP
******************************************************************************************** */

/*******************************************************************************************
*	DES
******************************************************************************************** */

/*******************************************************************************************
*	Ethernet MACA, MACB - All EMAC stuff is now in tnetd53xx_emac.h
******************************************************************************************** */

/* ***************************************************************************************************
	To avoid RCR freeze condition, the number of RCR entries must be larger than the number of
	free buffers. With buffer size 64 bytes, we need 26 such buffers for a complete ethernet
	frame (MTU). Let's say that we need roughly 32 buffers per ethernet MTU. Then we can buffer
	up to 8 (if number of RCR entries is 32*8) full ethernet frames in the device.
****************************************************************************************************** */

/* ***************************************************************************************************
	Number of RX buffer descriptors per packet is equal to: Ether MTU divided by RX buffer size.
	We need to add one buffer descriptor for possible padding buffer and one for the CRC buffer.
	The total number of buffer descriptors that the device may need is the number of RCR entries
	(which gives us total number of RX packets that the device may handle) times the number of
	buffer descriptors per packet.
****************************************************************************************************** */

/* *****************************************************************************************************
	Number of Rx BD's must be less than num RCR entries, otherwise we will run into RCR freeze when
	we fill up RCR entries with incoming packets.
******************************************************************************************************** */
/*(TNETD53XX_EMAC_RX_BD_PER_PKT * TNETD53XX_EMAC_TCR_NUM_ENTRIES)*/


/*******************************************************************************************
*	Memory Controller
******************************************************************************************** */

/* Memory Controller defines */
#define TNETD53XX_MEMC_BASE		0x08610800
#define TNETD53XX_MEMC_REVISION_REG		(TNETD53XX_MEMC_BASE + 0x00)
#define TNETD53XX_MEMC_XTL_CNTRL_REG		(TNETD53XX_MEMC_BASE + 0x04)
#define	TNETD53XX_MEMC_SDRAM_CS1_CTRL_REG 	(TNETD53XX_MEMC_BASE + 0x08)
#define TNETD53XX_MEMC_REFRESH_REG		(TNETD53XX_MEMC_BASE + 0x0C)
#define TNETD53XX_MEMC_ASYNC_CS2_REG		(TNETD53XX_MEMC_BASE + 0x10)
#define TNETD53XX_MEMC_ASYNC_CS3_REG		(TNETD53XX_MEMC_BASE + 0x14)
#define TNETD53XX_MEMC_ASYNC_CS4_REG 		(TNETD53XX_MEMC_BASE + 0x18)
#define TNETD53XX_MEMC_ASYNC_CS5_REG 		(TNETD53XX_MEMC_BASE + 0x1C)

/* External Control Register defines - offset 8*/
#define TNETD53XX_MEMC_SDRAM_CFG_SELF_REF 0x80000000
#define TNETD53XX_MEMC_SDRAM_CFG_NAROW		0x00004000
#define TNETD53XX_MEMC_SDRAM_CFG_CAS2		0x00000000
#define TNETD53XX_MEMC_SDRAM_CFG_CAS3		0x00002000
#define TNETD53XX_MEMC_SDRAM_CFG_BURST		0x00001000
#define TNETD53XX_MEMC_SDRAM_CFG_DDR		0x00000800
#define TNETD53XX_MEMC_SDRAM_CFG_ROWS(X)	( ((X) & 0x07) <<8)
#define TNETD53XX_MEMC_SDRAM_CFG_PRECHRG	0x00000080
#define TNETD53XX_MEMC_SDRAM_CFG_IBANK(X)	( ((X) & 0x07) <<4)
#define TNETD53XX_MEMC_SDRAM_CFG_EBANK1		0x00000000
#define TNETD53XX_MEMC_SDRAM_CFG_EBANK2		0x00000008
#define TNETD53XX_MEMC_SDRAM_CFG_PGSZ(X) 	( ((X) & 0x07) <<0)
#define TNETD53XX_MEMC_SDRAM_CFG_PGSZ_8		0x0
#define TNETD53XX_MEMC_SDRAM_CFG_PGSZ_9   	0x1
#define TNETD53XX_MEMC_SDRAM_CFG_PGSZ_10  	0x2
#define TNETD53XX_MEMC_SDRAM_CFG_PGSZ_11  	0x3


/* Memory Controller Async Regiser defines offset 10,14,18,1C */
#define TNETD53XX_MEMC_ASYNC_EWAIT		0x40000000
#define TNETD53XX_MEMC_ASYNC_WSU(X)		( ((X) & 0x0F) <<26)
#define TNETD53XX_MEMC_ASYNC_WSTROBE(X)		( ((X) & 0x3F) <<20)
#define TNETD53XX_MEMC_ASYNC_WHOLD(X)		( ((X) & 0x07) <<17)
#define TNETD53XX_MEMC_ASYNC_RSU(X)		( ((X) & 0x0F) <<13)
#define TNETD53XX_MEMC_ASYNC_RSTROBE(X)  	( ((X) & 0x3F) <<7)
#define TNETD53XX_MEMC_ASYNC_RHOLD(X)		( ((X) & 0x07) <<4)
#define TNETD53XX_MEMC_ASYNC_TA(X)		( ((X) & 0x03) <<2)
#define TNETD53XX_MEMC_ASYNC_08BIT_BUS  	0x00000000
#define TNETD53XX_MEMC_ASYNC_16BIT_BUS   	0x00000001
#define TNETD53XX_MEMC_ASYNC_32BIT_BUS  	0x00000002
#define TNETD53XX_MEMC_ASYNC_64BIT_BUS		0x00000003 /* Future use */


/* Refresh rate register defines - offset 0c */
#define	TNETD53XX_MEMC_REFRESH_LINE_TRAP  	0x80000000
#define	TNETD53XX_MEMC_REFRESH_ASYNC_TO		0x40000000
#define	TNETD53XX_MEMC_REFRESH_RATE_MASK	0x00000FFF

/*******************************************************************************************
*	GPIO
******************************************************************************************** */

#define	TNETD53XX_GPIO_BASE				0x08610900


#define TNETD53XX_GPIO_DINR		 		(TNETD53XX_GPIO_BASE +0x00)

/* Ava-I/Ava-D support */
#define TNETD53XX_GPIO_DINR_UARTB_DSR	   0x00000001
#define TNETD53XX_GPIO_DINR_UARTB_RI	   0x00000002
#define TNETD53XX_GPIO_DINR_UARTB_TD	   0x00000004
#define TNETD53XX_GPIO_DINR_UARTB_RD	   0x00000008
#define TNETD53XX_GPIO_DINR_UARTB_CTS	   0x00000010
#define TNETD53XX_GPIO_DINR_UARTB_RTS	   0x00000020
#define TNETD53XX_GPIO_DINR_UARTB_CD	   0x00000040
#define TNETD53XX_GPIO_DINR_UARTB_DTR	   0x00000080
#define TNETD53XX_GPIO_DINR_SCC_CLK		   0x00000100
#define TNETD53XX_GPIO_DINR_SCC_CS_N	   0x00000200
#define TNETD53XX_GPIO_DINR_SCC_DIN		   0x00000400
#define TNETD53XX_GPIO_DINR_SCC_DOUT	   0x00000800
#define TNETD53XX_GPIO_DINR_SCC_RINGIN1	0x00001000
#define TNETD53XX_GPIO_DINR_SCC_RINGIN2	0x00002000
#define TNETD53XX_GPIO_DINR_SCC_RINGIN3	0x00004000
#define TNETD53XX_GPIO_DINR_SCC_RINGIN4	0x00008000
#define TNETD53XX_GPIO_DINR_SCC_CLKIN	   0x00010000
#define TNETD53XX_GPIO_DINR_EINT0		   0x00020000
#define TNETD53XX_GPIO_DINR_EINT1		   0x00040000
#define TNETD53XX_GPIO_DINR_EINT2		   0x00080000
#define TNETD53XX_GPIO_DINR_EINT3		   0x00100000
#define TNETD53XX_GPIO_DINR_GPIO0		   0x00200000
#define TNETD53XX_GPIO_DINR_GPIO1		   0x00400000
#define TNETD53XX_GPIO_DINR_GPIO2		   0x00800000
#define TNETD53XX_GPIO_DINR_GPIO3		   0x01000000

/* PUMA support */
#define TNET_PUMA_GPIO_DINR_UARTA_RD		0x00000001
#define TNET_PUMA_GPIO_DINR_UARTA_TD		0x00000002
#define TNET_PUMA_GPIO_DINR_UARTA_RTS		0x00000004
#define TNET_PUMA_GPIO_DINR_UARTA_CTS		0x00000008
#define TNET_PUMA_GPIO_DINR_IICM_D			0x00000010
#define TNET_PUMA_GPIO_DINR_IICM_CLK		0x00000020
#define TNET_PUMA_GPIO_DINR_SCC_PCM_FS		0x00000040
#define TNET_PUMA_GPIO_DINR_SCC_CLK			0x00000080
#define TNET_PUMA_GPIO_DINR_SCC_CS_N		0x00000100
#define TNET_PUMA_GPIO_DINR_SCC_DIN			0x00000200
#define TNET_PUMA_GPIO_DINR_SCC_DOUT		0x00000400
#define TNET_PUMA_GPIO_DINR_SCC_RINGIN1	0x00000800
#define TNET_PUMA_GPIO_DINR_SCC_RINGIN2	0x00001000
#define TNET_PUMA_GPIO_DINR_SCC_RINGIN3	0x00002000
#define TNET_PUMA_GPIO_DINR_SCC_RINGIN4	0x00004000
#define TNET_PUMA_GPIO_DINR_SCC_PCM_CLK	0x00008000
#define TNET_PUMA_GPIO_DINR_EINT0			0x00010000
#define TNET_PUMA_GPIO_DINR_EINT1			0x00020000
#define TNET_PUMA_GPIO_DINR_EINT2			0x00040000
#define TNET_PUMA_GPIO_DINR_EINT3			0x00080000
#define TNET_PUMA_GPIO_DINR_GPIO0			0x00100000
#define TNET_PUMA_GPIO_DINR_GPIO1			0x00200000
#define TNET_PUMA_GPIO_DINR_GPIO2			0x00400000
#define TNET_PUMA_GPIO_DINR_GPIO3			0x00800000
#define TNET_PUMA_GPIO_DINR_GPIO4			0x01000000
#define TNET_PUMA_GPIO_DINR_GPIO5			0x02000000
#define TNET_PUMA_GPIO_DINR_VLYNQ_RXD1		0x04000000
#define TNET_PUMA_GPIO_DINR_SCC_CLKIN		0x08000000



#define TNETD53XX_GPIO_DOUTR	 			(TNETD53XX_GPIO_BASE+0x04)

/* Ava-I/Ava-D support */
#define TNETD53XX_GPIO_DOUTR_UARTB_DSR	   0x00000001
#define TNETD53XX_GPIO_DOUTR_UARTB_RI	   0x00000002
#define TNETD53XX_GPIO_DOUTR_UARTB_TD	   0x00000004
#define TNETD53XX_GPIO_DOUTR_UARTB_RD	   0x00000008
#define TNETD53XX_GPIO_DOUTR_UARTB_CTS	   0x00000010
#define TNETD53XX_GPIO_DOUTR_UARTB_RTS	   0x00000020
#define TNETD53XX_GPIO_DOUTR_UARTB_CD	   0x00000040
#define TNETD53XX_GPIO_DOUTR_UARTB_DTR	   0x00000080
#define TNETD53XX_GPIO_DOUTR_SCC_CLK	   0x00000100
#define TNETD53XX_GPIO_DOUTR_SCC_CS_N	   0x00000200
#define TNETD53XX_GPIO_DOUTR_SCC_DIN	   0x00000400
#define TNETD53XX_GPIO_DOUTR_SCC_DOUT	   0x00000800
#define TNETD53XX_GPIO_DOUTR_SCC_RINGIN1	0x00001000
#define TNETD53XX_GPIO_DOUTR_SCC_RINGIN2	0x00002000
#define TNETD53XX_GPIO_DOUTR_SCC_RINGIN3	0x00004000
#define TNETD53XX_GPIO_DOUTR_SCC_RINGIN4	0x00008000
#define TNETD53XX_GPIO_DOUTR_SCC_CLKIN	   0x00010000
#define TNETD53XX_GPIO_DOUTR_EINT0		   0x00020000
#define TNETD53XX_GPIO_DOUTR_EINT1		   0x00040000
#define TNETD53XX_GPIO_DOUTR_EINT2		   0x00080000
#define TNETD53XX_GPIO_DOUTR_EINT3		   0x00100000
#define TNETD53XX_GPIO_DOUTR_GPIO0		   0x00200000
#define TNETD53XX_GPIO_DOUTR_GPIO1		   0x00400000
#define TNETD53XX_GPIO_DOUTR_GPIO2		   0x00800000
#define TNETD53XX_GPIO_DOUTR_GPIO3		   0x01000000

/* PUMA support */
#define TNET_PUMA_GPIO_DOUTR_UARTA_RD		0x00000001
#define TNET_PUMA_GPIO_DOUTR_UARTA_TD		0x00000002
#define TNET_PUMA_GPIO_DOUTR_UARTA_RTS		0x00000004
#define TNET_PUMA_GPIO_DOUTR_UARTA_CTS		0x00000008
#define TNET_PUMA_GPIO_DOUTR_IICM_D			0x00000010
#define TNET_PUMA_GPIO_DOUTR_IICM_CLK		0x00000020
#define TNET_PUMA_GPIO_DOUTR_SCC_PCM_FS	0x00000040
#define TNET_PUMA_GPIO_DOUTR_SCC_CLK		0x00000080
#define TNET_PUMA_GPIO_DOUTR_SCC_CS_N		0x00000100
#define TNET_PUMA_GPIO_DOUTR_SCC_DIN		0x00000200
#define TNET_PUMA_GPIO_DOUTR_SCC_DOUT		0x00000400
#define TNET_PUMA_GPIO_DOUTR_SCC_RINGIN1	0x00000800
#define TNET_PUMA_GPIO_DOUTR_SCC_RINGIN2	0x00001000
#define TNET_PUMA_GPIO_DOUTR_SCC_RINGIN3	0x00002000
#define TNET_PUMA_GPIO_DOUTR_SCC_RINGIN4	0x00004000
#define TNET_PUMA_GPIO_DOUTR_SCC_PCM_CLK	0x00008000
#define TNET_PUMA_GPIO_DOUTR_EINT0			0x00010000
#define TNET_PUMA_GPIO_DOUTR_EINT1			0x00020000
#define TNET_PUMA_GPIO_DOUTR_EINT2			0x00040000
#define TNET_PUMA_GPIO_DOUTR_EINT3			0x00080000
#define TNET_PUMA_GPIO_DOUTR_GPIO0			0x00100000
#define TNET_PUMA_GPIO_DOUTR_GPIO1			0x00200000
#define TNET_PUMA_GPIO_DOUTR_GPIO2			0x00400000
#define TNET_PUMA_GPIO_DOUTR_GPIO3			0x00800000
#define TNET_PUMA_GPIO_DOUTR_GPIO4			0x01000000
#define TNET_PUMA_GPIO_DOUTR_GPIO5			0x02000000
#define TNET_PUMA_GPIO_DOUTR_VLYNQ_RXD1	0x04000000
#define TNET_PUMA_GPIO_DOUTR_SCC_CLKIN		0x08000000
#define TNET_PUMA_GPIO_DOUTR_TESTD_IN_EN	0x40000000 /* SELECT MONITORING ONLY MODE OF PHY/MAC/DMA */
#define TNET_PUMA_GPIO_DOUTR_TESTD_EN		0x80000000 /* ENABLE PHY/MAC/DMA MODE MUXING TO PINS     */



#define TNETD53XX_GPIO_PDIRR	 		  (TNETD53XX_GPIO_BASE+0x0008)

/* Ava-I/Ava-D support */
#define TNETD53XX_GPIO_PDIRR_UARTB_DSR	   0x00000001
#define TNETD53XX_GPIO_PDIRR_UARTB_RI	   0x00000002
#define TNETD53XX_GPIO_PDIRR_UARTB_TD	   0x00000004
#define TNETD53XX_GPIO_PDIRR_UARTB_RD	   0x00000008
#define TNETD53XX_GPIO_PDIRR_UARTB_CTS	   0x00000010
#define TNETD53XX_GPIO_PDIRR_UARTB_RTS	   0x00000020
#define TNETD53XX_GPIO_PDIRR_UARTB_CD	   0x00000040
#define TNETD53XX_GPIO_PDIRR_UARTB_DTR	   0x00000080
#define TNETD53XX_GPIO_PDIRR_SCC_CLK	   0x00000100
#define TNETD53XX_GPIO_PDIRR_SCC_CS_N	   0x00000200
#define TNETD53XX_GPIO_PDIRR_SCC_DIN	   0x00000400
#define TNETD53XX_GPIO_PDIRR_SCC_DOUT	   0x00000800
#define TNETD53XX_GPIO_PDIRR_SCC_RINGIN1	0x00001000
#define TNETD53XX_GPIO_PDIRR_SCC_RINGIN2	0x00002000
#define TNETD53XX_GPIO_PDIRR_SCC_RINGIN3	0x00004000
#define TNETD53XX_GPIO_PDIRR_SCC_RINGIN4	0x00008000
#define TNETD53XX_GPIO_PDIRR_SCC_CLKIN	   0x00010000
#define TNETD53XX_GPIO_PDIRR_EINT0		   0x00020000
#define TNETD53XX_GPIO_PDIRR_EINT1		   0x00040000
#define TNETD53XX_GPIO_PDIRR_EINT2		   0x00080000
#define TNETD53XX_GPIO_PDIRR_EINT3		   0x00100000
#define TNETD53XX_GPIO_PDIRR_GPIO0		   0x00200000
#define TNETD53XX_GPIO_PDIRR_GPIO1		   0x00400000
#define TNETD53XX_GPIO_PDIRR_GPIO2		   0x00800000
#define TNETD53XX_GPIO_PDIRR_GPIO3		   0x01000000

/* PUMA support */
#define TNET_PUMA_GPIO_PDIRR_UARTA_RD		0x00000001
#define TNET_PUMA_GPIO_PDIRR_UARTA_TD		0x00000002
#define TNET_PUMA_GPIO_PDIRR_UARTA_RTS		0x00000004
#define TNET_PUMA_GPIO_PDIRR_UARTA_CTS		0x00000008
#define TNET_PUMA_GPIO_PDIRR_IICM_D			0x00000010
#define TNET_PUMA_GPIO_PDIRR_IICM_CLK		0x00000020
#define TNET_PUMA_GPIO_PDIRR_SCC_PCM_FS	0x00000040
#define TNET_PUMA_GPIO_PDIRR_SCC_CLK		0x00000080
#define TNET_PUMA_GPIO_PDIRR_SCC_CS_N		0x00000100
#define TNET_PUMA_GPIO_PDIRR_SCC_DIN		0x00000200
#define TNET_PUMA_GPIO_PDIRR_SCC_DOUT		0x00000400
#define TNET_PUMA_GPIO_PDIRR_SCC_RINGIN1	0x00000800
#define TNET_PUMA_GPIO_PDIRR_SCC_RINGIN2	0x00001000
#define TNET_PUMA_GPIO_PDIRR_SCC_RINGIN3	0x00002000
#define TNET_PUMA_GPIO_PDIRR_SCC_RINGIN4	0x00004000
#define TNET_PUMA_GPIO_PDIRR_SCC_PCM_CLK	0x00008000
#define TNET_PUMA_GPIO_PDIRR_EINT0			0x00010000
#define TNET_PUMA_GPIO_PDIRR_EINT1			0x00020000
#define TNET_PUMA_GPIO_PDIRR_EINT2			0x00040000
#define TNET_PUMA_GPIO_PDIRR_EINT3			0x00080000
#define TNET_PUMA_GPIO_PDIRR_GPIO0			0x00100000
#define TNET_PUMA_GPIO_PDIRR_GPIO1			0x00200000
#define TNET_PUMA_GPIO_PDIRR_GPIO2			0x00400000
#define TNET_PUMA_GPIO_PDIRR_GPIO3			0x00800000
#define TNET_PUMA_GPIO_PDIRR_GPIO4			0x01000000
#define TNET_PUMA_GPIO_PDIRR_GPIO5			0x02000000
#define TNET_PUMA_GPIO_PDIRR_VLYNQ_RXD1	0x04000000
#define TNET_PUMA_GPIO_PDIRR_SCC_CLKIN		0x08000000



#define TNETD53XX_GPIO_ENR	 				(TNETD53XX_GPIO_BASE+0x000c)

/* Ava-I/Ava-D support */
#define TNETD53XX_GPIO_ENR_UARTB_DSR	0x00000001
#define TNETD53XX_GPIO_ENR_UARTB_RI		0x00000002
#define TNETD53XX_GPIO_ENR_UARTB_TD		0x00000004
#define TNETD53XX_GPIO_ENR_UARTB_RD		0x00000008
#define TNETD53XX_GPIO_ENR_UARTB_CTS	0x00000010
#define TNETD53XX_GPIO_ENR_UARTB_RTS	0x00000020
#define TNETD53XX_GPIO_ENR_UARTB_CD		0x00000040
#define TNETD53XX_GPIO_ENR_UARTB_DTR	0x00000080
#define TNETD53XX_GPIO_ENR_SCC_CLK		0x00000100
#define TNETD53XX_GPIO_ENR_SCC_CS_N		0x00000200
#define TNETD53XX_GPIO_ENR_SCC_DIN		0x00000400
#define TNETD53XX_GPIO_ENR_SCC_DOUT		0x00000800
#define TNETD53XX_GPIO_ENR_SCC_RINGIN1	0x00001000
#define TNETD53XX_GPIO_ENR_SCC_RINGIN2	0x00002000
#define TNETD53XX_GPIO_ENR_SCC_RINGIN3	0x00004000
#define TNETD53XX_GPIO_ENR_SCC_RINGIN4	0x00008000
#define TNETD53XX_GPIO_ENR_SCC_CLKIN	0x00010000
#define TNETD53XX_GPIO_ENR_EINT0		   0x00020000
#define TNETD53XX_GPIO_ENR_EINT1		   0x00040000
#define TNETD53XX_GPIO_ENR_EINT2		   0x00080000
#define TNETD53XX_GPIO_ENR_EINT3		   0x00100000
#define TNETD53XX_GPIO_ENR_GPIO0		   0x00200000
#define TNETD53XX_GPIO_ENR_GPIO1		   0x00400000
#define TNETD53XX_GPIO_ENR_GPIO2		   0x00800000
#define TNETD53XX_GPIO_ENR_GPIO3		   0x01000000

/* PUMA support */
#define TNET_PUMA_GPIO_ENR_UARTA_RD			0x00000001
#define TNET_PUMA_GPIO_ENR_UARTA_TD			0x00000002
#define TNET_PUMA_GPIO_ENR_UARTA_RTS		0x00000004
#define TNET_PUMA_GPIO_ENR_UARTA_CTS		0x00000008
#define TNET_PUMA_GPIO_ENR_IICM_D			0x00000010
#define TNET_PUMA_GPIO_ENR_IICM_CLK			0x00000020
#define TNET_PUMA_GPIO_ENR_SCC_PCM_FS		0x00000040
#define TNET_PUMA_GPIO_ENR_SCC_CLK			0x00000080
#define TNET_PUMA_GPIO_ENR_SCC_CS_N			0x00000100
#define TNET_PUMA_GPIO_ENR_SCC_DIN			0x00000200
#define TNET_PUMA_GPIO_ENR_SCC_DOUT			0x00000400
#define TNET_PUMA_GPIO_ENR_SCC_RINGIN1		0x00000800
#define TNET_PUMA_GPIO_ENR_SCC_RINGIN2		0x00001000
#define TNET_PUMA_GPIO_ENR_SCC_RINGIN3		0x00002000
#define TNET_PUMA_GPIO_ENR_SCC_RINGIN4		0x00004000
#define TNET_PUMA_GPIO_ENR_SCC_PCM_CLK		0x00008000
#define TNET_PUMA_GPIO_ENR_EINT0	   		0x00010000
#define TNET_PUMA_GPIO_ENR_EINT1		   	0x00020000
#define TNET_PUMA_GPIO_ENR_EINT2			   0x00040000
#define TNET_PUMA_GPIO_ENR_EINT3          0x00080000
#define TNET_PUMA_GPIO_ENR_GPIO0	         0x00100000
#define TNET_PUMA_GPIO_ENR_GPIO1	         0x00200000
#define TNET_PUMA_GPIO_ENR_GPIO2	         0x00400000
#define TNET_PUMA_GPIO_ENR_GPIO3	         0x00800000
#define TNET_PUMA_GPIO_ENR_GPIO4	         0x01000000
#define TNET_PUMA_GPIO_ENR_GPIO5	         0x02000000
#define TNET_PUMA_GPIO_ENR_VLYNQ_RXD1		0x04000000
#define TNET_PUMA_GPIO_ENR_SCC_CLKIN		0x08000000


#define TNETD53XX_GPIO_CVR	 			(TNETD53XX_GPIO_BASE+0x0014)
#define TNETD53XX_GPIO_CVR_ID			0x0000ffff


/*******************************************************************************************
*	Device ID
******************************************************************************************** */

/*******************************************************************************************
*	Clock Control
******************************************************************************************** */
#define	TNETD53XX_CLKC_BASE					0x08610a00

#define	TNETD53XX_CLKC_PDCR					(TNETD53XX_CLKC_BASE + 0x00 )
#define	TNETD53XX_CLKC_PDCR_USB				0x00000001
#define	TNETD53XX_CLKC_PDCR_WDT				0x00000002
#define	TNETD53XX_CLKC_PDCR_UART0			0x00000004
#define	TNETD53XX_CLKC_PDCR_UART1			0x00000008
#define	TNETD53XX_CLKC_PDCR_IIC0			0x00000010
#define	TNETD53XX_CLKC_PDCR_GPIO			0x00000040
#define	TNETD53XX_CLKC_PDCR_PCI				0x00000080
#define	TNETD53XX_CLKC_PDCR_SAR				0x00000100
#define	TNETD53XX_CLKC_PDCR_BBIF			0x00000200
#define	TNETD53XX_CLKC_PDCR_EMIF			0x00000400
#define	TNETD53XX_CLKC_PDCR_HDLC			0x00000800
#define	TNETD53XX_CLKC_PDCR_DSP				0x00001000
#define	TNETD53XX_CLKC_PDCR_RAM				0x00002000
#define	TNETD53XX_CLKC_PDCR_ROM				0x00004000
#define	TNETD53XX_CLKC_PDCR_PDMA			0x00008000
#define	TNETD53XX_CLKC_PDCR_BIST			0x00010000
#define	TNETD53XX_CLKC_PDCR_DES				0x00020000
#define	TNETD53XX_CLKC_PDCR_TMR0			0x00040000
#define	TNETD53XX_CLKC_PDCR_TMR1			0x00080000
#define	TNETD53XX_CLKC_PDCR_MAC0			0x00100000
#define	TNETD53XX_CLKC_PDCR_SCC				0x00200000
#define	TNETD53XX_CLKC_PDCR_MAC1			0x00400000
#define	TNETD53XX_CLKC_PDCR_GLBPD			0xc0000000

#define	TNETD53XX_CLKC_PCLKCR		 		(TNETD53XX_CLKC_BASE + 0x04 )
#define	TNETD53XX_CLKC_PCLKCR_PCI			0x00010000
#define	TNETD53XX_CLKC_PCLKCR_BBIF_VBUS		0x00020000
#define	TNETD53XX_CLKC_PCLKCR_BBIF			0x00060000
#define	TNETD53XX_CLKC_PCLKCR_SCC			0x00080000
#define	TNETD53XX_CLKC_PCLKCR_DSP			0x00100000
#define	TNETD53XX_CLKC_PCLKCR_USB			0x00200000

#define TNETD53XX_CLKC_PDUCR			 	(TNETD53XX_CLKC_BASE + 0x08 )
#define TNETD53XX_CLKC_PDUCR_UARTB_DSR		0x00000001
#define TNETD53XX_CLKC_PDUCR_UARTB_RI		0x00000002
#define TNETD53XX_CLKC_PDUCR_UARTB_TD		0x00000004
#define TNETD53XX_CLKC_PDUCR_UARTB_RD		0x00000008
#define TNETD53XX_CLKC_PDUCR_UARTB_CTS		0x00000010
#define TNETD53XX_CLKC_PDUCR_UARTB_RTS		0x00000020
#define TNETD53XX_CLKC_PDUCR_UARTB_CD		0x00000040
#define TNETD53XX_CLKC_PDUCR_UARTB_DTR		0x00000080
#define TNETD53XX_CLKC_PDUCR_SCC_DCLK		0x00000100
#define TNETD53XX_CLKC_PDUCR_SCC_CS_N		0x00000200
#define TNETD53XX_CLKC_PDUCR_SCC_DIN		0x00000400
#define TNETD53XX_CLKC_PDUCR_SCC_DOUT		0x00000800
#define TNETD53XX_CLKC_PDUCR_SCC_RINGIN1	0x00001000
#define TNETD53XX_CLKC_PDUCR_SCC_RINGIN2	0x00002000
#define TNETD53XX_CLKC_PDUCR_SCC_RINGIN3	0x00004000
#define TNETD53XX_CLKC_PDUCR_SCC_RINGIN4	0x00008000
#define TNETD53XX_CLKC_PDUCR_SCC_CLKIN		0x00010000
#define TNETD53XX_CLKC_PDUCR_EINT0			0x00020000
#define TNETD53XX_CLKC_PDUCR_EINT1			0x00040000
#define TNETD53XX_CLKC_PDUCR_EINT2			0x00080000
#define TNETD53XX_CLKC_PDUCR_EINT3			0x00100000
#define TNETD53XX_CLKC_PDUCR_UARTA_RD		0x00200000
#define TNETD53XX_CLKC_PDUCR_GPIO0			0x00400000
#define TNETD53XX_CLKC_PDUCR_GPIO1			0x00800000
#define TNETD53XX_CLKC_PDUCR_GPIO2			0x01000000
#define TNETD53XX_CLKC_PDUCR_GPIO3			0x02000000
#define TNETD53XX_CLKC_PDUCR_IICM_D			0x04000000
#define TNETD53XX_CLKC_PDUCR_BBIF			0x08000000
#define TNETD53XX_CLKC_PDUCR_EJTAG			0x10000000

#define TNETD53XX_CLKC_WKCR		 			(TNETD53XX_CLKC_BASE + 0x0C)
#define TNETD53XX_CLKC_WKCR_WAKE_EN			0x000000ff
#define TNETD53XX_CLKC_WKCR_WAKE_POL		0x00ff0000

#define TNETD53XX_CLKC_SCLKCR 				(TNETD53XX_CLKC_BASE + 0x20)
#define TNETD53XX_CLKC_SCLKCR_REG 			TNETD53XX_CLKC_SCLKCR /* Kept for backward comp. */
#define TNETD53XX_CLKC_SCLKCR_CLKDIV		0x0000000f /* Divide XTAL clock by (CLKDIV+1) After PLL */
#define TNETD53XX_CLKC_SCLKCR_CLKPREDIV	0x001f0000 /* Divide XTAL clock by (CLKPREDIV+1) before PLL */
/* 10MHz (8MHz for USB) <= PLL Input freq <= 100MHz,
   PLL Out freq <= 250MHz
 */

#define 	TNETD53XX_CLKC_SYSCLKMD_REG      (TNETD53XX_CLKC_BASE + 0x30)

#define TNETD53XX_CLKC_UCLKCR	 			   (TNETD53XX_CLKC_BASE + 0x40)
#define TNETD53XX_CLKC_UCLKCR_USBCLKDIV	0x0000000f
#define TNETD53XX_CLKC_UCLKCR2	 			(TNETD53XX_CLKC_BASE + 0x60)

/*******************************************************************************************
*	Watchdog Timer
******************************************************************************************** */

/*******************************************************************************************
*	Timer 1
******************************************************************************************** */

/*******************************************************************************************
*	Timer 2
******************************************************************************************** */

/*******************************************************************************************
*	UART A, B
******************************************************************************************** */
#define	TNETD53XX_UARTA_BASE		0x08610E00
#define	TNETD53XX_UARTB_BASE		0x08610F00
#define TNETD53XX_UART_FREQ		TNETD53XX_VBUS_FREQ					 

/*******************************************************************************************
*	I2CM  NOTE: ALL I2C registers are 16 bit!
******************************************************************************************** */
#define 	TNETD53XX_I2C_BASE	0x08611000

#define 	TNETD53XX_I2C_DATA_REG_HIGH_REG 	(TNETD53XX_I2C_BASE + TNETD53XX_REG16_OFFSET)
#define 	TNETD53XX_I2C_DATA_REG_HIGH_SLAVE_DEV_ADDR	 0x007f
#define 	TNETD53XX_I2C_DATA_REG_HIGH_WRITE_MODE	     0x0200
#define 	TNETD53XX_I2C_DATA_REG_HIGH_READ_MODE	 	 0x0300
#define 	TNETD53XX_I2C_DATA_REG_HIGH_END_BURST		 0x0400


#define 	TNETD53XX_I2C_DATA_REG_LOW_REG 		(TNETD53XX_I2C_BASE+0x0004 + TNETD53XX_REG16_OFFSET)

#define 	TNETD53XX_I2C_CFG_REG 				(TNETD53XX_I2C_BASE+0x0008 + TNETD53XX_REG16_OFFSET)
#define 	TNETD53XX_I2C_CFG_INT_MASK					0x0001
#define 	TNETD53XX_I2C_CFG_SPIKE_FILTER				0x7000

#define 	TNETD53XX_I2C_READ_DATA_REG_REG 	(TNETD53XX_I2C_BASE+0x000C + TNETD53XX_REG16_OFFSET)
#define 	TNETD53XX_I2C_READ_DATA_REG_DATA				0x00ff
#define 	TNETD53XX_I2C_READ_DATA_REG_WRITE_ALLOWED		0x2000
#define 	TNETD53XX_I2C_READ_DATA_REG_INT_FLAG			0x4000
#define 	TNETD53XX_I2C_READ_DATA_REG_ERR_FLAG			0x8000

#define TNETD53XX_I2C_CLOCK_DIVIDER_REG 		(TNETD53XX_I2C_BASE+0x0010 + TNETD53XX_REG16_OFFSET)

/*******************************************************************************************
*	USB
******************************************************************************************** */

/*******************************************************************************************
*	PCI Configuration Registers
******************************************************************************************** */

/*******************************************************************************************
*	DMA - 4 channels 0-3. transfers physical addresses! up to 64k*32bit transfer at a time.
******************************************************************************************** */
#define TNETD53XX_PDMA_CNTRL_BASE	0x08611400

#define TNETD53XX_PDMA_SRCADDR(channel) \
     (TNETD53XX_PDMA_CNTRL_BASE	+ ((channel) & 3) * 0x40)

#define TNETD53XX_PDMA_DSTADDR(channel) \
     (TNETD53XX_PDMA_CNTRL_BASE	+ 0x04 + ((channel) & 3) * 0x40)

#define TNETD53XX_PDMA_TFRCTL(channel)  \
     (TNETD53XX_PDMA_CNTRL_BASE	+ 0x08 + ((channel) & 3) * 0x40)
#define TNETD53XX_PDMA_TFRCTL_STARTSTOP                (1ul<<31)  /* write 1 to start/stop data */
#define TNETD53XX_PDMA_TFRCTL_ACTIVE                   (1ul<<30)  /* read 1 while transfer is still in progress */
#define TNETD53XX_PDMA_TFRCTL_DSTADDR_MODE_INCRM       (0ul<<24)
#define TNETD53XX_PDMA_TFRCTL_DSTADDR_MODE_FIXED       (2ul<<24)
#define TNETD53XX_PDMA_TFRCTL_SRCADDR_MODE_INCRM       (0ul<<22)
#define TNETD53XX_PDMA_TFRCTL_SRCADDR_MODE_FIXED       (2ul<<22)
#define TNETD53XX_PDMA_TFRCTL_BURST1                   (0ul<<20)  /* 1*UINT32 burst */
#define TNETD53XX_PDMA_TFRCTL_BURST2                   (1ul<<20)  /* 2*UINT32 burst */
#define TNETD53XX_PDMA_TFRCTL_BURST4                   (2ul<<20)  /* 4*UINT32 burst */
#define TNETD53XX_PDMA_TFRCTL_TFR_LENGTH(x)            ((x) & 0x01fffc )  /* x = TFR length in bytes */

/*******************************************************************************************
*	Reset Control
******************************************************************************************** */
#define	TNETD53XX_RST_CNTRL_BASE	0x08611600
#define	TNETD53XX_RST_CNTRL_PRCR	(TNETD53XX_RST_CNTRL_BASE + 0x00 )

/* Reset bits for the peripherals */
#define TNETD53XX_RST_CNTRL_PRCR_UART0	0x00000001
#define TNETD53XX_RST_CNTRL_PRCR_UART1	0x00000002
#define TNETD53XX_RST_CNTRL_PRCR_IIC0	0x00000004
#define TNETD53XX_RST_CNTRL_PRCR_TMR0	0x00000008
#define TNETD53XX_RST_CNTRL_PRCR_TMR1	0x00000010
#define TNETD53XX_RST_CNTRL_PRCR_DES	0x00000020
#define TNETD53XX_RST_CNTRL_PRCR_GPIO	0x00000040
#define TNETD53XX_RST_CNTRL_PRCR_BBIF	0x00000080
#define TNETD53XX_RST_CNTRL_PRCR_USB	0x00000100
#define TNETD53XX_RST_CNTRL_PRCR_SAR	0x00000200
#define TNETD53XX_RST_CNTRL_PRCR_HDLC	0x00000400
#define TNETD53XX_RST_CNTRL_PRCR_PCI	0x00010000
#define TNETD53XX_RST_CNTRL_PRCR_EMAC0	0x00020000	/* EMAC0 reset */
#define TNETD53XX_RST_CNTRL_PRCR_PDMA	0x00040000
#define TNETD53XX_RST_CNTRL_PRCR_BIST	0x00080000
#define TNETD53XX_RST_CNTRL_PRCR_DSP	0x00100000  /* AvaI */
#define TNETD53XX_RST_CNTRL_PRCR_VLYNQ	0x00100000  /* Puma, PumaS */
#define TNETD53XX_RST_CNTRL_PRCR_EMAC1	0x00200000	/* EMAC1 reset */

#define TNETD53XX_RST_CNTRL_SWRCR	(TNETD53XX_RST_CNTRL_BASE + 0x04 )
#define	TNETD53XX_RST_SWRCR_SWR0	0x00000001
#define TNETD53XX_RST_SWRCR_SWR1	0x00000002

#define TNETD53XX_RST_CNTRL_RSR		(TNETD53XX_RST_CNTRL_BASE + 0x08 )
#define	TNETD53XX_RST_RSR_RSCAUSE	0x00000003

/*******************************************************************************************
*	BIST Control
******************************************************************************************** */

/*******************************************************************************************
*	DSP Subsystem Control
******************************************************************************************** */
#define	TNETD53XX_DSPC_BASE	 	0x08611800		

#define TNETD53XX_DSPC_DSPCR1	 	(TNETD53XX_DSPC_BASE + 0x00)
#define TNETD53XX_DSPC_DSPCR1_RST_A	0x00000001
#define TNETD53XX_DSPC_DSPCR1_RST_B	0x00000002
#define TNETD53XX_DSPC_DSPCR1_CLKMD	0x00000100

#define TNETD53XX_DSPC_DSPCR1_HPIRSN	0x00000200
#define TNETD53XX_DSPC_DSPCR1_INTA	0x00010000
#define TNETD53XX_DSPC_DSPCR1_INTB	0x00020000
#define TNETD53XX_DSPC_DSPCR1_NMIA	0x01000000
#define TNETD53XX_DSPC_DSPCR1_NMIB	0x02000000

#define TNETD53XX_DSPC_DSPCR2	 	(TNETD53XX_DSPC_BASE + 0x04)
#define TNETD53XX_DSPC_DSPCR2_GPIOA_IN	0x0000000f
#define TNETD53XX_DSPC_DSPCR2_GPIOB_IN	0x00000f00

#define TNETD53XX_DSPC_DSPSR	 	(TNETD53XX_DSPC_BASE + 0x08)
#define TNETD53XX_DSPC_DSPSR_GPIOA_OUT	0x0000000f
#define TNETD53XX_DSPC_DSPSR_GPIOB_OUT	0x00000f00

#define	TNETD53XX_DSP_HPI_BASE		0x04000000

/*******************************************************************************************
*	Voice/Codec Interface
******************************************************************************************** */
#if 0

#define TNETD53XX_VOIC_BASE						0x08611900

#define TNETD53XX_VOIC_PCMCR1	 				(TNETD53XX_VOIC_BASE + 0x00)
#define TNETD53XX_VOIC_PCMCR1_PCM_CLKDIV		0x00000fff
#define TNETD53XX_VOIC_PCMCR1_PCE				0x00008000
#define TNETD53XX_VOIC_PCMCR1_FS_PER			0x3fff0000

#define TNETD53XX_VOIC_PCMCR2		 			(TNETD53XX_VOIC_BASE + 0x04)
#define TNETD53XX_VOIC_PCMCR2_BSPA_RXCLK_SRC	0x00010000
#define TNETD53XX_VOIC_PCMCR2_BSPA_RXCLK_SYNC	0x00020000
#define TNETD53XX_VOIC_PCMCR2_BSPA_TXCLK_SRC	0x00040000
#define TNETD53XX_VOIC_PCMCR2_BSPA_TXCLK_SYNC	0x00080000
#define TNETD53XX_VOIC_PCMCR2_BSPB_RXCLK_SRC	0x01000000
#define TNETD53XX_VOIC_PCMCR2_BSPB_RXCLK_SYNC	0x02000000
#define TNETD53XX_VOIC_PCMCR2_BSPB_TXCLK_SRC	0x04000000
#define TNETD53XX_VOIC_PCMCR2_BSPB_TXCLK_SYNC	0x08000000

#define TNETD53XX_VOIC_SPCR1					(TNETD53XX_VOIC_BASE + 0x10)
#define TNETD53XX_VOIC_SPCR1_DCLK_DIV			0x0000ffff
#define TNETD53XX_VOIC_SPCR1_DATALEN			0x001f0000
#define TNETD53XX_VOIC_SPCR1_RCMD				0x00200000
#define TNETD53XX_VOIC_SPCR1_WCMD				0x00400000
#define TNETD53XX_VOIC_SPCR1_DCE				0x01000000
#define TNETD53XX_VOIC_SPCR1_INTEN				0x02000000
#define TNETD53XX_VOIC_SPCR1_DPIN_MODE			0x80000000

#define TNETD53XX_VOIC_SPDR 					(TNETD53XX_VOIC_BASE + 0x18)
#define TNETD53XX_VOIC_SPDR_SP_WDATA			0x0000ffff
#define TNETD53XX_VOIC_SPDR_WST					0x80000000

#define TNETD53XX_VOIC_SPBR 					(TNETD53XX_VOIC_BASE + 0x1c)
#define TNETD53XX_VOIC_SPBR_SP_RDATA			0x0000ffff
#define TNETD53XX_VOIC_SPBR_RST					0x80000000

#define TNETD53XX_VOIC_RCR						(TNETD53XX_VOIC_BASE + 0x20)
#define TNETD53XX_VOIC							TNETD53XX_VOIC_RCR /* Kept for backward compatibility...*/
#define TNETD53XX_VOIC_RCR_REN1					0x00000001
#define TNETD53XX_VOIC_RCR_REN2					0x00000002
#define TNETD53XX_VOIC_RCR_REN3					0x00000004
#define TNETD53XX_VOIC_RCR_REN4					0x00000008

/* Note the two timer registers below are 16 bit */
#define TNETD53XX_VOIC_TMCNTRL_BASE				(TNETD53XX_VOIC_BASE + 0x40 + TNETD53XX_REG16_OFFSET)
#define TNETD53XX_VOIC_TMCNTRL_REG				(TNETD53XX_VOIC_BASE + 0x42)
#define TNETD53XX_VOIC_TMCNT_REG				(TNETD53XX_VOIC_BASE + 0x46)

#endif /* 0 */
/*******************************************************************************************
*	Device Configuratin Latch (DCL)
******************************************************************************************** */
#define	TNETD53XX_LATCH_CNTRL_BASE		     0x08611A00
#define	TNETD53XX_LATCH_CNTRL_BOOTCR       (TNETD53XX_LATCH_CNTRL_BASE + 0x00 )

#define	TNETD53XX_LATCH_CNTRL_BOOTCR_PLLBP 0x00000020


/*******************************************************************************************
*	DSL Interface
******************************************************************************************** */
#define TNETD53XX_DSLIF_BASE                         		   0x08611B00

#define TNETD53XX_DSLIF_TX_ENABLE_REG                         (TNETD53XX_DSLIF_BASE + 0x00000000)
#define TNETD53XX_DSLIF_TX_DISABLE                            0x00000000;
#define TNETD53XX_DSLIF_TX_ENABLE_IF_0_FAST                   0x00000001;
#define TNETD53XX_DSLIF_TX_ENABLE_IF_1_INTV                   0x00000002;

#define TNETD53XX_DSLIF_RX_ENABLE_REG                         (TNETD53XX_DSLIF_BASE + 0x00000004)
#define TNETD53XX_DSLIF_RX_DISABLE                            0x00000000;
#define TNETD53XX_DSLIF_RX_ENABLE_IF_0_FAST                   0x00000001;
#define TNETD53XX_DSLIF_RX_ENABLE_IF_1_INTV                   0x00000002;

#define TNETD53XX_DSLIF_ENABLE_POLLING_REG                    (TNETD53XX_DSLIF_BASE + 0x00000008)
#define TNETD53XX_DSLIF_ENABLE_POLLING                        0x00000001;
#define TNETD53XX_DSLIF_SUSPEND_POLLING                       0x00000000;

#define TNETD53XX_DSLIF_TX_FIFO_0_PTR_ADDR_REG                (TNETD53XX_DSLIF_BASE + 0x0000000c)
#define TNETD53XX_DSLIF_TX_FIFO_0_BASE_ADDR_REG               (TNETD53XX_DSLIF_BASE + 0x00000010)
#define TNETD53XX_DSLIF_TX_FIFO_0_LEN_REG                     (TNETD53XX_DSLIF_BASE + 0x00000014)
#define TNETD53XX_DSLIF_TX_FIFO_0_POLLRATE_REG                (TNETD53XX_DSLIF_BASE + 0x00000018)
#define TNETD53XX_DSLIF_RX_FIFO_0_PTR_ADDR_REG                (TNETD53XX_DSLIF_BASE + 0x0000001c)
#define TNETD53XX_DSLIF_RX_FIFO_0_BASE_ADDR_REG               (TNETD53XX_DSLIF_BASE + 0x00000020)
#define TNETD53XX_DSLIF_RX_FIFO_0_LEN_REG                     (TNETD53XX_DSLIF_BASE + 0x00000024)
#define TNETD53XX_DSLIF_RX_FIFO_0_POLLRATE_REG                (TNETD53XX_DSLIF_BASE + 0x00000028)
#define TNETD53XX_DSLIF_TX_FIFO_1_PTR_ADDR_REG                (TNETD53XX_DSLIF_BASE + 0x0000002c)
#define TNETD53XX_DSLIF_TX_FIFO_1_BASE_ADDR_REG               (TNETD53XX_DSLIF_BASE + 0x00000030)
#define TNETD53XX_DSLIF_TX_FIFO_1_LEN_REG                     (TNETD53XX_DSLIF_BASE + 0x00000034)
#define TNETD53XX_DSLIF_TX_FIFO_1_POLLRATE_REG                (TNETD53XX_DSLIF_BASE + 0x00000038)
#define TNETD53XX_DSLIF_RX_FIFO_1_PTR_ADDR_REG                (TNETD53XX_DSLIF_BASE + 0x0000003c)
#define TNETD53XX_DSLIF_RX_FIFO_1_BASE_ADDR_REG               (TNETD53XX_DSLIF_BASE + 0x00000040)
#define TNETD53XX_DSLIF_RX_FIFO_1_LEN_REG                     (TNETD53XX_DSLIF_BASE + 0x00000044)
#define TNETD53XX_DSLIF_RX_FIFO_1_POLLRATE_REG                (TNETD53XX_DSLIF_BASE + 0x00000048)

/*******************************************************************************************
*	Interrupt Control
*
*	To form an interrupt source for the function call, OR with one of:
*
*	TNETD53XX_INTR_CNTRL_SRC_PRIMARY	- for primary interrupt source
*	TNETD53XX_INTR_CNTRL_SRC_SECONDARY	- for secondary interrupt source
*
******************************************************************************************** */
#define	TNETD53XX_INTR_CNTRL_SRC_PRIMARY	   0x80000000
#define	TNETD53XX_INTR_CNTRL_SRC_SECONDARY	0x40000000

/* ******************************************************
Interrupt sources on TNETD53xx interrupt controller.
The reserved sources are skipped, but accounted for.
********************************************************* */
#define	TNETD53XX_INTR_CNTRL_SRC_SECOND		0
#define	TNETD53XX_INTR_CNTRL_SRC_EXTERNAL0	1
#define	TNETD53XX_INTR_CNTRL_SRC_EXTERNAL1	2
#define	TNETD53XX_INTR_CNTRL_SRC_EXTERNAL2	3
#define	TNETD53XX_INTR_CNTRL_SRC_EXTERNAL3	4
#define	TNETD53XX_INTR_CNTRL_SRC_TIMER0		5
#define	TNETD53XX_INTR_CNTRL_SRC_TIMER1		6
#define	TNETD53XX_INTR_CNTRL_SRC_UART0		7
#define	TNETD53XX_INTR_CNTRL_SRC_UART1		8
#define	TNETD53XX_INTR_CNTRL_SRC_PDMA0		9
#define	TNETD53XX_INTR_CNTRL_SRC_PDMA1		10
#define	TNETD53XX_INTR_CNTRL_SRC_HDLC_TXA	11
#define	TNETD53XX_INTR_CNTRL_SRC_HDLC_TXB	12
#define	TNETD53XX_INTR_CNTRL_SRC_HDLC_RXA	13
#define	TNETD53XX_INTR_CNTRL_SRC_HDLC_RXB	14
#define	TNETD53XX_INTR_CNTRL_SRC_SAR_TXA	   15
#define	TNETD53XX_INTR_CNTRL_SRC_SAR_TXB	   16
#define	TNETD53XX_INTR_CNTRL_SRC_SAR_RXA	   17
#define	TNETD53XX_INTR_CNTRL_SRC_SAR_RXB	   18
#define	TNETD53XX_INTR_CNTRL_SRC_EMAC_TXA	19
#define	TNETD53XX_INTR_CNTRL_SRC_EMAC_RXA	20
#define	TNETD53XX_INTR_CNTRL_SRC_DSP0		   21
#define	TNETD53XX_INTR_CNTRL_SRC_VLYNQ		   21 /* same as DSP0 */
#define	TNETD53XX_INTR_CNTRL_SRC_DSP1		   22
#define	TNETD53XX_INTR_CNTRL_SRC_DES		   23
#define	TNETD53XX_INTR_CNTRL_SRC_USB		   24
#define	TNETD53XX_INTR_CNTRL_SRC_PCIA		   25
#define	TNETD53XX_INTR_CNTRL_SRC_PCIB		   26
#define	TNETD53XX_INTR_CNTRL_SRC_PCICf		27
/* reserved sources ... */
#define	TNETD53XX_INTR_CNTRL_SRC_I2CM		   29
#define	TNETD53XX_INTR_CNTRL_SRC_PDMA2		30
#define	TNETD53XX_INTR_CNTRL_SRC_PDMA3		31
#define	TNETD53XX_INTR_CNTRL_SRC_CODEC		32
#define	TNETD53XX_INTR_CNTRL_SRC_EMAC_TXB	33
#define	TNETD53XX_INTR_CNTRL_SRC_EMAC_RXB	34
/* reserved sources ... */
#define	TNETD53XX_INTR_CNTRL_SRC_PUMA_DOCSIS 39
/* reserved sources ... */

#define	TNETD53XX_INTR_CNTRL_SRC_HDLC_STATUS	0
#define	TNETD53XX_INTR_CNTRL_SRC_SAR_STATUS	   1
#define	TNETD53XX_INTR_CNTRL_SRC_EMACA_STATUS	2
#define	TNETD53XX_INTR_CNTRL_SRC_EMACA_LNK_CHNG 3
#define	TNETD53XX_INTR_CNTRL_SRC_EMACA_MNGMNT  4
#define	TNETD53XX_INTR_CNTRL_SRC_PCI_STATUS	   5
/* reserved sources... */
#define	TNETD53XX_INTR_CNTRL_SRC_EXT_MEMORY	   7
#define	TNETD53XX_INTR_CNTRL_SRC_DSPA_WATCHDOG	8
#define	TNETD53XX_INTR_CNTRL_SRC_DSPB_WATCHDOG	9
/* reserved sources... */
#define	TNETD53XX_INTR_CNTRL_SRC_EMACB_STATUS	20
#define	TNETD53XX_INTR_CNTRL_SRC_EMACB_LNK_CHNG 21
#define	TNETD53XX_INTR_CNTRL_SRC_EMACB_MNGMNT	22
/* reserved sources... */

/* Plese DO NOT delete this any more... */
#define TNETD53XX_INTR_LEVEL_LOW         0x01
#define TNETD53XX_INTR_LEVEL_HIGH        0x00
#define TNETD53XX_INTR_TRIG_LEVEL        0x00
#define TNETD53XX_INTR_TRIG_EDGE         0x01


/*******************************************************************************************
*	HDLC
******************************************************************************************** */

/*******************************************************************************************
*	CS0 (FLASH)
******************************************************************************************** */
#define	TNETD53XX_FLASH_BASE		0x10000000

/*******************************************************************************************
*	CS1 (SDRAM)
******************************************************************************************** */
#define TNETD53XX_SDRAM_BASE            0x14000000

/*******************************************************************************************
*	CS2 (SDRAM)
******************************************************************************************** */

/*******************************************************************************************
*	CS3 (Asynchronous RAM)
******************************************************************************************** */
#define TNETD53XX_CS3_BASE              0x1C000000

/*******************************************************************************************
*	CS4 (Asynchronous RAM)
******************************************************************************************** */

/*******************************************************************************************
*	CS5 (Asynchronous RAM)
******************************************************************************************** */

/*******************************************************************************************
*	Internal ROM
******************************************************************************************** */

/*******************************************************************************************
*	PCI
******************************************************************************************** */
/* TLB defines to access PCI memory */
#define TLB_PAGESIZE_4KB	0x000
#define TLB_PAGESIZE_16KB	0x003
#define TLB_PAGESIZE_64KB	0x00F
#define TLB_PAGESIZE_256KB	0x03F
#define TLB_PAGESIZE_1MB	0x0FF
#define TLB_PAGESIZE_4MB	0x3FF
#define TLB_PAGESIZE_16MB	0xFFF

/* ODD/EVEN physical address selector bit */
#define TLB_PAGESIZE_4KB_ODD_PHYSICAL_ADDRESS 	 (1ul << 12)
#define TLB_PAGESIZE_16KB_ODD_PHYSICAL_ADDRESS   (1ul << 14)
#define TLB_PAGESIZE_64KB_ODD_PHYSICAL_ADDRESS   (1ul << 16)
#define TLB_PAGESIZE_256KB_ODD_PHYSICAL_ADDRESS  (1ul << 18)
#define TLB_PAGESIZE_1MB_ODD_PHYSICAL_ADDRESS    (1ul << 20)
#define TLB_PAGESIZE_4MB_ODD_PHYSICAL_ADDRESS    (1ul << 22)
#define TLB_PAGESIZE_16MB_ODD_PHYSICAL_ADDRESS   (1ul << 24)

#define TLB_FLAGS_GLOBAL	0x0001
#define TLB_FLAGS_VALID		0x0002
#define TLB_FLAGS_WRITEABLE	0x0004
#define TLB_FLAGS_CACHEABLE	0x0008

#ifndef NTLBID
#define NTLBID		16	/* max number of tlb entry pairs */
#endif


/*******************************************************************************************
*	Jade core specific
******************************************************************************************** */

#ifdef  BIG_ENDIAN
#define TNETD53XX_REG16_OFFSET 2
#elif defined(LITTLE_ENDIAN)
#define TNETD53XX_REG16_OFFSET 0
#else
#error Need to define endianism in hswpform.h
#endif

#define KSEG0_BASE			0x80000000
#define KSEG1_BASE			0xA0000000
#define KSEGMASK			0x1FFFFFFF

#define K0BASE                          0x80000000
#define K1BASE                          0xA0000000

#ifndef	PHYS_ADDR
#define PHYS_ADDR(X)                    ((X) & 0X1FFFFFFF)
#endif

#ifndef	PHYS_TO_K0
#define PHYS_TO_K0(X)                   (PHYS_ADDR(X)|K0BASE)
#endif

#ifndef	PHYS_TO_K1
#define PHYS_TO_K1(X)                   (PHYS_ADDR(X)|K1BASE)
#endif

#ifndef TNETD53XX_REG8
#define TNETD53XX_REG8( addr )              (*(volatile UINT8 *) PHYS_TO_K1(addr))
#endif

#ifndef TNETD53XX_REG16
#define TNETD53XX_REG16( addr )     		(*(volatile UINT16 *)PHYS_TO_K1(addr))
#endif

#ifndef TNETD53XX_REG32
#define TNETD53XX_REG32( addr )     		(*(volatile UINT32 *)PHYS_TO_K1(addr))
#endif

#ifdef	_LINK_KSEG0_			/* Application is linked into KSEG0 space */
#define	VIRT_ADDR(X)			PHYS_TO_K0(PHYS_ADDR(X))
#endif

#ifdef	_LINK_KSEG1_			/* Application is linked into KSEG1 space */
#define	VIRT_ADDR(X)			PHYS_TO_K1(PHYS_ADDR(X))
#endif

#if !defined(_LINK_KSEG0_) && !defined(_LINK_KSEG1_)
#error	You must define _LINK_KSEG0_ or _LINK_KSEG1_ to compile the code.
#endif


#define C0_PRID_PROC_IMP_MASK 		0x0000FF00
#define C0_PRID_PROC_REV_MASK 		0x000000FF
#define MIPS_4000_ID			0x00000400
#define MIPS_4000C_ID			0x00008000
#define TLBHI_VPNMASK			0xfffff000

/* Below are Jade core specific */
#define CFG0_4K_IL_MASK   		0x00380000
#define CFG0_4K_IL_SHIFT		19
#define CFG0_4K_IA_MASK			0x00070000
#define CFG0_4K_IA_SHIFT		16
#define CFG0_4K_IS_MASK			0x01c00000
#define CFG0_4K_IS_SHIFT		22

#define CFG0_4K_DL_MASK   		0x00001c00
#define CFG0_4K_DL_SHIFT		10
#define CFG0_4K_DA_MASK			0x00000380
#define CFG0_4K_DA_SHIFT		7
#define CFG0_4K_DS_MASK			0x0000E000
#define CFG0_4K_DS_SHIFT		13

#define C0_TAGHI			$29

/* These ugly macros are to access the -1 registers, like config1 */
#define MFC0_SEL1_OPCODE(dst, src)\
	  	.word (0x40000000 | ((dst)<<16) | ((src)<<11) | 1);\
		nop; \
		nop; \
		nop

#define MTC0_SEL1_OPCODE(dst, src)\
	  	.word (0x40800000 | ((dst)<<16) | ((src)<<11) | 1);\
		nop; \
		nop; \
		nop

#ifdef _ASMLANGUAGE
/* The GET_PHYA values assume registers as inputs */
#define GET_PHYA(R)			and R, R, ~KSEGMASK;
#define KSEG0A(R) 			GET_PHYA(R) \
					or R, KSEG0_BASE;

#define KSEG1A(R)			GET_PHYA(R) \
					or R, R, KSEG1_BASE;
                                    
#define GET_PHY(A)			((A) & KSEGMASK)
#define KSEG0(A)			(GET_PHY(A)|KSEG0_BASE)
#define KSEG1(A)			(GET_PHY(A)|KSEG1_BASE)

#else /* C or C++ code */
#ifndef KSEG0
#define KSEG0(X)			(GET_PHY(X) | KSEG0_BASE)
#endif

#ifndef KSEG1
#define KSEG1(X)			(GET_PHY(X) | KSEG1_BASE)
#endif

#ifndef GET_PHY
#define GET_PHY(X)			((X) & KSEGMASK)
#endif
#endif /* _ASMLANGUAGE */

#ifndef	_ASMLANGUAGE

/**************************************
* Interrupt controller interface 
***************************************/
/* Interrupt service routine */
typedef	void (*TNETD53XX_INTR_CNTRL_ISR)(void*);

extern void	tnetd53xx_intr_cntrl_interrupt(void);
extern int	tnetd53xx_intr_cntrl_iconnect(
   UINT32 source,
   TNETD53XX_INTR_CNTRL_ISR isr,
   void* arg
   );

extern int	tnetd53xx_intr_cntrl_init( void );
extern int	tnetd53xx_intr_cntrl_ienable( UINT32 source );
extern int	tnetd53xx_intr_active_level( UINT32 source, UINT32 level, UINT32 edge);


/********************** ATM Interface Functions ****************************************/

/* Interface functions for END driver */
//extern void* get_end_driver_control_device();

/* Interrupt controller interface */
extern void tnetd53xx_tcr_post_a_service_routine(void);
extern void tnetd53xx_tcr_post_b_service_routine(void);
extern void tnetd53xx_rcr_post_a_service_routine(void);
extern void tnetd53xx_rcr_post_b_service_routine(void);		
extern UINT32 tnetd53xx_atm_status(void);

/* User-provided routines */
extern UINT32 *sysAtmAlignedMalloc(UINT32 nBytes, UINT32 ByteAlignVal);
extern void sysAtmFree(void *pointer);

extern int  tnetd53xx_wdt_program_disable_timer_value(UINT16 disable_value);
extern void tnetd53xx_wdt_initialization(void);
extern int  tnetd53xx_wdt_program_lock_register(UINT16 kick_value);
extern int  tnetd53xx_wdt_program_initial_timer_value(UINT16 initial_timer_value);
extern int  tnetd53xx_wdt_program_prescal_value(UINT16 prescale_value);
extern int  tnetd53xx_wdt_program_disable_timer_value(UINT16 disable_value);
extern int  tnetd53xx_wdt_is_available(void);

#endif	/* _ASMLANGUAGE */

#endif	/* ___TNETD53XX_H___ */


