/****************************************************************************
* FILE PURPOSE:
*****************************************************************************
* FILE NAME:
*
*
*
* (C) Copyright 2001 Telogy Networks, Inc.
*
*****************************************************************************/

/*******************************************************************************************
*	tnetd53xxerr.h		TYPEDEFs, CONSTs, and API for TNETD53xx Errors
********************************************************************************************
*	PLEASE, READ THE NOTES BELOW TO BETTER UNDERSTAND HOW THIS FILE IS STRUCTURED.
*
*	These are const, typedef, and api definitions for Avalanche.
*
*	NOTES:
******************************************************************************************** */

#ifndef ___TNETD53XXERR_H___
#define ___TNETD53XXERR_H___
#ifndef _ASMLANGUAGE		/* This file is not for assembly language */

/* Error definitions */
typedef	enum TNETD53XX_ERR_t
{
	TNETD53XX_ERR_OK		= 0,	/* OK or SUCCESS */
	TNETD53XX_ERR_ERROR		= -1,	/* Unspecified/Generic ERROR */

	/* Pointers and args */
	TNETD53XX_ERR_INVARG		= -2,	/* Invaild argument to the call */
	TNETD53XX_ERR_NULLPTR		= -3,	/* NULL pointer */
	TNETD53XX_ERR_BADPTR		= -4,	/* Bad (out of mem) pointer */

	/* Memory issues */
	TNETD53XX_ERR_ALLOC_FAIL	= -10,	/* allocation failed */
	TNETD53XX_ERR_FREE_FAIL		= -11,	/* free failed */
	TNETD53XX_ERR_MEM_CORRUPT	= -12,	/* corrupted memory */
	TNETD53XX_ERR_BUF_LINK		= -13,	/* buffer linking failed */

	/* Device issues */
	TNETD53XX_ERR_DEVICE_TIMEOUT	= -20,	/* device timeout on read/write */
	TNETD53XX_ERR_DEVICE_MALFUNC	= -21,	/* device malfunction */
	TNETD53XX_ERR_DEVICE_NOT_EXIST	= -22,	/* device not exist*/
	TNETD53XX_ERR_DEVICE_NOT_READY	= -23,	/* device not ready */

	TNETD53XX_ERR_INVID		= -30	/* Invalid ID */

} TNETD53XX_ERR;

typedef	enum TNETD53XX_MODULE_ID_t
{
	TNETD53XX_MODULE_ID_SRAM	= 0,
	TNETD53XX_MODULE_ID_BBIF,
	TNETD53XX_MODULE_ID_BBIF_CNTRL,
	TNETD53XX_MODULE_ID_ATM,
	TNETD53XX_MODULE_ID_USB_MCU,
	TNETD53XX_MODULE_ID_DSP,
	TNETD53XX_MODULE_ID_DES,
	TNETD53XX_MODULE_ID_EMAC,
	TNETD53XX_MODULE_ID_MEM_CNTRL,
	TNETD53XX_MODULE_ID_GPIO,
	TNETD53XX_MODULE_ID_CLK_CNTRL,
	TNETD53XX_MODULE_ID_WDT,
	TNETD53XX_MODULE_ID_TIMER,
	TNETD53XX_MODULE_ID_UART,
	TNETD53XX_MODULE_ID_I2CM,
	TNETD53XX_MODULE_ID_USB,
	TNETD53XX_MODULE_ID_PCI_CFG,
	TNETD53XX_MODULE_ID_DMA,
	TNETD53XX_MODULE_ID_RST_CNTRL,
	TNETD53XX_MODULE_ID_BIST_CNTRL,
	TNETD53XX_MODULE_ID_DSP_CNTRL,
	TNETD53XX_MODULE_ID_CODEC,
	TNETD53XX_MODULE_ID_DCL,
	TNETD53XX_MODULE_ID_DSL,
	TNETD53XX_MODULE_ID_INTR_CNTRL,
	TNETD53XX_MODULE_ID_HDLC,
	TNETD53XX_MODULE_ID_ROM,
	TNETD53XX_MODULE_ID_PCI
} TNETD53XX_MODULE_ID;



/* *************************************************************************
	Function that returns a human-readable description of the error
**************************************************************************** */
typedef	int (*TNETD53XX_LOG_FN)(char* fmtstr, ... );

extern	TNETD53XX_ERR	tnetd53xx_err_init(TNETD53XX_MODULE_ID module, void* log_fn, unsigned int mask);

extern	TNETD53XX_ERR tnetd53xx_err_log(TNETD53XX_ERR error, unsigned int mask,
				  TNETD53XX_MODULE_ID module, char* fmtstr, ...);

/* *************************************************************************
	These two functions set the logging bit for the specified module.
	The layout and meaning of the logging configuration is up to the module.
**************************************************************************** */
extern	TNETD53XX_ERR	tnetd53xx_err_mask_set(TNETD53XX_MODULE_ID module, unsigned int mask);
extern	unsigned int	tnetd53xx_err_mask_get(TNETD53XX_MODULE_ID module);

/* Error reporting capability */
#ifndef	TNETD53XX_NO_LOG

/* *************************************************************************************
	The user must define TNETD53XX_MODULE in the module .c file to be one of the
	module ID's defined in tnetd53xxerr.h file. For example, in emac.c, the
	user should define this constant as TNETD53XX_MODULE_ID_EMAC.
**************************************************************************************** */
#define	TNETD53XX_PERROR(error,fmtstr,a1,a2,a3,a4,a5)\
	tnetd53xx_err_log(error,0xFFFFFFFF,TNETD53XX_MODULE,fmtstr,a1,a2,a3,a4,a5)

#define	TNETD53XX_LOG(mask,fmtstr,a1,a2,a3,a4,a5)\
	tnetd53xx_err_log(TNETD53XX_ERR_OK,mask,TNETD53XX_MODULE,fmtstr,a1,a2,a3,a4,a5)

#else	/* For production code, we can take all logs out (compile-time macro TNETD53XX_NO_LOG must be defined) */

#define	TNETD53XX_PERROR(error, fmtstr, a1, a2, a3, a4, a5)	{/* do nothing */} 
#define	TNETD53XX_LOG(mask, fmtstr, log_f, a1, a2, a3, a4)	{/* do nothing */}

#endif	/* TNETD53XX_NO_LOG */

#endif	/* _ASMLANGUAGE */
#endif	/* ___TNETD53XXERR_H___ */


