/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*
 *  ------------------------------------------------------------------------------
 *   Module      : vlnq.c
 *   Owner       : 
 *  ------------------------------------------------------------------------------
 *   Application : Cable Modem
 *   Subsystem   : Infrastructure / BSP
 *  ------------------------------------------------------------------------------
 *   Description :
 *   The module implements the VLYNQ device driver to interface with any VLYNQ
 *   device connected to PUMA or higher CPU.
 *   Based on Yaron Vaknin's VLYNQ-VDSP code.
 *   This file is included from "tnetd53xx.h"
 *  ------------------------------------------------------------------------------
 *   Author    : Joseph Gabbay
 *  ------------------------------------------------------------------------------
 */
#define _VLYNQ_C_

#include "pal.h"


/************
 * Includes
 ************/                      
#define _LINK_KSEG0_
#define LITTLE_ENDIAN

#include "tnetd53xx.h"
#include "vlynq.h"

/****************
 * Local Macros
 ****************/

/* local module */
#define VLYNQ_CTL              (*TNETD53XX_VLYNQ_CTL(pDev->module_base))
#define VLYNQ_STS              (*TNETD53XX_VLYNQ_STS(pDev->module_base))
#define VLYNQ_TXMAP            (*TNETD53XX_VLYNQ_TXMAP(pDev->module_base))
#define VLYNQ_RXMAPSIZE(x)     (*TNETD53XX_VLYNQ_RXMAPSIZE(pDev->module_base, (x)))
#define VLYNQ_RXMAPOFST(x)     (*TNETD53XX_VLYNQ_RXMAPOFFSET(pDev->module_base, (x)))
#define VLYNQ_IVR(ivr)         (*TNETD53XX_VLYNQ_IVR(pDev->module_base, (ivr)))
#define VLYNQ_IVR_03TO00       (*TNETD53XX_VLYNQ_IVR_03TO00(pDev->module_base))
#define VLYNQ_IVR_07TO04       (*TNETD53XX_VLYNQ_IVR_07TO04(pDev->module_base))
#define VLYNQ_INTPTR           (*TNETD53XX_VLYNQ_INTPTR(pDev->module_base))
#define VLYNQ_INTCR            (*TNETD53XX_VLYNQ_INTCR(pDev->module_base))

/* remote module */
#define VLYNQ_R_CTL            (*TNETD53XX_VLYNQ_REMOTE_CTL(pDev->module_base))
#define VLYNQ_R_STS            (*TNETD53XX_VLYNQ_REMOTE_STS(pDev->module_base))
#define VLYNQ_R_TXMAP          (*TNETD53XX_VLYNQ_REMOTE_TXMAP(pDev->module_base))
#define VLYNQ_R_RXMAPSIZE(x)   (*TNETD53XX_VLYNQ_REMOTE_RXMAPSIZE(pDev->module_base, (x)))
#define VLYNQ_R_RXMAPOFST(x)   (*TNETD53XX_VLYNQ_REMOTE_RXMAPOFFSET(pDev->module_base, (x)))
#define VLYNQ_R_IVR(ivr)       (*TNETD53XX_VLYNQ_REMOTE_IVR(pDev->module_base, (ivr)))
#define VLYNQ_R_IVR_03TO00     (*TNETD53XX_VLYNQ_REMOTE_IVR_03TO00(pDev->module_base))
#define VLYNQ_R_IVR_07TO04     (*TNETD53XX_VLYNQ_REMOTE_IVR_07TO04(pDev->module_base))
#define VLYNQ_R_INTPTR         (*TNETD53XX_VLYNQ_REMOTE_INTPTR(pDev->module_base))
#define VLYNQ_R_INTCR          (*TNETD53XX_VLYNQ_REMOTE_INTCR(pDev->module_base))

/* helpers */
#define VLYNQ_ALIGN4(x)       ((x) &= ~3)

#define VLYNQ_RESETCB(arg) \
   if( pDev->reset_cb != NULL)   \
   {                             \
      (pDev->reset_cb)(pDev, (arg));  \
   }  

#define VLYNQ_STS_FLD_WIDTH(sts)    (((sts) & TNETD53XX_VLYNQ_STS_SWIDTH) >> 24)
#define VLYNQ_CTL_INTVEC(x)         (((x) & 31) << 8)

/*******************
 * Local Functions
 *******************/
static int           tnetd53xx_detectVlynqModule    ( VLYNQ_DEV *pDev);
static int           tnetd53xx_vlynqLinkCheck       ( VLYNQ_DEV *pDev);
static void          tnetd53xx_vlynqMicroDelay      ( UINT32  clktime);
static TNETD53XX_ERR tnetd53xx_vlynqResetModule     ( VLYNQ_DEV *pDev, int assert);
static void          tnetd53xx_vlynqLocalMdlHandler ( VLYNQ_DEV *pDev);
static void          tnetd53xx_vlynqRemoteMdlHandler( VLYNQ_DEV *pDev);
#ifndef __linux__
static TNETD53XX_ERR tnetd53xx_vlynqInterruptInit   ( VLYNQ_DEV *pDev);
#endif

/*
 * ----------------------------------------------------------------------------
 *  This function tests the CPU_TYPE and determines how many VLYNQ modules
 *  exist.
 *
 *  Returns numof modules or 0. and fills the module address in pDec structure.
 * -----------------------------------------------------------------------------
 */
static int tnetd53xx_detectVlynqModule( VLYNQ_DEV *pDev)
{
	UINT32 PumaVlBase[] = {TNETD53XX_PUMA_VLBASE_1};   /* Puma and Puma-S */ /* Demiurg : hopefully Sangam too */

   unsigned int   nAvailableModules = 0;
   unsigned short cpu_type;

#ifndef __linux__

   cpu_type= ReadCpuType();

   if( (CPU_PUMA == cpu_type) || (CPU_PUMAS == cpu_type))
   {

#endif

      nAvailableModules = VLYNQ_ARRSIZE(PumaVlBase);

      if(pDev != NULL)
      {
         /* check VLYNQ module index , must be 1,2,3,4,..,num of available modules */
         if( VLYNQ_INRANGE(pDev->dev_idx, nAvailableModules , 1) )
         {
            /* Get VLYNQ module base address and store it for later use. */
					 printk("TNETD73XX_VLYNQ0_BASE : %#x, PumaVlBase[pDev->dev_idx - 1] : %#x \n", TNETD73XX_VLYNQ0_BASE, PumaVlBase[pDev->dev_idx - 1]); 
					 pDev->module_base = PHYS_TO_K1(PumaVlBase[pDev->dev_idx - 1]);
         }
      }

#ifndef __linux__
   }
#endif

   return nAvailableModules;
}/* END tnetd53xx_detectVlynqModule() */

/*
 * ----------------------------------------------------------------------------
 *  This function tests the current VLYNQ for a link.  An
 *  arbitrary amount of time is allowed for the link to
 *  come up.
 *
 *  Returns OK if link exists, NOK if no link was found.
 * -----------------------------------------------------------------------------
 */
static int tnetd53xx_vlynqLinkCheck( VLYNQ_DEV *pDev)
{
	int i;

   /* give link chance to come up */
   for (i=0; (i<1000) && ((VLYNQ_STS & TNETD53XX_VLYNQ_STS_LINK)==0); i++) {
		 
	 };
	 
   /* check status register return OK if link is found. */

	 printk("VLYNQ_STS : %#x \n", VLYNQ_STS);

   if( VLYNQ_STS & TNETD53XX_VLYNQ_STS_LINK)
     return OK;

   return NOK;  /* link is not found */
} /* END tnetd53xx_vlynqLinkCheck() */


/*
 * ----------------------------------------------------------------------------
 * Function : tnetd53xx_vlynqMicroDelay
 *
 * Input   : UINT32 clktime
 *            designates the number of micro seconds to delay
 *
 * Output   : void
 *            			  
 * Process  :  Introdues a Delay for "clktime" CPU clocks. In this function, we have to
 *  wait for "clktime" by either reading the CPU count register or run a loop. In the
 *  case of reading the CPU count register, we have to pull in the BSP (and OS) specific
 *  assembly functions. In the case of C code, we can do this by declaring a volatile
 *  variable that tells the compiler not to do optimization on accesses to it (because
 *  at compile time the "volatile" means "i don't know where else this global variable
 *  may be accessed from). This way we escape from pulling in the BSP assembly functions.
 *  In either implementation, we have to guarantee that we wait the specified number of
 *  CPU clocks.
 *
 * Note(s)  : None
 * -----------------------------------------------------------------------------
 */
static void tnetd53xx_vlynqMicroDelay( UINT32 clktime)
{
   int i = 0;
   
   static volatile int vlynqMicroDelayValue = 0;
   volatile int    *ptr = &vlynqMicroDelayValue;
   
   *ptr = 0;
   
   clktime *= 80; /* ~4*23 */
   
   /* We are assuming that the each cycle takes about 23 assembly instructions. */
   for(i = 0; i < (clktime + 23)/23; i++)
   {
      *ptr = *ptr + 1;
   }
}/* END tnetd53xx_vlynqMicroDelay() */


/*
 * ----------------------------------------------------------------------------
 * Function : tnetd53xx_vlynqResetModule
 *
 * Input   : pDev
 *
 * Output   : TNETD53XX_ERR 
 *            			  
 * Process  :  Reset the module given by pDev->dev_idx from reset state
 *             in order to turn it on. Works with Puma/Puma-S.
 *
 * -----------------------------------------------------------------------------
 */
static TNETD53XX_ERR tnetd53xx_vlynqResetModule( VLYNQ_DEV *pDev, int assert)
{
   unsigned long oldLevel;
   volatile UINT32 *ptr;

   /*
      Check CPU type to see that it supports VLYNQ and how many modules
   */
   if( pDev->dev_idx < tnetd53xx_detectVlynqModule(NULL))
      return TNETD53XX_ERR_DEVICE_NOT_EXIST;

   ptr = (UINT32*)PHYS_TO_K1(TNETD53XX_RST_CNTRL_PRCR);

   if( assert)
   {
      /* put in reset state */
      /*oldLevel = rt_os_splx(MAX_ILEV); L.M */ /* lock interrupts */
      *ptr &= ~TNETD53XX_RST_CNTRL_PRCR_VLYNQ;
      /*rt_os_splx(oldLevel);  */ /* unlock interrupts */
      tnetd53xx_vlynqMicroDelay(8);
   }
   else
   {
      /* take out from reset state */
      /*oldLevel = rt_os_splx(MAX_ILEV); L.M*/ /* lock interrupts */
      *ptr |= TNETD53XX_RST_CNTRL_PRCR_VLYNQ;
      /*rt_os_splx(oldLevel);  */ /* unlock interrupts */
      /* wait at least 2000 clocks for VLYNQ to set up (according to module spec)
         0.08msec @ 25MHz. 1msec is more than enough. */
      PAL_sleepLong(100);
   }

   return TNETD53XX_ERR_OK;
}/* END tnetd53xx_vlynqResetModule() */


/*
 * ----------------------------------------------------------------------------
 * Function : tnetd53xx_vlynqLocalMdlHandler
 *
 * Input    : VLYNQ_DEV *pDev
 *    The handler argument. Calls user reporter handler
 *    pointed to by pDev->report_cb.
 *
 * Output   : void
 *            			  
 * Process  : This is the local VLYNQ module interrupt handler 
 *
 * -----------------------------------------------------------------------------
 */
static void tnetd53xx_vlynqLocalMdlHandler( VLYNQ_DEV *pDev)
{
   VLYNQ_REPORT_CB func;
   UINT32          dwStatRegVal;

   /* Callback function is read from the device pointer that
      is passed as an argument */
   func = pDev->report_cb;

   /* read local status register */
   dwStatRegVal = VLYNQ_STS;

   /* clear pending events */
   VLYNQ_STS = dwStatRegVal;

   /* call user reporter */
   if( func != NULL)
      func( pDev, VLYNQ_LOCAL_DVC, dwStatRegVal);

}/* END tnetd53xx_vlynqLocalMdlHandler() */

/*
 * ----------------------------------------------------------------------------
 * Function : tnetd53xx_vlynqRemoteMdlHandler
 *
 * Input    : VLYNQ_DEV *pDev
 *    The handler argument. Calls user reporter handler
 *    pointed to by pDev->report_cb.
 *
 * Output   : void
 *            			  
 * Process  : This is the remote VLYNQ module interrupt handler 
 *
 * -----------------------------------------------------------------------------
 */
static void tnetd53xx_vlynqRemoteMdlHandler( VLYNQ_DEV *pDev)
{
   VLYNQ_REPORT_CB func;
   UINT32          dwStatRegVal;

   /* Callback function is read from the device pointer that
      is passed as an argument */
   func = pDev->report_cb;

   /* read remote status register */
   dwStatRegVal = VLYNQ_R_STS;

   /* clear pending events */
   VLYNQ_R_STS = dwStatRegVal;

   /* call user reporter */
   if( func != NULL)
      func( pDev, VLYNQ_REMOTE_DVC, dwStatRegVal);

}/* END tnetd53xx_vlynqRemoteMdlHandler() */

#ifndef __linux__

/*
 * ----------------------------------------------------------------------------
 * Function : tnetd53xx_vlynqInterruptInit
 *
 * Input   : pDev
 *
 * Output   : TNETD53XX_ERR code
 *  TNETD53XX_ERR_ALLOC_FAIL - Function failed to allocate enough memory
 *    for the interrupt control blocks (vlynq interrupts) 		  
 *				
 *  TNETD53XX_ERR_ERROR - Function failed to connect the vlynqPumaRootHndlr, 
 *	   VLYNQ main interrupt handler to the general PUMA VLYNQ 
 *	   interrupt #21 on the Puma (root device)
 *	   Function failed to enable the general PUMA VLYNQ interrupt source
 *    #21 on the Puma (root device)
 *  TNETD53XX_ERR_OK - Function initialized the interrupt mechanism  successfully.
 *   									  
 * Process  : Function initialize the VLYNQ interrupt mechanism as follows:
 *		- Map remote interrupts to PUMA's VLYNQ interrupt status register
 *      (keep them as disable yet)
 *    - Initialize the root VLYNQ interrupt pointer register to point 
 *      the interrupt Pending/Set Register 
 *		- Install the root and remote VLYNQ modules interrupt handlers
 *		  into the VLYNQ interrupt mechanism
 *		- Enable the root and remote VLYNQ modules interrupts
 *		- Enable the general PUMA VLYNQ interrupt #21 on the Puma (root device)
 *		- Connect the vlynqPumaRootHndlr, VLYNQ main interrupt handler to the 
 *      general PUMA VLYNQ interrupt #21 on the Puma (root device)
 *		- Enable the general PUMA VLYNQ interrupt source #21 on the Puma (root device)
 *            			  
 * -----------------------------------------------------------------------------
 */
static TNETD53XX_ERR tnetd53xx_vlynqInterruptInit( VLYNQ_DEV *pDev)
{
   TNETD53XX_ERR retCode = TNETD53XX_ERR_OK;
   UINT32 val;
/*   int i;*/

   /* Allocate memory for VLYNQ_INTR_CNTRL_ICBs */
   pDev->pIntrCB = (VLYNQ_INTR_CNTRL_ICB *)PAL_memAlloc( 0,32*sizeof(VLYNQ_INTR_CNTRL_ICB),0,0);

   if( pDev->pIntrCB == NULL)
   {
      retCode = TNETD53XX_ERR_ALLOC_FAIL;
      return retCode;
   }

   /* Map interrupts, disable all. except for root IRQ.
    * bit0 - Local VLYNQ device (Puma)
    * bit1 - Remote VLYNQ device.
    */
   VLYNQ_IVR_03TO00 = 0;
   VLYNQ_IVR_07TO04 = 0;

   /* local interrupt pointer register should point to
    * the Interrupt Pending/Set register
    */
   VLYNQ_INTPTR = TNETD53XX_VLYNQ_INTSR_OFFSET;
   
   /* Map local and remote VLYNQ modules interrupt handlers. */
   pDev->pIntrCB[VLYNQ_MODULE_LOCAL].isr = tnetd53xx_vlynqLocalMdlHandler;
   pDev->pIntrCB[VLYNQ_MODULE_LOCAL].arg = pDev;
   pDev->pIntrCB[VLYNQ_MODULE_REMOTE].isr = tnetd53xx_vlynqRemoteMdlHandler;
   pDev->pIntrCB[VLYNQ_MODULE_REMOTE].arg = pDev;

   /* Enable the local and remote VLYNQ modules interrupts. */
   /* Configure the local VLYNQ's control register
      This enables the VLYNQ module status interrupt on the Puma (which means a VLYNQ
      error has occurred if you get this interrupt), tells the VLYNQ to write the 
      Interrupt Pending/Set register with the status received in Interrupt packets from
      the remote, and tells the VLYNQ to post interrupts in the Interrupt Status/Clear
      register.  We'll choose to leave "intvec" field of the control register as a 0 
      (the reset value), which means that bit 0 of the Interrupt Status register will
      correspond to the local VLYNQ device
   */
   val = VLYNQ_CTL;
   val &= ~TNETD53XX_VLYNQ_CTL_INTVEC;
   val |= (TNETD53XX_VLYNQ_CTL_INTENABLE
         | TNETD53XX_VLYNQ_CTL_INT2CFG   /* consider 8 LSB of VLYNQ_INTPTR */
         | TNETD53XX_VLYNQ_CTL_INTLOCAL  /* local interrupts */
         | VLYNQ_CTL_INTVEC(VLYNQ_MODULE_LOCAL));
   VLYNQ_CTL = val;

   /* Configure the remote VLYNQ's control register
      This enables the remote VLYNQ's module status interrupt, and sets the "intvec" field 
      of the remote control register to '1', meaning that bit 1 of the Interrupt Status 
      register will correspond to the remote VLYNQ device!
   */ 
   val = VLYNQ_R_CTL;
   val &= ~TNETD53XX_VLYNQ_CTL_INTVEC;
   val |= (TNETD53XX_VLYNQ_CTL_INTENABLE
         | VLYNQ_CTL_INTVEC(VLYNQ_MODULE_REMOTE));
   VLYNQ_R_CTL = val;

   /* Connect tnetd53xx_vlynq_RootISR, VLYNQ main interrupt handler to the 
      general PUMA VLYNQ interrupt #21 on the Puma (root device) */
   retCode = sysHwIntConnect(
      TNETD53XX_INTR_CNTRL_SRC_PRIMARY | TNETD53XX_INTR_CNTRL_SRC_VLYNQ,
		tnetd53xx_vlynq_RootISR,
      pDev);

   /* Enable the general PUMA VLYNQ interrupt source #21 on the Puma (root device) */
   if( TNETD53XX_ERR_OK == retCode)
   {
      retCode = sysHwVlynqIntEnable();
   }
   
   /* If failure, return allocated resources to the system */
   if( TNETD53XX_ERR_OK != retCode)
   {
      /* Handle failure */
      PAL_memFree(0,pDev->pIntrCB,0);
   }

   return retCode;
}/* END tnetd53xx_vlynqInterruptInit() */

#endif // __linux__

static void tnetd53xx_vlynq_ConfigClock(VLYNQ_DEV* pDev)
{
   UINT32 tmp;

   switch( pDev->clk_source)
   {
   case VLYNQ_CLK_SOURCE_LOCAL:  /* we output the clock, clk_div in range [1..8]. */
      tmp = ((pDev->clk_div - 1) << 16) | TNETD53XX_VLYNQ_CTL_CLKDIR;
      VLYNQ_CTL = tmp;
      break;
   case VLYNQ_CLK_SOURCE_REMOTE: /* we need to set the clock pin as input */
     tmp = ((pDev->clk_div - 1) << 16) | TNETD53XX_VLYNQ_CTL_CLKDIR;
     VLYNQ_R_CTL = tmp;
      VLYNQ_CTL = 0ul;
      break;
   default:                      /* do nothing about the clock, but clear other bits. */
      tmp = ~(TNETD53XX_VLYNQ_CTL_CLKDIR | TNETD53XX_VLYNQ_CTL_CLKDIV);
      VLYNQ_CTL &= tmp;
      break;
   }
}/* END tnetd53xx_vlynq_ConfigClock() */


/*
 * ----------------------------------------------------------------------------
 * Function : tnetd53xx_vlynq_Init
 *
 * Input    : VLYNQ_DEV *pDev
 *            Structure elements, set to 0 all non relevant elements.
 *            See vlynq.h for field details:
 *  uint              dev_idx;	    1,2,3... index of VLYNQ module
 *  UINT32            module_base;   Set to 0. Filled by the driver.
 *  eVLYNQ_DRV_STATE  state;         Set to VLYNQ_DRV_STATE_UNINIT.
 *  eVLYNQ_CLK_SOURCE	clk_source;	 Set according to your HW.
 *  uint              clk_div;       Set VLYNQ clock divider [1..8]
 *  eVLYNQ_BUS_WIDTH  width;		    Valid VLYNQ bus width, filled by driver.
 *  VLYNQ_MEMORY_MAP	local_mem;	    Set to address of device memory window.
 *  VLYNQ_MEMORY_MAP	remote_mem;	    Set memory regions of the remote device.
 *  VLYNQ_INTR_CNTRL_ICB *pIntrCB;   Interrupt ICB. set to NULL.
 *  VLYNQ_REPORT_CB	report_cb;	 
 *  VLYNQ_RESET_REMOTE reset_cb;   
 *
 *  UINT32 opt  Option mask (bit field)
 *
 * Output   : Type:  TNETD53XX_ERR
 *            Value:
 *            TNETD53XX_ERR_OK - success.
 *            Error code - Failure.
 *            			  
 * Process  : Function initializes the VLYNQ module. Must be called first.
 *
 * -----------------------------------------------------------------------------
 */
TNETD53XX_ERR tnetd53xx_vlynq_Init( VLYNQ_DEV* pDev, UINT32 opt)
{
   unsigned int nAvailableModules;
   volatile TNETD53XX_ERR retCode;
   UINT32 tmp;


   /* check driver state, must be "uninit"ialized. */
   if( pDev->state != VLYNQ_DRV_STATE_UNINIT)
   {
      return TNETD53XX_ERR_ERROR;  /* already initialized ?? */
   }
 
   /*
      Check CPU type to see that it supports VLYNQ and how many modules
      if not exist, set nAvailableModules to 0 to fail the function.
   */
   nAvailableModules = tnetd53xx_detectVlynqModule(pDev);
	 
   /* check VLYNQ module index , must be 1,2,3,4,..,num of available modules */
   if( VLYNQ_OUTRANGE(pDev->dev_idx, nAvailableModules , 1) )
   {
      return TNETD53XX_ERR_DEVICE_NOT_EXIST;
   }
 
   /* update the driver state */
   pDev->state = VLYNQ_DRV_STATE_ININIT;

   /* validate arguments */
   if(   VLYNQ_OUTRANGE(pDev->clk_source, VLYNQ_CLK_SOURCE_REMOTE, VLYNQ_CLK_SOURCE_NONE)
      || VLYNQ_OUTRANGE(pDev->clk_div, 8 , 1)
      || (pDev->pIntrCB != NULL)
      )
   {
      return TNETD53XX_ERR_INVARG;
   }
 
   /* Put local VLYNQ module in reset state. */
   retCode = tnetd53xx_vlynqResetModule(pDev, 1);
   if( retCode != TNETD53XX_ERR_OK)
   {
      return retCode;
   }
 
   /* Assert reset the remote device, call reset_cb, reset CB holds Reset according to
      the device needs. */
   VLYNQ_RESETCB( VLYNQ_RESET_ASSERT);

   /* Reset the VLYNQ module, and wait for it to be ready. */
   retCode = tnetd53xx_vlynqResetModule(pDev, 0);
   if( retCode != TNETD53XX_ERR_OK)
   {
      return retCode;
   }
   
   /* Handle VLYNQ clock, HW default (Sense On Reset) is
      usually input for all the devices. */
   if( pDev->clk_source == VLYNQ_CLK_SOURCE_LOCAL) {
     if( opt & VLYNQ_OPTION_INIT_CONFIG_CLK) {
      tnetd53xx_vlynq_ConfigClock(pDev);
   }
   }

   /* Call reset_cb again. It will release the remote device
      from reset, and wait for a while.
   */
	 
   VLYNQ_RESETCB( VLYNQ_RESET_DEASSERT);
	 
   /* If clock source is the remote device, initialize it if there is a link */
   if( pDev->clk_source == VLYNQ_CLK_SOURCE_REMOTE)
   {
      if( opt & VLYNQ_OPTION_INIT_CONFIG_CLK)
      {
         if( tnetd53xx_vlynqLinkCheck(pDev) == OK)
            tnetd53xx_vlynq_ConfigClock(pDev);
      }
   }

   if( opt & VLYNQ_OPTION_INIT_CHECK_LINK)
   {
      /* Now comes the moment of truth... Check for a valid link */
      if( tnetd53xx_vlynqLinkCheck(pDev) != OK)
      {
         /* Handle this case ans abort */
         pDev->state = VLYNQ_DRV_STATE_ERROR;
         VLYNQ_RESETCB( VLYNQ_RESET_INITFAIL);
         return TNETD53XX_ERR_ERROR;
      }
   }


   /* Avoid clock generators clashing */
   if( opt & VLYNQ_OPTION_INIT_CONFIG_CLK)
   {
      if( tnetd53xx_vlynqLinkCheck(pDev) == OK)
      {
         if( pDev->clk_source == VLYNQ_CLK_SOURCE_LOCAL)
         {
            /* If clock source is local, turn off the remote clock */
            VLYNQ_R_CTL = 0;
         }
         else if( pDev->clk_source == VLYNQ_CLK_SOURCE_REMOTE)
         {
            /* If clock source is remote, turn off the remote clock */
            VLYNQ_CTL = 0;
         }
	  else
         {
            /* If clock source is external, turn off the local and remote clocks */
            VLYNQ_R_CTL = 0;
            VLYNQ_CTL = 0;
         }
   }
   else
      {
         if( opt & VLYNQ_OPTION_INIT_CHECK_LINK)
         {
            /* Handle this case and abort */
            pDev->state = VLYNQ_DRV_STATE_ERROR;
            VLYNQ_RESETCB( VLYNQ_RESET_INITFAIL);
            return TNETD53XX_ERR_ERROR;
         }
      }
   }
 
   /* Configure VLYNQ memory mapping, host side */
   if( opt & VLYNQ_OPTION_INIT_HOST_REGIONS)
   {
      int i;
			
      /* configure the VLYNQ portal window to a PHYSICAL address of the local CPU */
      VLYNQ_ALIGN4(pDev->local_mem.TxMap);
      VLYNQ_TXMAP = PHYS_ADDR(pDev->local_mem.TxMap);
      for( i=0; i<TNETD53XX_VLYNQ_MAX_MEMORY_REGIONS; i++)
      {
         /* Physical address on the remote */
         int map = i+1;
         VLYNQ_R_RXMAPSIZE(map) =  0;
         if( pDev->remote_mem.RxSize[i])
         {
            VLYNQ_ALIGN4(pDev->remote_mem.RxOffset[i]);            
            VLYNQ_ALIGN4(pDev->remote_mem.RxSize[i]);
            VLYNQ_R_RXMAPOFST(map) =  pDev->remote_mem.RxOffset[i];
            VLYNQ_R_RXMAPSIZE(map) =  pDev->remote_mem.RxSize[i];
         }
      }
   }


   /* Configure VLYNQ memory mapping, remote side */
   if( opt & VLYNQ_OPTION_INIT_REMOTE_REGIONS)
   {
      int i;
			
      /* configure the VLYNQ portal window to a PHYSICAL address of the remote */
      VLYNQ_ALIGN4(pDev->remote_mem.TxMap);            
      VLYNQ_R_TXMAP = pDev->remote_mem.TxMap;
      for( i=0; i<TNETD53XX_VLYNQ_MAX_MEMORY_REGIONS; i++)
      {
         /* Physical address on the local */
         int map = i+1;
         VLYNQ_RXMAPSIZE(map) =  0;
         if( pDev->local_mem.RxSize[i])
         {
            VLYNQ_ALIGN4(pDev->local_mem.RxOffset[i]);            
            VLYNQ_ALIGN4(pDev->local_mem.RxSize[i]);
            VLYNQ_RXMAPOFST(map) =  PHYS_ADDR(pDev->local_mem.RxOffset[i]);
            VLYNQ_RXMAPSIZE(map) =  PHYS_ADDR(pDev->local_mem.RxSize[i]);
         }
      }
   }


   /* Calculate VLYNQ bus width */
   pDev->width = 3 + VLYNQ_STS_FLD_WIDTH(VLYNQ_STS) + VLYNQ_STS_FLD_WIDTH(VLYNQ_R_STS);

   /* chance to initializa the device, e.g. to boost
      VLYNQ clock by modifying pDev->clk_div or and verify the width. */
   VLYNQ_RESETCB( VLYNQ_RESET_LINKESTABLISH);

   /* Handle VLYNQ clock, HW default (Sense On Reset) is
      usually input for all the devices. */
   if( opt & VLYNQ_OPTION_INIT_CONFIG_CLK)
   {
      tnetd53xx_vlynq_ConfigClock(pDev);
   }


   /* last check */
   if( opt & VLYNQ_OPTION_INIT_CHECK_LINK)
   {
      if( tnetd53xx_vlynqLinkCheck(pDev) != OK)
      {
         /* Handle this case ans abort */
         pDev->state = VLYNQ_DRV_STATE_ERROR;
         VLYNQ_RESETCB( VLYNQ_RESET_INITFAIL);
         return TNETD53XX_ERR_ERROR;
      }
   }
	 
   /* Clear errors in case they occured. wc bits in status register */
   if( opt & VLYNQ_OPTION_INIT_CLEAR_ERRORS)
   {
      tmp = VLYNQ_STS & (TNETD53XX_VLYNQ_STS_RERROR | TNETD53XX_VLYNQ_STS_LERROR);
      VLYNQ_STS = tmp;
      tmp = VLYNQ_R_STS & (TNETD53XX_VLYNQ_STS_RERROR | TNETD53XX_VLYNQ_STS_LERROR);
      VLYNQ_R_STS = tmp;
   }

#ifndef __linux__

   /* Set up interrupts */
   if( opt & VLYNQ_OPTION_INIT_INTERRUPTS )
   {
      retCode = tnetd53xx_vlynqInterruptInit(pDev);
      if( TNETD53XX_ERR_OK != retCode)
      {
         pDev->state = VLYNQ_DRV_STATE_ERROR;
         VLYNQ_RESETCB( VLYNQ_RESET_INITFAIL2);
         return retCode;
      }
   }
#else
	dda_init_vlynq_int();	
#endif

   /* finish initialization */
   pDev->state = VLYNQ_DRV_STATE_RUN;
   VLYNQ_RESETCB( VLYNQ_RESET_INITOK);
   return TNETD53XX_ERR_OK;
}/* END tnetd53xx_vlynq_Init() */



/*
 * ----------------------------------------------------------------------------
 * Function : tnetd53xx_vlynq_Install_ISR
 *
 * Input    : VLYNQ_DEV *pDev
 *   eVLYNQ_REMOTE_INTRRUPT_SRC   nSrc = the IRQ
 *   int          nMapVec, [0 - 31] = What bit remote writes in order to send the IRQ
 *   eVLYNQ_INTR_POLARITY         nPol,
 *   eVLYNQ_INTR_TYPE             nInt,
 *   VLYNQ_INTR_CNTRL_ISR         isr,
 *   void                        *arg       normally set to same value as pDev
 *
 *
 * Output   : Type:  TNETD53XX_ERR
 *            Value:
 *            TNETD53XX_ERR_OK - success.
 *            Error code - Failure.
 *            			  
 * Process  : Function installs an ISR and map it to a VLYNQ vector.
 *            The ISR serves the remote device.
 *
 * -----------------------------------------------------------------------------
 */
TNETD53XX_ERR tnetd53xx_vlynq_Install_ISR
(
    VLYNQ_DEV                   *pDev,
    eVLYNQ_REMOTE_INTRRUPT_SRC   nSrc,
    int                          nMapVec, /* 0 - 31 */
    eVLYNQ_INTR_POLARITY         nPol,
    eVLYNQ_INTR_TYPE             nIntT,
    VLYNQ_INTR_CNTRL_ISR         isr,
    void                        *arg  /* normally should be set to same
                                         value as pDev (NULL has same affect) */
)
{
   unsigned long oldLevel;
   UINT32   val, mask, sm;

   /* Ensure VLYNQ driver is running properly */
   if( VLYNQ_DRV_STATE_RUN != pDev->state )
   {
      return TNETD53XX_ERR_ERROR;
   }

   /* Test arguments */
   if(   isr  == NULL
      || VLYNQ_OUTRANGE( nSrc, VLYNQ_REMOTE_NUM_OF_INTRRUPTS-1, 0)
      || (nMapVec == VLYNQ_MODULE_LOCAL) || (nMapVec == VLYNQ_MODULE_REMOTE) /* reserved bits */
      || VLYNQ_OUTRANGE( nMapVec, 31, 0)
      || VLYNQ_OUTRANGE( nPol , VLYNQ_INTR_ACTIVE_LOW, VLYNQ_INTR_ACTIVE_HIGH)
      || VLYNQ_OUTRANGE( nIntT, VLYNQ_INTR_PULSED, VLYNQ_INTR_LEVEL)
   )
   {
      return TNETD53XX_ERR_INVARG;
   }

   /* override arg if NULL */
   if( arg == NULL)
      arg = pDev;

   mask = ~TNETD53XX_VLYNQ_IVR_INTALL(nSrc);
   sm = TNETD53XX_VLYNQ_IVR2FIELD( nMapVec & 0x1F, nSrc);
   if( VLYNQ_INTR_PULSED == nIntT )
      sm |= TNETD53XX_VLYNQ_IVR_INTTYPE(nSrc);
   if( VLYNQ_INTR_ACTIVE_LOW == nPol)
      sm |= TNETD53XX_VLYNQ_IVR_INTPOL(nSrc);

   /* critical section begin */
   /*oldLevel = rt_os_splx(MAX_ILEV);L.M */

   /* set up the interrupt, it is not enabled yet */
   val  = ((VLYNQ_R_IVR( nSrc) & mask) | sm);
   VLYNQ_R_IVR( nSrc) = val;
   
   /* Install the interrupt handler */
   pDev->pIntrCB[nMapVec].isr = isr;
   pDev->pIntrCB[nMapVec].arg = arg;

   /* critical section end */
   /*rt_os_splx(oldLevel);*/

   return TNETD53XX_ERR_OK;
}/* END tnetd53xx_vlynq_Install_ISR() */

/*
 * ----------------------------------------------------------------------------
 * Function : tnetd53xx_vlynq_IRQ_Enable
 *
 * Input    : VLYNQ_DEV *pDev
 *   eVLYNQ_REMOTE_INTRRUPT_SRC  nSrc,
 *   int                         flag, 0 = disable, else = enable
 *
 *
 * Output   : Type:  TNETD53XX_ERR
 *            Value:
 *            TNETD53XX_ERR_OK - success.
 *            Error code - Failure.
 *            			  
 * Process  : Enables / disables IRQ that is coming from a remote device.
 *
 * -----------------------------------------------------------------------------
 */
TNETD53XX_ERR tnetd53xx_vlynq_IRQ_Enable
(
    VLYNQ_DEV                  *pDev,
    eVLYNQ_REMOTE_INTRRUPT_SRC  nSrc,
    int                         flag	/* 0 = disable, else = enable */
)
{
/*   UINT32   val;*/
   unsigned long oldLevel;

   /* Ensure VLYNQ driver is running properly */
   if( VLYNQ_DRV_STATE_RUN != pDev->state )
   {
      return TNETD53XX_ERR_ERROR;
   }

   /* Test arguments */
   if( VLYNQ_OUTRANGE( nSrc, VLYNQ_REMOTE_NUM_OF_INTRRUPTS-1, 0))
   {
      return TNETD53XX_ERR_INVARG;
   }

    /* critical section begin */
   /*oldLevel = rt_os_splx(MAX_ILEV); L.M */

   /* set up the interrupt, it is not enabled yet */
   if( flag)
   {
      VLYNQ_R_IVR(nSrc) |= TNETD53XX_VLYNQ_IVR_INTEN(nSrc);
   }
   else
   {
      VLYNQ_R_IVR(nSrc) &= ~TNETD53XX_VLYNQ_IVR_INTEN(nSrc);
   }
    
   /* critical section end */
   /*rt_os_splx(oldLevel);*/

   return TNETD53XX_ERR_OK;
}/* END tnetd53xx_vlynq_IRQ_Enable() */

/*
 * ----------------------------------------------------------------------------
 * Function : tnetd53xx_vlynq_RootISR
 *
 * Input    : VLYNQ_DEV *pDev
 *
 *
 * Output   : none
 *            			  
 * Process  : generic ISR to serve the VLYNQ module.
 *
 * Note(s)  : This handler is to be connected to the general PUMA VLYNQ 
 *            interrupt #21 on the Puma (root device)
 * -----------------------------------------------------------------------------
 */
void tnetd53xx_vlynq_RootISR( VLYNQ_DEV *pDev)
{
   int         source;      /* Bit position of pending interrupt, start from 0 */
   UINT32      interrupts, clrInterrupts;
   VLYNQ_INTR_CNTRL_ISR isr;
   void *arg;

   /* Get pending interrupts  */
   interrupts = VLYNQ_INTCR;
   /* clear the int CR register */
   VLYNQ_INTCR = interrupts;
   /* save them for further analysis */
   clrInterrupts = interrupts;

   if( VLYNQ_LOG_ENABLE)
      printf("tnetd53xx_vlynq_RootISR dev %u. INTCR = 0x%08lx\n", pDev->dev_idx, clrInterrupts);


   /* Scan interrupt bits */
   source =0;
   while( clrInterrupts != 0)
   {
      /* test if bit is set? */
      if( 0x1ul & clrInterrupts)
      {
         /* call appropriate ISR if user installed it. */
         isr = pDev->pIntrCB[source].isr;
         if( isr != NULL)
         {
            arg = pDev->pIntrCB[source].arg;
            isr(arg);
         }
      }

      /* Next source bit */
      clrInterrupts >>= 1;
      ++source;
   }/*endWhile*/

}/* END tnetd53xx_vlynq_RootISR() */


/*
 * ----------------------------------------------------------------------------
 * Function : tnetd53xx_vlynq_Unload
 *
 * Input    : VLYNQ_DEV *pDev
 *
 *
 * Output   : none
 *            			  
 * Process  : Shut down the VLYNQ interface, free all resources taken by the driver.
 *
 * -----------------------------------------------------------------------------
 */
TNETD53XX_ERR tnetd53xx_vlynq_Unload( VLYNQ_DEV* pDev)
{
   volatile TNETD53XX_ERR retCode = TNETD53XX_ERR_OK;

   /* Specify the reason on unloading */
   pDev->state = VLYNQ_DRV_STATE_UNLOAD;

#ifndef __linux__

   /* First, disable all VLYNQ interrupts by closing the master IRQ vector */
   retCode = sysHwVlynqIntDisable();

#endif

   /* Clean allocated resources by the VLYNQ driver, and invalidate pointer. */
   if( pDev->pIntrCB)
   {
      PAL_memFree( 0,pDev->pIntrCB,0);
      pDev->pIntrCB = NULL;
   }

   /* Give the user a notification before VLYNQ shut down.
    * The user may suspend tasks, notify object not to use this VLYNQ device any more.
    * This mechanism enables asynchronous shut down, for example,
    * The driver might check up and decide to shut down, if things go wrong with HW.
    */
   VLYNQ_RESETCB( VLYNQ_RESET_UNLOAD);

   /* Put local VLYNQ module in reset state.
    * The VLYNQ interface is not accessible any more !!!
    */
   retCode = tnetd53xx_vlynqResetModule( pDev, 1);
   pDev->state = VLYNQ_DRV_STATE_UNINIT;

   /* Let the user a notification after VLYNQ is dead.  The user might
    * decide to reestablish VLYNQ.
    */
   VLYNQ_RESETCB(VLYNQ_RESET_ONFINALRELEASE);

   return retCode;
}/* END tnetd53xx_vlynq_Unload() */

#undef _VLYNQ_C_



