/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalAcxAccess.c
 *   PURPOSE: Support access to the ACX100 registers and memory
 *   
 ****************************************************************************/

#include "whalCommon.h"
#include "whalAcxAccess.h"
#include "whalAcxRegs.h"
#include "acx_hw.h"

/*
 * Registers access
 * ----------------
 * Implementing the following functions for each type of WLAN bus and Endianess:
 * whal_acxRegRead
 * whal_acxRegWrite
 * whal_acxRegSetBitVal
 * whal_acxRegResetBitVal
 * whal_acxRegIsBitSet
 * whal_acxRegSetEndian
 *
 * Note:
 * ACX100_REG_ADDRESS expands to function call that gets the ACX registers
 *  address, see whalCommon.h and whalApi.c
 */
 
#if 0 /* Implementation moved to #define in whalAcxAccess.h */
UINT32 whal_acxRegRead(UINT32 Offset)
{
	register int rVal;
	rVal = *(volatile UINT32 *)(ACX100_REG_ADDRESS+Offset);
	return (ENDIAN_HANDLE_LONG(rVal));
}

void whal_acxRegWrite(UINT32 Offset, UINT32 Val)
{
	register int rVal = Val;
	*(volatile UINT32 *)(ACX100_REG_ADDRESS+Offset) = ENDIAN_HANDLE_LONG(rVal);
}

void whal_acxRegSetBitVal(UINT32 Offset, UINT32 BitVal)
{
	register int rVal = BitVal;
	*(volatile UINT32 *)(ACX100_REG_ADDRESS+Offset) |= ENDIAN_HANDLE_LONG(rVal);
}

void whal_acxRegResetBitVal(UINT32 Offset, UINT32 BitVal)
{
	register int rVal = BitVal;
	*(volatile UINT32 *)(ACX100_REG_ADDRESS+Offset) &= ~ENDIAN_HANDLE_LONG(rVal);
}
#endif

int whal_acxRegIsBitSet(UINT32 AcxReg, UINT32 BitVal)
{
	register UINT32 RegVal;

	RegVal = whal_acxRegRead( AcxReg );
	if (RegVal & BitVal)
		return 1;
	return 0;
}

int whal_acxRegSetEndian(int val)
{
	whal_acxRegWrite(ACX_REG_SLV_END_CTL, val);
	return 0;
}

/*
 * Memory Access
 * -------------
 * Implementing the following functions for each type of WLAN bus and Endianess:
 * whal_acxMemCopy
 * whal_acxMemSetByteBits
 * whal_acxMemResetByteBits
 * whal_acxMemGetByte
 * whal_acxMemSetByte
 * 
 */
/***************** WLAN_BUS_TYPE_CARDBUS *************/
#if( WLAN_BUS_TYPE == WLAN_BUS_TYPE_CARDBUS)
#warning *** WhalAcxAccess.c - Implementing CardBus memory access. ***

void whal_acxMemCopy(char *Dest, char *Src, int Len)
{
	register volatile unsigned short *shortDest = (volatile unsigned short *)Dest;
	register volatile unsigned short *shortSrc = (unsigned short *)Src;
	register volatile unsigned char *byteSrc = (unsigned char *)Src;
	register int Size = Len >> 1;
	unsigned short TempWord;

#if 0
   /* check alignment */
	if (((int)Dest & 1) || ((int)Src & 1))
		printf ("whal_acxMemCopy: fatal error .................\n");
#endif

	if ((int)Dest&1)  /* copy byte after byte */
	{
		/* 
		 * dest addresse is in odd address 
		 */

		/* copy the first byte */
		shortDest = (unsigned short *)((int)shortDest - 1);
		TempWord = (*shortDest & 0xff00) | (*byteSrc++);
		*shortDest++ = TempWord;
		Len--;
		Size = Len >> 1;

		/* now the dest address is in even address but the src address is in odd address */
		while (Size--)
		{
			/* TempWord = ( ((*byteSrc++)<<8) | (*byteSrc++)); */
			TempWord  = (*byteSrc++)<<8; 
			TempWord |= (*byteSrc++); 
			*shortDest++ = TempWord;
		}
		Len -= Size<<1;

		
		/* handle the last byte */
		if (Len > 0)
		{
			TempWord = (*shortDest & 0x00ff) | ((*byteSrc++)<<8);
			*shortDest = TempWord;
		}
	}
	else
	{
		/* 
		 * both addresses are align to 2 bytes 
		 */
		while (Size--)
			*shortDest++ = *shortSrc++;

	if (Len & 1)
	{
		TempWord = (*shortDest & 0x00ff) | (*shortSrc & 0xff00);
		*shortDest = TempWord;
	}
}

void whal_acxMemSetByteBits(UINT32 Addr, int ByteOffset, UINT8 BitsVal)
{
	volatile UINT16 *CtlWordAddr = (UINT16 *)(Addr + (ByteOffset & 0xfffffffe));
   if (ByteOffset & 1)
		*CtlWordAddr |= (UINT16)BitsVal;
	else
		*CtlWordAddr |= (UINT16)(BitsVal << 8);
}

void whal_acxMemResetByteBits(UINT32 Addr, int ByteOffset, UINT8 BitsVal)
{
	volatile UINT16 *CtlWordAddr = (UINT16 *)(Addr + (ByteOffset & 0xfffffffe));
   if (ByteOffset & 1)
		*CtlWordAddr &= ~(UINT16)BitsVal;
	else
		*CtlWordAddr &= ~(UINT16)(BitsVal << 8);
}

UINT8 whal_acxMemGetByte(UINT32 Addr, int ByteOffset)
{
	volatile UINT16 *CtlWordAddr = (UINT16 *)(Addr + (ByteOffset & 0xfffffffe));
   if (ByteOffset & 1)
		return ((UINT8)( (*CtlWordAddr >> 0) & 0xff ));
	else
		return ((UINT8)( (*CtlWordAddr >> 8) & 0xff ));
}

void whal_acxMemSetByte(UINT32 Addr, int ByteOffset, UINT8 Val)
{
	volatile UINT16 *CtlWordAddr = (UINT16 *)(Addr + (ByteOffset & 0xfffffffe));
   if (ByteOffset & 1)
		*CtlWordAddr = ((Val << 0) & 0x00ff) | (*CtlWordAddr & 0xff00);
	else
		*CtlWordAddr = ((Val << 8) & 0xff00) | (*CtlWordAddr & 0x00ff);
}

/***************** WLAN_BUS_TYPE_VLYNQ || WLAN_BUS_TYPE_PCI *************/
#elif( (WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ) \
    || (WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI))

void whal_acxMemCopy(char *Dest, char *Src, int Len)
{
   register volatile unsigned short *shortDest = (volatile unsigned short *)Dest;
	register volatile unsigned short *shortSrc = (unsigned short *)Src;
	register int Size = Len >> 1;
	unsigned short TempWord;

	while (Size--)
		*shortDest++ = *shortSrc++;

#if( USE_ENDIAN_SWAP ==1)
	if (Len & 1)
	{
		TempWord = (*shortDest & 0x00ff) | (*shortSrc & 0xff00);
		*shortDest = TempWord;
	}
#else
	if (Len & 1)
	{
		TempWord = (*shortDest & 0xff00) | (*shortSrc & 0x00ff);
		*shortDest = TempWord;
	}
#endif
}

void whal_acxMemSetByteBits(UINT32 Addr, int ByteOffset, UINT8 BitsVal)
{
   volatile UINT8 *ptr = (UINT8 *)(Addr + ByteOffset);
   *ptr |= BitsVal;
}

void whal_acxMemResetByteBits(UINT32 Addr, int ByteOffset, UINT8 BitsVal)
{
   volatile UINT8 *ptr = (UINT8 *)(Addr + ByteOffset);
   *ptr &= ~BitsVal;
}

UINT8 whal_acxMemGetByte(UINT32 Addr, int ByteOffset)
{
	volatile UINT8 *ptr = (UINT8 *)(Addr + ByteOffset);
	return (*ptr);
}

void whal_acxMemSetByte(UINT32 Addr, int ByteOffset, UINT8 Val)
{
	volatile UINT8 *ptr = (UINT8 *)(Addr + ByteOffset);
   *ptr = Val;
}


/***************** TODO: More buses *************/
#else
#error *** WhalAcxAccess.c - WLAN Bus Memory access is not implemented for this bus type. ***
#endif /* WLAN_BUS_TYPE  */


