/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalAcxAccess.h
 *   PURPOSE: Support access to the ACX100 registers and memory
 *   
 ****************************************************************************/

#ifndef _WHAL_ACX_REG_ACCESS_H
#define _WHAL_ACX_REG_ACCESS_H

#include "pal.h"
#include "whalCommon.h"
/* 
 * Acx Registers Api
 */
extern int		whal_acxRegIsBitSet(UINT32 AcxReg, UINT32 BitVal);
extern int		whal_acxRegSetEndian(int BitVal);

/* 
 * Acx memory Api
 */
extern void		whal_acxMemCopy(char *Dest, char *Src, int len);
extern UINT8	whal_acxMemGetByte(UINT32 Addr, int ByteOffset);
extern void		whal_acxMemSetByte(UINT32 Addr, int ByteOffset, UINT8 Val);
extern void		whal_acxMemSetByteBits(UINT32 Addr, int ByteOffset, UINT8 BitsVal);
extern void		whal_acxMemResetByteBits(UINT32 Addr, int ByteOffset, UINT8 BitsVal);

#define whal_acxMemGetU16(Addr) ( ENDIAN_HANDLE_WORD(*(volatile UINT16 *)(Addr)) ) 
#define whal_acxMemSetU16(Addr, Val) ( *(volatile UINT16 *)(Addr) = ENDIAN_HANDLE_WORD((UINT16)(Val)) )
#define whal_acxMemSetU16_Bits(Addr, Val) ( *(volatile UINT16 *)(Addr) |=  ENDIAN_HANDLE_WORD((UINT16)(Val)) )
#define whal_acxMemResetU16_Bits(Addr, Val) ( *(volatile UINT16 *)(Addr) &= ~ENDIAN_HANDLE_WORD((UINT16)(Val)) )

#define whal_acxMemGetU32(Addr) ( ENDIAN_HANDLE_LONG(*(volatile UINT32 *)(Addr)) ) 
#define whal_acxMemSetU32(Addr, Val) ( *(volatile UINT32 *)(Addr) = ENDIAN_HANDLE_LONG((UINT32)(Val)) )
#define whal_acxMemSetU32_Bits(Addr, Val) ( *(volatile UINT32 *)(Addr) |=  ENDIAN_HANDLE_LONG((UINT32)(Val)) )
#define whal_acxMemResetU32_Bits(Addr, Val) ( *(volatile UINT32 *)(Addr) &= ~ENDIAN_HANDLE_LONG((UINT32)(Val)) )

/* Clearing long, word, byte - no need to convert endianess, it is just a move zero */
#define whal_acxMemClrU32(Addr) ( *(volatile UINT32 *)(Addr) = 0)
#define whal_acxMemClrU16(Addr) ( *(volatile UINT16 *)(Addr) = 0)
#define whal_acxMemClrU8(Addr)  ( *(volatile UINT8 *)(Addr) = 0)

/* Read ACX memory, no endianess conversion */
#define whal_acxMemGetRawU32(Addr) ( *(volatile UINT32 *)(Addr) ) 
#define whal_acxMemGetRawU16(Addr) ( *(volatile UINT16 *)(Addr) ) 
#define whal_acxMemGetRawU8(Addr) ( *(volatile UINT8 *)(Addr) ) 

/*
 * ACX Register Access macros
 */
#define whal_acxRegRead( ofst)    \
   whal_acxMemGetU32( ACX100_REG_ADDRESS+(UINT32)(ofst))
#define whal_acxRegWrite( ofst, v)   \
   whal_acxMemSetU32( ACX100_REG_ADDRESS+(UINT32)(ofst), (v))
#define whal_acxRegSetBitVal( ofst, bitVal) \
   whal_acxMemSetU32_Bits( ACX100_REG_ADDRESS+(UINT32)(ofst), (bitVal))
#define whal_acxRegResetBitVal( ofst, bitVal) \
   whal_acxMemResetU32_Bits( ACX100_REG_ADDRESS+(UINT32)(ofst), (bitVal))

#endif


