/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalAcxConfig.c
 *   PURPOSE: Handle the ACX100 information elements:
 *			MemoryMap				Configure and Interrogate
 *			MemoryConfigOption		Configure Rx/Tx memory blocks
 *			QueueConfigration		Configure Rx queue and Tx Queues
 *			RxConfig				Configure Rx filters
 *			StationId				Configure the AP MAC address
 *   
 ****************************************************************************/

#include "whalCommon.h"
#include "whalAcxAccess.h"
#include "whal.h"
#include "AcxCommands.h"
#include "whalAcxMbox.h"
#include "AcxDataIf.h"
#include "whalTxQueue.h"
#include "whalRxQueue.h"
#include "whalParams.h"
#include "whalAcxConfig.h"
#include "whalTrace.h"
#include "whalAcxRegs.h"
#include "whalAcxAccess.h"
#include "drvDbMain.h"


/****************************************************************************
 *                      whal_acxConfigMemory()
 ****************************************************************************
 * DESCRIPTION:	Configure wlan hardware memory 
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigMemory(UINT32 HostRxDescriptorsAddr)
{
	WlanElm_ConfigMemory_T AcxElm_ConfigMemory;
	WlanElm_ConfigMemory_T *pCfg = &AcxElm_ConfigMemory;
	DmaParams_T *pDmaParams = whal_ParamsGetDmaParams(0);
	int Qid;
	UINT8 WoneIndex, LocalQid, TotalWones, WoneTxQueues, TotalTxQueues = 0;

	PAL_memSet(pCfg,0,sizeof(*pCfg));

	/*
	 * Set configuration fields 
	 */
	pCfg->numStations	= ENDIAN_HANDLE_WORD(pDmaParams->NumStations);
	pCfg->mblkSize		= ENDIAN_HANDLE_WORD(pDmaParams->RxBlockSize);
	pCfg->rxMblkRatio	=  pDmaParams->RxBlocksRatio;
	pCfg->txXfrMblkRatio = pDmaParams->TxXfrBlocksRatio;
	pCfg->numRxQueues	= 1;
	
	wdrv_dbGetNumSupWone(&TotalWones);
	for (WoneIndex = 0, Qid = 0; WoneIndex < TotalWones; WoneIndex++)
	{	
   		/*
	 	 * Tx queue config 
		 */

		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		pDmaParams = whal_ParamsGetDmaParams(WoneIndex);
		
		for (LocalQid=0; LocalQid<WoneTxQueues; LocalQid++,Qid++)                              
	    {                                                                  
			pCfg->TxQueueConfig[Qid].NumDesc = pDmaParams->TxNumDesc[LocalQid];
			pCfg->TxQueueConfig[Qid].Attr	= pDmaParams->TxQPriority[LocalQid];
      		printf(" TxQueue[%d] NumDesc  = %u\n", Qid, (UINT)(pCfg->TxQueueConfig[Qid].NumDesc));
      		printf(" TxQueue[%d] Priority = %u\n", Qid, (UINT)(pCfg->TxQueueConfig[Qid].Attr));
	    }                                                                  
		TotalTxQueues+= WoneTxQueues;
	}
	
	pCfg->numTxQueues	= TotalTxQueues;

	/*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = ACX_ELM_ID_MEMORY_CONF; 
	pCfg->ElmHdr.Len = sizeof(WlanElm_ConfigMemory_T) - sizeof(ElmHdr_T) 
                    - (sizeof(ACXtxQueueConfig) * (NUM_TX_QUEUES - TotalTxQueues)) ;

#if 1
	pCfg->hostifOptions	= HOSTIF_MASTER;  /* According to FW and WSP src code */
#else
  	pCfg->hostifOptions	= HOSTIF_VLYNQ_MASTER_HOST_INDIRECT;
#endif

	/*
	 * Rx queue config 
	 */
#if( WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)
	pCfg->RxQueueConfig.HostRxDescriptorsAddr =
      ENDIAN_HANDLE_LONG( VLYNQ_HOST2ACX111_SDRAM_PTR(HostRxDescriptorsAddr));
#elif( WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)
	pCfg->RxQueueConfig.HostRxDescriptorsAddr =
      ENDIAN_HANDLE_LONG( PCI_HOST2ACX111_SDRAM_PTR(HostRxDescriptorsAddr));
#else
#error   whalAcxConfig.c -- Check bus type.
#endif

	pCfg->RxQueueConfig.NumDesc  = pDmaParams->RxQueue_NumDesc;
	pCfg->RxQueueConfig.Priority = pDmaParams->RxQueue_Priority;
	pCfg->RxQueueConfig.Type	 = pDmaParams->RxQueue_Type;

#if( WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)
	printf("\nwhal_acxConfigMemory() --\nHostRxDescriptorsAddr = %08x\n",
		   VLYNQ_HOST2ACX111_SDRAM_PTR(HostRxDescriptorsAddr));
#elif( WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)
	printf("\nwhal_acxConfigMemory() --\nHostRxDescriptorsAddr = %08x\n",
           PCI_HOST2ACX111_SDRAM_PTR(HostRxDescriptorsAddr));
#else
#error   whalAcxConfig.c -- Check bus type.
#endif

   printf(" RxQueue NumDesc  = %u\n", (UINT)(pCfg->RxQueueConfig.NumDesc));
   printf(" RxQueue Priority = %u\n", (UINT)(pCfg->RxQueueConfig.Priority));
   printf(" RxQueue type     = %u\n", (UINT)(pCfg->RxQueueConfig.Type));


   	/*
	 * Send the configuration command
	 */
   puts(" ");

   return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), CONFIG_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxConfigRead_QueueHead()
 ****************************************************************************
 * DESCRIPTION:	Read the Queue addresses after memory init 
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigRead_QueueHead(WlanElm_QueueHead_T *pCfg)
{
	int Qid;
	int Stt;
	UINT8 WoneIndex, LocalQid, TotalWones, WoneTxQueues, TotalTxQueues = 0;

    PAL_memSet( pCfg, 0 , sizeof(*pCfg));

   	wdrv_dbGetNumSupWone(&TotalWones);
   	
	for (WoneIndex = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		TotalTxQueues+= WoneTxQueues;
	}

   	/*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = ACX_ELM_ID_QUEUE_HEAD; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T)
									 -(sizeof(ACXoneQueueHead) * (NUM_TX_QUEUES - TotalTxQueues));

	/*
	 * Send the interrogation command, and fill the structure
	 */
	Stt = whal_acxMboxInterrogate(pCfg, sizeof(*pCfg));

	/*
	 * solve endian problem
	 */
	pCfg->rxMemBlkQ  = (UINT32 *)ENDIAN_HANDLE_LONG((UINT32)(pCfg->rxMemBlkQ));
	pCfg->txMemBlkQ  = (UINT32 *)ENDIAN_HANDLE_LONG((UINT32)(pCfg->txMemBlkQ));
	pCfg->RxQueueHead.Addr = ENDIAN_HANDLE_LONG(pCfg->RxQueueHead.Addr);

   puts("whal_acxConfigRead_QueueHead() -\n");
   printf(" RxMemBlkQ   - %08x\n", (int)pCfg->rxMemBlkQ );
   printf(" RxQueueHead - %08x\n", (int)pCfg->RxQueueHead.Addr);
   printf(" TxMemBlkQ   - %08x\n", (int)pCfg->txMemBlkQ );
   
	for (WoneIndex = 0, Qid = 0; WoneIndex < TotalWones; WoneIndex++)
	{
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
   
		for (LocalQid=0; LocalQid<WoneTxQueues; LocalQid++,Qid++)                              
	    {                                                                  
			pCfg->TxQueueHead[Qid].Addr= ENDIAN_HANDLE_LONG(pCfg->TxQueueHead[Qid].Addr);
      		printf(" TxMemBlkQ[%u]- %08x\n", Qid, pCfg->TxQueueHead[Qid].Addr);
		}
	}
   puts(" ");

   return( Stt );
}


/****************************************************************************
 *                      whal_acxConfigRxConfig()
 ****************************************************************************
 * DESCRIPTION:	Configure RxConfig information element to the ACX100
 *		This information element allows the host to configure the types of 
 *      frames that are received by the ACX100.
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigRxConfig()
{
	AcxElm_RxConfig_T AcxElm_RxConfig;
	AcxElm_RxConfig_T *pCfg = &AcxElm_RxConfig;

   PAL_memSet(pCfg , 0, sizeof(*pCfg));
	/*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = ACX_ELM_ID_RX_CONFIG; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);

	/*
	 * Set configuration fields 
	 */
	pCfg->ConfigOptions = ENDIAN_HANDLE_WORD(whal_ParamsGetGenParams(0)->RxConfigOption);
	pCfg->FilterOptions = ENDIAN_HANDLE_WORD(whal_ParamsGetGenParams(0)->RxFilterOption);

	/*
	 * Send the configuration command
	 */
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), CONFIG_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxConfigStationId()
 ****************************************************************************
 * DESCRIPTION:	Configure StationId information element to the ACX100
 *		This information element specifies the MAC Address assigned to the AP. 
 *		This default value is the permanent MAC address that is stored in the 
 *		adaptors non-volatile memory.
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigStationId()
{
	AcxElm_StationId_T AcxElm_StationId;
	AcxElm_StationId_T *pCfg = &AcxElm_StationId;
	UINT8 *SrcMacAddr = whal_ParamsGetSrcMac();
	int i;

   PAL_memSet(pCfg , 0, sizeof(*pCfg));
	/*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = DOT11_ELM_ID_STATION_ID; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set configuration fields (reversed order, see docs)
	 */
	for (i=0; i<6; i++)
		pCfg->StationId[i] = SrcMacAddr[5-i];

	/*
	 * Send the configuration command
	 */
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), CONFIG_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxConfigShortRetryLimit()
 ****************************************************************************
 * DESCRIPTION:	Configure The short retry limit
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigShortRetryLimit()
{
	AcxElm_RetryLimit_T AcxElm_RetryLimit;
	AcxElm_RetryLimit_T *pCfg = &AcxElm_RetryLimit;

   PAL_memSet(pCfg , 0, sizeof(*pCfg));
	/*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = DOT11_SHORT_RETRY_LIMIT; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set configuration fields 
	 */
	pCfg->RetryLimit = whal_ParamsGetGenParams(0)->ShortRetryLimit;

	/*
	 * Send the configuration command
	 */
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), CONFIG_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxConfigCWminWin()
 ****************************************************************************
 * DESCRIPTION:	Configure The CWmin
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_acxConfigCWminWin()
{
	UINT16 CwMin = whal_ParamsGetGenParams(0)->CWmin;
	whal_acxRegWrite(ACX_CONT_WIND_CFG_REG, (UINT32)(((UINT32)CwMin & ACX_CONT_WIND_MIN_MASK) | ACX_CONT_WIND_MAX));
}

/****************************************************************************
 *                      whal_acxConfigFragThreshold()
 ****************************************************************************
 * DESCRIPTION:	Configure The fragmentation threshold
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigFragThreshold()
{
	AcxElm_FragThreshold_T AcxElm_FragThreshold;
	AcxElm_FragThreshold_T *pCfg = &AcxElm_FragThreshold;

   PAL_memSet(pCfg , 0, sizeof(*pCfg));
	/*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = DOT11_ELM_ID_FRAG_THRESHOLD; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set configuration fields 
	 */
	pCfg->FragThreshold = ENDIAN_HANDLE_WORD(whal_ParamsGetGenParams(0)->FragmentThreshold);

	/*
	 * Send the configuration command
	 */
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), CONFIG_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxConfigLongRetryLimit()
 ****************************************************************************
 * DESCRIPTION:	Configure The long retry limit
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigLongRetryLimit()
{
	AcxElm_RetryLimit_T AcxElm_RetryLimit;
	AcxElm_RetryLimit_T *pCfg = &AcxElm_RetryLimit;

   PAL_memSet(pCfg , 0, sizeof(*pCfg));
	/*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = DOT11_LONG_RETRY_LIMIT; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set configuration fields 
	 */
	pCfg->RetryLimit = whal_ParamsGetGenParams(0)->LongRetryLimit;

	/*
	 * Send the configuration command
	 */
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), CONFIG_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxConfigBssPowerSave()
 ****************************************************************************
 * DESCRIPTION:	Configure the Bss in/not power save 
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigBssPowerSave(int BssInPs_Val)
{
	AcxElm_BssPowerSave_T AcxElm_BssPowerSave;
	AcxElm_BssPowerSave_T *pCfg = &AcxElm_BssPowerSave;

   PAL_memSet(pCfg , 0, sizeof(*pCfg));
	/*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = ACX_ELM_ID_BSS_IN_PS; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set configuration fields 
	 */
	pCfg->BssInPs = BssInPs_Val;

#ifdef TIWLAN_DEBUG
	if (whal_trcIsEnable())
		whal_trcAddCtrl("CNFG", "AcxPowerSave ", BssInPs_Val);
#endif
	/*
	 * Send the configuration command
	 */
#if 0
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), CONFIG_TIMEOUT));
#else
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), NO_WAIT));
#endif
}

/****************************************************************************
 *                      whal_acxConfigCurrRegDomain()
 ****************************************************************************
 * DESCRIPTION:	Configure the Regulatory domain 
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigCurrRegDomain(UINT8 apRegDomain, UINT8 radioBand)
{
	AcxElm_CurrRegDomain_T AcxElm_CurrRegDomain;
	AcxElm_CurrRegDomain_T *pCfg = &AcxElm_CurrRegDomain;

   PAL_memSet(pCfg , 0, sizeof(*pCfg));
	/*
	 * Set information element header 
	 */
   	if(radioBand == RADIO_BAND_2_4GHZ)

		pCfg->ElmHdr.Id = DOT11_CUR_REG_DOMAIN; /*(DOT11_ID | 0x000A)*/
	else
		pCfg->ElmHdr.Id = DOT11_CUR_5GHZ_REG_DOMAIN ; /*(DOT11_ID | 0x0011)*/

	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set configuration fields 
	 */

        pCfg->Domain = apRegDomain;
	/*
	 * Send the configuration command
	 */
	
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), CONFIG_TIMEOUT));
}


/****************************************************************************
 *                      whal_acxConfigPowerLevel()
 ****************************************************************************
 * DESCRIPTION:	Configure the Power Level 
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigPowerLevel()
{
	AcxElm_PowerLevel_T AcxElm_PowerLevel;
	AcxElm_PowerLevel_T *pCfg = &AcxElm_PowerLevel;

   PAL_memSet(pCfg , 0, sizeof(*pCfg));
	/*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = DOT11_CUR_PWR_LVL; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set configuration fields 
	 */
	pCfg->PowerLevel = whal_ParamsGetGenParams(0)->PowerLevel;

	/*
	 * Send the configuration command
	 */
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), CONFIG_TIMEOUT));
}


/****************************************************************************
 *                      whal_acxConfigRateFallback()
 ****************************************************************************
 * DESCRIPTION:	Configure the Rate fallback 
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigRateFallback()
{
	AcxElm_RateFallback_T AcxElm_RateFallback;
	AcxElm_RateFallback_T *pCfg = &AcxElm_RateFallback;

   PAL_memSet(pCfg , 0, sizeof(*pCfg));
	/*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = ACX_RATE_FALLBACK; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set configuration fields 
	 */
	pCfg->RateFallback = whal_ParamsGetGenParams(0)->RateFallback;

	/*
	 * Send the configuration command
	 */
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), CONFIG_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxConfigRxIntr()
 ****************************************************************************
 * DESCRIPTION:	Configure the RxData interrupt threshold 
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigRxIntr(UINT32 Threshold, UINT32 Timeout_Usec)
{
	AcxElm_IntConfig_T AcxElm_IntConfig;
	AcxElm_IntConfig_T *pCfg = &AcxElm_IntConfig;

   PAL_memSet(pCfg , 0, sizeof(*pCfg));
	/*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = ACX_INT_CONFIG; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set configuration fields 
	 */
	pCfg->RxDataThreshold	= ENDIAN_HANDLE_LONG(Threshold);
	pCfg->RxDataTimeout		= ENDIAN_HANDLE_LONG(Timeout_Usec);

	/*
	 * Send the configuration command
	 */
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), CONFIG_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxConfigPrint_ConfigRxIntr()
 ****************************************************************************
 * DESCRIPTION:	Print the Rx interrupt configuration
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigPrint_ConfigRxIntr()
{
	AcxElm_IntConfig_T AcxElm_IntConfig;
	AcxElm_IntConfig_T *pCfg = &AcxElm_IntConfig;
	int Stt;

   PAL_memSet(pCfg , 0, sizeof(*pCfg));
	/*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id  = ACX_INT_CONFIG;
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T); 

	/*
	 * Send the interrogation command
	 */
	Stt = whal_acxMboxInterrogate(pCfg, sizeof(*pCfg));

	printf("whal_acxConfigPrint_ConfigRxIntr:\n");
	printf("	RxDataThreshold = 0x%x\n", ENDIAN_HANDLE_LONG(pCfg->RxDataThreshold));
	printf("	RxDataTimeout   = 0x%x\n", ENDIAN_HANDLE_LONG(pCfg->RxDataTimeout));
	printf("	RxDataWatermark = 0x%x\n", ENDIAN_HANDLE_LONG(pCfg->RxDataWatermark));
	printf("	RxDataTime      = 0x%x\n", ENDIAN_HANDLE_LONG(pCfg->RxDataTime));
	printf("	RxDataFrames    = 0x%x\n", pCfg->RxDataFrames);
	return( Stt );
}


/****************************************************************************
 *                      whal_acxConfigMemoryMap()
 ****************************************************************************
 * DESCRIPTION:	Configure MemoryMap information element to the ACX100
 *		This information element allows the host to define where data is stored 
 *		in ACX100 memory in terms of start and end address to the following 
 *		memory areas:
 *			Firmware code
 *			WEP cache
 *			Packet templates
 *			Rx/Tx queues
 *			Rx/Tx buffers
 * 
 * INPUTS:	
 *		AcxElm_MemoryMap_T *pMap	pointer to the memory map structure
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigMemoryMap( AcxElm_MemoryMap_T *pMap )
{
	AcxElm_MemoryMap_T SwapMap;
	int *pSwap, *pOrig;
	int i;

	/*
	 * Set information element header 
	 */
	SwapMap.ElmHdr.Id  = ACX_ELM_ID_MEM_MAP;
	SwapMap.ElmHdr.Len = sizeof(SwapMap) - sizeof(ElmHdr_T); 

	/*
	 * solve endian problem (all fields are 32 bit)
	 */
	pOrig = (int *)&pMap->CodeStart;
	pSwap = (int *)&SwapMap.CodeStart;
	for (i=0; i<MEM_MAP_NUM_FILEDS; i++)
		pSwap[i] = ENDIAN_HANDLE_LONG(pOrig[i]);
	/*
	 * Send the configuration command
	 */
	return( whal_acxMboxConfigure(&SwapMap, sizeof(SwapMap), CONFIG_TIMEOUT) );
}

/****************************************************************************
 *                      whal_acxConfigRead_MemoryMap()
 ****************************************************************************
 * DESCRIPTION:	Interrogate MemoryMap information element from the ACX100
 *		The information element consist of memory areas with Start and end address
 *		for each areas. The memory areas are:
 *			Firmware code
 *			WEP cache
 *			Packet templates
 *			Rx/Tx queues
 *			Rx/Tx buffers
 * 
 * INPUTS:	
 *		AcxElm_MemoryMap_T *pMap	pointer to the memory map structure
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigRead_MemoryMap( AcxElm_MemoryMap_T *pMap )
{
	int i, Stt;
	int *pSwap;

	/*
	 * Set information element header 
	 */
	pMap->ElmHdr.Id  = ACX_ELM_ID_MEM_MAP;
	pMap->ElmHdr.Len = sizeof(*pMap) - sizeof(ElmHdr_T); 

	/*
	 * Send the interrogation command
	 */
	Stt = whal_acxMboxInterrogate(pMap, sizeof(*pMap));

	/*
	 * solve endian problem (all fields are 32 bit)
	 */
	pSwap = (int *)&pMap->CodeStart;
	for (i=0; i<MEM_MAP_NUM_FILEDS; i++)
		pSwap[i] = ENDIAN_HANDLE_LONG(pSwap[i]);

	return( Stt );
}

/****************************************************************************
 *                      whal_acxConfigGetSet_MemoryMap()
 ****************************************************************************
 * DESCRIPTION:	Set specific field with new value
 * 
 * INPUTS:	
 *		AcxElm_MemoryMap_T *pMap	pointer to the memory map structure
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigGetSet_MemoryMap( AcxElm_MemoryMap_T *pMemMap, int Idx)
{
    whal_acxConfigRead_MemoryMap(pMemMap);
	switch (Idx)
	{
		case IDX_WEP_CACHE_START:
			pMemMap->WEPCacheStart = pMemMap->CodeEnd;
			break;
		case IDX_WEP_CACHE_END: /* for empty wep cache */
			pMemMap->WEPCacheEnd = pMemMap->CodeEnd;
			break;
		case IDX_STA_TABLE_START:
			pMemMap->staTableStart = pMemMap->CodeEnd;
			break;
		case IDX_STA_TABLE_END: /* only for empty station table */
			pMemMap->staTableEnd = pMemMap->CodeEnd;
			break;
		case IDX_TEMPLATE_START:
			pMemMap->PacketTemplateStart = pMemMap->staTableEnd;
			break;
		case IDX_QUEUE_START:
			pMemMap->QueueMemoryStart = (UINT8 *)pMemMap->PacketTemplateEnd; /* +4  */
			break;
		case IDX_POOL_START:
			pMemMap->PacketMemoryPoolStart = (UINT8 *)(((UINT32)pMemMap->QueueMemoryEnd+0x100) & 0xffffff00);
			break;
		default:
			break;
	}
	return OK;
}

/****************************************************************************
 *                      whal_acxConfigRead_ConfigOptions()
 ****************************************************************************
 * DESCRIPTION:	Read ConfigOption information element from the ACX100. 
 *		This is a special case where the data is already in the mailbox
 *		after ACX100 reset and no interrogate command should be sent.
 *		This read-only IE provides information stored in the adaptors 
 *		non-volatile memory to host.
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigRead_ConfigOptions()
{
	AcxElm_ConfigOptions_T *pElm=whal_ParamsGetConfigOptions();
	AcxElm_ConfigOptionsFixed_T *pFix = &pElm->ConfigOptionsFixed;
	char ConfigOptionsBuf[sizeof(AcxElm_ConfigOptions_T)];
	UINT8  *pSrc;
	UINT8	NumLists;
	UINT8	TableId;
	int	i, j, Len;
	UINT8  *pPowerLevel;

	/* 
	 * The ConfigOptions information element is ready after reset on the mailbox 
	 */
	whal_acxMboxReadParameters( ConfigOptionsBuf, sizeof(ConfigOptionsBuf) );

	/* 
	 * copy the fixed size of the info element 
	 */
	pSrc = (UINT8 *)&ConfigOptionsBuf;
	Len = sizeof(pElm->ConfigOptionsFixed);
	PAL_memCopy( pFix, pSrc, Len);
	pFix->EndofMemory = ENDIAN_HANDLE_LONG(pFix->EndofMemory);
	pFix->ProbeDelay  = ENDIAN_HANDLE_WORD(pFix->ProbeDelay);
	pSrc += Len;

	/* 
	 * copy the variable length lists 
	 */
	NumLists = *pSrc++;

	for (i=0; i<NumLists; i++)
	{
		TableId  = *pSrc++;
		Len = *pSrc++;
		switch (TableId)
		{
			case ELM_ID_ANTENNAS:
				pElm->AntennaList.ElmId = TableId;
				pElm->AntennaList.Num = Len;
				for (j=0; j<Len; j++)
					pElm->AntennaList.Table[j] = *pSrc++;
				break;

			case ELM_ID_POWER_LEVELS:
				pElm->SupportedPowerLevels.ElmId = TableId;
				pElm->SupportedPowerLevels.Num = Len;
				for (j=0; j<Len; j++)
				{
					pPowerLevel = (UINT8 *)&pElm->SupportedPowerLevels.Table[j];
					pPowerLevel[1] = *pSrc++;
					pPowerLevel[0] = *pSrc++;
				}
				break;

			case ELM_ID_SUPPORTED_RATES:
				pElm->SupportedDataRates.ElmId = TableId;
				pElm->SupportedDataRates.Num = Len;
				for (j=0; j<Len; j++)
					pElm->SupportedDataRates.Table[j] = *pSrc++;
				break;

			case ELM_ID_REG_DOMAINS: 
				pElm->SupportedRegDomains.ElmId = TableId;
				pElm->SupportedRegDomains.Num = Len;
				for (j=0; j<Len; j++)
					pElm->SupportedRegDomains.Table[j] = *pSrc++;
				break;

			case ELM_ID_PRODUCT_ID:
				pElm->ProductId.ElmId = TableId;
				pElm->ProductId.Num = Len;
				for (j=0; j<Len; j++)
					pElm->ProductId.Table[j] = *pSrc++;
				pElm->ProductId.Table[j] = 0; 
				break;

			case ELM_ID_MANUFACTOR_ID:
				pElm->ManufactureId.ElmId = TableId;
				pElm->ManufactureId.Num = Len;
				for (j=0; j<Len; j++)
					pElm->ManufactureId.Table[j] = *pSrc++;
				pElm->ManufactureId.Table[j] = 0; 
				break;

			default:
				printf("whal_acxConfigRead_ConfigOptions: Unknown list ID=0x%x\n", TableId);
				return NOK;
		}
	}

	return OK;
}

/****************************************************************************
 *                      whal_acxConfigRead_FwVersion()
 ****************************************************************************
 * DESCRIPTION:	Read the FW version information element from the ACX100
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigRead_FwVersionForRestart(whal_userMboxCB userCB)
{
	AcxElm_AcxVersion_T pElem;	

	/*
	 * Set information element header 
	 */
	pElem.ElmHdr.Id  = ACX_ELM_ID_FW_REV;
	pElem.ElmHdr.Len = sizeof(pElem) - sizeof(ElmHdr_T); 

	/*
	 * Send the interrogation command
	 */
	return whal_acxMboxInterrogateNoWait(CMD_INTERROGATE,&pElem, sizeof(pElem), userCB, NULL);
}

/****************************************************************************
 *                      whal_acxConfigRead_FwVersion()
 ****************************************************************************
 * DESCRIPTION:	Read the FW version information element from the ACX100
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigRead_FwVersion()
{
	AcxElm_AcxVersion_T *pElem=whal_ParamsGetAcxVersion();	
	int Stt;

	/*
	 * Set information element header 
	 */
	pElem->ElmHdr.Id  = ACX_ELM_ID_FW_REV;
	pElem->ElmHdr.Len = sizeof(*pElem) - sizeof(ElmHdr_T); 

	/*
	 * Send the interrogation command
	 */
	Stt = whal_acxMboxInterrogate(pElem, sizeof(*pElem));

	return( Stt );
}


/****************************************************************************
 *                      whal_acxConfigDefaultKeyId()
 ****************************************************************************
 * DESCRIPTION:	Configure The default Key Id
 * 
 * INPUTS:	
 *		int Key				The default key id to use
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigWepDefaultKeyId(int Key)
{
	WlanElm_WepDefaultKeyId_T WlanElm_WepDefaultKeyId;
	WlanElm_WepDefaultKeyId_T *pCfg = &WlanElm_WepDefaultKeyId;
	BOOL sts;

	/*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = DOT11_WEP_DEFAULT_KEY_ID; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set fields
	 */
	pCfg->DefaultKeyId = (UINT8)Key;

	/*
	 * Send the configuration command
	 */
	
	sts = whal_acxMboxConfigure(pCfg, sizeof(*pCfg), NO_WAIT);

#if 0
	/* Debug */
	if(sts == NOK)
		printf("@@@@@@@@@@@ failed to set DefaultKeyID: %d\n", Key);
	else
		printf("@@@@@@@@@@@ succeeded to set DefaultKeyID: %d\n", Key);
#endif

	return sts;
}


/****************************************************************************
 *                      whal_acxConfigFeatureOptions()
 ****************************************************************************
 * DESCRIPTION:	Configure The Feature Options
 * 
 * INPUTS:	
 *		UINT32 FeatureOptions		the selected options
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigFeatureOptions(UINT32 FeatureOptions)
{
	AcxElm_FeatureConfig_T AcxElm_FeatureConfig;
	AcxElm_FeatureConfig_T *pCfg = &AcxElm_FeatureConfig;

    PAL_memSet( pCfg, 0 , sizeof(*pCfg));
   /*
    * Set information element header 
    */
	pCfg->ElmHdr.Id = ACX_FEATURE_CONFIG; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);

 	
	/*
	 * Set fields
	 */
	pCfg->Options = ENDIAN_HANDLE_LONG(FeatureOptions | FEAT_MAC_CLK_80);
	pCfg->dataflowOptions = 0;
	/*
	 * Send the configuration command
	 */
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), CONFIG_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxConfigRead_Statistics()
 ****************************************************************************
 * DESCRIPTION:	Interrogate Statistics from the ACX100
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigRead_Statistics( ACXStatisticsStruct *pElem, whal_userMboxCB userCB, UINT8 *replyBuf )
{
	int Stt;

	/*
	 * Set information element header 
	 */
	pElem->ElmHdr.Id  = ACX_ELM_ID_STATISTICS;
	pElem->ElmHdr.Len = sizeof(*pElem) - sizeof(ElmHdr_T); 

	/*
	 * Send the interrogation command
	 */
   Stt = whal_acxMboxInterrogateNoWait(CMD_INTERROGATE,pElem, sizeof(*pElem), userCB, replyBuf);

	return( Stt );
}

/****************************************************************************
 *              
 ****************************************************************************
 * DESCRIPTION:	
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigPrint_Statistics( char *buf)
{
	char tmp_buf[1024*3];
	int len = 0;

	ACXStatisticsStruct *pElem= (ACXStatisticsStruct *)buf;

	/* Isr */
	len +=sprintf(tmp_buf+len,"------  Isr statistics  -------------------\n");

	len +=sprintf(tmp_buf+len,"ACXRxProcs  = %10u\n", pElem->isr.ACXRxProcs);

	len +=sprintf(tmp_buf+len,"RxHeaders   = %10u, RxCompletes       = %10u\n", pElem->isr.RxHeaders, pElem->isr.RxCompletes);
	len +=sprintf(tmp_buf+len,"RxRdys      = %10u, RxMemOverflow     = %10u\n", pElem->isr.RxRdys, pElem->isr.RxMemOverflow);
	len +=sprintf(tmp_buf+len,"ACXTxProcs  = %10u, ACXTxExchComplete = %10u\n", pElem->isr.ACXTxProcs, pElem->isr.ACXTxExchComplete);
	len +=sprintf(tmp_buf+len,"DecryptDone       = %10u\n", pElem->isr.DecryptDone);
	len +=sprintf(tmp_buf+len,"HwPMModeChanges   = %10u\n", pElem->isr.HwPMModeChanges);
	len +=sprintf(tmp_buf+len,"HostAcknowledges  = %10u\n", pElem->isr.HostAcknowledges);
	len +=sprintf(tmp_buf+len,"PCI_PM            = %10u\n", pElem->isr.PCI_PM);
	len +=sprintf(tmp_buf+len,"ACMWakeups        = %10u\n", pElem->isr.ACMWakeups);
	len +=sprintf(tmp_buf+len,"ACXCommands = %10u, Cmd_Cmplt= %10u\n", pElem->isr.ACXCommands, pElem->isr.Cmd_Cmplt);
	len +=sprintf(tmp_buf+len,"DMA0Done    = %10u, DMA1Done = %10u\n", pElem->isr.DMA0Done, pElem->isr.DMA1Done);
	len +=sprintf(tmp_buf+len,"IRQs = %10u, FIQs = %10u\n", pElem->isr.IRQs, pElem->isr.FIQs);

	/* Rx */
	len +=sprintf(tmp_buf+len,"------  Rx  statistics  -------------------\n");
	len +=sprintf(tmp_buf+len,"RxDroppedFrame    = %u\n", pElem->rx.RxDroppedFrame);
	len +=sprintf(tmp_buf+len,"RxFcsErr          = %u\n", pElem->rx.RxFcsErr);
	len +=sprintf(tmp_buf+len,"RxHdrOverflow     = %u\n", pElem->rx.RxHdrOverflow);
	len +=sprintf(tmp_buf+len,"RxHWStuck         = %u\n", pElem->rx.RxHWStuck);
	len +=sprintf(tmp_buf+len,"RxOutOfMem        = %u\n", pElem->rx.RxOutOfMem);
	len +=sprintf(tmp_buf+len,"RxXfrHintTrig     = %u\n", pElem->rx.RxXfrHintTrig);
    len +=sprintf(tmp_buf+len,"RxACIevents       = %u\n", pElem->rx.RxACIevents);
	len +=sprintf(tmp_buf+len,"RxACIresets       = %u\n", pElem->rx.RxACIresets);



	/* Tx */
	len +=sprintf(tmp_buf+len,"------  Tx  statistics  -------------------\n");
	len +=sprintf(tmp_buf+len,"TxInDescOverflow  = %u\n", pElem->tx.TxInternalDescOverflow);

   /* DMA */
	len +=sprintf(tmp_buf+len,"------  DMA  statistics  ------------------\n");
	len +=sprintf(tmp_buf+len,"RxDMARequested    = %u\n", pElem->dma.RxDMARequested	);
	len +=sprintf(tmp_buf+len,"RxDMAErrors       = %u\n", pElem->dma.RxDMAErrors		);
	len +=sprintf(tmp_buf+len,"TxDMARequested    = %u\n", pElem->dma.TxDMARequested	);
	len +=sprintf(tmp_buf+len,"TxDMAErrors       = %u\n", pElem->dma.TxDMAErrors		);

	/* Wep */
	len +=sprintf(tmp_buf+len,"------  Wep statistics  -------------------\n");
	len +=sprintf(tmp_buf+len,"WepDefaultKeyCount= %u\n", pElem->wep.WepDefaultKeyCount);
	len +=sprintf(tmp_buf+len,"dot11DefKeyMIB    = %u\n", pElem->wep.dot11DefKeyMIB);
	len +=sprintf(tmp_buf+len,"WepAddrKeyCount   = %u\n", pElem->wep.WepAddrKeyCount);
	len +=sprintf(tmp_buf+len,"WepDecryptFail    = %u\n", pElem->wep.WepDecryptFail);
	len +=sprintf(tmp_buf+len,"WepKeyNotFound    = %u\n", pElem->wep.WepKeyNotFound);
    len +=sprintf(tmp_buf+len,"WepInterrupts	  = %u\n",pElem->wep.WepInterrupt);
	len +=sprintf(tmp_buf+len,"WepPackets        = %u\n", pElem->wep.WepPackets);


   /* Pwr */
	len +=sprintf(tmp_buf+len,"------  Pwr statistics  -------------------\n");
	len +=sprintf(tmp_buf+len,"AbortTooShort     = %u\n", pElem->pwr.AbortTooShort);
	len +=sprintf(tmp_buf+len,"LPPDBegin         = %u\n", pElem->pwr.LPPDBegin    );
	len +=sprintf(tmp_buf+len,"Match             = %u\n", pElem->pwr.Match        );
	len +=sprintf(tmp_buf+len,"Noisy             = %u\n", pElem->pwr.Noisy        );
	len +=sprintf(tmp_buf+len,"RxStartCtr        = %u\n", pElem->pwr.RxStartCtr   );
	len +=sprintf(tmp_buf+len,"RxTooShort        = %u\n", pElem->pwr.RxTooShort   );
	len +=sprintf(tmp_buf+len,"TxStartCtr        = %u\n", pElem->pwr.TxStartCtr   );
	len +=sprintf(tmp_buf+len,"TxTooShort        = %u\n", pElem->pwr.TxTooShort   );

    /* Aes */
	len +=sprintf(tmp_buf+len,"------  Aes statistics  -------------------\n");
    len +=sprintf(tmp_buf+len,"AesDecryptFail      = %u\n",pElem->aes.AesDecryptFail);
    len +=sprintf(tmp_buf+len,"AesDecryptInterrupt = %u\n",pElem->aes.AesDecryptInterrupt);
    len +=sprintf(tmp_buf+len,"AesDecryptPackets   = %u\n",pElem->aes.AesDecryptPackets);
    len +=sprintf(tmp_buf+len,"AesEncryptFail      = %u\n",pElem->aes.AesEncryptFail);
    len +=sprintf(tmp_buf+len,"AesEncryptInterrupt = %u\n",pElem->aes.AesEncryptInterrupt);
    len +=sprintf(tmp_buf+len,"AesEncryptPackets   = %u\n",pElem->aes.AesEncryptPackets);

    /* Mic */
	len +=sprintf(tmp_buf+len,"------  Mic statistics  -------------------\n");
    len +=sprintf(tmp_buf+len,"MicCalcFailure      = %u\n",pElem->mic.MicCalcFailure);
    len +=sprintf(tmp_buf+len,"MicRxPkts           = %u\n",pElem->mic.MicRxPkts);

    /* Event */
	len +=sprintf(tmp_buf+len,"------  Event statistics  -----------------\n");
    len +=sprintf(tmp_buf+len,"calibration         = %u\n",pElem->event.calibration);
    len +=sprintf(tmp_buf+len,"heartbeat           = %u\n",pElem->event.heartbeat);
    len +=sprintf(tmp_buf+len,"oomLate             = %u\n",pElem->event.oomLate);
    len +=sprintf(tmp_buf+len,"phyTransmitError    = %u\n",pElem->event.phyTransmitError);
    len +=sprintf(tmp_buf+len,"rxMemEmpty          = %u\n",pElem->event.rxMemEmpty);
    len +=sprintf(tmp_buf+len,"rxMismatch          = %u\n",pElem->event.rxMismatch);
    len +=sprintf(tmp_buf+len,"rxPool              = %u\n",pElem->event.rxPool);
    len += sprintf(tmp_buf +len,"txStuck             = %u\n",pElem->event.txStuck);

	/* copy data to org buf*/
	memcpy(buf,tmp_buf,len);
    
	return( len);
}

/****************************************************************************
 *                      whal_acxConfigRead_FcsError()
 ****************************************************************************
 * DESCRIPTION:	Interrogate Fcs error counter from the ACX100
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigRead_FcsError( WlanElm_FcsError_T *pElem, whal_userMboxCB userCB, UINT8 *replyBuf)
{
	int Stt;

	/*
	 * Set information element header 
	 */
	pElem->ElmHdr.Id  = ACX_ELM_ID_FCS_ERROR_CNT;
	pElem->ElmHdr.Len = sizeof(*pElem) - sizeof(ElmHdr_T); 

#if 0
	/*
	 * Send the interrogation command
	 */
	Stt = whal_acxMboxInterrogate(pElem, sizeof(*pElem));

	/*
	 * solve endian problem 
	 */
	pElem->FcsErrorCount = ENDIAN_HANDLE_LONG(pElem->FcsErrorCount);
#endif

	/*
	 * Send the interrogation command
	 */
	Stt = whal_acxMboxInterrogateNoWait(CMD_INTERROGATE,pElem, sizeof(*pElem), userCB, replyBuf);

	return( Stt );
}

#define WONE_GUARD_TIME 		2000
#define WONE_DEFAULT_SLICE 		20000

/****************************************************************************
 *                      whal_acxConfigWoneConfig()
 ****************************************************************************
 * DESCRIPTION:	Configure The Feature Options
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigWoneConfig (void)
{
	WlanElm_WoneConfig_T 			AcxElm_WoneConfig;
	WlanElm_WoneConfig_T *	pCfg = &AcxElm_WoneConfig;
	UINT8 							WoneIndex,
									TotalWones,
									WoneTxQueues,
									StartQueue = 0;

   PAL_memSet( pCfg, 0 , sizeof(*pCfg));
   /*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = ACX_WONE_CONFIG; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set fields
	 */
	 
	wdrv_dbGetNumSupWone(&TotalWones);
	pCfg->WoneNets		= TotalWones;
	pCfg->GuardTime		= ENDIAN_HANDLE_LONG( WONE_GUARD_TIME );
	
	for (WoneIndex = 0; WoneIndex < TotalWones; WoneIndex++)
	{	
		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		pCfg->Wone[WoneIndex].QueuesNum = WoneTxQueues;
		pCfg->Wone[WoneIndex].StartQueue = StartQueue;
		
		StartQueue += WoneTxQueues;
	}
	/*
	 * Send the configuration command
	 */
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), CONFIG_TIMEOUT));
}


/****************************************************************************
 *                      whal_acxConfigWoneSlice()
 ****************************************************************************
 * DESCRIPTION:	Configure The Feature Options
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigWoneSlice ()
{
	WlanElm_WoneSlice_T 			AcxElm_WoneSlice;
	WlanElm_WoneSlice_T *	pCfg = &AcxElm_WoneSlice;
	UINT8 							WoneIndex,
									TotalWones;
	UINT8							Slice;

   PAL_memSet( pCfg, 0 , sizeof(*pCfg));
   /*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = ACX_WONE_SLICE; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set fields
	 */
	 
	wdrv_dbGetNumSupWone(&TotalWones);
	for (WoneIndex = 0; WoneIndex < TotalWones; WoneIndex++)
	{	
		wdrv_dbGetUsageTime(WoneIndex, &Slice);  
   /*    pCfg->Wone[WoneIndex].Slice = ENDIAN_HANDLE_LONG( (UINT32)Slice * 1024 );   */   
        /* FW 3.0.... */
        pCfg->Wone[WoneIndex].Slice = ENDIAN_HANDLE_LONG( (UINT32)Slice );          

	}
	/*
	 * Send the configuration command
	 */
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), NO_WAIT));
}

/****************************************************************************
 *                      whal_acxConfigReadWoneConfig()
 ****************************************************************************
 * DESCRIPTION:	Reads the WONE Config information element from ACX device.
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
#if 0
int whal_acxConfigReadWoneConfig ()
{
	WlanElm_WoneConfig_T 			AcxElm_WoneConfig;
	WlanElm_WoneConfig_T *	pCfg = &AcxElm_WoneConfig;
	int 					Stt;
	UINT8 							WoneIndex,
									TotalWones,
									WoneTxQueues,
									StartQueue = 0;

   	PAL_memSet( pCfg, 0 , sizeof(*pCfg));
   /*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = ACX_WONE_CONFIG; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
		 
	/*
	 * Send the query command
	 */
	Stt = whal_acxMboxInterrogate(pCfg, sizeof(*pCfg));
	
	pCfg->GuardTime		= ENDIAN_HANDLE_LONG( pCfg->GuardTime );
	
	printf(" == WoneConfig IE ==\n Nets:%02X Guard time:%08X\n", pCfg->WoneNets, pCfg->GuardTime );
	
	for (WoneIndex = 0;WoneIndex < MAX_WONE_NET; WoneIndex++)
	{
		printf("    Net[%d]: start queue = %d, total queues = %d\n",
					WoneIndex,
					pCfg->Wone[WoneIndex].StartQueue,
					pCfg->Wone[WoneIndex].QueuesNum );
	}
	
	return (Stt);
}
#endif

/****************************************************************************
 *                      whal_acxConfigReadWoneSlice()
 ****************************************************************************
 * DESCRIPTION:	Reads the WONE Slice information element from ACX device.
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
#if 0
int whal_acxConfigReadWoneSlice ()
{
	WlanElm_WoneSlice_T 			AcxElm_WoneSlice;
	WlanElm_WoneSlice_T *	pCfg = &AcxElm_WoneSlice;
	int						Stt;
	UINT8 							WoneIndex,
									TotalWones,
									WoneTxQueues;

   PAL_memSet( pCfg, 0 , sizeof(*pCfg));
   /*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = ACX_WONE_SLICE; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Send the query command
	 */
	Stt = whal_acxMboxInterrogate(pCfg, sizeof(*pCfg));
	 
	printf(" == WoneSlice IE ==\n" );
	for (WoneIndex = 0; WoneIndex < MAX_WONE_NET; WoneIndex++)
	{	
		pCfg->Wone[WoneIndex].Slice = ENDIAN_HANDLE_LONG( pCfg->Wone[WoneIndex].Slice );
		printf("    Net[%d]: slice = %d\n", WoneIndex, pCfg->Wone[WoneIndex].Slice);
	}
	
	return (Stt);
}
#endif

/****************************************************************************
 *                      whal_acxConfigEdcfTid()
 ****************************************************************************
 * DESCRIPTION:	Configure an EDCF traffic type (tied to a Tx queue) &
 * packet Bursting.
 * 
 * INPUTS:	The configured Tx queue index and its priority.
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/

int whal_acxConfigAccessCategory(TxQueueId_T Qid, edcf_info_t * edcfInfo)
{
	WlanElm_AccessCategory_T    ACXElm_AccessCategory;
	WlanElm_AccessCategory_T *  pCfg = &ACXElm_AccessCategory;

    PAL_memSet( pCfg, 0 , sizeof(*pCfg) );
    /*
     * Set information element header
	 * ==============================
	 */
	pCfg->ElmHdr.Id = ACX_TID_CFG; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set fields 
	 * ==========
	 */
	pCfg->qID = Qid;

      pCfg->acID = edcfInfo->acID;

      pCfg->accessCategory.trafficType = edcfInfo->trafficType;
      pCfg->accessCategory.aifsn = edcfInfo->aifsn;
      pCfg->accessCategory.cwMin = edcfInfo->cwMin;
      pCfg->accessCategory.cwMax = ENDIAN_HANDLE_WORD(edcfInfo->cwMax);
      pCfg->accessCategory.dot11EDCATableMSDULifetime = ENDIAN_HANDLE_LONG(edcfInfo->dot11EDCATableMSDULifetime);
      pCfg->accessCategory.rxTimeout = ENDIAN_HANDLE_WORD(edcfInfo->rxTimeout);
      pCfg->accessCategory.ackPolicy = edcfInfo->ackPolicy;
      pCfg->accessCategory.shortRetryLimit = edcfInfo->shortRetryLimit;
      pCfg->accessCategory.longRetryLimit = edcfInfo->longRetryLimit;
      pCfg->accessCategory.retryPremption = edcfInfo->retryPremption;


    pCfg->accessCategory.txopLimit = ENDIAN_HANDLE_WORD(edcfInfo->txopLimit);      
    pCfg->accessCategory.txopContinuation = edcfInfo->txopContinuation;    
       

      	printf("Write EDCF-IE: TxQueue = %u, trafficType %u AIFSN = %u, CWmin = %u, CWmax = %u\n",
		pCfg->qID, pCfg->accessCategory.trafficType, pCfg->accessCategory.aifsn, 
               pCfg->accessCategory.cwMin, ENDIAN_HANDLE_WORD(pCfg->accessCategory.cwMax));
        printf("MSDULifetime %d ,rxTimeout %u ,ackPolicy %u, shortRetryLimit %u \n",
               ENDIAN_HANDLE_LONG(pCfg->accessCategory.dot11EDCATableMSDULifetime),
                ENDIAN_HANDLE_WORD(pCfg->accessCategory.rxTimeout),
               pCfg->accessCategory.ackPolicy, pCfg->accessCategory.shortRetryLimit);
        printf("longRetryLimit %u , retryPremption %u , txopLimit %u , txopContinuation %u\n",
               pCfg->accessCategory.longRetryLimit, pCfg->accessCategory.retryPremption,
               ENDIAN_HANDLE_WORD(pCfg->accessCategory.txopLimit), pCfg->accessCategory.txopContinuation);
                                                      
	/*
	 * Send the configuration command
	 * ==============================
	 */
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), CONFIG_TIMEOUT));
}



/****************************************************************************
 *                      whal_acxConfigReadEdcfTid()
 ****************************************************************************
 * DESCRIPTION:	Reads the EDCF Traffic Type information element from ACX device.
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigReadEdcfTid(TxQueueId_T Qid)
{
	WlanElm_AccessCategory_T    ACXElm_AccessCategory;
	WlanElm_AccessCategory_T *  pCfg = &ACXElm_AccessCategory;
	int Stt;

    PAL_memSet( pCfg, 0 , sizeof(*pCfg) );
    /*
     * Set information element header and Tx Queue ID:
	 * ===============================================
	 */
	pCfg->ElmHdr.Id = ACX_TID_CFG; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	pCfg->qID = Qid;
	
	/*
	 * Send the query command and print response:
	 */
	Stt = whal_acxMboxInterrogate(pCfg, sizeof(*pCfg));
	 
	printf("Read  EDCF-IE: TxQ = %u, AC = %u, TrafType = %u, AIFS = %u, CWmin = %u, CWmax = %u\n",
		pCfg->qID, pCfg->acID, pCfg->accessCategory.trafficType, pCfg->accessCategory.aifsn, 
		pCfg->accessCategory.cwMin, ENDIAN_HANDLE_WORD(pCfg->accessCategory.cwMax));
	
	
    printf("Read Packet Bursting IE: TxQ = %u txopContinuation = %u txopLimit = %u\n",
           pCfg->qID,pCfg->accessCategory.txopContinuation,
           pCfg->accessCategory.txopLimit);
	return (Stt);
}



/****************************************************************************
 *                      whal_acxConfigQueueThresh()
 ****************************************************************************
 * DESCRIPTION:	Configure a limit to a Tx queue buffers usage from the ACX pool.
 * 
 * INPUTS:	The configured Tx queue index, and its threshold.
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/

int whal_acxConfigQueueThresh(TxQueueId_T Qid, UINT32 Threshold)
{
	WlanElm_QueueThresh_T    ACXElm_QueueThresh;
	WlanElm_QueueThresh_T *  pCfg = &ACXElm_QueueThresh;

    PAL_memSet( pCfg, 0 , sizeof(*pCfg) );
    /*
     * Set information element header
	 * ==============================
	 */
	pCfg->ElmHdr.Id = ACX_QUEUE_THRESH; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set fields 
	 * ==========
	 */
	pCfg->qID = ENDIAN_HANDLE_LONG((UINT32)Qid);
	pCfg->threshold = ENDIAN_HANDLE_LONG(Threshold); /* Threshold in units of 5%. */

	printf("Write Queue Threshold IE: TxQueue = %u, Threshold = %u percent\n",
		ENDIAN_HANDLE_LONG(pCfg->qID), Threshold * 5);

	/*
	 * Send the configuration command
	 * ==============================
	 */
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), CONFIG_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxConfigSlotTime()
 ****************************************************************************
 * DESCRIPTION:	Configure slot time.
 * 
 * INPUTS:	
 *           SlotTimeVal		The SlotTime value in the Capabilities
 *           woneIndex          wone index
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/

int whal_acxConfigSlotTime(UINT8 SlotTimeVal, UINT8 woneIndex)
{
	WlanElm_SlotTime_T    ACXElm_SlotTime;
	WlanElm_SlotTime_T *  pCfg = &ACXElm_SlotTime;

    PAL_memSet( pCfg, 0 , sizeof(*pCfg) );
    /*
     * Set information element header
	 * ==============================
	 */                   
	pCfg->ElmHdr.Id = ACX_ELM_ID_SLOT_TIME_CONF; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);

	/*
	 * Set fields 
	 * ==========
	 */
    pCfg->slotTime  = SlotTimeVal;    


    pCfg->woneIndex = woneIndex;

    return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), NO_WAIT));
}

/****************************************************************************
 *                      whal_acxConfigRadioParams()
 ****************************************************************************
 * DESCRIPTION:	Configure radio params.
 * 
 * INPUTS:	
 *  RadioParam_t * ptr = radio parameters structure
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigRadioParams( RadioParam_t * ptr)
{
   int retCode = OK;

   /* Override Radio Power table if required */
   if( ptr->overrideBitmask & RADIO_PARAM_OVERRIDE_POWERTABLE)
   {
	   WlanElm_RadioSetPowerTable_T  ACXElm_RadioPwr;
	   WlanElm_RadioSetPowerTable_T *pCfg = &ACXElm_RadioPwr;
      PAL_memSet( pCfg, 0 , sizeof(*pCfg) );

      /* Set information element header */                   
	   pCfg->ElmHdr.Id = ACX_ELM_ID_RADIO_SET_PARAMS; 
	   pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
      /* set parameters, convert endianess if required */
      pCfg->parameterType = ENDIAN_HANDLE_LONG(ptr->powerTable_paramType);
	   PAL_memCopy( pCfg->parameter, ptr->powerTable_param, sizeof(pCfg->parameter));
      /* configure the ACX */
      puts("whal_acxConfigRadioParams() -- Overriding Radio Power table\n");
      /* d( pCfg, sizeof(*pCfg), 1); */
      retCode = whal_acxMboxConfigure( pCfg, sizeof(*pCfg), NO_WAIT);
   }

   /* Override more radio parameters if required */

   return( retCode);
}/* END whal_acxConfigRadioParams() */


/****************************************************************************
 *                      whal_acxConfigRead_HDK_Version()
 ****************************************************************************
 * DESCRIPTION:	Interrogate the HDK version from the ACX100
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int 
whal_acxConfigRead_HDK_Version(WlanElm_HDKModuleVersion_t *  pElem, whal_userMboxCB userCB, UINT8 *replyBuf )
{
	int Stt;

    PAL_memSet( pElem, 0 , sizeof(*pElem) );
    /*
     * Set information element header:
	 * ===============================================
	 */
	pElem->ElmHdr.Id = ACX_HDK_VERSION; 
	pElem->ElmHdr.Len = sizeof(*pElem) - sizeof(ElmHdr_T);
	
	/*
	 * Send the query command:
	 */
	Stt = whal_acxMboxInterrogateNoWait(CMD_INTERROGATE, pElem, sizeof(*pElem), userCB, replyBuf);

	return (Stt);
}




int whal_acxConfigVWiLeds(void)
{
	ACXMisc_t Cfg;
	int ret;
	extern UINT16 tiap_activity_led_mask;
	extern UINT8 tiap_gpio_owner;

	if (!tiap_gpio_owner)
		return OK;

	PAL_memSet(&Cfg , 0, sizeof(Cfg));
	Cfg.ElmHdr.Id  = ACX_MISC_CONFIG;
	Cfg.ElmHdr.Len = sizeof(Cfg) - sizeof(ElmHdr_T);

	 /* interrogate VWi board led mask */
	ret = whal_acxMboxInterrogate(&Cfg, sizeof(Cfg));
	if (!(tiap_activity_led_mask = Cfg.txActivityLed))
		tiap_activity_led_mask = 0x100; /* workaround */

	/* config gpio */
	whal_apiMacWrite(ACX_GPO_CFG, ~tiap_activity_led_mask); 	/* set pin to normal GPIO (muxed) */
	whal_apiMacWrite(ACX_GPIO_OE_REG, ~tiap_activity_led_mask); 	/* set pin as output */

	PAL_memSet(&Cfg , 0, sizeof(Cfg));
	 /* signal fw that from now on the driver controls the led(s) */
	Cfg.ElmHdr.Id  = ACX_MISC_CONFIG;
	Cfg.ElmHdr.Len = sizeof(Cfg) - sizeof(ElmHdr_T);
	return (whal_acxMboxConfigure(&Cfg, sizeof(Cfg), CONFIG_TIMEOUT));
}

/****************************************************************************
 *                      whal_acxConfigReadAciCounter()
 ****************************************************************************
 * DESCRIPTION:	Get ACI counter value from FW.
 * 
 * INPUTS:	
			
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigReadAciCounter(AcxElm_AciEvent_T *pCfg, whal_userMboxCB userCB, UINT8 *replyBuf)
{
	/*
	 * Set information element header 
	 */

	pCfg->ElmHdr.Id = DOT11_ACI; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	return whal_acxMboxInterrogateNoWait(CMD_INTERROGATE, pCfg, sizeof(*pCfg), userCB, replyBuf);
}

/****************************************************************************
 *                      whal_acxConfigReadMediumUsage()
 ****************************************************************************
 * DESCRIPTION:	Get Medium usage and usage period value from FW.
 * 
 * INPUTS: 
			
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigReadMediumUsage(AcxElm_MediumUsage_T *acxElem, whal_userMboxCB userCB, UINT8 *replyBuf)
{
	acxElem->ElmHdr.Id  = ACX_MEDIUM_USAGE;
	acxElem->ElmHdr.Len = sizeof(*acxElem) - sizeof(ElmHdr_T);

	return whal_acxMboxInterrogateNoWait(CMD_INTERROGATE, acxElem, sizeof(*acxElem), userCB, replyBuf);
}

/****************************************************************************
 *                      whal_acxConfigReadEnergyDetection()
 ****************************************************************************
 * DESCRIPTION:	Get Energy Detection threshold from FW.
 * 
 * INPUTS:	
			
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigReadEnergyDetection(WlanElm_EDThreshold_T *acxElem, whal_userMboxCB userCB, UINT8 *replyBuf) 
{
	acxElem->ElmHdr.Id  = DOT11_ED_THRESHOLD;
	acxElem->ElmHdr.Len = sizeof(*acxElem) - sizeof(ElmHdr_T);

	return whal_acxMboxInterrogateNoWait(CMD_INTERROGATE, acxElem, sizeof(*acxElem), userCB, replyBuf);
}

/****************************************************************************
 *                      whal_acxConfigEnergyDetect()
 ****************************************************************************
 * DESCRIPTION:	Configure energy detection threshold
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigEnergyDetect(UINT32 edThresh)
{
	WlanElm_EDThreshold_T edThreshElem;
	WlanElm_EDThreshold_T *pCfg = &edThreshElem;
	

    memset(pCfg , 0, sizeof(*pCfg));
	/*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = DOT11_ED_THRESHOLD; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set configuration fields 
	 */
	pCfg->dot11EDThreshold = ENDIAN_HANDLE_LONG(edThresh);

	/*
	 * Send the configuration command
	 */
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), NO_WAIT));
}


/****************************************************************************
 *                      whal_acxConfigPDThreshold()
 ****************************************************************************
 * DESCRIPTION:	Configure packet detection threshold
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigPDThreshold(UINT16 pdThresh)
{
	WlanElm_PDThreshold_T pdThreshElem;
	WlanElm_PDThreshold_T *pCfg = &pdThreshElem;
	

    memset(pCfg , 0, sizeof(*pCfg));
	/*
	 * Set information element header 
	 */
	pCfg->ElmHdr.Id = DOT11_PD_THRESHOLD; 
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);
	
	/*
	 * Set configuration fields 
	 */
	pCfg->dot11PDThreshold = ENDIAN_HANDLE_LONG((UINT32)pdThresh);

	/*
	 * Send the configuration command
	 */
	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), NO_WAIT));
}

/****************************************************************************
 *                      whal_acxConfigReadPacketDetection()
 ****************************************************************************
 * DESCRIPTION:	Get Packet Detection threshold from FW.
 * 
 * INPUTS:	
			
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigReadPacketDetection( WlanElm_PDThreshold_T *acxElem, whal_userMboxCB userCB, UINT8 *replyBuf)
{
	acxElem->ElmHdr.Id  = DOT11_PD_THRESHOLD;
	acxElem->ElmHdr.Len = sizeof(*acxElem) - sizeof(ElmHdr_T);

	return whal_acxMboxInterrogateNoWait(CMD_INTERROGATE, acxElem, sizeof(*acxElem), userCB, replyBuf);

}

/****************************************************************************
 *                      whal_apiSetRXdot11Mmode()
 ****************************************************************************
 * DESCRIPTION:	Set RX dot11 mode : 1 - OFDM, 2 - CCK, 3 - both
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/

int whal_acxConfigRXdot11Mode(UINT32 mode)
{
	WlanElm_RxDot11Mode_T acxElem;
	WlanElm_RxDot11Mode_T *pCfg = &acxElem;

	pCfg->ElmHdr.Id = DOT11_RX_DOT11_MODE;
	pCfg->ElmHdr.Len = sizeof(*pCfg) - sizeof(ElmHdr_T);

	pCfg->dot11RxMode = ENDIAN_HANDLE_LONG(mode);

	return (whal_acxMboxConfigure(pCfg, sizeof(*pCfg), NO_WAIT));

}


/****************************************************************************
 *                      whal_acxConfigReadRXdot11Mmode()
 ****************************************************************************
 * DESCRIPTION:	Read RX dot11 mode
 * 
 * INPUTS:	
			
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxConfigReadRXdot11Mmode(WlanElm_RxDot11Mode_T *acxElem, whal_userMboxCB userCB, UINT8 *replyBuf)
{
	
	acxElem->ElmHdr.Id  = DOT11_RX_DOT11_MODE;
	acxElem->ElmHdr.Len = sizeof(*acxElem) - sizeof(ElmHdr_T);

	return whal_acxMboxInterrogateNoWait(CMD_INTERROGATE, acxElem, sizeof(*acxElem), userCB, replyBuf);
}

