/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 2000-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/
/**************************************************************************/
/*                                                                        */
/*   MODULE:  whalAcxEeprom.c                                             */ 
/*   PURPOSE: ACX100 EEPROM access routines implemenatation               */
/*                                                                        */
/**************************************************************************/

#include "whalAcxRegs.h"
#include "whalCommon.h"
#include "whalAcxAccess.h"
#include "whalAcxEeprom.h"

/* #define _EEPROM_DEBUG_ */
/************************************************************************
*        UINT32 whal_acxEepromReadByte(UINT16 wAddr,UINT8 *pbVal)       *
*************************************************************************
* DESCRIPTION:  Reads a single byte from EEPROM.                        *
*                                                                       *
* INPUT:        wAddr - 16-bits EEPROM addrress                         *
*               pbVal - pointer the to output parameter - EEPROM value  *     
*                                                                       *
* OUTPUT:       *pbVal contains EEPROM value                            *   
*                                                                       *
* RETURN:       OK - successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32 whal_acxEepromReadByte(UINT16 wAddr,UINT8 *pbVal)
{
   int i = 0x00;

    wAddr &= 0x07ff;
    whal_acxRegWrite(ACX_EE_ADDR_REG,(UINT32)wAddr);
    whal_acxRegWrite(ACX_EE_CTL_REG,EE_READ);

    while((whal_acxRegRead(ACX_EE_CTL_REG) & EE_READ) != 0x00)
    {
        if(i++ > EEPROM_ACCESS_TO)
        {
            return NOK; /* timeout */
        }
    }
    
    *pbVal = (UINT8)(whal_acxRegRead(ACX_EE_DATA_REG) & 0xff);
    return OK;
}

/************************************************************************
*      UINT32 whal_acxEepromWriteByteNoUnp(UINT16 wAddr,UINT8 bVal)     *
*************************************************************************
* DESCRIPTION:  Writes a single byte to EEPROM                          *  
*                                                                       *
* INPUT:        wAddr - 16-bits EEPROM addrress                         *
*               bVal  - new value                                       *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       OK - successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32 whal_acxEepromWriteByteNoUnp(UINT16 wAddr,UINT8 bVal)
{
   volatile int i=0;

   wAddr &= 0x07ff;
   whal_acxRegWrite(ACX_EE_ADDR_REG,(UINT32)wAddr);
   whal_acxRegWrite(ACX_EE_DATA_REG,bVal);
   whal_acxRegWrite(ACX_EE_CTL_REG,EE_WRITE);
    
	while((whal_acxRegRead(ACX_EE_CTL_REG) & EE_WRITE) != 0x00)
   {
		volatile int y=0;

   	for( ; y<100; y++) {}
      if(i++ > EEPROM_ACCESS_TO)
      {
          return NOK; /* timeout */
      }
   }

   return OK;
}

/************************************************************************
*           void whal_acxEepromProtect(void)                            *
*************************************************************************
* DESCRIPTION:  Set EEPROM write protection.                            *
*               Inhibits writing to the EEPROM.                         *
*                                                                       *
* INPUT:        N/A                                                     *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       N/A                                                     *
*************************************************************************/
void whal_acxEepromProtect(void)
{
}

/************************************************************************
*           void whal_acxEepromUnprotect(void)                          *
*************************************************************************
* DESCRIPTION:  Remove EEPROM write protection.                         *
*               Enables writing to the EEPROM.                          *
*                                                                       *
* INPUT:        N/A                                                     *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       N/A                                                     *
*************************************************************************/
void whal_acxEepromUnprotect(void)
{
   /* reset GPIO4 to unprotect EEPROM- MR#2799 */
   static int p =0;
   if( !p) {
      UINT32 mask = 1ul << 4;
      whal_acxRegResetBitVal( ACX_GPO_CFG, mask);        /* set pin to normal GPIO (muxed) */
      whal_acxRegResetBitVal( ACX_GPIO_OE_REG, mask);    /* set pin as output */
 	   whal_acxRegResetBitVal( ACX_GPIO_OUT_REG, mask);   /* reset the WP pin  */
      p = 1;
   }
}

/************************************************************************
*          UINT32 whal_acxEepromWriteByte(UINT16 wAddr,UINT8 bVal)      *
*************************************************************************
* DESCRIPTION:  Writes a single byte to EEPROM                          *  
*                                                                       *
* INPUT:        wAddr - 16-bits EEPROM addrress                         *
*               bVal  - new value                                       *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       OK - successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32 whal_acxEepromWriteByte(UINT16 wAddr,UINT8 bVal)
{
	UINT32 retCode;
	
   whal_acxEepromUnprotect();
   PAL_sleepLong(100);
   retCode = whal_acxEepromWriteByteNoUnp(wAddr, bVal);
   PAL_sleepLong(100);
   whal_acxEepromProtect();
   return retCode;
}

/************************************************************************
*           UINT32 whal_acxEepromGetCalValue(UINT8 *pbVal)              *
*************************************************************************
* DESCRIPTION:  Reads oscillator cal. value from EEPROM                 *
*                                                                       *
* INPUT:        N/A                                                     *
*                                                                       *
* OUTPUT:       pbVal - pointer to the output parameter                 *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32 whal_acxEepromGetCalValue(UINT8 *pbVal)
{
    return whal_acxEepromReadByte(ACX_EEPROM_OSC_ADDR,pbVal);
}

/************************************************************************
*           UINT32 whal_acxEepromSetCalValue(UINT8 bVal)                *
*************************************************************************
* DESCRIPTION:  Writes new oscillator cal. value to EEPROM              *
*                                                                       *
* INPUT:        bVal - new oscillator cal. value                        *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout or invalid value                           *
*************************************************************************/
UINT32 whal_acxEepromSetCalValue(UINT8 bVal)
{
   if(bVal > MAX_OSC_CAL)
   {
      return NOK;       
   }
   
   return whal_acxEepromWriteByte(ACX_EEPROM_OSC_ADDR,bVal);
}

/************************************************************************
*           UINT32 whal_acxEepromGetBiasValue(UINT8 *pbVal)             *
*************************************************************************
* DESCRIPTION:  Reads bias value from EEPROM                            *
*                                                                       *
* INPUT:        N/A                                                     *
*                                                                       *
* OUTPUT:       pbVal - pointer to the output parameter                 *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32 whal_acxEepromGetBiasValue(UINT8 *pbVal)
{
    return whal_acxEepromReadByte(ACX_EEPROM_BIAS_ADDR,pbVal);
}

/************************************************************************
*           UINT32 whal_acxEepromSetBiasValue(UINT8 bVal)               *
*************************************************************************
* DESCRIPTION:  Writes new bias value to EEPROM                         *
*                                                                       *
* INPUT:        bVal - new bias value                                   *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32 whal_acxEepromSetBiasValue(UINT8 bVal)
{
    return whal_acxEepromWriteByte(ACX_EEPROM_BIAS_ADDR,bVal);
}

/************************************************************************
*           UINT32 whal_acxEepromGetDACValue(UINT8 *pbVal)              *
*************************************************************************
* DESCRIPTION:  Reads DAC value from EEPROM                             *
*                                                                       *
* INPUT:        N/A                                                     *
*                                                                       *
* OUTPUT:       pbVal - pointer to the output parameter                 *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32 whal_acxEepromGetDACValue(UINT8 *pbVal)
{
    return whal_acxEepromReadByte(ACX_EEPROM_DAC_ADDR,pbVal);
}

/************************************************************************
*           UINT32 whal_acxEepromSetDACValue(UINT8 bVal)                *
*************************************************************************
* DESCRIPTION:  Writes new DAC value to EEPROM                          *
*                                                                       *
* INPUT:        bVal - new DAC value                                    *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32 whal_acxEepromSetDACValue(UINT8 bVal)
{
    return whal_acxEepromWriteByte(ACX_EEPROM_DAC_ADDR,bVal);
}

/************************************************************************
*           int whal_acxEepromLoadBaseBandTable(void)                   *
*************************************************************************
* DESCRIPTION:  Loads BB registers table from EEPROM                    *
*                                                                       *
* INPUT:        N/A                                                     *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       N/A                                                     *
*************************************************************************/
int whal_acxEepromLoadBaseBandTable(void)
{
   UINT8   bRegVal;
   UINT16  wTableAddress;
   UINT8   bNumberOfEntries;  
   UINT16  wTableEnd;
   UINT8   bEntrySize; 
   UINT8   bbAddr;
   UINT8   bbData;
   int addr;

   if(whal_acxEepromReadByte(0x14a,&bRegVal) != OK)
   {
      #ifdef _EEPROM_DEBUG_
      printf("ERROR on %s: %d\n",__FILE__, __LINE__);
      #endif
      return NOK;
   }
   
   wTableAddress = bRegVal;
   
   if(whal_acxEepromReadByte(0x14b,&bRegVal) != OK)
   {
      #ifdef _EEPROM_DEBUG_
      printf("ERROR on %s: %d\n",__FILE__, __LINE__);
      #endif
      return NOK;
   }
   
   wTableAddress |= bRegVal << 8;

   if(whal_acxEepromReadByte(wTableAddress,&bRegVal) != OK)
   {
      #ifdef _EEPROM_DEBUG_
      printf("ERROR on %s: %d\n",__FILE__, __LINE__);
      #endif
      return NOK;
   }
   
   bNumberOfEntries = bRegVal;

   if(whal_acxEepromReadByte(wTableAddress+1,&bRegVal) != OK)
   {
      #ifdef _EEPROM_DEBUG_
      printf("ERROR on %s: %d\n",__FILE__, __LINE__);
      #endif
      return NOK;
   }

   bEntrySize = bRegVal;
   
#ifdef _EEPROM_DEBUG_
   printf("\tTable Address: 0x%x\n\tNumber of elements: 0x%x\n\tEntry Size: 0x%x\n",
                                              wTableAddress,bNumberOfEntries,bEntrySize);

#endif

   wTableEnd = wTableAddress + (bEntrySize * (bNumberOfEntries + 1));
   
   for(addr = wTableAddress + 2; addr < wTableEnd; addr+= bEntrySize)
   {
      if(whal_acxEepromReadByte(addr,&bbAddr)!= OK)
      {
         #ifdef _EEPROM_DEBUG_
         printf("ERROR on %s: %d\n",__FILE__, __LINE__);
         #endif
         return NOK;         
      }                        
   
      if(whal_acxEepromReadByte(addr+1,&bbData)!= OK)
      {
         #ifdef _EEPROM_DEBUG_
         printf("ERROR on %s: %d\n",__FILE__, __LINE__);
         #endif
         return NOK;         
      }

      /*
      whal_acxWritePHYReg(bbAddr,bbData);
      -- the follwing 3 statements do the same thing
      */
	   whal_acxRegWrite(ACX_PHY_ADDR_REG, bbAddr);
	   whal_acxRegWrite(ACX_PHY_DATA_REG, bbData);
	   whal_acxRegWrite(ACX_PHY_CTRL_REG, ACX_PHY_REG_WR_MASK);
#ifdef _EEPROM_DEBUG_
      printf("0x%x: 0x%x\n",bbAddr,bbData);
#endif
   }
   return OK;
}/* END whal_acxEepromLoadBaseBandTable() */

/************************************************************************
*   UINT32 whal_acxEepromGetAGCCell(UINT8 bTableOffset, UINT8 *pbVal)   *
*************************************************************************
* DESCRIPTION:  Reads one cell from ACG table                           *
*                                                                       *
* INPUT:        bTableOffset - zero-based offset of the cell in AGC     *   
*                              table                                    *
* OUTPUT:       pbVal - pointer to the output parameter                 *       
*                                                                       *
* RETURN:       OK                                                      *
*               NOK - EEPROM write error                                *
*               INVALID_PARAMETER1 - invalid parameter 1                *  
*************************************************************************/
UINT32 whal_acxEepromGetAGCCell(UINT8 bTableOffset, UINT8 *pbVal)
{
    if(bTableOffset > MAX_AGC_TABLE_ENTRIES)
    {
        *pbVal = 0x00;
        return INVALID_PARAMETER1;
    }

    return whal_acxEepromReadByte(ACX_EEPROM_AGC_TABLE_ADDR+bTableOffset,pbVal);
}
 
/************************************************************************
*   UINT32 whal_acxEepromSetAGCCell(UINT8 bTableOffset, UINT8 bVal)     *
*************************************************************************
* DESCRIPTION:  Writes one AGC table cell to EEPROM                     *
*                                                                       *
* INPUT:        bTableOffset - zero-based offset of the cell in AGC     *   
*                              table                                    *
*               bVal         - new cell value                           *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       OK                                                      *
*               NOK - EEPROM write error                                *
*               INVALID_PARAMETER1 - invalid parameter 1                *  
*************************************************************************/
UINT32 whal_acxEepromSetAGCCell(UINT8 bTableOffset, UINT8 bVal)
{
    if(bTableOffset > MAX_AGC_TABLE_ENTRIES)
    {
        return INVALID_PARAMETER1;
    }

    return whal_acxEepromWriteByte(ACX_EEPROM_AGC_TABLE_ADDR+bTableOffset, bVal);
} 

/************************************************************************
*           UINT32 whal_acxEepromGetRadioType(UINT8 *pbVal)             *
*************************************************************************
* DESCRIPTION:  Reads radio type value from EEPROM                      *
*                                                                       *
* INPUT:        N/A                                                     *
*                                                                       *
* OUTPUT:       pbVal - pointer to the output parameter                 *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32	whal_acxEepromGetRadioType(UINT8 *pbVal)
{
    return whal_acxEepromReadByte(ACX_EEPROM_CARD_TYPE_ADDR, pbVal);
}

/************************************************************************
*           UINT32 whal_acxEepromSetRadioType(UINT8 bVal)               *
*************************************************************************
* DESCRIPTION:  Writes new radio type to EEPROM                         *
*                                                                       *
* INPUT:        bVal - new radio type value                             *
*                                                                       *
* OUTPUT:       N/A                                                     *
*                                                                       *
* RETURN:       OK   successful                                         *
*               NOK  timeout                                            *
*************************************************************************/
UINT32	whal_acxEepromSetRadioType(UINT8 bVal)
{
    return whal_acxEepromWriteByte(ACX_EEPROM_CARD_TYPE_ADDR, bVal);
}


