/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2001 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/*********************************************************************************
*                                                                                     
*   MODULE:  whalAcxMbox.h                                                                            
*   PURPOSE: Acx mailbox object api
*                                                                                                          
**********************************************************************************/
#ifndef _WHAL_whal_acxMbox_H
#define _WHAL_whal_acxMbox_H

#include "AcxCommands.h"

/* Timeouts in ticks */
#define NO_WAIT				0
#define INIT_TIMEOUT		10
#define CONFIG_TIMEOUT		10
#define READ_TIMEOUT		10		
#define PREV_TIMEOUT		100
#define CALIBRATE_TIMEOUT		100

#define MAX_CMND_PARAMS		384
#define MAX_INFO_PARAMS		40

typedef void (*whal_userMboxCB)(BOOL Sts, UINT8 *buf);

typedef struct 
{
	UINT16 CmdType;
	UINT16 Status;
	UINT8  parameters[MAX_CMND_PARAMS];
} CmndMbox_T;

typedef struct 
{
	UINT16 InfoType;
	UINT16 Status;
	UINT8  parameters[MAX_INFO_PARAMS];
} InfoMbox_T;



#define MAX_MBOX_SIZE 384
/* don't reduce the queue size - wOne will not work */
#define MBOX_QUEUE_NUM_ENTRIES 16 
#define MBOX_QUEUE_SIZE_MASK 0xF
#define MBOX_CMD_FAILURE_THRESHOLD 100 /* ms */

typedef struct
{
	UINT8	cmdBuf[MAX_MBOX_SIZE];
	UINT32	cmdLen;
	whal_userMboxCB cb;
	UINT8   *replyBuf;
	AcxMboxCommand_T cmdType;
} whalMboxQueue_T;

extern int whal_acxMboxInit(void);
extern int whal_acxMboxCreate(void);
extern int whal_acxMboxUnload(void);
extern int whal_acxMboxCommand(AcxMboxCommand_T MboxCmd, char *ParamsBuf, int ParamsLen, UINT32 Timeout);
extern int whal_acxMboxConfigure( void *ParamsBuf, UINT32 ParamsLen, UINT32 Timeout);
extern int whal_acxMboxInterrogate( void *ParamsBuf, UINT32 ParamsLen );
extern int whal_acxMboxInterrogateNoWait( AcxMboxCommand_T Cmd,void *ParamsBuf, UINT32 ParamsLen, whal_userMboxCB userCB, UINT8 *replyBuf);
extern int whal_acxMboxReadParameters( char *Buf, int Size );
extern void whal_acxMboxSetCommandStatus(UINT16 StatusVal);
extern int  whal_acxMboxSetComplete(void);
extern int whal_acxMboxPrint(char *buf);
extern int wdrv_mailbox_getEvent(UINT32 event);

#endif