/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalAcxProcess.c
 *   PURPOSE: Implements action on the Acx card (Reset, Run, SendCmd, Sw Download)
 *
 ****************************************************************************/

#include "whalCommon.h"
#include "whal.h"
#include "whalAcxRegs.h"
#include "whalInterrupt.h"
#include "whalAcxAccess.h"
#include "whalAcxProcess.h"
#include "whalAcxMbox.h"
#include "whalAcxConfig.h"
#include "whalRxQueue.h"
#include "whalTxQueue.h"
#include "whalRxBufs.h"
#include "whalRxMaster.h"
#include "whalTxBufs.h"
#include "whalParams.h"
#include "whalAcxCmd.h"
#include "whalAcxEeprom.h"
#include "drvDbMain.h"
#include "pal.h"
#include "acx_hw.h"

#define FW_DL_MAX_SIZE 0x20000

/* 
   Global optional User Call-Back. The user can define this function as extern in his
   BSP and return the Host SDRAM Base address he wants to exposure to the VLYNQ remote
   module.  See implementation in whal_acxConfigDma() .
   
e.g:  from the BSP (outside the WLAN Driver module scope): 

#define HOST_SDRAM_BASE_ADDRESS    0x14000000

extern	unsigned int(*getHostSdram_CB)(void);  
unsigned int acxGetHostSdram(void);

    getHostSdram_CB=acxGetHostSdram;
	UINT32 acxGetHostSdram(void) { HOST_SDRAM_BASE_ADDRESS }
   
*/   
UINT32(*getHostSdram_CB)(void) = NULL;  

/****************************************************************************
 *               External labels, taken from acx100_bin.S
 ****************************************************************************
 * 
 ****************************************************************************/

UINT8 * Acx100_bin_0_Array = NULL;
UINT8 * Acx100_bin_0_Array_Extra = NULL;
UINT32 Acx100_bin_0_Array_Len;
UINT32 Acx100_bin_0_Array_Extra_Len;

extern UINT8   _ACX111_REG_binArrayStart[];    /* Debug image           */
extern UINT8   _ACX111_REG_binArrayEnd[];      /* end of debug image    */
extern UINT8   _ACX111_REG_Version[];          /* debug image version   */

/* Radio modules */
/* RFMD */
extern UINT8   _Acx100_radio_0_Version[];      /* radio image 0 version       */
extern UINT8   _Acx100_radio_0_ArrayStart [];  /* binary radio image 0        */
extern UINT8   _Acx100_radio_0_ArrayEnd[];     /* end of binary radio image 0 */
/* Maxim */
extern UINT8   _Acx100_radio_1_Version[];      /* radio image 1 version       */
extern UINT8   _Acx100_radio_1_ArrayStart[];   /* binary radio image 1        */
extern UINT8   _Acx100_radio_1_ArrayEnd[];     /* end of binary radio image 1 */
#if 0
/* Radia bg */
extern UINT8   _Acx100_radio_2_Version[];      /* radio image 2 version       */
extern UINT8   _Acx100_radio_2_ArrayStart[];   /* binary radio image 2        */
extern UINT8   _Acx100_radio_2_ArrayEnd[];     /* end of binary radio image 2 */
/* Radia abg */
extern UINT8   _Acx100_radio_3_Version[];      /* radio image 3 version       */
extern UINT8   _Acx100_radio_3_ArrayStart[];   /* binary radio image 3        */
extern UINT8   _Acx100_radio_3_ArrayEnd[];     /* end of binary radio image 3 */
#endif



/****************************************************************************
 *                      OBJECT DATA BASE + INTERNALS
 ****************************************************************************
 * 
 ****************************************************************************/
int  whal_acxProcWaitInitComplete(void);
void whal_acxProcRun(void);
void whal_acxProcReset(void);
int  whal_acxProcLoadFwImage(UINT32 Acx_BaseMemAddr, UINT32 AddrOffset, UINT32 *RetDataLen);
int  whal_acxProcDownloadFwImageFTP(unsigned char **pBuf, int *Len, int Type);
int  whal_acxProcDownloadFwImageTFTP(unsigned char **pBuf, int *Len, int Type);
int  whal_acxProcGetFwImage(unsigned char **pBuf, int *Len, unsigned char **, int *);
int  whal_acxProcGetRadioImage(unsigned char **pBuf, int *Len);
int  whal_acxProcInitiateDebug(void);

int  whal_acxConfigDma(void);
#if 0
static void	acxProc_FwBinNumSet (UINT8 aVal);
static UINT8 acxProc_FwBinNumGet (void);

static UINT8 fwBinNumSelected = 0xFF;
#endif

/****************************************************************************
 *                      whal_acxProcMemoryConfigure()
 ****************************************************************************
 * DESCRIPTION:	Configure the TNETW1130 memory
 * 
 * INPUTS:  None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
UINT32 wDbgHostAddr = ACX111_VL1_REMOTE_BASE;
int wDbgWait1=0;
int whal_acxProcMemoryConfigure(void)
{
	DmaParams_T *pDmaParams = whal_ParamsGetDmaParams(0);
	dot11_tim_T *pWlanElm_Tim = whal_ParamsGetElm_Tim();
	int	 PrivacyImplemented = whal_ParamsGetWepPrivacyOptionImplemented();
	WlanElm_QueueHead_T WlanElm_QueueHead;
	UINT32 TxQueuesSize;
	UINT32 HostRxDescriptorsAddr;
	char MacAddr[6];
	int Stt;
	int Qid;
	int  i, KeySize, DefaultKeyId;
	char Key[KEY_SIZE_MAX_BYTES];
	char EthAddr[6] = {0x3, 0x4, 0x5, 0x6, 0x7, 0x8};
	int    originalSupportTIcapTemplate;
	int saveLengthTim;
	UINT8 WoneIndex, LocalQid, TotalWones, WoneTxQueues;
	edcf_info_t  edcfInfo;
    int   edcfInfo_RC;

	UINT32  queueThresh;

  
   wdrv_dbGetNumSupWone(&TotalWones);


	/*
	 * Configure Beacon and ProbeResponse templates
	 * --------------------------------------------
	 */
	 
	for (WoneIndex = 0 ; WoneIndex < TotalWones ; WoneIndex++)
	{
	 	
		printf("whal_acxProcConfigure: templates (beacon, probe response, tim) for [%d] Network\n", WoneIndex);
		/* configure the ACX100 templates */
#ifndef NO_PBCC_CHECK
		if (whal_ParamsGetNvParams()->ProbePatch == 1)
			printf("whal_acxProcConfigure: NO probe response template\n");
		else
#endif
	
		{
            /* First ProbeRsp template should be of max. len, so include Rsne IE if required with max length */
			dot11_TIcap_T TIcapStruct;
#if 0
			printf("@@@@@@@ len = %d\n",pWlanElm_Rsne->hdr.eleLen);
			for(i =0;i<pWlanElm_Rsne->hdr.eleLen;i++)
			{
				printf("LALALA0:char %d = 0x%x\n",i,pWlanElm_Rsne->rsne[i]);
			}
#endif

            /*if ((pWlanElm_Rsne != NULL) && (pWlanElm_Rsne->hdr.eleLen))
            {
                saveLengthRsne = pWlanElm_Rsne->hdr.eleLen;
                pWlanElm_Rsne->hdr.eleLen = DOT11_RSNE_MAX;
            }*/
			originalSupportTIcapTemplate = whal_ParamsGetGenParams(0)->SupportTIcapTemplate;
			/* First ProbeRsp template should be of max. len, so include 4x IE */
			whal_ParamsGetGenParams(0)->SupportTIcapTemplate = TRUE; 
			/* Set the TI Capability (4x) to its maximum size so the Probe Response template's IE has the maximum size */
			PAL_memSet(&TIcapStruct,0,sizeof(dot11_TIcap_T));
			TIcapStruct.hdr.eleLen = DOT11_TI_CAP_MAX_LEN;
			whal_ParamsSetElm_TIcapPrivateSupport(&TIcapStruct);
			/* whal_acxCmdProbeRespTemplate(WoneIndex); */
			whal_ParamsSetElm_TIcapSupport();	/* Restore the IE to the real one */
			whal_ParamsGetGenParams(0)->SupportTIcapTemplate = originalSupportTIcapTemplate;
            /*if ((pWlanElm_Rsne != NULL) && (pWlanElm_Rsne->hdr.eleLen))
            {
                pWlanElm_Rsne->hdr.eleLen = saveLengthRsne;
            }*/
			whal_acxCmdProbeRespTemplate(WoneIndex);
		}
	
		whal_acxCmdBeaconTemplate(WoneIndex); 
	
		/* Tim template, first reserve space (len=MAX), second init to empty */
		saveLengthTim = pWlanElm_Tim->hdr.eleLen;
		pWlanElm_Tim->hdr.eleLen = 3+DOT11_PARTIAL_VIRTUAL_BITMAP_MAX;
        /*if ((pWlanElm_Rsne != NULL) && (pWlanElm_Rsne->hdr.eleLen))
        {
            pWlanElm_Rsne->hdr.eleLen = DOT11_RSNE_MAX;
        }*/
		
	    whal_acxCmdTimTemplate(WoneIndex);
	   
		pWlanElm_Tim->hdr.eleLen = saveLengthTim;
        /*if ((pWlanElm_Rsne != NULL) && (pWlanElm_Rsne->hdr.eleLen))
        {
            pWlanElm_Rsne->hdr.eleLen = saveLengthRsne;
        }*/
		whal_acxCmdTimTemplate(WoneIndex);		
	}

	if (TotalWones > 1)
	{
		whal_acxCmdWoneToggleTemplate();
	}    

	/*
	 * Configure memory queues and pools
	 * ---------------------------------
	 */

	HostRxDescriptorsAddr = (UINT32)whal_rxMaster_getHostQueueAddr();

	printf("whal_acxProcConfigure: Host Queue is in 0x%X\n", (int)HostRxDescriptorsAddr);
	Stt = whal_acxConfigMemory(HostRxDescriptorsAddr);
	if (Stt != OK)
	{
		printf("whal_acxProcMemoryConfigure: ERROR: whal_acxConfigMemory failed\n");
		return(NOK);
	}
	
	/*
	 * Configure wep (empty)
	 * ---------------------
	 */
	printf("whal_acxProcConfigure: WEP cache\n");


	/* set start address (after the code ends */
	/*whal_acxConfigGetSet_MemoryMap(pMemMap, IDX_WEP_CACHE_START);*/

	/* set mapping keys and default keys */
	
	if (PrivacyImplemented)
	{
#if 0
		/* load mapping keys, config keys with "WepOn" */
		whal_ParamsGetWepKeyMappingSize(&NumMappingKeys);
		for (i=0; i<NumMappingKeys; i++)
		{
			if (whal_ParamsGetWepKeyMappingWepOn(i) == 1)
			{
				whal_ParamsGetWepKeyMappingMacAddr(i, MacAddr);
				whal_ParamsGetWepKeyMappingKeyVal(i, Key);
				whal_ParamsGetWepKeyMappingKeySize(i, &KeySize);
				whal_acxCmdSetKey(KEY_ADD_OR_REPLACE, MacAddr, ENTRY_INDEX_IGNORE,
									 KeySize, KEY_WEP_ADDR, IGNORE_KEY_ID, Key);
			}
		}
#endif
		/* Set default keys */
		for (i=0; i<WEP_NUM_DEFAULT_KEYS; i++)
		{
			whal_ParamsGetWepDefaultKey(i, Key);
			whal_ParamsGetWepDefaultKeySize(i, &KeySize);
			/* MacAddr is ignored for Default Key setting */
			PAL_memCopy(MacAddr, EthAddr, 6);			
			whal_acxCmdSetKey(KEY_ADD_OR_REPLACE, MacAddr, ENTRY_INDEX_IGNORE,
								 KeySize, KEY_WEP_DEFAULT, i, Key);
		}		

		/* Set default key Id */
		whal_ParamsGetWepDefaultKeyId(&DefaultKeyId);
		whal_acxConfigWepDefaultKeyId(DefaultKeyId);
	}

	/*
	 * Configure Feature Options
	 * -------------------------
	 */

	whal_acxConfigFeatureOptions(0);	

	/*
	 * Read Queue heads
	 * ----------------
	 */
	printf("whal_acxProcConfigure: read queue heads\n");	
	Stt = whal_acxConfigRead_QueueHead(&WlanElm_QueueHead);
	if (Stt != OK)
	{
		printf("whal_acxProcMemoryConfigure: ERROR: whal_acxConfigRead_QueueHead failed\n");
		return(NOK);
	}

	while(wDbgWait1)
	{
		PAL_sleepLong(500);
	}
	/* 
	 * configure RxQueue/TxQueue objects 
	 */
	printf("whal_acxProcConfigure: configure RxQueue/TxQueue objects\n");
	
	TxQueuesSize = 0;
	
	for (WoneIndex = 0, Qid = 0; WoneIndex < TotalWones; WoneIndex++)
	{	
   		/*
	 	 * Tx queue config 
		 */

		wdrv_dbGetNumSupWoneQueues( WoneIndex, &WoneTxQueues );
		for (LocalQid=0; LocalQid<WoneTxQueues; LocalQid++,Qid++)                              
		{
			TxQueuesSize  += whal_txQueue_Config(Qid, (UINT32)WlanElm_QueueHead.TxQueueHead[Qid].Addr);

			/* If required, configure the ACX Tx queue EDCF QoS parameters & packet bursting parameters. */
			edcfInfo_RC = wdrv_dbGetEdcfParams(WoneIndex, LocalQid, &edcfInfo);
            if(edcfInfo_RC)
                whal_acxConfigAccessCategory(Qid, &edcfInfo);

			/* If required, configure the ACX Tx queue buffers usage threshold. */
			if ( wdrv_dbGetQueueThresh(WoneIndex, LocalQid, &queueThresh) )
				whal_acxConfigQueueThresh(Qid, queueThresh);
		}
	}

	/* 
	 * calculate the number of blocks
	 */
	pDmaParams->NumRxBlocks = ((UINT32)WlanElm_QueueHead.txMemBlkQ - (UINT32)WlanElm_QueueHead.rxMemBlkQ)/pDmaParams->RxBlockSize;
	pDmaParams->NumTxBlocks = pDmaParams->NumRxBlocks*(20-pDmaParams->RxBlocksRatio)/pDmaParams->RxBlocksRatio;

	whal_ParamsPrintMemMap();
	return OK;
}


/****************************************************************************
 *                      whal_acxProcInitiate()
 ****************************************************************************
 * DESCRIPTION:	Download firmware code to the ACX100 and run it
 * 
 * INPUTS:  None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int wDbgWait = 0;

int whal_acxProcInitiate(void)
{
	int Stt;
	UINT32 DeviceVendorId;
	UINT32 ImageDataLen;
	int FwNumber;

   FwNumber = whal_ParamsGetNvParams()->FwNumber;
   if( FwNumber  == 3 )
   {
      Stt = whal_acxProcInitiateDebug();
      return Stt;
   }

#if( USE_ENDIAN_SWAP == 0)
	/*for  little endian system.*/
	whal_acxRegWrite(ACX_REG_SLV_END_CTL, 0x00000000);
#else
	/* use endian swap */
	whal_acxRegWrite(ACX_REG_SLV_END_CTL, 0xffffffff);
#endif

	while(wDbgWait)
	{
		PAL_sleepLong(500);
	}
	/*
	 * Identification test
	 */
    DeviceVendorId = whal_acxRegRead(REG_DEVICE_VENDOR_ID);
	if (DeviceVendorId != ((ACX_DEVICE_ID<<16) | ACX_VENDOR_ID))
	{
		printf("whal_acxProcInitiate: ERROR, Fail to identify ACX100 card\n");
		return NOK;
	}

	printf("whal_acxProcInitiate: found DEVICE_VENDOR ID = 0x%x\n", DeviceVendorId);

	whal_acxProcReset();

	/*
	 * Read Halt bit
	 */
	if (whal_acxRegIsBitSet( ACX_REG_ECPU_CONTROL, ECPU_CONTROL_HALT ))
	{
		printf("whal_acxProcInitiate: Cpu halt -> download code\n");
		if (! Acx100_bin_0_Array ) {
			Acx100_bin_0_Array = _Acx100_bin_0_ArrayStart;
			Acx100_bin_0_Array_Len =  _Acx100_bin_0_ArrayEnd - _Acx100_bin_0_ArrayStart;		
		}
		Stt = whal_acxProcLoadFwImage(ACX100_MEM_ADDRESS, 0, &ImageDataLen);
		if (Stt != 1)
		{
			printf("whal_acxProcResetRun: download failed\n");
			return(-1);
		}
	}


   whal_acxConfigDma();
   printf("whal_acxProcInitiate: run\n");
   whal_acxProcRun();


   if ( !whal_acxProcWaitInitComplete() )
   {
		printf("whal_acxProcInitiate: Timeout waiting for the ACX100 to complete Initialization\n");
		return (-1);
   }

	return(0);
}

/****************************************************************************
 *                      whal_acxProcRun()
 ****************************************************************************
 * DESCRIPTION:	Run the ACX100 firmware
 * 
 * INPUTS:  None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_acxProcRun(void)
{
	/*
	 * Run Acx Cpu
	 */
	whal_acxRegResetBitVal(ACX_REG_ECPU_CONTROL, ECPU_CONTROL_HALT);
	/*PAL_sleepLong(500);*/
}

/****************************************************************************
 *                      whal_acxProcReset()
 ****************************************************************************
 * DESCRIPTION:	Reset the ACX100
 * 
 * INPUTS:  None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_acxProcReset(void)
{
	/*
	 * Halt the Acx Cpu
	 */
	whal_acxRegSetBitVal(ACX_REG_ECPU_CONTROL, ECPU_CONTROL_HALT);
}

/****************************************************************************
 *                      whal_acxProcWaitInitComplete()
 ****************************************************************************
 * DESCRIPTION:	Wait (polling) for init complete interrupt
 * 
 * INPUTS:  None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxProcWaitInitComplete(void)
{
	int i;

	printf("whal_acxProcWaitInitComplete:\n");
	for(i=0; i<40; i++) {
		PAL_sleepLong(50);
		if (whal_acxIntrCheckAndAck(INTR_INIT_COMPLETE))
			return(1);
	}

   return(0);
}

/****************************************************************************
 *                      whal_acxProcLoadFwImage()
 ****************************************************************************
 * DESCRIPTION:	Load Image from the host and download into the Acx100
 *                AddrOffset == 0 : FW is loaded
 *                AddrOffset != 0 : Module is loaded
 * 
 * INPUTS:  None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxProcLoadFwImage(UINT32 Acx_BaseMemAddr, UINT32 AddrOffset, UINT32 *RetDataLen)
{
	unsigned char *pNextChar;
	unsigned int  checksumCalc = 0;
	unsigned int  idx, temp;
	int i;
	unsigned int  FileCsum;
	unsigned int  DataLen;
	unsigned int  AcxOffset;
	unsigned int  NumWrittenChars;
	int fRC, Status;

	unsigned char *FileBuf = 0, *ExtraFileBuf = 0;
	int FileLen, ExtraFileLen;

	//	PAL_sleepLong(100);
	printf( "whal_acxProcLoadFwImage: 0x%x, 0x%x\n", Acx_BaseMemAddr, AddrOffset);
	//	PAL_sleepLong(100);
	/*
	 * get image 
	 */
	if (AddrOffset == 0)
	{
		/* firmware image */
	  puts("whal_acxProcLoadFwImage() -- Loading FW image");
	  Status = whal_acxProcGetFwImage(&FileBuf, &FileLen, &ExtraFileBuf, &ExtraFileLen);
	}
	else
	{
		/* radio image */
      puts("whal_acxProcLoadFwImage() -- Loading radio image");
		Status = whal_acxProcGetRadioImage(&FileBuf, &FileLen);
	}

	//	PAL_sleepLong(100);
	printf( "whal_acxProcLoadFwImage: 1, pBuf=0x%x, len=0x%x. Extra pBuf=%#x, len=%#x\n", (int)FileBuf, FileLen, (int)ExtraFileBuf, ExtraFileLen);
	//	PAL_sleepLong(100);
	if (Status < 0)
	{
		printf ("whal_acxProcLoadImage: can't load Image\n");
		return -1;
	}

	//	PAL_sleepLong(100);
	printf( "whal_acxProcLoadFwImage: 2, pBuf=0x%x, len=0x%x. Extra pBuf=%#x, len=%#x\n", (int)FileBuf, FileLen, (int)ExtraFileBuf, ExtraFileLen);
	//	PAL_sleepLong(100);
	/*
	 * Extract and Calculate CheckSum and Length
	 */
	FileCsum = ENDIAN_HANDLE_LONG(*(unsigned long *)&FileBuf[0]);
	DataLen  = ENDIAN_HANDLE_LONG(*(unsigned long *)&FileBuf[4]);

	*RetDataLen = DataLen;
	pNextChar = &FileBuf[4];

	checksumCalc = 0;

	/* Add the NumBytes to the checksum calculation. */
	for ( idx = 0; idx < 4; idx++ )  {
		checksumCalc += (unsigned int)*pNextChar++;
	}


	AcxOffset = AddrOffset;
	NumWrittenChars = 0;
	i = 3;
	temp = 0;

	//	PAL_sleepLong(100);
	printf( "whal_acxProcLoadFwImage: 3, pBuf=0x%x, len=0x%x, DataLen=0x%x\n", (int)FileBuf, FileLen, DataLen);
	//	PAL_sleepLong(100);
	/*
	 * download the image into the MemMapped area in the Acx100 board
	 */
	while ( NumWrittenChars < DataLen )  	{
		temp |= ((unsigned int)(*pNextChar)) << ( 8 * i );
		checksumCalc += ((unsigned int)(*pNextChar));
		++pNextChar;
		++NumWrittenChars;
		if ( --i < 0 ) 		{
			temp = ENDIAN_HANDLE_LONG(temp);

         /*
         if( AddrOffset)
         {
            printf( "whal_acxMemCopy(%p, %p, %d)\n", (char *)(Acx_BaseMemAddr+AcxOffset), (char *)&temp, sizeof(temp));
            PAL_sleepLong(100);
         }
         */

			whal_acxMemCopy((char *)(Acx_BaseMemAddr+AcxOffset), (char *)&temp, sizeof(temp));

			AcxOffset += 4;
			temp = 0;
			i = 3;
		}
		if (pNextChar >= FileBuf + FileLen)
			pNextChar = ExtraFileBuf;
	}

	//	PAL_sleepLong(100);
	printf( "whal_acxProcLoadFwImage: 4, pBuf=0x%x, len=0x%x\n", (int)FileBuf, FileLen);
	//	PAL_sleepLong(100);
	/*
	 * CheckSum
	 */
	printf ("whal_acxProcLoadFwImage: Checksum, calc=0x%x, file=0x%x\n", checksumCalc, FileCsum);
	fRC = (checksumCalc == FileCsum);
	PAL_sleepLong(300);
	//	printf( "pBuf=0x%x, len=0x%x\n", (int)FileBuf, FileLen);
	//	PAL_sleepLong(1000);

	return(fRC);
}

/****************************************************************************
 *                      whal_acxProcDownloadFwImageFTP()
 ****************************************************************************
 * DESCRIPTION:	Download image from the host (via FTP)
 * 
 * INPUTS:  None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxProcDownloadFwImageFTP(unsigned char **pBuf, int *Len, int Type)
{
	return OK;
#if 0
	char FileName[256] = "host:c:/firmware/";
	char InputFileName[40];
	unsigned char *FileBuf;
	int FileLen;
	int fd;

	/* 
	 * Prepare file name 
	 */
	if (Type == 0)
		strcpy(InputFileName, "acx100.bin");
	else
		strcpy(InputFileName, "radio.bin");

	strcat(FileName, InputFileName);
	printf("whal_acxProcDownloadFwImageFTP: Filename=%s\n", FileName);

	/* 
	 * Initiate the loading 
	 */
	FileBuf = PAL_memAlloc(0,FW_DL_MAX_SIZE,0,0);
	fd = open(FileName, O_RDONLY, 0);
	if (fd < 0)
	{
		printf("whal_acxProcDownloadFwImageFTP: Can't open file %s\n", FileName);
		return -1;
	}

	/* 
	 * Load 
	 */
	FileLen = read(fd, FileBuf, FW_DL_MAX_SIZE);
	printf("whal_acxProcDownloadFwImageFTP: read buf=0x%x, len=%d\n", FileBuf, FileLen);

	/* 
	 * Close and return 
	 */	
	close(fd);

	*pBuf = FileBuf;
	*Len = FileLen;

	return(0);
#endif
}

#ifndef TIWLAN_NO_FIRMWARE_TFTP

/****************************************************************************
 *                      whal_acxProcDownloadFwImage()
 ****************************************************************************
 * DESCRIPTION:	Download image from the host (via TFTP)
 * 
 * INPUTS:  None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_acxProcDownloadFwImageTFTP(unsigned char **pBuf, int *Len, int Type)
{
	return OK;
#if 0
	char FileName[256] = "";
	char InputFileName[40];
	unsigned char *FileBuf;
	unsigned long IpAddr_Long;
	unsigned char IpAddr_Str[16];
	int FileLen;
    int	err;
	int fd;

	/* 
	 * Prepare file name 
	 */
	if (Type == 0)
		strcpy(InputFileName, "acx100.bin");
	else
		strcpy(InputFileName, "radio.bin");

	strcat(FileName, InputFileName);
	printf("whal_acxProcDownloadFwImage(TFTP): Filename=%s\n", FileName);

	FileBuf = PAL_memAlloc(0,FW_DL_MAX_SIZE,0,0);
	FileLen = FW_DL_MAX_SIZE;

	/* 
	 * Initiate the loading 
	 */
	cm_hwGetHostIP(&IpAddr_Long);
	IpAddr_Long = htonl(IpAddr_Long);
	sprintf( IpAddr_Str,"%d.%d.%d.%d%",((IpAddr_Long >> 24) & 0xFF),
                                       ((IpAddr_Long >> 16) & 0xFF),
                                       ((IpAddr_Long >> 8) & 0xFF),
                                        (IpAddr_Long & 0xFF));
	cm_init_TFTP_client();

	/* 
	 * Load 
	 */
	if(err = cm_TFTP_get_file(IpAddr_Str, FileName, FileBuf, &FileLen, NULL))
	{
		printf("whal_acxProcDownloadFwImage(TFTP): ERROR cm_TFTP_get_file\n", FileName);
		return -1;
	}
	printf("whal_acxProcDownloadFwImage(TFTP): read buf=0x%x, len=%d\n", FileBuf, FileLen);


	/* 
	 * Close and return 
	 */	
	*pBuf = FileBuf;
	*Len = FileLen;

	return(0);
#endif
}

#endif /* TIWLAN_NO_FIRMWARE_TFTP */

/****************************************************************************
 *                      whal_acxProcPrintFwVersions()
 ****************************************************************************
 * DESCRIPTION:	Prints FW versions
 * 
 * INPUTS:  None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	None
 ****************************************************************************/

int whal_acxProcPrintFwVersions(char *buf)
{
	int len = 0;

	len += sprintf(buf + len, "Autodetection is not supported anymore !\n");
	len += sprintf(buf + len, "Compiled for %s radio\n", TIWLAN_RADIO_NAME);

#if 0
	len +=sprintf(buf+len,"FwNumber  0, version=Autodetect\n");
	len +=sprintf(buf+len,"FwNumber  1, version=%s\n", _Acx100_bin_0_Version);
	len +=sprintf(buf+len,"FwNumber  2, version=%s\n", _Acx100_bin_1_Version);
	len +=sprintf(buf+len,"FwNumber  3, version=%s\n", _ACX111_REG_Version);
    len +=sprintf(buf+len,"FwNumber 10, download using FTP\n");
	len +=sprintf(buf+len,"FwNumber 11, download using TFTP\n");
	len +=sprintf(buf+len,"Selected FwNumber = %d\n", whal_ParamsGetNvParams()->FwNumber);

#endif // 0

	return len;
}


/****************************************************************************
 *                      whal_acxProcGetFwImage()
 ****************************************************************************
 * DESCRIPTION:	get firmware image pointer and length
 *		The FW file ACX100.bin converted to ASM file using BinToAsm util
 *		and have two global symbols Acx100_binArrayStart and Acx100_binArrayEnd
 *		for each image
 * 
 * INPUTS:  None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/

/*
Demiurg : I just killed FW autodetection
*/

STATUS whal_acxProcGetFwImage(unsigned char ** pBuf, int * len, unsigned char ** extraBuf, int * extraLen) {
	printf("Compiled for %s radio\n", TIWLAN_RADIO_NAME);

#if defined(TIWLAN_FW_RADIA)
	wdrv_dbSetRadioFrequency(RADIO_BAND_2_4GHZ);
#elif defined(TIWLAN_FW_RADIA_DB)
	wdrv_dbSetRadioFrequency(RADIO_BAND_2_4_AND_5GHZ);
#elif defined(TIWLAN_FW_RADIA_CRT)
	wdrv_dbSetRadioFrequency(RADIO_BAND_2_4GHZ);
#elif
	 #error "Unknwon radio type !!!\n"
#endif

	*pBuf = Acx100_bin_0_Array;
	*len = Acx100_bin_0_Array_Len;
	if (Acx100_bin_0_Array_Extra) {
		*extraBuf = Acx100_bin_0_Array_Extra;
		*extraLen = Acx100_bin_0_Array_Extra_Len;
	}
	
	return OK;
}

#if 0

STATUS whal_acxProcGetFwImage(unsigned char ** pBuf, int * len) {
	printf("Compiled for %s radio\n", TIWLAN_RADIO_NAME);

#if defined(TIWLAN_FW_RADIA)
	wdrv_dbSetRadioFrequency(RADIO_BAND_2_4GHZ);
    *pBuf = _Acx100_bin_0_ArrayStart;
    *len = (int)_Acx100_bin_0_ArrayEnd - (int)_Acx100_bin_0_ArrayStart;

#elif defined(TIWLAN_FW_RADIA_DB)
	wdrv_dbSetRadioFrequency(RADIO_BAND_2_4_AND_5GHZ);
    *pBuf = _Acx100_bin_0_ArrayStart;
    *len = (int)_Acx100_bin_0_ArrayEnd - (int)_Acx100_bin_0_ArrayStart;

#elif defined(TIWLAN_FW_RADIA_CRT)
	wdrv_dbSetRadioFrequency(RADIO_BAND_2_4GHZ);
    *pBuf = _Acx100_bin_0_ArrayStart;
    *len = (int)_Acx100_bin_0_ArrayEnd - (int)_Acx100_bin_0_ArrayStart;
#elif
	 #error "Unknwon radio type !!!\n"
#endif
	 
    return OK;
}

#endif

#if 0
int whal_acxProcGetFwImage(unsigned char **pBuf, int *Len)
{
	int Status=OK;
	int FwNumber;

	FwNumber = whal_ParamsGetNvParams()->FwNumber;
	
	if (FwNumber == 0) /* Auto detect */
	{
      UINT8 radioT;

      /* Workaround: Read EEPROM and detect the type of FW required */
     whal_acxEepromGetRadioType( &radioT);

      if ( radioT == 0x16)
      {
         /* bg radio */
         puts("Detecting Radia bg radio");

           wdrv_dbSetRadioFrequency(RADIO_BAND_2_4GHZ);
		   *pBuf = _Acx100_bin_0_ArrayStart;
		   *Len = (int)_Acx100_bin_0_ArrayEnd - (int)_Acx100_bin_0_ArrayStart;
		   acxProc_FwBinNumSet (0);
      }
      else
      if(  radioT == 0x17)
      {
         /* abg radio */
         puts("Detecting Radia abg radio");

         wdrv_dbSetRadioFrequency(RADIO_BAND_2_4_AND_5GHZ);

		   *pBuf = _Acx100_bin_1_ArrayStart;
		   *Len = (int)_Acx100_bin_1_ArrayEnd - (int)_Acx100_bin_1_ArrayStart;
		   acxProc_FwBinNumSet (1);
      }
      else
      {
         /* Assume bg radio */
         puts("**** Auto detection failed, assume bg radio ****");
		   *pBuf = _Acx100_bin_0_ArrayStart;
		   *Len = (int)_Acx100_bin_0_ArrayEnd - (int)_Acx100_bin_0_ArrayStart;
		   acxProc_FwBinNumSet (0);
      }
	}
	else  /* No auto detection, FW#x is set manually */
	if (FwNumber == 1)
	{
		*pBuf = _Acx100_bin_0_ArrayStart;
		*Len = (UINT32)_Acx100_bin_0_ArrayEnd - (UINT32)_Acx100_bin_0_ArrayStart;
		acxProc_FwBinNumSet (0);
	}
	else
	if (FwNumber == 2)
	{
		*pBuf = _Acx100_bin_1_ArrayStart;
		*Len = (UINT32)_Acx100_bin_1_ArrayEnd - (UINT32)_Acx100_bin_1_ArrayStart;
		acxProc_FwBinNumSet (1);
	}
	else
	if (FwNumber == 3)
	{
		*pBuf = _ACX111_REG_binArrayStart;
		*Len = (UINT32)_ACX111_REG_binArrayEnd - (UINT32)_ACX111_REG_binArrayStart;
		acxProc_FwBinNumSet (3);
      puts("ACX111 debug FW....");
	}
	else if (FwNumber == 10)
	{
#ifndef TIWLAN_NO_FIRMWARE_TFTP
		/* Load image from the host (via FTP) */
		Status = whal_acxProcDownloadFwImageFTP(pBuf, Len, 0);
#else
		puts("Ooops");
#endif
	}
	else
	if (FwNumber == 11)
	{
#ifndef TIWLAN_NO_FIRMWARE_TFTP
		/* Load image from the host (via TFTP) */
		Status = whal_acxProcDownloadFwImageTFTP(pBuf, Len, 0);
#else
		puts("Ooops");
#endif
	}
	else
	{
		printf("whal_acxProcGetFwImage: FATAL ERROR unknown FirmwareNumber = 0x%x !!!!!!!!\n", FwNumber);
		return NOK;
	}

	return(Status);
}

#endif // 0

/****************************************************************************
 *                      whal_acxProcPrintRadioVersions()
 ****************************************************************************
 * DESCRIPTION:	Prints Radio Module versions
 * 
 * INPUTS:  None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	None
 ****************************************************************************/

int whal_acxProcPrintRadioVersions(char *buf)
{
	int len =0;

	len +=sprintf(buf  +len,"RadioNumber  0, version=Autodetect\n");
	len +=sprintf(buf  +len,"RadioNumber  1, version=%s\n", _Acx100_radio_0_Version);
	len +=sprintf(buf  +len,"RadioNumber  2, version=%s\n", _Acx100_radio_1_Version);

    len +=sprintf(buf  +len,"RadioNumber 10, download using FTP\n");
	len +=sprintf(buf  +len,"RadioNumber 11, download using TFTP\n");
	len +=sprintf(buf  +len,"Selected RadioNumber = %d\n", whal_ParamsGetNvParams()->RadioNumber);

	return len;
}

/****************************************************************************
 *                      whal_acxProcGetRadioImage()
 ****************************************************************************
 * DESCRIPTION:	Gets the Radio Module image
 * 
 * INPUTS:  None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	None
 ****************************************************************************/
int whal_acxProcGetRadioImage( unsigned char **pBuf, int *Len)
{
	int Status=OK;
	char RadioType;
	int  RadioNumber = whal_ParamsGetNvParams()->RadioNumber;

	//	PAL_sleepLong(100);
	printf( "******* RadioNumber=%d\n", RadioNumber);
	if( RadioNumber == 0) /* Auto detect */
	{
		RadioType = whal_ParamsGetConfigOptions()->ConfigOptionsFixed.NVSVersion[1];
		if ( RadioType == 0x0d)  /* MAXIM */
		{
			printf("whal_acxProcGetRadioImage: AutoDetect MAXIM Radio\n");
			*pBuf = _Acx100_radio_0_ArrayStart;
			*Len = (UINT32)_Acx100_radio_0_ArrayEnd - (UINT32)_Acx100_radio_0_ArrayStart;
		}
		else
		if ( RadioType == 0x11)  /* RFMD */
		{
			printf("whal_acxProcGetRadioImage: AutoDetect RFMD Radio\n");
			*pBuf = _Acx100_radio_1_ArrayStart;
			*Len = (UINT32)_Acx100_radio_1_ArrayEnd - (UINT32)_Acx100_radio_1_ArrayStart;
		}
		else
		if ( RadioType == 0x20)  /* old RFMD */
		{
			printf("whal_acxProcGetRadioImage: AutoDetect RFMD Radio (old nvram structure)\n");
			*pBuf = _Acx100_radio_1_ArrayStart;
			*Len = (UINT32)_Acx100_radio_1_ArrayEnd - (UINT32)_Acx100_radio_1_ArrayStart;
		}
#if 0
		if ( RadioType == 0x40)  /* Radia bg */
		{
			printf("whal_acxProcGetRadioImage: AutoDetect Radia bg Radio\n");
			*pBuf = _Acx100_radio_2_ArrayStart;
			*Len = (UINT32)_Acx100_radio_2_ArrayEnd - (UINT32)_Acx100_radio_2_ArrayStart;
		}
		else
		if ( RadioType == 0x50)  /* Radia abg */
		{
			printf("whal_acxProcGetRadioImage: AutoDetect Radia abg Radio\n");
			*pBuf = _Acx100_radio_3_ArrayStart;
			*Len = (UINT32)_Acx100_radio_3_ArrayEnd - (UINT32)_Acx100_radio_3_ArrayStart;
		}
#endif
		else
		{
         /* Invalid / Unspcified radio number */
			printf(" !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
			printf("\n");
			printf("whal_acxProcGetRadioImage: FATAL ERROR unknown radio type = 0x%x\n", RadioType);
			printf("\n");
			printf(" !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
			printf("\n");
			printf("whal_acxProcGetRadioImage: RECOVERY: use RFMD radio as default choice\n");
			printf("\n");
			printf(" !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
			printf("\n");
			*pBuf = _Acx100_radio_1_ArrayStart;
			*Len = (UINT32)_Acx100_radio_1_ArrayEnd - (UINT32)_Acx100_radio_1_ArrayStart;
			return NOK;
		}
	}
	else  /* manual selection */
	if (RadioNumber == 1)
	{
		printf("whal_acxProcGetRadioImage: MAXIM Radio\n");
		*pBuf = _Acx100_radio_0_ArrayStart;
		*Len = (UINT32)_Acx100_radio_0_ArrayEnd - (UINT32)_Acx100_radio_0_ArrayStart;
	}
	else
	if (RadioNumber == 2)
	{
		printf("whal_acxProcGetRadioImage: RFMD Radio\n");
		*pBuf = _Acx100_radio_1_ArrayStart;
		*Len = (UINT32)_Acx100_radio_1_ArrayEnd - (UINT32)_Acx100_radio_1_ArrayStart;
	}
#if 0
	else
	if (RadioNumber == 3)
	{
		printf("whal_acxProcGetRadioImage: Radia bg Radio\n");
		*pBuf = _Acx100_radio_2_ArrayStart;
		*Len = (UINT32)_Acx100_radio_2_ArrayEnd - (UINT32)_Acx100_radio_2_ArrayStart;
   }
	else
	if (RadioNumber == 4)
	{
		printf("whal_acxProcGetRadioImage: Radia abg Radio\n");
		*pBuf = _Acx100_radio_3_ArrayStart;
		*Len = (UINT32)_Acx100_radio_3_ArrayEnd - (UINT32)_Acx100_radio_3_ArrayStart;
	}
#endif
	else
	if (RadioNumber == 10)
	{
#ifndef TIWLAN_NO_FIRMWARE_TFTP
		/* Load image from the host (via FTP) */
		printf("whal_acxProcGetRadioImage: using FTP\n");
		Status = whal_acxProcDownloadFwImageFTP(pBuf, Len, 1);
#else
		puts("Ooops");
#endif
	}
	else
	if (RadioNumber == 11)
	{
#ifndef TIWLAN_NO_FIRMWARE_TFTP
		/* Load image from the host (via TFTP) */
		printf("whal_acxProcGetRadioImage: using TFTP\n");
		Status = whal_acxProcDownloadFwImageTFTP(pBuf, Len, 1);
#else
		puts("Ooops");
#endif
	}
	else
	{
		printf("whal_acxProcGetRadioImage: FATAL ERROR unknown RadioNumber = 0x%x !!!!!!!!\n", RadioNumber);
		return NOK;
	}

PAL_sleepLong(100);
printf( "pBuf=0x%x, len=0x%x\n", (int)*pBuf, (int)*Len);
PAL_sleepLong(100);

	return(Status);
}



/****************************************************************************
 *                      acxProc_FwBinNumSet()
 *                      acxProc_FwBinNumGet()
 ****************************************************************************
 * DESCRIPTION:	Sets/Gets the Radio Module image
 * 
 * INPUTS:  None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	None
 ****************************************************************************/
#if 0
static void	acxProc_FwBinNumSet (UINT8 aVal)
{
	fwBinNumSelected = aVal;
}

static UINT8 acxProc_FwBinNumGet (void)
{
	return (fwBinNumSelected);
}
#endif

/****************************************************************************
 *                      whal_acxProcInitiateDebug()
 ****************************************************************************
 * DESCRIPTION:	Loads debug image for eCPU emulator.
 * 
 * INPUTS:  None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	None
 ****************************************************************************/
volatile int cccc=0;
int  whal_acxProcInitiateDebug(void)
{
   UINT32 ImageDataLen;
   int Stt;

whal_acxProcInitiateDebug_0001:
   puts("Initiate Debug FW...");
#if (USE_ENDIAN_SWAP == 0)
	/*for little endian system.*/
	whal_acxRegWrite(ACX_REG_SLV_END_CTL, 0x00000000);
#else
	/* use endian swap */
	whal_acxRegWrite(ACX_REG_SLV_END_CTL, 0xffffffff);
#endif
	whal_acxProcReset();

	/*
	 * Read Halt bit
	 */
	if (whal_acxRegIsBitSet( ACX_REG_ECPU_CONTROL, ECPU_CONTROL_HALT ))
	{
		printf("whal_acxProcInitiateDebug: Cpu halt -> download code\n");
		Stt = whal_acxProcLoadFwImage(ACX100_MEM_ADDRESS, 0, &ImageDataLen);
		if (Stt != 1)
		{
			printf("whal_acxProcResetRun: download failed\n");
			return(-1);
		}
	}

   printf("whal_acxProcInitiateDebug: run\n");
   whal_acxConfigDma();
   whal_acxProcRun();

   /* Debug FW does not return InitComplete, wait for it to come up */
   puts("Now, issue a CC reset, download the FW via JTAG,\n"
        "then set cccc=1 to continue, cccc=2 to load Debug FW again...");
   do
   {
      PAL_sleepLong(1000);
   } while(!cccc);
   if( cccc >= 2) { cccc=0; goto whal_acxProcInitiateDebug_0001;}
   cccc = 0;

   puts("Verifying setup is OK.");
   if ( !whal_acxProcWaitInitComplete() )
   {
		printf("whal_acxProcInitiateDebug: Timeout waiting for the ACX100 to complete Initialization\n");
		return (-1);
   }

   return 0;
}


/****************************************************************************
 *                      whal_acxConfigDma()
 ****************************************************************************
 * DESCRIPTION:	Configure ACX DMA for Indirect Memory Access Master Mode.
 * 
 * INPUTS:  None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	None
 ****************************************************************************/
int  whal_acxConfigDma(void)
{
	UINT32 HostSdramBaseAddress = EVM3_SDRAM_MEM_ADDR;  /* Default if not applied User Callback */
	
	/*
	   The Call Back is initialized with NULL value. If you intends to assign a function to the Empty Call-Back,
	   make sure that the assignment is done before this point...
	*/
   if( getHostSdram_CB != NULL )
     HostSdramBaseAddress = getHostSdram_CB();
		
   /* configure ACX DMA controls to enable HW translation of pointers */
#if( WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)
   whal_acxRegWrite(DMA_HDESC_OFFSET, ACX111_VL1_REMOTE_BASE - PHYS_ADDR(HostSdramBaseAddress)); 
   whal_acxRegWrite(DMA_HDATA_OFFSET, ACX111_VL1_REMOTE_BASE - PHYS_ADDR(HostSdramBaseAddress));
#elif( WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)
   whal_acxRegWrite(DMA_HDESC_OFFSET, - PHYS_ADDR(HostSdramBaseAddress));
   whal_acxRegWrite(DMA_HDATA_OFFSET, - PHYS_ADDR(HostSdramBaseAddress));
#endif
#if( USE_ENDIAN_SWAP == 1)
   /* Big endian CPU host */
   whal_acxRegWrite(DMA_GLB_CFG, 1);
#else
   /* Little endian CPU host */
   whal_acxRegWrite(DMA_GLB_CFG, 0);
#endif

   whal_acxRegWrite(DMA_CFG0   , 0x10000);
   whal_acxRegWrite(DMA_CFG1   , 0x10000); 
   
   return 0;
}

