/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalApi.h
 *   PURPOSE: whal module API (Init, Receive MPDU, send MSDU, ...)
 *
 ****************************************************************************/

#ifndef _WHAL_API_H
#define _WHAL_API_H

typedef void *MEM_HANDLE;

#define WEP_ACTION_ADD	KEY_ADD_OR_REPLACE
#define WEP_ACTION_DEL	KEY_REMOVE_V

/* 
 * Callbacks 
 */
/*typedef void (*whal_UsrInitFuncCB)(int retArg);*/		/* User init ack callback function */
/*typedef void (*RxData_Callback_T)(int Val);*/			/* Callback for Rx Data interrupt */
/*typedef void (*DeviceError_Callback_T)(void);*/		/* Callback for device error interrupt */
/* 
 * Initiation/Control api 
 */
extern int  whal_apiInit(whal_UsrInitFuncCB CbFunc, int retArg);
extern int  whal_apiUnload(int stage);
extern int  whal_apiConfig();
extern int  whal_apiStartBss();
extern int  whal_apiStopBss();
extern int  whal_apiAcxReset();
extern int  whal_apiRxIntrEnable(int IntrOn);
extern int  wpa_restart();
extern void whal_apiSendProbeResp(UINT8 *DstMac, dot11_SSID_T *pWlanElm_Ssid, UINT8 wone_index);

/* 
 * Rx api 
 */
extern MEM_HANDLE whal_apiRxRecv(int *Len, RxAttr_T *pRxAttr);
extern void whal_apiRxClean();
extern int  whal_apiSetRxDataCB(RxData_Callback_T RxDataCB, int Threshold);

/* 
 * Tx api 
 */
extern int  whal_apiSendMsdu(TxQueueId_T TxQid, MEM_HANDLE Msdu_Handle, TxAttr_T *pTxAttr);
extern int  whal_apiTxCheckSize(TxQueueId_T TxQid, int Len);
extern int whal_apiTxGetBufsNum(TxQueueId_T TxQid, UINT32 *FreeDescNum, UINT32 *UsedDescNum);
extern int  whal_apiSetTxStatusCB(TxStatus_Callback_T TxStatusCB);
extern void whal_apiTxClean(TxQueueId_T TxQid);
extern void whal_apiTxCleanAll();
extern int  whal_apiSetBssPs(int BssPsMode);
extern int whal_apiSetTxCompleteThreshold(int Trsh);

/* 
 * Params api 
 */
extern int whal_apiSetRadioChannel(UINT8 WoneIndex,int Channel);
extern int whal_apiSetFragmentThreshold(int FragSize);
extern int whal_apiSetRtsThreshold(int RtsSize);
extern int whal_apiSetShortRetryLimit(UINT8 Val);
extern int whal_apiSetLongRetryLimit(UINT8 Val);
extern int whal_apiSetMaxTxMsduLifetime(UINT32 Val);
extern int whal_apiSetPbcc		(UINT8 WoneIndex, int PbccVal);
extern int whal_apiSetSlotTime	(UINT8 WoneIndex, int SlotTimeVal);
extern int whal_apiSetMappingKey(int Action, char *MacAddr, int KeySize,int KeyType,int Index,int KeyId, char *Key);
extern int whal_apiSetDefaultKey(int WepAction, int KeyId,     int KeySize, char *Key);
extern int whal_apiSetDefaultKeyId(int KeyId);
extern int whal_apiSetWepExcludeUnencrypted(int Val);
extern int whal_apiSetWepOptionImplemented(int Val);
extern int whal_apiSetWepInvoke(int Val);
extern int whal_apiSetEnergyDetect(UINT32 ed);
extern int whal_apiSetPDthreshold(UINT16 pdThresh);
extern int whal_apiGetPDthreshold(UINT32 *pdThresh);
extern int whal_apiGetEnergyDetect(UINT32 *ed);
extern int whal_apiSetRxDot11mode(UINT32 mode);
extern int whal_apiGetRxDot11mode(UINT32 *mode);
extern int whal_apiGetACIcounter(UINT32 *count);
extern int whal_apiGetMediumUsage(UINT32 *usage, UINT32 *intvl);


/* 
 * Counters 
 */
extern int whal_apiGetFcsError(UINT32 *Val);
extern int whal_apiGetFailedCount		(UINT8 WoneIndex, UINT32 *Val);
extern int whal_apiGetRetryCount		(UINT8 WoneIndex, UINT32 *Val);
extern int whal_apiGetMultipleRetryCount(UINT8 WoneIndex, UINT32 *Val);
extern int whal_apiGetAckFailCount		(UINT8 WoneIndex, UINT32 *Val);
extern int whal_apiGetRtsFailCount		(UINT8 WoneIndex, UINT32 *Val);
extern int whal_apiGetRtsSuccCount		(UINT8 WoneIndex, UINT32 *Val);
extern int whal_apiGetTxFrameCount		(UINT8 WoneIndex, UINT32 *Val);
extern int whal_apiGetTxFragmentCount	(UINT8 WoneIndex, UINT32 *Val);
extern int whal_apiGetTxMulticastCount	(UINT8 WoneIndex, UINT32 *Val);
extern int whal_apiGetExcludeUnencryptedCount(UINT32 *Val);
extern int whal_apiGetUnencryptableCount(UINT32 *Val);
extern int whal_apiGetIcvErrorCount(UINT32 *Val);
extern int whal_apiGetFwVersion(char *FwVersionStr);
extern int whal_apiGetHDKVersion(UINT8 *buf);



/*
 * Hal menu
 */
extern int whal_apiPrintFwVersion(char *buf);
extern int whal_apiPrintTxInfo(int Qid, BOOL QprintEn,char *buf);
extern int whal_apiPrintTxQueue(int Qid,char *buf);
extern int whal_apiPrintTxIndicator(char *buf);
extern int whal_apiPrintTxBufs(void);
extern int whal_apiPrintRxInfo(char *buf);
extern int whal_apiPrintRxQueue(char *buf);
extern int whal_apiPrintRxBufs(char *buf);
extern int whal_apiPrintIntr(char *buf);
extern int whal_apiPrintMbox(char *buf);
extern int whal_apiPrintAcxParams(char *buf);
extern int whal_apiPrintAcxStatistics(char *buf);
extern int whal_apiPrintRestartStatistics(char *buf);
extern int whal_apiPrintRegs(char *buf);

/*
 * Misc
 */
extern int		whal_apiRxPromiscuous(int Val);
extern int		whal_apiModeSniffer(int Val);
extern void		whal_apiPhyWrite(UINT32 PhyRegAddr, UINT32 DataVal);
extern UINT32	whal_apiPhyRead(UINT32 PhyRegAddr,char *buf);
extern void		whal_apiMacWrite(UINT32 MacRegAddr, UINT32 DataVal);
extern UINT32	whal_apiMacRead(UINT32 MacRegAddr,char *buf);



/*
 * Hal serial EEPROM
 */
int whal_apiDumpEEPROM(char *buf, int buf_len);
int whal_apiReadEEPROM(int addr,char *buf);
int whal_apiWriteEEPROM(int addr, int val);

/*
 * Hal nvram
 */
int  whal_apiPrintRadioVersions(char *buf);
int  whal_apiPrintFwVersions(char *buf);
void whal_apiSelectFwNumber(int Val);
void whal_apiTraceEnable(int Val);
int  whal_apiTracePrint(int Num, int From,char *buf);
void whal_apiSetNvByte(int offset, int Val);
void whal_apiSetQueueSizeType(int Val);
int  whal_apiTracePrintRxStat( char *buf );

/*
 * ACX100 BIT
 */
int whal_apiAcxBIT_Fcc(int band, int chID, int rate, int mode);
int whal_apiAcxBIT_Telec(int band, int chID);
int whal_apiAcxBIT_perTxCfg
(
   int   band,
   int   chID,
   int   rate,
   int   mode,
   int   preamble,     /* 0=long 1=short*/
   int   numFrames,
   int   gap,          /* should be >= 200usec */
   int   bodySize,     /* [0..2304] multiples of 4 */
   char *destMac       /* destination MAC address (6 octets) */
);
int whal_apiAcxBIT_perRxCfg(int band, int chID);
int whal_apiAcxBIT_perClrStats(void);
int whal_apiAcxBIT_perTxStop(void);
int whal_apiAcxBIT_perRxStats(char *buf);
int whal_apiAcxBIT_perTxStats(char *buf);


int whal_apiAcxBIT_ExitTest(void);

/* 
 * Built in test
 */

void whal_apiRestartWatchDog_PacketReceived();
void whal_apiRestartWatchDog_PacketTransmitted();
void whal_apiIncrementDeviceErrorCounter();
void whal_apiUpdateFWWAStatistics();
void whal_apiEnableAllInterrupts();
void whal_apiDisableAllInterrupts();

/* radar detection API*/
int  whal_apiRadarDetSendStartCmd(UINT8 channel,UINT8 band);
int  whal_apiRadarDetSendStopCmd(UINT16 *event_diff,UINT8 *number_of_events);

#endif   /* _WHAL_API_H */

