/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2004 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalFrameBuilder.c
 *   PURPOSE: build 802.11 frames
 *		Beacon template
 *		Probe Response template
 *		Authentication frame #2 - For unit testing
 *		Association Response	- For unit testing
 *		Data frame header		- For unit testing
 * 
 ****************************************************************************/

#include "whalCommon.h"
#include "whalParams.h"
#include "whalFrameBuilder.h"
#include "drvDbMain.h"
#include "drvDbRegDomain.h"
#include "mgmtSm.h"

/* Performance check */
#define WFRAMB_PERF_DBG

typedef struct {
	UINT16			algoNumber;
	UINT16			transSeqNumber;
	UINT16			status;
/*	dot11_Chlg_T	*challenge; */
} internal_dot11_auth_T;

typedef struct {
	UINT16				capabilities;
	UINT16				status;
	UINT16				AID;
	dot11_supRates_T	supportedRates;
} internal_dot11_assocRsp_T;

extern PAL_spinLockHandle whalLock;

/****************************************************************************
 *                      whal_frameBuildBeacon()
 ****************************************************************************
 * DESCRIPTION:	Build beacon frame
 *
 * INPUTS:	
 *		dot11_beacon *pFrame		Beacon frame pointer
 *		int *FrameSize				The frame size to be return
 *
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_frameBuildBeacon(UINT8 WoneIndex, dot11_beacon *pFrame, int *FrameSize)
{
	UINT8 *DstMac = whal_ParamsGetBcastMac();

	dot11_SSID_T        *pWlanElm_Ssid = whal_ParamsGetElm_Ssid();
	dot11_supRates_T    *pWlanElm_SupportedRates = whal_ParamsGetElm_SupportedRates(WoneIndex);
	dot11_dsParams_T    *pWlanElm_DsParams = whal_ParamsGetElm_DsParams(WoneIndex);
	dot11_cfParams_T    *pWlanElm_Cf = whal_ParamsGetElm_Cf();
	GenParams_T         *pGenParams = whal_ParamsGetGenParams(WoneIndex);
	int                 idx, size;
    UINT16              hiddenSSID;
    UINT8               *pSSID;

	PAL_memSet((void *)pFrame, 0, sizeof(*pFrame));

	/*
	 * Mgmt header 
	 */
    MACDUP2( &pFrame->hdr.DA, DstMac);
    {
	   register UINT8* pBssId = whal_ParamsGetBssId();
	   MACDUP2( &pFrame->hdr.BSSID, pBssId);
    }
    {
	   register UINT8* pSrcMac = whal_ParamsGetSrcMac();
	   MACDUP2( &pFrame->hdr.SA, pSrcMac);
    }

	/*
	 * Other fields
	 */
	pFrame->beaconInterval = ENDIAN_HANDLE_WORD(whal_ParamsGetBeaconInterval());
	pFrame->capabilities = ENDIAN_HANDLE_WORD(pGenParams->BeaconCapabilities);
#ifndef NO_PBCC_CHECK
	if (whal_ParamsGetNvParams()->BeaconPatch == 1)
		pFrame->capabilities = ENDIAN_HANDLE_WORD((UINT16)(whal_ParamsGetNvParams()->BeaconPatch_Caps));
#endif

	size = sizeof(dot11_mgmtHeader_T) + 8 + 2 + 2;

	/*
	 * Informataion elements
	 */
	idx = 0;

	/* SSID */
	PAL_memCopy(&pFrame->infoElements[idx], pWlanElm_Ssid, 2+pWlanElm_Ssid->hdr.eleLen);
    /* Hidden SSID - set SSID buffer with 0x00 */
    wdrv_dbGetHiddenSsidStatus(&hiddenSSID);
    if (hiddenSSID)
    {
        pSSID = &pFrame->infoElements[idx]+2;
        wdrv_mgmtSetHiddenSSID(pSSID,pWlanElm_Ssid->hdr.eleLen);
    }
	idx += 2+pWlanElm_Ssid->hdr.eleLen;

	/* Supported rates */
#ifndef NO_PBCC_CHECK
	if (whal_ParamsGetNvParams()->BeaconPatch == 1)
		pWlanElm_SupportedRates = (dot11_supRates_T *)whal_ParamsGetNv_BeaconSupportedRates();
#endif
	PAL_memCopy(&pFrame->infoElements[idx], pWlanElm_SupportedRates, 2+pWlanElm_SupportedRates->hdr.eleLen);
	idx += 2+pWlanElm_SupportedRates->hdr.eleLen;

	/* DS parameters */
    pWlanElm_DsParams->currChannel = whal_ParamsGetRadioChannel(WoneIndex);
	PAL_memCopy(&pFrame->infoElements[idx], pWlanElm_DsParams, 2+pWlanElm_DsParams->hdr.eleLen);
	idx +=2+pWlanElm_DsParams->hdr.eleLen;
	
#ifndef NO_PCF_CHECK
	if (whal_ParamsGetNvParams()->PcfPatch == 1)
	{
		PAL_memCopy(&pFrame->infoElements[idx], pWlanElm_Cf, 2+pWlanElm_Cf->hdr.eleLen);
		idx +=2+pWlanElm_Cf->hdr.eleLen;
		pFrame->capabilities = ENDIAN_HANDLE_WORD(pGenParams->BeaconCapabilities | (UINT16)(whal_ParamsGetNvParams()->PcfPatch_Caps));
	}
#endif


	size += idx;

	/*
	 * return the frame size 
	 */
	*FrameSize = size;

	pFrame->hdr.fc = ENDIAN_HANDLE_WORD(DOT11_FC_BEACON);

	return OK;
}


int whal_frameBuildBeaconFromTim(UINT8 WoneIndex, char *pInfoElements, int *FrameSize)
{
	dot11_tim_T         *pWlanElm_Tim = whal_ParamsGetElm_Tim();
    dot11_CountryInfo_T *pWlanElm_CountryInfo = whal_ParamsGetElm_CountryInfo(WoneIndex);
	dot11_ERP_T         *pWlanElm_ERP = whal_ParamsGetElm_ERP(WoneIndex);
	dot11_extSupRates_T *pWlanElm_ExtSupportedRates = whal_ParamsGetElm_ExtSupportedRates(WoneIndex);
	dot11_rsne_T        *pWlanElm_Rsne = whal_ParamsGetElm_Rsne();
	int					idx=0;
    UINT8               domainCapStatus, specReqStatus;

	/*
	 * Informataion elements
	 */

	/* TIM */
	PAL_spinLock(&whalLock); /* START OF CRITICAL SECTION */
	PAL_memCopy(&pInfoElements[idx], pWlanElm_Tim, 2+pWlanElm_Tim->hdr.eleLen);
	idx += 2+pWlanElm_Tim->hdr.eleLen;
   	PAL_spinUnLock(&whalLock);  /* END OF CRITICAL SECTION */

    /* Coutry Information - only if 802.11d is enabled. */
    wdrv_dbGetdot11MultiDomainCapabilityStatus(&domainCapStatus);
    wdrv_dbGetdot11SpectrumManagementRequiredStatus(&specReqStatus);
    if (domainCapStatus) /* 802.11d enabled */
    {
        PAL_memCopy(&pInfoElements[idx], pWlanElm_CountryInfo, 2+pWlanElm_CountryInfo->hdr.eleLen);
        idx += 2+pWlanElm_CountryInfo->hdr.eleLen;
        if (specReqStatus) /* 802.11h enabled */
        {
            /* TBD.. */
        }
    }

	/* ERP */
	if (pWlanElm_ERP != NULL)
	{
		PAL_spinLock(&whalLock); /* START OF CRITICAL SECTION */
		PAL_memCopy(&pInfoElements[idx], pWlanElm_ERP, 2+pWlanElm_ERP->hdr.eleLen);
		idx += 2+pWlanElm_ERP->hdr.eleLen;
   		PAL_spinUnLock(&whalLock);  /* END OF CRITICAL SECTION */
	}

	/* Extended Supported rates (length zero is legal as it is optional IE) */
	if (pWlanElm_ExtSupportedRates->hdr.eleLen > 0)
	{
		PAL_memCopy(&pInfoElements[idx], pWlanElm_ExtSupportedRates, 2+pWlanElm_ExtSupportedRates->hdr.eleLen);
		idx += 2+pWlanElm_ExtSupportedRates->hdr.eleLen;
	}

	/* RSN */
    if ((pWlanElm_Rsne != NULL) && (pWlanElm_Rsne->hdr.eleLen))
	{
		PAL_memCopy(&pInfoElements[idx], pWlanElm_Rsne, 2+pWlanElm_Rsne->hdr.eleLen);
		idx +=2+pWlanElm_Rsne->hdr.eleLen;
	}

	/*
	 * return the frame size 
	 */
	*FrameSize = idx;

	return OK;
}


/****************************************************************************
 *                      whal_frameBuildProbeResp()
 ****************************************************************************
 * DESCRIPTION:	Build probe response frame
 *
 * INPUTS:	
 *		dot11_probeRsp *pFrame		Probe response frame pointer
 *		int *FrameSize				The frame size to be return
 *		UINT8 *DstMac				Destination MAC address 
 *										for Probe response testing
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_frameBuildProbeResp(UINT8 WoneIndex, 
                             dot11_probeRsp *pFrame, 
                             int *FrameSize, 
                             UINT8 *DstMac,
                             dot11_SSID_T *pWlanElm_Ssid)
{
	dot11_supRates_T    *pWlanElm_SupportedRates = whal_ParamsGetElm_SupportedRates(WoneIndex);
	dot11_dsParams_T    *pWlanElm_DsParams = whal_ParamsGetElm_DsParams(WoneIndex);
    dot11_CountryInfo_T *pWlanElm_CountryInfo = whal_ParamsGetElm_CountryInfo(WoneIndex);
	dot11_cfParams_T    *pWlanElm_Cf = whal_ParamsGetElm_Cf();
	dot11_ERP_T         *pWlanElm_ERP = whal_ParamsGetElm_ERP(WoneIndex);
	dot11_extSupRates_T *pWlanElm_ExtSupportedRates = whal_ParamsGetElm_ExtSupportedRates(WoneIndex);
	dot11_rsne_T        *pWlanElm_Rsne = whal_ParamsGetElm_Rsne();
	dot11_TIcap_T       *pWlanElm_4xSupport = whal_ParamsGetElm_TIcapSupport();
	GenParams_T         *pGenParams = whal_ParamsGetGenParams(WoneIndex);
	int idx, size;
    UINT8               domainCapStatus, specReqStatus;

	PAL_memSet((void *)pFrame, 0, sizeof(*pFrame));

	/*
	 * Mgmt header 
	 */
    MACDUP2( &pFrame->hdr.DA, DstMac);
    {
       register UINT8* pBssId = whal_ParamsGetBssId();
       MACDUP2( &pFrame->hdr.BSSID, pBssId);
    }
    {
       register UINT8* pSrcMac = whal_ParamsGetSrcMac();
       MACDUP2( &pFrame->hdr.SA, pSrcMac);
    }

	/*
	 * Other fields
	 */
	pFrame->beaconInterval = ENDIAN_HANDLE_WORD(whal_ParamsGetBeaconInterval());
	pFrame->capabilities = ENDIAN_HANDLE_WORD(pGenParams->ProbResCapabilities);
#ifndef NO_PBCC_CHECK
	if (whal_ParamsGetNvParams()->ProbePatch == 1)
		pFrame->capabilities = ENDIAN_HANDLE_WORD((UINT16)(whal_ParamsGetNvParams()->ProbePatch_Caps));
#endif

	size = sizeof(dot11_mgmtHeader_T) + 8 + 2 + 2;

	/*
	 * Informataion elements
	 */
	idx = 0;

	/* SSID */
	PAL_memCopy(&pFrame->infoElements[idx], pWlanElm_Ssid, 2+pWlanElm_Ssid->hdr.eleLen);
	idx += 2+pWlanElm_Ssid->hdr.eleLen;

	/* Supported rates */
#ifndef NO_PBCC_CHECK
	if (whal_ParamsGetNvParams()->ProbePatch == 1)
		pWlanElm_SupportedRates = (dot11_supRates_T *)whal_ParamsGetNv_ProbeRespSupportedRates();
#endif
	PAL_memCopy(&pFrame->infoElements[idx], pWlanElm_SupportedRates, 2+pWlanElm_SupportedRates->hdr.eleLen);
	idx += 2+pWlanElm_SupportedRates->hdr.eleLen;

	/* DS parameters */
    pWlanElm_DsParams->currChannel = whal_ParamsGetRadioChannel(WoneIndex);
	PAL_memCopy(&pFrame->infoElements[idx], pWlanElm_DsParams, 2+pWlanElm_DsParams->hdr.eleLen);
	idx +=2+pWlanElm_DsParams->hdr.eleLen;

#ifndef NO_PCF_CHECK
	if (whal_ParamsGetNvParams()->PcfPatch == 1)
	{
		PAL_memCopy(&pFrame->infoElements[idx], pWlanElm_Cf, 2+pWlanElm_Cf->hdr.eleLen);
		idx +=2+pWlanElm_Cf->hdr.eleLen;
		pFrame->capabilities = ENDIAN_HANDLE_WORD(pGenParams->ProbResCapabilities | (UINT16)(whal_ParamsGetNvParams()->PcfPatch_Caps));
	}
#endif

    /* Coutry Information - only if 802.11d is enabled. */
    wdrv_dbGetdot11MultiDomainCapabilityStatus(&domainCapStatus);
    wdrv_dbGetdot11SpectrumManagementRequiredStatus(&specReqStatus);
    if (domainCapStatus) /* 802.11d enabled */
    {
        PAL_memCopy(&pFrame->infoElements[idx], pWlanElm_CountryInfo, 2+pWlanElm_CountryInfo->hdr.eleLen);
        idx += (2 + pWlanElm_CountryInfo->hdr.eleLen);
        if (specReqStatus) /* 802.11h enabled */
        {
            /* TBD.. */
        }
    }

	/* ERP */
	if (pWlanElm_ERP != NULL)
	{
		PAL_memCopy(&pFrame->infoElements[idx], pWlanElm_ERP, 2+pWlanElm_ERP->hdr.eleLen);
		idx += 2+pWlanElm_ERP->hdr.eleLen;
	}

	/* Extended Supported rates (length zero is legal as it is optional IE) */
	if (pWlanElm_ExtSupportedRates->hdr.eleLen > 0)
	{
		PAL_memCopy(&pFrame->infoElements[idx], pWlanElm_ExtSupportedRates, 2+pWlanElm_ExtSupportedRates->hdr.eleLen);
		idx += 2+pWlanElm_ExtSupportedRates->hdr.eleLen;
	}

	/* RSN */
    if ((pWlanElm_Rsne != NULL) && (pWlanElm_Rsne->hdr.eleLen))
	{
		PAL_memCopy(&pFrame->infoElements[idx], pWlanElm_Rsne, 2+pWlanElm_Rsne->hdr.eleLen);
		idx +=2+pWlanElm_Rsne->hdr.eleLen;
	}

	/* 4X TEMPLATE */
	if (whal_ParamsGetGenParams(0)->SupportTIcapTemplate == 1)
	{
		PAL_memCopy(&pFrame->infoElements[idx], pWlanElm_4xSupport, 2+pWlanElm_4xSupport->hdr.eleLen);
		idx += 2+pWlanElm_4xSupport->hdr.eleLen;
	}

	size += idx;

	/*
	 * return the frame size 
	 */
	*FrameSize = size;

	pFrame->hdr.fc = ENDIAN_HANDLE_WORD(DOT11_FC_PROBE_RESP);

	return OK;
}

/****************************************************************************
 *                      whal_frameBuildAssResp()
 ****************************************************************************
 * DESCRIPTION:	Build Association response
 *	
 * INPUTS:	None
 *
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_frameBuildAssResp(UINT8 WoneIndex, dot11_mgmtFrame_T *pFrame, UINT32 *FrameSize, UINT8 *DstMac)
{
	internal_dot11_assocRsp_T *pBody = (internal_dot11_assocRsp_T *)pFrame->body;
	dot11_supRates_T *pWlanElm_SupportedRates = whal_ParamsGetElm_SupportedRates(WoneIndex);
	GenParams_T *pGenParams = whal_ParamsGetGenParams(WoneIndex);
	int i;
	/*
	 * Mgmt header 
	 */
	pFrame->hdr.fc = DOT11_FC_ASSOC_RESP;

#ifdef WFRAMB_PERF_DBG
   MACDUP2( &pFrame->hdr.DA, DstMac);
   MACDUP2( &pFrame->hdr.BSSID, whal_ParamsGetBssId());
   MACDUP2( &pFrame->hdr.SA, whal_ParamsGetSrcMac());
#else
	PAL_memCopy(&pFrame->hdr.DA, DstMac, 6);
	PAL_memCopy(&pFrame->hdr.BSSID, whal_ParamsGetBssId(), 6);
	PAL_memCopy(&pFrame->hdr.SA, whal_ParamsGetSrcMac(), 6);  
#endif

	pFrame->hdr.duration = 0;
	pFrame->hdr.seqCtrl = 0;

	/*
	 * Frame body
	 */
	pBody->AID = ENDIAN_HANDLE_WORD(1); 
	pBody->status = 0; /* OK */
	pBody->capabilities = ENDIAN_HANDLE_WORD(pGenParams->BeaconCapabilities);
	pBody->supportedRates.hdr.eleId = pWlanElm_SupportedRates->hdr.eleId;
	pBody->supportedRates.hdr.eleLen = pWlanElm_SupportedRates->hdr.eleLen;
	for (i=0; i<pWlanElm_SupportedRates->hdr.eleLen; i++)
		pBody->supportedRates.rates[i] = pWlanElm_SupportedRates->rates[i]; 

	*FrameSize = sizeof(dot11_assocRsp_T) + sizeof(dot11_mgmtHeader_T) 
		- DOT11_SUPPORTED_RATES_MAX + pWlanElm_SupportedRates->hdr.eleLen;
	return 0;
}

/****************************************************************************
 *                      whal_frameBuildAuth()
 ****************************************************************************
 * DESCRIPTION:	Build Authentication frame #2 - for debug
 *	
 * INPUTS:	None
 *
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_frameBuildAuth(dot11_mgmtFrame_T *pFrame, UINT32 *FrameSize, UINT8 *DstMac)
{
	internal_dot11_auth_T *pBody = (internal_dot11_auth_T *)pFrame->body;
	/*
	 * Mgmt header 
	 */
	pFrame->hdr.fc = DOT11_FC_AUTH;

#ifdef WFRAMB_PERF_DBG
   MACDUP2( &pFrame->hdr.DA, DstMac);
   MACDUP2( &pFrame->hdr.BSSID, whal_ParamsGetBssId());
   MACDUP2( &pFrame->hdr.SA, whal_ParamsGetSrcMac());
#else
	PAL_memCopy(&pFrame->hdr.DA, DstMac, 6);
	PAL_memCopy(&pFrame->hdr.BSSID, whal_ParamsGetBssId(), 6);
	PAL_memCopy(&pFrame->hdr.SA, whal_ParamsGetSrcMac(), 6);  
#endif

	pFrame->hdr.duration = 0;
	pFrame->hdr.seqCtrl = 0;

	/*
	 * Frame body
	 */
	pBody->algoNumber = 0; /* open system */
	pBody->transSeqNumber = ENDIAN_HANDLE_WORD(2);
	pBody->status = 0; /* OK */

	*FrameSize = sizeof(internal_dot11_auth_T) + sizeof(dot11_mgmtHeader_T);
	return 0;
}

/****************************************************************************
 *                      whal_frameBuildDataHeader()
 ****************************************************************************
 * DESCRIPTION:	Build data frame header - for debug
 *	
 * INPUTS:	None
 *
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
typedef struct tLLC_HEADER {

  unsigned char dstSAP;
  unsigned char srcSAP;
  unsigned char Control[2];

} LLC_HEADER, *PLLC_HEADER;

typedef struct tDATA_FRAME_HEADER {

  dot11_header_T     sdot11Header;
  LLC_HEADER       sLLCHeader;
/*  SNAP_HEADER      sSNAPHeader; */

} DATA_FRAME_HEADER, *PDATA_FRAME_HEADER;


int whal_frameBuildDataHeader(UINT8 *pHdrBlock, int *HdrBlockSize, UINT8 *DstMac)
{
	DATA_FRAME_HEADER *pFrame = (DATA_FRAME_HEADER *)pHdrBlock;
	/*
	 * Data header 
	 */
	pFrame->sdot11Header.fc = DOT11_FC_DATA|DOT11_FC_FROM_DS;

	/*
	 * Set addresses, ToDs/FromDs = 0,1  --> Dst, Bss, Src
	 */
    MACDUP2( &pFrame->sdot11Header.address1, DstMac);				   /* DstId address */ 
    {
       register UINT8* pSrcMac = whal_ParamsGetSrcMac();
       MACDUP2( &pFrame->sdot11Header.address2, pSrcMac); /* BSSID address */
       MACDUP2( &pFrame->sdot11Header.address3, pSrcMac); /* SrcId address */
    }
	pFrame->sdot11Header.duration = 0;
	pFrame->sdot11Header.seqCtrl = 0;

	/*
	 * LLC header
	 */
	pFrame->sLLCHeader.dstSAP = 0x5f;
	pFrame->sLLCHeader.srcSAP = 0x4e;
	pFrame->sLLCHeader.Control[0] = 0x84;
	pFrame->sLLCHeader.Control[1] = 0xa8;

	*HdrBlockSize = 0x1c;

	return 0;
}

/****************************************************************************
 *                      whal_frameBuildHdrTxFromRx()
 ****************************************************************************
 * DESCRIPTION:	Build Tx frame header from Rx header - for debug
 *	
 * INPUTS:	None
 *
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_frameBuildHdrTxFromRx(UINT8 *pHdrBlock)
{
	dot11_header_T *pFrame = (dot11_header_T *)pHdrBlock;
	UINT8 RxDstMac[6];
	UINT8 RxSrcMac[6];
	UINT8 RxBssId[6];
	/*
	 * Data header 
	 */
	pFrame->fc = DOT11_FC_DATA|DOT11_FC_FROM_DS;
	/*
	 * Set addresses, ToDs/FromDs = 0,1  --> Dst, Bss, Src
	 */

	/* copy the src address from the rx frame */
    MACDUP2( RxBssId, &pFrame->address1 );
    MACDUP2( RxSrcMac, &pFrame->address2);
    MACDUP2( RxDstMac, &pFrame->address3);

    MACDUP2( &pFrame->address1, RxDstMac);  /* DstId address */
    MACDUP2( &pFrame->address2, RxBssId );  /* BSSID address */
    MACDUP2( &pFrame->address3, RxSrcMac);  /* SrcId address */
	
    pFrame->duration = 0;
    pFrame->seqCtrl = 0;

	return 0;
}
