/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2001 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalInterrupt.h
 *   PURPOSE: Handle all interrupt issues (ISR, trigger)
 * 
 ****************************************************************************/

#ifndef _WHAL_INTERRUPT_H
#define _WHAL_INTERRUPT_H

#define INTR_RX_DATA		0x00000001
#define INTR_TX_DATA		0x00000002
#define INTR_TX_TRF         0x00000004
#define INTR_RX_TRF         0x00000008
#define INTR_INFO			0x00000400
#define INTR_INIT_COMPLETE	0x00004000
#define INTR_CMD_COMPLETE	0x00000200
#define INTR_DEVICE_ERROR	0x00001000

/*
 * Receive interrupts
 */
extern void whal_acxIntrInit(void);
extern void whal_acxIntrHandler(void);
extern void whal_acxIntrEnable(UINT32 Intr);
extern void whal_acxIntrDisable(UINT32 Intr);
extern void whal_acxRxIntrDisable(void);
extern void whal_acxRxIntrEnable(void);
extern int	whal_acxIntrCheck(UINT32 Intr);
extern int	whal_acxIntrCheckAndAck(UINT32 Intr);
extern int  whal_acxIntrSetRxDataCB(void *external_RxData_Callback, int Threshold);
extern int  whal_acxIntrSetTxDataCB(void *external_TxData_Callback);
extern int	whal_acxIntrSetDeviceErrorCB(void *external_DeviceError_Callback);

/*
 * Trigger(Send) interrupts
 */
extern void whal_acxIntrTrigger(UINT32 Intr);
extern void whal_acxIntrTriggerCmd(void);
extern void whal_acxIntrTriggerRxProc(void);
extern void whal_acxIntrTriggerTxProc(void);

void whal_acxMboxIntrMask(int Intr, BOOL en);

extern int whal_acxIntrPrint(char *buf);
#endif


