/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalRxHostDesc.c
 *   PURPOSE: ACX100 Rx Host Descriptor Manager
 *
 ****************************************************************************/

#include "pal.h"  
#include "whalHostDesc.h"
#include "acx_hw.h"

/* Descriptors and pointer conversion macros */
#if 0 
#define DESC_ENDIAN_WORD(x) ENDIAN_HANDLE_WORD(x)
#define DESC_ENDIAN_LONG(x) ENDIAN_HANDLE_LONG(x)
adsfasdf??+;
#else
#define DESC_ENDIAN_WORD(x) (x)
#define DESC_ENDIAN_LONG(x) (x)
#endif

#if( WLAN_BUS_TYPE == WLAN_BUS_TYPE_VLYNQ)
#define HOST2ACX_PTR(x)       (DESC_ENDIAN_LONG(VLYNQ_HOST2ACX111_SDRAM_PTR((UINT32)(x))))
#define ACX2HOST_PTR(x)       (VLYNQ_ACX111_TO_HOST_SDRAM_PTR_CACHED( DESC_ENDIAN_LONG((UINT32)(x))))
#elif( WLAN_BUS_TYPE == WLAN_BUS_TYPE_PCI)
#define HOST2ACX_PTR(x)       (DESC_ENDIAN_LONG(PCI_HOST2ACX111_SDRAM_PTR((UINT32)(x))))
#define ACX2HOST_PTR(x)       (PCI_ACX111_TO_HOST_SDRAM_PTR_CACHED( DESC_ENDIAN_LONG((UINT32)(x))))
#else
#error   whalHostDesc.c -- Check bus type
#endif


 /****************************************************************************
 *                      whal_HostDesc_setNextDesc()
 ****************************************************************************
 * DESCRIPTION:
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
void whal_HostDesc_setNextDesc( HostDesc_T* hostDesc, HostDesc_T *nextHostDesc )
{
   hostDesc->next_desc_ptr = HOST2ACX_PTR( nextHostDesc);
}

 /****************************************************************************
 *                      whal_HostDesc_getNext()
 ****************************************************************************
 * DESCRIPTION:
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
HostDesc_T* whal_HostDesc_getNext(  HostDesc_T* this )
{
    /* We don't have to invalidate this pointer since it's never modified from the HW. */ 
	return (HostDesc_T*) ACX2HOST_PTR( this->next_desc_ptr);
}

 /****************************************************************************
 *                      whal_HostDesc_setBuffAddr()
 ****************************************************************************
 * DESCRIPTION:
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
void whal_HostDesc_setBuffAddr(  HostDesc_T* this, UINT8* dataBuff , UINT32 bufToken, UINT16 len)
{
   this->blk_ptr = HOST2ACX_PTR( dataBuff);
	this->blk_offset = 0;
	this->blk_len = DESC_ENDIAN_WORD( len);
	this->bufToken = bufToken;
}

 /****************************************************************************
 *                      whal_HostDesc_getDataBuff()
 ****************************************************************************
 * DESCRIPTION:
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
void whal_HostDesc_getDataBuff(  HostDesc_T* this, UINT8** dataBuff, UINT16* len)
{
    PAL_cacheInvalidate(0,
       this,                    /* virtual address */
       sizeof(HostDesc_T));    /* number of bytes to invalidate */
   
   *dataBuff =  (UINT8*)ACX2HOST_PTR( this->blk_ptr);
	*len =       DESC_ENDIAN_WORD( this->blk_len);
}

 /****************************************************************************
 *                      whal_HostDesc_get()
 ****************************************************************************
 * DESCRIPTION:
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
void whal_HostDesc_getAttr( HostDesc_T* this, HostDescAttr_T* attr)
{
	/* Host descriptors are allocated and accessed through uncached pointers
	   since the data can be changed by the ACX without notice */
    PAL_cacheInvalidate(0,
       this,                    /* virtual address */
       sizeof(HostDesc_T));    /* number of bytes to invalidate */

     attr->blk_ptr    = ACX2HOST_PTR( this->blk_ptr);
     attr->blk_offset = DESC_ENDIAN_WORD( this->blk_offset);
     attr->blk_len    = DESC_ENDIAN_WORD( this->blk_len);
     attr->ctrl       = DESC_ENDIAN_WORD( this->ctrl);
     attr->next       = ACX2HOST_PTR( this->next_desc_ptr);
     attr->status     = ENDIAN_HANDLE_LONG( this->status); /* Status is written by ACX rather than DMA */
     attr->msduPtr    = this->msduPtr;
     attr->bufToken   = this->bufToken;
}


 /****************************************************************************
 *                      whal_HostDesc_clearToken()
 ****************************************************************************
 * DESCRIPTION:
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
void whal_HostDesc_clearToken( HostDesc_T* this)
{
	this->bufToken = 0;
}


/****************************************************************************
 *                      whal_HostDesc_get()
 ****************************************************************************
 * DESCRIPTION:
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
void whal_HostDesc_setAttr(  HostDesc_T* this, HostDescAttr_T* attr)
{
   this->msduPtr    = (UINT32) attr->msduPtr; /* Cookie */
   this->blk_ptr    = HOST2ACX_PTR( attr->blk_ptr);
   this->blk_offset = DESC_ENDIAN_WORD( attr->blk_offset);
   this->blk_len    = DESC_ENDIAN_WORD( attr->blk_len );
   this->ctrl       = DESC_ENDIAN_WORD( attr->ctrl );
   this->status     = ENDIAN_HANDLE_LONG( attr->status );
   this->bufToken   = attr->bufToken;

   /*sysFlushDataCache( this, sizeof(HostDesc_T) );*/
   PAL_cacheFlush( 0, this, sizeof(HostDesc_T) );
}


 /****************************************************************************
 *                      whal_HostDesc_setCtrl()
 ****************************************************************************
 * DESCRIPTION:
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
void whal_HostDesc_setCtrl(  HostDesc_T* this, UINT16 ctrl)
{
	this->ctrl = ENDIAN_HANDLE_WORD( ctrl);
}


 /****************************************************************************
 *                      whal_HostDesc_setStatus()
 ****************************************************************************
 * DESCRIPTION:
 *
 * INPUTS:	None	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK. 
 ****************************************************************************/
void whal_HostDesc_setStatus(  HostDesc_T* this, UINT32 status)
{
	this->status = ENDIAN_HANDLE_LONG( status);
}

