/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalRxHostQueue.h
 *   PURPOSE: ACX100 Rx Queue manager
 *
 ****************************************************************************/ 

#ifndef WHAL_HOST_QUEUE_H
#define WHAL_HOST_QUEUE_H 
 
#include "whalHostDesc.h"

typedef struct  {
    UINT32  targetAborts;
    UINT32  masterAborts;
    UINT32  reqBeforeComplete;
    UINT32  unexpectHostError;
    UINT32  unexpectAcxError;
    UINT32  pciNotAbled;                        
}  HostQStat_Obj;
 


typedef struct 
{
	UINT32		 NumOfDesc;
	UINT32		 BlockSize;
	HostDesc_T  *FirstDesc;
	HostDesc_T  *NextToRead;
	HostDesc_T  *NextToInsert;
	UINT32       CurrNumDescInQ; /* NumOfDescInQ is the current number of descriptors with
                                 valid buffers associated with them  */
	UINT32       MaxCurrNumDescInQ;  /* Use for statistics and debugging. */ 
	UINT32       MinCurrNumDescInQ;  /* Use for statistics and debugging. */ 
    HostQStat_Obj statistics;
	UINT8       *BlocksPool;
} HostQueue_Obj;
   
 
HostQueue_Obj* whal_HostQueue_init( UINT32 NumBlocks, UINT32  BlockSize);	 
STATUS whal_HostQueue_getNext(HostQueue_Obj* this, HostDescAttr_T* pDscAttr);
STATUS whal_HostQueue_insert(HostQueue_Obj* this, HostDescAttr_T* pDscAttr);
int		whal_HostQueue_show(HostQueue_Obj* this,char *buf);
STATUS  whal_HostQueue_freeMsdu(HostQueue_Obj* this);
STATUS  whal_HostQueue_CleanAll(HostQueue_Obj* this);
void whal_HostQueue_unload( HostQueue_Obj *pHostQueue, UINT32  BlockSize);

static __inline__ UINT32  whal_HostQueue_getFirstDescAddr(HostQueue_Obj* this)
{
	return (UINT32) this->FirstDesc;
}

static __inline__ UINT32  whal_HostQueue_getNextToInsertDescAddr(HostQueue_Obj* this)
{
	return (UINT32) this->NextToInsert;
}
static __inline__ STATUS  whal_HostDesc_getFreeBufsNum(HostQueue_Obj* this, UINT32 *numOfFreeBds)
{
    *numOfFreeBds =  (this->NumOfDesc - this->CurrNumDescInQ );
    return OK;
}


#endif /* WHAL_RX_HOST_QUEUE_H */
