/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalTrace.c
 *   PURPOSE: Trace on Rx/Tx packets
 *   
 ****************************************************************************/

#include "whalCommon.h"
#include "whal.h"
#include "whalRx.h"
#include "whalRxQueue.h"
#include "whalRxBufs.h"
#include "whalRxDesc.h"
#include "whalTx.h"
#include "whalTxQueue.h"
#include "whalTxBufs.h"
#include "whalTxDesc.h"
#include "memMngr.h"
#include "whalInterrupt.h"
#include "whalAcxAccess.h"
#include "whalParams.h"
#include "whalTrace.h"
#include "whalAcxRegs.h"

/****************************************************************************
 *                      OBJECT DATA BASE
 ****************************************************************************
 *
 *	TrcDb - Rx Trace buffer of 5000 packets
 *
 ****************************************************************************/
#define TRACER_MAX_EVENTS		5000
#define TRACER_MAX_ACT_LEN		5
#define TRACER_MAX_OBJ_LEN		3
#define TRACER_HDR_LEN			24
#define TRACER_MAX_STR_LEN		51

typedef struct
{
	UINT32			time;
	UINT16			MpduLen;
	UINT8			Ctl;
	UINT8			Rate;
	UINT8			Status[4];
	RxPLCPHeader_T  PlcpHdr;
    UINT16			Len4Plcp;
	UINT32			PlcpDiffLen;

	dot11_header_T	FrameHeader;
	int				FrameData;
} TRACER_DATA;

typedef struct
{
	char			String[TRACER_MAX_STR_LEN];
	int				IntValue;
} TRACER_CTRL;

typedef struct
{
	UINT32			TimStamp;
	char			Action[TRACER_MAX_ACT_LEN];
	char			Object[TRACER_MAX_OBJ_LEN];
#if 0
	UINT16			MpduLen;
	UINT8			Ctl;
	UINT8			Rate;
	UINT8			Status[4];
	dot11_header_T	FrameHeader;
	int				FrameData;
#else
	union
	{
		TRACER_DATA		TrcData;
		TRACER_CTRL		TrcCtrl;
	}Info;
#endif
} TRACER_EVENT;

typedef struct
{
	int Enable;
	int Idx;
	int Num;
	TRACER_EVENT Evt[TRACER_MAX_EVENTS];
} TRACER_DB;

TRACER_DB TrcDb;

PAL_spinLockHandle	TrcLock;
#define PROTECT_ON()  { PAL_spinLock(&TrcLock);	}
#define PROTECT_OFF() { PAL_spinUnLock(&TrcLock);	}

#if 0
FILE *TraceFd=0;
#endif
int whal_trcPrintLine(int EventIndex);
int whal_trcSendLine(int EventIndex);
int whal_trcOpenFile(void);


#ifndef NO_PLCP_PATCH
int PlcpDiffCounter	= 0;
UINT32 PlcpMaxDiffLen = 0;
UINT32 PlcpMinDiffLen = 10000;
#endif


/* Rx Rates Counters */
UINT32   whalRxCounterRate1  = 0;
UINT32   whalRxCounterRate2  = 0;
UINT32   whalRxCounterRate5  = 0;
UINT32   whalRxCounterRate6  = 0;
UINT32   whalRxCounterRate9  = 0;
UINT32   whalRxCounterRate11 = 0;
UINT32   whalRxCounterRate12 = 0;
UINT32   whalRxCounterRate18 = 0;
UINT32   whalRxCounterRate22 = 0;
UINT32   whalRxCounterRate24 = 0;
UINT32   whalRxCounterRate36 = 0;
UINT32   whalRxCounterRate48 = 0;
UINT32   whalRxCounterRate54 = 0;


/****************************************************************************
 *                      whal_trcAddRx()
 ****************************************************************************
 * DESCRIPTION:	Add Rx info line to the DB
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_trcAddRx(WlanRxStatusInfo_T *rxStatusInfo, char *Action)
{
#if 0
	TRACER_EVENT *pEvt;
	UINT32 FrameAddress;
	TRACER_DATA	*pData;
	int LenExtBit;
	int RateVal;
	int PbccVal;
	int CalcLen;

	PROTECT_ON();

	pEvt = &TrcDb.Evt[TrcDb.Idx];
	pData = &pEvt->Info.TrcData;
	
	/*
	 * Common
	 */
	strncpy(pEvt->Action, Action, TRACER_MAX_ACT_LEN-1);
	strncpy(pEvt->Object, "Rx", TRACER_MAX_OBJ_LEN-1);

#if 0
	pEvt->TimStamp = whal_acxRegRead(ACX_MAC_TIMESTAMP_REG /* Mac TimeStamp counter */);
#else
	pEvt->TimStamp	= ENDIAN_HANDLE_LONG( rxStatusInfo->time_stamp );
#endif

	/* 
	 * Data
	 */
    /* our data length is only from mac header without plcp header */
	pData->MpduLen	 =  (ENDIAN_HANDLE_WORD( rxStatusInfo->length ) & 0x0fff) - sizeof(RxPLCPHeader_T);
    /*	pData->time		 = local_RxDesc.time; */
	pData->Ctl		 = 0;
	pData->Rate		 = rxStatusInfo->plcp_signal;
	pData->Status[0] = rxStatusInfo->rxm_status;
	pData->Status[1] = rxStatusInfo->rx_snr;
	pData->Status[2] = rxStatusInfo->rx_level;
	pData->Status[3] = rxStatusInfo->mod_mode;



    switch (pData->Rate) {
    case 0x01 : pData->Rate = 0x01; whalRxCounterRate1++ ; break;
    case 0x14 : pData->Rate = 0x02; whalRxCounterRate2++ ; break;
    case 0x37 : pData->Rate = 0x05; whalRxCounterRate5++ ; break;
    case 0x0B : pData->Rate = 0x06; whalRxCounterRate6++ ; break;
    case 0x0F : pData->Rate = 0x09; whalRxCounterRate9++ ; break;
    case 0x6E : pData->Rate = 0x11; whalRxCounterRate11++; break;
    case 0x0A : pData->Rate = 0x12; whalRxCounterRate12++; break;
    case 0x0E : pData->Rate = 0x18; whalRxCounterRate18++; break;
    case 0xDC : pData->Rate = 0x22; whalRxCounterRate22++; break;
    case 0x09 : pData->Rate = 0x24; whalRxCounterRate24++; break;
    case 0x0D : pData->Rate = 0x36; whalRxCounterRate36++; break;
    case 0x08 : pData->Rate = 0x48; whalRxCounterRate48++; break;
    case 0x0C : pData->Rate = 0x54; whalRxCounterRate54++; break;
    }

	FrameAddress = (UINT32)rxStatusInfo + sizeof(WlanRxStatusInfo_T) + sizeof(RxPLCPHeader_T);
	PAL_memCopy((char *)&pData->FrameHeader, (char *)FrameAddress, TRACER_HDR_LEN);
	pData->FrameHeader.fc = ENDIAN_HANDLE_WORD(pData->FrameHeader.fc);
	pData->FrameHeader.seqCtrl = ENDIAN_HANDLE_WORD(pData->FrameHeader.seqCtrl);
	pData->FrameHeader.duration = ENDIAN_HANDLE_WORD(pData->FrameHeader.duration);
	PAL_memCopy((char *)&pData->FrameData, (char *)FrameAddress+TRACER_HDR_LEN, 4);

  /* #ifndef NO_PLCP_PATCH */
#if 0

	if (whal_ParamsGetNvParams()->PlcpPatch == 1)
	{
		whal_acxMemCopy((char *)&pData->Len4Plcp, (char *)FrameAddress-0x10 /* Len */, 2);
		pData->Len4Plcp = ENDIAN_HANDLE_WORD(pData->Len4Plcp);
		whal_acxMemCopy((char *)&pData->PlcpHdr,  (char *)FrameAddress-0x04 /* PLCP HEADER */, 4);
		pData->PlcpHdr.Length = ENDIAN_HANDLE_WORD(pData->PlcpHdr.Length);

		if ((pData->PlcpHdr.Service & 0x80) == 0x80)
			LenExtBit = 1;
		else
			LenExtBit = 0;

		switch (pData->PlcpHdr.Signal)
		{
			case 0x0A: RateVal = 2;		PbccVal = 0; break;
			case 0x14: RateVal = 4;		PbccVal = 0; break;
			case 0x37: RateVal = 11;	PbccVal = 0; break;
			case 0xb7: RateVal = 11;	PbccVal = 1; break;
			case 0x6e: RateVal = 22;	PbccVal = 0; break;
			case 0xee: RateVal = 22;	PbccVal = 1; break;
			case 0xdc: RateVal = 44;	PbccVal = 1; break;
		}

		CalcLen = (( ((int)(pData->PlcpHdr.Length) * RateVal) >> 4) - PbccVal) - LenExtBit;

		pData->PlcpDiffLen = CalcLen - pData->Len4Plcp;

		if (pData->PlcpDiffLen != 0)
		{
			PlcpDiffCounter++;
			if (pData->PlcpDiffLen >= PlcpMaxDiffLen)
				PlcpMaxDiffLen = pData->PlcpDiffLen;
			if (pData->PlcpDiffLen <= PlcpMinDiffLen)
				PlcpMinDiffLen = pData->PlcpDiffLen;
		}
	}
	else
	{
		pData->PlcpDiffLen = 0;	
	}
#endif


	if (whal_ParamsGetNvParams()->TraceFile == 1)
	{
		whal_trcSendLine(TrcDb.Idx);
	}
	if (whal_ParamsGetNvParams()->TraceOut == 1)
	{
		whal_trcPrintLine(TrcDb.Idx);
	}
	/* 
	 * Prepare next index
	 */
	if (++TrcDb.Idx >= TRACER_MAX_EVENTS)
		TrcDb.Idx = 0;

	if (TrcDb.Num < TRACER_MAX_EVENTS)
		TrcDb.Num++;

	PROTECT_OFF();
#endif
	return(0);
}



/****************************************************************************
 *                      whal_trcAddTx()
 ****************************************************************************
 * DESCRIPTION:	Add Tx info line to the DB
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_trcAddTx(TxDesc_T *pTxDesc,UINT32 FrameAddress, char *Action)
{
#if 0
	TRACER_EVENT *pEvt;
    TxDesc_T local_TxDesc;
	TRACER_DATA	*pData;

	PROTECT_ON();

	pEvt = &TrcDb.Evt[TrcDb.Idx];
	pData = &pEvt->Info.TrcData;
	
	/*
	 * Common
	 */
	strncpy(pEvt->Action, Action, TRACER_MAX_ACT_LEN-1);
	strncpy(pEvt->Object, "Tx", TRACER_MAX_OBJ_LEN-1);
#if 1
	pEvt->TimStamp = whal_acxRegRead(ACX_MAC_TIMESTAMP_REG /* Mac TimeStamp counter */);
#else
	pEvt->TimStamp	= local_RxDesc.time;
#endif

	/* 
	 * Data
	 */
	whal_txDesc_Read(&local_TxDesc, pTxDesc);

	pData->MpduLen	= local_TxDesc.MPDULen;
	pData->time		 = 0;
	pData->Ctl		= local_TxDesc.Ctl;
	pData->Rate		= RateConvertMask_Mask16_to_Val8(local_TxDesc.Rate);
	pData->Status[0] = local_TxDesc.Ctl2;
	pData->Status[1] = local_TxDesc.AckFailures;
	pData->Status[2] = local_TxDesc.RTSSuccess;
	pData->Status[3] = local_TxDesc.RTSFailures;

    /*logMsg("whal_trcAddTx: FrameAddress: 0x%x\n",FrameAddress,0,0,0,0,0);*/



	PAL_memCopy((char *)&pData->FrameHeader, (char *)FrameAddress, TRACER_HDR_LEN);
	pData->FrameHeader.fc = ENDIAN_HANDLE_WORD(pData->FrameHeader.fc);
	pData->FrameHeader.seqCtrl = ENDIAN_HANDLE_WORD(pData->FrameHeader.seqCtrl);
	pData->FrameHeader.duration = ENDIAN_HANDLE_WORD(pData->FrameHeader.duration);
	PAL_memCopy((char *)&pData->FrameData, (char *)FrameAddress+TRACER_HDR_LEN, 4);


#ifndef NO_PLCP_PATCH
	pData->PlcpDiffLen = 0;
#endif



	if (whal_ParamsGetNvParams()->TraceFile == 1)
	{
		whal_trcSendLine(TrcDb.Idx);
	}
	if (whal_ParamsGetNvParams()->TraceOut == 1)
	{
		whal_trcPrintLine(TrcDb.Idx);
	}
	/* 
	 * Prepare next index
	 */
	if (++TrcDb.Idx >= TRACER_MAX_EVENTS)
		TrcDb.Idx = 0;

	if (TrcDb.Num < TRACER_MAX_EVENTS)
		TrcDb.Num++;

	PROTECT_OFF();
#endif
	return(0);
}

/****************************************************************************
 *                      whal_trcAddTc()
 ****************************************************************************
 * DESCRIPTION:	Add Tx complete info line to the DB
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_trcAddTc(TxDesc_T *pTxDesc, char *Action)
{
#if 0
	TRACER_EVENT *pEvt;
	UINT32 FrameAddress;
    UINT16 *len;
	TxDesc_T local_TxDesc;
	TRACER_DATA	*pData;
    HostDesc_T *pHostDesc;


#if 0 /* interrupt context problem */
	PROTECT_ON();
#endif

	pEvt = &TrcDb.Evt[TrcDb.Idx];
	pData = &pEvt->Info.TrcData;
	/*
	 * Common
	 */
	strncpy(pEvt->Action, Action, TRACER_MAX_ACT_LEN-1);
	strncpy(pEvt->Object, "Tx", TRACER_MAX_OBJ_LEN-1);

#if 1
	pEvt->TimStamp = whal_acxRegRead(ACX_MAC_TIMESTAMP_REG /* Mac TimeStamp counter */);
#else
	pEvt->TimStamp	= local_RxDesc.time;
#endif

	/* 
	 * Data
	 */
	whal_txDesc_Read(&local_TxDesc, pTxDesc);

	pData->MpduLen	 = local_TxDesc.MPDULen;
	pData->time		 = 0;
	pData->Ctl		 = local_TxDesc.Ctl;
	pData->Rate		 = RateConvertMask_Mask16_to_Val8(local_TxDesc.Rate);
	pData->Status[0] = local_TxDesc.Ctl2;
	pData->Status[1] = local_TxDesc.AckFailures;
	pData->Status[2] = local_TxDesc.RTSSuccess;
	pData->Status[3] = local_TxDesc.RTSFailures;

    /* sending LOCAL Tx descriptor */
    FrameAddress = whal_txDesc_GetAcxBufAddr(local_TxDesc);

    /*logMsg("whal_trcAddTc: FrameAddress: 0x%x\n",FrameAddress,0,0,0,0,0);*/

    tiDCacheBlocksInvalidate(0,
        FrameAddress,                    /* virtual address */
        TRACER_HDR_LEN + 4 );    /* number of bytes to invalidate */


	PAL_memCopy((char *)&pData->FrameHeader, (char *)FrameAddress, TRACER_HDR_LEN);
	pData->FrameHeader.fc = ENDIAN_HANDLE_WORD(pData->FrameHeader.fc);
	pData->FrameHeader.seqCtrl = ENDIAN_HANDLE_WORD(pData->FrameHeader.seqCtrl);
	pData->FrameHeader.duration = ENDIAN_HANDLE_WORD(pData->FrameHeader.duration);
	PAL_memCopy((char *)&pData->FrameData, (char *)FrameAddress+TRACER_HDR_LEN, 4);

	if (whal_ParamsGetNvParams()->TraceFile == 1)
	{
		whal_trcSendLine(TrcDb.Idx);
	}
	if (whal_ParamsGetNvParams()->TraceOut == 1)
	{
		whal_trcPrintLine(TrcDb.Idx);
	}
	/* 
	 * Prepare next index
	 */
	if (++TrcDb.Idx >= TRACER_MAX_EVENTS)
		TrcDb.Idx = 0;

	if (TrcDb.Num < TRACER_MAX_EVENTS)
		TrcDb.Num++;

#if 0 /* interrupt context problem */
	PROTECT_OFF();
#endif

#endif
	return(0);
}

/****************************************************************************
 *                      whal_trcAddTx()
 ****************************************************************************
 * DESCRIPTION:	Add Tx info line to the DB
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_trcAddCtrl(char *Action, char *String, int IntValue)
{
#if 0
	TRACER_EVENT *pEvt;
	TRACER_CTRL	*pCtrl;

	PROTECT_ON();

	pEvt = &TrcDb.Evt[TrcDb.Idx];
	pCtrl = &pEvt->Info.TrcCtrl;
	/*
	 * Common
	 */
	strncpy(pEvt->Action, Action, TRACER_MAX_ACT_LEN-1);
	strncpy(pEvt->Object, "Cr", TRACER_MAX_OBJ_LEN-1);

#if 1
	pEvt->TimStamp = whal_acxRegRead(ACX_MAC_TIMESTAMP_REG /* Mac TimeStamp counter */);
#else
	pEvt->TimStamp	= local_RxDesc.time;
#endif

	/* 
	 * Control
	 */
	strncpy(pCtrl->String, String, TRACER_MAX_STR_LEN-1);
	pCtrl->IntValue = IntValue;

	if (whal_ParamsGetNvParams()->TraceFile == 1)
	{
		whal_trcSendLine(TrcDb.Idx);
	}
	if (whal_ParamsGetNvParams()->TraceOut == 1)
	{
		whal_trcPrintLine(TrcDb.Idx);
	}
	/* 
	 * Prepare next index
	 */
	if (++TrcDb.Idx >= TRACER_MAX_EVENTS)
		TrcDb.Idx = 0;

	if (TrcDb.Num < TRACER_MAX_EVENTS)
		TrcDb.Num++;

	PROTECT_OFF();
#endif
	return(0);
}

void whal_trcInfo(void)
{
	printf("whal_trcInfo: Number of events = %d\n", TrcDb.Num);
}

void sprint_mac(char *to, char *mac)
{
	int i;
	char *pBuf=to;
	for (i=2; i<6; i++, pBuf+=2)
		sprintf(pBuf, "%02x", mac[i]);
	*(pBuf) = 0;
}

int whal_trcOpenFile(void)
{
#if 0
	char OutputFileName[] = "Trace1.txt";
	/*
	 * Load image from the host (via FTP)
	 */
	char FileName[256] = "host:c:/firmware/";

	strcat(FileName, OutputFileName);
	printf("whal_trcOpenFile: Filename=%s\n", FileName);
	TraceFd = fopen(FileName, "w");
	if (TraceFd == 0)
	{
		printf("whal_trcOpenFile: Can't open file %s\n", FileName);
		return -1;
	}
#endif
	return(0);
}


int whal_trcSendLine(int EventIndex)
{
#if 0
	TRACER_EVENT *pEvt;
	TRACER_DATA	*pData;
	TRACER_CTRL	*pCtrl;
	dot11_header_T	 *pHdr;
	char Addr1[20], Addr2[20], Addr3[20];

	if (TraceFd == 0)
		return 0;

			pEvt = &TrcDb.Evt[EventIndex];
			pData = &pEvt->Info.TrcData;
			pCtrl = &pEvt->Info.TrcCtrl;
			pHdr = &pData->FrameHeader;

			fprintf(TraceFd, "%04d: %10lu %02s %4s ", 
				EventIndex, pEvt->TimStamp, pEvt->Object, pEvt->Action);

			if ( strcmp(pEvt->Object, "Cr") == 0 )
			{
				fprintf(TraceFd, "%s %08x", 
					pCtrl->String, pCtrl->IntValue);
			}
			else
			{
				sprint_mac(Addr1, pHdr->address1.addr);
				sprint_mac(Addr2, pHdr->address2.addr);
				sprint_mac(Addr3, pHdr->address3.addr);
				fprintf(TraceFd, "%04x %02x %02x %08x ",
					pData->MpduLen, pData->Ctl, pData->Rate, *(UINT32 *)pData->Status);
				fprintf(TraceFd, "%04x %8s %8s %8s ",
					pHdr->fc, Addr1, Addr2, Addr3); 
#ifndef NO_PLCP_PATCH
				if (TrcDb.Enable & 0x02)
					fprintf(TraceFd, "%4d ", pData->PlcpDiffLen);
#endif
				if (TrcDb.Enable & 0x04)
					fprintf(TraceFd, "%04x %08x %08x", pData->FrameHeader.duration, pData->time, pData->FrameData);
			}

			fprintf(TraceFd, "\n\r");

			fflush(TraceFd);
#endif
	return 0;

}

int whal_trcPrintLine(int EventIndex)
{
#if 0
	TRACER_EVENT *pEvt;
	TRACER_DATA	*pData;
	TRACER_CTRL	*pCtrl;
	dot11_header_T	 *pHdr;
	char Addr1[20], Addr2[20], Addr3[20];

			pEvt = &TrcDb.Evt[EventIndex];
			pData = &pEvt->Info.TrcData;
			pCtrl = &pEvt->Info.TrcCtrl;
			pHdr = &pData->FrameHeader;

			printf("%04d: %10lu %02s %4s ", 
				EventIndex, pEvt->TimStamp, pEvt->Object, pEvt->Action);

			if ( strcmp(pEvt->Object, "Cr") == 0 )
			{
				printf("%s %08x", 
					pCtrl->String, pCtrl->IntValue);
			}
			else
			{
				sprint_mac(Addr1, pHdr->address1.addr);
				sprint_mac(Addr2, pHdr->address2.addr);
				sprint_mac(Addr3, pHdr->address3.addr);
				printf("%04x %02x %02x %08x ",
					pData->MpduLen, pData->Ctl, pData->Rate, *(UINT32 *)pData->Status);
				printf("%04x %8s %8s %8s ",
					pHdr->fc, Addr1, Addr2, Addr3); 
#ifndef NO_PLCP_PATCH
				if (TrcDb.Enable & 0x02)
					printf("%4d ", pData->PlcpDiffLen);
#endif
				if (TrcDb.Enable & 0x04)
					printf("%04x %08x %08x", pData->FrameHeader.duration, pData->time, pData->FrameData);
			}

			printf("\n");
#endif
	return 0;

}

/****************************************************************************
 *                      whal_trcPrint()
 ****************************************************************************
 * DESCRIPTION:	Print the DB events
 * 
 * INPUTS:	
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_trcPrint(int Num, int From ,char *buf)
{
#if 0
	TRACER_EVENT *pEvt;
	TRACER_DATA	*pData;
	TRACER_CTRL	*pCtrl;
	dot11_header_T	 *pHdr;
	char Addr1[20], Addr2[20], Addr3[20];
	int NextIdx, i ,len;

	if (Num < 0 || From < 0)
		return(-1);

	if (From >= TrcDb.Num)
	{
		printf("whal_trcPrint: Only %d events\n", TrcDb.Num);
		return(-1);
	}

	if ((From+Num) >= TrcDb.Num)
		Num = TrcDb.Num-From;

	if (TrcDb.Idx > 0)
		NextIdx = TrcDb.Idx-1;
	else
		NextIdx = TRACER_MAX_EVENTS-1;

	for(i=0; i<From; i++)
	{
		if (NextIdx > 0)
			NextIdx--;
		else
			NextIdx = TRACER_MAX_EVENTS-1;
	}

	if (TrcDb.Num < Num)
		Num = TrcDb.Num;

	if (Num == 0)
	{
		len += sprintf(buf,"	Empty\n");
		return(len);
	}


	if (TrcDb.Enable & 0x01) /* brief */
	{

		len + =sprintf(buf+len,"Idx     u_second Tp Act  Len  Cl Rt Status   CTRL Addr1    Addr2    Addr3    ");
		if (TrcDb.Enable & 0x02)
			len + =sprintf(buf+len,"Pclp ");
		if (TrcDb.Enable & 0x04)
			len + =sprintf(buf+len,"Dura TimStamp FirstDat");
		len + =sprintf(buf+len,"\n");

		len + =sprintf(buf+len,"---   ---------- -- ---- ---- -- -- -------- ---- -------- -------- -------- ");
#ifndef NO_PLCP_PATCH
		if (TrcDb.Enable & 0x02)
			len + =sprintf(buf+len,"---- ");
#endif
		if (TrcDb.Enable & 0x04)
			len + =sprintf(buf+len,"---- -------- --------");
		len + =sprintf(buf+len,"\n");
		for(i=0; i<Num; i++)
		{
			pEvt = &TrcDb.Evt[NextIdx];
			pData = &pEvt->Info.TrcData;
			pCtrl = &pEvt->Info.TrcCtrl;
			pHdr = &pData->FrameHeader;

			len + =sprintf(buf+len,"%04d: %10lu %02s %4s ", 
				NextIdx, pEvt->TimStamp, pEvt->Object, pEvt->Action);

			if ( strcmp(pEvt->Object, "Cr") == 0 )
			{
				len + =sprintf(buf+len,"%s %08x", 
					pCtrl->String, pCtrl->IntValue);
			}
			else
			{
				sprint_mac(Addr1, pHdr->address1.addr);
				sprint_mac(Addr2, pHdr->address2.addr);
				sprint_mac(Addr3, pHdr->address3.addr);
				len + =sprintf(buf+len,"%04x %02x %02x %08x ",
					pData->MpduLen, pData->Ctl, pData->Rate, *(UINT32 *)pData->Status);
				len + =sprintf(buf+len,"%04x %8s %8s %8s ",
					pHdr->fc, Addr1, Addr2, Addr3); 
#ifndef NO_PLCP_PATCH
				if (TrcDb.Enable & 0x02)
					len + =sprintf(buf+len,"%4d ", pData->PlcpDiffLen);
#endif
				if (TrcDb.Enable & 0x04)
					len + =sprintf(buf+len,"%04x %08x %08x", pData->FrameHeader.duration, pData->time, pData->FrameData);
			}

			len + =sprintf(buf+len,"\n");

			if (NextIdx > 0)
				NextIdx--;
			else
				NextIdx = TRACER_MAX_EVENTS-1;
		}
	}
#endif
	return(0);
}

/****************************************************************************
 *                      whal_trcIsEnable()
 ****************************************************************************
 * DESCRIPTION:	return the enable value
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	Enable value
 ****************************************************************************/
int whal_trcIsEnable(void)
{
	return TrcDb.Enable;
}

/****************************************************************************
 *                      whal_trcEnable()
 ****************************************************************************
 * DESCRIPTION:	enable the tracing
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	None
 ****************************************************************************/
void whal_trcEnable(int val)
{
	TrcDb.Enable = val;
}

/****************************************************************************
 *                      whal_trcInit()
 ****************************************************************************
 * DESCRIPTION:	initiate the module
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	None
 ****************************************************************************/
void whal_trcInit()
{
	PAL_spinLock_init(&TrcLock);

	TrcDb.Enable = whal_ParamsGetNvParams()->TraceEnable;

   if(TrcDb.Enable)
      printf("\n ***********    Whal Tracer Enable !!!   ***********\n\n");
   else
      printf("\n Whal Tracer Disable\n");


	TrcDb.Idx = 0;
	TrcDb.Num = 0;

	if (whal_ParamsGetNvParams()->TraceFile == 1)
	{
		whal_trcOpenFile();
	}
}

/****************************************************************************
 *                      whal_trcUnload()
 ****************************************************************************
 * DESCRIPTION:	Unload the module
 * 
 * INPUTS:	None
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	None
 ****************************************************************************/
void whal_trcUnload(void)
{
}


#ifndef NO_PLCP_PATCH
void PlcpPrint(void)
{
	printf("PlcpDiffCounter = %d\n", PlcpDiffCounter);
	printf("PlcpMaxDiffLen	= %d\n", PlcpMaxDiffLen);
	printf("PlcpMinDiffLen	= %d\n", PlcpMinDiffLen);
}

int whal_trcPrintRxStat( char *buf )
{
	int len = 0;
    len += sprintf(buf," Recieved rates statistics :\n");
    len += sprintf(buf +len,"  1M  : %6d\n" , whalRxCounterRate1 );
    len +=sprintf(buf+len,"  2M  : %6d\n" , whalRxCounterRate2 );
    len +=sprintf(buf+len,"  5M  : %6d\n" , whalRxCounterRate5 );
    len +=sprintf(buf+len,"  6M  : %6d\n" , whalRxCounterRate6 );
    len +=sprintf(buf+len," 11M  : %6d\n" , whalRxCounterRate11 );
    len +=sprintf(buf+len," 12M  : %6d\n" , whalRxCounterRate12 );
    len +=sprintf(buf+len," 18M  : %6d\n" , whalRxCounterRate18 );
    len +=sprintf(buf+len," 22M  : %6d\n" , whalRxCounterRate22 );
    len +=sprintf(buf+len," 24M  : %6d\n" , whalRxCounterRate24 );
    len +=sprintf(buf+len," 36M  : %6d\n" , whalRxCounterRate36 );
    len +=sprintf(buf+len," 48M  : %6d\n" , whalRxCounterRate48 );
    len += sprintf(buf+len ," 54M  : %6d\n" , whalRxCounterRate54 );

	return len;
}




#endif
