/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalAcxCommands.h                
 *   PURPOSE: Contains ACX100 mailbox commands defines/structures
 *   
 ****************************************************************************/

#ifndef _WHAL_ACX_COMMANDS_H
#define _WHAL_ACX_COMMANDS_H

#include "802_11Defs.h" 


/*
 * ACX100 Command definitions
 * --------------------------
 */
typedef enum 
{
/* 0x0000 */	CMD_RESET,
/* 0x0001 */	CMD_INTERROGATE,
/* 0x0002 */	CMD_CONFIGURE,
/* 0x0003 */	CMD_ENABLE_RX,
/* 0x0004 */	CMD_ENABLE_TX,
/* 0x0005 */	CMD_DISABLE_RX,
/* 0x0006 */	CMD_DISABLE_TX,
/* 0x0007 */	CMD_FLUSH_TX,
/* 0x0008 */	CMD_SCAN,
/* 0x0009 */	CMD_STOP_SCAN,
/* 0x000a */	CMD_CONF_TIM, /* virtual bit map */
/* 0x000b */	CMD_JOIN,
/* 0x000c */	CMD_SET_KEY,
/* 0x000d */	CMD_READ_MEMORY,	/* read from register, memory,RAM table					*/
/* 0x000e */	CMD_WRITE_MEMORY,	/* write to register, memory,RAM table					*/
/* 0x000f */	CMD_SLEEP,
/* 0x0010 */	CMD_WAKE,
/* 0x0011 */	CMD_DCS_START,							
/* 0x0012 */	CMD_INIT_MEMORY,
/* 0x0013 */	CMD_CONF_BEACON,
/* 0x0014 */	CMD_CONF_PROBE_RESP,
/* 0x0015 */	CMD_CONF_NULL_DATA,
/* 0x0016 */	CMD_CONF_PROBE_REQ,
/* 0x0017 */	CMD_TEST,
/* 0x0018 */	CMD_RADIO_INIT,
/* 0x0019 */	CMD_RADIO_CALIBRATE,

} AcxMboxCommand_T;

/*
 * ACX100 Command return status
 * ----------------------------
 */
typedef enum 
{
	CMD_STATUS_IDLE,
	CMD_STATUS_CMD_COMPLETE,
	CMD_STATUS_UNKNOWN_CMD,
	CMD_STATUS_UNKNOWN_IE,
	CMD_STATUS_CMD_CMP_CHAN_INVD,
	CMD_STATUS_CMD_RJT_CHAN_INVD,
	CMD_STATUS_CMD_RJT_MAC_INVD,
	CMD_STATUS_CMD_RJT_RD_ONLY,
	CMD_STATUS_CMD_RJT_PWRUP_IE,
} AcxMboxCommandStatus_T;

/*
 * Configure beacon template 
 * -------------------------
 */
#pragma pack(1)
typedef struct 
{
	UINT16 len;
	dot11_beacon BeaconFrame;
} AcxCmd_ConfBeacon_T;
#pragma pack()

/*
 * Configure probe response template
 * ---------------------------------
 */
#pragma pack(1)
typedef struct 
{
	UINT16 len;
	dot11_probeRsp ProbeRespFrame;
} AcxCmd_ConfProbeResp_T;
#pragma pack()

/*
 * Configure tim template
 * ----------------------
 */
#pragma pack(1)
typedef struct 
{
	UINT16 len;
	char infoElements[sizeof( dot11_tim_T ) +
                      sizeof( dot11_ERP_T ) +
                      sizeof( dot11_extSupRates_T ) +
					  sizeof( dot11_rsne_T )];
} AcxCmd_ConfTim_T;
#pragma pack()

/*
 * Enable Rx mailbox command
 * -------------------------
 */
#pragma pack(1)
typedef struct 
{
	UINT8	ChannelNumber;
	UINT8	Dum1;
	UINT16	Dum2;
} AcxCmd_EnableTx_T;
#pragma pack()

/*
 * Enable Rx mailbox command
 * -------------------------
 */
#pragma pack(1)
typedef struct 
{
	UINT8	ChannelNumber;
	UINT8	Dum1;
	UINT16	Dum2;
} AcxCmd_EnableRx_T;
#pragma pack()


/*
 * Start BSS mailbox command
 * -------------------------
 */
/*Rate definitions */
#define RATE_1				0x0A		/* 1 megabit QPSK  */
#define RATE_2				0x14		/* 2 megabit QPSK  */
#define RATE_5_5			0x37		/* 5.5 CCK  */
#define RATE_5_5_PBCC	0xB7		/* 5.5 PBCC */
#define RATE_11			0x6E		/* 11 CCK */
#define RATE_11_PBCC		0xEE		/* 11 PBCC */
#define RATE_22_PBCC		0xDC		/* 22 PBCC */

/* Acx100 Preample */
#define PREAMBLE_OPT_SHORT		0x01	/* use short preamble */
#define PREAMBLE_OPT_LONG		0x00

/* Defines for BSS_Type */
/* the following values match with DEV_MD fields in DEV_MODE register, 0x01 is not used */
#define BSS_TYPE_AD_HOC	0x0			/* BSS is in Ad Hoc mode */
#define BSS_TYPE_INFRA	0x2			/* BSS is in Infrastructure mode */
#define BSS_TYPE_AP		0x3			/* this device is an Access Point  */

#define BSS_TYPE_OFFSET				0
#define BSS_TYPE_MASK				0x07
#define	BSS_TYPE_BAND_OFFSET		4
#define BSS_TYPE_BAND_MASK			0x10
#define	BSS_TYPE_WONE_IDX_OFFSET	6
#define	BSS_TYPE_WONE_IDX_MASK		0xC0


#pragma pack(1)
typedef struct 
{
	UINT8		BssId[6];
	UINT16		BeaconInterval;
	UINT16		BasicRateSet;

    UINT8		DtimInterval;
	UINT8		ACX100_Tx_Rate;

	UINT8		PreambleOpt;
	
/*******************************************************
 *	The BSS_Type field includes following sub-fields:
 *		- BSS Type   - Bits[0:2]
 *		- Reserved   - Bit [ 3 ]
 *		- Band	     - Bit [ 4 ]
 *		- Reserved   - Bit [ 5 ]
 *		- Wone Index - Bits[6:7]
 */
	UINT8		BSS_Type;
/*******************************************************/
	
#if 0 /* (!!!) TNETW1130 check */
	UINT16		ChannelNumber;
	UINT8		Band;
#else
	UINT8		ChannelNumber;
#endif
   UINT8    SSID_length;

   UINT8    SSID[32];
} AcxCmd_StartBss_T;
#pragma pack()

/*
 * Radio Init
 * ----------
 */
#pragma pack(1)
typedef struct 
{
    UINT32       Addr;
    UINT32       NumBytes;
} AcxCmd_RadioInit_T;
#pragma pack()

/*
 * Radio Calibration
 * -----------------
 */
#pragma pack(1)
typedef struct 
{
    UINT32       Option;
    UINT32       Reserved;
} AcxCmd_RadioCalibration_T;
#pragma pack()


/*******************************************
 * ACX100 Test (BIT)
 * =================
 *
 * Tests description:
 * ==================
 * FCC         = Continuous modulated transmission (should not emit carrier)
 * TELEC       = Continuous unmodulated carrier transmission (carrier only)
 * PER_TX_CFG  = Configures the TNETW1100 hardware per the specified parameters,
 *               recomputes the FCS, and begins transmitting 802.11 frames.
 * PER_RX_CFG  = Configures the TNETW1100 hardware per the specified parameters
 *               and starts receiving frames.
 * PER_CLR_STATS = Clears all PER statistics (Rx and Tx)
 * PER_TX_STOP = Stops a PER test in progress.
 * PER_RX_STATS= Reads receive statistics for a PER test.
 * PER_TX_STATS= Reads transmit statistics for a PER test.
 */
enum
{
/* 0x01 */   TESTID_FCC   = 0x01,  /* param1= channel number, param2= transmit data rate  */
/* 0x02 */   TESTID_TELEC,         /* param1= channel number, param2 ignored.             */
/* 0x03 */   TESTID_PER_TX_CFG,    /* Packet Error Rate - Transmitter configuration       */
/* 0x04 */   TESTID_PER_RX_CFG,    /* Packet Error Rate - Rcvr configuration, param1= chID*/
/* 0x05 */   TESTID_PER_CLR_STATS, /* Packet Error Rate - clear statistics, no params.    */
/* 0x06 */   TESTID_PER_TX_STOP,   /* Packet Error Rate - stop test, no params.           */
/* 0x07 */   TESTID_PER_RX_STATS,  /* PER - read RX statistics into structure             */
/* 0x08 */   TESTID_PER_TX_STATS,  /* PER - read TX statistics into structure             */

   TESTID_EXIT  = 0XFF   /* No params, The driver must reset and reload firmware */
};

enum
{
/* 0 */   TEST_MOD_QPSK,
/* 1 */   TEST_MOD_CCK,
/* 2 */   TEST_MOD_PBCC,
/* 3 */   TEST_MOD_OFDM,

   TEST_MOD_NUMOF
};

enum
{
/* 0 */   TEST_MOD_LONG_PREAMBLE,
/* 1 */   TEST_MOD_SHORT_PREAMBLE
};

#define TEST_MOD_MIN_GAP           200
#define TEST_MOD_MIN_TX_BODYLEN    0
#define TEST_MOD_MAX_TX_BODYLEN    2304

#define TEST_MOD_IS_GAP_OK(gap)     ((gap) >= TEST_MOD_MIN_GAP)

#define TEST_MOD_IS_TX_BODYLEN_OK(len)  \
   (INRANGE((len), TEST_MOD_MIN_TX_BODYLEN, TEST_MOD_MAX_TX_BODYLEN) && \
   (((len) & 3) == 0) )

#define TEST_MOD_IS_PREAMBLE_OK(p)  \
   INRANGE((p), TEST_MOD_LONG_PREAMBLE, TEST_MOD_SHORT_PREAMBLE)


/* Basic test structure, applied for the following tests:
 * FCC, TELEC, EXIT, TESTID_PER_CLR_STATS, TESTID_PER_TX_STOP,
 * PER_RX_CFG
 */
#pragma pack(1)
typedef struct
{
   UINT8    testID;  /* One of TESTID_xxx */
   UINT8    param1;
   UINT8    param2;
   UINT8    param3;
} AcxCmd_Test_T;


/* Applied for the following tests:
 * PER_TX_CFG
 */
typedef struct
{
   UINT8    testID;        /* One of TESTID_xxx       */
   UINT8    Channel;       /* same as FCC test param1 */
   UINT8    DataRate;      /* same as FCC test param2 */
   UINT8    ModPreamble;   /* same as FCC test param3 */
   UINT32   NumFrames;     /* Number of test frames to transmit.   */
   UINT32   InterFrameGap; /* Gap in uSec >= 200 */
   UINT16   FrameBodySize; /* [0..2304] in multiples of 4, actual length is +36 */
   UINT8    Dest[MAC_ADDR_LEN] ;    /* Destination MAC address 6 bytes */
   UINT8    TestModeCtrl;  /* bit mask:
                              bit0: When set, the firmware provides the frame body data.
                              When cleared, the host provides the frame body
                              data. The FCS is always generated by hardware.
                              bit1: When set, automatic sequence numbering is enabled.
                              When cleared, the sequence number is fixed. */
   UINT8    Dum1; /* pad */
   UINT8    Dum2;
   UINT8    Dum3;
} AcxCmd_Test_PER_TX_CFG_T;

/* Applied for the following tests:
 * PER_RX_STATS
 */
typedef struct
{
   UINT8    testID;       /* One of TESTID_xxx */
   UINT8    Reserved;
   UINT16   FrameBodyOffset;
   UINT32   NumFramesRcvd;
   UINT32   NumChecksumErrors;
   UINT32   NumFCSErrors;
   UINT32   NumPLCPErrors;
   UINT8    SNRMin;
   UINT8    SNRMax;
   UINT8    SNRAvg;
   UINT8    RxLevelMin;
   UINT8    RxLevelMax;
   UINT8    RxLevelAvg;
   UINT8    Dum1;    /* pad */
   UINT8    Dum2;
} AcxCmd_Test_PER_RX_STATS;

/* Applied for the following tests:
 * PER_TX_STATS
 */
typedef struct
{
   UINT8    testID;       /* One of TESTID_xxx */
   UINT8    Reserved;
   UINT16   FrameBodyOffset;
   UINT32   NumFramesTx;
   UINT8    Dum1;    /* pad */
   UINT8    Dum2;
} AcxCmd_Test_PER_TX_STATS;

#if 0
typedef union
{
   AcxCmd_Test_T              simple;
   AcxCmd_Test_PER_TX_CFG_T   perTxCfg;
   AcxCmd_Test_PER_RX_STATS   perRxStats;
   AcxCmd_Test_PER_TX_STATS   perTxStats;
} AcxCmd_Test_CmdUnion;
#endif   /*0*/



#pragma pack()

#endif   /* _WHAL_ACX_COMMANDS_H */

