/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalAcxInfoElements.h                
 *   PURPOSE: Contains information element defines/structures 
 *				used by the ACX100 and host               
 *   
 ****************************************************************************/

#ifndef _WHAL_ACX_INFO_ELEMENTS_H
#define _WHAL_ACX_INFO_ELEMENTS_H

#include "whal.h"
/*
 * Information element identification field
 * ----------------------------------------
 */
#define ACX_ID		0x0000	/* ACX100 Info elements */
#define DOT11_ID	0x1000	/* 802.11 Info elements */

/* reserved							(ACX_ID | 0x0001) */
#define ACX_PWRMGMT_OPT				(ACX_ID | 0x0002)
#define	ACX_ELM_ID_MEMORY_CONF	(ACX_ID | 0x0003)
#define ACX_ELM_ID_SLOT_TIME_CONF	(ACX_ID | 0x0004)
#define	ACX_ELM_ID_QUEUE_HEAD	(ACX_ID | 0x0005)
#define	ACX_RATE_FALLBACK		(ACX_ID | 0x0006)
#define	ACX_ELM_ID_RADIO_BAND	(ACX_ID | 0x0007)
#define	ACX_ELM_ID_MEM_MAP		(ACX_ID | 0x0008)  
#define	ACX_ELM_ID_SCAN_STATUS	(ACX_ID | 0x0009)  
#define	ACX_AID						(ACX_ID | 0x000A)
#define  ACX_ELM_ID_RADIO_SET_PARAMS   (ACX_ID | 0x000B)
#define	ACX_CONFIG					(ACX_ID | 0x000C)
#define ACX_ELM_ID_FW_REV		(ACX_ID | 0x000D)
#define	ACX_ELM_ID_FCS_ERROR_CNT (ACX_ID | 0x000E)
#define	ACX_MEDIUM_USAGE			(ACX_ID | 0x000F)
#define	ACX_ELM_ID_RX_CONFIG	(ACX_ID | 0x0010)
#define ACX_QUEUE_THRESH            (ACX_ID | 0x0011)
#define ACX_ELM_ID_BSS_IN_PS    (ACX_ID | 0x0012)
#define ACX_ELM_ID_STATISTICS   (ACX_ID | 0x0013)
#define ACX_INT_CONFIG          (ACX_ID | 0x0014)
#define ACX_FEATURE_CONFIG		(ACX_ID | 0x0015)
/* reserved						(ACX_ID | 0x0016) */
#define ACX_MISC_CONFIG			(ACX_ID | 0x0017)
#define ACX_WONE_CONFIG			(ACX_ID | 0x0018)
#define ACX_WONE_SLICE			(ACX_ID | 0x0019)
#define ACX_TID_CFG			(ACX_ID | 0x001A)
#define ACX_CAL_ASSESSMENT		(ACX_ID | 0x001E)
#define MAX_ACX_IE			((ACX_CAL_ASSESSMENT + 1) & ~ACX_ID)  /* SEE NOTE */
#define ACX_DYNAMIC_CHANNEL_SELC		(ACX_ID | 0x001C) 
#define ACX_HDK_VERSION			(ACX_ID | 0x0022)


#define DOT11_ELM_ID_STATION_ID	(DOT11_ID | 0x0001) 
#define DOT11_ELM_ID_FRAG_THRESHOLD  (DOT11_ID | 0x0002)
/* Reserved                                     0x0003 */
#define DOT11_ELM_ID_MAX_RX_LIFETIME (DOT11_ID | 0x0004) /*Victoria only*/
#define DOT11_SHORT_RETRY_LIMIT	(DOT11_ID | 0x0005)
#define DOT11_LONG_RETRY_LIMIT	(DOT11_ID | 0x0006)
#define DOT11_WEP_DEFAULT_KEYS	(DOT11_ID | 0x0007)
#define DOT11_MAX_TX_LIFETIME		(DOT11_ID | 0x0008)
#define DOT11_GROUP_ADDR			(DOT11_ID | 0x0009)
#define DOT11_CUR_REG_DOMAIN	(DOT11_ID | 0x000A)
#define DOT11_CUR_ANT			    (DOT11_ID | 0x000B)
/* Reserved                                     0x000C */
#define DOT11_CUR_PWR_LVL			(DOT11_ID | 0x000D)
#define DOT11_CUR_CCA_MODE			(DOT11_ID | 0x000E)
#define DOT11_ED_THRESHOLD			(DOT11_ID | 0x000F)  
#define DOT11_WEP_DEFAULT_KEY_ID (DOT11_ID | 0x0010)
#define DOT11_CUR_5GHZ_REG_DOMAIN   (DOT11_ID | 0x0011)
#define DOT11_PD_THRESHOLD			(DOT11_ID | 0x0012)
#define DOT11_ACI                   (DOT11_ID | 0x0013)
#define DOT11_RX_DOT11_MODE         (DOT11_ID | 0x0014)
#define MAX_DOT11_IE				((DOT11_WEP_DEFAULT_KEY_ID + 1) & ~DOT11_ID)

#define MAX_INFO_ELE MAX_DOT11_IE + 1 	/* the largest of any type of info ele		*/
#define ID_TYPE_MASK 0xF000				/* the first 4 bits define the table to use */
#define ID_INDEX	0x0FFF


/*
 * Information element header
 * --------------------------
 */
#pragma pack(1)
typedef struct 
{
	UINT16	Id;
	UINT16	Len;
} ElmHdr_T;
#pragma pack()

/*
 * Queue configuration information element
 * ---------------------------------------
 */
#pragma pack(1)
typedef struct 
{
	UINT32	QueueAddr;
	UINT8	QueuePri;
	UINT8	NumberOfTxDescriptors; /*Replaced field - previously: uint8 pad1*/
	UINT16	Reserved2;
} TxQueueConfig_T;
#pragma pack()

#define RX_QUEUE_AUTO_RESET	1


#pragma pack(1)
typedef struct 
{
/*
 * IE FOR 1100 
 * -----------
 */   

	ElmHdr_T	ElmHdr;
	UINT32	QueueAreaSize;
	UINT32  RxQueueAddress;
	UINT8	RxQueueOptions;
	UINT8	NumTxQueues;
	UINT8	NumberOfRxDescriptors;
	UINT8	Pad;
	UINT32	QueueIndicatorAddress;
	UINT32	HostQueueIndicatorAddress;
	TxQueueConfig_T TxQueueConfig[/*2 */NUM_TX_QUEUES];
} AcxElm_QueueConfig_T;
#pragma pack()

/*
 * Memory configuration information element
 * ----------------------------------------
 */

#pragma pack(1)
typedef struct 
{
/*
 * IE FOR 1100 
 * -----------
 */   

	ElmHdr_T	ElmHdr;
	UINT32	DMAConfig;
	UINT32	*HostReceiveIndDesc;
	UINT32	*RxMemBlockHead;
	UINT32	*TxMemBlockHead;
	UINT16	RxMemBlockNum;
	UINT16	TxMemBlockNum;
} AcxElm_MemoryConfigOptions_T;
#pragma pack()




/*
 * Memory configuration information element
 * ----------------------------------------
 */
#define IDX_WEP_CACHE_START	3
#define IDX_WEP_CACHE_END	4
#define IDX_STA_TABLE_START	5
#define IDX_STA_TABLE_END	6
#define IDX_TEMPLATE_START	7
#define IDX_QUEUE_START		9
#define IDX_POOL_START		11
#define MEM_MAP_NUM_FILEDS	12
#pragma pack(1)
typedef struct 
{
	ElmHdr_T	ElmHdr;				
	void	*CodeStart;
	void	*CodeEnd;
	UINT8	*WEPCacheStart;
	UINT8	*WEPCacheEnd;
	void	*staTableStart;
	void	*staTableEnd;
	void	*PacketTemplateStart;
	void	*PacketTemplateEnd;
	void	*QueueMemoryStart;
	void	*QueueMemoryEnd;
	void	*PacketMemoryPoolStart;
	void	*PacketMemoryPoolEnd;
} AcxElm_MemoryMap_T;
#pragma pack()

#pragma pack(1)
typedef struct
{
    UINT32 radioModuleLocation;
    UINT32 moduleSize;
}RadioInit_t;
#pragma pack()

/*
 * Rx configuration (filter) information element
 * ---------------------------------------------
 */
#define RX_CFG_PROMISCUOUS				0x0008	/* promiscuous - receives all valid frames */
#define RX_CFG_BSSID					0x0020	/* receives frames from any BSSID */
#define RX_CFG_MAC						0x0010	/* receives frames destined to any MAC address */
#ifdef	USE_OLD_WRONG__FILTER_CONFIG
#define RX_CFG_ENABLE_ANY_DEST_MAC		0x0010	
#define RX_CFG_ENABLE_ONLY_MY_BSSID		0x0000  
#define RX_CFG_ENABLE_ANY_BSSID			0x0020  
#else
#define RX_CFG_ENABLE_ANY_DEST_MAC		0x0000
#define RX_CFG_ENABLE_ONLY_MY_DEST_MAC  0x0010
#define RX_CFG_ENABLE_ONLY_MY_BSSID		0x0020  
#define RX_CFG_ENABLE_ANY_BSSID			0x0000  
#endif
#define RX_CFG_DISABLE_BCAST			0x0200	/* discards all broadcast frames */
#define RX_CFG_ENABLE_ONLY_MY_SSID		0x0400

/* defines for ConfigOptions */
#define CFG_RX_RAW			0x0002	/* 1 = write all data from baseband to frame buffer */
										/* including PHY header */
#define CFG_RX_FCS			0x0004	/* 1 = write FCS to end of frame in memory */
										/* 0 = do not write FCS to memory */
#define CFG_RX_ALL_GOOD		0x0008	/* promiscuous mode, receive all good frames */
#define CFG_UNI_FILTER_EN	0x0010	/* local MAC address filter enable */
#define CFG_BSSID_FILTER_EN 0x0020  /* BSSID filter enable */
#define CFG_MC_FILTER_EN	0x0040	/* 0 = receive all multicast */
										/* 1 = use one or both multicast address filters */
#define CFG_MC_ADDR0_EN		0x0080	/* 1 = receive frames from mc_addr0 */
										/* 0 = do not use this filter */
#define CFG_MC_ADDR1_EN		0x0100  /* 1 = receive frames from mc_addr1 */
										/* 0 = do not use this filter  */
#define CFG_BC_REJECT_EN	0x0200	/* 0 = receive all broadcast 1 = filter all broadcast */
#define CFG_SSID_FILTER_EN  0x0400  /*  SSID Filter Enable */
#define CFG_INT_FCS_ERROR	0x0800	/* 1 = give rx complete interrupt for FCS errors */
#define CFG_INT_WEP_ONLY	0x1000	/* 1 = only give rx header interrupt if frame is encrypted */
#define CFG_WR_RX_STATUS	0x2000  /* 1 = write three receive status words to top of rxd MPDU */
										/* 0 = do not write three
										 * status words (added rev 1.5)
										 * */
#define CONFIG_OPTIONS_DEFAULT	(CFG_WR_RX_STATUS | CFG_UNI_FILTER_EN ) /* this value is what ConfigOptions will be */
										/* Initialized to by the ACX100 */

/* defines for Filter Options */
/* Rx Filter Enables    The rx filter enables control what type of receive frames will be
   rejected or received by the rx hardware   1 = frame is written to memory,
   0 = not written to memory, rejected */

#define CFG_RX_RSV_EN		0x0001	/* reserved types and subtypes */
#define CFG_RX_RCTS_ACK		0x0002	/* rts, cts, ack frames */
#define CFG_RX_PRSP_EN		0x0004	/* probe response */
#define CFG_RX_PREQ_EN		0x0008	/* probe request */
#define CFG_RX_MGMT_EN		0x0010	/* type = management */
#define CFG_RX_FCS_ERROR	0x0020	/* frames with FCS errors */
#define CFG_RX_DATA_EN		0x0040	/* type = data */
#define CFG_RX_CTL_EN		0x0080	/* type = control */
#define CFG_RX_CF_EN		0x0100	/* contention free frames */
#define CFG_RX_BCN_EN		0x0200	/* beacons */
#define CFG_RX_AUTH_EN		0x0400	/* authentication, deauthentication */
#define CFG_RX_ASSOC_EN		0x0800	/* association related frames (all 5 subtypes  assoc req/resp,
									 * reassoc req/resp, disassoc)  */
#pragma pack(1)
typedef struct 
{
	ElmHdr_T  ElmHdr;
	UINT16	ConfigOptions;
	UINT16	FilterOptions;
} AcxElm_RxConfig_T;
#pragma pack()


/*
 * Bss Power Save
 * --------------
 */
#pragma pack(1)
typedef struct 
{
	ElmHdr_T	ElmHdr;
	UINT8    	BssInPs;
	UINT8		Pad;
} AcxElm_BssPowerSave_T;  
#pragma pack()

/*
 * Station Id information element
 * ------------------------------
 */
#pragma pack(1)
typedef struct 
{
	ElmHdr_T	ElmHdr;
	UINT8    	StationId[6];
} AcxElm_StationId_T;  
#pragma pack()

/*
 * Rate fallback information element
 * ---------------------------------
 */
#define RATE_FALLBACK_DISABLED	0
#define RATE_FALLBACK_3_RETRIES	3
#pragma pack(1)
typedef struct 
{
	ElmHdr_T	ElmHdr;
	UINT8    	RateFallback;
	UINT8		Pad;
} AcxElm_RateFallback_T;  
#pragma pack()

/*
 * Current Regulatory domain
 * -------------------------
 */
#pragma pack(1)
typedef struct 
{
	ElmHdr_T	ElmHdr;
	UINT8    	Domain;
	UINT8		Pad;
} AcxElm_CurrRegDomain_T;  
#pragma pack()

/*
 * Retry limit
 * -----------
 */
#pragma pack(1)
typedef struct 
{
	ElmHdr_T	ElmHdr;
	UINT8    	RetryLimit;
	UINT8		Pad;
} AcxElm_RetryLimit_T;  
#pragma pack()


/*
 * Power Level
 * -----------
 */
#pragma pack(1)
typedef struct 
{
	ElmHdr_T	ElmHdr;
	UINT8    	PowerLevel;
	UINT8		Pad;
} AcxElm_PowerLevel_T;  
#pragma pack()


/*
 * Fragmentation Threshold
 * -----------------------
 */
#pragma pack(1)
typedef struct 
{
	ElmHdr_T	ElmHdr;
	UINT16    	FragThreshold;
} AcxElm_FragThreshold_T;  
#pragma pack()


/*
 * Acx100 Firmaware version
 * ------------------------
 */
#define FW_VERSION_LEN	20
#define HW_VERSION_LEN	4

#pragma pack(1)
typedef struct
{
	ElmHdr_T	ElmHdr;
	UINT8    	FwVersion[FW_VERSION_LEN];
	UINT8		HwVersion[HW_VERSION_LEN];
} AcxElm_AcxVersion_T; 
#pragma pack()

/*
 * Scan Status information element
 * ----------------------------------
 */
#define NO_SCAN_IN_PROGRESS 0x0
#define SCAN_IN_PROGRESS    0x4

#pragma pack(1)
typedef struct 
{
	ElmHdr_T	ElmHdr;
    UINT32      Status;
} AcxElm_ScanStatus_T;
#pragma pack()

/*
 * Config options information element
 * ----------------------------------
 */
#define ACX_MAX_ANTENNAS	2
#define ELM_ID_ANTENNAS		0
#pragma pack(1)
typedef struct
{
    UINT8	ElmId;
    UINT8	Num;
    UINT8	Table[ ACX_MAX_ANTENNAS ];
} WlanElm_Antennas_T;
#pragma pack()
    
/*
 * Power level information element
 * -------------------------------
 */
#define ACX_MAX_POWER_LEVELS	8
#define ELM_ID_POWER_LEVELS		1
#pragma pack(1)
typedef struct
{
    UINT8	ElmId;    
    UINT8	Num;
    UINT16	Table[ ACX_MAX_POWER_LEVELS ];
} WlanElm_SupportedPowerLevels_T;
#pragma pack()


/*
 * Supported rates information element
 * -----------------------------------
 */
#define SUPPORTED_RATES_MAX		8
#define ELM_ID_SUPPORTED_RATES	2
#pragma pack(1)
typedef struct
{
    UINT8	ElmId;    
    UINT8	Num;
    UINT8	Table[ SUPPORTED_RATES_MAX ];
} WlanElm_SupportedDataRates_T;
#pragma pack()

    
/*
 * reg domains information element
 * -------------------------------
 */
#define ACX_MAX_REG_DOMAINS		6
#define ELM_ID_REG_DOMAINS		3
#pragma pack(1)
typedef struct
{
    UINT8	ElmId;    
    UINT8	Num;
    UINT8	Table[ ACX_MAX_REG_DOMAINS ];
} WlanElm_RegDomaind_T;
#pragma pack()
 
 
/*
 * Product Id information element
 * -------------------------------
 */
#define DOT11_MAX_PRODUCT_ID_STRING_LENGTH 128
#define ELM_ID_PRODUCT_ID		4
#pragma pack(1)
typedef struct 
{
    UINT8	ElmId;    
    UINT8	Num;
    UINT8	Table[ DOT11_MAX_PRODUCT_ID_STRING_LENGTH ];
} WlanElm_ProductId_T;
#pragma pack()
    
/*
 * Manufacture Id information element
 * ----------------------------------
 */
#define DOT11_MAX_MANUF_ID_STRING_LENGTH   128
#define ELM_ID_MANUFACTOR_ID	5
#pragma pack(1)
typedef struct 
{
    UINT8	ElmId;    
    UINT8	Num;
    UINT8	Table[ DOT11_MAX_MANUF_ID_STRING_LENGTH ];
} WlanElm_ManufactorId_T;
#pragma pack()

#define ACX_MAX_NVS_VERSION_LENGTH		8

/*
 * Config Options information element
 * ----------------------------------
 */
#pragma pack(1)
typedef struct
{
#if 0 /* (!!!) not present */
    UINT16	ConfigOptionsID;    // 0x000C   
    UINT16  ConfigOptionsLength;
#endif
    char    NVSVersion[ ACX_MAX_NVS_VERSION_LENGTH ];
    UINT8   dot11MacAddress[6];
    UINT16  ProbeDelay;
    UINT32  EndofMemory;
    UINT8   dot11CCAModeSupported;  
    UINT8   dot11DiversitySupport;
    UINT8   dot11ShortPreambleOptionImplemented;
    UINT8   dot11PBCCOptionImplemented; 
    UINT8   dot11ChanneAgilityPresent;
    UINT8   dot11PHYType;
    UINT8   dot11TempType;
} AcxElm_ConfigOptionsFixed_T;
#pragma pack()

#pragma pack(1)
typedef struct
{
    AcxElm_ConfigOptionsFixed_T		ConfigOptionsFixed;

    WlanElm_Antennas_T				AntennaList;
    WlanElm_SupportedPowerLevels_T	SupportedPowerLevels;
    WlanElm_SupportedDataRates_T	SupportedDataRates;
    WlanElm_RegDomaind_T			SupportedRegDomains;
    WlanElm_ProductId_T				ProductId;
    WlanElm_ManufactorId_T			ManufactureId;
} AcxElm_ConfigOptions_T; 
#pragma pack()

/*
 * Interrupt config
 * ----------------
 */
#pragma pack(1)
typedef struct
{
	ElmHdr_T  ElmHdr;
	UINT32		RxDataThreshold;    /* set by host, in bytes			*/
	UINT32		RxDataTimeout;      /* set by host, in uS				*/
    UINT32		RxDataWatermark;    /* current value queued, in bytes	*/
    UINT32		RxDataTime;         /* current time remaining in uS, if RxDataFrames is non-zero */
    UINT8		RxDataFrames;       /* current #frames queued			*/
} AcxElm_IntConfig_T;
#pragma pack()


/*
 * Default Keys information element
 * --------------------------------
 */
#define KEY_SIZE_232_BITS		232
#define KEY_SIZE_104_BITS		104
#define KEY_SIZE_040_BITS		40
#define KEY_SIZE_MAX_BYTES		32

#define WEP_NUM_DEFAULT_KEYS	4
#define WEP_NUM_MAPPING_KEYS	10

/*
 * Default Key Id information element
 * ----------------------------------
 */
#pragma pack(1)
typedef struct
{
	ElmHdr_T  ElmHdr;
	UINT8   DefaultKeyId;
	UINT8   Pad;
} WlanElm_WepDefaultKeyId_T;
#pragma pack()

/*
 * Fcs Error Count
 * ---------------
 */
#pragma pack(1)
typedef struct
{
	ElmHdr_T  ElmHdr;
	UINT32	  FcsErrorCount;
} WlanElm_FcsError_T;
#pragma pack()



/*
 * Max Tx Msdu lifetime
 * --------------------
 */

/*
 * IE FOR 1100 
 * -----------
 */   
#pragma pack(1)
typedef struct
{
	ElmHdr_T  ElmHdr;
	UINT32	  MaxTxMsduLifetime;
} WlanElm_MaxTxMsduLifetime_T;
#pragma pack()

/*
 * Memory block size
 * --------------------
 */
/*
 * IE FOR 1100 
 * -----------
 */   
#pragma pack(1)
typedef struct
{
	ElmHdr_T  ElmHdr;
	UINT16	  MemBlockSize;
} WlanElm_MemBlockSize_T;
#pragma pack()

#ifndef uint32
#define uint32 UINT32
#endif

#ifndef uint16
#define uint16 UINT16
#endif

#ifndef uint8
#define uint8 UINT8
#endif
 
#pragma pack(1)
typedef struct TxStatistics_t {
    uint32 TxInternalDescOverflow;
} TxStatistics_t;

typedef struct RxStatistics_t {
    uint32 RxOutOfMem;
    uint32 RxHdrOverflow;
    uint32 RxHWStuck;
    uint32 RxDroppedFrame;
    uint32 RxFcsErr;
    uint32 RxXfrHintTrig;
    uint32 RxACIevents;
    uint32 RxACIresets;

} RxStatistics_t;


typedef struct DMAStatistics_t {
    uint32 RxDMARequested;
    uint32 RxDMAErrors;
    uint32 TxDMARequested;
    uint32 TxDMAErrors;
} DMAStatistics_t;

    
typedef struct IsrStatistics_t {
	uint32 Cmd_Cmplt;         /* Host command complete */
	uint32 FIQs;              /* fiqisr() */
	uint32 RxHeaders;         /* (INT_STS_ND & INT_TRIG_RX_HEADER) */
	uint32 RxCompletes;       /* (INT_STS_ND & INT_TRIG_RX_CMPLT) */
	uint32 RxMemOverflow;     /* (INT_STS_ND & INT_TRIG_NO_RX_BUF) */
	uint32 RxRdys;            /* (INT_STS_ND & INT_TRIG_S_RX_RDY) */
	uint32 IRQs;              /* irqisr() */
	uint32 ACXTxProcs;        /* (INT_STS_ND & INT_TRIG_TX_PROC) */
	uint32 DecryptDone;       /* (INT_STS_ND & INT_TRIG_DECRYPT_DONE) */
	uint32 DMA0Done;          /* (INT_STS_ND & INT_TRIG_DMA0) */
	uint32 DMA1Done;          /* (INT_STS_ND & INT_TRIG_DMA1) */
	uint32 ACXTxExchComplete; /* (INT_STS_ND & INT_TRIG_TX_EXC_CMPLT) */
	uint32 ACXCommands;       /* (INT_STS_ND & INT_TRIG_COMMAND) */
	uint32 ACXRxProcs;        /* (INT_STS_ND & INT_TRIG_RX_PROC) */
	uint32 HwPMModeChanges;	  /* (INT_STS_ND & INT_TRIG_PM_802) */
	uint32 HostAcknowledges;  /* (INT_STS_ND & INT_TRIG_ACKNOWLEDGE) */
	uint32 PCI_PM;            /* (INT_STS_ND & INT_TRIG_PM_PCI) */
	uint32 ACMWakeups;        /* (INT_STS_ND & INT_TRIG_ACM_WAKEUP) */

} IsrStatistics_t;

typedef struct WepStatistics_t
{ 
    uint32 WepAddrKeyCount;      /* Count of Wep address keys configured */ 
    uint32 WepDefaultKeyCount;   /* Count of default keys configured  */
    uint32 dot11DefKeyMIB;       /* Current default key being used for Tx */ 
    uint32 WepKeyNotFound;       /* count of number of times that Wep key not found on lookup  */
    uint32 WepDecryptFail;       /* count of number of times that Wep key decryption failed  */
    uint32 WepPackets;           /* Wep Packets Decrypted */
    uint32 WepInterrupt;         /* Wep Decrypt Interrupts */
} WepStatistics_t; 

typedef struct PwrStatistics_t {/* Power Conservation statistics*/
    uint32 TxStartCtr;          /*Tx pwr cons powers down RX I/Q DAC's at TxStart time*/           
    uint32 TxTooShort;          /*Tx pwr cons does not occur due to lack of time */          
    uint32 RxStartCtr;          /*Rx power conservation does power down the Tx block*/
    uint32 RxTooShort;          /*Rx power conservation does not occur due to lack of time*/
    uint32 LPPDBegin;           /*incr every time LPPD causes MAC,BB to doze*/
    uint32 Noisy;               /*LPPD doze did not occur because of high noise level on a frame*/
    uint32 AbortTooShort;       /*LPPD doze did not occur because the frame was too short*/
    uint32 Match;               /*LPPD doze did not occur because the frame is directed to this station (or multi/broadcast)*/
} PwrStatistics_t;

typedef struct MicStatistics_t
{
    uint32 MicRxPkts;
    uint32 MicCalcFailure;
}MicStatistics_t;

typedef struct AesStatistics_t
{
    uint32 AesEncryptFail;
    uint32 AesDecryptFail;
    uint32 AesEncryptPackets;
    uint32 AesDecryptPackets;
    uint32 AesEncryptInterrupt;
    uint32 AesDecryptInterrupt;
}AesStatistics_t;

typedef struct EventStatistics_t 
{
    uint32 heartbeat;
    uint32 calibration;
    uint32 rxMismatch;
    uint32 rxMemEmpty;
    uint32 rxPool;
    uint32 oomLate;
    uint32 phyTransmitError;
    uint32 txStuck;
} EventStatistics_t;



typedef struct ACXStatisticsStruct 
{
	ElmHdr_T  ElmHdr;
    TxStatistics_t   tx;
    RxStatistics_t   rx;
	DMAStatistics_t  dma;
	IsrStatistics_t  isr;
    WepStatistics_t  wep;
    PwrStatistics_t  pwr;
    AesStatistics_t  aes;
    MicStatistics_t  mic;
    EventStatistics_t event;
} ACXStatisticsStruct; 
#pragma pack()

/* Provides expandability for future features */
#define FEAT_ELP_ENABLE     0x01         /*Enable extreme low power mode*/
#define FEAT_PWR_CONS_TX    0x02         /*Pwr conservation during TX*/
#define FEAT_PWR_CONS_RX    0x04         /*Pwr conservation during RX*/
#define FEAT_LPPD           0x08         /*Low Power Packet Detection*/
#define FEAT_HOST_CAL_ICV   0x10         /* Host calculates and inserts ICV for TKIP */
#define FEAT_4X             0x20         /* 4x throughput enhancement (WACK) */
#define FEAT_MAC_CLK_80     0x40         /* Enable 80MHz ARM7 CLK */

/* bit defines for dataflowOptions: */
#define DF_ENCRYPTION_DISABLE BIT_0
#define DF_SNIFF_MODE_ENABLE  BIT_7

/* defines for Pwr Conservation (PwrConsData.PwrFlag global) */
#define PWRFLAG_NONE        0x00
#define PWRFLAG_MASK        (FEAT_PWR_CONS_TX | FEAT_PWR_CONS_RX | FEAT_LPPD)

#pragma pack(1)
typedef struct {
	ElmHdr_T  ElmHdr;
    uint32        Options;     /* bit[0] == 1 enables ELP 
                                  bit[1] == 1 enables TX power conservation
                                  bit[2] == 1 enables RX power conservation
                                  bit[3] == 1 enables LPPD
                                  */
    uint32 dataflowOptions;
} AcxElm_FeatureConfig_T;
#pragma pack()

/* values for indicator */
#define KEY0_TX_ENABLE       0x01 
#define KEY1_TX_ENABLE       0x02
#define KEY0_RX_ENABLE       0x04
#define KEY1_RX_ENABLE       0x08
#define TX_IV_EXHAUST        0x10
#define TOGGLE_TX_KEY_ENABLE 0x80


/*copied this enum from Victoria FW host_if.h*/
typedef uint8 HostIFConfig_e;
typedef enum
{
    HOSTIF_MASTER                            = 0,

    /* Though documented, these modes are invalid (Tom).
       May cause unpredicted FW behavior. */
    HOSTIF_PCI_MASTER_HOST_DIRECT            = 1,
    HOSTIF_PCI_MASTER_HOST_INDIRECT          = 2,
    HOSTIF_VLYNQ_MASTER_HOST_DIRECT          = 3,
    HOSTIF_VLYNQ_MASTER_HOST_INDIRECT        = 4,
    HOSTIF_VLYNQ_PCI_MASTER_HOST_DIRECT      = 5,
    HOSTIF_VLYNQ_TO_PCI_MASTER_HOST_INDIRECT = 6,
    HOSTIF_VLYNQ_TO_USB2                     = 7,

    HOSTIF_USB                               = 8,
    HOSTIF_SLAVE                             = 9
}IntegerHostIFConfig_e;

#define QUEUE_PRI_HIGHEST		0		/* higest queue priority */
#define QUEUE_PRI_LOWEST		0xf	/* lowest queue priority */

#pragma pack(1)
typedef struct
{
	uint8  NumDesc;
	uint8  Reserved1;
	uint8  Reserved2;
	uint8  Attr;
}ACXtxQueueConfig;

typedef struct
{
	uint8  NumDesc;
	uint8  Reserved;
	uint8  Type;
	uint8  Priority;
	uint32 HostRxDescriptorsAddr;
}ACXrxQueueConfig;
#pragma pack()

/*
 * Config memory
 * --------------------
 */
#pragma pack(1)
typedef struct 
{
   ElmHdr_T  ElmHdr;
   uint16  numStations;
   uint16  mblkSize;
   uint8   rxMblkRatio;
   uint8   numRxQueues;
   uint8   numTxQueues;
   HostIFConfig_e hostifOptions;
   uint8	txXfrMblkRatio;
   uint8	reserved1[3];
   ACXrxQueueConfig RxQueueConfig;
   ACXtxQueueConfig TxQueueConfig[/*2 */NUM_TX_QUEUES];
} WlanElm_ConfigMemory_T;
#pragma pack()

/*
 * queues head
 * --------------------
 */
#pragma pack(1)
typedef struct
{
	uint32  Addr;
	uint8   Reserved[3];
	uint8   Priority;
}ACXoneQueueHead;

typedef struct
{
   ElmHdr_T  ElmHdr;
   uint32  *txMemBlkQ;
   uint32  *rxMemBlkQ;
   ACXoneQueueHead RxQueueHead;
   ACXoneQueueHead TxQueueHead[/*2*/ NUM_TX_QUEUES];
} WlanElm_QueueHead_T;
#pragma pack()


/*
 *  WONE Configuration 
 * --------------------
 */
 
#pragma pack(1)
typedef struct
{
	uint8			StartQueue;
	uint8			QueuesNum;
	uint16			Reserved;
} WoneConfig_T;

typedef struct
{
	ElmHdr_T		ElmHdr;
	uint8			WoneNets;
	uint8			Reserved[3];
	uint32			GuardTime;
	WoneConfig_T	Wone[MAX_WONE_NET];
} WlanElm_WoneConfig_T;
#pragma pack()




/*
 *  WONE Slice 
 * --------------------
 */
 
typedef struct
{
	uint32			Slice;
	uint32			Reserved;
} WoneSlice_T;

#pragma pack(1)
typedef struct
{
	ElmHdr_T		ElmHdr;
	WoneSlice_T		Wone[MAX_WONE_NET];
} WlanElm_WoneSlice_T;
#pragma pack()




/*
 *  EDCF Configuration 
 * --------------------
 */

#define AIFS_PIFS  1  /* slot number setting to start transmission at PIFS interval - not sure if we can do this yet */
#define AIFS_DIFS  2  /*slot number setting to start transmission at DIFS interval - normal DCF access */
#define MAX_AIFS   0xFF
#define MAX_EDCF_SLOTS  10   /* FW allows AIFSN up to this number of additional slots beyond DIFS. */
#define NO_RX_TIMEOUT   0    /* rxTimeout default value */
#define SINGLE_MPDU_TXOP_LIMIT  0    /* Only send a single MPDU when a TXOP is obtained. */
#define MSDU_LIFETIME_DEFAULT   512  /* In TUs (1024 usec). */

typedef enum
{
    ACK_IMMEDIATE_POLICY = 0,   /* means expect a standard ACK in response to each MPDU */
    NO_ACK_POLICY = 1,          /* No ACK is expected when transmitting - 
                                    makes bursts possible with only a SIFS between MPDUs */
    /* ALT_ACK_POLICY = 2,         Reserved usage according to draft 4.1 */
    /* BLOCK_ACK_POLICY = 3,       Requires a block ack after a burst has been sent */
    
    MAX_QOS_ACK_POLICY = 0xFF
}ACKPolicy_e;

typedef enum
{
    AC0 = 0,
    AC_DCF = AC0, /* use category 0 for normal DCF access upon initialization - all queues initialized to this */
    AC1 = 1,
    AC2 = 2,
    AC3 = 3,
    NUM_ACCESS_CATEGORIES = 4,
    MAX_ACCESS_CATEGORY = 0xFF
}AccessCategory_e;

typedef enum 
{
    DATA_DCF = 0,       /* MSDUs are sent completely including retrys - normal legacy traffic */
    QOS_DATA_EDCF = 1   /* MPDUs are sent according to TXOP limits. */
}TrafficAccessType_e;


#pragma pack(1)

typedef struct 
{
    UINT8         reserved;
    UINT8		  trafficType;  /* Use TrafficAccessType_e values */
    UINT8		  aifsn;
    UINT8		  cwMin;
    UINT16		  cwMax;
    UINT16		  txopLimit;   /*number of 32us intervals */
    UINT32		  dot11EDCATableMSDULifetime; /* num of TU (1024 usec) intervals. */
    UINT16        rxTimeout;    
    UINT8         ackPolicy;  /* Use ACKPolicy_e values */
    UINT8         shortRetryLimit;
    UINT8         longRetryLimit;
    UINT8	  retryPremption;
    UINT8         txopContinuation;
}AccessCategory_T;





typedef struct
{
	ElmHdr_T		  ElmHdr;
    UINT8             qID;      /* this tells us the transmit queue to link to the access category */
    UINT8			  acID;     /* Use AccessCategory_e values. This tells us which EDCFCfg is being read or written from the NUM_ACCESS_CATEGORIES available */
    UINT16            reserved;
    AccessCategory_T  accessCategory;  /* configuration that applies to EDCF Access Categories */
}WlanElm_AccessCategory_T;

#pragma pack()


/*
 * Used for processing the ACX_QUEUE_THRESH command, in order to set a limit to the 
 *   buffer pool usage by a specific tx queue in the ACX.
 *
 * threshold represents a number of tx memblocks expressed in 5% increments of total
 *   ACX buffer pool.
 * curr_usage is used to retrieve the current amount of memory a queue is using.
 */
#pragma pack(1)
typedef struct 
{
	ElmHdr_T	  ElmHdr;
    UINT32        qID;
    UINT32        threshold;
    UINT16        curr_usage;
    UINT16        total_avail;
}WlanElm_QueueThresh_T;
#pragma pack()


/*
 *  Slot Time Configuration 
 * ------------------------
 */

#define    SLOT_TIME_LONG  0      /* Long Slot Time  */
#define    SLOT_TIME_SHORT 1     /* Short Slot Time */


#pragma pack(1)
typedef struct
{
	ElmHdr_T	ElmHdr;
    UINT8       woneIndex;       
    UINT8       slotTime;  
    UINT8       reserved[6];
} WlanElm_SlotTime_T;
#pragma pack()

/************************
 * Radio parameters
 ************************/

/*#define MAX_RADIO_PARAM_POWER_LEVELS   5
#define MAX_RADIO_PARAM_POWER_RATES    4
#define MAX_RADIO_PARAM_POWER_TABLE    (MAX_RADIO_PARAM_POWER_RATES*MAX_RADIO_PARAM_POWER_LEVELS)
#define RADIO_PARAM_POWER_TABLE      1
*/
#pragma pack(1)
typedef struct
{
	ElmHdr_T	ElmHdr;
   UINT32   parameterType;
   INT8     parameter[MAX_RADIO_PARAM_POWER_TABLE];
} WlanElm_RadioSetPowerTable_T;
#pragma pack()

/* RADAR DETECTION STRUCT*/
#define EVENTS_TIME_VEC_MAX_SIZE  120
#pragma pack(1)
typedef struct
{
   ElmHdr_T	ElmHdr;
   UINT16   events_num;
   UINT16   resrved; 
   UINT16   events_vec    [EVENTS_TIME_VEC_MAX_SIZE];
} radar_detection_events_info;
#pragma pack()

/*
 *  CRT radio module calibration state
 *
*/

#define RUNTIME_CALIBRATION_NOT_NEEDED  1
#define RUNTIME_CALIBRATION_NEEDED      2

#pragma pack(1)
typedef struct
{
	ElmHdr_T    ElmHdr;
	UINT32		RunTimeCalState;        /* run time calibration state for CRT radio module */
} AcxElm_RunTimeCalState_T;
#pragma pack()



#pragma pack(1)
typedef struct 
{
    uint16  referenceDesignId;          /* Reference design Id supported */
    uint8   nvsMajorVersion;            /* First EEPROM version supported */
    uint8   nvsMinorVersion;
    uint8   nvsMinorMinorVersion;

} HDKReferenceDesign_t;
#pragma pack()

#pragma pack(1)
typedef struct 
{

	ElmHdr_T		  	ElmHdr;
    uint8               majorVersion;                         
	uint8               minorVersion;
	uint8               minorMinorVersion;
	uint8               hdkModuleReleaseType;               /* 'A'-'Z' => Engineering Bld, 0 => Release Bld, 1 => Unsupported Bld */
	uint8               radioModuleType;                    /* The radio that is currently supported by the HDK module */
	uint8               numberOfReferenceDesignsSupported;  /* The number of reference designs supported by the HDK module */

    HDKReferenceDesign_t*   referenceDesignsSupported;         /* Array of reference_design supported */

} WlanElm_HDKModuleVersion_t;
 
#pragma pack()


#pragma pack(1)
typedef struct
{
	ElmHdr_T  ElmHdr;
	uint16  txActivityLed;      // GPIO_OUT bit for this LED
	uint16  fwInitLed;          // GPIO_OUT bit for this LED
	uint16  diagnosticLed;      // GPIO_OUT bit for this LED
	uint16  reserved;
} ACXMisc_t;

#pragma pack()

#pragma pack(1)
typedef struct
{
	ElmHdr_T	ElmHdr;
    UINT32 		mediumUsage;
    UINT32 		period;
}AcxElm_MediumUsage_T;
#pragma pack()


#pragma pack(1)
typedef struct
{
	ElmHdr_T	ElmHdr;
    UINT32 		dot11EDThreshold;
}WlanElm_EDThreshold_T;
#pragma pack()

#pragma pack(1)
typedef struct
{
	ElmHdr_T	ElmHdr;
    UINT32 		dot11PDThreshold;
}WlanElm_PDThreshold_T;
#pragma pack()

#pragma pack(1)
typedef struct
{
	ElmHdr_T	ElmHdr;
    UINT32 		dot11RxMode;
}WlanElm_RxDot11Mode_T;
#pragma pack()

#pragma pack(1)
typedef struct
{
	ElmHdr_T	ElmHdr;
    UINT32 		aciEventsCounter;
}AcxElm_AciEvent_T;

#pragma pack()




#endif /* _WHAL_ACX_INFO_ELEMENTS_H */

