/*****************************************************************************

  FILENAME:       public_rates.h

  DESCRIPTION:    defines the allowable physical layer rate settings.
                    PBCC rates are multiples of 100kbps
                    OFDM rates are same as 802.11a/g standard enumerations

   Copyright (c) 2001, 2002 Texas Instruments Inc. 
   All Rights Reserved This program is the confidential and proprietary 
   product of Texas Instruments Inc.  Any Unauthorized use, reproduction or 
   transfer of this program is strictly prohibited.


*****************************************************************************/
#ifndef _PUBLIC_RATES_H
#define _PUBLIC_RATES_H

/*
 * from other defs file 
 */
#define  BIT_0   0x00000001
#define  BIT_1   0x00000002
#define  BIT_2   0x00000004
#define  BIT_3   0x00000008
#define  BIT_4   0x00000010
#define  BIT_5   0x00000020
#define  BIT_6   0x00000040
#define  BIT_7   0x00000080
#define  BIT_8   0x00000100
#define  BIT_9   0x00000200
#define BIT_10   0x00000400
#define BIT_11   0x00000800
#define BIT_12   0x00001000
#define BIT_13   0x00002000
#define BIT_14   0x00004000
#define BIT_15   0x00008000


#define     BIT_RATE_1MBPS   BIT_0
#define     BIT_RATE_2MBPS   BIT_1
#define     BIT_RATE_5_5MBPS BIT_2
#define     BIT_RATE_6MBPS   BIT_3
#define     BIT_RATE_9MBPS   BIT_4
#define     BIT_RATE_11MBPS  BIT_5
#define     BIT_RATE_12MBPS  BIT_6
#define     BIT_RATE_18MBPS  BIT_7
#define     BIT_RATE_22MBPS  BIT_8
#define     BIT_RATE_24MBPS  BIT_9
#define     BIT_RATE_36MBPS  BIT_10
#define     BIT_RATE_48MBPS  BIT_11
#define     BIT_RATE_54MBPS  BIT_12




typedef uint8 Rate_e;
typedef enum
{
    /*Supported Rate field settings in beacons, probe responses...*/
    
    NO_RATE      = 0,
    
    RATE_1MBPS   = 0x0A,
    RATE_2MBPS   = 0x14,
    RATE_5_5MBPS = 0x37,
#ifdef DOT11_G
    RATE_6MBPS   = 0x0B,
    RATE_9MBPS   = 0x0F,
#endif
    RATE_11MBPS  = 0x6E,
#ifdef DOT11_G
    RATE_12MBPS  = 0x0A,
    RATE_18MBPS  = 0x0E,
#endif
    RATE_22MBPS  = 0xDC,
#ifdef DOT11_G
    RATE_24MBPS  = 0x09,
    RATE_36MBPS  = 0x0D,
    RATE_48MBPS  = 0x08,
    RATE_54MBPS  = 0x0C,

    MINIMUM_RATE = RATE_6MBPS,
    MAX_NUM_RATES =   14,
#else
    MINIMUM_RATE = RATE_1MBPS,
    MAX_NUM_RATES = 6,
#endif

    END_RATES = 0xFF
}IntegerRate_e;

/*Seneca Rate definitions, included to allow easy compile without doing
a lot of global replacements */
#define RATE_1				0x0A		/* 1 megabit */
#define RATE_2				0x14		/* 2 megabit */
#define RATE_5_5			0x37		/* 5.5 CCK */
#define RATE_5_5_PBCC		0xB7		/* 5.5 PBCC */
#define RATE_11				0x6E		/* 11 CCK */
#define RATE_11_PBCC		0xEE		/* 11 PBCC */
#define RATE_22_PBCC		0xDC		/* 22 PBCC */


#define SHORT_PREAMBLE_BIT  BIT_0               /*CCK or Barker depending on the rate*/
#define OFDM_RATE_BIT       BIT_6
#define PBCC_RATE_BIT       BIT_7

#define TEST_CCK_MOD(x)  (!((x) & (PBCC_RATE_BIT | OFDM_RATE_BIT)))
#define TEST_PBCC_MOD(x) ((x) & PBCC_RATE_BIT)
#define TEST_OFDM_MOD(x) ((x) & OFDM_RATE_BIT)
#define TEST_SHORT_PREAMBLE(x) ((x) & SHORT_PREAMBLE_BIT)



typedef uint8 Mod_e;
typedef enum
{
    CCK_LONG = 0,
    CCK_SHORT = SHORT_PREAMBLE_BIT,
    PBCC_LONG = PBCC_RATE_BIT,
    PBCC_SHORT = PBCC_RATE_BIT | SHORT_PREAMBLE_BIT,
#ifdef DOT11_G
    OFDM = OFDM_RATE_BIT,
#endif
    
    END_MODULATIONS = 0xFF
}IntegerMod_e;


typedef uint16 BasicRateSet_t;
/*BasicRateSet_t*/
#ifdef ACX100
#define    RATE_SET_1MBPS    BIT_0
#define    RATE_SET_2MBPS    BIT_1
#define    RATE_SET_5_5MBPS  BIT_2
#define    RATE_SET_11MBPS   BIT_3
#define    RATE_SET_22MBPS   BIT_4
#else   /*ACX111*/
#define    RATE_SET_1MBPS    BIT_0
#define    RATE_SET_2MBPS    BIT_1
#define    RATE_SET_5_5MBPS  BIT_2
#define    RATE_SET_11MBPS   BIT_5
#define    RATE_SET_22MBPS   BIT_8
#endif


#ifndef ACX100
#ifdef DOT11_G
typedef uint8 OFDMRateSet_t;

/*OFDMRateSet_t = bits 16-23 of BASIC_RATE_CFG*/
#define    OFDM_6MBPS     BIT_0
#define    OFDM_9MBPS     BIT_1
#define    OFDM_12MBPS    BIT_2
#define    OFDM_18MBPS    BIT_3
#define    OFDM_24MBPS    BIT_4
#define    OFDM_36MBPS    BIT_5
#define    OFDM_48MBPS    BIT_6
#define    OFDM_54MBPS    BIT_7

#endif
#endif


/*
//---------------------------------------------------------------------------

//
//Transmit-Descriptor RATE-SET field definitions...
//

//
//Define a new "Rate-Set" for TX path that incorporates the 
//Rate & Modulation info into a single 16-bit field.
//
//TxdRateSet_t:
//  b15    - Indicates Preamble type (1=SHORT, 0=LONG).
//           Notes:
//             Must be LONG (0) for 1Mbps rate.
//             Does not apply (set to 0) for RevG-OFDM rates.
//  b14    - Indicates PBCC encoding (1=PBCC, 0=not).
//           Notes:
//             Does not apply (set to 0) for rates 1 and 2 Mbps.
//             Does not apply (set to 0) for RevG-OFDM rates.
//  b13    - Unused (set to 0).
//  b12-b0 - Supported Rate indicator bits as defined below.
//
*/
typedef uint16 TxdRateSet_t;

#define TXD_RATESET_SHORT_PREAMBLE  BIT_15
#define TXD_RATESET_PBCC            BIT_14

/*
//Use this to mask against the TxdRateSet_t field from the TxDescriptor
//to get the Supported-RATES indicator bits.
*/
#define TXD_RATE_MASK       0x1FFF



                                           /* Possible Options with these rates:*/
                                           /*  Preamble          PBCC*/
                                           /* Long  Short*/
                                           /* ----  -----        --------------*/
#define    TXD_RATESET_NO_RATE    0
#define    TXD_RATESET_1MBPS      BIT_0    /* yes   no           no*/
#define    TXD_RATESET_2MBPS      BIT_1    /* yes   yes          no*/
#define    TXD_RATESET_5_5MBPS    BIT_2    /* yes   yes          yes*/

                                           /* (note: short/long/pbcc do not apply to OFDM (revG) rates)*/
#define    TXD_RATESET_6MBPS      BIT_3    /* n/a   n/a          n/a*/
#define    TXD_RATESET_9MBPS      BIT_4    /* n/a   n/a          n/a*/

#define    TXD_RATESET_11MBPS     BIT_5    /* yes   yes          yes*/

                                           /* (note: short/long/pbcc do not apply to OFDM (revG) rates)*/
#define    TXD_RATESET_12MBPS     BIT_6    /* n/a   n/a          n/a*/
#define    TXD_RATESET_18MBPS     BIT_7    /* n/a   n/a          n/a*/

#define    TXD_RATESET_22MBPS     BIT_8    /* yes   yes          TRUE*/

                                           /* (note: short/long/pbcc do not apply to OFDM (revG) rates)*/
#define    TXD_RATESET_24MBPS     BIT_9    /* n/a   n/a          n/a*/
#define    TXD_RATESET_36MBPS     BIT_10   /* n/a   n/a          n/a*/
#define    TXD_RATESET_48MBPS     BIT_11   /* n/a   n/a          n/a*/
#define    TXD_RATESET_54MBPS     BIT_12   /* n/a   n/a          n/a */
  
#define    TXD_RATESET_HIGHEST_RATE  TXD_RATESET_54MBPS





#endif
