/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2001 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalAcxRegs.h
 *   PURPOSE: Contains ACX100 registers defines/structures
 *   
 ****************************************************************************/

#ifndef _WHAL_ACX_REGS_H
#define _WHAL_ACX_REGS_H

#include "rgm_acx111.h"

#define IS_MASK_ON(  field, bitmask ) (  (bitmask) == ( (field) &  (bitmask) ) )
#define IS_MASK_OFF( field, bitmask ) ( ~(bitmask) == ( (field) | ~(bitmask) ) )

#define IS_ANY_BIT_ON( field, bitmask ) ( (field) & (bitmask) )
#define ARE_ALL_BITS_ON( field, bitmask ) IS_MASK_ON( field, bitmask )

#define MASK_BITS_ON(  field, bitmask ) ( (field) |=  (bitmask) ) 
#define MASK_BITS_OFF( field, bitmask ) ( (field) &= ~(bitmask) )

#define MASK_BITS_TOGGLE( field, bitmask ) ( (field) ^= (bitmask) )


/*===============================================
   Device/Vendor Id - 32bit RW 
 ------------------------------------------
    This register holds the device and vendor id of the ACX100 card 
 ===============================================*/
#define REG_DEVICE_VENDOR_ID				( CPC_REGION ) 
	#define ACX_DEVICE_ID      0x9066
	#define ACX_VENDOR_ID      0x104C

/*
 * ACX100 Registers.
 */
#define ACX_SOR_CFG_REG			(SOR_CFG)
#define ACX_BCN_SEQ_NUM_REG		(BCN_SEQ_NUM)
#define ACX_ENABLE_REG			(ENABLE)
#define ACX_MAC_TIMESTAMP_REG	(MAC_TIMESTAMP)

/*======================================================================
                      Interrupt Registers                               
=======================================================================*/
/*=============================================
  Interrupt Trigger Register (Write only)              
  ------------------------------------------ 
  The host sets bits in this register to issue (trigger) 
  the corresponding interrupts (if not masked) to the
  embedded CPU. This register is self clearing. 
  0 -  CMD    - Host Command Interrupt        
                Setting this bit masks the    
                interrupt that the host issues
                to inform the ACX100 that it  
                has sent a command to the     
                ACX100 Command Mailbox.
  1 -  ACK    - Host Acknowlege Interrupt
                The host sets this bit to
                acknowlege that it receveid
                the unsolicited information
                from the ATX info mailbox.
  2 -  TXPRC  - TX Process Interrupt
                The host sets this bit to
                inform the ACX100 that a Tx
                descriptor is in the queue.
  3 -  RXPRC    RX Process Interrupt          
                The host sets this bit to     
                inform the ACX100 that a Rx   
                descriptor is in the queue.   
  14-15 Reserved                              
                                              
  Default: N/A                                
==============================================*/
#define ACX_REG_INTERRUPT_TRIG				( INT_TRIG )
	#define INTR_TRIG_CMD		0x00000001
	#define INTR_TRIG_ACK		0x00000002
	#define INTR_TRIG_TX_PROC	0x00000004
	#define INTR_TRIG_RX_PROC	0x00000008

/*=============================================
  Host Interrupt Mask Register - 16bit (RW)
  ------------------------------------------  
  Setting a bit in this register masks the
  corresponding interrupt to the host.
  0 - RXD   - Rx Data Interrupt
  1 - TXD   - Tx Data Interrupt
  2 - TXXFR - Tx Transfer Interrupt
  3 - RXXFR - Rx Transfer Interrupt
  4 - DTIM  - DTIM Interrupt
  5 - BEAC  - Beacon Interrupt
  6 - TICK  - Timeout Interrupt
  7 - KEYNF -
  8 - IVFAIL  IV/ICV Failure Interrupt
  9 - CDCMP - Command Complete Interrupt
 10 - INFO  - Information Interrupt
 11 - OVRFL - Overflow Interrupt
 12 - PROCE
 13 - FCSTH
 14 - ICOMP -Initialization Complete Interrupt
 15 Reserved
 Default: 0x0001
*==============================================*/
#define ACX_REG_INTERRUPT_MASK				( HINT_MASK )
	#define ACX_INTR_RX_DATA		0x00000001
	#define ACX_INTR_TX_DATA		0x00000002
	#define ACX_INTR_INIT_COMPLETE	0x00004000
	#define ACX_INTR_CMD_COMPLETE	0x00000200
	#define ACX_INTR_ALL			0x0000ffff
/*=============================================
  Host Interrupt Mask Set 16bit, (Write only) 
  ------------------------------------------  
 Setting a bit in this register sets          
 the corresponding bin in ACX_HINT_MASK register
 without effecting the mask                   
 state of other bits (0 = no effect).         
==============================================*/
#define ACX_HINT_MASK_SET_REG          HINT_MASK_SET

/*=============================================
  Host Interrupt Mask Clear 16bit,(Write only)
  ------------------------------------------  
 Setting a bit in this register clears        
 the corresponding bin in ACX_HINT_MASK register
 without effecting the mask                   
 state of other bits (0 = no effect).         
=============================================*/
#define ACX_HINT_MASK_CLR_REG          HINT_MASK_CLR

/*=============================================
  Host Interrupt Status Nondestructive Read   
  16bit,(Read only)                           
  ------------------------------------------  
 The host can read this register to determine 
 which interrupts are active.                 
 Reading this register doesn't                
 effect its content.                          
=============================================*/
#define ACX_REG_INTERRUPT_NO_CLEAR			( HINT_STS_ND )

/*=============================================
  Host Interrupt Status Clear on Read  Register
  16bit,(Read only)                           
  ------------------------------------------  
 The host can read this register to determine 
 which interrupts are active.                 
 Reading this register clears it,             
 thus making all interrupts inactive.         
==============================================*/
#define ACX_REG_INTERRUPT_CLEAR				( HINT_STS_CLR )

/*=============================================
  Host Interrupt Acknowledge Register         
  16bit,(Write only)                          
  ------------------------------------------  
 The host can set individual bits in this     
 register to clear (acknowledge) the corresp. 
 interrupt status bits in the HINT_STS_CLR and
 HINT_STS_ND registers, thus making the       
 assotiated interrupt inactive. (0-no effect) 
==============================================*/
#define ACX_REG_INTERRUPT_ACK				( HINT_ACK )


/*===============================================
   Host Software Reset - 32bit RW 
 ------------------------------------------
    [31:1] Reserved 
        0  SOFT_RESET Soft Reset  - When this bit is set,
         it holds the ACX100 in a soft reset state. 
         This reset disables all MAC and baseband processor 
         clocks except the CardBus/PCI interface clock. 
         It also initializes all MAC state machines except 
         the host interface. It does not reload the
         contents of the EEPROM. When this bit is cleared 
         (not self-clearing), the ACX100
         exits the software reset state.
===============================================*/
#define ACX_REG_SLV_SOFT_RESET				( SLV_SOFT_RESET )
	#define SLV_SOFT_RESET_BIT		0x00000001

/*===============================================
 EEPROM Burst Read Start  - 32bit RW 
 ------------------------------------------
 [31:1] Reserved 
     0  ACX_EE_START -  EEPROM Burst Read Start 0
        Setting this bit starts a burst read from 
        the external EEPROM. 
        If this bit is set (after reset) before an EEPROM read/write, 
        the burst read starts at EEPROM address 0.
        Otherwise, it starts at the address 
        following the address of the previous access. 
        TheACX100 hardware clears this bit automatically.
        
        Default: 0x00000000
*================================================*/
#define ACX_REG_EE_START					( EE_START )
	#define START_EEPROM_MGR	0x00000001

/*=======================================================================
                        Embedded ARM CPU Control
========================================================================*/
/*===============================================
 Halt eCPU   - 32bit RW 
 ------------------------------------------
    0 HALT_ECPU Halt Embedded CPU - This bit is the 
      compliment of bit 1 (MDATA2) in the SOR_CFG register. 
      During a hardware reset, this bit holds 
      the inverse of MDATA2.
      When downloading firmware from the host, 
      set this bit (pull down MDATA2). 
      The host clears this bit after downloading the firmware into 
      zero-wait-state SSRAM.
      When loading firmware from Flash, clear this bit (pull up MDATA2) 
      so that the eCPU can run the bootloader code in Flash
    HALT_ECPU eCPU State
    --------------------
    1 halt eCPU
    0 enable eCPU
===============================================*/
#define ACX_REG_ECPU_CONTROL				( ECPU_CTRL )
	#define ECPU_CONTROL_HALT	0x00000001				

/*===============================================
  Host Slave Access Endian Control -32bit, RW
 ------------------------------------------
    [31:4]  Reserved 
    [3:1]   ENDIAN_CTL Slave Access Endian Control  - this field 
            provides endian control for host direct and indirect 
            memory access, when the ACX100 is bus slave, according 
            to the following table. Bit 3 affects indirect mem-ory
            space accesses, bit 2 affects shared RAM accesses, 
            and bit 1 affects register accesses. 
            CIS region accesses are always little endian.
        0   Reserved 

===============================================*/
#define ACX_REG_SLV_END_CTL					( SLV_END_CTL )
	#define SLV_BIG_ENDIAN_ALL	0xffffffff



/*=======================================================================
                    Command/Information Mailbox Pointers
========================================================================*/

/*===============================================
   Command Mailbox Pointer - 32bit RW 
 ------------------------------------------
    This register holds the start address of 
    the command mailbox located in the ACX100 memory. 
    The host must read this pointer after a reset to 
    find the location of the command mailbox. 
    The ACX100 initializes the command mailbox 
    pointer with the default address of the command mailbox.
    The command mailbox pointer is not valid until after 
    the host receives the Init Complete interrupt from 
    the ACX100.
===============================================*/
#define REG_COMMAND_MAILBOX_PTR				( SCR_PAD0 ) 
#define REG_COMMAND_SCR_PAD0				( SCR_PAD0 ) 

/*===============================================
   Information Mailbox Pointer - 32bit RW 
 ------------------------------------------
    This register holds the start address of 
    the information mailbox located in the ACX100 memory. 
    The host must read this pointer after a reset to find 
    the location of the information mailbox. 
    The ACX100 initializes the information mailbox pointer 
    with the default address of the information mailbox. 
    The information mailbox pointer is not valid 
    until after the host receives the Init Complete interrupt from
    the ACX100.
===============================================*/
#define REG_INFO_MAILBOX_PTR				( SCR_PAD1 ) 


/*=======================================================================
                   Misc
========================================================================*/


/*===============================================
   Phy regs
 ===============================================*/
#define ACX_PHY_ADDR_REG                SBB_ADDR
#define ACX_PHY_DATA_REG                SBB_DATA
#define ACX_PHY_CTRL_REG                SBB_CTL
#define ACX_PHY_REG_WR_MASK             0x00000001ul
#define ACX_PHY_REG_RD_MASK             0x00000002ul    


#define ACX_RX_CFG_REG      RX_CFG


/*===============================================
 EEPROM Read/Write Request 32bit RW 
 ------------------------------------------
 1 EE_READ - EEPROM Read Request 1 - Setting this bit 
   loads a single byte of data into the EE_DATA 
   register from the EEPROM location specified in 
   the EE_ADDR register. 
   The ACX100 hardware clears this bit automatically. 
   EE_DATA is valid when this bit is cleared.
 0 EE_WRITE  - EEPROM Write Request  - Setting this bit 
   writes a single byte of data from the EE_DATA register into the
   EEPROM location specified in the EE_ADDR register. 
   The ACX100 hardware clears this bit automatically.
*===============================================*/
#define ACX_EE_CTL_REG                      EE_CTL
#define EE_WRITE                            0x00000001ul
#define EE_READ                             0x00000002ul

/*===============================================
 EEPROM Address  - 32bit RW 
 ------------------------------------------
 This register specifies the address 
 within the EEPROM from/to which to read/write data.
===============================================*/
#define ACX_EE_ADDR_REG                     EE_ADDR

/*===============================================
 EEPROM Data  - 32bit RW 
 ------------------------------------------
    This register either holds the read 8 bits of 
    data from the EEPROM or the write data 
    to be written to the EEPROM.
===============================================*/
#define ACX_EE_DATA_REG                     EE_DATA

/*===============================================
 EEPROM Base Address  - 32bit RW 
 ------------------------------------------
    This register holds the upper nine bits 
    [23:15] of the 24-bit ACX100 memory 
    address for burst reads from EEPROM accesses. 
    The EEPROM provides the lower 15 bits of this address. 
    The MSB of the address from the EEPROM is ignored.
===============================================*/
#define ACX_EE_CFG                          EE_CFG  

/*===============================================
  GPIO Output Values  -32bit, RW
 ------------------------------------------
    [31:16]  Reserved 
    [15: 0]  Specify the output values (at the output driver inputs) for
             GPIO[15:0], respectively.
===============================================*/
#define ACX_GPIO_OUT_REG            GPIO_OUT
#define ACX_GPIO_OE_REG             GPIO_OE
#define ACX_GPO_CFG                 GPO_CFG
#define ACX_MAX_GPIO_LINES          15

/*===============================================
  Contention window  -32bit, RW
 ------------------------------------------
    [31:26]  Reserved 
    [25:16]  Max (0x3ff)
    [15:07]  Reserved
    [06:00]  Current contention window value - default is 0x1F
===============================================*/
#define ACX_CONT_WIND_CFG_REG    CONT_WIND_CFG
#define ACX_CONT_WIND_MIN_MASK   0x0000007f
#define ACX_CONT_WIND_MAX        0x03ff0000

#endif   /* _WHAL_ACX_REGS_H */

