#include "stdio.h"
#include "string.h"
#include "whalCommon.h"
#include "drvHostBufs.h"

/*
 * --------------------------------------------------------------------------
 *		Data pool simulator
 * --------------------------------------------------------------------------
 */
MSDU_DESC *MsduAlloc(int DataLen)
{
	MSDU_DESC *pNewMsdu;
	struct Host_Buf_Desc *CurrDesc;
	struct Host_Buf_Desc *PrevDesc;
	int NumBlocks;
	int i;

	
	/* Allocate Msdu descriptor */
	pNewMsdu = (MSDU_DESC *)malloc(sizeof(MSDU_DESC));

	/* Allocate Frame header */
	pNewMsdu->pHdrBlock = (char *)malloc(HOST_BUF_SIZE);

	NumBlocks = DataLen/HOST_BUF_SIZE;
	if (DataLen % HOST_BUF_SIZE)
		NumBlocks++;

	pNewMsdu->NumDataBlocks = NumBlocks;
	pNewMsdu->DataBlocksSize = DataLen;
	pNewMsdu->HdrBlockSize = 0;
	pNewMsdu->pHostBufDesc = 0;

	for (i=0; i<NumBlocks; i++)
	{
		/* Allocate buffer descriptor with buffer */
		CurrDesc = (HOST_BUF_DESC *)malloc(sizeof(HOST_BUF_DESC));
		CurrDesc->pData = (char *)malloc(HOST_BUF_SIZE);
		CurrDesc->Offset = 0;
		CurrDesc->Size = HOST_BUF_SIZE;
		CurrDesc->Next = 0;
		if (i == 0)
		{
			pNewMsdu->pHostBufDesc = CurrDesc;
			PrevDesc = CurrDesc;
		}
		else
		{
			PrevDesc->Next = CurrDesc;
		}
		PrevDesc = CurrDesc;
	}

	return pNewMsdu;
}

int MsduFree(MSDU_DESC *pNewMsdu)
{
	if (pNewMsdu->pHostBufDesc != 0)
		BufsFree(pNewMsdu->pHostBufDesc);
	free(pNewMsdu->pHdrBlock);
	free(pNewMsdu);

	return 0;
}

void MsduDataFill(MSDU_DESC *pMsdu)
{
	struct Host_Buf_Desc *CurrDesc;
	UINT16 *pWord;
	int i,j;

	CurrDesc = pMsdu->pHostBufDesc;
	/* 
	 * Fill blocks 
	 */
	for (i=0; i<pMsdu->NumDataBlocks; i++)
	{
#if 0
		CurrDesc->Offset = 0;
		CurrDesc->Size = HOST_BUF_SIZE;
#else
		/* check also the offset (don't use odd offset) */
		CurrDesc->Offset = i << 1;
		CurrDesc->Size = HOST_BUF_SIZE - CurrDesc->Offset;
		pMsdu->DataBlocksSize -= CurrDesc->Offset;
#endif
		pWord = (UINT16 *)CurrDesc->pData;
		/* fill it with block num value for debug */
		for (j=0; j<HOST_BUF_SIZE/2; j++)
		{
			pWord[j] = (i << 8) | (j & 0xff);
		}
		CurrDesc = CurrDesc->Next;
	}
}



HOST_BUF_DESC *BufsAlloc(int DataLen)
{
	struct Host_Buf_Desc *pHead;
	struct Host_Buf_Desc *CurrDesc;
	struct Host_Buf_Desc *PrevDesc;
	int NumBlocks;
	int i;

	NumBlocks = DataLen/HOST_BUF_SIZE;
	if (DataLen % HOST_BUF_SIZE)
		NumBlocks++;

	pHead = 0;

	for (i=0; i<NumBlocks; i++)
	{
		/* Allocate buffer descriptor with buffer */
		CurrDesc = (HOST_BUF_DESC *)malloc(sizeof(HOST_BUF_DESC));
		CurrDesc->pData = (char *)malloc(HOST_BUF_SIZE);
		CurrDesc->Offset = 0;
		CurrDesc->Size = HOST_BUF_SIZE;
		CurrDesc->Next = 0;
		if (i == 0)
		{
			pHead = CurrDesc;
			PrevDesc = CurrDesc;
		}
		else
		{
			PrevDesc->Next = CurrDesc;
		}
		PrevDesc = CurrDesc;
	}

	return pHead;
}

int BufsFree(HOST_BUF_DESC *pBufDesc)
{
	if (pBufDesc == NULL)
		return 0;
	if (pBufDesc->Next != 0)
		BufsFree(pBufDesc->Next);

	free(pBufDesc->pData);
	free(pBufDesc);

	return 0;
}

