/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998-2003 Texas Instruments Incorporated           |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/****************************************************************************
 *
 *   MODULE:  whalUnitTest.c
 *   PURPOSE: Hal unit testing module
 *		whal_testRoot			- Initiate and run the Hal tests
 *		whal_testMain			- Test Rx frames and response to Auth and Ass
 *		whal_testMgmt(Count)	- Send <Count> management frames
 *		whal_testData(Count)	- Send <Count> data frames
 * 
 ****************************************************************************/

#include "stdio.h"
#include "string.h"
#include "memMngr.h"
#include "whalCommon.h"
#include "whalFrameBuilder.h"
#include "whal.h"
#include "wlanSniffer.h"

#if 0

/****************************************************************************
 *                      OBJECT DATA BASE + INTENALS
 ****************************************************************************
 *
 ****************************************************************************/
void	whal_testSend_Auth(UINT8 *DstMac);
void	whal_testSend_AssResp(UINT8 *DstMac);
void	whal_testSend_ProbeResp(UINT8 *DstMac);
int		whal_testFsm(UINT8 *MgmtBuf, UINT32 MgmtBufLen);
int		whal_testMain();
void    print_mac(char *hdr, char *mac);
void	whal_testBridge(MEM_HANDLE MpduHandle, UINT32 DataLen, RxAttr_T *MpduRxAttr);
void	whal_testTxStatus(UINT8 *TxDestMacAddr, UINT8 TxStatus, UINT8 ActualTxRate);
void	whal_testRxData(int Val);
int		Archive_Include();

UINT8   StaMac[6]	= {0x00,0x00,0xde,0xad,0xbe,0xef};
int TestDebug = 0;
int BridgeCount= 0;
int MgmtCount= 0;
int ControlCount= 0;
int OtherCount= 0;
int SendFailCount = 0;
int RxDataInterruptCounter = 0;
int SendFailVec[20];
int SendFailInit = 0;
int SendRetries = 100000;
int Retry;
UINT32 ReadLen;
RxAttr_T RxAttr;


/****************************************************************************
 *                      whal_testRoot()
 ****************************************************************************
 * DESCRIPTION:	root function, Initiate and run the Hal tests
 *
 * INPUTS:	None
 *
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
whal_testRoot()
{
	int Status;

	printf("whal_testRoot: Entry\n");

	Status = wapp_utilsInit();
	Status = wdrv_utilsInit();
	Status = wdrv_dbInit();
	Status = wlan_memMngrInit();

	/*
	 * Initiate and start the AP
	 */
    Status = whal_apiInit(NULL,0);
	if (Status != OK)
	{
		printf("whal_testRoot: ERROR, Fail to initiate whal\n");
		return NOK; 
	}

	whal_apiSetTxStatusCB((TxStatus_Callback_T)whal_testTxStatus);
	whal_apiSetRxDataCB((RxData_Callback_T)whal_testRxData, 1);

    whal_apiStartBss();

	/*
	 * run tests 
	 */
	whal_testMain();
	
	/*
	 * Wait forever
	 */
	while(1)
	{
		rt_osTimerWakeAfter(100);
	}

	/* Overide archive excludes */
	Archive_Include();
}
#ifdef WIN32
main()
{
	whal_testRoot();
}
#endif


int StartHalTest = 0;
int whal_testInit()
{
	if(StartHalTest)
		whal_testRoot();
	return 0;
}

/****************************************************************************
 *                      whal_testMain()
 ****************************************************************************
 * DESCRIPTION:	Main test function
 *		1. Always read incoming frames
 *		2. Enable stations to register, response to Auth and Ass
 *		3. Periodic clean (reclaim) the Rx and the Tx queue 
 *
 * INPUTS:	None
 *
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_testMain()
{
	dot11_mgmtHeader_T *pDot11Header;
	MEM_HANDLE MpduHandle;


	while (1)
	{
#if 0
		rt_osTimerWakeAfter(1);
#endif
		MpduHandle = (MEM_HANDLE *)whal_apiRxRecv(&ReadLen, &RxAttr);
		if (MpduHandle != NULL)
		{
			whal_utilsSniffer( memMgr_BufData(MpduHandle ));
			
			pDot11Header = (dot11_mgmtHeader_T *)(memMgr_BufData(MpduHandle));

			if ((pDot11Header->fc & DOT11_FC_TYPE_MASK) == DOT11_FC_TYPE_MGMT)
			{
				whal_testFsm(memMgr_BufData(MpduHandle), ReadLen);
				MgmtCount++;
			}
			else
			if ((pDot11Header->fc & DOT11_FC_TYPE_MASK) == DOT11_FC_TYPE_DATA)
			{
				whal_testBridge(MpduHandle, ReadLen, &RxAttr);
				BridgeCount++;
			}
			else
			if ((pDot11Header->fc & DOT11_FC_TYPE_MASK) == DOT11_FC_TYPE_CTRL)
			{
				ControlCount++;
			}
			else
				OtherCount++;

		}

		if (MpduHandle != 0)
			wlan_memMngrFreeBD(((mem_BD_T *)MpduHandle)->handle);
		whal_apiRxClean();
		whal_apiTxClean(UcastTxQid);
		whal_apiTxClean(BcastTxQid);
	}

	return 0;
}

void whal_testBridge(MEM_HANDLE MpduHandle, UINT32 MpduLen, RxAttr_T *MpduRxAttr)
{
	UINT8 *DataBuf = memMgr_BufData(MpduHandle);
	dot11_header_T *pDot11Header = (dot11_header_T *)DataBuf;
	mem_MSDU_T *pMsdu;
	TxAttr_T TxAttr;
	TxQueueId_T Qid;
	int HdrSize = sizeof(dot11_mgmtHeader_T);
	int Status;
	int j;
	int Index;

	if (TestDebug)
		printf("whal_testBridge: FrameControl=0x%x\n", pDot11Header->fc);

	/*
	 * 1. Allocate MSDU only for header
	 * 2. Copy the header from the Rx Mpdu and change MAC fileds
	 * 3. Chain the MPDU to the MSDU
	 * --------------------------------
	 * When the fragmentation will use header block as all other blocks
	 * The header will not need to be copy
	 */

	/* 
	 * Allocate MSDU for header only
	 */
	wlan_memMngrAllocMSDU(&pMsdu, HdrSize);

	/* 
	 * Copy and change Mpdu MAC header to the MSDU header
	 */
	memcpy(memMgr_MsduHdrAddr(pMsdu), memMgr_BufData(MpduHandle), HdrSize);
	memMgr_MsduFirstLen(pMsdu) = HdrSize;
	memMgr_MsduHdrLen(pMsdu) = HdrSize;
	whal_frameBuildHdrTxFromRx(memMgr_MsduHdrAddr(pMsdu));

	/* 
	 * skip Mpdu header
	 */
	((mem_BD_T *)MpduHandle)->dataOffset += HdrSize;
	((mem_BD_T *)MpduHandle)->length     -= HdrSize;

	/* 
	 * Msdu point to the Mpdu as its data blocks
	 */
	memMgr_MsduNextAddr(pMsdu) = (mem_BD_T *)MpduHandle;
	memMgr_MsduDataSize(pMsdu) = MpduLen; /* already includes the header size */

	/*
 	 * Check for which queue
	 */
	if (genMacCompare(pDot11Header->address3.addr, whal_ParamsGetBcastMac()) == OK)
		Qid = BcastTxQid;
	else
		Qid = UcastTxQid;
	/* 
	 * check with retries if there is enough place in the ACX100 memory 
	 */
	TxAttr.Rate = MpduRxAttr->Rate;
	for(Retry=0; Retry<SendRetries; Retry++)
	{
		Status = whal_apiTxCheckSize(Qid, MpduLen);
		if (Status == OK)
			break;
	}

#if 1 /* test retries */
	if (!SendFailInit)
	{
		for(j=0; j<11; j++)
			SendFailVec[j] = 0;
		SendFailInit = 1;
	}

	Index = Retry / (SendRetries/10);
	SendFailVec[Index]++;
	if (Retry==SendRetries)
		SendFailCount++;
#endif

	/* 
	 * Send the MSDU 
	 */
	Status = whal_apiSendMsdu(Qid, (MEM_HANDLE)pMsdu, &TxAttr);

	/* Free only Msdu and its header, upper layer will free the data bufs (pMpdu) */
	memMgr_MsduNextAddr(pMsdu) = 0;
	wlan_memMngrFreeMSDU(memMgr_MsduHandle(pMsdu));
}


/****************************************************************************
 *                      whal_testFsm()
 ****************************************************************************
 * DESCRIPTION:	Enable stations to register, response to Auth and Ass
 *
 * INPUTS:	None
 *
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
int whal_testFsm(UINT8 *MgmtBuf, UINT32 MgmtBufLen)
{
	dot11_mgmtHeader_T *pDot11Header = (dot11_mgmtHeader_T *)MgmtBuf;

	if (TestDebug)
		printf("whal_testHandleFrame:FrameControl=0x%x\n", pDot11Header->fc);

	switch (pDot11Header->fc)
	{
		case DOT11_FC_AUTH:
			printf("Recv AUTH, Send response\n");
			whal_testSend_Auth(pDot11Header->SA.addr);
			break;
		case DOT11_FC_ASSOC_REQ:
			printf("Recv ASS_REQ, Send response\n");
			whal_testSend_AssResp(pDot11Header->SA.addr);
			break;
#if 0
		case DOT11_FC_PROBE_REQ:
			printf("Recv PROBE_REQ, Send response\n");
			whal_testSend_ProbeResp(pDot11Header->SA.addr);
			break;
#endif
		default:
			if (TestDebug)
				printf("Frame=0x%x, ignored...\n", pDot11Header->fc);
			break;
	}
	return 0;
}

/****************************************************************************
 *                      whal_testSend_Auth()
 ****************************************************************************
 * DESCRIPTION:	Send Auth frame #2
 *
 * INPUTS:	None
 *
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
void whal_testSend_Auth(UINT8 *DstMac)
{
	mem_MSDU_T *pMsdu;
	TxAttr_T TxAttr;
	UINT32 FrameSize;

	/* 
	 * Allocate MSDU 
	 */
	wlan_memMngrAllocMSDU(&pMsdu, 100/*dummy for one block*/);

	/* 
	 * Fill MSDU, Header+Data in the same host block
	 */
	whal_frameBuildAuth((dot11_mgmtFrame_T *)memMgr_MsduHdrAddr(pMsdu), &FrameSize, DstMac);
	memMgr_MsduFirstLen(pMsdu) = FrameSize;
	memMgr_MsduDataSize(pMsdu) = memMgr_MsduFirstLen(pMsdu);
	memMgr_MsduHdrLen(pMsdu) = 24; /*sizeof(dot11_header_T);*/
	/* 
	 * Send the MSDU 
	 */
	print_mac("whal_testSend_Auth to: ", DstMac);
	TxAttr.Rate = RATE_1;
	whal_apiSendMsdu(UcastTxQid, (MEM_HANDLE)pMsdu, &TxAttr);

	wlan_memMngrFreeMSDU(memMgr_MsduHandle(pMsdu));
}

/****************************************************************************
 *                      whal_testSend_AssResp()
 ****************************************************************************
 * DESCRIPTION:	Send Association response
 *
 * INPUTS:	None
 *
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
void whal_testSend_AssResp(UINT8 *DstMac)
{
	mem_MSDU_T *pMsdu;
	TxAttr_T TxAttr;
	UINT32 FrameSize;

	/* 
	 * Allocate MSDU 
	 */
	wlan_memMngrAllocMSDU(&pMsdu, 100 /* dummy */);

	/* 
	 * Fill MSDU, Header+Data in the same host block
	 */
	whal_frameBuildAssResp((dot11_mgmtFrame_T *)memMgr_MsduHdrAddr(pMsdu), &FrameSize, DstMac);
	memMgr_MsduFirstLen(pMsdu) = FrameSize;
	memMgr_MsduDataSize(pMsdu) = memMgr_MsduFirstLen(pMsdu);
	memMgr_MsduHdrLen(pMsdu) = 24; /*sizeof(dot11_header_T);*/

	/* 
	 * Send the MSDU 
	 */
	print_mac("whal_testSend_AssResp to: ", DstMac);
	TxAttr.Rate = RATE_1;
	whal_apiSendMsdu(UcastTxQid, (MEM_HANDLE)pMsdu, &TxAttr);

	wlan_memMngrFreeMSDU(memMgr_MsduHandle(pMsdu));
}

/****************************************************************************
 *                      whal_testSend_ProbeResp()
 ****************************************************************************
 * DESCRIPTION:	Send Probe response
 *
 * INPUTS:	None
 *
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
void whal_testSend_ProbeResp(UINT8 *DstMac)
{
	mem_MSDU_T *pMsdu;
	TxAttr_T TxAttr;
	UINT32 FrameSize;

	/* 
	 * Allocate MSDU 
	 */
	wlan_memMngrAllocMSDU(&pMsdu, 100 /* dummy */);

	/* 
	 * Fill MSDU, Header+Data in the same host block
	 */
	whal_frameBuildProbeResp(0,(dot11_probeRsp *)memMgr_MsduHdrAddr(pMsdu), &FrameSize, DstMac,whal_ParamsGetElm_Ssid());
	memMgr_MsduFirstLen(pMsdu) = FrameSize;
	memMgr_MsduDataSize(pMsdu) = memMgr_MsduFirstLen(pMsdu);
	memMgr_MsduHdrLen(pMsdu) = 24; /*sizeof(dot11_header_T);*/

	/* 
	 * Send the MSDU 
	 */
	print_mac("whal_testSend_ProbeResp to: ", DstMac);
	TxAttr.Rate = RATE_1;
	whal_apiSendMsdu(UcastTxQid, (MEM_HANDLE)pMsdu, &TxAttr);

	wlan_memMngrFreeMSDU(memMgr_MsduHandle(pMsdu));
}

/****************************************************************************
 *                      whal_testSend_Data()
 ****************************************************************************
 * DESCRIPTION:	Send Data frame
 *
 * INPUTS:	None
 *
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
int whal_testSend_Data(int DataLen, UINT8 *DstMac, int Rate)
{
	mem_MSDU_T *pMsdu;
	mem_BD_T   *pDataBufs;
	UINT32 HdrSize;
	TxAttr_T TxAttr;
	int Status;

	/* 
	 * Allocate MSDU only for data header
	 */
	wlan_memMngrAllocMSDU(&pMsdu, 100 /* dummy */);
	whal_frameBuildDataHeader(memMgr_MsduHdrAddr(pMsdu), &HdrSize, DstMac);
	memMgr_MsduFirstLen(pMsdu) = HdrSize;
	memMgr_MsduHdrLen(pMsdu) = 24; /*sizeof(dot11_header_T);*/

	/* 
	 * Allocate data buffers and fill with data
	 */
	wlan_memMngrAllocDataBuf(&pDataBufs, DataLen);
	memMgr_MsduNextAddr(pMsdu) = pDataBufs;
	memMgr_MsduDataSize(pMsdu) = DataLen+HdrSize;
#if 0
	MsduDataFill(pMsdu);
#endif
	/* 
	 * Send the MSDU 
	 */
	TxAttr.Rate = Rate;
	Status = whal_apiSendMsdu(UcastTxQid, (MEM_HANDLE)pMsdu, &TxAttr);

	wlan_memMngrFreeMSDU(memMgr_MsduHandle(pMsdu));

	return (Status);
}


/****************************************************************************
 *                      TestMgmt()
 ****************************************************************************
 * DESCRIPTION:	Send <Count> management frames
 *
 * INPUTS:	None
 *
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
void whal_testMgmt(int Count)
{
	int i;

	for (i=0; i<Count; i++)
		whal_testSend_AssResp (StaMac);
}

/****************************************************************************
 *                      TestData()
 ****************************************************************************
 * DESCRIPTION:	Send <Count> data frames
 *
 * INPUTS:	None
 *
 * OUTPUT:	None
 * 
 * RETURNS:	
 ****************************************************************************/
void whal_testData(int Count, int Size, int Rate)
{
	int StartTick, EndTick, TimeMsec;
	int Status;
	int RealCount=0;

	StartTick = rt_osTimerGetTick();
	while(1)
	{
		Status = whal_testSend_Data (Size, StaMac, Rate);
		if (Status == OK)
			RealCount++;

		if (RealCount == Count)
			break;
	}
	EndTick = rt_osTimerGetTick();
	TimeMsec = (EndTick - StartTick) * 1000 / sysClkRateGet();
	if (TimeMsec==0) TimeMsec=1;

	printf("whal_testData: Rate=%d Packet/Sec (Count=%d), Tmsec=%d\n", RealCount*1000/TimeMsec, Count, TimeMsec);
}

void whal_testTxStatus(UINT8 *TxDestMacAddr, UINT8 ActualTxRate, UINT8 TxStatus)
{
}

void whal_testRxData(int Val)
{
	RxDataInterruptCounter++;
}

void print_mac(char *hdr, char *mac)
{
	int i;
	printf("%s:", hdr);
	for (i=0; i<6; i++)
		printf("%02x", mac[i]);
	printf("\n");
}


void Test_Print()
{
	printf("Counters: Mgmt=%d, Data=%d, Ctrl=%d, Other=%d\n"
		, MgmtCount, BridgeCount, ControlCount, OtherCount);
	printf("Counters: SendFail=%d, RxDataInterruptCounter=%d\n"
		, MgmtCount, RxDataInterruptCounter);
}

char TestBuf[1024];

void Test_Mem(int Dest, int Src, int Count, int BlockSize)
{
	int StartTick, EndTick, TimeMsec, i;

	StartTick = rt_osTimerGetTick();
	for (i=0; i<Count; i++)
	{
		whal_acxMemCopy(Dest, Src , BlockSize);
	}
	EndTick = rt_osTimerGetTick();
	TimeMsec = (EndTick - StartTick) * 1000 / sysClkRateGet();

	printf("Test_Mem: Rate=%d Pkt/Sec, PktSize=%d\n", Count*1000/TimeMsec, BlockSize);
}

int TestSem()
{
	rt_os_sem_t TestSemId;
	UINT32 StartTime;
	UINT32 EndTime;



	if(rt_osSemCreate("MBOX_ENTR", 1, OS_SEM_FIFO, &TestSemId) != OK)
    {
		printf("TestSem: FATAL error creating MBOX_ENTR semaphore\n");
        return NOK;
    } 
	
	StartTime = _GetMipsCount();
	rt_osTimerWakeAfter(1);
	EndTime = _GetMipsCount();
	printf("Test Time 1 (%12u-%12u=%12u)=%12u [nsec]\n", EndTime, StartTime, EndTime-StartTime, (EndTime-StartTime)*16);

	StartTime = _GetMipsCount();
	rt_osTimerWakeAfter(2);
	EndTime = _GetMipsCount();
	printf("Test Time 2 (%12u-%12u=%12u)=%12u [nsec]\n", EndTime, StartTime, EndTime-StartTime, (EndTime-StartTime)*16);

	StartTime = _GetMipsCount();
	rt_osTimerWakeAfter(3);
	EndTime = _GetMipsCount();
	printf("Test Time 3 (%12u-%12u=%12u)=%12u [nsec]\n", EndTime, StartTime, EndTime-StartTime, (EndTime-StartTime)*16);

	StartTime = _GetMipsCount();
	EndTime = _GetMipsCount();
	printf("Test Time 1 (%12u-%12u=%12u)=%12u [nsec]\n", EndTime, StartTime, EndTime-StartTime, (EndTime-StartTime)*16);

	StartTime = _GetMipsCount();
	rt_osSemTake(TestSemId);
	EndTime = _GetMipsCount();
	printf("Test Time 1 (%12u-%12u=%12u)=%12u [nsec]\n", EndTime, StartTime, EndTime-StartTime, (EndTime-StartTime)*16);

	StartTime = _GetMipsCount();
	rt_osSemGive(TestSemId);
	EndTime = _GetMipsCount();
	printf("Test Time 1 (%12u-%12u=%12u)=%12u [nsec]\n", EndTime, StartTime, EndTime-StartTime, (EndTime-StartTime)*16);

}
#if 0
void MpduDataFill(mem_BD_T *pMpdu)
{
	mem_BD_T   *CurrDesc;
	UINT16 *pWord;
	int i,j;

	CurrDesc = pMpdu;
	/* 
	 * Fill blocks 
	 */
	for (i=0, CurrDesc=pMpdu; CurrDesc != NULL; CurrDesc = memMgr_MsduNextAddr(CurrDesc), i++)
	{
#if 0	/* use it only if the DataLen will be determined by this function */

		/* check also the offset (don't use odd offset) */
		memMgr_BufOffset(CurrDesc) = i << 1;
		memMgr_BufLength(CurrDesc) = 2048 - memMgr_BufOffset(CurrDesc);
#endif
		pWord = (UINT16 *)memMgr_BufData(CurrDesc);
		/* fill it with block num value for debug */
		for (j=0; j<memMgr_BufLength(CurrDesc)/2; j++)
		{
			pWord[j] = (i << 8) | (j & 0xff);
		}
	}
}
#endif
int Archive_Include()
{
	whal_testRoot();
	Test_Print();
	whal_testData(1, 100, 10);
	whal_testMgmt(1);
	whal_txBufs_Print();
	whal_txQueue_Print(1);
	whal_rxBufs_Print();
	whal_rxQueue_Print(1);
	whal_acxIntrPrint();
	whal_acxMboxPrint();
	Test_Mem(1, 1, 0, 10);
	return 0;
}


Test_Single_Write(UINT32 Addr)
{
	register volatile unsigned short *shortDest = (volatile unsigned short *)Addr;
	UINT32 StartTime;
	UINT32 EndTime;
	StartTime = _GetMipsCount();
	*shortDest = 0x1234;
	EndTime = _GetMipsCount();
	printf("Test Time 1 (%12u-%12u=%12u)=%12u [nsec]\n", EndTime, StartTime, EndTime-StartTime, (EndTime-StartTime)*16);
}

Test_Single_Read(UINT32 Addr)
{
	register volatile unsigned short *shortDest = (volatile unsigned short *)Addr;
	register volatile unsigned short Val;
	UINT32 StartTime;
	UINT32 EndTime;

	StartTime = _GetMipsCount();
	Val = *shortDest;
	EndTime = _GetMipsCount();
	printf("Test Time 1 (%12u-%12u=%12u)=%12u [nsec]\n", EndTime, StartTime, EndTime-StartTime, (EndTime-StartTime)*16);
}

Test_Single()
{
	UINT32 StartTime;
	UINT32 EndTime;

	StartTime = _GetMipsCount();
	EndTime = _GetMipsCount();
	printf("Test Time 1 (%12u-%12u=%12u)=%12u [nsec]\n", EndTime, StartTime, EndTime-StartTime, (EndTime-StartTime)*16);
}

#endif