#ifndef WLAN_SNIFFER_H
#define WLAN_SNIFFER_H
#include "pform_types.h"
#include "wlanSniffer.h"
#include "802_11Defs.h"


#define swap16(x) ( ((x&0xff00)>>8) | ((x & 0x00ff)<<8) )  

void parseAssocReqBudy( UINT8 *budyPtr );
void parseAssocRespBudy( UINT8 *budyPtr );
void parseReAssocReqBudy( UINT8 *budyPtr );
void printStringElement(UINT8 **elmPtr);
void printHexElement(UINT8 **elmPtr);
void printFrameConrolOther( UINT16 FrameControl );
void printCapability( UINT16 cabability );
void  printAddresses(dot11_header_T * pDot11Hdr, UINT16 FrameControl );


int snifferOn = 1;


/****************************************************************************
 *                      whal_rxQueueFilter()
 ****************************************************************************
 * DESCRIPTION:	Filter the frame if needed
 * 
 * INPUTS:	
 *		RxQueueId_T Qid		The Id of the Queue
 *		RxDesc_T *pRxDesc	The Rx descriptor address to be filter
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void whal_utilsSniffer(char *pPacket)
{

   dot11_header_T *pDot11Hdr;
   UINT16 FrameControl;
   UINT8  *budyPtr; 


   if( snifferOn == 0 )
      return;
   
   PAL_memCopy( &FrameControl, pPacket, 2 );

	if ((FrameControl & DOT11_FC_TYPE_MASK) == DOT11_FC_TYPE_MGMT)
	{
      printf("--------------------- MGMT ------------------------------\n");
      /* Managment frame */
      switch( FrameControl & DOT11_FC_TYPESUBTYPE_MASK){
  /* Management subtypes */
      case DOT11_FC_SUB_ASSOC_REQ :
         printf("ASSOC_REQ");
         break;
      case DOT11_FC_SUB_ASSOC_RESP :
         printf("ASSOC_RESP");
         break;
      case DOT11_FC_SUB_REASSOC_REQ :
         printf("REASSOC_REQ");
         break;
      case DOT11_FC_SUB_REASSOC_RESP :
         printf("REASSOC_RESP");
         break;
      case DOT11_FC_SUB_PROBE_REQ :
         printf("PROBE_REQ");
         break;
      case DOT11_FC_SUB_PROBE_RESP :
         printf("PROBE_RESP");
         break;
      case DOT11_FC_SUB_BEACON :
         printf("BEACON");
         break;
      case DOT11_FC_SUB_ATIM :
         printf("ATIM");
         break;
      case DOT11_FC_SUB_DISASSOC :
         printf("DISASSOCIATION");
         break;
      case DOT11_FC_SUB_AUTH :
         printf("AUTHENTICATION");
         break;
      case DOT11_FC_SUB_DEAUTH :
         printf("DEAUTHICATION");
         break;
      }

      printf("\n");
      printFrameConrolOther( FrameControl );
      printf("\n");

      pDot11Hdr = (dot11_header_T*) pPacket;

      printAddresses( pDot11Hdr, FrameControl );


      printf("Frag#=%d  Seq#=%d\n", 
         swap16(pDot11Hdr->seqCtrl) & DOT11_SC_FRAG_NUM_MASK ,
        (swap16(pDot11Hdr->seqCtrl) & DOT11_SC_SEQ_NUM_MASK) >> 4 );


      budyPtr = pPacket + sizeof( dot11_mgmtHeader_T );
      
      switch( FrameControl & DOT11_FC_TYPESUBTYPE_MASK){
      case DOT11_FC_SUB_ASSOC_REQ :
         parseAssocReqBudy( budyPtr );
         break;

      case DOT11_FC_SUB_ASSOC_RESP :
         parseAssocRespBudy( budyPtr );
         break;

      case DOT11_FC_SUB_REASSOC_REQ :
         parseReAssocReqBudy( budyPtr );
         break;

      case DOT11_FC_SUB_REASSOC_RESP :
         parseAssocRespBudy(  budyPtr );
         break;
#if 0
      case DOT11_FC_SUB_PROBE_REQ :
         printf("PROBE_REQ");
         break;
      case DOT11_FC_SUB_PROBE_RESP :
         printf("PROBE_RESP");
         break;
      case DOT11_FC_SUB_BEACON :
         printf("BEACON");
         break;
      case DOT11_FC_SUB_ATIM :
         printf("ATIM");
         break;
      case DOT11_FC_SUB_DISASSOC :
         printf("DISASSOCIATION");
         break;
      case DOT11_FC_SUB_AUTH :
         printf("AUTHENTICATION");
         break;
      case DOT11_FC_SUB_DEAUTH :
         printf("DEAUTHICATION");
         break;
#endif
      }



	}

	else if ((FrameControl & DOT11_FC_TYPE_MASK) == DOT11_FC_TYPE_CTRL)
	{
      printf("--------------------- CONTROL ------------------------------\n");
      switch( FrameControl & DOT11_FC_TYPESUBTYPE_MASK){ 
      case DOT11_FC_RTS :
         printf("Request To Send");
         break;
      case DOT11_FC_CTS :
         printf("Clear To Send");
         break;
      case DOT11_FC_ACK :
         printf("ACK");
         break;
      case DOT11_FC_CF_END :
         printf("CF_END");
         break;
      case DOT11_FC_CF_END_CF_ACK :
         printf("CF_END_CF_ACK");
         break;
      }

      printf("\n");
      printFrameConrolOther( FrameControl );

      pDot11Hdr = (dot11_header_T*) pPacket;

      printAddresses( pDot11Hdr, FrameControl );

      printf("Frag#=%d  Seq#=%d\n", 
         swap16(pDot11Hdr->seqCtrl) & DOT11_SC_FRAG_NUM_MASK ,
        (swap16(pDot11Hdr->seqCtrl) & DOT11_SC_SEQ_NUM_MASK) >> 4 ); 
   
   }
   
   else if((FrameControl & DOT11_FC_TYPE_MASK) == DOT11_FC_TYPE_DATA)
   {

      printf("--------------------- DATA ------------------------------\n");

      switch( FrameControl & DOT11_FC_TYPESUBTYPE_MASK){ 
      case DOT11_FC_DATA :
         printf("DATA");
         break;
      case DOT11_FC_DATA_CF_ACK :
         printf("DATA + CF_ACK");
         break;
      case DOT11_FC_DATA_CF_POLL :
         printf("DATA + CF_POLL");
         break;
      case DOT11_FC_DATA_CF_ACK_CF_POLL :
         printf("DATA + CF_ACK + CF_POLL");
         break;
      case DOT11_FC_DATA_NULL_FUNCTION :
         printf("DATA_NULL_FUNCTION");
         break;
      case DOT11_FC_CF_ACK :
         printf("CF_ACK");
         break;
      case DOT11_FC_CF_POLL :
         printf("CF_POLL");
         break;
      case DOT11_FC_CF_ACK_CF_POLL :
         printf("CF_ACK + CF_POLL");
         break;
      }

      printf("\n");
      printFrameConrolOther( FrameControl );
      pDot11Hdr = (dot11_header_T*) pPacket;

      printAddresses( pDot11Hdr, FrameControl );

      printf("Frag#=%d  Seq#=%d\n", 
         swap16(pDot11Hdr->seqCtrl) & DOT11_SC_FRAG_NUM_MASK ,
        (swap16(pDot11Hdr->seqCtrl) & DOT11_SC_SEQ_NUM_MASK) >> 4 );

   }

   else {
      printf("Sniffer: Currupted packet");
   }

   printf("------------------------------------------------------\n");

	return;
}


/****************************************************************************
 *                      printFrameConrolOther()
 ****************************************************************************
 * DESCRIPTION:	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void printFrameConrolOther( UINT16 FrameControl )
{

      printf("TO_DS  FROM_DS  MORE_FRAG  RETRY  PWR_MGMT  MORE_DATA  WEP  ORDER\n");  
      if( FrameControl & DOT11_FC_TO_DS ) printf("  1    ");
      else printf("  0    ");
      
      if( FrameControl & DOT11_FC_FROM_DS ) printf("   1    ");
      else printf("   0    ");
      
      if( FrameControl & DOT11_FC_MORE_FRAG ) printf("    1      ");
      else printf("    0      ");
      
      if( FrameControl & DOT11_FC_RETRY ) printf("  1     ");
      else printf("   0     ");
      
      if( FrameControl & DOT11_FC_PWR_MGMT ) printf("    1     ");
      else printf("    0     ");

      if( FrameControl & DOT11_FC_MORE_DATA ) printf("   1      ");
      else printf("   0      ");

      if( FrameControl & DOT11_FC_WEP ) printf(" 1   ");
      else printf(" 0   ");
      
      if( FrameControl & DOT11_FC_ORDER ) printf("  1    ");
      else printf("  0    ");

      printf("\n");
}


/****************************************************************************
 *                      parseAssocReqBudy()
 ****************************************************************************
 * DESCRIPTION:	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void parseAssocReqBudy( UINT8 *budyPtr )
{
   UINT16 Capability, listenInterval;
   
   
   PAL_memCopy( &Capability, budyPtr, 2 );
   printCapability( Capability );
   budyPtr+=2;
   
   PAL_memCopy( &listenInterval, budyPtr, 2 );
   printf("listenInterval = %d\n", listenInterval);
   budyPtr+=2;
   
   printf("SSID = ");
   printStringElement( &budyPtr );
   
   printf("Supported Rates = ");
   printHexElement( &budyPtr );
}


/****************************************************************************
 *                      parseAssocReqBudy()
 ****************************************************************************
 * DESCRIPTION:	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void parseAssocRespBudy( UINT8 *budyPtr )
{
 
   UINT16 *pField, Capability;
   
   pField = (UINT16*) budyPtr;

   PAL_memCopy( &Capability, budyPtr, 2 );
   printCapability( Capability );
   budyPtr+=2;
   pField++;

   printf("STATUS =  0x%02X\n", *pField );
   budyPtr+=2;
   pField++;
   
   printf("ASSOCIATION ID = %d\n", *pField );
   budyPtr+=2;

   printf("Supported Rates = ");
   printHexElement( &budyPtr );

}

/****************************************************************************
 *                      parseReAssocRespBudy()
 ****************************************************************************
 * DESCRIPTION:	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void parseReAssocReqBudy( UINT8 *budyPtr )
{

   UINT16 *pField;

   pField = (UINT16*) budyPtr;
   
   printCapability( *pField );
   budyPtr+=2;
   pField++;

   printf("listenInterval = %d\n", *pField);
   budyPtr+=2;
   pField++;

   printf("Current AP Address=%02X-%02X-%02X-%02X-%02X-%02X\n",
         budyPtr[0],budyPtr[1],budyPtr[2],budyPtr[3],
         budyPtr[4],budyPtr[5] );
   budyPtr+=6;
      
   printf("SSID = ");
   printStringElement( &budyPtr );
   
   printf("Supported Rates = ");
   printHexElement( &budyPtr );

}


/****************************************************************************
 *                      parseAssocReqBudy()
 ****************************************************************************
 * DESCRIPTION:	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void printHexElement(UINT8 **budyPtr)
{
   UINT8 hdrId, hdrLen;
   int i;
   
   hdrId = **budyPtr;
   (*budyPtr)++;
   hdrLen = **budyPtr;
   (*budyPtr)++;
   
   for( i=0; i < hdrLen-1 ; i++, (*budyPtr)++)
      printf("%02X,", *(*budyPtr));
   printf("%02X\n", *(*budyPtr));
}


/****************************************************************************
 *                      parseAssocReqBudy()
 ****************************************************************************
 * DESCRIPTION:	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void printStringElement(UINT8 **budyPtr)
{
   UINT8 printArray[ 256 ];
   UINT8 hdrId, hdrLen;
   
   hdrId = **budyPtr;
   (*budyPtr)++;
   hdrLen = **budyPtr;
   (*budyPtr)++;

   PAL_memCopy( printArray, (*budyPtr), hdrLen );
   (*budyPtr)+= hdrLen;
   printArray[hdrLen] = 0;
   printf("%s\n", printArray );

}

/****************************************************************************
 *                      parseAssocReqBudy()
 ****************************************************************************
 * DESCRIPTION:	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void printCapability( UINT16 cabability )
{
      printf("CAPABILITY : ESS  IESS  POLLABLE  POLL-REQ  PRIVACY\n");
      printf("             ");
      if( cabability & DOT11_CAPABILITY_ESS ) printf(" 1   ");
      else printf(" 0   ");
      
      if( cabability & DOT11_CAPABILITY_IESS ) printf("  1   ");
      else printf("  0   ");
      
      if( cabability & DOT11_CAPABILITY_CF_POLLABE ) printf("   1      ");
      else printf("   0      ");
      
      if( cabability & DOT11_CAPABILITY_CF_POLL_REQ ) printf("   1      ");
      else printf("   0      ");
      
      if( cabability & DOT11_CAPABILITY_PRIVACY ) printf("   1");
      else printf("   0");

      printf("\n");   
}


/****************************************************************************
 *                      parseAssocReqBudy()
 ****************************************************************************
 * DESCRIPTION:	
 * 
 * INPUTS:	
 *		
 *		
 * 
 * OUTPUT:	None
 * 
 * RETURNS:	OK or NOK
 ****************************************************************************/
void  printAddresses(dot11_header_T * pDot11Hdr, UINT16 FrameControl )
{
   UINT8 mac[6] = {0};
   UINT8 *DA=mac,*SA=mac,*BSSID=mac;

   if(FrameControl & DOT11_FC_TO_DS){ 
      if(FrameControl & DOT11_FC_FROM_DS ){
            /* TO-DS = 1, FROM_DS=1 Not Supported */
      }
      else {
         /* TO-DS = 1, FROM_DS=0 .*/
         BSSID = (UINT8*)&pDot11Hdr->address1;
         SA    = (UINT8*)&pDot11Hdr->address2;
         DA    = (UINT8*)&pDot11Hdr->address3;
      }
   }
   else {
      if(FrameControl & DOT11_FC_FROM_DS ){
         /* TO-DS = 0, FROM_DS=1 .*/
         DA    = (UINT8*)&pDot11Hdr->address1;
         BSSID = (UINT8*)&pDot11Hdr->address2;
         SA    = (UINT8*)&pDot11Hdr->address3;
      }
      else {
         /* TO-DS = 0, FROM_DS=0 .*/
         DA = (UINT8*)&pDot11Hdr->address1;
         SA = (UINT8*)&pDot11Hdr->address2;
         BSSID = (UINT8*)&pDot11Hdr->address3;
      }
   }

      printf("Dest=%02X-%02X-%02X-%02X-%02X-%02X\n",
         DA[0],DA[1],DA[2],DA[3],
         DA[4],DA[5] );

      printf("Src=%02X-%02X-%02X-%02X-%02X-%02X\n",
         SA[0],SA[1],SA[2],SA[3],
         SA[4],SA[5] );

      printf("BSSID=%02X-%02X-%02X-%02X-%02X-%02X\n",
         BSSID[0],BSSID[1],BSSID[2],BSSID[3],
         BSSID[4],BSSID[5] );

}
#endif