#ifndef __TIWLAN_DDA_H
#define __TIWLAN_DDA_H

/*#include <linux/modversions.h>*/
#include <linux/module.h>
#include <linux/types.h>
#include <linux/init.h>
/*#include <linux/mm.h>*/
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <linux/mm.h>
#include <linux/slab.h>
/*#include <linux/string.h>
#include <linux/ethtool.h>
#include <linux/inetdevice.h>
#include <linux/bitops.h>*/
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
#include <linux/tqueue.h>
#else
#include <linux/workqueue.h>
#endif
#include <linux/if.h>
#include <asm-mips/addrspace.h>
#include "drvDbDefs.h"
#include "memMngr.h"

#include "whalAcxRegs.h"


#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
typedef void irqreturn_t;
#define IRQ_NONE
#define IRQ_HANDLED
#define IRQ_RETVAL(x)
#endif

#ifndef vbase1
# define vbase1 TNETD53XX_PUMA_VLBASE_1
#else
# error "name clashing, vbase1 is defined"
#endif

/* we want dynamic name */
/*TODO: pass the dynamic name to the user*/
#define DDA_DEVNAME	"wlan%d"

#define DDA_DRV_RECEIVED_GO (1)
#define DDA_DRV_STARTED	(1<<2)

struct dda_priv {
	struct net_device_stats stats;
	__u32 wpa_authenticator_pid, driver_started;
	struct sock * user_nl;
};


/* wireless extensions staff */
#define DDA_IOCTL_PARAM (SIOCIWFIRSTPRIV + 0)
#define DDA_IOCTL_GET_PARAM (SIOCIWFIRSTPRIV + 1)

enum {
	DDA_PARAM_PTYPE = 1,
};

typedef enum {
	TIWLAN_DDA_RX_EVENT_SEND_FROM_DDA,
	TIWLAN_DDA_RX_EVENT_SEND_FROM_DDC
} tiwlan_ddaRxEvent_t;

void tiwlan_ddaDeleteMacAddrFunc(char *mac);
UINT8 tiwlan_ddaRand(void);
void tiwlan_ddaSrand(UINT16 seed);

STATUS tiwlan_ddaReceive(mem_MSDU_T *);

#define TIWLAN_DDA_BUFFER_FROM_TOKEN(token) (((struct sk_buff *)token)->data)  /* Lior */
STATUS tiwlan_ddaRxAllocBufferIfNeeded(UINT32 len, UINT32 * dataBufToken, UINT8 ** buf);

STATUS tiwlan_ddaNetLinkSend(void * data, int data_len);

STATUS tiwlan_ddaCreateWpaNL(void);

STATUS tiwlan_ddaGo(initConfigStruct *init_cfg);

STATUS tiwlan_dda_wpaGo(void);

STATUS tiwlan_dda_waitForWpa(void);

STATUS tiwlan_ddaInitInterrupts(void);

STATUS tiwlan_ddaSuspend(void);
STATUS tiwlan_ddaResume(void);

STATUS tiwlan_ddaInitPSPVlynq(UINT32 * RegAddr, UINT32 * MemAddr);
STATUS tiwlan_ddaUnloadPSPVlynq(void);

#define tiwlan_ddaRxFreeBuffer(x) \
do{ \
	if (!(((struct sk_buff *)x)->list)) dev_kfree_skb((struct sk_buff *)x);\
	else printk("kfree_skb warning: the skb is on the list !!!\n");\
} while (0);

#define boardInitAcx111Addr(RegAddr, MemAddr) tiwlan_ddaInitPSPVlynq(RegAddr, MemAddr)
#define boardUnloadAcx111() tiwlan_ddaUnloadPSPVlynq()

STATUS wdrv_DDA_queue_stop(void);
STATUS wdrv_DDA_queue_start(void);

#ifdef TIWLAN_DEBUG
void inline bm_trace(int loc, unsigned long p1, unsigned long p2);
#else
#define bm_init()
#define bm_trace(a, b, c) 
#define bm_shutdown()
#define bm_disable()
#endif




#ifdef DDA_WLAN_LED

#define TIAP_LED_OFF		0
#define TIAP_LED_ON		1
#define TIAP_LED_BLINK_OFF	2
#define TIAP_LED_BLINK_ON	3
#define TIAP_LED_FLASH		4

#if !defined(CONFIG_MIPS_PUMAS)
UINT8 wdrv_dbGetApBridgeMode(void);
#if defined(CONFIG_MIPS_AR7VWI)

extern atomic_t tiap_led_oper_state, tiap_led_blinked;
extern UINT16 tiap_activity_led_mask;

extern __u8 tiap_gpio_owner;
extern void whal_apiMacWrite(UINT32 MacRegAddr, UINT32 DataVal);

__inline__ static void tiwlan_ddaLedAction(int action)
{
	/* It mustn't be a function call in an inline function like this one.
	 * Performance is crucial.
	 */
	if (wdrv_dbGetApBridgeMode() || !tiap_gpio_owner)
		return;

	switch(action)
	{
		case TIAP_LED_OFF:
			whal_apiMacWrite(ACX_GPIO_OUT_REG, tiap_activity_led_mask); /*turn it off*/
			atomic_set(&tiap_led_oper_state, action);
			break;
		case TIAP_LED_ON:
			whal_apiMacWrite(ACX_GPIO_OUT_REG, ~tiap_activity_led_mask); /*turn it on*/
			atomic_set(&tiap_led_oper_state, action);
			break;
		case TIAP_LED_BLINK_OFF:
		case TIAP_LED_BLINK_ON:
			atomic_set(&tiap_led_blinked, 1);
			break;
	}
}

#define PAL_ConfigVWiLeds whal_acxConfigVWiLeds

#else
extern void *wlan_led;
#define PAL_ConfigVWiLeds()
#define tiwlan_ddaLedAction(action)	\
	if (!wdrv_dbGetApBridgeMode())	\
		avalanche_led_action(wlan_led, action)
#endif
#else
#define tiwlan_ddaLedAction(action)
#define PAL_ConfigVWiLeds()
#endif
#endif



#endif //__TIWLAN_DDA_H

