/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2003 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*                                                                         */
/*   MODULE:  dda_proc.c                                                */
/*   PURPOSE: implimitation functions for proc fs support for managemnet   */
/*			  appl.													       */
/*                                                                         */
/***************************************************************************/

/***********************************************************
	TODO:
*******************************************************/

/***********************************************************
	includes
*******************************************************/
/*#include <linux/modversions.h>*/
#include <linux/proc_fs.h>
#include <linux/wait.h>
#include <asm/page.h>
#include <linux/compatmac.h>
#include "dda_proc.h"
#include "dda.h"
#include "dda_proc_fs_defs.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "whalApi.h"
#include "mgmtSm.h"
#include "mainsm.h"
#include "whalDefs.h"
#include "bssbridge.h"
#include "dda_contexts.h"
#include "mgmtParser.h"
#include "powerSave.h"
#include "Test/mgmtTst.h"
#include "release.h"



/***********************************************************
	defs
*******************************************************/
typedef int 		(*PROC_READ_FUNC) (char* buf, char **start, off_t offset, int count,int *eof, void *data);
typedef int 		(*PROC_WRITE_FUNC) (struct file *file, const char *buffer,unsigned long count, void *data);

extern UINT8 whal_ParamsGetRadioChannel(UINT8 WoneIndex);

/* # define DDA_PROC_FS_TEST_ONLY */

#define DDA_PROC_ENRTY_NUM    128
#define CMD_NOT_SUPPORT_STR   "cmd not supported."
#define GET_MAJOR_SW_VER 5
#define GET_MINOR_SW_VER 1  

/***********************************************************
	macro DDA_CREATE_PROC
	desc: create proc entry,return NOK case fail
*******************************************************/
#define DDA_CREATE_PROC(name,read_func,write_func,mod,parent,new_file)\
do { \
		if(proc_count > DDA_PROC_ENRTY_NUM -2 )\
		{\
			printk("\n\rdda_proc_fs:error creating %s proc entry",name);\
			return NOK; \
		}\
        if( !(new_file = dda_create_proc_entry(name,read_func,write_func,mod,parent)) )\
		{ \
			printk("\n\rdda_proc_fs:error creating %s proc entry",name);\
			return NOK; \
		}\
} while(0)

#if 1

#define PROC_CHECK_MAGIC() do {\
if (params.s.magic != PROC_PARAMS_MAGIC) { \
printk("You can not access this /proc entry from command line!\n"); \
 return -1; \
} \
} while(0);

#endif


/***********************************************************
	static variables
*******************************************************/


/********************************************************************
	* proc_entries: all the proc_entries will be addedd to proc_entries
	  ,the order will be proc dir -> index x
		 				       proc subdir/file entry "A"-> index x+1
						       proc subdir/file entry "B"-> index x+2
						            ...
							   dir -> index y

  * proc count will hold the number of proc entries created.						   
**********************************************************************/
static struct proc_dir_entry* proc_entries[DDA_PROC_ENRTY_NUM];
static int proc_count;


/***********************************************************
	froc function decleration
*******************************************************/

#ifdef DDA_PROC_FS_TEST_ONLY
void test_dda_procfs(void);
#endif

/******* MAIN funtions *********/

int proc_main_read_ver(char* page, char **start, off_t offset, int count,int *eof, void *data);

/******* BIT functions *********/
int proc_bit_write_fcc(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_bit_write_telec(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_bit_write_txCfg(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_bit_write_rxCfg(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_bit_write_clrStat(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_bit_write_txStop(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_bit_write_bitStop(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_bit_read_rxStat(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_bit_read_txStat(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_bit_read_test(char* page, char **start, off_t offset, int count,int *eof, void *data);

/************* mgmt func *************/
int proc_mgmt_read_staTable(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_mgmt_read_staHash(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_mgmt_read_staRates(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_mgmt_read_4xtable(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_mgmt_write_deAuth(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_mgmt_write_disAss(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_mgmt_read_counters(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_mgmt_read_show(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_mgmt_write_aging(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_mgmt_write_setLog(struct file *file, const char *buffer,unsigned long count, void *data);
/* mgmt test func */
int proc_mgmt_write_null(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_mgmt_write_TstAuth1(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_mgmt_write_TstAuth3(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_mgmt_write_TstAss(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_mgmt_write_TstReass(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_mgmt_write_TstDeAuth(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_mgmt_write_TstDisAss(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_mgmt_write_TstClass3(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_mgmt_write_TstRemStat(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_mgmt_write_TstReset(struct file *file, const char *buffer,unsigned long count, void *data);


/************** wpa func **********/
int proc_wpa_write_TstReset(struct file *file, const char *buffer,unsigned long count, void *data);

/************** privacy func **********/
int proc_priv_read_notify(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_priv_read_cfg(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_priv_read_wepkeys(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_priv_read_counters(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_priv_write_invoke(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_priv_write_exclude(struct file *file, const char *buffer,unsigned long count, void *data);

/***********  serv func*/
int proc_srv_read_rates(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_srv_read_counters(char* page, char **start, off_t offset, int count,int *eof, void *data);
#if 0
int proc_srv_read_wlan_params(char* page, char **start, off_t offset, int count,int *eof, void *data);
#endif
int proc_srv_read_radioFreq(char* page, char **start, off_t offset, int count,int *eof, void *data);  
int proc_srv_read_radioParamas(char* page, char **start, off_t offset, int count,int *eof, void *data);  
int proc_srv_read_power_level(char* page, char **start, off_t offset, int count,int *eof, void *data);  

/************** show func **********/
int proc_show_read_bss(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_show_read_ps(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_show_read_lanMib(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_show_read_lanStat(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_show_read_lanPool(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_show_read_mgmtStat(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_show_read_RA(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_show_read_txStat(char* page, char **start, off_t offset, int count,int *eof, void *data);

/*************  hal func ********/
int proc_whal_read_fwVer(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_read_acxParams(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_read_rxInfo(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_read_txIdnt(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_read_rxQueue(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_read_txBuf(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_read_Intr(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_read_Mbox(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_read_acxStat(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_read_restratStat(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_show_InterfWAparams(char* buf, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_read_vlyncErr(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_read_led(char *, char **, off_t, int, int *, void *);
int proc_whal_write_led(struct file *, const char *, unsigned long, void *);
/* hal diag*/
int proc_whal_read_eed(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_write_eew(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_whal_write_trsh(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_whal_write_BssPs(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_whal_read_Regs(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_write_reg(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_whal_read_reg(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_write_macReg(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_whal_write_rx_promis(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_whal_write_sniffmod(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_whal_read_fwversions(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_write_FwSelc(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_whal_write_QueueSizeType(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_whal_read_radioVers(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_write_SelcRadionum(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_whal_write_NvByte(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_whal_write_traceEn(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_whal_read_traceRxStat(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_whal_read_RxBufs(char* page, char **start, off_t offset, int count,int *eof, void *data);

/*4x*/
int proc_4x_write_max_concatsize(struct file *file, const char *buffer,unsigned long count, void *data);
int proc_4x_show_max_concatsize(char* page, char **start, off_t offset, int count,int *eof, void *data);

/*proc entries for debug purposes */
int proc_debug_read(char* page, char **start, off_t offset, int count,int *eof, void *data);
int proc_debug_write(struct file *file, const char *buffer,unsigned long count, void *data);

/*******************************************************************************
		FUNCTION IMPL.
*******************************************************************************/


/*******************************************************************************
DESCRIPTION: clean all proc entries acording to creation order
			 first we find a directory and then we reomve all its children		   
             the order of the proc entries is:  dir = index x
											    dir child  = x +1
												dir child  = x +2
												    ....
												next dir  = y. 
			according to WLAN tree
INPUTS:	non											

OUTPUT:	non		

RETURNS:non			                                                    
*******************************************************************************/
void	dda_clean_proc(void)
{
	int i,j;
	for(i = proc_count -1;i >= 0 ; i--)
	{
		/* if we have a dir ...*/
		if(S_ISDIR(proc_entries[i]->mode)) 
		{
			printk("removing proc dir %s\n",proc_entries[i]->name);
			/* run on all dir entries*/
			for(j=proc_count - 1;j>=i;j--)
			{
/*				printk("removing proc entry %s ,index %d\n",proc_entries[j]->name,j);*/
				if(j == 0) /* remove base wlan*/
					remove_proc_entry(proc_entries[j]->name,&proc_root);
				else if (j == i) /* remove dir*/
					  remove_proc_entry(proc_entries[j]->name,proc_entries[0]);
				else /* remove regular entry*/
				 	  remove_proc_entry(proc_entries[j]->name,proc_entries[i]);
			}
			proc_count = i;	
		}
	}

	printk("dda proc: finish removing proc entries\n");
}
/*******************************************************************************
DESCRIPTION: create proc entry	   
                                                                         
INPUTS:proc entry name,read,write func,proc mod, and the proc entry parent			

OUTPUT:			

RETURNS: entry pointer or NULL case error.			                                                    
*******************************************************************************/
static struct proc_dir_entry*
dda_create_proc_entry(const char *name,
	 				  PROC_READ_FUNC read_func,
					  PROC_WRITE_FUNC write_func,
					  mode_t mode,
					  struct proc_dir_entry *parent
											) {

	struct proc_dir_entry* new_file = create_proc_entry(name, mode, parent);

	if (new_file) 	{
		new_file->data       = (void *)new_file;
        new_file->read_proc  = read_func;
        new_file->write_proc = write_func;
		new_file->owner = THIS_MODULE;
	}
	else
		return NULL;

	return new_file;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
BOOL IsChannelInsideBandRange(int band, int chID)
{
	BOOL found = FALSE;
	int ch;

	if (band == RADIO_BAND_2_4GHZ)
	{
		if ((chID < 1) || (chID > 14))
			return FALSE;
		return TRUE;
	}
	else
	{
		if ((chID < 36) || (chID > 161))
			return FALSE;
		/* channel is inside the range, let's verify its value */
		for (ch=36; ch<=64;ch++)
			if (ch == chID)
			{
				found=TRUE;
				break;
			}
		if (!found)
			for (ch=100; ch<=140;ch++)
				if (ch == chID)
				{
					found=TRUE;
					break;
				}
		if (!found)
			for (ch=149; ch<=161;ch++)
				if (ch == chID)
				{
					found=TRUE;
					break;
				}
	}
	return found;
}
/*******************************************************************************
DESCRIPTION: init all proc fs dirs/files  supporting wlan driver.	   
           
			 proc WLAN tree =

			   /proc
					wlan 
						bit
						mgmt
						wpa
						privacy
						serv
						show
						4x
						hal
							
			 note: 
			 
			 1.to add proc entry,
			   use DDA_CREATE_PROC and implement read/write func
			  (if u want !).
             
			 2.proc new dir shloud be place under /proc/wlan  !!!.
			 
			 3.when u finish the add,check if total entries count
			   ,include dirs, is under DDA_PROC_ENRTY_NUM,if not so,
			   increase the define.
INPUTS: NON			

OUTPUT: NON			

RETURNS:OK
		NOK			                                                    
*******************************************************************************/
STATUS dda_init_proc_fs(void)
{
	struct proc_dir_entry *wlan_base,*current_dir;
	int i;

	proc_count =0;
	wlan_base = current_dir = NULL;
	
	for(i=0; i < DDA_PROC_ENRTY_NUM;i++)
		proc_entries[i]	= NULL;

	/*  create base wlan proc dir*/
	DDA_CREATE_PROC(WLAN_PROC_BASE_DIR_NAME,NULL,NULL,S_IFDIR,&proc_root,wlan_base);
	proc_entries[proc_count++] = wlan_base;
	

	/* create BIT menu entries*/
	DDA_CREATE_PROC(WLAN_PROC_BIT_DIR_NAME,NULL,NULL,S_IFDIR,wlan_base,current_dir);
	proc_entries[proc_count++] = current_dir;
	DDA_CREATE_PROC(WLAN_PROC_BIT_FCC,NULL,proc_bit_write_fcc,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_BIT_TELEC,NULL,proc_bit_write_telec,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_BIT_TXCFG,NULL,proc_bit_write_txCfg,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_BIT_RXCFG,NULL,proc_bit_write_rxCfg,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_BIT_CLRSTAT,NULL,proc_bit_write_clrStat,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_BIT_TXSTOP,NULL,proc_bit_write_txStop,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_BIT_BITSTOP,NULL,proc_bit_write_bitStop,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_BIT_RXSTAT,proc_bit_read_rxStat,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_BIT_TXSTAT,proc_bit_read_txStat,NULL,0,current_dir,proc_entries[proc_count++]);

#ifdef TIWLAN_DEBUG
	/* test entry, for debug only */
	DDA_CREATE_PROC("test",proc_bit_read_test,NULL,0,current_dir,proc_entries[proc_count++]);
#endif // DEBUG

	/* create mgmt menu entries*/
	DDA_CREATE_PROC(WLAN_PROC_MGMT_DIR_NAME,NULL,NULL,S_IFDIR,wlan_base,current_dir);
	proc_entries[proc_count++] = current_dir;
	DDA_CREATE_PROC(WLAN_PROC_MGMT_DEAUTH,NULL,proc_mgmt_write_deAuth,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_DISASS,NULL,proc_mgmt_write_disAss,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_STATABLE,proc_mgmt_read_staTable,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_4XTABLE,proc_mgmt_read_4xtable,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_STAHASH,proc_mgmt_read_staHash,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_STARATES,proc_mgmt_read_staRates,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_COUNTERS,proc_mgmt_read_counters,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_SETLOG,NULL,proc_mgmt_write_setLog,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_AGING,NULL,proc_mgmt_write_aging,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_SHOW,proc_mgmt_read_show,NULL,0,current_dir,proc_entries[proc_count++]);
	/* create mgmt test menu entries*/
	DDA_CREATE_PROC(WLAN_PROC_MGMT_TST_NULL,NULL,proc_mgmt_write_null,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_TST_AUTH1,NULL,proc_mgmt_write_TstAuth1,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_TST_AUTH3,NULL,proc_mgmt_write_TstAuth3,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_TST_ASS,NULL,proc_mgmt_write_TstAss,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_TST_REASS, NULL,proc_mgmt_write_TstReass,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_TST_DEAUTH,NULL,proc_mgmt_write_TstDeAuth,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_TS_DEASS,NULL,proc_mgmt_write_TstDisAss,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_TST_CLASS3,NULL,proc_mgmt_write_TstClass3,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_TST_REM_STAT,NULL,proc_mgmt_write_TstRemStat,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_MGMT_TST_RESET,NULL,proc_mgmt_write_TstReset,0,current_dir,proc_entries[proc_count++]);

	/* create wpa menu entries*/
	DDA_CREATE_PROC(WLAN_PROC_WPA_DIR_NAME,NULL,NULL,S_IFDIR,wlan_base,current_dir);
	proc_entries[proc_count++] = current_dir;
	DDA_CREATE_PROC(WLAN_PROC_WPA_RESET,NULL,proc_wpa_write_TstReset,0,current_dir,proc_entries[proc_count++]);

	/* create privacy menu entries*/
	DDA_CREATE_PROC(WLAN_PROC_PRIVACY_DIR_NAME,NULL,NULL,S_IFDIR,wlan_base,current_dir);
	proc_entries[proc_count++] = current_dir;
	DDA_CREATE_PROC(WLAN_PROC_PRIVACY_CFG,proc_priv_read_cfg,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_PRIVACY_NOTIFY,proc_priv_read_notify,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_PRIVACY_WEP_KEYS,proc_priv_read_wepkeys,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_PRIVACY_COUNTERS,proc_priv_read_counters,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_PRIVACY_INVOKE,NULL,proc_priv_write_invoke,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_PRIVACY_EXCLUDE,NULL,proc_priv_write_exclude,0,current_dir,proc_entries[proc_count++]);

	/* create serv menu entries*/
	DDA_CREATE_PROC(WLAN_PROC_SERV_DIR_NAME,NULL,NULL,S_IFDIR,wlan_base,current_dir);
	proc_entries[proc_count++] = current_dir;
#if 0
	DDA_CREATE_PROC(WLAN_PROC_SERV_WLAN_PARAMS,proc_srv_read_wlan_params,NULL,0,current_dir,proc_entries[proc_count++]);
#endif
	DDA_CREATE_PROC(WLAN_PROC_SERV_RATES,proc_srv_read_rates,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_SERV_COUNTERS,proc_srv_read_counters,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_SERV_RADIO_FREQ,proc_srv_read_radioFreq,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_SERV_RADIO_PARAMS,proc_srv_read_radioParamas,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_SERV_POWER_LEVEL,proc_srv_read_power_level,NULL,0,current_dir,proc_entries[proc_count++]);

	/* create show menu entries*/
	DDA_CREATE_PROC(WLAN_PROC_SHOW_DIR_NAME,NULL,NULL,S_IFDIR,wlan_base,current_dir);
	proc_entries[proc_count++] = current_dir;
	DDA_CREATE_PROC(WLAN_PROC_SHOW_BSS,proc_show_read_bss,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_SHOW_PS,proc_show_read_ps,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_SHOW_TX_STAT,proc_show_read_txStat,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_SHOW_LAN_MIB,proc_show_read_lanMib,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_SHOW_LAN_STAT,proc_show_read_lanStat,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_SHOW_LAN_POOL,proc_show_read_lanPool,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_SHOW_MEM_STAT,proc_show_read_mgmtStat,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_SHOW_RATE_ADAP,proc_show_read_RA,NULL,0,current_dir,proc_entries[proc_count++]);

	/* create 4x menu entries*/
	DDA_CREATE_PROC(WLAN_PROC_4X_DIRNAME, NULL,NULL,S_IFDIR,wlan_base,current_dir);
	proc_entries[proc_count++] = current_dir;

	DDA_CREATE_PROC(WLAN_PROC_4X_MAX_CONCATSIZE, NULL, proc_4x_write_max_concatsize, 0, current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_SHOW_4X_MAX_CONCATSIZE, proc_4x_show_max_concatsize, NULL, 0, current_dir,proc_entries[proc_count++]);
	/* NO NEED*/

	/* create hal menu entries*/
	DDA_CREATE_PROC(WLAN_PROC_HAL_DIR_NAME,NULL,NULL,S_IFDIR,wlan_base,current_dir);
	proc_entries[proc_count++] = current_dir;
	DDA_CREATE_PROC(WLAN_PROC_HAL_FWVER,proc_whal_read_fwVer,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_ACX_PARAM,proc_whal_read_acxParams,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_RX_INFO,proc_whal_read_rxInfo,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_TX_IDNT,proc_whal_read_txIdnt,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_RX_QUEUE,proc_whal_read_rxQueue,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_TXBUF,proc_whal_read_txBuf,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_INTR,proc_whal_read_Intr,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_MBOX,proc_whal_read_Mbox,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_ACXSTAT,proc_whal_read_acxStat,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_RESTART_STAT,proc_whal_read_restratStat,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_LED,proc_whal_read_led,proc_whal_write_led,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_INTERF_WD_PARAMS,proc_whal_show_InterfWAparams,NULL,0,current_dir,proc_entries[proc_count++]);

#if 0
	DDA_CREATE_PROC(WLAN_PROC_HAL_VLYNC_ERR,proc_whal_read_vlyncErr,NULL,0,current_dir,proc_entries[proc_count++]);
#endif
	/* hal diag*/
	DDA_CREATE_PROC(WLAN_PROC_HAL_EED,proc_whal_read_eed,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_EEW,NULL,proc_whal_write_eew,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_SET_TX_COMPTRSH,NULL,proc_whal_write_trsh,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_SETPS_MODE,NULL,proc_whal_write_BssPs,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_PRINTREGS,proc_whal_read_Regs,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_PHY_REG_WRITE,NULL,proc_whal_write_reg,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_MAC_REG_WRITE,NULL,proc_whal_write_macReg,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_SET_RX_PROMIS,NULL,proc_whal_write_rx_promis,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_MODE_SNIFFER,NULL,proc_whal_write_sniffmod,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_FW_VERS,proc_whal_read_fwversions,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_SELEC_FW_NUM,NULL,proc_whal_write_FwSelc,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_SETY_Q_SIZE_TYPE,NULL,proc_whal_write_QueueSizeType,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_RADIO_VER,proc_whal_read_radioVers,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_SET_NVBYTE,NULL,proc_whal_write_NvByte,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_SELEC_RADIO_NUM,NULL,proc_whal_write_SelcRadionum,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_TRACE_ENABLE,NULL,proc_whal_write_traceEn,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_TRACE_RX_STAT,proc_whal_read_traceRxStat,NULL,0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_HAL_RX_BUFFS,proc_whal_read_RxBufs,NULL,0,current_dir,proc_entries[proc_count++]);
	
	/* create production menu entries*/
	DDA_CREATE_PROC(WLAN_PROC_PROD_DIR_NAME,NULL,NULL,S_IFDIR,wlan_base,current_dir);
	proc_entries[proc_count++] = current_dir;
	DDA_CREATE_PROC(WLAN_PROC_PROD_VERSION, proc_main_read_ver, NULL, 0, current_dir,proc_entries[proc_count++]);

    /* create proc entries for debug purposes */
	DDA_CREATE_PROC(WLAN_PROC_DEBUG_DIR_NAME,NULL,NULL,S_IFDIR,wlan_base,current_dir);
	proc_entries[proc_count++] = current_dir;
	DDA_CREATE_PROC(WLAN_PROC_DEBUG_READ_ENTRY,proc_debug_read, NULL, 0,current_dir,proc_entries[proc_count++]);
	DDA_CREATE_PROC(WLAN_PROC_DEBUG_WRITE_ENTRY, NULL, proc_debug_write, 0,current_dir,proc_entries[proc_count++]);


	/* NO NEED*/
#ifdef DDA_PROC_FS_TEST_ONLY
	test_dda_procfs();
#endif
	return OK;
}

/******* MAIN functions *********/

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/

int proc_main_read_ver(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = sprintf(page,"%s, %d/%d/%d", SW_VERSION_STR, SW_RELEASE_MONTH, SW_RELEASE_DAY,SW_RELEASE_YEAR);
	}
	
	*eof = 1;

	return len;
}

/******* BIT functions *********/
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_bit_write_fcc(struct file *file, const char *buffer,unsigned long count, void *data)
{ 

	proc_params in_params;
	int band,chID,rate,mode;

	if( count == 0)
	{	
		printk("proc_bit_write_fcc:receive error params size");
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);

	
	band = in_params.s.Param1; /* band*/
    chID = in_params.s.Param2; /* cahhnel id*/
	rate = in_params.s.Param3; /* rate*/
	mode = in_params.s.Param4; /* mode*/

	if (IsChannelInsideBandRange(band,chID) == FALSE)
		return -1;

	/* OK.. let's do it */
	wdrv_dbSetEnRestart(0, FALSE);	/* Disable Restart WA w/o writing to NV */
	wdrv_mgmtRadioCalibrateStop();	/* Disable radio calibration */

	printf("%s\n",__func__);
	if(whal_apiAcxBIT_Fcc(band, chID, rate, mode) != OK)
		return -1;
	
	return count; 
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_bit_write_telec(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;
	int band,chID;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);
	band = in_params.s.Param1; /* band*/
    chID = in_params.s.Param2; /* cahhnel id*/



   if (band < 0)
	   return -1;
   if(chID <= 0)
	   return -1;
   if (IsChannelInsideBandRange(band,chID) == FALSE)
   {
	   return -1;
   }

   wdrv_dbSetEnRestart(0, FALSE);	/* Disable Restart WA w/o writing to NV */
   wdrv_mgmtRadioCalibrateStop();	/* Disable radio calibration */

   if(whal_apiAcxBIT_Telec(band, chID) != OK)
	   return -1;
   
   return count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_bit_write_txCfg(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;
	int band,chID,rate,mode,preamble,numFrames,gap,bodySize;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);

	band = in_params.s.Param1; /* band*/
    chID = in_params.s.Param2; /* cahhnel id*/
	rate = in_params.s.Param3; /* rate*/;
	mode = in_params.s.Param4; /* mode*/;
	preamble = in_params.s.Param5; /* preamble*/
    numFrames = in_params.s.Param6; /* frame num*/
	gap = in_params.s.Param7; /* gap*/;
	bodySize = in_params.s.Param8; /* frame budy size*/;

	wdrv_dbSetEnRestart(0, FALSE);	/* Disable Restart WA w/o writing to NV */

    if(whal_apiAcxBIT_perTxCfg(band, chID, rate, mode, preamble,
							   numFrames, gap, bodySize, (char*)in_params.s.mac
							  ) != OK)
		return -1;

	return count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_bit_write_rxCfg(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;
	int band,chID;


	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);
	band = in_params.s.Param1; /* band*/
    chID = in_params.s.Param2; /* channel id*/


   if (band < 0)
	   return -1;
   if(chID <= 0)
	   return -1;
   if (IsChannelInsideBandRange(band,chID) == FALSE)
   {
	   return -1;
   }

   wdrv_dbSetEnRestart(0, FALSE);	/* Disable Restart WA w/o writing to NV */

   if(whal_apiAcxBIT_perRxCfg(band, chID) != OK)
	   return -1;
   return count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_bit_write_clrStat(struct file *file, const char *buffer,unsigned long count, void *data)
{
	if(whal_apiAcxBIT_perClrStats() != OK)
		return -1;
	return count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_bit_write_txStop(struct file *file, const char *buffer,unsigned long count, void *data)
{
	if(whal_apiAcxBIT_perTxStop() != OK)
		return -1;
	return count; 
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_bit_write_bitStop(struct file *file, const char *buffer,unsigned long count, void *data)
{
   whal_apiAcxBIT_ExitTest();
   whal_apiAcxReset();

#ifdef VXWORKS
	cm_hwReset(); /* nothing else matters... */
#endif
	return count; 
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_bit_read_rxStat(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = whal_apiAcxBIT_perRxStats(page);
	}

	*eof = 1;
	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_bit_read_txStat(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = whal_apiAcxBIT_perTxStats(page);
	}

	*eof = 1;
	return len;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_bit_read_test(char* page, char **start, off_t offset, int count,int *eof, void *data) {
	char buff[32];
	int len = 0;

	if (offset == 0) {
		tiwlan_ddaNetLinkSend(buff, 32);
		len = sprintf(page, "%s Done !!!\n", __PRETTY_FUNCTION__);
	}
	*eof = 1;

	return len;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_read_staTable(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	static int len = 0;
	static char buf[10000];
	int new_count;

	if (offset == 0) {
		len = wdrv_dbPrintStaTable(buf);
	}

	if (offset >= len)
	{
        *eof = 1;
        return 0;
    }

    new_count =  count + offset > len ? len - offset : count;
   
    memcpy(page, buf + offset, new_count);
    offset += new_count;
   
    *eof = 0;

    *start = page;

    return new_count;

}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_read_staHash(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = wdrv_dbPrintHashTable(page);
	}

	*eof = 1;
	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_read_staRates(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	
	static int len = 0;
	static char buf[10000];
	int new_count;

	if (offset == 0) {
		len = wdrv_dbPrintRatesTable(buf);
	}

	if (offset >= len)
	{
        *eof = 1;
        return 0;
    }

    new_count =  count + offset > len ? len - offset : count;
   
    memcpy(page, buf + offset, new_count);
    offset += new_count;
   
    *eof = 0;

    *start = page;

    return new_count;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/


int proc_mgmt_read_4xtable(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	static int		len = 0;
    static char		buf[20000];
	int 			new_count;

    /*
        TODO: sooner or later, there will be buffer overflow here !!!
    */
    

	if (offset == 0) {
		len = wdrv_dbPrintFourXTable(buf);
	}
    
    if (offset >= len)
	{
        *eof = 1;
        return 0;
    }

    new_count =  count + offset > len ? len - offset : count;
   
    memcpy(page, buf + offset, new_count);
    offset += new_count;
   
    *eof = 0;

    *start = page;

    return new_count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_write_deAuth(struct file *file, const char *buffer,unsigned long count, void *data)
{

	mgmtMessage_T	msg;
	proc_params in_params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);

	msg.type = DEAUTH_MNG;
	PAL_memCopy(msg.content.deAuthMng.macAddress, in_params.s.mac, MAC_ADDR_LEN);
	msg.content.deAuthMng.reason = in_params.s.Param1 /* reason */;
    wdrv_DDA_mgmt_setEvent(&msg);
	
	return count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_write_disAss(struct file *file, const char *buffer,unsigned long count, void *data)
{
	mgmtMessage_T	msg;
	proc_params in_params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);

	msg.type = DISASSOC_MNG;
	PAL_memCopy(msg.content.deAuthMng.macAddress, in_params.s.mac, MAC_ADDR_LEN);
	msg.content.deAuthMng.reason = in_params.s.Param1; /* reason */
    wdrv_DDA_mgmt_setEvent(&msg);
	
	return count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_read_counters(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0){
		len = wdrv_mgmtPrintCounters(page);
	}

	*eof = 1;
	
	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_read_show(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	BOOL	enable4x;
	dot11_TIcap_T	TIcap;
	int len = 0;

	
	if (offset == 0) {
		wdrv_dbGet4xEnable(&enable4x);
		wdrv_dbGetTIInfoElement(TIcap.cap, &(TIcap.hdr.eleLen));		
		
		len+=wdrv_dbPrintNumOfStations(page);
	
		if (wdrv_mgmtLogEnable())
			len += sprintf(page +len,"Management Log:		Enable\n");
		else
			len += sprintf(page +len,"Management Log:		Disable\n");
	
		if (wdrv_mgmtAgingEnable())
			len += sprintf(page +len,"Management Aging:	Enable\n");
		else
			len += sprintf(page +len,"Management Aging:	Disable\n");
	
		if(enable4x == TRUE)
		{
			int i;
			len += sprintf(page +len,"4x:	Enable\n");
			len += sprintf(page +len,"0x%x ", TIcap.hdr.eleId);
			len += sprintf(page +len,"0x%x ", TIcap.hdr.eleLen);
			for(i=0;i < TIcap.hdr.eleLen;i++)
				len += sprintf(page +len,"0x%x ", TIcap.cap[i]);
			len += sprintf(page +len,"\n");
		}
		else
			len += sprintf(page +len,"4x:	Disable\n");

	}
		
	*eof = 1;
	
	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_write_aging(struct file *file, const char *buffer,unsigned long count, void *data)
{
	int flag;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&flag,buffer, count);
	if ((flag!= 0) && (flag != 1))
	{
		return -1;
	}
	wdrv_mgmtSetAging((BOOL)flag);

	return count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_write_setLog(struct file *file, const char *buffer,unsigned long count, void *data)
{
	int flag;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&flag,buffer, count);
	if ((flag!= 0) && (flag != 1))
	{
		return -1;
	}

	wdrv_mgmtSetLog((BOOL)flag);
	return count;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_write_null(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&params,buffer, count);
	wdrv_mgmtTstNull(
					  params.s.Param1 /*fc*/	
					 ,params.s.Param2 /*SA*/
					 ,params.s.Param3 /*da*/
					 ,params.s.Param4 /*bssid*/
					 );

	return count;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_write_TstAuth1(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&params,buffer, count);
	wdrv_mgmtTstAuth1(
					  params.s.Param1 /*SA*/
					 ,params.s.Param2 /*algoNum*/
					 );
	return count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_write_TstAuth3(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&params,buffer, count);
	wdrv_mgmtTstAuth3(
					  params.s.Param1 /* SA */
					 ,(UINT16)params.s.Param2 /* algoNum*/
					 ,params.s.Param3 /* challenge */
					 );
	return count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_write_TstAss(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&params,buffer, count);
	wdrv_mgmtTstAssoc(
					  params.s.Param1		  /* SA */
					 ,(BOOL)params.s.Param2 /* pbcc*/
					 ,(BOOL)params.s.Param3 /* privacy */
					 ,(BOOL)params.s.Param4 /* fourx */
					 );
	return count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_write_TstReass(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&params,buffer, count);
	wdrv_mgmtTstReassoc(
					    params.s.Param1		  /* SA */
					   ,(BOOL)params.s.Param2 /* pbcc*/
					   ,(BOOL)params.s.Param3 /* privacy */
					   ,(BOOL)params.s.Param4 /* fourx */
					   );
	return count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_write_TstDeAuth(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&params,buffer, count);
	wdrv_mgmtTstDeAuth (
					    params.s.Param1		  /* SA */
					   ,(UINT16)params.s.Param2 /* reason*/
					   );
	return count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_write_TstDisAss(struct file *file, const char *buffer,unsigned long count, void *data)
{
	
	proc_params params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&params,buffer, count);
	wdrv_mgmtTstDisAssoc (
					      params.s.Param1		  /* SA */
					     ,(UINT16)params.s.Param2 /* reason*/
					     );
	return count;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_write_TstClass3(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&params,buffer, count);
	wdrv_mgmtTstClass3(
					   params.s.Param1 /* SA */
					  ,(BOOL)params.s.Param2 /* index*/
					  );
	return count;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_write_TstRemStat(struct file *file, const char *buffer,unsigned long count, void *data)
{
	/* call proc_mgmt_write_deAuth (this whats happend in command.c)*/
	return count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_mgmt_write_TstReset(struct file *file, const char *buffer,unsigned long count, void *data)
{
	wdrv_mgmtReset();
	return count;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_wpa_write_TstReset(struct file *file, const char *buffer,unsigned long count, void *data)
{
	return count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_priv_read_notify(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	UINT16	value;
	__u8 macaddr[ETH_ALEN];
	int len=0;

	if (offset == 0) {
	
		wdrv_dbGetStationConfigDisassociateReason(&value);
		wdrv_dbGetStationConfigDisassociateStation(macaddr);
		len +=sprintf(page+len,"Disassociate reason: %d\n", value);
		len +=sprintf(page+len,"Disassociates station: <%02X-%02X-%02X-%02X-%02X-%02X>\n", macaddr[0],macaddr[1],macaddr[2],macaddr[3],
			  macaddr[4],macaddr[5]);
	
		wdrv_dbGetStationConfigDeauthenticateReason(&value);
		wdrv_dbGetStationConfigDeauthenticateStation(macaddr);
		len +=sprintf(page+len,"Deauthenticate reason: %d\n", value);
		len +=sprintf(page+len,"Deauthenticate station: <%02X-%02X-%02X-%02X-%02X-%02X>\n", macaddr[0],macaddr[1],macaddr[2],macaddr[3],
			  macaddr[4],macaddr[5]);
	
		wdrv_dbGetStationConfigDeauthenticateFailStatus(&value);
		wdrv_dbGetStationConfigeauthenticateFailStation((char*)macaddr);
		len +=sprintf(page+len,"Deauthenticate Fail status: %d\n", value);
		len +=sprintf(page+len,"Deauthenticate Fail station: <%02X-%02X-%02X-%02X-%02X-%02X>\n", macaddr[0],macaddr[1],macaddr[2],macaddr[3],
			  macaddr[4],macaddr[5]);
	
	}
	*eof = 1;
	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_priv_read_cfg(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0,value,i;
	UINT32 value32;
	UINT16 value16;
	UINT8  value8;

	if (offset == 0) {
	
		wdrv_dbGetStationConfigPrivacyOptionImplemented(&value);
		if (value == TRUE)
			len +=sprintf(page+len,"Privacy option:		On\n");
		else
			len +=sprintf(page+len,"Privacy option:		Off\n");
	
		wdrv_dbGetPrivacyInvoked(&value);
		if (value == TRUE)
			len +=sprintf(page+len,"Privacy Invoked:	On\n");
		else
			len +=sprintf(page+len,"Privacy Invoked:	Off\n");
	
		wdrv_dbGetPrivacyExcludeUnencrypted(&value);
		if (value == TRUE)
			len +=sprintf(page+len,"Exclude Unencrypted:	On\n");
		else
			len +=sprintf(page+len,"Exclude Unencrypted:	Off\n");
	
		wdrv_dbGetPrivacyWEPDefaultKeyId(&value8);
		len +=sprintf(page+len,"WEP Default Key Id:	%u\n", value8);
	
	
		wdrv_dbGetStationConfigAuthRspTimeout(&value32);
		len +=sprintf(page+len,"Auth response timeout:	%d msecs\n", value32);
		for (i = 0; i < MAX_AUTH_ALGORITHM_SUPPORTED; i++)
		{
			
			wdrv_dbGetAuthAlgo(i, (UINT16 *)&value16);
			if (value16 == MIB_OPEN_SYSTEM)
				len +=sprintf(page+len,"Authentication algorithm: Open system	");
			else if (value16 == MIB_SHARED_KEY)
				len +=sprintf(page+len,"Authentication algorithm: Shared key	");
			else 
				len +=sprintf(page+len,"Authentication algorithm: Unknown	");
	
			wdrv_dbGetAuthAlgoEnable(i, &value);
			if (value == TRUE)
				len +=sprintf(page+len,"Enable\n");
			else
				len +=sprintf(page+len,"Disable\n");
		}
	}
	
	*eof = 1;
	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_priv_read_wepkeys(char* page, char **start, off_t offset, int count,int *eof, void *data)
{

	char key[WEP_KEY_LEN_MAX];
	int i, j,len = 0;
	UINT16 Keylen;

	if (offset == 0) {
	
		for (i = 0; i < DOT11_DEFS_KEY_NAL_LEN; i++)
		{
			wdrv_dbGetDefaultWepKey(i, &Keylen, key);
			len+=sprintf(page+len,"Default key index: %d\n", i);
			len+=sprintf(page+len,"len: %d\n", Keylen);
			len+=sprintf(page+len,"key value: ");
			for (j = 0; j < Keylen; j++)
				len+=sprintf(page+len,"0x%hhX ", key[j]);
			len+=sprintf(page+len,"\n\n");
		}
	}

	*eof = 1;
	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_priv_read_counters(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;
	UINT32 count14, count15, count16, count17;

	if (offset == 0) {
	
		wdrv_dbGetCountersWEPUndecryptableCount(&count14);
		wdrv_dbGetPrivacyWEPICVErrorCount(&count15);
		wdrv_dbGetPrivacyWEPExcludedCount(&count16);
		wdrv_dbGetPrivacyTKIPMicFailure(&count17);
	
		len = sprintf(page,"***Privacy Counters***\n"); 
		len +=sprintf(page+len,"WEP Undecryptable Count				= %d\n", count14); 
		len +=sprintf(page+len,"WEP ICV Error Count				= %d\n", count15); 
		len +=sprintf(page+len,"WEP Excluded Count				= %d\n", count16); 
		len +=sprintf(page+len,"TKIP MIC Failure Count				= %d\n", count17); 
	}

	*eof = 1;
	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_priv_write_invoke(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);

	if ((in_params.s.Param1	!= 0) && (in_params.s.Param1 != 1))
	{
		return -1;
	}
	wdrv_dbSetPrivacyInvoked((BOOL)in_params.s.Param1);

	return count;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_priv_write_exclude(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);

	if ((in_params.s.Param1	!= 0) && (in_params.s.Param1 != 1))
	{
		return -1;
	}
	wdrv_dbSetPrivacyExcludeUnencrypted((BOOL)in_params.s.Param1);

	return count;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_4x_write_max_concatsize(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);

	if ((in_params.s.Param1	< WLAN_MIN_CONCAT_SIZE) || (in_params.s.Param1 > WLAN_MAX_CONCAT_SIZE))
	{
		printf("4x Max Concat size: %d - the value is out of range\n", in_params.s.Param1); 
		return -1;
	}
	wdrv_txSetMaxConcatSize((UINT32)in_params.s.Param1);

	return count;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_4x_show_max_concatsize(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;
	
	if (offset == 0) {
		len = sprintf(page, "Max Concatenation Size    =%8d\n", wdrv_txGetMaxConcatSize());
	}
	
	*eof = 1;
	
	return len;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_srv_read_rates(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	int i, len= 0;

	if (offset == 0)
	{
	
		char rates[CCK_PBCC_RATES_MAX+OFDM_RATES_MAX];
		UINT8 ratesLen;
		char extRates[OFDM_RATES_MAX];
		UINT8 extRatesLen;
		char allRates[CCK_PBCC_RATES_MAX+OFDM_RATES_MAX];
		UINT8 allRatesLen;
	
		UINT8 wone_index;
		UINT8  num_sup_wone;
		wdrv_dbGetNumSupWone(&num_sup_wone);
		for (wone_index = 0 ; wone_index < num_sup_wone; wone_index ++)
		{
			wdrv_dbGetStationConfigOperationalRateSet(wone_index, rates, &ratesLen);
			wdrv_dbGetStationConfigOperationalOFDMRateSet(wone_index, extRates, &extRatesLen);
			wdrv_dbGetStationConfigOperationalAllRateSet(wone_index, allRates, &allRatesLen);
	
			len +=sprintf(page +len , "\n"); 
			len +=sprintf(page +len ,"Wone Index %d\n",wone_index); 
	
			len +=sprintf(page +len ,"Rates in Legacy IE =    "); 
			for (i = 0; i < ratesLen; i++)
			{
				len +=sprintf(page +len ,"  "); 
				len += wdrv_printRateInMbpsUnit(rates[i], page + len);
			}
			len +=sprintf(page +len ,"\n\n"); 
			len +=sprintf(page +len ,"Rates in Extended IE =    "); 
			for (i = 0; i < extRatesLen; i++)
			{
				len +=sprintf(page +len ,"  "); 
			len += wdrv_printRateInMbpsUnit(extRates[i], page + len);
			}
			len +=sprintf(page +len ,"\n\n"); 
			len +=sprintf(page +len ,"All Rates =    "); 
			for (i = 0; i < allRatesLen; i++)
			{
				len += sprintf(page+len,"  "); 
				len += wdrv_printRateInMbpsUnit(allRates[i], page + len);
			}
			len += sprintf(page+len,"\n"); 
		}
	}

	*eof = 1;
	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_srv_read_counters(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	UINT32 count1, count2, count3, count4, count5, count6, count7;
	UINT32 count8, count9, count10, count11, count12, count13, count14;
	UINT32 count15, count16;

	int len = 0;

	if (offset == 0) {
	
			wdrv_dbGetCountersTransmittedFragmentCount(&count1);
			wdrv_dbGetCountersMulticastTransmittedFrameCount(&count2);
			wdrv_dbGetCountersFailedCount(&count3);
			wdrv_dbGetCountersRetryCount(&count4);
			wdrv_dbGetCountersMultipleRetryCount(&count5);
			wdrv_dbGetCountersFrameDuplicateCount(&count6);
			wdrv_dbGetCountersRTSSuccessCount(&count7);
			wdrv_dbGetCountersRTSFailureCount(&count8);
			wdrv_dbGetCountersACKFailureCount(&count9);
			wdrv_dbGetCountersReceivedFragmentCount(&count10);
			wdrv_dbGetCountersMulticastReceivedFrameCount(&count11);
			wdrv_dbGetCountersFCSErrorCount(&count12);
			wdrv_dbGetCountersTransmittedFrameCount(&count13);
			wdrv_dbGetCountersWEPUndecryptableCount(&count14);
			wdrv_dbGetPrivacyWEPICVErrorCount(&count15);
			wdrv_dbGetPrivacyWEPExcludedCount(&count16);
		
			len += sprintf(page + len,"***Counters***\n"); 
			len += sprintf(page + len,"Transmitted Fragment Count			= %d\n", count1); 
			len += sprintf(page + len,"Multicast Transmitted Frame Count		= %d\n", count2); 
			len += sprintf(page + len,"Failed Count					= %d\n", count3); 
			len += sprintf(page + len,"Retry Count					= %d\n", count4); 
			len += sprintf(page + len,"Multiple Retry Count				= %d\n", count5); 
			len += sprintf(page + len,"Frame Duplicate Count				= %d\n", count6); 
			len += sprintf(page + len,"RTS Success Count				= %d\n", count7); 
			len += sprintf(page + len,"RTS Failure Count				= %d\n", count8); 
			len += sprintf(page + len,"ACK Failure Count				= %d\n", count9); 
			len += sprintf(page + len,"Received Fragment Count				= %d\n", count10); 
			len += sprintf(page + len,"Multicast Received Frame Count			= %d\n", count11); 
			len += sprintf(page + len,"FCS Error Count					= %d\n", count12); 
			len += sprintf(page + len,"Transmitted Frame Count				= %d\n", count13); 
			len += sprintf(page + len,"WEP Undecryptable Count				= %d\n", count14); 
			len += sprintf(page + len,"WEP ICV Error Count				= %d\n", count15); 
			len += sprintf(page + len,"WEP Excluded Count				= %d\n", count16); 
	
	}

	*eof=1;
	return len;
}

/*******************************************************************************
DESCRIPTION: This function is not registered as proc entry anymore.
	     It is called by ioctl and has been left in this file since no other
	     place fit for it	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_srv_read_wlan_params(UINT8 currentConfigWoneInex, char *page)
{
	int i , len = 0;


	unsigned char  macaddr[ETH_ALEN], ssid[50], bssid[ETH_ALEN];
	UINT8 shortRetryLimit, longRetryLimit, dtim, channel, defaultActiveRate;
	UINT8 RateFallback, RateAdaptation;
	UINT16 capabilities, beacon, rtsThreshold, fragThreshold, stationsNotPBCCCapabale;
	UINT32 transmitLifetime, receiveLifetime;
	BOOL currentPBCCSupport;
	UINT8 PBCCAlgorithmMode; 
	UINT32 swMinor, swMajor;
	UINT8 SlotTimeMode, PreambleMode;
	UINT16 SlotTimeThreshold;
	UINT16 stationsNotSlotTimeCapabale, stationsNotShortPreambleCapabale;
	BOOL currentShortSlotTimeSupport, currentShortPreambleSupport;
	UINT8 ERPProtectionMode, ERPProtectionType;
	UINT8 ERPProtectionTI;
	BOOL currentERPProtectionMode;
	char allRates[CCK_PBCC_RATES_MAX+OFDM_RATES_MAX];
	//char rates[CCK_PBCC_RATES_MAX+OFDM_RATES_MAX];
	UINT8 allRatesLen;
	UINT8 respRateMode;
	UINT8 dot11Mode,CWminMode;
	UINT16 CWminWindowSize;
	UINT32	DynAlgoTimeout;
	UINT32 RadioCalibrateOption;
	UINT16 RadioCalibrateInterval;
	UINT8 band;
	UINT8 num_sup_wone;
	UINT16 HiddenSSID;

	UINT8 usageTimeMode;
	UINT8 usageTime;

	UINT16 MultipleSsidFlag;
	char MultipleSsidTable[NUM_MULTIPLE_SSID][WHAL_SSID_STRING_LEN];
	UINT8  idx;
	tVHCF_PARAMS  vhcfParams;
	
	wdrv_dbGetNumSupWone(&num_sup_wone);
	if (num_sup_wone == WONE_DUAL_BAND )
	{
		len += sprintf(page +len,"\n---------  Dual AP ---------\n");

		wdrv_dbGetUsageTimeMode(&usageTimeMode);
		if(usageTimeMode)
			len += sprintf(page + len,"\n--------- Long Usage Time ---------\n");
		else  
			len +=sprintf(page + len,"\n---------  Short Usage Time ---------\n");
	}
	len += sprintf(page + len,"\n--------- Active Set Wone Index is %d ---------\n",currentConfigWoneInex);  

	wdrv_dbGetBand(currentConfigWoneInex,&band);


	swMajor = GET_MAJOR_SW_VER;
	swMinor = GET_MINOR_SW_VER;

	wdrv_dbGetOperationMacAddress(macaddr);
	wdrv_dbGetBssid(bssid);	
	wdrv_dbGetSsid(ssid);
	wdrv_dbGetHiddenSsidStatus(&HiddenSSID);
	wdrv_dbGetMultipleSsidStatus(&MultipleSsidFlag);
	if (MultipleSsidFlag)
	{
		UINT32 i;

		memset(MultipleSsidTable, 0, sizeof(MultipleSsidTable));
		for (i=0; i<NUM_MULTIPLE_SSID; i++)
			wdrv_dbGetElementFromMultipleSsidActualTable((char *)MultipleSsidTable[i], i+1);
	}

	wdrv_dbGetStationConfigOperationalAllRateSet(currentConfigWoneInex,allRates, &allRatesLen);
	wdrv_dbGetDefaultActiveRate(currentConfigWoneInex,&defaultActiveRate);
	wdrv_dbGetApCapabilities(currentConfigWoneInex,&capabilities);
	wdrv_dbGetChannel(currentConfigWoneInex,&channel);
	wdrv_dbGetStationConfigBeaconPeriod(&beacon);
	wdrv_dbGetStationConfigDtimPeriod(&dtim);

	wdrv_dbGetOperationRTSThreshold(&rtsThreshold);
	wdrv_dbGetOperationShortRetryLimit(&shortRetryLimit);
	wdrv_dbGetOperationLongRetryLimit(&longRetryLimit);
	wdrv_dbGetRateFallback(&RateFallback);
	wdrv_dbGetRateAdaptation(currentConfigWoneInex,&RateAdaptation);
	wdrv_dbGetOperationFragmentationThreshold(&fragThreshold);
	wdrv_dbGetOperationMaxTransmitMSDULifetime(&transmitLifetime);
	wdrv_dbGetOperationMaxReceiveLifetime(&receiveLifetime);
	wdrv_dbGetPBCCAlgorithmMode(currentConfigWoneInex,&PBCCAlgorithmMode);
	wdrv_dbGetNumOfStationNotPBCCCapable(&stationsNotPBCCCapabale);
	wdrv_dbGetCurrentPBCCSupportMode(&currentPBCCSupport);
	wdrv_dbGetDot11Mode(currentConfigWoneInex,&dot11Mode);
	wdrv_dbGetRadioCalibrateOption(&RadioCalibrateOption);
	wdrv_dbGetRadioCalibrateInterval(&RadioCalibrateInterval);

	wdrv_dbGetPreambleMode(currentConfigWoneInex,&PreambleMode);
	wdrv_dbGetNumOfStationNotShortPreambleCapable(&stationsNotShortPreambleCapabale);
	wdrv_dbGetCurrentShortPreambleSupport(&currentShortPreambleSupport);

	wdrv_dbGetSlotTimeMode(currentConfigWoneInex,&SlotTimeMode);
	wdrv_dbGetSlotTimeThreshold(&SlotTimeThreshold);
	wdrv_dbGetNumOfStationNotSlotTimeCapable(&stationsNotSlotTimeCapabale);
	wdrv_dbGetCurrentShortSlotTimeSupport(&currentShortSlotTimeSupport);

	wdrv_dbGetERPProtectionMode(currentConfigWoneInex,&ERPProtectionMode);
	wdrv_dbGetCurrentERPProtectionMode(&currentERPProtectionMode);
	wdrv_dbGetERPProtectionType(currentConfigWoneInex,&ERPProtectionType);
	wdrv_dbGetERPProtectionTI(currentConfigWoneInex,&ERPProtectionTI);

	wdrv_dbGetDynamicAlgorithmsTimeout(&DynAlgoTimeout);

	wdrv_dbGetRespRateMode(&respRateMode);

	wdrv_dbGetCwMinMode(currentConfigWoneInex,&CWminMode);
	wdrv_dbGetCwMinWinSize(&CWminWindowSize);
	wdrv_dbGetUsageTime(currentConfigWoneInex,&usageTime);
	wdrv_dbGetVhcfParams(&vhcfParams);


	len += sprintf(page + len,"Software Version:	%d.%d\n",swMajor, swMinor);


	len += sprintf(page + len,"WLAN MAC = <%02X-%02X-%02X-%02X-%02X-%02X>\n",		
			macaddr[0],macaddr[1],macaddr[2],macaddr[3],
			macaddr[4],macaddr[5] );

	len += sprintf(page + len,"WLAN BSSID = <%02X-%02X-%02X-%02X-%02X-%02X>\n",		
			bssid[0],bssid[1],bssid[2],bssid[3],
			bssid[4],bssid[5] );

	len += sprintf(page + len,"SSID = %s\n", ssid); 

	if (band == RADIO_BAND_5GHZ)    
		len += sprintf(page + len,"Band = 5 GHZ\n");  
	else
		len += sprintf(page + len,"Band = 2.4 GHZ\n"); 

	len += sprintf(page + len,"Hidden SSID mode is %s\n", (HiddenSSID==1) ? "Enabled" : "Disabled");
	len += sprintf(page + len,"Multiple SSID mode is %s\n", (MultipleSsidFlag==1) ? "Enabled" : "Disabled");
	if (MultipleSsidFlag==1) 
	{
		len += sprintf(page + len,"List of Multiple SSID:\n");
		for (i=0; i<NUM_MULTIPLE_SSID; i++) 
		{
			if (MultipleSsidTable[i][0] == '\0')
				break;
			len += sprintf(page + len,"\t%d. %s\n",i+1,MultipleSsidTable[i]);
		}
		if (i==0)
			len += sprintf(page + len,"\tNone.\n");
	}

	len += sprintf(page + len,"Capabilities = AGILE  PBCC  ShortPreamble  Privacy  CFPollReq  CFPoll  IBSS  ESS\n"); 

	len += sprintf(page + len,"                 %d    ", ((capabilities >> CAP_AGILE_SHIFT) & CAP_AGILE_MASK));
	len += sprintf(page + len,"  %d   ", ((capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK));
	len += sprintf(page + len,"      %d       ", ((capabilities >> CAP_PREAMBLE_SHIFT) & CAP_PREAMBLE_MASK));
	len += sprintf(page + len,"    %d     ", ((capabilities >> CAP_PRIVACY_SHIFT) & CAP_PRIVACY_MASK));
	len += sprintf(page + len,"    %d      ", ((capabilities >> CAP_CF_REQ_SHIFT) & CAP_CF_REQ_MASK));
	len += sprintf(page + len,"   %d    ", ((capabilities >> CAP_CF_POLL_SHIFT) & CAP_CF_POLL_MASK));
	len += sprintf(page + len,"  %d   ", ((capabilities >> CAP_IBSS_SHIFT) & CAP_IBSS_MASK));
	len += sprintf(page + len,"  %d\n", ((capabilities >> CAP_ESS_SHIFT) & CAP_ESS_MASK));

	len += sprintf(page + len,"Capabilities = RSN  ShortSlotTime\n"); 
	len += sprintf(page + len,"                %d         ", ((capabilities >> CAP_RSN_SHIFT) & CAP_RSN_MASK));
	len += sprintf(page + len,"%d\n", ((capabilities >> CAP_SLOTTIME_SHIFT) & CAP_SLOTTIME_MASK));

	len += sprintf(page + len,"Rates =	"); 
	for (i = 0; i < allRatesLen; i++)
	{
		len += sprintf(page + len,"  "); 
		len += wdrv_printRateInMbpsUnit(allRates[i], page + len);
	}
	len += sprintf(page + len,"\n"); 

	len += sprintf(page + len,"Default active rate = "); 
	len += sprintf(page + len,"  "); 
	len += wdrv_printRateInMbpsUnit(defaultActiveRate, page + len);
	len += sprintf(page + len,"\n"); 

	len += sprintf(page + len,"Channel = %d\n", channel); 

	if (channel == 14)
		len += sprintf(page + len,"Domain = MKK\n"); 
	else
		len += sprintf(page + len,"Domain = ETSI\n"); 

	len += sprintf(page + len,"Beacon period = %d msecs\n", beacon); 

	len += sprintf(page + len,"DTIM period = %d beacons\n", dtim); 

	len += sprintf(page + len,"rtsThreshold = %d bytes\n", rtsThreshold); 
	len += sprintf(page + len,"shortRetryLimit = %d\n", shortRetryLimit); 
	len += sprintf(page + len,"longRetryLimit = %d\n", longRetryLimit); 
	len += sprintf(page + len,"rate fallback = %d\n", RateFallback); 
	if (RateAdaptation)
	{
		len += sprintf(page + len,"rate adaptation = Enabled\n");
	}
	else
	{
		len += sprintf(page + len,"rate adaptation = Disabled\n");
	}
	len += sprintf(page + len,"fragThreshold = %d bytes\n", fragThreshold); 
	len += sprintf(page + len,"transmitLifetime = %d msecs\n", transmitLifetime); 
	len += sprintf(page + len,"receiveLifetime = %d msecs\n", receiveLifetime); 
	len += sprintf(page + len,"Radio Calibrate option=%d interval=%d\n", RadioCalibrateOption, RadioCalibrateInterval); 

	if (dot11Mode == DOT11_MODE_MIXED)
	{
		len += sprintf(page + len,"Dot11 mode = Mixed mode\n");
	}
	else if (dot11Mode == DOT11_MODE_B_ONLY)
	{
		len += sprintf(page + len,"Dot11 mode = B only mode\n");
	}
	else if(dot11Mode == DOT11_MODE_OFDM_ONLY)
	{
		len += sprintf(page + len,"Dot11 mode = G only mode\n");
	}
	else if(dot11Mode == DOT11_MODE_B_ONLY_PLUS)
	{
		len += sprintf(page + len,"Dot11 mode = B+ only mode\n");
	}
	else
		len += sprintf(page + len,"Dot11 mode = A only mode\n");


	if (respRateMode == ASSOCRESP_RATE_ALL_AP)
	{
		len += sprintf(page + len,"AssocResp rate mode = %d (All AP rates)\n", ASSOCRESP_RATE_ALL_AP);
	}
	else if (ASSOCRESP_RATE_AP_AND_STA)
	{
		len += sprintf(page + len,"AssocResp rate mode = %d (Only AP rates supported by STA)\n", ASSOCRESP_RATE_AP_AND_STA);
	}

	len += sprintf(page + len,"Enhanced Dynamic Algorithms Timeout = %d ms\n", DynAlgoTimeout);

	len += sprintf(page + len,"CWmin mode = %d, CWminWindowSize = %d\n", CWminMode,CWminWindowSize);


	/* PBCC support section*/
	len += sprintf(page + len,"--------------------------------------------\n"); 
	if (PBCCAlgorithmMode == PBCC_ALGO_DISABLED)
		len += sprintf(page + len,"PBCC Algorithm			Always off\n");
	else if (PBCCAlgorithmMode == PBCC_ALGO_LOCAL_STATIONS)
		len += sprintf(page + len,"PBCC Algorithm			Local Stations\n");
	else if (PBCCAlgorithmMode == PBCC_ALGO_DYNAMIC)
		len += sprintf(page + len,"PBCC Algorithm			Dynamic\n");
	else if (PBCCAlgorithmMode == PBCC_ALGO_ENHANCED_DYNAMIC)
		len += sprintf(page + len,"PBCC Algorithm			Enhanced Dynamic\n");	
	else if (PBCCAlgorithmMode == PBCC_ALGO_ENABLED)
		len += sprintf(page + len,"PBCC Algorithm			Always on\n");
	else
		len += sprintf(page + len,"PBCC Algorithm			Unrecognized\n");

	if ((PBCCAlgorithmMode == PBCC_ALGO_LOCAL_STATIONS) || (PBCCAlgorithmMode == PBCC_ALGO_DYNAMIC) || (PBCCAlgorithmMode == PBCC_ALGO_ENHANCED_DYNAMIC))
	{
		len += sprintf(page + len,"Number of station that are not PBCC capable:	%d\n", stationsNotPBCCCapabale);
		if ((capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)
		{	
			if (currentPBCCSupport)
				len += sprintf(page + len,"Current PBCC support		On\n");
			else
				len += sprintf(page + len,"Current PBCC support		Off\n");
		}
	}

	/* Preamble support section*/
	len += sprintf(page + len,"--------------------------------------------\n"); 
	if (PreambleMode == PREAMBLE_MODE_LONG)
		len += sprintf(page + len,"Preamble Mode                   Always Long\n");
	else if (PreambleMode == PREAMBLE_MODE_SHORT)
		len += sprintf(page + len,"Preamble Mode                   Always Short\n");
	else if (PreambleMode == PREAMBLE_MODE_LOCAL_STATIONS)
	{
		len += sprintf(page + len,"Preamble Mode                   Local Stations\n");
		if (currentShortPreambleSupport)
			len += sprintf(page + len,"Current Short Preamble support  On\n");
		else
			len += sprintf(page + len,"Current Short Preamble support  Off\n");
		len += sprintf(page + len,"Num of stations not capable     %d\n", stationsNotShortPreambleCapabale);
	}

	/* Slot Time support section*/
	len += sprintf(page + len,"--------------------------------------------\n"); 
	if (SlotTimeMode == SLOTTIME_MODE_DISABLED)
		len += sprintf(page + len,"Slot Time Mode                  Always off\n");
	else if (SlotTimeMode == SLOTTIME_MODE_ENABLED)
		len += sprintf(page + len,"Slot Time Mode                  Always on\n");
	else if (SlotTimeMode == SLOTTIME_MODE_LOCAL_STATIONS)
	{
		len += sprintf(page + len,"Slot Time Mode                  Local Stations\n");
		len += sprintf(page + len,"Slot Time Threshold             %d\n", SlotTimeThreshold);
		if (currentShortSlotTimeSupport)
			len += sprintf(page + len,"Current Short Slot Time support On\n");
		else
			len += sprintf(page + len,"Current Short Slot Time support Off\n");
		len += sprintf(page + len,"Num of stations not capable     %d\n", stationsNotSlotTimeCapabale);
	}
	else if (SlotTimeMode == SLOTTIME_MODE_ENHANCED_DYNAMIC)
	{
		len += sprintf(page + len,"Slot Time Mode                  Enhanced Dynamic\n");
		len += sprintf(page + len,"Slot Time Threshold             %d\n", SlotTimeThreshold);
		if (currentShortSlotTimeSupport)
			len += sprintf(page + len,"Current Short Slot Time support On\n");
		else
			len += sprintf(page + len,"Current Short Slot Time support Off\n");
		len += sprintf(page + len,"Num of stations not capable     %d\n", stationsNotSlotTimeCapabale);
	}



	/* ERP Protection support section*/
	len += sprintf(page + len,"--------------------------------------------\n"); 
	if (ERPProtectionMode == PROTECTION_MODE_DISABLED)
	{
		len += sprintf(page + len,"ERP Protection Mode             Always off\n");
	}
	else
	{
		if(ERPProtectionMode == PROTECTION_MODE_LOCAL_STATIONS)		
			len += sprintf(page + len,"ERP Protection Mode             Local Stations\n");
		else if(ERPProtectionMode == PROTECTION_MODE_DYNAMIC)
			len += sprintf(page + len,"ERP Protection Mode             Dynamic\n");
		else if(ERPProtectionMode == PROTECTION_MODE_ENHANCED_DYNAMIC)
			len += sprintf(page + len,"ERP Protection Mode             Enhanced Dynamic\n");
		else if (ERPProtectionMode == PROTECTION_MODE_ENABLED)
			len += sprintf(page + len,"ERP Protection Mode             Always on\n");
		else
			len += sprintf(page + len," Undefined ERP Protection Mode\n");


		if (currentERPProtectionMode)
		{
			len += sprintf(page + len,"Current ERP Protection Mode     On\n");
		}
		else
		{
			len += sprintf(page + len,"Current ERP Protection Mode     Off\n");
		}

		if(ERPProtectionTI)
			len += sprintf(page + len,"ERP Protection Type             TI-proprietary\n");
		else
		{
			if (ERPProtectionType == PROTECTION_TYPE_RTS_CTS)
				len += sprintf(page + len,"ERP Protection Type             RTS/CTS\n");
			else
				len += sprintf(page + len,"ERP Protection Type             CTS TO SELF\n");
		}
	}
	if(num_sup_wone == 2)
		len += sprintf(page + len,"Wone Usage Time is %d\n",usageTime);

	/* Show vHCF configuration parameters */
	len += sprintf(page + len,"--------------------------------------------\n"); 
	if(vhcfParams.vhcfEnable)   
	{	
		len += sprintf(page + len,"vHCF Enabled:\n");
		for(idx = 0; idx < MAX_VHCF_CLASSIFIERS ; idx++)
		{
			if (vhcfParams.vhcfClassifier[idx].Protocol != VHCF_PROT_NONE)
			{

				vhcfParams.vhcfClassifier[idx].DestIp = htonl(vhcfParams.vhcfClassifier[idx].DestIp);
				len += sprintf(page + len,"Classifier #%u: Prot = %s, Dest_IP = %u.%u.%u.%u, Dest_Port = %u\n",
						idx, 
						(vhcfParams.vhcfClassifier[idx].Protocol == VHCF_PROT_TCP) ? "TCP" : "UDP",
						/*(vhcfParams.vhcfClassifier[idx].DestIp >> 24) & 0xFF,
						  (vhcfParams.vhcfClassifier[idx].DestIp >> 16) & 0xFF,
						  (vhcfParams.vhcfClassifier[idx].DestIp >>  8) & 0xFF,
						  (vhcfParams.vhcfClassifier[idx].DestIp      ) & 0xFF,*/						  
						NIPQUAD(vhcfParams.vhcfClassifier[idx].DestIp),
						vhcfParams.vhcfClassifier[idx].DestPort);

				//len += sprintf(page + len,"Classifier #%u: Prot = %s, Dest_IP = %u.%u.%u.%u, Dest_Port = %u\n",
				//	idx,
			}
			else
				len += sprintf(page + len,"Classifier #%u: Not Used.\n", idx);
		}
	}
	else
		len += sprintf(page + len,"vHCF Disabled!\n");
	len += sprintf(page + len, "Ap bridge mode %s\n", wdrv_dbGetApBridgeMode() ? "enabled" : "disabled");

	return len;
	
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_srv_read_radioFreq(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	UINT8 value;
	int len = 0;

	if (offset == 0) {
		wdrv_dbGetRadioFrequency(&value);
		len = sprintf(page,"%u\n",value);
	}
	

	*eof = 1;
	return len;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_srv_read_power_level(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	UINT8 value;
	int len = 0;

	if (offset == 0) {
		wdrv_dbGetPowerLevel(whal_ParamsGetRadioChannel(0), &value);
		len = sprintf(page, "%hu\n", value);
	}
	

	*eof = 1;
	return len;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_srv_read_radioParamas(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0, i;

	if (offset == 0) {
	
			RadioParam_t rdp = {0};
			const char *sMsg1[] =
			{
				"100%",
				"50%",
				"25%",
				"12.5%",
				"lowest"
			};
		
			
			//wdrv_dbGetRadioParams(&rdp);
			if( sizeof(RadioParam_t) > PAGE_SIZE )
				return -ENOMEM;
		
			wdrv_dbGetRadioParams((RadioParam_t *)page);
			return sizeof(RadioParam_t);
		
			/*len +=sprintf(page + len,"RADIO PARAMS:\n");
			len +=sprintf(page + len,"Bitmask = 0x%x\n",params.overrideBitmask);
			len +=sprintf(page + len,"Power table param type = 0x%x\n",params.powerTable_paramType);
			len +=sprintf(page + len,"Power table:");
			for(i=0;i<MAX_RADIO_PARAM_POWER_TABLE;i++)
				len +=sprintf(page + len,"0x%02x ",params.powerTable_param[i]);
			len +=sprintf(page + len,"\n");*/
		
		
		
			// Display the table
			len = sprintf(page + len, "\n\tRates\t:\t54-48\t36-18\t 12-6\t 11b\n");
			len += sprintf(page + len, "-------------------------------------------------------------------\n");
			for( i =0 ; i < MAX_RADIO_PARAM_POWER_LEVELS; i++)
			{
				int j = i * MAX_RADIO_PARAM_POWER_RATES;
				len += sprintf(page + len, "LVL %d (%s)\t:\t", i+1, sMsg1[i]);
				len += sprintf(page + len, "%4d\t%4d\t%4d\t%4d\n",
						(__s8)rdp.powerTable_param[j  ],
						(__s8)rdp.powerTable_param[j+1],
						(__s8)rdp.powerTable_param[j+2],
						(__s8)rdp.powerTable_param[j+3]
					  );
			}
			if( rdp.overrideBitmask & RADIO_PARAM_OVERRIDE_POWERTABLE) {
				len += sprintf(page + len, "Radio Power table overrides HW settings\n");
			} else {
				len += sprintf(page + len, "Radio Power table uses HW settings\n");
			}

	}
	*eof = 1;

	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_show_read_bss(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = wdrv_bssBridgeShowStatistics(page);
	}
	*eof = 1;

	return  len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_show_read_ps(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = wdrv_powerSavePrintDb(page);
	}
	*eof = 1;
	return 	len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_show_read_lanMib(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	/* TODO:do we have support in MIB2 for linux */
	*eof = 1;
	return -1;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_show_read_lanStat(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
/* TODO:do we have support in MIB2 for linux */
#if 0
	int len = 0;
	bm_print(page, &len);

	*eof = 1;
	return len;
#else
	*eof = 1;
	return -1;
#endif
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_show_read_lanPool(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
/* TODO:do we have support in MIB2 for linux */
	*eof = 1;
	return -1;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_show_read_mgmtStat(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
	
			memMngr_Statistics_T s;
			
			
			len = sprintf(page,"------------ MemMngr Stat ------------");
			if( OK == wlan_memMngrPrintStatistics(&s))
			{
				len +=sprintf(page+len,"msduPool BDs location   @ %p\n", s.msduPool);
				len +=sprintf(page+len,"bdPool data BDs         @ %p\n", s.bdPool);
				len +=sprintf(page+len,"List of data buffers    @ %p\n", s.dataBufPool);
				len +=sprintf(page+len,"firstFreeMSDU           @ %p\n", s.firstFreeMSDU);
				len +=sprintf(page+len,"firstFreeBD             @ %p\n", s.firstFreeBD);
				len +=sprintf(page+len,"firstFreeDataBuf64      @ %p\n", s.firstFreeDataBuf[0]);
				len +=sprintf(page+len,"firstFreeDataBuf2048    @ %p\n", s.firstFreeDataBuf[1]);
				
				len +=sprintf(page+len,"numFreeMSDU             = %d\n", s.numFreeMSDU     );
				len +=sprintf(page+len,"numFreeBD               = %d\n", s.numFreeBD       );
				len +=sprintf (page+len,"numFreeDataBuf64       = %d\n", s.numFreeDataBuf[0]);
				len +=sprintf (page+len,"numFreeDataBuf2048     = %d\n", s.numFreeDataBuf[1]);
			}
	}

	*eof = 1;
	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_show_read_RA(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = wdrv_txPrintRateAdaptation(buf);
	}
	*eof = 1;
	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_show_read_txStat(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = wdrv_txPrintStatistics(buf);
	}
	*eof = 1;
	return len;
}


/*************  hal func ********/

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_fwVer(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = whal_apiPrintFwVersion(buf);
		len += whal_apiGetHDKVersion(buf + len);
	}
	*eof = 1;
	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_acxParams(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = whal_apiPrintAcxParams(buf);
	}
	*eof = 1;
	return len;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_rxInfo(char* page, char **start, off_t off, int count,int *eof, void *data) {
	static char buf[20000];
	/*
		TODO: sooner or later, there will be buffer overflow here !!!
	*/
	static int len;
	int new_count;

	if (off == 0) {
		len = whal_apiPrintRxInfo(buf);
	}

	if (off >= len) {
		*eof = 1;
		return 0;
	}

	new_count =  count + off > len ? len - off : count;
	
	memcpy(page, buf + off, new_count);
	
	off += new_count;
	
	*eof = 0;

	*start = page;

	return new_count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_txIdnt(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = whal_apiPrintTxIndicator(buf);
	}
	*eof = 1;
	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_rxQueue(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = whal_apiPrintRxQueue(buf);
	}
	
	*eof = 1;

	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_txBuf(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	/* was only implimented for 1100*/
	*eof = 1;
	return -1;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_Intr(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = whal_apiPrintIntr(buf);
	}
	*eof = 1;
	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_Mbox(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = whal_apiPrintMbox(buf);
	}
	*eof = 1;
	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_acxStat(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = whal_apiPrintAcxStatistics(buf);
	}
	*eof = 1;
	return len;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/

int proc_whal_show_InterfWAparams(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;
	UINT32 aciRateThr, aciEventThr, aciTimeout, nbTimeout,  busyMedium;
	UINT16 pdHighThresh, pdLowThresh;

	if (offset == 0)
	{
	
		wdrv_dbGetAciRateThreshold(&aciRateThr);
		wdrv_dbGetAciEventsThreshold(&aciEventThr);
		wdrv_dbGetAciStateTimeout(&aciTimeout);
		wdrv_dbGetNbStateTimeout(&nbTimeout);
		wdrv_dbGetPacketDetectLowThresh(&pdLowThresh);
		wdrv_dbGetPacketDetectHighThresh(&pdHighThresh);
		wdrv_dbGetMediumBusyThresh(&busyMedium);
	
		len =  sprintf(buf+len, "Interference workaround configuration:\n");
		len += sprintf(buf+len, "ACI rate threshold     = %u\n", aciRateThr);
		len += sprintf(buf+len, "ACI events threshold   = %u\n", aciEventThr);
		len += sprintf(buf+len, "ACI state timeout      = %u sec\n", aciTimeout);
		len += sprintf(buf+len, "NB state timeout       = %u sec\n", nbTimeout);
		len += sprintf(buf+len, "Low PD threshold       = 0x%hx\n", pdLowThresh);
		len += sprintf(buf+len, "High PD threshold      = 0x%hx\n", pdHighThresh);
		len += sprintf(buf+len, "Busy medium threshold  = %u percents\n", busyMedium);
	
		/*FW configuration*/
		{
			UINT32 pdThresh, edThresh, rxDot11Mode;
	
			whal_apiGetPDthreshold(&pdThresh);
			whal_apiGetEnergyDetect(&edThresh);
			whal_apiGetRxDot11mode(&rxDot11Mode);
			len += sprintf(buf+len, "\nFW configuration:\n");
			len += sprintf(buf+len, "PD threshold  = 0x%x\n", pdThresh);
			len += sprintf(buf+len, "ED threshold  = 0x%x\n", edThresh);
			len += sprintf(buf+len, "RX dot11 mode = %d\n", rxDot11Mode);
			
		}
	}



	*eof=1;

	return len;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_restratStat(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = whal_apiPrintRestartStatistics(buf);
	}
	*eof = 1;
	return len;
}

int proc_whal_read_led(char *page, char **start, off_t offset, int count, int *eof, void *data)
{
	int len = 0;
	extern __u8 tiap_gpio_owner;

	len = sprintf(page, "%s owns gpio\n", tiap_gpio_owner ? "driver" : "fw");

	return len+1;
}

int proc_whal_write_led(struct file *file, const char *buffer, unsigned long count, void *data)
{
	int len = sizeof(char);
	char byte;
	extern __u8 tiap_gpio_owner;
	__u16 tmp;

	if(copy_from_user(&byte, buffer, len))
		return -EFAULT;
	sscanf(&byte, "%hu", &tmp);
	tiap_gpio_owner = (__u8)tmp;

	return count;
}

#if 0 /* Demiurg : ?? */

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_vlyncErr(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = sprintf(buf,"%d Vlynq errors encountered\n",get_acx111_vlynq_errorCount());
	}
	*eof = 1;
	return len;
}

#endif
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_eed(char* page, char **start, off_t offset, int count,int *eof, void *data) {
	static char buf[30000];
	int new_count;
	static int len;
	/*
		TODO: sooner or later, there will be buffer overflow here !!!
	*/

	if (offset == 0) {
		len = whal_apiDumpEEPROM(buf, sizeof(buf));
	}
	
	if (offset >= len) {
		*eof = 1;
		return 0;
	}

	new_count =  count + offset > len ? len - offset : count;
	
	memcpy(page, buf + offset, new_count);
	offset += new_count;
	
	*eof = 0;

	*start = page;

	return new_count;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_write_eew(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;
	int addr,val;


	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);
	addr = in_params.s.Param1; /* band*/
    val = in_params.s.Param2; /* channel id*/

	if(whal_apiWriteEEPROM(addr,val) != OK)
		return -1;
	
	return count;

}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_write_trsh(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;
	int trsh ;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);
	trsh = in_params.s.Param1; /* treshhold */

	if(whal_apiSetTxCompleteThreshold(trsh)== OK)
		return count;
	return -1;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_write_BssPs(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;
	int BssPsMode;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);
	BssPsMode = in_params.s.Param1; /* BssPsMode */

	if(whal_apiSetBssPs(BssPsMode) != OK)
		return -1;
	return count; 
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_Regs(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = whal_apiPrintRegs(buf);
	}
	*eof = 1;
	return len;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_write_reg(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;
	int addr,val;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);
	addr = in_params.s.Param1; /* RegAdd */
	val = in_params.s.Param2; /* val */

	whal_apiPhyWrite(addr,val);
	
	return count;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_write_macReg(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;
	int mac_addr,val;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);
	mac_addr = in_params.s.Param1; /* RegAdd */
	val = in_params.s.Param2; /* val */

	whal_apiMacWrite(mac_addr,val);
	
	return count;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_write_rx_promis(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);

	whal_apiRxPromiscuous(
						 in_params.s.Param1/* promiscuous flag*/	
						 );
	
	return count;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_write_sniffmod(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);

	whal_apiModeSniffer(
					   in_params.s.Param1/* sniff flag*/	
					   );
	
	return count;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_fwversions(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset) {
		len = whal_apiPrintFwVersions(buf);
	}
	*eof = 1;
	return len;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_write_FwSelc(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);

	whal_apiSelectFwNumber(
							in_params.s.Param1/*  selection val*/	
						  );
	return count;
	
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_write_QueueSizeType(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);

	whal_apiSetQueueSizeType(
							 in_params.s.Param1/*  queue size*/	
						    );
	return count;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_radioVers(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = whal_apiPrintRadioVersions(buf);
	}
	*eof = 1;
	return len;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_write_SelcRadionum(struct file *file, const char *buffer,unsigned long count, void *data)
{
	
	proc_params in_params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);

	whal_apiSelectRadioNumber(
					 		  in_params.s.Param1/*  selection val*/	
						     );
	return count;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_write_NvByte(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;

	if( count == 0)
	{	
		return -1;
	}


	copy_from_user((char*)&in_params, buffer, count);

	whal_apiSetNvByte(
					  in_params.s.Param1 ,/* radio num*/	
					  in_params.s.Param2 /*  selection val*/
					 );
	return count;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_write_traceEn(struct file *file, const char *buffer,unsigned long count, void *data)
{
	proc_params in_params;

	if( count == 0)
	{	
		return -1;
	}

	copy_from_user((char*)&in_params, buffer, count);

	whal_apiTraceEnable(
					    in_params.s.Param1/* val*/	
					   );
	return count;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int proc_whal_read_traceRxStat(char* buf, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = whal_apiTracePrintRxStat(buf);
	}
	*eof = 1;
	return len;
}


/*******************************************************************************
DESCRIPTION:

INPUTS:

OUTPUT:

RETURNS:
*******************************************************************************/
int proc_whal_read_RxBufs(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	int len = 0;

	if (offset == 0) {
		len = whal_apiPrintRxBufs(page);
	}
	*eof = 1;
	return len;
}

int proc_debug_read(char* page, char **start, off_t offset, int count,int *eof, void *data)
{
	static char buf[30000];
	int new_count;
	static int len;
	
	if (offset == 0) {
		/* Here comes debug read */
	}
	
	if (offset >= len) {
		*eof = 1;
		return 0;
	}

	new_count =  count + offset > len ? len - offset : count;
	
	memcpy(page, buf + offset, new_count);
	offset += new_count;
	
	*eof = 0;

	*start = page;

	return new_count;

}

int proc_debug_write(struct file *file, const char *buffer,unsigned long count, void *data)
{
	/* Here comes debug write */
   
	return count;
}

/*******************************  TESTTTT ***************************/
 
#ifdef DDA_PROC_FS_TEST_ONLY
void test_dda_procfs(void)
{
	proc_params in_params;
	in_params.s.Param1 = 1; /* band*/
    in_params.s.Param2 = 4; /* cahhnel id*/
	in_params.s.Param3 = 12; /* rate*/
	in_params.s.Param4 = 18; /* mode*/

/*	dda_write_proc_entry(WLAN_PROC_BIT_DIR_NAME,WLAN_PROC_BIT_FCC,(char*)&in_params,sizeof(proc_params));*/
}
#endif

