/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2003 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/***************************************************************************/
/*                                                                         */
/*   MODULE:  dda_proc_fs.h                                                */
/*   PURPOSE: api functions for proc use		                           */
/*                                                                         */
/***************************************************************************/

#ifndef  _DDA_PROC_FS_DEFS_
#define  _DDA_PROC_FS_DEFS_

#include <linux/types.h>
#include <linux/if_ether.h>

/*****************************************************
params struct 
 use in input parameters func.
 NOTE:
   1. fill parametrs according to the proc_func order !!!   	
   2. the data size givven to the proc func = sizeof(proc_params_strcut)	
**********************************************************/
#define PROC_PARAMS_MAGIC 0x42DEAD42

#pragma pack(1)
typedef union _proc_params{
    struct {
        __u32 magic;
	__u32 Param1;
	__u32 Param2;
	__u32 Param3;
	__u32 Param4;
	__u32 Param5;
	__u32 Param6;
	__u32 Param7;
	__u32 Param8;
        __u8 mac[6];
    } s;
    char buf[4096];
} proc_params;

#pragma pack() 


/* proc dir names */
#define   WLAN_PROC_BASE_DIR_PATH     "/proc/wlan/"
#define   WLAN_PROC_BASE_DIR_NAME     "wlan"
#define   WLAN_PROC_BIT_DIR_NAME	  "bit"
#define   WLAN_PROC_MGMT_DIR_NAME	  "mgmt"
#define   WLAN_PROC_WPA_DIR_NAME	  "wpa"
#define   WLAN_PROC_PRIVACY_DIR_NAME  "privacy"
#define   WLAN_PROC_SERV_DIR_NAME	  "serv"
#define   WLAN_PROC_SHOW_DIR_NAME	  "show"
#define   WLAN_PROC_HAL_DIR_NAME	  "hal"
#define   WLAN_PROC_PROD_DIR_NAME	  "production"
#define   WLAN_PROC_DEBUG_DIR_NAME	  "debug"


/*bit entries name */
#define  WLAN_PROC_BIT_FCC     "fcc"
#define  WLAN_PROC_BIT_TELEC   "telec"
#define  WLAN_PROC_BIT_TXCFG   "perTxCfg"
#define  WLAN_PROC_BIT_RXCFG   "perRxCfg"
#define  WLAN_PROC_BIT_CLRSTAT "perClrStat"
#define  WLAN_PROC_BIT_TXSTOP  "perTxStop"
#define  WLAN_PROC_BIT_BITSTOP "bitStop"
#define  WLAN_PROC_BIT_RXSTAT  "perRxStat"
#define  WLAN_PROC_BIT_TXSTAT  "perTxStat"

/* mgmt entries name*/
#define  WLAN_PROC_MGMT_DEAUTH    "deAuth"
#define  WLAN_PROC_MGMT_DISASS    "disAss"
#define  WLAN_PROC_MGMT_STATABLE  "staTable"
#define  WLAN_PROC_MGMT_4XTABLE   "4xTable"
#define  WLAN_PROC_MGMT_STAHASH	  "staHash"
#define  WLAN_PROC_MGMT_STARATES  "staRates"
#define  WLAN_PROC_MGMT_COUNTERS  "counters"
#define  WLAN_PROC_MGMT_SETLOG    "setLog"
#define  WLAN_PROC_MGMT_AGING     "aging"
#define  WLAN_PROC_MGMT_SHOW      "show"
/* create mgmt test menu entries*/
#define  WLAN_PROC_MGMT_TST_NULL      "tstnull"
#define  WLAN_PROC_MGMT_TST_AUTH1     "tstAuth1"
#define  WLAN_PROC_MGMT_TST_AUTH3     "tstAuth3"
#define  WLAN_PROC_MGMT_TST_ASS       "tstAss"
#define  WLAN_PROC_MGMT_TST_REASS     "tstReass"
#define  WLAN_PROC_MGMT_TST_DEAUTH    "tstDeAuth"
#define  WLAN_PROC_MGMT_TS_DEASS      "tstDeAss"
#define  WLAN_PROC_MGMT_TST_CLASS3    "tstClass3"
#define  WLAN_PROC_MGMT_TST_CHALLENGE "tstChallenge"
#define  WLAN_PROC_MGMT_TST_FINDSTA   "tstFindSta"
#define  WLAN_PROC_MGMT_TST_FIND_INS  "tstFindAndInsertSta"
#define  WLAN_PROC_MGMT_TST_REM_STAT  "tstRemStat"
#define  WLAN_PROC_MGMT_TST_RESET     "tstReset"

/* wpa entries */
#define  WLAN_PROC_WPA_RESET     "tstReset"

/* privacy menu*/
#define  WLAN_PROC_PRIVACY_CFG "cfg"
#define  WLAN_PROC_PRIVACY_NOTIFY   "notify"
#define  WLAN_PROC_PRIVACY_WEP_KEYS "wepkeys"
#define  WLAN_PROC_PRIVACY_COUNTERS "counters"
#define  WLAN_PROC_PRIVACY_INVOKE   "invoke"
#define  WLAN_PROC_PRIVACY_EXCLUDE  "exclude"

/*serv menu*/
#define  WLAN_PROC_SERV_WLAN_PARAMS "printWlanParams"
#define  WLAN_PROC_SERV_RATES       "printRates"
#define  WLAN_PROC_SERV_COUNTERS    "counters"
#define  WLAN_PROC_SERV_RADIO_FREQ   "radioFreq"
#define  WLAN_PROC_SERV_RADIO_PARAMS "radioParams"
#define  WLAN_PROC_SERV_POWER_LEVEL "power_level"


/* show menu */
#define  WLAN_PROC_SHOW_BSS       "bss"
#define  WLAN_PROC_SHOW_PS        "ps"
#define  WLAN_PROC_SHOW_TX_STAT	  "txStat"
#define  WLAN_PROC_SHOW_LAN_MIB   "lanMib"
#define  WLAN_PROC_SHOW_LAN_STAT  "lanStat"
#define  WLAN_PROC_SHOW_LAN_POOL  "lanPool"
#define  WLAN_PROC_SHOW_MEM_STAT  "memStat"
#define  WLAN_PROC_SHOW_RATE_ADAP "RateAdap"

/* hal menu */
#define  WLAN_PROC_HAL_FWVER	    "fwVer"
#define  WLAN_PROC_HAL_ACX_PARAM    "acxParams"
#define  WLAN_PROC_HAL_RX_INFO      "rxInfo"
#define  WLAN_PROC_HAL_TX_IDNT      "txIdnt"
#define  WLAN_PROC_HAL_RX_QUEUE     "rxQueue"
#define  WLAN_PROC_HAL_TXBUF        "txBuf"
#define  WLAN_PROC_HAL_INTR         "intr"
#define  WLAN_PROC_HAL_MBOX         "mbox"
#define  WLAN_PROC_HAL_ACXSTAT      "acxStat"
#define  WLAN_PROC_HAL_RESTART_STAT "restartStat"
#define  WLAN_PROC_HAL_VLYNC_ERR    "vlyncErr"
#define  WLAN_PROC_HAL_LED          "led"
#define  WLAN_PROC_INTERF_WD_PARAMS "interfWDparams"

/* hal diag*/
#define  WLAN_PROC_HAL_EED				"eed"
#define  WLAN_PROC_HAL_EEW				"eew"
#define  WLAN_PROC_HAL_SET_TX_COMPTRSH  "SetTxCompleteTrsh"
#define  WLAN_PROC_HAL_SETPS_MODE       "SetPsMode"
#define  WLAN_PROC_HAL_PRINTREGS		"PrintRegs"
#define  WLAN_PROC_HAL_PHY_REG_WRITE    "PhyRegWrite"
#define  WLAN_PROC_HAL_MAC_REG_WRITE    "MacRegWrite"
#define  WLAN_PROC_HAL_SET_RX_PROMIS    "SetRxPromiscuous"
#define  WLAN_PROC_HAL_MODE_SNIFFER     "ModeSniffer"
#define  WLAN_PROC_HAL_FW_VERS			"PrintFwVersions"
#define  WLAN_PROC_HAL_SELEC_FW_NUM     "SelectFwNumber"
#define  WLAN_PROC_HAL_SETY_Q_SIZE_TYPE "SetQueueSizeType"
#define  WLAN_PROC_HAL_RADIO_VER        "PrintRadioVer"
#define  WLAN_PROC_HAL_SET_NVBYTE       "SetNvByte"
#define  WLAN_PROC_HAL_SELEC_RADIO_NUM  "SelectRadioNum"
#define  WLAN_PROC_HAL_TRACE_ENABLE     "TraceEnable"
#define  WLAN_PROC_HAL_TRACE_RX_STAT    "TraceRxStat"
#define  WLAN_PROC_HAL_RX_BUFFS         "rxBuffs"  

/* 4x */
#define WLAN_PROC_4X_MAX_CONCATSIZE "max_concatsize"
#define WLAN_PROC_SHOW_4X_MAX_CONCATSIZE "show_max_concatsize"
#define WLAN_PROC_4X_DIRNAME	    "4x"

#define  WLAN_PROC_PROD_VERSION "version"

/* debug entries */
#define WLAN_PROC_DEBUG_READ_ENTRY "debug_read"
#define WLAN_PROC_DEBUG_WRITE_ENTRY "debug_write"

#endif /* _DDA_PROC_FS_DEFS_ */

