/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998 - 2003 Texas Instruments Incorporated         |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/



/***************************************************************************/
/*                                                                         */
/*   MODULE:  dda_wpa_if.c                                                 */ 
/*   PURPOSE: implementation API for ddc core to access the wpa module 	   */										       
/*                                                                         */
/***************************************************************************/


/***************************************************************************/
/*           	includes                                                   */
/***************************************************************************/
#include "dda_wpa_interface.h"
#include <WPA/util/linux_ipc_kernel.h>
#include "dda.h"
#include "mgmtSm.h"
#include "drvDbMain.h"
#include "drvDbHash.h"
#include "AcxInfoElements.h"
#include "mgmtIf.h"
#include "bssbridge.h"
#include "tx.h"
#include "dda_contexts.h"
#include "whalApi.h"


/***************************************************************************/
/*           	static vars                                                */
/***************************************************************************/


DDA_DECLARE_WAIT_MECHANISM(WaitForWpaModule);
#define WAIT_FOR_WPA_MODULE_TIMEOUT     10000

/*#define FUNC  printk("%s\n" , __func__); */

typedef struct 
{
	BOOL  inUse;
	wpa_temporal_key_t  keys;
}station_wpa_recovery_element_t;

typedef struct 
{
	wpa_cipher_t cipher;
	station_wpa_recovery_element_t stations[MAX_STATIONS];
}wpa_recovery_t;


static wpa_recovery_t wpa_recovery; 

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
static void add_station_to_recovery(wpa_cipher_t cipher,int index,wpa_temporal_key_t *keys)
{
	wpa_recovery.cipher = cipher;
	memcpy(&(wpa_recovery.stations[index].keys),keys,sizeof(wpa_temporal_key_t));
	wpa_recovery.stations[index].inUse = TRUE;	
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
static void remove_station_from_recovery(int index)
{
	wpa_recovery.stations[index].inUse = FALSE;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
static void formatEapolToEthernet(mem_MSDU_T*  msduPtr,char *pEapol ,UINT32 *dataLen)	
{
	/* First thing is to rmove the WLAN header and generate a ethernet 2 header instead. */	
	UINT32   		lengthDelta;
	EthernetHeader_T EthHeader;
	dot11_header_T  	*pdot11Header;
	UINT8 			*dataBuf;
	Wlan_LlcHeader_T  *pWlanSnapHeader;   

	dataBuf = memMgr_BufData(msduPtr->firstBDPtr);
	pdot11Header = (dot11_header_T*) dataBuf;
	pWlanSnapHeader = (Wlan_LlcHeader_T*)((ULONG)dataBuf + (ULONG)WLAN_HDR_LEN);

	CopyMacAddress(&EthHeader.DstAddr, &pdot11Header->address3);
	CopyMacAddress(&EthHeader.SrcAddr, &pdot11Header->address2);
	lengthDelta = WLAN_HDR_LEN + WLAN_SNAP_HDR_LEN - ETHERNET_HDR_LEN;
	EthHeader.TypeLength = pWlanSnapHeader->Type;

	dataBuf += lengthDelta;
	memcpy( dataBuf, (void*)&EthHeader, ETHERNET_HDR_LEN );

	msduPtr->firstBDPtr->dataOffset = lengthDelta;
	msduPtr->dataLen -= lengthDelta;
	msduPtr->firstBDPtr->length -= lengthDelta;

	/* Now, take care of fragmentation. */
	if (wlan_memMngrMsduIsFragmented(msduPtr))
	{
		wlan_memMngrMsduReassemble(msduPtr, pEapol);
	}
	else
		memcpy(pEapol , msduPtr->firstBDPtr->data + msduPtr->firstBDPtr->dataOffset,msduPtr->dataLen); 


	*dataLen = msduPtr->dataLen;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
static mem_MSDU_T *formatEthernetToEapol(char *buf, int dataLen, char *DA)	
{
	mem_MSDU_T*		MSDUPtr;
	UINT32			totalLen;
	dot11_header_T 	*pdot11Header;
	Wlan_LlcHeader_T	*pLlcHeader;
	mem_BD_T		*BD;
	char			*dataP, *bufP;
	char			apMac[MAC_ADDR_LEN];

	totalLen = WLAN_HDR_LEN + sizeof(Wlan_LlcHeader_T) + dataLen - sizeof(EthernetHeader_T); 

	if (wlan_memMngrAllocMSDU(&MSDUPtr, totalLen) != OK)
		return NULL;

	MSDUPtr->dataLen = totalLen;
	MSDUPtr->headerLen = WLAN_HDR_LEN;

	BD = MSDUPtr->firstBDPtr;

	BD->dataOffset = 0;

	BD->length = totalLen - dataLen;

	pdot11Header = (dot11_header_T *)(BD->data + BD->dataOffset) ;

	/* build frame control first */
	pdot11Header->fc = 0;
	
	pdot11Header->fc |= DOT11_FC_PROT_VERSION;

	pdot11Header->fc |= DOT11_FC_TYPE_DATA;

	pdot11Header->fc |= (FALSE << DOT11_FC_TO_DS_SHIFT);

	pdot11Header->fc |= (TRUE << DOT11_FC_FROM_DS_SHIFT);

	pdot11Header->fc |= (FALSE << DOT11_FC_MORE_FRAG_SHIFT);

	pdot11Header->fc |= (FALSE << DOT11_FC_RETRY_SHIFT);

	pdot11Header->fc |= (FALSE << DOT11_FC_PWR_MGMT_SHIFT);

	pdot11Header->fc |= (FALSE << DOT11_FC_MORE_DATA_SHIFT);

	pdot11Header->fc |= (FALSE << DOT11_FC_WEP_SHIFT);

	pdot11Header->fc |= (FALSE << DOT11_FC_ORDER_SHIFT);

	/* Build the addresses */
	wdrv_dbGetBssid(apMac);

	memcpy(&pdot11Header->address1, buf, MAC_ADDR_LEN);		/* dest */
	memcpy(&pdot11Header->address2, apMac, MAC_ADDR_LEN);	/* BSSID */
	memcpy(&pdot11Header->address3, apMac, MAC_ADDR_LEN);	/* src */


	pLlcHeader = (Wlan_LlcHeader_T *)(BD->data + BD->dataOffset + WLAN_HDR_LEN);

		/* Now build the LLC header. */
	pLlcHeader->DSAP = SNAP_CHANNEL_ID;
	pLlcHeader->SSAP = SNAP_CHANNEL_ID;
	pLlcHeader->Control = LLC_CONTROL_UNNUMBERED_INFORMATION;
	pLlcHeader->OUI[0] = SNAP_OUI_RFC1042_BYTE0;
	pLlcHeader->OUI[1] = SNAP_OUI_RFC1042_BYTE1;
	pLlcHeader->OUI[2] = SNAP_OUI_RFC1042_BYTE2;
	pLlcHeader->Type = htons(EAPOL_ETHER_TYPE);

	bufP = buf + 14;

	dataP = (char *)(BD->data + BD->dataOffset + WLAN_HDR_LEN + sizeof(Wlan_LlcHeader_T));

	wlan_memMngrFillField(&dataP, bufP, dataLen, &BD);

/*	wdrv_mgmtPrintFrame(MSDUPtr);*/

	return MSDUPtr; 
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
STATUS dda_wpa_drv_send_hello(void)
{
	wpaMessage_T	msg;
	UINT32 rc;

	msg.type = WPA_DRV_HELLO;

	//	printk("WDRV_WPA: sending hello msg to wpa module\n");
	rc = ipc_connect_buf_write((void *)&msg, sizeof(wpaMessage_T));
	if (rc)
	{
		printk("WDRV_WPA_KERNEL: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
		return NOK;
	}
	return OK;

}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
STATUS dda_wpa_restart_module(void)
{

	int i;
	char mac[6];
	/* restrat */
	for(i = 0;i<MAX_STATIONS;i++)
	{
		if(wpa_recovery.stations[i].inUse == TRUE)
		{
			wdrv_dbGetStaMacAddress(i,mac);
			dda_wpa_set_mapping_key(mac,wpa_recovery.cipher,&wpa_recovery.stations[i].keys);	
		}
	}

	return OK;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/

STATUS dda_wpa_notify_station_connect(UINT16 idx) {
	UINT32 wpaMode;
	char   mac[MAC_ADDR_LEN];
	wpaMessage_T	msg;
	UINT32 rc;

/*	FUNC;*/

	wdrv_dbGetWpaMode(&wpaMode);

	if (wpaMode == WPA_STA_TYPE_WEP_LEGACY)
	{			
		wdrv_dbGetStaMacAddress(idx, mac);
		wdrv_mgmtMacAuthorize(mac);
		return OK;
	}
	else 
	{		
		msg.type = NOTIFY_STATION_CONNECT;
		wdrv_dbGetStaMacAddress(idx, (char*)msg.content.stationConnect.mac);
		msg.content.stationConnect.index = idx;
		wdrv_dbGetStationRsnIe(idx,&(msg.content.stationConnect.rsne));
		
/*		printk(" &&&& rsne size DBBB = %d\n", msg.content.stationConnect.rsne.hdr.eleLen);*/
		rc = ipc_connect_buf_write((void *)&msg, sizeof(wpaMessage_T));
		if (rc == NOK)
		{
			printk("WDRV_WPA_KERNEL: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
			return NOK;
		}
	}
	return OK;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
STATUS dda_wpa_notify_station_disconnect(UINT16 idx) {
	wpaMessage_T	msg;
	UINT32 rc;
	UINT32 wpaMode;

/*	FUNC;*/

	wdrv_dbGetWpaMode(&wpaMode);

	if (wpaMode == WPA_STA_TYPE_WEP_LEGACY)
	{			
		return OK;
	}
	else
	{	
		msg.type = NOTIFY_STATION_DIS_CONNECT;
		wdrv_dbGetStaMacAddress(idx, msg.content.stationDisConnect.mac);
		
		rc = ipc_connect_buf_write((void *)&msg, sizeof(wpaMessage_T));
		if (rc)
		{
			printk("WDRV_WPA_KERNEL: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
			return NOK;
		}
	}

	return OK;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
STATUS dda_wpa_notify_eapolReceived(mem_MSDU_T*  msduPtr, BOOL encrypted) {

	wpaMessage_T	msg;
	UINT32 rc;
	UINT32 wpaMode;

/*	FUNC;*/

	wdrv_dbGetWpaMode(&wpaMode);

	if (wpaMode == WPA_STA_TYPE_WEP_LEGACY)
	{	
		wlan_memMngrFreeMSDU(msduPtr->handle);
		return OK;
	}
	else
	{
		formatEapolToEthernet(msduPtr,(msg.content.eapolFrame.pEapol), &msg.content.eapolFrame.eapolDataLen);
#if 0 
		printk("KERNELLLL receive eapol len %d\n ",msg.content.eapolFrame.eapolDataLen);
		for(i = 0;i<msg.content.eapolFrame.eapolDataLen;i++)
		{
			printk("%02x ", msg.content.eapolFrame.pEapol[i]);
			if(!(i & 0x7))
				printk("\n");
		}
#endif
		msg.type = EAPOL_FRAME;
		msg.content.eapolFrame.encrypted = encrypted;

		rc = ipc_connect_buf_write((void *)&msg, sizeof(wpaMessage_T));
		wlan_memMngrFreeMSDU(msduPtr->handle);

		if(rc)
		{
			printk("WDRV_WPA_KERNEL: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
			return NOK;
		}
	}

	return OK;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
STATUS dda_wpa_first_dataFromStation(UINT16 idx, BOOL isEapol) {
	wpaMessage_T	msg;
	UINT32 rc;
	UINT32 wpaMode;

/*	FUNC;*/

	wdrv_dbGetWpaMode(&wpaMode);

	if (wpaMode == WPA_STA_TYPE_WEP_LEGACY)
	{			
		return OK;
	}
	else
	{	
		msg.type = FIRST_DATA_FROM_STATION;
		wdrv_dbGetStaMacAddress(idx, msg.content.firstDataFromStation.mac);
		msg.content.firstDataFromStation.isEapol = isEapol;
		
		rc = ipc_connect_buf_write((void *)&msg, sizeof(wpaMessage_T));
		if (rc)
		{
			printk("WDRV_WPA_KERNEL: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
			return NOK;
		}
	}

	return OK;
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
STATUS dda_wpa_rxMicFailure(UINT16 idx) {

	wpaMessage_T	msg;
	UINT32 rc;

	UINT32 wpaMode;

/*	FUNC;*/

	wdrv_dbGetWpaMode(&wpaMode);

	if (wpaMode == WPA_STA_TYPE_WEP_LEGACY)
	{			
		return OK;
	}
	else
	{	
		msg.type = MIC_FAILURE;
		wdrv_dbGetStaMacAddress(idx, msg.content.micFailure.mac);
		
		rc = ipc_connect_buf_write((void *)&msg, sizeof(wpaMessage_T));
		if (rc)
		{
			printk("WDRV_WPA_KERNEL: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
			return NOK;
		}
	}

	return OK;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
STATUS   dda_wpa_SetStationSecAttack(Uint16 index,BOOL flag) {
	wpaMessage_T	msg;
	UINT32 rc;

	UINT32 wpaMode;

/*	FUNC;*/

	wdrv_dbGetWpaMode(&wpaMode);

	if (wpaMode == WPA_STA_TYPE_WEP_LEGACY)
	{			
		return OK;
	}
	else
	{	
		msg.type = SET_STA_ATTACK;
		msg.content.StaAttack.flag  = flag;
		msg.content.StaAttack.index = index;
		
		rc = ipc_connect_buf_write((void *)&msg, sizeof(wpaMessage_T));
		if (rc)
		{
			printk("WDRV_WPA_KERNEL: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
			return NOK;
		}
	}

	return OK;

}



/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
STATUS dda_wpa_canStationAss(Uint16 index,Uint8 *info_elem, int info_elem_len)
{

	
	wpaMessage_T	msg;
	int rc,wpaMode;

/*	FUNC;*/
	wdrv_dbGetWpaMode(&wpaMode);

	if (wpaMode == WPA_STA_TYPE_WEP_LEGACY)
	{
		dda_wpa_canStationAssRes(index,1);
		setBcastState(TRUE);
		return OK;	
	}
/*	printk("checking if station index %d can ass",index);*/

	msg.type = CAN_STA_ASS;
	
	msg.content.canStaAss.index = index;
	msg.content.canStaAss.info_elem_len = info_elem_len;

	memcpy(msg.content.canStaAss.info_elem,info_elem,info_elem_len);
	
	rc = ipc_connect_buf_write((void *)&msg, sizeof(wpaMessage_T));
	
	if (rc)
	{
		printk("WDRV_WPA_KERNEL: (%s) Error in 'ipc_write_port'. Error Code = %d\n",__func__, rc);
		return NOK;
	}

	return OK;
}



/****************************  driver api for user cmd    ***********************************************/




/*******************************************************************************
		dda_wpa_receive

DESCRIPTION:receive a msg from wpa user mode process,update driver according
			to cmd type
	 		   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
STATUS dda_wpa_receive(void *message, UINT32 len) {

	wpaMessage_T *msg = (wpaMessage_T *)message;
	unsigned long flags;

	if(len != sizeof(wpaMessage_T))
	{
		printk("dda_wpa_receive:receive msg with error len %d,expected %d\n",len,sizeof(wpaMessage_T));
	}

	switch(msg->type)
	{
	case NOTIFY_STATION_CONNECT:
	case NOTIFY_STATION_DIS_CONNECT:
	case FIRST_DATA_FROM_STATION:
	case MIC_FAILURE:
	case CAN_STA_ASS:
	case RESTART_MODULE:
		printk("\ndda_wpa_receive:receive no kernel msg to handle ,type %d\n",msg->type);
		return NOK;
	case EAPOL_FRAME:
		dda_wpa_send_eapol(msg->content.eapolFrame.pEapol,
						   msg->content.eapolFrame.eapolDataLen,
						   msg->content.eapolFrame.encrypted);
		break;
	case SET_DEFAULT_KEY:	
		dda_wpa_set_default_key(msg->content.setDefaultKey.index,
								msg->content.setDefaultKey.cipher,
								&msg->content.setDefaultKey.temporal_key
							   );
		break;
	case SET_DEFAULT_KEY_ID:	
		dda_wpa_set_default_key_id(msg->content.setDefaultKey.index);
		break;
	case REMOVE_DEFAULT_KEY:
		dda_wpa_remove_default_key(msg->content.removeDefaultKey.index,
								   msg->content.removeDefaultKey.cipher);
		break;
	case SET_MAP_KEY:
		dda_wpa_set_mapping_key(msg->content.setMapKey.mac,
								msg->content.setMapKey.cipher,
								&(msg->content.setMapKey.temporal_key));
		break;
	case REMOVE_MAP_KEY:
		dda_wpa_remove_mapping_key(msg->content.removeMapKey.mac,
								   msg->content.removeMapKey.cipher
								  );
		break;
	case DEAUTH_STA:
		dda_wpa_deauth_sta(msg->content.deAssAuthSta.mac);
		break;
	case DISASS_STA:
		dda_wpa_disassoc_sta(msg->content.deAssAuthSta.mac);
		break;
	case BLOCK_UNICAST:
		dda_wpa_block_unicast_traffic(msg->content.blockUcast.block,msg->content.blockUcast.mac);
		break;
	case BLOCK_MULTYCAST:
		dda_wpa_block_multicast_traffic(msg->content.blockMcast.block);
		break;
	case CAN_STA_ASS_RES:
		PAL_intrLock(flags);
		dda_wpa_canStationAssRes(msg->content.canStaAssRes.index,msg->content.canStaAssRes.flag);
		PAL_intrUnLock(flags);
		break;
	case WPA_HELLO:
		{
/*			int i;*/
//			printk("\nkernel wpa api: receive hello msg,connection alive.\n");

#if 0
			printf("@@@@@@  KERNEL INFO len = %d\n",msg->content.hello.info_len);

			for(i =0;i<msg->content.hello.info_len;i++)
			{
				printk("0x%x ",msg->content.hello.info[i]);
				if( !(i & 7))
					printk("\n");
			}
#endif
			wdrv_dbSetRsne(msg->content.hello.info_len ,msg->content.hello.info);
			PAL_sleepLong(1);/* just in case the driver not waiting for hello yet...*/
			dda_wpa_receive_hello();
			break;
		}
	default:
		printk("dda_wpa_receive:receive unknown msg to handle ,type %d\n",msg->type);
		
		return NOK; 
	}

	return OK;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void dda_wpa_canStationAssRes(Uint16 index,int ret)
{

	mgmtMessage_T message;
	message.type = WPA_ASSOC_REQ_REP;
	message.content.wpaAssCheckRep.index = index;
	message.content.wpaAssCheckRep.flag  = ret;

	wdrv_DDA_mgmt_setEvent(&message);
	return;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void dda_wpa_set_default_key(int index,wpa_cipher_t cipher,wpa_temporal_key_t *temporal_key)
{
	char bcastMac[MAC_ADDR_LEN] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

	if (index > 3)
		return;
	
	/* if cipher is TKIP then we have to configure the key in one of the mapping keys, 
		and in addition to set the broadcast MIC key */
	if (cipher == WPA_CIPHER_TKIP)
	{	
        UINT8 dummyKey[WPA_TKIP_ENCRYPTION_KEY_LEN];

		
   
        /* mic failure workaround - first insert dummy key for icv fail at station, then install mic key 
           and then the right tkip key */
        whal_apiSetMappingKey(KEY_ADD_OR_REPLACE, bcastMac, WPA_TKIP_ENCRYPTION_KEY_LEN, KEY_TKIP_GROUP, ENTRY_INDEX_IGNORE, index, dummyKey);

        whal_apiSetMappingKey(KEY_ADD_OR_REPLACE, bcastMac, WPA_MIC_KEY_LEN, KEY_MICHAEL_GROUP, MIC_TX_KEY_ENTRY_INDEX, index, temporal_key->tkip.tx_mic_key);
        whal_apiSetMappingKey(KEY_ADD_OR_REPLACE, bcastMac, WPA_TKIP_ENCRYPTION_KEY_LEN, KEY_TKIP_GROUP, ENTRY_INDEX_IGNORE, index, temporal_key->tkip.tkip_key);


        wdrv_dbSetBroadcastSuiteType(CIPHER_TKIP);


		if(REPORT_LEVEL(RL_WPA)) 
		{
			wdrv_utilsReport(RL_MGMT, PRI_DEBUG, "set_default_key: index :%d tx group key %x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x,%x\n",index,temporal_key->tkip.tkip_key[0],temporal_key->tkip.tkip_key[1],temporal_key->tkip.tkip_key[2],temporal_key->tkip.tkip_key[3],temporal_key->tkip.tkip_key[4],temporal_key->tkip.tkip_key[5],temporal_key->tkip.tkip_key[6],temporal_key->tkip.tkip_key[7], temporal_key->tkip.tkip_key[8],temporal_key->tkip.tkip_key[9],temporal_key->tkip.tkip_key[10],temporal_key->tkip.tkip_key[11],temporal_key->tkip.tkip_key[12],temporal_key->tkip.tkip_key[13],temporal_key->tkip.tkip_key[14],temporal_key->tkip.tkip_key[15]);
			wdrv_utilsReport(RL_MGMT, PRI_DEBUG, "set_default_key: tx group mic %x,%x,%x,%x,%x,%x,%x,%x\n",temporal_key->tkip.tx_mic_key[0],temporal_key->tkip.tx_mic_key[1],temporal_key->tkip.tx_mic_key[2],temporal_key->tkip.tx_mic_key[3],temporal_key->tkip.tx_mic_key[4],temporal_key->tkip.tx_mic_key[5],temporal_key->tkip.tx_mic_key[6],temporal_key->tkip.tx_mic_key[7]);
		}

	}
	
	else
	{
		if (cipher == WPA_CIPHER_WEP_40)
		{
            wdrv_dbSetDefaultWepKey((UINT8)index, WPA_WEP_40_KEY_LEN, temporal_key->wep.wep_key);
			wdrv_dbSetBroadcastSuiteType(CIPHER_WEP);
			whal_apiSetDefaultKey(WEP_ACTION_ADD, (UINT8)index, WPA_WEP_40_KEY_LEN, temporal_key->wep.wep_key);
		}
		else if (cipher == WPA_CIPHER_WEP_104)
		{
            wdrv_dbSetDefaultWepKey((UINT8)index,WPA_WEP_104_KEY_LEN,temporal_key->wep.wep_key);
			wdrv_dbSetBroadcastSuiteType(CIPHER_WEP);
			whal_apiSetDefaultKey(WEP_ACTION_ADD, (UINT8)index, WPA_WEP_104_KEY_LEN, temporal_key->wep.wep_key);
		}
		else if (cipher == WPA_CIPHER_WEP_232)
		{
            wdrv_dbSetDefaultWepKey((UINT8)index,WPA_WEP_232_KEY_LEN,temporal_key->wep.wep_key);
			wdrv_dbSetBroadcastSuiteType(CIPHER_WEP);
			whal_apiSetDefaultKey(WEP_ACTION_ADD, (UINT8)index, WPA_WEP_232_KEY_LEN, temporal_key->wep.wep_key);
		}
	}
}

/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void dda_wpa_set_default_key_id(int index)
{
	if (index > 3)
		return;

    wdrv_dbSetWepDefaultKeyId((UINT8)index);
	whal_apiSetDefaultKeyId((UINT8)index);
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void dda_wpa_remove_default_key(int index,wpa_cipher_t cipher)
{
	char bcastMac[MAC_ADDR_LEN] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
	wpa_temporal_key_t	temporal_key;

	memset(&temporal_key, 0, sizeof(wpa_temporal_key_t));
	
	/* if cipher is TKIP then we have to remove the key from one of the mapping keys, 
			and in addition to reset the broadcast MIC key */
	if (cipher == WPA_CIPHER_TKIP)
	{	
        /* DB must be set before FW incase of restart workaround */

		wdrv_dbSetBroadcastSuiteType(CIPHER_NONE);
  
        whal_apiSetMappingKey(KEY_REMOVE_V, bcastMac, WPA_TKIP_ENCRYPTION_KEY_LEN, KEY_TKIP_GROUP,ENTRY_INDEX_IGNORE, index, temporal_key.tkip.tkip_key);

        whal_apiSetMappingKey(KEY_REMOVE_V, bcastMac, WPA_MIC_KEY_LEN, KEY_MICHAEL_GROUP, MIC_TX_KEY_ENTRY_INDEX, index, temporal_key.tkip.tx_mic_key);

	}
	
	else
	{
		if (cipher == WPA_CIPHER_WEP_40)
		{
			wdrv_dbSetBroadcastSuiteType(CIPHER_NONE);
            wdrv_dbSetDefaultWepKey((UINT8)index, 0, NULL);
			whal_apiSetDefaultKey(KEY_REMOVE_V, (UINT8)index, WPA_WEP_40_KEY_LEN, temporal_key.wep.wep_key);
		}
		else if (cipher == WPA_CIPHER_WEP_104)
		{
			wdrv_dbSetBroadcastSuiteType(CIPHER_NONE);
            wdrv_dbSetDefaultWepKey((UINT8)index,0,NULL);
			whal_apiSetDefaultKey(KEY_REMOVE_V, (UINT8)index, WPA_WEP_104_KEY_LEN, temporal_key.wep.wep_key);
		}
		else if (cipher == WPA_CIPHER_WEP_232)
		{
			wdrv_dbSetBroadcastSuiteType(CIPHER_NONE);
            wdrv_dbSetDefaultWepKey((UINT8)index,0,NULL);
			whal_apiSetDefaultKey(KEY_REMOVE_V, (UINT8)index, WPA_WEP_232_KEY_LEN, temporal_key.wep.wep_key);
		}
	}
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void dda_wpa_set_mapping_key(char*/*ether_mac_t */mac,wpa_cipher_t cipher,wpa_temporal_key_t *temporal_key)
{
	UINT16	index;
/*	char *tmp;
	int i;
	UINT32 tmp_ci = cipher;*/

	if ( wdrv_dbFindStaIndex(mac, &index) != OK)
		return;
#if 0
	printk("\nkernel mac = %x,%x,%x,%x,%x,%x !!!",mac[0],
										    mac[1],
										    mac[2],
										    mac[3],
										    mac[4],
										    mac[5]);

	

	printk("\ncifer = %d !!",tmp_ci);

	tmp = (char*)temporal_key->tkip.tkip_key;
	printk("\nKEY : ");
	for (i = 0;i<8;i++)
	{
		printk("0x%x,",tmp[i]);
	}
	tmp = (char*)temporal_key->tkip.tx_mic_key;
	printk("\nTX :");
	for (i = 0;i<8;i++)
	{
		printk("0x%x,",tmp[i]);
	}
	tmp = (char*)temporal_key->tkip.rx_mic_key;
	printk("\nRX :");
	for (i = 0;i<8;i++)
	{
		printk("0x%x,",tmp[i]);
	}

#endif

	/* if cipher is TKIP then we have to set the station TX & RX MIC key in addition 
		to configuring the key to the HAL */
	if (cipher == WPA_CIPHER_TKIP)
	{	

       /* must be last incase of restart workaround*/
        wdrv_dbSetStationSuiteType(index, CIPHER_TKIP);

        whal_apiSetMappingKey(KEY_ADD_OR_REPLACE, mac, WPA_TKIP_ENCRYPTION_KEY_LEN, KEY_TKIP_PAIRWISE, ENTRY_INDEX_IGNORE, index, temporal_key->tkip.tkip_key);
        whal_apiSetMappingKey(KEY_ADD_OR_REPLACE, mac, WPA_MIC_KEY_LEN, KEY_MICHAEL_PAIRWISE, MIC_TX_KEY_ENTRY_INDEX, 0, temporal_key->tkip.tx_mic_key);
        whal_apiSetMappingKey(KEY_ADD_OR_REPLACE, mac, WPA_MIC_KEY_LEN, KEY_MICHAEL_PAIRWISE, MIC_RX_KEY_ENTRY_INDEX, 0, temporal_key->tkip.rx_mic_key);

		if(REPORT_LEVEL(RL_WPA)) 
		{
			wdrv_utilsReport(RL_MGMT, PRI_DEBUG, "set_mapping_key TX: %x,%x,%x,%x,%x,%x,%x,%x\n",temporal_key->tkip.tx_mic_key[0],temporal_key->tkip.tx_mic_key[1],temporal_key->tkip.tx_mic_key[2],temporal_key->tkip.tx_mic_key[3],temporal_key->tkip.tx_mic_key[4],temporal_key->tkip.tx_mic_key[5],temporal_key->tkip.tx_mic_key[6],temporal_key->tkip.tx_mic_key[7]);
			wdrv_utilsReport(RL_MGMT, PRI_DEBUG, "set_mapping_key RX: %x,%x,%x,%x,%x,%x,%x,%x\n",temporal_key->tkip.rx_mic_key[0],temporal_key->tkip.rx_mic_key[1],temporal_key->tkip.rx_mic_key[2],temporal_key->tkip.rx_mic_key[3],temporal_key->tkip.rx_mic_key[4],temporal_key->tkip.rx_mic_key[5],temporal_key->tkip.rx_mic_key[6],temporal_key->tkip.rx_mic_key[7]);
		}
	}
	else
	{
		if (cipher == WPA_CIPHER_WEP_40)
		{
            wdrv_dbSetStationEncryptionKey(index, WPA_WEP_40_KEY_LEN,temporal_key->wep.wep_key);
			wdrv_dbSetStationSuiteType(index, CIPHER_WEP);

            whal_apiSetMappingKey(KEY_ADD_OR_REPLACE, mac, WPA_WEP_40_KEY_LEN, KEY_WEP_ADDR, ENTRY_INDEX_IGNORE, index, temporal_key->wep.wep_key);

		}
		else if (cipher == WPA_CIPHER_WEP_104)
		{
            wdrv_dbSetStationEncryptionKey(index, WPA_WEP_104_KEY_LEN,temporal_key->wep.wep_key);
			wdrv_dbSetStationSuiteType(index, CIPHER_WEP);

            whal_apiSetMappingKey(KEY_ADD_OR_REPLACE, mac, WPA_WEP_104_KEY_LEN, KEY_WEP_ADDR, ENTRY_INDEX_IGNORE, index, temporal_key->wep.wep_key);
		}
		else if (cipher == WPA_CIPHER_WEP_232)
		{
            wdrv_dbSetStationEncryptionKey(index, WPA_WEP_232_KEY_LEN,temporal_key->wep.wep_key);
			wdrv_dbSetStationSuiteType(index, CIPHER_WEP);

            whal_apiSetMappingKey(KEY_ADD_OR_REPLACE, mac, WPA_WEP_232_KEY_LEN, KEY_WEP_ADDR, ENTRY_INDEX_IGNORE, index, temporal_key->wep.wep_key);
		}
	}

	add_station_to_recovery(cipher,index,temporal_key);
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void dda_wpa_remove_mapping_key(ether_mac_t mac,wpa_cipher_t cipher)
{
	UINT16	index;
	wpa_temporal_key_t	temporal_key;

	memset(&temporal_key, 0, sizeof(wpa_temporal_key_t));
	
	if(wdrv_dbFindStaIndex(mac, &index) != OK)
		return; 

	/* if cipher is TKIP then we have to remove the station TX & RX MIC key in addition 
		to removing the key from the HAL */
	if (cipher == WPA_CIPHER_TKIP)
	{	
		wdrv_dbSetStationSuiteType(index, CIPHER_NONE);
        wdrv_dbSetStationEncryptionKey(index, 0, NULL);
		wdrv_dbSetStationTxMicKey(index, NULL);
		wdrv_dbSetStationRxMicKey(index, NULL);
        whal_apiSetMappingKey(KEY_REMOVE_V, mac, WPA_MIC_KEY_LEN, KEY_MICHAEL_PAIRWISE, MIC_TX_KEY_ENTRY_INDEX, 0, temporal_key.tkip.tx_mic_key);
        whal_apiSetMappingKey(KEY_REMOVE_V, mac, WPA_MIC_KEY_LEN, KEY_MICHAEL_PAIRWISE, MIC_RX_KEY_ENTRY_INDEX, 0, temporal_key.tkip.rx_mic_key);

        whal_apiSetMappingKey(KEY_REMOVE_V, mac, WPA_TKIP_ENCRYPTION_KEY_LEN, KEY_TKIP_PAIRWISE, ENTRY_INDEX_IGNORE, 0, temporal_key.tkip.tkip_key);

	}
	
	else
	{
		if (cipher == WPA_CIPHER_WEP_40)
		{
			wdrv_dbSetStationSuiteType(index, CIPHER_NONE);
            wdrv_dbSetStationEncryptionKey(index, 0,NULL);
            whal_apiSetMappingKey(KEY_REMOVE_V, mac, WPA_WEP_40_KEY_LEN, KEY_WEP_ADDR, ENTRY_INDEX_IGNORE, index, temporal_key.wep.wep_key);
		}
		else if (cipher == WPA_CIPHER_WEP_104)
		{
			wdrv_dbSetStationSuiteType(index, CIPHER_NONE);
            wdrv_dbSetStationEncryptionKey(index, 0,NULL);
            whal_apiSetMappingKey(KEY_REMOVE_V, mac, WPA_WEP_104_KEY_LEN, KEY_WEP_ADDR, ENTRY_INDEX_IGNORE, index, temporal_key.wep.wep_key);
		}
		else if (cipher == WPA_CIPHER_WEP_232)
		{
			wdrv_dbSetStationSuiteType(index, CIPHER_NONE);
            wdrv_dbSetStationEncryptionKey(index, 0,NULL);
            whal_apiSetMappingKey(KEY_REMOVE_V, mac, WPA_WEP_232_KEY_LEN, KEY_WEP_ADDR, ENTRY_INDEX_IGNORE, index, temporal_key.wep.wep_key);
		}
	}

	remove_station_from_recovery(index);
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void dda_wpa_deauth_sta(ether_mac_t mac)
{
	wdrv_mgmtDeAuthMng(mac, STATUS_UNSPECIFIED);
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void dda_wpa_disassoc_sta(ether_mac_t mac)
{
	wdrv_mgmtDisAssocMng(mac, STATUS_UNSPECIFIED);
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void dda_wpa_set_ap_wpa_info_elem(u8 *wpa_info_elem,int len)
{
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
u64  dda_wpa_get_tsc()
{
	return 0;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void dda_wpa_block_multicast_traffic(int block)
{
	if (block)
		setBcastState(FALSE);
	else
		setBcastState(TRUE);

}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
void dda_wpa_block_unicast_traffic(int block,ether_mac_t mac)
{
#if 0
	FUNC
	printk("dda_wpa_block_unicast_traffic:mac = 0x%x,0x%x\n",mac[4],mac[5]);
	printk("dda_wpa_block_unicast_traffic block = %d\n",block);
#endif
	if (block)
		wdrv_mgmtMacDeAuthorize(mac);
	else
		wdrv_mgmtMacAuthorize(mac);
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int  dda_wpa_get_max_num_of_mapping_keys(void)
{
	return MAX_STATIONS;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
int  dda_wpa_send_eapol(void *buf, int len, int encrypt)
{
	UINT16 idx;
	ULONG txFlags;
	int rc;
	unsigned long flags;

/*	int i;*/

	char DA[MAC_ADDR_LEN];
	mem_MSDU_T *pMsdu = NULL;
    UINT8 wone_index = 0;

	if(REPORT_LEVEL(RL_WPA)) 
		wdrv_utilsReport(RL_WPA, PRI_EMERGENCY, "WDRV_WPA_KERNEL: 'send_eapol()' \n");
#if 0 
	PAL_sleepLong(1000);
	printk("\nKERNEL:EPOOL LEN = %d", len);

	for(i = 0;i<len;i++)
	{
		printk("\npct %d,0x%x\n",i,((char*)buf)[i]);
	}
#endif


	memcpy(DA, buf, MAC_ADDR_LEN);	
	
	rc = wdrv_dbFindStaIndex(DA, &idx); 

	if (rc != OK)
	{
		if(REPORT_LEVEL(RL_WPA)) 
			wdrv_utilsReport(RL_WPA, PRI_EMERGENCY, "WDRV_WPA_KERNEL: error in 'send_eapol()': destination has no index \n");
		return 0;
	}
	else if (!wdrv_mgmtIsStaAssoc(idx))
	{
		if(REPORT_LEVEL(RL_WPA)) 
			wdrv_utilsReport(RL_WPA, PRI_EMERGENCY, "WDRV_WPA_KERNEL: error in 'send_eapol()': destination is not associated\n");
		return 0;
	}

	pMsdu = formatEthernetToEapol(buf, len, DA);
	if (pMsdu == NULL)
	{
		if(REPORT_LEVEL(RL_WPA)) 
			wdrv_utilsReport(RL_WPA, PRI_EMERGENCY, "WDRV_WPA_KERNEL: MSDU is NULL\n");
		return 0;
	}

	PAL_intrLock(flags);
    wdrv_dbGetStationWoneIndex(idx, &wone_index);
	if (wone_index >= MAX_WONE_NET)
	{
		printf("%s: staIndex=%d, woneIndex = %d  !!!\n",__func__,idx,wone_index);
		wlan_memMngrFreeMSDU(pMsdu->handle);
		return 0;
	}

	/* Send it to the Tx module, we always send it in the mgmt queue */
	txFlags = WDRV_TX_HIGH_PRIORITY | WDRV_TX_UNICAST_MSDU | WDRV_TX_EAPOL_FRAME;
	
	if (encrypt == FALSE)
		txFlags |= WDRV_TX_FORCE_UN_ENCRYPTED; /* We send EAPOLs via the mgmt queue. */
		
	wdrv_txSendMsdu(pMsdu, idx, txFlags,wone_index);
	PAL_intrUnLock(flags);

	return len;
}


/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/
STATUS dda_wpa_wait_for_wpaHello(void)
{
	if((wdrv_DDA_sleep_timeout((void*)&WaitForWpaModule ,WAIT_FOR_WPA_MODULE_TIMEOUT)) == 0)
		return NOK;
	return OK;
}
/*******************************************************************************
DESCRIPTION:	   
                                                                         
INPUTS:			

OUTPUT:			

RETURNS:			                                                    
*******************************************************************************/  
void   dda_wpa_receive_hello(void) {
	int i;
	wdrv_DDA_wake((void*)&WaitForWpaModule);
#if 1
	/* init recovery struct */
	for(i = 0;i<MAX_STATIONS;i++)
	{
		wpa_recovery.stations[i].inUse = FALSE;
	}
#endif
}
