/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/


/***************************************************************************/
/*                                                                         */
/*    MODULE:	PAL                                                        */
/*    PURPOSE:	pal definitions                                            */
/*    authors:	Felix Rubinstein   <felixr@ti.com>                         */
/*              Shlomi Atias       <shlomi.atias@ti.com>                   */
/*                                                                         */
/***************************************************************************/



#ifndef __PAL_H
#define __PAL_H

/****************************
    includes
*****************************/

/*#include <linux/modversions.h>*/
#include <linux/config.h> /* for board type */
#include <linux/spinlock.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <asm/cacheops.h>
/*#include <asm-mips/softirq.h>*/
#include <linux/interrupt.h>
#include <linux/timer.h>
#include <asm/atomic.h>
#include <asm/byteorder.h>
#include "pform_types.h"

/* basic data types definitions */
typedef char Char; 
typedef u8 Uint8;
typedef u16 Uint16;
typedef u32 Uint32;
typedef int PAL_Result;
typedef void * Ptr;
typedef spinlock_t PAL_spinLockHandle;
typedef struct timer_list PALSRV_TcbObjTYPE;
typedef int PAL_MemAddrSpace;
typedef atomic_t PAL_atomic_t;

/*
	For debug
*/

#define FUNC printk("%s : %d\n", __FUNCTION__, __LINE__)

#define PAL_SOK (0)


/* macros definitions */

/* key definition */
#define RL_ROOT		1
#define RL_TEST		2
#define RL_TCB			3
#define RL_IDLE		4
#define RL_DB			5
#define RL_MAINSM	   6
#define RL_MGMT	   7
#define RL_HAL	      8
#define RL_TX        9
#define RL_SM	   10
#define RL_RX	   11
#define RL_RX_SNIFF	   12
#define RL_TX_SNIFF	   13
#define RL_BRIDGE	   14
#define RL_WPA			15
#define RL_HTTP	16
#define RL_4X		17
#define RL_MEM		18
#define RL_DYNAMIC_ALG	19

/* level definition */
typedef enum {
   PRI_ALWAYS_PRINT = 1,
   PRI_EMERGENCY,
   PRI_ALERT,
   PRI_CRITICAL,
   PRI_ERROR,
   PRI_WARNING,
   PRI_NOTICE,
   PRI_INFORMATION,
   PRI_DEBUG
} Priority_T;

extern signed int pal_replev;

struct pal_tcb_list {
	struct list_head list;
	PALSRV_TcbObjTYPE * timer;
	int pending, expires;
};

extern struct list_head pal_tcb_list_head;

extern BOOL recoveryIsInProcess;

#define TIWLAN_DEBUG_PRINT printk("%s : %d\n", __FUNCTION__, __LINE__)

/* PALSRV_report - Print routine implemented with debug levels
   key: the modules bitmask
   level: the debug level
   len: is ignored
   format: the format string

   Note: uses gcc extension namely - Macros with a Variable Number of Arguments
   TODO: The level argument should be dynamically mapped to one of linux kernel defines, i.e. KERN_DEBUG
 */
#define REPORT_DEBUG 0 /* for ReportMode debugging */

#define PALSRV_report(key, level, len, format, args...) \
do { \
			printk("%lu: ", jiffies/(1000/HZ)); \
			printk(format, ##args);\
} while(0)

/* PALSRV_report_set - set key to the modules bitmask
   key: the key value to set
 */
#define PALSRV_report_set(key, level) \
	do { pal_replev |= 1 << (key); } while(0)
	
/* PALSRV_report_unset - unset key from the modules bitmask
   key: the key value to unset
 */
#define PALSRV_report_unset(key, level) \
	do { pal_replev &= ~(1 << (key)); } while(0)

#define PALSRV_report_disable(level) \
	do { pal_replev = 0; } while(0)

#define PALSRV_report_get() pal_replev
/*
#else
#define PALSRV_report(key, level, len, format, args...)
#define PALSRV_report_set(key, level)
#define PALSRV_report_unset(key, level)
#define PALSRV_report_disable(level)
#endif
*/
#define printf(s,args...)	PALSRV_report(0,PRI_ALWAYS_PRINT,0,s,##args)
#define logMsg(s,args...)	PALSRV_report(0,PRI_ALWAYS_PRINT,0,s,##args)
#define wdrv_utilsReport(key, level, s, args...) PALSRV_report(key,level,0,s,##args)
#define puts(s, args...)	printf(s, ##args)	
	
#ifdef TIWLAN_DEBUG
	#define REPORT_LEVEL(key)	(pal_replev & (1<<(key)))
#else
	#define REPORT_LEVEL(key) 0
#endif

/* endianess converion */

#ifdef __LITTLE_ENDIAN
#define PAL_LITTLE_ENDIAN
#endif

#define PAL_LE2UINT16 le16_to_cpu
#define PAL_BE2UINT16 be16_to_cpu
#define	PAL_LE2UINT32 le32_to_cpu
#define	PAL_BE2UINT32 be32_to_cpu

#define ENDIAN_HANDLE_WORD  PAL_LE2UINT16
#define ENDIAN_HANDLE_LONG	PAL_LE2UINT32

/* PAL_spinLock_init - Initialize spin lock
   lock: the spin lock */
extern atomic_t spinLockNestCnt;
extern atomic_t intrLockNestCnt;

#define PAL_lockNesting_init() \
	(atomic_set(&spinLockNestCnt,0));\
	(atomic_set(&intrLockNestCnt,0));	

#define PAL_spinLock_init(lock) \
	({ spin_lock_init(lock); PAL_SOK; })
	
#define PAL_spinLockNestInc() \
	({atomic_inc(&spinLockNestCnt);})
	
#define PAL_spinLockNestDec() \
	({atomic_dec(&spinLockNestCnt);})

/* PAL_spinLock - Perform the following:
   1. TODO: Disable ACX interrupts
   2. Lock the spin lock and disables bottom halves.
 */
#define PAL_spinLock(lock) \
	do {  if(atomic_add_return(1, &spinLockNestCnt) == 1) spin_lock_bh(lock);\
	      else spin_lock(lock); } while(0)

/* PAL_spinUnLock - Perform the following:
   1. Unlock the spin lock and enable bottom halves.
   2. TODO: Disable ACX interrupts
 */

#define PAL_spinUnLock(lock) \
	do {  if(atomic_sub_return(1, &spinLockNestCnt) == 0) spin_unlock_bh(lock);\
		  else spin_unlock(lock); } while(0)

#define PAL_intrLock(flags) local_irq_save(flags)
#define PAL_intrUnLock(flags) local_irq_restore(flags)

#if 0
#define PAL_intrLock(flags) \
	do { if(atomic_add_return(1, &intrLockNestCnt) == 1) local_irq_save(flags); } while (0)
	
#define PAL_intrUnLock(flags) \
	do { if(atomic_sub_return(1, &intrLockNestCnt) == 0) local_irq_restore(flags); } while (0)

#endif

/* Atomic handling */
#define PAL_atomic_set(var, val) \
	do { atomic_set(var, val); } while (0);
	
#define PAL_atomic_read(var, val) \
	do { val = atomic_read(var); } while (0);

	
/* PAL_sleepShort - Delay execution for usecs microseconds
   Can be called from interrupt
   Note: if you know for sure what you're doing 
   #define notdef 
   before including linux/delay.h (I've warned you!)
 */
#define PAL_sleepShort(usecs) mdelay(usecs)


/* don't remove n, since constant argumentis are perfectly possible */
#define msecs_to_jiffies(msecs)						\
	({ 								\
		unsigned long n = msecs; 				\
		if ((unsigned long) n < MAX_SCHEDULE_TIMEOUT / HZ) 	\
			n = (unsigned long)(n*HZ+999)/1000+1; 		\
		else n = MAX_SCHEDULE_TIMEOUT; 				\
		n; 							\
	})

	
/* PAL_sleepLong - Delay execution for msecs milliseconds
 */	
#define PAL_sleepLong(msecs) 							\
	({	signed long timeout = (msecs); 					\
		while( timeout ) { 						\
			set_current_state(TASK_INTERRUPTIBLE); 			\
			timeout = schedule_timeout(msecs_to_jiffies(timeout)); 	\
		} 								\
		PAL_SOK; 							\
	})

#if 0
/* PALSRV_tcbNew - Initialize new timer */
#define PALSRV_tcbNew(timer) \
	({ struct pal_tcb_list * pal_tcb_list_elem = kmalloc(sizeof(struct pal_tcb_list), GFP_KERNEL); \
     init_timer(timer); \
     pal_tcb_list_elem->timer = timer; \
     list_add_tail(&pal_tcb_list_elem->list, &pal_tcb_list_head); \
     PAL_SOK; })

/* Since linux kernel is automatically removes itself from the list,
   defined as empty macro
 */
#define PALSRV_tcbDelete(timer) \
  ({ struct pal_tcb_list * pal_tcb_list_elem = NULL; \
     struct list_head * current = NULL; \
     del_timer(timer); \
     list_for_each(current, &pal_tcb_list_head) {\
       pal_tcb_list_elem = list_entry(current, struct pal_tcb_list, list); \
       if (pal_tcb_list_elem->timer == timer) {\
         list_del(pal_tcb_list_elem->list); \
         kfree(pal_tcb_list_elem); \
       } \
     } \
  })
  
/* PALSRV_tcbCancel - deactivate a timer and wait for the handler to finish 
   timer: the timer to be deactivated

   Note: It must not be called from interrupt contexts
 */
#define PALSRV_tcbCancel(timer) del_timer(timer)

#endif // 0

STATUS PALSRV_tcbNew(PALSRV_TcbObjTYPE * timer);
STATUS PALSRV_tcbDelete(PALSRV_TcbObjTYPE * timer);
STATUS PALSRV_tcbCancel(PALSRV_TcbObjTYPE * timer);
STATUS PALSRV_tcbSuspend(void);
STATUS PALSRV_tcbResume(void);

/* Palsrv_tcbQueueAct - start a timer
   timer: the timer to be added
   timeout: when the timer expires in milliseonds
   func: the function to be executed when the timer expires
   dat: data to be passed to the function
   periodic: ignored

   Note: don't change dat to data !!!
 */
#define PALSRV_tcbQueueAct(timer, timeout, func, dat, periodic) 	\
	({ 								\
	 	if( timeout ) { 					\
			(timer)->expires = jiffies + msecs_to_jiffies(timeout); 	\
			(timer)->data = (unsigned long)dat; 		\
			(timer)->function = (void (*)(unsigned long))func; 			\
			add_timer(timer); 				\
			PAL_SOK; 					\
		} 							\
		else ~PAL_SOK; 						\
	})


/* PALSRV_getTick - return number of clock ticks */
#define PALSRV_getTick() jiffies

/* PALSRV_msec2Ticks - convert number of milliseconds to number of clock ticks 
   msecs: number of milliseconds
 */
#define PALSRV_msec2ticks(msecs) msecs_to_jiffies(msecs)


/* PAL_vmPhy2VirtAddr - map physical address to virtual
   phyAddr: address to remap
   pVirtAddr: the result of the remap
 */
#define PAL_vmPhy2VirtAddr(phys, virt) \
		({ virt = phys_to_virt((volatile void *)(phys)); PAL_SOK; })

/* PAL_vmVirt2PhyAddr - map virtual address to physical
   pVirtAddr: address to remap
   pPhyAddr: the result of the remap
 */
#define PAL_vmVirt2PhyAddr(virt, phys) \
		({ phys = virt_to_phys((volatile void *)(virt)); PAL_SOK; })

#ifdef TIWLAN_DEBUG
extern atomic_t pal_alloc_count;
#endif

/* PAL_memAlloc - allocate memory
   size: how many bytes of memory are required
   segId: ignored
   alignment: ignored
   memAddr: ignored


   GFP_KERNEL - Allocate normal kernel ram.  May sleep.
 */
#ifdef TIWLAN_DEBUG
#define PAL_memAlloc(segId, size, alignment, memAddr) (atomic_add(1, &pal_alloc_count), kmalloc(size, GFP_KERNEL))
#else
#define PAL_memAlloc(segId, size, alignment, memAddr) kmalloc(size, GFP_KERNEL)
#endif
		
/* PAL_memFree - free previously allocated memory
   addr: pointer returned by PAL_memAlloc
 */
#ifdef TIWLAN_DEBUG
#define PAL_memFree(segId, addr, numBytes) (atomic_sub(1, &pal_alloc_count), kfree(addr) )
#else
#define PAL_memFree(segId, addr, numBytes) kfree(addr)
#endif


/* PAL_memCopy - Copy one area of memory to another
   dest: Where to copy to
   src: Where to copy from
   count: The size of the area
 */
#define PAL_memCopy(dest, src, count) memcpy(dest, src, count)

/* PAL_memSet - Fill a region of memory with the given value
   s: Pointer to the start of the area
   c: The byte to fill the area with
   count: The size of the area
 */
#define PAL_memSet(s, c, count) memset(s, c, count)


/* PAL_memCmp - Compare two areas of memory
   cs: One area of memory
   ct: Another area of memory
   count: The size of the area
 */
#define PAL_memCmp(cs, ct, count) memcmp(cs, ct, count)


/* excerted from include/asm-mips/r4kcache.h */
#define cache_op(op,addr)                                               \
	__asm__ __volatile__(                                           \
	"       .set    noreorder               \n"                     \
	"       .set    mips3\n\t               \n"                     \
	"       cache   %0, %1                  \n"                     \
	"       .set    mips0                   \n"                     \
	"       .set    reorder"                                        \
	:                                                               \
	: "i" (op), "m" (*(unsigned char *)(addr)))


#define cache_align_addr(a, end, addr, cl_size) 	\
do { 							\
	a = (addr) & ~((cl_size) - 1); 			\
	end = ((addr) + (size) - 1) & ~((cl_size) - 1); \
} while(0)

/* PAL_cacheInvalidate - Invalidate cache address
   addrSpace: ignored
   addr: address to invalidate from
   size: size byte
 */

__inline__ static PAL_Result PAL_cacheInvalidate(PAL_MemAddrSpace addrSpace, Ptr addr, Uint32 size)
{
	unsigned long a, end, cl_size = 16;

		/* no need to do it twice, since anyway size will be verified beforehand*/
		cache_align_addr(a, end, (unsigned long)addr, cl_size);

		while (1) {
			cache_op(Hit_Invalidate_D, a);
			if (a == end) break;
			a += cl_size;
		}
	/*}*/
	return PAL_SOK;
}

#if defined(CONFIG_MIPS_PUMAS)
//#warning "Disabling cash flush on Puma-S"
#define PAL_cacheFlush(a, b, c) 

#else
/* PAL_cacheFlush - Flush cache address
   addrSpace: ignored
   addr: address to flush from
   size: size byte
 */
__inline__ static PAL_Result PAL_cacheFlush(PAL_MemAddrSpace addrSpace, Ptr addr, Uint32 size)
{
	unsigned long a, end, cl_size = 16;

	 /* no need to do it twice, since anyway size will be verified beforehand*/
		cache_align_addr(a, end, (unsigned long)addr, cl_size);

		while (1) {
			cache_op(Hit_Writeback_D, a);
			if (a == end) break;
			a += cl_size;
		}
	/*}*/
	return PAL_SOK;
}


#endif // CONFIG_MIPS_PUMA_S


#define _min(a,b) ( (a) < (b) ? (a) : (b))
#define _max(a,b) ( (a) > (b) ? (a) : (b))


#endif /* __PAL_H */
