/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                ****                                  |**
**|                                ****                                  |**
**|                                ******o***                            |**
**|                          ********_///_****                           |**
**|                           ***** /_//_/ ****                          |**
**|                            ** ** (__/ ****                           |**
**|                                *********                             |**
**|                                 ****                                 |**
**|                                  ***                                 |**
**|                                                                      |**
**|     Copyright (c) 1998,1999,2000 Texas Instruments Incorporated      |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**| Permission is hereby granted to licensees of Texas Instruments       |**
**| Incorporated (TI) products to use this computer program for the sole |**
**| purpose of implementing a licensee product based on TI products.     |**
**| No other rights to reproduce, use, or disseminate this computer      |**
**| program, whether in part or in whole, are granted.                   |**
**|                                                                      |**
**| TI makes no representation or warranties with respect to the         |**
**| performance of this computer program, and specifically disclaims     |**
**| any responsibility for any damages, special or consequential,        |**
**| connected with the use of this program.                              |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

/**************************************************************************/
/*                                                                        */
/*   MODULE:  httpfunc.c                                                  */
/*   PURPOSE: HTTP functions										      */
/*                                                                        */
/**************************************************************************/

#if 0
#include "pform_types.h"
#include "whalNVRAM.h"
#include "802_11Defs.h"
#include "release.h"
#include "bspnvram.h" 
#include "app_driver_api.h"
#endif

#include <stdio.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "wcfglib.h"
#include "wcfg_ioctl.h"
#include "wcfg_net.h"
#include "cli.h"
#include "dda_proc_fs_defs.h"
#include "cfg_proc_if.h"
#include "drvDbRegDomainDefs.h"

#define	MAX_PRINT_STRING		128		

/* this definition is very important, it fixes the maximum number of bytes that can be returned to the html page.
		If you return a string that is bigger than this size, you will have a memory overflow. */
/*#define	MAX_RETURNED_STRING_LEN	800*/
#define	MAX_RETURNED_STRING_LEN	4096*8


extern wcfgd *wd;
struct storage storage_backup;

extern unsigned char htol(char str[]);

unsigned char hexChar2int(unsigned char c);

static char tempbuf[MAX_RETURNED_STRING_LEN];

static UINT16	stationCurrentIndex;
static UINT16   tableBW_CurrentIndex;
static UINT16   tableSSID_CurrentIndex;

static void create_js_msg_box(char *buf, char *message);

unsigned char hex(unsigned char c)
{
	if (c >= 'A' && c <= 'F')
		return (c - 'A' + 10);
	else if (c >= 'a' && c <= 'f')
		return (c - 'a' + 10);
	return c - '0';
}

unsigned char htol(char str[])
{
	unsigned char n;

	n = hex(str[0]) << 4;
	n |= hex(str[1]);
	return n;
}


typedef struct {    /* This is a storage for a list of Channel Range elements */
	tiap_param_cur_reg_domain_t cur_regdomain;
	UINT8                       ranges_nr;
	tiap_param_channel_rang_elem_t *rang_elem;
} cli_channel_range_tbl_elem_t;

/* 802.11d Allowed Channel Tables - hardcoded values */
static cli_channel_range_tbl_elem_t ch_rang_a[NUM_OF_REGULATORY_DOMAINS_A];
static cli_channel_range_tbl_elem_t ch_rang_b[NUM_OF_REGULATORY_DOMAINS_B];


int create_listening_socket(__u32);
int close_listening_socket(void);
	
static void restart(void)
{
	/* a trick.
	 * update storage.
	 */
	tiap_param_security_type_t security_type;


	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_SECURITY_TYPE, &security_type, FROM_STORAGE) )
		PRINT_CONFIG_ERR_MSG("Failed to get security type.");
	else {
		fflush(NULL);
		if(tiap_wcfg_set_basic(wd, TIAP_PARAM_SECURITY_TYPE, &security_type, UPDATE_STORAGE))
			PRINT_CONFIG_ERR_MSG("Failed to set security type.");
	}

	close_listening_socket();
	
	/*restart library*/
#if 0
	if( !(wd = tiap_wcfg_restart(wd)) ) {
		fprintf(stderr, "tiweb: Sorry.\n");
		exit(1);
	}
#else
	tiap_wcfg_system_reboot();
#endif

	if( create_listening_socket(INADDR_ANY) ) {
		fprintf(stderr, "tiweb: Sorry, couldn't create listening socket.\n");
		exit(1);
	}
}



static int is_privacy_enabled(void)
{
	tiap_param_security_type_t security_type;
	__u8 priv_opt = 0;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_SECURITY_TYPE, &security_type, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get security type.");
		return 0;
	}

	if( security_type == TIAP_PARAM_SECURITY_WEP ) {
		if( tiap_wcfg_get_basic(wd, TIAP_PARAM_PRIV_OPTION, &priv_opt, FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get privacy option.");
			return 0;
		}
	}

	if( priv_opt || security_type == TIAP_PARAM_SECURITY_802_1X ||
			security_type == TIAP_PARAM_SECURITY_WPA )
		return 1;

	return 0;
}


void wdrv_dbInitRangeTables(void)
{
	ch_rang_a[0].cur_regdomain.type = TIAP_PARAM_RD_US;
	ch_rang_a[0].ranges_nr = NUM_OF_RANGE_ELEMENTS_US;
	if( !(ch_rang_a[0].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t)*2)) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}

	ch_rang_a[0].rang_elem[0].start = REG_DOMAIN_US_START_CH_0;
	ch_rang_a[0].rang_elem[0].stop = REG_DOMAIN_US_STOP_CH_0;
	ch_rang_a[0].rang_elem[0].step = REG_DOMAIN_US_STEP_OPT_0;
	ch_rang_a[0].rang_elem[0].power_limit = REG_DOMAIN_US_POWER_LIMIT_CH_0;
	ch_rang_a[0].rang_elem[1].start = REG_DOMAIN_US_START_CH_1;
	ch_rang_a[0].rang_elem[1].stop = REG_DOMAIN_US_STOP_CH_1;
	ch_rang_a[0].rang_elem[1].step = REG_DOMAIN_US_STEP_OPT_1;
	ch_rang_a[0].rang_elem[1].power_limit = REG_DOMAIN_US_POWER_LIMIT_CH_1;

	ch_rang_a[1].cur_regdomain.type = TIAP_PARAM_RD_WORLD;
	ch_rang_a[1].ranges_nr = NUM_OF_RANGE_ELEMENTS_WORLD;
	if( !(ch_rang_a[1].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t)*3)) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_a[1].rang_elem[0].start = REG_DOMAIN_WORLD_START_CH_0;
	ch_rang_a[1].rang_elem[0].stop  = REG_DOMAIN_WORLD_STOP_CH_0;
	ch_rang_a[1].rang_elem[0].step = REG_DOMAIN_WORLD_STEP_OPT_0;
	ch_rang_a[1].rang_elem[0].power_limit = REG_DOMAIN_WORLD_POWER_LIMIT_CH_0;
	ch_rang_a[1].rang_elem[1].start = REG_DOMAIN_WORLD_START_CH_1;
	ch_rang_a[1].rang_elem[1].stop  = REG_DOMAIN_WORLD_STOP_CH_1;
	ch_rang_a[1].rang_elem[1].step = REG_DOMAIN_WORLD_STEP_OPT_1;
	ch_rang_a[1].rang_elem[1].power_limit = REG_DOMAIN_WORLD_POWER_LIMIT_CH_1;
	ch_rang_a[1].rang_elem[2].start = REG_DOMAIN_WORLD_START_CH_2;
	ch_rang_a[1].rang_elem[2].stop = REG_DOMAIN_WORLD_STOP_CH_2;
	ch_rang_a[1].rang_elem[2].step = REG_DOMAIN_WORLD_STEP_OPT_2;
	ch_rang_a[1].rang_elem[2].power_limit = REG_DOMAIN_WORLD_POWER_LIMIT_CH_2;

	/*              - G - band              */
	ch_rang_b[0].cur_regdomain.type = TIAP_PARAM_RD_FCC;
	ch_rang_b[0].ranges_nr = 1;
	if( !(ch_rang_b[0].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[0].rang_elem[0].start = REG_DOMAIN_FCC_START_CH_0;
	ch_rang_b[0].rang_elem[0].stop  = REG_DOMAIN_FCC_STOP_CH_0;
	ch_rang_b[0].rang_elem[0].step = REG_DOMAIN_FCC_STEP_OPT_0;
	ch_rang_b[0].rang_elem[0].power_limit = REG_DOMAIN_FCC_POWER_LIMIT_CH_0;

	ch_rang_b[1].cur_regdomain.type = TIAP_PARAM_RD_IC;
	ch_rang_b[1].ranges_nr = 1;
	if( !(ch_rang_b[1].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[1].rang_elem[0].start = REG_DOMAIN_IC_START_CH_0;
	ch_rang_b[1].rang_elem[0].stop  = REG_DOMAIN_IC_STOP_CH_0;
	ch_rang_b[1].rang_elem[0].step = REG_DOMAIN_IC_STEP_OPT_0;
	ch_rang_b[1].rang_elem[0].power_limit = REG_DOMAIN_IC_POWER_LIMIT_CH_0;

	ch_rang_b[2].cur_regdomain.type = TIAP_PARAM_RD_ETSI;
	ch_rang_b[2].ranges_nr = 1;
	if( !(ch_rang_b[2].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[2].rang_elem[0].start = REG_DOMAIN_ETSI_START_CH_0;
	ch_rang_b[2].rang_elem[0].stop  = REG_DOMAIN_ETSI_STOP_CH_0;
	ch_rang_b[2].rang_elem[0].step = REG_DOMAIN_ETSI_STEP_OPT_0;
	ch_rang_b[2].rang_elem[0].power_limit = REG_DOMAIN_ETSI_POWER_LIMIT_CH_0;

	ch_rang_b[3].cur_regdomain.type = TIAP_PARAM_RD_SPAIN;
	ch_rang_b[3].ranges_nr = 1;
	if( !(ch_rang_b[3].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[3].rang_elem[0].start = REG_DOMAIN_SPAIN_START_CH_0;
	ch_rang_b[3].rang_elem[0].stop  = REG_DOMAIN_SPAIN_STOP_CH_0;
	ch_rang_b[3].rang_elem[0].step = REG_DOMAIN_SPAIN_STEP_OPT_0;
	ch_rang_b[3].rang_elem[0].power_limit = REG_DOMAIN_SPAIN_POWER_LIMIT_CH_0;

	ch_rang_b[4].cur_regdomain.type = TIAP_PARAM_RD_FRANCE;
	ch_rang_b[4].ranges_nr = 1;
	if( !(ch_rang_b[4].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[4].rang_elem[0].start = REG_DOMAIN_FRANCE_START_CH_0;
	ch_rang_b[4].rang_elem[0].stop  = REG_DOMAIN_FRANCE_STOP_CH_0;
	ch_rang_b[4].rang_elem[0].step = REG_DOMAIN_FRANCE_STEP_OPT_0;
	ch_rang_b[4].rang_elem[0].power_limit = REG_DOMAIN_FRANCE_POWER_LIMIT_CH_0;

	ch_rang_b[5].cur_regdomain.type = TIAP_PARAM_RD_MKK;
	ch_rang_b[5].ranges_nr = 1;
	if( !(ch_rang_b[5].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t))) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[5].rang_elem[0].start = REG_DOMAIN_MKK_START_CH_0;
	ch_rang_b[5].rang_elem[0].stop = REG_DOMAIN_MKK_STOP_CH_0;
	ch_rang_b[5].rang_elem[0].step = REG_DOMAIN_MKK_STEP_OPT_0 | NO_OFDM_OPT;
	ch_rang_b[5].rang_elem[0].power_limit = REG_DOMAIN_MKK_POWER_LIMIT_CH_0;

	ch_rang_b[6].cur_regdomain.type = TIAP_PARAM_RD_MKK1;
	ch_rang_b[6].ranges_nr = 2;
	if( !(ch_rang_b[6].rang_elem =
				(tiap_param_channel_rang_elem_t *)malloc(sizeof(tiap_param_channel_rang_elem_t)*2)) ) {
		fprintf(stderr, "Allocation error.\n\n");
		exit(1);
	}
	ch_rang_b[6].rang_elem[0].start = REG_DOMAIN_MKK1_START_CH_0;
	ch_rang_b[6].rang_elem[0].stop  = REG_DOMAIN_MKK1_STOP_CH_0;
	ch_rang_b[6].rang_elem[0].step = REG_DOMAIN_MKK1_STEP_OPT_0;
	ch_rang_b[6].rang_elem[0].power_limit = REG_DOMAIN_MKK1_POWER_LIMIT_CH_0;
	ch_rang_b[6].rang_elem[1].start = REG_DOMAIN_MKK1_START_CH_1;
	ch_rang_b[6].rang_elem[1].stop  = REG_DOMAIN_MKK1_STOP_CH_1;
	ch_rang_b[6].rang_elem[1].step = REG_DOMAIN_MKK1_STEP_OPT_1 | NO_OFDM_OPT;
	ch_rang_b[6].rang_elem[1].power_limit = REG_DOMAIN_MKK1_POWER_LIMIT_CH_1;
}



static int get_domain_elems_num(int band, __u8 domain)
{
	switch (band)
	{
		case BAND_A:
			switch (domain)
			{
				case TIAP_PARAM_RD_US:
					return NUM_OF_RANGE_ELEMENTS_US;

				case TIAP_PARAM_RD_WORLD:
					return NUM_OF_RANGE_ELEMENTS_WORLD;

				default:
					break;
			}
			break;

		case BAND_B:
			switch (domain)
			{
				case TIAP_PARAM_RD_FCC:
					return 1;

				case TIAP_PARAM_RD_IC:
					return 1;

				case TIAP_PARAM_RD_ETSI:
					return 1;

				case TIAP_PARAM_RD_SPAIN:
					return 1;

				case TIAP_PARAM_RD_FRANCE:
					return 1;

				case TIAP_PARAM_RD_MKK:
					return 1;

				case TIAP_PARAM_RD_MKK1:
					return 2;

				default:
					break;
			}
			break;

		default:
			break;
	}
	return 0;
}


int wdrv_dbGetRangeElement(int band,
		tiap_param_cur_reg_domain_t cur_reg_domain,
		UINT8         RangeIndex,
		tiap_param_channel_rang_elem_t *rang_elem)
{
	if (RangeIndex == 0)    /* index is a positive number */
		return 1;

	switch (band)
	{
		case BAND_A:
			switch((__u8)cur_reg_domain.type)
			{
				case TIAP_PARAM_RD_US:
					if (RangeIndex > NUM_OF_RANGE_ELEMENTS_US)
						return 1;
					memcpy(rang_elem, &ch_rang_a[0].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_WORLD:
					if (RangeIndex > NUM_OF_RANGE_ELEMENTS_WORLD)
						return 1;
					memcpy(rang_elem, &ch_rang_a[1].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				default: return 1;
			}
			break;
		case BAND_B:
			switch((__u8)cur_reg_domain.type)
			{
				case TIAP_PARAM_RD_FCC:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[0].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_IC:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[1].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_ETSI:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[2].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_SPAIN:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[3].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_FRANCE:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[4].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_MKK:
					if (RangeIndex > 1)
						return 1;
					memcpy(rang_elem, &ch_rang_b[5].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				case TIAP_PARAM_RD_MKK1:
					if (RangeIndex > 2)
						return 1;
					memcpy(rang_elem, &ch_rang_b[6].rang_elem[RangeIndex-1],
							sizeof(*rang_elem));
					break;

				default:
					return 1;
			}

			break;

		default:
			return 1;
	}
	return OK;
}



/********************************
*		SETUP PAGE				*	
*********************************/

/****************************************************************************/
/*                      wlan_htmlGetApMacAddress(void)							*/
/****************************************************************************/
/* DESCRIPTION: Returns AP mac address. 									*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     AP mac address												*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetApMacAddress(void)
{
#if 0
	char address[MAC_ADDR_LEN];	

	tempbuf[0] = 0;
	
	wdrv_dbGetOperationMacAddress(address);
	
	sprintf(tempbuf,"%02X-%02X-%02X-%02X-%02X-%02X",
            address[0],address[1],address[2],address[3],address[4],address[5]);

	return tempbuf;
#endif

	int s;
	struct ifreq ifr;
	__u8 p[ETH_ALEN];

	if( (s =  socket(AF_INET, SOCK_DGRAM, 0)) == -1 ) {
		perror("");
		return NULL;
	}

	strncpy(ifr.ifr_name, wd->dev_name, IFNAMSIZ);
	ifr.ifr_hwaddr.sa_family = ARPHRD_ETHER;

	if( ioctl(s, SIOCGIFHWADDR, &ifr) < 0) {
		perror("");
		return NULL;
	}

	memcpy(p, ifr.ifr_hwaddr.sa_data, ETH_ALEN);
	close(s);
		
	sprintf(tempbuf,"%02X-%02X-%02X-%02X-%02X-%02X",
            p[0], p[1], p[2], p[3], p[4], p[5]);

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetChannel(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns AP channel. 										*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     AP channel													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetChannel(void)
{
#if 0
	unsigned char chanId;

	tempbuf[0] = 0;
	
	wdrv_dbGetChannel(0,&chanId);
	sprintf(tempbuf,"%d",chanId);

	return tempbuf;
#endif
	return NULL;
}

/****************************************************************************/
/*                      wlan_htmlGetSsid(void)									*/
/****************************************************************************/
/* DESCRIPTION: Returns the SSID. 											*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     SSID														*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetSsid(void)
{
#if 0
   char tempbuf2[DOT11_SSID_MAX+1];
	tempbuf2[0] = 0;
	
	wdrv_dbGetSsid(tempbuf2);
   sprintf(tempbuf, "\"%s\"", tempbuf2);
	return tempbuf;
#endif

	char ssid[IW_ESSID_MAX_SIZE+1];
	int i, j;
	
	if( tiap_wcfg_giwessid(wd, ssid, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get ssid.");
		return NULL;
	}
	
	strcpy(tempbuf, "\"");

	for(i=0, j=strlen(tempbuf); i<=strlen(ssid); i++, j++) {
		if( ssid[i] == '"' ) {
			tempbuf[j] = '\0';
			strcat(tempbuf, "&#148");
			j = strlen(tempbuf) - 1;
		}
		else tempbuf[j] = ssid[i];
	}
	
	strcat(tempbuf, "\"");

	return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlGetHiddenSsid(void)						    */
/****************************************************************************/
/* DESCRIPTION: Returns the SSID. 											*/ 
/*																		    */
/*																		    */
/* INPUT:       value of radio button                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     Hidden SSID Checked button                                  */
/*                                                                          */
/****************************************************************************/
char* wlan_htmlGetHiddenSsid(char *value)
{
#if 0
    UINT16 HiddenSSID;
	int mode;

	sscanf(value,"%d",&mode);
    
    wdrv_dbGetHiddenSsidStatus(&HiddenSSID);

    switch (HiddenSSID)
    {
        case 0:
            if (mode == 2) 
                return "checked";
            break;
        case 1:
            if (mode == 1) 
                return "checked";
            break;
    }
    
    return " ";
#endif

    __u16 hidden_ssid;
    int mode;

    sscanf(value,"%d",&mode);

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_HIDDEN_SSID, &hidden_ssid, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get hidden ssid.");
	    return NULL;
    }

    switch (hidden_ssid)
    {
	    case 0:
		    if (mode == 2) 
			    return "checked";
		    break;
	    case 1:
		    if (mode == 1) 
			    return "checked";
		    break;
    }

    return " ";

}


/****************************************************************************/
/*                      wlan_htmlVhcfCheck(void)								*/
/*                      wlan_htmlVhcfGetIpAddr1(void)							*/
/*                      wlan_htmlVhcfGetIpAddr2(void)							*/
/*                      wlan_htmlVhcfGetIpAddr3(void) 							*/
/*                      wlan_htmlVhcfGetIpAddr4(void) 							*/
/*                      wlan_htmlVhcfGetProtocol1(void)							*/
/*                      wlan_htmlVhcfGetPort1(void)								*/
/*                      wlan_htmlVhcfGetProtocol2(void)							*/
/*                      wlan_htmlVhcfGetPort2(void)								*/
/****************************************************************************/
/*																		    */
/* DESCRIPTION:  Return the vHCF parameters current status					*/ 
/*																		    */
/****************************************************************************/
char* wlan_htmlVhcfCheck(char* value)
{
#if 0
	int vhcfMode;
	UINT8 TotalWones;

	sscanf(value,"%d",&vhcfMode);

	wdrv_dbGetNumSupWone(&TotalWones);

	if (TotalWones == WONE_DUAL_BAND)
	{
		if (vhcfMode == 2) 
            return "checked disabled";
		else
            return " disabled";
	}

	if (wdrv_dbIsVhcfEnabled(void))
	{
        if (vhcfMode == 1)      
            return "checked";
		else
		    return " ";
	}
	else
	{
		if (vhcfMode == 2) 
            return "checked";
		else
			return " ";
    }
#endif

	int vhcfMode;
	__u8 wone_nr;
	tiap_param_vhcf_t vhcf;
	tiap_param_4x_t _4x;



	sscanf(value,"%d",&vhcfMode);

	/* If Dual AP is enabled, don't allow to set vHCF */
	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return NULL;
	}

	/* If 4x is enabled, don't allow to set vHCF */
	_4x.cmd = TIAP_PARAM_4X_4X;
	if (tiap_wcfg_get_4x(wd, &_4x, FROM_STORAGE))
	{
		PRINT_CONFIG_ERR_MSG("Reading configuration failed.");
		return NULL;
	}

	if (wone_nr == TIAP_PARAM_WONE_DUAL_BAND || _4x.value == TRUE) {
		if (vhcfMode == 2) 
			return "checked disabled";
		else
			return " disabled";
	}

	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return NULL;
	}

	if (vhcf.enable) {
		if (vhcfMode == 1)      
			return "checked";
		else
			return " ";
	}
	else {
		if (vhcfMode == 2) 
			return "checked";
		else
			return " ";
	}

}


char* wlan_htmlVhcfGetIpAddr1(void)
{
#if 0
    tVHCF_PARAMS  vhcfParams;

	tempbuf[0] = 0;
    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */
    sprintf(tempbuf,"%d",(vhcfParams.vhcfClassifier[0].DestIp & 0xFF000000)>>24);
	return tempbuf;
#endif

	tiap_param_vhcf_t vhcf;
	char *ip_str;
	int dummy,dig;
	struct in_addr in;

	tempbuf[0] = 0;
	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return NULL;
	}
	
	in.s_addr = vhcf.classifier[0].dest_ip;
	ip_str = inet_ntoa(in);
	sscanf(ip_str,"%d.%d.%d.%d",&dig, &dummy,&dummy,&dummy);
	
	sprintf(tempbuf,"%d",dig);
	
	
	return tempbuf;

}

char* wlan_htmlVhcfGetIpAddr2(void)
{
#if 0
    tVHCF_PARAMS  vhcfParams;

	tempbuf[0] = 0;
    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */
    sprintf(tempbuf,"%d",(vhcfParams.vhcfClassifier[0].DestIp & 0x00FF0000)>>16);
	return tempbuf;
#endif
	tiap_param_vhcf_t vhcf;
	char *ip_str;
	int dummy, dig;
	struct in_addr in;


	tempbuf[0] = 0;
	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return NULL;
	}

	in.s_addr = vhcf.classifier[0].dest_ip;
	ip_str = inet_ntoa(in);
	sscanf(ip_str,"%d.%d.%d.%d", &dummy,&dig,&dummy,&dummy);
	sprintf(tempbuf,"%d",dig);
	return tempbuf;

}

char* wlan_htmlVhcfGetIpAddr3(void)
{
#if 0
    tVHCF_PARAMS  vhcfParams;

	tempbuf[0] = 0;
    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */
    sprintf(tempbuf,"%d",(vhcfParams.vhcfClassifier[0].DestIp & 0x0000FF00)>>8);
	return tempbuf;
#endif
	tiap_param_vhcf_t vhcf;
	char *ip_str;
	int dummy, dig;
	struct in_addr in;

	tempbuf[0] = 0;
	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return NULL;
	}

	in.s_addr = vhcf.classifier[0].dest_ip;
	ip_str = inet_ntoa(in);
	sscanf(ip_str,"%d.%d.%d.%d", &dummy,&dummy,&dig,&dummy);
	sprintf(tempbuf,"%d",dig);

	return tempbuf;
}

char* wlan_htmlVhcfGetIpAddr4(void)
{
#if 0
    tVHCF_PARAMS  vhcfParams;

	tempbuf[0] = 0;
    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */
    sprintf(tempbuf,"%d",(vhcfParams.vhcfClassifier[0].DestIp & 0x000000FF));
	return tempbuf;
#endif
	tiap_param_vhcf_t vhcf;
	char *ip_str;
	int dummy, dig;
	struct in_addr in;

	tempbuf[0] = 0;
	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return NULL;
	}

	in.s_addr = vhcf.classifier[0].dest_ip;
	ip_str = inet_ntoa(in);
	sscanf(ip_str,"%d.%d.%d.%d", &dummy,&dummy,&dummy,&dig);
	sprintf(tempbuf,"%d",dig);

	return tempbuf;

}

char* wlan_htmlVhcfGetProtocol1(char *option)
{
#if 0
    tVHCF_PARAMS  vhcfParams;
	int inputOption;

    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */

    sscanf(option,"%d",&inputOption);

	if (inputOption == vhcfParams.vhcfClassifier[0].Protocol + 1)
		return "selected";
	else
		return " ";
#endif

	tiap_param_vhcf_t vhcf;
	int inputOption;

	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return NULL;
	}
	
	sscanf(option,"%d",&inputOption);
	
	if (inputOption == vhcf.classifier[0].proto + 1)
		return "selected";
	else return " ";
}

char* wlan_htmlVhcfGetPort1(void)
{
#if 0
    tVHCF_PARAMS  vhcfParams;

	tempbuf[0] = 0;
    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */
    sprintf(tempbuf,"%d",vhcfParams.vhcfClassifier[0].DestPort);
	return tempbuf;
#endif

	tiap_param_vhcf_t vhcf;

	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return NULL;
	}

	sprintf(tempbuf,"%d", vhcf.classifier[0].dest_port);
	return tempbuf;
}

char* wlan_htmlVhcfGetProtocol2(char *option)
{
#if 0
    tVHCF_PARAMS  vhcfParams;
	int inputOption;

    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */

    sscanf(option,"%d",&inputOption);

	if (inputOption == vhcfParams.vhcfClassifier[1].Protocol + 1)
		return "selected";
	else
		return " ";
#endif

	tiap_param_vhcf_t vhcf;
	int inputOption;

	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return NULL;
	}

	sscanf(option,"%d",&inputOption);

	if (inputOption == vhcf.classifier[1].proto + 1)
		return "selected";
	else
		return " ";
}

char* wlan_htmlVhcfGetPort2(void)
{
#if 0
    tVHCF_PARAMS  vhcfParams;

	tempbuf[0] = 0;
    wdrv_dbGetVhcfParams(&vhcfParams);  /* Get vHCF current configuration. */
    sprintf(tempbuf,"%d",vhcfParams.vhcfClassifier[1].DestPort);
	return tempbuf;
#endif
	tiap_param_vhcf_t vhcf;

	if( tiap_wcfg_get_vhcf(wd, &vhcf, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return NULL;
	}

	sprintf(tempbuf,"%d", vhcf.classifier[1].dest_port);
	return tempbuf;

}




char* wlan_htmlNotYet(void)
{
    return " ";
}

char* wlan_htmlNotYetValue(char *value)
{
    return "X";
}

/****************************************************************************/
/*                   wlan_htmlGetPucketBurstSupport(void)                       */
/****************************************************************************/
/* DESCRIPTION: Returns the Video Blust status.                             */ 
/*                                                                          */
/*                                                                          */
/* INPUT:       value of radio button                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     Checked button                                              */
/*																			*/
/****************************************************************************/
char* wlan_htmlGetPucketBurstSupport(char *value)
{
#if 0   /* unsupported yet */
    UINT32 VideoBlustSupport = 0;
	int mode;

	sscanf(value,"%d",&mode);
    
    switch (VideoBlustSupport)
    {
        case 0:
            if (mode == 2) 
                return "checked";
            break;
        case 1:
            if (mode == 1) 
                return "checked";
            break;
    }
    
    return " ";
#else
    return " disabled=true ";
#endif
	return NULL;
}


/****************************************************************************/
/*                    wlan_htmlGetOutputPowerLevel(void)                        */
/****************************************************************************/
/* DESCRIPTION: Returns the Output Power Level.                             */ 
/*                                                                          */
/*                                                                          */
/* INPUT:       value of radio button                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     Checked button                                              */
/*																			*/
/****************************************************************************/
char* wlan_htmlGetOutputPowerLevel(char *value)
{
#if 0
    UINT32 VideoBlustSupport = 0;
    UINT8 PowerLevel;
	int mode;


    wdrv_dbGetPowerLevel(whal_ParamsGetRadioChannel(0),&PowerLevel);

	sscanf(value,"%d",&mode);
    
    switch (PowerLevel)
    {
        case 1:
            if (mode == 0) 
                return "checked";
            break;
        case 2:
            if (mode == 1) 
                return "checked";
            break;
        case 3:
            if (mode == 2) 
                return "checked";
            break;
        case 4:
            if (mode == 3) 
                return "checked";
            break;
        case 5:
            if (mode == 4) 
                return "checked";
            break;
    }
    
    return " ";
#endif

    UINT8 PowerLevel;
    char buf[4096];
    int mode, len;

    if( (len = dda_read_proc_entry(WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_POWER_LEVEL,
				    buf, sizeof(buf))) == -1 ) {
	    PRINT_PROC_READ_ERR(WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_POWER_LEVEL);
	    return NULL;
    }
    sscanf(buf, "%hhu", &PowerLevel);

    sscanf(value,"%d",&mode);

    switch (PowerLevel)
    {
	    case 1:
		    if (mode == 0) 
			    return "checked";
		    break;
	    case 2:
		    if (mode == 1) 
			    return "checked";
		    break;
	    case 3:
		    if (mode == 2) 
			    return "checked";
		    break;
	    case 4:
		    if (mode == 3) 
			    return "checked";
		    break;
	    case 5:
		    if (mode == 4) 
			    return "checked";
		    break;
    }

    return " ";
}


/****************************************************************************/
/*                       wlan_htmlGet_A_G_PlusMode(void)                        */
/****************************************************************************/
/* DESCRIPTION: Returns the Video Blust status.                             */ 
/*                                                                          */
/*                                                                          */
/* INPUT:       value of radio button                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     Checked button                                              */
/*																			*/
/****************************************************************************/
char* wlan_htmlGet_A_G_PlusMode(char *value)
{
#if 0
	int mode;
    BOOL enable4x;

	sscanf(value,"%d",&mode);

    wdrv_dbGet4xEnable(&enable4x);
    switch (enable4x)
    {
        case FALSE:
            if (mode == 2) 
                return "checked";
            break;
        case TRUE:
            if (mode == 1) 
                return "checked";
            break;
    }
    
    return " ";
#endif

    int mode;
    tiap_param_4x_t _4x;
	tiap_param_vhcf_t vhcf;

    sscanf(value,"%d",&mode);


    _4x.cmd = TIAP_PARAM_4X_4X;
    if (tiap_wcfg_get_4x(wd, &_4x, !FROM_STORAGE)) {
	    PRINT_CONFIG_ERR_MSG("Failed to get 4x.");
	    return NULL;
    }

	if( tiap_wcfg_get_vhcf(wd, &vhcf, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
		return NULL;
	}
    
	
    switch (_4x.value)
    {
	    case FALSE:
		    if (mode == 2) 
			{
			    if (vhcf.enable) 
					return "checked disabled";
				else
					return "checked";
			}
			else
			{
				if (vhcf.enable) 
					return " disabled";
				else
					return " ";
			}
			break;
	    case TRUE:
		    if (mode == 1) 
			{
				if (vhcf.enable) 
					return "checked disabled";
				else
					return "checked";
			}
			else
			{
				if (vhcf.enable) 
					return " disabled";
				else
					return " ";

			}
		    break;
    }

    return " ";
}


/****************************************************************************/
/*                     wlan_htmlConfigEnergyDetection(void)                     */
/****************************************************************************/
/* DESCRIPTION: Returns the Interference Avoidance status.                  */ 
/*                                                                          */
/*                                                                          */
/* INPUT:       value of radio button                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     Checked button                                              */
/*																			*/
/****************************************************************************/
char* wlan_htmlConfigEnergyDetection(char *value)
{
#if 0
	int mode;
	UINT8 energyDetect;

	sscanf(value,"%d",&mode);

	wdrv_dbGetEnergyDetect(&energyDetect);
	switch (energyDetect)
	{
		case FALSE:
			if (mode == 2)
				return "checked";
			break;
        case TRUE:
            if (mode == 1) 
                return "checked";
            break;
	}
    
    return " ";
#endif

    int mode;
    __u8 energy_detect;

    sscanf(value,"%d",&mode);

    if(tiap_wcfg_get_basic(wd, TIAP_PARAM_ENERGY_DETECT, &energy_detect, !FROM_STORAGE)) {
	    PRINT_CONFIG_ERR_MSG("Failed to get energy detect configuration.");
	    return NULL;
    }

    switch (energy_detect)
    {
	    case FALSE:
		    if (mode == 2)
			    return "checked";
		    break;
	    case TRUE:
		    if (mode == 1) 
			    return "checked";
		    break;
    }

    return " ";

}

/****************************************************************************/
/*                     wlan_htmlConfigRadarDetectMode(void)                 */
/****************************************************************************/
/* DESCRIPTION: Returns the Radar detection mode							*/ 
/*                                                                          */
/*                                                                          */
/* INPUT:       value of radio button                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     Checked button                                              */
/*																			*/
/****************************************************************************/
char *wlan_htmlConfigRadarDetectMode(char *value)
{
    int mode;
    __u8 radar_detect;
    tiap_param_wone_t wone;
    __u8 wone_nr;

    sscanf(value,"%d",&mode);

    if(tiap_wcfg_get_basic(wd, TIAP_PARAM_RADAR_DETECT_MODE, &radar_detect, !FROM_STORAGE)) {
	    PRINT_CONFIG_ERR_MSG("Failed to get radar detect mode configuration.");
	    return NULL;
    }

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
	    return NULL;
    }

    if (wone_nr != TIAP_PARAM_WONE_DUAL_BAND)
    {
	    wone.index = 0;
	    wone.type = wone_band;
	    if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to get band.");
		    return NULL;
	    }

	    if( wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ )
		    return "disabled";
    }

    switch (radar_detect)
    {
	    case FALSE:
		    if (mode == 2)
			    return "checked";
		    break;
	    case TRUE:
		    if (mode == 1) 
			    return "checked";
		    break;
    }

    return " ";
}

/****************************************************************************/
/*                     wlan_htmlConfigBssIsolationMode(void)                */
/****************************************************************************/
/* DESCRIPTION: Returns the Bss Isolation mode							*/ 
/*                                                                          */
/*                                                                          */
/* INPUT:       value of radio button                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     Checked button                                              */
/*																			*/
/****************************************************************************/
char *wlan_htmlConfigBssIsolationMode(char *value)
{
    int mode;
    __u32 bss_bridge_state;

    sscanf(value,"%d",&mode);

    if(tiap_wcfg_get_basic(wd, TIAP_PARAM_BSS_BRIDGE_STATE, &bss_bridge_state, !FROM_STORAGE)) {
	    PRINT_CONFIG_ERR_MSG("Failed to get radar detect mode configuration.");
	    return NULL;
    }

    switch (bss_bridge_state)
    {
	    case TIAP_PARAM_BSS_BR_BLOCK:
		    if (mode == 1)
			    return "checked";
		    break;
	    default:
		    if (mode == 2) 
			    return "checked";
		    break;
    }

    return " ";
}



/****************************************************************************/
/*                      wlan_htmlSavePageSetup(void)							*/
/****************************************************************************/
/* DESCRIPTION: Update the driver database with the							*/
/*				parameters values from the setup page.						*/ 
/*																		    */
/*																		    */
/* INPUT:       Setup page parameters				                        */
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:																	*/
/*																			*/
/****************************************************************************/
char* wlan_htmlSavePageSetup(char *value, char *enableAp, char *ssid, char *chanId_bg, char *channId_a, 
                             char *bandMode_bg, char *bandMode_a, char *bandMode_both)
{

#if 0
	unsigned long channel, len, enAp, bandType;
    BOOL state, BandTypeWasSelected = FALSE;
    UINT8 TotWone, band;

	if (!strcmp(value,"Saving"))
	{
        /* Enable AP */
		sscanf(enableAp,"%d",&enAp);
		wdrv_dbGetWlanServiceState(&state);
        switch (enAp) 
        {
			case 1:     /* enable */
				if (state == TRUE)
					break;	/* already enabled */
                wdrv_WlanServiceEnable(void);
                break;
            case 2:     /* disable */
				if (state == FALSE)
					break;	/* already disabled */
                wdrv_WlanServiceDisable(void);
                break;
        }

        /* ssid */
        len = strlen(ssid);
        if (len <= DOT11_SSID_MAX)
           whal_nvram_setSsid(ssid, len,IMMEDIATELY);

        /* Band Type */
		sscanf(bandMode_bg,"%d",&bandType);
        if (bandType == 1)  /* band is B/G */
        {
            BandTypeWasSelected = TRUE;

            /* Check is working in dual band mode */
            wdrv_dbGetNumSupWone(&TotWone);
            /* if yes - move to single band mode */
            if (TotWone == WONE_DUAL_BAND) 
                wdrv_dbSetNumSupWone(1,FALSE);
            /* check if already configred to B/G */
            /* if not - switch to the B/G band. */
            wdrv_dbGetBand(0,&band);
            if (band != RADIO_BAND_2_4GHZ)
            {	
                wdrv_dbSetDot11Mode(0,0);
                whal_nvram_setDefaultsPartial(0,0);
            }
            /* set channel */
            sscanf(chanId_bg,"%d",&channel);
            wdrv_dbSetChannel(0,channel);
        }
        if (BandTypeWasSelected == FALSE) 
        {
            sscanf(bandMode_a,"%d",&bandType);
            if (bandType == 2)  /* band is A */
            {
                BandTypeWasSelected = TRUE;

                /* Check is working in dual band mode */
                wdrv_dbGetNumSupWone(&TotWone);
                /* if yes - move to single band mode */
                if (TotWone == WONE_DUAL_BAND) 
                    wdrv_dbSetNumSupWone(1,FALSE);
                /* check if already configred to A */
                /* if not - switch to the A band. */
                wdrv_dbGetBand(0,&band);
                if (band != RADIO_BAND_5GHZ)
                {	
                    wdrv_dbSetDot11Mode(0,4);
                    whal_nvram_setDefaultsPartial(0,4);
                }
                /* set channel */
                sscanf(channId_a,"%d",&channel);
                wdrv_dbSetChannel(0,channel);
            }
        }
        if (BandTypeWasSelected == FALSE) 
        {
            sscanf(bandMode_both,"%d",&bandType);
            if (bandType == 3)  /* band is both B/G  & A */
            {
                BandTypeWasSelected = TRUE;
                /* Check if working in single mode */
                wdrv_dbGetNumSupWone(&TotWone);
                /* if yes - switch to dual band mode.. */
                if (TotWone == WONE_SINGLE_BAND)
                {
                    wdrv_dbSetNumSupWone(2,FALSE);
                }
                /* ..and then set Wone 1 to B/G and Wone 2 to A */
                wdrv_dbGetBand(0,&band);
                if (band != RADIO_BAND_2_4GHZ)
                {	
                    wdrv_dbSetDot11Mode(0,0);
                    whal_nvram_setDefaultsPartial(0,0);
                }
                wdrv_dbGetBand(1,&band);
                if (band != RADIO_BAND_5GHZ)
                {	
                    wdrv_dbSetDot11Mode(1,4);
                    whal_nvram_setDefaultsPartial(1,4);
                }
                /* set channels */
                sscanf(chanId_bg,"%d",&channel);
                wdrv_dbSetChannel(0,channel);
                sscanf(channId_a,"%d",&channel);
                wdrv_dbSetChannel(1,channel);
            }
        }
        
        consoleReboot();
	}

	return "";
#endif

	unsigned long channel, len, enAp, bandType;
    BOOL BandTypeWasSelected = FALSE;
    __u8 state = 0;
    __u8 wone_nr;
    tiap_param_wone_t wone;

	if (!strcmp(value,"Saving"))
	{
        /* Enable AP */
		sscanf(enableAp,"%ld",&enAp);
		if( tiap_wcfg_get_basic(wd, TIAP_PARAM_OPER_STATE, &state, FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get AP's operating state.");
			return NULL;
		}
		switch (enAp) 
		{
			case 1:     /* enable */
				if (state == TRUE)
					break;	/* already enabled */
				state = 1;
				if( tiap_wcfg_set_basic(wd, TIAP_PARAM_OPER_STATE, &state, !UPDATE_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to set AP's operating state.");
					return NULL;
				}
				break;
			case 2:     /* disable */
				if (state == FALSE)
					break;	/* already disabled */
				state = 0;
				if( tiap_wcfg_set_basic(wd, TIAP_PARAM_OPER_STATE, &state, !UPDATE_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to set AP's operating state.");
					return NULL;
				}
				break;
		}
	
		/* ssid */
		len = strlen(ssid);
		if (len <= IW_ESSID_MAX_SIZE)
			if( tiap_wcfg_siwessid(wd, ssid, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set ssid.");
				return NULL;
			}

		/* Band Type */
		sscanf(bandMode_bg,"%ld",&bandType);
		if (bandType == 1)  /* band is B/G */
		{
			BandTypeWasSelected = TRUE;

			/* Check is working in dual band mode */
			if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
				return NULL;
			}

			/* if yes - move to single band mode */
			if (wone_nr == TIAP_PARAM_WONE_DUAL_BAND) {
				wone_nr = 1;
				if( tiap_wcfg_set_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !UPDATE_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to set wone number.");
					return NULL;
				}
			}

			/* check if already configred to B/G */
			/* if not - switch to the B/G band. */
			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return NULL;
			}
			if (wone.elem.band != TIAP_PARAM_RADIO_BAND_2_4GHZ)
				if( tiap_wcfg_set_dot11mode(wd, 0, 0, !UPDATE_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
					return NULL;
				}

			/* set channel */
			sscanf(chanId_bg,"%ld",&channel);
			wone.index = 0;
			wone.type = wone_ch;
			wone.elem.ch = channel;
			if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set channel.");
				return NULL;
			}
		}
		if (BandTypeWasSelected == FALSE) 
		{
			sscanf(bandMode_a,"%ld",&bandType);
			if (bandType == 2)  /* band is A */
			{
				BandTypeWasSelected = TRUE;

				/* Check is working in dual band mode */
				if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
					return NULL;
				}

				/* if yes - move to single band mode */
				if (wone_nr == TIAP_PARAM_WONE_DUAL_BAND) {
					wone_nr = 1;
					if( tiap_wcfg_set_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !UPDATE_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to set wone number.");
						return NULL;
					}
				}
				
				/* check if already configred to A */
				/* if not - switch to the A band. */
				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return NULL;
				}
				if (wone.elem.band != TIAP_PARAM_RADIO_BAND_5GHZ)
					if( tiap_wcfg_set_dot11mode(wd, 4, 0, !UPDATE_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
						return NULL;
					}
				
				/* set channel */
				sscanf(channId_a,"%ld",&channel);
				wone.index = 0;
				wone.type = wone_ch;
				wone.elem.ch = channel;
				if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to set channel.");
					return NULL;
				}
			}
		}
		if (BandTypeWasSelected == FALSE) 
		{
			sscanf(bandMode_both,"%ld",&bandType);
			if (bandType == 3)  /* band is both B/G  & A */
			{
				BandTypeWasSelected = TRUE;
				/* Check if working in single mode */
				if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
					return NULL;
				}
				/* if yes - switch to dual band mode.. */
				if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND) {
					wone_nr = 2;
					if( tiap_wcfg_set_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !UPDATE_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to set wone number.");
						return NULL;
					}
				}

				/* ..and then set Wone 1 to B/G and Wone 2 to A */
				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return NULL;
				}
				if (wone.elem.band != TIAP_PARAM_RADIO_BAND_2_4GHZ)
				{	
					if( tiap_wcfg_set_dot11mode(wd, 0, 0, !UPDATE_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
						return NULL;
					}
				}
				wone.index = 1;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return NULL;
				}

				if (wone.elem.band != TIAP_PARAM_RADIO_BAND_5GHZ)
					if( tiap_wcfg_set_dot11mode(wd, 4, 1, !UPDATE_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
						return NULL;
					}

				/* set channels */
				sscanf(chanId_bg,"%ld",&channel);
				wone.index = 0;
				wone.type = wone_ch;
				wone.elem.ch = channel;
				if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to set channel.");
					return NULL;
				}
				sscanf(channId_a,"%ld",&channel);
				wone.index = 1;
				wone.type = wone_ch;
				wone.elem.ch = channel;
				if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to set channel.");
					return NULL;
				}
			}
		}

		restart();
	}

	return "";
}

/****************************************************************************/
/*                wlan_htmlGetChannelSelectBandBG(void)							*/
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the channel drop down			*/ 
/*				text box - BAND B.											*/
/*																		    */
/*																		    */
/* INPUT:       Drop down value												*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"selected" if the value is identical to the AP's channel	*/
/*				"" if the value is different than the AP's channel			*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetChannelSelectBandBG(void)
{
#if 0
    UINT8 value, band, regDomain, uiRanges, ch, channel;
    UINT32  numOfRanges, index;
    IntegerRegulatoryDomains_e RegDomain, actualRegDomain;
    tChannelRangeElementStruct RangeElementStruct;
    char webBuffer[40];

    band = RADIO_BAND_2_4GHZ;
    wdrv_dbGetdot11CurrentRegDomain(band, &regDomain);
	wdrv_dbGetChannel(0,&channel);  // TBD: make sure to ask for the correct channel
    tempbuf[0] = '\0';

    switch (regDomain) 
    {
        case REG_DOMAIN_FCC:
        case REG_DOMAIN_US:
            actualRegDomain = REG_DOMAIN_FCC;
            break;
        case REG_DOMAIN_ETSI:
        case REG_DOMAIN_WORLD:
            actualRegDomain = REG_DOMAIN_ETSI;
            break;
        case REG_DOMAIN_IC:
        case REG_DOMAIN_SPAIN:
        case REG_DOMAIN_FRANCE:
        case REG_DOMAIN_MKK:
        case REG_DOMAIN_MKK1:
            actualRegDomain = regDomain;
            break;

        default:
            printf(tempbuf,"<OPTION selected value=0>0 ");
            break;
    }
    numOfRanges = wdrv_dbGetNumberOfDomainElements(WLAN_BAND_B,actualRegDomain);
    for (index = 1; index <= numOfRanges; index++) 
    {
        wdrv_dbGetRangeElement(WLAN_BAND_B,actualRegDomain,index,&RangeElementStruct);
        for (ch=RangeElementStruct.startCh;
             ch<=RangeElementStruct.stopCh;
             ch+=(RangeElementStruct.stepOpt & 0xF)) 
        {
            if (ch == channel) 
                sprintf(webBuffer,"<OPTION selected value=%d>%d ",ch,ch);
            else
                sprintf(webBuffer,"<OPTION value=%d>%d ",ch,ch);
            strcat(tempbuf,webBuffer);
        }
    }
    return tempbuf;
#endif

    UINT8 ch;
    UINT32  numOfRanges, i;
    char webBuffer[40];

    tiap_param_cur_reg_domain_t cur_regdomain, actual_regdomain;
    tiap_param_wone_t wone;
    tiap_param_channel_rang_elem_t rang_elem;

    cur_regdomain.band = TIAP_PARAM_RADIO_BAND_2_4GHZ;
    if( tiap_wcfg_get_cur_regdomain(wd, &cur_regdomain, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get current regulatory domain.");
	    return NULL;
    }
    
    wone.index = 0;
    wone.type = wone_ch;
    wone.elem.ch = 0;
    if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get channel.");
	    return NULL;
    }
    tempbuf[0] = '\0';

    switch (cur_regdomain.type) 
    {
	    case TIAP_PARAM_RD_FCC:
	    case TIAP_PARAM_RD_US:
		    actual_regdomain.type = TIAP_PARAM_RD_FCC;
		    break;
	    case TIAP_PARAM_RD_ETSI:
	    case TIAP_PARAM_RD_WORLD:
		    actual_regdomain.type = TIAP_PARAM_RD_ETSI;
		    break;
	    case TIAP_PARAM_RD_IC:
	    case TIAP_PARAM_RD_SPAIN:
	    case TIAP_PARAM_RD_FRANCE:
	    case TIAP_PARAM_RD_MKK:
		case TIAP_PARAM_RD_MKK1:
			actual_regdomain.type = cur_regdomain.type;
		    break;

	    default:
		    printf(tempbuf,"<OPTION selected value=0>0 ");
		    break;
    }
    numOfRanges = get_domain_elems_num(BAND_B, actual_regdomain.type);
    for (i = 1; i <= numOfRanges; i++) 
    {
	    wdrv_dbGetRangeElement(BAND_B, actual_regdomain, i, &rang_elem);
	    for (ch=rang_elem.start;
			    ch<=rang_elem.stop;
			    ch+=(rang_elem.step & 0xF)) 
	    {
		    if (ch == wone.elem.ch) 
			    sprintf(webBuffer,"<OPTION selected value=%d>%d ",ch,ch);
		    else
			    sprintf(webBuffer,"<OPTION value=%d>%d ",ch,ch);
		    strcat(tempbuf,webBuffer);
	    }
    }
    return tempbuf;

}


/****************************************************************************/
/*                wlan_htmlGetChannelSelectBandA(void)							*/
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the channel drop down			*/ 
/*				text box - BAND A.												    */
/*																		    */
/*																		    */
/* INPUT:       Drop down value												*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"selected" if the value is identical to the AP's channel	*/
/*				"" if the value is different than the AP's channel			*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetChannelSelectBandA(void)
{
#if 0
    UINT8 value, band, regDomain, uiRanges, ch, channel, TotWone;
    UINT32  numOfRanges, index;
    IntegerRegulatoryDomains_e RegDomain, actualRegDomain;
    tChannelRangeElementStruct RangeElementStruct;
    char webBuffer[40];

    band = RADIO_BAND_5GHZ;
    wdrv_dbGetdot11CurrentRegDomain(band, &regDomain);

    wdrv_dbGetNumSupWone(&TotWone);
    if (TotWone == WONE_DUAL_BAND) 
    	wdrv_dbGetChannel(1,&channel); /* Working in Dual band mode - band A assume to be on 1 */
    else
    	wdrv_dbGetChannel(0,&channel);  /* single mode */

    tempbuf[0] = '\0';

    switch (regDomain) 
    {
        case REG_DOMAIN_FCC:
        case REG_DOMAIN_US:
            actualRegDomain = REG_DOMAIN_US;
            break;
        case REG_DOMAIN_IC:
        case REG_DOMAIN_ETSI:
        case REG_DOMAIN_SPAIN:
        case REG_DOMAIN_FRANCE:
        case REG_DOMAIN_MKK:
        case REG_DOMAIN_MKK1:
        case REG_DOMAIN_WORLD:
            actualRegDomain = REG_DOMAIN_WORLD;
            break;

        default:
            printf(tempbuf,"<OPTION selected value=0>0 ");
            break;
    }
    numOfRanges = wdrv_dbGetNumberOfDomainElements(WLAN_BAND_A,actualRegDomain);
    for (index = 1; index <= numOfRanges; index++) 
    {
        wdrv_dbGetRangeElement(WLAN_BAND_A,actualRegDomain,index,&RangeElementStruct);
        for (ch=RangeElementStruct.startCh;
             ch<=RangeElementStruct.stopCh;
             ch+=(RangeElementStruct.stepOpt & 0xF)) 
        {
            if (ch == channel) 
                sprintf(webBuffer,"<OPTION selected value=%d>%d ",ch,ch);
            else
                sprintf(webBuffer,"<OPTION value=%d>%d ",ch,ch);
            strcat(tempbuf,webBuffer);
        }
    }
    return tempbuf;
#endif

    UINT8 ch;
    UINT32  numOfRanges, i;
    char webBuffer[40];
   
    __u8 wone_nr;
    tiap_param_cur_reg_domain_t cur_regdomain, actual_regdomain;
    tiap_param_wone_t wone;
    tiap_param_channel_rang_elem_t rang_elem;
	
    cur_regdomain.band = TIAP_PARAM_RADIO_BAND_5GHZ;
    if( tiap_wcfg_get_cur_regdomain(wd, &cur_regdomain, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get current regulatory domain.");
	    return NULL;
    }

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
	    return NULL;
    }

    wone.index = (wone_nr == TIAP_PARAM_WONE_DUAL_BAND ? 1 : 0);

    wone.type = wone_ch;
    wone.elem.ch = 0;
    if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get channel.");
	    return NULL;
    }

    tempbuf[0] = '\0';

    switch (cur_regdomain.type)
    {
	    case TIAP_PARAM_RD_FCC:
		case TIAP_PARAM_RD_US:
			actual_regdomain.type = TIAP_PARAM_RD_US;
		    break;
	    case TIAP_PARAM_RD_IC:
	    case TIAP_PARAM_RD_ETSI:
	    case TIAP_PARAM_RD_SPAIN:
	    case TIAP_PARAM_RD_FRANCE:
	    case TIAP_PARAM_RD_MKK:
	    case TIAP_PARAM_RD_MKK1:
		case TIAP_PARAM_RD_WORLD:
			actual_regdomain.type = TIAP_PARAM_RD_WORLD;
		    break;

	    default:
		    printf(tempbuf,"<OPTION selected value=0>0 ");
		    break;
    }
    numOfRanges = get_domain_elems_num(BAND_A, actual_regdomain.type);
    for (i = 1; i<= numOfRanges; i++) 
    {
	    wdrv_dbGetRangeElement(BAND_A, actual_regdomain, i, &rang_elem);
	    for (ch=rang_elem.start;
			    ch<=rang_elem.stop;
			    ch+=(rang_elem.step & 0xF)) 
	    {
			if (ch > 64) { /* currently we assume that such channels are not supported */
				continue;
			}
			/*Alexa: TODO: check the type of the board and if it supports
			channels higher than 64 to allow them */

		    if (ch == wone.elem.ch) 
			    sprintf(webBuffer,"<OPTION selected value=%d>%d ",ch,ch);
		    else
			    sprintf(webBuffer,"<OPTION value=%d>%d ",ch,ch);
		    strcat(tempbuf,webBuffer);
	    }
    }
    return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlGetBandMode(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the FW version. 									*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Fw version													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetBandMode(char *option)
{
#if 0
	UINT8 num_sup_wone, band, boardType;
	int bandType;

	sscanf(option,"%d",&bandType);

    wdrv_dbGetNumSupWone(&num_sup_wone);
    wdrv_dbGetRadioFrequency(&boardType);
	
    switch(bandType)
	{
		case 1: /* B */
			if (num_sup_wone == WONE_SINGLE_BAND)
            {
                wdrv_dbGetBand(0,&band);
                if (band != RADIO_BAND_5GHZ) 
                    return "checked";
            }
			break;

		case 2: /* A */
            if (boardType != RADIO_BAND_2_4_AND_5GHZ) 
                return " disabled = true ";
			if (num_sup_wone == WONE_SINGLE_BAND)
            {
                wdrv_dbGetBand(0,&band);
                if (band == RADIO_BAND_5GHZ) 
                    return "checked";
            }
			break;

        case 3: /* both */
            if (boardType != RADIO_BAND_2_4_AND_5GHZ) 
                return " disabled = true ";
			if (num_sup_wone == WONE_DUAL_BAND)
                return "checked";
            break;
    }
    return " ";
#endif

    UINT8 boardType;
    __u8 wone_nr;
    int bandType, len;
    char buf[4096];
    tiap_param_wone_t wone;

    sscanf(option,"%d",&bandType);

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
	    return NULL;
    }

    if( (len = dda_read_proc_entry(WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_RADIO_FREQ, buf, sizeof(buf))) == -1 ) {
	    PRINT_PROC_READ_ERR(WLAN_PROC_SERV_DIR_NAME, WLAN_PROC_SERV_RADIO_FREQ);
	    return NULL;
    }
    sscanf(buf, "%hhu", &boardType);

    switch(bandType)
    {
	    case 1: /* B */
		    if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
		    {
			    wone.index = 0;
			    wone.type = wone_band;
			    if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				    PRINT_CONFIG_ERR_MSG("Failed to get band.");
				    return NULL;
			    }
			    if (wone.elem.band != TIAP_PARAM_RADIO_BAND_5GHZ) 
				    return "checked";
		    }
		    break;

	    case 2: /* A */
		    if (boardType != TIAP_PARAM_RADIO_BAND_2_4_AND_5GHZ) 
			    return " disabled = true ";
		    if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
		    {
			    wone.index = 0;
			    wone.type = wone_band;
			    if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				    PRINT_CONFIG_ERR_MSG("Failed to get band.");
				    return NULL;
			    }
			    if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ) 
				    return "checked";
		    }
		    break;

	    case 3: /* both */
		    if (boardType != TIAP_PARAM_RADIO_BAND_2_4_AND_5GHZ)
			    return " disabled = true ";
		    if (wone_nr == TIAP_PARAM_WONE_DUAL_BAND)
			    return "checked";
		    break;
    }
    return " ";
}

/****************************************************************************/
/*                      wlan_htmlGetFwVersion(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the FW version. 									*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Fw version													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetFwVersion(void)
{
	int len;
	char buf[4096];

	buf[0] = '\0';

	if( (len = dda_read_proc_entry(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_FWVER, buf, sizeof(buf))) == -1)
		PRINT_PROC_READ_ERR(WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_FWVER);
	if( len > sizeof(buf) ) {
		fprintf(stderr, "\nweb: The returned length is bigger then the input one,"
				"in calling %s/%s proc entry\n",
				WLAN_PROC_HAL_DIR_NAME, WLAN_PROC_HAL_FWVER);
		return NULL;
	}
	memcpy(tempbuf, buf, sizeof(buf));
	tempbuf[len-1] = '\0';

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetSwVersion(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the SW version. 									*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Sw version													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetSwVersion(void)
{
	int len;
	char buf[4096];

	buf[0] = 0;
	
	if( (len = dda_read_proc_entry(WLAN_PROC_PROD_DIR_NAME,
                                        WLAN_PROC_PROD_VERSION, buf, sizeof(buf))) == -1 )
                PRINT_PROC_READ_ERR(WLAN_PROC_PROD_DIR_NAME, WLAN_PROC_PROD_VERSION);

	memcpy(tempbuf, buf, sizeof(buf));
	tempbuf[len]='\0';

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetIpAddr1(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[0] pf the AP's ip address						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[0] of the AP's ip address								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetIpAddr1(void)
{
#if 0
	unsigned long IPAddr;

	tempbuf[0] = 0;
	
	cm_hwGetLanIP(&IPAddr);

    sprintf(tempbuf,"%d",(IPAddr & 0xFF000000)>>24);

	return tempbuf;
#endif

	__u32 ipaddr;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFADDR, &ipaddr) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge IP address.");
		return NULL;
	}
	ipaddr = ntohl(ipaddr);
	sprintf(tempbuf,"%u",(ipaddr & 0xFF000000)>>24);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetIpAddr2(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[1] pf the AP's ip address						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[1] of the AP's ip address								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetIpAddr2(void)
{
#if 0
	unsigned long IPAddr;

	tempbuf[0] = 0;
	
	cm_hwGetLanIP(&IPAddr);

    sprintf(tempbuf,"%d",(IPAddr & 0x00FF0000)>>16);

	return tempbuf;
#endif
	__u32 ipaddr;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFADDR, &ipaddr) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge IP address.");
		return NULL;
	}
	ipaddr = ntohl(ipaddr);
	sprintf(tempbuf,"%u",(ipaddr & 0x00FF0000)>>16);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetIpAddr3(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[2] pf the AP's ip address						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[2] of the AP's ip address								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetIpAddr3(void)
{
#if 0
	unsigned long IPAddr;

	tempbuf[0] = 0;
	
	cm_hwGetLanIP(&IPAddr);

    sprintf(tempbuf,"%d",(IPAddr & 0x0000FF00)>>8);

	return tempbuf;
#endif
	__u32 ipaddr;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFADDR, &ipaddr) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge IP address.");
		return NULL;
	}
	ipaddr = ntohl(ipaddr);
	sprintf(tempbuf,"%u",(ipaddr & 0x0000FF00)>>8);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetIpAddr4(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[3] pf the AP's ip address						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[3] of the AP's ip address								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetIpAddr4(void)
{
#if 0
	unsigned long IPAddr;

	tempbuf[0] = 0;
	
	cm_hwGetLanIP(&IPAddr);

    sprintf(tempbuf,"%d",(IPAddr & 0x000000FF));

	return tempbuf;
#endif
	__u32 ipaddr;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFADDR, &ipaddr) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge IP address.");
		return NULL;
	}
	ipaddr = ntohl(ipaddr);
	sprintf(tempbuf,"%u",(ipaddr & 0x000000FF));

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetMask1(void)									*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[0] pf the AP's subnet mask						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[0] of the AP's subnet mask								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetMask1(void)
{
#if 0
	unsigned long IPMask;

	tempbuf[0] = 0;
	
	cm_hwGetLanIpMask(&IPMask);

    sprintf(tempbuf,"%d",(IPMask & 0xFF000000)>>24);

	return tempbuf;
#endif
	__u32 netmask;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFNETMASK, &netmask) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge netmask.");
		return NULL;
	}
	netmask = ntohl(netmask);
	sprintf(tempbuf,"%u",(netmask & 0xFF000000)>>24);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetMask2(void)									*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[1] pf the AP's subnet mask						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[1] of the AP's subnet mask								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetMask2(void)
{
#if 0
	unsigned long IPMask;

	tempbuf[0] = 0;
	
	cm_hwGetLanIpMask(&IPMask);

    sprintf(tempbuf,"%d",(IPMask & 0x00FF0000)>>16);

	return tempbuf;
#endif
	__u32 netmask;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFNETMASK, &netmask) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge netmask.");
		return NULL;
	}
	netmask = ntohl(netmask);
	sprintf(tempbuf,"%u",(netmask & 0x00FF0000)>>16);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetMask3(void)									*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[2] pf the AP's subnet mask						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[2] of the AP's subnet mask								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetMask3(void)
{
#if 0
	unsigned long IPMask;

	tempbuf[0] = 0;
	
	cm_hwGetLanIpMask(&IPMask);

    sprintf(tempbuf,"%d",(IPMask & 0x0000FF00)>>8);

	return tempbuf;
#endif
	__u32 netmask;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFNETMASK, &netmask) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge netmask.");
		return NULL;
	}
	netmask = ntohl(netmask);
	sprintf(tempbuf,"%u",(netmask & 0x0000FF00)>>8);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetMask4(void)									*/
/****************************************************************************/
/* DESCRIPTION: Returns byte[3] pf the AP's subnet mask						*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Byte[3] of the AP's subnet mask								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetMask4(void)
{
#if 0
	unsigned long IPMask;

	tempbuf[0] = 0;
	
	cm_hwGetLanIpMask(&IPMask);

    sprintf(tempbuf,"%d",(IPMask & 0x000000FF));

	return tempbuf;
#endif
	__u32 netmask;

	if( tiap_wcfg_get_ipaddr_or_netmask(wd->br_name, SIOCGIFNETMASK, &netmask) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get both driver and bridge netmask.");
		return NULL;
	}
	netmask = ntohl(netmask);
	sprintf(tempbuf,"%u",(netmask & 0x000000FF));

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetDomain(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the AP's domain										*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     AP's domain													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetDomain(void)
{
#if 0
    UINT8 regDomain;

    wdrv_dbGetdot11CurrentRegDomain(0,&regDomain);  
    switch (regDomain)
    {
        case REG_DOMAIN_FCC:
            return("FCC");
        
        case REG_DOMAIN_IC:
            return("IC");
        
        case REG_DOMAIN_ETSI:
            return("ETSI");
            
        case REG_DOMAIN_SPAIN:
            return("SPAIN");
        
        case REG_DOMAIN_FRANCE: 
            return("FRANCE");
        
        case REG_DOMAIN_MKK:
            return("MKK");
        
        case REG_DOMAIN_MKK1:
            return("MKK1");
        
        case REG_DOMAIN_US:
            return("US");
    
        case REG_DOMAIN_WORLD:
            return("WORLD");

        default :
            sprintf(tempbuf,"0x%X",regDomain);
            break;
    }
	return tempbuf;
#endif
	
	tiap_param_cur_reg_domain_t cur_regdomain;
	__u8 wone_nr;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return NULL;
	}

	cur_regdomain.band = 0;
	if( tiap_wcfg_get_cur_regdomain(wd, &cur_regdomain, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get current regulatory domain.");
		return NULL;
	}

	switch (cur_regdomain.type)
	{
		case TIAP_PARAM_RD_FCC:
			if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
				return("FCC");
			else
				return("FCC(2.4Ghz)   US(5Ghz)");

		case TIAP_PARAM_RD_IC:
			if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
				return("IC");
			else
				return("IC(2.4Ghz)  WORLD(5Ghz)");

		case TIAP_PARAM_RD_ETSI:
			if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
				return("ETSI");
			else
				return("ETSI(2.4Ghz)    WORLD(5Ghz)");

		case TIAP_PARAM_RD_SPAIN:
			if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
				return("SPAIN");
			else
				return("SPAIN(2.4Ghz)   WORLD(5Ghz)");

		case TIAP_PARAM_RD_FRANCE: 
			if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
				return("FRANCE");
			else
				return("FRANCE(2.4Ghz)   WORLD(5Ghz)");

		case TIAP_PARAM_RD_MKK:
			if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
				return("MKK");
			else
				return("MKK(2.4Ghz)   WORLD(5Ghz)");

		case TIAP_PARAM_RD_MKK1:
			if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
				return("MKK1");
			else
				return("MKK1(2.4Ghz)   WORLD(5Ghz)");

		case TIAP_PARAM_RD_US:
			return("US");

		case TIAP_PARAM_RD_WORLD:
			return("WORLD");

		default :
			sprintf(tempbuf,"0x%X", cur_regdomain.type);
			break;
	}

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlReboot(void)									*/
/****************************************************************************/
/* DESCRIPTION: Reboot the AP												*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:																	*/
/*																			*/
/****************************************************************************/
char* wlan_htmlReboot(char* value)
{
#if 0
	if (!strcmp(value,"Restart"))
	  consoleReboot();

	return "";
#endif
	return NULL;
}
 
/********************************
*		CONFIG PAGE				*	
*********************************/

/****************************************************************************/
/*                      wlan_htmlGetBeaconPeriod(void)							*/
/****************************************************************************/
/* DESCRIPTION: Returns the beacon period									*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     beacon period												*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetBeaconPeriod(void)
{
#if 0
	UINT16 beacon;
    UINT8 TotWone;

	tempbuf[0] = 0;
	
    wdrv_dbGetNumSupWone(&TotWone);
    if (TotWone == WONE_SINGLE_BAND) 
    {
        wdrv_dbGetStationConfigBeaconPeriod(&beacon);
        sprintf(tempbuf,"%d",beacon);
    }
    else
        strcpy(tempbuf,"-- disabled=true");

	return tempbuf;
#endif

	__u16 beacon;
	__u8 wone_nr;

	tempbuf[0] = 0;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return NULL;
	}
	if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND)
	{
		if( tiap_wcfg_get_basic(wd, TIAP_PARAM_BEACON_PERIOD, &beacon, !FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get beacon period.\n");
			return NULL;
		}
		sprintf(tempbuf,"%d",beacon);
	}
	else
		strcpy(tempbuf,"-- disabled=true");

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetDtimPeriod(void)							*/
/****************************************************************************/
/* DESCRIPTION: Returns the dtim period										*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     dtim period													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetDtimPeriod(void)
{
#if 0
	UINT8 dtim;

	tempbuf[0] = 0;
	
	wdrv_dbGetStationConfigDtimPeriod(&dtim);
	sprintf(tempbuf,"%d",dtim);

	return tempbuf;
#endif

	__u8 dtim;

	tempbuf[0] = 0;
	
	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_DTIM_PERIOD, &dtim, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get DTIM period.\n");
		return NULL;
	}
	sprintf(tempbuf,"%hhu",dtim);

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetRtsThreshold(void)							*/
/****************************************************************************/
/* DESCRIPTION: Returns the rts threshold									*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     rts threshold												*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetRtsThreshold(void)
{
#if 0
	UINT16 rts;

	tempbuf[0] = 0;
	
	wdrv_dbGetOperationRTSThreshold(&rts);
	sprintf(tempbuf,"%d",rts);

	return tempbuf;
#endif

	__s32 rts = 0;

	tempbuf[0] = 0;
	
	if( tiap_wcfg_giwrts(wd, &rts, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get rts period.\n");
		return NULL;
	}
	sprintf(tempbuf,"%hu", (__u16)rts);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetFragThreshold(void)							*/
/****************************************************************************/
/* DESCRIPTION: Returns the fragmentation threshold							*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     fragmentation threshold										*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetFragThreshold(void)
{
#if 0
	UINT16 frag;

	tempbuf[0] = 0;
	
	wdrv_dbGetOperationFragmentationThreshold(&frag);
	sprintf(tempbuf,"%d",frag);

	return tempbuf;
#endif
	__s32 frag;

	tempbuf[0] = 0;
	
	if( tiap_wcfg_giwfrag(wd, &frag, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get fragmentation threshold.");
		return NULL;
	}
	sprintf(tempbuf,"%hu", (__u16)frag);

	return tempbuf;

}

#if 0
static void CheckRateInList(UINT8 *ratesList, UINT8 ratesLen, UINT8 rate, char *str)
{
int i;
BOOL found=FALSE;

	for (i=0; i<ratesLen; i++)
	{
		if (rate == (ratesList[i]&0x7F))
		{
			if (ratesList[i]&0x80)
			{
				strcpy(str, "B");
			}
			else
			{
				strcpy(str, "Y");
			}
			found= TRUE;
			break;
		}
	}

	if (!found)
	{
		strcpy(str, "N");
	}
}
#endif


/****************************************************************************/
/*                      wlan_htmlSavePageConfig(void)							*/
/****************************************************************************/
/* DESCRIPTION: Update the driver database with the							*/
/*				parameters values from the config page.						*/ 
/*																		    */
/*																		    */
/* INPUT:       Config page parameters				                        */
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:																	*/
/*																			*/
/****************************************************************************/
char* wlan_htmlSavePageConfig(char* value, char *beaconPeriod, char *dtimPeriod, char *rtsThreshold, 
                              char *fragThreshold, char *inputDot11Mode, char *hiddenSSID,
                              char *packetBursting, char *outputPowerLevel, char *plusMode,
							  char *vhcfMode, char *vhcfIP1, char *vhcfIP2, char *vhcfIP3, char *vhcfIP4,
							  char *vhcfProtocol1, char *vhcfPort1, char *vhcfProtocol2, char *vhcfPort2,
							  char *energyDetection, char *radarDetection, char *bssIsolation)
{
#if 0
	unsigned long beacon, dtim, rts, frag, dot11Mode;
	char ratesSet[8];
	UINT8 ratesLen, currentMaxActive;
	BOOL quit;
	int  i, maxTxPowerLevel, mode4x;
	char str[4], support[3];

	UINT8 curRates[OFDM_RATES_MAX];
	UINT8 curRatesLen=0;
	UINT8 newRates[OFDM_RATES_MAX];
	UINT8 newRatesLen=0;
	UINT8 LastDot11Mode;
    
	char  rates[CCK_PBCC_RATES_MAX];
	UINT8 grates[OFDM_RATES_MAX];
	UINT8 ratesLength = 0;

	char addrate[OFDM_RATES_MAX], grate[OFDM_RATES_MAX]; 
    UINT8 TotalWones, band, woneIndex;
    tVHCF_PARAMS  vhcfParams;
	int  vhcfInput;
	UINT32  vhcfIP;


	if (!strcmp(value,"Saving"))
	{
        wdrv_dbGetNumSupWone(&TotalWones);

		/* beacon */
        if (TotalWones == WONE_SINGLE_BAND) 
        {
            sscanf(beaconPeriod,"%d",&beacon);
            wdrv_dbSetStationConfigBeaconPeriod(beacon);
        }

		/* dtim */
		sscanf(dtimPeriod,"%d",&dtim);
		wdrv_dbSetStationConfigDtimPeriod(dtim);

		/* rts threshold */
		sscanf(rtsThreshold,"%d",&rts);
		wdrv_dbSetOperationRTSThreshold(rts);

		/* frag threshold */
		sscanf(fragThreshold,"%d",&frag);
		if ((frag % 2) !=0 )
			printf("Invalid fragmentation threshold, must be an even number.\n");
		else
			wdrv_dbSetOperationFragmentationThreshold(frag);

		/* dot11mode */
        switch (TotalWones) 
        {
            case 1:
                wdrv_dbGetBand(0 ,&band);
                if (band == RADIO_BAND_5GHZ)
                    woneIndex = 99;             /* 99 is a value denotes 'not b/g' */
				else
					woneIndex = 0;
                break;

            case 2:
                wdrv_dbGetBand(0 ,&band);
                if (band == RADIO_BAND_2_4GHZ)
                    woneIndex = 0;
                else
                {
                    wdrv_dbGetBand(1 ,&band);
                    if (band == RADIO_BAND_2_4GHZ)
                        woneIndex = 1;
                    else
                        woneIndex = 99;   /* both Wone 0,1 are 'A' */
                }
                break;
        }
        if (woneIndex < 99) 
        {
		    wdrv_dbGetLastStoredDot11Mode(0,&LastDot11Mode);
		    sscanf(inputDot11Mode,"%d",&dot11Mode);
		    wdrv_dbSetDot11Mode(0,(UINT8)dot11Mode);
		    
		    if(dot11Mode != LastDot11Mode)
		    {
    			whal_nvram_setDefaultsPartial(0,(UINT8)dot11Mode);
	    	}
        }
              
        /* hidden SSID */
        if (strcmp(hiddenSSID,"1") == 0)
            wdrv_dbSetHiddenSsidStatus(1);
        else
            wdrv_dbSetHiddenSsidStatus(0);

		/* Power Level */
		sscanf(outputPowerLevel,"%d",&maxTxPowerLevel);
        if ((maxTxPowerLevel >= 0) && (maxTxPowerLevel <= 5))
        {
            wdrv_dbSetMaxTxPowerLevel((UINT8)(maxTxPowerLevel+1));
        }

        /* a/g+ ==> 4x */
        if (strcmp(plusMode,"1") == 0)
		{
            wdrv_dbSet4xEnable(TRUE);
			wdrv_dbSetConcatEnable(TRUE);
		}
        else if (strcmp(plusMode,"2") == 0) 
		{
            wdrv_dbSet4xEnable(FALSE);
			wdrv_dbSetConcatEnable(FALSE);
		}

		/* Interference Avoidance */
        if (strcmp(energyDetection,"1") == 0)
		{
			wdrv_dbSetEnergyDetect(TRUE, IMMEDIATELY);
		}
        else if (strcmp(energyDetection,"2") == 0) 
		{
			wdrv_dbSetEnergyDetect(FALSE, IMMEDIATELY);
		}


		/************************/
		/* Save vHCF Parameters */
		/************************/
		sscanf(vhcfMode,"%d",&vhcfInput);
		if (vhcfInput == 1)
		{
			vhcfParams.vhcfEnable = TRUE;

			
			sscanf(vhcfIP1,"%d",&vhcfInput);
			vhcfIP = ((UINT32)vhcfInput) << 24;
			sscanf(vhcfIP2,"%d",&vhcfInput);
			vhcfIP |= ((UINT32)vhcfInput) << 16;
			sscanf(vhcfIP3,"%d",&vhcfInput);
			vhcfIP |= ((UINT32)vhcfInput) << 8;
			sscanf(vhcfIP4,"%d",&vhcfInput);
			vhcfIP |= (UINT32)vhcfInput;
			
			
			
			vhcfParams.vhcfClassifier[0].DestIp = vhcfIP;
			vhcfParams.vhcfClassifier[1].DestIp = vhcfIP;

			sscanf(vhcfProtocol1,"%d",&vhcfInput);
			vhcfParams.vhcfClassifier[0].Protocol = vhcfInput - 1;
			sscanf(vhcfPort1,"%d",&vhcfInput);
			vhcfParams.vhcfClassifier[0].DestPort = vhcfInput;
			sscanf(vhcfProtocol2,"%d",&vhcfInput);
			vhcfParams.vhcfClassifier[1].Protocol = vhcfInput - 1;
			sscanf(vhcfPort2,"%d",&vhcfInput);
			vhcfParams.vhcfClassifier[1].DestPort = vhcfInput;
		}

		else
			vhcfParams.vhcfEnable = FALSE;

        wdrv_dbSetVhcfParams(&vhcfParams);  /* Save new setting to DB and NVRAM. */


		consoleReboot();
	}

	return "";
#endif


	__u8 dtim;
	__u16 beacon, rts, frag,input_dtim;
	int  maxTxPowerLevel;

	UINT8 woneIndex=111;
	int  vhcfInput;
	//UINT32  vhcfIP;
	char ipStr[20];
    struct in_addr in;

	__u8 wone_nr;
	tiap_param_wone_t wone;
	__u16 hidden_ssid;
	tiap_param_4x_t _4x;
	__u8 eneregy_detect;
	__u8 radar_detect;
	tiap_param_bss_br_state_t bss_bridge_state;
	tiap_param_vhcf_t vhcf;



	if (!strcmp(value,"Saving"))
	{
		storage_backup = wd->storage;
		
		if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
			return NULL;
		}

		
		/* beacon */
		if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND) 
		{ 
			if ( sscanf(beaconPeriod,"%hu",&beacon) <= 0){
				wd->storage = storage_backup;
				create_js_msg_box(tempbuf,"Beacon period must be a valid number");
				return tempbuf;
			}
			if (beacon < MIN_BEACON_PERIOD || beacon > MAX_BEACON_PERIOD) {
				wd->storage = storage_backup;
				create_js_msg_box(tempbuf,"Beacon period is out of range");
				return tempbuf;
			}

			if( tiap_wcfg_set_basic(wd, TIAP_PARAM_BEACON_PERIOD, &beacon, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set beacon period.\n");
				return NULL;
			}
		}

		
		/* dtim */
		if ( sscanf(dtimPeriod,"%hu",&input_dtim) <= 0) {
			wd->storage = storage_backup;
			create_js_msg_box(tempbuf,"DTIM period must be a valid number");
			return tempbuf;
		}
		
		if (input_dtim < MIN_DTIM_PERIOD || input_dtim > MAX_DTIM_PERIOD){
			wd->storage = storage_backup;
			create_js_msg_box(tempbuf,"DTIM period is out of range");
			return tempbuf;
		}

		dtim = input_dtim;
		if( tiap_wcfg_set_basic(wd, TIAP_PARAM_DTIM_PERIOD, &dtim, !UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set DTIM period.\n");
			return NULL;
		}

		
		/* rts threshold */
		if (sscanf(rtsThreshold,"%hu",&rts) <= 0 ) {
			wd->storage = storage_backup;
			create_js_msg_box(tempbuf,"Rts threshold must be a valid number");
			return tempbuf;
		}
		if (rts > MAX_RTS_THRESHOLD) {
			wd->storage = storage_backup;
			create_js_msg_box(tempbuf,"Rts threshold is out of range");
			return tempbuf;
		}
		
		if( tiap_wcfg_siwrts(wd, rts, !UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set rts period.\n");
			return NULL;
		}

		/* frag threshold */
		if (sscanf(fragThreshold,"%hu",&frag) <= 0){
			wd->storage = storage_backup;
			create_js_msg_box(tempbuf,"Frag threshold must be a valid number");
			return tempbuf;
		}

		if ((frag % 2) !=0 ){
			/* printf("Invalid fragmentation threshold, must be an even number.\n");*/
			wd->storage = storage_backup;
			create_js_msg_box(tempbuf,"Frag threshold must be an even number");
			return tempbuf;
		}
		else {
			if (frag < MIN_FRAG_THRESHOLD || frag > MAX_FRAG_THRESHOLD){
				wd->storage = storage_backup;
				create_js_msg_box(tempbuf,"Frag threshold is out of range");
				return tempbuf;
			}
			if( tiap_wcfg_siwfrag(wd, frag, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set fragmentation threshold.");
				return NULL;
			}
			
		}

		
		/* dot11mode */
		switch (wone_nr) 
		{
			case 1:
				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return NULL;
				}
				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
					woneIndex = 99;             /* 99 is a value denotes 'not b/g' */
				else
					woneIndex = 0;
				break;

			case 2:
				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return NULL;
				}
				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
					woneIndex = 0;
				else
				{
					wone.index = 1;
					wone.type = wone_band;
					if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to get band.");
						return NULL;
					}

					if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
						woneIndex = 1;
					else
						woneIndex = 99;   /* both Wone 0,1 are 'A' */
				}
				break;
		}

		
		
		if (woneIndex < 99) 
		{
			sscanf(inputDot11Mode,"%hhu",&wone.elem.dot11_mode);
			/* check if already configred to that dot11 mode. */
			/* if not switch to that mode */ 
			if (wone.elem.dot11_mode != wd->storage.ldb.dot11_mode[0].elem.dot11_mode){
			if( tiap_wcfg_set_dot11mode(wd, wone.elem.dot11_mode, 0, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
				return NULL;
			}
		}
		}

		


		/* hidden SSID */
		hidden_ssid = (!strcmp(hiddenSSID,"1") ? 1 : 0);

		if( tiap_wcfg_set_basic(wd, TIAP_PARAM_HIDDEN_SSID, &hidden_ssid, !UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set hidden ssid.");
			return NULL;
		}

		

		/* Power Level */
		sscanf(outputPowerLevel,"%d",&maxTxPowerLevel);
		if ((maxTxPowerLevel >= 0) && (maxTxPowerLevel <= 5))
		{
			__u8 i = (__u8)(maxTxPowerLevel+1);
			if( tiap_wcfg_set_basic(wd, TIAP_PARAM_MAX_TX_POWER_LEVEL, &i, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set tx power level.\n");
				return NULL;
			}
		}

		
		/* a/g+ ==> 4x */
		if (strcmp(plusMode,"1") == 0)
		{
			_4x.cmd = TIAP_PARAM_4X_4X;
			_4x.value = TRUE;
			if (tiap_wcfg_set_4x(wd, &_4x, 0, UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set 4x 4x mode.");
				return NULL;
			}

			_4x.cmd = TIAP_PARAM_4X_CAT;
			_4x.value = TRUE;
			if (tiap_wcfg_set_4x(wd, &_4x, 0, UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set 4x concatenation mode.");
				return NULL;
			}
		}
		else if (strcmp(plusMode,"2") == 0) 
		{
			_4x.cmd = TIAP_PARAM_4X_4X;
			_4x.value = FALSE;
			if (tiap_wcfg_set_4x(wd, &_4x, 0, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set 4x 4x mode.");
				return NULL;
			}

			_4x.cmd = TIAP_PARAM_4X_CAT;
			_4x.value = FALSE;
			if (tiap_wcfg_set_4x(wd, &_4x, 0, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set 4x concatenation mode.");
				return NULL;
			}
		}


		
		/* Interference Avoidance */
		if (strcmp(energyDetection,"1") == 0)
		{
			eneregy_detect = TRUE;
			if(tiap_wcfg_set_basic(wd, TIAP_PARAM_ENERGY_DETECT, &eneregy_detect, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set energy detect.");
				return NULL;
			}
		}
		else if (strcmp(energyDetection,"2") == 0) 
		{
			eneregy_detect = FALSE;
			if(tiap_wcfg_set_basic(wd, TIAP_PARAM_ENERGY_DETECT, &eneregy_detect, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set energy detect.");
				return NULL;
			}
		}

		
		/* Radar Detection */		
		if (strcmp(radarDetection,"1") == 0)
		{
			radar_detect = TRUE;
			if(tiap_wcfg_set_basic(wd, TIAP_PARAM_RADAR_DETECT_MODE, &radar_detect, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set radar detect mode");
				return NULL;
			}
		}
		else if (strcmp(radarDetection,"2") == 0) 
		{
			radar_detect = FALSE;
			if(tiap_wcfg_set_basic(wd, TIAP_PARAM_RADAR_DETECT_MODE, &radar_detect, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set radar detect mode");
				return NULL;
			}
		}

		
		/* BSS isolation */		
		if (strcmp(bssIsolation,"1") == 0)
		{
			bss_bridge_state = TIAP_PARAM_BSS_BR_BLOCK;
			if(tiap_wcfg_set_basic(wd, TIAP_PARAM_BSS_BRIDGE_STATE, &bss_bridge_state, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set bss isolation mode");
				return NULL;
			}
		}
		else if (strcmp(bssIsolation,"2") == 0) 
		{
			bss_bridge_state = TIAP_PARAM_BSS_BR_ENABLE;
			if(tiap_wcfg_set_basic(wd, TIAP_PARAM_BSS_BRIDGE_STATE, &bss_bridge_state, !UPDATE_STORAGE)) {
				PRINT_CONFIG_ERR_MSG("Failed to set bss isolation mode");
				return NULL;
			}
		}


		
		/************************/
		/* Save vHCF Parameters */
		/************************/
		sscanf(vhcfMode,"%d",&vhcfInput);
		if (vhcfInput == 1)
		{
			vhcf.enable = TRUE;

			sprintf(ipStr,"%s.%s.%s.%s", vhcfIP1, vhcfIP2, vhcfIP3, vhcfIP4);

			if (inet_aton(ipStr, &in))
			{
				vhcf.classifier[0].dest_ip = in.s_addr;
				vhcf.classifier[1].dest_ip = in.s_addr;
			}
			else
			{
				printf("Wrong IP format\n");
				return NULL;
			}


			sscanf(vhcfProtocol1,"%d",&vhcfInput);
			vhcf.classifier[0].proto = vhcfInput - 1;
			sscanf(vhcfPort1,"%d",&vhcfInput);
			vhcf.classifier[0].dest_port = (__u16)vhcfInput;
			sscanf(vhcfProtocol2,"%d",&vhcfInput);
			vhcf.classifier[1].proto = vhcfInput - 1;
			sscanf(vhcfPort2,"%d",&vhcfInput);
			vhcf.classifier[1].dest_port = (__u16)vhcfInput;
		}
		else {
			if( tiap_wcfg_get_vhcf(wd, &vhcf, FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get vhcf parameters.");
				return NULL;
			}
			vhcf.enable = FALSE;
		}

		if (strcmp(plusMode,"1") == 0) { /* 4x is enabled */
			vhcf.enable = FALSE;
		}
		if( tiap_wcfg_set_vhcf(wd, &vhcf, !UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set vhcf parameters.");
			return NULL;
		}

		restart();
	}

	return "";

	
}


/****************************************************************************/
/*                      wlan_htmlRestoreDefaults(void)							*/
/****************************************************************************/
/* DESCRIPTION: Rstores the AP defaults from nvram							*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:																	*/
/*																			*/
/****************************************************************************/
char* wlan_htmlRestoreDefaults(char* value)
{
#if 0
	if (!strcmp(value,"Defaults"))
	{
		wdrv_dbRestoreNvramDefaults(void);

		consoleReboot(void);
	}

	return "";
#endif
	if (!strcmp(value,"Defaults")) {
		if (tiap_wcfg_set_defaults(wd)) {
			PRINT_CONFIG_ERR_MSG("Failed to set defaults.");
			return NULL;
		}
		restart();
	}

	return "";

}


/****************************************************************************/
/*                     wlan_html GetUsageTimeMode(void)					*/
/****************************************************************************/
/* DESCRIPTION: Rstores the AP defaults from nvram							*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:																	*/
/*																			*/
/****************************************************************************/
char* wlan_htmlAdvancedGetUsageTimeMode(char *value)
{
#if 0
    UINT8 TotWone, usageTimeMode;
	int mode;
	
    wdrv_dbGetNumSupWone(&TotWone);
    if (TotWone == WONE_SINGLE_BAND) 
    {
		return " disabled=true";
    }

	sscanf(value,"%d",&mode);

	wdrv_dbGetUsageTimeMode(&usageTimeMode);
	switch (usageTimeMode)
	{
		case FALSE:
			if (mode == 2)
				return "checked";
			break;
        case TRUE:
            if (mode == 1) 
                return "checked";
            break;
	}
    return " ";
#endif

    int mode;
    __u8 wone_nr, time_mode;

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
	    return NULL;
    }

    if (wone_nr == TIAP_PARAM_WONE_SINGLE_BAND) 
	    return " disabled=true";

    sscanf(value,"%d",&mode);

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_TIME_MODE, &time_mode, FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to set time mode.");
	    return NULL;
    }

    switch (time_mode)
    {
	    case 0:/* short */
		    if (mode == 2)
			    return "checked";
		    break;
	    case 1: /* long */
		    if (mode == 1) 
			    return "checked";
		    break;
    }
    return " ";

}


/****************************************************************************/
/*                      wlan_htmlGetPreambleCheck(void)							*/
/****************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the preamble radio buttons			*/ 
/*																		    */
/*																		    */
/* INPUT:       Radio button value											*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"checked" if the value is identical to the AP's preamble mode	*/
/*				"" if the value is different than the AP's preamble mode	*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetPreambleCheck(char *value)
{
#if 0
	int preambleMode;
	int mode;
	UINT8 temp;

	sscanf(value,"%d",&mode);

	wdrv_dbGetPreambleMode(0,&temp);

	preambleMode = (int)temp;

	if (preambleMode == mode)
		return "checked";
	else 
		return " ";
#endif

	int mode;
	tiap_param_wone_t wone;

	sscanf(value,"%d",&mode);

	wone.index = 0;
	wone.type = wone_pream_mode;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get preamble.");
		return NULL;
	}

	if (wone.elem.pream_mode == mode)
		return "checked";
	else 
		return " ";

}


/****************************************************************************/
/*                      wlan_htmlGetSlotTimeCheck(void)							*/
/****************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the pbcc radio buttons				*/ 
/*																		    */
/*																		    */
/* INPUT:       Radio button value											*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"checked" if the value is identical to the AP's Slot        */
/*				 time mode													*/
/*				"" if the value is different than the AP's  mode			*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetSlotTimeCheck(char* value)
{
#if 0
	UINT8 temp;
	int   mode;
	int   SlotTimeAlgorithmMode;

	sscanf(value,"%d",&mode);

	wdrv_dbGetSlotTimeMode(0,&temp);
	SlotTimeAlgorithmMode = (int) temp;

	if (SlotTimeAlgorithmMode == mode)
		return "checked";
	else 
		return " ";
#endif
	int   mode;
	tiap_param_wone_t wone;

	sscanf(value,"%d",&mode);

	wone.index = 0;
	wone.type = wone_slot_mode;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get slot time.");
		return NULL;
	}

	if (wone.elem.slot_mode == mode)
		return "checked";
	else 
		return " ";

}

/****************************************************************************/
/*                      wlan_htmlGetPbccCheck(void)								*/
/****************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the pbcc radio buttons				*/ 
/*																		    */
/*																		    */
/* INPUT:       Radio button value											*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"checked" if the value is identical to the AP's pbcc mode	*/
/*				"" if the value is different than the AP's pbcc mode		*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetPbccCheck(char* value)
{
#if 0
	UINT8 temp;
	int   mode;
	int   PBCCAlgorithmMode;
    UINT8 TotalWones, band, woneIndex;

    wdrv_dbGetNumSupWone(&TotalWones);
    switch (TotalWones) 
    {
        case WONE_SINGLE_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_5GHZ)
                return " disabled=true ";
            woneIndex = 0;
            break;
    
        case WONE_DUAL_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_2_4GHZ)
                woneIndex = 0;
            else
            {
                wdrv_dbGetBand(1 ,&band);
                if (band == RADIO_BAND_2_4GHZ)
                    woneIndex = 1;
                else
                    return " disabled=true ";   /* both Wone 0,1 are 'A' */
            }
            break;
    }

	sscanf(value,"%d",&mode);

	wdrv_dbGetPBCCAlgorithmMode(woneIndex,&temp);
	PBCCAlgorithmMode = (int) temp;

	if (PBCCAlgorithmMode == mode)
		return "checked";
	else 
		return " ";
#endif

	int   mode;
	UINT8 woneIndex;

	__u8 wone_nr;
	tiap_param_wone_t wone;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return NULL;
	}

	switch (wone_nr) 
	{
		case TIAP_PARAM_WONE_SINGLE_BAND:

			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return NULL;
			}
			
			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
				return " disabled=true ";
			woneIndex = 0;
			break;

		case TIAP_PARAM_WONE_DUAL_BAND:

			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return NULL;
			}
			
			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
				woneIndex = 0;
			else
			{
				wone.index = 1;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return NULL;
				}
				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
					woneIndex = 1;
				else
					return " disabled=true ";   /* both Wone 0,1 are 'A' */
			}
			break;

		default:
			return NULL;
	}

	sscanf(value,"%d",&mode);

	wone.index = woneIndex;
	wone.type = wone_pbcc;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get band.");
		return NULL;
	}

	if (wone.elem.pbcc_mode == mode)
		return "checked";
	else 
		return " ";

}

/****************************************************************************/
/*                      wlan_htmlErpCheck(void)									*/
/****************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the pbcc radio buttons				*/ 
/*																		    */
/*																		    */
/* INPUT:       Radio button value											*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"checked" if the value is identical to the AP's ERP mode    */
/*				"" if the value is different than the AP's ERP mode		*/
/*																			*/
/****************************************************************************/
char* wlan_htmlErpCheck(char* value)
{
#if 0
	UINT8 temp;
	int   mode;
	int   ERPAlgorithmMode;
    UINT8 TotalWones, band, woneIndex;

    wdrv_dbGetNumSupWone(&TotalWones);
    switch (TotalWones) 
    {
        case WONE_SINGLE_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_5GHZ)
                return " disabled=true ";
            woneIndex = 0;
            break;
    
        case WONE_DUAL_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_2_4GHZ)
                woneIndex = 0;
            else
            {
                wdrv_dbGetBand(1 ,&band);
                if (band == RADIO_BAND_2_4GHZ)
                    woneIndex = 1;
                else
                    return " disabled=true ";   /* both Wone 0,1 are 'A' */
            }
            break;
    }

	sscanf(value,"%d",&mode);

	wdrv_dbGetERPProtectionMode(woneIndex,&temp);
	ERPAlgorithmMode = (int) temp;

	if (ERPAlgorithmMode == mode)
		return "checked";
	else 
		return " ";
#endif

	int   mode;
	UINT8 woneIndex;

	__u8 wone_nr;
	tiap_param_wone_t wone;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return NULL;
	}
	
	switch (wone_nr) 
	{
		case TIAP_PARAM_WONE_SINGLE_BAND:
			
			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return NULL;
			}
			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
				return " disabled=true ";
			woneIndex = 0;
			break;

		case TIAP_PARAM_WONE_DUAL_BAND:
			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return NULL;
			}
			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
				woneIndex = 0;
			else
			{
				wone.index = 1;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return NULL;
				}

				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
					woneIndex = 1;
				else
					return " disabled=true ";   /* both Wone 0,1 are 'A' */
			}
			break;
		default:
			return NULL;
	}

	sscanf(value,"%d",&mode);

	wone.index = woneIndex;
	wone.type = wone_erp_mode;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get erp mode.");
		return NULL;
	}


	if (wone.elem.erp_mode == mode)
		return "checked";
	else 
		return " ";

}

/****************************************************************************/
/*                      wlan_htmlDot11ModeCheck(void)							*/
/****************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the dot11mode buttons				*/ 
/*																		    */
/*																		    */
/* INPUT:       Radio button value											*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"checked" if the value is identical to the AP's dot11 mode  */
/*				"" if the value is different than the AP's dot11  mode		*/
/*																			*/
/****************************************************************************/
char* wlan_htmlDot11ModeCheck(char* value)
{
#if 0
	UINT8 temp;
	int   mode;
	int   dot11Mode;
    UINT8 TotalWones, band, woneIndex;

    wdrv_dbGetNumSupWone(&TotalWones);
    switch (TotalWones) 
    {
        case WONE_SINGLE_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_5GHZ)
                return " disabled=true ";
            woneIndex = 0;
            break;
    
        case WONE_DUAL_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_2_4GHZ)
                woneIndex = 0;
            else
            {
                wdrv_dbGetBand(1 ,&band);
                if (band == RADIO_BAND_2_4GHZ)
                    woneIndex = 1;
                else
                    return " disabled=true ";   /* both Wone 0,1 are 'A' */
            }
            break;
    }
	sscanf(value,"%d",&mode);

	wdrv_dbGetDot11Mode(woneIndex,&temp);
	dot11Mode = (int) temp;

    if (dot11Mode == 4)
        return " disabled=true ";
	else if (dot11Mode == mode)
		return "checked";
	else 
		return " ";
#endif

	int   mode;
	UINT8 woneIndex;

	__u8 wone_nr;
	tiap_param_wone_t wone;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return NULL;
	}

	switch (wone_nr) 
	{
		case TIAP_PARAM_WONE_SINGLE_BAND:
			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return NULL;
			}
			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
				return " disabled=true ";
			woneIndex = 0;
			break;

		case TIAP_PARAM_WONE_DUAL_BAND:
			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return NULL;
			}
			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
				woneIndex = 0;
			else
			{
				wone.index = 1;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return NULL;
				}
				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
					woneIndex = 1;
				else
					return " disabled=true ";   /* both Wone 0,1 are 'A' */
			}
			break;
		default: return NULL;
	}
	sscanf(value,"%d",&mode);

	wone.index = woneIndex;
	wone.type = wone_dot11_mode;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
		return NULL;
	}

	if (wone.elem.dot11_mode == 4)
		return " disabled=true ";
	else if (wone.elem.dot11_mode == mode)
		return "checked";
	else return " ";
}


char *wlan_html4xCheck(char* value)
{
#if 0
	BOOL temp;
	int  support;
	int  en;

	sscanf(value, "%d", &support);
	
	wdrv_dbGet4xEnable(&temp);

	if(temp == TRUE)
		strcpy(tempbuf, "ON");
	else
		strcpy(tempbuf, "OFF");

	return tempbuf;
#endif
	return NULL;
}

char *wlan_html4xConcatCheck(char* value)
{
#if 0
	BOOL temp;
	int  support;
	int  en;

	sscanf(value, "%d", &support);
	
	wdrv_dbGetConcatEnable(&temp);

	if(temp == TRUE)
		strcpy(tempbuf, "ON");
	else
		strcpy(tempbuf, "OFF");

	return tempbuf;
#endif
	return NULL;
}

char *wlan_html4xAckEmulCheck(char* value)
{
#if 0
	BOOL temp;
	int  support;
	int  en;

	sscanf(value, "%d", &support);
	
	wdrv_dbGetAckEmulEnable(&temp);

	if(temp == TRUE)
		strcpy(tempbuf, "ON");
	else
		strcpy(tempbuf, "OFF");

	return tempbuf;
#endif
	return NULL;
}


char* wlan_htmlGetAdditionalOperRateSet_G_Band(char *value)
{
#if 0
	UINT8 curRates[OFDM_RATES_MAX];
	UINT8 curRatesLen=0;
	int   rateInd, i;
	UINT8 rateValue[OFDM_RATES_MAX] =
			{ DOT11_RATE_6,  DOT11_RATE_9,
			  DOT11_RATE_12, DOT11_RATE_18,
			  DOT11_RATE_24, DOT11_RATE_36,
			  DOT11_RATE_48, DOT11_RATE_54 };

    UINT8 TotalWones, band, woneIndex, dot11Mode;
    
    wdrv_dbGetNumSupWone(&TotalWones);
    switch (TotalWones) 
    {
        case WONE_SINGLE_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_5GHZ)
                return "N disabled=true ";
            woneIndex = 0;
            break;

        case WONE_DUAL_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_2_4GHZ)
                woneIndex = 0;
            else
            {
                wdrv_dbGetBand(1 ,&band);
                if (band == RADIO_BAND_2_4GHZ)
                    woneIndex = 1;
                else
                    return "N disabled=true ";   /* both Wone 0,1 are 'A' */
            }
            break;
    }
    /* We are in B/G band - skipping the B modes, remain only with G */
    wdrv_dbGetDot11Mode(woneIndex,&dot11Mode);
    if ((dot11Mode != DOT11_MODE_MIXED) && (dot11Mode != DOT11_MODE_OFDM_ONLY))
        return "N disabled=true ";   /* not G only mode */

    tempbuf[1] = 0;

	sscanf(value,"%d",&rateInd);
	if(rateInd > OFDM_RATES_MAX || rateInd < 1)
	{
		tempbuf[0] = 'F';
		return tempbuf;
	}

	whal_nvram_getOperationalAdditionalRateSetLength(woneIndex,&curRatesLen);
	whal_nvram_getOperationalAdditionalRateSet(woneIndex,curRates, curRatesLen);

	for (i=0; i<curRatesLen; i++)
	{
		if(rateValue[rateInd-1] == (curRates[i]&0x7F))
			break;
	}

	if(i >= curRatesLen)
		tempbuf[0] = 'N';
	else
	{
		if(curRates[i] & 0x80)
			tempbuf[0] = 'B';
		else
			tempbuf[0] = 'Y';
	}

	return tempbuf;
#endif

	int   rateInd;

	UINT8 woneIndex;
	__u8 wone_nr;
	tiap_param_wone_t wone;

	wone.index = 0;
	wone.type = wone_ch;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get channel.");
		return NULL;
	}

    if (wone.elem.ch == 14)
        return "N disabled=true ";

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return NULL;
	}
    
    switch (wone_nr) 
    {
        case TIAP_PARAM_WONE_SINGLE_BAND:

		wone.index = 0;
		wone.type = wone_band;
		if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get band.");
			return NULL;
		}

            if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
                return "N disabled=true ";
            woneIndex = 0;
            break;

        case TIAP_PARAM_WONE_DUAL_BAND:

		wone.index = 0;
		wone.type = wone_band;
		if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get band.");
			return NULL;
		}

		if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
                woneIndex = 0;
            else
            {
		wone.index = 1;
		wone.type = wone_band;
		if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get band.");
			return NULL;
		}
                if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
                    woneIndex = 1;
                else
                    return "N disabled=true ";   /* both Wone 0,1 are 'A' */
            }
            break;

	default: 
	    return NULL;
    }
    /* We are in B/G band - skipping the B modes, remain only with G */
    wone.index = woneIndex;
    wone.type = wone_dot11_mode;
    if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
	    return NULL;
    }
    
    if ((wone.elem.dot11_mode != TIAP_PARAM_D11MODE_MIXED) && 
	(wone.elem.dot11_mode != TIAP_PARAM_D11MODE_OFDM))
        return "N disabled=true ";   /* not G only mode */

    tempbuf[1] = 0;

	sscanf(value,"%d",&rateInd);
	if(rateInd > TIAP_PARAM_OFDM_RATES_MAX || rateInd < 1)
	{
		tempbuf[0] = 'F';
		return tempbuf;
	}

	wone.index = woneIndex;
	wone.type = wone_oper_add;

	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get .11 mode.");
		return NULL;
	}

	if( wone.elem.ofdm_legacy.rates[rateInd-1] == TIAP_PARAM_RATE_BASIC_SUPPORTED )
		tempbuf[0] = 'B';
	else if( wone.elem.ofdm_legacy.rates[rateInd-1] == TIAP_PARAM_RATE_SUPPORTED )
		tempbuf[0] = 'Y';	
	else
		tempbuf[0] = 'N';

	return tempbuf;

}


char* wlan_htmlGetOFDMOperRateSet_G_Band(char *value)
{
#if 0
	UINT8 curRates[OFDM_RATES_MAX];
	UINT8 curRatesLen=0;
	int   rateInd, i;
	UINT8 rateValue[OFDM_RATES_MAX] =
			{ DOT11_RATE_6,  DOT11_RATE_9,
			  DOT11_RATE_12, DOT11_RATE_18,
			  DOT11_RATE_24, DOT11_RATE_36,
			  DOT11_RATE_48, DOT11_RATE_54 };

    UINT8 TotalWones, band, woneIndex, dot11Mode;
    
    wdrv_dbGetNumSupWone(&TotalWones);
    switch (TotalWones) 
    {
        case WONE_SINGLE_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_5GHZ)
                return "N disabled=true ";
            woneIndex = 0;
            break;
        case WONE_DUAL_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_2_4GHZ)
                woneIndex = 0;
            else
            {
                wdrv_dbGetBand(1 ,&band);
                if (band == RADIO_BAND_2_4GHZ)
                    woneIndex = 1;
                else
                    return "N disabled=true ";   /* both Wone 0,1 are 'A' */
            }
            break;
    }
    /* We are in B/G band - skipping the B modes, remain only with G */
    wdrv_dbGetDot11Mode(woneIndex,&dot11Mode);
    if ((dot11Mode != DOT11_MODE_MIXED) && (dot11Mode != DOT11_MODE_OFDM_ONLY))
        return "N disabled=true ";   /* not G only mode */
    
    tempbuf[1] = 0;

	sscanf(value,"%d",&rateInd);
	if(rateInd > OFDM_RATES_MAX || rateInd < 1)
	{
		tempbuf[0] = 'F';
		return tempbuf;
	}

	whal_nvram_getOperationalOFDMRateSetLength(0,&curRatesLen);
	whal_nvram_getOperationalOFDMRateSet(0,curRates, curRatesLen);

	for (i=0; i<curRatesLen; i++)
	{
		if(rateValue[rateInd-1] == (curRates[i]&0x7F))
			break;
	}

	if(i >= curRatesLen)
		tempbuf[0] = 'N';
	else
	{
		if(curRates[i] & 0x80)
			tempbuf[0] = 'B';
		else
			tempbuf[0] = 'Y';
	}

	return tempbuf;
#endif

	int   rateInd;

	UINT8 woneIndex;
	__u8 wone_nr;
	tiap_param_wone_t wone;

	wone.index = 0;
	wone.type = wone_ch;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get channel.");
		return NULL;
	}

    if (wone.elem.ch == 14)
        return "N disabled=true ";

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return NULL;
	}

	switch (wone_nr) 
	{
		case TIAP_PARAM_WONE_SINGLE_BAND:

			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return NULL;
			}

			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
				return "N disabled=true ";
			woneIndex = 0;
			break;

		case TIAP_PARAM_WONE_DUAL_BAND:

			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return NULL;
			}

			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
				woneIndex = 0;
			else
			{
				wone.index = 1;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return NULL;
				}
				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
					woneIndex = 1;
				else
					return "N disabled=true ";   /* both Wone 0,1 are 'A' */
			}
			break;

		default: 
			return NULL;
	}
	/* We are in B/G band - skipping the B modes, remain only with G */
	wone.index = woneIndex;
	wone.type = wone_dot11_mode;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
		return NULL;
	}

	if ((wone.elem.dot11_mode != TIAP_PARAM_D11MODE_MIXED) && 
		(wone.elem.dot11_mode != TIAP_PARAM_D11MODE_OFDM))
		return "N disabled=true ";   /* not G only mode */

	tempbuf[1] = 0;

	sscanf(value,"%d",&rateInd);
	if(rateInd > TIAP_PARAM_OFDM_RATES_MAX || rateInd < 1)
	{
		tempbuf[0] = 'F';
		return tempbuf;
	}

	wone.index = woneIndex;
	wone.type = wone_oper_ofdm;

	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get .11 mode.");
		return NULL;
	}

	if( wone.elem.ofdm_extend.rates[rateInd-1] == TIAP_PARAM_RATE_BASIC_SUPPORTED )
		tempbuf[0] = 'B';
	else if( wone.elem.ofdm_extend.rates[rateInd-1] == TIAP_PARAM_RATE_SUPPORTED )
		tempbuf[0] = 'Y';	
	else
		tempbuf[0] = 'N';

	return tempbuf;
}


char* wlan_htmlGetAdditionalOperRateSet_A_Band(char *value)
{
#if 0
	UINT8 curRates[OFDM_RATES_MAX];
	UINT8 curRatesLen=0;
	int   rateInd, i;
	UINT8 rateValue[OFDM_RATES_MAX] =
			{ DOT11_RATE_6,  DOT11_RATE_9,
			  DOT11_RATE_12, DOT11_RATE_18,
			  DOT11_RATE_24, DOT11_RATE_36,
			  DOT11_RATE_48, DOT11_RATE_54 };

    UINT8 TotalWones, band, woneIndex, dot11Mode;
    
    wdrv_dbGetNumSupWone(&TotalWones);
    switch (TotalWones) 
    {
        case WONE_SINGLE_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band != RADIO_BAND_5GHZ)
                return "N disabled=true ";
            woneIndex = 0;
            break;

        case WONE_DUAL_BAND:
            wdrv_dbGetBand(0 ,&band);
            if (band == RADIO_BAND_5GHZ)
                woneIndex = 0;
            else
            {
                wdrv_dbGetBand(1 ,&band);
                if (band == RADIO_BAND_5GHZ)
                    woneIndex = 1;
                else
                    return "N disabled=true ";   /* both Wone 0,1 are 'B/G' */
            }
            break;
    }
    /* We are in A band - Verifying A mode */
    wdrv_dbGetDot11Mode(woneIndex,&dot11Mode);
    if (dot11Mode != DOT11_MODE_A_ONLY)
        return "N disabled=true ";   /* not G only mode */

    tempbuf[1] = 0;

	sscanf(value,"%d",&rateInd);
	if(rateInd > OFDM_RATES_MAX || rateInd < 1)
	{
		tempbuf[0] = 'F';
		return tempbuf;
	}

	whal_nvram_getOperationalAdditionalRateSetLength(woneIndex,&curRatesLen);
	whal_nvram_getOperationalAdditionalRateSet(woneIndex,curRates, curRatesLen);

	for (i=0; i<curRatesLen; i++)
	{
		if(rateValue[rateInd-1] == (curRates[i]&0x7F))
			break;
	}

	if(i >= curRatesLen)
		tempbuf[0] = 'N';
	else
	{
		if(curRates[i] & 0x80)
			tempbuf[0] = 'B';
		else
			tempbuf[0] = 'Y';
	}

	return tempbuf;
#endif

	int   rateInd;

	UINT8 woneIndex;
	__u8 wone_nr;
	tiap_param_wone_t wone;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return NULL;
	}

	switch (wone_nr) 
	{
		case TIAP_PARAM_WONE_SINGLE_BAND:

			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return NULL;
			}

			if (wone.elem.band != TIAP_PARAM_RADIO_BAND_5GHZ)
				return "N disabled=true ";
			woneIndex = 0;
			break;

		case TIAP_PARAM_WONE_DUAL_BAND:

			wone.index = 0;
			wone.type = wone_band;
			if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get band.");
				return NULL;
			}

			if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
				woneIndex = 0;
			else
			{
				wone.index = 1;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return NULL;
				}
				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
					woneIndex = 1;
				else
					return "N disabled=true ";   /* both Wone 0,1 are 'A' */
			}
			break;

		default: 
			return NULL;
	}
	/* We are in B/G band - skipping the B modes, remain only with G */
	wone.index = woneIndex;
	wone.type = wone_dot11_mode;
	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to set .11 mode.");
		return NULL;
	}

	if (wone.elem.dot11_mode != TIAP_PARAM_D11MODE_A)
		return "N disabled=true ";   /* not G only mode */

	tempbuf[1] = 0;

	sscanf(value,"%d",&rateInd);
	if(rateInd > TIAP_PARAM_OFDM_RATES_MAX || rateInd < 1)
	{
		tempbuf[0] = 'F';
		return tempbuf;
	}

	wone.index = woneIndex;
	wone.type = wone_oper_add;

	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get .11 mode.");
		return NULL;
	}

	if( wone.elem.ofdm_legacy.rates[rateInd-1] == TIAP_PARAM_RATE_BASIC_SUPPORTED )
		tempbuf[0] = 'B';
	else if( wone.elem.ofdm_legacy.rates[rateInd-1] == TIAP_PARAM_RATE_SUPPORTED )
		tempbuf[0] = 'Y';	
	else
		tempbuf[0] = 'N';

	return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlGetMaxBasicRate(void)							*/
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the max basic rate drop down		*/ 
/*				text box.												    */
/*																		    */
/*																		    */
/* INPUT:       Drop down value												*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"selected" if the value is identical to the AP's max basic rate	*/
/*				"" if the value is different								*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetMaxBasicRate(char* option)
{
#if 0
	UINT8 maxBasic;
	int rate;
    UINT8 TotalWones, band, woneIndex;

	sscanf(option,"%d",&rate);

    if (rate == 0) 
    {
        wdrv_dbGetNumSupWone(&TotalWones);
        switch (TotalWones) 
        {
            case WONE_SINGLE_BAND:
                wdrv_dbGetBand(0 ,&band);
                if (band == RADIO_BAND_5GHZ)
                    return " disabled=true ";
                woneIndex = 0;
                break;
        
            case WONE_DUAL_BAND:
                wdrv_dbGetBand(0 ,&band);
                if (band == RADIO_BAND_2_4GHZ)
                    woneIndex = 0;
                else
                {
                    wdrv_dbGetBand(1 ,&band);
                    if (band == RADIO_BAND_2_4GHZ)
                        woneIndex = 1;
                    else
                        return " disabled=true ";   /* both Wone 0,1 are 'A' */
                }
                break;
        }
    }
	wdrv_dbGetMaxCCKbasicRate(woneIndex,&maxBasic);
	
	switch(maxBasic)
	{
		case 0x82:
			maxBasic = 1;
			break;

		case 0x84:
			maxBasic = 2;
			break;

		case 0x8B:
			maxBasic = 3;
			break;

		case 0x96:
			maxBasic = 4;
			break;
	}

	if (maxBasic == rate)
		return "selected ";
	else 
		return " ";
#endif

	UINT8 maxBasic;
	int rate, i;
	UINT8 woneIndex=0;

	__u8 wone_nr;
	tiap_param_wone_t wone;

	sscanf(option,"%d",&rate);

	if (rate == 0) 
	{
		if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
			return NULL;
		}
		
		switch (wone_nr) 
		{
			case TIAP_PARAM_WONE_SINGLE_BAND:

				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return NULL;
				}

				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
					return " disabled=true ";
				woneIndex = 0;
				break;

			case TIAP_PARAM_WONE_DUAL_BAND:

				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return NULL;
				}

				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
					woneIndex = 0;
				else
				{
					wone.index = 1;
					wone.type = wone_band;
					if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to get band.");
						return NULL;
					}
					
					if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
						woneIndex = 1;
					else
						return " disabled=true ";   /* both Wone 0,1 are 'A' */
				}
				break;
			default:
				return NULL;
		}
	}

	maxBasic = 0;

	wone.index = woneIndex;
	wone.type = wone_oper_rate;

	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get .11 mode.");
		return NULL;
	}

	for(i=4; i>=0; i--)
	{
		if(wone.elem.b_legacy.rates[i] == TIAP_PARAM_RATE_BASIC_SUPPORTED)
		{
			maxBasic = i;
			break;
		}
	}

	maxBasic++;

	if (maxBasic == rate)
		return "selected ";
	else 
		return " ";

}

/****************************************************************************/
/*                      wlan_htmlGetMaxActiveRate(void)						*/
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the max active rate drop down	*/ 
/*				text box.												    */
/*																		    */
/*																		    */
/* INPUT:       Drop down value												*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"selected" if the value is identical to the AP's max active rate	*/
/*				"" if the value is different								*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetMaxActiveRate(char* option)
{
#if 0
	UINT8 maxActive;
	int rate;
	UINT8 TotalWones, band, woneIndex;

	sscanf(option,"%d",&rate);

	if (rate == 0) 
	{
		wdrv_dbGetNumSupWone(&TotalWones);
		switch (TotalWones) 
		{
			case WONE_SINGLE_BAND:
				wdrv_dbGetBand(0 ,&band);
                if (band == RADIO_BAND_5GHZ)
                    return " disabled=true ";
                woneIndex = 0;
                break;
        
            case WONE_DUAL_BAND:
                wdrv_dbGetBand(0 ,&band);
                if (band == RADIO_BAND_2_4GHZ)
                    woneIndex = 0;
                else
                {
                    wdrv_dbGetBand(1 ,&band);
                    if (band == RADIO_BAND_2_4GHZ)
                        woneIndex = 1;
                    else
                        return " disabled=true ";   /* both Wone 0,1 are 'A' */
                }
                break;
        }
    }
	wdrv_dbGetMaxCCKactiveRate(woneIndex,&maxActive);
	
	switch(maxActive)
	{
		case 0x02:
		case 0x82:
			maxActive = 1;
			break;

		case 0x04:
		case 0x84:
			maxActive = 2;
			break;

		case 0x0B:
		case 0x8B:
			maxActive = 3;
			break;

		case 0x16:
		case 0x96:
			maxActive = 4;
			break;

		case 0x2C:
			maxActive = 5;
			break;
	}

	if (maxActive == rate)
		return "selected ";
	else 
		return " ";
#endif

	UINT8 maxActive;
	int rate, i;
	UINT8 woneIndex=0;

	__u8 wone_nr;
	tiap_param_wone_t wone;

	sscanf(option,"%d",&rate);

	if (rate == 0) 
	{
		if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &wone_nr, !FROM_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
			return NULL;
		}
		
		switch (wone_nr) 
		{
			case TIAP_PARAM_WONE_SINGLE_BAND:

				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return NULL;
				}

				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_5GHZ)
					return " disabled=true ";
				woneIndex = 0;
				break;

			case TIAP_PARAM_WONE_DUAL_BAND:

				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return NULL;
				}

				if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
					woneIndex = 0;
				else
				{
					wone.index = 1;
					wone.type = wone_band;
					if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to get band.");
						return NULL;
					}
					
					if (wone.elem.band == TIAP_PARAM_RADIO_BAND_2_4GHZ)
						woneIndex = 1;
					else
						return " disabled=true ";   /* both Wone 0,1 are 'A' */
				}
				break;
			default:
				return NULL;
		}
	}

	maxActive = 0;

	wone.index = woneIndex;
	wone.type = wone_oper_rate;

	if( tiap_wcfg_get_wone(wd, &wone, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get .11 mode.");
		return NULL;
	}

	for(i = 4; i>=0; i--)
	{
		if((wone.elem.b_legacy.rates[i] == TIAP_PARAM_RATE_SUPPORTED) ||
		   (wone.elem.b_legacy.rates[i] == TIAP_PARAM_RATE_BASIC_SUPPORTED))
		{
			maxActive = i;
			break;
		}
	}

	maxActive++;

	if (maxActive == rate)
		return "selected ";
	else 
		return " ";
}



/****************************************************************************/
/*                      wlan_htmlSoftwareDownload(void)                         */
/****************************************************************************/
/* DESCRIPTION: Performs software download and reboot                       */ 
/*                                                                          */
/*                                                                          */
/* INPUT:       NONE                                                        */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:                                                                 */
/*                                                                          */
/****************************************************************************/
char* wlan_htmlSoftwareDownload(char* value, char *sector, char *file, 
                                char *tftpip1, char *tftpip2, char *tftpip3, char *tftpip4)
{
#if 0
    unsigned long IPAddr, sectorId, len, ipAddr1, ipAddr2, ipAddr3, ipAddr4, TftpServerIp;
    unsigned char filename[65];

    if (!strcmp(value,"Downloading"))
    {
        cm_hwGetHostIP(&IPAddr);
        sscanf(sector,"%d",&sectorId);

        len = strlen(file);
        if (len <= 64)
        {
            memcpy(filename, file, len);
            filename[len] = '\0';
        }
        else
            return "";

        /* TFTP ip address */
        sscanf(tftpip1,"%d",&ipAddr1);
        sscanf(tftpip2,"%d",&ipAddr2);
        sscanf(tftpip3,"%d",&ipAddr3);
        sscanf(tftpip4,"%d",&ipAddr4);
        TftpServerIp = (ipAddr1 << 24) | (ipAddr2 << 16) | (ipAddr3 << 8) | ipAddr4;
        cm_hwSetHostIP(TftpServerIp);

        /*cm_serviceEraseAndDownload(sectorId, IPAddr, filename);*/
        if (wlan_DwldEraseBurnSwImage(TftpServerIp, filename, sectorId) == OK)
        {
            switch (sectorId) 
            {
                case 1:
                case 2:
                    cm_serviceSetDef(sectorId);
                break;
            }
            consoleReboot();
        }
    }

    return "";
#endif
	return NULL;
}
 
/****************************************************************************/
/*                      wlan_htmlGetDownloadSector(void)                        */
/****************************************************************************/
/* DESCRIPTION: returns the current sector the software running from        */ 
/*                                                                          */
/*                                                                          */
/* INPUT:       NONE.                                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     current sector the software running from                    */
/*                                                                          */
/****************************************************************************/
char* wlan_htmlGetRunningSector(void)
{
#if 0
    UINT32 sector;

    tempbuf[0] = 0;
    
    cm_serviceGetRunningSector(&sector);
    sprintf(tempbuf,"%d",sector);

    return tempbuf;
#endif
	return NULL;
}


/****************************************************************************/
/*                      wlan_htmlGetRunningFilename(void)                       */
/****************************************************************************/
/* DESCRIPTION: returns the current running image                           */ 
/*                                                                          */
/*                                                                          */
/* INPUT:       NONE.                                                       */
/* OUTPUT:      NONE.                                                       */
/*                                                                          */
/* RETURNS:     current running image                                       */
/*                                                                          */
/****************************************************************************/
char* wlan_htmlGetRunningFilename(void)
{
#if 0
    tempbuf[0] = 0;
    
    cm_serviceGetRunningImage(tempbuf);

    return tempbuf;
#endif
	return NULL;
}



/****************************************************************************/
/*                      wlan_htmlGetTftpIpAddr(void)                            */
/****************************************************************************/
/* DESCRIPTION: Returns byte[0] pf the AP's TFTP SERVER ip address          */ 
/*                                                                          */
/*																		    */
/* INPUT:       Component ID (1-4).                                         */
/* OUTPUT:      NONE.                                                       */
/*																			*/
/* RETURNS:     Byte[0] of the AP's TFTOP Server ip address                 */
/*                                                                          */
/****************************************************************************/
char* wlan_htmlGetTftpIpAddr(char *ipComponent)
{
#if 0
	unsigned long IPAddr, component;

    cm_hwGetHostIP(&IPAddr);
    sscanf(ipComponent,"%d",&component);

    switch (component) 
    {
        case 1:
            sprintf(tempbuf,"%d",IPAddr>>24);
            break;
        case 2:
            sprintf(tempbuf,"%d",(IPAddr>>16)&0xFF);
            break;
        case 3:
            sprintf(tempbuf,"%d",(IPAddr>>8)&0xFF);
            break;
        case 4:
            sprintf(tempbuf,"%d",(IPAddr & 0xFF));
            break;
    }
	return tempbuf;
#endif
	return NULL;
}




int printRatesInMbpsUnit(char *tmp, UINT8 rate)
{
#if 0
    UINT32 rate1, rate2, rate3;
    BOOL basic;
    int  len=0;

    if (rate)
    {
        rate1 = rate;
        basic = rate1 & 0x80;
        rate1 &= 0x7f;
        rate1 *= 500;
        rate2 = rate1/1000;
        rate3 = rate1%1000;

        if (rate3)
        {
            len = sprintf(tmp,"%d.%dMbps", rate2, 5);
            tmp += len;
        }
        else
        {
            len = sprintf(tmp,"%dMbps", rate2);
            tmp += len;
        }

        if (basic)
            len += sprintf(tmp,"B");

        return len;
    }
#endif
	return 0;
}


/****************************************************************************/
/*                      wlan_htmlGetAllRates(void)                              */
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the max basic rate drop down     */ 
/*              text box.                                                   */
/*                                                                          */
/*                                                                          */
/* INPUT:       Drop down value                                             */
/* OUTPUT:                                                                  */
/*                                                                          */
/* RETURNS: "selected" if the value is identical to the AP's max basic rate */
/*          "" if the value is different                                    */
/*                                                                          */
/****************************************************************************/
char *wlan_htmlGetAllRates(void)
{
/*
	UINT8 maxBasic;
	int rate;

	sscanf(option,"%d",&rate);

	wdrv_dbGetDefaultActiveRate(&maxBasic);
	switch(maxBasic)
	{
		case 0x82:
			maxBasic = 1;
			break;

		case 0x84:
			maxBasic = 2;
			break;

		case 0x8B:
			maxBasic = 3;
			break;

		case 0x96:
			maxBasic = 4;
			break;
	}

	if (maxBasic == rate)
		return "selected";
	else 
		return " ";
*/

#if 0
	char allRates[CCK_PBCC_RATES_MAX+OFDM_RATES_MAX];
	UINT8 allRatesLen;
	char *buf = tempbuf;
	int i;
	int len;

	tempbuf[0] = 0;

	wdrv_dbGetStationConfigOperationalAllRateSet(0,allRates, &allRatesLen);

	len = sprintf(buf," ");
	buf += len;
	for (i = 0; i < allRatesLen; i++)
	{		 
		len = printRatesInMbpsUnit(buf, allRates[i]);
		if(len)
		{
			buf += len;
			if(i<allRatesLen-1)
			{			
				len = sprintf(buf,"__");
				buf += len;
			}
		}
	}

	return tempbuf;
#endif
	return NULL;
}


/********************************
*		STATISTICS PAGE				*	
*********************************/

/* LAN SEND COUNTERS */


static tiap_param_net_stats stats;
/****************************************************************************/
/*                      wlan_htmlGetLanSendUnicast(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of unicast frames sent to the lan		*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of unicast frames sent to the lan					*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetLanSendUnicast(void)
{
		
	if( tiap_wcfg_get_priv_net_stats(wd, &stats) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get net statistics.");
		return NULL;
	}

	sprintf(tempbuf, "%d", stats.bss.tx_frames);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetLanSendMulticast(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of multicast frames sent to the lan		*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of multicast frames sent to the lan					*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetLanSendMulticast(void)
{

	sprintf(tempbuf, "%d", stats.bss.tx_multicast_frames);

	return tempbuf;
}

/* LAN RECEIVE COUNTERS */

/****************************************************************************/
/*                      wlan_htmlGetLanRecvUnicast(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of unicast frames recieved from the lan	*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of unicast frames recieved from the lan				*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetLanRecvUnicast(void)
{
	sprintf(tempbuf, "%u", stats.bss.rx_frames);

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetLanRecvMulticast(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of multicast frames recieved from the lan*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of multicast frames recieved from the lan			*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetLanRecvMulticast(void)
{
	sprintf(tempbuf, "%u", stats.bss.rx_multicast_frames);

	return tempbuf;

}


/* WIRELESS SEND COUNTERS */

/****************************************************************************/
/*                      wlan_htmlGetWirelessSendFragments(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of mpdus sent to the wlan				*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of mpdus sent to the wlan							*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessSendFragments(void)
{
	sprintf(tempbuf, "%u", stats.wireless.tx_mpdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetWirelessSendMsdus(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of msdus sent to the wlan				*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of msdus sent to the wlan							*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessSendMsdus(void)
{
	sprintf(tempbuf, "%u", stats.wireless.tx_msdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetWirelessSendMulticasts(void)				*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of multicast msdus sent to the wlan		*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of multicast msdus sent to the wlan					*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessSendMulticasts(void)
{
	sprintf(tempbuf, "%u", stats.wireless.tx_multicast_msdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetWirelessSendFailed(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of msdus failed to sent to the wlan		*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of msdus failed to sent to the wlan					*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessSendFailed(void)
{
	sprintf(tempbuf, "%u", stats.wireless.tx_failed_msdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetWirelessSendRetry(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of msdus that were retried to sent to the wlan				*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of msdus that were retried to sent to the wlan		*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessSendRetry(void)
{
	sprintf(tempbuf, "%u", stats.wireless.tx_retry_msdus);

	return tempbuf;
}


/* WIRELESS RECEIVE COUNTERS */

/****************************************************************************/
/*                      wlan_htmlGetWirelessRecvFragments(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of mpdus received from the wlan			*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of mpdus received from the wlan						*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessRecvFragments(void)
{
	sprintf(tempbuf, "%u", stats.wireless.rx_mpdus);

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetWirelessRecvMsdus(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of msdus received from the wlan			*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of msdus received from the wlan						*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessRecvMsdus(void)
{
	sprintf(tempbuf, "%u", stats.wireless.rx_msdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetWirelessRecvMulticasts(void)				*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of multicast msdus received from the wlan*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of multicast msdus received from the wlan			*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessRecvMulticasts(void)
{
	sprintf(tempbuf, "%u", stats.wireless.rx_multicast_msdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetWirelessRecvFcsError(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of mpdus received from the wlan with FCS error				*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of mpdus received from the wlan with FCS error		*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetWirelessRecvFcsError(void)
{
	sprintf(tempbuf, "%u", stats.wireless.rx_fcs_error_mpdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetMicFailureMSDUs(void)				*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of MIC failure MSDUs received from the wlan*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of MIC failure MSDUs received from the wlan			*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetMicFailureMSDUs(void)
{
	sprintf(tempbuf, "%u", stats.wireless.rx_mic_failure_msdus);

	return tempbuf;
}

/****************************************************************************/
/*                      wlan_htmlGetDecryptErrorMPDUs(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the number of decrypt error MPDUs					*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     number of decrypt error MPDUs								*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetDecryptErrorMPDUs(void)
{
	sprintf(tempbuf, "%u", stats.wireless.rx_decrypt_error_mpdus);

	return tempbuf;
}

/********************************
*       PRIVACY PAGE            *	
*********************************/

/****************************************************************************/
/*                      wlan_htmlGetWepCheck(void)								*/
/****************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the wep mode radio buttons			*/ 
/*																		    */
/*																		    */
/* INPUT:       Radio button value											*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"checked" if the value is identical to the AP's WEP & authentication modes	*/
/*				"" if the value is different than the AP's WEP & authentication modes mode			*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetWepCheck(char* value)
{
#if 0
	int inputWep, i, mode;
	BOOL privacyOption, openSystemEnable = FALSE, sharedKeyEnable = FALSE;
	UINT16 authAlgo;
	BOOL enable;

	sscanf(value,"%d",&inputWep);

	wdrv_dbGetStationConfigPrivacyOptionImplemented(&privacyOption);

	for (i = 0; i < MAX_AUTH_ALGORITHM_SUPPORTED; i++)
	{
		wdrv_dbGetAuthAlgo(i, &authAlgo);
		wdrv_dbGetAuthAlgoEnable(i, &enable);

		if ((authAlgo == MIB_OPEN_SYSTEM) && (enable == TRUE))
			openSystemEnable = TRUE;
		else if ((authAlgo == MIB_SHARED_KEY) && (enable == TRUE))
			sharedKeyEnable = TRUE;
	}
	
	if (!privacyOption)
		mode = 1;
	else
	{
		if ((openSystemEnable == TRUE) && (sharedKeyEnable == FALSE))
			mode = 2;

		else if ((openSystemEnable == FALSE) && (sharedKeyEnable == TRUE))
			mode = 3;

		else if ((openSystemEnable == TRUE) && (sharedKeyEnable == TRUE))
			mode = 4;
	}

	if (inputWep == mode)
		return "checked";
	else 
		return " ";
#endif
	return NULL;
}

/****************************************************************************/
/*                      wlan_htmlSavePagePrivacy(void)							*/
/****************************************************************************/
/* DESCRIPTION: Update the driver database with the							*/
/*				parameters values from the privacy page.					*/ 
/*																		    */
/*																		    */
/* INPUT:       Privacy page parameters				                        */
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:																	*/
/*																			*/
/****************************************************************************/
char* wlan_htmlSavePagePrivacy(char* value, char *wepCipherInput, char *keyIdInput, char *keyStr1Input, 
                               char *keyStr2Input, char *keyStr3Input, char *keyStr4Input, char *stationTypesInput_WEP, 
                               char *stationTypesInput_1X, char *stationTypesInput_WPA, char *privacyOnInput, 
                               char *privacyOffInput, char *pskModeInput, char *pskHexButton, char *pskStringButton, 
                               char *pskHexInput, char *pskStringInput, char *authTypeInput, char *keyIntervalInput, 
                               char *radiusIp1Input, char *radiusIp2Input, char *radiusIp3Input, char *radiusIp4Input, 
                               char *radiusPortInput, char *radiusSecretInput)
{
#if 0
    unsigned long authType, keyId, wepCipher, keyStr0, keyStr1, keyStr2, keyStr3, tempKeyByte, keyStrLen, 
				  ipAddr1, ipAddr2, ipAddr3, ipAddr4, radiusPort ,len, pskMode, pskModeHex, pskModeString, 
                  gtkInterval, privacyOn, privacyOff, workingMode, tempHexByte, 
                  stationTypes_WEP, stationTypes_1X, stationTypes_WPA;

	char key[WEP_KEY_LEN_MAX * 2], keyByte[(WEP_KEY_LEN_MAX * 2 * 2) + 2];
	char hex[32 * 2], hexByte[(32 * 2 * 2) + 2];
	int j, i;
	
	if (!strcmp(value,"Saving"))
	{

		/*		Working Mode  */
        stationTypes_WEP = atoi(stationTypesInput_WEP);
        stationTypes_1X = atoi(stationTypesInput_1X);
        stationTypes_WPA = atoi(stationTypesInput_WPA);
		workingMode = 0;

        switch (stationTypes_WEP | stationTypes_1X | stationTypes_WPA) 
        {
            case 2:     /* WEP */
                workingMode = DOT11_WPA_STA_TYPE_WEP_LEGACY;

                        /* Auth Type */
                sscanf(authTypeInput,"%d",&authType);
				
                switch (authType)
                {
                    case 1:
                        wdrv_dbSetAuthAlgo(0, MIB_OPEN_SYSTEM);
                        wdrv_dbSetAuthAlgoEnable(0, TRUE);
                        wdrv_dbSetAuthAlgo(1, MIB_SHARED_KEY);
                        wdrv_dbSetAuthAlgoEnable(1, FALSE);
                        break;
    
                    case 2:
                        wdrv_dbSetAuthAlgo(0, MIB_OPEN_SYSTEM);
                        wdrv_dbSetAuthAlgoEnable(0, FALSE);
                        wdrv_dbSetAuthAlgo(1, MIB_SHARED_KEY);
                        wdrv_dbSetAuthAlgoEnable(1, TRUE);
                        break;
    
                    case 3:
                        wdrv_dbSetAuthAlgo(0, MIB_OPEN_SYSTEM);
                        wdrv_dbSetAuthAlgoEnable(0, TRUE);
                        wdrv_dbSetAuthAlgo(1, MIB_SHARED_KEY);
                        wdrv_dbSetAuthAlgoEnable(1, TRUE);
                        break;
    
                    default:
                        break;
                }

                /* Privacy */
                sscanf(privacyOnInput,"%d",&privacyOn);
                sscanf(privacyOffInput,"%d",&privacyOff);
                if (privacyOn == 2) 
                    wdrv_dbSetStationConfigPrivacyOptionImplemented(1);
                else if (privacyOff == 1)
                    wdrv_dbSetStationConfigPrivacyOptionImplemented(0);

                /* key ID */
                sscanf(keyIdInput,"%d",&keyId);
                wdrv_dbSetPrivacyWEPDefaultKeyId((UINT8)keyId-1);

        		/* WEP Cipher */
        		sscanf(wepCipherInput,"%d",&wepCipher);
        		if (wepCipher == 1)
        		{
        			keyStrLen = 5;
        			wdrv_dbSetDefaultCipher(DOT11_CIPHER_WEP_40);
        		}
        		else if (wepCipher == 2)
        		{
        			keyStrLen = 13;
        			wdrv_dbSetDefaultCipher(DOT11_CIPHER_WEP_104);
        		}
        		else if (wepCipher == 3)
        		{
        			keyStrLen = 29;
        			wdrv_dbSetDefaultCipher(DOT11_CIPHER_WEP_232);
        		} 

                /* key 0 */
                memset(keyByte, 0, sizeof(keyByte));

		
                for (j = 0; j < keyStrLen; j++)
                {
                    memcpy(&keyByte[j * 2], keyStr1Input + (j * 2), 2);
                    sscanf(&keyByte[j * 2], "%x", &tempKeyByte);
                    key[j] = (char)tempKeyByte;
                }
                wdrv_dbSetWEPDefaultKeys(0, keyStrLen, key);

                /* key 1 */
                memset(keyByte, 0, sizeof(keyByte));

                for (j = 0; j < keyStrLen; j++)
                {
                    memcpy(&keyByte[j * 2], keyStr2Input + (j * 2), 2);
                    sscanf(&keyByte[j * 2], "%x", &tempKeyByte);
                    key[j] = (char)tempKeyByte;
                }
                wdrv_dbSetWEPDefaultKeys(1, keyStrLen, key);

                /* key 2 */
                memset(keyByte, 0, sizeof(keyByte));

                for (j = 0; j < keyStrLen; j++)
                {
                    memcpy(&keyByte[j * 2], keyStr3Input + (j * 2), 2);
                    sscanf(&keyByte[j * 2], "%x", &tempKeyByte);
                    key[j] = (char)tempKeyByte;
                }
                wdrv_dbSetWEPDefaultKeys(2, keyStrLen, key);

                /* key 3 */
                memset(keyByte, 0, sizeof(keyByte));

                for (j = 0; j < keyStrLen; j++)
                {
                    memcpy(&keyByte[j * 2], keyStr4Input + (j * 2), 2);
                    sscanf(&keyByte[j * 2], "%x", &tempKeyByte);
                    key[j] = (char)tempKeyByte;
                }
                wdrv_dbSetWEPDefaultKeys(3, keyStrLen, key);
                break;

            case 1:     /* WPA */
                workingMode = DOT11_WPA_STA_TYPE_WPA;
                /*		PSK		*/
                /* Mode */
                sscanf(pskModeInput,"%d",&pskMode);
                sscanf(pskHexButton,"%d",&pskModeHex);
                sscanf(pskStringButton,"%d",&pskModeString);
                if (pskMode ==1) 
                {
                    wdrv_dbSetPassPhraseMode(DOT11_WPA_PSK_PARAM_NONE);
                }
                else if (pskModeHex == 2) 
                {
                    pskMode = pskModeHex;
                    /* Hex number */
                    memset(hexByte, 0, sizeof(hexByte));

                    for (j = 0; j < 32; j++)
                    {
                        memcpy(&hexByte[j * 2], pskHexInput + (j * 2), 2);
                        sscanf(&hexByte[j * 2], "%x", &tempHexByte);
                        hex[j] = (char)tempHexByte;
                    }
                    wdrv_dbSetPskHex((UINT8 *)hex);
                    wdrv_dbSetPassPhraseMode(DOT11_WPA_PSK_PARAM_HEX);
                }
                else if (pskModeString == 4) 
                {
                    pskMode = pskModeString;
                    /* Password */
                    len = strlen(pskStringInput);
                    if (len <= DOT11_MAX_RADIUS_SECRET_LEN)
                        whal_nvram_setPskPwd(pskStringInput, len, IMMEDIATELY);
                    wdrv_dbSetPassPhraseMode(DOT11_WPA_PSK_PARAM_ASCII);
                }

                /* Note: do not put a break here!
                   Proceed to Key Interval, in case 2 (1.x) */

            case 4:     /* 802.1x */
                if (workingMode != DOT11_WPA_STA_TYPE_WPA)   /* in case proceeds from 'case 4' above */
                    workingMode = DOT11_WPA_STA_TYPE_WEP_8021X;
                /* GTKK Interval */
                sscanf(keyIntervalInput,"%d",&gtkInterval);
				
				if ((gtkInterval > 0) && (gtkInterval < MIN_GTK_PERIOD))
					gtkInterval = MIN_GTK_PERIOD;

				wdrv_dbSetGTKInterval(gtkInterval * 1000);
                break;
            
            default:
                return "";
        }
				
		wdrv_dbSetWpaMode(workingMode);
		
		/*		RADIUS		*/
        if (workingMode != DOT11_WPA_STA_TYPE_WEP_LEGACY) 
        {
            /* ip address */
		    sscanf(radiusIp1Input,"%d",&ipAddr1);
		    sscanf(radiusIp2Input,"%d",&ipAddr2);
		    sscanf(radiusIp3Input,"%d",&ipAddr3);
		    sscanf(radiusIp4Input,"%d",&ipAddr4);
		    wdrv_dbSetRadiusIpAddr(ipAddr1, ipAddr2, ipAddr3, ipAddr4);
    
	      	/* port */
		    sscanf(radiusPortInput,"%d",&radiusPort);
		    wdrv_dbSetRadiusPort(radiusPort);

		    /* secret */
		    len = strlen(radiusSecretInput);
		    if (len <= DOT11_MAX_RADIUS_SECRET_LEN)
			    whal_nvram_setRadiusSecret(radiusSecretInput, len, IMMEDIATELY);
        }

		consoleReboot(void);
	}

	return "";
#endif

	unsigned long authType, keyId, wepCipher, tempKeyByte, keyStrLen, 
	ipAddr1, ipAddr2, ipAddr3, ipAddr4, len, pskMode, pskModeHex, pskModeString, 
	gtkInterval, privacyOn, privacyOff, workingMode, 
	stationTypes_WEP, stationTypes_1X, stationTypes_WPA;

	char keyByte[(TIAP_WPA_WEP_KEY_LEN_MAX * 2 * 2) + 2];
	char inputStr[TIAP_WPA_WEP_KEY_LEN_MAX*2];
	
	int j;

	__u8 priv_option;
	__u16 radius_port;
	tiap_param_auth_algo_t tbl[TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED];
	tiap_wpa_wep_key_t wep_key;

	if (!strcmp(value,"Saving"))
	{

		storage_backup = wd->storage;


		/*		Working Mode  */
		stationTypes_WEP = atoi(stationTypesInput_WEP);
		stationTypes_1X = atoi(stationTypesInput_1X);
		stationTypes_WPA = atoi(stationTypesInput_WPA);
		workingMode = 0;

		switch (stationTypes_WEP | stationTypes_1X | stationTypes_WPA) 
		{
			case 2:     /* WEP */
				workingMode |= TIAP_PARAM_SECURITY_WEP;

				/* Auth Type */
				sscanf(authTypeInput,"%ld",&authType);
				
				switch (authType)
				{
					case 1:
						tbl[0].algo = TIAP_PARAM_MIB_OPEN_SYSTEM;
						tbl[0].enable = TRUE;
						
						tbl[1].algo = TIAP_PARAM_MIB_SHARED_KEY;
						tbl[1].enable = FALSE;
						if (tiap_wcfg_set_auth_type(wd, tbl)) {
							PRINT_CONFIG_ERR_MSG("Failed to set authentication algorithm table");
							return NULL;
						}
						break;

					case 2:
						tbl[0].algo = TIAP_PARAM_MIB_OPEN_SYSTEM;
						tbl[0].enable = FALSE;
						
						tbl[1].algo = TIAP_PARAM_MIB_SHARED_KEY;
						tbl[1].enable = TRUE;
						if (tiap_wcfg_set_auth_type(wd, tbl)) {
							PRINT_CONFIG_ERR_MSG("Failed to set authentication algorithm table");
							return NULL;
						}
						break;

					case 3:
						tbl[0].algo = TIAP_PARAM_MIB_OPEN_SYSTEM;
						tbl[0].enable = TRUE;
						
						
						tbl[1].algo = TIAP_PARAM_MIB_SHARED_KEY;
						tbl[1].enable = TRUE;
						if (tiap_wcfg_set_auth_type(wd, tbl)) {
							PRINT_CONFIG_ERR_MSG("Failed to set authentication algorithm table");
							return NULL;
						}
						break;

					default:
						break;
				}

				/* Privacy */
				sscanf(privacyOnInput,"%ld",&privacyOn);
				sscanf(privacyOffInput,"%ld",&privacyOff);
				if (privacyOn == 2)
				{
					priv_option = 1;
					if (tiap_wcfg_set_basic(wd, TIAP_PARAM_PRIV_OPTION, &priv_option, !UPDATE_STORAGE)) 
					{
						PRINT_CONFIG_ERR_MSG("failed to set privacy option");
						return NULL;
					}
				}
				else if (privacyOff == 1)
				{
					priv_option = 0;
					if (tiap_wcfg_set_basic(wd, TIAP_PARAM_PRIV_OPTION, &priv_option, !UPDATE_STORAGE))
				       	{
						PRINT_CONFIG_ERR_MSG("failed to set privacy option");
						return NULL;
					}
				}

				/* key ID */
				sscanf(keyIdInput,"%ld",&keyId);
			
				if (tiap_wcfg_set_wep_default_key_id(wd, (UINT8)keyId-1))
					PRINT_CONFIG_ERR_MSG("failed to set wep default key id");

				/* WEP Cipher */
				sscanf(wepCipherInput,"%lu",&wepCipher);
				
				if (wepCipher == 1)
				{
					keyStrLen = 5;
					tiap_wcfg_set_default_cipher(wd, TIAP_WPA_CIPHER_WEP_40);
				}
				else if (wepCipher == 2)
				{
					keyStrLen = 13;
					tiap_wcfg_set_default_cipher(wd, TIAP_WPA_CIPHER_WEP_104);
				}
				else if (wepCipher == 3)
				{
					keyStrLen = 29;
					tiap_wcfg_set_default_cipher(wd, TIAP_WPA_CIPHER_WEP_232);
				} 
				else {
					printf("%s: invalid web cipher = %lu", __func__, wepCipher);
					return NULL;
				}

				

				/*	
				if (	strlen(keyStr1Input) != keyStrLen * 2 ||
					strlen(keyStr2Input) != keyStrLen * 2 ||
					strlen(keyStr3Input) != keyStrLen * 2 ||
					strlen(keyStr4Input) != keyStrLen * 2) 
				{
					wd->storage = storage_backup;
					sprintf(msg, "Each WEP key should be of length %ld",keyStrLen*2);
					create_js_msg_box(tempbuf,msg);
					return tempbuf;

				}
				*/

				/* key 0 */
				memset(keyByte, 0, sizeof(keyByte));
				
				memset(inputStr,'0',sizeof(inputStr));
				memcpy(inputStr, keyStr1Input, min(sizeof(inputStr), strlen(keyStr1Input)));

				for (j = 0; j < keyStrLen; j++)
				{
					//memcpy(&keyByte[j * 2], keyStr1Input + (j * 2), 2);
					memcpy(&keyByte[j * 2], inputStr + (j * 2), 2);
					sscanf(&keyByte[j * 2], "%lx", &tempKeyByte);
					wep_key.key[j] = (char)tempKeyByte;

				}

				wep_key.len = keyStrLen;
				if(tiap_wcfg_set_def_wep_key(wd, 0, &wep_key))
					PRINT_CONFIG_ERR_MSG("failed to set wep default key 5");

				/* key 1 */
				memset(keyByte, 0, sizeof(keyByte));
				
				memset(inputStr,'0',sizeof(inputStr));
				memcpy(inputStr, keyStr2Input,  min(sizeof(inputStr), strlen(keyStr2Input)));

				for (j = 0; j < keyStrLen; j++)
				{
					//memcpy(&keyByte[j * 2], keyStr2Input + (j * 2), 2);
					memcpy(&keyByte[j * 2], inputStr + (j * 2), 2);
					sscanf(&keyByte[j * 2], "%lx", &tempKeyByte);
					wep_key.key[j] = (char)tempKeyByte;
				}
				wep_key.len = keyStrLen;
				if(tiap_wcfg_set_def_wep_key(wd, 1, &wep_key))
					PRINT_CONFIG_ERR_MSG("failed to set wep default key 5");

				/* key 2 */
				memset(keyByte, 0, sizeof(keyByte));
				memset(inputStr,'0',sizeof(inputStr));
				memcpy(inputStr, keyStr3Input,  min(sizeof(inputStr), strlen(keyStr3Input)));


				for (j = 0; j < keyStrLen; j++)
				{
					//memcpy(&keyByte[j * 2], keyStr3Input + (j * 2), 2);
					memcpy(&keyByte[j * 2], inputStr + (j * 2), 2);
					sscanf(&keyByte[j * 2], "%lx", &tempKeyByte);
					wep_key.key[j] = (char)tempKeyByte;
				}
				wep_key.len = keyStrLen;
				if(tiap_wcfg_set_def_wep_key(wd, 2, &wep_key))
					PRINT_CONFIG_ERR_MSG("failed to set wep default key 5");

				/* key 3 */
				memset(keyByte, 0, sizeof(keyByte));
				memset(inputStr,'0',sizeof(inputStr));
				memcpy(inputStr, keyStr4Input,  min(sizeof(inputStr), strlen(keyStr4Input)));


				for (j = 0; j < keyStrLen; j++)
				{
					//memcpy(&keyByte[j * 2], keyStr4Input + (j * 2), 2);
					memcpy(&keyByte[j * 2], inputStr + (j * 2), 2);
					sscanf(&keyByte[j * 2], "%lx", &tempKeyByte);
					wep_key.key[j] = (char)tempKeyByte;
				}
				wep_key.len = keyStrLen;
				if(tiap_wcfg_set_def_wep_key(wd, 3, &wep_key))
					PRINT_CONFIG_ERR_MSG("failed to set wep default key 5");
			
				break;

			case 1:     /* WPA */
				workingMode |= TIAP_PARAM_SECURITY_WPA;
				/*		PSK		*/
				/* Mode */
				sscanf(pskModeInput,"%ld",&pskMode);
				sscanf(pskHexButton,"%ld",&pskModeHex);
				sscanf(pskStringButton,"%ld",&pskModeString);
				if (pskMode ==1) 
				{
					if (tiap_wcfg_set_psk_passphrase_mode(wd, TIAP_WPA_PSK_PARAM_NONE))
						PRINT_CONFIG_ERR_MSG("Failed to set psk passphrase mode");
				}
				else if (pskModeHex == 2) 
				{
#if 0
					pskMode = pskModeHex;
					/* Hex number */
					memset(hexByte, 0, sizeof(hexByte));

					for (j = 0; j < 32; j++)
					{
						memcpy(&hexByte[j * 2], pskHexInput + (j * 2), 2);
						sscanf(&hexByte[j * 2], "%x", &tempHexByte);
						hex[j] = (char)tempHexByte;
					}
					wdrv_dbSetPskHex((UINT8 *)hex);
					wdrv_dbSetPassPhraseMode(DOT11_WPA_PSK_PARAM_HEX);
#endif
					PRINT_NOT_SUPPORTED_MSG("Psk hex mode");
				}
				else if (pskModeString == 4) 
				{
					pskMode = pskModeString;
					/* Password */
					len = strlen(pskStringInput);
					if (len > TIAP_WPA_MAX_PSK_PWD_LEN || len < TIAP_WPA_MIN_PSK_PWD_LEN) {
						wd->storage = storage_backup;
						create_js_msg_box(tempbuf,"Valid PSK password length is from 8 up to 63 symbols inclusive");
						return tempbuf;
					}


						if (tiap_wcfg_set_psk_passwd(wd, pskStringInput, len)) {
							PRINT_CONFIG_ERR_MSG("Failed to set psk passphrase");
							return NULL;
						}
					
					
					if (tiap_wcfg_set_psk_passphrase_mode(wd, TIAP_WPA_PSK_PARAM_ASCII))
						PRINT_CONFIG_ERR_MSG("Failed to set psk passphrase mode");
				}

				/* Note: do not put a break here!
				   Proceed to Key Interval, in case 2 (1.x) */

			case 4:     /* 802.1x */
				if (!(workingMode & TIAP_PARAM_SECURITY_WPA))   /* in case proceeds from 'case 4' above */
					workingMode |= TIAP_PARAM_SECURITY_802_1X;
				/* GTKK Interval */
				sscanf(keyIntervalInput,"%ld",&gtkInterval);

				if ((gtkInterval > 0) && (gtkInterval < TIAP_WPA_MIN_GTK_PERIOD))
					gtkInterval = TIAP_WPA_MIN_GTK_PERIOD;

				if (tiap_wcfg_set_gtk_interval(wd, gtkInterval * 1000)) {
					PRINT_CONFIG_ERR_MSG("Failed to set gtk interval");
					return NULL;
				}
				break;

			default:
				return "";
		}

		if(tiap_wcfg_set_basic(wd, TIAP_PARAM_SECURITY_TYPE, &workingMode, !UPDATE_STORAGE)) {
			PRINT_CONFIG_ERR_MSG("Failed to set security type.\n");
			return NULL;
		}

		/*		RADIUS		*/
		if (!(workingMode & TIAP_PARAM_SECURITY_WEP)) 
		{
			__u32 ipaddr;
			unsigned char *p = (unsigned char *)&ipaddr;
			int fmt_err = 0;

			/* ip address */
			if (sscanf(radiusIp1Input,"%ld",&ipAddr1)<= 0) fmt_err = 1;
			if (sscanf(radiusIp2Input,"%ld",&ipAddr2)<= 0) fmt_err = 1;
			if (sscanf(radiusIp3Input,"%ld",&ipAddr3)<= 0) fmt_err = 1;
			if (sscanf(radiusIp4Input,"%ld",&ipAddr4)<= 0) fmt_err = 1;

			if (fmt_err) 
				fprintf(stderr, "Wrong radius server IP format\n");
			else
			{
				p[0] = ipAddr1;
				p[1] = ipAddr2;
				p[2] = ipAddr3;
				p[3] = ipAddr4;
				if( tiap_wcfg_set_radius_ipaddr(wd, ipaddr) ) {
					PRINT_CONFIG_ERR_MSG("Failed to set radius IP address.");
					return NULL;
				}
			}

			/* port */
			sscanf(radiusPortInput,"%hu",&radius_port);
			if( tiap_wcfg_set_radius_port(wd, radius_port) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set radius port.");
				return NULL;
			}

			/* secret */
			len = strlen(radiusSecretInput);
			if (len <= TIAP_WPA_MAX_RADIUS_SECRET_LEN)
				if( tiap_wcfg_set_radius_secret(wd, radiusSecretInput, len) ) {
					PRINT_CONFIG_ERR_MSG("Failed to set radius secret.");
					return NULL;
				}
		}

		restart();
	}

	return "";

}

/****************************************************************************/
/*                      wlan_htmlGetKeyIdCheck(void)							*/
/****************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the default key id radio buttons	*/ 
/*																		    */
/*																		    */
/* INPUT:       Radio button value											*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"checked" if the value is identical to the AP's default key id value	*/
/*				"" if the value is different than the AP's default key id value			*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetKeyIdCheck(char* value)
{
#if 0
	int inputKeyId;
	UINT8 keyId;

	sscanf(value,"%d",&inputKeyId);

	wdrv_dbGetPrivacyWEPDefaultKeyId(&keyId);

	if (inputKeyId == keyId)
		return "checked";
	else 
		return " ";
#endif
	int inputKeyId;
	__u8 key_id;

	sscanf(value,"%d",&inputKeyId);

	tiap_wcfg_get_wep_default_key_id(wd, &key_id);

	if (inputKeyId == key_id)
		return "checked";
	else 
		return " ";
}

/****************************************************************************/
/*                      wlan_htmlGetKeyLenSelect0(void)							*/
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the first default wep key len drop down		*/ 
/*				menu.													    */
/*																		    */
/*																		    */
/* INPUT:       Drop down value												*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"selected" if the value is identical to the first default wep key len	*/
/*				"" if the value is different than the first default wep key len		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetKeyLenSelect0(char* option)
{
#if 0
	UINT8 keyLen;
	int inputKeyLen;
	char key[WEP_KEY_LEN_MAX];

	sscanf(option,"%d",&inputKeyLen);

	wdrv_dbGetWEPDefaultKeys(0, &keyLen, key);

	if (inputKeyLen == keyLen)
		return "selected";
	else 
		return " ";
#endif
	return NULL;
}

/****************************************************************************/
/*                      wlan_htmlGetKeyLenSelect1(void)							*/
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the second default wep key len drop down		*/ 
/*				menu.													    */
/*																		    */
/*																		    */
/* INPUT:       Drop down value												*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"selected" if the value is identical to the second wep default key len	*/
/*				"" if the value is different than the second wep default key len		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetKeyLenSelect1(char* option)
{
#if 0
	UINT8 keyLen;
	int inputKeyLen;
	char key[WEP_KEY_LEN_MAX];

	sscanf(option,"%d",&inputKeyLen);

	wdrv_dbGetWEPDefaultKeys(1, &keyLen, key);

	if (inputKeyLen == keyLen)
		return "selected";
	else 
		return " ";
#endif
	return NULL;
}

/****************************************************************************/
/*                      wlan_htmlGetKeyLenSelect2(void)							*/
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the third wep default key len drop down		*/ 
/*				menu.													    */
/*																		    */
/*																		    */
/* INPUT:       Drop down value												*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"selected" if the value is identical to the third wep default key len	*/
/*				"" if the value is different than the third wep default key len		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetKeyLenSelect2(char* option)
{
#if 0
	UINT8 keyLen;
	int inputKeyLen;
	char key[WEP_KEY_LEN_MAX];

	sscanf(option,"%d",&inputKeyLen);

	wdrv_dbGetWEPDefaultKeys(2, &keyLen, key);

	if (inputKeyLen == keyLen)
		return "selected";
	else 
		return " ";
#endif
	return NULL;
}

/****************************************************************************/
/*                      wlan_htmlGetKeyLenSelect3(void)							*/
/****************************************************************************/
/* DESCRIPTION: Select/unselect a value of the fourth wep default key len drop down		*/ 
/*				menu.													    */
/*																		    */
/*																		    */
/* INPUT:       Drop down value												*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		"selected" if the value is identical to the fourth wep default key len	*/
/*				"" if the value is different than the fourth wep default key len		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetKeyLenSelect3(char* option)
{
#if 0
	UINT8 keyLen;
	int inputKeyLen;
	char key[WEP_KEY_LEN_MAX];

	sscanf(option,"%d",&inputKeyLen);

	wdrv_dbGetWEPDefaultKeys(3, &keyLen, key);

	if (inputKeyLen == keyLen)
		return "selected";
	else 
		return " ";
#endif
	return NULL;
}


/********************************
*		MGMT PAGE				*	
*********************************/

/****************************************************************************/
/*                      wlan_htmlGetMgmtTableFirstRow(void)						*/
/****************************************************************************/
/* DESCRIPTION: Returns the management table first row (table header)		*/ 
/*																		    */
/*																		    */
/* INPUT:       															*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		Returns the management table first row. Formats it to html format		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetMgmtTableFirstRow(void)
{
	unsigned long len = 0;
	stationCurrentIndex = 0;

	tempbuf[0] = 0;
	
	len = sprintf(tempbuf,"<TABLE height=65 width=\"72%%\" border=1>\n");

	/* first row */
	len += sprintf(tempbuf + len,"<TBODY>\n");
	len += sprintf(tempbuf + len,"<TR>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"90%%\" bgColor=#9999ff colSpan=6 height=12>Associated STA Table</TD>\n");
	len += sprintf(tempbuf + len,"</TR>\n");

	return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlGetMgmtTableSecondRow(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns the management table second row (table header)		*/ 
/*																		    */
/*																		    */
/* INPUT:       															*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		Returns the management table secind row. Formats it to html format		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetMgmtTableSecondRow(void)
{
	unsigned long len = 0;

	tempbuf[0] = 0;
	
	/* second row */
	len += sprintf(tempbuf + len,"<tr>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"25%%\" bgColor=#ccccff height=11>Mac Address</TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"20%%\" bgColor=#ccccff height=11>State</TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"15%%\" bgColor=#ccccff height=11>SSID #</TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"15%%\" bgColor=#ccccff height=11>Active Rate</TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"15%%\" bgColor=#ccccff height=11>Ban STA</TD>\n");
	len += sprintf(tempbuf + len,"<TD align=middle width=\"15%%\" bgColor=#ccccff height=11>Security Status</TD>\n");
	len += sprintf(tempbuf + len,"</tr>\n");

	return tempbuf;
}


/****************************************************************************/
/*                      wlan_htmlGetMgmtTableDynamicRow(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns a management table row								*/ 
/*																		    */
/*																		    */
/* INPUT:       															*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:		Returns a management table row. Formats it to html format		*/
/*																			*/
/****************************************************************************/
char *wlan_htmlGetMgmtTableDynamicRow(void)
{
#if 0
	char address[MAC_ADDR_LEN], formatAddr[40];	
	UINT16 capabilities;
	UINT8 activeRate;
	unsigned long len = 0;
	UINT32 state;
    char ssid[33];
    int ssidLen;

	tempbuf[0] = 0;
	
	if (stationCurrentIndex == MAX_STATIONS)
		return tempbuf;

	wdrv_mgmtGetState(stationCurrentIndex, &state);
	
	if (state != STATE_IDLE)
	{
		/* mac address */
		wdrv_dbGetStaMacAddress(stationCurrentIndex, address);
        sprintf(formatAddr,"%02X-%02X-%02X-%02X-%02X-%02X",address[0],address[1],address[2],address[3],address[4],address[5]);
        if (strcmp(formatAddr,"00-00-00-00-00-00") == 0)
            return tempbuf;
		/* now pour it to the buffer */
		len += sprintf(tempbuf + len,"<tr>\n");
		len += sprintf(tempbuf + len,"<td align=middle width=\"25%%\" height=\"11\">%02X-%02X-%02X-%02X-%02X-%02X</td>\n", 
                       address[0],address[1],address[2],address[3],address[4],address[5]);
        /* formatAddr is used below - Banned Station */

		/* state */
		/* now pour it to the buffer */
		switch (state)	
		{
			case STATE_AUTH_KEY:
				len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"middle\"><font color=\"#00ffff\"><b>Auth key</b></font></td>\n");
				break;

			case STATE_AUTHENTICATED:
				len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#00ccff\"><b>Authenticated</b></td>\n");
				break;

			case STATE_ASSOCIATED:
				len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#00cc00\"><b>Associated</b></td>\n");
				break;

            case STATE_AUTHORIZED:
                len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#0000FF\"><b>Authorized</b></td>\n");
                break;

			default:
				break;
		}

        /* ssid */

        wdrv_dbGetStaSsidByIndex(stationCurrentIndex,ssid,&ssidLen);
        ssid[ssidLen] = '\0';
        if (strlen(ssid) == 0) 
            strcpy(ssid,"-");
        len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">%s</td>\n",ssid);
#if 0
		/* pbcc */
		if ((state == STATE_ASSOCIATED) || (state == STATE_AUTHORIZED))
		{
			wdrv_dbGetStationCapabilities(stationCurrentIndex, &capabilities);
			if ((capabilities >> CAP_PBCC_SHIFT) & CAP_PBCC_MASK)  		/* station supports PBCC */
				len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">Enable</td>\n");
			else
				len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">Disable</td>\n");
		}
		else
			len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">-----</td>\n");
#endif

		/* active rate */	
		if ((state == STATE_ASSOCIATED) || (state == STATE_AUTHORIZED))
		{
			wdrv_dbGetActiveRate(stationCurrentIndex, &activeRate);
			switch (activeRate)	
			{
				case DOT11_RATE_1:
					len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">1 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_2:
					len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">2 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_5_5:
					len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">5.5 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_6:
					len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">6 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_9:
					len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">9 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_11:
					len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">11 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_12:
					len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">12 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_18:
					len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">18 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_22:
					len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">22 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_24:
					len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">24 Mbit/sec</td>\n");
					break;
	
				case DOT11_RATE_36:
					len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">36 Mbit/sec</td>\n");
					break;
	
				case DOT11_RATE_48:
					len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">48 Mbit/sec</td>\n");
					break;
	
				case DOT11_RATE_54:
					len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">54 Mbit/sec</td>\n");
					break;

				default:
					break;
			}
		} /* STATE_ASSOCIATED */
		else
			len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\">-----</td>\n");

        /* Banned Station */

        /*<TD align=middle width="15%" height=11><INPUT style="FONT-SIZE: 8pt; POSITION: relative" onclick='this.value="MAC1"' type=submit value="    " name=BanSTA0></TD></TR>*/
        len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\"><INPUT style=\"FONT-SIZE: 8pt; POSITION: relative\" onclick='this.value=\"%s\"' type=submit value=\"    \" name=BanSTA></TD>\n",formatAddr);
		if (wdrv_wpaGetStationSecAttack(stationCurrentIndex))
                len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#FF0000\"><b>Attacker</b></td>\n");
		else
                len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#00CC99\"><b>Trusty</b></td>\n");
        len += sprintf(tempbuf + len,"</tr>\n");
	}	/* !STATE_IDLE */

	stationCurrentIndex++;
	if (stationCurrentIndex == MAX_STATIONS)
		len += sprintf(tempbuf + len,"</table>\n");

	return tempbuf;
#endif
















	__u8 hwaddr[ETH_ALEN], formatAddr[40];	
	unsigned long len = 0;
	__u32 state;
	char ssid[33];

	tiap_param_statbl_t sta_tbl[TIAP_PARAM_MAX_STATIONS];

	tempbuf[0] = 0;

	if (stationCurrentIndex == TIAP_PARAM_MAX_STATIONS)
		return tempbuf;

	if( tiap_wcfg_get_sta_table(wd, sta_tbl) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get sta table.");
		return NULL;
	}
	state = sta_tbl[stationCurrentIndex].state;

	if (state != TIAP_PARAM_SM_STATE_IDLE)
	{
		/* mac address */
		memcpy(hwaddr, sta_tbl[stationCurrentIndex].hwaddr, ETH_ALEN);
		sprintf(formatAddr,"%02X-%02X-%02X-%02X-%02X-%02X",hwaddr[0],hwaddr[1],hwaddr[2],
				hwaddr[3],hwaddr[4],hwaddr[5]);
		if (strcmp(formatAddr,"00-00-00-00-00-00") == 0)
			return tempbuf;
		/* now pour it to the buffer */
		len += sprintf(tempbuf + len,"<tr>\n");
		len += sprintf(tempbuf + len,
				"<td align=middle width=\"25%%\" height=\"11\">%02X-%02X-%02X-%02X-%02X-%02X</td>\n", 
				hwaddr[0],hwaddr[1],hwaddr[2],hwaddr[3],hwaddr[4],hwaddr[5]);
		/* formatAddr is used below - Banned Station */

		/* state */
		/* now pour it to the buffer */
		switch (state)	
		{
			case TIAP_PARAM_SM_STATE_AUTH_KEY:
				len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"middle\"><font color=\"#00ffff\"><b>Auth key</b></font></td>\n");
				break;

			case TIAP_PARAM_SM_STATE_AUTHENTICATED:
				len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#00ccff\"><b>Authenticated</b></td>\n");
				break;

			case TIAP_PARAM_SM_STATE_WAIT_FOR_ASS_RES:
				len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#0000FF\"><b>Wait for assoc response</b></td>\n");
				break;

			case TIAP_PARAM_SM_STATE_ASSOCIATED:
				len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#00cc00\"><b>Associated</b></td>\n");
				break;

			case TIAP_PARAM_SM_STATE_AUTHORIZED:
				len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#0000FF\"><b>Authorized</b></td>\n");
				break;	

			default:
				break;
		}

		/* ssid */
		strcpy(ssid, sta_tbl[stationCurrentIndex].ssid);
		if (strlen(ssid) == 0) 
			strcpy(ssid,"-");
		len += sprintf(tempbuf + len,
			"<td width=\"15%%\" height=\"11\" align=\"center\">%s</td>\n",
			ssid);

		/* active rate */	
		if ((state == TIAP_PARAM_SM_STATE_ASSOCIATED) || (state == TIAP_PARAM_SM_STATE_AUTHORIZED))
		{
			switch (sta_tbl[stationCurrentIndex].active_rate)	
			{
				case DOT11_RATE_1:
					len += sprintf(tempbuf + len,
						"<td width=\"15%%\" height=\"11\""
					       " align=\"center\">1 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_2:
					len += sprintf(tempbuf + len,
						"<td width=\"15%%\" height=\"11\""
					       " align=\"center\">2 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_5_5:
					len += sprintf(tempbuf + len,
						"<td width=\"15%%\" height=\"11\""
					       " align=\"center\">5.5 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_6:
					len += sprintf(tempbuf + len,
						"<td width=\"15%%\" height=\"11\""
					       " align=\"center\">6 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_9:
					len += sprintf(tempbuf + len,
						"<td width=\"15%%\" height=\"11\""
					       " align=\"center\">9 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_11:
					len += sprintf(tempbuf + len,
						"<td width=\"15%%\" height=\"11\""
					       " align=\"center\">11 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_12:
					len += sprintf(tempbuf + len,
						"<td width=\"15%%\" height=\"11\""
					       " align=\"center\">12 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_18:
					len += sprintf(tempbuf + len,
						"<td width=\"15%%\" height=\"11\""
					       " align=\"center\">18 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_22:
					len += sprintf(tempbuf + len,
						"<td width=\"15%%\" height=\"11\""
					       " align=\"center\">22 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_24:
					len += sprintf(tempbuf + len,
						"<td width=\"15%%\" height=\"11\""
					       " align=\"center\">24 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_36:
					len += sprintf(tempbuf + len,
						"<td width=\"15%%\" height=\"11\""
					       " align=\"center\">36 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_48:
					len += sprintf(tempbuf + len,
						"<td width=\"15%%\" height=\"11\""
					       " align=\"center\">48 Mbit/sec</td>\n");
					break;

				case DOT11_RATE_54:
					len += sprintf(tempbuf + len,
						"<td width=\"15%%\" height=\"11\""
					       " align=\"center\">54 Mbit/sec</td>\n");
					break;
	
				default:
					break;
			}
		} /* STATE_ASSOCIATED */
		else
			len += sprintf(tempbuf + len,
				"<td width=\"15%%\" height=\"11\" align=\"center\">-----</td>\n");

		/* Banned Station */

		len += sprintf(tempbuf + len,
			"<td width=\"15%%\" height=\"11\" align=\"center\">"
			"<INPUT style=\"FONT-SIZE: 8pt; POSITION: relative\" "
			"onclick='this.value=\"%s\"' type=submit value=\"    \""
			" name=BanSTA></TD>\n", formatAddr);
		
		//#warning "TODO: don't forget to implement wdrv_wpaGetStationSecAttack"
		/*undefine it when wdrv_wpaGetStationSecAttack will be implemented*/
		#if 0

			if (wdrv_wpaGetStationSecAttack(stationCurrentIndex))
				len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#FF0000\"><b>Attacker</b></td>\n");
			else
				len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#00CC99\"><b>Trusty</b></td>\n");
		#else 
			len += sprintf(tempbuf + len,"<td width=\"20%%\" height=\"11\" align=\"center\"><font color=\"#00CC9
					9\"><b>Trusty</b></td>\n");
		#endif


		len += sprintf(tempbuf + len,"</tr>\n");
	}	/* !STATE_IDLE */

	stationCurrentIndex++;
	if (stationCurrentIndex == TIAP_PARAM_MAX_STATIONS)
		len += sprintf(tempbuf + len,"</table>\n");

	return tempbuf;

	
}


/****************************************************************************/
/*                    wlan_htmlGetMgmt_BW_TableFirstRow(void)                   */
/****************************************************************************/
/* DESCRIPTION: Returns the management table first row (table header)       */ 
/*                                                                          */
/*                                                                          */
/* INPUT:                                                                   */
/* OUTPUT:                                                                  */
/*                                                                          */
/* RETURNS:	Returns the management Black & White table first row.           */
/*          Formats it to html format                                       */
/*                                                                          */
/****************************************************************************/
char* wlan_htmlGetMgmt_BW_TableFirstRow(void)
{
	unsigned long len = 0;
	tableBW_CurrentIndex = 0;

	tempbuf[0] = 0;
	
	/* first row */
    len += sprintf(tempbuf + len,"<tr>\n");
    len += sprintf(tempbuf + len,"<TD align=middle width=\"20%%\" bgColor=#ccccff height=11>Delete</TD>\n");
    len += sprintf(tempbuf + len,"<TD align=middle width=\"80%%\" bgColor=#ccccff height=11>Mac Address</TD>\n");
    len += sprintf(tempbuf + len,"</tr>\n");

	return tempbuf;
	return NULL;
}



/****************************************************************************/
/*                    wlan_htmlGetMgmt_BW_TableDynamicRow(void)				    */
/****************************************************************************/
/* DESCRIPTION: Returns a B&W management table row						    */ 
/*																		    */
/*																		    */
/* INPUT:       															*/
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:	Returns a management B&W table row.                             */
/*          Formats it to html format		                                */
/*																			*/
/****************************************************************************/
char* wlan_htmlGetMgmt_BW_TableDynamicRow(void)
{
#if 0
	char address[MAC_ADDR_LEN], zeroBuf[MAC_ADDR_LEN];	
	char tmpTable[MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN], *ptr;
    int length = MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN;
    int index;
	UINT16 capabilities;
	UINT8 activeRate;
	unsigned long len = 0;
	UINT32 state;
    STATUS status;

	tempbuf[0] = 0;
	
	if (tableBW_CurrentIndex == MGMT_ACCESS_LIST_SIZE)
		return tempbuf;

    /* look for an entry in the B&W table */
    memset(zeroBuf,0,MAC_ADDR_LEN);

	status = mgmtAccessGetList(tmpTable, &length);
    if (status != OK) 
        return tempbuf;

    for (index=1, ptr=tmpTable; 
         index<=MGMT_ACCESS_LIST_SIZE; 
         index++, ptr+=MAC_ADDR_LEN) 
    {
        if (memcmp(ptr,zeroBuf,MAC_ADDR_LEN) == 0)
            continue;
        if (index > tableBW_CurrentIndex)
            break;
    }
    if (index == MGMT_ACCESS_LIST_SIZE+1) /* end of list */
        return tempbuf;
    tableBW_CurrentIndex = index;
    /* ptr is pointing to the current MacAddr */
    memcpy(address,ptr,MAC_ADDR_LEN);

/************************

    <TR>
    <TD align=middle width="20%" height=11>
    <P align=center><INPUT style="FONT-SIZE: 8pt; POSITION: relative" type=button value="    " name=DelSTA0></P></TD>
    <TD align=middle width="80%" height=11>00-08-21-31-18-40</TD>
    </TR>

*************************/

    /* now pour it to the buffer */
    len += sprintf(tempbuf + len,"<tr>\n");
    len += sprintf(tempbuf + len,"<td align=middle width=\"20%%\" height=\"11\">\n");
    len += sprintf(tempbuf + len,"<P align=center><INPUT style=\"FONT-SIZE: 8pt; POSITION: relative\" onclick='this.value=\"%d\"' type=submit value=\"    \" name=DelSTA></P></TD>\n",tableBW_CurrentIndex);
    len += sprintf(tempbuf + len,"<td align=middle width=\"80%%\" height=11>%02X-%02X-%02X-%02X-%02X-%02X</td>\n",
                   address[0],address[1],address[2],address[3],address[4],address[5]);
    len += sprintf(tempbuf + len,"</tr>\n");

    /*len += sprintf(tempbuf + len,"<td width=\"15%%\" height=\"11\" align=\"center\"><INPUT style=\"FONT-SIZE: 8pt; POSITION: relative\" onclick='this.value=\"%s\"' type=submit value=\"    \" name=BanSTA></TD>\n",formatAddr);*/
	return tempbuf;
#endif

	char address[ETH_ALEN], zeroBuf[ETH_ALEN];	
	__u8 (*ptr)[ETH_ALEN] = NULL;
	int i;
	unsigned long len = 0;
	tiap_param_access_list_t access_list;


	tempbuf[0] = 0;

	if (tableBW_CurrentIndex == TIAP_PARAM_ACCESS_LIST_SIZE)
		return tempbuf;

	/* look for an entry in the B&W table */
	memset(zeroBuf, 0, ETH_ALEN);

	access_list.cmd = TIAP_PARAM_ACCESS_LST;
	if (tiap_wcfg_get_access_list(wd, &access_list, !FROM_STORAGE)) {
		PRINT_CONFIG_ERR_MSG("Failed to get sta table.");
		return NULL;
	}


	for (i=1, ptr=(__u8 (*)[ETH_ALEN])access_list.list; i<=TIAP_PARAM_ACCESS_LIST_SIZE; i++) 
	{
		if (memcmp(ptr[i-1],zeroBuf,ETH_ALEN) == 0)
			continue;
		if (i > tableBW_CurrentIndex)
			break;
	}
	if (i == TIAP_PARAM_ACCESS_LIST_SIZE+1) /* end of list */
		return tempbuf;
	tableBW_CurrentIndex = i;
	/* ptr is pointing to the current MacAddr */
	memcpy(address, ptr[i-1], ETH_ALEN);

	/* now pour it to the buffer */
	len += sprintf(tempbuf + len,"<tr>\n");
	len += sprintf(tempbuf + len,"<td align=middle width=\"20%%\" height=\"11\">\n");
	len += sprintf(tempbuf + len,"<P align=center><INPUT style=\"FONT-SIZE: 8pt; POSITION: relative\" onclick='this.value=\"%d\"' type=submit value=\"    \" name=DelSTA></P></TD>\n",tableBW_CurrentIndex);
	len += sprintf(tempbuf + len,"<td align=middle width=\"80%%\" height=11>%02hhX-%02hhX-%02hhX-%02hhX-%02hhX-%02hhX</td>\n",
			address[0],address[1],address[2],address[3],address[4],address[5]);
	len += sprintf(tempbuf + len,"</tr>\n");

	return tempbuf;

}


/****************************************************************************/
/*                    wlan_htmlSavePageIpSettings(void)							*/
/****************************************************************************/
/* DESCRIPTION: Update the driver database with the							*/
/*				parameters values from the IP Settings page.    			*/ 
/*																		    */
/*																		    */
/* INPUT:       Setup page parameters				                        */
/* OUTPUT:      															*/
/*																			*/
/* RETURNS:																	*/
/*																			*/
/****************************************************************************/


char* wlan_htmlSavePageIpSettings(char* value, 
                                  char *ip1, char *ip2, char *ip3, char *ip4, 
                                  char *mask1, char *mask2, char *mask3, char *mask4,
                                  char *defGateway1, char *defGateway2, char *defGateway3, char *defGateway4)
{
#if 0
    unsigned long us, len, ipAddress, subnetMask;
    unsigned long ipAddr1, ipAddr2, ipAddr3, ipAddr4, 
                  subnetMask1, subnetMask2, subnetMask3, subnetMask4,
                  gateway1, gateway2, gateway3, gateway4;

	if (!strcmp(value,"Saving"))
	{
        /* ip address */
        sscanf(ip1,"%d",&ipAddr1);
        sscanf(ip2,"%d",&ipAddr2);
        sscanf(ip3,"%d",&ipAddr3);
        sscanf(ip4,"%d",&ipAddr4);
        ipAddress = (ipAddr1 << 24) | (ipAddr2 << 16) | (ipAddr3 << 8) | ipAddr4;
        cm_hwSetLanIP(ipAddress);
        
        /* subnet mask */
        sscanf(mask1,"%d",&subnetMask1);
        sscanf(mask2,"%d",&subnetMask2);
        sscanf(mask3,"%d",&subnetMask3);
        sscanf(mask4,"%d",&subnetMask4);
        subnetMask = (subnetMask1 << 24) | (subnetMask2 << 16) | (subnetMask3 << 8) | subnetMask4;
        cm_hwSetLanIpMask(subnetMask);

        /* default gateway */
		sscanf(defGateway1,"%d",&gateway1);
		sscanf(defGateway2,"%d",&gateway2);
		sscanf(defGateway3,"%d",&gateway3);
		sscanf(defGateway4,"%d",&gateway4);
		wdrv_dbSetDefaultGateway(gateway1, gateway2, gateway3, gateway4);

        consoleReboot();
    }
    
    return "";
#endif

    __u32 ipAddress = 0, subnetMask = 0;
    __u32 ipAddr1, ipAddr2, ipAddr3, ipAddr4, 
    subnetMask1, subnetMask2, subnetMask3, subnetMask4,
    gateway1, gateway2, gateway3, gateway4;
    __u32 gateway;
    __u8 *p = (__u8 *)&gateway;
    int ip_err = 0, subnet_err = 0, gtw_err = 0;

    if (!strcmp(value,"Saving"))
    {
	    /* ip address */
	    if ( sscanf(ip1,"%u",&ipAddr1) <= 0) ip_err = 1;
	    if ( sscanf(ip2,"%u",&ipAddr2) <= 0) ip_err = 1;
	    if ( sscanf(ip3,"%u",&ipAddr3) <= 0) ip_err = 1;
	    if ( sscanf(ip4,"%u",&ipAddr4) <= 0) ip_err = 1;

	    if (ip_err){
		    fprintf(stderr, "\ntiweb: Wrong IP format\n");
	    }
	    else {

		    ipAddress = (ipAddr1 << 24) | (ipAddr2 << 16) | (ipAddr3 << 8) | ipAddr4;
		    ipAddress = htonl(ipAddress);
		    ip_err = tiap_wcfg_set_ipaddr_or_netmask(wd->br_name, SIOCSIFADDR, ipAddress);
		    if(ip_err) 
			    fprintf(stderr, "\ntiweb: Failed to set IP address for %s.\n", wd->br_name);
	    }

	    /* subnet mask */
	    if ( sscanf(mask1,"%u",&subnetMask1) <=0) subnet_err = 1;
	    if ( sscanf(mask2,"%u",&subnetMask2) <=0) subnet_err = 1;
	    if ( sscanf(mask3,"%u",&subnetMask3) <=0) subnet_err = 1;
	    if ( sscanf(mask4,"%u",&subnetMask4) <=0) subnet_err = 1;

	    if (subnet_err){
		    fprintf(stderr, "\ntiweb: Wrong Subnet mask format\n");
	    }
	    else {
		    subnetMask = (subnetMask1 << 24) | (subnetMask2 << 16) | (subnetMask3 << 8) | subnetMask4;
		    subnetMask = htonl(subnetMask);
		    subnet_err = tiap_wcfg_set_ipaddr_or_netmask(wd->br_name, SIOCSIFNETMASK, subnetMask);
		    if(subnet_err) 
			    fprintf(stderr, "\ntiweb: Failed to set netmask for %s.\n", wd->br_name);
			    
	    }
	    /* default gateway */
	    if ( sscanf(defGateway1,"%u",&gateway1) <=0) gtw_err = 1;
	    if ( sscanf(defGateway2,"%u",&gateway2) <=0) gtw_err = 1;
	    if ( sscanf(defGateway3,"%u",&gateway3) <=0) gtw_err = 1;
	    if ( sscanf(defGateway4,"%u",&gateway4) <=0) gtw_err = 1;

	    if (gtw_err)
		    fprintf(stderr, "\ntiweb: Wrong Default Gateway format\n");
	    
	    else {
		    /* delete previous default route */
		    gtw_err = tiap_wcfg_get_default_gateway(&gateway);
		    if( gtw_err ) {
			    fprintf(stderr, "tiweb: Warning, failed to get default route, "
					    "probably IP address of the default route device "
					    "was changed recently.\n");
			    fflush(stderr);
		    }
		    else if( tiap_wcfg_set_route(wd->br_name, 0, 0, gateway, SIOCDELRT) ) {
			    fprintf(stderr, "tiweb: Warning, failed to delete default route.\n");
			    fflush(stderr);
		    }
		    p[0] = (__u8)gateway1;
		    p[1] = (__u8)gateway2;
		    p[2] = (__u8)gateway3;
		    p[3] = (__u8)gateway4;
		    //gateway = htonl(gateway);

		    gtw_err = tiap_wcfg_set_route(wd->br_name, 0, 0, gateway, SIOCADDRT);
		    if( gtw_err )
			    fprintf(stderr, "\ntiweb: Failed to set default route.\n");
	    }

	    if (ip_err || gtw_err || subnet_err)
	    {
			fprintf(stderr, "\ntiweb: Failed to configure one or more of IP parameters.\n"
					"	  Warning: previous settings will be restored after reboot\n");
	    }
	    else
	    {
		    tiap_wcfg_set_storage_ipaddr(wd, SIOCSIFADDR, ipAddress); /*store it in storage*/
		    tiap_wcfg_set_storage_ipaddr(wd, SIOCSIFNETMASK, subnetMask); /*store it in storage*/
		    tiap_wcfg_set_storage_ipaddr(wd, SIOCADDRT, gateway); /*store it in storage*/
	    }
    }

    return "";

}



/********************************************************************************/
/*                      wlan_htmlGetApEnable(void)                                  */
/********************************************************************************/
/* DESCRIPTION: Check/uncheck a value of the AP Enable buttons                  */ 
/*                                                                              */
/*                                                                              */
/* INPUT:       Radio button value                                              */
/* OUTPUT:      															    */
/*                                                                              */
/* RETURNS:		"checked" if the value is identical to the AP's service state   */
/*				"" if the value is different than the AP's preamble mode        */
/*                                                                              */
/********************************************************************************/
char* wlan_htmlGetApEnable(char *value)
{
#if 0
	int preambleMode;
	int mode;
	UINT8 temp;
    BOOL state;

	sscanf(value,"%d",&mode);

	wdrv_dbGetWlanServiceState(&state);

    switch (state) 
    {
        case TRUE:
            if (mode == 1) 
                return "checked";
            break;
        case FALSE:
            if (mode == 2) 
                return "checked";
            break;
    }
    return " ";
#endif
	__u8 state;
	int mode;

	sscanf(value,"%d",&mode);

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_OPER_STATE, &state, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get AP's operating state.");
		return NULL;
	}

	switch (state) 
	{
		case TRUE:
			if (mode == 1) 
				return "checked";
			break;
		case FALSE:
			if (mode == 2) 
				return "checked";
			break;
	}
	return " ";
}


/****************************************************************************/
/*                 wlan_htmlGetEnableDisableButtonName(void)				    */
/****************************************************************************/
/* DESCRIPTION: Returns the name for the Enable/Disable button              */ 
/*              If the AP is disabled then returns Enable and vice versa.   */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Fw version													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetEnableDisableButtonName(char *value)
{
#if 0
    BOOL state;

	wdrv_dbGetWlanServiceState(&state);

    if (state == TRUE) 
        return "Disable AP";
    else
        return "Enable AP";
#endif

	__u8 state;

	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_OPER_STATE, &state, !FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get AP's operating state.");
		return NULL;
	}

	return (state == TRUE ? "Disabling AP" : "Enabling AP");
}


/****************************************************************************/
/*                      wlan_htmlUpdateEnableDisableAP(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns name for the button when pressing on it             */ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Fw version													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlUpdateEnableDisableAP(char *value)
{
#if 0
	if (!strcmp(value,"Disabling AP"))
	{
        wdrv_WlanServiceDisable(void);          /* Enable */
    }
	else if (!strcmp(value,"Enabling AP"))
	{
        wdrv_WlanServiceEnable(void);           /* Disable */
    }

    return "";
#endif

     __u8 state;
    
    if( !value ) {
	    fprintf(stderr, "%s: invalid value: %s\n", __func__, "NULL");
	    return NULL;
    }

    if (!strcmp(value,"Disabling AP"))
	    state = 0;
    else if (!strcmp(value,"Enabling AP"))
	    state = 1;
    else return "";

    if( tiap_wcfg_set_basic(wd, TIAP_PARAM_OPER_STATE, &state, UPDATE_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to set AP's operating state.");
	    fprintf(stderr, "%s: invalid value: %s\n", __func__, "NULL");
	    return NULL;
    }

    return "";
}


/****************************************************************************/
/*                  wlan_htmlGetUpdateButtonValueApEnable(void)					*/
/****************************************************************************/
/* DESCRIPTION: Returns the FW version. 									*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     Fw version													*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetUpdateButtonValueApEnable(void)
{
#if 0
    BOOL state;

	wdrv_dbGetWlanServiceState(&state);

    if (state == TRUE) 
        return "Disabling AP";
    else
        return "Enabling AP";
#endif

    __u8 state;

    if( tiap_wcfg_get_basic(wd, TIAP_PARAM_OPER_STATE, &state, !FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get AP's operating state.");
	    return NULL;
    }

    return (state == TRUE ? "Disabling AP" : "Enabling AP");
}


char* wlan_htmlUpdateStationPageMgmt(char *stationMAC)
{
#if 0
    int uintval, pDigitPos, pPos, hyphenCount, length;
    UINT8 enable, type, macval[6];
    char tmp_buf[MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN];

    
    if (stationMAC[0] != 'N')
    {
        whal_nvram_getAccessListEnable(&enable);
        if (enable == 0) 
        {
            return " "; /* BW table is disbled */
        }
        memset(macval,0,6);
        hyphenCount=0;

        uintval = 0;
        pDigitPos = 0;

        length = strlen(stationMAC);
        if (length > 17) 
            return FALSE;

        for (pPos = 0; pPos<length;pPos++) 
        {
            if (stationMAC[pPos] == '-') 
            {
                if (hyphenCount > 5) 
                    return " "; /* its the sixth '-' */
                if ((uintval == 0) && (pDigitPos == 0))
                    return " "; /* string begins with hyphen */
                macval[hyphenCount] = uintval;
                hyphenCount++;
                uintval = 0;
                pDigitPos = 0;
            } 
            else if (isxdigit(stationMAC[pPos])) 
            {
                pDigitPos++;
                if (pDigitPos > 2) 
                    return FALSE;
                uintval = uintval*16;
                if (isdigit(stationMAC[pPos]))
                    uintval += (stationMAC[pPos]-'0');
                else
                    uintval += hexChar2int(stationMAC[pPos]);
            } 
            else 
            {
                return " ";         /* bad value */
            }
        }
        
        if (hyphenCount < 5) 
            return " ";             /* have to have exactly 5 '-' */

        /* its the last octet */
        if ((uintval == 0) && (pDigitPos == 0))
           return " ";
        macval[hyphenCount] = uintval;

        whal_nvram_getAccessListType(&type);

        if (type == 1) /* table is black*/
            mgmtAccessSetMAC(macval);
        else /* remove the mac */
            mgmtAccessRemMACbyAddr(macval);
        
        /* Update table in NVRAM */
        length = MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN;
        if (mgmtAccessGetList(tmp_buf, &length) != OK) 
            return " ";
        else
    	    whal_nvram_setAccessList(tmp_buf, IMMEDIATELY);
    }
    return " ";
#endif












    int uintval, pDigitPos, pPos, hyphenCount, length;
    __u8 macval[6];
    tiap_param_access_list_t access_list;

    if (stationMAC[0] != 'N')
    {
	access_list.cmd = TIAP_PARAM_ACCESS_LST;
	if (tiap_wcfg_get_access_list(wd, &access_list, FROM_STORAGE)) {
		PRINT_CONFIG_ERR_MSG("Reading configuration failed");
		return NULL;
	}

        if (access_list.enabled == 0) 
        {
            return " "; /* BW table is disbled */
        }
        memset(macval,0,6);
        hyphenCount=0;

        uintval = 0;
        pDigitPos = 0;

        length = strlen(stationMAC);
        if (length > 17) 
            return FALSE;

        for (pPos = 0; pPos<length;pPos++) 
        {
            if (stationMAC[pPos] == '-') 
            {
                if (hyphenCount > 5) 
                    return " "; /* its the sixth '-' */
                if ((uintval == 0) && (pDigitPos == 0))
                    return " "; /* string begins with hyphen */
                macval[hyphenCount] = uintval;
                hyphenCount++;
                uintval = 0;
                pDigitPos = 0;
            } 
            else if (isxdigit(stationMAC[pPos])) 
            {
                pDigitPos++;
                if (pDigitPos > 2) 
                    return FALSE;
                uintval = uintval*16;
                if (isdigit(stationMAC[pPos]))
                    uintval += (stationMAC[pPos]-'0');
                else
                    uintval += hexChar2int(stationMAC[pPos]);
            } 
            else 
            {
                return " ";         /* bad value */
            }
        }
        
        if (hyphenCount < 5) 
            return " ";             /* have to have exactly 5 '-' */

        /* its the last octet */
        if ((uintval == 0) && (pDigitPos == 0))
           return " ";
        macval[hyphenCount] = uintval;


        if (access_list.type == 1) { /* table is black*/
		access_list.cmd = TIAP_PARAM_ACCESS_ADD;
		memcpy(access_list.list[0], macval, ETH_ALEN);
	}
        else { /* remove the mac */
	    access_list.cmd = TIAP_PARAM_ACCESS_DEL_BY_ADDR;
	    memcpy(access_list.list[0], macval, ETH_ALEN);
	}

        /* Update table in NVRAM */
	if (tiap_wcfg_set_access_list(wd, &access_list)) {
		PRINT_CONFIG_ERR_MSG("Failed to set access list.");
		return NULL;
	}
    }
    return " ";

}


char* wlan_htmlUpdate_BW_DeletePageMgmt(char *id)
{
#if 0
    UINT32 staID;
    STATUS status;
    int length = MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN;
    char tmp_buf[MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN];

    sscanf(id,"%d",&staID);
    if ((staID > 0) && (staID < MGMT_ACCESS_LIST_SIZE))
    {
        mgmtAccessRemMAC(staID);

        /* Update table in NVRAM */
    	status = mgmtAccessGetList(tmp_buf, &length);
        if (status != OK) 
        {
            printf("remAccessMAC: Fatal Error from mgmtAccessGetList (file %s)\n",__FILE__);
            return " ";
        }
    	whal_nvram_setAccessList(tmp_buf, IMMEDIATELY);
    }
    return " ";
#endif

    UINT32 staID;
    tiap_param_access_list_t access_lst;

    sscanf(id,"%d",&staID);
    if ((staID > 0) && (staID < TIAP_PARAM_ACCESS_LIST_SIZE))
    {
	    access_lst.cmd = TIAP_PARAM_ACCESS_LST;
	    if (tiap_wcfg_get_access_list(wd, &access_lst, !FROM_STORAGE)) {
		    PRINT_CONFIG_ERR_MSG("Failed to get access list.");
		    return NULL;
	    }
	    
	    access_lst.cmd = TIAP_PARAM_ACCESS_DEL_BY_INDEX;
	    access_lst.index = staID;

	    /* Update table in NVRAM */
	    if (tiap_wcfg_set_access_list(wd, &access_lst)) {
		    PRINT_CONFIG_ERR_MSG("Failed to set access list.");
		    return NULL;
	    }
    }
    return " ";
}


char* wlan_htmlUpdate_BW_AddMacPageMgmt(char *value, char *mac1, char *mac2, 
                                        char *mac3, char *mac4, char *mac5, char *mac6)
{
#if 0
    UINT32 staID;
    UINT8 macAddr[6];
    char tmpTable[MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN];
    int length = MGMT_ACCESS_LIST_SIZE*MAC_ADDR_LEN;

    if (strcmp(value,"Add") == 0) 
    {
        /* check that each MAC component has a length of 2 */
        if ((strlen(mac1) != 2) ||
            (strlen(mac2) != 2) ||
            (strlen(mac3) != 2) ||
            (strlen(mac4) != 2) ||
            (strlen(mac5) != 2) ||
            (strlen(mac6) != 2))
        {
            return " ";
        }
        /* check that each component character is a hex digit */
        if ((isxdigit(mac1[0]) == FALSE) ||
            (isxdigit(mac1[1]) == FALSE) ||
            (isxdigit(mac2[0]) == FALSE) ||
            (isxdigit(mac2[1]) == FALSE) ||
            (isxdigit(mac3[0]) == FALSE) ||
            (isxdigit(mac3[1]) == FALSE) ||
            (isxdigit(mac4[0]) == FALSE) ||
            (isxdigit(mac4[1]) == FALSE) ||
            (isxdigit(mac5[0]) == FALSE) ||
            (isxdigit(mac5[1]) == FALSE) ||
            (isxdigit(mac6[0]) == FALSE) ||
            (isxdigit(mac6[1]) == FALSE))
        {
            return " ";
        }

        macAddr[0] = htol(mac1);
        macAddr[1] = htol(mac2);
        macAddr[2] = htol(mac3);
        macAddr[3] = htol(mac4);
        macAddr[4] = htol(mac5);
        macAddr[5] = htol(mac6);

        if (mgmtAccessSetMAC(macAddr) != OK)
        {
            return " ";
        }
        /* Update table in NVRAM */
        if (mgmtAccessGetList(tmpTable, &length) == OK)
        {
            whal_nvram_setAccessList(tmpTable, IMMEDIATELY);
        }
    }

    return " ";
#endif

    __u8 macAddr[ETH_ALEN];
    tiap_param_access_list_t access_lst;

    if (strcmp(value,"Add") == 0) 
    {
	    /* check that each MAC component has a length of 2 */
	    if ((strlen(mac1) != 2) ||
			    (strlen(mac2) != 2) ||
			    (strlen(mac3) != 2) ||
			    (strlen(mac4) != 2) ||
			    (strlen(mac5) != 2) ||
			    (strlen(mac6) != 2))
	    {
		    return " ";
	    }
	    /* check that each component character is a hex digit */
	    if ((isxdigit(mac1[0]) == FALSE) ||
			    (isxdigit(mac1[1]) == FALSE) ||
			    (isxdigit(mac2[0]) == FALSE) ||
			    (isxdigit(mac2[1]) == FALSE) ||
			    (isxdigit(mac3[0]) == FALSE) ||
			    (isxdigit(mac3[1]) == FALSE) ||
			    (isxdigit(mac4[0]) == FALSE) ||
			    (isxdigit(mac4[1]) == FALSE) ||
			    (isxdigit(mac5[0]) == FALSE) ||
			    (isxdigit(mac5[1]) == FALSE) ||
			    (isxdigit(mac6[0]) == FALSE) ||
			    (isxdigit(mac6[1]) == FALSE))
	    {
		    return " ";
	    }

	    macAddr[0] = htol(mac1);
	    macAddr[1] = htol(mac2);
	    macAddr[2] = htol(mac3);
	    macAddr[3] = htol(mac4);
	    macAddr[4] = htol(mac5);
	    macAddr[5] = htol(mac6);


	    access_lst.cmd = TIAP_PARAM_ACCESS_LST;

	    /*get access list type*/
	    if (tiap_wcfg_get_access_list(wd, &access_lst, FROM_STORAGE)) {
		    PRINT_CONFIG_ERR_MSG("Failed to get access list.");
		    return NULL;
	    }

	    access_lst.cmd = TIAP_PARAM_ACCESS_ADD;
	    memcpy(access_lst.list[0], macAddr, ETH_ALEN);

	    /* Update table in NVRAM and driver */
	    if (tiap_wcfg_set_access_list(wd, &access_lst)) {
		    PRINT_CONFIG_ERR_MSG("Failed to set access list.");
		    return NULL;
	    }
    }

    return " ";
}


/********************************************************************************/
/*                    wlan_htmlGetBWTableStatusPageMgmt(void)                       */
/********************************************************************************/
/* DESCRIPTION: Select a value for the BW table radio buttons                   */ 
/*                                                                              */
/*                                                                              */
/* INPUT:       Radio button value                                              */
/* OUTPUT:      															    */
/*                                                                              */
/* RETURNS:		"checked" if the value is identical to the BW's table state     */
/*				"" if the value is different than the BW table states           */
/*                                                                              */
/********************************************************************************/
char* wlan_htmlGetBWTableStatusPageMgmt(char *value)
{
#if 0
	int preambleMode;
	int mode;
	UINT8 temp, enable, type;
    BOOL state;

	sscanf(value,"%d",&mode);

    whal_nvram_getAccessListEnable(&enable);
    if (enable == 1)
        whal_nvram_getAccessListType(&type);

    switch (mode) 
    {
        case 1:     /* White */
            if ((enable == 1) && (type == 0))
                return "checked";
            break;
        case 2:     /* Black */
            if ((enable == 1) && (type == 1))
                return "checked";
            break;
        case 3:     /* Disable */
            if (enable == 0) 
                return "checked";
            break;
    }
    return " ";
#endif

    int mode;

    tiap_param_access_list_t access_list;

    access_list.cmd = TIAP_PARAM_ACCESS_LST;
    if (tiap_wcfg_get_access_list(wd, &access_list, !FROM_STORAGE)) {
	    PRINT_CONFIG_ERR_MSG("Failed to get access list.");
	    return NULL;
    }

    sscanf(value,"%d",&mode);

    switch (mode) 
    {
	    case 1:     /* White */
		    if ((access_list.enabled == 1) && (access_list.type == 0))
			    return "checked";
		    break;
	    case 2:     /* Black */
		    if ((access_list.enabled == 1) && (access_list.type == 1))
			    return "checked";
		    break;
	    case 3:     /* Disable */
		    if (access_list.enabled == 0) 
			    return "checked";
		    break;
    }
    return " ";

}


/********************************************************************************/
/*                    wlan_htmlGetBWTableStatusPageMgmt(void)                       */
/********************************************************************************/
/* DESCRIPTION: Select a value for the BW table radio buttons                   */ 
/*                                                                              */
/*                                                                              */
/* INPUT:       Radio button value                                              */
/* OUTPUT:      															    */
/*                                                                              */
/* RETURNS:		"checked" if the value is identical to the BW's table state     */
/*				"" if the value is different than the BW table states           */
/*                                                                              */
/********************************************************************************/
char* wlan_htmlSavePageMgmt(char *value, char *BWstatus, char *MultipleSSID, char *stam) 
{
#if 0
	int bwTableStatus, multipleSsidEnable;
	int mode;
	UINT8 temp, enable, type;
    BOOL state;

	if (strcmp(value,"Saving")==0) 
    {
        sscanf(BWstatus,"%d",&bwTableStatus);
        sscanf(MultipleSSID,"%d",&multipleSsidEnable);

        switch (bwTableStatus) 
        {
            case 1:     /* White */
                whal_nvram_setAccessListEnable(1, IMMEDIATELY);     /* Enable */
                whal_nvram_setAccessListType(0, IMMEDIATELY);       /* White */
                break;
            case 2:     /* Black */
                whal_nvram_setAccessListEnable(1, IMMEDIATELY);     /* Enable */
                whal_nvram_setAccessListType(1, IMMEDIATELY);       /* Black */
                break;
            case 3:     /* Disable */
                whal_nvram_setAccessListEnable(0, IMMEDIATELY);     /* Disable */
                break;
        }
        switch (multipleSsidEnable) 
        {
            case 1:     /* Enable */
                wdrv_dbSetMultipleSsidStatus(1);
                break;
            case 2:     /* Disable */
                wdrv_dbSetMultipleSsidStatus(0);
                break;
        }
        consoleReboot();                                /* Reboot the AP */
    }
    return " ";
#endif

    int bwTableStatus, multi_ssid_enabled;
    tiap_param_access_list_t access_lst;
    tiap_param_multi_ssid_t multi_ssid;

    if (strcmp(value,"Saving")==0) 
    {	
	    sscanf(BWstatus,"%d",&bwTableStatus);

	    access_lst.cmd = TIAP_PARAM_ACCESS_LST;
	    if (tiap_wcfg_get_access_list(wd, &access_lst, FROM_STORAGE)) {
		    PRINT_CONFIG_ERR_MSG("Reading configuration failed");
		    return NULL;
	    }

	    switch (bwTableStatus) 
	    {
		    case 1:	/* White */
			    access_lst.enabled = 1;
			    access_lst.type = 0;

			    break;
		    case 2:     /* Black */
			    access_lst.enabled = 1;
			    access_lst.type = 1;
			    break;
		    case 3:     /* Disable */
			    access_lst.enabled = 0;
			    break;
		    default:
			    return NULL;
	    } 

		access_lst.cmd = TIAP_PARAM_ACCESS_LST;
	    if (tiap_wcfg_set_access_list(wd, &access_lst)) {
		    PRINT_CONFIG_ERR_MSG("Configuration failed");
		    return NULL;
	    }


	    if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid status.");
		    return NULL;
	    }

	    if( strcmp(MultipleSSID, "NO_PARAM") )
	    {
		    sscanf(MultipleSSID,"%d",&multi_ssid_enabled);

		    switch (multi_ssid_enabled) 
	    {
		    case 1:     /* Enable */
			    multi_ssid.enabled = 1;
			    break;
		    case 2:     /* Disable */
			    multi_ssid.enabled = 0;
			    break;
		    default:
			    return NULL;
	    }

	    multi_ssid.cmd = TIAP_PARAM_MULTI_SSID_SET;
		    if( tiap_wcfg_set_multi_ssid(wd, &multi_ssid, UPDATE_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to set multiple ssid status.");
		    return NULL;
	    }
	    }
	    
	    restart();                                /* Reboot the AP */
    }
    return " ";
}


/********************************************************************************/
/*                  wlan_htmlGetMultipleSsidStatusPageMgmt(void)                    */
/********************************************************************************/
/* DESCRIPTION: Select a value for the Mutiple SSID table radio buttons         */ 
/*                                                                              */
/*                                                                              */
/* INPUT:       Radio button value                                              */
/* OUTPUT:                                                                      */
/*                                                                              */
/* RETURNS:     "checked" if the value is identical to the SSID table state     */
/*              "" if the value is different than the SSID table states         */
/*                                                                              */
/********************************************************************************/
char* wlan_htmlGetMultipleSsidStatusPageMgmt(char *value)
{
#if 0
    int mode;
    UINT16 temp, enable, type;
    BOOL state;

    sscanf(value,"%d",&mode);

    if (wdrv_dbIsPrivacyEnabled())
    {
        return " disabled";
    }

    wdrv_dbGetMultipleSsidStatus(&enable);

    switch (mode) 
    {
        case 1:     /* enable */
            if (enable == 1)
                return "checked";
            break;
        case 2:     /* disable */
            if (enable == 0)
                return "checked";
            break;
    }
    return " ";
#endif

    int mode;
    tiap_param_multi_ssid_t multi_ssid;

    sscanf(value,"%d",&mode);

    if( is_privacy_enabled() )
    	    return " disabled";

    if (tiap_wcfg_get_multi_ssid(wd, &multi_ssid, !FROM_STORAGE)) {
	    PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid");
	    return NULL;
    }

    switch (mode) 
    {
	    case 1:     /* enable */
		    if (multi_ssid.enabled == 1)
			    return "checked";
		    break;
	    case 2:     /* disable */
		    if (multi_ssid.enabled == 0)
			    return "checked";
		    break;
    }
    return " ";

}


char* wlan_htmlUpdate_SSID_AddMultipleSSIDPageMgmt(char *value, char *pStrSSID)
{
#if 0
    UINT8 apSSID[WHAL_SSID_STRING_LEN+1];
    UINT8 SSID2add[WHAL_SSID_STRING_LEN+1];
    UINT8 bLen = WHAL_SSID_STRING_LEN;
    UINT8 apSsidLen;
    int   index, loop;
    char tempTable[NUM_MULTIPLE_SSID][WHAL_SSID_STRING_LEN];
    UINT32 len = NUM_MULTIPLE_SSID*WHAL_SSID_STRING_LEN;

    if (strcmp(value,"Add") == 0) 
    {
        if (strlen(pStrSSID) > WHAL_SSID_STRING_LEN) 
        {
            return " ";
        }
        /* Look for an empty place in Multiple SSID table */
        whal_nvram_getMultipleSsidTable(tempTable[0],&len);
        index = 0;
        while (index < NUM_MULTIPLE_SSID) 
        {
            if (tempTable[index][0] == '\0') 
                break;
            index++;
        }
        if (index == NUM_MULTIPLE_SSID) 
        {
            return " ";     /* Table is full */
        }

        bLen = strlen(pStrSSID);
        if(bLen > 0)
        {
            /* Check that the new value differs from AP SSID */
            whal_nvram_getSsid(apSSID, &apSsidLen);
            apSSID[apSsidLen] = 0;
            if (strcmp(pStrSSID,apSSID)==0)
            {
                /* The new SSID equals the AP SSID. Setting aborted! */
                return " ";
            }
            memset(SSID2add,0,WHAL_SSID_STRING_LEN+1);
            strcpy(SSID2add,pStrSSID);
            /* Check that the new SSID not already exists */
            for (loop=0; loop<index;loop++)
            {
                if (memcmp(SSID2add,tempTable[loop],WHAL_SSID_STRING_LEN) == 0)
                {
                    return " ";     /* SSID already exists */
                }
            }
            wdrv_dbSetElementToMultipleSsidTable(pStrSSID,bLen);
        }

    }
    return " ";
#endif

    UINT8 SSID2add[IW_ESSID_MAX_SIZE+1];
    UINT8 bLen = IW_ESSID_MAX_SIZE;
    int   index, loop;

    tiap_param_multi_ssid_t multi_ssid;
    char ssid[IW_ESSID_MAX_SIZE+1];

    if (strcmp(value,"Add") == 0) 
    {
        if (strlen(pStrSSID) > IW_ESSID_MAX_SIZE) 
        {
            return " ";
        }
        /* Look for an empty place in Multiple SSID table */
	if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid status.");
		return NULL;
	}
        index = 0;
        while (index < TIAP_PARAM_NUM_MULTIPLE_SSID) 
        {
            if (multi_ssid.tbl.ssid[index][0] == '\0') 
                break;
            index++;
        }
        if (index == TIAP_PARAM_NUM_MULTIPLE_SSID) 
            return " ";     /* Table is full */

        bLen = strlen(pStrSSID);
        if(bLen > 0)
        {
            /* Check that the new value differs from AP SSID */
	    if( tiap_wcfg_giwessid(wd, ssid, FROM_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to get ssid.");
		    return NULL;
	    }
            if (strcmp(pStrSSID, ssid)==0)
            {
                /* The new SSID equals the AP SSID. Setting aborted! */
                return " ";
            }
            memset(SSID2add,0,IW_ESSID_MAX_SIZE+1);
            strcpy(SSID2add,pStrSSID);
            /* Check that the new SSID not already exists */
            for (loop=0; loop<index;loop++)
                if (memcmp(SSID2add,multi_ssid.tbl.ssid[loop],IW_ESSID_MAX_SIZE) == 0)
                    return " ";     /* SSID already exists */

	    multi_ssid.cmd = TIAP_PARAM_MULTI_SSID_ADD;
	    strncpy(multi_ssid.tbl.ssid[0], pStrSSID, IW_ESSID_MAX_SIZE);
	    if( tiap_wcfg_set_multi_ssid(wd, &multi_ssid, UPDATE_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to add ssid to multiple ssid table.");
		    return NULL;
	    }
        }

    }
    return " ";
}


char* wlan_htmlUpdate_SSID_DeleteSSIDPageMgmt(char *delSSID)
{
#if 0
    char tempTable[NUM_MULTIPLE_SSID][WHAL_SSID_STRING_LEN];
    UINT32 len = NUM_MULTIPLE_SSID*WHAL_SSID_STRING_LEN;
    int index, chosen;

    chosen = atoi(delSSID);
    if (chosen > 0)
    {
        whal_nvram_getMultipleSsidTable(tempTable[0],&len);
        index = 0;
        while (index < NUM_MULTIPLE_SSID) 
        {
            if (tempTable[index][0] == '\0') 
                break;
            index++;
        }
        if (index == 0) 
        {
            /* The Multiple SSID table is empty */
            return " ";
        }
        
        if (chosen > index) 
           return " ";

        /* removing SSID element #%d from the Multiple SSID table */
        wdrv_dbRemoveElementFromMultipleSsidTable(chosen);
    }
    return " ";
#endif

    int index, chosen;
    tiap_param_multi_ssid_t multi_ssid;

    chosen = atoi(delSSID);
    if (chosen > 0)
    {
	    if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid status.");
		    return NULL;
	    }
	    
	    index = 0;
	    while (index < TIAP_PARAM_NUM_MULTIPLE_SSID) 
	    {
		    if (multi_ssid.tbl.ssid[index][0] == '\0') 
			    break;
		    index++;
	    }
	    if (index == 0) 
		    /* The Multiple SSID table is empty */
		    return " ";

	    if (chosen > index) 
		    return " ";

	    /* removing SSID element #%d from the Multiple SSID table */
	    multi_ssid.cmd = TIAP_PARAM_MULTI_SSID_DEL;
	    multi_ssid.tbl.ssid_id = chosen-1;
	    if( tiap_wcfg_set_multi_ssid(wd, &multi_ssid, UPDATE_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to delete from multiple ssid table.");
		    return NULL;
	    }
    }
    return " ";
}


char* wlan_htmlGetMgmt_SSID_TableFirstRow(void)
{
    unsigned long len = 0;
    tableSSID_CurrentIndex = 0;

    tempbuf[0] = 0;
    
    /* first row */
    len += sprintf(tempbuf + len,"<tr>\n");
    len += sprintf(tempbuf + len,"<TD align=middle width=\"20%%\" bgColor=#ccccff height=11>Delete</TD>\n");
    len += sprintf(tempbuf + len,"<TD align=middle width=\"60%%\" bgColor=#ccccff height=11>SSID</TD>\n");
    len += sprintf(tempbuf + len,"<TD align=middle width=\"20%%\" bgColor=#ccccff height=11>#</TD>\n");
    len += sprintf(tempbuf + len,"</tr>\n");

    return tempbuf;
}


/****************************************************************************/
/*                  wlan_htmlGetMgmt_SSID_TableDynamicRow(void)                 */
/****************************************************************************/
/* DESCRIPTION: Returns a Multiple SSID management table row                */ 
/*                                                                          */
/*                                                                          */
/* INPUT:                                                                   */
/* OUTPUT:                                                                  */
/*                                                                          */
/* RETURNS:	Returns a management Multiple SSID table row.                   */
/*          Formats it to html format                                       */
/*                                                                          */
/****************************************************************************/
char* wlan_htmlGetMgmt_SSID_TableDynamicRow(void)
{
#if 0
    char currentSSID[WHAL_SSID_STRING_LEN+1];
    char ssidTable[NUM_MULTIPLE_SSID][WHAL_SSID_STRING_LEN];
    int length = NUM_MULTIPLE_SSID*WHAL_SSID_STRING_LEN;
    int index;
    unsigned long len = 0;

    tempbuf[0] = 0;

    if (tableSSID_CurrentIndex == NUM_MULTIPLE_SSID)
        return tempbuf;

    if (whal_nvram_getMultipleSsidTable(ssidTable[0],&length) != OK)
        return tempbuf;

    for (index=1; index <= NUM_MULTIPLE_SSID; index++)
    {
        if (ssidTable[index-1][0] == '\0') 
            continue;
        if (index > tableSSID_CurrentIndex)
            break;
    }

    if (index == NUM_MULTIPLE_SSID+1) /* end of list */
        return tempbuf;
    tableSSID_CurrentIndex = index;
    memset(currentSSID,0,WHAL_SSID_STRING_LEN+1);
    memcpy(currentSSID,ssidTable[index-1],WHAL_SSID_STRING_LEN);

/************************

  <TR>
    <TD align=middle width="20%" height=11>
    <INPUT style="FONT-SIZE: 8pt; POSITION: relative" type=button value="    " name=DelSSID></TD>
    <TD align=left width=60 height=11>Second_SSID</TD>
    <TD align=middle width="20%" height=11>1</TD>
  </TR>

*************************/

    /* now pour it to the buffer */
    len += sprintf(tempbuf + len,"<tr>\n");
    len += sprintf(tempbuf + len,"<td align=middle width=\"20%%\" height=\"11\">\n");
    len += sprintf(tempbuf + len,"<INPUT style=\"FONT-SIZE: 8pt; POSITION: relative\" onclick='this.value=\"%d\"' type=submit value=\"    \" name=DelSSID></TD>\n",
                   tableSSID_CurrentIndex);
    len += sprintf(tempbuf + len,"<td align=middle width=\"60%%\" height=11>%s</td>\n",currentSSID);
    len += sprintf(tempbuf + len,"<td align=middle width=\"20%%\" height=\"11\">%d\n",tableSSID_CurrentIndex);
    len += sprintf(tempbuf + len,"</tr>\n");

    return tempbuf;
#endif

    char currentSSID[IW_ESSID_MAX_SIZE+1];
    int i;
    unsigned long len = 0;

    tiap_param_multi_ssid_t multi_ssid;

    tempbuf[0] = 0;

    if (tableSSID_CurrentIndex == TIAP_PARAM_NUM_MULTIPLE_SSID)
        return tempbuf;

    if (tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE)) {
	    PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid");
	    return NULL;
    }

    for (i=1; i <= TIAP_PARAM_NUM_MULTIPLE_SSID; i++)
    {
        if (multi_ssid.tbl.ssid[i-1][0] == '\0') 
            continue;
        if (i > tableSSID_CurrentIndex)
            break;
    }

    if (i == TIAP_PARAM_NUM_MULTIPLE_SSID+1) /* end of list */
        return tempbuf;
    tableSSID_CurrentIndex = i;
    memset(currentSSID,0,IW_ESSID_MAX_SIZE+1);
    memcpy(currentSSID,multi_ssid.tbl.ssid[i-1],IW_ESSID_MAX_SIZE);

/************************

  <TR>
    <TD align=middle width="20%" height=11>
    <INPUT style="FONT-SIZE: 8pt; POSITION: relative" type=button value="    " name=DelSSID></TD>
    <TD align=left width=60 height=11>Second_SSID</TD>
    <TD align=middle width="20%" height=11>1</TD>
  </TR>

*************************/

    /* now pour it to the buffer */
    len += sprintf(tempbuf + len,"<tr>\n");
    len += sprintf(tempbuf + len,"<td align=middle width=\"20%%\" height=\"11\">\n");
    len += sprintf(tempbuf + len,"<INPUT style=\"FONT-SIZE: 8pt; POSITION: relative\" onclick='this.value=\"%d\"' type=submit value=\"    \" name=DelSSID></TD>\n",
                   tableSSID_CurrentIndex);
    len += sprintf(tempbuf + len,"<td align=middle width=\"60%%\" height=11>%s</td>\n",currentSSID);
    len += sprintf(tempbuf + len,"<td align=middle width=\"20%%\" height=\"11\">%d\n",tableSSID_CurrentIndex);
    len += sprintf(tempbuf + len,"</tr>\n");

    return tempbuf;

}


char* wlan_htmlSavePageAdvanced(char* value, char *preambleMode, char *inputSlotTimeMode,
                                char *inputPbccMode, char *inputErpMode, char *InputMaxBasic, char *inputMaxActive,
                                char *addrate1, char *addrate2, char *addrate3, char *addrate4,
                                char *addrate5, char *addrate6, char *addrate7, char *addrate8,
                                char *grate1, char *grate2, char *grate3, char *grate4,
                                char *grate5, char *grate6, char *grate7, char *grate8,
                                char *arate1, char *arate2, char *arate3, char *arate4,
                                char *arate5, char *arate6, char *arate7, char *arate8, char* usageTimeMode)
{
#if 0
    unsigned long ratePbcc, maxBasic, maxActive;
    char ratesSet[8];
    unsigned long pbccMode, slotTimeMode, preamble, erpMode, usageTimeModeValue;
    UINT8 ratesLen, currentMaxActive;
    BOOL quit;
    int  i;
    char str[4], support[3];
    char rateString[OFDM_RATES_MAX][3] =
        { " 6", " 9", "12", "18", "24", "36", "48", "54" };
    UINT8 rateValue[OFDM_RATES_MAX] = { DOT11_RATE_6,  DOT11_RATE_9,
                                        DOT11_RATE_12, DOT11_RATE_18,
                                        DOT11_RATE_24, DOT11_RATE_36,
                                        DOT11_RATE_48, DOT11_RATE_54 };
    UINT8 curRates[OFDM_RATES_MAX];
    UINT8 curRatesLen=0;
    UINT8 newRates[OFDM_RATES_MAX];
    UINT8 newRatesLen=0;
    UINT8 dot11Mode;

    char  rates[CCK_PBCC_RATES_MAX];
    UINT8 grates[OFDM_RATES_MAX], arate[OFDM_RATES_MAX];
    UINT8 ratesLength = 0;

    char addrate[OFDM_RATES_MAX], grate[OFDM_RATES_MAX]; 
    UINT8 TotalWones, band1, band2, woneIndex;
    BOOL  doIt = TRUE;

    if (!strcmp(value,"Saving"))
    {
        wdrv_dbGetNumSupWone(&TotalWones);
        switch (TotalWones) 
        {
            case WONE_SINGLE_BAND:
                wdrv_dbGetBand(0 ,&band1);
                if (band1 == RADIO_BAND_5GHZ)
                    doIt = FALSE;
                woneIndex = 0;
                break;

            case WONE_DUAL_BAND:
                wdrv_dbGetBand(0 ,&band1);
                if (band1 == RADIO_BAND_2_4GHZ)
                    woneIndex = 0;
                else
                {
                    wdrv_dbGetBand(1 ,&band2);
                    if (band2 == RADIO_BAND_2_4GHZ)
                        woneIndex = 1;
                    else
                        doIt = FALSE;   /* both Wone 0,1 are 'A' */
                }
				sscanf(usageTimeMode,"%d",&usageTimeModeValue);	/* only in WONE */
				if (usageTimeModeValue == 2)
					usageTimeModeValue = 0;	/* Set to FALSE */
				wdrv_dbSetUsageTimeMode((UINT8)usageTimeModeValue, TRUE);
                break;
        }

        /* preamble */
        sscanf(preambleMode,"%d",&preamble);
        wdrv_dbSetPreambleMode(0,(UINT8)preamble);

        /* slot time */
        sscanf(inputSlotTimeMode,"%d",&slotTimeMode);
        wdrv_dbSetSlotTimeMode(0,(UINT8)slotTimeMode);

        if (doIt) 
        {
            /* ERP */
            sscanf(inputErpMode,"%d",&erpMode);
            wdrv_dbSetERPProtectionMode(0,(UINT8)erpMode);
        
            /* PBCC */
            sscanf(inputPbccMode,"%d",&pbccMode);
            wdrv_dbSetPBCCAlgorithmMode(0,(UINT8)pbccMode);

            /* rates */     
            memset(ratesSet, 0, 8);     
            ratesLen = 0;
            sscanf(InputMaxBasic,"%d",&maxBasic);
            sscanf(inputMaxActive,"%d",&maxActive);
        
            if (maxBasic <= maxActive)  
            {
                switch (maxActive)
                {
                    case 5: 
                        ratesSet[4] = 0x2C;
                        ratesLen++;
                        
                    case 4:
                        ratesLen++;
                        ratesSet[3] = 0x16;
                        
                    case 3:
                        ratesLen++;
                        ratesSet[2] = 0x0B;
                        
                    case 2:
                        ratesLen++;
                        ratesSet[1] = 0x04;
                        
                    case 1:
                        ratesSet[0] = 0x02;
                        ratesLen++;
                        break;
                        
                    default:
                        break;
                }
                switch (maxBasic)
                {
                    case 4:
                        ratesSet[3] = 0x96;
        
                    case 3:
                        ratesSet[2] = 0x8B;
    
                    case 2:
                        ratesSet[1] = 0x84;
    
                    case 1:
                        ratesSet[0] = 0x82;
                        break;
    
                    default:
                        break;
                }
                wdrv_dbSetStationConfigOperationalRateSet(0,ratesSet, ratesLen);
            }
        }

        if (doIt)   /* We are in B/G band - skipping the B modes, remain only with G */
        {
            wdrv_dbGetDot11Mode(woneIndex,&dot11Mode);
            if ((dot11Mode == DOT11_MODE_MIXED) || (dot11Mode == DOT11_MODE_OFDM_ONLY))
                doIt = TRUE;   /* this is G only mode */
            else
                doIt = FALSE;
        }
        
        if (doIt) 
        {
            /* Read current rates setting */

            sscanf(addrate1,"%c",&addrate[0]);
            sscanf(addrate2,"%c",&addrate[1]);
            sscanf(addrate3,"%c",&addrate[2]);
            sscanf(addrate4,"%c",&addrate[3]);
            sscanf(addrate5,"%c",&addrate[4]);
            sscanf(addrate6,"%c",&addrate[5]);
            sscanf(addrate7,"%c",&addrate[6]);
            sscanf(addrate8,"%c",&addrate[7]);
            sscanf(grate1,"%c",&grate[0]);
            sscanf(grate2,"%c",&grate[1]);
            sscanf(grate3,"%c",&grate[2]);
            sscanf(grate4,"%c",&grate[3]);
            sscanf(grate5,"%c",&grate[4]);
            sscanf(grate6,"%c",&grate[5]);
            sscanf(grate7,"%c",&grate[6]);
            sscanf(grate8,"%c",&grate[7]);

            whal_nvram_getOperationalAdditionalRateSetLength(woneIndex,&curRatesLen);
            whal_nvram_getOperationalAdditionalRateSet(woneIndex,curRates, curRatesLen);
            for (i=0; i<OFDM_RATES_MAX; i++)
            {
                CheckRateInList(curRates, curRatesLen, rateValue[i], str);
                if ((addrate[i] != 'Y') && (addrate[i] != 'y') && (addrate[i] != 'N') && (addrate[i] != 'n') && (addrate[i] != 'B') && (addrate[i] != 'b'))
                    memcpy(&addrate[i], str, 1);
                if ((addrate[i] == 'Y') || (addrate[i] == 'y'))
                    newRates[newRatesLen++] = rateValue[i];
                else if ((addrate[i] == 'B') || (addrate[i] == 'b'))
                    newRates[newRatesLen++] = (rateValue[i] | 0x80);
            }
            wdrv_dbSetStationConfigOperationalAdditionalRateSet(woneIndex,newRates, newRatesLen);
            newRatesLen = 0;
            memset(newRates, 0, OFDM_RATES_MAX);
            whal_nvram_getOperationalOFDMRateSetLength(woneIndex,&curRatesLen);
            whal_nvram_getOperationalOFDMRateSet(woneIndex,curRates, curRatesLen);
            for (i=0; i<OFDM_RATES_MAX; i++)
            {
                CheckRateInList(curRates, curRatesLen, rateValue[i], str);

                if ((grate[i] != 'Y') && (grate[i] != 'y') && (grate[i] != 'N') && (grate[i] != 'n') && (grate[i] != 'B') && (grate[i] != 'b'))
                    memcpy(&grate[i], str, 1);
                if ((grate[i] == 'Y') || (grate[i] == 'y'))
                    newRates[newRatesLen++] = rateValue[i];
                else if ((grate[i] == 'B') || (grate[i] == 'b'))
                    newRates[newRatesLen++] = (rateValue[i] | 0x80);
            }
            wdrv_dbSetStationConfigOperationalOFDMRateSet(woneIndex,newRates, newRatesLen);
        }
        
        doIt = FALSE;
        switch (TotalWones) 
        {
            case 1:
                if (band1 == RADIO_BAND_5GHZ)
                    doIt = TRUE;
                woneIndex = 0;
                break;

            case 2:
                doIt = TRUE;
                wdrv_dbGetBand(0 ,&band1);
                if (band1 == RADIO_BAND_5GHZ)
                    woneIndex = 0;
                else
                {
                    wdrv_dbGetBand(1 ,&band2);
                    if (band2 == RADIO_BAND_5GHZ)
                        woneIndex = 1;
                    else
                        doIt = FALSE;   /* both Wone 0,1 are 'A' */
                }
                break;
        }
        if (doIt) 
        {
            /* Read current rates setting */

            sscanf(arate1,"%c",&arate[0]);
            sscanf(arate2,"%c",&arate[1]);
            sscanf(arate3,"%c",&arate[2]);
            sscanf(arate4,"%c",&arate[3]);
            sscanf(arate5,"%c",&arate[4]);
            sscanf(arate6,"%c",&arate[5]);
            sscanf(arate7,"%c",&arate[6]);
            sscanf(arate8,"%c",&arate[7]);

            newRatesLen = 0;
            memset(newRates, 0, OFDM_RATES_MAX);
            whal_nvram_getOperationalOFDMRateSetLength(woneIndex,&curRatesLen);
            whal_nvram_getOperationalOFDMRateSet(woneIndex,curRates, curRatesLen);
            for (i=0; i<OFDM_RATES_MAX; i++)
            {
                CheckRateInList(curRates, curRatesLen, rateValue[i], str);

                if ((arate[i] != 'Y') && (arate[i] != 'y') && (arate[i] != 'N') && (arate[i] != 'n') && (arate[i] != 'B') && (arate[i] != 'b'))
                    memcpy(&arate[i], str, 1);
                if ((arate[i] == 'Y') || (arate[i] == 'y'))
                    newRates[newRatesLen++] = rateValue[i];
                else if ((arate[i] == 'B') || (arate[i] == 'b'))
                    newRates[newRatesLen++] = (rateValue[i] | 0x80);
            }
            wdrv_dbSetStationConfigOperationalOFDMRateSet(woneIndex,newRates, newRatesLen);
        }

        consoleReboot();
    }

    return "";
#endif

    unsigned long maxBasic, maxActive;
    char ratesSet[8];
    UINT8 ratesLen;
    int  i,k;
    /*char str[4];*/
    UINT8 newRates[TIAP_PARAM_OFDM_RATES_MAX];
    UINT8 newRatesLen=0;

    /*UINT8 rateValue[TIAP_PARAM_OFDM_RATES_MAX] = { DOT11_RATE_6,  DOT11_RATE_9,
	    DOT11_RATE_12, DOT11_RATE_18,
	    DOT11_RATE_24, DOT11_RATE_36,
	    DOT11_RATE_48, DOT11_RATE_54 };*/
    UINT8 arate[TIAP_PARAM_OFDM_RATES_MAX];

    char addrate[TIAP_PARAM_OFDM_RATES_MAX], grate[TIAP_PARAM_OFDM_RATES_MAX]; 
    UINT8 band1, band2, woneIndex=0;
    BOOL  doIt = TRUE;

    __u8 num_sup_wone;
    tiap_param_wone_t wone;
    __u8 time_mode, pream_mode, slot_mode, pbcc_mode, erp_mode;
    char rates[TIAP_PARAM_CCK_PBCC_RATES_MAX];

    if (!strcmp(value,"Saving"))
    {
	if( tiap_wcfg_get_basic(wd, TIAP_PARAM_WONE_NUM, &num_sup_wone, FROM_STORAGE) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get wone number.");
		return NULL;
	}
        switch (num_sup_wone) 
        {
            case TIAP_PARAM_WONE_SINGLE_BAND:

						wone.index = 0;
						wone.type = wone_band;
						if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
							PRINT_CONFIG_ERR_MSG("Failed to get band.");
							return NULL;
						}
				
						band1 = wone.elem.band;
					
						if (band1 == TIAP_PARAM_RADIO_BAND_5GHZ)
							doIt = FALSE;
						
						woneIndex = 0;
						break;

			case TIAP_PARAM_WONE_DUAL_BAND:
		
						wone.index = 0;
						wone.type = wone_band;
						if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
							PRINT_CONFIG_ERR_MSG("Failed to get band.");
							return NULL;
						}
				
						band1 = wone.elem.band;
						
						if (band1 == TIAP_PARAM_RADIO_BAND_2_4GHZ)
								woneIndex = 0;
						else
						{
							wone.index = 1;
							wone.type = wone_band;
							if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
								PRINT_CONFIG_ERR_MSG("Failed to get band.");
								return NULL;
							}
				
							band2 = wone.elem.band;
				
							if (band2 == TIAP_PARAM_RADIO_BAND_2_4GHZ)
										woneIndex = 1;
									else
										doIt = FALSE;   /* both Wone 0,1 are 'A' */
						}
						
						
						sscanf(usageTimeMode, "%hhu", &time_mode);	/* only in WONE */
						
						if (time_mode == 2)
							time_mode = 0;	/* Set to FALSE */
				
						if( tiap_wcfg_set_basic(wd, TIAP_PARAM_WONE_TIME_MODE, &time_mode, !UPDATE_STORAGE) ) {
							PRINT_CONFIG_ERR_MSG("Failed to set time mode.");
							return NULL;
						}
				
								break;

				default:
						return NULL;
        }

        /* preamble */
        sscanf(preambleMode, "%hhu", &pream_mode);
		wone.index = 0;
		wone.type = wone_pream_mode;
		wone.elem.pream_mode = pream_mode;

		if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set preamble mode.");
			return NULL;
		}
	
		/* slot time */
		sscanf(inputSlotTimeMode, "%hhu", &slot_mode);
		wone.index = 0;
		wone.type = wone_slot_mode;
		wone.elem.pream_mode = slot_mode;

		if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
			PRINT_CONFIG_ERR_MSG("Failed to set slot mode.");
			return NULL;
		}

        if (doIt) 
        {
			/* ERP */
			sscanf(inputErpMode, "%hhu", &erp_mode);
			wone.index = 0;
			wone.type = wone_erp_mode;
			wone.elem.pream_mode = erp_mode;
	
			if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set erp mode.");
				return NULL;
			}
        
            /* PBCC */
            sscanf(inputPbccMode, "%hhu", &pbcc_mode);
			wone.index = 0;
			wone.type = wone_pbcc;
			wone.elem.pream_mode = pbcc_mode;
	
			if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set pbcc mode.");
				return NULL;
			}

			/* rates */     
			memset(ratesSet, TIAP_PARAM_RATE_NOT_SUPPORTED, 5);     
			ratesLen = 5;
			sscanf(InputMaxBasic,"%ld",&maxBasic);
			sscanf(inputMaxActive,"%ld",&maxActive);

			for(i=1, k=0; i<=ratesLen; i++)
			{
				if(i > maxActive)
					rates[k++] = TIAP_PARAM_RATE_NOT_SUPPORTED;
				else if(i <= maxBasic)
					rates[k++] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
				else
					rates[k++] = TIAP_PARAM_RATE_SUPPORTED;
			}
		
			wone.index = 0;
			wone.type = wone_oper_rate;
			memcpy((void *)wone.elem.b_legacy.rates, (void *)rates, ratesLen);
			wone.elem.b_legacy.len = ratesLen;
		
			if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set operational rates.");
				return NULL;
			}
	}

        if (doIt)   /* We are in B/G band - skipping the B modes, remain only with G */
        {
	    wone.index = woneIndex;
	    wone.type = wone_dot11_mode;

	    if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to get .11 mode.");
		    return NULL;
	    }

            if ((wone.elem.dot11_mode == TIAP_PARAM_D11MODE_MIXED) || (wone.elem.dot11_mode == TIAP_PARAM_D11MODE_OFDM))
                doIt = TRUE;   /* this is G only mode */
            else
                doIt = FALSE;
        }
        
        if (doIt) 
        {
            /* Read current rates setting */

            sscanf(addrate1,"%c",&addrate[0]);
            sscanf(addrate2,"%c",&addrate[1]);
            sscanf(addrate3,"%c",&addrate[2]);
            sscanf(addrate4,"%c",&addrate[3]);
            sscanf(addrate5,"%c",&addrate[4]);
            sscanf(addrate6,"%c",&addrate[5]);
            sscanf(addrate7,"%c",&addrate[6]);
            sscanf(addrate8,"%c",&addrate[7]);
            sscanf(grate1,"%c",&grate[0]);
            sscanf(grate2,"%c",&grate[1]);
            sscanf(grate3,"%c",&grate[2]);
            sscanf(grate4,"%c",&grate[3]);
            sscanf(grate5,"%c",&grate[4]);
            sscanf(grate6,"%c",&grate[5]);
            sscanf(grate7,"%c",&grate[6]);
            sscanf(grate8,"%c",&grate[7]);

	    wone.index = woneIndex;
	    wone.type = wone_oper_add;

	    if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to get .11 mode.");
		    return NULL;
	    }

	    
		for (i=0; i<TIAP_PARAM_OFDM_RATES_MAX; i++)
		{
			if(i > wone.elem.ofdm_legacy.len)
				break;

			if((addrate[i] != 'y')&&(addrate[i] != 'Y')&&(addrate[i] != 'n')&&(addrate[i] != 'N')&&(addrate[i] != 'b')&&(addrate[i] != 'B'))
			{
				if(wone.elem.ofdm_legacy.rates[i] == TIAP_PARAM_RATE_SUPPORTED)
					addrate[i] = 'Y';
				else if(wone.elem.ofdm_legacy.rates[i] == TIAP_PARAM_RATE_BASIC_SUPPORTED)
					addrate[i] = 'B';
				else if(wone.elem.ofdm_legacy.rates[i] == TIAP_PARAM_RATE_NOT_SUPPORTED)
					addrate[i] = 'N';
			}

			if ((addrate[i] == 'Y') || (addrate[i] == 'y'))
				newRates[newRatesLen++] = TIAP_PARAM_RATE_SUPPORTED;
			else if ((addrate[i] == 'B') || (addrate[i] == 'b'))
				newRates[newRatesLen++] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
			else
				newRates[newRatesLen++] = TIAP_PARAM_RATE_NOT_SUPPORTED;
		}

	    wone.index = woneIndex;
	    wone.type = wone_oper_add;
	    memcpy((void *)&wone.elem.ofdm_legacy.rates, (void *)&newRates,
			    sizeof(wone.elem.ofdm_legacy.rates));
	    wone.elem.ofdm_legacy.len = newRatesLen;
	    if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to set operational additional rates.");
		    return NULL;
	    }

	    
        newRatesLen = 0;
        memset(newRates, 0, TIAP_PARAM_OFDM_RATES_MAX);

	    wone.index = woneIndex;
	    wone.type = wone_oper_ofdm;

	    if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to get operational ofdm rates.");
		    return NULL;
	    }



		for (i=0; i<TIAP_PARAM_OFDM_RATES_MAX; i++)
		{
			if(i > wone.elem.ofdm_extend.len)
				break;

			if((grate[i] != 'y')&&(grate[i] != 'Y')&&(grate[i] != 'n')&&(grate[i] != 'N')&&(grate[i] != 'b')&&(grate[i] != 'B'))
			{
				if(wone.elem.ofdm_extend.rates[i] == TIAP_PARAM_RATE_SUPPORTED)
					grate[i] = 'Y';
				else if(wone.elem.ofdm_extend.rates[i] == TIAP_PARAM_RATE_BASIC_SUPPORTED)
					grate[i] = 'B';
				else if(wone.elem.ofdm_extend.rates[i] == TIAP_PARAM_RATE_NOT_SUPPORTED)
					grate[i] = 'N';
			}

			if ((grate[i] == 'Y') || (grate[i] == 'y'))
				newRates[newRatesLen++] = TIAP_PARAM_RATE_SUPPORTED;
			else if ((grate[i] == 'B') || (grate[i] == 'b'))
				newRates[newRatesLen++] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
			else
				newRates[newRatesLen++] = TIAP_PARAM_RATE_NOT_SUPPORTED;
		}


	    wone.index = woneIndex;
	    wone.type = wone_oper_ofdm;
	    memcpy((void *)&wone.elem.ofdm_extend.rates, (void *)&newRates,
			    sizeof(wone.elem.ofdm_extend.rates));
	    wone.elem.ofdm_extend.len = newRatesLen;
	    if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
		    PRINT_CONFIG_ERR_MSG("Failed to set operational ofdm rates.");
		    return NULL;
	    }
        }
        
        doIt = FALSE;
        switch (num_sup_wone)
        {
            case 1:
                if (band1 == TIAP_PARAM_RADIO_BAND_5GHZ)
                    doIt = TRUE;
                woneIndex = 0;
                break;

            case 2:
                doIt = TRUE;

				wone.index = 0;
				wone.type = wone_band;
				if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
					PRINT_CONFIG_ERR_MSG("Failed to get band.");
					return NULL;
				}

				band1 = wone.elem.band;
		
                if (band1 == TIAP_PARAM_RADIO_BAND_5GHZ)
                    woneIndex = 0;
                else
                {
					wone.index = 1;
					wone.type = wone_band;
					if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
						PRINT_CONFIG_ERR_MSG("Failed to get band.");
						return NULL;
					}

					band2 = wone.elem.band;
					
					if (band2 == TIAP_PARAM_RADIO_BAND_5GHZ)
						woneIndex = 1;
					else
						doIt = FALSE;   /* both Wone 0,1 are 'A' */
				}
                break;
        }
        if (doIt) 
        {
            /* Read current rates setting */

            sscanf(arate1,"%c",&arate[0]);
            sscanf(arate2,"%c",&arate[1]);
            sscanf(arate3,"%c",&arate[2]);
            sscanf(arate4,"%c",&arate[3]);
            sscanf(arate5,"%c",&arate[4]);
            sscanf(arate6,"%c",&arate[5]);
            sscanf(arate7,"%c",&arate[6]);
            sscanf(arate8,"%c",&arate[7]);

            newRatesLen = 0;
            memset(newRates, 0, TIAP_PARAM_OFDM_RATES_MAX);
	    
			wone.index = woneIndex;
			wone.type = wone_oper_add;

			if( tiap_wcfg_get_wone(wd, &wone, FROM_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to get operational ofdm rates.");
				return NULL;
			}
			
			for (i=0; i<TIAP_PARAM_OFDM_RATES_MAX; i++)
			{
				if(i > wone.elem.ofdm_legacy.len)
					break;

				if((arate[i] != 'y')&&(arate[i] != 'Y')&&(arate[i] != 'n')&&(arate[i] != 'N')&&(arate[i] != 'b')&&(arate[i] != 'B'))
				{
					if(wone.elem.ofdm_legacy.rates[i] == TIAP_PARAM_RATE_SUPPORTED)
						arate[i] = 'Y';
					else if(wone.elem.ofdm_legacy.rates[i] == TIAP_PARAM_RATE_BASIC_SUPPORTED)
						arate[i] = 'B';
					else if(wone.elem.ofdm_legacy.rates[i] == TIAP_PARAM_RATE_NOT_SUPPORTED)
						arate[i] = 'N';
				}

				if ((arate[i] == 'Y') || (arate[i] == 'y'))
					newRates[newRatesLen++] = TIAP_PARAM_RATE_SUPPORTED;
				else if ((arate[i] == 'B') || (arate[i] == 'b'))
					newRates[newRatesLen++] = TIAP_PARAM_RATE_BASIC_SUPPORTED;
				else
					newRates[newRatesLen++] = TIAP_PARAM_RATE_NOT_SUPPORTED;
			}

			wone.index = woneIndex;
			wone.type = wone_oper_add;
			memcpy((void *)&wone.elem.ofdm_legacy.rates, (void *)&newRates,
					sizeof(wone.elem.ofdm_legacy.rates));
			wone.elem.ofdm_legacy.len = newRatesLen;
			if( tiap_wcfg_set_wone(wd, &wone, !UPDATE_STORAGE) ) {
				PRINT_CONFIG_ERR_MSG("Failed to set operational ofdm rates.");
				return NULL;
			}
        }

	restart();
    }

    return "";

	
}


/******************************************************************************
*                               hexChar2int(void)                                 *
*******************************************************************************
DESCRIPTION:	Return the decimal value of an hexadecimal character.  						
																										
INPUT:			c - hexadecimal character.
OUTPUT:		   None.
 
RETURN:			decimal value.
																										
******************************************************************************/
unsigned char hexChar2int(unsigned char c)
{
   if (c >= 'A' && c <= 'F')
      return (c - 'A' + 10);
   else if (c >= 'a' && c <= 'f')
      return (c - 'a' + 10);
   return c - '0';
}


/*******************************************************************************

                                    Privacy                                     

********************************************************************************/

char* wlan_htmlPrivacyPageNotePerMultipleSsidActivated(void)
{
#if 0
    UINT16 enable;

    wdrv_dbGetMultipleSsidStatus(&enable);
    if (enable)
    {
        strcpy(tempbuf,"<P><b><font color=\"#FF0000\">Note:</font></b> Privacy is deactivated, due to use of Multiple SSID. ");
        strcat(tempbuf,"See <A href=\"mgmt.htm\">Management page</A>.</P>");
        return tempbuf;
    }
    else
        return " ";
#endif

    tiap_param_multi_ssid_t multi_ssid;

    if( tiap_wcfg_get_multi_ssid(wd, &multi_ssid, FROM_STORAGE) ) {
	    PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid status.");
	    return NULL;
    }

    if (multi_ssid.enabled)
    {
	    strcpy(tempbuf,"<P><b><font color=\"#FF0000\">Note:</font></b> Privacy is deactivated, due to use of Multiple SSID. ");
	    strcat(tempbuf,"See <A href=\"mgmt.htm\">Management page</A>.</P>");
	    return tempbuf;
    }
    else
	    return " ";
}

char* wlan_htmlMgmtPageNotePerPrivacyActivated(void)
{
#if 0
    if (wdrv_dbIsPrivacyEnabled())
    {
        strcpy(tempbuf,"<P><b><font color=\"#FF0000\">Note:</font></b> Multiple SSID is deactivated, due to use of Privacy. ");
        strcat(tempbuf,"See <A href=\"privacy.htm\">Privacy page</A>.</P>");
        return tempbuf;
    }
    else
        return " ";
#endif

    if (is_privacy_enabled())
    {
	    strcpy(tempbuf,"<P><b><font color=\"#FF0000\">Note:</font></b> Multiple SSID is deactivated, due to use of Privacy. ");
	    strcat(tempbuf,"See <A href=\"privacy.htm\">Privacy page</A>.</P>");
	    return tempbuf;
    }
    else
	    return " ";

}

char* wlan_htmlGetPrivacyGroupKeyInterval(void)
{
#if 0
    UINT32 value;

    wdrv_dbGetGTKInterval(&value);	
	value = value / 1000;
	sprintf(tempbuf,"%d",value);
    return tempbuf;
#endif

    __u32 value;

    tiap_wcfg_get_gtk_interval(wd, &value);
    value = value / 1000;
    sprintf(tempbuf,"%d",value);
    return tempbuf;
}

char* wlan_htmlGetPrivacyPskStringValue(void)
{
#if 0
	static char tempbuf2[MAX_RETURNED_STRING_LEN];
    tempbuf2[0] = 0;

	wdrv_dbGetPskPwd(tempbuf2);

	sprintf(tempbuf, "\"%s\"", tempbuf2);
	return tempbuf;
#endif

	__s8 passwd[TIAP_WPA_MAX_PSK_PWD_LEN+1];
	__u8 len;

	tiap_wcfg_get_psk_passwd(wd, passwd, &len);
	passwd[len] = '\0';

	sprintf(tempbuf, "\"%s\"", passwd);
	return tempbuf;
}

char* wlan_htmlGetPrivacyRadiusSecret(void)
{
#if 0
	static char tempbuf2[MAX_RETURNED_STRING_LEN];
    tempbuf2[0] = 0;

	wdrv_dbGetRadiusSecret(tempbuf2);

	sprintf(tempbuf, "\"%s\"", tempbuf2);
	return tempbuf;
#endif

	__s8 secret[TIAP_WPA_MAX_RADIUS_SECRET_LEN+1];
	__u8 len;

	tiap_wcfg_get_radius_secret(wd, secret, &len);
	secret[len] = '\0';
	sprintf(tempbuf, "\"%s\"", secret);
	return tempbuf;
}


char* wlan_htmlGetPrivacyRadiusPort(void)
{
#if 0
    UINT32 value;

    wdrv_dbGetRadiusPort(&value);	
	sprintf(tempbuf,"%d",value);
    return tempbuf;
#endif
    
    __u16 port;
    
    tiap_wcfg_get_radius_port(wd, &port);
    sprintf(tempbuf,"%hu", port);

    return tempbuf;

}

char* wlan_htmlGetPrivacyRadiusAddr(char *ipComponent)
{
#if 0
    UINT8 addr1, addr2, addr3, addr4;
	unsigned long component;

    wdrv_dbGetRadiusIpAddr(&addr1,&addr2,&addr3,&addr4);
    sscanf(ipComponent,"%d",&component);

    switch (component) 
    {
        case 1:
            sprintf(tempbuf,"%d",addr1);
            break;
        case 2:
            sprintf(tempbuf,"%d",addr2);
            break;
        case 3:
            sprintf(tempbuf,"%d",addr3);
            break;
        case 4:
            sprintf(tempbuf,"%d",addr4);
            break;
    }
	return tempbuf;
#endif

	__u32 ipaddr;
	__u8 *p = (__u8 *)&ipaddr;
	unsigned long component;

	tiap_wcfg_get_radius_ipaddr(wd, &ipaddr);
	sscanf(ipComponent,"%ld",&component);

	switch (component) 
	{
		case 1:
			sprintf(tempbuf,"%hhu", p[0]);
			break;
		case 2:
			sprintf(tempbuf,"%hhu", p[1]);
			break;
		case 3:
			sprintf(tempbuf,"%hhu", p[2]);
			break;
		case 4:
			sprintf(tempbuf,"%hhu", p[3]);
			break;
	}
	return tempbuf;
}


char* wlan_htmlGetPrivacyPskHexValue(void)
{
#if 0
	UINT32 hexNumber[8];
	int j, stringLen = 0;

	hexNumber[0] = 0;
	
    wdrv_dbGetPskHex((UINT8 *)hexNumber);

	for (j = 0; j < 8; j++)
		stringLen += sprintf(tempbuf + stringLen,"%02X", hexNumber[j]);

	
	return tempbuf;
#endif

	strncpy(tempbuf, "psk hex value: feature is not supported in apdk 5.6", sizeof(tempbuf));
	
	return tempbuf;
}


char* wlan_htmlGetPrivacyPsk(char *option)
{
#if 0
    int inputKeyId;
	UINT32 keyId;

	sscanf(option,"%d",&inputKeyId);

	wdrv_dbGetPassPhraseMode(&keyId);

	if (inputKeyId == keyId)
		return "checked";
	else 
		return " ";
#endif
	int inputKeyId;
	__u32 value = 0;

	sscanf(option,"%d",&inputKeyId);

	tiap_wcfg_get_psk_passphrase_mode(wd, &value);

	if (inputKeyId == value)
		return "checked";
	else 
		return " ";
}

char* wlan_htmlGetPrivacyOn(char* value)
{
#if 0
	int input;
	BOOL option;

	sscanf(value,"%d",&input);

    wdrv_dbGetStationConfigPrivacyOptionImplemented(&option);

	if (input == option)
		return "checked";
	else 
		return " ";
#endif

	int input;
	__u8 priv_option;

	sscanf(value,"%d",&input);

	if (tiap_wcfg_get_basic(wd, TIAP_PARAM_PRIV_OPTION, &priv_option, !FROM_STORAGE)) {
		PRINT_CONFIG_ERR_MSG("failed to get privacy option");
		return NULL;
	}

	if (input == priv_option)
		return "checked";
	else 
		return " ";

}

char* wlan_htmlGetPrivacyDefaultGateway(char *ipComponent)
{
#if 0
    UINT8 addr1, addr2, addr3, addr4;
	unsigned long component;

    wdrv_dbGetDefaultGateway(&addr1,&addr2,&addr3,&addr4);
    sscanf(ipComponent,"%d",&component);

    switch (component) 
    {
        case 1:
            sprintf(tempbuf,"%d",addr1);
            break;
        case 2:
            sprintf(tempbuf,"%d",addr2);
            break;
        case 3:
            sprintf(tempbuf,"%d",addr3);
            break;
        case 4:
            sprintf(tempbuf,"%d",addr4);
            break;
    }
	return tempbuf;
#endif

	unsigned long component;
	__u32 gateway;
	__u8 *p = (__u8 *)&gateway;

	tiap_wcfg_get_default_gateway(&gateway);
	sscanf(ipComponent,"%ld",&component);

	switch (component) 
	{
		case 1:
			sprintf(tempbuf,"%hhu", p[0]);
			break;
		case 2:
			sprintf(tempbuf,"%hhu", p[1]);
			break;
		case 3:
			sprintf(tempbuf,"%hhu", p[2]);
			break;
		case 4:
			sprintf(tempbuf,"%hhu", p[3]);
			break;
	}
	return tempbuf;
}

char *wlan_htmlGetStationTypes(char* value)
{
#if 0
	int input;
	UINT32 mode;
    UINT16 enable;

	sscanf(value,"%d",&input);

    wdrv_dbGetMultipleSsidStatus(&enable);
    if (enable)
    {
        return " disabled";
    }


    wdrv_dbGetWpaMode(&mode);

	if ((mode & DOT11_WPA_STA_TYPE_WEP_LEGACY) && (input == 2))
		return "checked";
	if ((mode & DOT11_WPA_STA_TYPE_WEP_8021X) && (input == 4))
		return "checked";
	if ((mode & DOT11_WPA_STA_TYPE_WPA) && (input == 1))
		return "checked";

	return " ";
#endif

	int input;
	tiap_param_multi_ssid_t multi_ssid;
	tiap_param_security_type_t mode;

	sscanf(value,"%d",&input);

	if (tiap_wcfg_get_multi_ssid(wd, &multi_ssid, !FROM_STORAGE)) {
		PRINT_CONFIG_ERR_MSG("Failed to get multiple ssid");
		return NULL;
	}
	
	if (multi_ssid.enabled)
		return " disabled";

	if (tiap_wcfg_get_basic(wd, TIAP_PARAM_SECURITY_TYPE, &mode, !FROM_STORAGE)) {
		PRINT_CONFIG_ERR_MSG("failed to read security type");
		return NULL;
	}

	if ((mode & TIAP_PARAM_SECURITY_WEP) && (input == 2))
		return "checked";
	if ((mode & TIAP_PARAM_SECURITY_802_1X) && (input == 4))
		return "checked";
	if ((mode & TIAP_PARAM_SECURITY_WPA) && (input == 1))
		return "checked";

	return " ";
}


char* wlan_htmlGetPrivacyWepCipher(char *option)
{
#if 0
    int inputKeyId;
	UINT32 keyId;

	sscanf(option,"%d",&inputKeyId);

	wdrv_dbGetDefaultCipher(&keyId);

    switch (keyId) 
    {
        case DOT11_CIPHER_WEP_40:
            if (inputKeyId == 0) 
                return "checked";
            break;
        case DOT11_CIPHER_WEP_104:
            if (inputKeyId == 1) 
                return "checked";
            break;
        case DOT11_CIPHER_WEP_232:
            if (inputKeyId == 2) 
                return "checked";
            break;
        default:
            break;
    }
    return " ";
#endif

    int inputKeyId;
    __u32 wep_cipher;

    sscanf(option,"%d",&inputKeyId);

    tiap_wcfg_get_default_cipher(wd, &wep_cipher);

    switch (wep_cipher) 
    {
	    case TIAP_WPA_CIPHER_WEP_40:
		    if (inputKeyId == 0) 
			    return "checked";
		    break;
	    case TIAP_WPA_CIPHER_WEP_104:
		    if (inputKeyId == 1) 
			    return "checked";
		    break;
	    case TIAP_WPA_CIPHER_WEP_232:
		    if (inputKeyId == 2) 
			    return "checked";
		    break;
	    default:
		    break;
    }
    return " ";

}

char* wlan_htmlGetPrivacyAuthMode(char *option)
{   
#if 0
	int     i, inputOption;
	BOOL    openSystemEnable, sharedKeyEnable;
	UINT16 authAlgo;
	BOOL enable;

    
	openSystemEnable = FALSE;
    sharedKeyEnable = FALSE;

    sscanf(option,"%d",&inputOption);

	for (i = 0; i < MAX_AUTH_ALGORITHM_SUPPORTED; i++)
	{
		wdrv_dbGetAuthAlgo(i, &authAlgo);
		wdrv_dbGetAuthAlgoEnable(i, &enable);

		if ((authAlgo == MIB_OPEN_SYSTEM) && (enable == TRUE))
			openSystemEnable = TRUE;
		else if ((authAlgo == MIB_SHARED_KEY) && (enable == TRUE))
			sharedKeyEnable = TRUE;
	}

	if ((inputOption == 1) && (openSystemEnable) && (!sharedKeyEnable))
		return "selected";
	else if ((inputOption == 2) && (!openSystemEnable) && (sharedKeyEnable))
		return "selected";
	else if ((inputOption == 3) && (openSystemEnable) && (sharedKeyEnable))
		return "selected";
	else
		return " ";
#endif

	int     i, inputOption;
	BOOL    openSystemEnable, sharedKeyEnable;
	tiap_param_auth_algo_t tbl[TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED];


	openSystemEnable = FALSE;
	sharedKeyEnable = FALSE;

	sscanf(option,"%d",&inputOption);

	for (i = 0; i < TIAP_PARAM_MAX_AUTH_ALG_SUPPORTED; i++)
	{
		if (tiap_wcfg_get_auth_type(wd, tbl, !FROM_STORAGE)) {
			PRINT_CONFIG_ERR_MSG("Failed to get authentication algorithm table");
			return NULL;
		}

		if ((tbl[i].algo == TIAP_PARAM_MIB_OPEN_SYSTEM) && (tbl[i].enable == TRUE))
			openSystemEnable = TRUE;
		else if ((tbl[i].algo == TIAP_PARAM_MIB_SHARED_KEY) && (tbl[i].enable == TRUE))
			sharedKeyEnable = TRUE;
	}

	if ((inputOption == 1) && (openSystemEnable) && (!sharedKeyEnable))
		return "selected";
	else if ((inputOption == 2) && (!openSystemEnable) && (sharedKeyEnable))
		return "selected";
	else if ((inputOption == 3) && (openSystemEnable) && (sharedKeyEnable))
		return "selected";
	else
		return " ";

}

/****************************************************************************/
/*                      wlan_htmlGetKeyVal0(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the first wep default key							*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     first wep default key										*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetKeyVal0(void)
{
#if 0
	UINT32 wepCipher;
	char key[WEP_KEY_LEN_MAX];
	UINT8 keyLen;
	int j, stringLen = 0;

	tempbuf[0] = 0;
	
	wdrv_dbGetWEPDefaultKeys(0, &keyLen, key);

	wdrv_dbGetDefaultCipher(&wepCipher);
    switch (wepCipher) 
    {
        case DOT11_CIPHER_WEP_40:
			keyLen = 5;
            break;

        case DOT11_CIPHER_WEP_104:
			keyLen = 13;
            break;

        case DOT11_CIPHER_WEP_232:
			keyLen = 29;
            break;

        default:
            break;
    }

	for (j = 0; j < keyLen; j++)
		stringLen += sprintf(tempbuf + stringLen,"%02X", key[j]);

	return tempbuf;
#endif

	__u32 wep_cipher;
	tiap_wpa_wep_key_t wep_key;
	int j, stringLen = 0;

	tempbuf[0] = 0;

	if( tiap_wcfg_get_def_wep_key(wd, 0, &wep_key) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get default wep key.");
		return NULL;
	}

	tiap_wcfg_get_default_cipher(wd, &wep_cipher);
	switch (wep_cipher) 
	{
		case TIAP_WPA_CIPHER_WEP_40:
			wep_key.len = 5;
			break;

		case TIAP_WPA_CIPHER_WEP_104:
			wep_key.len = 13;
			break;

		case TIAP_WPA_CIPHER_WEP_232:
			wep_key.len = 29;
			break;

		default:
			break;
	}

	for (j = 0; j < wep_key.len; j++)
		stringLen += sprintf(tempbuf + stringLen,"%02X", wep_key.key[j]);

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetKeyVal1(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the second wep default key							*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     second wep default key										*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetKeyVal1(void)
{
#if 0
	UINT32 wepCipher;
	char key[WEP_KEY_LEN_MAX];
	UINT8 keyLen;
	int j, stringLen = 0;

	tempbuf[0] = 0;
	
	wdrv_dbGetWEPDefaultKeys(1, &keyLen, key);

	wdrv_dbGetDefaultCipher(&wepCipher);
    switch (wepCipher) 
    {
        case DOT11_CIPHER_WEP_40:
			keyLen = 5;
            break;

        case DOT11_CIPHER_WEP_104:
			keyLen = 13;
            break;

        case DOT11_CIPHER_WEP_232:
			keyLen = 29;
            break;

        default:
            break;
    }

	for (j = 0; j < keyLen; j++)
		stringLen += sprintf(tempbuf + stringLen,"%02X", key[j]);

	return tempbuf;
#endif

	__u32 wep_cipher;
	tiap_wpa_wep_key_t wep_key;
	int j, stringLen = 0;

	tempbuf[0] = 0;

	if( tiap_wcfg_get_def_wep_key(wd, 1, &wep_key) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get default wep key.");
		return NULL;
	}

	tiap_wcfg_get_default_cipher(wd, &wep_cipher);

	switch (wep_cipher) 
	{
		case TIAP_WPA_CIPHER_WEP_40:
			wep_key.len = 5;
			break;

		case TIAP_WPA_CIPHER_WEP_104:
			wep_key.len = 13;
			break;

		case TIAP_WPA_CIPHER_WEP_232:
			wep_key.len = 29;
			break;

		default:
			break;
	}

	for (j = 0; j < wep_key.len; j++)
		stringLen += sprintf(tempbuf + stringLen,"%02X", wep_key.key[j]);

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetKeyVal2(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the third wep default key							*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     third wep default key										*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetKeyVal2(void)
{
#if 0
	UINT32 wepCipher;
	char key[WEP_KEY_LEN_MAX];
	UINT8 keyLen;
	int j, stringLen = 0;

	tempbuf[0] = 0;
	
	wdrv_dbGetWEPDefaultKeys(2, &keyLen, key);

	wdrv_dbGetDefaultCipher(&wepCipher);
    switch (wepCipher) 
    {
        case DOT11_CIPHER_WEP_40:
			keyLen = 5;
            break;

        case DOT11_CIPHER_WEP_104:
			keyLen = 13;
            break;

        case DOT11_CIPHER_WEP_232:
			keyLen = 29;
            break;

        default:
            break;
    }

	for (j = 0; j < keyLen; j++)
		stringLen += sprintf(tempbuf + stringLen,"%02X", key[j]);

	return tempbuf;
#endif

	__u32 wep_cipher;
	tiap_wpa_wep_key_t wep_key;
	int j, stringLen = 0;

	tempbuf[0] = 0;

	if( tiap_wcfg_get_def_wep_key(wd, 2, &wep_key) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get default wep key.");
		return NULL;
	}

	tiap_wcfg_get_default_cipher(wd, &wep_cipher);
	switch (wep_cipher) 
	{
		case TIAP_WPA_CIPHER_WEP_40:
			wep_key.len = 5;
			break;

		case TIAP_WPA_CIPHER_WEP_104:
			wep_key.len = 13;
			break;

		case TIAP_WPA_CIPHER_WEP_232:
			wep_key.len = 29;
			break;

		default:
			break;
	}

	for (j = 0; j < wep_key.len; j++)
		stringLen += sprintf(tempbuf + stringLen,"%02X", wep_key.key[j]);

	return tempbuf;

}

/****************************************************************************/
/*                      wlan_htmlGetKeyVal3(void)								*/
/****************************************************************************/
/* DESCRIPTION: Returns the fourth wep default key							*/ 
/*																		    */
/*																		    */
/* INPUT:       NONE.								                        */
/* OUTPUT:      NONE.														*/
/*																			*/
/* RETURNS:     fourth wep default key										*/
/*																			*/
/****************************************************************************/
char* wlan_htmlGetKeyVal3(void)
{
#if 0
	UINT32 wepCipher;
	char key[WEP_KEY_LEN_MAX];
	UINT8 keyLen;
	int j, stringLen = 0;

	tempbuf[0] = 0;
	
	wdrv_dbGetWEPDefaultKeys(3, &keyLen, key);

	wdrv_dbGetDefaultCipher(&wepCipher);
    switch (wepCipher) 
    {
        case DOT11_CIPHER_WEP_40:
			keyLen = 5;
            break;

        case DOT11_CIPHER_WEP_104:
			keyLen = 13;
            break;

        case DOT11_CIPHER_WEP_232:
			keyLen = 29;
            break;

        default:
            break;
    }

	for (j = 0; j < keyLen; j++)
		stringLen += sprintf(tempbuf + stringLen,"%02X", key[j]);

	return tempbuf;
#endif
	__u32 wep_cipher;
	tiap_wpa_wep_key_t wep_key;
	int j, stringLen = 0;

	tempbuf[0] = 0;
	
	if( tiap_wcfg_get_def_wep_key(wd, 3, &wep_key) ) {
		PRINT_CONFIG_ERR_MSG("Failed to get default wep key.");
		return NULL;
	}

	tiap_wcfg_get_default_cipher(wd, &wep_cipher);

	switch (wep_cipher) 
	{
		case TIAP_WPA_CIPHER_WEP_40:
			wep_key.len = 5;
			break;

		case TIAP_WPA_CIPHER_WEP_104:
			wep_key.len = 13;
			break;

		case TIAP_WPA_CIPHER_WEP_232:
			wep_key.len = 29;
			break;

		default:
			break;
	}

	for (j = 0; j < wep_key.len; j++)
		stringLen += sprintf(tempbuf + stringLen,"%02X", wep_key.key[j]);

	return tempbuf;

}

static void create_js_msg_box(char *buf, char *message)
{
	char *msg_box = "<SCRIPT LANGUAGE=\"JavaScript\">
			   window.alert(\"%s\")
			   </SCRIPT>";

	if (buf)
		sprintf(buf,msg_box, message);

}
